# This is an auto-generated file. Do not modify this file manually
from enum import Enum

class Station(Enum):
	"""
	This enum was generated using the data/parser.py script
	The data has been extracted from the VVS Steige CSV file
	This enum contains all unique stations from the CSV file

	The station names are sanitised to be valid Python identifiers
	The station names are in the format: 'de:08111:154:1:1'
	The last two values describe the platform and the direction and might not be included
	Note that even if you select a specific platform, the API will still return all possible connections
	"""

	WALDBURGSTRASSE_1 = 'de:08111:2:0:3'  # Rtg Hegel-Gymnasium/Pestalozzi-Schule (Stuttgart Stuttgart)
	WALDBURGSTRASSE_2 = 'de:08111:2:0:4'  # Rtg Vaihingen ZOB (Stuttgart Stuttgart)
	WALDBURGSTRASSE_3 = 'de:08115:4675:0:3'  # eRtg Elsa-Brändström-Straße (Böblingen Böblingen)
	WALDBURGSTRASSE_4 = 'de:08115:4675:0:4'  # Rtg Südwest (Böblingen Böblingen)
	FAUSTSTRASSE = 'de:08111:5'
	FAUSTSTRASSE_1 = 'de:08111:5:1:1'  # Rtg Vaihingen Schillerplatz (Stuttgart Stuttgart)
	FAUSTSTRASSE_2 = 'de:08111:5:1:2'  # Rtg Vaihingen Viadukt (Stuttgart Stuttgart)
	FAUSTSTRASSE_3 = 'de:08111:5:3:5'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	VAIHINGEN_VIADUKT = 'de:08111:6'
	VAIHINGEN_VIADUKT_1 = 'de:08111:6:1:1'  # Rtg Fauststr. (Stuttgart Stuttgart)
	VAIHINGEN_VIADUKT_2 = 'de:08111:6:1:2'  # Rtg Engelboldstr. (Stuttgart Stuttgart)
	ENGELBOLDSTRASSE = 'de:08111:8'
	ENGELBOLDSTRASSE_1 = 'de:08111:8:1:1'  # Rtg Vaihingen Viadukt (Stuttgart Stuttgart)
	ENGELBOLDSTRASSE_2 = 'de:08111:8:1:2'  # Rtg Kaltental (Stuttgart Stuttgart)
	ENGELBOLDSTRASSE_3 = 'de:08111:8:3:5'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	SÜDHEIMER_PLATZ = 'de:08111:13'
	SÜDHEIMER_PLATZ_1 = 'de:08111:13:1:1'  # Rtg Heslach Vogelrain (Stuttgart Stuttgart)
	SÜDHEIMER_PLATZ_2 = 'de:08111:13:2:2'  # Rtg Bihlplatz (Stuttgart Stuttgart)
	SÜDHEIMER_PLATZ_3 = 'de:08111:13:3:3'  # Seilbahn (Stuttgart Stuttgart)
	SÜDHEIMER_PLATZ_4 = 'de:08111:13:4:4'  # Bus Rtg einwärts (Stuttgart Stuttgart)
	BIHLPLATZ = 'de:08111:15'
	BIHLPLATZ_1 = 'de:08111:15:0:1'  # Rtg Südheimer Platz (Stuttgart Stuttgart)
	BIHLPLATZ_2 = 'de:08111:15:0:2'  # Rtg Erwin-Schöttle-Platz (Stuttgart Stuttgart)
	NECKARTOR = 'de:08111:25'
	NECKARTOR_1 = 'de:08111:25:0:1'  # Rtg Staatsgalerie (Stuttgart Stuttgart)
	NECKARTOR_2 = 'de:08111:25:0:2'  # Rtg Stöckach (Stuttgart Stuttgart)
	NECKARTOR_3 = 'de:08111:25:0:4'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	STÖCKACH = 'de:08111:26'
	STÖCKACH_1 = 'de:08111:26:1:1'  # Rtg Neckartor (Stuttgart Stuttgart)
	STÖCKACH_2 = 'de:08111:26:2:2'  # Rtg Metzstr./Karl-Olga-Krhs. (Stuttgart Stuttgart)
	STÖCKACH_3 = 'de:08111:26:3:4'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	METZSTRASSE = 'de:08111:27'
	METZSTRASSE_1 = 'de:08111:27:0:1'  # Rtg Stöckach (Stuttgart Stuttgart)
	METZSTRASSE_2 = 'de:08111:27:0:2'  # Rtg Mineralbäder (Stuttgart Stuttgart)
	MINERALBÄDER = 'de:08111:28'
	MINERALBÄDER_1 = 'de:08111:28:1:1'  # Rtg Metzstraße (Stuttgart Stuttgart)
	MINERALBÄDER_2 = 'de:08111:28:1:2'  # Rtg Mercedesstr./Wilhelma (Stuttgart Stuttgart)
	MERCEDESSTRASSE_1 = 'de:08111:30:0:1'  # Stadtbahn Rtg Mineralbäder (Stuttgart Stuttgart)
	MERCEDESSTRASSE_2 = 'de:08111:30:0:2'  # Stadtbahn Rtg Wilhelmpl./Stadion (Stuttgart Stuttgart)
	MERCEDESSTRASSE_3 = 'de:08118:6491:0:3'  # Rtg Hirschl Nord (Ditzingen Hirschlanden)
	MERCEDESSTRASSE_4 = 'de:08118:6491:0:4'  # Rtg Brandström/Hirschlander (Ditzingen Hirschlanden)
	UFF_KIRCHHOF = 'de:08111:32'
	UFF_KIRCHHOF_1 = 'de:08111:32:1:2'  # Rtg Ebitzweg/Augsburger Pl. (Stuttgart Stuttgart)
	UFF_KIRCHHOF_2 = 'de:08111:32:2:1'  # Rtg Wilhelmsplatz (Stuttgart Stuttgart)
	UFF_KIRCHHOF_3 = 'de:08111:32:3:3'  # NBus Rtg einwärts (Stuttgart Stuttgart)
	AUGSBURGER_PLATZ = 'de:08111:33'
	AUGSBURGER_PLATZ_1 = 'de:08111:33:1:1'  # Rtg Uff-Kirchhof (Stuttgart Stuttgart)
	AUGSBURGER_PLATZ_2 = 'de:08111:33:2:2'  # Rtg Nürnberger Str. (Stuttgart Stuttgart)
	AUGSBURGER_PLATZ_3 = 'de:08111:33:5:3'  # NBus (Stuttgart Stuttgart)
	NÜRNBERGER_STRASSE = 'de:08111:34'
	NÜRNBERGER_STRASSE_1 = 'de:08111:34:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	NÜRNBERGER_STRASSE_2 = 'de:08111:34:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	NÜRNBERGER_STRASSE_3 = 'de:08111:34:2:3'  # Gleis 3 (Stuttgart Stuttgart)
	NÜRNBERGER_STRASSE_4 = 'de:08111:34:2:4'  # Gleis 4 (Stuttgart Stuttgart)
	NÜRNBERGER_STRASSE_5 = 'de:08111:34:3:1'  # Rtg Augsburger Platz (Stuttgart Stuttgart)
	NÜRNBERGER_STRASSE_6 = 'de:08111:34:4:2'  # Rtg Antwerpener Str. (Stuttgart Stuttgart)
	NÜRNBERGER_STRASSE_7 = 'de:08111:34:5:3'  # NBus (Stuttgart Stuttgart)
	NÜRNBERGER_STRASSE_8 = 'de:08111:34:5:7'  # Ersatzhaltestelle U1E  Rtg Antwerp. Str. (Stuttgart Stuttgart)
	NÜRNBERGER_STRASSE_9 = 'de:08111:34:7:10'  # SEV S Rtg Bad Cannstatt (Stuttgart Stuttgart)
	ANTWERPENER_STRASSE = 'de:08111:35'
	ANTWERPENER_STRASSE_1 = 'de:08111:35:1:1'  # Rtg Nürnberger Str. (Stuttgart Stuttgart)
	ANTWERPENER_STRASSE_2 = 'de:08111:35:1:2'  # Rtg Beskidenstr. (Stuttgart Stuttgart)
	ANTWERPENER_STRASSE_3 = 'de:08111:35:2:3'  # NBus (Stuttgart Stuttgart)
	BESKIDENSTRASSE = 'de:08111:36'
	BESKIDENSTRASSE_1 = 'de:08111:36:1:1'  # Rtg Antwerpener Str. (Stuttgart Stuttgart)
	BESKIDENSTRASSE_2 = 'de:08111:36:2:2'  # Rtg Fellbach Höhenstr. (Stuttgart Stuttgart)
	BESKIDENSTRASSE_3 = 'de:08111:36:3:3'  # NBus (Stuttgart Stuttgart)
	FELLBACH_HÖHENSTRASSE = 'de:08119:37'
	FELLBACH_HÖHENSTRASSE_1 = 'de:08119:37:0:1'  # Rtg Beskidenstr. (Fellbach Fellbach)
	FELLBACH_HÖHENSTRASSE_2 = 'de:08119:37:0:2'  # Rtg Esslinger Str. (Fellbach Fellbach)
	FELLBACH_HÖHENSTRASSE_3 = 'de:08119:37:0:3'  # Rtg Fellbach Beskidenstr. (Fellbach Fellbach)
	HÖHENSTRASSE = 'de:08119:37'
	HÖHENSTRASSE_1 = 'de:08119:37:0:1'  # Rtg Beskidenstr. (Fellbach Fellbach)
	HÖHENSTRASSE_2 = 'de:08119:37:0:2'  # Rtg Esslinger Str. (Fellbach Fellbach)
	HÖHENSTRASSE_3 = 'de:08119:37:0:3'  # Rtg Fellbach Beskidenstr. (Fellbach Fellbach)
	ESSLINGER_STRASSE_1 = 'de:08119:38:1:1'  # Rtg Höhenstr. (Fellbach Fellbach)
	ESSLINGER_STRASSE_2 = 'de:08119:38:1:2'  # Rtg Schwabenlandhalle (Fellbach Fellbach)
	ESSLINGER_STRASSE_3 = 'de:08119:38:2:3'  # Rtg Schwabenlandhalle/Kelter (Fellbach Fellbach)
	ESSLINGER_STRASSE_4 = 'de:08119:38:2:4'  # Rtg Postdörfle (Fellbach Fellbach)
	ESSLINGER_STRASSE_5 = 'de:08116:2918:0:3'  # Rtg Löwen (Wolfschlugen Wolfschlugen)
	ESSLINGER_STRASSE_6 = 'de:08116:2918:0:4'  # Rtg Sielmingen/Neuhausen (Wolfschlugen Wolfschlugen)
	ESSLINGER_STRASSE_7 = 'de:08116:4054:0:3'  # Rtg Esslingen (Deizisau Deizisau)
	ESSLINGER_STRASSE_8 = 'de:08116:4054:0:4'  # Rtg Deizisau (Deizisau Deizisau)
	ESSLINGER_STRASSE_9 = 'de:08116:5007:0:3'  # 119 Rtg Weingartenstr. (Denkendorf Denkendorf)
	ESSLINGER_STRASSE_10 = 'de:08116:5007:0:4'  # Rtg Nellingen (Denkendorf Denkendorf)
	ESSLINGER_STRASSE_11 = 'de:08116:5007:0:5'  # 121 Rtg Weingartstr. (Denkendorf Denkendorf)
	ESSLINGER_STRASSE_12 = 'de:08116:5822:0:3'  # Rtg Plochingen Bf/ZOB (Plochingen Plochingen)
	ESSLINGER_STRASSE_13 = 'de:08116:5822:0:4'  # eRtg Pfostenberg (Plochingen Plochingen)
	FELLBACH_ESSLINGER_STRASSE = 'de:08119:38'
	FELLBACH_ESSLINGER_STRASSE_1 = 'de:08119:38:1:1'  # Rtg Höhenstr. (Fellbach Fellbach)
	FELLBACH_ESSLINGER_STRASSE_2 = 'de:08119:38:1:2'  # Rtg Schwabenlandhalle (Fellbach Fellbach)
	FELLBACH_ESSLINGER_STRASSE_3 = 'de:08119:38:2:3'  # Rtg Schwabenlandhalle/Kelter (Fellbach Fellbach)
	FELLBACH_ESSLINGER_STRASSE_4 = 'de:08119:38:2:4'  # Rtg Postdörfle (Fellbach Fellbach)
	FELLBACH_SCHWABENLANDHALLE = 'de:08119:39'
	FELLBACH_SCHWABENLANDHALLE_1 = 'de:08119:39:1:1'  # Rtg Esslinger Str. (Fellbach Fellbach)
	FELLBACH_SCHWABENLANDHALLE_2 = 'de:08119:39:2:2'  # Rtg Lutherkirche (Fellbach Fellbach)
	FELLBACH_SCHWABENLANDHALLE_3 = 'de:08119:39:3:5'  # Rtg Lutherkirche (Fellbach Fellbach)
	FELLBACH_SCHWABENLANDHALLE_4 = 'de:08119:39:3:6'  # Rtg Kienbachstr. (Fellbach Fellbach)
	SCHWABENLANDHALLE = 'de:08119:39'
	SCHWABENLANDHALLE_1 = 'de:08119:39:1:1'  # Rtg Esslinger Str. (Fellbach Fellbach)
	SCHWABENLANDHALLE_2 = 'de:08119:39:2:2'  # Rtg Lutherkirche (Fellbach Fellbach)
	SCHWABENLANDHALLE_3 = 'de:08119:39:3:5'  # Rtg Lutherkirche (Fellbach Fellbach)
	SCHWABENLANDHALLE_4 = 'de:08119:39:3:6'  # Rtg Kienbachstr. (Fellbach Fellbach)
	HESLACH_VOGELRAIN = 'de:08111:43'
	HESLACH_VOGELRAIN_1 = 'de:08111:43:0:1'  # Rtg Waldeck (Stuttgart Stuttgart)
	HESLACH_VOGELRAIN_2 = 'de:08111:43:0:2'  # Rtg Südheimer Platz (Stuttgart Stuttgart)
	HESLACH_VOGELRAIN_3 = 'de:08111:43:0:4'  # Rtg Rudolf-Sophien-Stift (Stuttgart Stuttgart)
	HESLACH_VOGELRAIN_4 = 'de:08111:43:0:5'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	HESLACH_VOGELRAIN_5 = 'de:08111:43:0:7'  # U9 Endhst./Rtg Südheimer Platz (Stuttgart Stuttgart)
	BERLINER_PLATZ__HOHE_STRASSE = 'de:08111:55'
	BERLINER_PLATZ__HOHE_STRASSE_1 = 'de:08111:55:1:2'  # Rtg Börsenpl/Hölderlinpl/Schloss-/Joh-st (Stuttgart Stuttgart)
	BERLINER_PLATZ__HOHE_STRASSE_2 = 'de:08111:55:2:1'  # Rtg Rotebühlplatz (Stuttgart Stuttgart)
	BERLINER_PLATZ__HOHE_STRASSE_3 = 'de:08111:55:3:4'  # Lnie 43 Rtg Rotebühlplatz (Stuttgart Stuttgart)
	BERLINER_PLATZ__HOHE_STRASSE_4 = 'de:08111:55:4:5'  # NBus (Stuttgart Stuttgart)
	BÜCHSENSTRASSE = 'de:08111:69:0:3'  # Rtg Kleiner Schlossplatz (Stuttgart Stuttgart)
	KLEINER_SCHLOSSPLATZ = 'de:08111:70:0:3'  # Rtg Hbf (Stuttgart Stuttgart)
	KARL_OLGA_KRANKENHAUS = 'de:08111:76'
	KARL_OLGA_KRANKENHAUS_1 = 'de:08111:76:1:1'  # Rtg Stöckach (Stuttgart Stuttgart)
	KARL_OLGA_KRANKENHAUS_2 = 'de:08111:76:1:2'  # Rtg Bergfriedhof (Stuttgart Stuttgart)
	KARL_OLGA_KRANKENHAUS_3 = 'de:08111:76:1:4'  # NBus (Stuttgart Stuttgart)
	BERGFRIEDHOF = 'de:08111:77'
	BERGFRIEDHOF_1 = 'de:08111:77:0:1'  # Rtg Karl-Olga-Krankenhaus (Stuttgart Stuttgart)
	BERGFRIEDHOF_2 = 'de:08111:77:0:2'  # Rtg Raitelsberg/Ostendplatz (Stuttgart Stuttgart)
	BERGFRIEDHOF_3 = 'de:08111:77:0:4'  # NBus (Stuttgart Stuttgart)
	OSTHEIM_LEO_VETTER_BAD = 'de:08111:78'
	OSTHEIM_LEO_VETTER_BAD_1 = 'de:08111:78:1:1'  # Rtg Ostendplatz (Stuttgart Stuttgart)
	OSTHEIM_LEO_VETTER_BAD_2 = 'de:08111:78:1:2'  # Rtg Gaisburg (Stuttgart Stuttgart)
	OSTHEIM_LEO_VETTER_BAD_3 = 'de:08111:78:2:4'  # Linie 45 Rtg Tal-/Landhausstr. (Stuttgart Stuttgart)
	GAISBURG = 'de:08111:79'
	GAISBURG_1 = 'de:08111:79:0:1'  # Rtg Leo-Vetter-Bad (Stuttgart Stuttgart)
	GAISBURG_2 = 'de:08111:79:0:2'  # Rtg Wangener-/Landhausstr. (Stuttgart Stuttgart)
	WANGENER__LANDHAUSSTRASSE = 'de:08111:80'
	WANGENER__LANDHAUSSTRASSE_1 = 'de:08111:80:1:1'  # U9 Rtg Schlachthof (Stuttgart Stuttgart)
	WANGENER__LANDHAUSSTRASSE_2 = 'de:08111:80:1:2'  # U9 Rtg Brendle (Großmarkt) (Stuttgart Stuttgart)
	WANGENER__LANDHAUSSTRASSE_3 = 'de:08111:80:1:3'  # NBus Rtg einwärts (Stuttgart Stuttgart)
	WANGENER__LANDHAUSSTRASSE_4 = 'de:08111:80:1:4'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	WANGENER__LANDHAUSSTRASSE_5 = 'de:08111:80:2:7'  # U4 Rtg Gaisburg (Stuttgart Stuttgart)
	WANGENER__LANDHAUSSTRASSE_6 = 'de:08111:80:2:8'  # U4 Rtg Brendle (Großmarkt) (Stuttgart Stuttgart)
	BRENDLE__GROSSMARKT = 'de:08111:81'
	BRENDLE__GROSSMARKT_1 = 'de:08111:81:1:1'  # Rtg Wangener-/Landhausstr. (Stuttgart Stuttgart)
	BRENDLE__GROSSMARKT_2 = 'de:08111:81:2:2'  # Rtg Im Degen (Stuttgart Stuttgart)
	BRENDLE__GROSSMARKT_3 = 'de:08111:81:2:4'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	BRENDLE__GROSSMARKT_4 = 'de:08111:81:3:3'  # NBus Rtg einwärts (Stuttgart Stuttgart)
	IM_DEGEN = 'de:08111:82'
	IM_DEGEN_1 = 'de:08111:82:0:1'  # Rtg Brendle (Großmarkt) (Stuttgart Stuttgart)
	IM_DEGEN_2 = 'de:08111:82:0:2'  # Rtg Inselstr. (Stuttgart Stuttgart)
	IM_DEGEN_3 = 'de:08111:82:0:3'  # NBus Rtg einwärts (Stuttgart Stuttgart)
	IM_DEGEN_4 = 'de:08111:82:0:4'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	INSELSTRASSE = 'de:08111:83'
	INSELSTRASSE_1 = 'de:08111:83:0:1'  # Rtg Im Degen (Stuttgart Stuttgart)
	INSELSTRASSE_2 = 'de:08111:83:0:2'  # Rtg Wasenstr. (Stuttgart Stuttgart)
	INSELSTRASSE_3 = 'de:08111:83:0:3'  # NBus Rtg einwärts (Stuttgart Stuttgart)
	WASENSTRASSE_1 = 'de:08111:84:1:7'  # U13 Rtg Untertürkheim Bf (Stuttgart Stuttgart)
	WASENSTRASSE_2 = 'de:08111:84:1:8'  # U9/U13 Rtg Wangen Marktplatz (Stuttgart Stuttgart)
	WASENSTRASSE_3 = 'de:08111:84:2:1'  # U4/U9 Rtg Inselstr. (Stuttgart Stuttgart)
	WASENSTRASSE_4 = 'de:08111:84:2:2'  # U4 Rtg Untertürkheim Bf (Stuttgart Stuttgart)
	WASENSTRASSE_5 = 'de:08118:4889:1:3'  # eRtg Stadtbahn/Rathaus/Waiblingen (Remseck am Neckar Neckargröningen)
	WASENSTRASSE_6 = 'de:08118:4889:1:4'  # eRtg Lamm (Remseck am Neckar Neckargröningen)
	WASENSTRASSE_7 = 'de:08119:5226:0:3'  # West (Urbach Urbach)
	WASENSTRASSE_8 = 'de:08119:5226:0:4'  # Ost (Urbach Urbach)
	WASENSTRASSE_9 = 'de:08117:2154:0:H1'  # Rtg Falkenstraße (Göppingen Jebenhausen)
	WASENSTRASSE_10 = 'de:08117:2154:0:R1'  # Rtg Vorderer Berg (Göppingen Jebenhausen)
	GROSSGLOCKNERSTRASSE = 'de:08111:86'
	GROSSGLOCKNERSTRASSE_1 = 'de:08111:86:1:3'  # L.61 Rtg Untertürkheim Bf (Stuttgart Stuttgart)
	GROSSGLOCKNERSTRASSE_2 = 'de:08111:86:1:4'  # L.60/61 Rtg Mönchw./L.61 Rtg Neckarhalde (Stuttgart Stuttgart)
	NECKARHALDE_1 = 'de:08111:87:0:3'  # Rtg Haus am Weinberg (Stuttgart Stuttgart)
	NECKARHALDE_2 = 'de:08111:87:0:4'  # Rtg Großglocknerstr. (Stuttgart Stuttgart)
	NECKARHALDE_3 = 'de:08118:4885:0:3'  # eRtg Remstalstraße/Hochberg (Remseck am Neckar Neckarrems)
	NECKARHALDE_4 = 'de:08118:4885:0:4'  # eRtg Rathaus (Remseck am Neckar Neckarrems)
	EBNISEESTRASSE_1 = 'de:08111:89:0:3'  # Rtg Bergstaffelstr. (Stuttgart Stuttgart)
	EBNISEESTRASSE_2 = 'de:08111:89:0:4'  # Rtg Haus am Weinberg (Stuttgart Stuttgart)
	EBNISEESTRASSE_3 = 'de:08119:3729:0:3'  # eRtg Rathaus (Althütte Althütte)
	EBNISEESTRASSE_4 = 'de:08119:3729:0:4'  # eRtg Voggenhof Abzweig (Althütte Althütte)
	EBNISEESTRASSE_5 = 'de:08119:4988:0:3'  # eRtg Bruch (Weissach im Tal Oberweissach)
	EBNISEESTRASSE_6 = 'de:08119:4988:0:4'  # eRtg Weissachstraße (Weissach im Tal Oberweissach)
	BERGSTAFFELSTRASSE = 'de:08111:90'
	BERGSTAFFELSTRASSE_1 = 'de:08111:90:0:3'  # Rtg Obertürkheim Bf (Stuttgart Stuttgart)
	BERGSTAFFELSTRASSE_2 = 'de:08111:90:0:4'  # Rtg Ebniseestr. (Stuttgart Stuttgart)
	WOLLINSTRASSE = 'de:08111:94'
	WOLLINSTRASSE_1 = 'de:08111:94:0:3'  # Rtg Neuwirtshauskreuzung (Stuttgart Stuttgart)
	WOLLINSTRASSE_2 = 'de:08111:94:0:4'  # Rtg Zuff.Porsche/Nordsee-/Schwbd.Str. (Stuttgart Stuttgart)
	NEUWIRTSHAUSKREUZUNG = 'de:08111:95'
	NEUWIRTSHAUSKREUZUNG_1 = 'de:08111:95:1:3'  # Rtg Niebuhrweg (Stuttgart Stuttgart)
	NEUWIRTSHAUSKREUZUNG_2 = 'de:08111:95:1:4'  # Rtg Wollinstr. (Stuttgart Stuttgart)
	NIEBUHRWEG = 'de:08111:96'
	NIEBUHRWEG_1 = 'de:08111:96:0:3'  # Rtg Marco-Polo-Weg (Stuttgart Stuttgart)
	NIEBUHRWEG_2 = 'de:08111:96:0:4'  # Rtg Neuwirtshauskreuzung (Stuttgart Stuttgart)
	MARCO_POLO_WEG = 'de:08111:97'
	MARCO_POLO_WEG_1 = 'de:08111:97:0:3'  # Rtg Korntaler Straße (Stuttgart Stuttgart)
	MARCO_POLO_WEG_2 = 'de:08111:97:0:4'  # Rtg Niebuhrweg (Stuttgart Stuttgart)
	STAMMHEIM = 'de:08111:100'
	STAMMHEIM_1 = 'de:08111:100:1:1'  # Stadtbahn Endhaltestelle (Stuttgart Stuttgart)
	STAMMHEIM_2 = 'de:08111:100:2:3'  # L.412 Rtg Sonatenweg/L.508 Rtg Kornt.Str (Stuttgart Stuttgart)
	STAMMHEIM_3 = 'de:08111:100:2:4'  # Linien 412/508 Rtg Asperger Str. (Stuttgart Stuttgart)
	STAMMHEIM_4 = 'de:08111:100:2:5'  # Li 98 Rtg Asperger Str. (Stuttgart Stuttgart)
	KORNTALER_STRASSE_1 = 'de:08111:102:1:1'  # Rtg Heutingsheimer Str. (Stuttgart Stuttgart)
	KORNTALER_STRASSE_2 = 'de:08111:102:2:2'  # Rtg Stammheim (Stuttgart Stuttgart)
	KORNTALER_STRASSE_3 = 'de:08111:102:3:2'  # L.52/99/N4 Rtg M.-Polo-W./L.508 Rtg Sthm (Stuttgart Stuttgart)
	KORNTALER_STRASSE_4 = 'de:08111:102:3:3'  # Linie 508 Rtg Salzwiesenstr. (Stuttgart Stuttgart)
	KORNTALER_STRASSE_5 = 'de:08115:2366:0:3'  # Rtg Marbacher Str. (Leonberg Leonberg)
	HEUTINGSHEIMER_STRASSE = 'de:08111:103'
	HEUTINGSHEIMER_STRASSE_1 = 'de:08111:103:0:1'  # Rtg Wimpfener Str. (Stuttgart Stuttgart)
	HEUTINGSHEIMER_STRASSE_2 = 'de:08111:103:0:2'  # Rtg Korntaler Str. (Stuttgart Stuttgart)
	HEUTINGSHEIMER_STRASSE_3 = 'de:08111:103:0:4'  # NBus (Stuttgart Stuttgart)
	WIMPFENER_STRASSE = 'de:08111:104'
	WIMPFENER_STRASSE_1 = 'de:08111:104:0:1'  # Rtg Salzwiesenstr. (Stuttgart Stuttgart)
	WIMPFENER_STRASSE_2 = 'de:08111:104:0:2'  # Rtg Heutingsheimer Str. (Stuttgart Stuttgart)
	WIMPFENER_STRASSE_3 = 'de:08111:104:0:5'  # Rtg Stammheim (Stuttgart Stuttgart)
	KORNWESTHEIMER_STRASSE_1 = 'de:08111:105:0:3'  # eRtg Sonatenweg (Stuttgart Stuttgart)
	KORNWESTHEIMER_STRASSE_2 = 'de:08111:105:0:4'  # eRtg Max-Planck-Str. (Stuttgart Stuttgart)
	KORNWESTHEIMER_STRASSE_3 = 'de:08118:4519:0:3'  # eRtg Sportzentrum (Korntal-Münchingen Münchingen)
	KORNWESTHEIMER_STRASSE_4 = 'de:08118:4519:0:4'  # eRtg Müllerheim (Korntal-Münchingen Münchingen)
	ZAHN_NOPPER_STRASSE = 'de:08111:106'
	ZAHN_NOPPER_STRASSE_1 = 'de:08111:106:1:6'  # Linie 508 Rtg Zuffenhausen Bf (Stuttgart Stuttgart)
	ZAHN_NOPPER_STRASSE_2 = 'de:08111:106:2:7'  # Linie 508 Rtg Salzwiesenstr. (Stuttgart Stuttgart)
	ZAHN_NOPPER_STRASSE_3 = 'de:08111:106:3:3'  # Linie 99 Rtg Marconibrücke (Stuttgart Stuttgart)
	ZAHN_NOPPER_STRASSE_4 = 'de:08111:106:4:4'  # Linie 99 Rtg Zuffenhausen Bf (Stuttgart Stuttgart)
	ZAHN_NOPPER_STRASSE_5 = 'de:08111:106:5:5'  # Nachtbus (Stuttgart Stuttgart)
	KIRCHTALSTRASSE_1 = 'de:08111:107:1:1'  # Rtg Zuffenhausen Rathaus (Stuttgart Stuttgart)
	KIRCHTALSTRASSE_2 = 'de:08111:107:1:2'  # Rtg Salzwiesenstr. (Stuttgart Stuttgart)
	KIRCHTALSTRASSE_3 = 'de:08111:107:2:3'  # Linie 401 Rtg Zuffenhausen Rathaus (Stuttgart Stuttgart)
	KIRCHTALSTRASSE_4 = 'de:08111:107:2:4'  # Linie 401 Rtg Zuffenhausen Bf / NBus (Stuttgart Stuttgart)
	KIRCHTALSTRASSE_5 = 'de:08118:5448:0:3'  # Rtg Im Wiesengrund (Kornwestheim Kornwestheim)
	ZUFFENHAUSEN_RATHAUS = 'de:08111:108'
	ZUFFENHAUSEN_RATHAUS_1 = 'de:08111:108:1:1'  # Rtg Hohensteinstr. (Stuttgart Stuttgart)
	ZUFFENHAUSEN_RATHAUS_2 = 'de:08111:108:1:2'  # Rtg Kirchtalstr. (Stuttgart Stuttgart)
	ZUFFENHAUSEN_RATHAUS_3 = 'de:08111:108:2:4'  # NBus (Stuttgart Stuttgart)
	ZUFFENHAUSEN_RATHAUS_4 = 'de:08111:108:2:5'  # Linie 401 Rtg Friedhof (Stuttgart Stuttgart)
	ZUFFENHAUSEN_FRIEDHOF = 'de:08111:109'
	ZUFFENHAUSEN_FRIEDHOF_1 = 'de:08111:109:0:3'  # eRtg Johanneskirche (Stuttgart Stuttgart)
	ZUFFENHAUSEN_FRIEDHOF_2 = 'de:08111:109:0:4'  # eRtg Zuffenhausen Reibedanz (Stuttgart Stuttgart)
	FRIEDRICHSWAHL = 'de:08111:110'
	FRIEDRICHSWAHL_1 = 'de:08111:110:0:1'  # Rtg Borsigstr. (Stuttgart Stuttgart)
	FRIEDRICHSWAHL_2 = 'de:08111:110:0:2'  # Rtg Hohensteinstr. (Stuttgart Stuttgart)
	FRIEDRICHSWAHL_3 = 'de:08111:110:0:3'  # NBus (Stuttgart Stuttgart)
	BORSIGSTRASSE_1 = 'de:08111:111:0:1'  # Rtg Sieglestr. (Stuttgart Stuttgart)
	BORSIGSTRASSE_2 = 'de:08111:111:0:2'  # Rtg Friedrichswahl (Stuttgart Stuttgart)
	BORSIGSTRASSE_3 = 'de:08111:111:0:3'  # NBus Rtg einwärts (Stuttgart Stuttgart)
	BORSIGSTRASSE_4 = 'de:08118:7474:0:3'  # Rtg ZOB (Bietigheim-Bissingen Bissingen (LB))
	SIEGLESTRASSE = 'de:08111:112'
	SIEGLESTRASSE_1 = 'de:08111:112:0:1'  # Rtg Pragsattel (Stuttgart Stuttgart)
	SIEGLESTRASSE_2 = 'de:08111:112:0:2'  # Rtg Borsigstr. (Stuttgart Stuttgart)
	SIEGLESTRASSE_3 = 'de:08111:112:0:3'  # NBus (Stuttgart Stuttgart)
	SONATENWEG = 'de:08111:114'
	SONATENWEG_1 = 'de:08111:114:0:3'  # eRtg Stammheim/Friedhof (Stuttgart Stuttgart)
	SONATENWEG_2 = 'de:08111:114:0:4'  # eRtg Lindenstr. (Stuttgart Stuttgart)
	PRAGFRIEDHOF = 'de:08111:115'
	PRAGFRIEDHOF_1 = 'de:08111:115:1:1'  # Rtg Stadtbibliothek (Stuttgart Stuttgart)
	PRAGFRIEDHOF_2 = 'de:08111:115:1:2'  # Rtg Löwentorbrücke (Stuttgart Stuttgart)
	STAFFLENBERGSTRASSE = 'de:08111:123'
	STAFFLENBERGSTRASSE_1 = 'de:08111:123:0:1'  # Rtg Bubenbad (Stuttgart Stuttgart)
	STAFFLENBERGSTRASSE_2 = 'de:08111:123:0:2'  # Rtg Heidehofstraße (Stuttgart Stuttgart)
	STAFFLENBERGSTRASSE_3 = 'de:08111:123:0:3'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	BUBENBAD = 'de:08111:124'
	BUBENBAD_1 = 'de:08111:124:0:1'  # Rtg Payerstraße (Stuttgart Stuttgart)
	BUBENBAD_2 = 'de:08111:124:0:2'  # Rtg Stafflenbergstraße (Stuttgart Stuttgart)
	BUBENBAD_3 = 'de:08111:124:0:3'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	PAYERSTRASSE = 'de:08111:125'
	PAYERSTRASSE_1 = 'de:08111:125:0:1'  # Rtg Geroksruhe (Stuttgart Stuttgart)
	PAYERSTRASSE_2 = 'de:08111:125:0:2'  # Rtg Bubenbad (Stuttgart Stuttgart)
	PAYERSTRASSE_3 = 'de:08111:125:0:3'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	PAYERSTRASSE_4 = 'de:08111:125:0:5'  # Nachtbus (Stuttgart Stuttgart)
	GEROKSRUHE = 'de:08111:126'
	GEROKSRUHE_1 = 'de:08111:126:1:1'  # Rtg Stelle (Stuttgart Stuttgart)
	GEROKSRUHE_2 = 'de:08111:126:1:2'  # Rtg Payerstr. (Stuttgart Stuttgart)
	GEROKSRUHE_3 = 'de:08111:126:1:3'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	SILBERWALD = 'de:08111:129'
	SILBERWALD_1 = 'de:08111:129:1:1'  # Rtg Sillenbuch (Stuttgart Stuttgart)
	SILBERWALD_2 = 'de:08111:129:1:2'  # Rtg Ruhbank (Stuttgart Stuttgart)
	SILBERWALD_3 = 'de:08111:129:2:3'  # NBus (Stuttgart Stuttgart)
	WALDAU = 'de:08111:130'
	WALDAU_1 = 'de:08111:130:1:1'  # Rtg Ruhbank (Stuttgart Stuttgart)
	WALDAU_2 = 'de:08111:130:1:2'  # Rtg Bopser/Weinsteige (Stuttgart Stuttgart)
	HEUMADEN_1 = 'de:08111:136:1:1'  # Rtg Heumaden Bockelstr. (Stuttgart Stuttgart)
	HEUMADEN_2 = 'de:08111:136:1:2'  # Rtg Ruit (Stuttgart Stuttgart)
	HEUMADEN_3 = 'de:08111:136:1:3'  # U15 Rtg Heumaden Bockelstr. (Stuttgart Stuttgart)
	HEUMADEN_4 = 'de:08111:136:2:3'  # Rtg Kemnat Schönbergstr./Zeppelinstr. (Stuttgart Stuttgart)
	HEUMADEN_5 = 'de:08111:136:2:4'  # Rtg G.-Scholl-Gymn./Ruit Horbstr. (Stuttgart Stuttgart)
	HEUMADEN_6 = 'de:08119:6671:0:3'  # eRtg Eulenhof (Kaisersbach Kaisersbach)
	HEUMADEN_7 = 'de:08119:6671:0:4'  # eRtg Hauptstraße (Kaisersbach Kaisersbach)
	ISEGRIMWEG = 'de:08111:137'
	ISEGRIMWEG_1 = 'de:08111:137:0:3'  # Rtg Heumaden Bockelstr. (Stuttgart Stuttgart)
	ISEGRIMWEG_2 = 'de:08111:137:0:4'  # Rtg Heumaden Schule (Stuttgart Stuttgart)
	GERLINGEN_SIEDLUNG = 'de:08118:141'
	GERLINGEN_SIEDLUNG_1 = 'de:08118:141:0:1'  # Rtg Breitwiesen (Gerlingen Gerlingen)
	GERLINGEN_SIEDLUNG_2 = 'de:08118:141:0:2'  # Rtg Gerlingen (Gerlingen Gerlingen)
	GERLINGEN_SIEDLUNG_3 = 'de:08118:141:3:4'  # Rtg Gerlingen Breitweisenhaus (Gerlingen Gerlingen)
	SIEDLUNG_1 = 'de:08118:141:0:1'  # Rtg Breitwiesen (Gerlingen Gerlingen)
	SIEDLUNG_2 = 'de:08118:141:0:2'  # Rtg Gerlingen (Gerlingen Gerlingen)
	SIEDLUNG_3 = 'de:08118:141:3:4'  # Rtg Gerlingen Breitweisenhaus (Gerlingen Gerlingen)
	SIEDLUNG_4 = 'de:08116:2003:0:3'  # Rtg Ort (Aichtal Aich)
	SIEDLUNG_5 = 'de:08116:2003:0:4'  # Rtg Riedwiesen (Aichtal Aich)
	SIEDLUNG_6 = 'de:08116:2921:1:3'  # Rtg Hardt (Wolfschlugen Wolfschlugen)
	SIEDLUNG_7 = 'de:08116:2921:1:4'  # Rtg Reinhardtstr. (Wolfschlugen Wolfschlugen)
	SIEDLUNG_8 = 'de:08116:4162:0:3'  # Rtg Krummhardt (Aichwald Schanbach)
	SIEDLUNG_9 = 'de:08116:4162:0:4'  # Rtg Kindergarten/Kreuzung (Aichwald Schanbach)
	SIEDLUNG_10 = 'de:08116:4392:0:4'  # eRtg Zeller Straße (Ohmden Ohmden)
	SIEDLUNG_11 = 'de:08117:91:0:H1'  # Rtg Unterböhringen (Bad Überkingen Hausen (Bad Überkingen))
	SIEDLUNG_12 = 'de:08117:91:0:R1'  # Rtg Hirsch (Bad Überkingen Hausen (Bad Überkingen))
	SIEDLUNG_13 = 'de:08117:318:0:H1'  # Rtg Hirsch (Böhmenkirch Schnittlingen)
	SIEDLUNG_14 = 'de:08117:318:0:R1'  # Rtg Donzdorf (Böhmenkirch Schnittlingen)
	SIEDLUNG_15 = 'de:08117:6014:0:H1'  # Rtg Gasthaus Krone (Ottenbach Ottenbach)
	SIEDLUNG_16 = 'de:08117:6014:0:R1'  # Rtg Breitfelderhof (Ottenbach Ottenbach)
	SIEDLUNG_17 = 'de:08117:8005:0:H1'  # Rtg Seeweg (Ebersbach an der Fils Bünzwangen)
	SIEDLUNG_18 = 'de:08117:8005:0:R1'  # Rtg Silcherstr. (Ebersbach an der Fils Bünzwangen)
	SIEDLUNG_19 = 'de:08135:1156:0:1'  # Rtg Dorfplatz (Gerstetten Gussenstadt)
	SIEDLUNG_20 = 'de:08135:1156:0:2'  # Rtg Gerstetten (Gerstetten Gussenstadt)
	ROSSBAUM = 'de:08118:142:0:4'  # NBus (Gerlingen Gerlingen)
	GERLINGEN_ROSSBAUM = 'de:08118:142:0:4'  # NBus (Gerlingen Gerlingen)
	BREITWIESEN = 'de:08118:143'
	BREITWIESEN_1 = 'de:08118:143:1:1'  # Rtg Giebel (Gerlingen Gerlingen)
	BREITWIESEN_2 = 'de:08118:143:1:2'  # Rtg Siedlung (Gerlingen Gerlingen)
	BREITWIESEN_3 = 'de:08118:143:2:1'  # Rtg Margartenweg (Gerlingen Gerlingen)
	BREITWIESEN_4 = 'de:08118:143:2:4'  # Rtg Rossbaum (Gerlingen Gerlingen)
	GERLINGEN_BREITWIESEN = 'de:08118:143'
	GERLINGEN_BREITWIESEN_1 = 'de:08118:143:1:1'  # Rtg Giebel (Gerlingen Gerlingen)
	GERLINGEN_BREITWIESEN_2 = 'de:08118:143:1:2'  # Rtg Siedlung (Gerlingen Gerlingen)
	GERLINGEN_BREITWIESEN_3 = 'de:08118:143:2:1'  # Rtg Margartenweg (Gerlingen Gerlingen)
	GERLINGEN_BREITWIESEN_4 = 'de:08118:143:2:4'  # Rtg Rossbaum (Gerlingen Gerlingen)
	GIEBEL = 'de:08111:144'
	GIEBEL_1 = 'de:08111:144:1:1'  # Rtg Salamanderweg (Stuttgart Stuttgart)
	GIEBEL_2 = 'de:08111:144:1:2'  # Rtg Breitwiesen (Stuttgart Stuttgart)
	GIEBEL_3 = 'de:08111:144:2:7'  # Stadtbahn - U13 (Stuttgart Stuttgart)
	GIEBEL_4 = 'de:08111:144:3:3'  # Ankunft und Rtg E.-Reuter-Platz (Stuttgart Stuttgart)
	GIEBEL_5 = 'de:08111:144:3:4'  # NBus Rtg Gerlingen (Stuttgart Stuttgart)
	SALAMANDERWEG = 'de:08111:145'
	SALAMANDERWEG_1 = 'de:08111:145:1:1'  # Rtg Bergheimer Hof (Stuttgart Stuttgart)
	SALAMANDERWEG_2 = 'de:08111:145:1:2'  # Rtg Giebel (Stuttgart Stuttgart)
	SALAMANDERWEG_3 = 'de:08111:145:2:4'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	BERGHEIMER_HOF = 'de:08111:146'
	BERGHEIMER_HOF_1 = 'de:08111:146:1:1'  # Rtg Wolfbusch (Stuttgart Stuttgart)
	BERGHEIMER_HOF_2 = 'de:08111:146:1:2'  # Rtg Salamanderweg (Stuttgart Stuttgart)
	BERGHEIMER_HOF_3 = 'de:08111:146:2:4'  # NBus (Stuttgart Stuttgart)
	WOLFBUSCH = 'de:08111:147'
	WOLFBUSCH_1 = 'de:08111:147:1:1'  # Rtg Rastatter Str. (Stuttgart Stuttgart)
	WOLFBUSCH_2 = 'de:08111:147:1:2'  # Rtg Bergheimer Hof (Stuttgart Stuttgart)
	WOLFBUSCH_3 = 'de:08111:147:2:4'  # NBus (Stuttgart Stuttgart)
	RASTATTER_STRASSE = 'de:08111:148'
	RASTATTER_STRASSE_1 = 'de:08111:148:1:1'  # Rtg Löwenmarkt (Stuttgart Stuttgart)
	RASTATTER_STRASSE_2 = 'de:08111:148:1:2'  # Rtg Wolfbusch (Stuttgart Stuttgart)
	RASTATTER_STRASSE_3 = 'de:08111:148:2:4'  # NBus (Stuttgart Stuttgart)
	GERLINGEN_GOETHESTRASSE = 'de:08118:149'
	GERLINGEN_GOETHESTRASSE_1 = 'de:08118:149:0:3'  # Rtg Weilimdorfer Str. (Gerlingen Gerlingen)
	GERLINGEN_GOETHESTRASSE_2 = 'de:08118:149:0:4'  # Rtg Gerlingen Stadtbahn (Gerlingen Gerlingen)
	GOETHESTRASSE_1 = 'de:08118:149:0:3'  # Rtg Weilimdorfer Str. (Gerlingen Gerlingen)
	GOETHESTRASSE_2 = 'de:08118:149:0:4'  # Rtg Gerlingen Stadtbahn (Gerlingen Gerlingen)
	GOETHESTRASSE_3 = 'de:08116:3826:0:3'  # Rtg Kirchheimer Str. (Wendlingen am Neckar Wendlingen (N))
	GOETHESTRASSE_4 = 'de:08115:4635:0:3'  # eRtg Schützenweg (Böblingen Dagersheim)
	GOETHESTRASSE_5 = 'de:08116:6703:0:3'  # eRtg Metzinger Straße (Kohlberg Kohlberg)
	GOETHESTRASSE_6 = 'de:08118:6829:0:3'  # eRtg Mühlweg (Remseck am Neckar Aldingen)
	GOETHESTRASSE_7 = 'de:08118:6829:0:4'  # eRtg Berliner Straße (Remseck am Neckar Aldingen)
	GOETHESTRASSE_8 = 'de:08115:7019:0:3'  # Rtg In der Röte (Mötzingen Mötzingen)
	MATHILDENSTRASSE = 'de:08111:150:0:4'  # NBus (Stuttgart Stuttgart)
	LANDAUER_STRASSE = 'de:08111:151'
	LANDAUER_STRASSE_1 = 'de:08111:151:1:1'  # Rtg Pfostenwäldle (Stuttgart Stuttgart)
	LANDAUER_STRASSE_2 = 'de:08111:151:1:2'  # Rtg Löwenmarkt (Stuttgart Stuttgart)
	LANDAUER_STRASSE_3 = 'de:08111:151:2:4'  # NBus (Stuttgart Stuttgart)
	FEUERBACH_PFOSTENWÄLDLE = 'de:08111:152'
	FEUERBACH_PFOSTENWÄLDLE_1 = 'de:08111:152:1:1'  # Rtg Sportpark (Stuttgart Stuttgart)
	FEUERBACH_PFOSTENWÄLDLE_2 = 'de:08111:152:2:2'  # Rtg Landauer Str. (Stuttgart Stuttgart)
	SPORTPARK_FEUERBACH = 'de:08111:153'
	SPORTPARK_FEUERBACH_1 = 'de:08111:153:1:1'  # Rtg Föhrich (Stuttgart Stuttgart)
	SPORTPARK_FEUERBACH_2 = 'de:08111:153:1:2'  # Rtg Pfostenwäldle (Stuttgart Stuttgart)
	SPORTPARK_FEUERBACH_3 = 'de:08111:153:3:4'  # NBus auswärts (Stuttgart Stuttgart)
	SPORTPARK_FEUERBACH_4 = 'de:08111:153:3:5'  # NBus einwärts (Stuttgart Stuttgart)
	FÖHRICH = 'de:08111:154'
	FÖHRICH_1 = 'de:08111:154:1:1'  # Rtg Wilhelm-Geiger-Pl. (Stuttgart Stuttgart)
	FÖHRICH_2 = 'de:08111:154:2:2'  # Rtg Sportpark (Stuttgart Stuttgart)
	FÖHRICH_3 = 'de:08111:154:3:3'  # Rtg Hohewartstr. (Stuttgart Stuttgart)
	FÖHRICH_4 = 'de:08111:154:3:4'  # Rtg Wilhelm-Geiger-Pl. (Stuttgart Stuttgart)
	FÖHRICH_5 = 'de:08111:154:3:5'  # NBus Rtg Sportpark Feuerbach (Stuttgart Stuttgart)
	FÖHRICH_6 = 'de:08111:154:3:6'  # NBus Rtg Hohewartstraße (Stuttgart Stuttgart)
	LEOBENER_STRASSE = 'de:08111:156:0:3'  # NBus (Stuttgart Stuttgart)
	MAYBACHSTRASSE_1 = 'de:08111:158:1:1'  # Rtg Pragsattel (Stuttgart Stuttgart)
	MAYBACHSTRASSE_2 = 'de:08111:158:2:2'  # Rtg Feuerbach Bf (Stuttgart Stuttgart)
	MAYBACHSTRASSE_3 = 'de:08111:158:3:3'  # NBus Rtg Löwentorbrücke (Stuttgart Stuttgart)
	MAYBACHSTRASSE_4 = 'de:08111:158:4:4'  # NBus Rtg Feuerbach (Stuttgart Stuttgart)
	MAYBACHSTRASSE_5 = 'de:08115:3133:0:4'  # eRtg (Schönaich Schönaich)
	MAYBACHSTRASSE_6 = 'de:08118:5468:0:3'  # eRtg Villeneuvestraße (Kornwestheim Kornwestheim)
	MAYBACHSTRASSE_7 = 'de:08118:5468:0:4'  # eRtg Max-Eyth-Straße (Kornwestheim Kornwestheim)
	MAYBACHSTRASSE_8 = 'de:08116:5902:0:3'  # Rtg Frickenh. Bf (Frickenhausen Frickenhausen)
	MAYBACHSTRASSE_9 = 'de:08117:1204:0:H1'  # Rtg Grundschule (Rechberghausen Rechberghausen)
	DOBELSTRASSE = 'de:08111:159'
	DOBELSTRASSE_1 = 'de:08111:159:1:1'  # Rtg Bopser (Stuttgart Stuttgart)
	DOBELSTRASSE_2 = 'de:08111:159:1:2'  # Rtg Olgaeck (Stuttgart Stuttgart)
	NEUE_WEINSTEIGE_61 = 'de:08111:161'
	NEUE_WEINSTEIGE_61_1 = 'de:08111:161:0:3'  # Nachtbus Rtg Altenbergstaffel (Stuttgart Stuttgart)
	NEUE_WEINSTEIGE_61_2 = 'de:08111:161:0:4'  # Nachtbus Rtg Bopser (Stuttgart Stuttgart)
	ALTENBERGSTAFFEL = 'de:08111:162'
	ALTENBERGSTAFFEL_1 = 'de:08111:162:0:3'  # Nachtbus Rtg Weinsteige (Stuttgart Stuttgart)
	ALTENBERGSTAFFEL_2 = 'de:08111:162:0:4'  # Nachtbus Rtg Neue Weinsteige 61 (Stuttgart Stuttgart)
	WEINSTEIGE_1 = 'de:08111:163:1:1'  # Rtg Degerloch (Stuttgart Stuttgart)
	WEINSTEIGE_2 = 'de:08111:163:1:2'  # Rtg Bopser/Waldau (Stuttgart Stuttgart)
	WEINSTEIGE_3 = 'de:08111:163:2:3'  # N8 (Stuttgart Stuttgart)
	WEINSTEIGE_4 = 'de:08111:163:2:4'  # N8, N9, N10 (Stuttgart Stuttgart)
	WEINSTEIGE_5 = 'de:08119:3770:0:3'  # Rtg Burgunderstr. (Weinstadt Beutelsbach)
	WEINSTEIGE_6 = 'de:08116:3895:0:3'  # eRtg Egelsberg Teckstraße (Weilheim an der Teck Weilheim (T))
	WEINSTEIGE_7 = 'de:08116:3895:0:4'  # eRtg Limburgschule/Bissinger Strasse (Weilheim an der Teck Weilheim (T))
	KARL_PFAFF_STRASSE_1 = 'de:08111:164:0:3'  # Nachtbus Rtg Degerloch (B27) (Stuttgart Stuttgart)
	KARL_PFAFF_STRASSE_2 = 'de:08111:164:0:4'  # Nachtbus Rtg Weinsteige (Stuttgart Stuttgart)
	KARL_PFAFF_STRASSE_3 = 'de:08116:4036:0:3'  # Rtg Weil (Esslingen am Neckar Pliensauvorstadt)
	KARL_PFAFF_STRASSE_4 = 'de:08116:4036:0:5'  # Rtg Weil neu (Esslingen am Neckar Pliensauvorstadt)
	KARL_PFAFF_STRASSE_5 = 'de:08116:4036:0:6'  # Rtg Esslingen neu (Esslingen am Neckar Pliensauvorstadt)
	HAIGST = 'de:08111:165'
	HAIGST_1 = 'de:08111:165:1:1'  # Rtg Nägelestraße (Stuttgart Stuttgart)
	HAIGST_2 = 'de:08111:165:1:2'  # Rtg Wielandshöhe (Stuttgart Stuttgart)
	HAIGST_3 = 'de:08111:165:1:3'  # Linientaxi (Stuttgart Stuttgart)
	HAIGST_4 = 'de:08111:165:1:4'  # Zacke-Bus Rtg. Marienplatz (Stuttgart Stuttgart)
	VAIHINGER_STRASSE_1 = 'de:08111:170:1:1'  # Rtg SSB-Zentrum/Rohrer Weg (Stuttgart Stuttgart)
	VAIHINGER_STRASSE_2 = 'de:08111:170:2:2'  # Rtg Möhringen Bf (Stuttgart Stuttgart)
	VAIHINGER_STRASSE_3 = 'de:08111:170:3:3'  # Nachtbus (Stuttgart Stuttgart)
	VAIHINGER_STRASSE_4 = 'de:08118:5651:1:3'  # eRtg Sedanstraße/Industriestraße (Sersheim Sersheim)
	VAIHINGER_STRASSE_5 = 'de:08118:6962:0:3'  # eRtg Dorfplatz/B10/Aurich (Eberdingen Nussdorf)
	VAIHINGER_STRASSE_6 = 'de:08118:6962:0:4'  # eRtg Martinstraße (Eberdingen Nussdorf)
	LEINF__UNTERAICHEN = 'de:08116:174'
	LEINF__UNTERAICHEN_1 = 'de:08116:174:0:1'  # Rtg Leinfelden Bf (Leinfelden-Echterdingen Leinfelden)
	LEINF__UNTERAICHEN_2 = 'de:08116:174:0:2'  # Rtg Leinfelden Frank (Leinfelden-Echterdingen Leinfelden)
	UNTERAICHEN = 'de:08116:174'
	UNTERAICHEN_1 = 'de:08116:174:0:1'  # Rtg Leinfelden Bf (Leinfelden-Echterdingen Leinfelden)
	UNTERAICHEN_2 = 'de:08116:174:0:2'  # Rtg Leinfelden Frank (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN = 'de:08116:175'
	LEINFELDEN_1 = 'de:08116:175:1:1'  # Gleis 1 (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_2 = 'de:08116:175:2:2'  # Gleis 2 (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_3 = 'de:08116:175:3:7'  # Stadtbahn - Ankunft Endhst. (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_4 = 'de:08116:175:3:8'  # Stadtbahn Rtg Unteraichen (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_5 = 'de:08116:175:4:4'  # Bstg 3 Linien 82/86 Rtg Schönbuchstr. (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_6 = 'de:08116:175:4:5'  # Bstg 4 Linie 38 Rtg Bad /Linie 35 Ank. (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_7 = 'de:08116:175:5:2'  # Bstg 1 Linien 86/826 Rtg Neuer Markt (Leinfelden-Echterdingen Leinfelden)
	ROHRER_WEG = 'de:08111:183'
	ROHRER_WEG_1 = 'de:08111:183:0:1'  # Rtg Möhringen Freibad (Stuttgart Stuttgart)
	ROHRER_WEG_2 = 'de:08111:183:0:2'  # Rtg Vaihinger Str. (Stuttgart Stuttgart)
	GRÖTZINGEN_RAIFFEISENSTRASSE = 'de:08116:190'
	GRÖTZINGEN_RAIFFEISENSTRASSE_1 = 'de:08116:190:0:3'  # Rtg Aich (Aichtal Grötzingen)
	GRÖTZINGEN_RAIFFEISENSTRASSE_2 = 'de:08116:190:0:4'  # Rtg Ortsmitte (Aichtal Grötzingen)
	RAIFFEISENSTRASSE_1 = 'de:08116:190:0:3'  # Rtg Aich (Aichtal Grötzingen)
	RAIFFEISENSTRASSE_2 = 'de:08116:190:0:4'  # Rtg Ortsmitte (Aichtal Grötzingen)
	RAIFFEISENSTRASSE_3 = 'de:08116:1997:0:3'  # Rtg Harthausen Linde (Filderstadt Bonlanden)
	RAIFFEISENSTRASSE_4 = 'de:08116:1997:0:4'  # Rtg Bernhausen Fleinsbach (Filderstadt Bonlanden)
	RAIFFEISENSTRASSE_5 = 'de:08118:5607:0:3'  # eRtg Aussiedlerhöfe (Großbottwar Winzerhausen)
	RAIFFEISENSTRASSE_6 = 'de:08117:4011:0:R1'  # Rtg Am Meerberg (Göppingen Bartenbach (Göppingen))
	AM_ÄUSSEREN_GRABEN = 'de:08111:192'
	AM_ÄUSSEREN_GRABEN_1 = 'de:08111:192:0:1'  # Rtg Emerholz (Stuttgart Stuttgart)
	AM_ÄUSSEREN_GRABEN_2 = 'de:08111:192:0:2'  # Rtg Pflugfelder Str. (Stuttgart Stuttgart)
	EMERHOLZ = 'de:08111:193'
	EMERHOLZ_1 = 'de:08111:193:0:1'  # Rtg Withau (Stuttgart Stuttgart)
	EMERHOLZ_2 = 'de:08111:193:0:2'  # Rtg Greut (Stuttgart Stuttgart)
	WITHAU = 'de:08111:194'
	WITHAU_1 = 'de:08111:194:0:1'  # Rtg Greut (Stuttgart Stuttgart)
	WITHAU_2 = 'de:08111:194:0:2'  # Rtg Emerholz (Stuttgart Stuttgart)
	GREUT = 'de:08111:195'
	GREUT_1 = 'de:08111:195:0:1'  # Rtg Sporthalle (Stuttgart Stuttgart)
	GREUT_2 = 'de:08111:195:0:2'  # Rtg Withau (Stuttgart Stuttgart)
	STAMMHEIM_SPORTHALLE = 'de:08111:196'
	STAMMHEIM_SPORTHALLE_1 = 'de:08111:196:0:1'  # Rtg Wespenweg (Stuttgart Stuttgart)
	STAMMHEIM_SPORTHALLE_2 = 'de:08111:196:0:2'  # Rtg Greut (Stuttgart Stuttgart)
	WESPENWEG = 'de:08111:197'
	WESPENWEG_1 = 'de:08111:197:0:1'  # Rtg Freihofplatz (Stuttgart Stuttgart)
	WESPENWEG_2 = 'de:08111:197:0:2'  # Rtg Sporthalle (Stuttgart Stuttgart)
	FREIHOFPLATZ = 'de:08111:198'
	FREIHOFPLATZ_1 = 'de:08111:198:0:1'  # Rtg Am Kirchplatz (Stuttgart Stuttgart)
	FREIHOFPLATZ_2 = 'de:08111:198:0:2'  # Rtg Wespenweg (Stuttgart Stuttgart)
	AM_KIRCHPLATZ = 'de:08111:199:0:1'  # Rtg Kameralamtsstr. (Stuttgart Stuttgart)
	KAMERALAMTSSTRASSE = 'de:08111:200:0:1'  # Rtg Admiralweg (Stuttgart Stuttgart)
	ADMIRALWEG = 'de:08111:201:0:1'  # Rtg Schwärmerweg (Stuttgart Stuttgart)
	SCHWÄRMERWEG = 'de:08111:202:0:1'  # Rtg Segelfalterstr. (Stuttgart Stuttgart)
	SEGELFALTERSTRASSE = 'de:08111:203:0:1'  # Rtg Tagfalterweg (Stuttgart Stuttgart)
	TAGFALTERWEG = 'de:08111:204:0:1'  # Rtg Schwilkenhofstr. (Stuttgart Stuttgart)
	SCHWILKENHOFSTRASSE = 'de:08111:205:0:1'  # Rtg Sonatenweg (Stuttgart Stuttgart)
	STAMMHEIM_FRIEDHOF = 'de:08111:206:0:1'  # Rtg Generationenhaus (Stuttgart Stuttgart)
	GENERATIONENHAUS = 'de:08111:207:0:1'  # Rtg Freihofplatz (Stuttgart Stuttgart)
	MILLÖCKERSTRASSE = 'de:08111:211'
	MILLÖCKERSTRASSE_1 = 'de:08111:211:0:1'  # Rtg Eltinger Str. (Stuttgart Stuttgart)
	MILLÖCKERSTRASSE_2 = 'de:08111:211:0:2'  # Rtg Beethovenstr. (Stuttgart Stuttgart)
	MILLÖCKERSTRASSE_3 = 'de:08111:211:0:6'  # Ersatzhalte auswärts (Stuttgart Stuttgart)
	BEETHOVENSTRASSE_1 = 'de:08111:212:1:1'  # Rtg Millöckerstr. (Stuttgart Stuttgart)
	BEETHOVENSTRASSE_2 = 'de:08111:212:1:2'  # Rtg Lindpaintnerstr. (Stuttgart Stuttgart)
	BEETHOVENSTRASSE_3 = 'de:08111:212:3:3'  # Rtg Lindpaintnerstr. (Stuttgart Stuttgart)
	BEETHOVENSTRASSE_4 = 'de:08111:212:3:4'  # Rtg Eltinger Str. (Stuttgart Stuttgart)
	BEETHOVENSTRASSE_5 = 'de:08116:4231:0:3'  # eRtg Lettenäcker (Plochingen Plochingen)
	BEETHOVENSTRASSE_6 = 'de:08116:4231:0:4'  # eRtg Marquardtschule / Panoramastraße (Plochingen Plochingen)
	BEETHOVENSTRASSE_7 = 'de:08115:4589:0:3'  # Rtg Hausener Str. (Weil der Stadt Merklingen)
	BEETHOVENSTRASSE_8 = 'de:08115:4589:0:4'  # eRtg Ortsmitte (Weil der Stadt Merklingen)
	BEETHOVENSTRASSE_9 = 'de:08115:4821:0:3'  # eRtg Wolfenmühle (Weil im Schönbuch Neuweiler)
	BEETHOVENSTRASSE_10 = 'de:08119:5287:0:3'  # eRtg Täferhalde (Weissach im Tal Unterweissach)
	BEETHOVENSTRASSE_11 = 'de:08119:5287:0:4'  # eRtg Ungeheuerhof (Weissach im Tal Unterweissach)
	BEETHOVENSTRASSE_12 = 'de:08115:7004:0:3'  # eRtg Grafenweg (Herrenberg Herrenberg)
	BEETHOVENSTRASSE_13 = 'de:08115:7004:0:4'  # eRtg Hallenbad (Herrenberg Herrenberg)
	BEETHOVENSTRASSE_14 = 'de:08119:7550:0:3'  # Rtg Friedhof (Waiblingen Bittenfeld)
	BEETHOVENSTRASSE_15 = 'de:08136:7006:0:H1'  # Beethovenstraße (Lorch Lorch)
	WIELANDSTRASSE = 'de:08111:215'
	WIELANDSTRASSE_1 = 'de:08111:215:0:3'  # Rtg Herbsthalde (Stuttgart Stuttgart)
	WIELANDSTRASSE_2 = 'de:08111:215:0:4'  # Rtg Botnanger Sattel (Stuttgart Stuttgart)
	HERDERPLATZ = 'de:08111:216'
	HERDERPLATZ_1 = 'de:08111:216:1:1'  # Rtg Lindpaintnerstr. (Stuttgart Stuttgart)
	HERDERPLATZ_2 = 'de:08111:216:1:2'  # Rtg Vogelsang (Stuttgart Stuttgart)
	HERDERPLATZ_3 = 'de:08111:216:2:5'  # Linie 40 Rtg Vogelsang (Stuttgart Stuttgart)
	HERDERPLATZ_4 = 'de:08111:216:2:6'  # Linie 40 Rtg Herbsthalde (Stuttgart Stuttgart)
	HERDERPLATZ_5 = 'de:08111:216:3:3'  # Linie 50 Rtg Westbahnhof (Stuttgart Stuttgart)
	HERDERPLATZ_6 = 'de:08111:216:3:4'  # Linie 50 Rtg Herbsthalde (Stuttgart Stuttgart)
	VOGELSANG_1 = 'de:08111:217:1:1'  # Rtg Herderplatz (Stuttgart Stuttgart)
	VOGELSANG_2 = 'de:08111:217:1:5'  # Nachtbus (Stuttgart Stuttgart)
	VOGELSANG_3 = 'de:08111:217:2:3'  # Linie 40 Rtg Herderplatz (Stuttgart Stuttgart)
	VOGELSANG_4 = 'de:08111:217:3:2'  # Rtg Arndt-/Spittastr. (Stuttgart Stuttgart)
	VOGELSANG_5 = 'de:08235:10070:0:3'  # Rtg Waldlust Mötzingen (Nagold Nagold)
	VOGELSANG_6 = 'de:08235:10070:0:4'  # Rtg ZOB (Nagold Nagold)
	ARNDT__SPITTASTRASSE = 'de:08111:218'
	ARNDT__SPITTASTRASSE_1 = 'de:08111:218:1:1'  # Rtg Vogelsang (Stuttgart Stuttgart)
	ARNDT__SPITTASTRASSE_2 = 'de:08111:218:2:2'  # Rtg Schwab-/Bebelstr. (Stuttgart Stuttgart)
	ARNDT__SPITTASTRASSE_3 = 'de:08111:218:3:3'  # NBus (Stuttgart Stuttgart)
	RAITELSBERG = 'de:08111:222'
	RAITELSBERG_1 = 'de:08111:222:0:1'  # Rtg Bergfriedhof (Stuttgart Stuttgart)
	RAITELSBERG_2 = 'de:08111:222:0:2'  # Rtg Schlachthof (Stuttgart Stuttgart)
	RAITELSBERG_3 = 'de:08111:222:0:4'  # NBus (Stuttgart Stuttgart)
	WANGEN_MARKTPLATZ = 'de:08111:225'
	WANGEN_MARKTPLATZ_1 = 'de:08111:225:1:1'  # Rtg Wasenstr. (Stuttgart Stuttgart)
	WANGEN_MARKTPLATZ_2 = 'de:08111:225:1:2'  # Rtg Hedelfinger Str. (Stuttgart Stuttgart)
	HEDELFINGER_STRASSE = 'de:08111:226'
	HEDELFINGER_STRASSE_1 = 'de:08111:226:0:1'  # Rtg Wangen Marktplatz (Stuttgart Stuttgart)
	HEDELFINGER_STRASSE_2 = 'de:08111:226:0:2'  # Rtg Hedelfingen (Stuttgart Stuttgart)
	HEDELFINGEN = 'de:08111:227'
	HEDELFINGEN_1 = 'de:08111:227:1:1'  # Stadtbahn - Endhst. (Stuttgart Stuttgart)
	HEDELFINGEN_2 = 'de:08111:227:2:4'  # Rtg Otto-Hirsch-Brücken (Stuttgart Stuttgart)
	HEDELFINGEN_3 = 'de:08111:227:3:3'  # Rtg Ährenweg/Amstetter Str. (Stuttgart Stuttgart)
	BÖRSENPLATZ = 'de:08111:229'
	BÖRSENPLATZ_1 = 'de:08111:229:1:1'  # Rtg Berliner Platz (Stuttgart Stuttgart)
	BÖRSENPLATZ_2 = 'de:08111:229:1:2'  # Rtg Hbf (Stuttgart Stuttgart)
	WILHELMA = 'de:08111:254'
	WILHELMA_1 = 'de:08111:254:1:1'  # Rtg Mineralbäder (Stuttgart Stuttgart)
	WILHELMA_2 = 'de:08111:254:1:2'  # Rtg Rosensteinbrücke (Stuttgart Stuttgart)
	ROSENSTEINPARK = 'de:08111:262'
	ROSENSTEINPARK_1 = 'de:08111:262:0:1'  # Rtg Glockenstr. (Stuttgart Stuttgart)
	ROSENSTEINPARK_2 = 'de:08111:262:0:2'  # Rtg Rosensteinpark (Stuttgart Stuttgart)
	GLOCKENSTRASSE = 'de:08111:263'
	GLOCKENSTRASSE_1 = 'de:08111:263:0:1'  # Rtg Rosensteinbrücke (Stuttgart Stuttgart)
	GLOCKENSTRASSE_2 = 'de:08111:263:0:2'  # Rtg Rosensteinpark (Stuttgart Stuttgart)
	KIENBACHSTRASSE_1 = 'de:08111:264:0:1'  # Rtg Blick (Stuttgart Stuttgart)
	KIENBACHSTRASSE_2 = 'de:08111:264:0:2'  # Rtg Ebitzweg (Stuttgart Stuttgart)
	KIENBACHSTRASSE_3 = 'de:08119:2644:0:3'  # Rtg Eugenstr. (Fellbach Fellbach)
	KIENBACHSTRASSE_4 = 'de:08119:2644:0:4'  # Rtg Schwabenlandhalle/F.3 (Fellbach Fellbach)
	BLICK = 'de:08111:265'
	BLICK_1 = 'de:08111:265:0:1'  # Rtg Eszet (Stuttgart Stuttgart)
	BLICK_2 = 'de:08111:265:0:2'  # Rtg Kienbachstr. (Stuttgart Stuttgart)
	ESZET = 'de:08111:267'
	ESZET_1 = 'de:08111:267:0:1'  # Rtg Schlotterbeckstr. (Stuttgart Stuttgart)
	ESZET_2 = 'de:08111:267:0:2'  # Rtg Blick (Stuttgart Stuttgart)
	SCHLOTTERBECKSTRASSE = 'de:08111:268'
	SCHLOTTERBECKSTRASSE_1 = 'de:08111:268:0:1'  # Rtg Untertürkheim Bf (Stuttgart Stuttgart)
	SCHLOTTERBECKSTRASSE_2 = 'de:08111:268:0:2'  # Rtg Eszet (Stuttgart Stuttgart)
	AUWIESEN = 'de:08111:271'
	AUWIESEN_1 = 'de:08111:271:1:1'  # Rtg Hofen (Stuttgart Stuttgart)
	AUWIESEN_2 = 'de:08111:271:2:2'  # Rtg Mühlhausen (Stuttgart Stuttgart)
	AUWIESEN_3 = 'de:08111:271:3:3'  # Linie 54 Rtg Hofen (Stuttgart Stuttgart)
	AUWIESEN_4 = 'de:08111:271:4:4'  # Linie 54 Rtg Mühlhausen (Stuttgart Stuttgart)
	HOFEN_1 = 'de:08111:272:1:1'  # Rtg Max-Eyth-See (Stuttgart Stuttgart)
	HOFEN_2 = 'de:08111:272:1:2'  # Rtg Auwiesen (Stuttgart Stuttgart)
	HOFEN_3 = 'de:08111:272:3:3'  # Linie 54 Rtg Max-Eyth-See (Stuttgart Stuttgart)
	HOFEN_4 = 'de:08111:272:3:4'  # Linie 54 Rtg Auwiesen (Stuttgart Stuttgart)
	HOFEN_5 = 'de:08118:5696:0:3'  # eRtg Aussiedlerhöfe (Bönnigheim Hofen (Bönnigheim))
	HOFEN_6 = 'de:08118:5696:0:4'  # eRtg Sonnenhalde (Bönnigheim Hofen (Bönnigheim))
	WAGRAINÄCKER = 'de:08111:274'
	WAGRAINÄCKER_1 = 'de:08111:274:1:1'  # U14 Rtg Elbestr./U 12 Rtg Bottroper Str. (Stuttgart Stuttgart)
	WAGRAINÄCKER_2 = 'de:08111:274:1:2'  # Rtg Max-Eyth-See (Stuttgart Stuttgart)
	ELBESTRASSE = 'de:08111:275'
	ELBESTRASSE_1 = 'de:08111:275:1:1'  # Rtg Freibergstraße (Stuttgart Stuttgart)
	ELBESTRASSE_2 = 'de:08111:275:1:2'  # Rtg Wagrainäcker (Stuttgart Stuttgart)
	FREIBERGSTRASSE = 'de:08111:276'
	FREIBERGSTRASSE_1 = 'de:08111:276:0:1'  # Rtg Münster Rathaus (Stuttgart Stuttgart)
	FREIBERGSTRASSE_2 = 'de:08111:276:0:2'  # Rtg Elbestraße (Stuttgart Stuttgart)
	FREIBERGSTRASSE_3 = 'de:08111:276:0:3'  # Nachtbus auswärts (Stuttgart Stuttgart)
	MÜNSTER_RATHAUS = 'de:08111:277'
	MÜNSTER_RATHAUS_1 = 'de:08111:277:1:1'  # Rtg Münster Viadukt (Stuttgart Stuttgart)
	MÜNSTER_RATHAUS_2 = 'de:08111:277:1:2'  # Rtg Freibergstraße (Stuttgart Stuttgart)
	MÜNSTER_RATHAUS_3 = 'de:08111:277:2:4'  # NBus auswärts (Stuttgart Stuttgart)
	MÜNSTER_VIADUKT = 'de:08111:278'
	MÜNSTER_VIADUKT_1 = 'de:08111:278:0:1'  # Rtg Kraftwerk Münster (Stuttgart Stuttgart)
	MÜNSTER_VIADUKT_2 = 'de:08111:278:0:2'  # Rtg Münster Rathaus (Stuttgart Stuttgart)
	WESERSTRASSE = 'de:08111:279'
	WESERSTRASSE_1 = 'de:08111:279:0:3'  # Rtg Bottroper Str. (Stuttgart Stuttgart)
	WESERSTRASSE_2 = 'de:08111:279:0:4'  # Nachtbus (Stuttgart Stuttgart)
	KRAFTWERK_MÜNSTER = 'de:08111:280'
	KRAFTWERK_MÜNSTER_1 = 'de:08111:280:0:1'  # Rtg Mühlsteg (Stuttgart Stuttgart)
	KRAFTWERK_MÜNSTER_2 = 'de:08111:280:0:2'  # Rtg Münster Viadukt (Stuttgart Stuttgart)
	MÜHLSTEG = 'de:08111:281'
	MÜHLSTEG_1 = 'de:08111:281:0:1'  # Rtg Rosensteinbrücke (Stuttgart Stuttgart)
	MÜHLSTEG_2 = 'de:08111:281:0:2'  # Rtg Kraftwerk Münster (Stuttgart Stuttgart)
	MÜHLSTEG_3 = 'de:08111:281:0:3'  # NBus (Stuttgart Stuttgart)
	SUTTNERSTRASSE = 'de:08111:288'
	SUTTNERSTRASSE_1 = 'de:08111:288:1:1'  # Rtg Himmelsleiter (Stuttgart Stuttgart)
	SUTTNERSTRASSE_2 = 'de:08111:288:1:2'  # Rtg Freiberg (Stuttgart Stuttgart)
	SUTTNERSTRASSE_3 = 'de:08111:288:2:4'  # NBus (Stuttgart Stuttgart)
	HIMMELSLEITER = 'de:08111:289'
	HIMMELSLEITER_1 = 'de:08111:289:1:1'  # Rtg Tappachstr. (Stuttgart Stuttgart)
	HIMMELSLEITER_2 = 'de:08111:289:1:2'  # Rtg Suttnerstr. (Stuttgart Stuttgart)
	HIMMELSLEITER_3 = 'de:08111:289:2:4'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	MITTNACHTSTRASSE = 'de:08111:296'
	MITTNACHTSTRASSE_1 = 'de:08111:296:1:1'  # Rtg Milchhof (Stuttgart Stuttgart)
	MITTNACHTSTRASSE_2 = 'de:08111:296:1:2'  # Rtg Nordbahnhof (Stuttgart Stuttgart)
	MILCHHOF = 'de:08111:297'
	MILCHHOF_1 = 'de:08111:297:0:1'  # Rtg Budapester Platz (Stuttgart Stuttgart)
	MILCHHOF_2 = 'de:08111:297:0:2'  # Rtg Mittnachtstr. (Stuttgart Stuttgart)
	BUDAPESTER_PLATZ = 'de:08111:299'
	BUDAPESTER_PLATZ_1 = 'de:08111:299:0:1'  # Rtg Hauptbahnhof (Stuttgart Stuttgart)
	BUDAPESTER_PLATZ_2 = 'de:08111:299:0:2'  # Rtg Milchhof (Stuttgart Stuttgart)
	FRIEDHOFSTRASSE = 'de:08111:303'
	FRIEDHOFSTRASSE_1 = 'de:08111:303:0:3'  # NBus auswärts (Stuttgart Stuttgart)
	FRIEDHOFSTRASSE_2 = 'de:08111:303:0:4'  # NBus einwärts (Stuttgart Stuttgart)
	TAL__OSTENDSTRASSE = 'de:08111:306'
	TAL__OSTENDSTRASSE_1 = 'de:08111:306:0:4'  # NBus auswärts (Stuttgart Stuttgart)
	TAL__OSTENDSTRASSE_2 = 'de:08111:306:0:5'  # NBus Rtg Wunnensteinstr. (Stuttgart Stuttgart)
	MUSBERG_KARLSTRASSE = 'de:08116:309'
	MUSBERG_KARLSTRASSE_1 = 'de:08116:309:0:3'  # Rtg Schafgartenstraße (Leinfelden-Echterdingen Musberg)
	MUSBERG_KARLSTRASSE_2 = 'de:08116:309:0:4'  # Rtg Ludwigstraße (Leinfelden-Echterdingen Musberg)
	KARLSTRASSE_1 = 'de:08116:309:0:3'  # Rtg Schafgartenstraße (Leinfelden-Echterdingen Musberg)
	KARLSTRASSE_2 = 'de:08116:309:0:4'  # Rtg Ludwigstraße (Leinfelden-Echterdingen Musberg)
	KARLSTRASSE_3 = 'de:08116:2009:0:3'  # Rtg Sielmingen Schul-/Sportzentrum (Filderstadt Bernhausen)
	KARLSTRASSE_4 = 'de:08116:2009:0:4'  # Rtg. Bernhausen Bahnhof (Filderstadt Bernhausen)
	KARLSTRASSE_5 = 'de:08115:3103:0:3'  # eRtg Parkstraße (Landratsamt) (Böblingen Böblingen)
	KARLSTRASSE_6 = 'de:08118:5683:0:3'  # Rtg Panoramastr. (Bietigheim-Bissingen Bissingen (LB))
	KARLSTRASSE_7 = 'de:08118:5683:0:4'  # Rtg Bahnhof (Bietigheim-Bissingen Bissingen (LB))
	KARLSTRASSE_8 = 'de:08119:5949:0:3'  # eRtg Lilienstraße/Fellbacher Straße (Kernen im Remstal Rommelshausen)
	KARLSTRASSE_9 = 'de:08119:5949:0:4'  # eRtg Rumold-Realschule (Kernen im Remstal Rommelshausen)
	KARLSTRASSE_10 = 'de:08119:5949:0:5'  # Rtg Esslingen (RELEX X20) (Kernen im Remstal Rommelshausen)
	FRIEDHOF_1 = 'de:08116:310:0:3'  # Rtg Wanderweg (Leinfelden-Echterdingen Musberg)
	FRIEDHOF_2 = 'de:08116:310:0:4'  # Rtg Schafgartenstraße (Leinfelden-Echterdingen Musberg)
	FRIEDHOF_3 = 'de:08116:2045:0:3'  # Rtg Schul-/Sportzentrum (Filderstadt Sielmingen)
	FRIEDHOF_4 = 'de:08116:2045:0:4'  # Rtg Lange Straße (Filderstadt Sielmingen)
	FRIEDHOF_5 = 'de:08116:2102:0:3'  # Rtg Messe West (Leinfelden-Echterdingen Echterdingen)
	FRIEDHOF_6 = 'de:08116:2102:0:4'  # Rtg Stadionstraße (Leinfelden-Echterdingen Echterdingen)
	FRIEDHOF_7 = 'de:08115:3319:0:3'  # Rtg Ernst-Bauer-Pl. (Renningen Renningen)
	FRIEDHOF_8 = 'de:08115:3319:0:4'  # Rtg Süd (Renningen Renningen)
	FRIEDHOF_9 = 'de:08119:3607:0:3'  # eRtg Johanneskirche (Backnang Backnang)
	FRIEDHOF_10 = 'de:08119:3737:0:3'  # Rtg. Rommelsh. Rathaus (Kernen im Remstal Rommelshausen)
	FRIEDHOF_11 = 'de:08116:4018:0:3'  # Rtg Egert Lindenplatz (Esslingen am Neckar Zell (Esslingen))
	FRIEDHOF_12 = 'de:08116:4018:0:4'  # Rtg Zell Bf (Esslingen am Neckar Zell (Esslingen))
	FRIEDHOF_13 = 'de:08116:4390:0:3'  # eRtg Ohmden Rathaus (Ohmden Ohmden)
	FRIEDHOF_14 = 'de:08116:4390:0:4'  # eRtg Jesingen Rathaus (Ohmden Ohmden)
	FRIEDHOF_15 = 'de:08116:4454:0:3'  # Rtg Unterensinger Str. (Nürtingen Zizishausen)
	FRIEDHOF_16 = 'de:08116:4454:0:4'  # Rtg Oberensingen (Nürtingen Zizishausen)
	FRIEDHOF_17 = 'de:08118:4560:0:3'  # eRtg Ditzingen/Korntal (Korntal-Münchingen Münchingen)
	FRIEDHOF_18 = 'de:08118:4560:0:4'  # eRtg Stiegelplatz (Korntal-Münchingen Münchingen)
	FRIEDHOF_19 = 'de:08115:4586:0:1'  # Rtg Renningen Ihninger Hof (Weil der Stadt Weil der Stadt)
	FRIEDHOF_20 = 'de:08115:4586:0:2'  # Rtg Bahnhof (Weil der Stadt Weil der Stadt)
	FRIEDHOF_21 = 'de:08115:4794:0:3'  # eRtg Würmstraße/Theodor-Heuss-Straße (Altdorf Altdorf (BB))
	FRIEDHOF_22 = 'de:08119:5129:0:3'  # eRtg Kirchplatz (Welzheim Welzheim)
	FRIEDHOF_23 = 'de:08119:5129:0:4'  # eRtg Amselweg (Welzheim Welzheim)
	FRIEDHOF_24 = 'de:08119:5207:0:3'  # eRtg Wallstraße/BZII (Winnenden Winnenden)
	FRIEDHOF_25 = 'de:08119:5381:0:3'  # eRtg Lippoldsweiler (Friedhofseite) (Auenwald Unterbrüden)
	FRIEDHOF_26 = 'de:08119:5381:0:4'  # eRtg Auenwaldhalle/Hügelstraße (Auenwald Unterbrüden)
	FRIEDHOF_27 = 'de:08118:5447:0:3'  # eRtg Lenzhalde (Kornwestheim Kornwestheim)
	FRIEDHOF_28 = 'de:08118:5558:0:3'  # eRtg Zehntscheuer (Ludwigsburg Poppenweiler)
	FRIEDHOF_29 = 'de:08118:5558:0:4'  # eRtg Poppenweiler Straße (Ludwigsburg Poppenweiler)
	FRIEDHOF_30 = 'de:08118:5718:0:3'  # eRtg Wasserturm (Besigheim Besigheim)
	FRIEDHOF_31 = 'de:08118:5933:0:3'  # eRtg Schlossstraße (Ingersheim Großingersheim)
	FRIEDHOF_32 = 'de:08118:5933:0:4'  # eRtg Brühlstraße (Ingersheim Großingersheim)
	FRIEDHOF_33 = 'de:08119:5958:0:3'  # Rtg Seniorenzentrum/Querspange (Waiblingen Waiblingen)
	FRIEDHOF_34 = 'de:08119:5958:0:4'  # eRtg Schwabstraße (Waiblingen Waiblingen)
	FRIEDHOF_35 = 'de:08115:6480:0:3'  # Rtg Gärtringer Str. (Nufringen Nufringen)
	FRIEDHOF_36 = 'de:08116:6494:0:3'  # eRtg Hegenloher Straße (Lichtenwald Thomashardt)
	FRIEDHOF_37 = 'de:08116:6494:0:4'  # eRtg Nachbarschaftsschule (Lichtenwald Thomashardt)
	FRIEDHOF_38 = 'de:08115:6529:0:3'  # Rtg Gymnasium (Holzgerlingen Holzgerlingen)
	FRIEDHOF_39 = 'de:08118:6825:0:3'  # eRtg Neckarhalde (Remseck am Neckar Neckarrems)
	FRIEDHOF_40 = 'de:08118:6825:0:4'  # eRtg Lindenstraße (Remseck am Neckar Neckarrems)
	FRIEDHOF_41 = 'de:08116:6865:0:3'  # Rtg Wilhelm-Raabe-Str. (Wendlingen am Neckar Wendlingen (N))
	FRIEDHOF_42 = 'de:08119:6996:0:3'  # eRtg Hohenbrach (Großerlach Grab)
	FRIEDHOF_43 = 'de:08119:6996:0:4'  # eRtg Trauzenbach (Großerlach Grab)
	FRIEDHOF_44 = 'de:08119:7551:0:3'  # Rtg Altes Rathaus (Waiblingen Bittenfeld)
	FRIEDHOF_45 = 'de:08118:7841:0:3'  # Rtg Silvanerstr. (Vaihingen an der Enz Roßwag)
	FRIEDHOF_46 = 'de:08117:53:0:H1'  # Rtg Bad Ditzenbach (Deggingen Deggingen)
	FRIEDHOF_47 = 'de:08117:53:0:R1'  # Rtg Ave Maria (Deggingen Deggingen)
	FRIEDHOF_48 = 'de:08117:96:0:R1'  # Rtg Stadtkirche (Geislingen an der Steige Geislingen (Steige))
	FRIEDHOF_49 = 'de:08117:201:0:2'  # Rtg Rathaus (Böhmenkirch Böhmenkirch)
	FRIEDHOF_50 = 'de:08117:201:0:H1'  # Rtg Post (Böhmenkirch Böhmenkirch)
	FRIEDHOF_51 = 'de:08117:1109:0:H1'  # Rtg Schw. Gmünd (Wäschenbeuren Wäschenbeuren)
	FRIEDHOF_52 = 'de:08117:1109:0:R1'  # Rtg Göppingen (Wäschenbeuren Wäschenbeuren)
	FRIEDHOF_53 = 'de:08117:12054:0:2'  # Rtg Rathaus (Hattenhofen Hattenhofen)
	FRIEDHOF_54 = 'de:08117:2054:0:H1'  # Rtg Schlierbach (Hattenhofen Hattenhofen)
	FRIEDHOF_55 = 'de:08117:3312:0:H1'  # Rtg Eschenbach (Heiningen Heiningen (GP))
	FRIEDHOF_56 = 'de:08117:3312:0:R1'  # Rtg Lindenstr. (Heiningen Heiningen (GP))
	FRIEDHOF_57 = 'de:08117:4034:0:H1'  # Rtg Industriegebiet (Börtlingen Börtlingen)
	FRIEDHOF_58 = 'de:08117:4034:0:R1'  # Rtg Kreissparkasse (Börtlingen Börtlingen)
	MUSBERG_FRIEDHOF = 'de:08116:310'
	MUSBERG_FRIEDHOF_1 = 'de:08116:310:0:3'  # Rtg Wanderweg (Leinfelden-Echterdingen Musberg)
	MUSBERG_FRIEDHOF_2 = 'de:08116:310:0:4'  # Rtg Schafgartenstraße (Leinfelden-Echterdingen Musberg)
	FILDERBAHNSTRASSE = 'de:08111:316:0:5'  # Nachtbus (Stuttgart Stuttgart)
	ELSTERSTAFFEL = 'de:08111:318:0:3'  # NBus Rtg Südheimer Platz (Stuttgart Stuttgart)
	LOSÄCKER = 'de:08111:319:0:3'  # NBus (Stuttgart Stuttgart)
	LAICHLESTRASSE = 'de:08118:320'
	LAICHLESTRASSE_1 = 'de:08118:320:0:3'  # NBus / Rtg Breitwiesen (Gerlingen Gerlingen)
	LAICHLESTRASSE_2 = 'de:08118:320:0:4'  # Rtg Leonberg (Gerlingen Gerlingen)
	GERLINGEN_LAICHLESTR = 'de:08118:320'
	GERLINGEN_LAICHLESTR_1 = 'de:08118:320:0:3'  # NBus / Rtg Breitwiesen (Gerlingen Gerlingen)
	GERLINGEN_LAICHLESTR_2 = 'de:08118:320:0:4'  # Rtg Leonberg (Gerlingen Gerlingen)
	GNESENER_STRASSE = 'de:08111:321'
	GNESENER_STRASSE_1 = 'de:08111:321:0:1'  # Rtg Kursaal (Stuttgart Stuttgart)
	GNESENER_STRASSE_2 = 'de:08111:321:0:2'  # Rtg Obere Ziegelei (Stuttgart Stuttgart)
	GNESENER_STRASSE_3 = 'de:08111:321:0:4'  # NBus (Stuttgart Stuttgart)
	KURSAAL = 'de:08111:322'
	KURSAAL_1 = 'de:08111:322:1:1'  # Rtg Daimlerplatz (Stuttgart Stuttgart)
	KURSAAL_2 = 'de:08111:322:1:2'  # Rtg Gnesener Str. (Stuttgart Stuttgart)
	KURSAAL_3 = 'de:08111:322:3:4'  # N5 (Stuttgart Stuttgart)
	DAIMLERPLATZ = 'de:08111:323'
	DAIMLERPLATZ_1 = 'de:08111:323:0:1'  # Rtg Wilhelmsplatz (Stuttgart Stuttgart)
	DAIMLERPLATZ_2 = 'de:08111:323:0:2'  # Rtg Kursaal (Stuttgart Stuttgart)
	NÄGELESTRASSE = 'de:08111:341'
	NÄGELESTRASSE_1 = 'de:08111:341:0:1'  # Rtg Zahnradbahnhof (Stuttgart Stuttgart)
	NÄGELESTRASSE_2 = 'de:08111:341:0:2'  # Rtg Haigst (Stuttgart Stuttgart)
	WIELANDSHÖHE = 'de:08111:342'
	WIELANDSHÖHE_1 = 'de:08111:342:0:1'  # Rtg Haigst (Stuttgart Stuttgart)
	WIELANDSHÖHE_2 = 'de:08111:342:0:2'  # Rtg Pfaffenweg (Stuttgart Stuttgart)
	WIELANDSHÖHE_3 = 'de:08111:342:0:3'  # Zackebus Rtg Degerloch (Stuttgart Stuttgart)
	WIELANDSHÖHE_4 = 'de:08111:342:0:4'  # Zackebus Rtg Marienplatz (Stuttgart Stuttgart)
	PFAFFENWEG = 'de:08111:343'
	PFAFFENWEG_1 = 'de:08111:343:0:1'  # Rtg Wielandshöhe (Stuttgart Stuttgart)
	PFAFFENWEG_2 = 'de:08111:343:0:2'  # Rtg Liststraße (Stuttgart Stuttgart)
	PFAFFENWEG_3 = 'de:08111:343:0:3'  # Zackebus Rtg Degerloch (Stuttgart Stuttgart)
	PFAFFENWEG_4 = 'de:08111:343:0:4'  # Zackebus Rtg Marienplatz (Stuttgart Stuttgart)
	LISTSTRASSE = 'de:08111:344'
	LISTSTRASSE_1 = 'de:08111:344:0:1'  # Rtg Pfaffenweg (Stuttgart Stuttgart)
	LISTSTRASSE_2 = 'de:08111:344:0:2'  # Rtg Marienplatz (Stuttgart Stuttgart)
	LISTSTRASSE_3 = 'de:08111:344:0:3'  # Zackebus Rtg Degerloch (Stuttgart Stuttgart)
	LISTSTRASSE_4 = 'de:08111:344:0:4'  # Zackebus Rtg Marienplatz (Stuttgart Stuttgart)
	SSB_ZENTRUM = 'de:08111:350'
	SSB_ZENTRUM_1 = 'de:08111:350:0:1'  # Rtg Wallgraben (Stuttgart Stuttgart)
	SSB_ZENTRUM_2 = 'de:08111:350:0:2'  # Rtg Vaihinger Str. (Stuttgart Stuttgart)
	PLIENINGER_STRASSE_1 = 'de:08111:353:0:1'  # Rtg Sigmaringer Str. (Stuttgart Stuttgart)
	PLIENINGER_STRASSE_2 = 'de:08111:353:0:2'  # Rtg Salzäcker (Stuttgart Stuttgart)
	PLIENINGER_STRASSE_3 = 'de:08111:353:0:5'  # NBus (Stuttgart Stuttgart)
	PLIENINGER_STRASSE_4 = 'de:08116:2916:1:3'  # Rtg Rathaus/Gestütshof (Ostfildern Scharnhausen)
	PLIENINGER_STRASSE_5 = 'de:08116:2916:1:4'  # Rtg Seemühlenweg (Ostfildern Scharnhausen)
	SIGMARINGER_STRASSE = 'de:08111:354'
	SIGMARINGER_STRASSE_1 = 'de:08111:354:1:1'  # Rtg Möhringen Bf (Stuttgart Stuttgart)
	SIGMARINGER_STRASSE_2 = 'de:08111:354:1:2'  # Rtg Plieninger Str. (Stuttgart Stuttgart)
	WALLGRABEN = 'de:08111:355'
	WALLGRABEN_1 = 'de:08111:355:1:1'  # Rtg Jurastr./Lapp Kabel (Stuttgart Stuttgart)
	WALLGRABEN_2 = 'de:08111:355:1:2'  # Rtg SSB-Zentrum (Stuttgart Stuttgart)
	WALLGRABEN_3 = 'de:08111:355:1:5'  # NBus (Stuttgart Stuttgart)
	JURASTRASSE = 'de:08111:356'
	JURASTRASSE_1 = 'de:08111:356:1:1'  # Rtg Vaihingen Bf (Stuttgart Stuttgart)
	JURASTRASSE_2 = 'de:08111:356:1:2'  # Rtg Wallgraben (Stuttgart Stuttgart)
	SALZÄCKER_1 = 'de:08111:360:1:1'  # Rtg Plieninger Str. (Stuttgart Stuttgart)
	SALZÄCKER_2 = 'de:08111:360:1:2'  # Rtg Landhaus (Stuttgart Stuttgart)
	SALZÄCKER_3 = 'de:08111:360:2:3'  # Nachtbus (Stuttgart Stuttgart)
	SALZÄCKER_4 = 'de:08118:5757:0:3'  # Rtg Stuttgarter Str. (Vaihingen an der Enz Vaihingen (E))
	SALZÄCKER_5 = 'de:08118:5757:0:4'  # Rtg Vaihinger Eck (Vaihingen an der Enz Vaihingen (E))
	SALZÄCKER_6 = 'de:08118:5757:0:5'  # Hans-Krieg-Str. Rtg Kehlstr. (Vaihingen an der Enz Vaihingen (E))
	FREYTAGWEG = 'de:08111:361'
	FREYTAGWEG_1 = 'de:08111:361:0:3'  # Rtg Europaplatz (Stuttgart Stuttgart)
	FREYTAGWEG_2 = 'de:08111:361:0:4'  # Rtg Fasanenhof (Stuttgart Stuttgart)
	SCHELMENWASEN_1 = 'de:08111:363:1:1'  # Rtg Stadionstr (Stuttgart Stuttgart)
	SCHELMENWASEN_2 = 'de:08111:363:1:2'  # Rtg EnBW City (Stuttgart Stuttgart)
	SCHELMENWASEN_3 = 'de:08116:4450:0:3'  # eRtg Steinachbrücke (Nürtingen Nürtingen)
	SCHELMENWASEN_4 = 'de:08116:4450:0:4'  # eRtg Hermann-Löns-Straße (Nürtingen Nürtingen)
	FASANENHOF = 'de:08111:364'
	FASANENHOF_1 = 'de:08111:364:1:1'  # Rtg Europaplatz (Stuttgart Stuttgart)
	FASANENHOF_2 = 'de:08111:364:1:2'  # Rtg Möhringen Freibad (Stuttgart Stuttgart)
	FASANENHOF_3 = 'de:08111:364:2:3'  # Rtg Freytagweg (Stuttgart Stuttgart)
	FASANENHOF_4 = 'de:08111:364:2:4'  # Rtg Kurt-Schumacher-Str. (Stuttgart Stuttgart)
	KELTERSTRASSE_1 = 'de:08111:370:0:3'  # NBus Rtg Elsterstaffel (Stuttgart Stuttgart)
	KELTERSTRASSE_2 = 'de:08119:3741:0:3'  # Rtg. KelterStadion (Kernen im Remstal Rommelshausen)
	KELTERSTRASSE_3 = 'de:08119:3741:0:4'  # Rtg. Seestr. (Kernen im Remstal Rommelshausen)
	KELTERSTRASSE_4 = 'de:08119:3749:0:3'  # Rtg Reinhold-Maier-Str. (Remshalden Grunbach)
	KELTERSTRASSE_5 = 'de:08119:3749:0:4'  # Rtg Kirchplatz (Remshalden Grunbach)
	KELTERSTRASSE_6 = 'de:08116:5859:1:1'  # R-Bahn (Frickenhausen Frickenhausen)
	KELTERSTRASSE_7 = 'de:08118:6720:0:3'  # Bus 556/557 (Bietigheim-Bissingen Bissingen (LB))
	KELTERSTRASSE_8 = 'de:08118:6720:0:4'  # Bus 553/554/540 (Bietigheim-Bissingen Bissingen (LB))
	TUNNELSTRASSE = 'de:08111:371:0:3'  # Nachtbus (Stuttgart Stuttgart)
	VILLASTRASSE = 'de:08111:372:0:4'  # NBus auswärts (Stuttgart Stuttgart)
	MOSELSTRASSE = 'de:08111:373:0:4'  # NBus auswärts (Stuttgart Stuttgart)
	STUTTGART_MOSELSTRASSE = 'de:08111:373:0:4'  # NBus auswärts (Stuttgart Stuttgart)
	REMBRANDTSTRASSE = 'de:08111:374:0:3'  # NBus einwärts (Stuttgart Stuttgart)
	STUTTGART_REMBRANDTSTRASSE = 'de:08111:374:0:3'  # NBus einwärts (Stuttgart Stuttgart)
	BIELER_WEG = 'de:08111:375:0:3'  # NBus Rtg Schlossplatz (Stuttgart Stuttgart)
	ULMENSTRASSE_1 = 'de:08111:376:0:3'  # NBus Rtg Marienhospital (Stuttgart Stuttgart)
	ULMENSTRASSE_2 = 'de:08115:7444:0:3'  # Rtg Nordost (Sindelfingen Maichingen)
	ULMENSTRASSE_3 = 'de:08115:7444:0:4'  # Rtg Südwest (Sindelfingen Maichingen)
	WANGEN_KELTER = 'de:08111:377:0:4'  # NBus Rtg Untertürkheim (Stuttgart Stuttgart)
	HÖFINGEN = 'de:08115:1003'
	HÖFINGEN_1 = 'de:08115:1003:1:1'  # Gleis 1 (Leonberg Höfingen)
	HÖFINGEN_2 = 'de:08115:1003:2:2'  # Gleis 2 (Leonberg Höfingen)
	ZAZENHAUSEN = 'de:08111:1200'
	ZAZENHAUSEN_1 = 'de:08111:1200:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	ZAZENHAUSEN_2 = 'de:08111:1200:2:2'  # Gleis 2 (Stuttgart Stuttgart)
	SOMMERRAIN_1 = 'de:08111:1300:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	SOMMERRAIN_2 = 'de:08111:1300:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	SOMMERRAIN_3 = 'de:08111:1300:2:3'  # Gleis 3 (Stuttgart Stuttgart)
	SOMMERRAIN_4 = 'de:08111:1300:2:4'  # Gleis 4 (Stuttgart Stuttgart)
	SOMMERRAIN_5 = 'de:08111:1300:3:4'  # Linien 54/56 Endhst/Rtg Edelweißweg (Stuttgart Stuttgart)
	SOMMERRAIN_6 = 'de:08119:5338:1:3'  # eRtg Föhrenbühl/Steinbach (Backnang Backnang)
	SOMMERRAIN_7 = 'de:08119:5338:1:4'  # eRtg Getrud-Bäumer-Weg/Hintere Tos (Backnang Backnang)
	MALMSHEIM = 'de:08115:1301'
	MALMSHEIM_1 = 'de:08115:1301:1:2'  # Gleis 2 (Renningen Malmsheim)
	MALMSHEIM_2 = 'de:08115:1301:5:1'  # Gleis 1 (Renningen Malmsheim)
	RUTESHEIM = 'de:08115:1302'
	RUTESHEIM_1 = 'de:08115:1302:1:2'  # Gleis 2 (Leonberg Leonberg)
	RUTESHEIM_2 = 'de:08115:1302:2:1'  # Gleis 1 (Leonberg Leonberg)
	RUTESHEIM_3 = 'de:08115:1302:3:3'  # Rtg Heuweg (Leonberg Leonberg)
	RUTESHEIM_4 = 'de:08115:1302:3:4'  # Rtg (Leonberg Leonberg)
	WEIL_DER_STADT = 'de:08115:1303'
	WEIL_DER_STADT_1 = 'de:08115:1303:1:1'  # Gleis 1 (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_2 = 'de:08115:1303:1:2'  # Gleis 2 (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_3 = 'de:08115:1303:2:1'  # Bstg 1 (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_4 = 'de:08115:1303:2:2'  # Bstg 2 (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_5 = 'de:08115:1303:2:3'  # Bstg 3 (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_6 = 'de:08115:1303:2:4'  # Bstg 4 (Weil der Stadt Weil der Stadt)
	BIETIGHEIM = 'de:08118:1400'
	BIETIGHEIM_1 = 'de:08118:1400:1:5'  # Gleis 5 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_2 = 'de:08118:1400:1:6'  # Gleis 6 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_3 = 'de:08118:1400:1:8'  # Gleis 8 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_4 = 'de:08118:1400:2:3'  # Gleis 3 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_5 = 'de:08118:1400:2:4'  # Gleis 4 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_6 = 'de:08118:1400:3:7'  # Gleis 7 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_7 = 'de:08118:1400:4:10'  # Gleis 10 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_8 = 'de:08118:1400:5:4'  # Bstg 4 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_9 = 'de:08118:1400:5:5'  # Bstg 5 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_10 = 'de:08118:1400:6:10'  # Bstg 10 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_11 = 'de:08118:1400:6:11'  # Bstg 11 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_12 = 'de:08118:1400:6:6'  # Bstg 6 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_13 = 'de:08118:1400:6:7'  # Bstg 7 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_14 = 'de:08118:1400:6:8'  # Bstg 8 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_15 = 'de:08118:1400:6:9'  # Bstg 9 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_16 = 'de:08118:1400:7:1'  # Bstg 1 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_17 = 'de:08118:1400:7:2'  # Bstg 2 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_18 = 'de:08118:1400:7:3'  # Bstg 3 (Bietigheim-Bissingen Bietigheim)
	KORNWESTHEIM = 'de:08118:1402'
	KORNWESTHEIM_1 = 'de:08118:1402:1:1'  # Gleis 1 (Kornwestheim Kornwestheim)
	KORNWESTHEIM_2 = 'de:08118:1402:1:3'  # Gleis 3 (Kornwestheim Kornwestheim)
	KORNWESTHEIM_3 = 'de:08118:1402:2:4'  # Gleis 4 (Kornwestheim Kornwestheim)
	KORNWESTHEIM_4 = 'de:08118:1402:2:5'  # Gleis 5 (Kornwestheim Kornwestheim)
	KORNWESTHEIM_5 = 'de:08118:1402:3:7'  # Gleis 7 (Kornwestheim Kornwestheim)
	KORNWESTHEIM_6 = 'de:08118:1402:4:1'  # Bstg 1 (Kornwestheim Kornwestheim)
	KORNWESTHEIM_7 = 'de:08118:1402:4:2'  # Bstg 2 (Kornwestheim Kornwestheim)
	KORNWESTHEIM_8 = 'de:08118:1402:4:3'  # Bstg 3 (Kornwestheim Kornwestheim)
	NEUWIRTSH___PORSCHEP = 'de:08111:1403'
	NEUWIRTSH___PORSCHEP_1 = 'de:08111:1403:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	NEUWIRTSH___PORSCHEP_2 = 'de:08111:1403:2:1'  # Gleis 1 (Stuttgart Stuttgart)
	BENNINGEN__N = 'de:08118:1500'
	BENNINGEN__N_1 = 'de:08118:1500:1:2'  # Gleis 2 (Benningen am Neckar Benningen (N))
	BENNINGEN__N_2 = 'de:08118:1500:2:1'  # Gleis 1 (Benningen am Neckar Benningen (N))
	FREIBERG__N = 'de:08118:1503'
	FREIBERG__N_1 = 'de:08118:1503:1:2'  # Gleis 2 (Freiberg am Neckar Freiberg (N))
	FREIBERG__N_2 = 'de:08118:1503:1:3'  # Gleis 3 (Freiberg am Neckar Freiberg (N))
	FREIBERG__N_3 = 'de:08118:1503:2:3'  # eRtg Stuttgarter Straße (Freiberg am Neckar Freiberg (N))
	FREIBERG__N_4 = 'de:08118:1503:2:4'  # eRtg Stadtzentrum (Freiberg am Neckar Freiberg (N))
	MÜNSTER = 'de:08111:1504'
	MÜNSTER_1 = 'de:08111:1504:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	MÜNSTER_2 = 'de:08111:1504:2:3'  # Ankunft/Rtg Weserstr. (Stuttgart Stuttgart)
	NELLMERSBACH = 'de:08119:1601'
	NELLMERSBACH_1 = 'de:08119:1601:1:1'  # Gleis 1 (Leutenbach Nellmersbach)
	NELLMERSBACH_2 = 'de:08119:1601:2:2'  # Gleis 2 (Leutenbach Nellmersbach)
	NEUSTADT_HOHENACKER = 'de:08119:1602'
	NEUSTADT_HOHENACKER_1 = 'de:08119:1602:1:2'  # Gleis 2 (Waiblingen Neustadt)
	NEUSTADT_HOHENACKER_2 = 'de:08119:1602:2:1'  # Gleis 1 (Waiblingen Neustadt)
	NEUSTADT_HOHENACKER_3 = 'de:08119:1602:3:3'  # Rtg Bittenfeld Kriegsberg (Waiblingen Neustadt)
	NEUSTADT_HOHENACKER_4 = 'de:08119:1602:3:4'  # Rtg Pfarräcker (Waiblingen Neustadt)
	SCHWAIKHEIM = 'de:08119:1603'
	SCHWAIKHEIM_1 = 'de:08119:1603:1:2'  # Gleis 2 (Schwaikheim Schwaikheim)
	SCHWAIKHEIM_2 = 'de:08119:1603:2:1'  # Gleis 1 (Schwaikheim Schwaikheim)
	SCHWAIKHEIM_3 = 'de:08119:1603:3:3'  # Rtg Café Rommel (Schwaikheim Schwaikheim)
	SCHWAIKHEIM_4 = 'de:08119:1603:3:4'  # Rtg Korb Mansler Str. (Schwaikheim Schwaikheim)
	EBITZWEG = 'de:08111:1604'
	EBITZWEG_1 = 'de:08111:1604:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	EBITZWEG_2 = 'de:08111:1604:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	EBITZWEG_3 = 'de:08111:1604:3:1'  # Rtg Kienbachstr. (Stuttgart Stuttgart)
	EBITZWEG_4 = 'de:08111:1604:3:2'  # Rtg Uff-Kirchhof (Stuttgart Stuttgart)
	STETTEN_BEINSTEIN = 'de:08119:1701'
	STETTEN_BEINSTEIN_1 = 'de:08119:1701:1:2'  # Gleis 2 (Weinstadt Endersbach)
	STETTEN_BEINSTEIN_2 = 'de:08119:1701:2:1'  # Gleis 1 (Weinstadt Endersbach)
	STETTEN_BEINSTEIN_3 = 'de:08119:1701:3:4'  # eRtg Stetten (Weinstadt Endersbach)
	GERADSTETTEN = 'de:08119:1702'
	GERADSTETTEN_1 = 'de:08119:1702:1:1'  # Gleis 1 (Remshalden Geradstetten)
	GERADSTETTEN_2 = 'de:08119:1702:2:2'  # Gleis 2 (Remshalden Geradstetten)
	GRUNBACH = 'de:08119:1703'
	GRUNBACH_1 = 'de:08119:1703:1:2'  # Gleis 2 (Remshalden Grunbach)
	GRUNBACH_2 = 'de:08119:1703:1:3'  # Gleis 3 (Remshalden Grunbach)
	GRUNBACH_3 = 'de:08119:1703:2:4'  # Rtg. Geradstetten/Stgt.-/Weststr. (Remshalden Grunbach)
	GRUNBACH_4 = 'de:08119:1703:2:5'  # Rtg Austraße (Remshalden Grunbach)
	WEILER__R = 'de:08119:1704'
	WEILER__R_1 = 'de:08119:1704:1:2'  # Gleis 2 (Schorndorf Weiler (Schorndorf))
	WEILER__R_2 = 'de:08119:1704:2:1'  # Gleis 1 (Schorndorf Weiler (Schorndorf))
	WINTERBACH = 'de:08119:1705'
	WINTERBACH_1 = 'de:08119:1705:1:1'  # Gleis 1 (Winterbach Winterbach)
	WINTERBACH_2 = 'de:08119:1705:2:2'  # Gleis 2 (Winterbach Winterbach)
	ALTBACH = 'de:08116:1800'
	ALTBACH_1 = 'de:08116:1800:1:1'  # Gleis 1 (Altbach Altbach)
	ALTBACH_2 = 'de:08116:1800:1:2'  # Gleis 2 (Altbach Altbach)
	ALTBACH_3 = 'de:08116:1800:1:3'  # Gleis 3 (Altbach Altbach)
	ALTBACH_4 = 'de:08116:1800:1:4'  # Gleis 4 (Altbach Altbach)
	METTINGEN = 'de:08116:1801'
	METTINGEN_1 = 'de:08116:1801:1:1'  # Gleis 1 (Esslingen am Neckar Mettingen)
	METTINGEN_2 = 'de:08116:1801:1:2'  # Gleis 2 (Esslingen am Neckar Mettingen)
	METTINGEN_3 = 'de:08116:1801:1:3'  # Gleis 3 (Esslingen am Neckar Mettingen)
	METTINGEN_4 = 'de:08116:1801:1:4'  # Gleis 4 (Esslingen am Neckar Mettingen)
	METTINGEN_5 = 'de:08116:1801:3:5'  # Rtg. Weinstraße (Cannstatter Str.) (Esslingen am Neckar Mettingen)
	OBERESSLINGEN = 'de:08116:1802'
	OBERESSLINGEN_1 = 'de:08116:1802:1:1'  # Gleis 1 (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_2 = 'de:08116:1802:1:2'  # Gleis 2 (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_3 = 'de:08116:1802:1:3'  # Gleis 3 (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_4 = 'de:08116:1802:1:4'  # Gleis 4 (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_5 = 'de:08116:1802:2:5'  # Rtg Zell (Esslingen am Neckar Oberesslingen)
	ZELL_1 = 'de:08116:1803:1:1'  # Gleis 1 (Esslingen am Neckar Zell (Esslingen))
	ZELL_2 = 'de:08116:1803:1:2'  # Gleis 2 (Esslingen am Neckar Zell (Esslingen))
	ZELL_3 = 'de:08116:1803:1:3'  # Gleis 3 (Esslingen am Neckar Zell (Esslingen))
	ZELL_4 = 'de:08116:1803:1:4'  # Gleis 4 (Esslingen am Neckar Zell (Esslingen))
	ZELL_5 = 'de:08116:1803:2:5'  # Rtg Kirche/Johannesstr./Zentrum (Esslingen am Neckar Zell (Esslingen))
	ZELL_6 = 'de:08116:1803:2:6'  # Rtg Heussteig/Oberesslingen (Esslingen am Neckar Zell (Esslingen))
	ZELL_7 = 'de:08116:1803:2:7'  # Rtg Johannestraße (Bachstraße) (Esslingen am Neckar Zell (Esslingen))
	ZELL_8 = 'de:08119:7486:0:3'  # Rtg Dieselstr. (Oppenweiler Oppenweiler)
	ZELL_9 = 'de:08119:7486:0:4'  # Rtg Aichelbach (Oppenweiler Oppenweiler)
	FILDERSTADT = 'de:08116:1905'
	FILDERSTADT_1 = 'de:08116:1905:1:1'  # Gleis 1 (Filderstadt Bernhausen)
	FILDERSTADT_2 = 'de:08116:1905:1:2'  # Gleis 2 (Filderstadt Bernhausen)
	ELLENTAL = 'de:08118:1910'
	ELLENTAL_1 = 'de:08118:1910:1:1'  # Rtg Mühlacker (Bietigheim-Bissingen Bietigheim)
	ELLENTAL_2 = 'de:08118:1910:1:2'  # Rtg S (Bietigheim-Bissingen Bietigheim)
	ELLENTAL_3 = 'de:08118:1910:2:3'  # Rtg Badepark (Bietigheim-Bissingen Bietigheim)
	ELLENTAL_4 = 'de:08118:1910:2:4'  # Rtg Schwarzwaldstr. (Bietigheim-Bissingen Bietigheim)
	BONLANDEN_RAIFFEISENSTRASSE = 'de:08116:1997'
	BONLANDEN_RAIFFEISENSTRASSE_1 = 'de:08116:1997:0:3'  # Rtg Harthausen Linde (Filderstadt Bonlanden)
	BONLANDEN_RAIFFEISENSTRASSE_2 = 'de:08116:1997:0:4'  # Rtg Bernhausen Fleinsbach (Filderstadt Bonlanden)
	BERNHAUSEN_JOHANNESKIRCHE = 'de:08116:1998'
	BERNHAUSEN_JOHANNESKIRCHE_1 = 'de:08116:1998:0:3'  # Rtg Rosenstraße (Filderstadt Bernhausen)
	BERNHAUSEN_JOHANNESKIRCHE_2 = 'de:08116:1998:0:4'  # Rtg Echterdingen Bf (Filderstadt Bernhausen)
	JOHANNESKIRCHE_1 = 'de:08116:1998:0:3'  # Rtg Rosenstraße (Filderstadt Bernhausen)
	JOHANNESKIRCHE_2 = 'de:08116:1998:0:4'  # Rtg Echterdingen Bf (Filderstadt Bernhausen)
	JOHANNESKIRCHE_3 = 'de:08119:3666:0:3'  # eRtg Backnang (Backnang Backnang)
	JOHANNESKIRCHE_4 = 'de:08111:6268:0:3'  # eRtg Kirchtalstraße (Stuttgart Stuttgart)
	ROSENSTRASSE_1 = 'de:08116:1999:0:3'  # Rtg Rathaus (Filderstadt Bernhausen)
	ROSENSTRASSE_2 = 'de:08116:1999:0:4'  # Rtg Echterdingen Bf (Filderstadt Bernhausen)
	ROSENSTRASSE_3 = 'de:08116:5018:0:3'  # Rtg Kemnat Altes Rathaus (Ostfildern Kemnat)
	ROSENSTRASSE_4 = 'de:08116:5018:0:4'  # Rtg Kemnat Zeppelinstr. (Ostfildern Kemnat)
	ROSENSTRASSE_5 = 'de:08119:5380:1:3'  # eRtg Alle Richtungen (Auenwald Oberbrüden)
	ROSENSTRASSE_6 = 'de:08118:6903:0:3'  # eRtg Vesouler Straße (Gerlingen Gerlingen)
	ROSENSTRASSE_7 = 'de:08118:6903:0:4'  # eRtg Leonberger Straße (Gerlingen Gerlingen)
	BERNHAUSEN_ROSENSTRASSE = 'de:08116:1999'
	BERNHAUSEN_ROSENSTRASSE_1 = 'de:08116:1999:0:3'  # Rtg Rathaus (Filderstadt Bernhausen)
	BERNHAUSEN_ROSENSTRASSE_2 = 'de:08116:1999:0:4'  # Rtg Echterdingen Bf (Filderstadt Bernhausen)
	AICH_RUDOLFSHÖHE = 'de:08116:2000'
	AICH_RUDOLFSHÖHE_1 = 'de:08116:2000:0:3'  # Rtg Ort/Grötzingen (Aichtal Aich)
	AICH_RUDOLFSHÖHE_2 = 'de:08116:2000:0:4'  # Rtg Aich Stern (Aichtal Aich)
	RUDOLFSHÖHE = 'de:08116:2000'
	RUDOLFSHÖHE_1 = 'de:08116:2000:0:3'  # Rtg Ort/Grötzingen (Aichtal Aich)
	RUDOLFSHÖHE_2 = 'de:08116:2000:0:4'  # Rtg Aich Stern (Aichtal Aich)
	ORT = 'de:08116:2001'
	ORT_1 = 'de:08116:2001:0:1'  # Rtg Rudolfshöhe (Aichtal Aich)
	ORT_2 = 'de:08116:2001:0:2'  # Rtg Neckartailfingen (Aichtal Aich)
	ORT_3 = 'de:08116:2001:0:3'  # Rtg Siedlung (Aichtal Aich)
	AICH_ORT = 'de:08116:2001'
	AICH_ORT_1 = 'de:08116:2001:0:1'  # Rtg Rudolfshöhe (Aichtal Aich)
	AICH_ORT_2 = 'de:08116:2001:0:2'  # Rtg Neckartailfingen (Aichtal Aich)
	AICH_ORT_3 = 'de:08116:2001:0:3'  # Rtg Siedlung (Aichtal Aich)
	AICH_SIEDLUNG = 'de:08116:2003'
	AICH_SIEDLUNG_1 = 'de:08116:2003:0:3'  # Rtg Ort (Aichtal Aich)
	AICH_SIEDLUNG_2 = 'de:08116:2003:0:4'  # Rtg Riedwiesen (Aichtal Aich)
	BAHNHOF_1 = 'de:08116:2006:1:1'  # Bus Rtg Degerloch (Filderstadt Bernhausen)
	BAHNHOF_2 = 'de:08116:2006:1:2'  # Bus Rtg Sielmingen/Nürtingen (Filderstadt Bernhausen)
	BAHNHOF_3 = 'de:08116:2006:2:3'  # Bus Rtg Harthausen/Plattenhardt (Filderstadt Bernhausen)
	BAHNHOF_4 = 'de:08116:2006:2:4'  # Bus Rtg Musberg/Echterd./Plattenh. (Filderstadt Bernhausen)
	BAHNHOF_5 = 'de:08116:4324:0:3'  # eRtg Maierhöfe (Holzmaden Holzmaden)
	BAHNHOF_6 = 'de:08116:4324:0:4'  # eRtg Rathaus (Holzmaden Holzmaden)
	BAHNHOF_7 = 'de:08118:5602:0:3'  # Rtg Murrer Str. (Steinheim an der Murr Steinheim (M))
	BAHNHOF_8 = 'de:08118:5602:0:4'  # Rtg Kelter (Steinheim an der Murr Steinheim (M))
	BAHNHOF_9 = 'de:08117:7619:1:H1'  # Rtg Lauterstein (Donzdorf Donzdorf)
	BAHNHOF_10 = 'de:08117:7619:1:R1'  # Rtg Süßen (Donzdorf Donzdorf)
	BAHNHOF_11 = 'de:08235:10165:0:3'  # Rtg Gamp Nagold (Rohrdorf Rohrdorf (CW))
	BAHNHOF_12 = 'de:08235:10165:0:4'  # Rtg In der Misse Altensteig (Rohrdorf Rohrdorf (CW))
	BERNHAUSEN_BAHNHOF = 'de:08116:2006'
	BERNHAUSEN_BAHNHOF_1 = 'de:08116:2006:1:1'  # Bus Rtg Degerloch (Filderstadt Bernhausen)
	BERNHAUSEN_BAHNHOF_2 = 'de:08116:2006:1:2'  # Bus Rtg Sielmingen/Nürtingen (Filderstadt Bernhausen)
	BERNHAUSEN_BAHNHOF_3 = 'de:08116:2006:2:3'  # Bus Rtg Harthausen/Plattenhardt (Filderstadt Bernhausen)
	BERNHAUSEN_BAHNHOF_4 = 'de:08116:2006:2:4'  # Bus Rtg Musberg/Echterd./Plattenh. (Filderstadt Bernhausen)
	STERN_1 = 'de:08116:2007:0:3'  # eRtg Ort (Aichtal Aich)
	STERN_2 = 'de:08116:4290:0:3'  # eRtg Blumenstraße (Unterensingen Unterensingen)
	STERN_3 = 'de:08116:4290:0:4'  # eRtg Linde (Unterensingen Unterensingen)
	AICH_STERN = 'de:08116:2007:0:3'  # eRtg Ort (Aichtal Aich)
	HAUPTSTRASSE_1 = 'de:08116:2008:1:4'  # Rtg Bernhausen Bf (Filderstadt Bernhausen)
	HAUPTSTRASSE_2 = 'de:08116:2008:2:3'  # Rtg Diepoldstr. (Filderstadt Bernhausen)
	HAUPTSTRASSE_3 = 'de:08116:2019:0:3'  # Rtg Am Lindle (Filderstadt Bonlanden)
	HAUPTSTRASSE_4 = 'de:08116:2019:0:4'  # Rtg Nord (Filderstadt Bonlanden)
	HAUPTSTRASSE_5 = 'de:08118:3577:0:3'  # eRtg Kelter (Großbottwar Großbottwar)
	HAUPTSTRASSE_6 = 'de:08118:3577:0:4'  # eRtg In den Frauengärten (Großbottwar Großbottwar)
	HAUPTSTRASSE_7 = 'de:08115:4587:0:3'  # eRtg Ortsmitte (Weil der Stadt Merklingen)
	HAUPTSTRASSE_8 = 'de:08115:4587:0:4'  # eRtg Metzelwiesen (Weil der Stadt Merklingen)
	HAUPTSTRASSE_9 = 'de:08115:4631:0:3'  # eRtg Alter Friedhof (Böblingen Dagersheim)
	HAUPTSTRASSE_10 = 'de:08115:4631:0:4'  # eRtg Haus der Vereine (Böblingen Dagersheim)
	HAUPTSTRASSE_11 = 'de:08119:4882:0:3'  # eRtg Rathaus (Waiblingen Hegnach)
	HAUPTSTRASSE_12 = 'de:08119:5076:0:3'  # Rtg Heumaden/Forststr. (Kaisersbach Kaisersbach)
	HAUPTSTRASSE_13 = 'de:08119:5076:0:4'  # Rtg Bruch (Kaisersbach Kaisersbach)
	HAUPTSTRASSE_14 = 'de:08119:5134:1:3'  # Rtg Plüderhausen (Urbach Urbach)
	HAUPTSTRASSE_15 = 'de:08119:5134:1:4'  # Rtg Schorndorf (Urbach Urbach)
	HAUPTSTRASSE_16 = 'de:08119:5314:0:3'  # Rtg Weiler (Leutenbach Leutenbach)
	HAUPTSTRASSE_17 = 'de:08119:5314:0:4'  # Rtg Winnenden (Leutenbach Leutenbach)
	HAUPTSTRASSE_18 = 'de:08118:5369:0:3'  # eRtg Schweißbrücke (Marbach am Neckar Rielingshausen)
	HAUPTSTRASSE_19 = 'de:08118:5369:0:4'  # eRtg Egelsee (Marbach am Neckar Rielingshausen)
	HAUPTSTRASSE_20 = 'de:08118:5570:1:3'  # eRtg Wiegehalle/Riedbachstraße/Beihinger (Pleidelsheim Pleidelsheim)
	HAUPTSTRASSE_21 = 'de:08118:5570:1:4'  # eRtg Am Friedhof (Pleidelsheim Pleidelsheim)
	HAUPTSTRASSE_22 = 'de:08118:5573:0:3'  # Rtg Talstr. (Steinheim an der Murr Höpfigheim)
	HAUPTSTRASSE_23 = 'de:08118:5573:0:4'  # Rtg Endwiesenstr. (Steinheim an der Murr Höpfigheim)
	HAUPTSTRASSE_24 = 'de:08118:5636:0:3'  # eRtg Industriegebiet (Markgröningen Unterriexingen)
	HAUPTSTRASSE_25 = 'de:08118:5636:0:4'  # eRtg Am Türmle/Glemsbrücke (Markgröningen Unterriexingen)
	HAUPTSTRASSE_26 = 'de:08118:5677:1:3'  # Rtg Ludwigsburger Str. (Sachsenheim Großsachsenheim)
	HAUPTSTRASSE_27 = 'de:08118:5677:1:4'  # Rtg Bahnhof (Sachsenheim Großsachsenheim)
	HAUPTSTRASSE_28 = 'de:08115:5837:0:3'  # Rtg Norden (Rutesheim Perouse)
	HAUPTSTRASSE_29 = 'de:08115:5837:0:4'  # Kringel (Rutesheim Perouse)
	HAUPTSTRASSE_30 = 'de:08115:5947:1:3'  # eRtg Rathaus(Backhaus)/See (Weil im Schönbuch Weil im Schönbuch)
	HAUPTSTRASSE_31 = 'de:08115:5947:1:4'  # eRtg Troppel (Weil im Schönbuch Weil im Schönbuch)
	HAUPTSTRASSE_32 = 'de:08117:2053:0:H1'  # Rtg Friedhof (Hattenhofen Hattenhofen)
	HAUPTSTRASSE_33 = 'de:08425:2640:0:1'  # Rtg Laichingen (Laichingen Machtolsheim)
	HAUPTSTRASSE_34 = 'de:08425:2640:0:2'  # Rtg Bühlenhausen (Laichingen Machtolsheim)
	BERNHAUSEN_HAUPTSTRASSE = 'de:08116:2008'
	BERNHAUSEN_HAUPTSTRASSE_1 = 'de:08116:2008:1:4'  # Rtg Bernhausen Bf (Filderstadt Bernhausen)
	BERNHAUSEN_HAUPTSTRASSE_2 = 'de:08116:2008:2:3'  # Rtg Diepoldstr. (Filderstadt Bernhausen)
	BERNHAUSEN_KARLSTRASSE = 'de:08116:2009'
	BERNHAUSEN_KARLSTRASSE_1 = 'de:08116:2009:0:3'  # Rtg Sielmingen Schul-/Sportzentrum (Filderstadt Bernhausen)
	BERNHAUSEN_KARLSTRASSE_2 = 'de:08116:2009:0:4'  # Rtg. Bernhausen Bahnhof (Filderstadt Bernhausen)
	FRÖBELSTRASSE_1 = 'de:08116:2012:0:3'  # Rtg Diepoldstr. (Filderstadt Bernhausen)
	FRÖBELSTRASSE_2 = 'de:08116:2012:0:4'  # Rtg Nord-West-Ring (Filderstadt Bernhausen)
	FRÖBELSTRASSE_3 = 'de:08119:3652:0:3'  # eRtg Schmidener Straße (Waiblingen Waiblingen)
	FRÖBELSTRASSE_4 = 'de:08119:3652:0:4'  # eRtg Ludwigsburger Straße (Waiblingen Waiblingen)
	BERNHAUSEN_FRÖBELSTR = 'de:08116:2012'
	BERNHAUSEN_FRÖBELSTR_1 = 'de:08116:2012:0:3'  # Rtg Diepoldstr. (Filderstadt Bernhausen)
	BERNHAUSEN_FRÖBELSTR_2 = 'de:08116:2012:0:4'  # Rtg Nord-West-Ring (Filderstadt Bernhausen)
	BONLANDEN_BONLANDENER_STRASSE = 'de:08116:2015'
	BONLANDEN_BONLANDENER_STRASSE_1 = 'de:08116:2015:0:3'  # Rtg Harthausen Linde (Filderstadt Bonlanden)
	BONLANDEN_BONLANDENER_STRASSE_2 = 'de:08116:2015:0:4'  # Rtg Kreuzäcker/Gutenhalde (Filderstadt Bonlanden)
	BONLANDENER_STRASSE = 'de:08116:2015'
	BONLANDENER_STRASSE_1 = 'de:08116:2015:0:3'  # Rtg Harthausen Linde (Filderstadt Bonlanden)
	BONLANDENER_STRASSE_2 = 'de:08116:2015:0:4'  # Rtg Kreuzäcker/Gutenhalde (Filderstadt Bonlanden)
	BONLANDEN_FILDERKLINIK = 'de:08116:2016'
	BONLANDEN_FILDERKLINIK_1 = 'de:08116:2016:0:3'  # Rtg Fildorado (Filderstadt Bonlanden)
	BONLANDEN_FILDERKLINIK_2 = 'de:08116:2016:0:4'  # Rtg Uhlberg (Filderstadt Bonlanden)
	FILDERKLINIK = 'de:08116:2016'
	FILDERKLINIK_1 = 'de:08116:2016:0:3'  # Rtg Fildorado (Filderstadt Bonlanden)
	FILDERKLINIK_2 = 'de:08116:2016:0:4'  # Rtg Uhlberg (Filderstadt Bonlanden)
	BONLANDEN_FILDORADO = 'de:08116:2017'
	BONLANDEN_FILDORADO_1 = 'de:08116:2017:0:3'  # Rtg Am Lindle (Filderstadt Bonlanden)
	BONLANDEN_FILDORADO_2 = 'de:08116:2017:0:4'  # Rtg Filderklinik (Filderstadt Bonlanden)
	FILDORADO = 'de:08116:2017'
	FILDORADO_1 = 'de:08116:2017:0:3'  # Rtg Am Lindle (Filderstadt Bonlanden)
	FILDORADO_2 = 'de:08116:2017:0:4'  # Rtg Filderklinik (Filderstadt Bonlanden)
	BONLANDEN_GUTENHALDE = 'de:08116:2018'
	BONLANDEN_GUTENHALDE_1 = 'de:08116:2018:0:3'  # Rtg Aich Riedwiesen (Filderstadt Bonlanden)
	BONLANDEN_GUTENHALDE_2 = 'de:08116:2018:0:4'  # Rtg Bonlandener Str. (Filderstadt Bonlanden)
	GUTENHALDE = 'de:08116:2018'
	GUTENHALDE_1 = 'de:08116:2018:0:3'  # Rtg Aich Riedwiesen (Filderstadt Bonlanden)
	GUTENHALDE_2 = 'de:08116:2018:0:4'  # Rtg Bonlandener Str. (Filderstadt Bonlanden)
	BONLANDEN_HAUPTSTRASSE = 'de:08116:2019'
	BONLANDEN_HAUPTSTRASSE_1 = 'de:08116:2019:0:3'  # Rtg Am Lindle (Filderstadt Bonlanden)
	BONLANDEN_HAUPTSTRASSE_2 = 'de:08116:2019:0:4'  # Rtg Nord (Filderstadt Bonlanden)
	REUSSENSTEINWEG_1 = 'de:08116:2020:0:3'  # Bus Rtg Echterdingen Bf (Filderstadt Bonlanden)
	REUSSENSTEINWEG_2 = 'de:08116:2020:0:4'  # Bus Rtg Degerloch (Filderstadt Bonlanden)
	REUSSENSTEINWEG_3 = 'de:08116:3908:0:1'  # Rtg Talbachbrücke (Hochdorf Hochdorf (ES))
	REUSSENSTEINWEG_4 = 'de:08116:3908:0:2'  # Rtg Reichenbach (Hochdorf Hochdorf (ES))
	BONLANDEN_REUSSENSTEINWEG = 'de:08116:2020'
	BONLANDEN_REUSSENSTEINWEG_1 = 'de:08116:2020:0:3'  # Bus Rtg Echterdingen Bf (Filderstadt Bonlanden)
	BONLANDEN_REUSSENSTEINWEG_2 = 'de:08116:2020:0:4'  # Bus Rtg Degerloch (Filderstadt Bonlanden)
	BONLANDEN_NORD = 'de:08116:2021'
	BONLANDEN_NORD_1 = 'de:08116:2021:1:3'  # Rtg Hauptstr./Rainäckerstr. (Filderstadt Bonlanden)
	BONLANDEN_NORD_2 = 'de:08116:2021:1:4'  # Rtg Bernhausen Fleinsbach (Filderstadt Bonlanden)
	NORD_1 = 'de:08116:2021:1:3'  # Rtg Hauptstr./Rainäckerstr. (Filderstadt Bonlanden)
	NORD_2 = 'de:08116:2021:1:4'  # Rtg Bernhausen Fleinsbach (Filderstadt Bonlanden)
	NORD_3 = 'de:08115:3108:0:3'  # Rtg See (Weil im Schönbuch Weil im Schönbuch)
	NORD_4 = 'de:08115:3108:0:4'  # Rtg Breitenstein / Neuweiler (Weil im Schönbuch Weil im Schönbuch)
	NORD_5 = 'de:08115:3197:1:3'  # Rtg Landhaussiedlung (Sindelfingen Maichingen)
	NORD_6 = 'de:08115:3197:1:4'  # Rtg Solo (Sindelfingen Maichingen)
	NORD_7 = 'de:08115:3197:2:1'  # Gleis 1 (Sindelfingen Maichingen)
	NORD_8 = 'de:08115:3197:2:2'  # Gleis 2 (Sindelfingen Maichingen)
	NORD_9 = 'de:08119:5801:1:1'  # R-Bahn (Rudersberg Rudersberg)
	NORD_10 = 'de:08119:5801:2:3'  # Richtung Oberndorf Bf/Brunnenplatz (Rudersberg Rudersberg)
	NORD_11 = 'de:08119:5801:2:4'  # Richtung Rudersberg Brühlstr. (Rudersberg Rudersberg)
	NORD_12 = 'de:08119:5801:2:5'  # Rtg Rudersberg Schulzentrum (Rudersberg Rudersberg)
	NORD_13 = 'de:08118:6492:0:3'  # Rtg Rathausplatz (Ditzingen Hirschlanden)
	NORD_14 = 'de:08118:6492:0:4'  # Rtg Mercedesstr. (Ditzingen Hirschlanden)
	BONLANDEN_ROTZEIL = 'de:08116:2022'
	BONLANDEN_ROTZEIL_1 = 'de:08116:2022:0:3'  # Rtg Plattenhardt Industriegebiet (Filderstadt Bonlanden)
	BONLANDEN_ROTZEIL_2 = 'de:08116:2022:0:4'  # Rtg Marktstr. (Filderstadt Bonlanden)
	ROTZEIL = 'de:08116:2022'
	ROTZEIL_1 = 'de:08116:2022:0:3'  # Rtg Plattenhardt Industriegebiet (Filderstadt Bonlanden)
	ROTZEIL_2 = 'de:08116:2022:0:4'  # Rtg Marktstr. (Filderstadt Bonlanden)
	HARTHAUSEN_LINDE = 'de:08116:2023'
	HARTHAUSEN_LINDE_1 = 'de:08116:2023:0:3'  # Rtg Kirche (Filderstadt Harthausen)
	HARTHAUSEN_LINDE_2 = 'de:08116:2023:0:4'  # Rtg Bonl. Raiffeisenstr./Bonlandener Str (Filderstadt Harthausen)
	LINDE_1 = 'de:08116:2023:0:3'  # Rtg Kirche (Filderstadt Harthausen)
	LINDE_2 = 'de:08116:2023:0:4'  # Rtg Bonl. Raiffeisenstr./Bonlandener Str (Filderstadt Harthausen)
	LINDE_3 = 'de:08116:4089:0:3'  # eRtg Betz (Köngen Köngen)
	LINDE_4 = 'de:08116:4093:0:3'  # eRtg Stern (Unterensingen Unterensingen)
	LINDE_5 = 'de:08116:4093:0:4'  # eRtg Zizishausen (Unterensingen Unterensingen)
	LINDE_6 = 'de:08116:4493:1:3'  # Rtg Metzingen (Neuffen Kappishäusern)
	LINDE_7 = 'de:08116:4493:1:4'  # Rtg Neuffen/Kohlberg (Neuffen Kappishäusern)
	LINDE_8 = 'de:08116:4493:1:5'  # Rtg Kappishäusern (Neuffen Kappishäusern)
	LINDE_9 = 'de:08117:2101:0:H1'  # Rtg Heimbach (Göppingen Jebenhausen)
	LINDE_10 = 'de:08117:2101:0:R1'  # Rtg Herdweg (Göppingen Jebenhausen)
	KIRCHE_1 = 'de:08116:2024:1:3'  # Rtg Rossbergstr. (Filderstadt Harthausen)
	KIRCHE_2 = 'de:08116:2024:1:4'  # Rtg Linde (Filderstadt Harthausen)
	KIRCHE_3 = 'de:08116:2108:0:3'  # Rtg Musberg Friedhof/Wanderweg (Leinfelden-Echterdingen Musberg)
	KIRCHE_4 = 'de:08116:2108:0:4'  # Rtg Sonnenhalde (Leinfelden-Echterdingen Musberg)
	KIRCHE_5 = 'de:08118:2638:0:3'  # Rtg Kallenberg Lackfabrik (612A) (Korntal-Münchingen Münchingen)
	KIRCHE_6 = 'de:08116:2932:0:3'  # Rtg Ruit (Ostfildern Ruit)
	KIRCHE_7 = 'de:08116:2932:0:4'  # L.35 Rtg Scharnh.Str./L.131 Rtg Zinsholz (Ostfildern Ruit)
	KIRCHE_8 = 'de:08119:3710:1:3'  # eRtg Pflegeheim (Schorndorf Weiler (Schorndorf))
	KIRCHE_9 = 'de:08119:3710:1:4'  # eRtg Rathaus/Pfarrstraße (Schorndorf Weiler (Schorndorf))
	KIRCHE_10 = 'de:08116:4102:0:3'  # eRtg Kriegerdenkmal (Esslingen am Neckar Liebersbronn)
	KIRCHE_11 = 'de:08116:4102:0:4'  # eRtg Neue Straße (Esslingen am Neckar Liebersbronn)
	KIRCHE_12 = 'de:08116:4340:0:3'  # eRtg Mörikeschule (Kirchheim unter Teck Ötlingen)
	KIRCHE_13 = 'de:08119:4501:0:3'  # Rtg Schlichten Strutstr. (Schorndorf Schlichten)
	KIRCHE_14 = 'de:08119:4501:0:4'  # Rtg Thomashardt Abzw. Baiereck (Schorndorf Schlichten)
	KIRCHE_15 = 'de:08116:4505:0:3'  # eRtg Thomashardter Straße (Lichtenwald Hegenlohe)
	KIRCHE_16 = 'de:08116:4505:0:4'  # eRtg Bannmühle (Lichtenwald Hegenlohe)
	KIRCHE_17 = 'de:08115:4632:1:3'  # eRtg Aidlinger Straße (Böblingen Dagersheim)
	KIRCHE_18 = 'de:08115:4632:1:4'  # eRtg Schützenweg (Böblingen Dagersheim)
	KIRCHE_19 = 'de:08115:4743:0:3'  # Rtg Sindlinger Str. (Herrenberg Haslach)
	KIRCHE_20 = 'de:08115:4743:0:4'  # Rtg Randenstr. (Herrenberg Haslach)
	KIRCHE_21 = 'de:08115:4835:0:3'  # Rtg Erholungsheim (Herrenberg Gültstein)
	KIRCHE_22 = 'de:08115:4835:0:4'  # Rtg Tailfingen (Herrenberg Gültstein)
	KIRCHE_23 = 'de:08115:4846:0:3'  # eRtg Nelkenstraße (Herrenberg Affstätt)
	KIRCHE_24 = 'de:08116:5000:0:3'  # eRtg Ort (Aichtal Neuenhaus)
	KIRCHE_25 = 'de:08116:5000:0:4'  # eRtg Grörach/Häfnerstraße (Aichtal Neuenhaus)
	KIRCHE_26 = 'de:08118:5712:0:3'  # eRtg Buchenrain (Sachsenheim Ochsenbach)
	KIRCHE_27 = 'de:08118:5712:0:4'  # eRtg Ochsen (Sachsenheim Ochsenbach)
	KIRCHE_28 = 'de:08115:7106:1:3'  # Rtg Waldenbuch Hasenhof (Steinenbronn Steinenbronn)
	KIRCHE_29 = 'de:08115:7106:1:4'  # Rtg Im Alten See/Schönaich Sulzbachtal (Steinenbronn Steinenbronn)
	KIRCHE_30 = 'de:08117:66:0:H1'  # Rtg Bahnhofstraße (Mühlhausen im Täle Mühlhausen im Täle)
	KIRCHE_31 = 'de:08117:3018:0:H1'  # Rtg Bachstr. (Schlat Schlat)
	KIRCHE_32 = 'de:08117:3018:0:R1'  # Rtg Enzbachweg (Schlat Schlat)
	KIRCHE_33 = 'de:08117:3323:0:H1'  # Rtg Krzg. Gruib. Auend. (Gammelshausen Gammelshausen)
	KIRCHE_34 = 'de:08117:3323:0:R1'  # Rtg Gam. Kreisverkehr (Gammelshausen Gammelshausen)
	KIRCHE_35 = 'de:08117:3501:0:H1'  # Ankunft (Hohenstadt Hohenstadt (GP))
	KIRCHE_36 = 'de:08117:7406:0:H1'  # Rtg Schlierbach (Ebersbach an der Fils Roßwälden)
	KIRCHE_37 = 'de:08117:7406:0:R1'  # Rtg Ebersbach (Ebersbach an der Fils Roßwälden)
	KIRCHE_38 = 'de:08117:7682:0:2'  # Rtg Rathaus (Lauterstein Nenningen)
	KIRCHE_39 = 'de:08117:7682:0:R1'  # Rtg Donzdorf (Lauterstein Nenningen)
	KIRCHE_40 = 'de:08117:7803:0:H1'  # eRtg Bachbrücke (Schlierbach Schlierbach)
	KIRCHE_41 = 'de:08117:7803:0:R1'  # eRtg Hattenhofer Straße (Schlierbach Schlierbach)
	KIRCHE_42 = 'de:08136:7037:0:1'  # Rtg. Weitmars Rechbergstr. (Lorch Weitmars)
	KIRCHE_43 = 'de:08425:2641:0:1'  # auswärts (Laichingen Machtolsheim)
	HARTHAUSEN_KIRCHE = 'de:08116:2024'
	HARTHAUSEN_KIRCHE_1 = 'de:08116:2024:1:3'  # Rtg Rossbergstr. (Filderstadt Harthausen)
	HARTHAUSEN_KIRCHE_2 = 'de:08116:2024:1:4'  # Rtg Linde (Filderstadt Harthausen)
	DRIEFBRUNNEN = 'de:08116:2025'
	DRIEFBRUNNEN_1 = 'de:08116:2025:0:3'  # Rtg Kronenbrunnen/Weilerhau (von Westen) (Filderstadt Plattenhardt)
	DRIEFBRUNNEN_2 = 'de:08116:2025:0:4'  # Rtg Waldstr/Weilerhau (von Osten) (Filderstadt Plattenhardt)
	PLATTENHARDT_DRIEFBRUNNEN = 'de:08116:2025'
	PLATTENHARDT_DRIEFBRUNNEN_1 = 'de:08116:2025:0:3'  # Rtg Kronenbrunnen/Weilerhau (von Westen) (Filderstadt Plattenhardt)
	PLATTENHARDT_DRIEFBRUNNEN_2 = 'de:08116:2025:0:4'  # Rtg Waldstr/Weilerhau (von Osten) (Filderstadt Plattenhardt)
	PLATTENHARDT_INDUSTRIEGEBIET = 'de:08116:2026'
	PLATTENHARDT_INDUSTRIEGEBIET_1 = 'de:08116:2026:0:3'  # Rtg Plattenhardt Rathaus (Filderstadt Plattenhardt)
	PLATTENHARDT_INDUSTRIEGEBIET_2 = 'de:08116:2026:0:4'  # Rtg Bonlanden Rotzeil (Filderstadt Plattenhardt)
	INDUSTRIEGEBIET_1 = 'de:08116:2026:0:3'  # Rtg Plattenhardt Rathaus (Filderstadt Plattenhardt)
	INDUSTRIEGEBIET_2 = 'de:08116:2026:0:4'  # Rtg Bonlanden Rotzeil (Filderstadt Plattenhardt)
	INDUSTRIEGEBIET_3 = 'de:08116:4154:0:3'  # eRtg Industriegebiet Nord/Kreuzung (Aichwald Aichschieß)
	INDUSTRIEGEBIET_4 = 'de:08116:4154:0:4'  # eRtg Rotes Kreuz (Aichwald Aichschieß)
	INDUSTRIEGEBIET_5 = 'de:08116:4287:0:3'  # Rtg Unterensingen (Wendlingen am Neckar Wendlingen (N))
	INDUSTRIEGEBIET_6 = 'de:08116:4287:0:4'  # Rtg Wendlingen (Wendlingen am Neckar Wendlingen (N))
	INDUSTRIEGEBIET_7 = 'de:08116:4410:0:3'  # eRtg Gartenstraße (Kirchheim unter Teck Nabern)
	INDUSTRIEGEBIET_8 = 'de:08116:4410:0:4'  # eRtg Katholische Kirche (Kirchheim unter Teck Nabern)
	INDUSTRIEGEBIET_9 = 'de:08118:5635:0:3'  # Rtg Hauptstr. (Markgröningen Unterriexingen)
	INDUSTRIEGEBIET_10 = 'de:08118:5635:0:4'  # Rtg Markgröningen (Markgröningen Unterriexingen)
	INDUSTRIEGEBIET_11 = 'de:08119:5968:0:3'  # eRtg Lehnenberg Kreuzung (Winnenden Birkmannsweiler)
	INDUSTRIEGEBIET_12 = 'de:08119:5968:0:4'  # eRtg Ortsmitte (Winnenden Birkmannsweiler)
	INDUSTRIEGEBIET_13 = 'de:08116:7101:0:3'  # Rtg Bahnhof (Leinfelden-Echterdingen Echterdingen)
	INDUSTRIEGEBIET_14 = 'de:08116:7101:0:4'  # Rtg Fasanenhof Abzw. (Leinfelden-Echterdingen Echterdingen)
	INDUSTRIEGEBIET_15 = 'de:08117:2023:0:H2'  # Linie 178/N91 (Albershausen Albershausen)
	INDUSTRIEGEBIET_16 = 'de:08117:2023:0:R1'  # Linie 911,914 Rtg Albershausen (Albershausen Albershausen)
	INDUSTRIEGEBIET_17 = 'de:08117:4036:0:H1'  # Rtg Breech Krzg. Rattenh. (Börtlingen Börtlingen)
	INDUSTRIEGEBIET_18 = 'de:08117:4036:0:R1'  # Rtg Friedhof (Börtlingen Börtlingen)
	PLATTENHARDT_SCHNECKEN = 'de:08116:2027'
	PLATTENHARDT_SCHNECKEN_1 = 'de:08116:2027:0:3'  # Bus - Li.32/35/77 Rtg HCZS/ESBF/BLIN (Filderstadt Plattenhardt)
	PLATTENHARDT_SCHNECKEN_2 = 'de:08116:2027:0:4'  # Rtg Rathaus (Filderstadt Plattenhardt)
	SCHNECKEN = 'de:08116:2027'
	SCHNECKEN_1 = 'de:08116:2027:0:3'  # Bus - Li.32/35/77 Rtg HCZS/ESBF/BLIN (Filderstadt Plattenhardt)
	SCHNECKEN_2 = 'de:08116:2027:0:4'  # Rtg Rathaus (Filderstadt Plattenhardt)
	PLATTENHARDT_RATHAUS = 'de:08116:2028'
	PLATTENHARDT_RATHAUS_1 = 'de:08116:2028:1:3'  # Rtg Krone/Kronenbrunnen/Schillerstr. (Filderstadt Plattenhardt)
	PLATTENHARDT_RATHAUS_2 = 'de:08116:2028:1:4'  # Rtg Schönbuchstr./Industriegebiet (Filderstadt Plattenhardt)
	RATHAUS_1 = 'de:08116:2028:1:3'  # Rtg Krone/Kronenbrunnen/Schillerstr. (Filderstadt Plattenhardt)
	RATHAUS_2 = 'de:08116:2028:1:4'  # Rtg Schönbuchstr./Industriegebiet (Filderstadt Plattenhardt)
	RATHAUS_3 = 'de:08116:2036:1:3'  # Rtg Kantstr. (Filderstadt Sielmingen)
	RATHAUS_4 = 'de:08116:2036:1:4'  # Rtg Bernh. Karlstr. (Filderstadt Sielmingen)
	RATHAUS_5 = 'de:08116:2044:0:3'  # Rtg Bahnhof (Filderstadt Bernhausen)
	RATHAUS_6 = 'de:08116:2044:0:4'  # Rtg Rosenstraße (Filderstadt Bernhausen)
	RATHAUS_7 = 'de:08116:2066:0:3'  # Rtg Schule (Altenriet Altenriet)
	RATHAUS_8 = 'de:08116:2066:0:4'  # Rtg Schule Bernhausen (Altenriet Altenriet)
	RATHAUS_9 = 'de:08119:2372:0:3'  # Rtg Alemannenstr. (Fellbach Oeffingen)
	RATHAUS_10 = 'de:08119:2377:1:4'  # Rtg Dorfwiesen (Fellbach Schmiden)
	RATHAUS_11 = 'de:08119:2377:1:6'  # Rtg Bühnerstr. (Fellbach Schmiden)
	RATHAUS_12 = 'de:08116:2820:0:3'  # eRtg Linde (Neuffen Kappishäusern)
	RATHAUS_13 = 'de:08116:2820:0:4'  # eRtg Feuersee (Neuffen Kappishäusern)
	RATHAUS_14 = 'de:08116:2917:1:1'  # Rtg Plieninger Str. (Ostfildern Scharnhausen)
	RATHAUS_15 = 'de:08116:2917:1:2'  # Rtg Neuhausen Zabergäustr. (Ostfildern Scharnhausen)
	RATHAUS_16 = 'de:08116:2917:2:3'  # Rtg Lenzhalde (Ostfildern Scharnhausen)
	RATHAUS_17 = 'de:08116:2917:2:4'  # Rtg Plieninger Str. (Ostfildern Scharnhausen)
	RATHAUS_18 = 'de:08115:3209:0:3'  # eRtg Hallenbad (Sindelfingen Maichingen)
	RATHAUS_19 = 'de:08115:3209:0:4'  # eRtg Maichingen (Sindelfingen Maichingen)
	RATHAUS_20 = 'de:08118:3419:2:3'  # eRtg ZOB (Ludwigsburg Ludwigsburg)
	RATHAUS_21 = 'de:08118:3419:3:4'  # eRtg Residenzschl./Blühendes Barock/LRA (Ludwigsburg Ludwigsburg)
	RATHAUS_22 = 'de:08118:3515:0:3'  # eRtg Turnhalle/Bahnhofstraße (Erdmannhausen Erdmannhausen)
	RATHAUS_23 = 'de:08118:3515:0:4'  # eRtg Waage (Erdmannhausen Erdmannhausen)
	RATHAUS_24 = 'de:08119:3527:0:3'  # Richtung Bahnhof (Kirchberg an der Murr Kirchberg (M))
	RATHAUS_25 = 'de:08119:3527:0:4'  # Richtung Schule (Kirchberg an der Murr Kirchberg (M))
	RATHAUS_26 = 'de:08118:3590:0:3'  # eRtg Katharinenstraße (Vaihingen an der Enz Gündelbach)
	RATHAUS_27 = 'de:08119:3615:0:3'  # eRtg Bottwarstraße (Backnang Waldrems)
	RATHAUS_28 = 'de:08119:3615:0:4'  # eRtg Schozachstraße (Backnang Waldrems)
	RATHAUS_29 = 'de:08119:3669:0:3'  # Rtg Isarstr. (Backnang Maubach)
	RATHAUS_30 = 'de:08119:3669:0:4'  # Rtg Maubach (Backnang Maubach)
	RATHAUS_31 = 'de:08119:3702:1:3'  # Rtg Geradstetten Mitte (Remshalden Geradstetten)
	RATHAUS_32 = 'de:08119:3702:1:4'  # Rtg D.-Bonhoeffer-Str. (Remshalden Geradstetten)
	RATHAUS_33 = 'de:08119:3744:0:3'  # Rtg Rumold-Realschule (Kernen im Remstal Rommelshausen)
	RATHAUS_34 = 'de:08119:3768:0:3'  # Rtg Ortsmitte (Weinstadt Endersbach)
	RATHAUS_35 = 'de:08119:3768:0:4'  # Rtg Pestalozzistr. (Weinstadt Endersbach)
	RATHAUS_36 = 'de:08116:3803:1:3'  # Rtg Haldenrainweg/Losburgstr. (Altbach Altbach)
	RATHAUS_37 = 'de:08116:3808:0:3'  # Rtg Kreissparkasse (Baltmannsweiler Baltmannsweiler)
	RATHAUS_38 = 'de:08116:4057:1:3'  # eRtg Schule (Deizisau Deizisau)
	RATHAUS_39 = 'de:08116:4057:1:4'  # Rtg Alte Bergstraße (Deizisau Deizisau)
	RATHAUS_40 = 'de:08116:4057:1:5'  # Rtg Esslingen (Deizisau Deizisau)
	RATHAUS_41 = 'de:08116:4088:0:6'  # Ersatzhalt Li 151 (Köngen Köngen)
	RATHAUS_42 = 'de:08116:4166:0:3'  # eRtg Krummhardt Ort (Aichwald Aichelberg (Aichwald))
	RATHAUS_43 = 'de:08115:4210:0:3'  # Rtg Germanenstraße (Holzgerlingen Holzgerlingen)
	RATHAUS_44 = 'de:08115:4210:0:4'  # Rtg Holzgerlingen (Holzgerlingen Holzgerlingen)
	RATHAUS_45 = 'de:08116:4218:0:3'  # Rtg Notzingen Ost (Notzingen Notzingen)
	RATHAUS_46 = 'de:08116:4218:0:4'  # Rtg Wernau/Kirchheim/Hochdorf (Notzingen Notzingen)
	RATHAUS_47 = 'de:08116:4222:0:1'  # Rtg Talbachbrücke (Hochdorf Hochdorf (ES))
	RATHAUS_48 = 'de:08116:4222:0:2'  # Rtg Aspen (Hochdorf Hochdorf (ES))
	RATHAUS_49 = 'de:08116:4222:0:3'  # Rtg Ulmer Str/Weberei (Hochdorf Hochdorf (ES))
	RATHAUS_50 = 'de:08116:4284:0:3'  # eRtg Hochwang Rathaus/Bahnhofstraße (Lenningen Unterlenningen)
	RATHAUS_51 = 'de:08116:4284:0:4'  # eRtg Oberlenningen (Lenningen Unterlenningen)
	RATHAUS_52 = 'de:08116:4316:1:3'  # eRtg Lindenstraße (Bempflingen Bempflingen)
	RATHAUS_53 = 'de:08116:4316:1:4'  # eRtg Riederich Stuttg. Straße (Bempflingen Bempflingen)
	RATHAUS_54 = 'de:08116:4323:0:3'  # Rtg Holzmaden (Kirchheim unter Teck Jesingen)
	RATHAUS_55 = 'de:08116:4323:0:4'  # Rtg Kirchheim von Holzmaden (Kirchheim unter Teck Jesingen)
	RATHAUS_56 = 'de:08116:4323:0:5'  # Rtg Ohmden (Kirchheim unter Teck Jesingen)
	RATHAUS_57 = 'de:08116:4323:0:6'  # Rtg Kirchheim von Ohmden (Kirchheim unter Teck Jesingen)
	RATHAUS_58 = 'de:08116:4391:0:3'  # eRtg Zeller Straße (Ohmden Ohmden)
	RATHAUS_59 = 'de:08116:4391:0:4'  # eRtg Friedhof (Ohmden Ohmden)
	RATHAUS_60 = 'de:08116:4405:0:3'  # eRtg Alle Richtungen (Weilheim an der Teck Hepsisau)
	RATHAUS_61 = 'de:08116:4415:0:3'  # Rtg See (Bissingen an der Teck Bissingen an der Teck (T))
	RATHAUS_62 = 'de:08116:4415:0:4'  # Rtg Wilhelmstraße Grundschule (Bissingen an der Teck Bissingen an der Teck (T))
	RATHAUS_63 = 'de:08116:4417:0:3'  # eRtg Rathaus (Bissingen an der Teck Ochsenwang)
	RATHAUS_64 = 'de:08116:4424:0:3'  # Rtg Reußensteinstraße (Lenningen Schopfloch (Lenningen))
	RATHAUS_65 = 'de:08116:4424:0:4'  # Rtg Kreislerstraße (Lenningen Schopfloch (Lenningen))
	RATHAUS_66 = 'de:08116:4465:0:3'  # eRtg Grafenberger-Metzinger Straße (Bempflingen Kleinbettlingen)
	RATHAUS_67 = 'de:08116:4466:0:3'  # eRtg Frickenhäuser Straße (Frickenhausen Tischardt)
	RATHAUS_68 = 'de:08116:4466:0:4'  # eRtg Tischardter Straße (Frickenhausen Tischardt)
	RATHAUS_69 = 'de:08116:4480:0:3'  # eRtg Halde (Nürtingen Neckarhausen)
	RATHAUS_70 = 'de:08116:4480:0:4'  # eRtg Verdistraße/Neckarhausener Straße (Nürtingen Neckarhausen)
	RATHAUS_71 = 'de:08116:4483:1:3'  # eRtg Nürtinger Straße (Neckartailfingen Neckartailfingen)
	RATHAUS_72 = 'de:08116:4483:1:4'  # eRtg Tübinger Straße / Eichendorffstraße (Neckartailfingen Neckartailfingen)
	RATHAUS_73 = 'de:08116:4487:0:3'  # eRtg Marktplatz (Neckartenzlingen Neckartenzlingen)
	RATHAUS_74 = 'de:08116:4487:0:4'  # eRtg Auwiesenschule (Neckartenzlingen Neckartenzlingen)
	RATHAUS_75 = 'de:08115:4509:0:3'  # A Rtg Bondorf (Mötzingen Mötzingen)
	RATHAUS_76 = 'de:08115:4509:0:4'  # B Rtg Lindenhof/Goethestr. (Mötzingen Mötzingen)
	RATHAUS_77 = 'de:08115:4509:0:5'  # C Rtg Nagold (Mötzingen Mötzingen)
	RATHAUS_78 = 'de:08118:4526:0:3'  # Rtg Hemmingen (Hemmingen Hemmingen)
	RATHAUS_79 = 'de:08118:4526:0:4'  # Rtg Schwieberdingen (Hemmingen Hemmingen)
	RATHAUS_80 = 'de:08115:4550:0:3'  # Rtg Carl-Zeiss-Straße (Leonberg Gebersheim)
	RATHAUS_81 = 'de:08115:4550:0:4'  # Rtg Rutesheim Dieselstraße (Leonberg Gebersheim)
	RATHAUS_82 = 'de:08115:4585:0:3'  # eRtg Dätzinger Weg (Grafenau Dätzingen)
	RATHAUS_83 = 'de:08115:4585:0:4'  # eRtg Schule (Grafenau Dätzingen)
	RATHAUS_84 = 'de:08115:4592:0:3'  # Rtg Im Tal (Weil der Stadt Münklingen)
	RATHAUS_85 = 'de:08115:4592:0:4'  # Rtg Neuhauser Str. (Weil der Stadt Münklingen)
	RATHAUS_86 = 'de:08115:4727:1:3'  # Rtg Feuerwehrgerätehaus (Gärtringen Gärtringen)
	RATHAUS_87 = 'de:08115:4727:1:4'  # Rtg Gärtringen (Gärtringen Gärtringen)
	RATHAUS_88 = 'de:08115:4729:0:3'  # eRtg Jennerstraße/Gärtringen (Gärtringen Rohrau)
	RATHAUS_89 = 'de:08115:4747:0:3'  # Rtg Post (Gäufelden Öschelbronn (Gäufelden))
	RATHAUS_90 = 'de:08115:4747:0:4'  # Rtg Stuttgarter Straße (Gäufelden Öschelbronn (Gäufelden))
	RATHAUS_91 = 'de:08115:4747:0:5'  # Ersatz Rtg. Stgt. Str. (Gäufelden Öschelbronn (Gäufelden))
	RATHAUS_92 = 'de:08115:4770:0:3'  # eRtg Kapellenberg (Grafenau Döffingen)
	RATHAUS_93 = 'de:08115:4770:0:4'  # eRtg Ulrichweg/Seeger (Grafenau Döffingen)
	RATHAUS_94 = 'de:08115:4777:0:3'  # eRtg Katholische Kirche (Aidlingen Aidlingen)
	RATHAUS_95 = 'de:08115:4777:0:4'  # eRtg Furthmühle (Aidlingen Aidlingen)
	RATHAUS_96 = 'de:08115:4780:0:3'  # eRtg Bergsiedlung/Backhaus (Aidlingen Deufringen)
	RATHAUS_97 = 'de:08115:4780:0:4'  # eRtg Aidlinger Straße (Aidlingen Deufringen)
	RATHAUS_98 = 'de:08115:4788:0:3'  # eRtg Alle Richtungen (Weil im Schönbuch Breitenstein)
	RATHAUS_99 = 'de:08115:4820:0:3'  # eRtg Hennersdorfer Straße (Weil im Schönbuch Neuweiler)
	RATHAUS_100 = 'de:08115:4831:0:3'  # Rtg Obere Äcker (Herrenberg Kayh)
	RATHAUS_101 = 'de:08115:4831:0:4'  # Rtg Herrenberger Straße (Herrenberg Kayh)
	RATHAUS_102 = 'de:08115:4838:0:3'  # eRtg Brühlstraße (Deckenpfronn Deckenpfronn)
	RATHAUS_103 = 'de:08115:4838:0:4'  # eRtg Gültingen/Lerchenberg (Deckenpfronn Deckenpfronn)
	RATHAUS_104 = 'de:08115:4847:0:3'  # eRtg Jennerstraße (Herrenberg Affstätt)
	RATHAUS_105 = 'de:08115:4862:0:3'  # Rtg Altingen Kirche (Gäufelden Tailfingen)
	RATHAUS_106 = 'de:08115:4862:0:4'  # Rtg Maueräcker (Gäufelden Tailfingen)
	RATHAUS_107 = 'de:08119:4879:0:3'  # eRtg Katholische Kirche (Waiblingen Hegnach)
	RATHAUS_108 = 'de:08119:4965:0:3'  # eRtg Abzweig Neufürstenhütte (Großerlach Großerlach)
	RATHAUS_109 = 'de:08119:4965:0:4'  # eRtg Mainhardt (Großerlach Großerlach)
	RATHAUS_110 = 'de:08119:4976:0:3'  # eRtg Friedhof (Auenwald Unterbrüden)
	RATHAUS_111 = 'de:08119:4995:1:3'  # Rtg Ebniseestr. (Althütte Althütte)
	RATHAUS_112 = 'de:08119:4995:1:4'  # Rtg Ortsmitte (Althütte Althütte)
	RATHAUS_113 = 'de:08116:5008:0:3'  # Rtg Neuhäuser Str. (Denkendorf Denkendorf)
	RATHAUS_114 = 'de:08119:5031:0:3'  # eRtg Schozachstraße (Backnang Heiningen (Backnang))
	RATHAUS_115 = 'de:08119:5031:0:4'  # eRtg Marbacher Straße (Backnang Heiningen (Backnang))
	RATHAUS_116 = 'de:08119:5038:0:3'  # eRtg Heutensbacher Straße (Allmersbach im Tal Heutensbach)
	RATHAUS_117 = 'de:08119:5107:1:3'  # Rtg Mitte/Miedelsbacher Str. (Schorndorf Haubersbronn)
	RATHAUS_118 = 'de:08119:5107:1:4'  # eRtg Haubersbronn/Schorndorf (Schorndorf Haubersbronn)
	RATHAUS_119 = 'de:08119:5148:0:3'  # Rtg Torstr. (Waiblingen Neustadt)
	RATHAUS_120 = 'de:08119:5148:0:4'  # Rtg Alte Schule (Waiblingen Neustadt)
	RATHAUS_121 = 'de:08119:5164:0:3'  # Rtg Mühlweg (Waiblingen Beinstein)
	RATHAUS_122 = 'de:08119:5164:0:4'  # Rtg Endersbacher/Kleinhepp. Str. (Waiblingen Beinstein)
	RATHAUS_123 = 'de:08119:5178:0:3'  # End- und Startpunkt (Weinstadt Strümpfelbach (Weinstadt))
	RATHAUS_124 = 'de:08119:5179:0:3'  # eRtg Ulrichstraße (Weinstadt Beutelsbach)
	RATHAUS_125 = 'de:08119:5179:0:4'  # Rtg Benzach/Beutelsbach Bf (Weinstadt Beutelsbach)
	RATHAUS_126 = 'de:08119:5193:1:3'  # eRtg Kirche/Pfarrstraße/Pflegeheim (Schorndorf Weiler (Schorndorf))
	RATHAUS_127 = 'de:08119:5193:1:4'  # eRtg Dürrbach / Rehfeld (Schorndorf Weiler (Schorndorf))
	RATHAUS_128 = 'de:08119:5313:0:3'  # eRtg Winnenden (Leutenbach Leutenbach)
	RATHAUS_129 = 'de:08119:5313:0:4'  # eRtg Hauptstraße (Leutenbach Leutenbach)
	RATHAUS_130 = 'de:08119:5324:0:3'  # eRtg Biegel (Backnang Backnang)
	RATHAUS_131 = 'de:08119:5375:0:3'  # eRtg Auenwaldhalle (Weissach im Tal Unterweissach)
	RATHAUS_132 = 'de:08119:5410:0:3'  # eRtg Schwanen (Waiblingen Waiblingen)
	RATHAUS_133 = 'de:08119:5419:0:3'  # eRtg Winnender Straße (Schwaikheim Schwaikheim)
	RATHAUS_134 = 'de:08119:5419:0:4'  # Rtg Cafe Rommel/Bismarckstr. (Schwaikheim Schwaikheim)
	RATHAUS_135 = 'de:08118:5462:0:3'  # eRtg Johannesstraße (Kornwestheim Kornwestheim)
	RATHAUS_136 = 'de:08118:5579:0:3'  # Rtg Ilsfelder Str. (Besigheim Ottmarsheim)
	RATHAUS_137 = 'de:08118:5579:0:4'  # Rtg Besigheim (Besigheim Ottmarsheim)
	RATHAUS_138 = 'de:08118:5612:0:3'  # eRtg Nussbaumweg (Oberstenfeld Oberstenfeld)
	RATHAUS_139 = 'de:08118:5612:0:4'  # eRtg Lichtenberger Straße/Kath. Kirche (Oberstenfeld Oberstenfeld)
	RATHAUS_140 = 'de:08118:5621:0:3'  # eRtg Industriegebiet/Keltenmuseum (Eberdingen Hochdorf (Eberdingen))
	RATHAUS_141 = 'de:08118:5621:0:4'  # eRtg Pulverdingen B10 (Eberdingen Hochdorf (Eberdingen))
	RATHAUS_142 = 'de:08118:5644:0:3'  # Rtg Schillerschule (Bietigheim-Bissingen Bissingen (LB))
	RATHAUS_143 = 'de:08118:5644:0:4'  # Rtg Gerok-/Ludwigsburger Str. (Bietigheim-Bissingen Bissingen (LB))
	RATHAUS_144 = 'de:08115:5836:1:3'  # Pos 1 Rtg Dieselstraße Bosch (Rutesheim Rutesheim)
	RATHAUS_145 = 'de:08115:5836:1:4'  # Pos 2 Rtg Drescherstr. Festhalle (Rutesheim Rutesheim)
	RATHAUS_146 = 'de:08115:5848:0:3'  # eRtg Wankelstraße (Renningen Malmsheim)
	RATHAUS_147 = 'de:08115:5848:0:5'  # Rtg Perouser Str. (Renningen Malmsheim)
	RATHAUS_148 = 'de:08116:5903:0:3'  # Rtg Kappis/Tischardt (Kohlberg Kohlberg)
	RATHAUS_149 = 'de:08116:5903:0:4'  # Rtg Neuffen/Hörnlesweg (Kohlberg Kohlberg)
	RATHAUS_150 = 'de:08115:6045:1:3'  # Rtg Hirschlander/Alter Bfweg (Leonberg Höfingen)
	RATHAUS_151 = 'de:08115:6045:1:4'  # Rtg Am Schlossberg (Leonberg Höfingen)
	RATHAUS_152 = 'de:08111:6074:1:2'  # Stadtbahn - von Österreichischer Platz (Stuttgart Stuttgart)
	RATHAUS_153 = 'de:08111:6074:2:1'  # Rtg Österreichischer Pl./Rotebühlplatz (Stuttgart Stuttgart)
	RATHAUS_154 = 'de:08111:6074:3:8'  # Stadtbahn - von Rotelbühlplatz (Stuttgart Stuttgart)
	RATHAUS_155 = 'de:08111:6074:5:3'  # Linien 43/44 Rtg Wilhelmsbau/Österr. Pl. (Stuttgart Stuttgart)
	RATHAUS_156 = 'de:08111:6074:6:4'  # Linien 43/44 Rtg Charlottenplatz (Stuttgart Stuttgart)
	RATHAUS_157 = 'de:08116:6715:1:3'  # eRtg Gutenberger Straße 17 (Lenningen Oberlenningen)
	RATHAUS_158 = 'de:08119:6767:1:3'  # eRtg Leutenbach (Leutenbach Weiler zum Stein)
	RATHAUS_159 = 'de:08119:6767:1:4'  # eRtg Im Hummerholz (Leutenbach Weiler zum Stein)
	RATHAUS_160 = 'de:08119:6893:0:3'  # alle Richtungen (Berglen Bretzenacker)
	RATHAUS_161 = 'de:08118:6934:0:3'  # eRtg Talhöfe/Unterriexingen (Oberriexingen Oberriexingen)
	RATHAUS_162 = 'de:08118:7141:1:3'  # Rtg Alte Steige (Gerlingen Gerlingen)
	RATHAUS_163 = 'de:08116:7413:0:3'  # eRtg Bettlinger Forum/Herion (Großbettlingen Großbettlingen)
	RATHAUS_164 = 'de:08116:7413:0:4'  # eRtg Heerweg (Großbettlingen Großbettlingen)
	RATHAUS_165 = 'de:08117:2:0:2'  # Rtg Schildwachtweg (Geislingen an der Steige Geislingen (Steige))
	RATHAUS_166 = 'de:08117:2:0:H1'  # Rtg Geisl.(St) ZOB (Geislingen an der Steige Geislingen (Steige))
	RATHAUS_167 = 'de:08117:20:0:H1'  # Ankunft (Gingen an der Fils Gingen (F))
	RATHAUS_168 = 'de:08117:26:0:R1'  # Rtg Moltkestr. (Kuchen Kuchen (F))
	RATHAUS_169 = 'de:08117:51:0:H1'  # Rtg Turnhalle (Deggingen Reichenbach im Täle (Deggingen))
	RATHAUS_170 = 'de:08117:58:0:R1'  # Rtg Wiesensteig (Mühlhausen im Täle Mühlhausen im Täle)
	RATHAUS_171 = 'de:08117:64:0:H1'  # Rtg Brunnengarten/Ankunft (Wiesensteig Wiesensteig)
	RATHAUS_172 = 'de:08117:64:0:R1'  # Rtg Gasthaus See (Wiesensteig Wiesensteig)
	RATHAUS_173 = 'de:08117:203:0:H1'  # Rtg Söhnstetten (Böhmenkirch Böhmenkirch)
	RATHAUS_174 = 'de:08117:203:0:R1'  # Rtg Lauterstein/Geislingen (Böhmenkirch Böhmenkirch)
	RATHAUS_175 = 'de:08117:329:0:H1'  # Rtg Lauterstein (Lauterstein Nenningen)
	RATHAUS_176 = 'de:08117:329:0:R1'  # Rtg Kirche (Lauterstein Nenningen)
	RATHAUS_177 = 'de:08117:1102:0:H1'  # Rtg Bahnhofstr/Krettenhof (Birenbach Birenbach)
	RATHAUS_178 = 'de:08117:11102:0:2'  # Rtg Brückenstr (Birenbach Birenbach)
	RATHAUS_179 = 'de:08117:2052:0:H1'  # Rtg Hattenhofen (Hattenhofen Hattenhofen)
	RATHAUS_180 = 'de:08117:2052:0:R1'  # Rtg Göppingen (Hattenhofen Hattenhofen)
	RATHAUS_181 = 'de:08117:2201:0:H1'  # Rtg Sparwiesen (Uhingen Uhingen)
	RATHAUS_182 = 'de:08117:3232:0:3'  # Rtg Mühlhausen (Gruibingen Gruibingen)
	RATHAUS_183 = 'de:08117:3232:0:R1'  # Rtg Grundschule (Gruibingen Gruibingen)
	RATHAUS_184 = 'de:08117:3502:0:H1'  # Rtg Göserhof (Drackenstein Oberdrackenstein)
	RATHAUS_185 = 'de:08117:3502:0:R1'  # Rtg Drackenstein (Drackenstein Oberdrackenstein)
	RATHAUS_186 = 'de:08117:7412:0:R1'  # Rtg Filsbrücke (Ebersbach an der Fils Ebersbach (F))
	RATHAUS_187 = 'de:08117:7616:0:H1'  # Rtg Süßen Bf (Süßen Süßen)
	RATHAUS_188 = 'de:08117:7616:0:R1'  # Rtg stadtauswärts (Süßen Süßen)
	RATHAUS_189 = 'de:08117:7631:0:H1'  # Rtg Turnhalle (Donzdorf Reichenbach unter Rechberg (Donzdorf))
	RATHAUS_190 = 'de:08117:7641:0:H1'  # Rtg Reichenbach u.R. (Donzdorf Winzingen)
	RATHAUS_191 = 'de:08117:7641:0:R1'  # Rtg Rauhwiesenstr. (Donzdorf Winzingen)
	RATHAUS_192 = 'de:08117:7686:0:H1'  # Rtg Schnittlingen (Geislingen an der Steige Stötten)
	RATHAUS_193 = 'de:08117:7686:0:R1'  # Rtg Aussiedlerhöfe (Geislingen an der Steige Stötten)
	RATHAUS_194 = 'de:08117:9303:0:H1'  # Rtg Kornhausplatz (Göppingen Göppingen)
	RATHAUS_195 = 'de:08415:24319:0:0'  # Rtg Häslach Süd (Walddorfhäslach Häslach)
	RATHAUS_196 = 'de:08415:24319:0:4'  # Rtg Altenriet Schule (Walddorfhäslach Häslach)
	RATHAUS_197 = 'de:08415:28624:0:4'  # Rtg Bad Urach (Römerstein Donnstetten)
	RATHAUS_198 = 'de:08235:1510:1:1'  # eRtg Im Gailer (Gechingen Gechingen)
	RATHAUS_199 = 'de:08235:1510:1:4'  # eRtg Bergwald (Gechingen Gechingen)
	RATHAUS_200 = 'de:08235:10207:0:3'  # Rtg In der Misse Altensteig (Ebhausen Ebhausen)
	RATHAUS_201 = 'de:08235:10207:0:4'  # Rtg Breiter Weg Nagold (Ebhausen Ebhausen)
	RATHAUS_202 = 'de:08235:7881:1:3'  # Alle Richtungen (Altensteig Altensteig)
	RATHAUS_203 = 'de:08236:3609:0:3'  # Rtg Mühlhauser Str. (Tiefenbronn Lehningen)
	RATHAUS_204 = 'de:08236:3609:0:4'  # Rtg Hausen (Tiefenbronn Lehningen)
	RATHAUS_205 = 'de:08425:2613:0:1'  # auswärts (Nellingen Nellingen (UL))
	RATHAUS_206 = 'de:08425:2613:0:2'  # einwärts (Nellingen Nellingen (UL))
	WALDSTRASSE_1 = 'de:08116:2029:0:3'  # Rtg Driefbrunnen (Filderstadt Plattenhardt)
	WALDSTRASSE_2 = 'de:08116:2029:0:4'  # Rtg Stetten Kasparswald (Filderstadt Plattenhardt)
	WALDSTRASSE_3 = 'de:08118:5605:0:3'  # eRtg KleinbottwarSeestraße/Marbach (Großbottwar Großbottwar)
	WALDSTRASSE_4 = 'de:08118:5605:0:4'  # eRtg Bachstraße/Rosenplatz (Großbottwar Großbottwar)
	PLATTENHARDT_WALDSTRASSE = 'de:08116:2029'
	PLATTENHARDT_WALDSTRASSE_1 = 'de:08116:2029:0:3'  # Rtg Driefbrunnen (Filderstadt Plattenhardt)
	PLATTENHARDT_WALDSTRASSE_2 = 'de:08116:2029:0:4'  # Rtg Stetten Kasparswald (Filderstadt Plattenhardt)
	ALEMANNENSTRASSE_1 = 'de:08116:2030:0:3'  # Rtg Neuhausen (Filderstadt Sielmingen)
	ALEMANNENSTRASSE_2 = 'de:08116:2030:0:4'  # Rtg Friedrichstraße (Filderstadt Sielmingen)
	ALEMANNENSTRASSE_3 = 'de:08119:2374:0:3'  # Rtg Schmiden Dorfwiesen (Fellbach Oeffingen)
	ALEMANNENSTRASSE_4 = 'de:08119:2374:0:4'  # Rtg Gemeindezentrum (Fellbach Oeffingen)
	SIELMINGEN_ALEMANNENSTRASSE = 'de:08116:2030'
	SIELMINGEN_ALEMANNENSTRASSE_1 = 'de:08116:2030:0:3'  # Rtg Neuhausen (Filderstadt Sielmingen)
	SIELMINGEN_ALEMANNENSTRASSE_2 = 'de:08116:2030:0:4'  # Rtg Friedrichstraße (Filderstadt Sielmingen)
	SIELMINGEN_FRIEDRICHSTRASSE = 'de:08116:2031'
	SIELMINGEN_FRIEDRICHSTRASSE_1 = 'de:08116:2031:1:3'  # Rtg Alemannenstraße (Filderstadt Sielmingen)
	SIELMINGEN_FRIEDRICHSTRASSE_2 = 'de:08116:2031:1:4'  # Rtg Rathaus (Filderstadt Sielmingen)
	FRIEDRICHSTRASSE_1 = 'de:08116:2031:1:3'  # Rtg Alemannenstraße (Filderstadt Sielmingen)
	FRIEDRICHSTRASSE_2 = 'de:08116:2031:1:4'  # Rtg Rathaus (Filderstadt Sielmingen)
	FRIEDRICHSTRASSE_3 = 'de:08119:2379:0:3'  # Rtg Kleiner Ostring (Fellbach Schmiden)
	FRIEDRICHSTRASSE_4 = 'de:08119:2379:0:4'  # Rtg Jakob-/Butterstr. (Fellbach Schmiden)
	FRIEDRICHSTRASSE_5 = 'de:08118:7436:0:3'  # eRtg Remseck (Ludwigsburg Ludwigsburg)
	FRIEDRICHSTRASSE_6 = 'de:08118:7436:0:4'  # eRtg Ludwigsburg (Ludwigsburg Ludwigsburg)
	FRIEDRICHSTRASSE_7 = 'de:08117:4002:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	FRIEDRICHSTRASSE_8 = 'de:08117:4002:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	SIELMINGEN_HARTHÄUSER_WEG = 'de:08116:2032'
	SIELMINGEN_HARTHÄUSER_WEG_1 = 'de:08116:2032:0:3'  # Rtg Wolfschlugen (Filderstadt Sielmingen)
	SIELMINGEN_HARTHÄUSER_WEG_2 = 'de:08116:2032:0:4'  # Rtg Hausäcker (Filderstadt Sielmingen)
	HARTHÄUSER_WEG = 'de:08116:2032'
	HARTHÄUSER_WEG_1 = 'de:08116:2032:0:3'  # Rtg Wolfschlugen (Filderstadt Sielmingen)
	HARTHÄUSER_WEG_2 = 'de:08116:2032:0:4'  # Rtg Hausäcker (Filderstadt Sielmingen)
	SIELMINGEN_KANTSTRASSE = 'de:08116:2033'
	SIELMINGEN_KANTSTRASSE_1 = 'de:08116:2033:0:3'  # Rtg Hausäcker (Filderstadt Sielmingen)
	SIELMINGEN_KANTSTRASSE_2 = 'de:08116:2033:0:4'  # Rtg Rathaus (Filderstadt Sielmingen)
	KANTSTRASSE_1 = 'de:08116:2033:0:3'  # Rtg Hausäcker (Filderstadt Sielmingen)
	KANTSTRASSE_2 = 'de:08116:2033:0:4'  # Rtg Rathaus (Filderstadt Sielmingen)
	KANTSTRASSE_3 = 'de:08117:116:0:H1'  # Rtg Wölkbad (Geislingen an der Steige Geislingen (Steige))
	KANTSTRASSE_4 = 'de:08117:116:0:R1'  # Rtg Weing./Tegelberg (Geislingen an der Steige Geislingen (Steige))
	KANTSTRASSE_5 = 'de:08117:1000:0:H1'  # Rtg Stauferpark (Göppingen Göppingen)
	KANTSTRASSE_6 = 'de:08117:1000:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	SIELMINGEN_LANGE_STRASSE = 'de:08116:2035'
	SIELMINGEN_LANGE_STRASSE_1 = 'de:08116:2035:0:3'  # Rtg Friedhof (Filderstadt Sielmingen)
	SIELMINGEN_LANGE_STRASSE_2 = 'de:08116:2035:0:4'  # Rtg Rathaus (Filderstadt Sielmingen)
	LANGE_STRASSE_1 = 'de:08116:2035:0:3'  # Rtg Friedhof (Filderstadt Sielmingen)
	LANGE_STRASSE_2 = 'de:08116:2035:0:4'  # Rtg Rathaus (Filderstadt Sielmingen)
	LANGE_STRASSE_3 = 'de:08118:5574:0:3'  # eRtg Hessigheimer Straße (Mundelsheim Mundelsheim)
	LANGE_STRASSE_4 = 'de:08118:5574:0:4'  # eRtg SÜWAG (Mundelsheim Mundelsheim)
	SIELMINGEN_RATHAUS = 'de:08116:2036'
	SIELMINGEN_RATHAUS_1 = 'de:08116:2036:1:3'  # Rtg Kantstr. (Filderstadt Sielmingen)
	SIELMINGEN_RATHAUS_2 = 'de:08116:2036:1:4'  # Rtg Bernh. Karlstr. (Filderstadt Sielmingen)
	BERNHÄUSER_STRASSE = 'de:08111:2037'
	BERNHÄUSER_STRASSE_1 = 'de:08111:2037:0:3'  # Rtg In den Entenäckern (Stuttgart Stuttgart)
	BERNHÄUSER_STRASSE_2 = 'de:08111:2037:0:4'  # Rtg Plieningen Post (Stuttgart Stuttgart)
	KASERNE_MÖHRINGEN = 'de:08111:2038'
	KASERNE_MÖHRINGEN_1 = 'de:08111:2038:0:3'  # Rtg Steckfeld (Stuttgart Stuttgart)
	KASERNE_MÖHRINGEN_2 = 'de:08111:2038:0:4'  # Rtg Landhauskreuzung (Stuttgart Stuttgart)
	PLIENINGEN_POST = 'de:08111:2039'
	PLIENINGEN_POST_1 = 'de:08111:2039:1:3'  # Rtg Ressestr./Bernhäuser Str. (Stuttgart Stuttgart)
	PLIENINGEN_POST_2 = 'de:08111:2039:2:4'  # Rtg Fraubronnstr./Im Köpfert (Stuttgart Stuttgart)
	PLIENINGEN_POST_3 = 'de:08111:2039:2:6'  # NBus Rtg Fraubronnstr. (Stuttgart Stuttgart)
	SCHARNHÄUSER_BRÜCKE = 'de:08111:2040'
	SCHARNHÄUSER_BRÜCKE_1 = 'de:08111:2040:0:3'  # Rtg Seemühlenweg (Stuttgart Stuttgart)
	SCHARNHÄUSER_BRÜCKE_2 = 'de:08111:2040:0:4'  # Rtg Ressestr. (Stuttgart Stuttgart)
	SCHARNHÄUSER_BRÜCKE_3 = 'de:08111:2040:0:5'  # Ersatz Rtg Seemühlenweg (Stuttgart Stuttgart)
	SCHARNHÄUSER_BRÜCKE_4 = 'de:08111:2040:0:6'  # Ersatz Rtg Ressestraße Post (Stuttgart Stuttgart)
	STECKFELD = 'de:08111:2041'
	STECKFELD_1 = 'de:08111:2041:0:3'  # Rtg Im Chausseefeld (Stuttgart Stuttgart)
	STECKFELD_2 = 'de:08111:2041:0:4'  # Rtg Kaserne Möhringen (Stuttgart Stuttgart)
	SCHUL__SPORTZENTRUM = 'de:08116:2042'
	SCHUL__SPORTZENTRUM_1 = 'de:08116:2042:0:3'  # Rtg Bernhausen (Filderstadt Sielmingen)
	SCHUL__SPORTZENTRUM_2 = 'de:08116:2042:0:4'  # Rtg Friedhof (Filderstadt Sielmingen)
	SIELMINGEN_SCHUL__SPORTZENTRUM = 'de:08116:2042'
	SIELMINGEN_SCHUL__SPORTZENTRUM_1 = 'de:08116:2042:0:3'  # Rtg Bernhausen (Filderstadt Sielmingen)
	SIELMINGEN_SCHUL__SPORTZENTRUM_2 = 'de:08116:2042:0:4'  # Rtg Friedhof (Filderstadt Sielmingen)
	NEUHÄUSER_STRASSE_1 = 'de:08116:2043:0:3'  # Rtg Sielmingen (Filderstadt Bernhausen)
	NEUHÄUSER_STRASSE_2 = 'de:08116:2043:0:4'  # Rtg Hauptstr. (Filderstadt Bernhausen)
	NEUHÄUSER_STRASSE_3 = 'de:08116:5009:0:3'  # Rtg Neuhausen Traube (Denkendorf Denkendorf)
	NEUHÄUSER_STRASSE_4 = 'de:08116:5009:0:4'  # Rtg Vogelsangstr. (Denkendorf Denkendorf)
	BERNHAUSEN_NEUHÄUSER_STR = 'de:08116:2043'
	BERNHAUSEN_NEUHÄUSER_STR_1 = 'de:08116:2043:0:3'  # Rtg Sielmingen (Filderstadt Bernhausen)
	BERNHAUSEN_NEUHÄUSER_STR_2 = 'de:08116:2043:0:4'  # Rtg Hauptstr. (Filderstadt Bernhausen)
	BERNHAUSEN_RATHAUS = 'de:08116:2044'
	BERNHAUSEN_RATHAUS_1 = 'de:08116:2044:0:3'  # Rtg Bahnhof (Filderstadt Bernhausen)
	BERNHAUSEN_RATHAUS_2 = 'de:08116:2044:0:4'  # Rtg Rosenstraße (Filderstadt Bernhausen)
	SIELMINGEN_FRIEDHOF = 'de:08116:2045'
	SIELMINGEN_FRIEDHOF_1 = 'de:08116:2045:0:3'  # Rtg Schul-/Sportzentrum (Filderstadt Sielmingen)
	SIELMINGEN_FRIEDHOF_2 = 'de:08116:2045:0:4'  # Rtg Lange Straße (Filderstadt Sielmingen)
	BONLANDEN_WALDORFSCHULE = 'de:08116:2046:0:3'  # alle Richtungen (Filderstadt Bonlanden)
	WALDORFSCHULE_1 = 'de:08116:2046:0:3'  # alle Richtungen (Filderstadt Bonlanden)
	WALDORFSCHULE_2 = 'de:08115:3109:0:3'  # eRtg (Böblingen Böblingen)
	PLATTENHARDT_WEILERHAU = 'de:08116:2047'
	PLATTENHARDT_WEILERHAU_1 = 'de:08116:2047:0:3'  # Rtg Lailensäcker (Filderstadt Plattenhardt)
	PLATTENHARDT_WEILERHAU_2 = 'de:08116:2047:0:4'  # Rtg Bernhausen Bf (Filderstadt Plattenhardt)
	WEILERHAU = 'de:08116:2047'
	WEILERHAU_1 = 'de:08116:2047:0:3'  # Rtg Lailensäcker (Filderstadt Plattenhardt)
	WEILERHAU_2 = 'de:08116:2047:0:4'  # Rtg Bernhausen Bf (Filderstadt Plattenhardt)
	ELISABETH_SELBERT_GYMNASIUM = 'de:08116:2049'
	ELISABETH_SELBERT_GYMNASIUM_1 = 'de:08116:2049:0:3'  # Rtg Plattenhardt Griebenäcker (Filderstadt Bernhausen)
	ELISABETH_SELBERT_GYMNASIUM_2 = 'de:08116:2049:0:4'  # Rtg Filharmonie (Filderstadt Bernhausen)
	BERNH__E__SELBERT_GYMNASIUM = 'de:08116:2049'
	BERNH__E__SELBERT_GYMNASIUM_1 = 'de:08116:2049:0:3'  # Rtg Plattenhardt Griebenäcker (Filderstadt Bernhausen)
	BERNH__E__SELBERT_GYMNASIUM_2 = 'de:08116:2049:0:4'  # Rtg Filharmonie (Filderstadt Bernhausen)
	LEIPZIGER_PLATZ = 'de:08111:2050'
	LEIPZIGER_PLATZ_1 = 'de:08111:2050:0:3'  # Rtg Herweghstr. (Stuttgart Stuttgart)
	LEIPZIGER_PLATZ_2 = 'de:08111:2050:0:4'  # Rtg Seyfferstr. (Stuttgart Stuttgart)
	SCHILLERSTRASSE_1 = 'de:08116:2051:0:3'  # Rtg Schnecken (Filderstadt Plattenhardt)
	SCHILLERSTRASSE_2 = 'de:08116:2051:0:4'  # Rtg Griebenäcker (Filderstadt Plattenhardt)
	SCHILLERSTRASSE_3 = 'de:08116:2907:0:4'  # Rtg Lettenstr./Wilheilmstr. (Neuhausen auf den Fildern Neuhausen (F))
	SCHILLERSTRASSE_4 = 'de:08115:3217:0:3'  # eRtg Lessingstraße (Schönaich Schönaich)
	SCHILLERSTRASSE_5 = 'de:08115:3217:0:4'  # eRtg Evangelische Kirche (Schönaich Schönaich)
	SCHILLERSTRASSE_6 = 'de:08116:3905:0:3'  # Rtg Schulzentrum (Reichenbach an der Fils Reichenbach (F))
	SCHILLERSTRASSE_7 = 'de:08116:3905:0:4'  # Rtg Stuttgarter-/Hauptstr. (Reichenbach an der Fils Reichenbach (F))
	SCHILLERSTRASSE_8 = 'de:08116:3937:0:3'  # eRtg Eichendorffstraße (Neckartailfingen Neckartailfingen)
	SCHILLERSTRASSE_9 = 'de:08116:3937:0:4'  # eRtg Neckartartailfingen Schule (Neckartailfingen Neckartailfingen)
	SCHILLERSTRASSE_10 = 'de:08118:5759:0:3'  # eRtg Stromberggymnasium (Vaihingen an der Enz Kleinglattbach)
	SCHILLERSTRASSE_11 = 'de:08118:5759:0:4'  # eRtg Weizenstraße (Vaihingen an der Enz Kleinglattbach)
	SCHILLERSTRASSE_12 = 'de:08115:5826:0:4'  # Rtg Dieselstraße (Rutesheim Rutesheim)
	SCHILLERSTRASSE_13 = 'de:08116:6707:0:3'  # eRtg Linde (Kohlberg Kohlberg)
	SCHILLERSTRASSE_14 = 'de:08117:14:0:H1'  # Rtg Gemeindezentrum (Kuchen Kuchen (F))
	SCHILLERSTRASSE_15 = 'de:08117:1108:0:H1'  # Rtg Friedhof (Wäschenbeuren Wäschenbeuren)
	SCHILLERSTRASSE_16 = 'de:08117:2113:0:H1'  # Schillerstraße (Zell unter Aichelberg Zell (A))
	SCHILLERSTRASSE_17 = 'de:08117:2113:0:R1'  # Rtg Göppinger Straße (Zell unter Aichelberg Zell (A))
	SCHILLERSTRASSE_18 = 'de:08117:4037:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	SCHILLERSTRASSE_19 = 'de:08117:4037:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	SCHILLERSTRASSE_20 = 'de:08136:3029:0:H1'  # Rtg Schw. Gmünd (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHILLERSTRASSE_21 = 'de:08136:3029:0:R1'  # Rtg Göppingen (Schwäbisch Gmünd Schwäbisch Gmünd)
	PLATTENHARDT_SCHILLERSTRASSE = 'de:08116:2051'
	PLATTENHARDT_SCHILLERSTRASSE_1 = 'de:08116:2051:0:3'  # Rtg Schnecken (Filderstadt Plattenhardt)
	PLATTENHARDT_SCHILLERSTRASSE_2 = 'de:08116:2051:0:4'  # Rtg Griebenäcker (Filderstadt Plattenhardt)
	RAINÄCKERSTRASSE = 'de:08116:2052'
	RAINÄCKERSTRASSE_1 = 'de:08116:2052:0:3'  # Bus Rtg Echterdingen Bf (Filderstadt Bonlanden)
	RAINÄCKERSTRASSE_2 = 'de:08116:2052:0:4'  # Bus Rtg Degerloch (Filderstadt Bonlanden)
	BONLANDEN_RAINÄCKERSTRASSE = 'de:08116:2052'
	BONLANDEN_RAINÄCKERSTRASSE_1 = 'de:08116:2052:0:3'  # Bus Rtg Echterdingen Bf (Filderstadt Bonlanden)
	BONLANDEN_RAINÄCKERSTRASSE_2 = 'de:08116:2052:0:4'  # Bus Rtg Degerloch (Filderstadt Bonlanden)
	SENEFELDERSTRASSE = 'de:08111:2053'
	SENEFELDERSTRASSE_1 = 'de:08111:2053:1:3'  # Rtg Schwabstr. (Stuttgart Stuttgart)
	SENEFELDERSTRASSE_2 = 'de:08111:2053:2:4'  # Rtg Feuersee (Stuttgart Stuttgart)
	MARIENSTRASSE_1 = 'de:08111:2054:0:3'  # Rtg Feuersee (Stuttgart Stuttgart)
	MARIENSTRASSE_2 = 'de:08111:2054:0:4'  # Rtg Österreichischer Platz (Stuttgart Stuttgart)
	MARIENSTRASSE_3 = 'de:08116:4245:0:3'  # eRtg Bergstraße (Wernau (Neckar) Wernau (N))
	MARIENSTRASSE_4 = 'de:08117:2042:0:R1'  # Rtg Uditorium (Uhingen Uhingen)
	MARIENSTRASSE_5 = 'de:08117:7006:0:H1'  # Rtg Hirschstr. (Göppingen Faurndau)
	MARIENSTRASSE_6 = 'de:08117:7006:0:R1'  # Rtg Ludwigstr. (Göppingen Faurndau)
	SCHÖNBUCHSTRASSE_1 = 'de:08116:2055:0:3'  # Rtg Spitzäcker (Filderstadt Plattenhardt)
	SCHÖNBUCHSTRASSE_2 = 'de:08116:2055:0:4'  # Rtg Rathaus (Filderstadt Plattenhardt)
	SCHÖNBUCHSTRASSE_3 = 'de:08115:4694:0:4'  # Rtg Holzgerlingen (Böblingen Böblingen)
	PLATTENHARDT_SCHÖNBUCHSTRASSE = 'de:08116:2055'
	PLATTENHARDT_SCHÖNBUCHSTRASSE_1 = 'de:08116:2055:0:3'  # Rtg Spitzäcker (Filderstadt Plattenhardt)
	PLATTENHARDT_SCHÖNBUCHSTRASSE_2 = 'de:08116:2055:0:4'  # Rtg Rathaus (Filderstadt Plattenhardt)
	PLATTENHARDT_ALTENHEIM = 'de:08116:2056:0:3'  # Bus Rtg Echterd./Bernh. (Filderstadt Plattenhardt)
	ALTENHEIM_1 = 'de:08116:2056:0:3'  # Bus Rtg Echterd./Bernh. (Filderstadt Plattenhardt)
	ALTENHEIM_2 = 'de:08119:5432:1:3'  # Rtg Gartenstr. (Fellbach Fellbach)
	FRIEDENSTRASSE = 'de:08111:2057'
	FRIEDENSTRASSE_1 = 'de:08111:2057:0:3'  # Rtg Staatsgalerie (Stuttgart Stuttgart)
	FRIEDENSTRASSE_2 = 'de:08111:2057:0:4'  # Rtg Urachstr. (Stuttgart Stuttgart)
	URACHSTRASSE = 'de:08111:2058'
	URACHSTRASSE_1 = 'de:08111:2058:0:3'  # Rtg Friedenstr. (Stuttgart Stuttgart)
	URACHSTRASSE_2 = 'de:08111:2058:0:4'  # Rtg Schwarenbergstr. (Stuttgart Stuttgart)
	SCHWARENBERGSTRASSE = 'de:08111:2059'
	SCHWARENBERGSTRASSE_1 = 'de:08111:2059:0:3'  # Rtg Urachstr. (Stuttgart Stuttgart)
	SCHWARENBERGSTRASSE_2 = 'de:08111:2059:0:4'  # Rtg Ostendplatz (Stuttgart Stuttgart)
	WILHELMSBAU = 'de:08111:2061'
	WILHELMSBAU_1 = 'de:08111:2061:0:3'  # Rtg Rotebühlplatz (Stuttgart Stuttgart)
	WILHELMSBAU_2 = 'de:08111:2061:0:4'  # Rtg Rathaus/Österr.Pl. (Stuttgart Stuttgart)
	MITTE_1 = 'de:08116:2062:0:2'  # Rtg Neckartailfingen (Schlaitdorf Schlaitdorf)
	MITTE_2 = 'de:08116:2062:0:3'  # Rtg Häslach/Altenriet (Schlaitdorf Schlaitdorf)
	MITTE_3 = 'de:08118:3327:1:2'  # Pos 2 Rtg Glemstal hinten (Schwieberdingen Schwieberdingen)
	MITTE_4 = 'de:08118:3327:1:3'  # Pos 3 Rtg Glemstal vorne (Schwieberdingen Schwieberdingen)
	MITTE_5 = 'de:08118:3327:1:4'  # Pos 1 Rtg H.-Essig-Str./Ludwigsburg Str. (Schwieberdingen Schwieberdingen)
	MITTE_6 = 'de:08119:3703:1:3'  # Rtg Markgraben (Remshalden Geradstetten)
	MITTE_7 = 'de:08119:3703:1:4'  # Rtg Kelterwiesenbach/Rathaus (Remshalden Geradstetten)
	MITTE_8 = 'de:08116:4380:0:3'  # eRtg West (Nürtingen Reudern)
	MITTE_9 = 'de:08116:4380:0:4'  # eRtg Ost (Nürtingen Reudern)
	MITTE_10 = 'de:08116:4407:0:3'  # eRtg Schlossgärten (Neidlingen Neidlingen)
	MITTE_11 = 'de:08116:4407:0:4'  # eRtg Fabrikstraße (Neidlingen Neidlingen)
	MITTE_12 = 'de:08115:4735:1:3'  # Rtg Gärtringen (Nufringen Nufringen)
	MITTE_13 = 'de:08115:4735:1:4'  # Rtg Herrenberg (Nufringen Nufringen)
	MITTE_14 = 'de:08119:5803:0:1'  # R-Bahn (Schorndorf Haubersbronn)
	MITTE_15 = 'de:08118:6982:0:3'  # eRtg Realschule Remseck (Remseck am Neckar Pattonville)
	MITTE_16 = 'de:08118:6982:0:4'  # eRtg Berufsschule (Remseck am Neckar Pattonville)
	SCHLAITDORF_MITTE = 'de:08116:2062'
	SCHLAITDORF_MITTE_1 = 'de:08116:2062:0:2'  # Rtg Neckartailfingen (Schlaitdorf Schlaitdorf)
	SCHLAITDORF_MITTE_2 = 'de:08116:2062:0:3'  # Rtg Häslach/Altenriet (Schlaitdorf Schlaitdorf)
	SÜD_1 = 'de:08415:29041:0:1'  # Rtg Altenriet Schule (Walddorfhäslach Häslach)
	SÜD_2 = 'de:08415:29041:0:2'  # Rtg Seestr. (Walddorfhäslach Häslach)
	SÜD_3 = 'de:08115:3195:1:1'  # Rtg Renningen (Renningen Renningen)
	SÜD_4 = 'de:08115:3195:2:3'  # Rtg Kirche (Renningen Renningen)
	SÜD_5 = 'de:08115:3195:2:4'  # Rtg Friedhof (Renningen Renningen)
	SÜD_6 = 'de:08115:3195:3:2'  # Rtg Böblingen (Renningen Renningen)
	SÜD_7 = 'de:08116:5777:1:2'  # Gleis 2 (Kirchheim unter Teck Kirchheim (T))
	HÄSLACH_SÜD = 'de:08415:29041'
	HÄSLACH_SÜD_1 = 'de:08415:29041:0:1'  # Rtg Altenriet Schule (Walddorfhäslach Häslach)
	HÄSLACH_SÜD_2 = 'de:08415:29041:0:2'  # Rtg Seestr. (Walddorfhäslach Häslach)
	HÄSLACH_SEESTRASSE = 'de:08415:29040'
	HÄSLACH_SEESTRASSE_1 = 'de:08415:29040:0:1'  # Rtg Süd (Walddorfhäslach Häslach)
	HÄSLACH_SEESTRASSE_2 = 'de:08415:29040:0:2'  # Rtg Schlaitdorf Rathaus (Walddorfhäslach Häslach)
	SEESTRASSE_1 = 'de:08415:29040:0:1'  # Rtg Süd (Walddorfhäslach Häslach)
	SEESTRASSE_2 = 'de:08415:29040:0:2'  # Rtg Schlaitdorf Rathaus (Walddorfhäslach Häslach)
	SEESTRASSE_3 = 'de:08119:3636:0:3'  # eRtg Winnenden (Winnenden Winnenden)
	SEESTRASSE_4 = 'de:08119:3636:0:4'  # eRtg Backnanger Straße (Winnenden Winnenden)
	SEESTRASSE_5 = 'de:08119:3743:0:3'  # eRtg Schafstraße (Kernen im Remstal Rommelshausen)
	SEESTRASSE_6 = 'de:08115:4856:0:3'  # eRtg Schwabenstraße (Schönaich Schönaich)
	SEESTRASSE_7 = 'de:08115:4856:0:4'  # eRtg Mittlere Mühle (Schönaich Schönaich)
	SEESTRASSE_8 = 'de:08116:6773:0:3'  # eRtg Bahnhof/Weilheim (Holzmaden Holzmaden)
	SEESTRASSE_9 = 'de:08118:6986:1:3'  # eRtg Horrenwinkel (Steinheim an der Murr Kleinbottwar)
	SEESTRASSE_10 = 'de:08118:6986:1:4'  # eRtg Waldstraße (Steinheim an der Murr Kleinbottwar)
	SEESTRASSE_11 = 'de:08117:2035:0:H1'  # Rtg Fabrikstr. (Uhingen Uhingen)
	SEESTRASSE_12 = 'de:08117:2035:0:R1'  # Rtg. Baiereck (Uhingen Uhingen)
	ALTENRIET_SCHULE = 'de:08116:2065'
	ALTENRIET_SCHULE_1 = 'de:08116:2065:0:3'  # Rtg Rathaus (Altenriet Altenriet)
	ALTENRIET_SCHULE_2 = 'de:08116:2065:0:4'  # Rtg Häslach Süd/Walddorf Hohebildstr. (Altenriet Altenriet)
	SCHULE_1 = 'de:08116:2065:0:3'  # Rtg Rathaus (Altenriet Altenriet)
	SCHULE_2 = 'de:08116:2065:0:4'  # Rtg Häslach Süd/Walddorf Hohebildstr. (Altenriet Altenriet)
	SCHULE_3 = 'de:08116:2948:0:3'  # eRtg Lindenplatz (Nürtingen Oberensingen)
	SCHULE_4 = 'de:08118:3338:0:3'  # eRtg Beilsteiner Straße (Oberstenfeld Oberstenfeld)
	SCHULE_5 = 'de:08119:3530:0:3'  # eRtg Ankunft/Abfahrt Bus (Kirchberg an der Murr Kirchberg (M))
	SCHULE_6 = 'de:08118:3564:0:3'  # eRtg Neusatz/Ottmarsh. Straße (Hessigheim Hessigheim)
	SCHULE_7 = 'de:08119:3719:0:3'  # eRtg Alle Richtungen (Althütte Althütte)
	SCHULE_8 = 'de:08116:4058:0:3'  # eRtg Rathaus (Deizisau Deizisau)
	SCHULE_9 = 'de:08116:4058:0:4'  # eRtg Olga-/Keplerstr. (Deizisau Deizisau)
	SCHULE_10 = 'de:08116:4157:0:3'  # eRtg Krummhardt Abzweig (Aichwald Schanbach)
	SCHULE_11 = 'de:08115:4194:0:3'  # eRtg Kreuzstraße (Grafenau Dätzingen)
	SCHULE_12 = 'de:08116:4202:0:1'  # Rtg Rüdern Bürgerhaus RSKN (Esslingen am Neckar Sulzgries)
	SCHULE_13 = 'de:08116:4202:0:3'  # eRtg Bergstraße (Esslingen am Neckar Sulzgries)
	SCHULE_14 = 'de:08116:4429:0:3'  # eRtg Hölzlachstraße (Frickenhausen Frickenhausen)
	SCHULE_15 = 'de:08116:4429:0:4'  # eRtg Reutte (Frickenhausen Frickenhausen)
	SCHULE_16 = 'de:08116:4484:0:3'  # eRtg Neckarbrücke/Schillerstr/Eichendorf (Neckartailfingen Neckartailfingen)
	SCHULE_17 = 'de:08116:4484:0:4'  # eRtg Stuttgarter Str/Obere Liesäcker (Neckartailfingen Neckartailfingen)
	SCHULE_18 = 'de:08116:4490:0:4'  # alle Richtungen (Bempflingen Bempflingen)
	SCHULE_19 = 'de:08119:4986:0:3'  # eRtg Alle Richtungen (Althütte Sechselberg)
	SCHULE_20 = 'de:08119:5037:0:3'  # eRtg Rathaus (Allmersbach im Tal Allmersbach im Tal)
	SCHULE_21 = 'de:08119:5044:0:3'  # eRtg Alle Richtungen (Spiegelberg Spiegelberg)
	SCHULE_22 = 'de:08119:5319:0:3'  # Rtg Rathaus/Nellmersb. Leutenb.Str. (Leutenbach Leutenbach)
	SCHULE_23 = 'de:08119:5319:0:4'  # Rtg Rathaus (Leutenbach Leutenbach)
	SCHULE_24 = 'de:08118:5544:0:3'  # eRtg Brühlstraße (Ingersheim Großingersheim)
	SCHULE_25 = 'de:08118:5615:0:3'  # Rtg Oberstenfeld (Oberstenfeld Gronau)
	SCHULE_26 = 'de:08118:5615:0:4'  # Rtg Beilstein/Gronau (Oberstenfeld Gronau)
	SCHULE_27 = 'de:08118:5630:0:3'  # eRtg Allmandklinge (Sachsenheim Hohenhaslach)
	SCHULE_28 = 'de:08119:6640:0:1'  # Rtg Gewerbegebiet (Alfdorf Alfdorf)
	SCHULE_29 = 'de:08119:6640:0:3'  # eRtg Alle Richtungen (Alfdorf Alfdorf)
	SCHULE_30 = 'de:08116:6775:0:3'  # eRtg Lindenplatz/Kohlberg (Neuffen Neuffen)
	SCHULE_31 = 'de:08116:6775:0:4'  # eRtg Uracher Weg (Neuffen Neuffen)
	SCHULE_32 = 'de:08119:6935:0:3'  # Rtg Hof (Backnang Sachsenweiler)
	SCHULE_33 = 'de:08115:7025:0:3'  # Rtg alle Richtungen (Herrenberg Mönchberg)
	SCHULE_34 = 'de:08119:7504:0:3'  # existiert nicht (Großerlach Großerlach)
	SCHULE_35 = 'de:08119:7504:0:4'  # Rtg Liemersbach_Abzw. Hohenegarten (Großerlach Großerlach)
	SCHULE_36 = 'de:08117:10498:0:1'  # Rtg Aufhausen (Geislingen an der Steige Aufhausen (Geislingen))
	SCHULE_37 = 'de:08117:17683:0:1'  # Rtg Geislingen (Böhmenkirch Treffelhausen)
	SCHULE_38 = 'de:08416:6354:0:1'  # eRtg Altes Rathaus (Dettenhausen Dettenhausen)
	SCHULE_39 = 'de:08416:6354:0:4'  # eRtg Bahnhofstraße (Dettenhausen Dettenhausen)
	SCHULE_40 = 'de:08125:984:0:1'  # eRtg Freibad (Wüstenrot Wüstenrot)
	SCHULE_41 = 'de:08125:984:0:2'  # eRtg Firma Zügel (Wüstenrot Wüstenrot)
	SCHULE_42 = 'de:08425:2577:0:1'  # Alle Richtungen (Amstetten Schalkstetten)
	ALTENRIET_RATHAUS = 'de:08116:2066'
	ALTENRIET_RATHAUS_1 = 'de:08116:2066:0:3'  # Rtg Schule (Altenriet Altenriet)
	ALTENRIET_RATHAUS_2 = 'de:08116:2066:0:4'  # Rtg Schule Bernhausen (Altenriet Altenriet)
	HOHEBILDSTRASSE = 'de:08415:29043'
	HOHEBILDSTRASSE_1 = 'de:08415:29043:0:1'  # Rtg Rathaus (Walddorfhäslach Walddorf)
	HOHEBILDSTRASSE_2 = 'de:08415:29043:0:2'  # Rtg Altenriet Schule (Walddorfhäslach Walddorf)
	WALDDORF_HOHEBILDSTRASSE = 'de:08415:29043'
	WALDDORF_HOHEBILDSTRASSE_1 = 'de:08415:29043:0:1'  # Rtg Rathaus (Walddorfhäslach Walddorf)
	WALDDORF_HOHEBILDSTRASSE_2 = 'de:08415:29043:0:2'  # Rtg Altenriet Schule (Walddorfhäslach Walddorf)
	RATHAUS__NOTARIAT = 'de:08415:24320:0:1'  # Linien H&B sowie Melchinger (Walddorfhäslach Walddorf)
	WALDDORF_RATHAUS__NOTARIAT = 'de:08415:24320:0:1'  # Linien H&B sowie Melchinger (Walddorfhäslach Walddorf)
	LEINFELDEN_NEUER_MARKT = 'de:08116:2069'
	LEINFELDEN_NEUER_MARKT_1 = 'de:08116:2069:3:5'  # Rtg Leinfelder Haus (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_NEUER_MARKT_2 = 'de:08116:2069:3:6'  # Rtg Stadtbücherei (Leinfelden-Echterdingen Leinfelden)
	NEUER_MARKT = 'de:08116:2069'
	NEUER_MARKT_1 = 'de:08116:2069:3:5'  # Rtg Leinfelder Haus (Leinfelden-Echterdingen Leinfelden)
	NEUER_MARKT_2 = 'de:08116:2069:3:6'  # Rtg Stadtbücherei (Leinfelden-Echterdingen Leinfelden)
	NECKARTAILFINGER_STRASSE = 'de:08116:2070'
	NECKARTAILFINGER_STRASSE_1 = 'de:08116:2070:0:3'  # Rtg Rathaus (Schlaitdorf Schlaitdorf)
	NECKARTAILFINGER_STRASSE_2 = 'de:08116:2070:0:4'  # Rtg Neckartailfingen (Schlaitdorf Schlaitdorf)
	SCHLAITDORF_NECKARTAILF__STR = 'de:08116:2070'
	SCHLAITDORF_NECKARTAILF__STR_1 = 'de:08116:2070:0:3'  # Rtg Rathaus (Schlaitdorf Schlaitdorf)
	SCHLAITDORF_NECKARTAILF__STR_2 = 'de:08116:2070:0:4'  # Rtg Neckartailfingen (Schlaitdorf Schlaitdorf)
	FLEINSBACH = 'de:08116:2071'
	FLEINSBACH_1 = 'de:08116:2071:0:3'  # Rtg Bonlanden Nord/Filharmonie (Filderstadt Bernhausen)
	FLEINSBACH_2 = 'de:08116:2071:0:4'  # Rtg Bernhausen Bf (Filderstadt Bernhausen)
	BERNHAUSEN_FLEINSBACH = 'de:08116:2071'
	BERNHAUSEN_FLEINSBACH_1 = 'de:08116:2071:0:3'  # Rtg Bonlanden Nord/Filharmonie (Filderstadt Bernhausen)
	BERNHAUSEN_FLEINSBACH_2 = 'de:08116:2071:0:4'  # Rtg Bernhausen Bf (Filderstadt Bernhausen)
	BERNHAUSEN_TÜV = 'de:08116:2072'
	BERNHAUSEN_TÜV_1 = 'de:08116:2072:0:3'  # Rtg Bahnhof/Luftfrachtzentrum West (Filderstadt Bernhausen)
	BERNHAUSEN_TÜV_2 = 'de:08116:2072:0:4'  # Rtg Stetten Gewerbegebiet (Filderstadt Bernhausen)
	BERNHAUSEN_TÜV_3 = 'de:08116:2072:0:6'  # Rtg Zaunäcker (Filderstadt Bernhausen)
	TÜV = 'de:08116:2072'
	TÜV_1 = 'de:08116:2072:0:3'  # Rtg Bahnhof/Luftfrachtzentrum West (Filderstadt Bernhausen)
	TÜV_2 = 'de:08116:2072:0:4'  # Rtg Stetten Gewerbegebiet (Filderstadt Bernhausen)
	TÜV_3 = 'de:08116:2072:0:6'  # Rtg Zaunäcker (Filderstadt Bernhausen)
	PLATTENHARDT_FINKENSTRASSE = 'de:08116:2074:0:3'  # Rtg Driefbrunnen (Filderstadt Plattenhardt)
	FINKENSTRASSE = 'de:08116:2074:0:3'  # Rtg Driefbrunnen (Filderstadt Plattenhardt)
	FILHARMONIE = 'de:08116:2076'
	FILHARMONIE_1 = 'de:08116:2076:0:3'  # Rtg Plattenhardt (Filderstadt Bernhausen)
	FILHARMONIE_2 = 'de:08116:2076:0:4'  # Rtg Bernhausen Bf (Filderstadt Bernhausen)
	BERNHAUSEN_FILHARMONIE = 'de:08116:2076'
	BERNHAUSEN_FILHARMONIE_1 = 'de:08116:2076:0:3'  # Rtg Plattenhardt (Filderstadt Bernhausen)
	BERNHAUSEN_FILHARMONIE_2 = 'de:08116:2076:0:4'  # Rtg Bernhausen Bf (Filderstadt Bernhausen)
	DIEPOLDSTRASSE = 'de:08116:2077'
	DIEPOLDSTRASSE_1 = 'de:08116:2077:1:3'  # Rtg Hauptstr. (Filderstadt Bernhausen)
	DIEPOLDSTRASSE_2 = 'de:08116:2077:2:4'  # Rtg Fröbelstr. (Filderstadt Bernhausen)
	BERNHAUSEN_DIEPOLDSTRASSE = 'de:08116:2077'
	BERNHAUSEN_DIEPOLDSTRASSE_1 = 'de:08116:2077:1:3'  # Rtg Hauptstr. (Filderstadt Bernhausen)
	BERNHAUSEN_DIEPOLDSTRASSE_2 = 'de:08116:2077:2:4'  # Rtg Fröbelstr. (Filderstadt Bernhausen)
	BONLANDEN_MARKTSTRASSE = 'de:08116:2078'
	BONLANDEN_MARKTSTRASSE_1 = 'de:08116:2078:0:3'  # Rtg Metzinger Str. (Filderstadt Bonlanden)
	BONLANDEN_MARKTSTRASSE_2 = 'de:08116:2078:0:4'  # Rtg Rotzeil (Filderstadt Bonlanden)
	MARKTSTRASSE_1 = 'de:08116:2078:0:3'  # Rtg Metzinger Str. (Filderstadt Bonlanden)
	MARKTSTRASSE_2 = 'de:08116:2078:0:4'  # Rtg Rotzeil (Filderstadt Bonlanden)
	MARKTSTRASSE_3 = 'de:08117:1037:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	NORD_WEST_RING = 'de:08116:2079'
	NORD_WEST_RING_1 = 'de:08116:2079:0:3'  # Rtg Fröbelstr. (Filderstadt Bernhausen)
	NORD_WEST_RING_2 = 'de:08116:2079:0:4'  # Rtg In den Entenäckern (Filderstadt Bernhausen)
	BERNHAUSEN_NORD_WEST_RING = 'de:08116:2079'
	BERNHAUSEN_NORD_WEST_RING_1 = 'de:08116:2079:0:3'  # Rtg Fröbelstr. (Filderstadt Bernhausen)
	BERNHAUSEN_NORD_WEST_RING_2 = 'de:08116:2079:0:4'  # Rtg In den Entenäckern (Filderstadt Bernhausen)
	HARTHAUSEN_ROSSBERGSTRASSE = 'de:08116:2080'
	HARTHAUSEN_ROSSBERGSTRASSE_1 = 'de:08116:2080:0:3'  # Rtg Carl-Zeiss-Str. (Filderstadt Harthausen)
	HARTHAUSEN_ROSSBERGSTRASSE_2 = 'de:08116:2080:0:4'  # Rtg Kirche (Filderstadt Harthausen)
	ROSSBERGSTRASSE = 'de:08116:2080'
	ROSSBERGSTRASSE_1 = 'de:08116:2080:0:3'  # Rtg Carl-Zeiss-Str. (Filderstadt Harthausen)
	ROSSBERGSTRASSE_2 = 'de:08116:2080:0:4'  # Rtg Kirche (Filderstadt Harthausen)
	HARTHAUSEN_CARL_ZEISS_STRASSE = 'de:08116:2081'
	HARTHAUSEN_CARL_ZEISS_STRASSE_1 = 'de:08116:2081:0:3'  # Ankunft (Filderstadt Harthausen)
	HARTHAUSEN_CARL_ZEISS_STRASSE_2 = 'de:08116:2081:0:4'  # Rtg Rossbergstr. (Filderstadt Harthausen)
	CARL_ZEISS_STRASSE_1 = 'de:08116:2081:0:3'  # Ankunft (Filderstadt Harthausen)
	CARL_ZEISS_STRASSE_2 = 'de:08116:2081:0:4'  # Rtg Rossbergstr. (Filderstadt Harthausen)
	CARL_ZEISS_STRASSE_3 = 'de:08115:5824:0:3'  # Rtg Rathaus (Leonberg Gebersheim)
	CARL_ZEISS_STRASSE_4 = 'de:08115:5824:0:4'  # Rtg Leonberg (Leonberg Gebersheim)
	CARL_ZEISS_STRASSE_5 = 'de:08118:6516:0:3'  # eRtg Ilsfelder Straße (Besigheim Ottmarsheim)
	AM_LINDLE = 'de:08116:2082'
	AM_LINDLE_1 = 'de:08116:2082:0:3'  # Rtg Hauptstraße (Filderstadt Bonlanden)
	AM_LINDLE_2 = 'de:08116:2082:0:4'  # Rtg Fildorado (Filderstadt Bonlanden)
	BONLANDEN_AM_LINDLE = 'de:08116:2082'
	BONLANDEN_AM_LINDLE_1 = 'de:08116:2082:0:3'  # Rtg Hauptstraße (Filderstadt Bonlanden)
	BONLANDEN_AM_LINDLE_2 = 'de:08116:2082:0:4'  # Rtg Fildorado (Filderstadt Bonlanden)
	BERNHAUSEN_NORD_WEST_UMFAHRUNG = 'de:08116:2084:0:4'  # Rtg Bernhausen TÜV (Filderstadt Bernhausen)
	NORD_WEST_UMFAHRUNG = 'de:08116:2084:0:4'  # Rtg Bernhausen TÜV (Filderstadt Bernhausen)
	HAUSÄCKER = 'de:08116:2085'
	HAUSÄCKER_1 = 'de:08116:2085:0:3'  # Rtg Harthäuser Weg (Filderstadt Sielmingen)
	HAUSÄCKER_2 = 'de:08116:2085:0:4'  # Rtg Kantstr. (Filderstadt Sielmingen)
	SIELMINGEN_HAUSÄCKER = 'de:08116:2085'
	SIELMINGEN_HAUSÄCKER_1 = 'de:08116:2085:0:3'  # Rtg Harthäuser Weg (Filderstadt Sielmingen)
	SIELMINGEN_HAUSÄCKER_2 = 'de:08116:2085:0:4'  # Rtg Kantstr. (Filderstadt Sielmingen)
	HARTHAUSEN_HASENHEIM = 'de:08116:2086'
	HARTHAUSEN_HASENHEIM_1 = 'de:08116:2086:0:3'  # Rtg Grötz. Schönblick (Filderstadt Harthausen)
	HARTHAUSEN_HASENHEIM_2 = 'de:08116:2086:0:4'  # Rtg Carl-Zeiss-Str. (Filderstadt Harthausen)
	HASENHEIM = 'de:08116:2086'
	HASENHEIM_1 = 'de:08116:2086:0:3'  # Rtg Grötz. Schönblick (Filderstadt Harthausen)
	HASENHEIM_2 = 'de:08116:2086:0:4'  # Rtg Carl-Zeiss-Str. (Filderstadt Harthausen)
	KLINKERNFELD = 'de:08116:2087'
	KLINKERNFELD_1 = 'de:08116:2087:0:3'  # Rtg Bernhausen Bf (Filderstadt Bernhausen)
	KLINKERNFELD_2 = 'de:08116:2087:0:4'  # Rtg Mühlenbuckel (Filderstadt Bernhausen)
	BERNHAUSEN_KLINKERNFELD = 'de:08116:2087'
	BERNHAUSEN_KLINKERNFELD_1 = 'de:08116:2087:0:3'  # Rtg Bernhausen Bf (Filderstadt Bernhausen)
	BERNHAUSEN_KLINKERNFELD_2 = 'de:08116:2087:0:4'  # Rtg Mühlenbuckel (Filderstadt Bernhausen)
	BERNHAUSEN_MÜHLENBUCKEL = 'de:08116:2088'
	BERNHAUSEN_MÜHLENBUCKEL_1 = 'de:08116:2088:0:3'  # Rtg Klinkernfeld (Filderstadt Bernhausen)
	BERNHAUSEN_MÜHLENBUCKEL_2 = 'de:08116:2088:0:4'  # Rtg Sielmingen Schul-/Sportzentrum (Filderstadt Bernhausen)
	MÜHLENBUCKEL = 'de:08116:2088'
	MÜHLENBUCKEL_1 = 'de:08116:2088:0:3'  # Rtg Klinkernfeld (Filderstadt Bernhausen)
	MÜHLENBUCKEL_2 = 'de:08116:2088:0:4'  # Rtg Sielmingen Schul-/Sportzentrum (Filderstadt Bernhausen)
	TURNACKERSTRASSE = 'de:08116:2089'
	TURNACKERSTRASSE_1 = 'de:08116:2089:0:3'  # Rtg Johannesstraße (Filderstadt Bernhausen)
	TURNACKERSTRASSE_2 = 'de:08116:2089:0:4'  # Rtg Luftfrachtzentrum West (Filderstadt Bernhausen)
	BERNHAUSEN_TURNACKERSTR = 'de:08116:2089'
	BERNHAUSEN_TURNACKERSTR_1 = 'de:08116:2089:0:3'  # Rtg Johannesstraße (Filderstadt Bernhausen)
	BERNHAUSEN_TURNACKERSTR_2 = 'de:08116:2089:0:4'  # Rtg Luftfrachtzentrum West (Filderstadt Bernhausen)
	BERNH__LUFTFRACHTZENTRUM_WEST = 'de:08116:2090'
	BERNH__LUFTFRACHTZENTRUM_WEST_1 = 'de:08116:2090:0:3'  # Rtg Turnackerstraße (Filderstadt Bernhausen)
	BERNH__LUFTFRACHTZENTRUM_WEST_2 = 'de:08116:2090:0:4'  # Rtg Bernhausen Nord-West-Umfahrung (Filderstadt Bernhausen)
	LUFTFRACHTZENTRUM_WEST = 'de:08116:2090'
	LUFTFRACHTZENTRUM_WEST_1 = 'de:08116:2090:0:3'  # Rtg Turnackerstraße (Filderstadt Bernhausen)
	LUFTFRACHTZENTRUM_WEST_2 = 'de:08116:2090:0:4'  # Rtg Bernhausen Nord-West-Umfahrung (Filderstadt Bernhausen)
	KREUZÄCKER = 'de:08116:2091'
	KREUZÄCKER_1 = 'de:08116:2091:0:3'  # Rtg Bonlandener Str. (Filderstadt Bonlanden)
	KREUZÄCKER_2 = 'de:08116:2091:0:4'  # Rtg Metzinger Str. (Filderstadt Bonlanden)
	BONLANDEN_KREUZÄCKER = 'de:08116:2091'
	BONLANDEN_KREUZÄCKER_1 = 'de:08116:2091:0:3'  # Rtg Bonlandener Str. (Filderstadt Bonlanden)
	BONLANDEN_KREUZÄCKER_2 = 'de:08116:2091:0:4'  # Rtg Metzinger Str. (Filderstadt Bonlanden)
	RESSESTRASSE = 'de:08111:2092'
	RESSESTRASSE_1 = 'de:08111:2092:0:3'  # Rtg Scharnhäuser Brücke (Stuttgart Stuttgart)
	RESSESTRASSE_2 = 'de:08111:2092:0:4'  # Rtg Plieningen Post (Stuttgart Stuttgart)
	RESSESTRASSE_3 = 'de:08111:2092:0:5'  # Ersatzhst Rtg Scharnhäuser Brücke (Stuttgart Stuttgart)
	RESSESTRASSE_4 = 'de:08111:2092:0:6'  # Ersatzhst Rtg Plieningen Post (Stuttgart Stuttgart)
	IN_DEN_ENTENÄCKERN = 'de:08111:2093'
	IN_DEN_ENTENÄCKERN_1 = 'de:08111:2093:0:3'  # Rtg Bernhausen Nord-West-Ring (Stuttgart Stuttgart)
	IN_DEN_ENTENÄCKERN_2 = 'de:08111:2093:0:4'  # Rtg Bernhäuser Str. (Stuttgart Stuttgart)
	IM_KÖPFERT = 'de:08111:2094'
	IM_KÖPFERT_1 = 'de:08111:2094:0:3'  # Rtg Messe West (Stuttgart Stuttgart)
	IM_KÖPFERT_2 = 'de:08111:2094:0:4'  # Rtg Plieningen Post (Stuttgart Stuttgart)
	ESELSMÜHLE = 'de:08116:2098'
	ESELSMÜHLE_1 = 'de:08116:2098:0:3'  # Rtg Musberg Seebruckenmühle (Klärwerk) (Leinfelden-Echterdingen Musberg)
	ESELSMÜHLE_2 = 'de:08116:2098:0:4'  # Rtg Leinfelden Riedweg (Leinfelden-Echterdingen Musberg)
	MUSBERG_ESELSMÜHLE = 'de:08116:2098'
	MUSBERG_ESELSMÜHLE_1 = 'de:08116:2098:0:3'  # Rtg Musberg Seebruckenmühle (Klärwerk) (Leinfelden-Echterdingen Musberg)
	MUSBERG_ESELSMÜHLE_2 = 'de:08116:2098:0:4'  # Rtg Leinfelden Riedweg (Leinfelden-Echterdingen Musberg)
	LEINFELDEN_RIEDWEG = 'de:08116:2099'
	LEINFELDEN_RIEDWEG_1 = 'de:08116:2099:0:1'  # Rtg Spielkartenmuseum (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_RIEDWEG_2 = 'de:08116:2099:0:2'  # Rtg Musberg Wanderweg/Eselsmühle (Leinfelden-Echterdingen Leinfelden)
	RIEDWEG = 'de:08116:2099'
	RIEDWEG_1 = 'de:08116:2099:0:1'  # Rtg Spielkartenmuseum (Leinfelden-Echterdingen Leinfelden)
	RIEDWEG_2 = 'de:08116:2099:0:2'  # Rtg Musberg Wanderweg/Eselsmühle (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_HANS_BRÜMMER_PLATZ = 'de:08116:2100'
	LEINFELDEN_HANS_BRÜMMER_PLATZ_1 = 'de:08116:2100:0:3'  # Rtg Oberaichen Bahnhof (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_HANS_BRÜMMER_PLATZ_2 = 'de:08116:2100:0:4'  # Rtg Oberaichen Waldheim (Leinfelden-Echterdingen Leinfelden)
	HANS_BRÜMMER_PLATZ = 'de:08116:2100'
	HANS_BRÜMMER_PLATZ_1 = 'de:08116:2100:0:3'  # Rtg Oberaichen Bahnhof (Leinfelden-Echterdingen Leinfelden)
	HANS_BRÜMMER_PLATZ_2 = 'de:08116:2100:0:4'  # Rtg Oberaichen Waldheim (Leinfelden-Echterdingen Leinfelden)
	BONLÄNDER_STRASSE = 'de:08116:2101'
	BONLÄNDER_STRASSE_1 = 'de:08116:2101:0:3'  # Rtg Holderweg (Leinfelden-Echterdingen Echterdingen)
	BONLÄNDER_STRASSE_2 = 'de:08116:2101:0:4'  # Rtg Waldhornstr. (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_BONLÄNDER_STRASSE = 'de:08116:2101'
	ECHTERDINGEN_BONLÄNDER_STRASSE_1 = 'de:08116:2101:0:3'  # Rtg Holderweg (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_BONLÄNDER_STRASSE_2 = 'de:08116:2101:0:4'  # Rtg Waldhornstr. (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_FRIEDHOF = 'de:08116:2102'
	ECHTERDINGEN_FRIEDHOF_1 = 'de:08116:2102:0:3'  # Rtg Messe West (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_FRIEDHOF_2 = 'de:08116:2102:0:4'  # Rtg Stadionstraße (Leinfelden-Echterdingen Echterdingen)
	FLUGHAFEN_MESSE = 'de:08116:2103'
	FLUGHAFEN_MESSE_1 = 'de:08116:2103:1:1'  # Gleis 1 (Leinfelden-Echterdingen Echterdingen)
	FLUGHAFEN_MESSE_2 = 'de:08116:2103:1:2'  # Gleis 2 (Leinfelden-Echterdingen Echterdingen)
	FLUGHAFEN_MESSE_3 = 'de:08116:2103:3:1'  # Linien X10, X60 (Leinfelden-Echterdingen Echterdingen)
	FLUGHAFEN_MESSE_4 = 'de:08116:2103:3:2'  # Linien 122, 828 (Leinfelden-Echterdingen Echterdingen)
	FLUGHAFEN_MESSE_5 = 'de:08116:2103:3:3'  # Linien 806, 812, 813, X3 (Leinfelden-Echterdingen Echterdingen)
	FLUGHAFEN_MESSE_6 = 'de:08116:2103:3:4'  # Linie 65 (Leinfelden-Echterdingen Echterdingen)
	FLUGHAFEN_MESSE_7 = 'de:08116:2103:3:A'  # Ankunft (Leinfelden-Echterdingen Echterdingen)
	FLUGHAFEN_MESSE_8 = 'de:08116:2103:4:1'  # Stadtbahn - Endhst (Leinfelden-Echterdingen Echterdingen)
	OBERAICHEN = 'de:08116:2105'
	OBERAICHEN_1 = 'de:08116:2105:1:1'  # Gleis 1 (Leinfelden-Echterdingen Leinfelden)
	OBERAICHEN_2 = 'de:08116:2105:2:2'  # Gleis 2 (Leinfelden-Echterdingen Leinfelden)
	OBERAICHEN_3 = 'de:08116:2105:3:3'  # Linie 38 Rtg Manosquer Str. (Leinfelden-Echterdingen Leinfelden)
	OBERAICHEN_4 = 'de:08116:2105:4:4'  # Linie 38 Rtg Hans-Brümmer-Platz (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_OBERAICHEN_WALDHEIM = 'de:08116:2106'
	LEINFELDEN_OBERAICHEN_WALDHEIM_1 = 'de:08116:2106:0:3'  # Rtg Schmellbachtal/H.-Brümmer-Pl. (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_OBERAICHEN_WALDHEIM_2 = 'de:08116:2106:0:4'  # Rtg Musberg Eulenberg (Leinfelden-Echterdingen Leinfelden)
	OBERAICHEN_WALDHEIM = 'de:08116:2106'
	OBERAICHEN_WALDHEIM_1 = 'de:08116:2106:0:3'  # Rtg Schmellbachtal/H.-Brümmer-Pl. (Leinfelden-Echterdingen Leinfelden)
	OBERAICHEN_WALDHEIM_2 = 'de:08116:2106:0:4'  # Rtg Musberg Eulenberg (Leinfelden-Echterdingen Leinfelden)
	EULENBERG = 'de:08116:2107'
	EULENBERG_1 = 'de:08116:2107:0:3'  # Rtg Oberaichen Waldheim (Leinfelden-Echterdingen Musberg)
	EULENBERG_2 = 'de:08116:2107:0:4'  # Rtg Klingenstraße (Leinfelden-Echterdingen Musberg)
	MUSBERG_EULENBERG = 'de:08116:2107'
	MUSBERG_EULENBERG_1 = 'de:08116:2107:0:3'  # Rtg Oberaichen Waldheim (Leinfelden-Echterdingen Musberg)
	MUSBERG_EULENBERG_2 = 'de:08116:2107:0:4'  # Rtg Klingenstraße (Leinfelden-Echterdingen Musberg)
	MUSBERG_KIRCHE = 'de:08116:2108'
	MUSBERG_KIRCHE_1 = 'de:08116:2108:0:3'  # Rtg Musberg Friedhof/Wanderweg (Leinfelden-Echterdingen Musberg)
	MUSBERG_KIRCHE_2 = 'de:08116:2108:0:4'  # Rtg Sonnenhalde (Leinfelden-Echterdingen Musberg)
	LUDWIGSTRASSE_1 = 'de:08116:2109:0:3'  # Rtg Musberg Sonnenhalde (Leinfelden-Echterdingen Musberg)
	LUDWIGSTRASSE_2 = 'de:08116:2109:0:4'  # Rtg Klingenstr. (Leinfelden-Echterdingen Musberg)
	LUDWIGSTRASSE_3 = 'de:08117:7016:0:H1'  # Rtg Marienstr. (Göppingen Faurndau)
	LUDWIGSTRASSE_4 = 'de:08117:7016:0:R1'  # Rtg Schubartstr. (Göppingen Faurndau)
	MUSBERG_LUDWIGSTRASSE = 'de:08116:2109'
	MUSBERG_LUDWIGSTRASSE_1 = 'de:08116:2109:0:3'  # Rtg Musberg Sonnenhalde (Leinfelden-Echterdingen Musberg)
	MUSBERG_LUDWIGSTRASSE_2 = 'de:08116:2109:0:4'  # Rtg Klingenstr. (Leinfelden-Echterdingen Musberg)
	KLINGENSTRASSE_1 = 'de:08116:2110:0:3'  # Rtg Eulenberg (Leinfelden-Echterdingen Musberg)
	KLINGENSTRASSE_2 = 'de:08116:2110:0:4'  # Rtg Musberg Ludwigstr. (Leinfelden-Echterdingen Musberg)
	KLINGENSTRASSE_3 = 'de:08118:3601:0:3'  # eRtg Birkhau (Affalterbach Affalterbach)
	KLINGENSTRASSE_4 = 'de:08118:3601:0:4'  # eRtg Waage (Affalterbach Affalterbach)
	KLINGENSTRASSE_5 = 'de:08119:5157:0:3'  # Endstation bzw. Start (Waiblingen Bittenfeld)
	MUSBERG_KLINGENSTRASSE = 'de:08116:2110'
	MUSBERG_KLINGENSTRASSE_1 = 'de:08116:2110:0:3'  # Rtg Eulenberg (Leinfelden-Echterdingen Musberg)
	MUSBERG_KLINGENSTRASSE_2 = 'de:08116:2110:0:4'  # Rtg Musberg Ludwigstr. (Leinfelden-Echterdingen Musberg)
	STETTEN__F__HOHLWEG = 'de:08116:2111:0:4'  # Rtg Echterd. Bonländer Str. (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	HOHLWEG = 'de:08116:2111:0:4'  # Rtg Echterd. Bonländer Str. (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__HOLDERWEG = 'de:08116:2112'
	STETTEN__F__HOLDERWEG_1 = 'de:08116:2112:0:3'  # Rtg Hof/Untere Halde (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__HOLDERWEG_2 = 'de:08116:2112:0:4'  # Rtg Hohlweg (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	HOLDERWEG = 'de:08116:2112'
	HOLDERWEG_1 = 'de:08116:2112:0:3'  # Rtg Hof/Untere Halde (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	HOLDERWEG_2 = 'de:08116:2112:0:4'  # Rtg Hohlweg (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	KASPARSWALD = 'de:08116:2114'
	KASPARSWALD_1 = 'de:08116:2114:0:3'  # Rtg Plattenhardt Waldstr. (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	KASPARSWALD_2 = 'de:08116:2114:0:4'  # Rtg Hof (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__KASPARSWALD = 'de:08116:2114'
	STETTEN__F__KASPARSWALD_1 = 'de:08116:2114:0:3'  # Rtg Plattenhardt Waldstr. (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__KASPARSWALD_2 = 'de:08116:2114:0:4'  # Rtg Hof (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__HOF = 'de:08116:2115'
	STETTEN__F__HOF_1 = 'de:08116:2115:0:3'  # Rtg Kasparswald (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__HOF_2 = 'de:08116:2115:0:4'  # Rtg Holderweg (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	HOF_1 = 'de:08116:2115:0:3'  # Rtg Kasparswald (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	HOF_2 = 'de:08116:2115:0:4'  # Rtg Holderweg (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	HOF_3 = 'de:08119:5347:0:3'  # eRtg Marienheim (Backnang Sachsenweiler)
	HOF_4 = 'de:08119:5347:0:4'  # eRtg Am Dresselbach (Backnang Sachsenweiler)
	SCHATTENGRUND = 'de:08111:2118'
	SCHATTENGRUND_1 = 'de:08111:2118:0:3'  # Rtg Am Schattwald (Stuttgart Stuttgart)
	SCHATTENGRUND_2 = 'de:08111:2118:0:4'  # Rtg Bruderhaus (Stuttgart Stuttgart)
	UNTERE_HALDE_1 = 'de:08116:2121:0:3'  # Rtg Weidach (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	UNTERE_HALDE_2 = 'de:08116:2121:0:4'  # Rtg Holderweg (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	UNTERE_HALDE_3 = 'de:08115:7037:1:1'  # R-Bahn (Weil im Schönbuch Weil im Schönbuch)
	STETTEN__F__UNTERE_HALDE = 'de:08116:2121'
	STETTEN__F__UNTERE_HALDE_1 = 'de:08116:2121:0:3'  # Rtg Weidach (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__UNTERE_HALDE_2 = 'de:08116:2121:0:4'  # Rtg Holderweg (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	LINDACHSCHULE = 'de:08116:2122'
	LINDACHSCHULE_1 = 'de:08116:2122:1:3'  # Rtg Forststr. (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	LINDACHSCHULE_2 = 'de:08116:2122:1:4'  # Rtg Weidach (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__LINDACHSCHULE = 'de:08116:2122'
	STETTEN__F__LINDACHSCHULE_1 = 'de:08116:2122:1:3'  # Rtg Forststr. (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__LINDACHSCHULE_2 = 'de:08116:2122:1:4'  # Rtg Weidach (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	WEIDACH = 'de:08116:2123'
	WEIDACH_1 = 'de:08116:2123:0:3'  # Rtg Lindachschule (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	WEIDACH_2 = 'de:08116:2123:0:4'  # Rtg Untere Halde (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__WEIDACH = 'de:08116:2123'
	STETTEN__F__WEIDACH_1 = 'de:08116:2123:0:3'  # Rtg Lindachschule (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__WEIDACH_2 = 'de:08116:2123:0:4'  # Rtg Untere Halde (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__ERLACH = 'de:08116:2124'
	STETTEN__F__ERLACH_1 = 'de:08116:2124:0:3'  # Abfahrt Rtg Forststraße (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__ERLACH_2 = 'de:08116:2124:0:4'  # Ankunft (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	ERLACH_1 = 'de:08116:2124:0:3'  # Abfahrt Rtg Forststraße (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	ERLACH_2 = 'de:08116:2124:0:4'  # Ankunft (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	ERLACH_3 = 'de:08119:7514:0:3'  # eRtg Hohenbrach (Großerlach Großerlach)
	ERLACH_4 = 'de:08119:7514:0:4'  # eRtg Murrhardt/Mainhardt (Großerlach Großerlach)
	LEINFELDEN_SCHMELLBACHTAL = 'de:08116:2125'
	LEINFELDEN_SCHMELLBACHTAL_1 = 'de:08116:2125:0:3'  # Rtg Leinf. Oberaichen Waldheim (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_SCHMELLBACHTAL_2 = 'de:08116:2125:0:4'  # Rtg Rohr Berghaustr. (Leinfelden-Echterdingen Leinfelden)
	SCHMELLBACHTAL = 'de:08116:2125'
	SCHMELLBACHTAL_1 = 'de:08116:2125:0:3'  # Rtg Leinf. Oberaichen Waldheim (Leinfelden-Echterdingen Leinfelden)
	SCHMELLBACHTAL_2 = 'de:08116:2125:0:4'  # Rtg Rohr Berghaustr. (Leinfelden-Echterdingen Leinfelden)
	VEILCHENWEG = 'de:08116:2127'
	VEILCHENWEG_1 = 'de:08116:2127:0:3'  # Rtg Meisenweg (Leinfelden-Echterdingen Leinfelden)
	VEILCHENWEG_2 = 'de:08116:2127:0:4'  # Rtg Manosquer Str. (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_VEILCHENWEG = 'de:08116:2127'
	LEINFELDEN_VEILCHENWEG_1 = 'de:08116:2127:0:3'  # Rtg Meisenweg (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_VEILCHENWEG_2 = 'de:08116:2127:0:4'  # Rtg Manosquer Str. (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_MANOSQUER_STRASSE = 'de:08116:2129'
	LEINFELDEN_MANOSQUER_STRASSE_1 = 'de:08116:2129:0:3'  # Rtg Veilchenweg (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_MANOSQUER_STRASSE_2 = 'de:08116:2129:0:4'  # Rtg Oberaichen (Leinfelden-Echterdingen Leinfelden)
	MANOSQUER_STRASSE = 'de:08116:2129'
	MANOSQUER_STRASSE_1 = 'de:08116:2129:0:3'  # Rtg Veilchenweg (Leinfelden-Echterdingen Leinfelden)
	MANOSQUER_STRASSE_2 = 'de:08116:2129:0:4'  # Rtg Oberaichen (Leinfelden-Echterdingen Leinfelden)
	ECHTERDINGEN_ZAUNACKER = 'de:08116:2130'
	ECHTERDINGEN_ZAUNACKER_1 = 'de:08116:2130:0:3'  # Rtg Stetten Gewerbegebiet (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_ZAUNACKER_2 = 'de:08116:2130:0:4'  # Rtg Friedhof (Leinfelden-Echterdingen Echterdingen)
	ZAUNACKER = 'de:08116:2130'
	ZAUNACKER_1 = 'de:08116:2130:0:3'  # Rtg Stetten Gewerbegebiet (Leinfelden-Echterdingen Echterdingen)
	ZAUNACKER_2 = 'de:08116:2130:0:4'  # Rtg Friedhof (Leinfelden-Echterdingen Echterdingen)
	SPAICHINGER_STRASSE = 'de:08111:2131'
	SPAICHINGER_STRASSE_1 = 'de:08111:2131:0:3'  # Rtg Schemppstr. (Stuttgart Stuttgart)
	SPAICHINGER_STRASSE_2 = 'de:08111:2131:0:4'  # Nachtbus (Stuttgart Stuttgart)
	SPAICHINGER_STRASSE_3 = 'de:08111:2131:0:6'  # Rtg Augustinum (Stuttgart Stuttgart)
	ECHTERD__NIKOLAUS_OTTO_STRASSE = 'de:08116:2132'
	ECHTERD__NIKOLAUS_OTTO_STRASSE_1 = 'de:08116:2132:0:3'  # Rtg Bahnhof (Leinfelden-Echterdingen Echterdingen)
	ECHTERD__NIKOLAUS_OTTO_STRASSE_2 = 'de:08116:2132:0:4'  # Rtg Magellanstraße (Leinfelden-Echterdingen Echterdingen)
	NIKOLAUS_OTTO_STRASSE = 'de:08116:2132'
	NIKOLAUS_OTTO_STRASSE_1 = 'de:08116:2132:0:3'  # Rtg Bahnhof (Leinfelden-Echterdingen Echterdingen)
	NIKOLAUS_OTTO_STRASSE_2 = 'de:08116:2132:0:4'  # Rtg Magellanstraße (Leinfelden-Echterdingen Echterdingen)
	MARTIN_LUTHER_STRASSE_1 = 'de:08116:2134:0:3'  # Rtg Stangen (Leinfelden-Echterdingen Echterdingen)
	MARTIN_LUTHER_STRASSE_2 = 'de:08116:2134:0:4'  # Rtg Burgstraße (Leinfelden-Echterdingen Echterdingen)
	MARTIN_LUTHER_STRASSE_3 = 'de:08118:5510:0:3'  # Rtg ZOB (Ludwigsburg Ludwigsburg)
	MARTIN_LUTHER_STRASSE_4 = 'de:08118:5510:0:4'  # Rtg Kurfürstenstraße/Albrechstr. (Ludwigsburg Ludwigsburg)
	ECHTERDINGEN_M__LUTHER_STRASSE = 'de:08116:2134'
	ECHTERDINGEN_M__LUTHER_STRASSE_1 = 'de:08116:2134:0:3'  # Rtg Stangen (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_M__LUTHER_STRASSE_2 = 'de:08116:2134:0:4'  # Rtg Burgstraße (Leinfelden-Echterdingen Echterdingen)
	GRÖTZINGEN_FROSCHEGERT = 'de:08116:2135'
	GRÖTZINGEN_FROSCHEGERT_1 = 'de:08116:2135:0:3'  # eRtg Schönblick (Aichtal Grötzingen)
	GRÖTZINGEN_FROSCHEGERT_2 = 'de:08116:2135:0:4'  # eRtg Rudolfshöhe (Aichtal Grötzingen)
	FROSCHEGERT = 'de:08116:2135'
	FROSCHEGERT_1 = 'de:08116:2135:0:3'  # eRtg Schönblick (Aichtal Grötzingen)
	FROSCHEGERT_2 = 'de:08116:2135:0:4'  # eRtg Rudolfshöhe (Aichtal Grötzingen)
	FORSTSTRASSE_1 = 'de:08116:2136:0:3'  # Rtg Erlach (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	FORSTSTRASSE_2 = 'de:08116:2136:0:4'  # Rtg Lindachschule (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	FORSTSTRASSE_3 = 'de:08119:7542:0:3'  # eRtg Hauptstraße (Kaisersbach Kaisersbach)
	FORSTSTRASSE_4 = 'de:08119:7542:0:4'  # eRtg Grairich (Kaisersbach Kaisersbach)
	STETTEN__F__FORSTSTRASSE = 'de:08116:2136'
	STETTEN__F__FORSTSTRASSE_1 = 'de:08116:2136:0:3'  # Rtg Erlach (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__FORSTSTRASSE_2 = 'de:08116:2136:0:4'  # Rtg Lindachschule (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	SONNENHALDE_1 = 'de:08116:2138:0:3'  # Rtg Musberg Kirche (Leinfelden-Echterdingen Musberg)
	SONNENHALDE_2 = 'de:08116:2138:0:4'  # Rtg Ludwigstr. (Leinfelden-Echterdingen Musberg)
	SONNENHALDE_3 = 'de:08116:5011:0:3'  # Rtg Sonnenhalde (Ostfildern Scharnhausen)
	SONNENHALDE_4 = 'de:08116:5011:0:4'  # Rtg Kreuzbrunnen (Ostfildern Scharnhausen)
	SONNENHALDE_5 = 'de:08118:6700:0:3'  # eRtg Hofen (Bönnigheim Hofen (Bönnigheim))
	SONNENHALDE_6 = 'de:08118:6700:0:4'  # eRtg Erligheim (Bönnigheim Hofen (Bönnigheim))
	MUSBERG_SONNENHALDE = 'de:08116:2138'
	MUSBERG_SONNENHALDE_1 = 'de:08116:2138:0:3'  # Rtg Musberg Kirche (Leinfelden-Echterdingen Musberg)
	MUSBERG_SONNENHALDE_2 = 'de:08116:2138:0:4'  # Rtg Ludwigstr. (Leinfelden-Echterdingen Musberg)
	SIELMINGER_STRASSE = 'de:08116:2139'
	SIELMINGER_STRASSE_1 = 'de:08116:2139:0:3'  # Rtg Stetten (F) Untere Halde / Hof (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	SIELMINGER_STRASSE_2 = 'de:08116:2139:0:4'  # Rtg Echterdingen Zaunacker (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__SIELMINGER_STR = 'de:08116:2139'
	STETTEN__F__SIELMINGER_STR_1 = 'de:08116:2139:0:3'  # Rtg Stetten (F) Untere Halde / Hof (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__SIELMINGER_STR_2 = 'de:08116:2139:0:4'  # Rtg Echterdingen Zaunacker (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	ECHTERDINGEN_MAGELLANSTR = 'de:08116:2141'
	ECHTERDINGEN_MAGELLANSTR_1 = 'de:08116:2141:0:3'  # Rtg Nikolaus-Otto-Straße (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_MAGELLANSTR_2 = 'de:08116:2141:0:4'  # Rtg Stangen (Leinfelden-Echterdingen Echterdingen)
	MAGELLANSTRASSE = 'de:08116:2141'
	MAGELLANSTRASSE_1 = 'de:08116:2141:0:3'  # Rtg Nikolaus-Otto-Straße (Leinfelden-Echterdingen Echterdingen)
	MAGELLANSTRASSE_2 = 'de:08116:2141:0:4'  # Rtg Stangen (Leinfelden-Echterdingen Echterdingen)
	KELTERRAIN = 'de:08116:2142'
	KELTERRAIN_1 = 'de:08116:2142:0:3'  # Rtg Leinfelden Sportzentrum (Leinfelden-Echterdingen Echterdingen)
	KELTERRAIN_2 = 'de:08116:2142:0:4'  # eRtg Tübinger Straße (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_KELTERRAIN = 'de:08116:2142'
	ECHTERDINGEN_KELTERRAIN_1 = 'de:08116:2142:0:3'  # Rtg Leinfelden Sportzentrum (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_KELTERRAIN_2 = 'de:08116:2142:0:4'  # eRtg Tübinger Straße (Leinfelden-Echterdingen Echterdingen)
	SIELMINGEN_MÖHRINGER_STRASSE = 'de:08116:2143:0:3'  # Rtg Reutlinger Straße (Filderstadt Sielmingen)
	MÖHRINGER_STRASSE = 'de:08116:2143:0:3'  # Rtg Reutlinger Straße (Filderstadt Sielmingen)
	REUTLINGER_STRASSE_1 = 'de:08116:2144:0:3'  # Rtg Plattenh. Griebenäcker (Filderstadt Sielmingen)
	REUTLINGER_STRASSE_2 = 'de:08116:4031:0:3'  # Rtg Esslingen (Esslingen am Neckar Esslingen (N))
	REUTLINGER_STRASSE_3 = 'de:08116:4031:0:4'  # Rtg Zell (Esslingen am Neckar Esslingen (N))
	REUTLINGER_STRASSE_4 = 'de:08116:5019:0:3'  # Rtg Scharnhausen Nagelstr. (Ostfildern Kemnat)
	REUTLINGER_STRASSE_5 = 'de:08116:5019:0:4'  # Rtg Altes Rathaus (Ostfildern Kemnat)
	REUTLINGER_STRASSE_6 = 'de:08111:6567:0:3'  # Rtg Waldenbucher Pl./Albschule (Stuttgart Stuttgart)
	REUTLINGER_STRASSE_7 = 'de:08111:6567:0:4'  # Rtg Zur Anhöhe (Stuttgart Stuttgart)
	REUTLINGER_STRASSE_8 = 'de:08117:1007:0:H1'  # Rtg Eislingen (Göppingen Göppingen)
	REUTLINGER_STRASSE_9 = 'de:08117:1007:0:R1'  # Rtg Freibad (Göppingen Göppingen)
	SIELMINGEN_REUTLINGER_STRASSE = 'de:08116:2144:0:3'  # Rtg Plattenh. Griebenäcker (Filderstadt Sielmingen)
	ECHTERDINGEN_BURGSTRASSE = 'de:08116:2145'
	ECHTERDINGEN_BURGSTRASSE_1 = 'de:08116:2145:0:3'  # Rtg M.-Luther-Str. (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_BURGSTRASSE_2 = 'de:08116:2145:0:4'  # Rtg Friedhof (Leinfelden-Echterdingen Echterdingen)
	BURGSTRASSE_1 = 'de:08116:2145:0:3'  # Rtg M.-Luther-Str. (Leinfelden-Echterdingen Echterdingen)
	BURGSTRASSE_2 = 'de:08116:2145:0:4'  # Rtg Friedhof (Leinfelden-Echterdingen Echterdingen)
	BURGSTRASSE_3 = 'de:08119:6698:0:3'  # eRtg Feuerwehr (Urbach Urbach)
	BURGSTRASSE_4 = 'de:08119:6698:0:4'  # Rtg Seehalde (Urbach Urbach)
	LEINFELDEN_STADTBÜCHEREI = 'de:08116:2147'
	LEINFELDEN_STADTBÜCHEREI_1 = 'de:08116:2147:0:3'  # Rtg Neuer Markt/Spielkartenmuseum (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_STADTBÜCHEREI_2 = 'de:08116:2147:0:4'  # Rtg Leinfelden Bf (Leinfelden-Echterdingen Leinfelden)
	STADTBÜCHEREI = 'de:08116:2147'
	STADTBÜCHEREI_1 = 'de:08116:2147:0:3'  # Rtg Neuer Markt/Spielkartenmuseum (Leinfelden-Echterdingen Leinfelden)
	STADTBÜCHEREI_2 = 'de:08116:2147:0:4'  # Rtg Leinfelden Bf (Leinfelden-Echterdingen Leinfelden)
	GEWERBEGEBIET_1 = 'de:08116:2156:0:3'  # Rtg Sielminger Str. (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	GEWERBEGEBIET_2 = 'de:08116:2156:0:4'  # Rtg Bernhausen TÜV (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	GEWERBEGEBIET_3 = 'de:08119:3256:0:1'  # Rtg Adelstetten (Alfdorf Alfdorf)
	GEWERBEGEBIET_4 = 'de:08118:5708:0:3'  # eRtg Post (Freudental Freudental)
	GEWERBEGEBIET_5 = 'de:08118:5708:0:4'  # eRtg Hohenhaslach Staffel (Freudental Freudental)
	GEWERBEGEBIET_6 = 'de:08115:6483:0:3'  # Rtg Grabenstr. (Nufringen Nufringen)
	GEWERBEGEBIET_7 = 'de:08115:6483:0:4'  # Rtg Gärtr. Str. (Nufringen Nufringen)
	GEWERBEGEBIET_8 = 'de:08116:6523:0:3'  # Rtg Kreuzung (Lichtenwald Thomashardt)
	GEWERBEGEBIET_9 = 'de:08116:6523:0:4'  # Rtg Hegenloher Str. (Lichtenwald Thomashardt)
	GEWERBEGEBIET_10 = 'de:08119:7445:1:3'  # eRtg Waldrems B14 (Leutenbach Nellmersbach)
	GEWERBEGEBIET_11 = 'de:08119:7445:1:4'  # eRtg Bahnhofstraße (Leutenbach Nellmersbach)
	GEWERBEGEBIET_12 = 'de:08117:5020:0:H1'  # Rtg Schopflenberg (Uhingen Sparwiesen)
	GEWERBEGEBIET_13 = 'de:08117:5020:0:R1'  # Rtg Uhingen (Uhingen Sparwiesen)
	GEWERBEGEBIET_14 = 'de:08425:2635:0:1'  # auswärts (Merklingen Merklingen)
	GEWERBEGEBIET_15 = 'de:08425:2635:0:2'  # einwärts (Merklingen Merklingen)
	STETTEN__F__GEWERBEGEBIET = 'de:08116:2156'
	STETTEN__F__GEWERBEGEBIET_1 = 'de:08116:2156:0:3'  # Rtg Sielminger Str. (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	STETTEN__F__GEWERBEGEBIET_2 = 'de:08116:2156:0:4'  # Rtg Bernhausen TÜV (Leinfelden-Echterdingen Stetten (Leinfelden-Echt.))
	FILDER_AIRPORT_AREAL = 'de:08116:2157'
	FILDER_AIRPORT_AREAL_1 = 'de:08116:2157:0:3'  # Rtg Flughafen (Filderstadt Bernhausen)
	FILDER_AIRPORT_AREAL_2 = 'de:08116:2157:0:4'  # Rtg Turnackerstr. (Filderstadt Bernhausen)
	BERNH__FILDER_AIRPORT_AREAL = 'de:08116:2157'
	BERNH__FILDER_AIRPORT_AREAL_1 = 'de:08116:2157:0:3'  # Rtg Flughafen (Filderstadt Bernhausen)
	BERNH__FILDER_AIRPORT_AREAL_2 = 'de:08116:2157:0:4'  # Rtg Turnackerstr. (Filderstadt Bernhausen)
	MUSBERG_SCHAFGARTENSTR = 'de:08116:2158'
	MUSBERG_SCHAFGARTENSTR_1 = 'de:08116:2158:0:3'  # Rtg Friedhof (Leinfelden-Echterdingen Musberg)
	MUSBERG_SCHAFGARTENSTR_2 = 'de:08116:2158:0:4'  # Rtg Karlstraße (Leinfelden-Echterdingen Musberg)
	SCHAFGARTENSTRASSE = 'de:08116:2158'
	SCHAFGARTENSTRASSE_1 = 'de:08116:2158:0:3'  # Rtg Friedhof (Leinfelden-Echterdingen Musberg)
	SCHAFGARTENSTRASSE_2 = 'de:08116:2158:0:4'  # Rtg Karlstraße (Leinfelden-Echterdingen Musberg)
	IM_KÖLLER = 'de:08116:2159'
	IM_KÖLLER_1 = 'de:08116:2159:0:3'  # Rtg Neuhausen (Filderstadt Sielmingen)
	IM_KÖLLER_2 = 'de:08116:2159:0:4'  # Rtg Bernhausen (Filderstadt Sielmingen)
	SIELMINGEN_IM_KÖLLER = 'de:08116:2159'
	SIELMINGEN_IM_KÖLLER_1 = 'de:08116:2159:0:3'  # Rtg Neuhausen (Filderstadt Sielmingen)
	SIELMINGEN_IM_KÖLLER_2 = 'de:08116:2159:0:4'  # Rtg Bernhausen (Filderstadt Sielmingen)
	KLEINHOHENHEIM = 'de:08111:2186:0:3'  # Rtg Schönberg (Stuttgart Stuttgart)
	BUCHWALD = 'de:08111:2187:0:3'  # Ankunft/Rtg Fuchsrain (Stuttgart Stuttgart)
	FUCHSRAIN_1 = 'de:08111:2188:0:3'  # Rtg Albert-Schäffle-Str. (Stuttgart Stuttgart)
	FUCHSRAIN_2 = 'de:08111:2188:0:4'  # Rtg Buchwald (Stuttgart Stuttgart)
	FUCHSRAIN_3 = 'de:08116:2949:0:3'  # eRtg Hans-von-Hutten-Platz (Nürtingen Hardt)
	ALBERT_SCHÄFFLE_STRASSE = 'de:08111:2189'
	ALBERT_SCHÄFFLE_STRASSE_1 = 'de:08111:2189:0:3'  # Rtg Gablenberg (Stuttgart Stuttgart)
	ALBERT_SCHÄFFLE_STRASSE_2 = 'de:08111:2189:0:4'  # Rtg Fuchsrain (Stuttgart Stuttgart)
	DOGGENBURG = 'de:08111:2190'
	DOGGENBURG_1 = 'de:08111:2190:1:3'  # Rtg Kräherwald (Stuttgart Stuttgart)
	DOGGENBURG_2 = 'de:08111:2190:1:4'  # Rtg Am Bismarckturm (Stuttgart Stuttgart)
	DOGGENBURG_3 = 'de:08111:2190:1:6'  # Rtg Viktor-Köchl-Weg (Stuttgart Stuttgart)
	VIKTOR_KÖCHL_WEG = 'de:08111:2191'
	VIKTOR_KÖCHL_WEG_1 = 'de:08111:2191:0:3'  # Rtg Doggenburg (Stuttgart Stuttgart)
	VIKTOR_KÖCHL_WEG_2 = 'de:08111:2191:0:4'  # Rtg Schottstr. (Stuttgart Stuttgart)
	SCHOTTSTRASSE = 'de:08111:2192'
	SCHOTTSTRASSE_1 = 'de:08111:2192:0:3'  # Rtg Viktor-Köchl-Weg (Stuttgart Stuttgart)
	SCHOTTSTRASSE_2 = 'de:08111:2192:0:4'  # Rtg Ehrenhalde (Stuttgart Stuttgart)
	EHRENHALDE = 'de:08111:2193'
	EHRENHALDE_1 = 'de:08111:2193:0:3'  # Rtg Schottstr. (Stuttgart Stuttgart)
	EHRENHALDE_2 = 'de:08111:2193:0:4'  # Rtg Dillmannstr. (Stuttgart Stuttgart)
	DILLMANNSTRASSE = 'de:08111:2194'
	DILLMANNSTRASSE_1 = 'de:08111:2194:0:3'  # Rtg Ehrenhalde (Stuttgart Stuttgart)
	DILLMANNSTRASSE_2 = 'de:08111:2194:0:4'  # Rtg Hölderlinstr. (Stuttgart Stuttgart)
	HÖLDERLINSTRASSE_1 = 'de:08111:2195:0:3'  # Rtg Dillmannstr. (Stuttgart Stuttgart)
	HÖLDERLINSTRASSE_2 = 'de:08111:2195:0:4'  # Rtg Linden-Museum (Stuttgart Stuttgart)
	HÖLDERLINSTRASSE_3 = 'de:08118:3596:0:3'  # eRtg König-Wilhelm-Platz (Marbach am Neckar Marbach (N))
	LINDEN_MUSEUM = 'de:08111:2196'
	LINDEN_MUSEUM_1 = 'de:08111:2196:1:3'  # Rtg Hegel-/Seidenstr./Rosenb.-/Seidenstr (Stuttgart Stuttgart)
	LINDEN_MUSEUM_2 = 'de:08111:2196:1:5'  # Rtg Hölderlinstr. (Stuttgart Stuttgart)
	LINDEN_MUSEUM_3 = 'de:08111:2196:2:4'  # Rtg Katharinenhospital (Stuttgart Stuttgart)
	LINDEN_MUSEUM_4 = 'de:08111:2196:3:6'  # Rtg Berliner Pl. (Stuttgart Stuttgart)
	WILHELM__OLGASTRASSE = 'de:08111:2198'
	WILHELM__OLGASTRASSE_1 = 'de:08111:2198:0:3'  # Rtg Olgaeck (Stuttgart Stuttgart)
	WILHELM__OLGASTRASSE_2 = 'de:08111:2198:0:4'  # Rtg Falbenhennenstr. (Stuttgart Stuttgart)
	FALBENHENNENSTRASSE = 'de:08111:2199'
	FALBENHENNENSTRASSE_1 = 'de:08111:2199:0:3'  # Rtg Wilhelm-/Olgastr. (Stuttgart Stuttgart)
	FALBENHENNENSTRASSE_2 = 'de:08111:2199:0:4'  # Rtg Zellerstr. (Stuttgart Stuttgart)
	FALBENHENNENSTRASSE_3 = 'de:08111:2199:0:5'  # NBus (Stuttgart Stuttgart)
	GABLENBERG = 'de:08111:2200'
	GABLENBERG_1 = 'de:08111:2200:0:3'  # Rtg Libanonstr. (Stuttgart Stuttgart)
	GABLENBERG_2 = 'de:08111:2200:0:4'  # Rtg A.-Schäffle-Str./Planckstr. (Stuttgart Stuttgart)
	LIBANONSTRASSE = 'de:08111:2201'
	LIBANONSTRASSE_1 = 'de:08111:2201:0:3'  # Rtg Wagenburgstr. (Stuttgart Stuttgart)
	LIBANONSTRASSE_2 = 'de:08111:2201:0:4'  # Rtg Gablenberg (Stuttgart Stuttgart)
	WAGENBURGSTRASSE = 'de:08111:2202'
	WAGENBURGSTRASSE_1 = 'de:08111:2202:1:1'  # L.40 Ankunft/L.42 Rtg Ostendplatz (Stuttgart Stuttgart)
	WAGENBURGSTRASSE_2 = 'de:08111:2202:1:2'  # Rtg Ostendplatz (Stuttgart Stuttgart)
	WAGENBURGSTRASSE_3 = 'de:08111:2202:1:3'  # Rtg Libanonstr. (Stuttgart Stuttgart)
	WAGENBURGSTRASSE_4 = 'de:08111:2202:1:4'  # Rtg Wunnensteinstr. (Stuttgart Stuttgart)
	KATHARINENHOSPITAL = 'de:08111:2203'
	KATHARINENHOSPITAL_1 = 'de:08111:2203:0:3'  # Rtg Linden-Museum (Stuttgart Stuttgart)
	KATHARINENHOSPITAL_2 = 'de:08111:2203:0:4'  # Rtg Hauptbahnhof (Stuttgart Stuttgart)
	ROSENBERG__JOHANNESSTRASSE = 'de:08111:2204'
	ROSENBERG__JOHANNESSTRASSE_1 = 'de:08111:2204:0:3'  # Rtg Rosenbergplatz (Stuttgart Stuttgart)
	ROSENBERG__JOHANNESSTRASSE_2 = 'de:08111:2204:0:4'  # Rtg Rosenberg-/Seidenstr. (Stuttgart Stuttgart)
	ROSENBERGPLATZ = 'de:08111:2205'
	ROSENBERGPLATZ_1 = 'de:08111:2205:0:3'  # Rtg Schwab-/Bebelstr. (Stuttgart Stuttgart)
	ROSENBERGPLATZ_2 = 'de:08111:2205:0:4'  # Rtg Rosenberg-/Johannesstr. (Stuttgart Stuttgart)
	BRUDERHAUS = 'de:08111:2206'
	BRUDERHAUS_1 = 'de:08111:2206:0:3'  # Rtg Schattengrund (Stuttgart Stuttgart)
	BRUDERHAUS_2 = 'de:08111:2206:0:4'  # Rtg Gerlingen Glemstal (Stuttgart Stuttgart)
	SCHWAB__REINSBURGSTRASSE = 'de:08111:2207'
	SCHWAB__REINSBURGSTRASSE_1 = 'de:08111:2207:0:3'  # Rtg Schickhardtschule (Stuttgart Stuttgart)
	SCHWAB__REINSBURGSTRASSE_2 = 'de:08111:2207:0:4'  # Rtg Schwabstr. (Stuttgart Stuttgart)
	SCHWAB__REINSBURGSTRASSE_3 = 'de:08111:2207:0:5'  # Rtg Rotebühl-/Reinsburgstr. (Stuttgart Stuttgart)
	SCHWAB__REINSBURGSTRASSE_4 = 'de:08111:2207:0:6'  # Rtg Silberburg-/Reinsburgstr. (Stuttgart Stuttgart)
	SCHICKHARDTSCHULE = 'de:08111:2208'
	SCHICKHARDTSCHULE_1 = 'de:08111:2208:0:3'  # Rtg Erwin-Schoettle-Platz (Stuttgart Stuttgart)
	SCHICKHARDTSCHULE_2 = 'de:08111:2208:0:4'  # Rtg Schwab-/Reinsburgstr. (Stuttgart Stuttgart)
	RUDOLF_SOPHIEN_STIFT = 'de:08111:2209'
	RUDOLF_SOPHIEN_STIFT_1 = 'de:08111:2209:0:3'  # Rtg Heslach Vogelrain (Stuttgart Stuttgart)
	RUDOLF_SOPHIEN_STIFT_2 = 'de:08111:2209:0:4'  # Rtg Universität (Stuttgart Stuttgart)
	SINDELFINGEN_DAIMLERSTEG = 'de:08115:2210'
	SINDELFINGEN_DAIMLERSTEG_1 = 'de:08115:2210:1:3'  # Rtg Stadion (Sindelfingen Sindelfingen)
	SINDELFINGEN_DAIMLERSTEG_2 = 'de:08115:2210:1:4'  # Rtg ZOB (Sindelfingen Sindelfingen)
	DAIMLERSTEG = 'de:08115:2210'
	DAIMLERSTEG_1 = 'de:08115:2210:1:3'  # Rtg Stadion (Sindelfingen Sindelfingen)
	DAIMLERSTEG_2 = 'de:08115:2210:1:4'  # Rtg ZOB (Sindelfingen Sindelfingen)
	DROSSELWEG_1 = 'de:08115:2211:0:3'  # Rtg Königsknoll (Sindelfingen Sindelfingen)
	DROSSELWEG_2 = 'de:08115:2211:0:4'  # Rtg Mahdentalstr. (Sindelfingen Sindelfingen)
	DROSSELWEG_3 = 'de:08119:2500:0:3'  # Rtg Altenheim (Fellbach Fellbach)
	DROSSELWEG_4 = 'de:08115:6748:0:3'  # eRtg Elsterweg (Grafenau Döffingen)
	SINDELFINGEN_DROSSELWEG = 'de:08115:2211'
	SINDELFINGEN_DROSSELWEG_1 = 'de:08115:2211:0:3'  # Rtg Königsknoll (Sindelfingen Sindelfingen)
	SINDELFINGEN_DROSSELWEG_2 = 'de:08115:2211:0:4'  # Rtg Mahdentalstr. (Sindelfingen Sindelfingen)
	SINDELFINGEN_FELDBERGSTRASSE = 'de:08115:2212'
	SINDELFINGEN_FELDBERGSTRASSE_1 = 'de:08115:2212:0:3'  # Rtg Stg Am Schattwald (Sindelfingen Sindelfingen)
	SINDELFINGEN_FELDBERGSTRASSE_2 = 'de:08115:2212:0:4'  # Rtg Spitzholzstr. (Sindelfingen Sindelfingen)
	SINDELFINGEN_FELDBERGSTRASSE_3 = 'de:08115:2212:0:5'  # Bus Rtg ZOB (Sindelfingen Sindelfingen)
	FELDBERGSTRASSE_1 = 'de:08115:2212:0:3'  # Rtg Stg Am Schattwald (Sindelfingen Sindelfingen)
	FELDBERGSTRASSE_2 = 'de:08115:2212:0:4'  # Rtg Spitzholzstr. (Sindelfingen Sindelfingen)
	FELDBERGSTRASSE_3 = 'de:08115:2212:0:5'  # Bus Rtg ZOB (Sindelfingen Sindelfingen)
	FELDBERGSTRASSE_4 = 'de:08115:4668:0:3'  # eRtg Eichendorff-Schule (Böblingen Böblingen)
	SINDELFINGEN_ELTINGER_STRASSE = 'de:08115:2213:0:3'  # eRtg Essegger Straße (Sindelfingen Sindelfingen)
	ELTINGER_STRASSE_1 = 'de:08115:2213:0:3'  # eRtg Essegger Straße (Sindelfingen Sindelfingen)
	ELTINGER_STRASSE_2 = 'de:08111:6210:1:1'  # Rtg Botnang (Stuttgart Stuttgart)
	ELTINGER_STRASSE_3 = 'de:08111:6210:1:2'  # Rtg Millöckerstr. (Stuttgart Stuttgart)
	ELTINGER_STRASSE_4 = 'de:08111:6210:2:3'  # Rtg Beethovenstr./Händelstr. (Stuttgart Stuttgart)
	ELTINGER_STRASSE_5 = 'de:08111:6210:2:4'  # Rtg Botnang (Stuttgart Stuttgart)
	MESSEHALLE = 'de:08115:2214'
	MESSEHALLE_1 = 'de:08115:2214:0:3'  # Rtg Mahdentalstr. (Sindelfingen Sindelfingen)
	MESSEHALLE_2 = 'de:08115:2214:0:4'  # Rtg Mönchsbrunnen (Sindelfingen Sindelfingen)
	SINDELFINGEN_MESSEHALLE = 'de:08115:2214'
	SINDELFINGEN_MESSEHALLE_1 = 'de:08115:2214:0:3'  # Rtg Mahdentalstr. (Sindelfingen Sindelfingen)
	SINDELFINGEN_MESSEHALLE_2 = 'de:08115:2214:0:4'  # Rtg Mönchsbrunnen (Sindelfingen Sindelfingen)
	PFARRWIESENGYMNASIUM = 'de:08115:2216'
	PFARRWIESENGYMNASIUM_1 = 'de:08115:2216:0:3'  # Rtg Spitzholzstr. (Sindelfingen Sindelfingen)
	PFARRWIESENGYMNASIUM_2 = 'de:08115:2216:0:4'  # Rtg Rotbühl (Sindelfingen Sindelfingen)
	PFARRWIESENGYMNASIUM_3 = 'de:08115:2216:0:6'  # Ersatzhst Rtg Spitzholzstr. (Sindelfingen Sindelfingen)
	PFARRWIESENGYMNASIUM_4 = 'de:08115:2216:0:7'  # Ersatzhst Rtg Rotbühl (Sindelfingen Sindelfingen)
	SINDELFINGEN_PFARRWIESENGYMN = 'de:08115:2216'
	SINDELFINGEN_PFARRWIESENGYMN_1 = 'de:08115:2216:0:3'  # Rtg Spitzholzstr. (Sindelfingen Sindelfingen)
	SINDELFINGEN_PFARRWIESENGYMN_2 = 'de:08115:2216:0:4'  # Rtg Rotbühl (Sindelfingen Sindelfingen)
	SINDELFINGEN_PFARRWIESENGYMN_3 = 'de:08115:2216:0:6'  # Ersatzhst Rtg Spitzholzstr. (Sindelfingen Sindelfingen)
	SINDELFINGEN_PFARRWIESENGYMN_4 = 'de:08115:2216:0:7'  # Ersatzhst Rtg Rotbühl (Sindelfingen Sindelfingen)
	SINDELFINGEN_ROTBÜHL = 'de:08115:2218'
	SINDELFINGEN_ROTBÜHL_1 = 'de:08115:2218:0:3'  # Rtg Pfarrwiesengymnasium (Sindelfingen Sindelfingen)
	SINDELFINGEN_ROTBÜHL_2 = 'de:08115:2218:0:4'  # Rtg Zimmerstr. (Sindelfingen Sindelfingen)
	ROTBÜHL = 'de:08115:2218'
	ROTBÜHL_1 = 'de:08115:2218:0:3'  # Rtg Pfarrwiesengymnasium (Sindelfingen Sindelfingen)
	ROTBÜHL_2 = 'de:08115:2218:0:4'  # Rtg Zimmerstr. (Sindelfingen Sindelfingen)
	SINDELFINGEN_SPITZHOLZSTRASSE = 'de:08115:2220'
	SINDELFINGEN_SPITZHOLZSTRASSE_1 = 'de:08115:2220:0:3'  # Rtg Feldbergstr. (Sindelfingen Sindelfingen)
	SINDELFINGEN_SPITZHOLZSTRASSE_2 = 'de:08115:2220:0:4'  # Rtg Pfarrwiesengymnasium (Sindelfingen Sindelfingen)
	SPITZHOLZSTRASSE = 'de:08115:2220'
	SPITZHOLZSTRASSE_1 = 'de:08115:2220:0:3'  # Rtg Feldbergstr. (Sindelfingen Sindelfingen)
	SPITZHOLZSTRASSE_2 = 'de:08115:2220:0:4'  # Rtg Pfarrwiesengymnasium (Sindelfingen Sindelfingen)
	SINDELFINGEN_STADION = 'de:08115:2221'
	SINDELFINGEN_STADION_1 = 'de:08115:2221:0:3'  # Rtg Zimmerstr. (Sindelfingen Sindelfingen)
	SINDELFINGEN_STADION_2 = 'de:08115:2221:0:4'  # Rtg Daimlersteg (Sindelfingen Sindelfingen)
	STADION_1 = 'de:08115:2221:0:3'  # Rtg Zimmerstr. (Sindelfingen Sindelfingen)
	STADION_2 = 'de:08115:2221:0:4'  # Rtg Daimlersteg (Sindelfingen Sindelfingen)
	STADION_3 = 'de:08115:3130:0:3'  # Rtg Hallenbad (Waldenbuch Waldenbuch)
	STADION_4 = 'de:08115:3130:0:4'  # Rtg Steinenbronn (Waldenbuch Waldenbuch)
	STADION_5 = 'de:08116:4320:0:3'  # eRtg Fauslerstraße (Kirchheim unter Teck Kirchheim (T))
	STADION_6 = 'de:08116:4320:0:4'  # eRtg Freibad/Kirchheim (Kirchheim unter Teck Kirchheim (T))
	STADION_7 = 'de:08115:4824:0:3'  # eRtg Elsenhalde (Schönaich Schönaich)
	STADION_8 = 'de:08116:5006:0:3'  # Rtg Esslinger Str. (Denkendorf Denkendorf)
	STADION_9 = 'de:08116:5006:0:4'  # Rtg Nellingen Denkendorfer Str. (Denkendorf Denkendorf)
	STADION_10 = 'de:08118:5469:0:3'  # eRtg Eichenweg (Kornwestheim Kornwestheim)
	SINDELF__MERCEDES_BENZ_HST__A = 'de:08115:2222:0:3'  # eRtg Mercedes-Benz Hst. B (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_HST__A = 'de:08115:2222:0:3'  # eRtg Mercedes-Benz Hst. B (Sindelfingen Sindelfingen)
	SINDELFINGEN_WALDHEIM = 'de:08115:2223'
	SINDELFINGEN_WALDHEIM_1 = 'de:08115:2223:0:3'  # Rtg Mönchsbrunnen (Sindelfingen Sindelfingen)
	SINDELFINGEN_WALDHEIM_2 = 'de:08115:2223:0:4'  # Rtg Stg Gründgensstr. (Sindelfingen Sindelfingen)
	WALDHEIM = 'de:08115:2223'
	WALDHEIM_1 = 'de:08115:2223:0:3'  # Rtg Mönchsbrunnen (Sindelfingen Sindelfingen)
	WALDHEIM_2 = 'de:08115:2223:0:4'  # Rtg Stg Gründgensstr. (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZIMMERSTRASSE = 'de:08115:2224'
	SINDELFINGEN_ZIMMERSTRASSE_1 = 'de:08115:2224:0:3'  # Rtg Rotbühl (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZIMMERSTRASSE_2 = 'de:08115:2224:0:4'  # Rtg Stadion (Sindelfingen Sindelfingen)
	ZIMMERSTRASSE = 'de:08115:2224'
	ZIMMERSTRASSE_1 = 'de:08115:2224:0:3'  # Rtg Rotbühl (Sindelfingen Sindelfingen)
	ZIMMERSTRASSE_2 = 'de:08115:2224:0:4'  # Rtg Stadion (Sindelfingen Sindelfingen)
	MAHDENTALSTRASSE = 'de:08115:2225'
	MAHDENTALSTRASSE_1 = 'de:08115:2225:0:3'  # Rtg Drosselweg (Sindelfingen Sindelfingen)
	MAHDENTALSTRASSE_2 = 'de:08115:2225:0:4'  # Rtg Messehalle (Sindelfingen Sindelfingen)
	SINDELF__MAHDENTALSTRASSE = 'de:08115:2225'
	SINDELF__MAHDENTALSTRASSE_1 = 'de:08115:2225:0:3'  # Rtg Drosselweg (Sindelfingen Sindelfingen)
	SINDELF__MAHDENTALSTRASSE_2 = 'de:08115:2225:0:4'  # Rtg Messehalle (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_HST__B = 'de:08115:2226:0:3'  # eRtg Mercedes-Benz Hst. I (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_HST__B = 'de:08115:2226:0:3'  # eRtg Mercedes-Benz Hst. I (Sindelfingen Sindelfingen)
	STADTHALLE_1 = 'de:08115:2227:0:3'  # Rtg Marktplatz (Rath.) (Sindelfingen Sindelfingen)
	STADTHALLE_2 = 'de:08115:2227:0:4'  # Rtg Königsknoll (Sindelfingen Sindelfingen)
	STADTHALLE_3 = 'de:08118:2627:0:3'  # Rtg Hauffstr. (Korntal-Münchingen Korntal)
	STADTHALLE_4 = 'de:08116:4266:1:3'  # eRtg Krankenhaus (Kirchheim unter Teck Kirchheim (T))
	STADTHALLE_5 = 'de:08116:4266:1:4'  # eRtg Bahnhof/ZOB (Kirchheim unter Teck Kirchheim (T))
	STADTHALLE_6 = 'de:08119:5294:1:3'  # eRtg Alle Richtungen (Murrhardt Murrhardt)
	STADTHALLE_7 = 'de:08118:5733:0:3'  # Rtg Neue Bahnhofstr. u. Gym (Vaihingen an der Enz Vaihingen (E))
	STADTHALLE_8 = 'de:08118:5733:0:4'  # Linie 578 (Vaihingen an der Enz Vaihingen (E))
	STADTHALLE_9 = 'de:08118:5733:0:5'  # Rtg Grabenstr. (Vaihingen an der Enz Vaihingen (E))
	STADTHALLE_10 = 'de:08117:7622:0:2'  # Rtg R1 Bahnhof (Donzdorf Donzdorf)
	STADTHALLE_11 = 'de:08117:7622:0:H1'  # Rtg Schulzentrum / Bahnhof (Donzdorf Donzdorf)
	STADTHALLE_12 = 'de:08117:9007:0:R1'  # Ankunft (Göppingen Göppingen)
	SINDELFINGEN_STADTHALLE = 'de:08115:2227'
	SINDELFINGEN_STADTHALLE_1 = 'de:08115:2227:0:3'  # Rtg Marktplatz (Rath.) (Sindelfingen Sindelfingen)
	SINDELFINGEN_STADTHALLE_2 = 'de:08115:2227:0:4'  # Rtg Königsknoll (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_HST__C = 'de:08115:2228:0:3'  # eRtg Mercedes-Benz Hst. D (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_HST__C = 'de:08115:2228:0:3'  # eRtg Mercedes-Benz Hst. D (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_HST__D = 'de:08115:2229:0:3'  # eRtg Mercedes-Benz Hst. E (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_HST__D = 'de:08115:2229:0:3'  # eRtg Mercedes-Benz Hst. E (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_HST__E = 'de:08115:2230:0:3'  # eRtg Mercedes-Benz Tor III (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_HST__E = 'de:08115:2230:0:3'  # eRtg Mercedes-Benz Tor III (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_HST__F = 'de:08115:2231:0:3'  # eRtg Mercedes-Benz Hst. A (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_HST__F = 'de:08115:2231:0:3'  # eRtg Mercedes-Benz Hst. A (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_HST__I = 'de:08115:2234:0:3'  # eRtg Hst. K (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_HST__I = 'de:08115:2234:0:3'  # eRtg Hst. K (Sindelfingen Sindelfingen)
	KILLESBERG = 'de:08111:2235'
	KILLESBERG_1 = 'de:08111:2235:1:1'  # Stadtbahn - Endhst. (Stuttgart Stuttgart)
	KILLESBERG_2 = 'de:08111:2235:2:1'  # Linie 44 Rtg Kunstakademie (Stuttgart Stuttgart)
	KILLESBERG_3 = 'de:08111:2235:2:2'  # Linien 43/50 Rtg Feuerbacher Weg (Stuttgart Stuttgart)
	KILLESBERG_4 = 'de:08111:2235:2:3'  # Linien 43/50 Ankunft/N2 (Stuttgart Stuttgart)
	OSKAR_SCHLEMMER_STRASSE = 'de:08111:2236'
	OSKAR_SCHLEMMER_STRASSE_1 = 'de:08111:2236:0:3'  # Rtg Killesberg (Stuttgart Stuttgart)
	OSKAR_SCHLEMMER_STRASSE_2 = 'de:08111:2236:0:4'  # Rtg Höhenfreibad (Stuttgart Stuttgart)
	MERCEDES_BENZ_HST__K = 'de:08115:2237:0:3'  # Rtg Hst. C / Tor 1 (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_HST__K = 'de:08115:2237:0:3'  # Rtg Hst. C / Tor 1 (Sindelfingen Sindelfingen)
	GLASPALAST = 'de:08115:2240:0:3'  # alle Richtungen (Sindelfingen Sindelfingen)
	SINDELFINGEN_GLASPALAST = 'de:08115:2240:0:3'  # alle Richtungen (Sindelfingen Sindelfingen)
	KUNSTAKADEMIE = 'de:08111:2241'
	KUNSTAKADEMIE_1 = 'de:08111:2241:0:3'  # Rtg Viergiebelweg (Stuttgart Stuttgart)
	KUNSTAKADEMIE_2 = 'de:08111:2241:0:4'  # Rtg Killesberg (Stuttgart Stuttgart)
	VIERGIEBELWEG = 'de:08111:2242'
	VIERGIEBELWEG_1 = 'de:08111:2242:0:3'  # Rtg Obere Mönchhalde (Stuttgart Stuttgart)
	VIERGIEBELWEG_2 = 'de:08111:2242:0:4'  # Rtg Kunstakademie (Stuttgart Stuttgart)
	OBERE_MÖNCHHALDE = 'de:08111:2243'
	OBERE_MÖNCHHALDE_1 = 'de:08111:2243:0:3'  # Rtg Helfferichstr. (Stuttgart Stuttgart)
	OBERE_MÖNCHHALDE_2 = 'de:08111:2243:0:4'  # Rtg Viergiebelweg (Stuttgart Stuttgart)
	HELFFERICHSTRASSE = 'de:08111:2244'
	HELFFERICHSTRASSE_1 = 'de:08111:2244:0:3'  # Rtg Am Kriegsbergturm (Stuttgart Stuttgart)
	HELFFERICHSTRASSE_2 = 'de:08111:2244:0:4'  # Rtg Obere Mönchhalde (Stuttgart Stuttgart)
	AM_KRIEGSBERGTURM = 'de:08111:2245'
	AM_KRIEGSBERGTURM_1 = 'de:08111:2245:0:3'  # Rtg Im Kaisemer (Stuttgart Stuttgart)
	AM_KRIEGSBERGTURM_2 = 'de:08111:2245:0:4'  # Rtg Helffrichstr. (Stuttgart Stuttgart)
	IM_KAISEMER = 'de:08111:2246'
	IM_KAISEMER_1 = 'de:08111:2246:0:3'  # Rtg Postdörfle (Stuttgart Stuttgart)
	IM_KAISEMER_2 = 'de:08111:2246:0:4'  # Rtg Am Kriegsbergturm (Stuttgart Stuttgart)
	POSTDÖRFLE_1 = 'de:08111:2247:0:3'  # Rtg Stadtbibliothek (Stuttgart Stuttgart)
	POSTDÖRFLE_2 = 'de:08111:2247:0:4'  # Rtg Im Kaisemer (Stuttgart Stuttgart)
	POSTDÖRFLE_3 = 'de:08119:2647:0:3'  # Rtg Esslinger Str. (Fellbach Fellbach)
	POSTDÖRFLE_4 = 'de:08119:2647:0:4'  # Rtg Adlerstr. (Fellbach Fellbach)
	KLEISTSTRASSE = 'de:08111:2248'
	KLEISTSTRASSE_1 = 'de:08111:2248:0:3'  # Rtg Westbahnhof (Stuttgart Stuttgart)
	KLEISTSTRASSE_2 = 'de:08111:2248:0:4'  # Rtg Leipziger Platz (Stuttgart Stuttgart)
	AM_RÖMERKASTELL = 'de:08111:2250'
	AM_RÖMERKASTELL_1 = 'de:08111:2250:0:1'  # Rtg Nastplatz (Stuttgart Stuttgart)
	AM_RÖMERKASTELL_2 = 'de:08111:2250:0:2'  # Rtg Hallschlag (Stuttgart Stuttgart)
	AM_RÖMERKASTELL_3 = 'de:08111:2250:0:3'  # Rtg Altenburg (Stuttgart Stuttgart)
	ALTENBURG = 'de:08111:2251'
	ALTENBURG_1 = 'de:08111:2251:0:3'  # Rtg Martinskirche (Stuttgart Stuttgart)
	ALTENBURG_2 = 'de:08111:2251:0:4'  # Rtg Am Römerkastell (Stuttgart Stuttgart)
	MARTINSKIRCHE_1 = 'de:08111:2252:0:3'  # Rtg Rosensteinbrücke (Stuttgart Stuttgart)
	MARTINSKIRCHE_2 = 'de:08116:4317:1:3'  # eRtg Teckstraße/Austraße/Ziegelwasen (Kirchheim unter Teck Kirchheim (T))
	NASTPLATZ = 'de:08111:2258'
	NASTPLATZ_1 = 'de:08111:2258:0:3'  # Rtg Am Römerkastell (Stuttgart Stuttgart)
	NASTPLATZ_2 = 'de:08111:2258:0:4'  # Rtg Dessauer Str. (Stuttgart Stuttgart)
	DESSAUER_STRASSE = 'de:08111:2259'
	DESSAUER_STRASSE_1 = 'de:08111:2259:0:3'  # Rtg Nastplatz (Stuttgart Stuttgart)
	DESSAUER_STRASSE_2 = 'de:08111:2259:0:4'  # Rtg Bottroper Str. (Stuttgart Stuttgart)
	BOTTROPER_STRASSE = 'de:08111:2260'
	BOTTROPER_STRASSE_1 = 'de:08111:2260:1:1'  # Rtg Hallschlag (Stuttgart Stuttgart)
	BOTTROPER_STRASSE_2 = 'de:08111:2260:1:2'  # Rtg Wagrainäcker (Stuttgart Stuttgart)
	BOTTROPER_STRASSE_3 = 'de:08111:2260:2:3'  # Rtg Dessauer Str. (Stuttgart Stuttgart)
	BOTTROPER_STRASSE_4 = 'de:08111:2260:2:4'  # Rtg Münster Bf (Stuttgart Stuttgart)
	BISMARCKSTAFFEL = 'de:08111:2261'
	BISMARCKSTAFFEL_1 = 'de:08111:2261:0:3'  # Rtg Westbahnhof (Stuttgart Stuttgart)
	BISMARCKSTAFFEL_2 = 'de:08111:2261:0:4'  # Rtg Rotebühl-/Reinsburgstr. (Stuttgart Stuttgart)
	ROTEBÜHL__REINSBURGSTRASSE = 'de:08111:2262'
	ROTEBÜHL__REINSBURGSTRASSE_1 = 'de:08111:2262:0:3'  # Rtg Bismarckstaffel (Stuttgart Stuttgart)
	ROTEBÜHL__REINSBURGSTRASSE_2 = 'de:08111:2262:0:4'  # Rtg Rotebühl-/Reinsburgstr. (Stuttgart Stuttgart)
	SEYFFERSTRASSE = 'de:08111:2263'
	SEYFFERSTRASSE_1 = 'de:08111:2263:0:3'  # Rtg Leipziger Platz (Stuttgart Stuttgart)
	SEYFFERSTRASSE_2 = 'de:08111:2263:0:4'  # Rtg Schwabstr. (Stuttgart Stuttgart)
	SILBERBURG__REINSBURGSTRASSE = 'de:08111:2264'
	SILBERBURG__REINSBURGSTRASSE_1 = 'de:08111:2264:0:3'  # Rtg Schwab-/Reinsburgstr. (Stuttgart Stuttgart)
	SILBERBURG__REINSBURGSTRASSE_2 = 'de:08111:2264:0:4'  # Rtg Feuersee (Stuttgart Stuttgart)
	HERBSTHALDE = 'de:08111:2265'
	HERBSTHALDE_1 = 'de:08111:2265:0:3'  # Rtg Herderplatz (Stuttgart Stuttgart)
	HERBSTHALDE_2 = 'de:08111:2265:0:4'  # Rtg Wielandstr. (Stuttgart Stuttgart)
	WEILIMDORF_BF = 'de:08111:2270'
	WEILIMDORF_BF_1 = 'de:08111:2270:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	WEILIMDORF_BF_2 = 'de:08111:2270:2:1'  # Gleis 1 (Stuttgart Stuttgart)
	WEILIMDORF_BF_3 = 'de:08111:2270:3:3'  # Rtg Ditzinger Str./Bf (Unterf.) (Stuttgart Stuttgart)
	SINDELF__MERCEDES_BENZ_HST__N = 'de:08115:2281:0:3'  # Rtg Hst. F (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_HST__N = 'de:08115:2281:0:3'  # Rtg Hst. F (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_HST__O = 'de:08115:2282:0:3'  # Rtg TOR III (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_HST__O = 'de:08115:2282:0:3'  # Rtg TOR III (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_HST__P = 'de:08115:2283:0:1'  # Rtg ??? (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_HST__P = 'de:08115:2283:0:1'  # Rtg ??? (Sindelfingen Sindelfingen)
	FEUERWEHR_1 = 'de:08118:2299:0:1'  # Rtg Einkaufszentrum (Gerlingen Gerlingen)
	FEUERWEHR_2 = 'de:08118:2299:0:2'  # Rtg Goethestraße (Gerlingen Gerlingen)
	FEUERWEHR_3 = 'de:08116:3817:0:3'  # eRtg Rathaus (Altbach Altbach)
	FEUERWEHR_4 = 'de:08116:3817:0:4'  # eRtg Badstraße (Altbach Altbach)
	FEUERWEHR_5 = 'de:08119:5336:0:3'  # eRtg Gartenstraße (Backnang Backnang)
	FEUERWEHR_6 = 'de:08119:5336:0:4'  # eRtg Biegel (Backnang Backnang)
	FEUERWEHR_7 = 'de:08119:6696:0:3'  # Rtg Burgstr./Schloss (Urbach Urbach)
	FEUERWEHR_8 = 'de:08119:6696:0:4'  # Rtg Hölderlinweg (Urbach Urbach)
	FEUERWEHR_9 = 'de:08119:7096:0:3'  # Rtg Pfarrgartenstr. (Kirchberg an der Murr Kirchberg (M))
	FEUERWEHR_10 = 'de:08117:4016:0:H1'  # Rtg Schlossmarkt (Rechberghausen Rechberghausen)
	FEUERWEHR_11 = 'de:08117:4016:0:R1'  # Rtg Schurwaldsch. (Rechberghausen Rechberghausen)
	GERLINGEN_FEUERWEHR = 'de:08118:2299'
	GERLINGEN_FEUERWEHR_1 = 'de:08118:2299:0:1'  # Rtg Einkaufszentrum (Gerlingen Gerlingen)
	GERLINGEN_FEUERWEHR_2 = 'de:08118:2299:0:2'  # Rtg Goethestraße (Gerlingen Gerlingen)
	ALTE_STEIGE_1 = 'de:08118:2301:0:1'  # Rtg Hirschbergstraße (Gerlingen Gerlingen)
	ALTE_STEIGE_2 = 'de:08118:2301:0:2'  # Rtg Richthofenstraße (Gerlingen Gerlingen)
	ALTE_STEIGE_3 = 'de:08116:4104:0:3'  # eRtg Neue Straße (Esslingen am Neckar Liebersbronn)
	ALTE_STEIGE_4 = 'de:08116:4104:0:4'  # eRtg Jägerhaus (Esslingen am Neckar Liebersbronn)
	ALTE_STEIGE_5 = 'de:08119:6991:0:3'  # eRtg Alle Richtungen (Schorndorf Schorndorf)
	GERLINGEN_ALTE_STEIGE = 'de:08118:2301'
	GERLINGEN_ALTE_STEIGE_1 = 'de:08118:2301:0:1'  # Rtg Hirschbergstraße (Gerlingen Gerlingen)
	GERLINGEN_ALTE_STEIGE_2 = 'de:08118:2301:0:2'  # Rtg Richthofenstraße (Gerlingen Gerlingen)
	BOPSERWALDSTRASSE = 'de:08118:2302:1:3'  # Rtg Ramtel (Gerlingen Gerlingen)
	GERLINGEN_BOPSERWALDSTR = 'de:08118:2302:1:3'  # Rtg Ramtel (Gerlingen Gerlingen)
	GERLINGEN_BURGKLINGE = 'de:08118:2303:0:4'  # Rtg Alte Steige (Gerlingen Gerlingen)
	BURGKLINGE = 'de:08118:2303:0:4'  # Rtg Alte Steige (Gerlingen Gerlingen)
	FÜLLERSTRASSE = 'de:08118:2304:0:3'  # Rtg Forchenrainstr. (Gerlingen Gerlingen)
	GERLINGEN_FÜLLERSTRASSE = 'de:08118:2304:0:3'  # Rtg Forchenrainstr. (Gerlingen Gerlingen)
	GERLINGEN_GLEMSTAL = 'de:08118:2305'
	GERLINGEN_GLEMSTAL_1 = 'de:08118:2305:0:3'  # Rtg Bruderhaus (Gerlingen Gerlingen)
	GERLINGEN_GLEMSTAL_2 = 'de:08118:2305:0:4'  # Rtg Waldfreibad (Gerlingen Gerlingen)
	GLEMSTAL_1 = 'de:08118:2305:0:3'  # Rtg Bruderhaus (Gerlingen Gerlingen)
	GLEMSTAL_2 = 'de:08118:2305:0:4'  # Rtg Waldfreibad (Gerlingen Gerlingen)
	GLEMSTAL_3 = 'de:08118:3322:1:3'  # eRtg Hemmingen (Schwieberdingen Schwieberdingen)
	GLEMSTAL_4 = 'de:08118:3322:1:4'  # eRtg Mitte (Schwieberdingen Schwieberdingen)
	HIRSCHBERGSTRASSE_1 = 'de:08118:2306:0:3'  # Rtg Schillerhöhe Post (Gerlingen Gerlingen)
	HIRSCHBERGSTRASSE_2 = 'de:08118:2306:0:4'  # Rtg Burgklinge (Gerlingen Gerlingen)
	HIRSCHBERGSTRASSE_3 = 'de:08118:3456:0:3'  # eRtg Tammer Straße (Ludwigsburg Eglosheim)
	HIRSCHBERGSTRASSE_4 = 'de:08118:3456:0:4'  # Rtg Asperg Hohenstaufenstr. (Ludwigsburg Eglosheim)
	GERLINGEN_HIRSCHBERGSTRASSE = 'de:08118:2306'
	GERLINGEN_HIRSCHBERGSTRASSE_1 = 'de:08118:2306:0:3'  # Rtg Schillerhöhe Post (Gerlingen Gerlingen)
	GERLINGEN_HIRSCHBERGSTRASSE_2 = 'de:08118:2306:0:4'  # Rtg Burgklinge (Gerlingen Gerlingen)
	GERLINGEN_HOFWIESENSTRASSE = 'de:08118:2307'
	GERLINGEN_HOFWIESENSTRASSE_1 = 'de:08118:2307:0:3'  # Rtg Rathaus (Gerlingen Gerlingen)
	GERLINGEN_HOFWIESENSTRASSE_2 = 'de:08118:2307:0:4'  # Rtg Ditz. Gewerbegebiet Süd (Gerlingen Gerlingen)
	GERLINGEN_HOFWIESENSTRASSE_3 = 'de:08118:2307:0:5'  # 635 Rtg Goethestr (Gerlingen Gerlingen)
	HOFWIESENSTRASSE = 'de:08118:2307'
	HOFWIESENSTRASSE_1 = 'de:08118:2307:0:3'  # Rtg Rathaus (Gerlingen Gerlingen)
	HOFWIESENSTRASSE_2 = 'de:08118:2307:0:4'  # Rtg Ditz. Gewerbegebiet Süd (Gerlingen Gerlingen)
	HOFWIESENSTRASSE_3 = 'de:08118:2307:0:5'  # 635 Rtg Goethestr (Gerlingen Gerlingen)
	RAMTEL = 'de:08118:2308'
	RAMTEL_1 = 'de:08118:2308:1:3'  # Rtg Leonberg Golfplatz/Füllerstr. (Gerlingen Gerlingen)
	RAMTEL_2 = 'de:08118:2308:1:4'  # Rtg Waldsiedlung (Gerlingen Gerlingen)
	GERLINGEN_RAMTEL = 'de:08118:2308'
	GERLINGEN_RAMTEL_1 = 'de:08118:2308:1:3'  # Rtg Leonberg Golfplatz/Füllerstr. (Gerlingen Gerlingen)
	GERLINGEN_RAMTEL_2 = 'de:08118:2308:1:4'  # Rtg Waldsiedlung (Gerlingen Gerlingen)
	LEONBERG_SCHLEIERMACHERSTR = 'de:08115:2309:0:3'  # Rtg Samariterstift (Leonberg Leonberg)
	SCHLEIERMACHERSTRASSE = 'de:08115:2309:0:3'  # Rtg Samariterstift (Leonberg Leonberg)
	PLANCKSTRASSE = 'de:08111:2310'
	PLANCKSTRASSE_1 = 'de:08111:2310:0:3'  # Rtg Gablenberg (Stuttgart Stuttgart)
	PLANCKSTRASSE_2 = 'de:08111:2310:0:4'  # Rtg Straußstaffel (Stuttgart Stuttgart)
	PLANCKSTRASSE_3 = 'de:08111:2310:0:5'  # Nachtbus (Stuttgart Stuttgart)
	STRAUSSSTAFFEL = 'de:08111:2311'
	STRAUSSSTAFFEL_1 = 'de:08111:2311:0:3'  # Rtg Planckstr. (Stuttgart Stuttgart)
	STRAUSSSTAFFEL_2 = 'de:08111:2311:0:4'  # Rtg Heidehofstr. (Stuttgart Stuttgart)
	KLINIK_SCHILLERHÖHE = 'de:08118:2312'
	KLINIK_SCHILLERHÖHE_1 = 'de:08118:2312:0:3'  # Rtg Schillerhöhe Post (Gerlingen Gerlingen)
	KLINIK_SCHILLERHÖHE_2 = 'de:08118:2312:0:4'  # Rtg Solitude (Gerlingen Gerlingen)
	GERL__KLINIK_SCHILLERHÖHE = 'de:08118:2312'
	GERL__KLINIK_SCHILLERHÖHE_1 = 'de:08118:2312:0:3'  # Rtg Schillerhöhe Post (Gerlingen Gerlingen)
	GERL__KLINIK_SCHILLERHÖHE_2 = 'de:08118:2312:0:4'  # Rtg Solitude (Gerlingen Gerlingen)
	GERL__SCHILLERHÖHE_BOSCH = 'de:08118:2313'
	GERL__SCHILLERHÖHE_BOSCH_1 = 'de:08118:2313:0:3'  # Rtg Waldsiedlung (Gerlingen Gerlingen)
	GERL__SCHILLERHÖHE_BOSCH_2 = 'de:08118:2313:0:4'  # Rtg Schillerhöhe Post/R.-Bosch-Pl (Gerlingen Gerlingen)
	SCHILLERHÖHE_BOSCH = 'de:08118:2313'
	SCHILLERHÖHE_BOSCH_1 = 'de:08118:2313:0:3'  # Rtg Waldsiedlung (Gerlingen Gerlingen)
	SCHILLERHÖHE_BOSCH_2 = 'de:08118:2313:0:4'  # Rtg Schillerhöhe Post/R.-Bosch-Pl (Gerlingen Gerlingen)
	GERLINGER_TOR = 'de:08118:2314'
	GERLINGER_TOR_1 = 'de:08118:2314:0:3'  # Rtg Schillerhöhe Bosch (Gerlingen Gerlingen)
	GERLINGER_TOR_2 = 'de:08118:2314:0:4'  # Rtg Klinik Schillerhöhe (Gerlingen Gerlingen)
	GERLINGER_TOR_3 = 'de:08118:2314:0:5'  # Rtg R.-Bosch-Platz (Gerlingen Gerlingen)
	GERLINGER_TOR_4 = 'de:08118:2314:0:6'  # Rtg Hirschbergstr. (Gerlingen Gerlingen)
	GERLINGEN_GERLINGER_TOR = 'de:08118:2314'
	GERLINGEN_GERLINGER_TOR_1 = 'de:08118:2314:0:3'  # Rtg Schillerhöhe Bosch (Gerlingen Gerlingen)
	GERLINGEN_GERLINGER_TOR_2 = 'de:08118:2314:0:4'  # Rtg Klinik Schillerhöhe (Gerlingen Gerlingen)
	GERLINGEN_GERLINGER_TOR_3 = 'de:08118:2314:0:5'  # Rtg R.-Bosch-Platz (Gerlingen Gerlingen)
	GERLINGEN_GERLINGER_TOR_4 = 'de:08118:2314:0:6'  # Rtg Hirschbergstr. (Gerlingen Gerlingen)
	SOLITUDE = 'de:08111:2316'
	SOLITUDE_1 = 'de:08111:2316:0:3'  # Rtg Gerl. Klinik Schillerhöhe (Stuttgart Stuttgart)
	SOLITUDE_2 = 'de:08111:2316:0:4'  # Rtg Forsthaus II (Stuttgart Stuttgart)
	GERLINGEN_KRUMMBACHTAL = 'de:08118:2317'
	GERLINGEN_KRUMMBACHTAL_1 = 'de:08118:2317:0:3'  # Rtg Glemstal (Gerlingen Gerlingen)
	GERLINGEN_KRUMMBACHTAL_2 = 'de:08118:2317:0:4'  # Rtg Leonberg Glemseck (Gerlingen Gerlingen)
	KRUMMBACHTAL = 'de:08118:2317'
	KRUMMBACHTAL_1 = 'de:08118:2317:0:3'  # Rtg Glemstal (Gerlingen Gerlingen)
	KRUMMBACHTAL_2 = 'de:08118:2317:0:4'  # Rtg Leonberg Glemseck (Gerlingen Gerlingen)
	WALDSIEDLUNG = 'de:08118:2318'
	WALDSIEDLUNG_1 = 'de:08118:2318:0:3'  # Rtg Bopserwaldstr. (Gerlingen Gerlingen)
	WALDSIEDLUNG_2 = 'de:08118:2318:0:4'  # Rtg Schillerhöhe Bosch (Gerlingen Gerlingen)
	GERLINGEN_WALDSIEDLUNG = 'de:08118:2318'
	GERLINGEN_WALDSIEDLUNG_1 = 'de:08118:2318:0:3'  # Rtg Bopserwaldstr. (Gerlingen Gerlingen)
	GERLINGEN_WALDSIEDLUNG_2 = 'de:08118:2318:0:4'  # Rtg Schillerhöhe Bosch (Gerlingen Gerlingen)
	LEONBERG_SAMARITERSTIFT = 'de:08115:2319'
	LEONBERG_SAMARITERSTIFT_1 = 'de:08115:2319:0:3'  # Rtg Seegarten (Leonberg Leonberg)
	LEONBERG_SAMARITERSTIFT_2 = 'de:08115:2319:0:4'  # Rtg Blosenbergkirche (Leonberg Leonberg)
	SAMARITERSTIFT = 'de:08115:2319'
	SAMARITERSTIFT_1 = 'de:08115:2319:0:3'  # Rtg Seegarten (Leonberg Leonberg)
	SAMARITERSTIFT_2 = 'de:08115:2319:0:4'  # Rtg Blosenbergkirche (Leonberg Leonberg)
	BILDSTÖCKLE = 'de:08115:2320'
	BILDSTÖCKLE_1 = 'de:08115:2320:0:3'  # Rtg Altstadt (Leonberg Leonberg)
	BILDSTÖCKLE_2 = 'de:08115:2320:0:4'  # Rtg Engelberg (Leonberg Leonberg)
	LEONBERG_BILDSTÖCKLE = 'de:08115:2320'
	LEONBERG_BILDSTÖCKLE_1 = 'de:08115:2320:0:3'  # Rtg Altstadt (Leonberg Leonberg)
	LEONBERG_BILDSTÖCKLE_2 = 'de:08115:2320:0:4'  # Rtg Engelberg (Leonberg Leonberg)
	ENGELBERG_1 = 'de:08115:2321:0:3'  # Rtg Bildstöckle (Leonberg Leonberg)
	ENGELBERG_2 = 'de:08115:2321:0:4'  # Rtg Golfplatz (Leonberg Leonberg)
	ENGELBERG_3 = 'de:08119:4082:0:3'  # Rtg. Engelberg/Goldboden (Winterbach Winterbach)
	ENGELBERG_4 = 'de:08119:4082:0:4'  # Rtg Bahnunterführung (Winterbach Winterbach)
	ENGELBERG_5 = 'de:08119:4082:0:5'  # Rtg Manolzw./Goldboden (Winterbach Winterbach)
	LEONBERG_ENGELBERG = 'de:08115:2321'
	LEONBERG_ENGELBERG_1 = 'de:08115:2321:0:3'  # Rtg Bildstöckle (Leonberg Leonberg)
	LEONBERG_ENGELBERG_2 = 'de:08115:2321:0:4'  # Rtg Golfplatz (Leonberg Leonberg)
	GARTENSTADT_1 = 'de:08115:2322:0:3'  # Rtg Rutesheim Gebersheim (Leonberg Leonberg)
	GARTENSTADT_2 = 'de:08115:2322:0:4'  # Rtg Krankenhaus (Leonberg Leonberg)
	GARTENSTADT_3 = 'de:08116:4024:0:3'  # Rtg Zell (Esslingen am Neckar Oberesslingen)
	GARTENSTADT_4 = 'de:08116:4024:0:4'  # Rtg Oberesslingen (Esslingen am Neckar Oberesslingen)
	LEONBERG_GARTENSTADT = 'de:08115:2322'
	LEONBERG_GARTENSTADT_1 = 'de:08115:2322:0:3'  # Rtg Rutesheim Gebersheim (Leonberg Leonberg)
	LEONBERG_GARTENSTADT_2 = 'de:08115:2322:0:4'  # Rtg Krankenhaus (Leonberg Leonberg)
	GLEMSECK = 'de:08115:2324'
	GLEMSECK_1 = 'de:08115:2324:0:3'  # Rtg Gerlingen Waldfreibad (Leonberg Leonberg)
	GLEMSECK_2 = 'de:08115:2324:0:4'  # Rtg Rappenhof (Leonberg Leonberg)
	LEONBERG_GLEMSECK = 'de:08115:2324'
	LEONBERG_GLEMSECK_1 = 'de:08115:2324:0:3'  # Rtg Gerlingen Waldfreibad (Leonberg Leonberg)
	LEONBERG_GLEMSECK_2 = 'de:08115:2324:0:4'  # Rtg Rappenhof (Leonberg Leonberg)
	GOLFPLATZ_1 = 'de:08115:2325:0:2'  # Rtg Engelberg (Leonberg Leonberg)
	GOLFPLATZ_2 = 'de:08115:2325:0:3'  # L98 Endhaltestelle/Rtg Gerlingen Ramtel (Leonberg Leonberg)
	GOLFPLATZ_3 = 'de:08115:2325:0:4'  # Rtg Gerlingen Ramtel (Leonberg Leonberg)
	GOLFPLATZ_4 = 'de:08118:5508:0:3'  # Rtg Ikea (Ludwigsburg Eglosheim)
	GOLFPLATZ_5 = 'de:08118:5508:0:4'  # Rtg Ludwigsburg (Ludwigsburg Eglosheim)
	LEONBERG_GOLFPLATZ = 'de:08115:2325'
	LEONBERG_GOLFPLATZ_1 = 'de:08115:2325:0:2'  # Rtg Engelberg (Leonberg Leonberg)
	LEONBERG_GOLFPLATZ_2 = 'de:08115:2325:0:3'  # L98 Endhaltestelle/Rtg Gerlingen Ramtel (Leonberg Leonberg)
	LEONBERG_GOLFPLATZ_3 = 'de:08115:2325:0:4'  # Rtg Gerlingen Ramtel (Leonberg Leonberg)
	HIRSCHBERGER_STRASSE = 'de:08115:2326'
	HIRSCHBERGER_STRASSE_1 = 'de:08115:2326:0:3'  # Rtg Göppinger Str. (Leonberg Ramtel)
	HIRSCHBERGER_STRASSE_2 = 'de:08115:2326:0:4'  # Elt. Tiroler Str. (Leonberg Ramtel)
	RAMTEL_HIRSCHBERGER_STR = 'de:08115:2326'
	RAMTEL_HIRSCHBERGER_STR_1 = 'de:08115:2326:0:3'  # Rtg Göppinger Str. (Leonberg Ramtel)
	RAMTEL_HIRSCHBERGER_STR_2 = 'de:08115:2326:0:4'  # Elt. Tiroler Str. (Leonberg Ramtel)
	LEONBERG_KRANKENHAUS = 'de:08115:2327'
	LEONBERG_KRANKENHAUS_1 = 'de:08115:2327:0:3'  # Rtg Bahnhof (Leonberg Leonberg)
	LEONBERG_KRANKENHAUS_2 = 'de:08115:2327:0:4'  # Rtg Gartenstadt (Leonberg Leonberg)
	KRANKENHAUS_1 = 'de:08115:2327:0:3'  # Rtg Bahnhof (Leonberg Leonberg)
	KRANKENHAUS_2 = 'de:08115:2327:0:4'  # Rtg Gartenstadt (Leonberg Leonberg)
	KRANKENHAUS_3 = 'de:08118:3533:1:3'  # Rtg Marbach (Marbach am Neckar Marbach (N))
	KRANKENHAUS_4 = 'de:08118:3533:1:4'  # Rtg Blattert/Kelter (Marbach am Neckar Marbach (N))
	KRANKENHAUS_5 = 'de:08115:4666:0:3'  # eRtg Altenheim Burghalde (Sindelfingen Sindelfingen)
	KRANKENHAUS_6 = 'de:08118:5656:0:3'  # eRtg Bolzstraße (Bietigheim-Bissingen Bietigheim)
	KRANKENHAUS_7 = 'de:08118:5656:0:4'  # eRtg DLW/Auwiesenbrücke (Bietigheim-Bissingen Bietigheim)
	KRANKENHAUS_8 = 'de:08118:5873:0:3'  # eRtg Steinbeisstraße (Vaihingen an der Enz Vaihingen (E))
	KRANKENHAUS_9 = 'de:08118:5873:0:4'  # eRtg Stadthalle (Vaihingen an der Enz Vaihingen (E))
	KRANKENHAUS_10 = 'de:08117:212:0:H1'  # Rtg Vossstr. (Geislingen an der Steige Geislingen (Steige))
	KRANKENHAUS_11 = 'de:08117:212:0:R1'  # Rtg Geisl.(St) ZOB (Geislingen an der Steige Geislingen (Steige))
	LEO_CENTER = 'de:08115:2328'
	LEO_CENTER_1 = 'de:08115:2328:0:3'  # Rtg Süd/Brennerstr./Mörike-Schule (Leonberg Eltingen)
	LEO_CENTER_2 = 'de:08115:2328:0:4'  # Rtg Römerstr./Belforter Pl. (Leonberg Eltingen)
	LEO_CENTER_3 = 'de:08115:2328:0:5'  # Rtg Bf (Linie 651) (Leonberg Eltingen)
	ELTINGEN_LEO_CENTER = 'de:08115:2328'
	ELTINGEN_LEO_CENTER_1 = 'de:08115:2328:0:3'  # Rtg Süd/Brennerstr./Mörike-Schule (Leonberg Eltingen)
	ELTINGEN_LEO_CENTER_2 = 'de:08115:2328:0:4'  # Rtg Römerstr./Belforter Pl. (Leonberg Eltingen)
	ELTINGEN_LEO_CENTER_3 = 'de:08115:2328:0:5'  # Rtg Bf (Linie 651) (Leonberg Eltingen)
	LEONB__JOH__KEPLER_GYMNASIUM = 'de:08115:2329'
	LEONB__JOH__KEPLER_GYMNASIUM_1 = 'de:08115:2329:0:3'  # Rtg Bahnhof (Leonberg Leonberg)
	LEONB__JOH__KEPLER_GYMNASIUM_2 = 'de:08115:2329:0:4'  # Rtg Törlensweg/Belforter Pl. (Leonberg Leonberg)
	JOHANNES_KEPLER_GYMNASIUM = 'de:08115:2329'
	JOHANNES_KEPLER_GYMNASIUM_1 = 'de:08115:2329:0:3'  # Rtg Bahnhof (Leonberg Leonberg)
	JOHANNES_KEPLER_GYMNASIUM_2 = 'de:08115:2329:0:4'  # Rtg Törlensweg/Belforter Pl. (Leonberg Leonberg)
	MERCEDES_BENZ_WELT = 'de:08111:2331:1:1'  # Rtg Fritz-Walter-Weg (Stuttgart Stuttgart)
	LEONBERG_RÖMERSTR = 'de:08115:2333'
	LEONBERG_RÖMERSTR_1 = 'de:08115:2333:0:3'  # Rtg Elt. Leo-Center (Leonberg Leonberg)
	LEONBERG_RÖMERSTR_2 = 'de:08115:2333:0:4'  # Rtg Bahnhof (Leonberg Leonberg)
	RÖMERSTRASSE_1 = 'de:08115:2333:0:3'  # Rtg Elt. Leo-Center (Leonberg Leonberg)
	RÖMERSTRASSE_2 = 'de:08115:2333:0:4'  # Rtg Bahnhof (Leonberg Leonberg)
	RÖMERSTRASSE_3 = 'de:08116:4264:1:3'  # Rtg Volksbank (Dettingen unter Teck Dettingen (T))
	RAPPENHOF = 'de:08115:2334'
	RAPPENHOF_1 = 'de:08115:2334:0:3'  # Rtg Glemseck (Leonberg Leonberg)
	RAPPENHOF_2 = 'de:08115:2334:0:4'  # Rtg Schumisberg (Leonberg Leonberg)
	LEONBERG_RAPPENHOF = 'de:08115:2334'
	LEONBERG_RAPPENHOF_1 = 'de:08115:2334:0:3'  # Rtg Glemseck (Leonberg Leonberg)
	LEONBERG_RAPPENHOF_2 = 'de:08115:2334:0:4'  # Rtg Schumisberg (Leonberg Leonberg)
	SCHUMISBERG = 'de:08115:2335'
	SCHUMISBERG_1 = 'de:08115:2335:0:3'  # Rtg Rappenhof (Leonberg Leonberg)
	SCHUMISBERG_2 = 'de:08115:2335:0:4'  # Rtg Elt. Leonberger Dreieck (Leonberg Leonberg)
	LEONBERG_SCHUMISBERG = 'de:08115:2335'
	LEONBERG_SCHUMISBERG_1 = 'de:08115:2335:0:3'  # Rtg Rappenhof (Leonberg Leonberg)
	LEONBERG_SCHUMISBERG_2 = 'de:08115:2335:0:4'  # Rtg Elt. Leonberger Dreieck (Leonberg Leonberg)
	WALDFRIEDHOF_1 = 'de:08115:2336:0:3'  # Bus-Li94 Ankunft (Leonberg Leonberg)
	WALDFRIEDHOF_2 = 'de:08115:2336:0:4'  # Rtg Ramtel Neue Ramtelstr. (Leonberg Leonberg)
	WALDFRIEDHOF_3 = 'de:08115:3211:0:3'  # eRtg Landhaussiedlung (Sindelfingen Maichingen)
	WALDFRIEDHOF_4 = 'de:08115:3211:0:4'  # eRtg Stuttgarter Straße/Eichholz (Sindelfingen Maichingen)
	WALDFRIEDHOF_5 = 'de:08115:3211:0:5'  # Einbahnstr (Sindelfingen Maichingen)
	WALDFRIEDHOF_6 = 'de:08116:4355:0:3'  # eRtg Wellinger Straße (Kirchheim unter Teck Kirchheim (T))
	WALDFRIEDHOF_7 = 'de:08116:4449:0:3'  # eRtg Schelmenwasen (Nürtingen Nürtingen)
	WALDFRIEDHOF_8 = 'de:08116:4449:0:4'  # eRtg Hermann-Löns-Straße (Nürtingen Nürtingen)
	WALDFRIEDHOF_9 = 'de:08115:4715:0:3'  # eRtg Ochsentrog (Böblingen Böblingen)
	WALDFRIEDHOF_10 = 'de:08115:4715:0:4'  # eRtg Bühler Straße (Böblingen Böblingen)
	WALDFRIEDHOF_11 = 'de:08115:4800:0:3'  # eRtg Grafenweg (Herrenberg Herrenberg)
	WALDFRIEDHOF_12 = 'de:08119:5322:0:3'  # eRtg Eichwald (Backnang Backnang)
	WALDFRIEDHOF_13 = 'de:08119:5322:0:4'  # eRtg Seehof (Backnang Backnang)
	WALDFRIEDHOF_14 = 'de:08111:6102:0:1'  # Waldfriedhof (Stuttgart Stuttgart)
	WALDFRIEDHOF_15 = 'de:08118:6907:0:3'  # eRtg Robert-Bosch-Platz (Gerlingen Gerlingen)
	WALDFRIEDHOF_16 = 'de:08116:6923:0:1'  # Rtg Tischardt (Kohlberg Kohlberg)
	WALDFRIEDHOF_17 = 'de:08116:6923:0:2'  # Rtg Kohlberg (Kohlberg Kohlberg)
	LEONBERG_WALDFRIEDHOF = 'de:08115:2336'
	LEONBERG_WALDFRIEDHOF_1 = 'de:08115:2336:0:3'  # Bus-Li94 Ankunft (Leonberg Leonberg)
	LEONBERG_WALDFRIEDHOF_2 = 'de:08115:2336:0:4'  # Rtg Ramtel Neue Ramtelstr. (Leonberg Leonberg)
	RAMTEL_GERLINGER_STR = 'de:08115:2338'
	RAMTEL_GERLINGER_STR_1 = 'de:08115:2338:0:3'  # Rtg Neue Ramtelstr. (Leonberg Ramtel)
	RAMTEL_GERLINGER_STR_2 = 'de:08115:2338:0:4'  # Rtg Göppinger Str. (Leonberg Ramtel)
	RAMTEL_GERLINGER_STR_3 = 'de:08115:2338:0:5'  # Rtg Aug.-Lämmle-Schule (Leonberg Ramtel)
	GERLINGER_STRASSE_1 = 'de:08115:2338:0:3'  # Rtg Neue Ramtelstr. (Leonberg Ramtel)
	GERLINGER_STRASSE_2 = 'de:08115:2338:0:4'  # Rtg Göppinger Str. (Leonberg Ramtel)
	GERLINGER_STRASSE_3 = 'de:08115:2338:0:5'  # Rtg Aug.-Lämmle-Schule (Leonberg Ramtel)
	GERLINGER_STRASSE_4 = 'de:08111:2620:0:3'  # Rtg Rappachstr. (Stuttgart Stuttgart)
	GERLINGER_STRASSE_5 = 'de:08111:2620:0:4'  # Rtg Hausenring (Stuttgart Stuttgart)
	GERLINGER_STRASSE_6 = 'de:08111:2620:0:5'  # Nachbus (Stuttgart Stuttgart)
	RAMTEL_GÖPPINGER_STR = 'de:08115:2339'
	RAMTEL_GÖPPINGER_STR_1 = 'de:08115:2339:0:3'  # Elt. Leonb. Dreieck/Gerlinger Str. (Leonberg Ramtel)
	RAMTEL_GÖPPINGER_STR_2 = 'de:08115:2339:0:4'  # Rtg Hirschberger Str. (Leonberg Ramtel)
	GÖPPINGER_STRASSE_1 = 'de:08115:2339:0:3'  # Elt. Leonb. Dreieck/Gerlinger Str. (Leonberg Ramtel)
	GÖPPINGER_STRASSE_2 = 'de:08115:2339:0:4'  # Rtg Hirschberger Str. (Leonberg Ramtel)
	GÖPPINGER_STRASSE_3 = 'de:08117:2117:0:H1'  # Rtg Schillerstr. (Zell unter Aichelberg Zell (A))
	GÖPPINGER_STRASSE_4 = 'de:08117:2117:0:R1'  # Rtg Bad Boll (Zell unter Aichelberg Zell (A))
	ELTINGEN_GLEMSECKSTRASSE = 'de:08115:2340'
	ELTINGEN_GLEMSECKSTRASSE_1 = 'de:08115:2340:0:3'  # Rtg Tiroler Str./Leonb. Dreieck (Leonberg Eltingen)
	ELTINGEN_GLEMSECKSTRASSE_2 = 'de:08115:2340:0:4'  # Rtg Carl-Schmincke-Str. (Leonberg Eltingen)
	GLEMSECKSTRASSE = 'de:08115:2340'
	GLEMSECKSTRASSE_1 = 'de:08115:2340:0:3'  # Rtg Tiroler Str./Leonb. Dreieck (Leonberg Eltingen)
	GLEMSECKSTRASSE_2 = 'de:08115:2340:0:4'  # Rtg Carl-Schmincke-Str. (Leonberg Eltingen)
	MÖRIKE_SCHULE = 'de:08115:2341'
	MÖRIKE_SCHULE_1 = 'de:08115:2341:0:3'  # Rtg Carl-Schmincke-Str. (Leonberg Eltingen)
	MÖRIKE_SCHULE_2 = 'de:08115:2341:0:4'  # Rtg Leo-Center (Leonberg Eltingen)
	ELTINGEN_MÖRIKE_SCHULE = 'de:08115:2341'
	ELTINGEN_MÖRIKE_SCHULE_1 = 'de:08115:2341:0:3'  # Rtg Carl-Schmincke-Str. (Leonberg Eltingen)
	ELTINGEN_MÖRIKE_SCHULE_2 = 'de:08115:2341:0:4'  # Rtg Leo-Center (Leonberg Eltingen)
	FORSTHAUS_II = 'de:08111:2342'
	FORSTHAUS_II_1 = 'de:08111:2342:0:3'  # Rtg Solitude (Stuttgart Stuttgart)
	FORSTHAUS_II_2 = 'de:08111:2342:0:4'  # Rtg Forsthaus I (Stuttgart Stuttgart)
	CANNSTATTER_WASEN = 'de:08111:2343'
	CANNSTATTER_WASEN_1 = 'de:08111:2343:0:1'  # Stadtbahn Rtg HB (Stuttgart Stuttgart)
	CANNSTATTER_WASEN_2 = 'de:08111:2343:0:2'  # Stadtbahn Ankunft (Stuttgart Stuttgart)
	TENNISPLATZ_1 = 'de:08111:2344:0:3'  # Rtg Metzgerhau (Stuttgart Stuttgart)
	TENNISPLATZ_2 = 'de:08111:2344:0:4'  # Rtg Birkenkopf (Stuttgart Stuttgart)
	TENNISPLATZ_3 = 'de:08118:5520:0:3'  # eRtg Karlshöhe (Ludwigsburg Ludwigsburg)
	TENNISPLATZ_4 = 'de:08118:5520:0:4'  # eRtg Robert-Franck-Allee (Ludwigsburg Ludwigsburg)
	LEONBERG_BELFORTER_PLATZ = 'de:08115:2345'
	LEONBERG_BELFORTER_PLATZ_1 = 'de:08115:2345:0:3'  # Rtg Süd (Leonberg Leonberg)
	LEONBERG_BELFORTER_PLATZ_2 = 'de:08115:2345:0:4'  # Rtg Nord/Nordwest (Leonberg Leonberg)
	BELFORTER_PLATZ = 'de:08115:2345'
	BELFORTER_PLATZ_1 = 'de:08115:2345:0:3'  # Rtg Süd (Leonberg Leonberg)
	BELFORTER_PLATZ_2 = 'de:08115:2345:0:4'  # Rtg Nord/Nordwest (Leonberg Leonberg)
	GENERATORSTRASSE = 'de:08111:2346'
	GENERATORSTRASSE_1 = 'de:08111:2346:0:3'  # Rtg Lilienthalstr. (Stuttgart Stuttgart)
	GENERATORSTRASSE_2 = 'de:08111:2346:0:4'  # Rtg Holderäcker (Stuttgart Stuttgart)
	KRANSTRASSE = 'de:08111:2347'
	KRANSTRASSE_1 = 'de:08111:2347:0:3'  # Rtg Hemminger Str. (Stuttgart Stuttgart)
	KRANSTRASSE_2 = 'de:08111:2347:0:4'  # Rtg Motorstr. 24 (Stuttgart Stuttgart)
	LILIENTHALSTRASSE = 'de:08111:2348'
	LILIENTHALSTRASSE_1 = 'de:08111:2348:0:3'  # Rtg Motorstr. 24 (Stuttgart Stuttgart)
	LILIENTHALSTRASSE_2 = 'de:08111:2348:0:4'  # Rtg Generatorstr. (Stuttgart Stuttgart)
	LEONBERG_SEEGARTEN = 'de:08115:2349'
	LEONBERG_SEEGARTEN_1 = 'de:08115:2349:0:3'  # Rtg Altstadt/Törlensweg (Leonberg Leonberg)
	LEONBERG_SEEGARTEN_2 = 'de:08115:2349:0:4'  # Rtg Samariterstift (Leonberg Leonberg)
	SEEGARTEN = 'de:08115:2349'
	SEEGARTEN_1 = 'de:08115:2349:0:3'  # Rtg Altstadt/Törlensweg (Leonberg Leonberg)
	SEEGARTEN_2 = 'de:08115:2349:0:4'  # Rtg Samariterstift (Leonberg Leonberg)
	MOTORSTRASSE_24 = 'de:08111:2350'
	MOTORSTRASSE_24_1 = 'de:08111:2350:0:3'  # Rtg Kranstr. (Stuttgart Stuttgart)
	MOTORSTRASSE_24_2 = 'de:08111:2350:0:4'  # Rtg Lilienthalstr. (Stuttgart Stuttgart)
	RAMTEL_NEUE_RAMTELSTR = 'de:08115:2351'
	RAMTEL_NEUE_RAMTELSTR_1 = 'de:08115:2351:0:3'  # Rtg Leonberg Waldfriedhof (Leonberg Ramtel)
	RAMTEL_NEUE_RAMTELSTR_2 = 'de:08115:2351:0:4'  # Rtg Gerlinger Str. (Leonberg Ramtel)
	NEUE_RAMTELSTRASSE = 'de:08115:2351'
	NEUE_RAMTELSTRASSE_1 = 'de:08115:2351:0:3'  # Rtg Leonberg Waldfriedhof (Leonberg Ramtel)
	NEUE_RAMTELSTRASSE_2 = 'de:08115:2351:0:4'  # Rtg Gerlinger Str. (Leonberg Ramtel)
	ALTSTADT_1 = 'de:08115:2352:1:3'  # Rtg Törlensweg (Leonberg Leonberg)
	ALTSTADT_2 = 'de:08115:2352:1:4'  # Rtg Bildstöckle (Leonberg Leonberg)
	ALTSTADT_3 = 'de:08115:3129:0:3'  # Rtg Postamt (Waldenbuch Waldenbuch)
	ALTSTADT_4 = 'de:08115:3129:0:4'  # Rtg Liebenau (Waldenbuch Waldenbuch)
	LEONBERG_ALTSTADT = 'de:08115:2352'
	LEONBERG_ALTSTADT_1 = 'de:08115:2352:1:3'  # Rtg Törlensweg (Leonberg Leonberg)
	LEONBERG_ALTSTADT_2 = 'de:08115:2352:1:4'  # Rtg Bildstöckle (Leonberg Leonberg)
	ELTINGEN_TIROLER_STRASSE = 'de:08115:2353'
	ELTINGEN_TIROLER_STRASSE_1 = 'de:08115:2353:0:3'  # Rtg Ramtel Hirschberger Str. (Leonberg Eltingen)
	ELTINGEN_TIROLER_STRASSE_2 = 'de:08115:2353:0:4'  # Rtg Glemseckstr. (Leonberg Eltingen)
	TIROLER_STRASSE = 'de:08115:2353'
	TIROLER_STRASSE_1 = 'de:08115:2353:0:3'  # Rtg Ramtel Hirschberger Str. (Leonberg Eltingen)
	TIROLER_STRASSE_2 = 'de:08115:2353:0:4'  # Rtg Glemseckstr. (Leonberg Eltingen)
	LEONBERG_TÖRLENSWEG = 'de:08115:2355'
	LEONBERG_TÖRLENSWEG_1 = 'de:08115:2355:0:3'  # Rtg J.-Kepler-Gymn. (Leonberg Leonberg)
	LEONBERG_TÖRLENSWEG_2 = 'de:08115:2355:0:4'  # Rtg Altstadt/Seegarten (Leonberg Leonberg)
	TÖRLENSWEG = 'de:08115:2355'
	TÖRLENSWEG_1 = 'de:08115:2355:0:3'  # Rtg J.-Kepler-Gymn. (Leonberg Leonberg)
	TÖRLENSWEG_2 = 'de:08115:2355:0:4'  # Rtg Altstadt/Seegarten (Leonberg Leonberg)
	BLOSENBERGKIRCHE = 'de:08115:2356:1:3'  # Rtg Schleiermacherstr. (Leonberg Leonberg)
	LEONBERG_BLOSENBERGKIRCHE = 'de:08115:2356:1:3'  # Rtg Schleiermacherstr. (Leonberg Leonberg)
	HEMMINGER_STRASSE_1 = 'de:08111:2357:0:3'  # Rtg Weilimdorf Bf (Stuttgart Stuttgart)
	HEMMINGER_STRASSE_2 = 'de:08111:2357:0:4'  # Rtg Kranstr. (Stuttgart Stuttgart)
	HEMMINGER_STRASSE_3 = 'de:08118:4529:1:3'  # eRtg Weissacher Straße (Ditzingen Heimerdingen)
	HEMMINGER_STRASSE_4 = 'de:08118:4529:1:4'  # eRtg Heimerdingen (Ditzingen Heimerdingen)
	ELWERTSTRASSE = 'de:08111:2358:0:4'  # Rtg Bad Cannstatt Bf (Stuttgart Stuttgart)
	LEONBERGER_DREIECK = 'de:08115:2359'
	LEONBERGER_DREIECK_1 = 'de:08115:2359:0:3'  # Leonberg Schumisberg (Leonberg Eltingen)
	LEONBERGER_DREIECK_2 = 'de:08115:2359:0:4'  # Rtg Ramtel Göppinger Str. (Leonberg Eltingen)
	ELTINGEN_LEONB__DREIECK = 'de:08115:2359'
	ELTINGEN_LEONB__DREIECK_1 = 'de:08115:2359:0:3'  # Leonberg Schumisberg (Leonberg Eltingen)
	ELTINGEN_LEONB__DREIECK_2 = 'de:08115:2359:0:4'  # Rtg Ramtel Göppinger Str. (Leonberg Eltingen)
	LEONBERGER_STRASSE_1 = 'de:08115:2360:0:3'  # Rtg LeoCenter (Leonberg Eltingen)
	LEONBERGER_STRASSE_2 = 'de:08115:2360:0:4'  # Rtg Leobad (Leonberg Eltingen)
	LEONBERGER_STRASSE_3 = 'de:08118:4554:0:3'  # eRtg Schulstraße (Gerlingen Gerlingen)
	LEONBERGER_STRASSE_4 = 'de:08118:4554:0:4'  # eRtg Ringstraße (Gerlingen Gerlingen)
	LEONBERGER_STRASSE_5 = 'de:08236:3514:0:1'  # Rtg Perouse (Heimsheim Heimsheim)
	LEONBERGER_STRASSE_6 = 'de:08236:3514:0:2'  # Rtg Heimsheim Marktplatz (Heimsheim Heimsheim)
	ELTINGEN_LEONBERGER_STR = 'de:08115:2360'
	ELTINGEN_LEONBERGER_STR_1 = 'de:08115:2360:0:3'  # Rtg LeoCenter (Leonberg Eltingen)
	ELTINGEN_LEONBERGER_STR_2 = 'de:08115:2360:0:4'  # Rtg Leobad (Leonberg Eltingen)
	LEONBERG_HEINRICH_ESSIG_STR = 'de:08115:2363:0:3'  # Rtg Feuerbacher Str. (Leonberg Leonberg)
	HEINRICH_ESSIG_STRASSE = 'de:08115:2363:0:3'  # Rtg Feuerbacher Str. (Leonberg Leonberg)
	WALTER_HELMES_WEG = 'de:08115:2364:0:3'  # Rtg Heinrich-Essig-Str. (Leonberg Leonberg)
	LEONBERG_W__HELMES_WEG = 'de:08115:2364:0:3'  # Rtg Heinrich-Essig-Str. (Leonberg Leonberg)
	LEONBERG_KORNTALER_STR = 'de:08115:2366:0:3'  # Rtg Marbacher Str. (Leonberg Leonberg)
	ULMER_STRASSE_1 = 'de:08115:2368:0:3'  # Rtg Gerlinger Str. (Leonberg Ramtel)
	ULMER_STRASSE_2 = 'de:08116:4225:0:3'  # eRtg Stuttgarter Straße (Plochingen Plochingen)
	ULMER_STRASSE_3 = 'de:08116:4225:0:4'  # eRtg Kreissparkasse (Plochingen Plochingen)
	ULMER_STRASSE_4 = 'de:08116:6498:1:3'  # Rtg Plochingen (Reichenbach an der Fils Reichenbach (F))
	ULMER_STRASSE_5 = 'de:08116:6498:1:4'  # Rtg Ebersbach (Reichenbach an der Fils Reichenbach (F))
	RAMTEL_ULMER_STRASSE = 'de:08115:2368:0:3'  # Rtg Gerlinger Str. (Leonberg Ramtel)
	DIESELSTRASSE_1 = 'de:08119:2370:1:3'  # Ankunft/Rtg Daimlerstr. (Fellbach Oeffingen)
	DIESELSTRASSE_2 = 'de:08115:4556:0:3'  # Rtg Leonberg Gartenstadt/Schillerstr. (Rutesheim Rutesheim)
	DIESELSTRASSE_3 = 'de:08115:4556:0:4'  # Rtg Rathaus (Rutesheim Rutesheim)
	DIESELSTRASSE_4 = 'de:08119:7487:0:3'  # Rtg Post (Oppenweiler Oppenweiler)
	DIESELSTRASSE_5 = 'de:08117:3302:0:H1'  # Rtg Heiningen (Göppingen Göppingen)
	DIESELSTRASSE_6 = 'de:08117:3302:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	OEFFINGEN_DIESELSTRASSE = 'de:08119:2370:1:3'  # Ankunft/Rtg Daimlerstr. (Fellbach Oeffingen)
	DAIMLERSTRASSE_1 = 'de:08119:2371:0:3'  # Rtg Rathaus (Fellbach Oeffingen)
	DAIMLERSTRASSE_2 = 'de:08119:2371:0:4'  # Rtg Dieselstr. (Fellbach Oeffingen)
	DAIMLERSTRASSE_3 = 'de:08115:3214:0:3'  # eRtg Maybachstraße (Schönaich Schönaich)
	DAIMLERSTRASSE_4 = 'de:08118:3433:0:3'  # eRtg Löscher Ost (Möglingen Möglingen)
	DAIMLERSTRASSE_5 = 'de:08118:3433:0:4'  # eRtg Lotter (Möglingen Möglingen)
	DAIMLERSTRASSE_6 = 'de:08118:5490:0:3'  # eRtg Friedenstraße 52 (Ludwigsburg Ludwigsburg)
	DAIMLERSTRASSE_7 = 'de:08118:5490:0:4'  # eRtg Steinbeisstraße (Ludwigsburg Ludwigsburg)
	DAIMLERSTRASSE_8 = 'de:08115:6533:0:3'  # Rtg Am Eschelbach (Holzgerlingen Holzgerlingen)
	DAIMLERSTRASSE_9 = 'de:08115:6533:0:4'  # Rtg R.-Bosch-Str. (Holzgerlingen Holzgerlingen)
	DAIMLERSTRASSE_10 = 'de:08115:7002:0:3'  # eRtg Affstätter Tal (Herrenberg Herrenberg)
	OEFFINGEN_DAIMLERSTRASSE = 'de:08119:2371'
	OEFFINGEN_DAIMLERSTRASSE_1 = 'de:08119:2371:0:3'  # Rtg Rathaus (Fellbach Oeffingen)
	OEFFINGEN_DAIMLERSTRASSE_2 = 'de:08119:2371:0:4'  # Rtg Dieselstr. (Fellbach Oeffingen)
	OEFFINGEN_RATHAUS = 'de:08119:2372:0:3'  # Rtg Alemannenstr. (Fellbach Oeffingen)
	OEFFINGEN_KLOSTERPLATZ = 'de:08119:2373:0:4'  # Rtg Daimlerstr. (Fellbach Oeffingen)
	KLOSTERPLATZ = 'de:08119:2373:0:4'  # Rtg Daimlerstr. (Fellbach Oeffingen)
	OEFFINGEN_ALEMANNENSTR = 'de:08119:2374'
	OEFFINGEN_ALEMANNENSTR_1 = 'de:08119:2374:0:3'  # Rtg Schmiden Dorfwiesen (Fellbach Oeffingen)
	OEFFINGEN_ALEMANNENSTR_2 = 'de:08119:2374:0:4'  # Rtg Gemeindezentrum (Fellbach Oeffingen)
	SCHMIDEN_DORFWIESEN = 'de:08119:2375'
	SCHMIDEN_DORFWIESEN_1 = 'de:08119:2375:0:3'  # Rtg Bühnerstr. (Fellbach Schmiden)
	SCHMIDEN_DORFWIESEN_2 = 'de:08119:2375:0:4'  # Rtg Oeffingen Alemannenstr. (Fellbach Schmiden)
	DORFWIESEN = 'de:08119:2375'
	DORFWIESEN_1 = 'de:08119:2375:0:3'  # Rtg Bühnerstr. (Fellbach Schmiden)
	DORFWIESEN_2 = 'de:08119:2375:0:4'  # Rtg Oeffingen Alemannenstr. (Fellbach Schmiden)
	SCHMIDEN_BÜHNERSTRASSE = 'de:08119:2376'
	SCHMIDEN_BÜHNERSTRASSE_1 = 'de:08119:2376:1:3'  # Rtg Gutenbergstr. (Fellbach Schmiden)
	SCHMIDEN_BÜHNERSTRASSE_2 = 'de:08119:2376:1:5'  # Ankunft/Rtg Friedrichstr. (Fellbach Schmiden)
	BÜHNERSTRASSE = 'de:08119:2376'
	BÜHNERSTRASSE_1 = 'de:08119:2376:1:3'  # Rtg Gutenbergstr. (Fellbach Schmiden)
	BÜHNERSTRASSE_2 = 'de:08119:2376:1:5'  # Ankunft/Rtg Friedrichstr. (Fellbach Schmiden)
	SCHMIDEN_RATHAUS = 'de:08119:2377'
	SCHMIDEN_RATHAUS_1 = 'de:08119:2377:1:4'  # Rtg Dorfwiesen (Fellbach Schmiden)
	SCHMIDEN_RATHAUS_2 = 'de:08119:2377:1:6'  # Rtg Bühnerstr. (Fellbach Schmiden)
	SCHMIDEN_JAKOB__BUTTERSTR = 'de:08119:2378'
	SCHMIDEN_JAKOB__BUTTERSTR_1 = 'de:08119:2378:0:4'  # Rtg Rathaus (Fellbach Schmiden)
	SCHMIDEN_JAKOB__BUTTERSTR_2 = 'de:08119:2378:0:6'  # Rtg Rathaus (Fellbach Schmiden)
	SCHMIDEN_JAKOB__BUTTERSTR_3 = 'de:08119:2378:0:8'  # Ersatz Rtg. Rathaus (Fellbach Schmiden)
	JAKOB__BUTTERSTRASSE = 'de:08119:2378'
	JAKOB__BUTTERSTRASSE_1 = 'de:08119:2378:0:4'  # Rtg Rathaus (Fellbach Schmiden)
	JAKOB__BUTTERSTRASSE_2 = 'de:08119:2378:0:6'  # Rtg Rathaus (Fellbach Schmiden)
	JAKOB__BUTTERSTRASSE_3 = 'de:08119:2378:0:8'  # Ersatz Rtg. Rathaus (Fellbach Schmiden)
	SCHMIDEN_FRIEDRICHSTR = 'de:08119:2379'
	SCHMIDEN_FRIEDRICHSTR_1 = 'de:08119:2379:0:3'  # Rtg Kleiner Ostring (Fellbach Schmiden)
	SCHMIDEN_FRIEDRICHSTR_2 = 'de:08119:2379:0:4'  # Rtg Jakob-/Butterstr. (Fellbach Schmiden)
	AUF_DER_GANS = 'de:08111:2380'
	AUF_DER_GANS_1 = 'de:08111:2380:0:3'  # Rtg Obere Ziegelei (Stuttgart Stuttgart)
	AUF_DER_GANS_2 = 'de:08111:2380:0:4'  # Rtg Kleiner Ostring (Stuttgart Stuttgart)
	GEMEINDEZENTRUM_1 = 'de:08119:2381:0:4'  # Rtg Klosterplatz (Fellbach Oeffingen)
	GEMEINDEZENTRUM_2 = 'de:08116:4148:0:3'  # eRtg Am Wiesengrund (Esslingen am Neckar Berkheim)
	GEMEINDEZENTRUM_3 = 'de:08116:4148:0:4'  # eRtg See (Esslingen am Neckar Berkheim)
	GEMEINDEZENTRUM_4 = 'de:08115:4860:0:3'  # Rtg Ev. Kirche Altes Rathaus (Jettingen Oberjettingen)
	GEMEINDEZENTRUM_5 = 'de:08115:4860:0:4'  # Rtg Sindlinger/Herrenberger Str. (Jettingen Oberjettingen)
	GEMEINDEZENTRUM_6 = 'de:08117:15:0:H1'  # Rtg Färberstr. (Kuchen Kuchen (F))
	OEFFINGEN_GEMEINDEZENTRUM = 'de:08119:2381:0:4'  # Rtg Klosterplatz (Fellbach Oeffingen)
	KLEINER_OSTRING = 'de:08111:2382'
	KLEINER_OSTRING_1 = 'de:08111:2382:0:3'  # Rtg Auf der Gans (Stuttgart Stuttgart)
	KLEINER_OSTRING_2 = 'de:08111:2382:0:4'  # Rtg Schmiden Friedrichstr. (Stuttgart Stuttgart)
	SPORTGELÄNDE_TENNWENGERT = 'de:08119:2383'
	SPORTGELÄNDE_TENNWENGERT_1 = 'de:08119:2383:0:3'  # Rtg Neckarrems Am unteren Schlossberg (Fellbach Oeffingen)
	SPORTGELÄNDE_TENNWENGERT_2 = 'de:08119:2383:0:4'  # Rtg Ludwigsburger Str. (Fellbach Oeffingen)
	OEFFINGEN_SPORTG__TENNWENGERT = 'de:08119:2383'
	OEFFINGEN_SPORTG__TENNWENGERT_1 = 'de:08119:2383:0:3'  # Rtg Neckarrems Am unteren Schlossberg (Fellbach Oeffingen)
	OEFFINGEN_SPORTG__TENNWENGERT_2 = 'de:08119:2383:0:4'  # Rtg Ludwigsburger Str. (Fellbach Oeffingen)
	OEFFINGEN_LUDWIGSBURGER_STR = 'de:08119:2384'
	OEFFINGEN_LUDWIGSBURGER_STR_1 = 'de:08119:2384:0:3'  # Rtg Sportgelände Tennwengert (Fellbach Oeffingen)
	OEFFINGEN_LUDWIGSBURGER_STR_2 = 'de:08119:2384:0:4'  # Rtg Alemannenstr. (Fellbach Oeffingen)
	LUDWIGSBURGER_STRASSE_1 = 'de:08119:2384:0:3'  # Rtg Sportgelände Tennwengert (Fellbach Oeffingen)
	LUDWIGSBURGER_STRASSE_2 = 'de:08119:2384:0:4'  # Rtg Alemannenstr. (Fellbach Oeffingen)
	LUDWIGSBURGER_STRASSE_3 = 'de:08118:3328:0:3'  # Rtg Bosch (Schwieberdingen Schwieberdingen)
	LUDWIGSBURGER_STRASSE_4 = 'de:08118:3328:0:4'  # Rtg Mitte (Schwieberdingen Schwieberdingen)
	LUDWIGSBURGER_STRASSE_5 = 'de:08118:3439:0:3'  # eRtg Sonnenbrunnen (Möglingen Möglingen)
	LUDWIGSBURGER_STRASSE_6 = 'de:08118:3451:0:3'  # eRtg Tamm (Tamm Tamm)
	LUDWIGSBURGER_STRASSE_7 = 'de:08118:3451:0:4'  # eRtg Alter Weg (Tamm Tamm)
	LUDWIGSBURGER_STRASSE_8 = 'de:08119:4876:0:3'  # Rtg Fröbelstraße/Talstr. (Waiblingen Waiblingen)
	LUDWIGSBURGER_STRASSE_9 = 'de:08119:4876:0:4'  # eRtg Christofstraße (Waiblingen Waiblingen)
	LUDWIGSBURGER_STRASSE_10 = 'de:08118:5459:0:3'  # Rtg Pflugfelder Str. (Kornwestheim Kornwestheim)
	LUDWIGSBURGER_STRASSE_11 = 'de:08118:5459:0:4'  # Rtg An der Wette (Kornwestheim Kornwestheim)
	LUDWIGSBURGER_STRASSE_12 = 'de:08118:5525:0:3'  # eRtg Ebelstraße (Ludwigsburg Hoheneck)
	LUDWIGSBURGER_STRASSE_13 = 'de:08118:5525:0:4'  # eRtg Bottwartalstraße (Ludwigsburg Hoheneck)
	LUDWIGSBURGER_STRASSE_14 = 'de:08118:5643:0:3'  # eRtg Rathaus (Bietigheim-Bissingen Bissingen (LB))
	LUDWIGSBURGER_STRASSE_15 = 'de:08118:5643:0:4'  # eRtg Tamm (Bietigheim-Bissingen Bissingen (LB))
	LUDWIGSBURGER_STRASSE_16 = 'de:08118:5649:0:3'  # Rtg Zimmerer Pfad (Sachsenheim Großsachsenheim)
	LUDWIGSBURGER_STRASSE_17 = 'de:08118:5649:0:4'  # Rtg Hauptstr./Sachsenheim (Sachsenheim Großsachsenheim)
	KRÄHERWALD = 'de:08111:2401'
	KRÄHERWALD_1 = 'de:08111:2401:1:3'  # Rtg Geibelstr. (Stuttgart Stuttgart)
	KRÄHERWALD_2 = 'de:08111:2401:1:4'  # Rtg Doggenburg/Leibnizstr. (Stuttgart Stuttgart)
	KRÄHERWALD_3 = 'de:08111:2401:2:7'  # Nachtbus (Stuttgart Stuttgart)
	GAUSSSTRASSE = 'de:08111:2402'
	GAUSSSTRASSE_1 = 'de:08111:2402:0:3'  # Rtg Leibnizstr. (Stuttgart Stuttgart)
	GAUSSSTRASSE_2 = 'de:08111:2402:0:4'  # Rtg Fichtestr. (Stuttgart Stuttgart)
	FICHTESTRASSE = 'de:08111:2403'
	FICHTESTRASSE_1 = 'de:08111:2403:0:3'  # Rtg Gaußstr. (Stuttgart Stuttgart)
	FICHTESTRASSE_2 = 'de:08111:2403:0:4'  # Rtg Honoldweg (Stuttgart Stuttgart)
	HONOLDWEG = 'de:08111:2404'
	HONOLDWEG_1 = 'de:08111:2404:0:3'  # Rtg Fichtestr. (Stuttgart Stuttgart)
	HONOLDWEG_2 = 'de:08111:2404:0:4'  # Rtg Hölderlinplatz (Stuttgart Stuttgart)
	WUNNENSTEINSTRASSE = 'de:08111:2405'
	WUNNENSTEINSTRASSE_1 = 'de:08111:2405:0:3'  # Rtg Tunnel Ostportal (Stuttgart Stuttgart)
	WUNNENSTEINSTRASSE_2 = 'de:08111:2405:0:4'  # Rtg Wagenburgstr. (Stuttgart Stuttgart)
	TUNNEL_OSTPORTAL = 'de:08111:2406'
	TUNNEL_OSTPORTAL_1 = 'de:08111:2406:0:3'  # Rtg Staatsgalerie (Stuttgart Stuttgart)
	TUNNEL_OSTPORTAL_2 = 'de:08111:2406:0:4'  # Rtg Wunnensteinstr. (Stuttgart Stuttgart)
	LEIBNIZSTRASSE_1 = 'de:08111:2407:1:4'  # Rtg Gaußstr. (Stuttgart Stuttgart)
	LEIBNIZSTRASSE_2 = 'de:08111:2407:2:3'  # Rtg Kräherwald (Stuttgart Stuttgart)
	LEIBNIZSTRASSE_3 = 'de:08118:3364:0:3'  # Rtg Lenzhalde (Kornwestheim Kornwestheim)
	LEIBNIZSTRASSE_4 = 'de:08118:3364:0:4'  # Rtg Im Moldengraben (Kornwestheim Kornwestheim)
	GEIBELSTRASSE = 'de:08111:2408'
	GEIBELSTRASSE_1 = 'de:08111:2408:0:3'  # Rtg Nikolauspflege (Stuttgart Stuttgart)
	GEIBELSTRASSE_2 = 'de:08111:2408:0:4'  # Rtg Kräherwald (Stuttgart Stuttgart)
	MÖRIKESTRASSE_1 = 'de:08111:2409:0:3'  # Rtg Marienplatz (Stuttgart Stuttgart)
	MÖRIKESTRASSE_2 = 'de:08111:2409:0:4'  # Rtg Marien-/Silberburgstr. (Stuttgart Stuttgart)
	MÖRIKESTRASSE_3 = 'de:08118:5476:0:3'  # eRtg Osterholz (Ludwigsburg Ludwigsburg)
	MÖRIKESTRASSE_4 = 'de:08118:5476:0:4'  # eRtg Im Waldeck (Ludwigsburg Ludwigsburg)
	MÖRIKESTRASSE_5 = 'de:08118:6514:0:3'  # eRtg Rathaus (Oberriexingen Oberriexingen)
	MÖRIKESTRASSE_6 = 'de:08116:6846:0:3'  # eRtg Bettlinger Forum (Großbettlingen Großbettlingen)
	MÖRIKESTRASSE_7 = 'de:08116:6846:0:4'  # Rtg Albstraße (Großbettlingen Großbettlingen)
	MÖRIKESTRASSE_8 = 'de:08117:1053:0:R1'  # Rtg Barbarossasee (Göppingen Göppingen)
	MÖRIKESTRASSE_9 = 'de:08117:3310:0:H1'  # Rtg Breitestraße (Heiningen Heiningen (GP))
	MÖRIKESTRASSE_10 = 'de:08117:3310:0:R1'  # Rtg Uhlandstraße (Heiningen Heiningen (GP))
	MARIEN__SILBERBURGSTRASSE = 'de:08111:2410'
	MARIEN__SILBERBURGSTRASSE_1 = 'de:08111:2410:0:3'  # Rtg Mörikestr. (Stuttgart Stuttgart)
	MARIEN__SILBERBURGSTRASSE_2 = 'de:08111:2410:0:4'  # Rtg Feuersee (Stuttgart Stuttgart)
	MARKUSKIRCHE = 'de:08111:2411'
	MARKUSKIRCHE_1 = 'de:08111:2411:0:3'  # Rtg Zellerstr. (Stuttgart Stuttgart)
	MARKUSKIRCHE_2 = 'de:08111:2411:0:4'  # Rtg Lehenstr. (Stuttgart Stuttgart)
	ZELLERSTRASSE = 'de:08111:2413'
	ZELLERSTRASSE_1 = 'de:08111:2413:0:3'  # Rtg Falbenhennenstr. (Stuttgart Stuttgart)
	ZELLERSTRASSE_2 = 'de:08111:2413:0:4'  # Rtg Markuskirche (Stuttgart Stuttgart)
	LEHENSTRASSE = 'de:08111:2414'
	LEHENSTRASSE_1 = 'de:08111:2414:0:3'  # Rtg Markuskirche (Stuttgart Stuttgart)
	LEHENSTRASSE_2 = 'de:08111:2414:0:4'  # Rtg Marienplatz (Stuttgart Stuttgart)
	METZGERHAU = 'de:08111:2415:1:3'  # Rtg Forsthaus II (Stuttgart Stuttgart)
	FORSTHAUS_I = 'de:08111:2416:1:4'  # Rtg Uni/Forsth.Parkpl./Tennisplatz (Stuttgart Stuttgart)
	UMGELTERWEG = 'de:08111:2417'
	UMGELTERWEG_1 = 'de:08111:2417:0:3'  # Rtg Im Himmerreich (Stuttgart Stuttgart)
	UMGELTERWEG_2 = 'de:08111:2417:0:4'  # Rtg Aspenwaldstr. (Stuttgart Stuttgart)
	ASPENWALDSTRASSE = 'de:08111:2418'
	ASPENWALDSTRASSE_1 = 'de:08111:2418:0:3'  # Rtg Umgelterweg (Stuttgart Stuttgart)
	ASPENWALDSTRASSE_2 = 'de:08111:2418:0:4'  # Rtg Auberlenweg (Stuttgart Stuttgart)
	HÄNDELSTRASSE = 'de:08111:2419'
	HÄNDELSTRASSE_1 = 'de:08111:2419:0:3'  # Rtg Eltinger Str. (Stuttgart Stuttgart)
	HÄNDELSTRASSE_2 = 'de:08111:2419:0:4'  # Rtg Laihle (Stuttgart Stuttgart)
	LAIHLE = 'de:08111:2420'
	LAIHLE_1 = 'de:08111:2420:0:3'  # Rtg Händelstr. (Stuttgart Stuttgart)
	LAIHLE_2 = 'de:08111:2420:0:4'  # Rtg Paul-Lincke-Str. (Stuttgart Stuttgart)
	BOTNANG_FREIBAD = 'de:08111:2421'
	BOTNANG_FREIBAD_1 = 'de:08111:2421:0:3'  # Rtg Paul-Lincke-Str. (Stuttgart Stuttgart)
	BOTNANG_FREIBAD_2 = 'de:08111:2421:0:4'  # Rtg Mähderklinge (Stuttgart Stuttgart)
	FEUERBACH_FRIEDHOF = 'de:08111:2422'
	FEUERBACH_FRIEDHOF_1 = 'de:08111:2422:0:3'  # Rtg Heimberg (Stuttgart Stuttgart)
	FEUERBACH_FRIEDHOF_2 = 'de:08111:2422:0:4'  # Rtg Wildensteinstr. (Stuttgart Stuttgart)
	HEIMBERG = 'de:08111:2423'
	HEIMBERG_1 = 'de:08111:2423:0:3'  # Rtg Mähderklinge (Stuttgart Stuttgart)
	HEIMBERG_2 = 'de:08111:2423:0:4'  # Rtg Friedhof (Stuttgart Stuttgart)
	WILDENSTEINSTRASSE = 'de:08111:2424'
	WILDENSTEINSTRASSE_1 = 'de:08111:2424:0:3'  # Rtg Friedhof (Stuttgart Stuttgart)
	WILDENSTEINSTRASSE_2 = 'de:08111:2424:0:4'  # Rtg Hohewartstr. (Stuttgart Stuttgart)
	FORSTHAUS_PARKPLATZ = 'de:08111:2425:1:3'  # Rtg Forsthaus I (Stuttgart Stuttgart)
	MÄHDERKLINGE = 'de:08111:2427'
	MÄHDERKLINGE_1 = 'de:08111:2427:0:3'  # Rtg Freibad (Stuttgart Stuttgart)
	MÄHDERKLINGE_2 = 'de:08111:2427:0:4'  # Rtg Heimberg (Stuttgart Stuttgart)
	ALTER_GUTSHOF = 'de:08111:2428:1:4'  # Ankunft/Rtg Burgholzhof (Stuttgart Stuttgart)
	PAUL_LINCKE_STRASSE = 'de:08111:2429'
	PAUL_LINCKE_STRASSE_1 = 'de:08111:2429:0:3'  # Rtg Laihle (Stuttgart Stuttgart)
	PAUL_LINCKE_STRASSE_2 = 'de:08111:2429:0:4'  # Rtg Freibad (Stuttgart Stuttgart)
	KIRCHHEIMER_STRASSE_1 = 'de:08111:2430:1:4'  # Rtg Im Lauch (Stuttgart Stuttgart)
	KIRCHHEIMER_STRASSE_2 = 'de:08116:3827:0:3'  # Rtg Eugenstr. (Wendlingen am Neckar Wendlingen (N))
	KIRCHHEIMER_STRASSE_3 = 'de:08116:4216:0:3'  # eRtg Brahmsstraße (Notzingen Notzingen)
	KIRCHHEIMER_STRASSE_4 = 'de:08116:4376:0:3'  # eRtg Plochinger Straße (Nürtingen Nürtingen)
	KIRCHHEIMER_STRASSE_5 = 'de:08118:5726:0:3'  # eRtg Burgermühle (Bönnigheim Bönnigheim)
	KIRCHHEIMER_STRASSE_6 = 'de:08118:5726:0:4'  # eRtg Schulzentrum (Bönnigheim Bönnigheim)
	KIRCHHEIMER_STRASSE_7 = 'de:08116:7819:0:3'  # Rtg Neuhausen (Köngen Köngen)
	KIRCHHEIMER_STRASSE_8 = 'de:08116:7819:0:4'  # Rtg Wendlingen (Köngen Köngen)
	KIRCHHEIMER_STRASSE_9 = 'de:08117:2115:0:H1'  # Rtg Weilheimer Straße (Zell unter Aichelberg Zell (A))
	KIRCHHEIMER_STRASSE_10 = 'de:08117:2115:0:R1'  # Rtg Kath. Kirche (Zell unter Aichelberg Zell (A))
	IM_LAUCH = 'de:08111:2431:0:4'  # Rtg Bußbachstr. (Stuttgart Stuttgart)
	BUSSBACHSTRASSE = 'de:08111:2432:0:4'  # Rtg Treiberstr. (Stuttgart Stuttgart)
	IN_DER_WERRE = 'de:08111:2433:0:3'  # Rtg Baumgartenweg (Stuttgart Stuttgart)
	BAUMGARTENWEG = 'de:08111:2434:0:3'  # Rtg Oberwiesenstr.55 (Stuttgart Stuttgart)
	OBERWIESENSTRASSE_55 = 'de:08111:2435:0:3'  # Rtg Sillenbuch (Stuttgart Stuttgart)
	FRIDINGER_STRASSE = 'de:08111:2436:0:3'  # Rtg In der Werre (Stuttgart Stuttgart)
	KÜHWASEN = 'de:08111:2437:0:3'  # Rtg Fridinger Str. (Stuttgart Stuttgart)
	TREIBERSTRASSE = 'de:08111:2438:0:4'  # Rtg Erdbeerweg (Stuttgart Stuttgart)
	ERDBEERWEG = 'de:08111:2439:0:4'  # Rtg Kühwasen (Stuttgart Stuttgart)
	IM_CHAUSSEEFELD = 'de:08111:2440'
	IM_CHAUSSEEFELD_1 = 'de:08111:2440:0:3'  # Rtg Plieningen Garbe (Stuttgart Stuttgart)
	IM_CHAUSSEEFELD_2 = 'de:08111:2440:0:4'  # Rtg Steckfeld (Stuttgart Stuttgart)
	IM_HIMMERREICH = 'de:08111:2450'
	IM_HIMMERREICH_1 = 'de:08111:2450:0:3'  # Rtg Forsthaus Parkplatz (Stuttgart Stuttgart)
	IM_HIMMERREICH_2 = 'de:08111:2450:0:4'  # Rtg Umgelterweg (Stuttgart Stuttgart)
	BIRKENKOPF = 'de:08111:2451'
	BIRKENKOPF_1 = 'de:08111:2451:1:3'  # Rtg Tennisplatz (Stuttgart Stuttgart)
	BIRKENKOPF_2 = 'de:08111:2451:1:4'  # Rtg Westbahnhof (Stuttgart Stuttgart)
	NIKOLAUSPFLEGE = 'de:08111:2452'
	NIKOLAUSPFLEGE_1 = 'de:08111:2452:0:3'  # Rtg Botnanger Sattel (Stuttgart Stuttgart)
	NIKOLAUSPFLEGE_2 = 'de:08111:2452:0:4'  # Rtg Geibelstr. (Stuttgart Stuttgart)
	AM_BISMARCKTURM = 'de:08111:2453'
	AM_BISMARCKTURM_1 = 'de:08111:2453:0:3'  # Rtg Doggenburg (Stuttgart Stuttgart)
	AM_BISMARCKTURM_2 = 'de:08111:2453:0:4'  # Rtg Feuerbacher Weg (Stuttgart Stuttgart)
	FEUERBACHER_WEG = 'de:08111:2454'
	FEUERBACHER_WEG_1 = 'de:08111:2454:1:3'  # Linien 43/50 Rtg Am Bismarckturm (Stuttgart Stuttgart)
	FEUERBACHER_WEG_2 = 'de:08111:2454:1:4'  # Linien 43/50 Rtg Killesberg (Stuttgart Stuttgart)
	HÖHENFREIBAD = 'de:08111:2455'
	HÖHENFREIBAD_1 = 'de:08111:2455:0:3'  # Rtg Oskar-Schlemmer-Str. (Stuttgart Stuttgart)
	HÖHENFREIBAD_2 = 'de:08111:2455:0:4'  # Rtg Pragsattel (Stuttgart Stuttgart)
	ASPERGER_STRASSE = 'de:08111:2459'
	ASPERGER_STRASSE_1 = 'de:08111:2459:0:1'  # Li 98 Rtg Pflugfelder Str. (Stuttgart Stuttgart)
	ASPERGER_STRASSE_2 = 'de:08111:2459:0:3'  # Rtg Pflugfelder Str. (Stuttgart Stuttgart)
	ASPERGER_STRASSE_3 = 'de:08111:2459:0:4'  # Rtg Stammheim (Stuttgart Stuttgart)
	BORKUMSTRASSE__NEUW__SIEDL = 'de:08111:2460'
	BORKUMSTRASSE__NEUW__SIEDL_1 = 'de:08111:2460:0:1'  # Rtg Zuff. Porsche (Stuttgart Stuttgart)
	BORKUMSTRASSE__NEUW__SIEDL_2 = 'de:08111:2460:0:2'  # Ankunft (Stuttgart Stuttgart)
	NORDSEE__SCHWIEBERD__STRASSE = 'de:08111:2461:0:4'  # Rtg Borkumstr. (Stuttgart Stuttgart)
	MARCONISTRASSE = 'de:08111:2463'
	MARCONISTRASSE_1 = 'de:08111:2463:1:3'  # Rtg Schlotwiese/Marconibrücke (Stuttgart Stuttgart)
	MARCONISTRASSE_2 = 'de:08111:2463:1:4'  # Rtg Zuffenhausen Porsche (Stuttgart Stuttgart)
	MARCONIBRÜCKE = 'de:08111:2464'
	MARCONIBRÜCKE_1 = 'de:08111:2464:1:3'  # Rtg Schlotwiese/Marconistr. (Stuttgart Stuttgart)
	MARCONIBRÜCKE_2 = 'de:08111:2464:1:4'  # Rtg Zahn-Nopper-Str. (Stuttgart Stuttgart)
	SCHLOTWIESE = 'de:08111:2465:0:4'  # Rtg alle Richtungen (Stuttgart Stuttgart)
	FRANKENSTRASSE_1 = 'de:08111:2467:0:3'  # Rtg Hohensteinstraße (Stuttgart Stuttgart)
	FRANKENSTRASSE_2 = 'de:08111:2467:0:4'  # Rtg Ohmstr. (Stuttgart Stuttgart)
	FRANKENSTRASSE_3 = 'de:08115:3206:0:3'  # eRtg ZOB Böbl. (Sindelfingen Sindelfingen)
	YITZHAK_RABIN_STRASSE = 'de:08111:2469'
	YITZHAK_RABIN_STRASSE_1 = 'de:08111:2469:0:3'  # Rtg Hebsacker Str./Pragsattel (Stuttgart Stuttgart)
	YITZHAK_RABIN_STRASSE_2 = 'de:08111:2469:0:4'  # Rtg Burgholzhof/Alter Gutshof (Stuttgart Stuttgart)
	BURGHOLZHOF = 'de:08111:2470'
	BURGHOLZHOF_1 = 'de:08111:2470:0:3'  # Rtg Yitzhak-Rabin-Str. (Stuttgart Stuttgart)
	BURGHOLZHOF_2 = 'de:08111:2470:0:4'  # Rtg Schnarrenberg (Stuttgart Stuttgart)
	SCHNARRENBERG = 'de:08111:2471'
	SCHNARRENBERG_1 = 'de:08111:2471:0:3'  # Rtg Hallschlag (Stuttgart Stuttgart)
	SCHNARRENBERG_2 = 'de:08111:2471:0:4'  # Rtg Burgholzhof (Stuttgart Stuttgart)
	IM_RAISER = 'de:08111:2472'
	IM_RAISER_1 = 'de:08111:2472:0:3'  # Rtg Zuffenhäuser Heide (Stuttgart Stuttgart)
	IM_RAISER_2 = 'de:08111:2472:0:4'  # Rtg Rilke-Realschule (Stuttgart Stuttgart)
	IM_RAISER_3 = 'de:08111:2472:0:5'  # NBus N5 Rtg Burgholzhof (Stuttgart Stuttgart)
	IM_RAISER_4 = 'de:08111:2472:0:6'  # NBus N4 Rtg Schozacher Str. (Stuttgart Stuttgart)
	HALLSCHLAG = 'de:08111:2473'
	HALLSCHLAG_1 = 'de:08111:2473:1:1'  # Rtg Riethmüllerhaus (Stuttgart Stuttgart)
	HALLSCHLAG_2 = 'de:08111:2473:1:2'  # Rtg Bottroper Str. (Stuttgart Stuttgart)
	HALLSCHLAG_3 = 'de:08111:2473:2:3'  # Rtg Am Römerkastell (Stuttgart Stuttgart)
	HALLSCHLAG_4 = 'de:08111:2473:2:4'  # Rtg Schnarrenberg (Stuttgart Stuttgart)
	RILKE_REALSCHULE = 'de:08111:2474'
	RILKE_REALSCHULE_1 = 'de:08111:2474:0:3'  # Rtg Im Raiser (Stuttgart Stuttgart)
	RILKE_REALSCHULE_2 = 'de:08111:2474:0:4'  # Rtg Tappachstr. (Stuttgart Stuttgart)
	ROBERT_BOSCH_KRANKENHAUS = 'de:08111:2475:0:4'  # Rtg Yitzhak-Rabin-Str. (Stuttgart Stuttgart)
	ALBERT_LUTHULI_PLATZ = 'de:08111:2476:0:4'  # Rtg R.-Bosch-Krankenh./Y.-Rabin-Str. (Stuttgart Stuttgart)
	HEBSACKER_STRASSE = 'de:08111:2477'
	HEBSACKER_STRASSE_1 = 'de:08111:2477:0:3'  # Rtg Yitzhak-Rabin-Str. (Stuttgart Stuttgart)
	HEBSACKER_STRASSE_2 = 'de:08111:2477:0:4'  # Rtg Zuffenhäuser Heide (Stuttgart Stuttgart)
	ZUFFENHÄUSER_HEIDE = 'de:08111:2478'
	ZUFFENHÄUSER_HEIDE_1 = 'de:08111:2478:0:3'  # Rtg Hebsacker Str. (Stuttgart Stuttgart)
	ZUFFENHÄUSER_HEIDE_2 = 'de:08111:2478:0:4'  # Rtg Im Raiser (Stuttgart Stuttgart)
	AALSTRASSE = 'de:08111:2480'
	AALSTRASSE_1 = 'de:08111:2480:0:3'  # Rtg Mühlhausen Schloss (Stuttgart Stuttgart)
	AALSTRASSE_2 = 'de:08111:2480:0:4'  # Rtg Steinbuttstr. (Stuttgart Stuttgart)
	FORELLENWEG = 'de:08111:2481'
	FORELLENWEG_1 = 'de:08111:2481:0:3'  # Rtg Steinbuttstr. (Stuttgart Stuttgart)
	FORELLENWEG_2 = 'de:08111:2481:0:4'  # Rtg Freiberg (Stuttgart Stuttgart)
	STEINBUTTSTRASSE = 'de:08111:2482'
	STEINBUTTSTRASSE_1 = 'de:08111:2482:0:3'  # Rtg Aalstr. (Stuttgart Stuttgart)
	STEINBUTTSTRASSE_2 = 'de:08111:2482:0:4'  # Rtg Forellenweg (Stuttgart Stuttgart)
	MARABUSTRASSE = 'de:08111:2483'
	MARABUSTRASSE_1 = 'de:08111:2483:0:3'  # Rtg Steinhaldenfeld (Stuttgart Stuttgart)
	MARABUSTRASSE_2 = 'de:08111:2483:0:4'  # Rtg Neugereut (Stuttgart Stuttgart)
	NEUGEREUT = 'de:08111:2484'
	NEUGEREUT_1 = 'de:08111:2484:1:1'  # Stadtbahn Endhaltestelle (Stuttgart Stuttgart)
	NEUGEREUT_2 = 'de:08111:2484:2:3'  # Rtg Marabustr. (Stuttgart Stuttgart)
	NEUGEREUT_3 = 'de:08111:2484:2:4'  # Rtg Schneideräckerstr. (Stuttgart Stuttgart)
	EDELWEISSWEG = 'de:08111:2487'
	EDELWEISSWEG_1 = 'de:08111:2487:0:3'  # Rtg Sommerrain (Stuttgart Stuttgart)
	EDELWEISSWEG_2 = 'de:08111:2487:0:4'  # Rtg Obere Ziegelei (Stuttgart Stuttgart)
	MUCKENSTURM = 'de:08111:2488'
	MUCKENSTURM_1 = 'de:08111:2488:0:3'  # Rtg Obere Ziegelei (Stuttgart Stuttgart)
	MUCKENSTURM_2 = 'de:08111:2488:0:4'  # Rtg Hauptfriedhof (Stuttgart Stuttgart)
	STEINHALDENFELD = 'de:08111:2490'
	STEINHALDENFELD_1 = 'de:08111:2490:1:1'  # Rtg Hauptfriedhof (Stuttgart Stuttgart)
	STEINHALDENFELD_2 = 'de:08111:2490:1:2'  # Rtg Neugereut (Stuttgart Stuttgart)
	STEINHALDENFELD_3 = 'de:08111:2490:2:3'  # Rtg Zuckerberg (Stuttgart Stuttgart)
	STEINHALDENFELD_4 = 'de:08111:2490:2:4'  # Rtg Marabustr. (Stuttgart Stuttgart)
	ZUCKERBERG = 'de:08111:2491'
	ZUCKERBERG_1 = 'de:08111:2491:2:3'  # Rtg Hauptfriedhof (Stuttgart Stuttgart)
	ZUCKERBERG_2 = 'de:08111:2491:2:4'  # Rtg Steinhaldenfeld (Stuttgart Stuttgart)
	HAUPTFRIEDHOF = 'de:08111:2492'
	HAUPTFRIEDHOF_1 = 'de:08111:2492:1:1'  # Rtg Obere Ziegelei (Stuttgart Stuttgart)
	HAUPTFRIEDHOF_2 = 'de:08111:2492:1:2'  # Rtg Steinhaldenfeld (Stuttgart Stuttgart)
	HAUPTFRIEDHOF_3 = 'de:08111:2492:2:3'  # Rtg Muckensturm (Stuttgart Stuttgart)
	HAUPTFRIEDHOF_4 = 'de:08111:2492:2:4'  # Rtg Zuckerberg (Stuttgart Stuttgart)
	RIETHMÜLLERHAUS = 'de:08111:2493'
	RIETHMÜLLERHAUS_1 = 'de:08111:2493:1:1'  # Rtg Züricher Str. (Stuttgart Stuttgart)
	RIETHMÜLLERHAUS_2 = 'de:08111:2493:1:2'  # Rtg Hallschlag (Stuttgart Stuttgart)
	ZÜRICHER_STRASSE_1 = 'de:08111:2494:1:1'  # Rtg Löwentor (Stuttgart Stuttgart)
	ZÜRICHER_STRASSE_2 = 'de:08111:2494:1:2'  # Rtg Riethmüllerhaus (Stuttgart Stuttgart)
	ZÜRICHER_STRASSE_3 = 'de:08119:3663:0:3'  # eRtg Carl-Kaelble-Straße (Backnang Backnang)
	ZÜRICHER_STRASSE_4 = 'de:08119:3663:0:4'  # eRtg Tuxer Weg (Backnang Backnang)
	FELLBACH_FRIEDRICH_LIST_STR = 'de:08119:2496'
	FELLBACH_FRIEDRICH_LIST_STR_1 = 'de:08119:2496:0:3'  # Rtg Rems-Murr-Center (Fellbach Fellbach)
	FELLBACH_FRIEDRICH_LIST_STR_2 = 'de:08119:2496:0:4'  # Rtg Maicklerstr. (Fellbach Fellbach)
	FRIEDRICH_LIST_STRASSE_1 = 'de:08119:2496:0:3'  # Rtg Rems-Murr-Center (Fellbach Fellbach)
	FRIEDRICH_LIST_STRASSE_2 = 'de:08119:2496:0:4'  # Rtg Maicklerstr. (Fellbach Fellbach)
	FRIEDRICH_LIST_STRASSE_3 = 'de:08119:3673:0:3'  # eRtg Hohrot (Aspach Großaspach)
	FRIEDRICH_LIST_STRASSE_4 = 'de:08119:3673:0:4'  # eRtg Klöpferbach (Aspach Großaspach)
	FRIEDRICH_LIST_STRASSE_5 = 'de:08116:4247:0:3'  # eRtg Max-Eyth-Straße (Wernau (Neckar) Wernau (N))
	FRIEDRICH_LIST_STRASSE_6 = 'de:08116:4247:0:4'  # Rtg Bruckenstr. (Wernau (Neckar) Wernau (N))
	FELLBACH_REMS_MURR_CENTER = 'de:08119:2497'
	FELLBACH_REMS_MURR_CENTER_1 = 'de:08119:2497:0:4'  # Rtg Friedrich-List-Str. (Fellbach Fellbach)
	FELLBACH_REMS_MURR_CENTER_2 = 'de:08119:2497:0:6'  # Ersatz Rtg Zeppelinschule ab 16.02.2016 (Fellbach Fellbach)
	FELLBACH_REMS_MURR_CENTER_3 = 'de:08119:2497:0:7'  # Ersatz 207 Rtg Zeppelin bei McDonalds (Fellbach Fellbach)
	FELLBACH_REMS_MURR_CENTER_4 = 'de:08119:2497:0:8'  # Linie 210 Rtg Beim Wasserturm (Fellbach Fellbach)
	REMS_MURR_CENTER = 'de:08119:2497'
	REMS_MURR_CENTER_1 = 'de:08119:2497:0:4'  # Rtg Friedrich-List-Str. (Fellbach Fellbach)
	REMS_MURR_CENTER_2 = 'de:08119:2497:0:6'  # Ersatz Rtg Zeppelinschule ab 16.02.2016 (Fellbach Fellbach)
	REMS_MURR_CENTER_3 = 'de:08119:2497:0:7'  # Ersatz 207 Rtg Zeppelin bei McDonalds (Fellbach Fellbach)
	REMS_MURR_CENTER_4 = 'de:08119:2497:0:8'  # Linie 210 Rtg Beim Wasserturm (Fellbach Fellbach)
	ZEPPELINSCHULE = 'de:08119:2498'
	ZEPPELINSCHULE_1 = 'de:08119:2498:0:3'  # Rtg Mozartstr./Cannst. Pl./Lutherkirche (Fellbach Fellbach)
	ZEPPELINSCHULE_2 = 'de:08119:2498:0:4'  # Rtg Waibl.-/Sdf Str./Rems-Murr-Center (Fellbach Fellbach)
	ZEPPELINSCHULE_3 = 'de:08119:2498:0:5'  # Rtg Silcherweg (Fellbach Fellbach)
	FELLBACH_ZEPPELINSCHULE = 'de:08119:2498'
	FELLBACH_ZEPPELINSCHULE_1 = 'de:08119:2498:0:3'  # Rtg Mozartstr./Cannst. Pl./Lutherkirche (Fellbach Fellbach)
	FELLBACH_ZEPPELINSCHULE_2 = 'de:08119:2498:0:4'  # Rtg Waibl.-/Sdf Str./Rems-Murr-Center (Fellbach Fellbach)
	FELLBACH_ZEPPELINSCHULE_3 = 'de:08119:2498:0:5'  # Rtg Silcherweg (Fellbach Fellbach)
	KLEINFELDFRIEDHOF = 'de:08119:2499:0:3'  # Rtg Drosselweg (Fellbach Fellbach)
	FELLBACH_KLEINFELDFRIEDHOF = 'de:08119:2499:0:3'  # Rtg Drosselweg (Fellbach Fellbach)
	FELLBACH_DROSSELWEG = 'de:08119:2500:0:3'  # Rtg Altenheim (Fellbach Fellbach)
	PAULUSKIRCHE_1 = 'de:08119:2501:0:3'  # Rtg Mozartstr. (Fellbach Fellbach)
	PAULUSKIRCHE_2 = 'de:08119:2501:0:4'  # Rtg Wernerstr. (Fellbach Fellbach)
	PAULUSKIRCHE_3 = 'de:08117:5:0:H1'  # Rtg Nel Mezzo (Geislingen an der Steige Geislingen (Steige))
	PAULUSKIRCHE_4 = 'de:08117:5:0:R1'  # Rtg Seebach Krankenh. (Geislingen an der Steige Geislingen (Steige))
	FELLBACH_PAULUSKIRCHE = 'de:08119:2501'
	FELLBACH_PAULUSKIRCHE_1 = 'de:08119:2501:0:3'  # Rtg Mozartstr. (Fellbach Fellbach)
	FELLBACH_PAULUSKIRCHE_2 = 'de:08119:2501:0:4'  # Rtg Wernerstr. (Fellbach Fellbach)
	KAPPELBERGSTRASSE = 'de:08119:2502'
	KAPPELBERGSTRASSE_1 = 'de:08119:2502:0:3'  # Rtg Untertürkheimer Str. (Fellbach Fellbach)
	KAPPELBERGSTRASSE_2 = 'de:08119:2502:0:4'  # Rtg Löwenbrunnen (Fellbach Fellbach)
	FELLBACH_KAPPELBERGSTRASSE = 'de:08119:2502'
	FELLBACH_KAPPELBERGSTRASSE_1 = 'de:08119:2502:0:3'  # Rtg Untertürkheimer Str. (Fellbach Fellbach)
	FELLBACH_KAPPELBERGSTRASSE_2 = 'de:08119:2502:0:4'  # Rtg Löwenbrunnen (Fellbach Fellbach)
	UNTERTÜRKHEIMER_STRASSE = 'de:08119:2503'
	UNTERTÜRKHEIMER_STRASSE_1 = 'de:08119:2503:0:3'  # Rtg Alte Kelter (Fellbach Fellbach)
	UNTERTÜRKHEIMER_STRASSE_2 = 'de:08119:2503:0:4'  # Rtg Kappelbergstr. (Fellbach Fellbach)
	FELLB__UNTERTÜRKHEIMER_STRASSE = 'de:08119:2503'
	FELLB__UNTERTÜRKHEIMER_STRASSE_1 = 'de:08119:2503:0:3'  # Rtg Alte Kelter (Fellbach Fellbach)
	FELLB__UNTERTÜRKHEIMER_STRASSE_2 = 'de:08119:2503:0:4'  # Rtg Kappelbergstr. (Fellbach Fellbach)
	ALTE_KELTER = 'de:08119:2504'
	ALTE_KELTER_1 = 'de:08119:2504:0:3'  # Rtg Steigstr./Eugenstr. (Fellbach Fellbach)
	ALTE_KELTER_2 = 'de:08119:2504:0:4'  # Rtg Untertürkheimer Str. (Fellbach Fellbach)
	FELLBACH_ALTE_KELTER = 'de:08119:2504'
	FELLBACH_ALTE_KELTER_1 = 'de:08119:2504:0:3'  # Rtg Steigstr./Eugenstr. (Fellbach Fellbach)
	FELLBACH_ALTE_KELTER_2 = 'de:08119:2504:0:4'  # Rtg Untertürkheimer Str. (Fellbach Fellbach)
	FELLBACH_STEIGSTRASSE = 'de:08119:2505'
	FELLBACH_STEIGSTRASSE_1 = 'de:08119:2505:0:3'  # Rtg Luginsland (Fellbach Fellbach)
	FELLBACH_STEIGSTRASSE_2 = 'de:08119:2505:0:4'  # Rtg Alte Kelter (Fellbach Fellbach)
	STEIGSTRASSE = 'de:08119:2505'
	STEIGSTRASSE_1 = 'de:08119:2505:0:3'  # Rtg Luginsland (Fellbach Fellbach)
	STEIGSTRASSE_2 = 'de:08119:2505:0:4'  # Rtg Alte Kelter (Fellbach Fellbach)
	LUGINSLAND = 'de:08111:2506'
	LUGINSLAND_1 = 'de:08111:2506:0:3'  # Rtg Gehrenwald (Stuttgart Stuttgart)
	LUGINSLAND_2 = 'de:08111:2506:0:4'  # Rtg Fellbach Steigstr. (Stuttgart Stuttgart)
	IM_HAG = 'de:08111:2507'
	IM_HAG_1 = 'de:08111:2507:0:3'  # Rtg Untert. Alter Friedhof (Stuttgart Stuttgart)
	IM_HAG_2 = 'de:08111:2507:0:4'  # Rtg Gehrenwald (Stuttgart Stuttgart)
	ALTE_KRONE = 'de:08111:2508:1:3'  # Rtg Untertürkheim Bf (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_KELTERPLATZ = 'de:08111:2509:1:4'  # L.60 Rtg Im Hag/L.61 Rtg UT Friedhof (Stuttgart Stuttgart)
	ROTENBERG = 'de:08111:2510:0:3'  # Ankunft/Rtg Sonnenbühl (Stuttgart Stuttgart)
	ASPEN_1 = 'de:08111:2511:0:3'  # Rtg Untertürkheim Friedhof (Stuttgart Stuttgart)
	ASPEN_2 = 'de:08111:2511:0:4'  # Rtg Sonnenbühl (Stuttgart Stuttgart)
	ASPEN_3 = 'de:08116:6772:0:3'  # eRtg Hochdorfer Straße (Hochdorf Hochdorf (ES))
	ASPEN_4 = 'de:08116:6772:0:4'  # Rtg Talbachbrücke (Hochdorf Hochdorf (ES))
	UNTERTÜRKHEIM_ALTER_FRIEDHOF = 'de:08111:2512:1:3'  # Rtg Alte Krone (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_FRIEDHOF = 'de:08111:2513'
	UNTERTÜRKHEIM_FRIEDHOF_1 = 'de:08111:2513:0:3'  # Rtg Untertürkheim Alter Friedhof (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_FRIEDHOF_2 = 'de:08111:2513:0:4'  # Rtg Aspen (Stuttgart Stuttgart)
	GARTENSTRASSE_1 = 'de:08119:2515:0:3'  # Rtg Kappelbergstr. (Fellbach Fellbach)
	GARTENSTRASSE_2 = 'de:08119:3660:0:3'  # eRtg Eintracht (Backnang Backnang)
	GARTENSTRASSE_3 = 'de:08119:3660:0:4'  # eRtg Feuerwehr/Burgplatz (Backnang Backnang)
	GARTENSTRASSE_4 = 'de:08116:4411:0:3'  # Rtg Bissingen (Kirchheim unter Teck Nabern)
	GARTENSTRASSE_5 = 'de:08116:4411:0:4'  # Rtg Dettingen (Kirchheim unter Teck Nabern)
	GARTENSTRASSE_6 = 'de:08116:4437:1:3'  # Rtg Balzholzer Str. Neuffen (Beuren Beuren)
	GARTENSTRASSE_7 = 'de:08116:4437:1:4'  # Rtg Bürgerhaus Erkenbrechtsweiler (Beuren Beuren)
	GARTENSTRASSE_8 = 'de:08118:5576:1:3'  # eRtg Sattlerei Eisele (Hessigheim Hessigheim)
	GARTENSTRASSE_9 = 'de:08118:5576:1:4'  # eRtg Mundelsh. Hessigheimer Str. (Hessigheim Hessigheim)
	GARTENSTRASSE_10 = 'de:08118:6949:0:3'  # eRtg Rathaus (Eberdingen Hochdorf (Eberdingen))
	GARTENSTRASSE_11 = 'de:08118:7563:0:3'  # eRtg Arsenalplatz (Ludwigsburg Ludwigsburg)
	GARTENSTRASSE_12 = 'de:08235:2540:0:1'  # Rtg Rathaus (Gechingen Gechingen)
	GARTENSTRASSE_13 = 'de:08235:2540:0:2'  # Rtg Bergwald (Gechingen Gechingen)
	FELLBACH_GARTENSTRASSE = 'de:08119:2515:0:3'  # Rtg Kappelbergstr. (Fellbach Fellbach)
	AUGUST_BRÄNDLE_STRASSE = 'de:08119:2516:0:3'  # Rtg Kappelbergstr. (Fellbach Fellbach)
	FELLBACH_AUGUST_BRÄNDLE_STRASSE = 'de:08119:2516:0:3'  # Rtg Kappelbergstr. (Fellbach Fellbach)
	SILCHERWEG = 'de:08119:2517'
	SILCHERWEG_1 = 'de:08119:2517:0:3'  # Rtg Lutherkirche (Fellbach Fellbach)
	SILCHERWEG_2 = 'de:08119:2517:0:4'  # Rtg Zeppelinschule (Fellbach Fellbach)
	FELLBACH_SILCHERWEG = 'de:08119:2517'
	FELLBACH_SILCHERWEG_1 = 'de:08119:2517:0:3'  # Rtg Lutherkirche (Fellbach Fellbach)
	FELLBACH_SILCHERWEG_2 = 'de:08119:2517:0:4'  # Rtg Zeppelinschule (Fellbach Fellbach)
	HAUS_AM_WEINBERG = 'de:08111:2519'
	HAUS_AM_WEINBERG_1 = 'de:08111:2519:0:3'  # Rtg Ebniseestr. (Stuttgart Stuttgart)
	HAUS_AM_WEINBERG_2 = 'de:08111:2519:0:4'  # Rtg Neckarhalde (Stuttgart Stuttgart)
	BÄCHLENWEG = 'de:08111:2520'
	BÄCHLENWEG_1 = 'de:08111:2520:0:3'  # Rtg Dürrbachstr. (Stuttgart Stuttgart)
	BÄCHLENWEG_2 = 'de:08111:2520:0:4'  # Rtg Ährenweg (Stuttgart Stuttgart)
	DÜRRBACHSTRASSE = 'de:08111:2521'
	DÜRRBACHSTRASSE_1 = 'de:08111:2521:0:3'  # Rtg Rohracker Kelter (Stuttgart Stuttgart)
	DÜRRBACHSTRASSE_2 = 'de:08111:2521:0:4'  # Rtg Bächlenweg (Stuttgart Stuttgart)
	ROHRACKER_KELTER = 'de:08111:2522'
	ROHRACKER_KELTER_1 = 'de:08111:2522:0:3'  # Rtg Luikenweg (Stuttgart Stuttgart)
	ROHRACKER_KELTER_2 = 'de:08111:2522:0:4'  # Rtg Dürrbachstr. (Stuttgart Stuttgart)
	TIEFENBACHSCHULE = 'de:08111:2523'
	TIEFENBACHSCHULE_1 = 'de:08111:2523:0:3'  # Rtg Rohracker (Stuttgart Stuttgart)
	TIEFENBACHSCHULE_2 = 'de:08111:2523:0:4'  # Rtg Luikenweg (Stuttgart Stuttgart)
	ROHRACKER = 'de:08111:2524:0:3'  # Ankunft/Rtg Tiefenbachschule (Stuttgart Stuttgart)
	LUIKENWEG = 'de:08111:2525'
	LUIKENWEG_1 = 'de:08111:2525:0:3'  # Rtg Tiefenbachschule (Stuttgart Stuttgart)
	LUIKENWEG_2 = 'de:08111:2525:0:4'  # Rtg Rohracker Kelter (Stuttgart Stuttgart)
	SCHILDFARNWEG = 'de:08111:2526'
	SCHILDFARNWEG_1 = 'de:08111:2526:0:3'  # Rtg Glaunerweg (Stuttgart Stuttgart)
	SCHILDFARNWEG_2 = 'de:08111:2526:0:4'  # Rtg Lederberg (Stuttgart Stuttgart)
	MÖNCHWEG = 'de:08111:2527:0:4'  # Rtg Untertürkheimer Kelterplatz (Stuttgart Stuttgart)
	SONNENBÜHL = 'de:08111:2528'
	SONNENBÜHL_1 = 'de:08111:2528:0:3'  # Rtg Aspen (Stuttgart Stuttgart)
	SONNENBÜHL_2 = 'de:08111:2528:0:4'  # Rtg Rotenberg (Stuttgart Stuttgart)
	UHLBACH = 'de:08111:2530:0:3'  # Ankunft/Rtg Terlanerstr. (Stuttgart Stuttgart)
	TERLANERSTRASSE = 'de:08111:2531'
	TERLANERSTRASSE_1 = 'de:08111:2531:0:3'  # Rtg Asang (Stuttgart Stuttgart)
	TERLANERSTRASSE_2 = 'de:08111:2531:0:4'  # Rtg Uhlbach (Stuttgart Stuttgart)
	ASANG = 'de:08111:2532'
	ASANG_1 = 'de:08111:2532:0:3'  # Rtg Im Mäder (Stuttgart Stuttgart)
	ASANG_2 = 'de:08111:2532:0:4'  # Rtg Terlanerstr. (Stuttgart Stuttgart)
	IM_MÄDER = 'de:08111:2533'
	IM_MÄDER_1 = 'de:08111:2533:0:3'  # Rtg Obertürkh. Bf (Göpp.Str) (Stuttgart Stuttgart)
	IM_MÄDER_2 = 'de:08111:2533:0:4'  # Rtg Asang (Stuttgart Stuttgart)
	OTTO_HIRSCH_BRÜCKEN = 'de:08111:2534'
	OTTO_HIRSCH_BRÜCKEN_1 = 'de:08111:2534:0:3'  # Rtg Hedelfingen (Stuttgart Stuttgart)
	OTTO_HIRSCH_BRÜCKEN_2 = 'de:08111:2534:0:4'  # Rtg Obertürkheim Bf (Stuttgart Stuttgart)
	ÄHRENWEG = 'de:08111:2535'
	ÄHRENWEG_1 = 'de:08111:2535:0:3'  # L.62 Rtg Bächlenweg/L.65 Rtg Lederberg (Stuttgart Stuttgart)
	ÄHRENWEG_2 = 'de:08111:2535:0:4'  # Rtg Hedelfingen (Stuttgart Stuttgart)
	LEDERBERG = 'de:08111:2536'
	LEDERBERG_1 = 'de:08111:2536:0:3'  # Rtg Schildfarnweg (Stuttgart Stuttgart)
	LEDERBERG_2 = 'de:08111:2536:0:4'  # Rtg Ährenweg (Stuttgart Stuttgart)
	GLAUNERWEG = 'de:08111:2537'
	GLAUNERWEG_1 = 'de:08111:2537:0:3'  # Rtg Heumaden Rose (Stuttgart Stuttgart)
	GLAUNERWEG_2 = 'de:08111:2537:0:4'  # Rtg Schildfarnweg (Stuttgart Stuttgart)
	HEUMADEN_ROSE = 'de:08111:2538'
	HEUMADEN_ROSE_1 = 'de:08111:2538:0:3'  # Rtg Heumaden Schule (Stuttgart Stuttgart)
	HEUMADEN_ROSE_2 = 'de:08111:2538:0:4'  # Rtg Glaunerweg (Stuttgart Stuttgart)
	HEUMADEN_SCHULE = 'de:08111:2539'
	HEUMADEN_SCHULE_1 = 'de:08111:2539:0:3'  # Rtg Isegrimweg (Stuttgart Stuttgart)
	HEUMADEN_SCHULE_2 = 'de:08111:2539:0:4'  # Rtg Heumaden Schule (Stuttgart Stuttgart)
	FRAUENKOPF = 'de:08111:2540:0:3'  # Ankunft/Rtg Eselweg (Stuttgart Stuttgart)
	ESELWEG = 'de:08111:2541'
	ESELWEG_1 = 'de:08111:2541:0:3'  # Rtg Frauenkopf (Stuttgart Stuttgart)
	ESELWEG_2 = 'de:08111:2541:0:4'  # Rtg Filderblickweg (Stuttgart Stuttgart)
	FILDERBLICKWEG = 'de:08111:2542'
	FILDERBLICKWEG_1 = 'de:08111:2542:0:3'  # Rtg Eselweg (Stuttgart Stuttgart)
	FILDERBLICKWEG_2 = 'de:08111:2542:0:4'  # Rtg Stelle (Stuttgart Stuttgart)
	OBERTÜRKH__BF__GÖPPINGER_STR = 'de:08111:2543:1:4'  # Rtg Bergstaffelstr./O.-Hirsch-Brücken (Stuttgart Stuttgart)
	AM_BERGWALD = 'de:08111:2544'
	AM_BERGWALD_1 = 'de:08111:2544:0:3'  # Rtg Lederberg (Stuttgart Stuttgart)
	AM_BERGWALD_2 = 'de:08111:2544:0:4'  # Rtg Ährenweg (Stuttgart Stuttgart)
	BIRKACHER_STRASSE = 'de:08111:2545'
	BIRKACHER_STRASSE_1 = 'de:08111:2545:0:3'  # Rtg Rotwiesenstr. (Stuttgart Stuttgart)
	BIRKACHER_STRASSE_2 = 'de:08111:2545:0:4'  # Rtg Riedenberg (Stuttgart Stuttgart)
	AUGUSTINUM = 'de:08111:2546'
	AUGUSTINUM_1 = 'de:08111:2546:0:3'  # Rtg Steinäcker (Stuttgart Stuttgart)
	AUGUSTINUM_2 = 'de:08111:2546:0:4'  # Rtg Spaichinger Str. (Stuttgart Stuttgart)
	STEINÄCKER = 'de:08111:2547'
	STEINÄCKER_1 = 'de:08111:2547:0:3'  # Rtg Riedenberg (Stuttgart Stuttgart)
	STEINÄCKER_2 = 'de:08111:2547:0:4'  # Rtg Augustinum (Stuttgart Stuttgart)
	RIEDENBERG = 'de:08111:2548'
	RIEDENBERG_1 = 'de:08111:2548:0:3'  # Rtg Birkacher Str. (Stuttgart Stuttgart)
	RIEDENBERG_2 = 'de:08111:2548:0:4'  # Rtg Steinäcker (Stuttgart Stuttgart)
	FRAUBRONNSTRASSE = 'de:08111:2553'
	FRAUBRONNSTRASSE_1 = 'de:08111:2553:0:3'  # Rtg Plieningen Post (Stuttgart Stuttgart)
	FRAUBRONNSTRASSE_2 = 'de:08111:2553:0:4'  # Rtg Windhalmweg (Stuttgart Stuttgart)
	WINDHALMWEG = 'de:08111:2554'
	WINDHALMWEG_1 = 'de:08111:2554:0:3'  # Rtg Fraubronnstr. (Stuttgart Stuttgart)
	WINDHALMWEG_2 = 'de:08111:2554:0:4'  # Rtg Plieningen Garbe (Stuttgart Stuttgart)
	UNIVERSITÄT_HOHENHEIM = 'de:08111:2555'
	UNIVERSITÄT_HOHENHEIM_1 = 'de:08111:2555:0:3'  # Rtg Plieningen Garbe (Stuttgart Stuttgart)
	UNIVERSITÄT_HOHENHEIM_2 = 'de:08111:2555:0:4'  # Rtg Fruwirthstr. (Stuttgart Stuttgart)
	FRUWIRTHSTRASSE = 'de:08111:2556'
	FRUWIRTHSTRASSE_1 = 'de:08111:2556:0:3'  # Rtg Universität Hohenheim (Stuttgart Stuttgart)
	FRUWIRTHSTRASSE_2 = 'de:08111:2556:0:4'  # Rtg Birkach Friedhof (Stuttgart Stuttgart)
	BIRKACH_FRIEDHOF = 'de:08111:2557'
	BIRKACH_FRIEDHOF_1 = 'de:08111:2557:0:3'  # Rtg Fruwirthstr. (Stuttgart Stuttgart)
	BIRKACH_FRIEDHOF_2 = 'de:08111:2557:0:4'  # Rtg Birkheckenstr. (Stuttgart Stuttgart)
	BIRKHECKENSTRASSE = 'de:08111:2558'
	BIRKHECKENSTRASSE_1 = 'de:08111:2558:1:3'  # Rtg Birkach Friedhof (Stuttgart Stuttgart)
	BIRKHECKENSTRASSE_2 = 'de:08111:2558:2:4'  # Rtg Dürnauer Weg/Birkach West (Stuttgart Stuttgart)
	BIRKACH_WEST = 'de:08111:2559'
	BIRKACH_WEST_1 = 'de:08111:2559:0:3'  # Rtg Birkheckenstr. (Stuttgart Stuttgart)
	BIRKACH_WEST_2 = 'de:08111:2559:0:4'  # Rtg Lerchenwiesen (Stuttgart Stuttgart)
	LERCHENWIESEN = 'de:08111:2560'
	LERCHENWIESEN_1 = 'de:08111:2560:0:3'  # Rtg Birkach West (Stuttgart Stuttgart)
	LERCHENWIESEN_2 = 'de:08111:2560:0:4'  # Rtg Rotwiesenstr. (Stuttgart Stuttgart)
	ROTWIESENSTRASSE = 'de:08111:2561'
	ROTWIESENSTRASSE_1 = 'de:08111:2561:0:1'  # Rtg Grünlingweg (Stuttgart Stuttgart)
	ROTWIESENSTRASSE_2 = 'de:08111:2561:0:2'  # Rtg Lerchenwiesen (Stuttgart Stuttgart)
	ROTWIESENSTRASSE_3 = 'de:08111:2561:0:3'  # Rtg Grünlingweg/Rotwiesenstr. (Stuttgart Stuttgart)
	GRÜNLINGWEG = 'de:08111:2562'
	GRÜNLINGWEG_1 = 'de:08111:2562:0:3'  # Rtg Rotwiesenstr. (Stuttgart Stuttgart)
	GRÜNLINGWEG_2 = 'de:08111:2562:0:4'  # Rtg Kolpingsiedlung (Stuttgart Stuttgart)
	KOLPINGSIEDLUNG = 'de:08111:2563'
	KOLPINGSIEDLUNG_1 = 'de:08111:2563:0:3'  # Rtg Grünlingweg (Stuttgart Stuttgart)
	KOLPINGSIEDLUNG_2 = 'de:08111:2563:0:4'  # Rtg Ruhbank (Fernsehturm) (Stuttgart Stuttgart)
	FERNSEHTURM = 'de:08111:2564:0:3'  # Rtg Ruhbank (Fernsehturm) (Stuttgart Stuttgart)
	KÖNIGSTRÄSSLE = 'de:08111:2565'
	KÖNIGSTRÄSSLE_1 = 'de:08111:2565:1:3'  # Rtg Fernsehturm (Stuttgart Stuttgart)
	KÖNIGSTRÄSSLE_2 = 'de:08111:2565:1:4'  # Rtg Zahnradbf/ZOB (Stuttgart Stuttgart)
	LEINFELDENER_STRASSE = 'de:08111:2566'
	LEINFELDENER_STRASSE_1 = 'de:08111:2566:0:3'  # Rtg Wurmlinger Str. (Stuttgart Stuttgart)
	LEINFELDENER_STRASSE_2 = 'de:08111:2566:0:4'  # Rtg Reutlinger Str. (Stuttgart Stuttgart)
	WURMLINGER_STRASSE = 'de:08111:2567:0:4'  # Rtg Epplestr. (Stuttgart Stuttgart)
	ZUR_ANHÖHE = 'de:08111:2568'
	ZUR_ANHÖHE_1 = 'de:08111:2568:0:3'  # Rtg Reutlinger Str. (Stuttgart Stuttgart)
	ZUR_ANHÖHE_2 = 'de:08111:2568:0:4'  # Rtg Hoffeld (Stuttgart Stuttgart)
	HOFFELD = 'de:08111:2569'
	HOFFELD_1 = 'de:08111:2569:0:3'  # Rtg Zur Anhöhe (Stuttgart Stuttgart)
	HOFFELD_2 = 'de:08111:2569:0:4'  # Ankunft (Stuttgart Stuttgart)
	DINKELSTRASSE__KARLSHOF = 'de:08111:2572'
	DINKELSTRASSE__KARLSHOF_1 = 'de:08111:2572:0:3'  # Rtg Asemwald (Stuttgart Stuttgart)
	DINKELSTRASSE__KARLSHOF_2 = 'de:08111:2572:0:4'  # Rtg Hohe Eiche (Stuttgart Stuttgart)
	ASEMWALD = 'de:08111:2573'
	ASEMWALD_1 = 'de:08111:2573:0:3'  # Rtg Dürnauer Weg (Stuttgart Stuttgart)
	ASEMWALD_2 = 'de:08111:2573:0:4'  # Rtg Kaserne Möhringen (Stuttgart Stuttgart)
	TALDORFER_STRASSE = 'de:08111:2574:0:3'  # Rtg Lerchenwiesen (Stuttgart Stuttgart)
	SCHÖNBERG_1 = 'de:08111:2575:1:3'  # Rtg Taldorfer Str. (Stuttgart Stuttgart)
	SCHÖNBERG_2 = 'de:08115:4763:0:3'  # eRtg Mittlere Mühle (Holzgerlingen Holzgerlingen)
	SCHÖNBERG_3 = 'de:08115:4763:0:4'  # eRtg Birkenstraße (Holzgerlingen Holzgerlingen)
	DÜRNAUER_WEG = 'de:08111:2576'
	DÜRNAUER_WEG_1 = 'de:08111:2576:0:3'  # Rtg Birkheckenstr. (Stuttgart Stuttgart)
	DÜRNAUER_WEG_2 = 'de:08111:2576:0:4'  # Rtg Asemwald (Stuttgart Stuttgart)
	IM_ELSENTAL = 'de:08111:2578'
	IM_ELSENTAL_1 = 'de:08111:2578:0:3'  # Rtg Dachswaldweg (Stuttgart Stuttgart)
	IM_ELSENTAL_2 = 'de:08111:2578:0:4'  # Rtg Waldeck (Stuttgart Stuttgart)
	OB_DEM_STEINBACH = 'de:08111:2579:0:3'  # Rtg Max-Planck-Institute (Stuttgart Stuttgart)
	JOHANNESGRABEN = 'de:08111:2580'
	JOHANNESGRABEN_1 = 'de:08111:2580:0:3'  # Rtg Alter Friedhof (Stuttgart Stuttgart)
	JOHANNESGRABEN_2 = 'de:08111:2580:0:4'  # Rtg Schranne (Stuttgart Stuttgart)
	KURT_SCHUMACHER_STRASSE = 'de:08111:2581'
	KURT_SCHUMACHER_STRASSE_1 = 'de:08111:2581:0:3'  # Rtg Fasanenhof (Stuttgart Stuttgart)
	KURT_SCHUMACHER_STRASSE_2 = 'de:08111:2581:0:4'  # Rtg Lohäckerstr. (Stuttgart Stuttgart)
	BERUFSBILDUNGSZENTRUM = 'de:08111:2582'
	BERUFSBILDUNGSZENTRUM_1 = 'de:08111:2582:0:3'  # Rtg Vaihingen Bf (Ost) (Stuttgart Stuttgart)
	BERUFSBILDUNGSZENTRUM_2 = 'de:08111:2582:0:5'  # NBus (Stuttgart Stuttgart)
	MÖHRINGEN_RATHAUS = 'de:08111:2583'
	MÖHRINGEN_RATHAUS_1 = 'de:08111:2583:0:3'  # Rtg Jelinstr. (Stuttgart Stuttgart)
	MÖHRINGEN_RATHAUS_2 = 'de:08111:2583:0:4'  # Rtg Möhringen Bf (Stuttgart Stuttgart)
	EUROPAPLATZ = 'de:08111:2584'
	EUROPAPLATZ_1 = 'de:08111:2584:1:3'  # N9 Rtg Landhaus /Ankunft 72 (Stuttgart Stuttgart)
	EUROPAPLATZ_2 = 'de:08111:2584:1:4'  # Rtg Freytagweg Ausgleichszeitpos. (Stuttgart Stuttgart)
	EUROPAPLATZ_3 = 'de:08111:2584:2:1'  # Rtg EnBW City (Stuttgart Stuttgart)
	EUROPAPLATZ_4 = 'de:08111:2584:2:2'  # Rtg Fasanenhof (Stuttgart Stuttgart)
	JELINSTRASSE = 'de:08111:2585'
	JELINSTRASSE_1 = 'de:08111:2585:0:3'  # Rtg Dornröschenweg (Stuttgart Möhringen)
	JELINSTRASSE_2 = 'de:08111:2585:0:4'  # Rtg Möhringen Rathaus (Stuttgart Möhringen)
	ENBW_CITY = 'de:08111:2586'
	ENBW_CITY_1 = 'de:08111:2586:1:1'  # Rtg Schelmenwasen (Stuttgart Stuttgart)
	ENBW_CITY_2 = 'de:08111:2586:1:2'  # Rtg Europaplatz (Stuttgart Stuttgart)
	ENBW_CITY_3 = 'de:08111:2586:2:3'  # Linien 77/806 Rtg Fasanenhof Abzw. (Stuttgart Stuttgart)
	HEIGELINSTRASSE = 'de:08111:2587:0:5'  # X7 Rtg Harthausen (Stuttgart Stuttgart)
	MAX_PLANCK_INSTITUTE = 'de:08111:2589'
	MAX_PLANCK_INSTITUTE_1 = 'de:08111:2589:0:3'  # Rtg Lauchhau (Stuttgart Stuttgart)
	MAX_PLANCK_INSTITUTE_2 = 'de:08111:2589:0:4'  # Rtg Am Schattwald (Stuttgart Stuttgart)
	BÜSNAUER_PLATZ = 'de:08111:2590:2:3'  # Rtg Ob dem Steinbach (Stuttgart Stuttgart)
	LAUCHHAU = 'de:08111:2591'
	LAUCHHAU_1 = 'de:08111:2591:0:1'  # Rtg Nobelstr./Am Feldrand (Stuttgart Stuttgart)
	LAUCHHAU_2 = 'de:08111:2591:0:2'  # Rtg Max-Planck-Institute (Stuttgart Stuttgart)
	LAUCHHAU_3 = 'de:08111:2591:0:3'  # Lauchhau Endhaltestelle (Stuttgart Stuttgart)
	AM_FELDRAND = 'de:08111:2592'
	AM_FELDRAND_1 = 'de:08111:2592:0:3'  # Rtg Katzenbachstr. (Stuttgart Stuttgart)
	AM_FELDRAND_2 = 'de:08111:2592:0:4'  # Rtg Lauchhau (Stuttgart Stuttgart)
	HEERSTRASSE_1 = 'de:08111:2593:0:3'  # Rtg Krehlstr. (Stuttgart Stuttgart)
	HEERSTRASSE_2 = 'de:08111:2593:0:4'  # Rtg Katzenbachstr. (Stuttgart Stuttgart)
	HEERSTRASSE_3 = 'de:08115:3219:0:3'  # Rtg Holzsteig (Herrenberg Gültstein)
	HEERSTRASSE_4 = 'de:08115:3219:0:4'  # Rtg Erholungsheim (Herrenberg Gültstein)
	DEGERLOCH_ALBSTRASSE = 'de:08111:2594'
	DEGERLOCH_ALBSTRASSE_1 = 'de:08111:2594:2:1'  # Rtg Peregrinastr. (Stuttgart Stuttgart)
	DEGERLOCH_ALBSTRASSE_2 = 'de:08111:2594:2:2'  # Rtg Degerloch (Stuttgart Stuttgart)
	VAIHINGEN_RATHAUS = 'de:08111:2595'
	VAIHINGEN_RATHAUS_1 = 'de:08111:2595:0:3'  # Rtg Schillerplatz (Stuttgart Stuttgart)
	VAIHINGEN_RATHAUS_2 = 'de:08111:2595:0:4'  # Rtg Kreuzstr. (Stuttgart Stuttgart)
	AM_SCHATTWALD = 'de:08111:2596'
	AM_SCHATTWALD_1 = 'de:08111:2596:0:3'  # Rtg Büsnauer Platz (Stuttgart Stuttgart)
	AM_SCHATTWALD_2 = 'de:08111:2596:0:4'  # Rtg Büsn. Pl./Feldbergstr./Schattengrund (Stuttgart Stuttgart)
	SYNERGIEPARK = 'de:08111:2598'
	SYNERGIEPARK_1 = 'de:08111:2598:0:3'  # Rtg LappKabel (Stuttgart Stuttgart)
	SYNERGIEPARK_2 = 'de:08111:2598:0:4'  # Rtg Vaih. Bf Ost (Stuttgart Stuttgart)
	LAPP_KABEL = 'de:08111:2599'
	LAPP_KABEL_1 = 'de:08111:2599:1:1'  # Rtg Dürrlewang (Stuttgart Stuttgart)
	LAPP_KABEL_2 = 'de:08111:2599:1:2'  # Rtg Wallgraben (Stuttgart Stuttgart)
	LAPP_KABEL_3 = 'de:08111:2599:2:7'  # Rtg Gewerbestr.(LK) (Stuttgart Stuttgart)
	SCHULZE_DELITZSCH_STR = 'de:08111:2600:0:3'  # Rtg Handwerkstr. (Stuttgart Stuttgart)
	SCHULZE_DELITZSCH_STRASSE = 'de:08111:2600:0:3'  # Rtg Handwerkstr. (Stuttgart Stuttgart)
	INDUSTRIESTRASSE_1 = 'de:08111:2601:0:3'  # Rtg Berufsbildungszentrum (Stuttgart Stuttgart)
	INDUSTRIESTRASSE_2 = 'de:08119:4972:0:3'  # Rtg Backnang (Backnang Backnang)
	INDUSTRIESTRASSE_3 = 'de:08119:4972:0:4'  # Rtg Weissach (Backnang Backnang)
	INDUSTRIESTRASSE_4 = 'de:08115:5839:0:3'  # eRtg Beethovenstraße (Weil der Stadt Merklingen)
	INDUSTRIESTRASSE_5 = 'de:08115:5839:0:4'  # eRtg Hauptstraße (Weil der Stadt Merklingen)
	INDUSTRIESTRASSE_6 = 'de:08118:7415:0:3'  # Rtg Kleinglattbach (Sersheim Sersheim)
	INDUSTRIESTRASSE_7 = 'de:08118:7415:0:4'  # Rtg Sersheim (Sersheim Sersheim)
	VAIHINGEN_ALTER_FRIEDHOF = 'de:08111:2602'
	VAIHINGEN_ALTER_FRIEDHOF_1 = 'de:08111:2602:0:3'  # Rtg Kreuzstr. (Stuttgart Stuttgart)
	VAIHINGEN_ALTER_FRIEDHOF_2 = 'de:08111:2602:0:4'  # Rtg Johannesgraben (Stuttgart Stuttgart)
	SCHRANNE = 'de:08111:2603'
	SCHRANNE_1 = 'de:08111:2603:0:3'  # Rtg Nobelstr. (Stuttgart Stuttgart)
	SCHRANNE_2 = 'de:08111:2603:0:4'  # Rtg Universität (Stuttgart Stuttgart)
	KNAPPENWEG = 'de:08111:2604'
	KNAPPENWEG_1 = 'de:08111:2604:0:3'  # Rtg Universität (Stuttgart Stuttgart)
	KNAPPENWEG_2 = 'de:08111:2604:0:4'  # Rtg Dachswald (Stuttgart Stuttgart)
	DACHSWALD = 'de:08111:2605'
	DACHSWALD_1 = 'de:08111:2605:0:3'  # Rtg Knappenweg (Stuttgart Stuttgart)
	DACHSWALD_2 = 'de:08111:2605:0:4'  # Rtg Dachswaldweg (Stuttgart Stuttgart)
	DACHSWALDWEG = 'de:08111:2609'
	DACHSWALDWEG_1 = 'de:08111:2609:0:3'  # Rtg Dachswald (Stuttgart Stuttgart)
	DACHSWALDWEG_2 = 'de:08111:2609:0:4'  # Rtg Im Elsental (Stuttgart Stuttgart)
	HANS_REHN_STIFT = 'de:08111:2610:0:3'  # Rtg Am Ochsenwald (Stuttgart Stuttgart)
	THINGSTRASSE = 'de:08111:2611'
	THINGSTRASSE_1 = 'de:08111:2611:0:3'  # Rtg Rohr Mitte (Stuttgart Stuttgart)
	THINGSTRASSE_2 = 'de:08111:2611:0:4'  # Rtg Rohrer Höhe (Stuttgart Stuttgart)
	OSTERBRONNSTRASSE = 'de:08111:2612'
	OSTERBRONNSTRASSE_1 = 'de:08111:2612:0:3'  # Rtg Dürrlewang (Stuttgart Stuttgart)
	OSTERBRONNSTRASSE_2 = 'de:08111:2612:0:4'  # Rtg Rohr Bf (Stuttgart Stuttgart)
	DÜRRLEWANG = 'de:08111:2613'
	DÜRRLEWANG_1 = 'de:08111:2613:0:3'  # Rtg Lambertweg (Stuttgart Stuttgart)
	DÜRRLEWANG_2 = 'de:08111:2613:0:4'  # Rtg Osterbronnstr. (Stuttgart Stuttgart)
	DÜRRLEWANG_3 = 'de:08111:2613:0:5'  # NBus (Stuttgart Stuttgart)
	DÜRRLEWANG_4 = 'de:08111:2613:1:1'  # Endhaltestelle (Stuttgart Stuttgart)
	LAMBERTWEG = 'de:08111:2614:0:3'  # Ankunft und Rtg Dürrlewang (Stuttgart Stuttgart)
	AM_OCHSENWALD = 'de:08111:2615'
	AM_OCHSENWALD_1 = 'de:08111:2615:0:1'  # Rtg Rohrer Höhe/H.-Rehn-Stift (Stuttgart Stuttgart)
	AM_OCHSENWALD_2 = 'de:08111:2615:0:2'  # Rtg Albblick/H.-Rehn-Stift (Stuttgart Stuttgart)
	AM_OCHSENWALD_3 = 'de:08111:2615:0:3'  # Rtg Rohrer Höhe/Albblick (Stuttgart Stuttgart)
	ROHRER_HÖHE = 'de:08111:2616'
	ROHRER_HÖHE_1 = 'de:08111:2616:0:3'  # Rtg Thingstraße (Stuttgart Stuttgart)
	ROHRER_HÖHE_2 = 'de:08111:2616:0:4'  # Rtg Am Ochsenwald (Stuttgart Stuttgart)
	HOLDERÄCKER = 'de:08111:2617'
	HOLDERÄCKER_1 = 'de:08111:2617:0:3'  # Rtg Generatorstr./WID Bf (Unterf.) (Stuttgart Stuttgart)
	HOLDERÄCKER_2 = 'de:08111:2617:0:4'  # Rtg Flachter Str. (Stuttgart Stuttgart)
	HAUSENRING = 'de:08111:2619'
	HAUSENRING_1 = 'de:08111:2619:0:3'  # Rtg Gerlinger Str. (Stuttgart Stuttgart)
	HAUSENRING_2 = 'de:08111:2619:0:4'  # Rtg Mönsheimer Weg (Stuttgart Stuttgart)
	DITZINGER_STRASSE = 'de:08111:2621'
	DITZINGER_STRASSE_1 = 'de:08111:2621:0:3'  # Rtg Weilemer Pfad (Stuttgart Stuttgart)
	DITZINGER_STRASSE_2 = 'de:08111:2621:0:4'  # Rtg Weilimdorf Bf (Stuttgart Stuttgart)
	FLACHTER_STRASSE_1 = 'de:08111:2623:0:3'  # Rtg Holderäcker (Stuttgart Stuttgart)
	FLACHTER_STRASSE_2 = 'de:08111:2623:0:4'  # Rtg Roßbachstr. (Stuttgart Stuttgart)
	FLACHTER_STRASSE_3 = 'de:08115:4568:0:3'  # eRtg Strudelbachhalle (Weissach Weissach (BB))
	FLACHTER_STRASSE_4 = 'de:08115:4568:0:4'  # Rtg Flacht Hohweg (Weissach Weissach (BB))
	WEILIMDORF_BF__UNTERFÜHRUNG = 'de:08111:2624'
	WEILIMDORF_BF__UNTERFÜHRUNG_1 = 'de:08111:2624:1:3'  # Rtg Bahnhof (Stuttgart Stuttgart)
	WEILIMDORF_BF__UNTERFÜHRUNG_2 = 'de:08111:2624:1:4'  # Rtg Hemminger Str./Holderäcker (Stuttgart Stuttgart)
	KORNTAL_BERGSTRASSE = 'de:08118:2626'
	KORNTAL_BERGSTRASSE_1 = 'de:08118:2626:0:3'  # Rtg Tachenbergstr./Jahnstr. (Korntal-Münchingen Korntal)
	KORNTAL_BERGSTRASSE_2 = 'de:08118:2626:0:4'  # Rtg Bahnhof (Warthstr) (Korntal-Münchingen Korntal)
	BERGSTRASSE_1 = 'de:08118:2626:0:3'  # Rtg Tachenbergstr./Jahnstr. (Korntal-Münchingen Korntal)
	BERGSTRASSE_2 = 'de:08118:2626:0:4'  # Rtg Bahnhof (Warthstr) (Korntal-Münchingen Korntal)
	BERGSTRASSE_3 = 'de:08116:4118:0:3'  # eRtg Kelter (Esslingen am Neckar Sulzgries)
	BERGSTRASSE_4 = 'de:08116:4118:0:4'  # eRtg Zentrum (Esslingen am Neckar Sulzgries)
	BERGSTRASSE_5 = 'de:08116:4246:0:3'  # eRtg Bruckenstraße (Wernau (Neckar) Wernau (N))
	BERGSTRASSE_6 = 'de:08117:4040:0:H1'  # Rtg Wangener Straße (Rechberghausen Rechberghausen)
	BERGSTRASSE_7 = 'de:08117:4040:0:R1'  # Rtg Reichenhardtstraße (Rechberghausen Rechberghausen)
	BERGSTRASSE_8 = 'de:08136:7009:0:H1'  # Bergstraße (Lorch Lorch)
	KORNTAL_STADTHALLE = 'de:08118:2627:0:3'  # Rtg Hauffstr. (Korntal-Münchingen Korntal)
	TACHENBERGSTRASSE = 'de:08118:2628'
	TACHENBERGSTRASSE_1 = 'de:08118:2628:0:3'  # Rtg Greutterstr. (Korntal-Münchingen Korntal)
	TACHENBERGSTRASSE_2 = 'de:08118:2628:0:4'  # Rtg Bergstr. (Korntal-Münchingen Korntal)
	KORNTAL_TACHENBERGSTRASSE = 'de:08118:2628'
	KORNTAL_TACHENBERGSTRASSE_1 = 'de:08118:2628:0:3'  # Rtg Greutterstr. (Korntal-Münchingen Korntal)
	KORNTAL_TACHENBERGSTRASSE_2 = 'de:08118:2628:0:4'  # Rtg Bergstr. (Korntal-Münchingen Korntal)
	FEHRBELLINER_STR = 'de:08111:2629'
	FEHRBELLINER_STR_1 = 'de:08111:2629:0:3'  # Rtg Löwenmarkt (Stuttgart Stuttgart)
	FEHRBELLINER_STR_2 = 'de:08111:2629:0:4'  # Rtg Greutterstr. (Stuttgart Stuttgart)
	GREUTTERSTRASSE = 'de:08111:2630'
	GREUTTERSTRASSE_1 = 'de:08111:2630:0:3'  # Rtg Fehrbelliner Str. (Stuttgart Stuttgart)
	GREUTTERSTRASSE_2 = 'de:08111:2630:0:4'  # Rtg Tachenbergstr. (Stuttgart Stuttgart)
	BAHNHOF__WARTHSTRASSE = 'de:08118:2631:1:3'  # Rtg Stadthalle (Korntal-Münchingen Korntal)
	KORNTAL_BF__WARTHSTR = 'de:08118:2631:1:3'  # Rtg Stadthalle (Korntal-Münchingen Korntal)
	HAUFFSTRASSE_1 = 'de:08118:2632:0:3'  # Rtg Bahnhof (Korntal-Münchingen Korntal)
	HAUFFSTRASSE_2 = 'de:08116:2952:0:3'  # eRtg Nürtinger Straße (Nürtingen Oberensingen)
	HAUFFSTRASSE_3 = 'de:08116:2952:0:4'  # eRtg Grötzinger Straße (Nürtingen Oberensingen)
	HAUFFSTRASSE_4 = 'de:08118:3414:1:3'  # eRtg Stadion (Kornwestheim Kornwestheim)
	HAUFFSTRASSE_5 = 'de:08117:1201:0:R1'  # Rtg Reichenhardtstraße (Rechberghausen Rechberghausen)
	KORNTAL_HAUFFSTRASSE = 'de:08118:2632:0:3'  # Rtg Bahnhof (Korntal-Münchingen Korntal)
	MÖNSHEIMER_WEG = 'de:08111:2635'
	MÖNSHEIMER_WEG_1 = 'de:08111:2635:0:3'  # Rtg Hausenring (Stuttgart Stuttgart)
	MÖNSHEIMER_WEG_2 = 'de:08111:2635:0:4'  # Weilemer Pfad (Stuttgart Stuttgart)
	JAHNSTRASSE_1 = 'de:08118:2636:0:3'  # Rtg Bergstr. (Korntal-Münchingen Korntal)
	JAHNSTRASSE_2 = 'de:08118:2636:0:4'  # Rtg Borkumstr.(Nwh.sdlg.) (Korntal-Münchingen Korntal)
	JAHNSTRASSE_3 = 'de:08116:4394:0:3'  # Rtg Maierhöfe (Weilheim an der Teck Weilheim (T))
	JAHNSTRASSE_4 = 'de:08116:4394:0:4'  # Rtg Schluchtweg (Weilheim an der Teck Weilheim (T))
	JAHNSTRASSE_5 = 'de:08118:5581:1:3'  # eRtg Auf dem Kies (Besigheim Besigheim)
	JAHNSTRASSE_6 = 'de:08118:5581:1:4'  # eRtg Freibad/FitKom (Besigheim Besigheim)
	JAHNSTRASSE_7 = 'de:08118:5581:1:5'  # eRtg Schülerbussteig Rtg Auf dem Kiess (Besigheim Besigheim)
	JAHNSTRASSE_8 = 'de:08118:6985:0:3'  # eRtg Friedhof (Ludwigsburg Poppenweiler)
	JAHNSTRASSE_9 = 'de:08118:6985:0:4'  # eRtg Poppenweiler Straße (Ludwigsburg Poppenweiler)
	JAHNSTRASSE_10 = 'de:08415:29042:0:1'  # Rtg Dettenhausen (Walddorfhäslach Walddorf)
	JAHNSTRASSE_11 = 'de:08415:29042:0:2'  # Rtg Rathaus (Walddorfhäslach Walddorf)
	KORNTAL_JAHNSTRASSE = 'de:08118:2636'
	KORNTAL_JAHNSTRASSE_1 = 'de:08118:2636:0:3'  # Rtg Bergstr. (Korntal-Münchingen Korntal)
	KORNTAL_JAHNSTRASSE_2 = 'de:08118:2636:0:4'  # Rtg Borkumstr.(Nwh.sdlg.) (Korntal-Münchingen Korntal)
	WEILEMER_PFAD = 'de:08111:2637'
	WEILEMER_PFAD_1 = 'de:08111:2637:0:3'  # Rtg Mönsheimer Weg (Stuttgart Stuttgart)
	WEILEMER_PFAD_2 = 'de:08111:2637:0:4'  # Rtg Ditzinger Str. (Stuttgart Stuttgart)
	MÜNCHINGEN_KALLENBERG_KIRCHE = 'de:08118:2638:0:3'  # Rtg Kallenberg Lackfabrik (612A) (Korntal-Münchingen Münchingen)
	FELLBACH_MAICKLERSTRASSE = 'de:08119:2639'
	FELLBACH_MAICKLERSTRASSE_1 = 'de:08119:2639:0:3'  # Rtg Friedrich-List-Str. (Fellbach Fellbach)
	FELLBACH_MAICKLERSTRASSE_2 = 'de:08119:2639:0:4'  # Rtg Wernerstr. (Fellbach Fellbach)
	MAICKLERSTRASSE = 'de:08119:2639'
	MAICKLERSTRASSE_1 = 'de:08119:2639:0:3'  # Rtg Friedrich-List-Str. (Fellbach Fellbach)
	MAICKLERSTRASSE_2 = 'de:08119:2639:0:4'  # Rtg Wernerstr. (Fellbach Fellbach)
	WERNERSTRASSE_1 = 'de:08119:2640:0:3'  # L.60 Rtg Pauluskirche/L.67 Rtg Maicklers (Fellbach Fellbach)
	WERNERSTRASSE_2 = 'de:08119:2640:0:4'  # Rtg Bahnhof (Fellbach Fellbach)
	WERNERSTRASSE_3 = 'de:08118:7641:0:3'  # Rtg Hoferstraße ZOB (Ludwigsburg Ludwigsburg)
	WERNERSTRASSE_4 = 'de:08118:7641:0:4'  # Rtg Osterholzallee Eglosheim (Ludwigsburg Ludwigsburg)
	FELLBACH_WERNERSTRASSE = 'de:08119:2640'
	FELLBACH_WERNERSTRASSE_1 = 'de:08119:2640:0:3'  # L.60 Rtg Pauluskirche/L.67 Rtg Maicklers (Fellbach Fellbach)
	FELLBACH_WERNERSTRASSE_2 = 'de:08119:2640:0:4'  # Rtg Bahnhof (Fellbach Fellbach)
	FELLBACH_STEINBEISSTRASSE = 'de:08119:2641:0:4'  # Rtg Max-Planck-Str. (Fellbach Fellbach)
	STEINBEISSTRASSE_1 = 'de:08119:2641:0:4'  # Rtg Max-Planck-Str. (Fellbach Fellbach)
	STEINBEISSTRASSE_2 = 'de:08116:3850:0:1'  # Rtg Alleen/Max-Planck/Röntgenstr. (Esslingen am Neckar Zell (Esslingen))
	STEINBEISSTRASSE_3 = 'de:08115:4702:0:3'  # Rtg Berliner Str. (Böblingen Böblingen)
	STEINBEISSTRASSE_4 = 'de:08115:4702:0:5'  # Rtg Nord Nordseite (Böblingen Böblingen)
	STEINBEISSTRASSE_5 = 'de:08118:5491:0:3'  # eRtg Daimlerstraße (Ludwigsburg Ludwigsburg)
	STEINBEISSTRASSE_6 = 'de:08118:5491:0:4'  # eRtg Robert-Bosch-Straße (Ludwigsburg Ludwigsburg)
	STEINBEISSTRASSE_7 = 'de:08118:5875:0:3'  # eRtg Vaihingen (E) (Vaihingen an der Enz Vaihingen (E))
	STEINBEISSTRASSE_8 = 'de:08118:5875:0:4'  # eRtg Krankenhaus (Vaihingen an der Enz Vaihingen (E))
	STEINBEISSTRASSE_9 = 'de:08118:6978:0:3'  # eRtg Abzweig Bahnhof (Markgröningen Markgröningen)
	STEINBEISSTRASSE_10 = 'de:08118:6978:0:4'  # eRtg Klinik/Schule (Markgröningen Markgröningen)
	STEINBEISSTRASSE_11 = 'de:08119:6993:0:3'  # eRtg Grafenbergweg (Schorndorf Schorndorf)
	STEINBEISSTRASSE_12 = 'de:08119:6993:0:4'  # eRtg Waiblinger Straße (Schorndorf Schorndorf)
	STEINBEISSTRASSE_13 = 'de:08117:1009:0:H1'  # Rtg Liebfrauenkirche (Eislingen/Fils Eislingen (F))
	WILHELM_PFITZER_STRASSE = 'de:08119:2642:0:3'  # Rtg Bahnhof (Fellbach Fellbach)
	FELLBACH_WILHELM_PFITZER_STR = 'de:08119:2642:0:3'  # Rtg Bahnhof (Fellbach Fellbach)
	MAX_PLANCK_STRASSE_1 = 'de:08119:2643:1:3'  # Rtg Wilhelm-Pfitzer-Str. (Fellbach Fellbach)
	MAX_PLANCK_STRASSE_2 = 'de:08116:4043:0:3'  # Rtg Zeppelinstr./Sirnauer Brücke (Esslingen am Neckar Oberesslingen)
	MAX_PLANCK_STRASSE_3 = 'de:08116:4043:0:4'  # Rtg Schwertmühle (Esslingen am Neckar Oberesslingen)
	MAX_PLANCK_STRASSE_4 = 'de:08116:4443:0:3'  # eRtg Oberer Graben (Neuffen Neuffen)
	MAX_PLANCK_STRASSE_5 = 'de:08116:4443:0:4'  # eRtg Balzholz Rathaus (Neuffen Neuffen)
	MAX_PLANCK_STRASSE_6 = 'de:08118:6809:0:3'  # Rtg Kornwestheimer Str. (Kornwestheim Kornwestheim)
	MAX_PLANCK_STRASSE_7 = 'de:08118:6809:0:4'  # Rtg Bolzstr. (Kornwestheim Kornwestheim)
	FELLBACH_MAX_PLANCK_STR = 'de:08119:2643:1:3'  # Rtg Wilhelm-Pfitzer-Str. (Fellbach Fellbach)
	FELLBACH_KIENBACHSTR = 'de:08119:2644'
	FELLBACH_KIENBACHSTR_1 = 'de:08119:2644:0:3'  # Rtg Eugenstr. (Fellbach Fellbach)
	FELLBACH_KIENBACHSTR_2 = 'de:08119:2644:0:4'  # Rtg Schwabenlandhalle/F.3 (Fellbach Fellbach)
	EUGENSTRASSE_1 = 'de:08119:2645:0:3'  # Rtg Alte Kelter (Fellbach Fellbach)
	EUGENSTRASSE_2 = 'de:08119:2645:0:4'  # Rtg Kienbachstr. (Fellbach Fellbach)
	EUGENSTRASSE_3 = 'de:08117:4907:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	EUGENSTRASSE_4 = 'de:08117:4907:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	FELLBACH_EUGENSTR = 'de:08119:2645'
	FELLBACH_EUGENSTR_1 = 'de:08119:2645:0:3'  # Rtg Alte Kelter (Fellbach Fellbach)
	FELLBACH_EUGENSTR_2 = 'de:08119:2645:0:4'  # Rtg Kienbachstr. (Fellbach Fellbach)
	FELLBACH_ADLERSTRASSE = 'de:08119:2646'
	FELLBACH_ADLERSTRASSE_1 = 'de:08119:2646:0:3'  # Rtg Postdörfle (Fellbach Fellbach)
	FELLBACH_ADLERSTRASSE_2 = 'de:08119:2646:0:4'  # Rtg Bahnhof (Fellbach Fellbach)
	ADLERSTRASSE_1 = 'de:08119:2646:0:3'  # Rtg Postdörfle (Fellbach Fellbach)
	ADLERSTRASSE_2 = 'de:08119:2646:0:4'  # Rtg Bahnhof (Fellbach Fellbach)
	ADLERSTRASSE_3 = 'de:08116:6723:0:4'  # Rtg Stadtplatz (Wernau (Neckar) Wernau (N))
	ADLERSTRASSE_4 = 'de:08115:7057:0:3'  # eRtg Im Vogelsang (Herrenberg Herrenberg)
	ADLERSTRASSE_5 = 'de:08117:6016:0:R1'  # Rtg Kitzener Str. (Ottenbach Ottenbach)
	FELLBACH_POSTDÖRFLE = 'de:08119:2647'
	FELLBACH_POSTDÖRFLE_1 = 'de:08119:2647:0:3'  # Rtg Esslinger Str. (Fellbach Fellbach)
	FELLBACH_POSTDÖRFLE_2 = 'de:08119:2647:0:4'  # Rtg Adlerstr. (Fellbach Fellbach)
	FELLBACH_LAUBSÄNGERWEG = 'de:08119:2648'
	FELLBACH_LAUBSÄNGERWEG_1 = 'de:08119:2648:0:3'  # Rtg Rom. Fellb. Str. (Fellbach Fellbach)
	FELLBACH_LAUBSÄNGERWEG_2 = 'de:08119:2648:0:4'  # Rtg Rommelshauser Str. (Fellbach Fellbach)
	LAUBSÄNGERWEG = 'de:08119:2648'
	LAUBSÄNGERWEG_1 = 'de:08119:2648:0:3'  # Rtg Rom. Fellb. Str. (Fellbach Fellbach)
	LAUBSÄNGERWEG_2 = 'de:08119:2648:0:4'  # Rtg Rommelshauser Str. (Fellbach Fellbach)
	RAPPACHSTRASSE = 'de:08111:2653:0:3'  # Rtg Giebel (Stuttgart Stuttgart)
	GEWERBESTRASSE = 'de:08111:2699:0:3'  # Rtg Schulze-Delitzsch-Str. (Stuttgart Stuttgart)
	HANDWERKSTRASSE = 'de:08111:2700:0:3'  # Rtg Industriestr. (Stuttgart Stuttgart)
	KREUZSTRASSE_1 = 'de:08111:2701:0:3'  # Rtg Vaih. Rathaus (Stuttgart Stuttgart)
	KREUZSTRASSE_2 = 'de:08111:2701:0:4'  # Rtg Vaih. Alter Friedhof (Stuttgart Stuttgart)
	KREUZSTRASSE_3 = 'de:08116:4029:0:3'  # eRtg Hirschlandhof/Krankenhaus (Esslingen am Neckar Oberesslingen)
	KREUZSTRASSE_4 = 'de:08116:4029:0:4'  # eRtg Steinhalde/Schorndorfer Str./Index (Esslingen am Neckar Oberesslingen)
	KREUZSTRASSE_5 = 'de:08115:4773:0:3'  # Rtg Rathaus (Grafenau Dätzingen)
	KREUZSTRASSE_6 = 'de:08115:4773:0:4'  # Rtg Döffingen (Grafenau Dätzingen)
	KREUZSTRASSE_7 = 'de:08119:5071:0:3'  # eRtg Eulenhof (Kaisersbach Kaisersbach)
	KREUZSTRASSE_8 = 'de:08119:5071:0:4'  # eRtg Gausmannsweiler (Kaisersbach Kaisersbach)
	KREUZSTRASSE_9 = 'de:08118:5614:0:3'  # eRtg Lichtenberger Straße (Oberstenfeld Oberstenfeld)
	KREUZSTRASSE_10 = 'de:08118:5614:0:4'  # eRtg Schule (Oberstenfeld Oberstenfeld)
	HAINBACH = 'de:08116:2788'
	HAINBACH_1 = 'de:08116:2788:0:1'  # Rtg Wäld. Betriebshof (Esslingen am Neckar Wäldenbronn)
	HAINBACH_2 = 'de:08116:2788:0:2'  # Rtg St. Bernh. Talstr (Esslingen am Neckar Wäldenbronn)
	WÄLDENBRONN_HAINBACH = 'de:08116:2788'
	WÄLDENBRONN_HAINBACH_1 = 'de:08116:2788:0:1'  # Rtg Wäld. Betriebshof (Esslingen am Neckar Wäldenbronn)
	WÄLDENBRONN_HAINBACH_2 = 'de:08116:2788:0:2'  # Rtg St. Bernh. Talstr (Esslingen am Neckar Wäldenbronn)
	MAILLE = 'de:08116:2800'
	MAILLE_1 = 'de:08116:2800:0:3'  # Rtg. Pliensauturm/Esslingen Hbf (Esslingen am Neckar Esslingen (N))
	MAILLE_2 = 'de:08116:2800:0:4'  # Rtg Stuttg. Str Pliensauvorstadt (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__MAILLE = 'de:08116:2800'
	ESSLINGEN__N__MAILLE_1 = 'de:08116:2800:0:3'  # Rtg. Pliensauturm/Esslingen Hbf (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__MAILLE_2 = 'de:08116:2800:0:4'  # Rtg Stuttg. Str Pliensauvorstadt (Esslingen am Neckar Esslingen (N))
	WEILSTRASSE = 'de:08116:2801'
	WEILSTRASSE_1 = 'de:08116:2801:1:3'  # Rtg Pliensauturm/Essl. Bf (Esslingen am Neckar Pliensauvorstadt)
	WEILSTRASSE_2 = 'de:08116:2801:1:4'  # Rtg Obererer Eisbergweg (Esslingen am Neckar Pliensauvorstadt)
	WEILSTRASSE_3 = 'de:08116:2801:1:5'  # Rtg Pfeifferklinge (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUV__WEILSTRASSE = 'de:08116:2801'
	PLIENSAUV__WEILSTRASSE_1 = 'de:08116:2801:1:3'  # Rtg Pliensauturm/Essl. Bf (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUV__WEILSTRASSE_2 = 'de:08116:2801:1:4'  # Rtg Obererer Eisbergweg (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUV__WEILSTRASSE_3 = 'de:08116:2801:1:5'  # Rtg Pfeifferklinge (Esslingen am Neckar Pliensauvorstadt)
	ZOLLBERG_MUTZENREIS = 'de:08116:2802'
	ZOLLBERG_MUTZENREIS_1 = 'de:08116:2802:0:3'  # Rtg Robert-Koch-Str. (Esslingen am Neckar Zollberg)
	ZOLLBERG_MUTZENREIS_2 = 'de:08116:2802:0:4'  # Rtg Pliensaufriedhof (Esslingen am Neckar Zollberg)
	MUTZENREIS = 'de:08116:2802'
	MUTZENREIS_1 = 'de:08116:2802:0:3'  # Rtg Robert-Koch-Str. (Esslingen am Neckar Zollberg)
	MUTZENREIS_2 = 'de:08116:2802:0:4'  # Rtg Pliensaufriedhof (Esslingen am Neckar Zollberg)
	ZOLLBERG = 'de:08116:2803'
	ZOLLBERG_1 = 'de:08116:2803:1:3'  # Rtg Ob. Eisbergweg Esslingen (Esslingen am Neckar Zollberg)
	ZOLLBERG_2 = 'de:08116:2803:1:4'  # Rtg Mutzenreisstr./Zollernplatz (Esslingen am Neckar Zollberg)
	PLIENSAUFRIEDHOF = 'de:08116:2804'
	PLIENSAUFRIEDHOF_1 = 'de:08116:2804:0:3'  # Rtg Mutzenreis (Esslingen am Neckar Zollberg)
	PLIENSAUFRIEDHOF_2 = 'de:08116:2804:0:4'  # Rtg Pfeifferklinge (Esslingen am Neckar Zollberg)
	ZOLLBERG_PLIENSAUFRIEDHOF = 'de:08116:2804'
	ZOLLBERG_PLIENSAUFRIEDHOF_1 = 'de:08116:2804:0:3'  # Rtg Mutzenreis (Esslingen am Neckar Zollberg)
	ZOLLBERG_PLIENSAUFRIEDHOF_2 = 'de:08116:2804:0:4'  # Rtg Pfeifferklinge (Esslingen am Neckar Zollberg)
	MUTZENREISSTRASSE = 'de:08116:2805'
	MUTZENREISSTRASSE_1 = 'de:08116:2805:0:3'  # Rtg Zollberg (Esslingen am Neckar Zollberg)
	MUTZENREISSTRASSE_2 = 'de:08116:2805:0:4'  # Rtg Fa. Festo/Nellingen (Esslingen am Neckar Zollberg)
	ZOLLBERG_MUTZENREISSTR = 'de:08116:2805'
	ZOLLBERG_MUTZENREISSTR_1 = 'de:08116:2805:0:3'  # Rtg Zollberg (Esslingen am Neckar Zollberg)
	ZOLLBERG_MUTZENREISSTR_2 = 'de:08116:2805:0:4'  # Rtg Fa. Festo/Nellingen (Esslingen am Neckar Zollberg)
	PLIENSAUTURM = 'de:08116:2806'
	PLIENSAUTURM_1 = 'de:08116:2806:1:3'  # Rtg Bahnhof/ZOB (Esslingen am Neckar Esslingen (N))
	PLIENSAUTURM_2 = 'de:08116:2806:1:4'  # Rtg stadtauswärts hinten (Esslingen am Neckar Esslingen (N))
	PLIENSAUTURM_3 = 'de:08116:2806:1:6'  # Rtg stadtauswärts vorne (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__PLIENSAUTURM = 'de:08116:2806'
	ESSLINGEN__N__PLIENSAUTURM_1 = 'de:08116:2806:1:3'  # Rtg Bahnhof/ZOB (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__PLIENSAUTURM_2 = 'de:08116:2806:1:4'  # Rtg stadtauswärts hinten (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__PLIENSAUTURM_3 = 'de:08116:2806:1:6'  # Rtg stadtauswärts vorne (Esslingen am Neckar Esslingen (N))
	WOLFSCHLUGEN_SCHUBERTSTR = 'de:08116:2808:0:4'  # Rtg Löwen (Wolfschlugen Wolfschlugen)
	SCHUBERTSTRASSE = 'de:08116:2808:0:4'  # Rtg Löwen (Wolfschlugen Wolfschlugen)
	JUNGBORN = 'de:08116:2809'
	JUNGBORN_1 = 'de:08116:2809:0:1'  # Rtg Owen (Nürtingen Nürtingen)
	JUNGBORN_2 = 'de:08116:2809:0:2'  # Rtg Nürtingen (Nürtingen Nürtingen)
	NÜRTINGEN_JUNGBORN = 'de:08116:2809'
	NÜRTINGEN_JUNGBORN_1 = 'de:08116:2809:0:1'  # Rtg Owen (Nürtingen Nürtingen)
	NÜRTINGEN_JUNGBORN_2 = 'de:08116:2809:0:2'  # Rtg Nürtingen (Nürtingen Nürtingen)
	ROSSDORF_SCHULE = 'de:08116:2810'
	ROSSDORF_SCHULE_1 = 'de:08116:2810:0:3'  # Rtg Roßdorf Kleeweg (Nürtingen Nürtingen)
	ROSSDORF_SCHULE_2 = 'de:08116:2810:0:4'  # Rtg Liebermannstr. (Nürtingen Nürtingen)
	NÜRTINGEN_ROSSDORF_SCHULE = 'de:08116:2810'
	NÜRTINGEN_ROSSDORF_SCHULE_1 = 'de:08116:2810:0:3'  # Rtg Roßdorf Kleeweg (Nürtingen Nürtingen)
	NÜRTINGEN_ROSSDORF_SCHULE_2 = 'de:08116:2810:0:4'  # Rtg Liebermannstr. (Nürtingen Nürtingen)
	ROSSDORF_KLEEWEG = 'de:08116:2811:0:4'  # Rtg alle Richtungen (Nürtingen Nürtingen)
	NÜRTINGEN_ROSSDORF_KLEEWEG = 'de:08116:2811:0:4'  # Rtg alle Richtungen (Nürtingen Nürtingen)
	MÜHLSTRASSE_1 = 'de:08116:2813:0:3'  # Rtg ZOB (Nürtingen Nürtingen)
	MÜHLSTRASSE_2 = 'de:08116:2813:0:4'  # Rtg Neckarbrücke (Nürtingen Nürtingen)
	MÜHLSTRASSE_3 = 'de:08118:5641:0:3'  # eRtg Theodor-Heuss-Straße (Oberriexingen Oberriexingen)
	MÜHLSTRASSE_4 = 'de:08117:303:0:R1'  # Rtg Stiegelwiese (Süßen Süßen)
	MÜHLSTRASSE_5 = 'de:08136:7048:0:1'  # Rtg Waldhausen (Lorch Waldhausen b Schornd (Lorch))
	MÜHLSTRASSE_6 = 'de:08136:7048:0:2'  # Rtg Weitmars (Lorch Waldhausen b Schornd (Lorch))
	NÜRTINGEN_MÜHLSTRASSE = 'de:08116:2813'
	NÜRTINGEN_MÜHLSTRASSE_1 = 'de:08116:2813:0:3'  # Rtg ZOB (Nürtingen Nürtingen)
	NÜRTINGEN_MÜHLSTRASSE_2 = 'de:08116:2813:0:4'  # Rtg Neckarbrücke (Nürtingen Nürtingen)
	FREIBAD_1 = 'de:08116:2814:0:3'  # eRtg Schlossgymn./Stadion/Eichendorffstr (Kirchheim unter Teck Kirchheim (T))
	FREIBAD_2 = 'de:08116:2814:0:4'  # eRtg Teckstraße (Kirchheim unter Teck Kirchheim (T))
	FREIBAD_3 = 'de:08115:3107:0:3'  # Rtg Thermalbad (Böblingen Böblingen)
	FREIBAD_4 = 'de:08115:3107:0:4'  # Rtg Silberweg/Stuttgarter Str. (Böblingen Böblingen)
	FREIBAD_5 = 'de:08119:3773:0:3'  # Rtg Grundschule (Kernen im Remstal Stetten (i. R.))
	FREIBAD_6 = 'de:08119:3773:0:4'  # Rtg Frauenländerstr. (Kernen im Remstal Stetten (i. R.))
	FREIBAD_7 = 'de:08119:5404:0:3'  # Rtg Stettener Str./WNer Str./Mercedesstr (Waiblingen Waiblingen)
	FREIBAD_8 = 'de:08119:5404:0:4'  # eRtg Stadtwerke (Waiblingen Waiblingen)
	FREIBAD_9 = 'de:08118:5729:0:3'  # eRtg Tripsdrill Erlebnispark (Bönnigheim Bönnigheim)
	FREIBAD_10 = 'de:08118:5729:0:4'  # eRtg Freibad (Bönnigheim Bönnigheim)
	FREIBAD_11 = 'de:08117:1006:0:H1'  # Rtg Eislingen (Göppingen Göppingen)
	FREIBAD_12 = 'de:08117:1006:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	FREIBAD_13 = 'de:08117:2030:0:1'  # Rtg Am Wiesenbach (Uhingen Uhingen)
	FREIBAD_14 = 'de:08117:2030:0:H1'  # Rtg Turnhalle (Uhingen Uhingen)
	KIRCHHEIM__T__FREIBAD = 'de:08116:2814'
	KIRCHHEIM__T__FREIBAD_1 = 'de:08116:2814:0:3'  # eRtg Schlossgymn./Stadion/Eichendorffstr (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__FREIBAD_2 = 'de:08116:2814:0:4'  # eRtg Teckstraße (Kirchheim unter Teck Kirchheim (T))
	WALDENBUCH_BONHOLZ = 'de:08115:2815'
	WALDENBUCH_BONHOLZ_1 = 'de:08115:2815:0:3'  # Rtg Dettenhausen Alte Post (Waldenbuch Waldenbuch)
	WALDENBUCH_BONHOLZ_2 = 'de:08115:2815:0:4'  # eRtg Postamt (Waldenbuch Waldenbuch)
	BONHOLZ_1 = 'de:08115:2815:0:3'  # Rtg Dettenhausen Alte Post (Waldenbuch Waldenbuch)
	BONHOLZ_2 = 'de:08115:2815:0:4'  # eRtg Postamt (Waldenbuch Waldenbuch)
	BONHOLZ_3 = 'de:08119:3257:0:1'  # Rtg Adelstetten (Alfdorf Alfdorf)
	MARQUARDTSCHULE = 'de:08116:2816:0:3'  # eRtg Beethovenstraße (Plochingen Plochingen)
	PLOCHINGEN_MARQUARDTSCHULE = 'de:08116:2816:0:3'  # eRtg Beethovenstraße (Plochingen Plochingen)
	WEILH__T__EGELSBERG_HOCHHAUS = 'de:08116:2817'
	WEILH__T__EGELSBERG_HOCHHAUS_1 = 'de:08116:2817:0:3'  # eRtg Teckstraße/Weilheim (Weilheim an der Teck Weilheim (T))
	WEILH__T__EGELSBERG_HOCHHAUS_2 = 'de:08116:2817:0:4'  # eRtg Egelsbergstraße (Weilheim an der Teck Weilheim (T))
	EGELSBERG_HOCHHAUS = 'de:08116:2817'
	EGELSBERG_HOCHHAUS_1 = 'de:08116:2817:0:3'  # eRtg Teckstraße/Weilheim (Weilheim an der Teck Weilheim (T))
	EGELSBERG_HOCHHAUS_2 = 'de:08116:2817:0:4'  # eRtg Egelsbergstraße (Weilheim an der Teck Weilheim (T))
	NEUFFENER_STRASSE = 'de:08116:2819'
	NEUFFENER_STRASSE_1 = 'de:08116:2819:1:3'  # Rtg Humboldtstraße (Frickenhausen Linsenhofen)
	NEUFFENER_STRASSE_2 = 'de:08116:2819:1:4'  # Rtg Neuffen (Frickenhausen Linsenhofen)
	LINSENHOFEN_NEUFFENER_STR = 'de:08116:2819'
	LINSENHOFEN_NEUFFENER_STR_1 = 'de:08116:2819:1:3'  # Rtg Humboldtstraße (Frickenhausen Linsenhofen)
	LINSENHOFEN_NEUFFENER_STR_2 = 'de:08116:2819:1:4'  # Rtg Neuffen (Frickenhausen Linsenhofen)
	KAPPISHÄUSERN_RATHAUS = 'de:08116:2820'
	KAPPISHÄUSERN_RATHAUS_1 = 'de:08116:2820:0:3'  # eRtg Linde (Neuffen Kappishäusern)
	KAPPISHÄUSERN_RATHAUS_2 = 'de:08116:2820:0:4'  # eRtg Feuersee (Neuffen Kappishäusern)
	ETZELWEG = 'de:08116:2821'
	ETZELWEG_1 = 'de:08116:2821:0:3'  # eRtg Rathaus (Kohlberg Kohlberg)
	ETZELWEG_2 = 'de:08116:2821:0:4'  # eRtg Hörnlesweg (Kohlberg Kohlberg)
	KOHLBERG_ETZELWEG = 'de:08116:2821'
	KOHLBERG_ETZELWEG_1 = 'de:08116:2821:0:3'  # eRtg Rathaus (Kohlberg Kohlberg)
	KOHLBERG_ETZELWEG_2 = 'de:08116:2821:0:4'  # eRtg Hörnlesweg (Kohlberg Kohlberg)
	FRICKENHAUSEN_BAHNHOF__UHLANDSTR = 'de:08116:2822:1:3'  # Rtg Eichendorffstr. (Frickenhausen Frickenhausen)
	BAHNHOF__UHLANDSTRASSE = 'de:08116:2822:1:3'  # Rtg Eichendorffstr. (Frickenhausen Frickenhausen)
	HÖLZLACHSTRASSE = 'de:08116:2824'
	HÖLZLACHSTRASSE_1 = 'de:08116:2824:0:3'  # eRtg Kelter (Frickenhausen Frickenhausen)
	HÖLZLACHSTRASSE_2 = 'de:08116:2824:0:4'  # eRtg Schule (Frickenhausen Frickenhausen)
	FRICKENHAUSEN_HÖLZLACHSTR = 'de:08116:2824'
	FRICKENHAUSEN_HÖLZLACHSTR_1 = 'de:08116:2824:0:3'  # eRtg Kelter (Frickenhausen Frickenhausen)
	FRICKENHAUSEN_HÖLZLACHSTR_2 = 'de:08116:2824:0:4'  # eRtg Schule (Frickenhausen Frickenhausen)
	SCHLIERBACHER_DREIECK = 'de:08116:2826'
	SCHLIERBACHER_DREIECK_1 = 'de:08116:2826:0:3'  # Rtg Hugo-Wolf-Weg/Kiebitzweg (Kirchheim unter Teck Kirchheim (T))
	SCHLIERBACHER_DREIECK_2 = 'de:08116:2826:0:4'  # Rtg Notzinger Straße (Kirchheim unter Teck Kirchheim (T))
	KIRCHH__T__SCHLIERB__DREIECK = 'de:08116:2826'
	KIRCHH__T__SCHLIERB__DREIECK_1 = 'de:08116:2826:0:3'  # Rtg Hugo-Wolf-Weg/Kiebitzweg (Kirchheim unter Teck Kirchheim (T))
	KIRCHH__T__SCHLIERB__DREIECK_2 = 'de:08116:2826:0:4'  # Rtg Notzinger Straße (Kirchheim unter Teck Kirchheim (T))
	NOTZINGER_STRASSE = 'de:08116:2827'
	NOTZINGER_STRASSE_1 = 'de:08116:2827:0:3'  # Rtg Schlierbacher Dreieck (Kirchheim unter Teck Kirchheim (T))
	NOTZINGER_STRASSE_2 = 'de:08116:2827:0:4'  # Rtg Saarstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__NOTZINGER_STR = 'de:08116:2827'
	KIRCHHEIM__T__NOTZINGER_STR_1 = 'de:08116:2827:0:3'  # Rtg Schlierbacher Dreieck (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__NOTZINGER_STR_2 = 'de:08116:2827:0:4'  # Rtg Saarstraße (Kirchheim unter Teck Kirchheim (T))
	IM_STOCKACH = 'de:08116:2831'
	IM_STOCKACH_1 = 'de:08116:2831:0:3'  # Rtg Lange Morgen (Weilheim an der Teck Weilheim (T))
	IM_STOCKACH_2 = 'de:08116:2831:0:4'  # Rtg Holzmaden (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__IM_STOCKACH = 'de:08116:2831'
	WEILHEIM__T__IM_STOCKACH_1 = 'de:08116:2831:0:3'  # Rtg Lange Morgen (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__IM_STOCKACH_2 = 'de:08116:2831:0:4'  # Rtg Holzmaden (Weilheim an der Teck Weilheim (T))
	AKADEMIEGÄRTEN = 'de:08116:2893:0:4'  # Rtg F.-S.-Schule (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__AKADEMIEGÄRTEN = 'de:08116:2893:0:4'  # Rtg F.-S.-Schule (Neuhausen auf den Fildern Neuhausen (F))
	NEUH__F__BERNHÄUSER_WEG = 'de:08116:2895'
	NEUH__F__BERNHÄUSER_WEG_1 = 'de:08116:2895:0:3'  # Rtg Bahnhofstraße (Neuhausen auf den Fildern Neuhausen (F))
	NEUH__F__BERNHÄUSER_WEG_2 = 'de:08116:2895:0:4'  # Rtg Industriegebiet West (Neuhausen auf den Fildern Neuhausen (F))
	BERNHÄUSER_WEG = 'de:08116:2895'
	BERNHÄUSER_WEG_1 = 'de:08116:2895:0:3'  # Rtg Bahnhofstraße (Neuhausen auf den Fildern Neuhausen (F))
	BERNHÄUSER_WEG_2 = 'de:08116:2895:0:4'  # Rtg Industriegebiet West (Neuhausen auf den Fildern Neuhausen (F))
	NOVIZENWEG_THW = 'de:08116:2896:0:3'  # Rtg F.-Schiller-Schule (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__NOVIZENWEG_THW = 'de:08116:2896:0:3'  # Rtg F.-Schiller-Schule (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__LETTENSTRASSE = 'de:08116:2897'
	NEUHAUSEN__F__LETTENSTRASSE_1 = 'de:08116:2897:0:3'  # Rtg Brühlsiedlung/Schillerstr (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__LETTENSTRASSE_2 = 'de:08116:2897:0:4'  # Rtg Mozartschule (Neuhausen auf den Fildern Neuhausen (F))
	LETTENSTRASSE = 'de:08116:2897'
	LETTENSTRASSE_1 = 'de:08116:2897:0:3'  # Rtg Brühlsiedlung/Schillerstr (Neuhausen auf den Fildern Neuhausen (F))
	LETTENSTRASSE_2 = 'de:08116:2897:0:4'  # Rtg Mozartschule (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__MOZARTSCHULE = 'de:08116:2898'
	NEUHAUSEN__F__MOZARTSCHULE_1 = 'de:08116:2898:0:3'  # Rtg Lettenstraße (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__MOZARTSCHULE_2 = 'de:08116:2898:0:4'  # Rtg Schlossplatz (Neuhausen auf den Fildern Neuhausen (F))
	MOZARTSCHULE = 'de:08116:2898'
	MOZARTSCHULE_1 = 'de:08116:2898:0:3'  # Rtg Lettenstraße (Neuhausen auf den Fildern Neuhausen (F))
	MOZARTSCHULE_2 = 'de:08116:2898:0:4'  # Rtg Schlossplatz (Neuhausen auf den Fildern Neuhausen (F))
	SCHARNH__PLIENINGER_STR__NORD = 'de:08116:2899:1:3'  # Rtg Neuhausen (Ostfildern Scharnhausen)
	PLIENINGER_STRASSE__NORD = 'de:08116:2899:1:3'  # Rtg Neuhausen (Ostfildern Scharnhausen)
	NAGELSTRASSE = 'de:08116:2900'
	NAGELSTRASSE_1 = 'de:08116:2900:0:3'  # Rtg Plieninger Str.(Nord) (Ostfildern Scharnhausen)
	NAGELSTRASSE_2 = 'de:08116:2900:0:4'  # Rtg Kemnat Reutlinger Str. (Ostfildern Scharnhausen)
	SCHARNHAUSEN_NAGELSTRASSE = 'de:08116:2900'
	SCHARNHAUSEN_NAGELSTRASSE_1 = 'de:08116:2900:0:3'  # Rtg Plieninger Str.(Nord) (Ostfildern Scharnhausen)
	SCHARNHAUSEN_NAGELSTRASSE_2 = 'de:08116:2900:0:4'  # Rtg Kemnat Reutlinger Str. (Ostfildern Scharnhausen)
	BAHNHOFSTRASSE_1 = 'de:08116:2901:1:3'  # Rtg Schlossplatz / Peronnas-Platz (Neuhausen auf den Fildern Neuhausen (F))
	BAHNHOFSTRASSE_2 = 'de:08116:2901:1:4'  # Rtg Industriegebiet Ost /Wilhelmstr. (Neuhausen auf den Fildern Neuhausen (F))
	BAHNHOFSTRASSE_3 = 'de:08116:2966:1:3'  # Rtg Schillerplatz/Weberstr. (Nürtingen Nürtingen)
	BAHNHOFSTRASSE_4 = 'de:08118:3432:1:3'  # eRtg Brühlstraße (Möglingen Möglingen)
	BAHNHOFSTRASSE_5 = 'de:08118:3432:1:4'  # eRtg Im Bornrain (Möglingen Möglingen)
	BAHNHOFSTRASSE_6 = 'de:08118:3551:1:3'  # eRtg Ankunft (Erdmannhausen Erdmannhausen)
	BAHNHOFSTRASSE_7 = 'de:08118:3551:1:4'  # eRtg Rathaus (Erdmannhausen Erdmannhausen)
	BAHNHOFSTRASSE_8 = 'de:08116:3834:0:1'  # Rtg Schützenstr. (Wendlingen am Neckar Wendlingen (N))
	BAHNHOFSTRASSE_9 = 'de:08116:4283:1:3'  # Rtg Brucken (Lenningen Unterlenningen)
	BAHNHOFSTRASSE_10 = 'de:08116:4283:1:4'  # Rtg Oberlenningen (Lenningen Unterlenningen)
	BAHNHOFSTRASSE_11 = 'de:08116:4326:0:3'  # Rtg ??? (Weilheim an der Teck Weilheim (T))
	BAHNHOFSTRASSE_12 = 'de:08115:4533:1:3'  # Rtg Marktplatz (Weissach Weissach (BB))
	BAHNHOFSTRASSE_13 = 'de:08115:4533:1:4'  # Rtg Heimerdingen/Wöhr (Weissach Weissach (BB))
	BAHNHOFSTRASSE_14 = 'de:08118:4558:1:3'  # eRtg Ankunft (Hemmingen Hemmingen)
	BAHNHOFSTRASSE_15 = 'de:08118:4558:1:4'  # eRtg Abfahrt (Hemmingen Hemmingen)
	BAHNHOFSTRASSE_16 = 'de:08115:4806:0:3'  # Rtg Elsenhalde (Schönaich Schönaich)
	BAHNHOFSTRASSE_17 = 'de:08115:4806:0:4'  # Rtg Neues Rathaus (Schönaich Schönaich)
	BAHNHOFSTRASSE_18 = 'de:08119:5245:1:3'  # eRtg Birkenallee (Bahnübergang) (Plüderhausen Plüderhausen)
	BAHNHOFSTRASSE_19 = 'de:08119:5321:1:3'  # Rtg B14 (Leutenbach Nellmersbach)
	BAHNHOFSTRASSE_20 = 'de:08119:5321:1:4'  # Rtg Leutenbacher Str. (Leutenbach Nellmersbach)
	BAHNHOFSTRASSE_21 = 'de:08119:6837:0:3'  # Rtg Endersbach Bf (Weinstadt Endersbach)
	BAHNHOFSTRASSE_22 = 'de:08119:6837:0:4'  # Rtg Beim Kreisel/Viadukt (Weinstadt Endersbach)
	BAHNHOFSTRASSE_23 = 'de:08118:7417:1:3'  # Rtg Danziger Weg (Sersheim Sersheim)
	BAHNHOFSTRASSE_24 = 'de:08118:7417:1:4'  # Rtg Vaihinger Straße (Sersheim Sersheim)
	BAHNHOFSTRASSE_25 = 'de:08117:17:0:H1'  # Rtg Buchsteiner (Gingen an der Fils Gingen (F))
	BAHNHOFSTRASSE_26 = 'de:08117:59:0:H1'  # Rtg Rathaus (Mühlhausen im Täle Mühlhausen im Täle)
	BAHNHOFSTRASSE_27 = 'de:08117:59:0:R1'  # Rtg Wiesensteig (Mühlhausen im Täle Mühlhausen im Täle)
	BAHNHOFSTRASSE_28 = 'de:08117:1103:0:H1'  # Rtg Wäschenbeuren (Birenbach Birenbach)
	BAHNHOFSTRASSE_29 = 'de:08117:1103:0:R1'  # Rtg Rechberghausen (Birenbach Birenbach)
	BAHNHOFSTRASSE_30 = 'de:08117:1107:0:H1'  # Rtg Schillerstr. (Wäschenbeuren Wäschenbeuren)
	NEUHAUSEN__F__BAHNHOFSTR = 'de:08116:2901'
	NEUHAUSEN__F__BAHNHOFSTR_1 = 'de:08116:2901:1:3'  # Rtg Schlossplatz / Peronnas-Platz (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__BAHNHOFSTR_2 = 'de:08116:2901:1:4'  # Rtg Industriegebiet Ost /Wilhelmstr. (Neuhausen auf den Fildern Neuhausen (F))
	BRÜHLSIEDLUNG = 'de:08116:2902'
	BRÜHLSIEDLUNG_1 = 'de:08116:2902:1:3'  # Linie 36,120 Rtg NHNW,Wolfschl (Neuhausen auf den Fildern Neuhausen (F))
	BRÜHLSIEDLUNG_2 = 'de:08116:2902:1:4'  # Linie 36,73,120 Rtg DG,ECBF,ESSL (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__BRÜHLSIEDLUNG = 'de:08116:2902'
	NEUHAUSEN__F__BRÜHLSIEDLUNG_1 = 'de:08116:2902:1:3'  # Linie 36,120 Rtg NHNW,Wolfschl (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__BRÜHLSIEDLUNG_2 = 'de:08116:2902:1:4'  # Linie 36,73,120 Rtg DG,ECBF,ESSL (Neuhausen auf den Fildern Neuhausen (F))
	PERONNAS_PLATZ = 'de:08116:2903'
	PERONNAS_PLATZ_1 = 'de:08116:2903:0:3'  # Rtg Traube (Neuhausen auf den Fildern Neuhausen (F))
	PERONNAS_PLATZ_2 = 'de:08116:2903:0:4'  # Rtg Bahnhofstr./Schlossplatz (Neuhausen auf den Fildern Neuhausen (F))
	NEUH__F__PERONNAS_PLATZ = 'de:08116:2903'
	NEUH__F__PERONNAS_PLATZ_1 = 'de:08116:2903:0:3'  # Rtg Traube (Neuhausen auf den Fildern Neuhausen (F))
	NEUH__F__PERONNAS_PLATZ_2 = 'de:08116:2903:0:4'  # Rtg Bahnhofstr./Schlossplatz (Neuhausen auf den Fildern Neuhausen (F))
	INDUSTRIEGEBIET_OST_1 = 'de:08116:2904:0:3'  # Rtg Bahnhofstraße (Neuhausen auf den Fildern Neuhausen (F))
	INDUSTRIEGEBIET_OST_2 = 'de:08116:2904:0:4'  # Rtg Bernhäuser Weg (Neuhausen auf den Fildern Neuhausen (F))
	INDUSTRIEGEBIET_OST_3 = 'de:08118:5844:0:3'  # eRtg Münchinger Str. (Altenw.) (Ditzingen Ditzingen)
	INDUSTRIEGEBIET_OST_4 = 'de:08117:3007:0:H1'  # Rtg Holzheim (Göppingen Göppingen)
	INDUSTRIEGEBIET_OST_5 = 'de:08117:3007:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	NEUH__F__INDUSTRIEGEBIET_OST = 'de:08116:2904'
	NEUH__F__INDUSTRIEGEBIET_OST_1 = 'de:08116:2904:0:3'  # Rtg Bahnhofstraße (Neuhausen auf den Fildern Neuhausen (F))
	NEUH__F__INDUSTRIEGEBIET_OST_2 = 'de:08116:2904:0:4'  # Rtg Bernhäuser Weg (Neuhausen auf den Fildern Neuhausen (F))
	NEUH__F__INDUSTRIEGEBIET_WEST = 'de:08116:2905'
	NEUH__F__INDUSTRIEGEBIET_WEST_1 = 'de:08116:2905:0:3'  # Rtg Bernhäuser Weg (Neuhausen auf den Fildern Neuhausen (F))
	NEUH__F__INDUSTRIEGEBIET_WEST_2 = 'de:08116:2905:0:4'  # Rtg Sielmingen Alemannenstraße (Neuhausen auf den Fildern Neuhausen (F))
	INDUSTRIEGEBIET_WEST = 'de:08116:2905'
	INDUSTRIEGEBIET_WEST_1 = 'de:08116:2905:0:3'  # Rtg Bernhäuser Weg (Neuhausen auf den Fildern Neuhausen (F))
	INDUSTRIEGEBIET_WEST_2 = 'de:08116:2905:0:4'  # Rtg Sielmingen Alemannenstraße (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__SCHLOSSPLATZ = 'de:08116:2906'
	NEUHAUSEN__F__SCHLOSSPLATZ_1 = 'de:08116:2906:1:3'  # Rtg Mozartschule (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__SCHLOSSPLATZ_2 = 'de:08116:2906:1:4'  # Rtg Peronnas-Platz (Neuhausen auf den Fildern Neuhausen (F))
	SCHLOSSPLATZ_1 = 'de:08116:2906:1:3'  # Rtg Mozartschule (Neuhausen auf den Fildern Neuhausen (F))
	SCHLOSSPLATZ_2 = 'de:08116:2906:1:4'  # Rtg Peronnas-Platz (Neuhausen auf den Fildern Neuhausen (F))
	SCHLOSSPLATZ_3 = 'de:08111:6022:1:1'  # Rtg Charlottenplatz (Stuttgart Stuttgart)
	SCHLOSSPLATZ_4 = 'de:08111:6022:1:2'  # Rtg Hauptbahnhof (Stuttgart Stuttgart)
	SCHLOSSPLATZ_5 = 'de:08111:6022:2:1'  # Linie 42 Rtg Charlottenplatz (Stuttgart Stuttgart)
	SCHLOSSPLATZ_6 = 'de:08111:6022:2:2'  # N2, N3, N10 (Stuttgart Stuttgart)
	SCHLOSSPLATZ_7 = 'de:08111:6022:2:3'  # N1, N4 (Stuttgart Stuttgart)
	SCHLOSSPLATZ_8 = 'de:08111:6022:2:4'  # N5, N6, N7, N8 (Stuttgart Stuttgart)
	SCHLOSSPLATZ_9 = 'de:08111:6022:2:5'  # Linie 44 Rtg Dorotheenstr. (Stuttgart Stuttgart)
	SCHLOSSPLATZ_10 = 'de:08111:6022:2:6'  # N9 (Stuttgart Stuttgart)
	NEUH__F__SCHILLERSTRASSE = 'de:08116:2907:0:4'  # Rtg Lettenstr./Wilheilmstr. (Neuhausen auf den Fildern Neuhausen (F))
	TRAUBE_1 = 'de:08116:2908:0:3'  # Rtg Schlosserstraße (Neuhausen auf den Fildern Neuhausen (F))
	TRAUBE_2 = 'de:08116:2908:0:4'  # Rtg Peronnas Platz (Neuhausen auf den Fildern Neuhausen (F))
	TRAUBE_3 = 'de:08119:5176:0:3'  # Rtg Kelter (Weinstadt Strümpfelbach (Weinstadt))
	TRAUBE_4 = 'de:08119:5176:0:4'  # Rtg Hundsäcker (Weinstadt Strümpfelbach (Weinstadt))
	TRAUBE_5 = 'de:08119:5382:0:3'  # Rtg Unterbr. Friedhof (Auenwald Lippoldsweiler)
	TRAUBE_6 = 'de:08119:5382:0:4'  # Rtg Naturfreundehaus/Am Brenner (Auenwald Lippoldsweiler)
	NEUHAUSEN__F__TRAUBE = 'de:08116:2908'
	NEUHAUSEN__F__TRAUBE_1 = 'de:08116:2908:0:3'  # Rtg Schlosserstraße (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__TRAUBE_2 = 'de:08116:2908:0:4'  # Rtg Peronnas Platz (Neuhausen auf den Fildern Neuhausen (F))
	WILHELMSTRASSE_1 = 'de:08116:2909:1:3'  # Rtg Lettenstr. (Neuhausen auf den Fildern Neuhausen (F))
	WILHELMSTRASSE_2 = 'de:08116:2909:1:4'  # Rtg Bahnhofstr. (Neuhausen auf den Fildern Neuhausen (F))
	WILHELMSTRASSE_3 = 'de:08115:3143:0:3'  # eRtg Röte (Weil im Schönbuch Weil im Schönbuch)
	WILHELMSTRASSE_4 = 'de:08115:3143:0:4'  # eRtg See (Weil im Schönbuch Weil im Schönbuch)
	WILHELMSTRASSE_5 = 'de:08118:3562:0:3'  # eRtg Bietigheimer Straße (Sachsenheim Kleinsachsenheim)
	WILHELMSTRASSE_6 = 'de:08116:4413:0:3'  # eRtg Grundschule (Bissingen an der Teck Bissingen an der Teck (T))
	WILHELMSTRASSE_7 = 'de:08116:4413:0:4'  # eRtg Fabrikstraße (Bissingen an der Teck Bissingen an der Teck (T))
	WILHELMSTRASSE_8 = 'de:08117:7:1:H1'  # Rtg Sternplatz ZOH (Geislingen an der Steige Geislingen (Steige))
	WILHELMSTRASSE_9 = 'de:08117:7:2:R1'  # Rtg Nel Mezzo (Geislingen an der Steige Geislingen (Steige))
	WILHELMSTRASSE_10 = 'de:08136:7042:0:1'  # Rtg (Lorch Lorch)
	WILHELMSTRASSE_11 = 'de:08136:7042:0:2'  # Rtg (Lorch Lorch)
	NEUHAUSEN__F__WILHELMSTR = 'de:08116:2909'
	NEUHAUSEN__F__WILHELMSTR_1 = 'de:08116:2909:1:3'  # Rtg Lettenstr. (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__WILHELMSTR_2 = 'de:08116:2909:1:4'  # Rtg Bahnhofstr. (Neuhausen auf den Fildern Neuhausen (F))
	HINDENBURGSTRASSE_1 = 'de:08116:2910:1:3'  # Rtg Nellingen (Ostfildern Nellingen)
	HINDENBURGSTRASSE_2 = 'de:08116:2910:1:4'  # Rtg Kaiserstr. (Ostfildern Nellingen)
	HINDENBURGSTRASSE_3 = 'de:08118:3442:1:1'  # Pos 1 (Möglingen Möglingen)
	HINDENBURGSTRASSE_4 = 'de:08118:3442:1:2'  # Pos 2 (Möglingen Möglingen)
	HINDENBURGSTRASSE_5 = 'de:08115:4571:0:3'  # Rtg Leonberg (Weissach Weissach (BB))
	HINDENBURGSTRASSE_6 = 'de:08115:4571:0:4'  # Rtg Porsche (Weissach Weissach (BB))
	HINDENBURGSTRASSE_7 = 'de:08115:4802:0:3'  # Rtg Altstadtgarage Kreiskrankenhaus (Herrenberg Herrenberg)
	HINDENBURGSTRASSE_8 = 'de:08115:4802:0:4'  # Rtg ZOB (Herrenberg Herrenberg)
	HINDENBURGSTRASSE_9 = 'de:08116:5023:0:3'  # Rtg Esslinger Str. (Denkendorf Denkendorf)
	NELLINGEN_HINDENBURGSTR = 'de:08116:2910'
	NELLINGEN_HINDENBURGSTR_1 = 'de:08116:2910:1:3'  # Rtg Nellingen (Ostfildern Nellingen)
	NELLINGEN_HINDENBURGSTR_2 = 'de:08116:2910:1:4'  # Rtg Kaiserstr. (Ostfildern Nellingen)
	PARKSTRASSE_1 = 'de:08116:2911:0:3'  # Rtg Zinsholz (Ostfildern Parksiedlung)
	PARKSTRASSE_2 = 'de:08116:2911:0:4'  # Rtg Herzog-Philipp-Platz (Ostfildern Parksiedlung)
	PARKSTRASSE_3 = 'de:08116:4038:0:3'  # Rtg Weil Stadion (Esslingen am Neckar Pliensauvorstadt)
	PARKSIEDLUNG_PARKSTRASSE = 'de:08116:2911'
	PARKSIEDLUNG_PARKSTRASSE_1 = 'de:08116:2911:0:3'  # Rtg Zinsholz (Ostfildern Parksiedlung)
	PARKSIEDLUNG_PARKSTRASSE_2 = 'de:08116:2911:0:4'  # Rtg Herzog-Philipp-Platz (Ostfildern Parksiedlung)
	RIEGELSTRASSE = 'de:08116:2912'
	RIEGELSTRASSE_1 = 'de:08116:2912:0:3'  # Rtg Hindenburgstr. (Ostfildern Nellingen)
	RIEGELSTRASSE_2 = 'de:08116:2912:0:4'  # Rtg Schulzentrum/Lindenschule (Ostfildern Nellingen)
	NELLINGEN_RIEGELSTRASSE = 'de:08116:2912'
	NELLINGEN_RIEGELSTRASSE_1 = 'de:08116:2912:0:3'  # Rtg Hindenburgstr. (Ostfildern Nellingen)
	NELLINGEN_RIEGELSTRASSE_2 = 'de:08116:2912:0:4'  # Rtg Schulzentrum/Lindenschule (Ostfildern Nellingen)
	RUIT_MEDIUS_KLINIK = 'de:08116:2913:0:3'  # Linie 131 Rtg Im Kapf (Ostfildern Ruit)
	MEDIUS_KLINIK_1 = 'de:08116:2913:0:3'  # Linie 131 Rtg Im Kapf (Ostfildern Ruit)
	MEDIUS_KLINIK_2 = 'de:08116:4265:0:3'  # Rtg Innenstadt (Kirchheim unter Teck Kirchheim (T))
	MEDIUS_KLINIK_3 = 'de:08116:4265:0:4'  # Rtg Ötlingen (Kirchheim unter Teck Kirchheim (T))
	MEDIUS_KLINIK_4 = 'de:08116:4445:0:3'  # Ankunft (Nürtingen Nürtingen)
	MEDIUS_KLINIK_5 = 'de:08116:4445:0:4'  # eRtg Säerstraße (Nürtingen Nürtingen)
	SCHARNHÄUSER_STRASSE = 'de:08116:2914'
	SCHARNHÄUSER_STRASSE_1 = 'de:08116:2914:0:3'  # Rtg Ruit Kirche (Ostfildern Ruit)
	SCHARNHÄUSER_STRASSE_2 = 'de:08116:2914:0:4'  # Rtg Scharnhausen Gestütshof (Ostfildern Ruit)
	RUIT_SCHARNHÄUSER_STR = 'de:08116:2914'
	RUIT_SCHARNHÄUSER_STR_1 = 'de:08116:2914:0:3'  # Rtg Ruit Kirche (Ostfildern Ruit)
	RUIT_SCHARNHÄUSER_STR_2 = 'de:08116:2914:0:4'  # Rtg Scharnhausen Gestütshof (Ostfildern Ruit)
	SCHARNHAUSEN_GESTÜTSHOF = 'de:08116:2915'
	SCHARNHAUSEN_GESTÜTSHOF_1 = 'de:08116:2915:0:3'  # Rtg Ruit Scharnhäuser Str. (Ostfildern Scharnhausen)
	SCHARNHAUSEN_GESTÜTSHOF_2 = 'de:08116:2915:0:4'  # Rtg Rathaus (Ostfildern Scharnhausen)
	GESTÜTSHOF = 'de:08116:2915'
	GESTÜTSHOF_1 = 'de:08116:2915:0:3'  # Rtg Ruit Scharnhäuser Str. (Ostfildern Scharnhausen)
	GESTÜTSHOF_2 = 'de:08116:2915:0:4'  # Rtg Rathaus (Ostfildern Scharnhausen)
	SCHARNHAUSEN_PLIENINGER_STR = 'de:08116:2916'
	SCHARNHAUSEN_PLIENINGER_STR_1 = 'de:08116:2916:1:3'  # Rtg Rathaus/Gestütshof (Ostfildern Scharnhausen)
	SCHARNHAUSEN_PLIENINGER_STR_2 = 'de:08116:2916:1:4'  # Rtg Seemühlenweg (Ostfildern Scharnhausen)
	SCHARNHAUSEN_RATHAUS = 'de:08116:2917'
	SCHARNHAUSEN_RATHAUS_1 = 'de:08116:2917:1:1'  # Rtg Plieninger Str. (Ostfildern Scharnhausen)
	SCHARNHAUSEN_RATHAUS_2 = 'de:08116:2917:1:2'  # Rtg Neuhausen Zabergäustr. (Ostfildern Scharnhausen)
	SCHARNHAUSEN_RATHAUS_3 = 'de:08116:2917:2:3'  # Rtg Lenzhalde (Ostfildern Scharnhausen)
	SCHARNHAUSEN_RATHAUS_4 = 'de:08116:2917:2:4'  # Rtg Plieninger Str. (Ostfildern Scharnhausen)
	WOLFSCHLUGEN_ESSLINGER_STR = 'de:08116:2918'
	WOLFSCHLUGEN_ESSLINGER_STR_1 = 'de:08116:2918:0:3'  # Rtg Löwen (Wolfschlugen Wolfschlugen)
	WOLFSCHLUGEN_ESSLINGER_STR_2 = 'de:08116:2918:0:4'  # Rtg Sielmingen/Neuhausen (Wolfschlugen Wolfschlugen)
	WOLFSCHLUGEN_LÖWEN = 'de:08116:2919'
	WOLFSCHLUGEN_LÖWEN_1 = 'de:08116:2919:0:3'  # Rtg Reinhardtstr. (Wolfschlugen Wolfschlugen)
	WOLFSCHLUGEN_LÖWEN_2 = 'de:08116:2919:0:4'  # Rtg Esslinger Str. (Wolfschlugen Wolfschlugen)
	LÖWEN_1 = 'de:08116:2919:0:3'  # Rtg Reinhardtstr. (Wolfschlugen Wolfschlugen)
	LÖWEN_2 = 'de:08116:2919:0:4'  # Rtg Esslinger Str. (Wolfschlugen Wolfschlugen)
	LÖWEN_3 = 'de:08117:13:0:H1'  # Rtg Schillerstr./Gingen (Kuchen Kuchen (F))
	LÖWEN_4 = 'de:08117:4032:0:R1'  # Rtg Untere Bruck (Börtlingen Börtlingen)
	REINHARDTSTRASSE = 'de:08116:2920'
	REINHARDTSTRASSE_1 = 'de:08116:2920:0:3'  # Rtg Siedlung (Wolfschlugen Wolfschlugen)
	REINHARDTSTRASSE_2 = 'de:08116:2920:0:4'  # Rtg Löwen (Wolfschlugen Wolfschlugen)
	WOLFSCHLUGEN_REINHARDTSTR = 'de:08116:2920'
	WOLFSCHLUGEN_REINHARDTSTR_1 = 'de:08116:2920:0:3'  # Rtg Siedlung (Wolfschlugen Wolfschlugen)
	WOLFSCHLUGEN_REINHARDTSTR_2 = 'de:08116:2920:0:4'  # Rtg Löwen (Wolfschlugen Wolfschlugen)
	WOLFSCHLUGEN_SIEDLUNG = 'de:08116:2921'
	WOLFSCHLUGEN_SIEDLUNG_1 = 'de:08116:2921:1:3'  # Rtg Hardt (Wolfschlugen Wolfschlugen)
	WOLFSCHLUGEN_SIEDLUNG_2 = 'de:08116:2921:1:4'  # Rtg Reinhardtstr. (Wolfschlugen Wolfschlugen)
	SCHLOSSERSTRASSE = 'de:08116:2922'
	SCHLOSSERSTRASSE_1 = 'de:08116:2922:0:3'  # Rtg Zabergäustraße (Neuhausen auf den Fildern Neuhausen (F))
	SCHLOSSERSTRASSE_2 = 'de:08116:2922:0:4'  # Rtg Traube (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__SCHLOSSERSTR = 'de:08116:2922'
	NEUHAUSEN__F__SCHLOSSERSTR_1 = 'de:08116:2922:0:3'  # Rtg Zabergäustraße (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__SCHLOSSERSTR_2 = 'de:08116:2922:0:4'  # Rtg Traube (Neuhausen auf den Fildern Neuhausen (F))
	HARDT = 'de:08116:2923'
	HARDT_1 = 'de:08116:2923:0:3'  # Rtg Oberens. Grötzinger Str. (Nürtingen Hardt)
	HARDT_2 = 'de:08116:2923:0:4'  # Rtg Wolfschlugen (Nürtingen Hardt)
	GRÖTZINGER_STRASSE = 'de:08116:2924'
	GRÖTZINGER_STRASSE_1 = 'de:08116:2924:1:3'  # Rtg Lindenplatz (Nürtingen Oberensingen)
	GRÖTZINGER_STRASSE_2 = 'de:08116:2924:1:4'  # Rtg Hardt (Nürtingen Oberensingen)
	GRÖTZINGER_STRASSE_3 = 'de:08116:2924:2:5'  # Rtg Hauffstraße (Nürtingen Oberensingen)
	GRÖTZINGER_STRASSE_4 = 'de:08116:2924:2:6'  # Rtg Lindenplatz (Nürtingen Oberensingen)
	OBERENSINGEN_GRÖTZINGER_STR = 'de:08116:2924'
	OBERENSINGEN_GRÖTZINGER_STR_1 = 'de:08116:2924:1:3'  # Rtg Lindenplatz (Nürtingen Oberensingen)
	OBERENSINGEN_GRÖTZINGER_STR_2 = 'de:08116:2924:1:4'  # Rtg Hardt (Nürtingen Oberensingen)
	OBERENSINGEN_GRÖTZINGER_STR_3 = 'de:08116:2924:2:5'  # Rtg Hauffstraße (Nürtingen Oberensingen)
	OBERENSINGEN_GRÖTZINGER_STR_4 = 'de:08116:2924:2:6'  # Rtg Lindenplatz (Nürtingen Oberensingen)
	OBERENSINGEN_LINDENPLATZ = 'de:08116:2925'
	OBERENSINGEN_LINDENPLATZ_1 = 'de:08116:2925:0:3'  # Rtg Cafe Stoll (Nürtingen Oberensingen)
	OBERENSINGEN_LINDENPLATZ_2 = 'de:08116:2925:0:4'  # Rtg Grötzinger Str. (Nürtingen Oberensingen)
	LINDENPLATZ_1 = 'de:08116:2925:0:3'  # Rtg Cafe Stoll (Nürtingen Oberensingen)
	LINDENPLATZ_2 = 'de:08116:2925:0:4'  # Rtg Grötzinger Str. (Nürtingen Oberensingen)
	LINDENPLATZ_3 = 'de:08119:3706:0:3'  # Rtg Rohrbronn/Winterbach (Remshalden Hebsack)
	LINDENPLATZ_4 = 'de:08119:3706:0:4'  # Rtg Markgraben (Remshalden Hebsack)
	LINDENPLATZ_5 = 'de:08116:4442:0:3'  # eRtg Schule/Kohlberg (Neuffen Neuffen)
	LINDENPLATZ_6 = 'de:08116:4442:0:4'  # eRtg Neuffen (Neuffen Neuffen)
	LINDENPLATZ_7 = 'de:08119:5030:0:3'  # eRtg Rathaus/Welzheimer Straße (Weissach im Tal Unterweissach)
	LINDENPLATZ_8 = 'de:08119:5030:0:4'  # eRtg Täferhalde/Edeka (Weissach im Tal Unterweissach)
	LINDENPLATZ_9 = 'de:08119:6046:0:3'  # eRtg Dorfhalle (Backnang Steinbach)
	LINDENPLATZ_10 = 'de:08119:6046:0:4'  # eRtg Sommerrain (Backnang Steinbach)
	CAFE_STOLL = 'de:08116:2926'
	CAFE_STOLL_1 = 'de:08116:2926:0:3'  # Rtg Nürtingen (Nürtingen Oberensingen)
	CAFE_STOLL_2 = 'de:08116:2926:0:4'  # Rtg Lindenplatz (Nürtingen Oberensingen)
	OBERENSINGEN_CAFE_STOLL = 'de:08116:2926'
	OBERENSINGEN_CAFE_STOLL_1 = 'de:08116:2926:0:3'  # Rtg Nürtingen (Nürtingen Oberensingen)
	OBERENSINGEN_CAFE_STOLL_2 = 'de:08116:2926:0:4'  # Rtg Lindenplatz (Nürtingen Oberensingen)
	NÜRTINGEN_NECKARBRÜCKE = 'de:08116:2927'
	NÜRTINGEN_NECKARBRÜCKE_1 = 'de:08116:2927:1:3'  # Rtg Mühlstr. (Nürtingen Nürtingen)
	NÜRTINGEN_NECKARBRÜCKE_2 = 'de:08116:2927:1:4'  # Rtg Oberensingen (Nürtingen Nürtingen)
	NÜRTINGEN_NECKARBRÜCKE_3 = 'de:08116:2927:2:5'  # Rtg Neckarhausen (Nürtingen Nürtingen)
	NÜRTINGEN_NECKARBRÜCKE_4 = 'de:08116:2927:2:6'  # Rtg Nürtingen (Nürtingen Nürtingen)
	NECKARBRÜCKE_1 = 'de:08116:2927:1:3'  # Rtg Mühlstr. (Nürtingen Nürtingen)
	NECKARBRÜCKE_2 = 'de:08116:2927:1:4'  # Rtg Oberensingen (Nürtingen Nürtingen)
	NECKARBRÜCKE_3 = 'de:08116:2927:2:5'  # Rtg Neckarhausen (Nürtingen Nürtingen)
	NECKARBRÜCKE_4 = 'de:08116:2927:2:6'  # Rtg Nürtingen (Nürtingen Nürtingen)
	NECKARBRÜCKE_5 = 'de:08118:3418:0:3'  # Rtg H.-Hesse (Ludwigsburg Ludwigsburg)
	NECKARBRÜCKE_6 = 'de:08118:3418:0:4'  # Rtg ZOB (Ludwigsburg Ludwigsburg)
	NECKARBRÜCKE_7 = 'de:08116:6860:0:3'  # eRtg Tübinger Straße (Neckartailfingen Neckartailfingen)
	NECKARBRÜCKE_8 = 'de:08116:6860:0:4'  # eRtg Schule (Neckartailfingen Neckartailfingen)
	NÜRT__ALBSTR___SIEDLERSTUBE = 'de:08116:2929'
	NÜRT__ALBSTR___SIEDLERSTUBE_1 = 'de:08116:2929:0:3'  # Rtg Amtsgericht (Nürtingen Nürtingen)
	NÜRT__ALBSTR___SIEDLERSTUBE_2 = 'de:08116:2929:0:4'  # Rtg Eichendorffstr. (Nürtingen Nürtingen)
	ALBSTRASSE__SIEDLERSTUBE = 'de:08116:2929'
	ALBSTRASSE__SIEDLERSTUBE_1 = 'de:08116:2929:0:3'  # Rtg Amtsgericht (Nürtingen Nürtingen)
	ALBSTRASSE__SIEDLERSTUBE_2 = 'de:08116:2929:0:4'  # Rtg Eichendorffstr. (Nürtingen Nürtingen)
	NÜRTINGEN = 'de:08116:2931'
	NÜRTINGEN_1 = 'de:08116:2931:1:1'  # Gleis 1 (Nürtingen Nürtingen)
	NÜRTINGEN_2 = 'de:08116:2931:2:2'  # Gleis 2 (Nürtingen Nürtingen)
	NÜRTINGEN_3 = 'de:08116:2931:2:3'  # Gleis 3 (Nürtingen Nürtingen)
	NÜRTINGEN_4 = 'de:08116:2931:3:1'  # Bstg 1 (Nürtingen Nürtingen)
	NÜRTINGEN_5 = 'de:08116:2931:3:10'  # Bstg 10 (Nürtingen Nürtingen)
	NÜRTINGEN_6 = 'de:08116:2931:3:12'  # Bstg 12 (Nürtingen Nürtingen)
	NÜRTINGEN_7 = 'de:08116:2931:3:2'  # Bstg 2 (Nürtingen Nürtingen)
	NÜRTINGEN_8 = 'de:08116:2931:3:3'  # Bstg 3 (Nürtingen Nürtingen)
	NÜRTINGEN_9 = 'de:08116:2931:3:4'  # Bstg 4 (Nürtingen Nürtingen)
	NÜRTINGEN_10 = 'de:08116:2931:3:5'  # Bstg 5 (Nürtingen Nürtingen)
	NÜRTINGEN_11 = 'de:08116:2931:3:6'  # Bstg 6 (Nürtingen Nürtingen)
	NÜRTINGEN_12 = 'de:08116:2931:3:7'  # Bstg 7 (Nürtingen Nürtingen)
	NÜRTINGEN_13 = 'de:08116:2931:3:8'  # Bstg 8 (Nürtingen Nürtingen)
	NÜRTINGEN_14 = 'de:08116:2931:3:9'  # Bstg 9 (Nürtingen Nürtingen)
	NÜRTINGEN_15 = 'de:08116:2931:3:99'  # Ankunft (Nürtingen Nürtingen)
	RUIT_KIRCHE = 'de:08116:2932'
	RUIT_KIRCHE_1 = 'de:08116:2932:0:3'  # Rtg Ruit (Ostfildern Ruit)
	RUIT_KIRCHE_2 = 'de:08116:2932:0:4'  # L.35 Rtg Scharnh.Str./L.131 Rtg Zinsholz (Ostfildern Ruit)
	RUIT_IM_KAPF = 'de:08116:2933'
	RUIT_IM_KAPF_1 = 'de:08116:2933:0:3'  # Rtg Krankenhaus (Ostfildern Ruit)
	RUIT_IM_KAPF_2 = 'de:08116:2933:0:4'  # Rtg Ruit (Ostfildern Ruit)
	IM_KAPF = 'de:08116:2933'
	IM_KAPF_1 = 'de:08116:2933:0:3'  # Rtg Krankenhaus (Ostfildern Ruit)
	IM_KAPF_2 = 'de:08116:2933:0:4'  # Rtg Ruit (Ostfildern Ruit)
	HERZOG_PHILIPP_PLATZ = 'de:08116:2935'
	HERZOG_PHILIPP_PLATZ_1 = 'de:08116:2935:0:3'  # Rtg Parkstr. (Ostfildern Parksiedlung)
	HERZOG_PHILIPP_PLATZ_2 = 'de:08116:2935:0:4'  # Rtg R.-Koch-Str. (Ostfildern Parksiedlung)
	PARKS__HERZOG_PHILIPP_PLATZ = 'de:08116:2935'
	PARKS__HERZOG_PHILIPP_PLATZ_1 = 'de:08116:2935:0:3'  # Rtg Parkstr. (Ostfildern Parksiedlung)
	PARKS__HERZOG_PHILIPP_PLATZ_2 = 'de:08116:2935:0:4'  # Rtg R.-Koch-Str. (Ostfildern Parksiedlung)
	ROBERT_KOCH_STRASSE_1 = 'de:08116:2936:0:3'  # Rtg R.-Koch-Str./Scharnhauser Park (Ostfildern Parksiedlung)
	ROBERT_KOCH_STRASSE_2 = 'de:08116:2936:0:4'  # Rtg Zollberg Mutzenreis (Ostfildern Parksiedlung)
	ROBERT_KOCH_STRASSE_3 = 'de:08118:7642:0:3'  # Rtg Auf dem Wasen (Ludwigsburg Ludwigsburg)
	PARKS__ROBERT_KOCH_STRASSE = 'de:08116:2936'
	PARKS__ROBERT_KOCH_STRASSE_1 = 'de:08116:2936:0:3'  # Rtg R.-Koch-Str./Scharnhauser Park (Ostfildern Parksiedlung)
	PARKS__ROBERT_KOCH_STRASSE_2 = 'de:08116:2936:0:4'  # Rtg Zollberg Mutzenreis (Ostfildern Parksiedlung)
	STEINEN = 'de:08116:2937:0:4'  # Rtg Paulinenstr. (Ostfildern Nellingen)
	NELLINGEN_STEINEN = 'de:08116:2937:0:4'  # Rtg Paulinenstr. (Ostfildern Nellingen)
	ZABERGÄUSTRASSE_1 = 'de:08116:2939:0:3'  # Rtg Schlosserstraße (Neuhausen auf den Fildern Neuhausen (F))
	ZABERGÄUSTRASSE_2 = 'de:08116:2939:0:4'  # Rtg Scharnhausen Rathaus (Neuhausen auf den Fildern Neuhausen (F))
	ZABERGÄUSTRASSE_3 = 'de:08111:6266:0:4'  # Rtg Friedhof (Stuttgart Stuttgart)
	NEUHAUSEN__F__ZABERGÄUSTR = 'de:08116:2939'
	NEUHAUSEN__F__ZABERGÄUSTR_1 = 'de:08116:2939:0:3'  # Rtg Schlosserstraße (Neuhausen auf den Fildern Neuhausen (F))
	NEUHAUSEN__F__ZABERGÄUSTR_2 = 'de:08116:2939:0:4'  # Rtg Scharnhausen Rathaus (Neuhausen auf den Fildern Neuhausen (F))
	ROSSDORF_HOLBEINSTRASSE = 'de:08116:2940'
	ROSSDORF_HOLBEINSTRASSE_1 = 'de:08116:2940:0:3'  # Rtg Berliner Str./Dürerstr. (Nürtingen Nürtingen)
	ROSSDORF_HOLBEINSTRASSE_2 = 'de:08116:2940:0:4'  # Rtg Liebermannstr./Dürerstr. (Nürtingen Nürtingen)
	NÜRT__ROSSDORF_HOLBEINSTRASSE = 'de:08116:2940'
	NÜRT__ROSSDORF_HOLBEINSTRASSE_1 = 'de:08116:2940:0:3'  # Rtg Berliner Str./Dürerstr. (Nürtingen Nürtingen)
	NÜRT__ROSSDORF_HOLBEINSTRASSE_2 = 'de:08116:2940:0:4'  # Rtg Liebermannstr./Dürerstr. (Nürtingen Nürtingen)
	NÜRTINGEN_BAUHOF = 'de:08116:2941'
	NÜRTINGEN_BAUHOF_1 = 'de:08116:2941:0:3'  # eRtg Weberstraße (Nürtingen Nürtingen)
	NÜRTINGEN_BAUHOF_2 = 'de:08116:2941:0:4'  # eRtg Realschule (Nürtingen Nürtingen)
	BAUHOF_1 = 'de:08116:2941:0:3'  # eRtg Weberstraße (Nürtingen Nürtingen)
	BAUHOF_2 = 'de:08116:2941:0:4'  # eRtg Realschule (Nürtingen Nürtingen)
	BAUHOF_3 = 'de:08117:1035:0:H1'  # Rtg Stadtwerke (Göppingen Göppingen)
	TISCHARDT_AUTMUT = 'de:08116:2943'
	TISCHARDT_AUTMUT_1 = 'de:08116:2943:0:3'  # Rtg Rathaus (Frickenhausen Tischardt)
	TISCHARDT_AUTMUT_2 = 'de:08116:2943:0:4'  # Rtg Großbettl. Metzinger Str. (Frickenhausen Tischardt)
	AUTMUT = 'de:08116:2943'
	AUTMUT_1 = 'de:08116:2943:0:3'  # Rtg Rathaus (Frickenhausen Tischardt)
	AUTMUT_2 = 'de:08116:2943:0:4'  # Rtg Großbettl. Metzinger Str. (Frickenhausen Tischardt)
	OBERENSINGEN_SCHULE = 'de:08116:2948:0:3'  # eRtg Lindenplatz (Nürtingen Oberensingen)
	HARDT_FUCHSRAIN = 'de:08116:2949:0:3'  # eRtg Hans-von-Hutten-Platz (Nürtingen Hardt)
	HANS_VON_HUTTEN_PLATZ = 'de:08116:2950:0:3'  # eRtg Kullenwiesen (Nürtingen Hardt)
	HARDT_HANS_VON_HUTTEN_PLATZ = 'de:08116:2950:0:3'  # eRtg Kullenwiesen (Nürtingen Hardt)
	KULLENWIESEN = 'de:08116:2951:0:4'  # eRtg Hauffstraße (Nürtingen Hardt)
	HARDT_KULLENWIESEN = 'de:08116:2951:0:4'  # eRtg Hauffstraße (Nürtingen Hardt)
	OBERENSINGEN_HAUFFSTRASSE = 'de:08116:2952'
	OBERENSINGEN_HAUFFSTRASSE_1 = 'de:08116:2952:0:3'  # eRtg Nürtinger Straße (Nürtingen Oberensingen)
	OBERENSINGEN_HAUFFSTRASSE_2 = 'de:08116:2952:0:4'  # eRtg Grötzinger Straße (Nürtingen Oberensingen)
	KROATENHOF = 'de:08116:2953'
	KROATENHOF_1 = 'de:08116:2953:0:3'  # eRtg Schillerplatz (Nürtingen Nürtingen)
	KROATENHOF_2 = 'de:08116:2953:0:4'  # eRtg Metzinger Straße (Nürtingen Nürtingen)
	NÜRTINGEN_KROATENHOF = 'de:08116:2953'
	NÜRTINGEN_KROATENHOF_1 = 'de:08116:2953:0:3'  # eRtg Schillerplatz (Nürtingen Nürtingen)
	NÜRTINGEN_KROATENHOF_2 = 'de:08116:2953:0:4'  # eRtg Metzinger Straße (Nürtingen Nürtingen)
	NEUENHAUS_IM_VOGELSANG = 'de:08116:2962'
	NEUENHAUS_IM_VOGELSANG_1 = 'de:08116:2962:0:3'  # eRtg Häfnerstraße (Aichtal Neuenhaus)
	NEUENHAUS_IM_VOGELSANG_2 = 'de:08116:2962:0:4'  # eRtg Liebenau (Aichtal Neuenhaus)
	IM_VOGELSANG_1 = 'de:08116:2962:0:3'  # eRtg Häfnerstraße (Aichtal Neuenhaus)
	IM_VOGELSANG_2 = 'de:08116:2962:0:4'  # eRtg Liebenau (Aichtal Neuenhaus)
	IM_VOGELSANG_3 = 'de:08115:7042:0:3'  # eRtg Zeisigweg (Herrenberg Herrenberg)
	HUMPFENTALSTRASSE = 'de:08116:2963'
	HUMPFENTALSTRASSE_1 = 'de:08116:2963:0:3'  # eRtg Enzenhardtplatz  / Humpfentalstraße (Nürtingen Nürtingen)
	HUMPFENTALSTRASSE_2 = 'de:08116:2963:0:4'  # eRtg Roßdorf Berliner Straße (Nürtingen Nürtingen)
	NÜRTINGEN_HUMPFENTALSTRASSE = 'de:08116:2963'
	NÜRTINGEN_HUMPFENTALSTRASSE_1 = 'de:08116:2963:0:3'  # eRtg Enzenhardtplatz  / Humpfentalstraße (Nürtingen Nürtingen)
	NÜRTINGEN_HUMPFENTALSTRASSE_2 = 'de:08116:2963:0:4'  # eRtg Roßdorf Berliner Straße (Nürtingen Nürtingen)
	ROSSDORF_LIEBERMANNSTRASSE = 'de:08116:2965:0:3'  # eRtg Roßdorf Holbeinstraße (Nürtingen Nürtingen)
	NÜRT__ROSSDORF_LIEBERMANNSTR = 'de:08116:2965:0:3'  # eRtg Roßdorf Holbeinstraße (Nürtingen Nürtingen)
	NÜRTINGEN_BAHNHOFSTRASSE = 'de:08116:2966:1:3'  # Rtg Schillerplatz/Weberstr. (Nürtingen Nürtingen)
	PAULINENSTRASSE = 'de:08116:2967'
	PAULINENSTRASSE_1 = 'de:08116:2967:1:3'  # Rtg Zollberg Mutzenreisstr. (Ostfildern Nellingen)
	PAULINENSTRASSE_2 = 'de:08116:2967:1:4'  # Rtg Hindenburgstr. (Ostfildern Nellingen)
	NELLINGEN_PAULINENSTRASSE = 'de:08116:2967'
	NELLINGEN_PAULINENSTRASSE_1 = 'de:08116:2967:1:3'  # Rtg Zollberg Mutzenreisstr. (Ostfildern Nellingen)
	NELLINGEN_PAULINENSTRASSE_2 = 'de:08116:2967:1:4'  # Rtg Hindenburgstr. (Ostfildern Nellingen)
	RUIT = 'de:08116:2968'
	RUIT_1 = 'de:08116:2968:1:1'  # Rtg Ruit Zinsholz (Ostfildern Ruit)
	RUIT_2 = 'de:08116:2968:1:2'  # Rtg Heumaden (Ostfildern Ruit)
	RUIT_3 = 'de:08116:2968:2:3'  # Rtg Ruit Im Kapf (Ostfildern Ruit)
	RUIT_4 = 'de:08116:2968:2:4'  # Rtg Ruit Kirche/Ruit Horbstr. (Ostfildern Ruit)
	RUIT_ZINSHOLZ = 'de:08116:2969'
	RUIT_ZINSHOLZ_1 = 'de:08116:2969:1:1'  # Rtg Parksiedlung (Ostfildern Ruit)
	RUIT_ZINSHOLZ_2 = 'de:08116:2969:1:2'  # Rtg Ruit (Ostfildern Ruit)
	RUIT_ZINSHOLZ_3 = 'de:08116:2969:2:3'  # Rtg Ruit Kirche (Ostfildern Ruit)
	RUIT_ZINSHOLZ_4 = 'de:08116:2969:2:4'  # Rtg Parkstr./Parksiedlung R.-K.-Str. (Ostfildern Ruit)
	ZINSHOLZ = 'de:08116:2969'
	ZINSHOLZ_1 = 'de:08116:2969:1:1'  # Rtg Parksiedlung (Ostfildern Ruit)
	ZINSHOLZ_2 = 'de:08116:2969:1:2'  # Rtg Ruit (Ostfildern Ruit)
	ZINSHOLZ_3 = 'de:08116:2969:2:3'  # Rtg Ruit Kirche (Ostfildern Ruit)
	ZINSHOLZ_4 = 'de:08116:2969:2:4'  # Rtg Parkstr./Parksiedlung R.-K.-Str. (Ostfildern Ruit)
	PARKSIEDLUNG = 'de:08116:2970'
	PARKSIEDLUNG_1 = 'de:08116:2970:1:1'  # Rtg Scharnhauser Park (Ostfildern Parksiedlung)
	PARKSIEDLUNG_2 = 'de:08116:2970:1:2'  # Rtg Ruit Zinsholz (Ostfildern Parksiedlung)
	SCHARNHAUSER_PARK = 'de:08116:2971'
	SCHARNHAUSER_PARK_1 = 'de:08116:2971:1:1'  # Rtg Kreuzbrunnen (Ostfildern Scharnhauser Park)
	SCHARNHAUSER_PARK_2 = 'de:08116:2971:1:2'  # Rtg Parksiedlung (Ostfildern Scharnhauser Park)
	SCHARNHAUSER_PARK_3 = 'de:08116:2971:2:3'  # Linie 122 Rtg Kreuzbrunnen (Ostfildern Scharnhauser Park)
	SCHARNHAUSER_PARK_4 = 'de:08116:2971:2:4'  # Linie 122 Rtg Robert-Koch-Str. (Ostfildern Scharnhauser Park)
	SCHARNHAUSER_PARK_5 = 'de:08116:2971:3:5'  # Rtg Ruit Zinsholz (Ostfildern Scharnhauser Park)
	SCHARNHAUSER_PARK_KREUZBRUNNEN = 'de:08116:2972'
	SCHARNHAUSER_PARK_KREUZBRUNNEN_1 = 'de:08116:2972:1:1'  # Rtg Nellingen Technische Akademie (Ostfildern Scharnhauser Park)
	SCHARNHAUSER_PARK_KREUZBRUNNEN_2 = 'de:08116:2972:1:2'  # Rtg Scharnhauser Park (Ostfildern Scharnhauser Park)
	SCHARNHAUSER_PARK_KREUZBRUNNEN_3 = 'de:08116:2972:2:3'  # Linie 122 Rtg Sonnenhalde (Ostfildern Scharnhauser Park)
	SCHARNHAUSER_PARK_KREUZBRUNNEN_4 = 'de:08116:2972:2:4'  # L.120 Rtg Riegelstr./L.122 Rtg Sch.Park (Ostfildern Scharnhauser Park)
	KREUZBRUNNEN = 'de:08116:2972'
	KREUZBRUNNEN_1 = 'de:08116:2972:1:1'  # Rtg Nellingen Technische Akademie (Ostfildern Scharnhauser Park)
	KREUZBRUNNEN_2 = 'de:08116:2972:1:2'  # Rtg Scharnhauser Park (Ostfildern Scharnhauser Park)
	KREUZBRUNNEN_3 = 'de:08116:2972:2:3'  # Linie 122 Rtg Sonnenhalde (Ostfildern Scharnhauser Park)
	KREUZBRUNNEN_4 = 'de:08116:2972:2:4'  # L.120 Rtg Riegelstr./L.122 Rtg Sch.Park (Ostfildern Scharnhauser Park)
	TECHNISCHE_AKADEMIE = 'de:08116:2973'
	TECHNISCHE_AKADEMIE_1 = 'de:08116:2973:0:1'  # Rtg Nellingen Ostfildern (Ostfildern Nellingen)
	TECHNISCHE_AKADEMIE_2 = 'de:08116:2973:0:2'  # Rtg Kreuzbrunnen (Ostfildern Nellingen)
	NELLINGEN_TECHNISCHE_AKADEMIE = 'de:08116:2973'
	NELLINGEN_TECHNISCHE_AKADEMIE_1 = 'de:08116:2973:0:1'  # Rtg Nellingen Ostfildern (Ostfildern Nellingen)
	NELLINGEN_TECHNISCHE_AKADEMIE_2 = 'de:08116:2973:0:2'  # Rtg Kreuzbrunnen (Ostfildern Nellingen)
	NELLINGEN_OSTFILDERN = 'de:08116:2974'
	NELLINGEN_OSTFILDERN_1 = 'de:08116:2974:1:1'  # Stadtbahn - Endhst. (Ostfildern Nellingen)
	NELLINGEN_OSTFILDERN_2 = 'de:08116:2974:2:4'  # L.119 Rtg Nussweg/L.120 Rtg Hindenburgst (Ostfildern Nellingen)
	OSTFILDERN = 'de:08116:2974'
	OSTFILDERN_1 = 'de:08116:2974:1:1'  # Stadtbahn - Endhst. (Ostfildern Nellingen)
	OSTFILDERN_2 = 'de:08116:2974:2:4'  # L.119 Rtg Nussweg/L.120 Rtg Hindenburgst (Ostfildern Nellingen)
	PLIENINGEN_SEEMÜHLENWEG = 'de:08111:2976'
	PLIENINGEN_SEEMÜHLENWEG_1 = 'de:08111:2976:0:3'  # Rtg Scharnhausen Plieninger Str (Stuttgart Stuttgart)
	PLIENINGEN_SEEMÜHLENWEG_2 = 'de:08111:2976:0:4'  # Rtg Scharnhauser Brücke (Stuttgart Stuttgart)
	PLIENINGEN_SEEMÜHLENWEG_3 = 'de:08111:2976:0:5'  # Pos 3 Rtg Seemühlenweg Pos 1 (Stuttgart Stuttgart)
	BEUREN_BÜRGERHAUS = 'de:08116:2980'
	BEUREN_BÜRGERHAUS_1 = 'de:08116:2980:0:3'  # eRtg Molkerei (Beuren Beuren)
	BEUREN_BÜRGERHAUS_2 = 'de:08116:2980:0:4'  # eRtg Freilichtmuseum (Beuren Beuren)
	BÜRGERHAUS_1 = 'de:08116:2980:0:3'  # eRtg Molkerei (Beuren Beuren)
	BÜRGERHAUS_2 = 'de:08116:2980:0:4'  # eRtg Freilichtmuseum (Beuren Beuren)
	BÜRGERHAUS_3 = 'de:08115:3215:0:3'  # Rtg Nord (Sindelfingen Maichingen)
	BÜRGERHAUS_4 = 'de:08115:3215:0:4'  # Rtg Süd (Sindelfingen Maichingen)
	BÜRGERHAUS_5 = 'de:08117:2061:0:H1'  # Rtg Kreuzung (Aichelberg Aichelberg (GP))
	KAISERSTRASSE_1 = 'de:08116:2981:0:3'  # Rtg Hindenburgstr. (Ostfildern Nellingen)
	KAISERSTRASSE_2 = 'de:08116:2981:0:4'  # Rtg Denkendorfer Str. (Ostfildern Nellingen)
	KAISERSTRASSE_3 = 'de:08119:3764:0:3'  # Rtg. Nordhaldenstr. (Weinstadt Beutelsbach)
	KAISERSTRASSE_4 = 'de:08119:3764:0:4'  # Rtg. Poststr. (Weinstadt Beutelsbach)
	NELLINGEN_KAISERSTRASSE = 'de:08116:2981'
	NELLINGEN_KAISERSTRASSE_1 = 'de:08116:2981:0:3'  # Rtg Hindenburgstr. (Ostfildern Nellingen)
	NELLINGEN_KAISERSTRASSE_2 = 'de:08116:2981:0:4'  # Rtg Denkendorfer Str. (Ostfildern Nellingen)
	STETTINER_STRASSE_1 = 'de:08116:2983:0:3'  # Rtg Ostpreußenweg (Nürtingen Nürtingen)
	STETTINER_STRASSE_2 = 'de:08116:2983:0:4'  # Rtg Tischardter Str./Enzenhardtplatz (Nürtingen Nürtingen)
	STETTINER_STRASSE_3 = 'de:08115:6744:0:3'  # eRtg Parkstraße (Landratsamt) (Böblingen Böblingen)
	STETTINER_STRASSE_4 = 'de:08118:6958:0:3'  # eRtg Hermann-Essig-Straße (Schwieberdingen Schwieberdingen)
	NÜRTINGEN_STETTINER_STR = 'de:08116:2983'
	NÜRTINGEN_STETTINER_STR_1 = 'de:08116:2983:0:3'  # Rtg Ostpreußenweg (Nürtingen Nürtingen)
	NÜRTINGEN_STETTINER_STR_2 = 'de:08116:2983:0:4'  # Rtg Tischardter Str./Enzenhardtplatz (Nürtingen Nürtingen)
	EIBENWEG = 'de:08116:2984'
	EIBENWEG_1 = 'de:08116:2984:0:3'  # eRtg Dammstraße (Nürtingen Nürtingen)
	EIBENWEG_2 = 'de:08116:2984:0:4'  # eRtg Auf dem Lerchenberg (Nürtingen Nürtingen)
	NÜRTINGEN_EIBENWEG = 'de:08116:2984'
	NÜRTINGEN_EIBENWEG_1 = 'de:08116:2984:0:3'  # eRtg Dammstraße (Nürtingen Nürtingen)
	NÜRTINGEN_EIBENWEG_2 = 'de:08116:2984:0:4'  # eRtg Auf dem Lerchenberg (Nürtingen Nürtingen)
	VENDELAUSTRASSE = 'de:08116:2985:0:3'  # eRtg Owen (Nürtingen Nürtingen)
	NÜRTINGEN_VENDELAUSTR = 'de:08116:2985:0:3'  # eRtg Owen (Nürtingen Nürtingen)
	OTTO_HAHN_STRASSE_1 = 'de:08116:2986:0:3'  # Rtg Steinbeisweg (Nürtingen Nürtingen)
	OTTO_HAHN_STRASSE_2 = 'de:08116:3953:0:3'  # eRtg Willi-Bleicher-Straße (Kirchheim unter Teck Kirchheim (T))
	OTTO_HAHN_STRASSE_3 = 'de:08116:3953:0:4'  # eRtg Bohnau (Kirchheim unter Teck Kirchheim (T))
	OTTO_HAHN_STRASSE_4 = 'de:08118:6847:0:3'  # eRtg Hermann-Hesse-Straße (Ludwigsburg Neckarweihingen)
	OTTO_HAHN_STRASSE_5 = 'de:08118:6847:0:4'  # eRtg Neckarbrücke (Ludwigsburg Neckarweihingen)
	NÜRTINGEN_OTTO_HAHN_STR = 'de:08116:2986:0:3'  # Rtg Steinbeisweg (Nürtingen Nürtingen)
	STEINBEISWEG = 'de:08116:2987:0:3'  # eRtg Otto-Hahn-Straße (Nürtingen Nürtingen)
	NÜRTINGEN_STEINBEISWEG = 'de:08116:2987:0:3'  # eRtg Otto-Hahn-Straße (Nürtingen Nürtingen)
	NÜRTINGEN_BF__POST = 'de:08116:2988:0:3'  # Rtg ZOB (Nürtingen Nürtingen)
	BAHNHOF__POST = 'de:08116:2988:0:3'  # Rtg ZOB (Nürtingen Nürtingen)
	NÜRTINGEN_TISCHARDTER_STR = 'de:08116:2990'
	NÜRTINGEN_TISCHARDTER_STR_1 = 'de:08116:2990:0:3'  # Rtg Stettiner Str. (Nürtingen Nürtingen)
	NÜRTINGEN_TISCHARDTER_STR_2 = 'de:08116:2990:0:4'  # Rtg Roßd. Berliner Str. (Nürtingen Nürtingen)
	TISCHARDTER_STRASSE_1 = 'de:08116:2990:0:3'  # Rtg Stettiner Str. (Nürtingen Nürtingen)
	TISCHARDTER_STRASSE_2 = 'de:08116:2990:0:4'  # Rtg Roßd. Berliner Str. (Nürtingen Nürtingen)
	TISCHARDTER_STRASSE_3 = 'de:08116:3820:0:3'  # eRtg Tischardt Rathaus (Kohlberg Kohlberg)
	TISCHARDTER_STRASSE_4 = 'de:08116:3820:0:4'  # eRtg Kohlberg Rathaus (Kohlberg Kohlberg)
	GROSSER_FORST = 'de:08116:2991'
	GROSSER_FORST_1 = 'de:08116:2991:0:3'  # Rtg Ostpreußenweg (Nürtingen Nürtingen)
	GROSSER_FORST_2 = 'de:08116:2991:0:4'  # Rtg Metzinger Str. (Nürtingen Nürtingen)
	NÜRTINGEN_GROSSER_FORST = 'de:08116:2991'
	NÜRTINGEN_GROSSER_FORST_1 = 'de:08116:2991:0:3'  # Rtg Ostpreußenweg (Nürtingen Nürtingen)
	NÜRTINGEN_GROSSER_FORST_2 = 'de:08116:2991:0:4'  # Rtg Metzinger Str. (Nürtingen Nürtingen)
	LISE_MEITNER_STR = 'de:08116:2993:0:3'  # Rtg Bunsenstr. (Nürtingen Oberensingen)
	OBERENSINGEN_LISE_MEITNER_STR = 'de:08116:2993:0:3'  # Rtg Bunsenstr. (Nürtingen Oberensingen)
	BUNSENSTR = 'de:08116:2994:0:3'  # Rtg Nürtingen ZOB (Nürtingen Oberensingen)
	OBERENSINGEN_BUNSENSTR = 'de:08116:2994:0:3'  # Rtg Nürtingen ZOB (Nürtingen Oberensingen)
	NÜRTINGEN_IM_GÄNSLESGRUND = 'de:08116:2995'
	NÜRTINGEN_IM_GÄNSLESGRUND_1 = 'de:08116:2995:0:3'  # Rtg Tischardter Str. (Nürtingen Nürtingen)
	NÜRTINGEN_IM_GÄNSLESGRUND_2 = 'de:08116:2995:0:4'  # Rtg Stettiner Str. (Nürtingen Nürtingen)
	IM_GÄNSLESGRUND = 'de:08116:2995'
	IM_GÄNSLESGRUND_1 = 'de:08116:2995:0:3'  # Rtg Tischardter Str. (Nürtingen Nürtingen)
	IM_GÄNSLESGRUND_2 = 'de:08116:2995:0:4'  # Rtg Stettiner Str. (Nürtingen Nürtingen)
	HOHES_GESTADE = 'de:08116:2996'
	HOHES_GESTADE_1 = 'de:08116:2996:0:3'  # Rtg Oberensingen (Nürtingen Nürtingen)
	HOHES_GESTADE_2 = 'de:08116:2996:0:4'  # Rtg Bauhof (Nürtingen Nürtingen)
	NÜRTINGEN_HOHES_GESTADE = 'de:08116:2996'
	NÜRTINGEN_HOHES_GESTADE_1 = 'de:08116:2996:0:3'  # Rtg Oberensingen (Nürtingen Nürtingen)
	NÜRTINGEN_HOHES_GESTADE_2 = 'de:08116:2996:0:4'  # Rtg Bauhof (Nürtingen Nürtingen)
	SCHELLINGSTRASSE_1 = 'de:08116:2998:0:1'  # Rtg Steinbeisweg/st.auswärts (Nürtingen Nürtingen)
	SCHELLINGSTRASSE_2 = 'de:08116:2998:0:2'  # Rtg Kirchheimer Str./Bf (Nürtingen Nürtingen)
	SCHELLINGSTRASSE_3 = 'de:08119:7092:0:3'  # eRtg Mendelssohnstraße (Waiblingen Waiblingen)
	NÜRTINGEN_SCHELLINGSTRASSE = 'de:08116:2998'
	NÜRTINGEN_SCHELLINGSTRASSE_1 = 'de:08116:2998:0:1'  # Rtg Steinbeisweg/st.auswärts (Nürtingen Nürtingen)
	NÜRTINGEN_SCHELLINGSTRASSE_2 = 'de:08116:2998:0:2'  # Rtg Kirchheimer Str./Bf (Nürtingen Nürtingen)
	AUTENSTRASSE = 'de:08118:3000'
	AUTENSTRASSE_1 = 'de:08118:3000:0:3'  # eRtg Hirschlander Straße (Ditzingen Ditzingen)
	AUTENSTRASSE_2 = 'de:08118:3000:0:4'  # eRtg Ditzingen (Ditzingen Ditzingen)
	DITZINGEN_AUTENSTRASSE = 'de:08118:3000'
	DITZINGEN_AUTENSTRASSE_1 = 'de:08118:3000:0:3'  # eRtg Hirschlander Straße (Ditzingen Ditzingen)
	DITZINGEN_AUTENSTRASSE_2 = 'de:08118:3000:0:4'  # eRtg Ditzingen (Ditzingen Ditzingen)
	DITZINGEN_GLEMSAUE = 'de:08118:3001:0:3'  # eRtg Ditzingen (Ditzingen Ditzingen)
	GLEMSAUE = 'de:08118:3001:0:3'  # eRtg Ditzingen (Ditzingen Ditzingen)
	HIRSCHLANDEN_PUMPSTATION = 'de:08118:3002'
	HIRSCHLANDEN_PUMPSTATION_1 = 'de:08118:3002:0:3'  # Rtg Ditzingen (Ditzingen Hirschlanden)
	HIRSCHLANDEN_PUMPSTATION_2 = 'de:08118:3002:0:4'  # Rtg Schöckingen (Ditzingen Hirschlanden)
	PUMPSTATION = 'de:08118:3002'
	PUMPSTATION_1 = 'de:08118:3002:0:3'  # Rtg Ditzingen (Ditzingen Hirschlanden)
	PUMPSTATION_2 = 'de:08118:3002:0:4'  # Rtg Schöckingen (Ditzingen Hirschlanden)
	HIRSCHLANDEN_RATHAUSPLATZ = 'de:08118:3003'
	HIRSCHLANDEN_RATHAUSPLATZ_1 = 'de:08118:3003:1:3'  # Rtg Turnhalle (Ditzingen Hirschlanden)
	HIRSCHLANDEN_RATHAUSPLATZ_2 = 'de:08118:3003:1:4'  # Rtg Nord/Pumpstation (Ditzingen Hirschlanden)
	RATHAUSPLATZ = 'de:08118:3003'
	RATHAUSPLATZ_1 = 'de:08118:3003:1:3'  # Rtg Turnhalle (Ditzingen Hirschlanden)
	RATHAUSPLATZ_2 = 'de:08118:3003:1:4'  # Rtg Nord/Pumpstation (Ditzingen Hirschlanden)
	TURNHALLE_1 = 'de:08118:3004:0:3'  # Rtg Kreisverkehr (Ditzingen Hirschlanden)
	TURNHALLE_2 = 'de:08118:3004:0:4'  # Rtg Rathausplatz (Ditzingen Hirschlanden)
	TURNHALLE_3 = 'de:08118:3516:0:3'  # Rtg Affalterbach (Erdmannhausen Erdmannhausen)
	TURNHALLE_4 = 'de:08118:3516:0:4'  # Rtg Rathaus (Erdmannhausen Erdmannhausen)
	TURNHALLE_5 = 'de:08119:5122:0:3'  # eRtg Michelau (Rudersberg Steinenberg)
	TURNHALLE_6 = 'de:08117:50:0:R1'  # Rtg Deggingen (Deggingen Reichenbach im Täle (Deggingen))
	TURNHALLE_7 = 'de:08117:2029:0:R1'  # Rtg Uditorium (Uhingen Uhingen)
	HIRSCHLANDEN_TURNHALLE = 'de:08118:3004'
	HIRSCHLANDEN_TURNHALLE_1 = 'de:08118:3004:0:3'  # Rtg Kreisverkehr (Ditzingen Hirschlanden)
	HIRSCHLANDEN_TURNHALLE_2 = 'de:08118:3004:0:4'  # Rtg Rathausplatz (Ditzingen Hirschlanden)
	SCHÖCKINGEN_ROSE = 'de:08118:3005'
	SCHÖCKINGEN_ROSE_1 = 'de:08118:3005:0:3'  # Rtg Hemmingen (Ditzingen Schöckingen)
	SCHÖCKINGEN_ROSE_2 = 'de:08118:3005:0:4'  # Rtg Kreisverkehr (Ditzingen Schöckingen)
	ROSE = 'de:08118:3005'
	ROSE_1 = 'de:08118:3005:0:3'  # Rtg Hemmingen (Ditzingen Schöckingen)
	ROSE_2 = 'de:08118:3005:0:4'  # Rtg Kreisverkehr (Ditzingen Schöckingen)
	SCHLOSS_1 = 'de:08118:3006:0:3'  # Rtg Silcherstr. (Ditzingen Schöckingen)
	SCHLOSS_2 = 'de:08115:3119:1:3'  # Rtg Bonholz (Waldenbuch Waldenbuch)
	SCHLOSS_3 = 'de:08115:3119:1:4'  # Richtung Gänswiese/Steinenbr. Goldäcker (Waldenbuch Waldenbuch)
	SCHLOSS_4 = 'de:08119:6692:0:3'  # eRtg Banrain (Urbach Urbach)
	SCHLOSS_5 = 'de:08119:6692:0:4'  # Rtg Wittumhalle (Urbach Urbach)
	SCHÖCKINGEN_SCHLOSS = 'de:08118:3006:0:3'  # Rtg Silcherstr. (Ditzingen Schöckingen)
	SCHÖCKINGEN_SILCHERSTR = 'de:08118:3007:1:3'  # Rtg Rose (Ditzingen Schöckingen)
	SILCHERSTRASSE_1 = 'de:08118:3007:1:3'  # Rtg Rose (Ditzingen Schöckingen)
	SILCHERSTRASSE_2 = 'de:08115:4850:0:3'  # Rtg Nebringen (Herrenberg Herrenberg)
	SILCHERSTRASSE_3 = 'de:08115:4850:0:4'  # Rtg Bahnhof (Herrenberg Herrenberg)
	SILCHERSTRASSE_4 = 'de:08119:5402:0:3'  # eRtg Ulrichstraße (Weinstadt Schnait)
	SILCHERSTRASSE_5 = 'de:08117:8004:0:H1'  # Rtg Siedlung (Ebersbach an der Fils Bünzwangen)
	SILCHERSTRASSE_6 = 'de:08117:8004:0:R1'  # Rtg Pläckertstr. (Ebersbach an der Fils Bünzwangen)
	SCHÖCKINGEN_RITTERSTR = 'de:08118:3008:0:3'  # eRtg Rose (Ditzingen Schöckingen)
	RITTERSTRASSE = 'de:08118:3008:0:3'  # eRtg Rose (Ditzingen Schöckingen)
	MARKT__BAUERNSTRASSE = 'de:08118:3009'
	MARKT__BAUERNSTRASSE_1 = 'de:08118:3009:0:3'  # eRtg Autenstraße (Ditzingen Ditzingen)
	MARKT__BAUERNSTRASSE_2 = 'de:08118:3009:0:4'  # eRtg Münchinger Straße (Ditzingen Ditzingen)
	DITZINGEN_MARKT__BAUERNSTR = 'de:08118:3009'
	DITZINGEN_MARKT__BAUERNSTR_1 = 'de:08118:3009:0:3'  # eRtg Autenstraße (Ditzingen Ditzingen)
	DITZINGEN_MARKT__BAUERNSTR_2 = 'de:08118:3009:0:4'  # eRtg Münchinger Straße (Ditzingen Ditzingen)
	DITZ__STADTBAD_ALTENWOHNANLAGE = 'de:08118:3010:0:3'  # eRtg Münchinger Straße (Ditzingen Ditzingen)
	STADTBAD_ALTENWOHNANLAGE = 'de:08118:3010:0:3'  # eRtg Münchinger Straße (Ditzingen Ditzingen)
	DITZ__GRÖNINGER_STR__BAUMHALDE = 'de:08118:3011:0:3'  # eRtg Gröninger-/Haldenstraße (Ditzingen Ditzingen)
	GRÖNINGER_STRASSE_BAUMHALDE = 'de:08118:3011:0:3'  # eRtg Gröninger-/Haldenstraße (Ditzingen Ditzingen)
	GRÖNINGER__HALDENSTRASSE = 'de:08118:3012:0:3'  # eRtg Korntaler-/Schubartstr/Maurener B (Ditzingen Ditzingen)
	DITZ__GRÖNINGER__HALDENSTR = 'de:08118:3012:0:3'  # eRtg Korntaler-/Schubartstr/Maurener B (Ditzingen Ditzingen)
	DITZ__KORNTALER__SCHUBARTSTR = 'de:08118:3014:0:3'  # eRtg Ditzenbrunner Straße (Ditzingen Ditzingen)
	KORNTALER__SCHUBARTSTRASSE = 'de:08118:3014:0:3'  # eRtg Ditzenbrunner Straße (Ditzingen Ditzingen)
	DITZ__DITZENBRUNNER__KNIELSTR = 'de:08118:3015:0:3'  # eRtg Hornbergstr./Lachebrückle (Ditzingen Ditzingen)
	DITZENBRUNNER__KNIELSTRASSE = 'de:08118:3015:0:3'  # eRtg Hornbergstr./Lachebrückle (Ditzingen Ditzingen)
	HORNBERGSTR__LACHEBRÜCKLE = 'de:08118:3016:0:3'  # eRtg Zollern-/Lichtensteinstraße (Ditzingen Ditzingen)
	DITZ__H_BERGSTR__LACHEBRÜCKLE = 'de:08118:3016:0:3'  # eRtg Zollern-/Lichtensteinstraße (Ditzingen Ditzingen)
	DITZ__ZOLLERN__L_STEINSTRASSE = 'de:08118:3017:0:3'  # eRtg Württembergstraße (Ditzingen Ditzingen)
	ZOLLERN__LICHTENSTEINSTRASSE = 'de:08118:3017:0:3'  # eRtg Württembergstraße (Ditzingen Ditzingen)
	AM_MAURENER_BERG = 'de:08118:3019:0:3'  # eRtg Bauern-/Breslauer Straße (Ditzingen Ditzingen)
	DITZINGEN_AM_MAURENER_BERG = 'de:08118:3019:0:3'  # eRtg Bauern-/Breslauer Straße (Ditzingen Ditzingen)
	BAUERN__BRESLAUER_STRASSE = 'de:08118:3020:0:3'  # eRtg Breslauer Straße/Kindergarten (Ditzingen Ditzingen)
	DITZ__BAUERN__BRESLAUER_STRASSE = 'de:08118:3020:0:3'  # eRtg Breslauer Straße/Kindergarten (Ditzingen Ditzingen)
	DITZ__BRESLAUER_STR__KINDERG = 'de:08118:3021:0:3'  # eRtg Autenstraße (Ditzingen Ditzingen)
	BRESLAUER_STRASSE_KINDERGARTEN = 'de:08118:3021:0:3'  # eRtg Autenstraße (Ditzingen Ditzingen)
	HIRSCHLANDER_STRASSE_1 = 'de:08118:3028:0:3'  # Rtg Hirschlanden (Ditzingen Ditzingen)
	HIRSCHLANDER_STRASSE_2 = 'de:08118:3028:0:4'  # Rtg Ditzingen (Ditzingen Ditzingen)
	HIRSCHLANDER_STRASSE_3 = 'de:08115:6050:0:3'  # Rtg Lachentor/Mercedesstr. (Leonberg Höfingen)
	HIRSCHLANDER_STRASSE_4 = 'de:08115:6050:0:4'  # Rtg Alter Bfweg/Rathaus (Leonberg Höfingen)
	DITZINGEN_HIRSCHLANDER_STR = 'de:08118:3028'
	DITZINGEN_HIRSCHLANDER_STR_1 = 'de:08118:3028:0:3'  # Rtg Hirschlanden (Ditzingen Ditzingen)
	DITZINGEN_HIRSCHLANDER_STR_2 = 'de:08118:3028:0:4'  # Rtg Ditzingen (Ditzingen Ditzingen)
	DITZINGEN_MÜNCHINGER_STR = 'de:08118:3029'
	DITZINGEN_MÜNCHINGER_STR_1 = 'de:08118:3029:0:3'  # eRtg Autenstraße (Ditzingen Ditzingen)
	DITZINGEN_MÜNCHINGER_STR_2 = 'de:08118:3029:0:4'  # eRtg Stiegelplatz (Ditzingen Ditzingen)
	MÜNCHINGER_STRASSE_1 = 'de:08118:3029:0:3'  # eRtg Autenstraße (Ditzingen Ditzingen)
	MÜNCHINGER_STRASSE_2 = 'de:08118:3029:0:4'  # eRtg Stiegelplatz (Ditzingen Ditzingen)
	MÜNCHINGER_STRASSE_3 = 'de:08118:3314:1:3'  # Pos 3 (Markgröningen Markgröningen)
	MÜNCHINGER_STRASSE_4 = 'de:08118:3314:2:4'  # Pos 4 (Markgröningen Markgröningen)
	MÜNCHINGER_STRASSE_5 = 'de:08118:6930:0:3'  # Rtg Schwieberdingen / Sägmühle (Hemmingen Hemmingen)
	MÜNCHINGER_STRASSE_6 = 'de:08118:6930:0:4'  # Rtg Hemmingen (Hemmingen Hemmingen)
	WÜRTTEMBERGSTRASSE_1 = 'de:08118:3030:0:3'  # eRtg Ditzingen (Ditzingen Ditzingen)
	WÜRTTEMBERGSTRASSE_2 = 'de:08116:6841:0:3'  # Rtg Mettingen / Hedelfingen (Esslingen am Neckar Weil)
	DITZINGEN_WÜRTTEMBERGSTRASSE = 'de:08118:3030:0:3'  # eRtg Ditzingen (Ditzingen Ditzingen)
	SCHÖCKINGEN_STEINBRUCH = 'de:08118:3031'
	SCHÖCKINGEN_STEINBRUCH_1 = 'de:08118:3031:0:3'  # eRtg Gewerbegebiet Süd (Ditzingen Schöckingen)
	SCHÖCKINGEN_STEINBRUCH_2 = 'de:08118:3031:0:4'  # eRtg Kreisverkehr (Ditzingen Schöckingen)
	STEINBRUCH = 'de:08118:3031'
	STEINBRUCH_1 = 'de:08118:3031:0:3'  # eRtg Gewerbegebiet Süd (Ditzingen Schöckingen)
	STEINBRUCH_2 = 'de:08118:3031:0:4'  # eRtg Kreisverkehr (Ditzingen Schöckingen)
	ZEISSSTRASSE = 'de:08118:3032:0:3'  # eRtg Industriegebiet Ost (Ditzingen Ditzingen)
	DITZINGEN_ZEISSSTRASSE = 'de:08118:3032:0:3'  # eRtg Industriegebiet Ost (Ditzingen Ditzingen)
	HERDWEG_1 = 'de:08118:3033:0:3'  # eRtg Zeissstraße (Ditzingen Ditzingen)
	HERDWEG_2 = 'de:08117:2100:0:H1'  # Rtg Linde (Göppingen Jebenhausen)
	HERDWEG_3 = 'de:08117:2100:0:R1'  # Rtg Göppingen (Göppingen Jebenhausen)
	DITZINGEN_HERDWEG = 'de:08118:3033:0:3'  # eRtg Zeissstraße (Ditzingen Ditzingen)
	FUCHSBAU_EINKAUFSMARKT = 'de:08118:3034:0:3'  # eRtg Autenstraße/Ditzingen (Ditzingen Ditzingen)
	DITZ__FUCHSBAU_EINKAUFSMARKT = 'de:08118:3034:0:3'  # eRtg Autenstraße/Ditzingen (Ditzingen Ditzingen)
	THALESPLATZ = 'de:08118:3035:0:3'  # eRtg Alle Richtungen (Ditzingen Ditzingen)
	DITZINGEN_THALESPLATZ = 'de:08118:3035:0:3'  # eRtg Alle Richtungen (Ditzingen Ditzingen)
	DORNIERSTRASSE_1 = 'de:08118:3036:0:1'  # Rtg Gewerbegebiet Süd (Ditzingen Ditzingen)
	DORNIERSTRASSE_2 = 'de:08118:3036:0:2'  # Rtg Ditzingen Bahnhof (Ditzingen Ditzingen)
	DORNIERSTRASSE_3 = 'de:08115:4721:0:3'  # eRtg Junkersstraße (Böblingen Böblingen)
	DORNIERSTRASSE_4 = 'de:08115:4721:0:4'  # eRtg Hulb (Böblingen Böblingen)
	DORNIERSTRASSE_5 = 'de:08116:6759:0:3'  # eRtg Körschtal (Esslingen am Neckar Sirnau)
	DORNIERSTRASSE_6 = 'de:08116:6759:0:4'  # eRtg Wolf-Hirth-Straße (Esslingen am Neckar Sirnau)
	DITZINGEN_DORNIERSTRASSE = 'de:08118:3036'
	DITZINGEN_DORNIERSTRASSE_1 = 'de:08118:3036:0:1'  # Rtg Gewerbegebiet Süd (Ditzingen Ditzingen)
	DITZINGEN_DORNIERSTRASSE_2 = 'de:08118:3036:0:2'  # Rtg Ditzingen Bahnhof (Ditzingen Ditzingen)
	KEMNAT_SCHÖNBERGSTR = 'de:08116:3050:0:5'  # Rtg alle Richtungen (Ostfildern Kemnat)
	SCHÖNBERGSTRASSE = 'de:08116:3050:0:5'  # Rtg alle Richtungen (Ostfildern Kemnat)
	BÖBLINGEN_ARBEITERZENTRUM = 'de:08115:3101:0:3'  # eRtg Friedrich-List-Platz/ZOB (Böblingen Böblingen)
	ARBEITERZENTRUM = 'de:08115:3101:0:3'  # eRtg Friedrich-List-Platz/ZOB (Böblingen Böblingen)
	THERMALBAD__STUTTGARTER_STR = 'de:08115:3102'
	THERMALBAD__STUTTGARTER_STR_1 = 'de:08115:3102:1:3'  # Rtg Freibad (Böblingen Böblingen)
	THERMALBAD__STUTTGARTER_STR_2 = 'de:08115:3102:1:4'  # Rtg Thermalbad (Böblingen Böblingen)
	BÖBL_THERMALBAD__STUTTG__STR = 'de:08115:3102'
	BÖBL_THERMALBAD__STUTTG__STR_1 = 'de:08115:3102:1:3'  # Rtg Freibad (Böblingen Böblingen)
	BÖBL_THERMALBAD__STUTTG__STR_2 = 'de:08115:3102:1:4'  # Rtg Thermalbad (Böblingen Böblingen)
	BÖBLINGEN_KARLSTRASSE = 'de:08115:3103:0:3'  # eRtg Parkstraße (Landratsamt) (Böblingen Böblingen)
	POSTPLATZ = 'de:08115:3104'
	POSTPLATZ_1 = 'de:08115:3104:0:1'  # Pos 1 (Böblingen Böblingen)
	POSTPLATZ_2 = 'de:08115:3104:0:2'  # Pos 2 (Böblingen Böblingen)
	POSTPLATZ_3 = 'de:08115:3104:0:3'  # Pos 3 (Böblingen Böblingen)
	BÖBLINGEN_POSTPLATZ = 'de:08115:3104'
	BÖBLINGEN_POSTPLATZ_1 = 'de:08115:3104:0:1'  # Pos 1 (Böblingen Böblingen)
	BÖBLINGEN_POSTPLATZ_2 = 'de:08115:3104:0:2'  # Pos 2 (Böblingen Böblingen)
	BÖBLINGEN_POSTPLATZ_3 = 'de:08115:3104:0:3'  # Pos 3 (Böblingen Böblingen)
	ELBENPLATZ = 'de:08115:3105:0:3'  # eRtg Klaffensteinstraße (Böblingen Böblingen)
	BÖBLINGEN_ELBENPLATZ = 'de:08115:3105:0:3'  # eRtg Klaffensteinstraße (Böblingen Böblingen)
	BÖBLINGEN_BRESLAUER_STR = 'de:08115:3106'
	BÖBLINGEN_BRESLAUER_STR_1 = 'de:08115:3106:0:3'  # eRtg Otto-Hahn-Gymnasium (Böblingen Böblingen)
	BÖBLINGEN_BRESLAUER_STR_2 = 'de:08115:3106:0:4'  # eRtg Kremser Str. (Böblingen Böblingen)
	BRESLAUER_STRASSE_1 = 'de:08115:3106:0:3'  # eRtg Otto-Hahn-Gymnasium (Böblingen Böblingen)
	BRESLAUER_STRASSE_2 = 'de:08115:3106:0:4'  # eRtg Kremser Str. (Böblingen Böblingen)
	BRESLAUER_STRASSE_3 = 'de:08118:6957:0:3'  # eRtg Stettiner Straße (Schwieberdingen Schwieberdingen)
	BRESLAUER_STRASSE_4 = 'de:08125:4676:0:1'  # eRtg Schmidhausen Abzweig (Beilstein Beilstein)
	BRESLAUER_STRASSE_5 = 'de:08125:4676:0:2'  # eRtg Apotheke (Beilstein Beilstein)
	BÖBLINGEN_FREIBAD = 'de:08115:3107'
	BÖBLINGEN_FREIBAD_1 = 'de:08115:3107:0:3'  # Rtg Thermalbad (Böblingen Böblingen)
	BÖBLINGEN_FREIBAD_2 = 'de:08115:3107:0:4'  # Rtg Silberweg/Stuttgarter Str. (Böblingen Böblingen)
	WEIL_I__S__NORD = 'de:08115:3108'
	WEIL_I__S__NORD_1 = 'de:08115:3108:0:3'  # Rtg See (Weil im Schönbuch Weil im Schönbuch)
	WEIL_I__S__NORD_2 = 'de:08115:3108:0:4'  # Rtg Breitenstein / Neuweiler (Weil im Schönbuch Weil im Schönbuch)
	BÖBLINGEN_WALDORFSCHULE = 'de:08115:3109:0:3'  # eRtg (Böblingen Böblingen)
	BÖBLINGEN_STUTTGARTER_STR = 'de:08115:3110'
	BÖBLINGEN_STUTTGARTER_STR_1 = 'de:08115:3110:0:3'  # eRtg Silberweg (Böblingen Böblingen)
	BÖBLINGEN_STUTTGARTER_STR_2 = 'de:08115:3110:0:4'  # eRtg Friedrich-List-Platz (Böblingen Böblingen)
	STUTTGARTER_STRASSE_1 = 'de:08115:3110:0:3'  # eRtg Silberweg (Böblingen Böblingen)
	STUTTGARTER_STRASSE_2 = 'de:08115:3110:0:4'  # eRtg Friedrich-List-Platz (Böblingen Böblingen)
	STUTTGARTER_STRASSE_3 = 'de:08118:3524:0:3'  # eRtg Freiberg (N) (Freiberg am Neckar Freiberg (N))
	STUTTGARTER_STRASSE_4 = 'de:08118:3524:0:4'  # eRtg Harteneckstraße (Freiberg am Neckar Freiberg (N))
	STUTTGARTER_STRASSE_5 = 'de:08116:4035:1:3'  # Rtg Weil/Nellingen/Schelztor (Esslingen am Neckar Pliensauvorstadt)
	STUTTGARTER_STRASSE_6 = 'de:08116:4035:1:4'  # Rtg Königsallee (Esslingen am Neckar Pliensauvorstadt)
	STUTTGARTER_STRASSE_7 = 'de:08116:4489:0:3'  # Rtg Neckartailfingen (Altdorf Altdorf (ES))
	STUTTGARTER_STRASSE_8 = 'de:08116:4489:0:4'  # Rtg Neckartenzlingen (Altdorf Altdorf (ES))
	STUTTGARTER_STRASSE_9 = 'de:08115:4746:0:3'  # Rtg Rathaus (Gäufelden Öschelbronn (Gäufelden))
	STUTTGARTER_STRASSE_10 = 'de:08115:4746:0:4'  # Rtg Nebringen (Gäufelden Öschelbronn (Gäufelden))
	STUTTGARTER_STRASSE_11 = 'de:08119:5315:0:3'  # eRtg Hauptstraße (Leutenbach Weiler zum Stein)
	STUTTGARTER_STRASSE_12 = 'de:08119:5315:0:4'  # eRtg Rathaus (Leutenbach Weiler zum Stein)
	STUTTGARTER_STRASSE_13 = 'de:08118:5865:0:3'  # eRtg Grabenstr. (Stadtm.) (Vaihingen an der Enz Vaihingen (E))
	STUTTGARTER_STRASSE_14 = 'de:08119:5974:0:4'  # eRtg Schorndorf (Schorndorf Schorndorf)
	STUTTGARTER_STRASSE_15 = 'de:08116:6495:0:3'  # eRtg Stuttgarter-/Hauptstraße (Reichenbach an der Fils Reichenbach (F))
	STUTTGARTER_STRASSE_16 = 'de:08116:6495:0:4'  # eRtg Ulmer Straße (Reichenbach an der Fils Reichenbach (F))
	STUTTGARTER_STRASSE_17 = 'de:08118:6948:0:3'  # eRtg Berliner Straße (Asperg Asperg)
	BÖBLINGEN_FRIEDRICH_LIST_PLATZ = 'de:08115:3112'
	BÖBLINGEN_FRIEDRICH_LIST_PLATZ_1 = 'de:08115:3112:0:3'  # Halt von Süd (Böblingen Böblingen)
	BÖBLINGEN_FRIEDRICH_LIST_PLATZ_2 = 'de:08115:3112:0:4'  # Halt von Ost (Böblingen Böblingen)
	FRIEDRICH_LIST_PLATZ = 'de:08115:3112'
	FRIEDRICH_LIST_PLATZ_1 = 'de:08115:3112:0:3'  # Halt von Süd (Böblingen Böblingen)
	FRIEDRICH_LIST_PLATZ_2 = 'de:08115:3112:0:4'  # Halt von Ost (Böblingen Böblingen)
	TÜBINGER_STRASSE_1 = 'de:08116:3113:0:3'  # eRtg Kelterrain (Leinfelden-Echterdingen Echterdingen)
	TÜBINGER_STRASSE_2 = 'de:08116:3113:0:4'  # eRtg Waldhornstraße (Leinfelden-Echterdingen Echterdingen)
	TÜBINGER_STRASSE_3 = 'de:08115:3128:1:3'  # Rtg Stettiner/Parkstr. (Böblingen Böblingen)
	TÜBINGER_STRASSE_4 = 'de:08115:3128:1:4'  # Rtg Schönbuchstr./Diezenhaldenstr. (Böblingen Böblingen)
	TÜBINGER_STRASSE_5 = 'de:08115:4839:0:3'  # eRtg Rathaus (Deckenpfronn Deckenpfronn)
	TÜBINGER_STRASSE_6 = 'de:08115:4839:0:4'  # eRtg Siemensstr. (Deckenpfronn Deckenpfronn)
	TÜBINGER_STRASSE_7 = 'de:08116:5820:0:3'  # eRtg Neckartailfingen (Neckartailfingen Neckartailfingen)
	TÜBINGER_STRASSE_8 = 'de:08116:5820:0:4'  # eRtg Neckarbrücke (Neckartailfingen Neckartailfingen)
	TÜBINGER_STRASSE_9 = 'de:08416:6352:0:1'  # Rtg Alte Post (Dettenhausen Dettenhausen)
	TÜBINGER_STRASSE_10 = 'de:08416:6352:0:4'  # Rtg Bf/Weil i.S. Kälberstelle/Pfrond. Sp (Dettenhausen Dettenhausen)
	ECHTERDINGEN_TÜBINGER_STR = 'de:08116:3113'
	ECHTERDINGEN_TÜBINGER_STR_1 = 'de:08116:3113:0:3'  # eRtg Kelterrain (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_TÜBINGER_STR_2 = 'de:08116:3113:0:4'  # eRtg Waldhornstraße (Leinfelden-Echterdingen Echterdingen)
	WALDENBUCH_LIEBENAU = 'de:08115:3118'
	WALDENBUCH_LIEBENAU_1 = 'de:08115:3118:0:3'  # Rtg Gänswiese (Waldenbuch Waldenbuch)
	WALDENBUCH_LIEBENAU_2 = 'de:08115:3118:0:4'  # Rtg Glashütte/Hallenbad (Waldenbuch Waldenbuch)
	WALDENBUCH_LIEBENAU_3 = 'de:08115:3118:0:5'  # Rtg Gänswiese (Waldenbuch Waldenbuch)
	LIEBENAU = 'de:08115:3118'
	LIEBENAU_1 = 'de:08115:3118:0:3'  # Rtg Gänswiese (Waldenbuch Waldenbuch)
	LIEBENAU_2 = 'de:08115:3118:0:4'  # Rtg Glashütte/Hallenbad (Waldenbuch Waldenbuch)
	LIEBENAU_3 = 'de:08115:3118:0:5'  # Rtg Gänswiese (Waldenbuch Waldenbuch)
	WALDENBUCH_SCHLOSS = 'de:08115:3119'
	WALDENBUCH_SCHLOSS_1 = 'de:08115:3119:1:3'  # Rtg Bonholz (Waldenbuch Waldenbuch)
	WALDENBUCH_SCHLOSS_2 = 'de:08115:3119:1:4'  # Richtung Gänswiese/Steinenbr. Goldäcker (Waldenbuch Waldenbuch)
	GLASHÜTTE = 'de:08115:3120'
	GLASHÜTTE_1 = 'de:08115:3120:0:3'  # Rtg Liebenau (Waldenbuch Waldenbuch)
	GLASHÜTTE_2 = 'de:08115:3120:0:4'  # Rtg Hallenbad (Waldenbuch Waldenbuch)
	WALDENBUCH_GLASHÜTTE = 'de:08115:3120'
	WALDENBUCH_GLASHÜTTE_1 = 'de:08115:3120:0:3'  # Rtg Liebenau (Waldenbuch Waldenbuch)
	WALDENBUCH_GLASHÜTTE_2 = 'de:08115:3120:0:4'  # Rtg Hallenbad (Waldenbuch Waldenbuch)
	WALDENBUCH_HALLENBAD = 'de:08115:3122'
	WALDENBUCH_HALLENBAD_1 = 'de:08115:3122:0:3'  # Rtg Glashütte/Liebenau (Waldenbuch Waldenbuch)
	WALDENBUCH_HALLENBAD_2 = 'de:08115:3122:0:4'  # Rtg Lerchenweg (Waldenbuch Waldenbuch)
	WALDENBUCH_HALLENBAD_3 = 'de:08115:3122:0:5'  # Rtg Austr. (Waldenbuch Waldenbuch)
	HALLENBAD_1 = 'de:08115:3122:0:3'  # Rtg Glashütte/Liebenau (Waldenbuch Waldenbuch)
	HALLENBAD_2 = 'de:08115:3122:0:4'  # Rtg Lerchenweg (Waldenbuch Waldenbuch)
	HALLENBAD_3 = 'de:08115:3122:0:5'  # Rtg Austr. (Waldenbuch Waldenbuch)
	HALLENBAD_4 = 'de:08115:3208:0:3'  # eRtg Sifi ZOB (Sindelfingen Maichingen)
	HALLENBAD_5 = 'de:08115:3208:0:4'  # eRtg Rathaus (Sindelfingen Maichingen)
	HALLENBAD_6 = 'de:08115:3233:0:3'  # am Hallenbad (Herrenberg Herrenberg)
	HALLENBAD_7 = 'de:08115:3233:0:4'  # Rtg Stadtfriedhof (Marienstraße) (Herrenberg Herrenberg)
	HALLENBAD_8 = 'de:08115:4692:1:3'  # eRtg Schönaicher First (Böblingen Böblingen)
	HALLENBAD_9 = 'de:08115:4692:1:4'  # eRtg Sporthalle (Böblingen Böblingen)
	HALLENBAD_10 = 'de:08118:5440:0:3'  # eRtg Realschule (Kornwestheim Kornwestheim)
	HALLENBAD_11 = 'de:08118:5440:0:4'  # eRtg Im Weizen (Kornwestheim Kornwestheim)
	HALLENBAD_12 = 'de:08118:7430:0:3'  # eRtg Bissinger Straße (Bietigheim-Bissingen Bissingen (LB))
	HALLENBAD_13 = 'de:08118:7430:0:4'  # eRtg Bietigheim (Bietigheim-Bissingen Bissingen (LB))
	HALLENBAD_14 = 'de:08117:1013:0:H1'  # Rtg Eislingen Bhf (Eislingen/Fils Eislingen (F))
	HALLENBAD_15 = 'de:08117:1013:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	WALDENBUCH_LERCHENWEG = 'de:08115:3123:0:4'  # Rtg Hasenhof (Waldenbuch Waldenbuch)
	LERCHENWEG = 'de:08115:3123:0:4'  # Rtg Hasenhof (Waldenbuch Waldenbuch)
	BÖBLINGEN_KLAFFENSTEINSTR = 'de:08115:3125'
	BÖBLINGEN_KLAFFENSTEINSTR_1 = 'de:08115:3125:0:3'  # eRtg Sporthalle (Böblingen Böblingen)
	BÖBLINGEN_KLAFFENSTEINSTR_2 = 'de:08115:3125:0:4'  # eRtg Postplatz (Böblingen Böblingen)
	KLAFFENSTEINSTRASSE = 'de:08115:3125'
	KLAFFENSTEINSTRASSE_1 = 'de:08115:3125:0:3'  # eRtg Sporthalle (Böblingen Böblingen)
	KLAFFENSTEINSTRASSE_2 = 'de:08115:3125:0:4'  # eRtg Postplatz (Böblingen Böblingen)
	PARKSTRASSE__LANDRATSAMT = 'de:08115:3126'
	PARKSTRASSE__LANDRATSAMT_1 = 'de:08115:3126:0:3'  # eRtg Böblingen (Böblingen Böblingen)
	PARKSTRASSE__LANDRATSAMT_2 = 'de:08115:3126:0:4'  # eRtg Tübinger Straße (Böblingen Böblingen)
	BÖBL__PARKSTR___LANDRATSAMT = 'de:08115:3126'
	BÖBL__PARKSTR___LANDRATSAMT_1 = 'de:08115:3126:0:3'  # eRtg Böblingen (Böblingen Böblingen)
	BÖBL__PARKSTR___LANDRATSAMT_2 = 'de:08115:3126:0:4'  # eRtg Tübinger Straße (Böblingen Böblingen)
	BÖBLINGEN_SCHÖNAICHER_STR = 'de:08115:3127'
	BÖBLINGEN_SCHÖNAICHER_STR_1 = 'de:08115:3127:0:3'  # Rtg Klaffensteinstr. (Böblingen Böblingen)
	BÖBLINGEN_SCHÖNAICHER_STR_2 = 'de:08115:3127:0:4'  # Rtg Hallenbad/Sporthalle (Böblingen Böblingen)
	SCHÖNAICHER_STRASSE = 'de:08115:3127'
	SCHÖNAICHER_STRASSE_1 = 'de:08115:3127:0:3'  # Rtg Klaffensteinstr. (Böblingen Böblingen)
	SCHÖNAICHER_STRASSE_2 = 'de:08115:3127:0:4'  # Rtg Hallenbad/Sporthalle (Böblingen Böblingen)
	BÖBLINGEN_TÜBINGER_STR = 'de:08115:3128'
	BÖBLINGEN_TÜBINGER_STR_1 = 'de:08115:3128:1:3'  # Rtg Stettiner/Parkstr. (Böblingen Böblingen)
	BÖBLINGEN_TÜBINGER_STR_2 = 'de:08115:3128:1:4'  # Rtg Schönbuchstr./Diezenhaldenstr. (Böblingen Böblingen)
	WALDENBUCH_ALTSTADT = 'de:08115:3129'
	WALDENBUCH_ALTSTADT_1 = 'de:08115:3129:0:3'  # Rtg Postamt (Waldenbuch Waldenbuch)
	WALDENBUCH_ALTSTADT_2 = 'de:08115:3129:0:4'  # Rtg Liebenau (Waldenbuch Waldenbuch)
	WALDENBUCH_STADION = 'de:08115:3130'
	WALDENBUCH_STADION_1 = 'de:08115:3130:0:3'  # Rtg Hallenbad (Waldenbuch Waldenbuch)
	WALDENBUCH_STADION_2 = 'de:08115:3130:0:4'  # Rtg Steinenbronn (Waldenbuch Waldenbuch)
	HASENHOF = 'de:08115:3131'
	HASENHOF_1 = 'de:08115:3131:0:3'  # Rtg Stadion (Waldenbuch Waldenbuch)
	HASENHOF_2 = 'de:08115:3131:0:4'  # Rtg Steinenbronn Kirche (Waldenbuch Waldenbuch)
	HASENHOF_3 = 'de:08115:3131:0:5'  # Hasenhof (Waldenbuch Waldenbuch)
	WALDENBUCH_HASENHOF = 'de:08115:3131'
	WALDENBUCH_HASENHOF_1 = 'de:08115:3131:0:3'  # Rtg Stadion (Waldenbuch Waldenbuch)
	WALDENBUCH_HASENHOF_2 = 'de:08115:3131:0:4'  # Rtg Steinenbronn Kirche (Waldenbuch Waldenbuch)
	WALDENBUCH_HASENHOF_3 = 'de:08115:3131:0:5'  # Hasenhof (Waldenbuch Waldenbuch)
	SCHÖNAICH_MAYBACHSTRASSE = 'de:08115:3133:0:4'  # eRtg (Schönaich Schönaich)
	SCHÖNAICH_FAULHABER = 'de:08115:3134:0:3'  # eRtg Stadion (Schönaich Schönaich)
	FAULHABER = 'de:08115:3134:0:3'  # eRtg Stadion (Schönaich Schönaich)
	THERMALBAD__HEXENBUCKEL = 'de:08115:3135:0:3'  # eRtg Thermalbad (Stuttgarter Str.) (Böblingen Böblingen)
	BÖBL__THERMALBAD__HEXENBUCKEL = 'de:08115:3135:0:3'  # eRtg Thermalbad (Stuttgarter Str.) (Böblingen Böblingen)
	BÖBLINGEN_KELTENBURGSTR = 'de:08115:3136:0:3'  # eRtg Diezenhalde (Zentrum) (Böblingen Böblingen)
	KELTENBURGSTRASSE = 'de:08115:3136:0:3'  # eRtg Diezenhalde (Zentrum) (Böblingen Böblingen)
	BÖBLINGEN_BREITENSTEINER_STR = 'de:08115:3137'
	BÖBLINGEN_BREITENSTEINER_STR_1 = 'de:08115:3137:0:3'  # Rtg Nord/Reiterhof (Böblingen Böblingen)
	BÖBLINGEN_BREITENSTEINER_STR_2 = 'de:08115:3137:0:4'  # Rtg Süd (Böblingen Böblingen)
	BREITENSTEINER_STRASSE = 'de:08115:3137'
	BREITENSTEINER_STRASSE_1 = 'de:08115:3137:0:3'  # Rtg Nord/Reiterhof (Böblingen Böblingen)
	BREITENSTEINER_STRASSE_2 = 'de:08115:3137:0:4'  # Rtg Süd (Böblingen Böblingen)
	BÖBLINGEN_MAIENPLATZ = 'de:08115:3138'
	BÖBLINGEN_MAIENPLATZ_1 = 'de:08115:3138:0:3'  # eRtg Waldburgstraße (Böblingen Böblingen)
	BÖBLINGEN_MAIENPLATZ_2 = 'de:08115:3138:0:4'  # Rtg West (Böblingen Böblingen)
	MAIENPLATZ = 'de:08115:3138'
	MAIENPLATZ_1 = 'de:08115:3138:0:3'  # eRtg Waldburgstraße (Böblingen Böblingen)
	MAIENPLATZ_2 = 'de:08115:3138:0:4'  # Rtg West (Böblingen Böblingen)
	BÖBL__DIEZENH___ZENTRUM__SÜD = 'de:08115:3139'
	BÖBL__DIEZENH___ZENTRUM__SÜD_1 = 'de:08115:3139:1:3'  # eRtg Diezenhalde (Zentrum) (Böblingen Böblingen)
	BÖBL__DIEZENH___ZENTRUM__SÜD_2 = 'de:08115:3139:1:4'  # eRtg Bühler Str. (Böblingen Böblingen)
	DIEZENHALDE__ZENTRUM__SÜD = 'de:08115:3139'
	DIEZENHALDE__ZENTRUM__SÜD_1 = 'de:08115:3139:1:3'  # eRtg Diezenhalde (Zentrum) (Böblingen Böblingen)
	DIEZENHALDE__ZENTRUM__SÜD_2 = 'de:08115:3139:1:4'  # eRtg Bühler Str. (Böblingen Böblingen)
	BÖBLINGEN_TECHNOLOGIEPARK = 'de:08115:3140'
	BÖBLINGEN_TECHNOLOGIEPARK_1 = 'de:08115:3140:0:3'  # eRtg Hanns-Klemm-Straße West (Böblingen Böblingen)
	BÖBLINGEN_TECHNOLOGIEPARK_2 = 'de:08115:3140:0:4'  # eRtg Schickardstraße (Böblingen Böblingen)
	TECHNOLOGIEPARK_1 = 'de:08115:3140:0:3'  # eRtg Hanns-Klemm-Straße West (Böblingen Böblingen)
	TECHNOLOGIEPARK_2 = 'de:08115:3140:0:4'  # eRtg Schickardstraße (Böblingen Böblingen)
	TECHNOLOGIEPARK_3 = 'de:08118:3531:0:3'  # eRtg Eichgraben (Marbach am Neckar Marbach (N))
	TECHNOLOGIEPARK_4 = 'de:08118:3531:0:4'  # eRtg Immanuel-Dornfeld.-Straße (Marbach am Neckar Marbach (N))
	LEONARDO_DA_VINCI_PLATZ = 'de:08115:3141'
	LEONARDO_DA_VINCI_PLATZ_1 = 'de:08115:3141:0:3'  # eRtg Goldberg Realschule (Böblingen Böblingen)
	LEONARDO_DA_VINCI_PLATZ_2 = 'de:08115:3141:0:4'  # Rtg Hanns-Klemm-Str. Ost (Böblingen Böblingen)
	BÖBL__LEONARDO_DA_VINCI_PL = 'de:08115:3141'
	BÖBL__LEONARDO_DA_VINCI_PL_1 = 'de:08115:3141:0:3'  # eRtg Goldberg Realschule (Böblingen Böblingen)
	BÖBL__LEONARDO_DA_VINCI_PL_2 = 'de:08115:3141:0:4'  # Rtg Hanns-Klemm-Str. Ost (Böblingen Böblingen)
	GANSSEE = 'de:08115:3142'
	GANSSEE_1 = 'de:08115:3142:0:3'  # eRtg Eichendorff-Schule (Böblingen Böblingen)
	GANSSEE_2 = 'de:08115:3142:0:4'  # Rtg Süd (Böblingen Böblingen)
	BÖBLINGEN_GANSSEE = 'de:08115:3142'
	BÖBLINGEN_GANSSEE_1 = 'de:08115:3142:0:3'  # eRtg Eichendorff-Schule (Böblingen Böblingen)
	BÖBLINGEN_GANSSEE_2 = 'de:08115:3142:0:4'  # Rtg Süd (Böblingen Böblingen)
	WEIL_I__S__WILHELMSTR = 'de:08115:3143'
	WEIL_I__S__WILHELMSTR_1 = 'de:08115:3143:0:3'  # eRtg Röte (Weil im Schönbuch Weil im Schönbuch)
	WEIL_I__S__WILHELMSTR_2 = 'de:08115:3143:0:4'  # eRtg See (Weil im Schönbuch Weil im Schönbuch)
	SEE__SEESTR = 'de:08115:3144:0:3'  # Rtg Nord (Weil im Schönbuch Weil im Schönbuch)
	WEIL_I__S__SEE__SEESTR = 'de:08115:3144:0:3'  # Rtg Nord (Weil im Schönbuch Weil im Schönbuch)
	BÖBLINGEN_FLUGFELD_ALLEE = 'de:08115:3145:0:4'  # Rtg Melli-Beese-Str. (Böblingen Böblingen)
	FLUGFELD_ALLEE = 'de:08115:3145:0:4'  # Rtg Melli-Beese-Str. (Böblingen Böblingen)
	KEILBERGSTRASSE = 'de:08115:3149'
	KEILBERGSTRASSE_1 = 'de:08115:3149:0:3'  # Rtg Reußensteinstr. (Böblingen Böblingen)
	KEILBERGSTRASSE_2 = 'de:08115:3149:0:4'  # Rtg Eichendorffschule (Böblingen Böblingen)
	BÖBLINGEN_KEILBERGSTRASSE = 'de:08115:3149'
	BÖBLINGEN_KEILBERGSTRASSE_1 = 'de:08115:3149:0:3'  # Rtg Reußensteinstr. (Böblingen Böblingen)
	BÖBLINGEN_KEILBERGSTRASSE_2 = 'de:08115:3149:0:4'  # Rtg Eichendorffschule (Böblingen Böblingen)
	BURKHARDTSMÜHLE = 'de:08115:3151'
	BURKHARDTSMÜHLE_1 = 'de:08115:3151:0:3'  # Rtg Liebenau (Waldenbuch Waldenbuch)
	BURKHARDTSMÜHLE_2 = 'de:08115:3151:0:4'  # Rtg Plattenhardt (Waldenbuch Waldenbuch)
	WALDENBUCH_BURKHARDTSMÜHLE = 'de:08115:3151'
	WALDENBUCH_BURKHARDTSMÜHLE_1 = 'de:08115:3151:0:3'  # Rtg Liebenau (Waldenbuch Waldenbuch)
	WALDENBUCH_BURKHARDTSMÜHLE_2 = 'de:08115:3151:0:4'  # Rtg Plattenhardt (Waldenbuch Waldenbuch)
	IHINGER_HOF = 'de:08115:3178'
	IHINGER_HOF_1 = 'de:08115:3178:0:1'  # Rtg Magstadt Ringstraße (Renningen Renningen)
	IHINGER_HOF_2 = 'de:08115:3178:0:2'  # Rtg Weil der Stadt Friedhof (Renningen Renningen)
	RENNINGEN_IHINGER_HOF = 'de:08115:3178'
	RENNINGEN_IHINGER_HOF_1 = 'de:08115:3178:0:1'  # Rtg Magstadt Ringstraße (Renningen Renningen)
	RENNINGEN_IHINGER_HOF_2 = 'de:08115:3178:0:2'  # Rtg Weil der Stadt Friedhof (Renningen Renningen)
	MAICHINGEN_BF__SCHLÜSSELÄCKER = 'de:08115:3179'
	MAICHINGEN_BF__SCHLÜSSELÄCKER_1 = 'de:08115:3179:0:3'  # Rtg Nord/Grünäckerstr. (Sindelfingen Maichingen)
	MAICHINGEN_BF__SCHLÜSSELÄCKER_2 = 'de:08115:3179:0:4'  # Rtg Süd/Teuchelweg (Sindelfingen Maichingen)
	BAHNHOF__SCHLÜSSELÄCKER = 'de:08115:3179'
	BAHNHOF__SCHLÜSSELÄCKER_1 = 'de:08115:3179:0:3'  # Rtg Nord/Grünäckerstr. (Sindelfingen Maichingen)
	BAHNHOF__SCHLÜSSELÄCKER_2 = 'de:08115:3179:0:4'  # Rtg Süd/Teuchelweg (Sindelfingen Maichingen)
	ALTES_RATHAUS_1 = 'de:08115:3180:0:3'  # Rtg Nord (Sindelfingen Maichingen)
	ALTES_RATHAUS_2 = 'de:08115:3180:0:4'  # Rtg Süd (Sindelfingen Maichingen)
	ALTES_RATHAUS_3 = 'de:08115:4798:0:3'  # eRtg Altdorfer Straße (Hildrizhausen Hildrizhausen)
	ALTES_RATHAUS_4 = 'de:08115:4798:0:4'  # eRtg IBM (Hildrizhausen Hildrizhausen)
	ALTES_RATHAUS_5 = 'de:08115:4863:0:3'  # Rtg Ev. Kirche Oberj. Gemeindezentrum (Jettingen Unterjettingen)
	ALTES_RATHAUS_6 = 'de:08115:4863:0:4'  # Rtg Kohlplatte / Kath. Kirche (Jettingen Unterjettingen)
	ALTES_RATHAUS_7 = 'de:08116:5017:0:3'  # Rtg Reutlinger Str. (Ostfildern Kemnat)
	ALTES_RATHAUS_8 = 'de:08116:5017:0:4'  # Rtg Rosenstr. (Ostfildern Kemnat)
	ALTES_RATHAUS_9 = 'de:08119:5156:0:3'  # Rtg Klingenstr. (Waiblingen Bittenfeld)
	ALTES_RATHAUS_10 = 'de:08119:5156:0:4'  # Rtg Am Zipfelbach (Waiblingen Bittenfeld)
	ALTES_RATHAUS_11 = 'de:08117:7666:0:H1'  # Rtg Böhmenkirch (Lauterstein Weißenstein)
	ALTES_RATHAUS_12 = 'de:08117:7666:0:R1'  # Rtg Nenningen (Lauterstein Weißenstein)
	ALTES_RATHAUS_13 = 'de:08416:6362:0:1'  # eRtg Schule (Dettenhausen Dettenhausen)
	ALTES_RATHAUS_14 = 'de:08416:6362:0:4'  # eRtg Turnhalle (Dettenhausen Dettenhausen)
	MAICHINGEN_ALTES_RATHAUS = 'de:08115:3180'
	MAICHINGEN_ALTES_RATHAUS_1 = 'de:08115:3180:0:3'  # Rtg Nord (Sindelfingen Maichingen)
	MAICHINGEN_ALTES_RATHAUS_2 = 'de:08115:3180:0:4'  # Rtg Süd (Sindelfingen Maichingen)
	BERLINER_STRASSE_1 = 'de:08115:3184:0:3'  # Rtg Allmendäcker (Sindelfingen Maichingen)
	BERLINER_STRASSE_2 = 'de:08115:3184:0:4'  # Rtg Hallenbad (Sindelfingen Maichingen)
	BERLINER_STRASSE_3 = 'de:08115:4513:0:3'  # Rtg Schießtäle ZOB Kalkofenstr. (Herrenberg Herrenberg)
	BERLINER_STRASSE_4 = 'de:08115:4513:0:4'  # Rtg Schulmeisterbuche (Herrenberg Herrenberg)
	BERLINER_STRASSE_5 = 'de:08115:4700:0:3'  # eRtg Königsberger Straße (Böblingen Böblingen)
	BERLINER_STRASSE_6 = 'de:08115:4700:0:4'  # eRtg Maurener Weg (Böblingen Böblingen)
	BERLINER_STRASSE_7 = 'de:08118:5657:0:3'  # eRtg Goethestraße (Remseck am Neckar Aldingen)
	BERLINER_STRASSE_8 = 'de:08118:5657:0:4'  # Rtg Pattonv. Realschule (Remseck am Neckar Aldingen)
	BERLINER_STRASSE_9 = 'de:08118:6947:0:3'  # eRtg Schäferstraße (Asperg Asperg)
	MAICHINGEN_BERLINER_STR = 'de:08115:3184'
	MAICHINGEN_BERLINER_STR_1 = 'de:08115:3184:0:3'  # Rtg Allmendäcker (Sindelfingen Maichingen)
	MAICHINGEN_BERLINER_STR_2 = 'de:08115:3184:0:4'  # Rtg Hallenbad (Sindelfingen Maichingen)
	TEUCHELWEG = 'de:08115:3185'
	TEUCHELWEG_1 = 'de:08115:3185:0:3'  # Rtg Maichingen Bf (Sindelfingen Maichingen)
	TEUCHELWEG_2 = 'de:08115:3185:0:4'  # Rtg Maichingen Ort (Sindelfingen Maichingen)
	MAICHINGEN_TEUCHELWEG = 'de:08115:3185'
	MAICHINGEN_TEUCHELWEG_1 = 'de:08115:3185:0:3'  # Rtg Maichingen Bf (Sindelfingen Maichingen)
	MAICHINGEN_TEUCHELWEG_2 = 'de:08115:3185:0:4'  # Rtg Maichingen Ort (Sindelfingen Maichingen)
	MALMSHEIM_SCHNALLENÄCKER = 'de:08115:3187'
	MALMSHEIM_SCHNALLENÄCKER_1 = 'de:08115:3187:0:3'  # Rtg Malmsheim (Renningen Malmsheim)
	MALMSHEIM_SCHNALLENÄCKER_2 = 'de:08115:3187:0:4'  # Rtg Renningen (Renningen Malmsheim)
	SCHNALLENÄCKER = 'de:08115:3187'
	SCHNALLENÄCKER_1 = 'de:08115:3187:0:3'  # Rtg Malmsheim (Renningen Malmsheim)
	SCHNALLENÄCKER_2 = 'de:08115:3187:0:4'  # Rtg Renningen (Renningen Malmsheim)
	AM_KINDELBERG = 'de:08115:3188:0:3'  # zum Wenden (Renningen Renningen)
	RENNINGEN_AM_KINDELBERG = 'de:08115:3188:0:3'  # zum Wenden (Renningen Renningen)
	RAITE = 'de:08115:3189'
	RAITE_1 = 'de:08115:3189:0:3'  # Rtg RBC (Renningen Renningen)
	RAITE_2 = 'de:08115:3189:0:4'  # Rtg Renningen (Renningen Renningen)
	RENNINGEN_RAITE = 'de:08115:3189'
	RENNINGEN_RAITE_1 = 'de:08115:3189:0:3'  # Rtg RBC (Renningen Renningen)
	RENNINGEN_RAITE_2 = 'de:08115:3189:0:4'  # Rtg Renningen (Renningen Renningen)
	BAHNHOF__INDUSTRIESTR = 'de:08115:3190'
	BAHNHOF__INDUSTRIESTR_1 = 'de:08115:3190:0:1'  # Rtg Keltenstraße (Renningen Renningen)
	BAHNHOF__INDUSTRIESTR_2 = 'de:08115:3190:0:2'  # Rtg Raite (Renningen Renningen)
	RENNINGEN_BF__INDUSTRIESTR = 'de:08115:3190'
	RENNINGEN_BF__INDUSTRIESTR_1 = 'de:08115:3190:0:1'  # Rtg Keltenstraße (Renningen Renningen)
	RENNINGEN_BF__INDUSTRIESTR_2 = 'de:08115:3190:0:2'  # Rtg Raite (Renningen Renningen)
	POST_1 = 'de:08115:3193:0:3'  # eRtg Renningen (Renningen Renningen)
	POST_2 = 'de:08115:3193:0:4'  # eRtg Ernst-Bauer-Platz (Renningen Renningen)
	POST_3 = 'de:08115:3232:0:3'  # eRtg Kirche (Böblingen Dagersheim)
	POST_4 = 'de:08116:4421:0:3'  # eRtg Schlatterhöhe B465 (Lenningen Gutenberg)
	POST_5 = 'de:08116:4421:0:4'  # eRtg Grünenbergstraße (Lenningen Gutenberg)
	POST_6 = 'de:08115:4748:0:3'  # Rtg Unterjettingen (Gäufelden Öschelbronn (Gäufelden))
	POST_7 = 'de:08115:4748:0:4'  # Rtg Rathaus (Gäufelden Öschelbronn (Gäufelden))
	POST_8 = 'de:08119:4899:0:3'  # eRtg Ortsmitte (Oppenweiler Oppenweiler)
	POST_9 = 'de:08119:5260:0:3'  # Rtg Buhlbronn (Schorndorf Schornbach)
	POST_10 = 'de:08119:5260:0:4'  # Rtg Schorndorf (Schorndorf Schornbach)
	POST_11 = 'de:08118:5707:1:3'  # eRtg Weißenhof Ost (Freudental Freudental)
	POST_12 = 'de:08118:5707:1:4'  # eRtg Wildparadies Tripsdrill/Staffel (Freudental Freudental)
	POST_13 = 'de:08118:5725:0:3'  # eRtg Gemeindehalle (Bönnigheim Hohenstein)
	POST_14 = 'de:08118:5725:0:4'  # Rtg Bönnigh. Burgermühle (Bönnigheim Hohenstein)
	POST_15 = 'de:08117:202:0:H1'  # Rtg Söhnstetten (Böhmenkirch Böhmenkirch)
	POST_16 = 'de:08117:202:0:R1'  # Rtg Lauterstein/Geislingen (Böhmenkirch Böhmenkirch)
	RENNINGEN_POST = 'de:08115:3193'
	RENNINGEN_POST_1 = 'de:08115:3193:0:3'  # eRtg Renningen (Renningen Renningen)
	RENNINGEN_POST_2 = 'de:08115:3193:0:4'  # eRtg Ernst-Bauer-Platz (Renningen Renningen)
	RENNINGEN_KELTENSTRASSE = 'de:08115:3194'
	RENNINGEN_KELTENSTRASSE_1 = 'de:08115:3194:0:5'  # Rtg Industriestr. (Renningen Renningen)
	RENNINGEN_KELTENSTRASSE_2 = 'de:08115:3194:0:6'  # Rtg Hummelbaum (Renningen Renningen)
	KELTENSTRASSE = 'de:08115:3194'
	KELTENSTRASSE_1 = 'de:08115:3194:0:5'  # Rtg Industriestr. (Renningen Renningen)
	KELTENSTRASSE_2 = 'de:08115:3194:0:6'  # Rtg Hummelbaum (Renningen Renningen)
	RENNINGEN_SÜD = 'de:08115:3195'
	RENNINGEN_SÜD_1 = 'de:08115:3195:1:1'  # Rtg Renningen (Renningen Renningen)
	RENNINGEN_SÜD_2 = 'de:08115:3195:2:3'  # Rtg Kirche (Renningen Renningen)
	RENNINGEN_SÜD_3 = 'de:08115:3195:2:4'  # Rtg Friedhof (Renningen Renningen)
	RENNINGEN_SÜD_4 = 'de:08115:3195:3:2'  # Rtg Böblingen (Renningen Renningen)
	MAGSTADT = 'de:08115:3196'
	MAGSTADT_1 = 'de:08115:3196:1:1'  # Rtg Renningen (Magstadt Magstadt)
	MAGSTADT_2 = 'de:08115:3196:2:2'  # Rtg Böblingen (Magstadt Magstadt)
	MAGSTADT_3 = 'de:08115:3196:3:3'  # Rtg Brauereiplatz (Magstadt Magstadt)
	MAICHINGEN_NORD = 'de:08115:3197'
	MAICHINGEN_NORD_1 = 'de:08115:3197:1:3'  # Rtg Landhaussiedlung (Sindelfingen Maichingen)
	MAICHINGEN_NORD_2 = 'de:08115:3197:1:4'  # Rtg Solo (Sindelfingen Maichingen)
	MAICHINGEN_NORD_3 = 'de:08115:3197:2:1'  # Gleis 1 (Sindelfingen Maichingen)
	MAICHINGEN_NORD_4 = 'de:08115:3197:2:2'  # Gleis 2 (Sindelfingen Maichingen)
	MAICHINGEN = 'de:08115:3198'
	MAICHINGEN_1 = 'de:08115:3198:1:1'  # Gleis 1 (Sindelfingen Maichingen)
	MAICHINGEN_2 = 'de:08115:3198:2:3'  # Pos 2 in Laurentiusstraße (Sindelfingen Maichingen)
	MAICHINGEN_3 = 'de:08115:3198:2:4'  # Pos 3 Talstraße ggü Bf (Sindelfingen Maichingen)
	MAICHINGEN_4 = 'de:08115:3198:2:5'  # Pos 1 Talstraße am Bahnhof (Sindelfingen Maichingen)
	SINDELFINGEN_ZOB = 'de:08115:3200'
	SINDELFINGEN_ZOB_1 = 'de:08115:3200:1:1'  # Bstg 1 (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZOB_2 = 'de:08115:3200:1:10'  # Rtg Daimlersteg (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZOB_3 = 'de:08115:3200:1:11'  # Bstg 11 (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZOB_4 = 'de:08115:3200:1:12'  # Bstg 12 (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZOB_5 = 'de:08115:3200:1:13'  # Bstg 13 (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZOB_6 = 'de:08115:3200:1:3'  # Bstg 3 (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZOB_7 = 'de:08115:3200:1:4'  # Bstg 4 (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZOB_8 = 'de:08115:3200:1:5'  # Bstg 5 (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZOB_9 = 'de:08115:3200:1:6'  # Bstg 6 (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZOB_10 = 'de:08115:3200:1:7'  # Bstg 7 (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZOB_11 = 'de:08115:3200:1:8'  # Bstg 8 (Sindelfingen Sindelfingen)
	SINDELFINGEN_ZOB_12 = 'de:08115:3200:1:9'  # Bstg 9 (Sindelfingen Sindelfingen)
	ZOB_1 = 'de:08115:3200:1:1'  # Bstg 1 (Sindelfingen Sindelfingen)
	ZOB_2 = 'de:08115:3200:1:10'  # Rtg Daimlersteg (Sindelfingen Sindelfingen)
	ZOB_3 = 'de:08115:3200:1:11'  # Bstg 11 (Sindelfingen Sindelfingen)
	ZOB_4 = 'de:08115:3200:1:12'  # Bstg 12 (Sindelfingen Sindelfingen)
	ZOB_5 = 'de:08115:3200:1:13'  # Bstg 13 (Sindelfingen Sindelfingen)
	ZOB_6 = 'de:08115:3200:1:3'  # Bstg 3 (Sindelfingen Sindelfingen)
	ZOB_7 = 'de:08115:3200:1:4'  # Bstg 4 (Sindelfingen Sindelfingen)
	ZOB_8 = 'de:08115:3200:1:5'  # Bstg 5 (Sindelfingen Sindelfingen)
	ZOB_9 = 'de:08115:3200:1:6'  # Bstg 6 (Sindelfingen Sindelfingen)
	ZOB_10 = 'de:08115:3200:1:7'  # Bstg 7 (Sindelfingen Sindelfingen)
	ZOB_11 = 'de:08115:3200:1:8'  # Bstg 8 (Sindelfingen Sindelfingen)
	ZOB_12 = 'de:08115:3200:1:9'  # Bstg 9 (Sindelfingen Sindelfingen)
	ZOB_13 = 'de:08116:7801:1:4'  # Bstg 4 (Esslingen am Neckar Esslingen (N))
	ZOB_14 = 'de:08116:7801:3:1'  # Bstg 1 (Esslingen am Neckar Esslingen (N))
	ZOB_15 = 'de:08116:7801:3:2'  # Bstg 2 (Esslingen am Neckar Esslingen (N))
	ZOB_16 = 'de:08116:7801:3:3'  # Bstg 3 (Esslingen am Neckar Esslingen (N))
	ZOB_17 = 'de:08116:7801:3:5'  # Bstg 5 (Esslingen am Neckar Esslingen (N))
	ZOB_18 = 'de:08116:7801:3:6'  # Bstg 6 (Esslingen am Neckar Esslingen (N))
	ZOB_19 = 'de:08116:7801:3:7'  # Bstg 7 (Esslingen am Neckar Esslingen (N))
	ZOB_20 = 'de:08116:7801:3:8'  # Bstg 8 (Esslingen am Neckar Esslingen (N))
	ZOB_21 = 'de:08116:7801:3:9'  # Bstg 9 (Esslingen am Neckar Esslingen (N))
	ZOB_22 = 'de:08235:9340:1:3'  # Steig 3 (Nagold Nagold)
	ZOB_23 = 'de:08235:9340:1:4'  # Steig 4 (Nagold Nagold)
	ZOB_24 = 'de:08425:2638:0:B'  # L30_350 (Laichingen Laichingen)
	ZOB_25 = 'de:08425:2638:0:D'  # L334_339_369 (Laichingen Laichingen)
	ZOB_26 = 'de:08425:2638:0:E'  # L335 (Laichingen Laichingen)
	SINDELFINGEN = 'de:08115:3201'
	SINDELFINGEN_1 = 'de:08115:3201:1:1'  # Gleis 1 (Sindelfingen Sindelfingen)
	SINDELFINGEN_2 = 'de:08115:3201:2:4'  # Rtg Ikea Böblingen (Sindelfingen Sindelfingen)
	SINDELFINGEN_3 = 'de:08115:3201:2:5'  # Rtg ZOB (Sindelfingen Sindelfingen)
	SINDELFINGEN_4 = 'de:08115:3201:3:3'  # Rtg Daimlersteg (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_TOR_VII = 'de:08115:3202:0:3'  # Rtg Nord (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_TOR_VII = 'de:08115:3202:0:3'  # Rtg Nord (Sindelfingen Sindelfingen)
	SINDELFINGEN_FRONÄCKERSTR = 'de:08115:3203'
	SINDELFINGEN_FRONÄCKERSTR_1 = 'de:08115:3203:0:3'  # Rtg West/Sifi Str. (Sindelfingen Sindelfingen)
	SINDELFINGEN_FRONÄCKERSTR_2 = 'de:08115:3203:0:4'  # Rtg Ost/Amundsenstr. (Sindelfingen Sindelfingen)
	FRONÄCKERSTRASSE = 'de:08115:3203'
	FRONÄCKERSTRASSE_1 = 'de:08115:3203:0:3'  # Rtg West/Sifi Str. (Sindelfingen Sindelfingen)
	FRONÄCKERSTRASSE_2 = 'de:08115:3203:0:4'  # Rtg Ost/Amundsenstr. (Sindelfingen Sindelfingen)
	SINDELF__GOLDBERG_GYMNASIUM = 'de:08115:3204'
	SINDELF__GOLDBERG_GYMNASIUM_1 = 'de:08115:3204:0:3'  # Rtg Böblingen (Sindelfingen Sindelfingen)
	SINDELF__GOLDBERG_GYMNASIUM_2 = 'de:08115:3204:0:4'  # Rtg Sindelfingen (Sindelfingen Sindelfingen)
	GOLDBERG_GYMNASIUM = 'de:08115:3204'
	GOLDBERG_GYMNASIUM_1 = 'de:08115:3204:0:3'  # Rtg Böblingen (Sindelfingen Sindelfingen)
	GOLDBERG_GYMNASIUM_2 = 'de:08115:3204:0:4'  # Rtg Sindelfingen (Sindelfingen Sindelfingen)
	GOTTLIEB_DAIMLER_SCHULE = 'de:08115:3205'
	GOTTLIEB_DAIMLER_SCHULE_1 = 'de:08115:3205:0:3'  # Rtg Böblingen/Goldberg (Sindelfingen Sindelfingen)
	GOTTLIEB_DAIMLER_SCHULE_2 = 'de:08115:3205:0:4'  # Rtg Sindelfingen (Sindelfingen Sindelfingen)
	SIND__GOTTLIEB_DAIMLER_SCHULE = 'de:08115:3205'
	SIND__GOTTLIEB_DAIMLER_SCHULE_1 = 'de:08115:3205:0:3'  # Rtg Böblingen/Goldberg (Sindelfingen Sindelfingen)
	SIND__GOTTLIEB_DAIMLER_SCHULE_2 = 'de:08115:3205:0:4'  # Rtg Sindelfingen (Sindelfingen Sindelfingen)
	SINDELFINGEN_FRANKENSTRASSE = 'de:08115:3206:0:3'  # eRtg ZOB Böbl. (Sindelfingen Sindelfingen)
	LANDHAUSSIEDLUNG = 'de:08115:3207'
	LANDHAUSSIEDLUNG_1 = 'de:08115:3207:0:3'  # Rtg Magstadt/Stuttgart (Sindelfingen Maichingen)
	LANDHAUSSIEDLUNG_2 = 'de:08115:3207:0:4'  # Rtg Maichingen (Sindelfingen Maichingen)
	MAICH__LANDHAUSSIEDLUNG = 'de:08115:3207'
	MAICH__LANDHAUSSIEDLUNG_1 = 'de:08115:3207:0:3'  # Rtg Magstadt/Stuttgart (Sindelfingen Maichingen)
	MAICH__LANDHAUSSIEDLUNG_2 = 'de:08115:3207:0:4'  # Rtg Maichingen (Sindelfingen Maichingen)
	MAICHINGEN_HALLENBAD = 'de:08115:3208'
	MAICHINGEN_HALLENBAD_1 = 'de:08115:3208:0:3'  # eRtg Sifi ZOB (Sindelfingen Maichingen)
	MAICHINGEN_HALLENBAD_2 = 'de:08115:3208:0:4'  # eRtg Rathaus (Sindelfingen Maichingen)
	MAICHINGEN_RATHAUS = 'de:08115:3209'
	MAICHINGEN_RATHAUS_1 = 'de:08115:3209:0:3'  # eRtg Hallenbad (Sindelfingen Maichingen)
	MAICHINGEN_RATHAUS_2 = 'de:08115:3209:0:4'  # eRtg Maichingen (Sindelfingen Maichingen)
	MAICHINGEN_SOLO = 'de:08115:3210'
	MAICHINGEN_SOLO_1 = 'de:08115:3210:0:4'  # Rtg Rathaus (Sindelfingen Maichingen)
	MAICHINGEN_SOLO_2 = 'de:08115:3210:0:5'  # Rtg Landhaus/Waldfriedhof (Sindelfingen Maichingen)
	SOLO = 'de:08115:3210'
	SOLO_1 = 'de:08115:3210:0:4'  # Rtg Rathaus (Sindelfingen Maichingen)
	SOLO_2 = 'de:08115:3210:0:5'  # Rtg Landhaus/Waldfriedhof (Sindelfingen Maichingen)
	MAICHINGEN_WALDFRIEDHOF = 'de:08115:3211'
	MAICHINGEN_WALDFRIEDHOF_1 = 'de:08115:3211:0:3'  # eRtg Landhaussiedlung (Sindelfingen Maichingen)
	MAICHINGEN_WALDFRIEDHOF_2 = 'de:08115:3211:0:4'  # eRtg Stuttgarter Straße/Eichholz (Sindelfingen Maichingen)
	MAICHINGEN_WALDFRIEDHOF_3 = 'de:08115:3211:0:5'  # Einbahnstr (Sindelfingen Maichingen)
	GOLDBERG = 'de:08115:3212'
	GOLDBERG_1 = 'de:08115:3212:1:1'  # Gleis 1 (Böblingen Böblingen)
	GOLDBERG_2 = 'de:08115:3212:2:2'  # Gleis 2 (Böblingen Böblingen)
	GOLDBERG_3 = 'de:08115:3212:3:3'  # eRtg Leipziger Str. (Böblingen Böblingen)
	GOLDBERG_4 = 'de:08115:3212:3:4'  # eRtg Silberweg (Böblingen Böblingen)
	SCHÖNAICH_DAIMLERSTRASSE = 'de:08115:3214:0:3'  # eRtg Maybachstraße (Schönaich Schönaich)
	MAICHINGEN_BÜRGERHAUS = 'de:08115:3215'
	MAICHINGEN_BÜRGERHAUS_1 = 'de:08115:3215:0:3'  # Rtg Nord (Sindelfingen Maichingen)
	MAICHINGEN_BÜRGERHAUS_2 = 'de:08115:3215:0:4'  # Rtg Süd (Sindelfingen Maichingen)
	SCHÖNAICH_EVANGELISCHE_KIRCHE = 'de:08115:3216'
	SCHÖNAICH_EVANGELISCHE_KIRCHE_1 = 'de:08115:3216:0:3'  # eRtg Neues Rathaus (Schönaich Schönaich)
	SCHÖNAICH_EVANGELISCHE_KIRCHE_2 = 'de:08115:3216:0:4'  # eRtg Schwabenstraße (Schönaich Schönaich)
	EVANGELISCHE_KIRCHE_1 = 'de:08115:3216:0:3'  # eRtg Neues Rathaus (Schönaich Schönaich)
	EVANGELISCHE_KIRCHE_2 = 'de:08115:3216:0:4'  # eRtg Schwabenstraße (Schönaich Schönaich)
	EVANGELISCHE_KIRCHE_3 = 'de:08115:4859:0:3'  # Rtg Altes Rathaus (Jettingen Unterjettingen)
	EVANGELISCHE_KIRCHE_4 = 'de:08115:4859:0:4'  # Rtg Gemeindezentrum (Jettingen Unterjettingen)
	EVANGELISCHE_KIRCHE_5 = 'de:08119:4929:0:3'  # eRtg Stettener Bank (Kernen im Remstal Stetten (i. R.))
	SCHÖNAICH_SCHILLERSTRASSE = 'de:08115:3217'
	SCHÖNAICH_SCHILLERSTRASSE_1 = 'de:08115:3217:0:3'  # eRtg Lessingstraße (Schönaich Schönaich)
	SCHÖNAICH_SCHILLERSTRASSE_2 = 'de:08115:3217:0:4'  # eRtg Evangelische Kirche (Schönaich Schönaich)
	IBM = 'de:08115:3218'
	IBM_1 = 'de:08115:3218:0:3'  # Rtg Ehningen (Ehningen Ehningen)
	IBM_2 = 'de:08115:3218:0:4'  # Rtg Hildrizhausen (Ehningen Ehningen)
	EHNINGEN_IBM = 'de:08115:3218'
	EHNINGEN_IBM_1 = 'de:08115:3218:0:3'  # Rtg Ehningen (Ehningen Ehningen)
	EHNINGEN_IBM_2 = 'de:08115:3218:0:4'  # Rtg Hildrizhausen (Ehningen Ehningen)
	GÜLTSTEIN_HEERSTRASSE = 'de:08115:3219'
	GÜLTSTEIN_HEERSTRASSE_1 = 'de:08115:3219:0:3'  # Rtg Holzsteig (Herrenberg Gültstein)
	GÜLTSTEIN_HEERSTRASSE_2 = 'de:08115:3219:0:4'  # Rtg Erholungsheim (Herrenberg Gültstein)
	HOLZGERLINGEN_TALSTRASSE = 'de:08115:3220'
	HOLZGERLINGEN_TALSTRASSE_1 = 'de:08115:3220:0:3'  # Rtg Stäuchle (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_TALSTRASSE_2 = 'de:08115:3220:0:4'  # Rtg Am Eschelbach (Holzgerlingen Holzgerlingen)
	TALSTRASSE_1 = 'de:08115:3220:0:3'  # Rtg Stäuchle (Holzgerlingen Holzgerlingen)
	TALSTRASSE_2 = 'de:08115:3220:0:4'  # Rtg Am Eschelbach (Holzgerlingen Holzgerlingen)
	TALSTRASSE_3 = 'de:08115:3250:0:3'  # Rtg Ehingen Bf (Ehningen Ehningen)
	TALSTRASSE_4 = 'de:08118:3335:0:3'  # eRtg Großbottwar Kelter (Großbottwar Hof und Lembach)
	TALSTRASSE_5 = 'de:08118:3335:0:4'  # eRtg Kelter (Großbottwar Hof und Lembach)
	TALSTRASSE_6 = 'de:08118:3591:0:3'  # eRtg Obere Seewiesen (Steinheim an der Murr Höpfigheim)
	TALSTRASSE_7 = 'de:08118:3591:0:4'  # eRtg Hauptstraße (Steinheim an der Murr Höpfigheim)
	TALSTRASSE_8 = 'de:08119:3651:0:3'  # Rtg Hans-Sachs-Weg (Waiblingen Waiblingen)
	TALSTRASSE_9 = 'de:08116:4137:0:3'  # eRtg Im Unteren Zehen (Esslingen am Neckar St. Bernhardt)
	TALSTRASSE_10 = 'de:08116:4137:0:4'  # eRtg Sängerkranz (Esslingen am Neckar St. Bernhardt)
	TALSTRASSE_11 = 'de:08116:4462:0:3'  # eRtg Ostpreußenweg (Nürtingen Raidwangen)
	TALSTRASSE_12 = 'de:08116:4462:0:4'  # eRtg Nürtinger Straße (Nürtingen Raidwangen)
	TALSTRASSE_13 = 'de:08115:4619:0:3'  # eRtg Hanns-Klemm-Straße Ost (Böblingen Böblingen)
	TALSTRASSE_14 = 'de:08118:5504:0:3'  # Rtg Bietigheimer Str. (Ludwigsburg Ludwigsburg)
	TALSTRASSE_15 = 'de:08118:5504:0:4'  # Rtg Abelstr. (Ludwigsburg Ludwigsburg)
	TALSTRASSE_16 = 'de:08118:5504:0:5'  # Rtg Heilbronner Str. (Ludwigsburg Ludwigsburg)
	TALSTRASSE_17 = 'de:08119:5962:0:3'  # eRtg Mittlere Brücke (Schorndorf Schorndorf)
	TALSTRASSE_18 = 'de:08118:6834:1:3'  # eRtg Aldinger Straße (Kornwestheim Kornwestheim)
	TALSTRASSE_19 = 'de:08118:6834:1:4'  # eRtg Enzstraße (Kornwestheim Kornwestheim)
	TALSTRASSE_20 = 'de:08117:1020:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	TALSTRASSE_21 = 'de:08117:1020:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	TALSTRASSE_22 = 'de:08136:7039:0:1'  # eRtg Lorch (Lorch Lorch)
	TALSTRASSE_23 = 'de:08136:7039:0:2'  # eRtg Edenhof (Lorch Lorch)
	EINSTEINSTRASSE_1 = 'de:08115:3223:0:3'  # Rtg Mönchberg Kayh (Herrenberg Gültstein)
	EINSTEINSTRASSE_2 = 'de:08115:3223:0:4'  # Rtg Holzsteig (Herrenberg Gültstein)
	EINSTEINSTRASSE_3 = 'de:08116:3955:0:3'  # eRtg Fauslerstraße (Kirchheim unter Teck Kirchheim (T))
	EINSTEINSTRASSE_4 = 'de:08116:3955:0:4'  # eRtg Heinkelstraße (Kirchheim unter Teck Kirchheim (T))
	EINSTEINSTRASSE_5 = 'de:08118:6959:0:3'  # eRtg Am Aischbach (Sersheim Sersheim)
	GÜLTSTEIN_EINSTEINSTRASSE = 'de:08115:3223'
	GÜLTSTEIN_EINSTEINSTRASSE_1 = 'de:08115:3223:0:3'  # Rtg Mönchberg Kayh (Herrenberg Gültstein)
	GÜLTSTEIN_EINSTEINSTRASSE_2 = 'de:08115:3223:0:4'  # Rtg Holzsteig (Herrenberg Gültstein)
	JUNKERSSTRASSE = 'de:08115:3224'
	JUNKERSSTRASSE_1 = 'de:08115:3224:0:3'  # Rtg Daimler (Böblingen Böblingen)
	JUNKERSSTRASSE_2 = 'de:08115:3224:0:4'  # Rtg Hulb (Böblingen Böblingen)
	BÖBLINGEN_JUNKERSSTRASSE = 'de:08115:3224'
	BÖBLINGEN_JUNKERSSTRASSE_1 = 'de:08115:3224:0:3'  # Rtg Daimler (Böblingen Böblingen)
	BÖBLINGEN_JUNKERSSTRASSE_2 = 'de:08115:3224:0:4'  # Rtg Hulb (Böblingen Böblingen)
	ALTSTADTGARAGE = 'de:08115:3225:0:3'  # eRtg Hallenbad (Herrenberg Herrenberg)
	HERRENBERG_ALTSTADTGARAGE = 'de:08115:3225:0:3'  # eRtg Hallenbad (Herrenberg Herrenberg)
	HOLZGERLINGEN_ALTENZENTRUM = 'de:08115:3228'
	HOLZGERLINGEN_ALTENZENTRUM_1 = 'de:08115:3228:0:3'  # eRtg Hohenzollernstraße (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_ALTENZENTRUM_2 = 'de:08115:3228:0:4'  # eRtg Rathaus (Holzgerlingen Holzgerlingen)
	ALTENZENTRUM = 'de:08115:3228'
	ALTENZENTRUM_1 = 'de:08115:3228:0:3'  # eRtg Hohenzollernstraße (Holzgerlingen Holzgerlingen)
	ALTENZENTRUM_2 = 'de:08115:3228:0:4'  # eRtg Rathaus (Holzgerlingen Holzgerlingen)
	AMUNDSENSTRASSE = 'de:08115:3229'
	AMUNDSENSTRASSE_1 = 'de:08115:3229:0:3'  # eRtg Sindelfinger Straße (Sindelfingen Sindelfingen)
	AMUNDSENSTRASSE_2 = 'de:08115:3229:0:4'  # eRtg Calwer Str. (Sindelfingen Sindelfingen)
	SINDELFINGEN_AMUNDSENSTRASSE = 'de:08115:3229'
	SINDELFINGEN_AMUNDSENSTRASSE_1 = 'de:08115:3229:0:3'  # eRtg Sindelfinger Straße (Sindelfingen Sindelfingen)
	SINDELFINGEN_AMUNDSENSTRASSE_2 = 'de:08115:3229:0:4'  # eRtg Calwer Str. (Sindelfingen Sindelfingen)
	SINDELFINGEN_WETTBACHPLATZ = 'de:08115:3230:0:4'  # eRtg Sifi ZOB (Sindelfingen Sindelfingen)
	WETTBACHPLATZ = 'de:08115:3230:0:4'  # eRtg Sifi ZOB (Sindelfingen Sindelfingen)
	GÜLTSTEIN = 'de:08115:3231:1:1'  # Gleis 1 (Herrenberg Gültstein)
	DAGERSHEIM_POST = 'de:08115:3232:0:3'  # eRtg Kirche (Böblingen Dagersheim)
	HERRENBERG_HALLENBAD = 'de:08115:3233'
	HERRENBERG_HALLENBAD_1 = 'de:08115:3233:0:3'  # am Hallenbad (Herrenberg Herrenberg)
	HERRENBERG_HALLENBAD_2 = 'de:08115:3233:0:4'  # Rtg Stadtfriedhof (Marienstraße) (Herrenberg Herrenberg)
	HOLZSTEIG = 'de:08115:3234'
	HOLZSTEIG_1 = 'de:08115:3234:0:3'  # Rtg Herrenberg Hallenbad Kreiskrankenh. (Herrenberg Gültstein)
	HOLZSTEIG_2 = 'de:08115:3234:0:4'  # Rtg Heerstraße Einsteinstraße (Herrenberg Gültstein)
	GÜLTSTEIN_HOLZSTEIG = 'de:08115:3234'
	GÜLTSTEIN_HOLZSTEIG_1 = 'de:08115:3234:0:3'  # Rtg Herrenberg Hallenbad Kreiskrankenh. (Herrenberg Gültstein)
	GÜLTSTEIN_HOLZSTEIG_2 = 'de:08115:3234:0:4'  # Rtg Heerstraße Einsteinstraße (Herrenberg Gültstein)
	HERRENBERG_KALKOFENSTR = 'de:08115:3235'
	HERRENBERG_KALKOFENSTR_1 = 'de:08115:3235:1:1'  # Bstg 1 (Herrenberg Herrenberg)
	HERRENBERG_KALKOFENSTR_2 = 'de:08115:3235:1:2'  # Bstg 2 (Herrenberg Herrenberg)
	HERRENBERG_KALKOFENSTR_3 = 'de:08115:3235:1:3'  # Bstg 3 (Herrenberg Herrenberg)
	KALKOFENSTRASSE = 'de:08115:3235'
	KALKOFENSTRASSE_1 = 'de:08115:3235:1:1'  # Bstg 1 (Herrenberg Herrenberg)
	KALKOFENSTRASSE_2 = 'de:08115:3235:1:2'  # Bstg 2 (Herrenberg Herrenberg)
	KALKOFENSTRASSE_3 = 'de:08115:3235:1:3'  # Bstg 3 (Herrenberg Herrenberg)
	SINDELF__G__U__H_SCHULE_GOLDB = 'de:08115:3236'
	SINDELF__G__U__H_SCHULE_GOLDB_1 = 'de:08115:3236:0:3'  # eRtg Goldberg Realschule (Sindelfingen Sindelfingen)
	SINDELF__G__U__H_SCHULE_GOLDB_2 = 'de:08115:3236:0:4'  # Rtg Nord (Sindelfingen Sindelfingen)
	GRUND__UND_HAUPTSCH__GOLDBERG = 'de:08115:3236'
	GRUND__UND_HAUPTSCH__GOLDBERG_1 = 'de:08115:3236:0:3'  # eRtg Goldberg Realschule (Sindelfingen Sindelfingen)
	GRUND__UND_HAUPTSCH__GOLDBERG_2 = 'de:08115:3236:0:4'  # Rtg Nord (Sindelfingen Sindelfingen)
	KÄTHE_KOLLWITZ_STRASSE = 'de:08115:3237'
	KÄTHE_KOLLWITZ_STRASSE_1 = 'de:08115:3237:0:3'  # eRtg Fröbelschule (Herrenberg Herrenberg)
	KÄTHE_KOLLWITZ_STRASSE_2 = 'de:08115:3237:0:4'  # Rtg ZOB (Herrenberg Herrenberg)
	HERRENBERG_KÄTHE_KOLLWITZ_STR = 'de:08115:3237'
	HERRENBERG_KÄTHE_KOLLWITZ_STR_1 = 'de:08115:3237:0:3'  # eRtg Fröbelschule (Herrenberg Herrenberg)
	HERRENBERG_KÄTHE_KOLLWITZ_STR_2 = 'de:08115:3237:0:4'  # Rtg ZOB (Herrenberg Herrenberg)
	IM_LETTEN = 'de:08115:3238'
	IM_LETTEN_1 = 'de:08115:3238:0:3'  # eRtg IBM (Ehningen Ehningen)
	IM_LETTEN_2 = 'de:08115:3238:0:4'  # eRtg Königsberger Straße (Ehningen Ehningen)
	EHNINGEN_IM_LETTEN = 'de:08115:3238'
	EHNINGEN_IM_LETTEN_1 = 'de:08115:3238:0:3'  # eRtg IBM (Ehningen Ehningen)
	EHNINGEN_IM_LETTEN_2 = 'de:08115:3238:0:4'  # eRtg Königsberger Straße (Ehningen Ehningen)
	SINDELFINGEN_WENGERTSTR = 'de:08115:3239:0:3'  # eRtg Altenheim Burghalde (Sindelfingen Sindelfingen)
	WENGERTSTRASSE = 'de:08115:3239:0:3'  # eRtg Altenheim Burghalde (Sindelfingen Sindelfingen)
	MELLI_BEESE_STRASSE = 'de:08115:3241:0:4'  # Rtg Calwer Str. (Hulb) (Sindelfingen Sindelfingen)
	SINDELFINGEN_MELLI_BEESE_STR = 'de:08115:3241:0:4'  # Rtg Calwer Str. (Hulb) (Sindelfingen Sindelfingen)
	HARFENBRÜCKE = 'de:08115:3242:0:3'  # Rtg Flugfeld-Allee (Sindelfingen Sindelfingen)
	SINDELFINGEN_HARFENBRÜCKE = 'de:08115:3242:0:3'  # Rtg Flugfeld-Allee (Sindelfingen Sindelfingen)
	HOLZGERLINGEN_ACHALMSTRASSE = 'de:08115:3244'
	HOLZGERLINGEN_ACHALMSTRASSE_1 = 'de:08115:3244:0:3'  # Rtg Hohenzollernstr/BB (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_ACHALMSTRASSE_2 = 'de:08115:3244:0:4'  # Rtg Germanenstr/Bf (Holzgerlingen Holzgerlingen)
	ACHALMSTRASSE_1 = 'de:08115:3244:0:3'  # Rtg Hohenzollernstr/BB (Holzgerlingen Holzgerlingen)
	ACHALMSTRASSE_2 = 'de:08115:3244:0:4'  # Rtg Germanenstr/Bf (Holzgerlingen Holzgerlingen)
	ACHALMSTRASSE_3 = 'de:08116:4448:0:3'  # eRtg Hermann-Löns-Straße (Nürtingen Nürtingen)
	ACHALMSTRASSE_4 = 'de:08116:4448:0:4'  # eRtg Rotenbergplatz (Nürtingen Nürtingen)
	TÜBINGER_ALLEE = 'de:08115:3245'
	TÜBINGER_ALLEE_1 = 'de:08115:3245:0:3'  # Rtg Nordost (Sindelfingen Sindelfingen)
	TÜBINGER_ALLEE_2 = 'de:08115:3245:0:4'  # Rtg Südwest (Sindelfingen Sindelfingen)
	SINDELFINGEN_TÜBINGER_ALLEE = 'de:08115:3245'
	SINDELFINGEN_TÜBINGER_ALLEE_1 = 'de:08115:3245:0:3'  # Rtg Nordost (Sindelfingen Sindelfingen)
	SINDELFINGEN_TÜBINGER_ALLEE_2 = 'de:08115:3245:0:4'  # Rtg Südwest (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_P305 = 'de:08115:3246'
	SINDELF__MERCEDES_BENZ_P305_1 = 'de:08115:3246:0:3'  # Rtg Mercedes Benz P 307 (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_P305_2 = 'de:08115:3246:0:4'  # Rtg Junkersstr (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_P305 = 'de:08115:3246'
	MERCEDES_BENZ_P305_1 = 'de:08115:3246:0:3'  # Rtg Mercedes Benz P 307 (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_P305_2 = 'de:08115:3246:0:4'  # Rtg Junkersstr (Sindelfingen Sindelfingen)
	LÜGLENWEG = 'de:08115:3248'
	LÜGLENWEG_1 = 'de:08115:3248:0:1'  # Rtg Waldfriedhof Grafenweg (Herrenberg Herrenberg)
	LÜGLENWEG_2 = 'de:08115:3248:0:2'  # Rtg Seniorenzentrum (Herrenberg Herrenberg)
	HERRENBERG_LÜGLENWEG = 'de:08115:3248'
	HERRENBERG_LÜGLENWEG_1 = 'de:08115:3248:0:1'  # Rtg Waldfriedhof Grafenweg (Herrenberg Herrenberg)
	HERRENBERG_LÜGLENWEG_2 = 'de:08115:3248:0:2'  # Rtg Seniorenzentrum (Herrenberg Herrenberg)
	EHNINGEN_TALSTRASSE = 'de:08115:3250:0:3'  # Rtg Ehingen Bf (Ehningen Ehningen)
	ZENTRUM_ZIEGELFELD = 'de:08115:3252:0:3'  # Rtg ??? (Herrenberg Herrenberg)
	HERRENBERG_ZENTRUM_ZIEGELFELD = 'de:08115:3252:0:3'  # Rtg ??? (Herrenberg Herrenberg)
	WESTRANDALLEE = 'de:08115:3254'
	WESTRANDALLEE_1 = 'de:08115:3254:0:1'  # Rtg Böblingen (Schönaich Schönaich)
	WESTRANDALLEE_2 = 'de:08115:3254:0:2'  # Rtg Schönaich Bahnhofstraße (Schönaich Schönaich)
	SCHÖNAICH_WESTRANDALLEE = 'de:08115:3254'
	SCHÖNAICH_WESTRANDALLEE_1 = 'de:08115:3254:0:1'  # Rtg Böblingen (Schönaich Schönaich)
	SCHÖNAICH_WESTRANDALLEE_2 = 'de:08115:3254:0:2'  # Rtg Schönaich Bahnhofstraße (Schönaich Schönaich)
	ALFDORF_GEWERBEGEBIET = 'de:08119:3256:0:1'  # Rtg Adelstetten (Alfdorf Alfdorf)
	ALFDORF_BONHOLZ = 'de:08119:3257:0:1'  # Rtg Adelstetten (Alfdorf Alfdorf)
	LACHENSTRASSE = 'de:08115:3258'
	LACHENSTRASSE_1 = 'de:08115:3258:0:1'  # Rtg Böblingen (Schönaich Schönaich)
	LACHENSTRASSE_2 = 'de:08115:3258:0:2'  # Rtg Neues Rathaus/Eiche (Schönaich Schönaich)
	SCHÖNAICH_LACHENSTRASSE = 'de:08115:3258'
	SCHÖNAICH_LACHENSTRASSE_1 = 'de:08115:3258:0:1'  # Rtg Böblingen (Schönaich Schönaich)
	SCHÖNAICH_LACHENSTRASSE_2 = 'de:08115:3258:0:2'  # Rtg Neues Rathaus/Eiche (Schönaich Schönaich)
	JOHANNES_TÄUFER_KIRCHE = 'de:08115:3295'
	JOHANNES_TÄUFER_KIRCHE_1 = 'de:08115:3295:0:3'  # Rtg Renninger Str. (Magstadt Magstadt)
	JOHANNES_TÄUFER_KIRCHE_2 = 'de:08115:3295:0:4'  # Rtg Ringstr. (Magstadt Magstadt)
	MAGSTADT_JOH__TÄUFER_KIRCHE = 'de:08115:3295'
	MAGSTADT_JOH__TÄUFER_KIRCHE_1 = 'de:08115:3295:0:3'  # Rtg Renninger Str. (Magstadt Magstadt)
	MAGSTADT_JOH__TÄUFER_KIRCHE_2 = 'de:08115:3295:0:4'  # Rtg Ringstr. (Magstadt Magstadt)
	MAGSTADT_OSWALDSTRASSE = 'de:08115:3296'
	MAGSTADT_OSWALDSTRASSE_1 = 'de:08115:3296:0:3'  # eRtg Johannes-Kepler-Schule (Magstadt Magstadt)
	MAGSTADT_OSWALDSTRASSE_2 = 'de:08115:3296:0:4'  # eRtg Hutwiesenstraße (Magstadt Magstadt)
	OSWALDSTRASSE = 'de:08115:3296'
	OSWALDSTRASSE_1 = 'de:08115:3296:0:3'  # eRtg Johannes-Kepler-Schule (Magstadt Magstadt)
	OSWALDSTRASSE_2 = 'de:08115:3296:0:4'  # eRtg Hutwiesenstraße (Magstadt Magstadt)
	HUTWIESENSTRASSE = 'de:08115:3297'
	HUTWIESENSTRASSE_1 = 'de:08115:3297:0:3'  # eRtg Oswaldstraße (Magstadt Magstadt)
	HUTWIESENSTRASSE_2 = 'de:08115:3297:0:4'  # eRtg Ringstraße (Magstadt Magstadt)
	MAGSTADT_HUTWIESENSTRASSE = 'de:08115:3297'
	MAGSTADT_HUTWIESENSTRASSE_1 = 'de:08115:3297:0:3'  # eRtg Oswaldstraße (Magstadt Magstadt)
	MAGSTADT_HUTWIESENSTRASSE_2 = 'de:08115:3297:0:4'  # eRtg Ringstraße (Magstadt Magstadt)
	MAGSTADT_BRAUEREIPLATZ = 'de:08115:3298'
	MAGSTADT_BRAUEREIPLATZ_1 = 'de:08115:3298:0:3'  # eRtg Renninger Straße (Magstadt Magstadt)
	MAGSTADT_BRAUEREIPLATZ_2 = 'de:08115:3298:0:4'  # eRtg Magstadt (Magstadt Magstadt)
	BRAUEREIPLATZ = 'de:08115:3298'
	BRAUEREIPLATZ_1 = 'de:08115:3298:0:3'  # eRtg Renninger Straße (Magstadt Magstadt)
	BRAUEREIPLATZ_2 = 'de:08115:3298:0:4'  # eRtg Magstadt (Magstadt Magstadt)
	KNIEBISSTRASSE = 'de:08115:3300'
	KNIEBISSTRASSE_1 = 'de:08115:3300:0:3'  # eRtg Brühlstraße 22 (Magstadt Magstadt)
	KNIEBISSTRASSE_2 = 'de:08115:3300:0:4'  # eRtg Renninger Straße (Magstadt Magstadt)
	MAGSTADT_KNIEBISSTRASSE = 'de:08115:3300'
	MAGSTADT_KNIEBISSTRASSE_1 = 'de:08115:3300:0:3'  # eRtg Brühlstraße 22 (Magstadt Magstadt)
	MAGSTADT_KNIEBISSTRASSE_2 = 'de:08115:3300:0:4'  # eRtg Renninger Straße (Magstadt Magstadt)
	ESSLINGER_WEG = 'de:08115:3301'
	ESSLINGER_WEG_1 = 'de:08115:3301:0:3'  # eRtg Johannes-Kepler-Schule (Magstadt Magstadt)
	ESSLINGER_WEG_2 = 'de:08115:3301:0:4'  # eRtg Waldfriedhof (Magstadt Magstadt)
	MAGSTADT_ESSLINGER_WEG = 'de:08115:3301'
	MAGSTADT_ESSLINGER_WEG_1 = 'de:08115:3301:0:3'  # eRtg Johannes-Kepler-Schule (Magstadt Magstadt)
	MAGSTADT_ESSLINGER_WEG_2 = 'de:08115:3301:0:4'  # eRtg Waldfriedhof (Magstadt Magstadt)
	MAGSTADT_J__KEPLER_SCHULE = 'de:08115:3302'
	MAGSTADT_J__KEPLER_SCHULE_1 = 'de:08115:3302:0:3'  # eRtg Oswaldstr./Marktplatz/Planstraße (Magstadt Magstadt)
	MAGSTADT_J__KEPLER_SCHULE_2 = 'de:08115:3302:0:4'  # eRtg Esslinger Weg/Hölderlinstraße (Magstadt Magstadt)
	JOHANNES_KEPLER_SCHULE = 'de:08115:3302'
	JOHANNES_KEPLER_SCHULE_1 = 'de:08115:3302:0:3'  # eRtg Oswaldstr./Marktplatz/Planstraße (Magstadt Magstadt)
	JOHANNES_KEPLER_SCHULE_2 = 'de:08115:3302:0:4'  # eRtg Esslinger Weg/Hölderlinstraße (Magstadt Magstadt)
	RENNINGER_STRASSE = 'de:08115:3304'
	RENNINGER_STRASSE_1 = 'de:08115:3304:0:3'  # eRtg Leonberg/Renningen (Magstadt Magstadt)
	RENNINGER_STRASSE_2 = 'de:08115:3304:0:4'  # eRtg Weilemer Str. (Magstadt Magstadt)
	MAGSTADT_RENNINGER_STR = 'de:08115:3304'
	MAGSTADT_RENNINGER_STR_1 = 'de:08115:3304:0:3'  # eRtg Leonberg/Renningen (Magstadt Magstadt)
	MAGSTADT_RENNINGER_STR_2 = 'de:08115:3304:0:4'  # eRtg Weilemer Str. (Magstadt Magstadt)
	ABZWEIG_BAHNHOF = 'de:08118:3306'
	ABZWEIG_BAHNHOF_1 = 'de:08118:3306:0:3'  # Rtg Markgröningen (Markgröningen Markgröningen)
	ABZWEIG_BAHNHOF_2 = 'de:08118:3306:0:4'  # Rtg Asperg (Markgröningen Markgröningen)
	MARKGRÖNINGEN_ABZW__BF = 'de:08118:3306'
	MARKGRÖNINGEN_ABZW__BF_1 = 'de:08118:3306:0:3'  # Rtg Markgröningen (Markgröningen Markgröningen)
	MARKGRÖNINGEN_ABZW__BF_2 = 'de:08118:3306:0:4'  # Rtg Asperg (Markgröningen Markgröningen)
	PAPPELWEG = 'de:08118:3307'
	PAPPELWEG_1 = 'de:08118:3307:0:3'  # eRtg Rebsteige (Markgröningen Markgröningen)
	PAPPELWEG_2 = 'de:08118:3307:0:4'  # eRtg Helene-Lange-Schule (Markgröningen Markgröningen)
	MARKGRÖNINGEN_PAPPELWEG = 'de:08118:3307'
	MARKGRÖNINGEN_PAPPELWEG_1 = 'de:08118:3307:0:3'  # eRtg Rebsteige (Markgröningen Markgröningen)
	MARKGRÖNINGEN_PAPPELWEG_2 = 'de:08118:3307:0:4'  # eRtg Helene-Lange-Schule (Markgröningen Markgröningen)
	PFULLINGER_STRASSE = 'de:08111:3308'
	PFULLINGER_STRASSE_1 = 'de:08111:3308:0:3'  # Rtg Tränke (Stuttgart Stuttgart)
	PFULLINGER_STRASSE_2 = 'de:08111:3308:0:4'  # Rtg Albschule (Stuttgart Stuttgart)
	TRÄNKE = 'de:08111:3309'
	TRÄNKE_1 = 'de:08111:3309:0:3'  # Rtg Hohe Eiche (Stuttgart Stuttgart)
	TRÄNKE_2 = 'de:08111:3309:0:4'  # Rtg Pfullinger Str. (Stuttgart Stuttgart)
	HAUS_FRANK = 'de:08118:3310'
	HAUS_FRANK_1 = 'de:08118:3310:0:3'  # Rtg Unterriexingen (Markgröningen Markgröningen)
	HAUS_FRANK_2 = 'de:08118:3310:0:4'  # Rtg Festplatz (Markgröningen Markgröningen)
	MARKGRÖNINGEN_HAUS_FRANK = 'de:08118:3310'
	MARKGRÖNINGEN_HAUS_FRANK_1 = 'de:08118:3310:0:3'  # Rtg Unterriexingen (Markgröningen Markgröningen)
	MARKGRÖNINGEN_HAUS_FRANK_2 = 'de:08118:3310:0:4'  # Rtg Festplatz (Markgröningen Markgröningen)
	MARKGR__H__LANGE_SCHULE = 'de:08118:3311'
	MARKGR__H__LANGE_SCHULE_1 = 'de:08118:3311:0:3'  # eRtg Pappelw./Münchinger Str./Haus Frank (Markgröningen Markgröningen)
	MARKGR__H__LANGE_SCHULE_2 = 'de:08118:3311:0:4'  # eRtg Mitte/Helenenstraße (Markgröningen Markgröningen)
	HELENE_LANGE_SCHULE = 'de:08118:3311'
	HELENE_LANGE_SCHULE_1 = 'de:08118:3311:0:3'  # eRtg Pappelw./Münchinger Str./Haus Frank (Markgröningen Markgröningen)
	HELENE_LANGE_SCHULE_2 = 'de:08118:3311:0:4'  # eRtg Mitte/Helenenstraße (Markgröningen Markgröningen)
	MARKGR__AM_WASSERTURM = 'de:08118:3312'
	MARKGR__AM_WASSERTURM_1 = 'de:08118:3312:0:3'  # eRtg Klinik/Schule (Markgröningen Markgröningen)
	MARKGR__AM_WASSERTURM_2 = 'de:08118:3312:0:4'  # eRtg Markgröninger Straße (Markgröningen Markgröningen)
	AM_WASSERTURM = 'de:08118:3312'
	AM_WASSERTURM_1 = 'de:08118:3312:0:3'  # eRtg Klinik/Schule (Markgröningen Markgröningen)
	AM_WASSERTURM_2 = 'de:08118:3312:0:4'  # eRtg Markgröninger Straße (Markgröningen Markgröningen)
	MITTE_WERNERSTRASSE = 'de:08118:3313'
	MITTE_WERNERSTRASSE_1 = 'de:08118:3313:1:3'  # eRtg Münchinger Straße/Mitte Volksbank (Markgröningen Markgröningen)
	MITTE_WERNERSTRASSE_2 = 'de:08118:3313:2:4'  # eRtg Abzw. Bahnhof/Sträßle (Markgröningen Markgröningen)
	MARKGRÖNINGEN_MITTE_WERNERSTR = 'de:08118:3313'
	MARKGRÖNINGEN_MITTE_WERNERSTR_1 = 'de:08118:3313:1:3'  # eRtg Münchinger Straße/Mitte Volksbank (Markgröningen Markgröningen)
	MARKGRÖNINGEN_MITTE_WERNERSTR_2 = 'de:08118:3313:2:4'  # eRtg Abzw. Bahnhof/Sträßle (Markgröningen Markgröningen)
	MARKGRÖNINGEN_MÜNCHINGER_STR = 'de:08118:3314'
	MARKGRÖNINGEN_MÜNCHINGER_STR_1 = 'de:08118:3314:1:3'  # Pos 3 (Markgröningen Markgröningen)
	MARKGRÖNINGEN_MÜNCHINGER_STR_2 = 'de:08118:3314:2:4'  # Pos 4 (Markgröningen Markgröningen)
	MARKGRÖNINGEN_STRÄSSLE = 'de:08118:3315'
	MARKGRÖNINGEN_STRÄSSLE_1 = 'de:08118:3315:0:3'  # Rtg Möglingen (Markgröningen Markgröningen)
	MARKGRÖNINGEN_STRÄSSLE_2 = 'de:08118:3315:0:4'  # Rtg Markgröningen (Markgröningen Markgröningen)
	STRÄSSLE = 'de:08118:3315'
	STRÄSSLE_1 = 'de:08118:3315:0:3'  # Rtg Möglingen (Markgröningen Markgröningen)
	STRÄSSLE_2 = 'de:08118:3315:0:4'  # Rtg Markgröningen (Markgröningen Markgröningen)
	MARKGR__STUTTGARTER_WEG = 'de:08118:3316'
	MARKGR__STUTTGARTER_WEG_1 = 'de:08118:3316:0:3'  # eRtg Münchinger Straße (Markgröningen Markgröningen)
	MARKGR__STUTTGARTER_WEG_2 = 'de:08118:3316:0:4'  # eRtg Endpunkt/Sträßle (Markgröningen Markgröningen)
	STUTTGARTER_WEG = 'de:08118:3316'
	STUTTGARTER_WEG_1 = 'de:08118:3316:0:3'  # eRtg Münchinger Straße (Markgröningen Markgröningen)
	STUTTGARTER_WEG_2 = 'de:08118:3316:0:4'  # eRtg Endpunkt/Sträßle (Markgröningen Markgröningen)
	MITTE_VOLKSBANK = 'de:08118:3317'
	MITTE_VOLKSBANK_1 = 'de:08118:3317:0:3'  # eRtg Heyd-Schule/Festplatz (Markgröningen Markgröningen)
	MITTE_VOLKSBANK_2 = 'de:08118:3317:2:5'  # Rtg Münchinger Str. (Markgröningen Markgröningen)
	MARKGRÖNINGEN_MITTE_VOLKSBANK = 'de:08118:3317'
	MARKGRÖNINGEN_MITTE_VOLKSBANK_1 = 'de:08118:3317:0:3'  # eRtg Heyd-Schule/Festplatz (Markgröningen Markgröningen)
	MARKGRÖNINGEN_MITTE_VOLKSBANK_2 = 'de:08118:3317:2:5'  # Rtg Münchinger Str. (Markgröningen Markgröningen)
	ERNST_BAUER_PLATZ = 'de:08115:3318'
	ERNST_BAUER_PLATZ_1 = 'de:08115:3318:1:3'  # Rtg Post/Renningen (Renningen Renningen)
	ERNST_BAUER_PLATZ_2 = 'de:08115:3318:1:4'  # Rtg Friedhof (Renningen Renningen)
	RENNINGEN_ERNST_BAUER_PL = 'de:08115:3318'
	RENNINGEN_ERNST_BAUER_PL_1 = 'de:08115:3318:1:3'  # Rtg Post/Renningen (Renningen Renningen)
	RENNINGEN_ERNST_BAUER_PL_2 = 'de:08115:3318:1:4'  # Rtg Friedhof (Renningen Renningen)
	RENNINGEN_FRIEDHOF = 'de:08115:3319'
	RENNINGEN_FRIEDHOF_1 = 'de:08115:3319:0:3'  # Rtg Ernst-Bauer-Pl. (Renningen Renningen)
	RENNINGEN_FRIEDHOF_2 = 'de:08115:3319:0:4'  # Rtg Süd (Renningen Renningen)
	HUMMELBAUM = 'de:08115:3320'
	HUMMELBAUM_1 = 'de:08115:3320:0:5'  # Rtg Keltenstr. (Renningen Renningen)
	HUMMELBAUM_2 = 'de:08115:3320:0:6'  # Rtg E.-Bauer-Platz (Renningen Renningen)
	RENNINGEN_HUMMELBAUM = 'de:08115:3320'
	RENNINGEN_HUMMELBAUM_1 = 'de:08115:3320:0:5'  # Rtg Keltenstr. (Renningen Renningen)
	RENNINGEN_HUMMELBAUM_2 = 'de:08115:3320:0:6'  # Rtg E.-Bauer-Platz (Renningen Renningen)
	BOSCH_TOR_III = 'de:08118:3321'
	BOSCH_TOR_III_1 = 'de:08118:3321:0:3'  # Rtg Ludwigsb. Str. (Schwieberdingen Schwieberdingen)
	BOSCH_TOR_III_2 = 'de:08118:3321:0:4'  # Rtg Zuff.Porsche/Mögl.Schwd.Str. (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_BOSCH_TOR_III = 'de:08118:3321'
	SCHWIEBERDINGEN_BOSCH_TOR_III_1 = 'de:08118:3321:0:3'  # Rtg Ludwigsb. Str. (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_BOSCH_TOR_III_2 = 'de:08118:3321:0:4'  # Rtg Zuff.Porsche/Mögl.Schwd.Str. (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_GLEMSTAL = 'de:08118:3322'
	SCHWIEBERDINGEN_GLEMSTAL_1 = 'de:08118:3322:1:3'  # eRtg Hemmingen (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_GLEMSTAL_2 = 'de:08118:3322:1:4'  # eRtg Mitte (Schwieberdingen Schwieberdingen)
	ROBERT_BOSCH_STRASSE_1 = 'de:08118:3323:0:3'  # eRtg Siemensstraße (Schwieberdingen Schwieberdingen)
	ROBERT_BOSCH_STRASSE_2 = 'de:08118:3323:0:4'  # eRtg Ludwigsburger Straße (Schwieberdingen Schwieberdingen)
	ROBERT_BOSCH_STRASSE_3 = 'de:08118:3399:0:3'  # eRtg Theo-Lorch-Werkstätten (Tamm Tamm)
	ROBERT_BOSCH_STRASSE_4 = 'de:08118:3399:0:4'  # eRtg Laiern-/R.-Bosch-Str. (Tamm Tamm)
	ROBERT_BOSCH_STRASSE_5 = 'de:08116:3819:0:3'  # Rtg Altenried Rathaus (Neckartenzlingen Neckartenzlingen)
	ROBERT_BOSCH_STRASSE_6 = 'de:08116:3819:0:4'  # Rtg Spitzacker/Marktplatz (Neckartenzlingen Neckartenzlingen)
	ROBERT_BOSCH_STRASSE_7 = 'de:08116:4272:0:3'  # eRtg Römerstraße (Dettingen unter Teck Dettingen (T))
	ROBERT_BOSCH_STRASSE_8 = 'de:08116:4272:0:4'  # eRtg Straßenmeisterei (Dettingen unter Teck Dettingen (T))
	ROBERT_BOSCH_STRASSE_9 = 'de:08115:4608:1:3'  # eRtg Tessiner Str. (Sindelfingen Darmsheim)
	ROBERT_BOSCH_STRASSE_10 = 'de:08118:5492:0:3'  # eRtg Steinbeisstraße (Ludwigsburg Pflugfelden)
	ROBERT_BOSCH_STRASSE_11 = 'de:08118:5492:0:4'  # eRtg Möglinger Straße (Ludwigsburg Pflugfelden)
	ROBERT_BOSCH_STRASSE_12 = 'de:08118:5560:0:3'  # eRtg Hohenzollernstraße (Pleidelsheim Pleidelsheim)
	ROBERT_BOSCH_STRASSE_13 = 'de:08118:5560:0:4'  # eRtg Beihinger Platz (Pleidelsheim Pleidelsheim)
	ROBERT_BOSCH_STRASSE_14 = 'de:08115:6534:0:3'  # Rtg Daimlerstr. (Holzgerlingen Holzgerlingen)
	ROBERT_BOSCH_STRASSE_15 = 'de:08115:6534:0:4'  # Rtg Siemensstr. (Holzgerlingen Holzgerlingen)
	ROBERT_BOSCH_STRASSE_16 = 'de:08119:7631:0:3'  # Rtg Welzheimer Str. (Schorndorf Schorndorf)
	ROBERT_BOSCH_STRASSE_17 = 'de:08119:7631:0:4'  # Rtg Schorndorf Bf (Schorndorf Schorndorf)
	SCHWIEBERD__ROBERT_BOSCH_STR = 'de:08118:3323'
	SCHWIEBERD__ROBERT_BOSCH_STR_1 = 'de:08118:3323:0:3'  # eRtg Siemensstraße (Schwieberdingen Schwieberdingen)
	SCHWIEBERD__ROBERT_BOSCH_STR_2 = 'de:08118:3323:0:4'  # eRtg Ludwigsburger Straße (Schwieberdingen Schwieberdingen)
	SCHWIEBERD__SIEMENSSTRASSE = 'de:08118:3324'
	SCHWIEBERD__SIEMENSSTRASSE_1 = 'de:08118:3324:0:3'  # eRtg Laiblinger Weg (Schwieberdingen Schwieberdingen)
	SCHWIEBERD__SIEMENSSTRASSE_2 = 'de:08118:3324:0:4'  # eRtg Robert-Bosch-Straße (Schwieberdingen Schwieberdingen)
	SIEMENSSTRASSE_1 = 'de:08118:3324:0:3'  # eRtg Laiblinger Weg (Schwieberdingen Schwieberdingen)
	SIEMENSSTRASSE_2 = 'de:08118:3324:0:4'  # eRtg Robert-Bosch-Straße (Schwieberdingen Schwieberdingen)
	SIEMENSSTRASSE_3 = 'de:08115:6535:0:3'  # Rtg R.-Bosch-Str. (Holzgerlingen Holzgerlingen)
	SIEMENSSTRASSE_4 = 'de:08115:6535:0:4'  # Rtg Max-Brose-Str. (Holzgerlingen Holzgerlingen)
	SIEMENSSTRASSE_5 = 'de:08115:7048:0:3'  # eRtg Oberjesingen (Deckenpfronn Deckenpfronn)
	SIEMENSSTRASSE_6 = 'de:08117:1136:0:H1'  # Rtg Hanglich (Wangen Wangen (GP))
	SIEMENSSTRASSE_7 = 'de:08117:1136:0:R1'  # Rtg Faurndau (Wangen Wangen (GP))
	SIEMENSSTRASSE_8 = 'de:08117:7799:0:H1'  # eRtg Auchtertstraße (Schlierbach Schlierbach)
	SIEMENSSTRASSE_9 = 'de:08117:7799:0:R1'  # eRtg Tannenhöfe (Schlierbach Schlierbach)
	REBSTEIGE = 'de:08118:3325:0:3'  # eRtg Pappelweg (Markgröningen Markgröningen)
	MARKGRÖNINGEN_REBSTEIGE = 'de:08118:3325:0:3'  # eRtg Pappelweg (Markgröningen Markgröningen)
	OBERESSL__BAHNHOF__SÜD = 'de:08116:3326'
	OBERESSL__BAHNHOF__SÜD_1 = 'de:08116:3326:1:3'  # Rtg Schwertmühle (Esslingen am Neckar Oberesslingen)
	OBERESSL__BAHNHOF__SÜD_2 = 'de:08116:3326:2:4'  # Rtg Neckarfreibad (Esslingen am Neckar Oberesslingen)
	BAHNHOF__SÜD = 'de:08116:3326'
	BAHNHOF__SÜD_1 = 'de:08116:3326:1:3'  # Rtg Schwertmühle (Esslingen am Neckar Oberesslingen)
	BAHNHOF__SÜD_2 = 'de:08116:3326:2:4'  # Rtg Neckarfreibad (Esslingen am Neckar Oberesslingen)
	SCHWIEBERDINGEN_MITTE = 'de:08118:3327'
	SCHWIEBERDINGEN_MITTE_1 = 'de:08118:3327:1:2'  # Pos 2 Rtg Glemstal hinten (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_MITTE_2 = 'de:08118:3327:1:3'  # Pos 3 Rtg Glemstal vorne (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_MITTE_3 = 'de:08118:3327:1:4'  # Pos 1 Rtg H.-Essig-Str./Ludwigsburg Str. (Schwieberdingen Schwieberdingen)
	SCHWIEBERD__LUDWIGSBURGER_STR = 'de:08118:3328'
	SCHWIEBERD__LUDWIGSBURGER_STR_1 = 'de:08118:3328:0:3'  # Rtg Bosch (Schwieberdingen Schwieberdingen)
	SCHWIEBERD__LUDWIGSBURGER_STR_2 = 'de:08118:3328:0:4'  # Rtg Mitte (Schwieberdingen Schwieberdingen)
	KLINIK_SCHULE = 'de:08118:3329'
	KLINIK_SCHULE_1 = 'de:08118:3329:0:3'  # Rtg Behindertenheim (Markgröningen Markgröningen)
	KLINIK_SCHULE_2 = 'de:08118:3329:0:4'  # Rtg Abzweigung Bahnhof (Markgröningen Markgröningen)
	MARKGR__KLINIK_SCHULE = 'de:08118:3329'
	MARKGR__KLINIK_SCHULE_1 = 'de:08118:3329:0:3'  # Rtg Behindertenheim (Markgröningen Markgröningen)
	MARKGR__KLINIK_SCHULE_2 = 'de:08118:3329:0:4'  # Rtg Abzweigung Bahnhof (Markgröningen Markgröningen)
	SCHWIEBERDINGEN_BRAUEREI = 'de:08118:3332:0:3'  # eRtg Ludwigsburger Straße (Schwieberdingen Schwieberdingen)
	BRAUEREI = 'de:08118:3332:0:3'  # eRtg Ludwigsburger Straße (Schwieberdingen Schwieberdingen)
	BIETIGH__FRIEDHOF_SANKT_PETER = 'de:08118:3333:0:4'  # Rtg Ellental (Bietigheim-Bissingen Bietigheim)
	FRIEDHOF_SANKT_PETER = 'de:08118:3333:0:4'  # Rtg Ellental (Bietigheim-Bissingen Bietigheim)
	HEIMERD__GEWERBEGEBIET_SÜD = 'de:08118:3334'
	HEIMERD__GEWERBEGEBIET_SÜD_1 = 'de:08118:3334:0:3'  # eRtg Heimerdingen (Ditzingen Heimerdingen)
	HEIMERD__GEWERBEGEBIET_SÜD_2 = 'de:08118:3334:0:4'  # eRtg Steinbruch (Ditzingen Heimerdingen)
	GEWERBEGEBIET_SÜD_1 = 'de:08118:3334:0:3'  # eRtg Heimerdingen (Ditzingen Heimerdingen)
	GEWERBEGEBIET_SÜD_2 = 'de:08118:3334:0:4'  # eRtg Steinbruch (Ditzingen Heimerdingen)
	GEWERBEGEBIET_SÜD_3 = 'de:08118:7001:0:3'  # Rtg Gerlingen Hofwiesenstr. (Ditzingen Ditzingen)
	GEWERBEGEBIET_SÜD_4 = 'de:08118:7001:0:4'  # Rtg Ditzingen Bf (Ditzingen Ditzingen)
	GEWERBEGEBIET_SÜD_5 = 'de:08118:7001:0:5'  # eRtg Thales (Ditzingen Ditzingen)
	HOF_UND_LEMBACH_TALSTRASSE = 'de:08118:3335'
	HOF_UND_LEMBACH_TALSTRASSE_1 = 'de:08118:3335:0:3'  # eRtg Großbottwar Kelter (Großbottwar Hof und Lembach)
	HOF_UND_LEMBACH_TALSTRASSE_2 = 'de:08118:3335:0:4'  # eRtg Kelter (Großbottwar Hof und Lembach)
	KELTER_1 = 'de:08118:3336:0:3'  # eRtg Ankunft/Abfahrt Bus (Großbottwar Hof und Lembach)
	KELTER_2 = 'de:08116:4430:1:3'  # Rtg Uhlandstr. (Frickenhausen Frickenhausen)
	KELTER_3 = 'de:08116:4430:1:4'  # Rtg Sternbergstr. (Frickenhausen Frickenhausen)
	KELTER_4 = 'de:08119:4928:0:4'  # eRtg Evangelische Kirche (Kernen im Remstal Stetten (i. R.))
	KELTER_5 = 'de:08119:4990:0:3'  # eRtg Däfern/Lutzenberg (Weissach im Tal Bruch (Weissach))
	KELTER_6 = 'de:08119:4990:0:4'  # eRtg Bruch (Weissach im Tal Bruch (Weissach))
	KELTER_7 = 'de:08119:5177:0:3'  # Rtg Rathaus (Weinstadt Strümpfelbach (Weinstadt))
	KELTER_8 = 'de:08119:5177:0:4'  # Rtg Traube (Weinstadt Strümpfelbach (Weinstadt))
	KELTER_9 = 'de:08119:5306:0:3'  # Rtg Hermann-Löns-Weg/Herm.-Schwab-Halle (Winnenden Winnenden)
	KELTER_10 = 'de:08119:5306:0:4'  # Rtg Wallstr./Stöckachschule (Winnenden Winnenden)
	KELTER_11 = 'de:08119:5400:0:3'  # Rtg Lützestr./Buchhaldenstr. (Weinstadt Schnait)
	KELTER_12 = 'de:08118:5603:0:3'  # eRtg Bahnhof (Steinheim an der Murr Steinheim (M))
	KELTER_13 = 'de:08118:5603:0:4'  # eRtg Kleinbottwarer Straße (Steinheim an der Murr Steinheim (M))
	KELTER_14 = 'de:08118:5609:0:3'  # eRtg Rosenplatz (Großbottwar Großbottwar)
	KELTER_15 = 'de:08118:5609:0:4'  # eRtg Flickenwiesen (Großbottwar Großbottwar)
	HOF_UND_LEMBACH_KELTER = 'de:08118:3336:0:3'  # eRtg Ankunft/Abfahrt Bus (Großbottwar Hof und Lembach)
	OBERSTENFELD_SCHULE = 'de:08118:3338:0:3'  # eRtg Beilsteiner Straße (Oberstenfeld Oberstenfeld)
	SCHWIEBERD__SCHULZENTRUM = 'de:08118:3339:0:3'  # alle Richtungen (Schwieberdingen Schwieberdingen)
	SCHULZENTRUM_1 = 'de:08118:3339:0:3'  # alle Richtungen (Schwieberdingen Schwieberdingen)
	SCHULZENTRUM_2 = 'de:08118:3553:0:3'  # eRtg Alle Richtungen (Großbottwar Großbottwar)
	SCHULZENTRUM_3 = 'de:08116:4235:0:3'  # Rtg West (Reichenbach an der Fils Reichenbach (F))
	SCHULZENTRUM_4 = 'de:08116:4235:0:4'  # Rtg Ost (Reichenbach an der Fils Reichenbach (F))
	SCHULZENTRUM_5 = 'de:08116:4296:0:3'  # eRtg Schützenstr./Anne-Frank-Schule (Wendlingen am Neckar Wendlingen (N))
	SCHULZENTRUM_6 = 'de:08116:5021:1:3'  # Rtg Riegelstraße/Kreuzbrunnen (Ostfildern Nellingen)
	SCHULZENTRUM_7 = 'de:08119:5121:0:3'  # eRtg Alle Richtungen (Rudersberg Rudersberg)
	SCHULZENTRUM_8 = 'de:08119:5131:0:3'  # eRtg Alle Richtungen (Welzheim Welzheim)
	SCHULZENTRUM_9 = 'de:08118:5727:0:3'  # Rtg Burgplatz/Kirchheimer Str. (Bönnigheim Bönnigheim)
	SCHULZENTRUM_10 = 'de:08118:5727:0:4'  # Rtg Freudental Besigh. Str. (Bönnigheim Bönnigheim)
	SCHULZENTRUM_11 = 'de:08115:5829:0:3'  # alle Richtungen (Rutesheim Rutesheim)
	SCHULZENTRUM_12 = 'de:08118:6988:0:3'  # eRtg Höpfigheimer Straße (Steinheim an der Murr Steinheim (M))
	SCHULZENTRUM_13 = 'de:08118:6988:0:4'  # eRtg Horrenwinkel (Steinheim an der Murr Steinheim (M))
	SCHULZENTRUM_14 = 'de:08117:7623:0:H1'  # Rtg Steingartenschule (Donzdorf Donzdorf)
	SCHULZENTRUM_15 = 'de:08125:32310:0:1'  # eRtg Langhans (Beilstein Beilstein)
	SCHULZENTRUM_16 = 'de:08425:2653:0:3'  # Linie 360 (Laichingen Laichingen)
	BIETIGHEIM_FREIBERGER_STR = 'de:08118:3342'
	BIETIGHEIM_FREIBERGER_STR_1 = 'de:08118:3342:0:3'  # eRtg Buch Buchzentrum (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_FREIBERGER_STR_2 = 'de:08118:3342:0:4'  # Rtg ZOB (Bietigheim-Bissingen Bietigheim)
	FREIBERGER_STRASSE = 'de:08118:3342'
	FREIBERGER_STRASSE_1 = 'de:08118:3342:0:3'  # eRtg Buch Buchzentrum (Bietigheim-Bissingen Bietigheim)
	FREIBERGER_STRASSE_2 = 'de:08118:3342:0:4'  # Rtg ZOB (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_AGENTUR_FÜR_ARBEIT = 'de:08118:3343'
	BIETIGHEIM_AGENTUR_FÜR_ARBEIT_1 = 'de:08118:3343:0:3'  # eRtg Buch Buchzentrum (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_AGENTUR_FÜR_ARBEIT_2 = 'de:08118:3343:0:4'  # Rtg Gustav-Rau-Str. (Bietigheim-Bissingen Bietigheim)
	AGENTUR_FÜR_ARBEIT_1 = 'de:08118:3343:0:3'  # eRtg Buch Buchzentrum (Bietigheim-Bissingen Bietigheim)
	AGENTUR_FÜR_ARBEIT_2 = 'de:08118:3343:0:4'  # Rtg Gustav-Rau-Str. (Bietigheim-Bissingen Bietigheim)
	AGENTUR_FÜR_ARBEIT_3 = 'de:08118:5529:0:3'  # eRtg Landratsamt (Ludwigsburg Ludwigsburg)
	AGENTUR_FÜR_ARBEIT_4 = 'de:08118:5529:0:4'  # eRtg Karlsplatz (Ludwigsburg Ludwigsburg)
	GUSTAV_RAU_STRASSE = 'de:08118:3344'
	GUSTAV_RAU_STRASSE_1 = 'de:08118:3344:0:3'  # eRtg Agentur für Arbeit (Bietigheim-Bissingen Bietigheim)
	GUSTAV_RAU_STRASSE_2 = 'de:08118:3344:0:4'  # Rtg Steinheimer Str. (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_GUSTAV_RAU_STR = 'de:08118:3344'
	BIETIGHEIM_GUSTAV_RAU_STR_1 = 'de:08118:3344:0:3'  # eRtg Agentur für Arbeit (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_GUSTAV_RAU_STR_2 = 'de:08118:3344:0:4'  # Rtg Steinheimer Str. (Bietigheim-Bissingen Bietigheim)
	SPIELBERG_BRUNNENWIESEN = 'de:08118:3346'
	SPIELBERG_BRUNNENWIESEN_1 = 'de:08118:3346:0:3'  # eRtg Ochsen (Sachsenheim Spielberg)
	SPIELBERG_BRUNNENWIESEN_2 = 'de:08118:3346:0:4'  # eRtg Brombergerhöfe (Sachsenheim Spielberg)
	BRUNNENWIESEN = 'de:08118:3346'
	BRUNNENWIESEN_1 = 'de:08118:3346:0:3'  # eRtg Ochsen (Sachsenheim Spielberg)
	BRUNNENWIESEN_2 = 'de:08118:3346:0:4'  # eRtg Brombergerhöfe (Sachsenheim Spielberg)
	KRONE_1 = 'de:08118:3347:0:3'  # eRtg Schule (Oberstenfeld Gronau)
	KRONE_2 = 'de:08118:3347:0:4'  # eRtg Prevorst Löwen (Oberstenfeld Gronau)
	KRONE_3 = 'de:08117:7663:0:H1'  # Rtg Am Kugelrain (Uhingen Baiereck)
	KRONE_4 = 'de:08117:7663:0:R1'  # Rtg Herrenbachstr. (Uhingen Baiereck)
	GRONAU_KRONE = 'de:08118:3347'
	GRONAU_KRONE_1 = 'de:08118:3347:0:3'  # eRtg Schule (Oberstenfeld Gronau)
	GRONAU_KRONE_2 = 'de:08118:3347:0:4'  # eRtg Prevorst Löwen (Oberstenfeld Gronau)
	HÜLBE = 'de:08118:3348:0:3'  # eRtg Breslauer Straße (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_HÜLBE = 'de:08118:3348:0:3'  # eRtg Breslauer Straße (Schwieberdingen Schwieberdingen)
	LAIBLINGER_WEG = 'de:08118:3349'
	LAIBLINGER_WEG_1 = 'de:08118:3349:0:3'  # Rtg Siemensstraße (Schwieberdingen Schwieberdingen)
	LAIBLINGER_WEG_2 = 'de:08118:3349:0:4'  # Rtg Bosch Tor IV (Schwieberdingen Schwieberdingen)
	SCHWIEBERD__LAIBLINGER_WEG = 'de:08118:3349'
	SCHWIEBERD__LAIBLINGER_WEG_1 = 'de:08118:3349:0:3'  # Rtg Siemensstraße (Schwieberdingen Schwieberdingen)
	SCHWIEBERD__LAIBLINGER_WEG_2 = 'de:08118:3349:0:4'  # Rtg Bosch Tor IV (Schwieberdingen Schwieberdingen)
	KLEINGLATTBACH_WEIZENSTRASSE = 'de:08118:3352'
	KLEINGLATTBACH_WEIZENSTRASSE_1 = 'de:08118:3352:0:3'  # eRtg Schillerstraße (Vaihingen an der Enz Kleinglattbach)
	KLEINGLATTBACH_WEIZENSTRASSE_2 = 'de:08118:3352:0:4'  # eRtg Im See (Vaihingen an der Enz Kleinglattbach)
	WEIZENSTRASSE = 'de:08118:3352'
	WEIZENSTRASSE_1 = 'de:08118:3352:0:3'  # eRtg Schillerstraße (Vaihingen an der Enz Kleinglattbach)
	WEIZENSTRASSE_2 = 'de:08118:3352:0:4'  # eRtg Im See (Vaihingen an der Enz Kleinglattbach)
	ENSINGER_STRASSE = 'de:08118:3353:0:3'  # eRtg Kleinglattbacher Straße (Vaihingen an der Enz Kleinglattbach)
	KLEINGLATTBACH_ENSINGER_STR = 'de:08118:3353:0:3'  # eRtg Kleinglattbacher Straße (Vaihingen an der Enz Kleinglattbach)
	EIERHÄUSLE = 'de:08118:3359'
	EIERHÄUSLE_1 = 'de:08118:3359:0:3'  # Rtg Hardthof (Schwieberdingen Schwieberdingen)
	EIERHÄUSLE_2 = 'de:08118:3359:0:4'  # Rtg Schwieberdingen Glemstal (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_EIERHÄUSLE = 'de:08118:3359'
	SCHWIEBERDINGEN_EIERHÄUSLE_1 = 'de:08118:3359:0:3'  # Rtg Hardthof (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_EIERHÄUSLE_2 = 'de:08118:3359:0:4'  # Rtg Schwieberdingen Glemstal (Schwieberdingen Schwieberdingen)
	KORNWESTHEIM_W___W = 'de:08118:3361:0:4'  # Endhaltestelle (Kornwestheim Kornwestheim)
	W___W = 'de:08118:3361:0:4'  # Endhaltestelle (Kornwestheim Kornwestheim)
	IM_MOLDENGRABEN = 'de:08118:3363'
	IM_MOLDENGRABEN_1 = 'de:08118:3363:0:3'  # Rtg Leibnizstr. (Kornwestheim Kornwestheim)
	IM_MOLDENGRABEN_2 = 'de:08118:3363:0:4'  # Rtg Talstr. (Kornwestheim Kornwestheim)
	KORNWESTHEIM_IM_MOLDENGRABEN = 'de:08118:3363'
	KORNWESTHEIM_IM_MOLDENGRABEN_1 = 'de:08118:3363:0:3'  # Rtg Leibnizstr. (Kornwestheim Kornwestheim)
	KORNWESTHEIM_IM_MOLDENGRABEN_2 = 'de:08118:3363:0:4'  # Rtg Talstr. (Kornwestheim Kornwestheim)
	KORNWESTHEIM_LEIBNIZSTRASSE = 'de:08118:3364'
	KORNWESTHEIM_LEIBNIZSTRASSE_1 = 'de:08118:3364:0:3'  # Rtg Lenzhalde (Kornwestheim Kornwestheim)
	KORNWESTHEIM_LEIBNIZSTRASSE_2 = 'de:08118:3364:0:4'  # Rtg Im Moldengraben (Kornwestheim Kornwestheim)
	PARKÄCKER = 'de:08118:3383'
	PARKÄCKER_1 = 'de:08118:3383:0:3'  # eRtg Mönchsbergstraße (Bietigheim-Bissingen Bietigheim)
	PARKÄCKER_2 = 'de:08118:3383:0:4'  # eRtg Im Feldle (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_PARKÄCKER = 'de:08118:3383'
	BIETIGHEIM_PARKÄCKER_1 = 'de:08118:3383:0:3'  # eRtg Mönchsbergstraße (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_PARKÄCKER_2 = 'de:08118:3383:0:4'  # eRtg Im Feldle (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_IM_FELDLE = 'de:08118:3384'
	BIETIGHEIM_IM_FELDLE_1 = 'de:08118:3384:0:3'  # eRtg Parkäcker (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_IM_FELDLE_2 = 'de:08118:3384:0:4'  # eRtg Freiberger Straße (Bietigheim-Bissingen Bietigheim)
	IM_FELDLE = 'de:08118:3384'
	IM_FELDLE_1 = 'de:08118:3384:0:3'  # eRtg Parkäcker (Bietigheim-Bissingen Bietigheim)
	IM_FELDLE_2 = 'de:08118:3384:0:4'  # eRtg Freiberger Straße (Bietigheim-Bissingen Bietigheim)
	BUCH_BUCHZENTRUM = 'de:08118:3385:0:4'  # eRtg Agentur für Arbeit/Freiberger Str. (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_BUCH_BUCHZENTRUM = 'de:08118:3385:0:4'  # eRtg Agentur für Arbeit/Freiberger Str. (Bietigheim-Bissingen Bietigheim)
	BISSINGEN_TAMMER_STR = 'de:08118:3386'
	BISSINGEN_TAMMER_STR_1 = 'de:08118:3386:0:3'  # eRtg Schubartstraße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_TAMMER_STR_2 = 'de:08118:3386:0:4'  # eRtg Südstraße (Bietigheim-Bissingen Bissingen (LB))
	TAMMER_STRASSE_1 = 'de:08118:3386:0:3'  # eRtg Schubartstraße (Bietigheim-Bissingen Bissingen (LB))
	TAMMER_STRASSE_2 = 'de:08118:3386:0:4'  # eRtg Südstraße (Bietigheim-Bissingen Bissingen (LB))
	TAMMER_STRASSE_3 = 'de:08118:5512:0:3'  # eRtg Frankfurter Straße (Ludwigsburg Eglosheim)
	TAMMER_STRASSE_4 = 'de:08118:5512:0:4'  # eRtg Falkenweg (Ludwigsburg Eglosheim)
	DÜRR_ROSENSTRASSE = 'de:08118:3388'
	DÜRR_ROSENSTRASSE_1 = 'de:08118:3388:0:3'  # eRtg Dürr/Marbacher Weg (Bietigheim-Bissingen Bissingen (LB))
	DÜRR_ROSENSTRASSE_2 = 'de:08118:3388:0:4'  # eRtg Panoramastraße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_DÜRR_ROSENSTR = 'de:08118:3388'
	BISSINGEN_DÜRR_ROSENSTR_1 = 'de:08118:3388:0:3'  # eRtg Dürr/Marbacher Weg (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_DÜRR_ROSENSTR_2 = 'de:08118:3388:0:4'  # eRtg Panoramastraße (Bietigheim-Bissingen Bissingen (LB))
	BIETIGHEIM_HAUS_AN_DER_METTER = 'de:08118:3390:0:4'  # Rtg Friedhof St. Peter (Bietigheim-Bissingen Bietigheim)
	HAUS_AN_DER_METTER = 'de:08118:3390:0:4'  # Rtg Friedhof St. Peter (Bietigheim-Bissingen Bietigheim)
	DÜRR_MARBACHER_WEG = 'de:08118:3394'
	DÜRR_MARBACHER_WEG_1 = 'de:08118:3394:0:3'  # eRtg Carl-Benz-Straße (Bietigheim-Bissingen Bissingen (LB))
	DÜRR_MARBACHER_WEG_2 = 'de:08118:3394:0:4'  # eRtg Karlstraße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_DÜRR_MARBACHER_WEG = 'de:08118:3394'
	BISSINGEN_DÜRR_MARBACHER_WEG_1 = 'de:08118:3394:0:3'  # eRtg Carl-Benz-Straße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_DÜRR_MARBACHER_WEG_2 = 'de:08118:3394:0:4'  # eRtg Karlstraße (Bietigheim-Bissingen Bissingen (LB))
	THEO_LORCH_WERKSTÄTTEN_1 = 'de:08118:3395:0:3'  # eRtg Bietigheim (Bietigheim-Bissingen Bietigheim)
	THEO_LORCH_WERKSTÄTTEN_2 = 'de:08118:7438:0:3'  # eRtg Ludwigsburg (Ludwigsburg Ludwigsburg)
	THEO_LORCH_WERKSTÄTTEN_3 = 'de:08118:7438:0:4'  # eRtg Grünbühl/Pattonville (Ludwigsburg Ludwigsburg)
	BIETIGHEIM_THEO_LORCH_WERKST = 'de:08118:3395:0:3'  # eRtg Bietigheim (Bietigheim-Bissingen Bietigheim)
	TAMM_ROBERT_BOSCH_STR = 'de:08118:3399'
	TAMM_ROBERT_BOSCH_STR_1 = 'de:08118:3399:0:3'  # eRtg Theo-Lorch-Werkstätten (Tamm Tamm)
	TAMM_ROBERT_BOSCH_STR_2 = 'de:08118:3399:0:4'  # eRtg Laiern-/R.-Bosch-Str. (Tamm Tamm)
	ASPERG_GYMNASIUM = 'de:08118:3400:0:3'  # eRtg Ankunft/Möglinger Straße (Asperg Asperg)
	GYMNASIUM_1 = 'de:08118:3400:0:3'  # eRtg Ankunft/Möglinger Straße (Asperg Asperg)
	GYMNASIUM_2 = 'de:08119:4918:0:3'  # eRtg Murrhardt (Murrhardt Murrhardt)
	GYMNASIUM_3 = 'de:08115:6477:0:3'  # eRtg Alle Richtungen (Weil der Stadt Weil der Stadt)
	GYMNASIUM_4 = 'de:08115:6506:0:3'  # eRtg Neuweiler (Holzgerlingen Holzgerlingen)
	GYMNASIUM_5 = 'de:08115:6506:2:4'  # eRtg Holzgerlingen (Holzgerlingen Holzgerlingen)
	GYMNASIUM_6 = 'de:08116:7185:0:3'  # Rtg Stangen (Leinfelden-Echterdingen Echterdingen)
	GYMNASIUM_7 = 'de:08116:7185:0:4'  # Rtg Leinfelder Haus (Leinfelden-Echterdingen Echterdingen)
	GYMNASIUM_8 = 'de:08118:7606:0:1'  # R-Bahn (Korntal-Münchingen Korntal)
	BIETIGHEIM_POSTSTRÄSSLE = 'de:08118:3401'
	BIETIGHEIM_POSTSTRÄSSLE_1 = 'de:08118:3401:0:3'  # eRtg Steinheimer Straße (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_POSTSTRÄSSLE_2 = 'de:08118:3401:0:4'  # Rtg Bolzstr./Schillerplatz (Bietigheim-Bissingen Bietigheim)
	POSTSTRÄSSLE = 'de:08118:3401'
	POSTSTRÄSSLE_1 = 'de:08118:3401:0:3'  # eRtg Steinheimer Straße (Bietigheim-Bissingen Bietigheim)
	POSTSTRÄSSLE_2 = 'de:08118:3401:0:4'  # Rtg Bolzstr./Schillerplatz (Bietigheim-Bissingen Bietigheim)
	MÖGLINGER_STRASSE_1 = 'de:08118:3402:0:1'  # Rtg Gymnasium/Möglingen Bahnhofstr. (Asperg Asperg)
	MÖGLINGER_STRASSE_2 = 'de:08118:3402:0:2'  # Rtg Stadtmitte (Asperg Asperg)
	MÖGLINGER_STRASSE_3 = 'de:08118:3429:0:3'  # Rtg Karlsruher Allee (Ludwigsburg Pflugfelden)
	MÖGLINGER_STRASSE_4 = 'de:08118:3429:0:4'  # Rtg Robert-Bosch-Str. (Ludwigsburg Pflugfelden)
	ASPERG_MÖGLINGER_STRASSE = 'de:08118:3402'
	ASPERG_MÖGLINGER_STRASSE_1 = 'de:08118:3402:0:1'  # Rtg Gymnasium/Möglingen Bahnhofstr. (Asperg Asperg)
	ASPERG_MÖGLINGER_STRASSE_2 = 'de:08118:3402:0:2'  # Rtg Stadtmitte (Asperg Asperg)
	ASPERG_SAARSTRASSE = 'de:08118:3404'
	ASPERG_SAARSTRASSE_1 = 'de:08118:3404:0:3'  # eRtg Asperg (Asperg Asperg)
	ASPERG_SAARSTRASSE_2 = 'de:08118:3404:0:4'  # eRtg Neckarstraße (Asperg Asperg)
	SAARSTRASSE_1 = 'de:08118:3404:0:3'  # eRtg Asperg (Asperg Asperg)
	SAARSTRASSE_2 = 'de:08118:3404:0:4'  # eRtg Neckarstraße (Asperg Asperg)
	SAARSTRASSE_3 = 'de:08116:3948:0:3'  # eRtg Austraße (Kirchheim unter Teck Kirchheim (T))
	SAARSTRASSE_4 = 'de:08116:3948:0:4'  # eRtg Kitteneshalde (Kirchheim unter Teck Kirchheim (T))
	WEINSTRASSE_1 = 'de:08118:3405:0:3'  # Rtg Markgröninger Str. (Asperg Asperg)
	WEINSTRASSE_2 = 'de:08118:3405:0:4'  # Rtg Stadtmitte (Asperg Asperg)
	WEINSTRASSE_3 = 'de:08116:4012:0:3'  # Rtg Mettingen (Esslingen am Neckar Mettingen)
	WEINSTRASSE_4 = 'de:08116:4012:0:4'  # Rtg Obertürkheim (Esslingen am Neckar Mettingen)
	ASPERG_WEINSTRASSE = 'de:08118:3405'
	ASPERG_WEINSTRASSE_1 = 'de:08118:3405:0:3'  # Rtg Markgröninger Str. (Asperg Asperg)
	ASPERG_WEINSTRASSE_2 = 'de:08118:3405:0:4'  # Rtg Stadtmitte (Asperg Asperg)
	ASPERG_WILHELMSPLATZ = 'de:08118:3406'
	ASPERG_WILHELMSPLATZ_1 = 'de:08118:3406:0:3'  # eRtg Volksbank (Asperg Asperg)
	ASPERG_WILHELMSPLATZ_2 = 'de:08118:3406:0:4'  # eRtg Asperg (Asperg Asperg)
	WILHELMSPLATZ_1 = 'de:08118:3406:0:3'  # eRtg Volksbank (Asperg Asperg)
	WILHELMSPLATZ_2 = 'de:08118:3406:0:4'  # eRtg Asperg (Asperg Asperg)
	WILHELMSPLATZ_3 = 'de:08118:6821:0:3'  # eRtg Poppenweiler Straße (Remseck am Neckar Hochdorf (Remseck))
	WILHELMSPLATZ_4 = 'de:08118:6821:0:4'  # eRtg Hochberger Straße (Remseck am Neckar Hochdorf (Remseck))
	HOHENSTAUFENSTRASSE_1 = 'de:08118:3407:0:3'  # Rtg Hirschbergstr. (Asperg Asperg)
	HOHENSTAUFENSTRASSE_2 = 'de:08118:3407:0:4'  # Rtg Asperg (Asperg Asperg)
	HOHENSTAUFENSTRASSE_3 = 'de:08119:5979:0:3'  # eRtg Kleingärten (Schorndorf Schorndorf)
	HOHENSTAUFENSTRASSE_4 = 'de:08119:5979:0:4'  # eRtg Reinhold-Maier-Platz (Schorndorf Schorndorf)
	ASPERG_HOHENSTAUFENSTR = 'de:08118:3407'
	ASPERG_HOHENSTAUFENSTR_1 = 'de:08118:3407:0:3'  # Rtg Hirschbergstr. (Asperg Asperg)
	ASPERG_HOHENSTAUFENSTR_2 = 'de:08118:3407:0:4'  # Rtg Asperg (Asperg Asperg)
	LUDWIGSBURG_ARSENALPLATZ = 'de:08118:3408'
	LUDWIGSBURG_ARSENALPLATZ_1 = 'de:08118:3408:1:3'  # eRtg ZOB (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_ARSENALPLATZ_2 = 'de:08118:3408:1:4'  # eRtg Rathaus (Ludwigsburg Ludwigsburg)
	ARSENALPLATZ = 'de:08118:3408'
	ARSENALPLATZ_1 = 'de:08118:3408:1:3'  # eRtg ZOB (Ludwigsburg Ludwigsburg)
	ARSENALPLATZ_2 = 'de:08118:3408:1:4'  # eRtg Rathaus (Ludwigsburg Ludwigsburg)
	EGLOSHEIM_A__BEBEL_STR = 'de:08118:3409'
	EGLOSHEIM_A__BEBEL_STR_1 = 'de:08118:3409:0:3'  # Richtung Nord (Ludwigsburg Eglosheim)
	EGLOSHEIM_A__BEBEL_STR_2 = 'de:08118:3409:0:4'  # Richtung Süd (Ludwigsburg Eglosheim)
	EGLOSHEIM_A__BEBEL_STR_3 = 'de:08118:3409:0:6'  # Rtg Weimarstr. (Ludwigsburg Eglosheim)
	AUGUST_BEBEL_STRASSE = 'de:08118:3409'
	AUGUST_BEBEL_STRASSE_1 = 'de:08118:3409:0:3'  # Richtung Nord (Ludwigsburg Eglosheim)
	AUGUST_BEBEL_STRASSE_2 = 'de:08118:3409:0:4'  # Richtung Süd (Ludwigsburg Eglosheim)
	AUGUST_BEBEL_STRASSE_3 = 'de:08118:3409:0:6'  # Rtg Weimarstr. (Ludwigsburg Eglosheim)
	LUDWIGSBURG_LOTTER = 'de:08118:3410'
	LUDWIGSBURG_LOTTER_1 = 'de:08118:3410:0:3'  # eRtg Daimlerstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_LOTTER_2 = 'de:08118:3410:0:4'  # eRtg Mörikestraße (Ludwigsburg Ludwigsburg)
	LOTTER = 'de:08118:3410'
	LOTTER_1 = 'de:08118:3410:0:3'  # eRtg Daimlerstraße (Ludwigsburg Ludwigsburg)
	LOTTER_2 = 'de:08118:3410:0:4'  # eRtg Mörikestraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_GÄNSFUSSALLEE = 'de:08118:3411'
	LUDWIGSBURG_GÄNSFUSSALLEE_1 = 'de:08118:3411:0:3'  # eRtg Jenischstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_GÄNSFUSSALLEE_2 = 'de:08118:3411:0:4'  # eRtg Hoferstraße/ZOB (Ludwigsburg Ludwigsburg)
	GÄNSFUSSALLEE = 'de:08118:3411'
	GÄNSFUSSALLEE_1 = 'de:08118:3411:0:3'  # eRtg Jenischstraße (Ludwigsburg Ludwigsburg)
	GÄNSFUSSALLEE_2 = 'de:08118:3411:0:4'  # eRtg Hoferstraße/ZOB (Ludwigsburg Ludwigsburg)
	HEILBRONNER_STRASSE = 'de:08118:3412'
	HEILBRONNER_STRASSE_1 = 'de:08118:3412:0:3'  # Richtung West (Ludwigsburg Ludwigsburg)
	HEILBRONNER_STRASSE_2 = 'de:08118:3412:0:4'  # Richtung Ost (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_HEILBRONNER_STR = 'de:08118:3412'
	LUDWIGSBURG_HEILBRONNER_STR_1 = 'de:08118:3412:0:3'  # Richtung West (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_HEILBRONNER_STR_2 = 'de:08118:3412:0:4'  # Richtung Ost (Ludwigsburg Ludwigsburg)
	HEUBERGSTRASSE = 'de:08118:3413:0:3'  # eRtg Hornbergstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_HEUBERGSTR = 'de:08118:3413:0:3'  # eRtg Hornbergstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_HAUFFSTR = 'de:08118:3414:1:3'  # eRtg Stadion (Kornwestheim Kornwestheim)
	LUDWIGSBURG_RUNDSPORTHALLE = 'de:08118:3415'
	LUDWIGSBURG_RUNDSPORTHALLE_1 = 'de:08118:3415:0:3'  # eRtg Martin-Luther-Straße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_RUNDSPORTHALLE_2 = 'de:08118:3415:0:4'  # eRtg August-Bebel-Straße (Ludwigsburg Ludwigsburg)
	RUNDSPORTHALLE_1 = 'de:08118:3415:0:3'  # eRtg Martin-Luther-Straße (Ludwigsburg Ludwigsburg)
	RUNDSPORTHALLE_2 = 'de:08118:3415:0:4'  # eRtg August-Bebel-Straße (Ludwigsburg Ludwigsburg)
	RUNDSPORTHALLE_3 = 'de:08119:5161:0:3'  # Rtg Beinstein (Waiblingen Waiblingen)
	RUNDSPORTHALLE_4 = 'de:08119:5161:0:4'  # Rtg Am Kätzenbach (Waiblingen Waiblingen)
	LUDWIGSBURG_UHLANDSTRASSE = 'de:08118:3416'
	LUDWIGSBURG_UHLANDSTRASSE_1 = 'de:08118:3416:0:3'  # eRtg Abelstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_UHLANDSTRASSE_2 = 'de:08118:3416:0:4'  # eRtg Ludwigsburg (Ludwigsburg Ludwigsburg)
	UHLANDSTRASSE_1 = 'de:08118:3416:0:3'  # eRtg Abelstraße (Ludwigsburg Ludwigsburg)
	UHLANDSTRASSE_2 = 'de:08118:3416:0:4'  # eRtg Ludwigsburg (Ludwigsburg Ludwigsburg)
	UHLANDSTRASSE_3 = 'de:08117:3309:0:H1'  # Rtg Mörikestr. (Heiningen Heiningen (GP))
	UHLANDSTRASSE_4 = 'de:08117:3309:0:R1'  # Rtg Adler (Heiningen Heiningen (GP))
	UHLANDSTRASSE_5 = 'de:08117:5002:0:H1'  # Rtg Panoramastr./Bahnhof (Göppingen Faurndau)
	UHLANDSTRASSE_6 = 'de:08117:5013:0:H1'  # Rtg Backhausen (Wangen Oberwälden)
	UHLANDSTRASSE_7 = 'de:08117:5013:0:R1'  # Rtg Rechberghausen (Wangen Oberwälden)
	MONREPOS = 'de:08118:3417:0:4'  # eRtg Eglosheim (Ludwigsburg Eglosheim)
	EGLOSHEIM_MONREPOS = 'de:08118:3417:0:4'  # eRtg Eglosheim (Ludwigsburg Eglosheim)
	LUDWIGSBURG_NECKARBRÜCKE = 'de:08118:3418'
	LUDWIGSBURG_NECKARBRÜCKE_1 = 'de:08118:3418:0:3'  # Rtg H.-Hesse (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_NECKARBRÜCKE_2 = 'de:08118:3418:0:4'  # Rtg ZOB (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_RATHAUS = 'de:08118:3419'
	LUDWIGSBURG_RATHAUS_1 = 'de:08118:3419:2:3'  # eRtg ZOB (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_RATHAUS_2 = 'de:08118:3419:3:4'  # eRtg Residenzschl./Blühendes Barock/LRA (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_SOLITUDEALLEE = 'de:08118:3420'
	LUDWIGSBURG_SOLITUDEALLEE_1 = 'de:08118:3420:0:3'  # eRtg Köhlstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_SOLITUDEALLEE_2 = 'de:08118:3420:0:4'  # eRtg Ludwigsburg (Ludwigsburg Ludwigsburg)
	SOLITUDEALLEE = 'de:08118:3420'
	SOLITUDEALLEE_1 = 'de:08118:3420:0:3'  # eRtg Köhlstraße (Ludwigsburg Ludwigsburg)
	SOLITUDEALLEE_2 = 'de:08118:3420:0:4'  # eRtg Ludwigsburg (Ludwigsburg Ludwigsburg)
	TEINACHER_STRASSE = 'de:08118:3421'
	TEINACHER_STRASSE_1 = 'de:08118:3421:0:3'  # eRtg Strombergstraße/Markgröninger Str. (Ludwigsburg Eglosheim)
	TEINACHER_STRASSE_2 = 'de:08118:3421:0:4'  # Rtg Rundsporthalle (Ludwigsburg Eglosheim)
	EGLOSHEIM_TEINACHER_STR = 'de:08118:3421'
	EGLOSHEIM_TEINACHER_STR_1 = 'de:08118:3421:0:3'  # eRtg Strombergstraße/Markgröninger Str. (Ludwigsburg Eglosheim)
	EGLOSHEIM_TEINACHER_STR_2 = 'de:08118:3421:0:4'  # Rtg Rundsporthalle (Ludwigsburg Eglosheim)
	MARKGRÖNINGER_STRASSE_1 = 'de:08118:3422:0:3'  # Rtg Hirschbergstr./Theurerstr. (Ludwigsburg Eglosheim)
	MARKGRÖNINGER_STRASSE_2 = 'de:08118:3422:0:4'  # Rtg Ludwigsburg (Ludwigsburg Eglosheim)
	MARKGRÖNINGER_STRASSE_3 = 'de:08118:3440:0:3'  # Rtg Markgröningen (Möglingen Möglingen)
	MARKGRÖNINGER_STRASSE_4 = 'de:08118:3440:0:4'  # Rtg Möglingen (Möglingen Möglingen)
	MARKGRÖNINGER_STRASSE_5 = 'de:08118:5481:0:3'  # eRtg Behindertenheim (Asperg Asperg)
	MARKGRÖNINGER_STRASSE_6 = 'de:08118:5481:0:4'  # eRtg Weinstraße (Asperg Asperg)
	EGLOSHEIM_MARKGRÖNINGER_STR = 'de:08118:3422'
	EGLOSHEIM_MARKGRÖNINGER_STR_1 = 'de:08118:3422:0:3'  # Rtg Hirschbergstr./Theurerstr. (Ludwigsburg Eglosheim)
	EGLOSHEIM_MARKGRÖNINGER_STR_2 = 'de:08118:3422:0:4'  # Rtg Ludwigsburg (Ludwigsburg Eglosheim)
	FRANKFURTER_STRASSE = 'de:08118:3423'
	FRANKFURTER_STRASSE_1 = 'de:08118:3423:1:3'  # stadtauswärts (Ludwigsburg Eglosheim)
	FRANKFURTER_STRASSE_2 = 'de:08118:3423:1:4'  # stadteinwärts (Ludwigsburg Eglosheim)
	EGLOSHEIM_FRANKFURTER_STR = 'de:08118:3423'
	EGLOSHEIM_FRANKFURTER_STR_1 = 'de:08118:3423:1:3'  # stadtauswärts (Ludwigsburg Eglosheim)
	EGLOSHEIM_FRANKFURTER_STR_2 = 'de:08118:3423:1:4'  # stadteinwärts (Ludwigsburg Eglosheim)
	KORNWESTH__ERNST_SIGLE_GYMN = 'de:08118:3424:0:3'  # eRtg Rathaus (Kornwestheim Kornwestheim)
	ERNST_SIGLE_GYMNASIUM = 'de:08118:3424:0:3'  # eRtg Rathaus (Kornwestheim Kornwestheim)
	BOTTWARTALSTRASSE = 'de:08118:3425'
	BOTTWARTALSTRASSE_1 = 'de:08118:3425:0:3'  # Rtg Ludwigsburger-/Beihinger Str. (Ludwigsburg Hoheneck)
	BOTTWARTALSTRASSE_2 = 'de:08118:3425:0:4'  # Rtg Reichertshalde (Ludwigsburg Hoheneck)
	HOHENECK_BOTTWARTALSTR = 'de:08118:3425'
	HOHENECK_BOTTWARTALSTR_1 = 'de:08118:3425:0:3'  # Rtg Ludwigsburger-/Beihinger Str. (Ludwigsburg Hoheneck)
	HOHENECK_BOTTWARTALSTR_2 = 'de:08118:3425:0:4'  # Rtg Reichertshalde (Ludwigsburg Hoheneck)
	NECKARWEIHINGEN_LECHTSTR = 'de:08118:3426'
	NECKARWEIHINGEN_LECHTSTR_1 = 'de:08118:3426:0:3'  # eRtg Peter-Hebel-Str./Hauptstr. 99 (Ludwigsburg Neckarweihingen)
	NECKARWEIHINGEN_LECHTSTR_2 = 'de:08118:3426:0:4'  # Rtg Hoheneck Heilbad/Lbg Neckarbrücke (Ludwigsburg Neckarweihingen)
	LECHTSTRASSE = 'de:08118:3426'
	LECHTSTRASSE_1 = 'de:08118:3426:0:3'  # eRtg Peter-Hebel-Str./Hauptstr. 99 (Ludwigsburg Neckarweihingen)
	LECHTSTRASSE_2 = 'de:08118:3426:0:4'  # Rtg Hoheneck Heilbad/Lbg Neckarbrücke (Ludwigsburg Neckarweihingen)
	NECKARWEIH__WENDEPLATTE = 'de:08118:3428'
	NECKARWEIH__WENDEPLATTE_1 = 'de:08118:3428:0:3'  # eRtg Hauptstraße 99 (Ludwigsburg Neckarweihingen)
	NECKARWEIH__WENDEPLATTE_2 = 'de:08118:3428:0:4'  # eRtg Schwarzwaldstraße (Ludwigsburg Neckarweihingen)
	WENDEPLATTE_1 = 'de:08118:3428:0:3'  # eRtg Hauptstraße 99 (Ludwigsburg Neckarweihingen)
	WENDEPLATTE_2 = 'de:08118:3428:0:4'  # eRtg Schwarzwaldstraße (Ludwigsburg Neckarweihingen)
	WENDEPLATTE_3 = 'de:08118:5548:0:3'  # eRtg Sonne (Remseck am Neckar Hochdorf (Remseck))
	WENDEPLATTE_4 = 'de:08127:28270:0:1'  # eRtg Alle Richtungen (Mainhardt Mainhardt)
	PFLUGFELDEN_MÖGLINGER_STR = 'de:08118:3429'
	PFLUGFELDEN_MÖGLINGER_STR_1 = 'de:08118:3429:0:3'  # Rtg Karlsruher Allee (Ludwigsburg Pflugfelden)
	PFLUGFELDEN_MÖGLINGER_STR_2 = 'de:08118:3429:0:4'  # Rtg Robert-Bosch-Str. (Ludwigsburg Pflugfelden)
	MÖGLINGEN_AUSSIEDLERHÖFE = 'de:08118:3430'
	MÖGLINGEN_AUSSIEDLERHÖFE_1 = 'de:08118:3430:0:3'  # eRtg Pflugfelder Straße (Möglingen Möglingen)
	MÖGLINGEN_AUSSIEDLERHÖFE_2 = 'de:08118:3430:0:4'  # eRtg Hindenburgstraße (Möglingen Möglingen)
	AUSSIEDLERHÖFE_1 = 'de:08118:3430:0:3'  # eRtg Pflugfelder Straße (Möglingen Möglingen)
	AUSSIEDLERHÖFE_2 = 'de:08118:3430:0:4'  # eRtg Hindenburgstraße (Möglingen Möglingen)
	AUSSIEDLERHÖFE_3 = 'de:08118:5619:0:3'  # eRtg Raiffeisenstraße (Großbottwar Großbottwar)
	AUSSIEDLERHÖFE_4 = 'de:08118:5619:0:4'  # eRtg In den Frauengärten/Bachstraße (Großbottwar Großbottwar)
	AUSSIEDLERHÖFE_5 = 'de:08118:6721:0:3'  # eRtg Schlossbergallee (Bönnigheim Hofen (Bönnigheim))
	AUSSIEDLERHÖFE_6 = 'de:08118:6721:0:4'  # eRtg Hofen (Bönnigheim Hofen (Bönnigheim))
	AUSSIEDLERHÖFE_7 = 'de:08115:6889:0:3'  # Rtg Haslach Randenstraße (Herrenberg Haslach)
	AUSSIEDLERHÖFE_8 = 'de:08115:6889:0:4'  # Rtg Schulmeisterbuche (Herrenberg Haslach)
	AUSSIEDLERHÖFE_9 = 'de:08117:320:0:H1'  # Rtg Rathaus (Geislingen an der Steige Stötten)
	AUSSIEDLERHÖFE_10 = 'de:08117:320:0:R1'  # Rtg Geisl.(St.) (Geislingen an der Steige Stötten)
	AUSSIEDLERHÖFE_11 = 'de:08117:3306:0:H1'  # Rtg Adler (Heiningen Heiningen (GP))
	AUSSIEDLERHÖFE_12 = 'de:08117:3306:0:R1'  # Rtg Göppingen (Heiningen Heiningen (GP))
	WARTBERGSTRASSE = 'de:08118:3431'
	WARTBERGSTRASSE_1 = 'de:08118:3431:0:3'  # Rtg Im Bornrain (Möglingen Möglingen)
	WARTBERGSTRASSE_2 = 'de:08118:3431:0:4'  # Rtg Löscher (Möglingen Möglingen)
	MÖGLINGEN_WARTBERGSTR = 'de:08118:3431'
	MÖGLINGEN_WARTBERGSTR_1 = 'de:08118:3431:0:3'  # Rtg Im Bornrain (Möglingen Möglingen)
	MÖGLINGEN_WARTBERGSTR_2 = 'de:08118:3431:0:4'  # Rtg Löscher (Möglingen Möglingen)
	MÖGLINGEN_BAHNHOFSTR = 'de:08118:3432'
	MÖGLINGEN_BAHNHOFSTR_1 = 'de:08118:3432:1:3'  # eRtg Brühlstraße (Möglingen Möglingen)
	MÖGLINGEN_BAHNHOFSTR_2 = 'de:08118:3432:1:4'  # eRtg Im Bornrain (Möglingen Möglingen)
	MÖGLINGEN_DAIMLERSTR = 'de:08118:3433'
	MÖGLINGEN_DAIMLERSTR_1 = 'de:08118:3433:0:3'  # eRtg Löscher Ost (Möglingen Möglingen)
	MÖGLINGEN_DAIMLERSTR_2 = 'de:08118:3433:0:4'  # eRtg Lotter (Möglingen Möglingen)
	MÖGLINGEN_TENNISPLÄTZE = 'de:08118:3434:0:3'  # eRtg Pflugfelden/Ludwigsburg (Möglingen Möglingen)
	TENNISPLÄTZE = 'de:08118:3434:0:3'  # eRtg Pflugfelden/Ludwigsburg (Möglingen Möglingen)
	MÖGL__HIRSCHGARTENWEG = 'de:08118:3435:1:3'  # eRtg Bahnhofstraße (Möglingen Möglingen)
	HIRSCHGARTENWEG = 'de:08118:3435:1:3'  # eRtg Bahnhofstraße (Möglingen Möglingen)
	IM_BORNRAIN = 'de:08118:3436'
	IM_BORNRAIN_1 = 'de:08118:3436:0:3'  # eRtg Bahnhofstraße (Möglingen Möglingen)
	IM_BORNRAIN_2 = 'de:08118:3436:0:4'  # eRtg Wartbergstraße (Möglingen Möglingen)
	MÖGLINGEN_IM_BORNRAIN = 'de:08118:3436'
	MÖGLINGEN_IM_BORNRAIN_1 = 'de:08118:3436:0:3'  # eRtg Bahnhofstraße (Möglingen Möglingen)
	MÖGLINGEN_IM_BORNRAIN_2 = 'de:08118:3436:0:4'  # eRtg Wartbergstraße (Möglingen Möglingen)
	MÖGLINGEN_LÖSCHER_OST = 'de:08118:3437'
	MÖGLINGEN_LÖSCHER_OST_1 = 'de:08118:3437:0:3'  # 533/534/536 Rtg Wartbergstr/Löscher West (Möglingen Möglingen)
	MÖGLINGEN_LÖSCHER_OST_2 = 'de:08118:3437:0:4'  # 508 Rtg Löscher West (Möglingen Möglingen)
	MÖGLINGEN_LÖSCHER_OST_3 = 'de:08118:3437:0:5'  # 533/534/536 Rtg Daimlerstr/Sonnenbrunnen (Möglingen Möglingen)
	LÖSCHER_OST = 'de:08118:3437'
	LÖSCHER_OST_1 = 'de:08118:3437:0:3'  # 533/534/536 Rtg Wartbergstr/Löscher West (Möglingen Möglingen)
	LÖSCHER_OST_2 = 'de:08118:3437:0:4'  # 508 Rtg Löscher West (Möglingen Möglingen)
	LÖSCHER_OST_3 = 'de:08118:3437:0:5'  # 533/534/536 Rtg Daimlerstr/Sonnenbrunnen (Möglingen Möglingen)
	LÖSCHER_WEST = 'de:08118:3438:0:3'  # eRtg Alle Richtungen (Möglingen Möglingen)
	MÖGLINGEN_LÖSCHER_WEST = 'de:08118:3438:0:3'  # eRtg Alle Richtungen (Möglingen Möglingen)
	MÖGLINGEN_LUDWIGSBURGER_STR = 'de:08118:3439:0:3'  # eRtg Sonnenbrunnen (Möglingen Möglingen)
	MÖGLINGEN_MARKGRÖNINGER_STR = 'de:08118:3440'
	MÖGLINGEN_MARKGRÖNINGER_STR_1 = 'de:08118:3440:0:3'  # Rtg Markgröningen (Möglingen Möglingen)
	MÖGLINGEN_MARKGRÖNINGER_STR_2 = 'de:08118:3440:0:4'  # Rtg Möglingen (Möglingen Möglingen)
	MÖGL__HINDENBURGSTRASSE = 'de:08118:3442'
	MÖGL__HINDENBURGSTRASSE_1 = 'de:08118:3442:1:1'  # Pos 1 (Möglingen Möglingen)
	MÖGL__HINDENBURGSTRASSE_2 = 'de:08118:3442:1:2'  # Pos 2 (Möglingen Möglingen)
	SCHWIEBERDINGER_STRASSE_1 = 'de:08118:3443:0:3'  # eRtg Ludwigsburger Straße/Münchingen (Möglingen Möglingen)
	SCHWIEBERDINGER_STRASSE_2 = 'de:08118:3443:0:4'  # eRtg Hindenburgstraße (Möglingen Möglingen)
	SCHWIEBERDINGER_STRASSE_3 = 'de:08118:3579:0:1'  # Rtg Schwieberding / Schwieberdinger Str. (Hemmingen Hemmingen)
	SCHWIEBERDINGER_STRASSE_4 = 'de:08118:3579:0:2'  # Rtg Hemmingen / Schwieberdinger Str. (Hemmingen Hemmingen)
	SCHWIEBERDINGER_STRASSE_5 = 'de:08118:3579:0:3'  # 501 Schlosshaldenstr. (Hemmingen Hemmingen)
	MÖGLINGEN_SCHWIEBERDINGER_STR = 'de:08118:3443'
	MÖGLINGEN_SCHWIEBERDINGER_STR_1 = 'de:08118:3443:0:3'  # eRtg Ludwigsburger Straße/Münchingen (Möglingen Möglingen)
	MÖGLINGEN_SCHWIEBERDINGER_STR_2 = 'de:08118:3443:0:4'  # eRtg Hindenburgstraße (Möglingen Möglingen)
	SONNENBRUNNEN = 'de:08118:3444'
	SONNENBRUNNEN_1 = 'de:08118:3444:0:3'  # eRtg Hindenburgstraße (Möglingen Möglingen)
	SONNENBRUNNEN_2 = 'de:08118:3444:0:4'  # eRtg Daimlerstraße (Möglingen Möglingen)
	MÖGL__SONNENBRUNNEN = 'de:08118:3444'
	MÖGL__SONNENBRUNNEN_1 = 'de:08118:3444:0:3'  # eRtg Hindenburgstraße (Möglingen Möglingen)
	MÖGL__SONNENBRUNNEN_2 = 'de:08118:3444:0:4'  # eRtg Daimlerstraße (Möglingen Möglingen)
	TAMM_HOHENST__TÜBINGER_STRASSE = 'de:08118:3446'
	TAMM_HOHENST__TÜBINGER_STRASSE_1 = 'de:08118:3446:0:3'  # Rtg Frankfurter Str. (Tamm Tamm)
	TAMM_HOHENST__TÜBINGER_STRASSE_2 = 'de:08118:3446:0:4'  # Rtg Ulmer Str. (Tamm Tamm)
	HOHENSTANGE_TÜBINGER_STRASSE = 'de:08118:3446'
	HOHENSTANGE_TÜBINGER_STRASSE_1 = 'de:08118:3446:0:3'  # Rtg Frankfurter Str. (Tamm Tamm)
	HOHENSTANGE_TÜBINGER_STRASSE_2 = 'de:08118:3446:0:4'  # Rtg Ulmer Str. (Tamm Tamm)
	LUDW__BREUNINGERLAND = 'de:08118:3448'
	LUDW__BREUNINGERLAND_1 = 'de:08118:3448:0:3'  # Richtung IKEA (Ludwigsburg Ludwigsburg)
	LUDW__BREUNINGERLAND_2 = 'de:08118:3448:0:4'  # Richtung raus (Ludwigsburg Ludwigsburg)
	BREUNINGERLAND_1 = 'de:08118:3448:0:3'  # Richtung IKEA (Ludwigsburg Ludwigsburg)
	BREUNINGERLAND_2 = 'de:08118:3448:0:4'  # Richtung raus (Ludwigsburg Ludwigsburg)
	BREUNINGERLAND_3 = 'de:08115:6043:1:3'  # Rtg Hofmeister/Schauwerk (Sindelfingen Sindelfingen)
	BREUNINGERLAND_4 = 'de:08115:6043:1:4'  # Rtg Textilzentrum (Sindelfingen Sindelfingen)
	REALSCHULE_1 = 'de:08118:3449:0:3'  # eRtg Ankunft/Ludwigsburger Straße (Tamm Tamm)
	REALSCHULE_2 = 'de:08116:4179:0:4'  # Rtg Esslingen/Zollernplatz (Esslingen am Neckar Zollberg)
	REALSCHULE_3 = 'de:08116:4251:0:3'  # eRtg Königsberger Straße (Wernau (Neckar) Wernau (N))
	REALSCHULE_4 = 'de:08116:4302:0:3'  # eRtg Nürtingen (Nürtingen Nürtingen)
	REALSCHULE_5 = 'de:08116:4302:0:4'  # Rtg Bauhof (Nürtingen Nürtingen)
	REALSCHULE_6 = 'de:08118:5441:0:3'  # eRtg Stauffenbergstraße (Kornwestheim Kornwestheim)
	REALSCHULE_7 = 'de:08118:5441:0:4'  # eRtg Hallenbad (Kornwestheim Kornwestheim)
	REALSCHULE_8 = 'de:08118:5742:0:3'  # Rtg Burgfeldschule (Sachsenheim Großsachsenheim)
	REALSCHULE_9 = 'de:08119:5802:0:3'  # Rtg Geradstetten Rathaus & Mitte (Remshalden Geradstetten)
	TAMM_REALSCHULE = 'de:08118:3449:0:3'  # eRtg Ankunft/Ludwigsburger Straße (Tamm Tamm)
	TAMM_LUDWIGSBURGER_STR = 'de:08118:3451'
	TAMM_LUDWIGSBURGER_STR_1 = 'de:08118:3451:0:3'  # eRtg Tamm (Tamm Tamm)
	TAMM_LUDWIGSBURGER_STR_2 = 'de:08118:3451:0:4'  # eRtg Alter Weg (Tamm Tamm)
	TAMM_HOHENST__STUTTGARTER_STR = 'de:08118:3452'
	TAMM_HOHENST__STUTTGARTER_STR_1 = 'de:08118:3452:0:3'  # Rtg Brächterstr. (Tamm Tamm)
	TAMM_HOHENST__STUTTGARTER_STR_2 = 'de:08118:3452:0:4'  # Rtg Ulmer Str. (Tamm Tamm)
	HOHENSTANGE_STUTTGARTER_STRASSE = 'de:08118:3452'
	HOHENSTANGE_STUTTGARTER_STRASSE_1 = 'de:08118:3452:0:3'  # Rtg Brächterstr. (Tamm Tamm)
	HOHENSTANGE_STUTTGARTER_STRASSE_2 = 'de:08118:3452:0:4'  # Rtg Ulmer Str. (Tamm Tamm)
	TAMM_HOHENSTANGE_ULMER_STRASSE = 'de:08118:3453'
	TAMM_HOHENSTANGE_ULMER_STRASSE_1 = 'de:08118:3453:0:3'  # eRtg Hohenstange Stuttgarter Straße (Tamm Tamm)
	TAMM_HOHENSTANGE_ULMER_STRASSE_2 = 'de:08118:3453:0:4'  # eRtg Hohenstange Tübinger Straße (Tamm Tamm)
	HOHENSTANGE_ULMER_STRASSE = 'de:08118:3453'
	HOHENSTANGE_ULMER_STRASSE_1 = 'de:08118:3453:0:3'  # eRtg Hohenstange Stuttgarter Straße (Tamm Tamm)
	HOHENSTANGE_ULMER_STRASSE_2 = 'de:08118:3453:0:4'  # eRtg Hohenstange Tübinger Straße (Tamm Tamm)
	KREISBERUFSSCHULE_1 = 'de:08118:3454:0:3'  # eRtg Köhlstraße (Ludwigsburg Ludwigsburg)
	KREISBERUFSSCHULE_2 = 'de:08118:5616:0:3'  # eRtg Auwiesenbrücke (Bietigheim-Bissingen Bietigheim)
	LUDW__KREISBERUFSSCHULE = 'de:08118:3454:0:3'  # eRtg Köhlstraße (Ludwigsburg Ludwigsburg)
	EGLOSHEIM_HIRSCHBERGSTR = 'de:08118:3456'
	EGLOSHEIM_HIRSCHBERGSTR_1 = 'de:08118:3456:0:3'  # eRtg Tammer Straße (Ludwigsburg Eglosheim)
	EGLOSHEIM_HIRSCHBERGSTR_2 = 'de:08118:3456:0:4'  # Rtg Asperg Hohenstaufenstr. (Ludwigsburg Eglosheim)
	OBERESSL__LERCHENÄCKER = 'de:08116:3459'
	OBERESSL__LERCHENÄCKER_1 = 'de:08116:3459:0:3'  # Süd (101) (Esslingen am Neckar Oberesslingen)
	OBERESSL__LERCHENÄCKER_2 = 'de:08116:3459:0:4'  # Nord (138) (Esslingen am Neckar Oberesslingen)
	LERCHENÄCKER_1 = 'de:08116:3459:0:3'  # Süd (101) (Esslingen am Neckar Oberesslingen)
	LERCHENÄCKER_2 = 'de:08116:3459:0:4'  # Nord (138) (Esslingen am Neckar Oberesslingen)
	LERCHENÄCKER_3 = 'de:08119:6933:0:3'  # Rtg Ferd.-Braun-Str. (Backnang Backnang)
	HOHENSTANGE_FRANKFURTER_STRASSE = 'de:08118:3460'
	HOHENSTANGE_FRANKFURTER_STRASSE_1 = 'de:08118:3460:0:3'  # eRtg Hohenstange Tübinger Straße (Tamm Tamm)
	HOHENSTANGE_FRANKFURTER_STRASSE_2 = 'de:08118:3460:0:4'  # eRtg IKEA (Tamm Tamm)
	TAMM_HOHENST__FRANKFURTER_STR = 'de:08118:3460'
	TAMM_HOHENST__FRANKFURTER_STR_1 = 'de:08118:3460:0:3'  # eRtg Hohenstange Tübinger Straße (Tamm Tamm)
	TAMM_HOHENST__FRANKFURTER_STR_2 = 'de:08118:3460:0:4'  # eRtg IKEA (Tamm Tamm)
	KARLSPLATZ = 'de:08118:3462'
	KARLSPLATZ_1 = 'de:08118:3462:0:3'  # eRtg Agentur für Arbeit (Ludwigsburg Ludwigsburg)
	KARLSPLATZ_2 = 'de:08118:3462:0:4'  # eRtg Ludwigsburg (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_KARLSPLATZ = 'de:08118:3462'
	LUDWIGSBURG_KARLSPLATZ_1 = 'de:08118:3462:0:3'  # eRtg Agentur für Arbeit (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_KARLSPLATZ_2 = 'de:08118:3462:0:4'  # eRtg Ludwigsburg (Ludwigsburg Ludwigsburg)
	AM_SONNENBERG = 'de:08118:3464'
	AM_SONNENBERG_1 = 'de:08118:3464:0:3'  # eRtg Aldinger Straße (Ludwigsburg Ludwigsburg)
	AM_SONNENBERG_2 = 'de:08118:3464:0:4'  # eRtg Netzestraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_AM_SONNENBERG = 'de:08118:3464'
	LUDWIGSBURG_AM_SONNENBERG_1 = 'de:08118:3464:0:3'  # eRtg Aldinger Straße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_AM_SONNENBERG_2 = 'de:08118:3464:0:4'  # eRtg Netzestraße (Ludwigsburg Ludwigsburg)
	BRÜHLSTRASSE_1 = 'de:08118:3465:0:3'  # eRtg Hindenburgstraße (Möglingen Möglingen)
	BRÜHLSTRASSE_2 = 'de:08118:5567:0:3'  # eRtg Tiefengasse (Ingersheim Großingersheim)
	BRÜHLSTRASSE_3 = 'de:08118:5567:0:4'  # eRtg Schlossstraße (Ingersheim Großingersheim)
	BRÜHLSTRASSE_4 = 'de:08115:7011:0:3'  # Rtg Gärtringen (Deckenpfronn Deckenpfronn)
	BRÜHLSTRASSE_5 = 'de:08115:7011:0:4'  # Rtg Rathaus (Deckenpfronn Deckenpfronn)
	BRÜHLSTRASSE_6 = 'de:08115:7020:0:3'  # eRtg Rathaus (Mötzingen Mötzingen)
	BRÜHLSTRASSE_7 = 'de:08115:7020:0:4'  # eRtg (Mötzingen Mötzingen)
	MÖGLINGEN_BRÜHLSTRASSE = 'de:08118:3465:0:3'  # eRtg Hindenburgstraße (Möglingen Möglingen)
	ROTBÄUMLESFELD = 'de:08118:3466'
	ROTBÄUMLESFELD_1 = 'de:08118:3466:0:3'  # eRtg Oststraße (Ludwigsburg Ludwigsburg)
	ROTBÄUMLESFELD_2 = 'de:08118:3466:0:4'  # eRtg Aldinger Straße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_ROTBÄUMLESFELD = 'de:08118:3466'
	LUDWIGSBURG_ROTBÄUMLESFELD_1 = 'de:08118:3466:0:3'  # eRtg Oststraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_ROTBÄUMLESFELD_2 = 'de:08118:3466:0:4'  # eRtg Aldinger Straße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_IKEA = 'de:08118:3467'
	LUDWIGSBURG_IKEA_1 = 'de:08118:3467:0:3'  # Ankunft (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_IKEA_2 = 'de:08118:3467:0:4'  # eRtg Breuningerland (Ludwigsburg Ludwigsburg)
	IKEA_1 = 'de:08118:3467:0:3'  # Ankunft (Ludwigsburg Ludwigsburg)
	IKEA_2 = 'de:08118:3467:0:4'  # eRtg Breuningerland (Ludwigsburg Ludwigsburg)
	IKEA_3 = 'de:08115:4803:0:3'  # Rtg Sindelfingen ZOB (Sindelfingen Sindelfingen)
	IKEA_4 = 'de:08115:4803:0:4'  # Rtg R.-Diesel-Str. (Sindelfingen Sindelfingen)
	ALBERT_SCHWEITZER_STRASSE_1 = 'de:08118:3468:0:3'  # eRtg Freibad/FitKom (Besigheim Besigheim)
	ALBERT_SCHWEITZER_STRASSE_2 = 'de:08115:6816:0:3'  # Rtg Brandströmstr. (Leonberg Höfingen)
	ALBERT_SCHWEITZER_STRASSE_3 = 'de:08115:6816:0:4'  # Rtg Lachentorstr. (Leonberg Höfingen)
	BESIGHEIM_A__SCHWEITZER_STR = 'de:08118:3468:0:3'  # eRtg Freibad/FitKom (Besigheim Besigheim)
	BESIGHEIM_ELSERRING = 'de:08118:3469:0:3'  # eRtg Altenzentrum (Besigheim Besigheim)
	ELSERRING = 'de:08118:3469:0:3'  # eRtg Altenzentrum (Besigheim Besigheim)
	PLEIDELSH__WIEGEH__RIEDB_STR = 'de:08118:3470:1:3'  # eRtg Haupstr./Murr (Pleidelsheim Pleidelsheim)
	WIEGEHALLE_RIEDBACHSTRASSE = 'de:08118:3470:1:3'  # eRtg Haupstr./Murr (Pleidelsheim Pleidelsheim)
	BURGSTALLER_STRASSE_1 = 'de:08119:3471:0:3'  # eRtg Rötesiedlung (Burgstetten Erbstetten)
	BURGSTALLER_STRASSE_2 = 'de:08119:3471:0:4'  # eRtg Ortsmitte (Burgstetten Erbstetten)
	BURGSTALLER_STRASSE_3 = 'de:08119:3528:0:4'  # eRtg Auf den Rüdern/Feuerwehr (Kirchberg an der Murr Kirchberg (M))
	ERBSTETTEN_BURGSTALLER_STR = 'de:08119:3471'
	ERBSTETTEN_BURGSTALLER_STR_1 = 'de:08119:3471:0:3'  # eRtg Rötesiedlung (Burgstetten Erbstetten)
	ERBSTETTEN_BURGSTALLER_STR_2 = 'de:08119:3471:0:4'  # eRtg Ortsmitte (Burgstetten Erbstetten)
	BEIHINGER_STRASSE_1 = 'de:08118:3472:0:3'  # Rtg Harteneckstr. (Ludwigsburg Hoheneck)
	BEIHINGER_STRASSE_2 = 'de:08118:3472:0:4'  # Rtg Bottwartalstr. (Ludwigsburg Hoheneck)
	BEIHINGER_STRASSE_3 = 'de:08118:5569:0:3'  # eRtg Wiegehalle (Pleidelsheim Pleidelsheim)
	BEIHINGER_STRASSE_4 = 'de:08118:5569:0:4'  # eRtg Hohenzollernstraße (Pleidelsheim Pleidelsheim)
	HOHENECK_BEIHINGER_STR = 'de:08118:3472'
	HOHENECK_BEIHINGER_STR_1 = 'de:08118:3472:0:3'  # Rtg Harteneckstr. (Ludwigsburg Hoheneck)
	HOHENECK_BEIHINGER_STR_2 = 'de:08118:3472:0:4'  # Rtg Bottwartalstr. (Ludwigsburg Hoheneck)
	HECKENWEG_1 = 'de:08118:3473:0:4'  # Rtg Friedhof (Besigheim Besigheim)
	HECKENWEG_2 = 'de:08119:7456:0:3'  # Rtg K.-G.-Pfleiderer-Str. (Winnenden Hertmannsweiler)
	BESIGHEIM_HECKENWEG = 'de:08118:3473:0:4'  # Rtg Friedhof (Besigheim Besigheim)
	AUF_DEM_WASEN = 'de:08118:3475:0:3'  # Rtg E.-Kranz-Str. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_AUF_DEM_WASEN = 'de:08118:3475:0:3'  # Rtg E.-Kranz-Str. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_E__KRANZ_STR = 'de:08118:3476:0:3'  # Rtg Comburgstr. (Ludwigsburg Ludwigsburg)
	ELISABETH_KRANZ_STRASSE = 'de:08118:3476:0:3'  # Rtg Comburgstr. (Ludwigsburg Ludwigsburg)
	FISCHERWÖRTH = 'de:08118:3485'
	FISCHERWÖRTH_1 = 'de:08118:3485:0:1'  # Rtg Ingersheim (Ingersheim Großingersheim)
	FISCHERWÖRTH_2 = 'de:08118:3485:0:2'  # Rtg Pleidelsheim (Ingersheim Großingersheim)
	GROSSINGERSHEIM_FISCHERWÖRTH = 'de:08118:3485'
	GROSSINGERSHEIM_FISCHERWÖRTH_1 = 'de:08118:3485:0:1'  # Rtg Ingersheim (Ingersheim Großingersheim)
	GROSSINGERSHEIM_FISCHERWÖRTH_2 = 'de:08118:3485:0:2'  # Rtg Pleidelsheim (Ingersheim Großingersheim)
	BRÜCKENSTRASSE_1 = 'de:08118:3499:0:1'  # Rtg Kleiningersheim (Hessigheim Hessigheim)
	BRÜCKENSTRASSE_2 = 'de:08118:3572:0:1'  # Rtg Mühle (Remseck am Neckar Aldingen)
	BRÜCKENSTRASSE_3 = 'de:08118:3572:0:2'  # Rtg Neckargröningen (Remseck am Neckar Aldingen)
	BRÜCKENSTRASSE_4 = 'de:08119:3696:0:3'  # eRtg Lange Weiden (Winnenden Winnenden)
	BRÜCKENSTRASSE_5 = 'de:08119:3696:0:4'  # eRtg Kronenplatz (Winnenden Winnenden)
	BRÜCKENSTRASSE_6 = 'de:08117:82:0:H1'  # Rtg Schurwaldweg (Birenbach Birenbach)
	BRÜCKENSTRASSE_7 = 'de:08117:86:0:H1'  # Rtg Süßen (Gingen an der Fils Gingen (F))
	BRÜCKENSTRASSE_8 = 'de:08117:86:0:R1'  # Rtg Brückenstr. (Gingen an der Fils Gingen (F))
	BRÜCKENSTRASSE_9 = 'de:08117:3003:0:R1'  # Rtg Kaiserbau (Ag. für Arb.) (Göppingen Göppingen)
	HESSIGHEIM_BRÜCKENSTRASSE = 'de:08118:3499:0:1'  # Rtg Kleiningersheim (Hessigheim Hessigheim)
	KIRSCHHALDENWEG = 'de:08118:3501'
	KIRSCHHALDENWEG_1 = 'de:08118:3501:0:3'  # eRtg Sachsenheimer Weg (Besigheim Besigheim)
	KIRSCHHALDENWEG_2 = 'de:08118:3501:0:4'  # eRtg Martinshaus/Besigheim/Enzplatz (Besigheim Besigheim)
	BESIGHEIM_KIRSCHHALDENWEG = 'de:08118:3501'
	BESIGHEIM_KIRSCHHALDENWEG_1 = 'de:08118:3501:0:3'  # eRtg Sachsenheimer Weg (Besigheim Besigheim)
	BESIGHEIM_KIRSCHHALDENWEG_2 = 'de:08118:3501:0:4'  # eRtg Martinshaus/Besigheim/Enzplatz (Besigheim Besigheim)
	OTTMARSHEIM_ILSFELDER_STR = 'de:08118:3502:0:3'  # eRtg Rathaus (Besigheim Ottmarsheim)
	ILSFELDER_STRASSE = 'de:08118:3502:0:3'  # eRtg Rathaus (Besigheim Ottmarsheim)
	HOHENHASL__BROMBERGERHÖFE = 'de:08118:3503'
	HOHENHASL__BROMBERGERHÖFE_1 = 'de:08118:3503:0:3'  # eRtg Brunnenwiesen (Sachsenheim Hohenhaslach)
	HOHENHASL__BROMBERGERHÖFE_2 = 'de:08118:3503:0:4'  # eRtg Allmandklinge (Sachsenheim Hohenhaslach)
	BROMBERGERHÖFE = 'de:08118:3503'
	BROMBERGERHÖFE_1 = 'de:08118:3503:0:3'  # eRtg Brunnenwiesen (Sachsenheim Hohenhaslach)
	BROMBERGERHÖFE_2 = 'de:08118:3503:0:4'  # eRtg Allmandklinge (Sachsenheim Hohenhaslach)
	WELLARIUM = 'de:08118:3504'
	WELLARIUM_1 = 'de:08118:3504:0:3'  # Rtg Höpfigheim (Steinheim an der Murr Steinheim (M))
	WELLARIUM_2 = 'de:08118:3504:0:4'  # Rtg Steinheim (Steinheim an der Murr Steinheim (M))
	STEINHEIM__M__WELLARIUM = 'de:08118:3504'
	STEINHEIM__M__WELLARIUM_1 = 'de:08118:3504:0:3'  # Rtg Höpfigheim (Steinheim an der Murr Steinheim (M))
	STEINHEIM__M__WELLARIUM_2 = 'de:08118:3504:0:4'  # Rtg Steinheim (Steinheim an der Murr Steinheim (M))
	BESIGHEIM_SCHÄUBER = 'de:08118:3505'
	BESIGHEIM_SCHÄUBER_1 = 'de:08118:3505:0:3'  # Rtg Sachsenheimer Weg (Besigheim Besigheim)
	BESIGHEIM_SCHÄUBER_2 = 'de:08118:3505:0:4'  # Ankunft (Besigheim Besigheim)
	SCHÄUBER = 'de:08118:3505'
	SCHÄUBER_1 = 'de:08118:3505:0:3'  # Rtg Sachsenheimer Weg (Besigheim Besigheim)
	SCHÄUBER_2 = 'de:08118:3505:0:4'  # Ankunft (Besigheim Besigheim)
	SACHSENHEIMER_WEG = 'de:08118:3506'
	SACHSENHEIMER_WEG_1 = 'de:08118:3506:0:3'  # Rtg Schäuber (Besigheim Besigheim)
	SACHSENHEIMER_WEG_2 = 'de:08118:3506:0:4'  # Rtg Kirschhaldenweg (Besigheim Besigheim)
	BESIGHEIM_SACHSENHEIMER_WEG = 'de:08118:3506'
	BESIGHEIM_SACHSENHEIMER_WEG_1 = 'de:08118:3506:0:3'  # Rtg Schäuber (Besigheim Besigheim)
	BESIGHEIM_SACHSENHEIMER_WEG_2 = 'de:08118:3506:0:4'  # Rtg Kirschhaldenweg (Besigheim Besigheim)
	BURGST__M__RÖTESIEDLUNG = 'de:08119:3507'
	BURGST__M__RÖTESIEDLUNG_1 = 'de:08119:3507:0:3'  # Rtg Erbstetten/Kirsch (Burgstetten Burgstall (Burgstetten))
	BURGST__M__RÖTESIEDLUNG_2 = 'de:08119:3507:0:4'  # Rtg Burgstall (Burgstetten Burgstall (Burgstetten))
	RÖTESIEDLUNG = 'de:08119:3507'
	RÖTESIEDLUNG_1 = 'de:08119:3507:0:3'  # Rtg Erbstetten/Kirsch (Burgstetten Burgstall (Burgstetten))
	RÖTESIEDLUNG_2 = 'de:08119:3507:0:4'  # Rtg Burgstall (Burgstetten Burgstall (Burgstetten))
	MURRBRÜCKE = 'de:08119:3508'
	MURRBRÜCKE_1 = 'de:08119:3508:0:3'  # Rtg Burgstall (Burgstetten Burgstall (Burgstetten))
	MURRBRÜCKE_2 = 'de:08119:3508:0:4'  # Rtg Kirchberg (Burgstetten Burgstall (Burgstetten))
	BURGSTALL__M__MURRBRÜCKE = 'de:08119:3508'
	BURGSTALL__M__MURRBRÜCKE_1 = 'de:08119:3508:0:3'  # Rtg Burgstall (Burgstetten Burgstall (Burgstetten))
	BURGSTALL__M__MURRBRÜCKE_2 = 'de:08119:3508:0:4'  # Rtg Kirchberg (Burgstetten Burgstall (Burgstetten))
	BURGSTALL__M__KELTERWEG = 'de:08119:3509'
	BURGSTALL__M__KELTERWEG_1 = 'de:08119:3509:0:3'  # eRtg Burgstall (M) (Burgstetten Burgstall (Burgstetten))
	BURGSTALL__M__KELTERWEG_2 = 'de:08119:3509:0:4'  # eRtg Rötesiedlung (Burgstetten Burgstall (Burgstetten))
	KELTERWEG_1 = 'de:08119:3509:0:3'  # eRtg Burgstall (M) (Burgstetten Burgstall (Burgstetten))
	KELTERWEG_2 = 'de:08119:3509:0:4'  # eRtg Rötesiedlung (Burgstetten Burgstall (Burgstetten))
	KELTERWEG_3 = 'de:08115:4804:0:3'  # eRtg Elsenhalde (Schönaich Schönaich)
	KELTERWEG_4 = 'de:08115:4804:0:4'  # eRtg Rauer Kapf (Schönaich Schönaich)
	BESIGHEIM_ULRICHSTRASSE = 'de:08118:3510'
	BESIGHEIM_ULRICHSTRASSE_1 = 'de:08118:3510:0:3'  # eRtg Friedrich-Breining-Straße (Besigheim Besigheim)
	BESIGHEIM_ULRICHSTRASSE_2 = 'de:08118:3510:0:4'  # eRtg Friedhof (Besigheim Besigheim)
	ULRICHSTRASSE_1 = 'de:08118:3510:0:3'  # eRtg Friedrich-Breining-Straße (Besigheim Besigheim)
	ULRICHSTRASSE_2 = 'de:08118:3510:0:4'  # eRtg Friedhof (Besigheim Besigheim)
	ULRICHSTRASSE_3 = 'de:08119:5398:0:3'  # Rtg Rathaus (Weinstadt Beutelsbach)
	ULRICHSTRASSE_4 = 'de:08119:5398:0:4'  # Rtg Schurwaldstr. (Weinstadt Beutelsbach)
	NEUSCHÖNTAL = 'de:08119:3511'
	NEUSCHÖNTAL_1 = 'de:08119:3511:0:3'  # eRtg Ortsmitte (Backnang Backnang)
	NEUSCHÖNTAL_2 = 'de:08119:3511:0:4'  # eRtg Backnang (Backnang Backnang)
	BACKNANG_NEUSCHÖNTAL = 'de:08119:3511'
	BACKNANG_NEUSCHÖNTAL_1 = 'de:08119:3511:0:3'  # eRtg Ortsmitte (Backnang Backnang)
	BACKNANG_NEUSCHÖNTAL_2 = 'de:08119:3511:0:4'  # eRtg Backnang (Backnang Backnang)
	ORTSMITTE_1 = 'de:08119:3513:0:3'  # eRtg Burgstaller Straße (Burgstetten Erbstetten)
	ORTSMITTE_2 = 'de:08119:3513:0:4'  # eRtg Backnang (Burgstetten Erbstetten)
	ORTSMITTE_3 = 'de:08119:3672:1:3'  # Rtg Sonnenhof (Aspach Kleinaspach)
	ORTSMITTE_4 = 'de:08119:3672:1:4'  # Rtg Schafhaus/Karlshof/Hardtwaldhalle (Aspach Kleinaspach)
	ORTSMITTE_5 = 'de:08116:4234:1:3'  # Rtg Plochingen/Schulzentrum (Reichenbach an der Fils Reichenbach (F))
	ORTSMITTE_6 = 'de:08116:4234:1:4'  # Rtg Bahnhof/Ulmer Straße (Reichenbach an der Fils Reichenbach (F))
	ORTSMITTE_7 = 'de:08116:4341:0:3'  # eRtg Ötlingen (Kirchheim unter Teck Ötlingen)
	ORTSMITTE_8 = 'de:08116:4349:1:3'  # eRtg Zähringer Straße (Kirchheim unter Teck Lindorf)
	ORTSMITTE_9 = 'de:08116:4349:1:4'  # eRtg Brühl (Kirchheim unter Teck Lindorf)
	ORTSMITTE_10 = 'de:08115:4537:0:4'  # Rtg Hauptstr. 77 (Leonberg Warmbronn)
	ORTSMITTE_11 = 'de:08115:4566:0:3'  # eRtg Sportplatz (Weissach Flacht)
	ORTSMITTE_12 = 'de:08115:4566:0:4'  # eRtg Hohweg (Weissach Flacht)
	ORTSMITTE_13 = 'de:08115:4588:1:3'  # eRtg Industriestraße (Weil der Stadt Merklingen)
	ORTSMITTE_14 = 'de:08115:4588:1:4'  # eRtg Hauptstraße (Weil der Stadt Merklingen)
	ORTSMITTE_15 = 'de:08119:4901:1:3'  # eRtg Murrtalschule (Oppenweiler Oppenweiler)
	ORTSMITTE_16 = 'de:08119:4901:1:4'  # eRtg Bühlfeld (Oppenweiler Oppenweiler)
	ORTSMITTE_17 = 'de:08119:4996:0:3'  # Rtg Rathaus (Althütte Althütte)
	ORTSMITTE_18 = 'de:08119:4996:0:4'  # Rtg Strohhof (Althütte Althütte)
	ORTSMITTE_19 = 'de:08116:5001:1:3'  # Rtg Raiffeisenstr. (Aichtal Grötzingen)
	ORTSMITTE_20 = 'de:08116:5001:1:4'  # Rtg Schönblick (Aichtal Grötzingen)
	ORTSMITTE_21 = 'de:08119:5032:0:3'  # eRtg Schule (Allmersbach im Tal Allmersbach im Tal)
	ORTSMITTE_22 = 'de:08119:5032:0:4'  # eRtg Wasenfeld (Allmersbach im Tal Allmersbach im Tal)
	ORTSMITTE_23 = 'de:08119:5118:1:3'  # Rtg Rudersberg (Rudersberg Schlechtbach)
	ORTSMITTE_24 = 'de:08119:5118:1:4'  # Rtg Lindentaler Str./Michelau (Rudersberg Schlechtbach)
	ORTSMITTE_25 = 'de:08119:5151:0:3'  # Rtg Hegnacher Str. (Waiblingen Hohenacker)
	ORTSMITTE_26 = 'de:08119:5151:0:4'  # Rtg Bildäckerstr. (Waiblingen Hohenacker)
	ORTSMITTE_27 = 'de:08119:5204:0:3'  # eRtg Industriegebiet (Winnenden Birkmannsweiler)
	ORTSMITTE_28 = 'de:08119:5204:0:4'  # eRtg Talaue (Winnenden Birkmannsweiler)
	ORTSMITTE_29 = 'de:08119:5268:1:3'  # eRtg Alle Richtungen (Berglen Oppelsbohm)
	ORTSMITTE_30 = 'de:08119:5268:1:4'  # Rtg. Kaisersbach (Berglen Oppelsbohm)
	ORTSMITTE_31 = 'de:08119:5274:0:3'  # eRtg Schulerhof (Winnenden Baach (Winnenden))
	ORTSMITTE_32 = 'de:08119:5274:0:4'  # eRtg Höfen (Winnenden Baach (Winnenden))
	ORTSMITTE_33 = 'de:08119:5275:0:3'  # eRtg Baach/Hertmannsweiler (Winnenden Höfen)
	ORTSMITTE_34 = 'de:08119:5275:0:4'  # eRtg Seehaldenweg (Winnenden Höfen)
	ORTSMITTE_35 = 'de:08119:5393:1:3'  # Rtg. Kalkofen (Weinstadt Endersbach)
	ORTSMITTE_36 = 'de:08119:5393:1:4'  # Rtg Endersbach Bf (gibt es nur Rück) (Weinstadt Endersbach)
	ORTSMITTE_37 = 'de:08118:5598:0:3'  # Rtg Hartlinde (Murr Murr)
	ORTSMITTE_38 = 'de:08118:5598:0:4'  # Rtg Blattert (Murr Murr)
	ORTSMITTE_39 = 'de:08118:5753:0:3'  # eRtg Nußdorfer Straße (Vaihingen an der Enz Aurich)
	ORTSMITTE_40 = 'de:08118:5753:0:4'  # eRtg Reitsteige (Vaihingen an der Enz Aurich)
	ORTSMITTE_41 = 'de:08118:5934:0:3'  # Rtg Brühlstr. (Ingersheim Großingersheim)
	ORTSMITTE_42 = 'de:08118:5934:0:4'  # Rtg Bietigheimer Str. (Ingersheim Großingersheim)
	ORTSMITTE_43 = 'de:08116:7935:0:3'  # eRtg Bolzplatz (Baltmannsweiler Hohengehren)
	ORTSMITTE_44 = 'de:08116:7935:0:4'  # eRtg Waldhorn (Baltmannsweiler Hohengehren)
	ORTSMITTE_45 = 'de:08117:208:0:H1'  # Roggenmühle (Geislingen an der Steige Eybach)
	ORTSMITTE_46 = 'de:08117:208:0:R1'  # Rtg Eybtalhalle (Geislingen an der Steige Eybach)
	ORTSMITTE_47 = 'de:08117:213:0:H1'  # Ankunft (Geislingen an der Steige Waldhausen (Geislingen))
	ORTSMITTE_48 = 'de:08117:213:0:R1'  # Rtg Eybach (Geislingen an der Steige Waldhausen (Geislingen))
	ORTSMITTE_49 = 'de:08117:1206:0:H1'  # Rtg Limesweg (Göppingen Maitis)
	ORTSMITTE_50 = 'de:08117:1208:0:H1'  # Rtg Reitprechts (Göppingen Lenglingen)
	ORTSMITTE_51 = 'de:08117:1208:0:R1'  # Rtg Maitis (Göppingen Lenglingen)
	ORTSMITTE_52 = 'de:08117:2044:0:H1'  # eRtg Industriegebiet (Albershausen Albershausen)
	ORTSMITTE_53 = 'de:08117:2044:0:R1'  # eRtg Uhinger-/Waldeckstraße (Albershausen Albershausen)
	ORTSMITTE_54 = 'de:08117:2107:0:H1'  # Rtg Gruibinger Straße (Bad Boll Bad Boll)
	ORTSMITTE_55 = 'de:08117:2107:0:R1'  # Rtg Bahnhofsallee (Bad Boll Bad Boll)
	ORTSMITTE_56 = 'de:08117:3339:0:H1'  # Rtg Boller Straße (Dürnau Dürnau)
	ORTSMITTE_57 = 'de:08117:3339:0:R1'  # Rtg Abzw. Kornberghalle (Dürnau Dürnau)
	ORTSMITTE_58 = 'de:08425:2632:0:2'  # einwärts (Merklingen Merklingen)
	ERBSTETTEN_ORTSMITTE = 'de:08119:3513'
	ERBSTETTEN_ORTSMITTE_1 = 'de:08119:3513:0:3'  # eRtg Burgstaller Straße (Burgstetten Erbstetten)
	ERBSTETTEN_ORTSMITTE_2 = 'de:08119:3513:0:4'  # eRtg Backnang (Burgstetten Erbstetten)
	ERDMANNHAUSEN = 'de:08118:3514:1:1'  # Gleis 1 (Erdmannhausen Erdmannhausen)
	ERDMANNHAUSEN_RATHAUS = 'de:08118:3515'
	ERDMANNHAUSEN_RATHAUS_1 = 'de:08118:3515:0:3'  # eRtg Turnhalle/Bahnhofstraße (Erdmannhausen Erdmannhausen)
	ERDMANNHAUSEN_RATHAUS_2 = 'de:08118:3515:0:4'  # eRtg Waage (Erdmannhausen Erdmannhausen)
	ERDMANNHAUSEN_TURNHALLE = 'de:08118:3516'
	ERDMANNHAUSEN_TURNHALLE_1 = 'de:08118:3516:0:3'  # Rtg Affalterbach (Erdmannhausen Erdmannhausen)
	ERDMANNHAUSEN_TURNHALLE_2 = 'de:08118:3516:0:4'  # Rtg Rathaus (Erdmannhausen Erdmannhausen)
	WAAGE_1 = 'de:08118:3517:1:3'  # eRtg Rathaus (Erdmannhausen Erdmannhausen)
	WAAGE_2 = 'de:08118:3517:1:4'  # eRtg Erdmannhäuser Straße (Erdmannhausen Erdmannhausen)
	WAAGE_3 = 'de:08118:3602:0:3'  # eRtg Marbach (N) (Affalterbach Affalterbach)
	WAAGE_4 = 'de:08118:3602:0:4'  # eRtg Klingenstraße (Affalterbach Affalterbach)
	ERDMANNHAUSEN_WAAGE = 'de:08118:3517'
	ERDMANNHAUSEN_WAAGE_1 = 'de:08118:3517:1:3'  # eRtg Rathaus (Erdmannhausen Erdmannhausen)
	ERDMANNHAUSEN_WAAGE_2 = 'de:08118:3517:1:4'  # eRtg Erdmannhäuser Straße (Erdmannhausen Erdmannhausen)
	ERBSTETTEN_LAUBERT = 'de:08119:3518'
	ERBSTETTEN_LAUBERT_1 = 'de:08119:3518:0:3'  # Rtg Erbstetten aussen (Burgstetten Erbstetten)
	ERBSTETTEN_LAUBERT_2 = 'de:08119:3518:0:4'  # Rtg Alle innen (Burgstetten Erbstetten)
	LAUBERT = 'de:08119:3518'
	LAUBERT_1 = 'de:08119:3518:0:3'  # Rtg Erbstetten aussen (Burgstetten Erbstetten)
	LAUBERT_2 = 'de:08119:3518:0:4'  # Rtg Alle innen (Burgstetten Erbstetten)
	BEIHINGER_PLATZ = 'de:08118:3519'
	BEIHINGER_PLATZ_1 = 'de:08118:3519:0:3'  # eRtg Feuerwehr (Freiberg am Neckar Freiberg (N))
	BEIHINGER_PLATZ_2 = 'de:08118:3519:0:4'  # eRtg Menzelstraße (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__BEIHINGER_PLATZ = 'de:08118:3519'
	FREIBERG__N__BEIHINGER_PLATZ_1 = 'de:08118:3519:0:3'  # eRtg Feuerwehr (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__BEIHINGER_PLATZ_2 = 'de:08118:3519:0:4'  # eRtg Menzelstraße (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__STADTZENTRUM = 'de:08118:3521'
	FREIBERG__N__STADTZENTRUM_1 = 'de:08118:3521:0:3'  # eRtg Freiberg (N) (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__STADTZENTRUM_2 = 'de:08118:3521:0:4'  # eRtg Bilfinger Straße (Freiberg am Neckar Freiberg (N))
	STADTZENTRUM = 'de:08118:3521'
	STADTZENTRUM_1 = 'de:08118:3521:0:3'  # eRtg Freiberg (N) (Freiberg am Neckar Freiberg (N))
	STADTZENTRUM_2 = 'de:08118:3521:0:4'  # eRtg Bilfinger Straße (Freiberg am Neckar Freiberg (N))
	FREIB__N__GEIS__DORFPLATZ = 'de:08118:3522'
	FREIB__N__GEIS__DORFPLATZ_1 = 'de:08118:3522:0:3'  # eRtg Bilfinger Straße (Freiberg am Neckar Freiberg (N))
	FREIB__N__GEIS__DORFPLATZ_2 = 'de:08118:3522:0:4'  # eRtg Mönchsbergstraße (Freiberg am Neckar Freiberg (N))
	GEISINGER_DORFPLATZ = 'de:08118:3522'
	GEISINGER_DORFPLATZ_1 = 'de:08118:3522:0:3'  # eRtg Bilfinger Straße (Freiberg am Neckar Freiberg (N))
	GEISINGER_DORFPLATZ_2 = 'de:08118:3522:0:4'  # eRtg Mönchsbergstraße (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__BILFINGER_STR = 'de:08118:3523'
	FREIBERG__N__BILFINGER_STR_1 = 'de:08118:3523:0:3'  # eRtg Stadtzentrum (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__BILFINGER_STR_2 = 'de:08118:3523:0:4'  # eRtg Geisinger Dorfplatz (Freiberg am Neckar Freiberg (N))
	BILFINGER_STRASSE = 'de:08118:3523'
	BILFINGER_STRASSE_1 = 'de:08118:3523:0:3'  # eRtg Stadtzentrum (Freiberg am Neckar Freiberg (N))
	BILFINGER_STRASSE_2 = 'de:08118:3523:0:4'  # eRtg Geisinger Dorfplatz (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__STUTTGARTER_STR = 'de:08118:3524'
	FREIBERG__N__STUTTGARTER_STR_1 = 'de:08118:3524:0:3'  # eRtg Freiberg (N) (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__STUTTGARTER_STR_2 = 'de:08118:3524:0:4'  # eRtg Harteneckstraße (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__HARTENECKSTR = 'de:08118:3525'
	FREIBERG__N__HARTENECKSTR_1 = 'de:08118:3525:0:3'  # eRtg Stuttgarter Straße (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__HARTENECKSTR_2 = 'de:08118:3525:0:4'  # eRtg Beihinger Straße (Freiberg am Neckar Freiberg (N))
	HARTENECKSTRASSE_1 = 'de:08118:3525:0:3'  # eRtg Stuttgarter Straße (Freiberg am Neckar Freiberg (N))
	HARTENECKSTRASSE_2 = 'de:08118:3525:0:4'  # eRtg Beihinger Straße (Freiberg am Neckar Freiberg (N))
	HARTENECKSTRASSE_3 = 'de:08118:5500:0:3'  # Rtg Kornbeckstraße (Ludwigsburg Ludwigsburg)
	HARTENECKSTRASSE_4 = 'de:08118:5500:0:4'  # Rtg Neuer Friedhof (Ludwigsburg Ludwigsburg)
	KIRCHBERG__M__MARBACHER_STR = 'de:08119:3526'
	KIRCHBERG__M__MARBACHER_STR_1 = 'de:08119:3526:0:3'  # Rtg Bahnhof /Marbach (Kirchberg an der Murr Kirchberg (M))
	KIRCHBERG__M__MARBACHER_STR_2 = 'de:08119:3526:0:4'  # Rtg Rathaus (Kirchberg an der Murr Kirchberg (M))
	MARBACHER_STRASSE_1 = 'de:08119:3526:0:3'  # Rtg Bahnhof /Marbach (Kirchberg an der Murr Kirchberg (M))
	MARBACHER_STRASSE_2 = 'de:08119:3526:0:4'  # Rtg Rathaus (Kirchberg an der Murr Kirchberg (M))
	MARBACHER_STRASSE_3 = 'de:08118:3599:0:3'  # eRtg Waage (Affalterbach Affalterbach)
	MARBACHER_STRASSE_4 = 'de:08119:3670:0:3'  # eRtg Rathaus (Backnang Heiningen (Backnang))
	MARBACHER_STRASSE_5 = 'de:08119:3670:0:4'  # eRtg Ankunft (Backnang Heiningen (Backnang))
	MARBACHER_STRASSE_6 = 'de:08119:6518:0:3'  # eRtg Fürstenhof (Aspach Großaspach)
	MARBACHER_STRASSE_7 = 'de:08119:6518:0:4'  # eRtg Klöpferbach (Aspach Großaspach)
	KIRCHBERG__M__RATHAUS = 'de:08119:3527'
	KIRCHBERG__M__RATHAUS_1 = 'de:08119:3527:0:3'  # Richtung Bahnhof (Kirchberg an der Murr Kirchberg (M))
	KIRCHBERG__M__RATHAUS_2 = 'de:08119:3527:0:4'  # Richtung Schule (Kirchberg an der Murr Kirchberg (M))
	KIRCHBERG__M__BURGSTALLER_STR = 'de:08119:3528:0:4'  # eRtg Auf den Rüdern/Feuerwehr (Kirchberg an der Murr Kirchberg (M))
	KIRCHBERG__M__NEUHOF = 'de:08119:3529:0:3'  # eRtg Ankunft/Abfahrt Bus (Kirchberg an der Murr Kirchberg (M))
	NEUHOF_1 = 'de:08119:3529:0:3'  # eRtg Ankunft/Abfahrt Bus (Kirchberg an der Murr Kirchberg (M))
	NEUHOF_2 = 'de:08119:5088:0:3'  # eRtg Breitenfürst Lorcher Straße (Welzheim Welzheim)
	NEUHOF_3 = 'de:08119:5088:0:4'  # eRtg Haghof (Welzheim Welzheim)
	KIRCHBERG__M__SCHULE = 'de:08119:3530:0:3'  # eRtg Ankunft/Abfahrt Bus (Kirchberg an der Murr Kirchberg (M))
	MARB__N__TECHNOLOGIEPARK = 'de:08118:3531'
	MARB__N__TECHNOLOGIEPARK_1 = 'de:08118:3531:0:3'  # eRtg Eichgraben (Marbach am Neckar Marbach (N))
	MARB__N__TECHNOLOGIEPARK_2 = 'de:08118:3531:0:4'  # eRtg Immanuel-Dornfeld.-Straße (Marbach am Neckar Marbach (N))
	MARBACH__N__ERDMANNH__STR = 'de:08118:3532'
	MARBACH__N__ERDMANNH__STR_1 = 'de:08118:3532:0:3'  # eRtg Waage (Marbach am Neckar Marbach (N))
	MARBACH__N__ERDMANNH__STR_2 = 'de:08118:3532:0:4'  # Rtg Bahnhof (Marbach am Neckar Marbach (N))
	ERDMANNHÄUSER_STRASSE = 'de:08118:3532'
	ERDMANNHÄUSER_STRASSE_1 = 'de:08118:3532:0:3'  # eRtg Waage (Marbach am Neckar Marbach (N))
	ERDMANNHÄUSER_STRASSE_2 = 'de:08118:3532:0:4'  # Rtg Bahnhof (Marbach am Neckar Marbach (N))
	MARBACH__N__KRANKENHAUS = 'de:08118:3533'
	MARBACH__N__KRANKENHAUS_1 = 'de:08118:3533:1:3'  # Rtg Marbach (Marbach am Neckar Marbach (N))
	MARBACH__N__KRANKENHAUS_2 = 'de:08118:3533:1:4'  # Rtg Blattert/Kelter (Marbach am Neckar Marbach (N))
	BESIGHEIM_PAUL_GERHARDT_HAUS = 'de:08118:3536'
	BESIGHEIM_PAUL_GERHARDT_HAUS_1 = 'de:08118:3536:0:3'  # Rtg Ulrichstr. (Besigheim Besigheim)
	BESIGHEIM_PAUL_GERHARDT_HAUS_2 = 'de:08118:3536:0:4'  # Rtg Friedhof/Husarenweg (Besigheim Besigheim)
	PAUL_GERHARDT_HAUS = 'de:08118:3536'
	PAUL_GERHARDT_HAUS_1 = 'de:08118:3536:0:3'  # Rtg Ulrichstr. (Besigheim Besigheim)
	PAUL_GERHARDT_HAUS_2 = 'de:08118:3536:0:4'  # Rtg Friedhof/Husarenweg (Besigheim Besigheim)
	MARBACH__N__KÖNIG_WILHELM_PL = 'de:08118:3537'
	MARBACH__N__KÖNIG_WILHELM_PL_1 = 'de:08118:3537:0:3'  # Rtg Hölderlinstr. (Marbach am Neckar Marbach (N))
	MARBACH__N__KÖNIG_WILHELM_PL_2 = 'de:08118:3537:0:4'  # Rtg Güntterstr. (Marbach am Neckar Marbach (N))
	KÖNIG_WILHELM_PLATZ = 'de:08118:3537'
	KÖNIG_WILHELM_PLATZ_1 = 'de:08118:3537:0:3'  # Rtg Hölderlinstr. (Marbach am Neckar Marbach (N))
	KÖNIG_WILHELM_PLATZ_2 = 'de:08118:3537:0:4'  # Rtg Güntterstr. (Marbach am Neckar Marbach (N))
	MARB__N__HÖ__DREIBRONNENSTR = 'de:08118:3539:0:3'  # eRtg Erdmannhäuser Str./Eichgraben (Marbach am Neckar Marbach (N))
	HÖRNLE_DREIBRONNENSTRASSE = 'de:08118:3539:0:3'  # eRtg Erdmannhäuser Str./Eichgraben (Marbach am Neckar Marbach (N))
	MARB__N__HÖRNLE_HOCHHAUS = 'de:08118:3540:1:3'  # eRtg Dreibronnenstraße (Marbach am Neckar Marbach (N))
	HÖRNLE_HOCHHAUS = 'de:08118:3540:1:3'  # eRtg Dreibronnenstraße (Marbach am Neckar Marbach (N))
	MARB__N__HÖ__WIESBAD__PLATZ = 'de:08118:3541:0:3'  # eRtg Hörnle Hochhaus (Marbach am Neckar Marbach (N))
	HÖRNLE_WIESBADENER_PLATZ = 'de:08118:3541:0:3'  # eRtg Hörnle Hochhaus (Marbach am Neckar Marbach (N))
	GRABENSTRASSE_1 = 'de:08118:3543:0:3'  # Rtg Eichgraben (Marbach am Neckar Marbach (N))
	GRABENSTRASSE_2 = 'de:08115:4581:0:3'  # eRtg Stadtgarten (Weil der Stadt Weil der Stadt)
	GRABENSTRASSE_3 = 'de:08115:4581:0:4'  # eRtg Würmbrücke (Weil der Stadt Weil der Stadt)
	GRABENSTRASSE_4 = 'de:08119:5988:0:3'  # Rtg Königsbau (Plüderhausen Plüderhausen)
	GRABENSTRASSE_5 = 'de:08119:5988:0:4'  # Rtg Postweg (Plüderhausen Plüderhausen)
	GRABENSTRASSE_6 = 'de:08115:7073:0:3'  # Rtg Dorfgemeinschaft Tennental (Gärtringen Gärtringen)
	GRABENSTRASSE_7 = 'de:08115:7073:0:4'  # Rtg Schwarzwaldstraße (Gärtringen Gärtringen)
	MARBACH__N__GRABENSTRASSE = 'de:08118:3543:0:3'  # Rtg Eichgraben (Marbach am Neckar Marbach (N))
	EICHGRABEN = 'de:08118:3544'
	EICHGRABEN_1 = 'de:08118:3544:1:3'  # Rtg Ludwigsburg (Marbach am Neckar Marbach (N))
	EICHGRABEN_2 = 'de:08118:3544:1:4'  # Rtg Bahnhof (Marbach am Neckar Marbach (N))
	EICHGRABEN_3 = 'de:08118:3544:2:1'  # Rtg Hörnle (Marbach am Neckar Marbach (N))
	EICHGRABEN_4 = 'de:08118:3544:2:2'  # Rtg Bahnhof (Marbach am Neckar Marbach (N))
	MARBACH__N__EICHGRABEN = 'de:08118:3544'
	MARBACH__N__EICHGRABEN_1 = 'de:08118:3544:1:3'  # Rtg Ludwigsburg (Marbach am Neckar Marbach (N))
	MARBACH__N__EICHGRABEN_2 = 'de:08118:3544:1:4'  # Rtg Bahnhof (Marbach am Neckar Marbach (N))
	MARBACH__N__EICHGRABEN_3 = 'de:08118:3544:2:1'  # Rtg Hörnle (Marbach am Neckar Marbach (N))
	MARBACH__N__EICHGRABEN_4 = 'de:08118:3544:2:2'  # Rtg Bahnhof (Marbach am Neckar Marbach (N))
	BURGST__M__AUF_DEN_RÜDERN = 'de:08119:3545'
	BURGST__M__AUF_DEN_RÜDERN_1 = 'de:08119:3545:0:3'  # eRtg Rathaus (Burgstetten Burgstall (Burgstetten))
	BURGST__M__AUF_DEN_RÜDERN_2 = 'de:08119:3545:0:4'  # eRtg Murrbrücke (Burgstetten Burgstall (Burgstetten))
	AUF_DEN_RÜDERN = 'de:08119:3545'
	AUF_DEN_RÜDERN_1 = 'de:08119:3545:0:3'  # eRtg Rathaus (Burgstetten Burgstall (Burgstetten))
	AUF_DEN_RÜDERN_2 = 'de:08119:3545:0:4'  # eRtg Murrbrücke (Burgstetten Burgstall (Burgstetten))
	KIRCHB__M__ZW__GROSSASP__STR = 'de:08119:3547'
	KIRCHB__M__ZW__GROSSASP__STR_1 = 'de:08119:3547:0:3'  # eRtg Frühmeßhof (Kirchberg an der Murr Kirchberg (M))
	KIRCHB__M__ZW__GROSSASP__STR_2 = 'de:08119:3547:0:4'  # eRtg Zwingelhausen Abzweig (Kirchberg an der Murr Kirchberg (M))
	ZWINGELH__GROSSASPACHER_STRASSE = 'de:08119:3547'
	ZWINGELH__GROSSASPACHER_STRASSE_1 = 'de:08119:3547:0:3'  # eRtg Frühmeßhof (Kirchberg an der Murr Kirchberg (M))
	ZWINGELH__GROSSASPACHER_STRASSE_2 = 'de:08119:3547:0:4'  # eRtg Zwingelhausen Abzweig (Kirchberg an der Murr Kirchberg (M))
	ZWINGELHAUSEN_KIRCHBERGER_STR = 'de:08119:3548'
	ZWINGELHAUSEN_KIRCHBERGER_STR_1 = 'de:08119:3548:0:3'  # eRtg Zwingelh. Großaspacher Straße (Kirchberg an der Murr Kirchberg (M))
	ZWINGELHAUSEN_KIRCHBERGER_STR_2 = 'de:08119:3548:0:4'  # eRtg K. Zwingelhäuser Straße (Kirchberg an der Murr Kirchberg (M))
	KIRCHB__M__ZWIN__KIRCHB__STR = 'de:08119:3548'
	KIRCHB__M__ZWIN__KIRCHB__STR_1 = 'de:08119:3548:0:3'  # eRtg Zwingelh. Großaspacher Straße (Kirchberg an der Murr Kirchberg (M))
	KIRCHB__M__ZWIN__KIRCHB__STR_2 = 'de:08119:3548:0:4'  # eRtg K. Zwingelhäuser Straße (Kirchberg an der Murr Kirchberg (M))
	STADIONHALLE = 'de:08118:3549:0:4'  # Rtg alle Richtungen (Marbach am Neckar Marbach (N))
	MARBACH__N__STADIONHALLE = 'de:08118:3549:0:4'  # Rtg alle Richtungen (Marbach am Neckar Marbach (N))
	MARBACH__N__SCHILLERHÖHE = 'de:08118:3550'
	MARBACH__N__SCHILLERHÖHE_1 = 'de:08118:3550:0:3'  # Rtg Ost (Marbach am Neckar Marbach (N))
	MARBACH__N__SCHILLERHÖHE_2 = 'de:08118:3550:0:4'  # Rtg West (Marbach am Neckar Marbach (N))
	SCHILLERHÖHE = 'de:08118:3550'
	SCHILLERHÖHE_1 = 'de:08118:3550:0:3'  # Rtg Ost (Marbach am Neckar Marbach (N))
	SCHILLERHÖHE_2 = 'de:08118:3550:0:4'  # Rtg West (Marbach am Neckar Marbach (N))
	ERDMANNHAUSEN_BAHNHOFSTR = 'de:08118:3551'
	ERDMANNHAUSEN_BAHNHOFSTR_1 = 'de:08118:3551:1:3'  # eRtg Ankunft (Erdmannhausen Erdmannhausen)
	ERDMANNHAUSEN_BAHNHOFSTR_2 = 'de:08118:3551:1:4'  # eRtg Rathaus (Erdmannhausen Erdmannhausen)
	GROSSBOTTWAR_SCHULZENTRUM = 'de:08118:3553:0:3'  # eRtg Alle Richtungen (Großbottwar Großbottwar)
	FREIBERG__N__STARENWEG = 'de:08118:3559'
	FREIBERG__N__STARENWEG_1 = 'de:08118:3559:0:3'  # eRtg Löchlesäcker (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__STARENWEG_2 = 'de:08118:3559:0:4'  # eRtg Bilfinger Straße (Freiberg am Neckar Freiberg (N))
	STARENWEG = 'de:08118:3559'
	STARENWEG_1 = 'de:08118:3559:0:3'  # eRtg Löchlesäcker (Freiberg am Neckar Freiberg (N))
	STARENWEG_2 = 'de:08118:3559:0:4'  # eRtg Bilfinger Straße (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__MENZELSTRASSE = 'de:08118:3560'
	FREIBERG__N__MENZELSTRASSE_1 = 'de:08118:3560:0:3'  # eRtg Beihinger Platz (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__MENZELSTRASSE_2 = 'de:08118:3560:0:4'  # eRtg Freiberg (N) (Freiberg am Neckar Freiberg (N))
	MENZELSTRASSE = 'de:08118:3560'
	MENZELSTRASSE_1 = 'de:08118:3560:0:3'  # eRtg Beihinger Platz (Freiberg am Neckar Freiberg (N))
	MENZELSTRASSE_2 = 'de:08118:3560:0:4'  # eRtg Freiberg (N) (Freiberg am Neckar Freiberg (N))
	BESIGHEIM_MARTINSHAUS = 'de:08118:3561'
	BESIGHEIM_MARTINSHAUS_1 = 'de:08118:3561:0:3'  # eRtg Kirschhaldenweg (Besigheim Besigheim)
	BESIGHEIM_MARTINSHAUS_2 = 'de:08118:3561:0:4'  # eRtg Enzplatz (Besigheim Besigheim)
	MARTINSHAUS = 'de:08118:3561'
	MARTINSHAUS_1 = 'de:08118:3561:0:3'  # eRtg Kirschhaldenweg (Besigheim Besigheim)
	MARTINSHAUS_2 = 'de:08118:3561:0:4'  # eRtg Enzplatz (Besigheim Besigheim)
	KLEINSACHSH__WILHELMSTRASSE = 'de:08118:3562:0:3'  # eRtg Bietigheimer Straße (Sachsenheim Kleinsachsenheim)
	PIEMONTESER_STRASSE = 'de:08118:3563:0:3'  # eRtg Rathaus (Erdmannhausen Erdmannhausen)
	ERDMANNHAUSEN_PIEMONTESER_STR = 'de:08118:3563:0:3'  # eRtg Rathaus (Erdmannhausen Erdmannhausen)
	HESSIGHEIM_SCHULE = 'de:08118:3564:0:3'  # eRtg Neusatz/Ottmarsh. Straße (Hessigheim Hessigheim)
	PLEIDELSHEIM_HOHENZOLLERNSTR = 'de:08118:3565'
	PLEIDELSHEIM_HOHENZOLLERNSTR_1 = 'de:08118:3565:0:3'  # eRtg Beihinger Straße (Pleidelsheim Pleidelsheim)
	PLEIDELSHEIM_HOHENZOLLERNSTR_2 = 'de:08118:3565:0:4'  # eRtg Robert-Bosch-Straße (Pleidelsheim Pleidelsheim)
	HOHENZOLLERNSTRASSE_1 = 'de:08118:3565:0:3'  # eRtg Beihinger Straße (Pleidelsheim Pleidelsheim)
	HOHENZOLLERNSTRASSE_2 = 'de:08118:3565:0:4'  # eRtg Robert-Bosch-Straße (Pleidelsheim Pleidelsheim)
	HOHENZOLLERNSTRASSE_3 = 'de:08115:4208:0:3'  # eRtg Lilienstr./Altenzentrum/Gymnasium (Holzgerlingen Holzgerlingen)
	HOHENZOLLERNSTRASSE_4 = 'de:08115:4208:0:4'  # eRtg Germanenstraße/Böblingen (Holzgerlingen Holzgerlingen)
	HOHENZOLLERNSTRASSE_5 = 'de:08115:4579:0:3'  # eRtg Magstadter Straße (Weil der Stadt Schafhausen)
	ALLMANDKLINGE = 'de:08118:3567'
	ALLMANDKLINGE_1 = 'de:08118:3567:0:3'  # eRtg Brombergerhöfe (Sachsenheim Hohenhaslach)
	ALLMANDKLINGE_2 = 'de:08118:3567:0:4'  # eRtg Steige (Sachsenheim Hohenhaslach)
	HOHENHASLACH_ALLMANDKLINGE = 'de:08118:3567'
	HOHENHASLACH_ALLMANDKLINGE_1 = 'de:08118:3567:0:3'  # eRtg Brombergerhöfe (Sachsenheim Hohenhaslach)
	HOHENHASLACH_ALLMANDKLINGE_2 = 'de:08118:3567:0:4'  # eRtg Steige (Sachsenheim Hohenhaslach)
	FLATTICHSCHULE = 'de:08118:3568:0:3'  # eRtg Sportzentrum (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_FLATTICHSCHULE = 'de:08118:3568:0:3'  # eRtg Sportzentrum (Korntal-Münchingen Münchingen)
	ALDINGEN_HORNBACH = 'de:08118:3570'
	ALDINGEN_HORNBACH_1 = 'de:08118:3570:1:1'  # Rtg Mühlhausen (Remseck am Neckar Aldingen)
	ALDINGEN_HORNBACH_2 = 'de:08118:3570:1:2'  # Rtg Mühle (Remseck am Neckar Aldingen)
	HORNBACH = 'de:08118:3570'
	HORNBACH_1 = 'de:08118:3570:1:1'  # Rtg Mühlhausen (Remseck am Neckar Aldingen)
	HORNBACH_2 = 'de:08118:3570:1:2'  # Rtg Mühle (Remseck am Neckar Aldingen)
	MÜHLE = 'de:08118:3571'
	MÜHLE_1 = 'de:08118:3571:1:1'  # Rtg Hornbach (Remseck am Neckar Aldingen)
	MÜHLE_2 = 'de:08118:3571:1:2'  # Rtg Brückenstr. (Remseck am Neckar Aldingen)
	ALDINGEN_MÜHLE = 'de:08118:3571'
	ALDINGEN_MÜHLE_1 = 'de:08118:3571:1:1'  # Rtg Hornbach (Remseck am Neckar Aldingen)
	ALDINGEN_MÜHLE_2 = 'de:08118:3571:1:2'  # Rtg Brückenstr. (Remseck am Neckar Aldingen)
	ALDINGEN_BRÜCKENSTRASSE = 'de:08118:3572'
	ALDINGEN_BRÜCKENSTRASSE_1 = 'de:08118:3572:0:1'  # Rtg Mühle (Remseck am Neckar Aldingen)
	ALDINGEN_BRÜCKENSTRASSE_2 = 'de:08118:3572:0:2'  # Rtg Neckargröningen (Remseck am Neckar Aldingen)
	REMSECK = 'de:08118:3573'
	REMSECK_1 = 'de:08118:3573:1:1'  # Stadtbahn - Endhst. (Remseck am Neckar Neckargröningen)
	REMSECK_2 = 'de:08118:3573:2:1'  # Linien 402/403/405 Rtg Haus der Jugend/ (Remseck am Neckar Neckargröningen)
	REMSECK_3 = 'de:08118:3573:2:2'  # L.431/451 Rtg H. d. Feuerw./L.432 Rtg LB (Remseck am Neckar Neckargröningen)
	REMSECK_4 = 'de:08118:3573:2:3'  # L. 432 Rtg Hegnach/L 451 Rtg Rainwiesen (Remseck am Neckar Neckargröningen)
	REMSECK_5 = 'de:08118:3573:2:4'  # Rtg Rathaus, Wasenstraße, Bank (Remseck am Neckar Neckargröningen)
	NECKARGRÖNINGEN_REMSECK = 'de:08118:3573'
	NECKARGRÖNINGEN_REMSECK_1 = 'de:08118:3573:1:1'  # Stadtbahn - Endhst. (Remseck am Neckar Neckargröningen)
	NECKARGRÖNINGEN_REMSECK_2 = 'de:08118:3573:2:1'  # Linien 402/403/405 Rtg Haus der Jugend/ (Remseck am Neckar Neckargröningen)
	NECKARGRÖNINGEN_REMSECK_3 = 'de:08118:3573:2:2'  # L.431/451 Rtg H. d. Feuerw./L.432 Rtg LB (Remseck am Neckar Neckargröningen)
	NECKARGRÖNINGEN_REMSECK_4 = 'de:08118:3573:2:3'  # L. 432 Rtg Hegnach/L 451 Rtg Rainwiesen (Remseck am Neckar Neckargröningen)
	NECKARGRÖNINGEN_REMSECK_5 = 'de:08118:3573:2:4'  # Rtg Rathaus, Wasenstraße, Bank (Remseck am Neckar Neckargröningen)
	PATTONVILLE_BERUFSSCHULE = 'de:08118:3574'
	PATTONVILLE_BERUFSSCHULE_1 = 'de:08118:3574:0:1'  # Rtg Ludwigsburg/Aldingen (Remseck am Neckar Pattonville)
	PATTONVILLE_BERUFSSCHULE_2 = 'de:08118:3574:0:2'  # Rtg Pattonville Mitte (Remseck am Neckar Pattonville)
	PATTONVILLE_BERUFSSCHULE_3 = 'de:08118:3574:0:3'  # vor Berufsschule (Remseck am Neckar Pattonville)
	BERUFSSCHULE = 'de:08118:3574'
	BERUFSSCHULE_1 = 'de:08118:3574:0:1'  # Rtg Ludwigsburg/Aldingen (Remseck am Neckar Pattonville)
	BERUFSSCHULE_2 = 'de:08118:3574:0:2'  # Rtg Pattonville Mitte (Remseck am Neckar Pattonville)
	BERUFSSCHULE_3 = 'de:08118:3574:0:3'  # vor Berufsschule (Remseck am Neckar Pattonville)
	PATTONV__REALSCH = 'de:08118:3576'
	PATTONV__REALSCH_1 = 'de:08118:3576:0:1'  # Rtg Pattonville Mitte (Remseck am Neckar Pattonville)
	PATTONV__REALSCH_2 = 'de:08118:3576:0:2'  # Realschule H/R (Remseck am Neckar Pattonville)
	PATTONV__REALSCH_3 = 'de:08118:3576:0:3'  # Endhalte / Rtg Aldingen (Remseck am Neckar Pattonville)
	REALSCHULE_REMSECK = 'de:08118:3576'
	REALSCHULE_REMSECK_1 = 'de:08118:3576:0:1'  # Rtg Pattonville Mitte (Remseck am Neckar Pattonville)
	REALSCHULE_REMSECK_2 = 'de:08118:3576:0:2'  # Realschule H/R (Remseck am Neckar Pattonville)
	REALSCHULE_REMSECK_3 = 'de:08118:3576:0:3'  # Endhalte / Rtg Aldingen (Remseck am Neckar Pattonville)
	GROSSBOTTWAR_HAUPTSTRASSE = 'de:08118:3577'
	GROSSBOTTWAR_HAUPTSTRASSE_1 = 'de:08118:3577:0:3'  # eRtg Kelter (Großbottwar Großbottwar)
	GROSSBOTTWAR_HAUPTSTRASSE_2 = 'de:08118:3577:0:4'  # eRtg In den Frauengärten (Großbottwar Großbottwar)
	IN_DEN_FRAUENGÄRTEN = 'de:08118:3578'
	IN_DEN_FRAUENGÄRTEN_1 = 'de:08118:3578:0:3'  # eRtg Hauptstraße (Großbottwar Großbottwar)
	IN_DEN_FRAUENGÄRTEN_2 = 'de:08118:3578:0:4'  # eRtg Holzweilerhof (Großbottwar Großbottwar)
	GROSSB__IN_DEN_FRAUENGÄRTEN = 'de:08118:3578'
	GROSSB__IN_DEN_FRAUENGÄRTEN_1 = 'de:08118:3578:0:3'  # eRtg Hauptstraße (Großbottwar Großbottwar)
	GROSSB__IN_DEN_FRAUENGÄRTEN_2 = 'de:08118:3578:0:4'  # eRtg Holzweilerhof (Großbottwar Großbottwar)
	HEMMINGEN_SCHWIEBERD__STR = 'de:08118:3579'
	HEMMINGEN_SCHWIEBERD__STR_1 = 'de:08118:3579:0:1'  # Rtg Schwieberding / Schwieberdinger Str. (Hemmingen Hemmingen)
	HEMMINGEN_SCHWIEBERD__STR_2 = 'de:08118:3579:0:2'  # Rtg Hemmingen / Schwieberdinger Str. (Hemmingen Hemmingen)
	HEMMINGEN_SCHWIEBERD__STR_3 = 'de:08118:3579:0:3'  # 501 Schlosshaldenstr. (Hemmingen Hemmingen)
	MARKGRÖNINGEN_FESTPLATZ = 'de:08118:3581:0:3'  # eRtg Heyd-Schule/Festplatz (Markgröningen Markgröningen)
	FESTPLATZ_1 = 'de:08118:3581:0:3'  # eRtg Heyd-Schule/Festplatz (Markgröningen Markgröningen)
	FESTPLATZ_2 = 'de:08116:7822:0:3'  # Rtg Tiefgarage/Waldhorn/Ortsmitte (Baltmannsweiler Baltmannsweiler)
	FESTPLATZ_3 = 'de:08116:7822:0:4'  # Rtg Schurwaldhöhe (Baltmannsweiler Baltmannsweiler)
	AM_FRIEDHOF_1 = 'de:08118:3582:0:3'  # eRtg Hauptstraße (Pleidelsheim Pleidelsheim)
	AM_FRIEDHOF_2 = 'de:08118:3582:0:4'  # eRtg Tiefengasse (Pleidelsheim Pleidelsheim)
	AM_FRIEDHOF_3 = 'de:08118:7088:0:3'  # Rtg Untermberg (Sachsenheim Großsachsenheim)
	AM_FRIEDHOF_4 = 'de:08118:7088:0:4'  # Rtg Großs. Hauptstr. (Sachsenheim Großsachsenheim)
	PLEIDELSHEIM_AM_FRIEDHOF = 'de:08118:3582'
	PLEIDELSHEIM_AM_FRIEDHOF_1 = 'de:08118:3582:0:3'  # eRtg Hauptstraße (Pleidelsheim Pleidelsheim)
	PLEIDELSHEIM_AM_FRIEDHOF_2 = 'de:08118:3582:0:4'  # eRtg Tiefengasse (Pleidelsheim Pleidelsheim)
	OSSWEILER_WEG = 'de:08118:3585'
	OSSWEILER_WEG_1 = 'de:08118:3585:0:3'  # eRtg Kindergarten Albstraße (Remseck am Neckar Aldingen)
	OSSWEILER_WEG_2 = 'de:08118:3585:0:4'  # eRtg Kaltenthalstraße (Remseck am Neckar Aldingen)
	ALDINGEN_OSSWEILER_WEG = 'de:08118:3585'
	ALDINGEN_OSSWEILER_WEG_1 = 'de:08118:3585:0:3'  # eRtg Kindergarten Albstraße (Remseck am Neckar Aldingen)
	ALDINGEN_OSSWEILER_WEG_2 = 'de:08118:3585:0:4'  # eRtg Kaltenthalstraße (Remseck am Neckar Aldingen)
	KINDERGARTEN_ALBSTRASSE = 'de:08118:3586'
	KINDERGARTEN_ALBSTRASSE_1 = 'de:08118:3586:0:3'  # eRtg Kindergarten Leonberger Straße (Remseck am Neckar Aldingen)
	KINDERGARTEN_ALBSTRASSE_2 = 'de:08118:3586:0:4'  # eRtg Oßweiler Weg (Remseck am Neckar Aldingen)
	ALD__KINDERGARTEN_ALBSTR = 'de:08118:3586'
	ALD__KINDERGARTEN_ALBSTR_1 = 'de:08118:3586:0:3'  # eRtg Kindergarten Leonberger Straße (Remseck am Neckar Aldingen)
	ALD__KINDERGARTEN_ALBSTR_2 = 'de:08118:3586:0:4'  # eRtg Oßweiler Weg (Remseck am Neckar Aldingen)
	ALD__KINDERGARTEN_LEONB__STR = 'de:08118:3587'
	ALD__KINDERGARTEN_LEONB__STR_1 = 'de:08118:3587:0:3'  # eRtg Bildungszentrum (Remseck am Neckar Aldingen)
	ALD__KINDERGARTEN_LEONB__STR_2 = 'de:08118:3587:0:4'  # eRtg Kindergarten Albstraße (Remseck am Neckar Aldingen)
	KINDERGARTEN_LEONBERGER_STRASSE = 'de:08118:3587'
	KINDERGARTEN_LEONBERGER_STRASSE_1 = 'de:08118:3587:0:3'  # eRtg Bildungszentrum (Remseck am Neckar Aldingen)
	KINDERGARTEN_LEONBERGER_STRASSE_2 = 'de:08118:3587:0:4'  # eRtg Kindergarten Albstraße (Remseck am Neckar Aldingen)
	BESIGHEIM_OBERAMTEIGASSE = 'de:08118:3588'
	BESIGHEIM_OBERAMTEIGASSE_1 = 'de:08118:3588:0:3'  # eRtg Kelterplatz (Besigheim Besigheim)
	BESIGHEIM_OBERAMTEIGASSE_2 = 'de:08118:3588:0:4'  # eRtg Auf dem Kies (Besigheim Besigheim)
	OBERAMTEIGASSE = 'de:08118:3588'
	OBERAMTEIGASSE_1 = 'de:08118:3588:0:3'  # eRtg Kelterplatz (Besigheim Besigheim)
	OBERAMTEIGASSE_2 = 'de:08118:3588:0:4'  # eRtg Auf dem Kies (Besigheim Besigheim)
	GÜNDELBACH_RATHAUS = 'de:08118:3590:0:3'  # eRtg Katharinenstraße (Vaihingen an der Enz Gündelbach)
	HÖPFIGHEIM_TALSTRASSE = 'de:08118:3591'
	HÖPFIGHEIM_TALSTRASSE_1 = 'de:08118:3591:0:3'  # eRtg Obere Seewiesen (Steinheim an der Murr Höpfigheim)
	HÖPFIGHEIM_TALSTRASSE_2 = 'de:08118:3591:0:4'  # eRtg Hauptstraße (Steinheim an der Murr Höpfigheim)
	OBERE_SEEWIESEN = 'de:08118:3592:0:3'  # eRtg Ankunft/Talstraße (Steinheim an der Murr Höpfigheim)
	HÖPFIGHEIM_OBERE_SEEWIESEN = 'de:08118:3592:0:3'  # eRtg Ankunft/Talstraße (Steinheim an der Murr Höpfigheim)
	BURGERMÜHLE = 'de:08118:3593'
	BURGERMÜHLE_1 = 'de:08118:3593:0:3'  # eRtg Schlossfeld (Bönnigheim Bönnigheim)
	BURGERMÜHLE_2 = 'de:08118:3593:0:4'  # eRtg Kirchheimer Str. (Bönnigheim Bönnigheim)
	BÖNNIGHEIM_BURGERMÜHLE = 'de:08118:3593'
	BÖNNIGHEIM_BURGERMÜHLE_1 = 'de:08118:3593:0:3'  # eRtg Schlossfeld (Bönnigheim Bönnigheim)
	BÖNNIGHEIM_BURGERMÜHLE_2 = 'de:08118:3593:0:4'  # eRtg Kirchheimer Str. (Bönnigheim Bönnigheim)
	BAD_BERKA_WEG = 'de:08118:3594'
	BAD_BERKA_WEG_1 = 'de:08118:3594:0:3'  # eRtg Steinheimer Straße (Marbach am Neckar Marbach (N))
	BAD_BERKA_WEG_2 = 'de:08118:3594:0:4'  # eRtg Schillerhöhe (Marbach am Neckar Marbach (N))
	MARBACH__N__BAD_BERKA_WEG = 'de:08118:3594'
	MARBACH__N__BAD_BERKA_WEG_1 = 'de:08118:3594:0:3'  # eRtg Steinheimer Straße (Marbach am Neckar Marbach (N))
	MARBACH__N__BAD_BERKA_WEG_2 = 'de:08118:3594:0:4'  # eRtg Schillerhöhe (Marbach am Neckar Marbach (N))
	WEIMARSTRASSE_1 = 'de:08118:3595:0:3'  # eRtg Schillerhöhe (Marbach am Neckar Marbach (N))
	WEIMARSTRASSE_2 = 'de:08118:5542:0:3'  # eRtg August-Bebel-Straße (Ludwigsburg Ludwigsburg)
	WEIMARSTRASSE_3 = 'de:08118:5542:0:4'  # eRtg Weststadt-Kirche (Ludwigsburg Ludwigsburg)
	MARBACH__N__WEIMARSTRASSE = 'de:08118:3595:0:3'  # eRtg Schillerhöhe (Marbach am Neckar Marbach (N))
	MARBACH__N__HÖLDERLINSTRASSE = 'de:08118:3596:0:3'  # eRtg König-Wilhelm-Platz (Marbach am Neckar Marbach (N))
	HÖPFIGHEIM_ENDWIESENSTRASSE = 'de:08118:3597'
	HÖPFIGHEIM_ENDWIESENSTRASSE_1 = 'de:08118:3597:0:3'  # eRtg Hauptstraße (Steinheim an der Murr Höpfigheim)
	HÖPFIGHEIM_ENDWIESENSTRASSE_2 = 'de:08118:3597:0:4'  # eRtg SÜWAG (Steinheim an der Murr Höpfigheim)
	ENDWIESENSTRASSE = 'de:08118:3597'
	ENDWIESENSTRASSE_1 = 'de:08118:3597:0:3'  # eRtg Hauptstraße (Steinheim an der Murr Höpfigheim)
	ENDWIESENSTRASSE_2 = 'de:08118:3597:0:4'  # eRtg SÜWAG (Steinheim an der Murr Höpfigheim)
	ERDMANNHAUSEN_STEIGE = 'de:08118:3598:0:4'  # Rtg Bahnhofstr. (Erdmannhausen Erdmannhausen)
	STEIGE_1 = 'de:08118:3598:0:4'  # Rtg Bahnhofstr. (Erdmannhausen Erdmannhausen)
	STEIGE_2 = 'de:08118:5710:1:3'  # Rtg Staffel (Sachsenheim Hohenhaslach)
	STEIGE_3 = 'de:08118:5710:1:4'  # Rtg Allmandklinge/Schule (Sachsenheim Hohenhaslach)
	STEIGE_4 = 'de:08118:6853:0:3'  # eRtg Traubenstraße (Remseck am Neckar Neckarrems)
	AFFALTERBACH_MARBACHER_STR = 'de:08118:3599:0:3'  # eRtg Waage (Affalterbach Affalterbach)
	AFFALTERBACH_BIRKHAU = 'de:08118:3600'
	AFFALTERBACH_BIRKHAU_1 = 'de:08118:3600:0:3'  # eRtg Wolfsölden/Burgstall (Affalterbach Affalterbach)
	AFFALTERBACH_BIRKHAU_2 = 'de:08118:3600:0:4'  # eRtg Klingenstraße (Affalterbach Affalterbach)
	BIRKHAU = 'de:08118:3600'
	BIRKHAU_1 = 'de:08118:3600:0:3'  # eRtg Wolfsölden/Burgstall (Affalterbach Affalterbach)
	BIRKHAU_2 = 'de:08118:3600:0:4'  # eRtg Klingenstraße (Affalterbach Affalterbach)
	AFFALTERBACH_KLINGENSTR = 'de:08118:3601'
	AFFALTERBACH_KLINGENSTR_1 = 'de:08118:3601:0:3'  # eRtg Birkhau (Affalterbach Affalterbach)
	AFFALTERBACH_KLINGENSTR_2 = 'de:08118:3601:0:4'  # eRtg Waage (Affalterbach Affalterbach)
	AFFALTERBACH_WAAGE = 'de:08118:3602'
	AFFALTERBACH_WAAGE_1 = 'de:08118:3602:0:3'  # eRtg Marbach (N) (Affalterbach Affalterbach)
	AFFALTERBACH_WAAGE_2 = 'de:08118:3602:0:4'  # eRtg Klingenstraße (Affalterbach Affalterbach)
	AFFALTERBACH_WOLFSÖLDEN = 'de:08118:3603:0:3'  # eRtg Birkhau/Winnenden (Affalterbach Affalterbach)
	WOLFSÖLDEN = 'de:08118:3603:0:3'  # eRtg Birkhau/Winnenden (Affalterbach Affalterbach)
	AOK_WALDHORN = 'de:08119:3605'
	AOK_WALDHORN_1 = 'de:08119:3605:0:3'  # eRtg Christophstraße (Backnang Backnang)
	AOK_WALDHORN_2 = 'de:08119:3605:0:4'  # eRtg Biegel (Backnang Backnang)
	BACKNANG_AOK_WALDHORN = 'de:08119:3605'
	BACKNANG_AOK_WALDHORN_1 = 'de:08119:3605:0:3'  # eRtg Christophstraße (Backnang Backnang)
	BACKNANG_AOK_WALDHORN_2 = 'de:08119:3605:0:4'  # eRtg Biegel (Backnang Backnang)
	BLUMENSTRASSE_1 = 'de:08119:3606:0:3'  # eRtg Kreisberufsschulzentrum (Backnang Backnang)
	BLUMENSTRASSE_2 = 'de:08116:4289:0:3'  # eRtg Industriegebiet (Unterensingen Unterensingen)
	BLUMENSTRASSE_3 = 'de:08116:4289:0:4'  # eRtg Stern (Unterensingen Unterensingen)
	BLUMENSTRASSE_4 = 'de:08116:4508:0:3'  # eRtg Schorndorfer Straße (Reichenbach an der Fils Reichenbach (F))
	BLUMENSTRASSE_5 = 'de:08116:4508:0:4'  # eRtg Blumen-/Wilhelmstraße (Reichenbach an der Fils Reichenbach (F))
	BLUMENSTRASSE_6 = 'de:08119:5144:1:3'  # Rtg Stadtmitte (Waiblingen Waiblingen)
	BLUMENSTRASSE_7 = 'de:08119:5144:1:4'  # Rtg Obere Bf-str. (Waiblingen Waiblingen)
	BLUMENSTRASSE_8 = 'de:08118:5568:1:3'  # alle Richtungen (Ingersheim Kleiningersheim)
	BACKNANG_BLUMENSTRASSE = 'de:08119:3606:0:3'  # eRtg Kreisberufsschulzentrum (Backnang Backnang)
	BACKNANG_FRIEDHOF = 'de:08119:3607:0:3'  # eRtg Johanneskirche (Backnang Backnang)
	SCHILLERPLATZ_1 = 'de:08119:3608:0:1'  # Rtg Rathaus (Backnang Backnang)
	SCHILLERPLATZ_2 = 'de:08119:3608:0:2'  # Rtg Maubacher Straße (Backnang Backnang)
	SCHILLERPLATZ_3 = 'de:08119:3608:0:3'  # Rtg ZOB (Backnang Backnang)
	SCHILLERPLATZ_4 = 'de:08116:4040:1:3'  # Rtg Schwimmbad (Esslingen am Neckar Esslingen (N))
	SCHILLERPLATZ_5 = 'de:08116:4040:1:4'  # Rtg Neckarfreibad (Esslingen am Neckar Esslingen (N))
	SCHILLERPLATZ_6 = 'de:08116:4304:0:4'  # Rtg Süd (Nürtingen Nürtingen)
	SCHILLERPLATZ_7 = 'de:08119:5216:0:3'  # eRtg Burggymnasium (Schorndorf Schorndorf)
	SCHILLERPLATZ_8 = 'de:08119:5216:0:4'  # eRtg Jakob-Raible-Anlage (Schorndorf Schorndorf)
	SCHILLERPLATZ_9 = 'de:08118:5724:0:3'  # eRtg Gemeindehalle (Kirchheim am Neckar Kirchheim (N))
	SCHILLERPLATZ_10 = 'de:08118:5724:0:4'  # eRtg Kirchheim (N) (Kirchheim am Neckar Kirchheim (N))
	SCHILLERPLATZ_11 = 'de:08118:7405:0:3'  # eRtg Kronenzentrum (Bietigheim-Bissingen Bietigheim)
	SCHILLERPLATZ_12 = 'de:08118:7405:0:4'  # eRtg Bolzstraße (Bietigheim-Bissingen Bietigheim)
	SCHILLERPLATZ_13 = 'de:08117:1001:0:H1'  # Rtg Rathaus (Göppingen Göppingen)
	BACKNANG_SCHILLERPLATZ = 'de:08119:3608'
	BACKNANG_SCHILLERPLATZ_1 = 'de:08119:3608:0:1'  # Rtg Rathaus (Backnang Backnang)
	BACKNANG_SCHILLERPLATZ_2 = 'de:08119:3608:0:2'  # Rtg Maubacher Straße (Backnang Backnang)
	BACKNANG_SCHILLERPLATZ_3 = 'de:08119:3608:0:3'  # Rtg ZOB (Backnang Backnang)
	MAX_BORN_GYMNASIUM = 'de:08119:3609'
	MAX_BORN_GYMNASIUM_1 = 'de:08119:3609:1:3'  # Rtg Blumenstr. (Backnang Backnang)
	MAX_BORN_GYMNASIUM_2 = 'de:08119:3609:1:4'  # Rtg Carl-Kaelble-Str. (Backnang Backnang)
	BACKNANG_MAX_BORN_GYMNASIUM = 'de:08119:3609'
	BACKNANG_MAX_BORN_GYMNASIUM_1 = 'de:08119:3609:1:3'  # Rtg Blumenstr. (Backnang Backnang)
	BACKNANG_MAX_BORN_GYMNASIUM_2 = 'de:08119:3609:1:4'  # Rtg Carl-Kaelble-Str. (Backnang Backnang)
	BACKNANG_KREISBERUFSSCHULZ = 'de:08119:3611'
	BACKNANG_KREISBERUFSSCHULZ_1 = 'de:08119:3611:0:3'  # eRtg Im Kusterfeld/Maubach (Backnang Backnang)
	BACKNANG_KREISBERUFSSCHULZ_2 = 'de:08119:3611:0:4'  # eRtg Friedhof (Backnang Backnang)
	KREISBERUFSSCHULZENTRUM = 'de:08119:3611'
	KREISBERUFSSCHULZENTRUM_1 = 'de:08119:3611:0:3'  # eRtg Im Kusterfeld/Maubach (Backnang Backnang)
	KREISBERUFSSCHULZENTRUM_2 = 'de:08119:3611:0:4'  # eRtg Friedhof (Backnang Backnang)
	BACKNANG_IM_WIESENGRUND = 'de:08119:3613'
	BACKNANG_IM_WIESENGRUND_1 = 'de:08119:3613:0:3'  # eRtg Berliner Ring (Backnang Backnang)
	BACKNANG_IM_WIESENGRUND_2 = 'de:08119:3613:0:4'  # eRtg Christophstraße (Backnang Backnang)
	IM_WIESENGRUND_1 = 'de:08119:3613:0:3'  # eRtg Berliner Ring (Backnang Backnang)
	IM_WIESENGRUND_2 = 'de:08119:3613:0:4'  # eRtg Christophstraße (Backnang Backnang)
	IM_WIESENGRUND_3 = 'de:08118:5450:0:3'  # eRtg Mühlhäuser Straße (Kornwestheim Kornwestheim)
	WALDREMS_RATHAUS = 'de:08119:3615'
	WALDREMS_RATHAUS_1 = 'de:08119:3615:0:3'  # eRtg Bottwarstraße (Backnang Waldrems)
	WALDREMS_RATHAUS_2 = 'de:08119:3615:0:4'  # eRtg Schozachstraße (Backnang Waldrems)
	WALDREMS_SCHOZACHSTR = 'de:08119:3616'
	WALDREMS_SCHOZACHSTR_1 = 'de:08119:3616:0:3'  # eRtg Rathaus (Backnang Waldrems)
	WALDREMS_SCHOZACHSTR_2 = 'de:08119:3616:0:4'  # eRtg Heiningen (Backnang Waldrems)
	SCHOZACHSTRASSE = 'de:08119:3616'
	SCHOZACHSTRASSE_1 = 'de:08119:3616:0:3'  # eRtg Rathaus (Backnang Waldrems)
	SCHOZACHSTRASSE_2 = 'de:08119:3616:0:4'  # eRtg Heiningen (Backnang Waldrems)
	BURGST__M__KIRSCHENHARDTHOF = 'de:08119:3617:0:3'  # eRtg Heidenhof (Burgstetten Burgstall (Burgstetten))
	KIRSCHENHARDTHOF = 'de:08119:3617:0:3'  # eRtg Heidenhof (Burgstetten Burgstall (Burgstetten))
	HERTMANNSWEILER_SCHMIEDE = 'de:08119:3622:0:3'  # e Rtg Winnenden/Backnang (Winnenden Hertmannsweiler)
	SCHMIEDE = 'de:08119:3622:0:3'  # e Rtg Winnenden/Backnang (Winnenden Hertmannsweiler)
	REMSPARK = 'de:08119:3626'
	REMSPARK_1 = 'de:08119:3626:0:3'  # Rtg Danziger Platz (Waiblingen Waiblingen)
	REMSPARK_2 = 'de:08119:3626:0:4'  # Rtg B.Wasserturm/Fellb. Rems-Murr-Center (Waiblingen Waiblingen)
	WAIBLINGEN_REMSPARK = 'de:08119:3626'
	WAIBLINGEN_REMSPARK_1 = 'de:08119:3626:0:3'  # Rtg Danziger Platz (Waiblingen Waiblingen)
	WAIBLINGEN_REMSPARK_2 = 'de:08119:3626:0:4'  # Rtg B.Wasserturm/Fellb. Rems-Murr-Center (Waiblingen Waiblingen)
	BÜRGERZENTRUM__HALLENBAD = 'de:08119:3627'
	BÜRGERZENTRUM__HALLENBAD_1 = 'de:08119:3627:0:3'  # Rtg stadtauswärts (201: Am Kätzenbach) (Waiblingen Waiblingen)
	BÜRGERZENTRUM__HALLENBAD_2 = 'de:08119:3627:0:4'  # Rtg Stadtmitte (Waiblingen Waiblingen)
	WAIBL__BÜRGERZ___HALLENBAD = 'de:08119:3627'
	WAIBL__BÜRGERZ___HALLENBAD_1 = 'de:08119:3627:0:3'  # Rtg stadtauswärts (201: Am Kätzenbach) (Waiblingen Waiblingen)
	WAIBL__BÜRGERZ___HALLENBAD_2 = 'de:08119:3627:0:4'  # Rtg Stadtmitte (Waiblingen Waiblingen)
	DEVIZESSTRASSE = 'de:08119:3628'
	DEVIZESSTRASSE_1 = 'de:08119:3628:0:3'  # Rtg Waiblingen Bf (Waiblingen Waiblingen)
	DEVIZESSTRASSE_2 = 'de:08119:3628:0:4'  # Rtg Rommelh. Str./Arbeitsagentur/Post (Waiblingen Waiblingen)
	WAIBLINGEN_DEVIZESSTRASSE = 'de:08119:3628'
	WAIBLINGEN_DEVIZESSTRASSE_1 = 'de:08119:3628:0:3'  # Rtg Waiblingen Bf (Waiblingen Waiblingen)
	WAIBLINGEN_DEVIZESSTRASSE_2 = 'de:08119:3628:0:4'  # Rtg Rommelh. Str./Arbeitsagentur/Post (Waiblingen Waiblingen)
	WAIBLINGEN_STADTWERKE = 'de:08119:3629'
	WAIBLINGEN_STADTWERKE_1 = 'de:08119:3629:0:3'  # eRtg Giselastraße (Waiblingen Waiblingen)
	WAIBLINGEN_STADTWERKE_2 = 'de:08119:3629:0:4'  # eRtg Freibad (Waiblingen Waiblingen)
	STADTWERKE_1 = 'de:08119:3629:0:3'  # eRtg Giselastraße (Waiblingen Waiblingen)
	STADTWERKE_2 = 'de:08119:3629:0:4'  # eRtg Freibad (Waiblingen Waiblingen)
	STADTWERKE_3 = 'de:08118:5466:0:3'  # eRtg Gänsfußallee (Ludwigsburg Ludwigsburg)
	STADTWERKE_4 = 'de:08118:5466:0:4'  # eRtg ZOB (Ludwigsburg Ludwigsburg)
	STADTWERKE_5 = 'de:08115:7009:0:3'  # Rtg Nufringen (Herrenberg Herrenberg)
	STADTWERKE_6 = 'de:08115:7009:0:4'  # Rtg Herrenberg (Herrenberg Herrenberg)
	STADTWERKE_7 = 'de:08117:1034:0:H1'  # Rtg Eislingen (Göppingen Göppingen)
	STADTWERKE_8 = 'de:08117:1034:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	WAIBLINGEN_BEIM_WASSERTURM = 'de:08119:3632:0:3'  # Rtg Rems-Murr-Center (Waiblingen Waiblingen)
	BEIM_WASSERTURM_1 = 'de:08119:3632:0:3'  # Rtg Rems-Murr-Center (Waiblingen Waiblingen)
	BEIM_WASSERTURM_2 = 'de:08119:6938:0:3'  # Rtg Rossbergstaffel (Backnang Backnang)
	BEIM_WASSERTURM_3 = 'de:08119:6938:0:4'  # Rtg Dresdener Ring (Backnang Backnang)
	WINNENDEN_BACKNANGER_STR = 'de:08119:3634'
	WINNENDEN_BACKNANGER_STR_1 = 'de:08119:3634:0:3'  # eRtg Karl-Georg-Pfleiderer-Straße (Winnenden Winnenden)
	WINNENDEN_BACKNANGER_STR_2 = 'de:08119:3634:0:4'  # eRtg Seestraße (Winnenden Winnenden)
	BACKNANGER_STRASSE_1 = 'de:08119:3634:0:3'  # eRtg Karl-Georg-Pfleiderer-Straße (Winnenden Winnenden)
	BACKNANGER_STRASSE_2 = 'de:08119:3634:0:4'  # eRtg Seestraße (Winnenden Winnenden)
	BACKNANGER_STRASSE_3 = 'de:08119:5035:0:3'  # eRtg Marktplatz/Rudersberg (Rudersberg Rudersberg)
	BACKNANGER_STRASSE_4 = 'de:08119:5035:0:4'  # eRtg Seelach (Rudersberg Rudersberg)
	MAX_EYTH_STRASSE_1 = 'de:08119:3635:0:3'  # eRtg Max-Eyth-/Daimlerstraße (Winnenden Winnenden)
	MAX_EYTH_STRASSE_2 = 'de:08119:3635:0:4'  # eRtg Winnenden (Winnenden Winnenden)
	MAX_EYTH_STRASSE_3 = 'de:08116:4248:0:3'  # eRtg Ziegelei (Wernau (Neckar) Wernau (N))
	MAX_EYTH_STRASSE_4 = 'de:08118:5456:0:3'  # Rtg Ludwigsburg (Kornwestheim Kornwestheim)
	MAX_EYTH_STRASSE_5 = 'de:08118:5456:0:4'  # Rtg Maybachstr. (Kornwestheim Kornwestheim)
	WINNENDEN_MAX_EYTH_STR = 'de:08119:3635'
	WINNENDEN_MAX_EYTH_STR_1 = 'de:08119:3635:0:3'  # eRtg Max-Eyth-/Daimlerstraße (Winnenden Winnenden)
	WINNENDEN_MAX_EYTH_STR_2 = 'de:08119:3635:0:4'  # eRtg Winnenden (Winnenden Winnenden)
	WINNENDEN_SEESTRASSE = 'de:08119:3636'
	WINNENDEN_SEESTRASSE_1 = 'de:08119:3636:0:3'  # eRtg Winnenden (Winnenden Winnenden)
	WINNENDEN_SEESTRASSE_2 = 'de:08119:3636:0:4'  # eRtg Backnanger Straße (Winnenden Winnenden)
	WINNENDEN_KRONENPLATZ = 'de:08119:3637'
	WINNENDEN_KRONENPLATZ_1 = 'de:08119:3637:0:3'  # Rtg Brückenstr./Wallstr./Ringstraße (Winnenden Winnenden)
	WINNENDEN_KRONENPLATZ_2 = 'de:08119:3637:0:4'  # Rtg Winnenden Bf (Winnenden Winnenden)
	WINNENDEN_KRONENPLATZ_3 = 'de:08119:3637:0:6'  # Ersatz Rtg. Winnenden Bf (Winnenden Winnenden)
	WINNENDEN_KRONENPLATZ_4 = 'de:08119:3637:0:7'  # Ersatz Rtg. Ringstr. (Winnenden Winnenden)
	KRONENPLATZ = 'de:08119:3637'
	KRONENPLATZ_1 = 'de:08119:3637:0:3'  # Rtg Brückenstr./Wallstr./Ringstraße (Winnenden Winnenden)
	KRONENPLATZ_2 = 'de:08119:3637:0:4'  # Rtg Winnenden Bf (Winnenden Winnenden)
	KRONENPLATZ_3 = 'de:08119:3637:0:6'  # Ersatz Rtg. Winnenden Bf (Winnenden Winnenden)
	KRONENPLATZ_4 = 'de:08119:3637:0:7'  # Ersatz Rtg. Ringstr. (Winnenden Winnenden)
	BACKNANG_STADTMÜHLE = 'de:08119:3638:0:3'  # Rtg Aspacher Brücke/Gerberstr. (Backnang Backnang)
	STADTMÜHLE = 'de:08119:3638:0:3'  # Rtg Aspacher Brücke/Gerberstr. (Backnang Backnang)
	BACKNANG_KUCHENGRUND_OVR = 'de:08119:3639'
	BACKNANG_KUCHENGRUND_OVR_1 = 'de:08119:3639:0:3'  # eRtg Ankunft OVR (Backnang Backnang)
	BACKNANG_KUCHENGRUND_OVR_2 = 'de:08119:3639:0:4'  # eRtg Im Kusterfeld (Backnang Backnang)
	KUCHENGRUND_OVR = 'de:08119:3639'
	KUCHENGRUND_OVR_1 = 'de:08119:3639:0:3'  # eRtg Ankunft OVR (Backnang Backnang)
	KUCHENGRUND_OVR_2 = 'de:08119:3639:0:4'  # eRtg Im Kusterfeld (Backnang Backnang)
	NEUE_ROMMELSHAUSER_STRASSE = 'de:08119:3640'
	NEUE_ROMMELSHAUSER_STRASSE_1 = 'de:08119:3640:0:3'  # eRtg Sommerhalde (Waiblingen Waiblingen)
	NEUE_ROMMELSHAUSER_STRASSE_2 = 'de:08119:3640:0:4'  # eRtg Devizesstraße (Waiblingen Waiblingen)
	WAIBL__NEUE_ROMMELSHAUSER_STR = 'de:08119:3640'
	WAIBL__NEUE_ROMMELSHAUSER_STR_1 = 'de:08119:3640:0:3'  # eRtg Sommerhalde (Waiblingen Waiblingen)
	WAIBL__NEUE_ROMMELSHAUSER_STR_2 = 'de:08119:3640:0:4'  # eRtg Devizesstraße (Waiblingen Waiblingen)
	MAX_EYTH__DAIMLERSTRASSE = 'de:08119:3641'
	MAX_EYTH__DAIMLERSTRASSE_1 = 'de:08119:3641:0:3'  # eRtg BBW (Winnenden Winnenden)
	MAX_EYTH__DAIMLERSTRASSE_2 = 'de:08119:3641:0:4'  # eRtg Max-Eyth-Straße (Winnenden Winnenden)
	WINN__MAX_EYTH__DAIMLERSTRASSE = 'de:08119:3641'
	WINN__MAX_EYTH__DAIMLERSTRASSE_1 = 'de:08119:3641:0:3'  # eRtg BBW (Winnenden Winnenden)
	WINN__MAX_EYTH__DAIMLERSTRASSE_2 = 'de:08119:3641:0:4'  # eRtg Max-Eyth-Straße (Winnenden Winnenden)
	WUNNEBAD = 'de:08119:3642'
	WUNNEBAD_1 = 'de:08119:3642:0:3'  # eRtg Burgunderweg (Winnenden Winnenden)
	WUNNEBAD_2 = 'de:08119:3642:0:4'  # eRtg Hermann-Schwab-Halle (Winnenden Winnenden)
	WINNENDEN_WUNNEBAD = 'de:08119:3642'
	WINNENDEN_WUNNEBAD_1 = 'de:08119:3642:0:3'  # eRtg Burgunderweg (Winnenden Winnenden)
	WINNENDEN_WUNNEBAD_2 = 'de:08119:3642:0:4'  # eRtg Hermann-Schwab-Halle (Winnenden Winnenden)
	WAIBLINGEN_WENDELKÖNIG = 'de:08119:3644:1:3'  # eRtg Galgenberg (Waiblingen Waiblingen)
	WENDELKÖNIG = 'de:08119:3644:1:3'  # eRtg Galgenberg (Waiblingen Waiblingen)
	WAIBLINGEN_GALGENBERG = 'de:08119:3645:1:4'  # Rtg Alte Steige (Waiblingen Waiblingen)
	GALGENBERG = 'de:08119:3645:1:4'  # Rtg Alte Steige (Waiblingen Waiblingen)
	SCHWANEN = 'de:08119:3646'
	SCHWANEN_1 = 'de:08119:3646:1:3'  # Rtg Feuerwehrgerätehaus/Ob. Rosberg (Waiblingen Waiblingen)
	SCHWANEN_2 = 'de:08119:3646:1:4'  # Rtg. Marktgasse (Waiblingen Waiblingen)
	WAIBLINGEN_SCHWANEN = 'de:08119:3646'
	WAIBLINGEN_SCHWANEN_1 = 'de:08119:3646:1:3'  # Rtg Feuerwehrgerätehaus/Ob. Rosberg (Waiblingen Waiblingen)
	WAIBLINGEN_SCHWANEN_2 = 'de:08119:3646:1:4'  # Rtg. Marktgasse (Waiblingen Waiblingen)
	WAIBLINGEN_SCHMIDENER_STRASSE = 'de:08119:3647'
	WAIBLINGEN_SCHMIDENER_STRASSE_1 = 'de:08119:3647:0:3'  # Rtg Hochwachtturm (Waiblingen Waiblingen)
	WAIBLINGEN_SCHMIDENER_STRASSE_2 = 'de:08119:3647:0:4'  # Rtg Hausgärten (Waiblingen Waiblingen)
	SCHMIDENER_STRASSE = 'de:08119:3647'
	SCHMIDENER_STRASSE_1 = 'de:08119:3647:0:3'  # Rtg Hochwachtturm (Waiblingen Waiblingen)
	SCHMIDENER_STRASSE_2 = 'de:08119:3647:0:4'  # Rtg Hausgärten (Waiblingen Waiblingen)
	WAIBLINGEN_HAUSGÄRTEN = 'de:08119:3648:0:3'  # Rtg R.-Wagner-Str./Schellingstr. (Waiblingen Waiblingen)
	HAUSGÄRTEN = 'de:08119:3648:0:3'  # Rtg R.-Wagner-Str./Schellingstr. (Waiblingen Waiblingen)
	BERUFSSCHULZENTRUM_1 = 'de:08119:3650:0:3'  # Rtg Hohenackerstr./Talstr./Hegn. Höhe (Waiblingen Waiblingen)
	BERUFSSCHULZENTRUM_2 = 'de:08119:3650:0:4'  # Rtg Waiblingen Bf (Waiblingen Waiblingen)
	BERUFSSCHULZENTRUM_3 = 'de:08117:118:0:H1'  # Rtg Lindenschule/Kuchen (Geislingen an der Steige Geislingen (Steige))
	BERUFSSCHULZENTRUM_4 = 'de:08117:118:0:R1'  # Rtg Wölkbad/Geislingen Bf (Geislingen an der Steige Geislingen (Steige))
	WAIBLINGEN_BERUFSSCHULZENTRUM = 'de:08119:3650'
	WAIBLINGEN_BERUFSSCHULZENTRUM_1 = 'de:08119:3650:0:3'  # Rtg Hohenackerstr./Talstr./Hegn. Höhe (Waiblingen Waiblingen)
	WAIBLINGEN_BERUFSSCHULZENTRUM_2 = 'de:08119:3650:0:4'  # Rtg Waiblingen Bf (Waiblingen Waiblingen)
	WAIBLINGEN_TALSTRASSE = 'de:08119:3651:0:3'  # Rtg Hans-Sachs-Weg (Waiblingen Waiblingen)
	WAIBLINGEN_FRÖBELSTRASSE = 'de:08119:3652'
	WAIBLINGEN_FRÖBELSTRASSE_1 = 'de:08119:3652:0:3'  # eRtg Schmidener Straße (Waiblingen Waiblingen)
	WAIBLINGEN_FRÖBELSTRASSE_2 = 'de:08119:3652:0:4'  # eRtg Ludwigsburger Straße (Waiblingen Waiblingen)
	WAIBLINGEN_GALERIE = 'de:08119:3654'
	WAIBLINGEN_GALERIE_1 = 'de:08119:3654:0:3'  # Rtg Neustädter Str. (Waiblingen Waiblingen)
	WAIBLINGEN_GALERIE_2 = 'de:08119:3654:0:4'  # Rtg Schwanen/Bürgerzentrum (Remsbr) (Waiblingen Waiblingen)
	GALERIE = 'de:08119:3654'
	GALERIE_1 = 'de:08119:3654:0:3'  # Rtg Neustädter Str. (Waiblingen Waiblingen)
	GALERIE_2 = 'de:08119:3654:0:4'  # Rtg Schwanen/Bürgerzentrum (Remsbr) (Waiblingen Waiblingen)
	WAIBLINGEN_GISELASTRASSE = 'de:08119:3655'
	WAIBLINGEN_GISELASTRASSE_1 = 'de:08119:3655:0:3'  # eRtg Stadtwerke (Waiblingen Waiblingen)
	WAIBLINGEN_GISELASTRASSE_2 = 'de:08119:3655:0:4'  # eRtg Stadtmitte (Waiblingen Waiblingen)
	GISELASTRASSE = 'de:08119:3655'
	GISELASTRASSE_1 = 'de:08119:3655:0:3'  # eRtg Stadtwerke (Waiblingen Waiblingen)
	GISELASTRASSE_2 = 'de:08119:3655:0:4'  # eRtg Stadtmitte (Waiblingen Waiblingen)
	MEISENWEG = 'de:08119:3656:0:3'  # eRtg Korber Steige (Waiblingen Waiblingen)
	WAIBLINGEN_MEISENWEG = 'de:08119:3656:0:3'  # eRtg Korber Steige (Waiblingen Waiblingen)
	BACKNANG_DRESDENER_RING = 'de:08119:3657'
	BACKNANG_DRESDENER_RING_1 = 'de:08119:3657:0:3'  # eRtg Stresemannstraße (Backnang Backnang)
	BACKNANG_DRESDENER_RING_2 = 'de:08119:3657:0:4'  # eRtg Berliner Ring (Backnang Backnang)
	DRESDENER_RING = 'de:08119:3657'
	DRESDENER_RING_1 = 'de:08119:3657:0:3'  # eRtg Stresemannstraße (Backnang Backnang)
	DRESDENER_RING_2 = 'de:08119:3657:0:4'  # eRtg Berliner Ring (Backnang Backnang)
	BACKNANG_STRESEMANNSTR = 'de:08119:3658:0:3'  # eRtg Rietenauer Weg (Backnang Backnang)
	STRESEMANNSTRASSE = 'de:08119:3658:0:3'  # eRtg Rietenauer Weg (Backnang Backnang)
	BACKNANG_RIETENAUER_WEG = 'de:08119:3659:0:3'  # eRtg Lerchenäcker/Dresdner Ring (Backnang Backnang)
	RIETENAUER_WEG = 'de:08119:3659:0:3'  # eRtg Lerchenäcker/Dresdner Ring (Backnang Backnang)
	BACKNANG_GARTENSTRASSE = 'de:08119:3660'
	BACKNANG_GARTENSTRASSE_1 = 'de:08119:3660:0:3'  # eRtg Eintracht (Backnang Backnang)
	BACKNANG_GARTENSTRASSE_2 = 'de:08119:3660:0:4'  # eRtg Feuerwehr/Burgplatz (Backnang Backnang)
	BIEGEL = 'de:08119:3661'
	BIEGEL_1 = 'de:08119:3661:0:3'  # Rtg Ost/AOK/Waldhorn (Backnang Backnang)
	BIEGEL_2 = 'de:08119:3661:0:4'  # Rtg West/Aspacher Brücke (Backnang Backnang)
	BACKNANG_BIEGEL = 'de:08119:3661'
	BACKNANG_BIEGEL_1 = 'de:08119:3661:0:3'  # Rtg Ost/AOK/Waldhorn (Backnang Backnang)
	BACKNANG_BIEGEL_2 = 'de:08119:3661:0:4'  # Rtg West/Aspacher Brücke (Backnang Backnang)
	BACKNANG_CARL_KAELBLE_STR = 'de:08119:3662'
	BACKNANG_CARL_KAELBLE_STR_1 = 'de:08119:3662:0:3'  # eRtg Max-Born-Gymnasium (Backnang Backnang)
	BACKNANG_CARL_KAELBLE_STR_2 = 'de:08119:3662:0:4'  # eRtg Züricher Straße (Backnang Backnang)
	CARL_KAELBLE_STRASSE = 'de:08119:3662'
	CARL_KAELBLE_STRASSE_1 = 'de:08119:3662:0:3'  # eRtg Max-Born-Gymnasium (Backnang Backnang)
	CARL_KAELBLE_STRASSE_2 = 'de:08119:3662:0:4'  # eRtg Züricher Straße (Backnang Backnang)
	BACKNANG_ZÜRICHER_STR = 'de:08119:3663'
	BACKNANG_ZÜRICHER_STR_1 = 'de:08119:3663:0:3'  # eRtg Carl-Kaelble-Straße (Backnang Backnang)
	BACKNANG_ZÜRICHER_STR_2 = 'de:08119:3663:0:4'  # eRtg Tuxer Weg (Backnang Backnang)
	MAUBACH_SALZBURGER_STR = 'de:08119:3664'
	MAUBACH_SALZBURGER_STR_1 = 'de:08119:3664:0:3'  # eRtg Tuxer Weg (Backnang Maubach)
	MAUBACH_SALZBURGER_STR_2 = 'de:08119:3664:0:4'  # eRtg Maubach (Backnang Maubach)
	SALZBURGER_STRASSE = 'de:08119:3664'
	SALZBURGER_STRASSE_1 = 'de:08119:3664:0:3'  # eRtg Tuxer Weg (Backnang Maubach)
	SALZBURGER_STRASSE_2 = 'de:08119:3664:0:4'  # eRtg Maubach (Backnang Maubach)
	ISARSTRASSE = 'de:08119:3665'
	ISARSTRASSE_1 = 'de:08119:3665:0:3'  # eRtg Rathaus (Backnang Waldrems)
	ISARSTRASSE_2 = 'de:08119:3665:0:4'  # eRtg Bottwarstraße (Backnang Waldrems)
	WALDREMS_ISARSTRASSE = 'de:08119:3665'
	WALDREMS_ISARSTRASSE_1 = 'de:08119:3665:0:3'  # eRtg Rathaus (Backnang Waldrems)
	WALDREMS_ISARSTRASSE_2 = 'de:08119:3665:0:4'  # eRtg Bottwarstraße (Backnang Waldrems)
	BACKNANG_JOHANNESKIRCHE = 'de:08119:3666:0:3'  # eRtg Backnang (Backnang Backnang)
	BACKNANG_HASENHÄLDE = 'de:08119:3668'
	BACKNANG_HASENHÄLDE_1 = 'de:08119:3668:0:3'  # Rtg Marienheim (Backnang Backnang)
	BACKNANG_HASENHÄLDE_2 = 'de:08119:3668:0:4'  # Rtg Burgplatz (Backnang Backnang)
	HASENHÄLDE = 'de:08119:3668'
	HASENHÄLDE_1 = 'de:08119:3668:0:3'  # Rtg Marienheim (Backnang Backnang)
	HASENHÄLDE_2 = 'de:08119:3668:0:4'  # Rtg Burgplatz (Backnang Backnang)
	MAUBACH_RATHAUS = 'de:08119:3669'
	MAUBACH_RATHAUS_1 = 'de:08119:3669:0:3'  # Rtg Isarstr. (Backnang Maubach)
	MAUBACH_RATHAUS_2 = 'de:08119:3669:0:4'  # Rtg Maubach (Backnang Maubach)
	HEININGEN_MARBACHER_STR = 'de:08119:3670'
	HEININGEN_MARBACHER_STR_1 = 'de:08119:3670:0:3'  # eRtg Rathaus (Backnang Heiningen (Backnang))
	HEININGEN_MARBACHER_STR_2 = 'de:08119:3670:0:4'  # eRtg Ankunft (Backnang Heiningen (Backnang))
	KLEINASP__ORTSMITTE = 'de:08119:3672'
	KLEINASP__ORTSMITTE_1 = 'de:08119:3672:1:3'  # Rtg Sonnenhof (Aspach Kleinaspach)
	KLEINASP__ORTSMITTE_2 = 'de:08119:3672:1:4'  # Rtg Schafhaus/Karlshof/Hardtwaldhalle (Aspach Kleinaspach)
	GROSSASPACH_FRIEDRICH_LIST_STR = 'de:08119:3673'
	GROSSASPACH_FRIEDRICH_LIST_STR_1 = 'de:08119:3673:0:3'  # eRtg Hohrot (Aspach Großaspach)
	GROSSASPACH_FRIEDRICH_LIST_STR_2 = 'de:08119:3673:0:4'  # eRtg Klöpferbach (Aspach Großaspach)
	GROSSASPACH_FORSTBODEN = 'de:08119:3674'
	GROSSASPACH_FORSTBODEN_1 = 'de:08119:3674:0:3'  # eRtg Karlshof (Aspach Großaspach)
	GROSSASPACH_FORSTBODEN_2 = 'de:08119:3674:0:4'  # eRtg Klöpferbach (Aspach Großaspach)
	FORSTBODEN = 'de:08119:3674'
	FORSTBODEN_1 = 'de:08119:3674:0:3'  # eRtg Karlshof (Aspach Großaspach)
	FORSTBODEN_2 = 'de:08119:3674:0:4'  # eRtg Klöpferbach (Aspach Großaspach)
	GERTRUD_BÄUMER_WEG = 'de:08119:3675'
	GERTRUD_BÄUMER_WEG_1 = 'de:08119:3675:0:3'  # eRtg Sommerrain (Backnang Backnang)
	GERTRUD_BÄUMER_WEG_2 = 'de:08119:3675:0:4'  # eRtg Eintracht (Backnang Backnang)
	BACKN__GERTRUD_BÄUMER_WEG = 'de:08119:3675'
	BACKN__GERTRUD_BÄUMER_WEG_1 = 'de:08119:3675:0:3'  # eRtg Sommerrain (Backnang Backnang)
	BACKN__GERTRUD_BÄUMER_WEG_2 = 'de:08119:3675:0:4'  # eRtg Eintracht (Backnang Backnang)
	SEEHOF = 'de:08119:3680'
	SEEHOF_1 = 'de:08119:3680:0:3'  # eRtg Waldfriedhof (Backnang Backnang)
	SEEHOF_2 = 'de:08119:3680:0:4'  # eRtg Seehofweg (Backnang Backnang)
	BACKNANG_SEEHOF = 'de:08119:3680'
	BACKNANG_SEEHOF_1 = 'de:08119:3680:0:3'  # eRtg Waldfriedhof (Backnang Backnang)
	BACKNANG_SEEHOF_2 = 'de:08119:3680:0:4'  # eRtg Seehofweg (Backnang Backnang)
	GEMEINDEHALLE_1 = 'de:08118:3682:1:3'  # Rtg Schillerplatz/Kircheim (Kirchheim am Neckar Kirchheim (N))
	GEMEINDEHALLE_2 = 'de:08118:3682:1:4'  # Rtg Post (Kirchheim am Neckar Kirchheim (N))
	GEMEINDEHALLE_3 = 'de:08116:3804:0:3'  # eRtg Pfostenberg (Altbach Altbach)
	GEMEINDEHALLE_4 = 'de:08116:3804:0:4'  # eRtg Haldenrainweg (Altbach Altbach)
	GEMEINDEHALLE_5 = 'de:08119:5081:0:3'  # eRtg Bohnreute (Murrhardt Kirchenkirnberg)
	GEMEINDEHALLE_6 = 'de:08119:5081:0:4'  # eRtg Tiefenmad (Murrhardt Kirchenkirnberg)
	GEMEINDEHALLE_7 = 'de:08119:5273:0:3'  # eRtg Schulerhof (Winnenden Höfen)
	GEMEINDEHALLE_8 = 'de:08119:5273:0:4'  # eRtg Ortsmitte (Winnenden Höfen)
	KIRCHHEIM__N__GEMEINDEHALLE = 'de:08118:3682'
	KIRCHHEIM__N__GEMEINDEHALLE_1 = 'de:08118:3682:1:3'  # Rtg Schillerplatz/Kircheim (Kirchheim am Neckar Kirchheim (N))
	KIRCHHEIM__N__GEMEINDEHALLE_2 = 'de:08118:3682:1:4'  # Rtg Post (Kirchheim am Neckar Kirchheim (N))
	WAIBLINGEN_ALFRED_DIEBOLD_WEG = 'de:08119:3684:1:3'  # Rtg Bajastr. (Waiblingen Waiblingen)
	ALFRED_DIEBOLD_WEG = 'de:08119:3684:1:3'  # Rtg Bajastr. (Waiblingen Waiblingen)
	WAIBLINGEN_BAJASTRASSE = 'de:08119:3685'
	WAIBLINGEN_BAJASTRASSE_1 = 'de:08119:3685:1:3'  # Rtg Salierstr. (Waiblingen Waiblingen)
	WAIBLINGEN_BAJASTRASSE_2 = 'de:08119:3685:1:4'  # Rtg Alfred-Leikam-Platz (Waiblingen Waiblingen)
	BAJASTRASSE = 'de:08119:3685'
	BAJASTRASSE_1 = 'de:08119:3685:1:3'  # Rtg Salierstr. (Waiblingen Waiblingen)
	BAJASTRASSE_2 = 'de:08119:3685:1:4'  # Rtg Alfred-Leikam-Platz (Waiblingen Waiblingen)
	ANTON_SCHMIDT_STRASSE = 'de:08119:3686:0:3'  # eRtg Gewerbestraße (Akademie) (Waiblingen Waiblingen)
	WAIBLINGEN_ANTON_SCHMIDT_STR = 'de:08119:3686:0:3'  # eRtg Gewerbestraße (Akademie) (Waiblingen Waiblingen)
	WAIBLINGEN_FINANZAMT = 'de:08119:3687'
	WAIBLINGEN_FINANZAMT_1 = 'de:08119:3687:0:3'  # eRtg Fronackerstraße (Waiblingen Waiblingen)
	WAIBLINGEN_FINANZAMT_2 = 'de:08119:3687:0:4'  # eRtg Waiblingen (Waiblingen Waiblingen)
	FINANZAMT_1 = 'de:08119:3687:0:3'  # eRtg Fronackerstraße (Waiblingen Waiblingen)
	FINANZAMT_2 = 'de:08119:3687:0:4'  # eRtg Waiblingen (Waiblingen Waiblingen)
	FINANZAMT_3 = 'de:08116:4095:0:3'  # Rtg Neckar Forum (Esslingen am Neckar Esslingen (N))
	FINKENBERG = 'de:08119:3688'
	FINKENBERG_1 = 'de:08119:3688:0:3'  # Rtg Korb Alfred-Leikam-Platz (Waiblingen Waiblingen)
	FINKENBERG_2 = 'de:08119:3688:0:4'  # Rtg Nachtigallenweg (Waiblingen Waiblingen)
	WAIBLINGEN_FINKENBERG = 'de:08119:3688'
	WAIBLINGEN_FINKENBERG_1 = 'de:08119:3688:0:3'  # Rtg Korb Alfred-Leikam-Platz (Waiblingen Waiblingen)
	WAIBLINGEN_FINKENBERG_2 = 'de:08119:3688:0:4'  # Rtg Nachtigallenweg (Waiblingen Waiblingen)
	KORB_BÜHL = 'de:08119:3689'
	KORB_BÜHL_1 = 'de:08119:3689:0:3'  # Rtg Kleinheppach Korber Str. (Korb Korb)
	KORB_BÜHL_2 = 'de:08119:3689:0:4'  # Rtg Steinreinach (Korb Korb)
	BÜHL = 'de:08119:3689'
	BÜHL_1 = 'de:08119:3689:0:3'  # Rtg Kleinheppach Korber Str. (Korb Korb)
	BÜHL_2 = 'de:08119:3689:0:4'  # Rtg Steinreinach (Korb Korb)
	WAIBLINGEN_SCHIPPERTSÄCKER = 'de:08119:3690:0:3'  # eRtg Schmidener Straße (Waiblingen Waiblingen)
	SCHIPPERTSÄCKER = 'de:08119:3690:0:3'  # eRtg Schmidener Straße (Waiblingen Waiblingen)
	WAIBL__UNTERE_MAYENNER_STR = 'de:08119:3691:0:3'  # eRtg Friedhof (Waiblingen Waiblingen)
	UNTERE_MAYENNER_STRASSE = 'de:08119:3691:0:3'  # eRtg Friedhof (Waiblingen Waiblingen)
	BOSKOPWEG = 'de:08119:3692:0:3'  # eRtg Wendelkönig (Waiblingen Waiblingen)
	WAIBLINGEN_BOSKOPWEG = 'de:08119:3692:0:3'  # eRtg Wendelkönig (Waiblingen Waiblingen)
	WAIBLINGEN_IM_BAUMSTÜCKLE = 'de:08119:3693:0:3'  # eRtg Boskopweg (Waiblingen Waiblingen)
	IM_BAUMSTÜCKLE = 'de:08119:3693:0:3'  # eRtg Boskopweg (Waiblingen Waiblingen)
	WINNENDEN_BRÜCKENSTR = 'de:08119:3696'
	WINNENDEN_BRÜCKENSTR_1 = 'de:08119:3696:0:3'  # eRtg Lange Weiden (Winnenden Winnenden)
	WINNENDEN_BRÜCKENSTR_2 = 'de:08119:3696:0:4'  # eRtg Kronenplatz (Winnenden Winnenden)
	CHRISTOFSTRASSE = 'de:08119:3698'
	CHRISTOFSTRASSE_1 = 'de:08119:3698:0:3'  # eRtg Ludwigsburger Straße (Waiblingen Waiblingen)
	CHRISTOFSTRASSE_2 = 'de:08119:3698:0:4'  # eRtg Fronackerstraße (Waiblingen Waiblingen)
	WAIBLINGEN_CHRISTOFSTRASSE = 'de:08119:3698'
	WAIBLINGEN_CHRISTOFSTRASSE_1 = 'de:08119:3698:0:3'  # eRtg Ludwigsburger Straße (Waiblingen Waiblingen)
	WAIBLINGEN_CHRISTOFSTRASSE_2 = 'de:08119:3698:0:4'  # eRtg Fronackerstraße (Waiblingen Waiblingen)
	WAIBLINGEN_SENIORENZENTRUM = 'de:08119:3699:0:3'  # Rtg Friedhof (Waiblingen Waiblingen)
	SENIORENZENTRUM_1 = 'de:08119:3699:0:3'  # Rtg Friedhof (Waiblingen Waiblingen)
	SENIORENZENTRUM_2 = 'de:08115:4801:0:3'  # eRtg Grafenweg (Herrenberg Herrenberg)
	SENIORENZENTRUM_3 = 'de:08115:4801:0:4'  # eRtg Hildrizhauser Str. (Herrenberg Herrenberg)
	ROHRBRONN_ABZW = 'de:08119:3701:0:3'  # Rtg Remsbrücke (Remshalden Rohrbronn)
	ABZWEIG_1 = 'de:08119:3701:0:3'  # Rtg Remsbrücke (Remshalden Rohrbronn)
	ABZWEIG_2 = 'de:08115:4206:0:3'  # Rtg Böblingen (Holzgerlingen Holzgerlingen)
	ABZWEIG_3 = 'de:08115:4206:0:4'  # Rtg Schaichhof (Holzgerlingen Holzgerlingen)
	ABZWEIG_4 = 'de:08116:4419:0:3'  # eRtg Grünenbergstraße (Lenningen Schlattstall)
	ABZWEIG_5 = 'de:08116:4419:0:4'  # eRtg Etterstraße (Lenningen Schlattstall)
	ABZWEIG_6 = 'de:08115:4789:0:3'  # eRtg Hennersdorfer Str/Rathaus (Weil im Schönbuch Neuweiler)
	ABZWEIG_7 = 'de:08115:4789:0:4'  # eRtg Nord (Weil im Schönbuch Neuweiler)
	ABZWEIG_8 = 'de:08115:4853:0:3'  # Rtg Kayh Herrenberger Straße (Herrenberg Mönchberg)
	ABZWEIG_9 = 'de:08115:4853:0:4'  # Rtg Weingartenstraße (Herrenberg Mönchberg)
	ABZWEIG_10 = 'de:08119:5197:0:3'  # eRtg Rohrbronn (Remshalden Hebsack)
	ABZWEIG_11 = 'de:08119:5197:0:4'  # eRtg Remsbrücke (Remshalden Hebsack)
	ABZWEIG_12 = 'de:08118:5752:0:3'  # eRtg Reitsteige (Vaihingen an der Enz Aurich)
	ABZWEIG_13 = 'de:08118:5752:0:4'  # eRtg Enzbrücke (Vaihingen an der Enz Aurich)
	ABZWEIG_14 = 'de:08115:5941:0:3'  # eRtg Lehenweiler Abzweig (Grafenau Dätzingen)
	ABZWEIG_15 = 'de:08115:5941:0:4'  # eRtg Seeger (Grafenau Dätzingen)
	GERADSTETTEN_RATHAUS = 'de:08119:3702'
	GERADSTETTEN_RATHAUS_1 = 'de:08119:3702:1:3'  # Rtg Geradstetten Mitte (Remshalden Geradstetten)
	GERADSTETTEN_RATHAUS_2 = 'de:08119:3702:1:4'  # Rtg D.-Bonhoeffer-Str. (Remshalden Geradstetten)
	GERADSTETTEN_MITTE = 'de:08119:3703'
	GERADSTETTEN_MITTE_1 = 'de:08119:3703:1:3'  # Rtg Markgraben (Remshalden Geradstetten)
	GERADSTETTEN_MITTE_2 = 'de:08119:3703:1:4'  # Rtg Kelterwiesenbach/Rathaus (Remshalden Geradstetten)
	GRUNBACH_REINHOLD_MAIER_STR = 'de:08119:3704'
	GRUNBACH_REINHOLD_MAIER_STR_1 = 'de:08119:3704:1:3'  # Rtg Austr/Stgt-Weststr (Remshalden Grunbach)
	GRUNBACH_REINHOLD_MAIER_STR_2 = 'de:08119:3704:1:4'  # Rtg Grundschule/Schorndorfer Str. (Remshalden Grunbach)
	REINHOLD_MAIER_STRASSE = 'de:08119:3704'
	REINHOLD_MAIER_STRASSE_1 = 'de:08119:3704:1:3'  # Rtg Austr/Stgt-Weststr (Remshalden Grunbach)
	REINHOLD_MAIER_STRASSE_2 = 'de:08119:3704:1:4'  # Rtg Grundschule/Schorndorfer Str. (Remshalden Grunbach)
	HEBSACK_LINDENPLATZ = 'de:08119:3706'
	HEBSACK_LINDENPLATZ_1 = 'de:08119:3706:0:3'  # Rtg Rohrbronn/Winterbach (Remshalden Hebsack)
	HEBSACK_LINDENPLATZ_2 = 'de:08119:3706:0:4'  # Rtg Markgraben (Remshalden Hebsack)
	HEBSACK_MARKGRABEN = 'de:08119:3707'
	HEBSACK_MARKGRABEN_1 = 'de:08119:3707:0:3'  # Rtg Lindenplatz (Remshalden Hebsack)
	HEBSACK_MARKGRABEN_2 = 'de:08119:3707:0:4'  # Rtg Geradstetten Mitte (Remshalden Hebsack)
	MARKGRABEN = 'de:08119:3707'
	MARKGRABEN_1 = 'de:08119:3707:0:3'  # Rtg Lindenplatz (Remshalden Hebsack)
	MARKGRABEN_2 = 'de:08119:3707:0:4'  # Rtg Geradstetten Mitte (Remshalden Hebsack)
	SCHORNDORF_WELZHEIMER_STR = 'de:08119:3708'
	SCHORNDORF_WELZHEIMER_STR_1 = 'de:08119:3708:0:3'  # Rtg Schlipfweg (Schorndorf Schorndorf)
	SCHORNDORF_WELZHEIMER_STR_2 = 'de:08119:3708:0:4'  # Rtg Kreisverkehr Welzheimer Str. (Schorndorf Schorndorf)
	SCHORNDORF_WELZHEIMER_STR_3 = 'de:08119:3708:0:5'  # Rtg K.-Haussmann-Weg/Hammerschlag (Schorndorf Schorndorf)
	SCHORNDORF_WELZHEIMER_STR_4 = 'de:08119:3708:0:6'  # Rtg. R.Bosch-Straße (Schorndorf Schorndorf)
	WELZHEIMER_STRASSE_1 = 'de:08119:3708:0:3'  # Rtg Schlipfweg (Schorndorf Schorndorf)
	WELZHEIMER_STRASSE_2 = 'de:08119:3708:0:4'  # Rtg Kreisverkehr Welzheimer Str. (Schorndorf Schorndorf)
	WELZHEIMER_STRASSE_3 = 'de:08119:3708:0:5'  # Rtg K.-Haussmann-Weg/Hammerschlag (Schorndorf Schorndorf)
	WELZHEIMER_STRASSE_4 = 'de:08119:3708:0:6'  # Rtg. R.Bosch-Straße (Schorndorf Schorndorf)
	WELZHEIMER_STRASSE_5 = 'de:08119:5091:0:3'  # eRtg Haubenwasen (Alfdorf Pfahlbronn)
	WELZHEIMER_STRASSE_6 = 'de:08119:5091:0:4'  # eRtg Lorcher Straße (Alfdorf Pfahlbronn)
	WELZHEIMER_STRASSE_7 = 'de:08119:5374:0:3'  # eRtg Edeka (Weissach im Tal Unterweissach)
	WELZHEIMER_STRASSE_8 = 'de:08119:5374:0:4'  # eRtg Rathaus/Lindenplatz (Weissach im Tal Unterweissach)
	WELZHEIMER_STRASSE_9 = 'de:08136:6514:0:3'  # Rtg Marktplatz (Gschwend Gschwend)
	WELZHEIMER_STRASSE_10 = 'de:08136:6514:0:4'  # Rtg Dinglesmad (Gschwend Gschwend)
	WEILER_KIRCHE = 'de:08119:3710'
	WEILER_KIRCHE_1 = 'de:08119:3710:1:3'  # eRtg Pflegeheim (Schorndorf Weiler (Schorndorf))
	WEILER_KIRCHE_2 = 'de:08119:3710:1:4'  # eRtg Rathaus/Pfarrstraße (Schorndorf Weiler (Schorndorf))
	BEUTELSBACH = 'de:08119:3711'
	BEUTELSBACH_1 = 'de:08119:3711:1:2'  # Gleis 2 (Weinstadt Beutelsbach)
	BEUTELSBACH_2 = 'de:08119:3711:2:1'  # Gleis 1 (Weinstadt Beutelsbach)
	WINTERBACH_PFLEGEHEIM = 'de:08119:3713'
	WINTERBACH_PFLEGEHEIM_1 = 'de:08119:3713:1:3'  # Rtg Remsbrücke/Bahnunterf. (Winterbach Winterbach)
	WINTERBACH_PFLEGEHEIM_2 = 'de:08119:3713:1:4'  # Rtg Weiler/Lerchenstr. (Winterbach Winterbach)
	PFLEGEHEIM_1 = 'de:08119:3713:1:3'  # Rtg Remsbrücke/Bahnunterf. (Winterbach Winterbach)
	PFLEGEHEIM_2 = 'de:08119:3713:1:4'  # Rtg Weiler/Lerchenstr. (Winterbach Winterbach)
	PFLEGEHEIM_3 = 'de:08115:4627:0:3'  # eRtg Krankenhaus (Sindelfingen Sindelfingen)
	PFLEGEHEIM_4 = 'de:08115:4627:0:4'  # eRtg Burghaldenfriedhof (Sindelfingen Sindelfingen)
	PFLEGEHEIM_5 = 'de:08117:7798:0:H1'  # eRtg Hattenhofer Straße (Schlierbach Schlierbach)
	PFLEGEHEIM_6 = 'de:08117:7798:0:R1'  # eRtg Auchtertstraße (Schlierbach Schlierbach)
	WINTERBACH_REMSBRÜCKE = 'de:08119:3714'
	WINTERBACH_REMSBRÜCKE_1 = 'de:08119:3714:0:3'  # Rtg Pflegeheim/Bahnunterf. (Winterbach Winterbach)
	WINTERBACH_REMSBRÜCKE_2 = 'de:08119:3714:0:4'  # Rtg Hebsack/Rohrbronn (Winterbach Winterbach)
	REMSBRÜCKE = 'de:08119:3714'
	REMSBRÜCKE_1 = 'de:08119:3714:0:3'  # Rtg Pflegeheim/Bahnunterf. (Winterbach Winterbach)
	REMSBRÜCKE_2 = 'de:08119:3714:0:4'  # Rtg Hebsack/Rohrbronn (Winterbach Winterbach)
	STUTTGARTER__WESTSTRASSE = 'de:08119:3715'
	STUTTGARTER__WESTSTRASSE_1 = 'de:08119:3715:0:3'  # Rtg Reinhold-Maier-Str. (Remshalden Grunbach)
	STUTTGARTER__WESTSTRASSE_2 = 'de:08119:3715:0:4'  # Rtg Großheppach Grunbacher Str. (Remshalden Grunbach)
	GRUNBACH_STUTTGARTER__WESTSTR = 'de:08119:3715'
	GRUNBACH_STUTTGARTER__WESTSTR_1 = 'de:08119:3715:0:3'  # Rtg Reinhold-Maier-Str. (Remshalden Grunbach)
	GRUNBACH_STUTTGARTER__WESTSTR_2 = 'de:08119:3715:0:4'  # Rtg Großheppach Grunbacher Str. (Remshalden Grunbach)
	SCHORNDORF_KÜNKELINSCHULE = 'de:08119:3716:0:3'  # eRtg Schlossstraße/Burggymnasium (Schorndorf Schorndorf)
	KÜNKELINSCHULE = 'de:08119:3716:0:3'  # eRtg Schlossstraße/Burggymnasium (Schorndorf Schorndorf)
	ALTHÜTTE_SCHULE = 'de:08119:3719:0:3'  # eRtg Alle Richtungen (Althütte Althütte)
	WAIBL__A__WINNENDER_STEIGE = 'de:08119:3720:0:3'  # eRtg Birkenweg (Waiblingen Waiblingen)
	ALTE_WINNENDER_STEIGE = 'de:08119:3720:0:3'  # eRtg Birkenweg (Waiblingen Waiblingen)
	BIRKENWEG_1 = 'de:08119:3721:0:3'  # eRtg Steingrube (Waiblingen Waiblingen)
	BIRKENWEG_2 = 'de:08116:3835:0:1'  # Rtg Altbacher Hof (Altbach Altbach)
	BIRKENWEG_3 = 'de:08118:7429:0:3'  # eRtg Westendstraße (Bietigheim-Bissingen Bissingen (LB))
	BIRKENWEG_4 = 'de:08118:7429:0:4'  # eRtg Kelterstraße (Bietigheim-Bissingen Bissingen (LB))
	BIRKENWEG_5 = 'de:08117:104:0:H1'  # Rtg Neuwiesen (B466) (Geislingen an der Steige Geislingen (Steige))
	WAIBLINGEN_BIRKENWEG = 'de:08119:3721:0:3'  # eRtg Steingrube (Waiblingen Waiblingen)
	WAIBLINGEN_STEINGRUBE = 'de:08119:3722:0:3'  # eRtg Junge Weinberge (Waiblingen Waiblingen)
	STEINGRUBE = 'de:08119:3722:0:3'  # eRtg Junge Weinberge (Waiblingen Waiblingen)
	JUNGE_WEINBERGE = 'de:08119:3723:0:3'  # eRtg Schwanen (Waiblingen Waiblingen)
	WAIBLINGEN_JUNGE_WEINBERGE = 'de:08119:3723:0:3'  # eRtg Schwanen (Waiblingen Waiblingen)
	WAIBLINGEN_OBERER_ROSBERG = 'de:08119:3724:0:3'  # eRtg Buchenweg (Waiblingen Waiblingen)
	OBERER_ROSBERG = 'de:08119:3724:0:3'  # eRtg Buchenweg (Waiblingen Waiblingen)
	WAIBLINGEN_BUCHENWEG = 'de:08119:3725:0:3'  # eRtg Im Baumstückle (Waiblingen Waiblingen)
	BUCHENWEG_1 = 'de:08119:3725:0:3'  # eRtg Im Baumstückle (Waiblingen Waiblingen)
	BUCHENWEG_2 = 'de:08119:5222:0:4'  # Rtg Lindenweg (Schorndorf Schorndorf)
	LÖWENBRUNNEN = 'de:08119:3726:0:4'  # Rtg Lutherkirche (Fellbach Fellbach)
	FELLBACH_LÖWENBRUNNEN = 'de:08119:3726:0:4'  # Rtg Lutherkirche (Fellbach Fellbach)
	IM_KUSTERFELD = 'de:08119:3728'
	IM_KUSTERFELD_1 = 'de:08119:3728:0:3'  # eRtg Kuchengrund/OVR (Backnang Backnang)
	IM_KUSTERFELD_2 = 'de:08119:3728:0:4'  # Rtg Paulinenpflege (Backnang Backnang)
	BACKNANG_IM_KUSTERFELD = 'de:08119:3728'
	BACKNANG_IM_KUSTERFELD_1 = 'de:08119:3728:0:3'  # eRtg Kuchengrund/OVR (Backnang Backnang)
	BACKNANG_IM_KUSTERFELD_2 = 'de:08119:3728:0:4'  # Rtg Paulinenpflege (Backnang Backnang)
	ALTHÜTTE_EBNISEESTRASSE = 'de:08119:3729'
	ALTHÜTTE_EBNISEESTRASSE_1 = 'de:08119:3729:0:3'  # eRtg Rathaus (Althütte Althütte)
	ALTHÜTTE_EBNISEESTRASSE_2 = 'de:08119:3729:0:4'  # eRtg Voggenhof Abzweig (Althütte Althütte)
	VORDERBÜCHELBERG = 'de:08119:3730'
	VORDERBÜCHELBERG_1 = 'de:08119:3730:0:3'  # eRtg Vorderbüchelberg Abzweig (Spiegelberg Spiegelberg)
	VORDERBÜCHELBERG_2 = 'de:08119:3730:0:4'  # Rtg Denteltal (Spiegelberg Spiegelberg)
	SPIEGELB__VORDERBÜCHELBERG = 'de:08119:3730'
	SPIEGELB__VORDERBÜCHELBERG_1 = 'de:08119:3730:0:3'  # eRtg Vorderbüchelberg Abzweig (Spiegelberg Spiegelberg)
	SPIEGELB__VORDERBÜCHELBERG_2 = 'de:08119:3730:0:4'  # Rtg Denteltal (Spiegelberg Spiegelberg)
	BACKNANG_TAUSSCHULE = 'de:08119:3731:0:3'  # eRtg Weidenweg (Backnang Backnang)
	TAUSSCHULE = 'de:08119:3731:0:3'  # eRtg Weidenweg (Backnang Backnang)
	LORTZINGSTRASSE = 'de:08119:3735:0:3'  # Rtg Rommelsh. Bf (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_LORTZINGSTRASSE = 'de:08119:3735:0:3'  # Rtg Rommelsh. Bf (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_WAGNERSTRASSE = 'de:08119:3736:0:3'  # Rtg. Lortzingstr. (Kernen im Remstal Rommelshausen)
	WAGNERSTRASSE = 'de:08119:3736:0:3'  # Rtg. Lortzingstr. (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_FRIEDHOF = 'de:08119:3737:0:3'  # Rtg. Rommelsh. Rathaus (Kernen im Remstal Rommelshausen)
	WIESENSTRASSE = 'de:08119:3738:0:3'  # Rtg. Hofgartenstr. (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_WIESENSTRASSE = 'de:08119:3738:0:3'  # Rtg. Hofgartenstr. (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_SCHLIENZ_TOURS = 'de:08119:3739:0:4'  # Rtg. Rudolf-Diesel-Str. (Kernen im Remstal Rommelshausen)
	SCHLIENZ_TOURS = 'de:08119:3739:0:4'  # Rtg. Rudolf-Diesel-Str. (Kernen im Remstal Rommelshausen)
	ROMMELSH__RUDOLF_DIESEL_STR = 'de:08119:3740:0:4'  # Rtg. Lilienstr. (Kernen im Remstal Rommelshausen)
	RUDOLF_DIESEL_STRASSE = 'de:08119:3740:0:4'  # Rtg. Lilienstr. (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_KELTERSTRASSE = 'de:08119:3741'
	ROMMELSHAUSEN_KELTERSTRASSE_1 = 'de:08119:3741:0:3'  # Rtg. KelterStadion (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_KELTERSTRASSE_2 = 'de:08119:3741:0:4'  # Rtg. Seestr. (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_KELTER_STADION = 'de:08119:3742:0:3'  # Rtg. Kelterstr. (Kernen im Remstal Rommelshausen)
	KELTER_STADION = 'de:08119:3742:0:3'  # Rtg. Kelterstr. (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_SEESTRASSE = 'de:08119:3743:0:3'  # eRtg Schafstraße (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_RATHAUS = 'de:08119:3744:0:3'  # Rtg Rumold-Realschule (Kernen im Remstal Rommelshausen)
	GRUNBACH_AUSTRASSE = 'de:08119:3745'
	GRUNBACH_AUSTRASSE_1 = 'de:08119:3745:0:3'  # Rtg Grunbach Bf (Remshalden Grunbach)
	GRUNBACH_AUSTRASSE_2 = 'de:08119:3745:0:4'  # Rtg Reinhold-Maier-Str. (Remshalden Grunbach)
	AUSTRASSE_1 = 'de:08119:3745:0:3'  # Rtg Grunbach Bf (Remshalden Grunbach)
	AUSTRASSE_2 = 'de:08119:3745:0:4'  # Rtg Reinhold-Maier-Str. (Remshalden Grunbach)
	AUSTRASSE_3 = 'de:08116:3947:0:3'  # eRtg Marktplatz (Kirchheim unter Teck Kirchheim (T))
	AUSTRASSE_4 = 'de:08116:3947:0:4'  # eRtg Saarstraße (Kirchheim unter Teck Kirchheim (T))
	AUSTRASSE_5 = 'de:08118:5746:0:3'  # eRtg Salzäcker (Vaihingen an der Enz Vaihingen (E))
	AUSTRASSE_6 = 'de:08116:6864:0:3'  # Rtg Friedhof (Wendlingen am Neckar Wendlingen (N))
	GERADSTETTEN_KELTERWIESENBACH = 'de:08119:3746'
	GERADSTETTEN_KELTERWIESENBACH_1 = 'de:08119:3746:0:3'  # Rtg Hauptschule (Remshalden Geradstetten)
	GERADSTETTEN_KELTERWIESENBACH_2 = 'de:08119:3746:0:4'  # Rtg Bahnüberführung (Remshalden Geradstetten)
	KELTERWIESENBACH = 'de:08119:3746'
	KELTERWIESENBACH_1 = 'de:08119:3746:0:3'  # Rtg Hauptschule (Remshalden Geradstetten)
	KELTERWIESENBACH_2 = 'de:08119:3746:0:4'  # Rtg Bahnüberführung (Remshalden Geradstetten)
	BAHNÜBERFÜHRUNG = 'de:08119:3748'
	BAHNÜBERFÜHRUNG_1 = 'de:08119:3748:1:3'  # Rtg Etyeker Platz (Remshalden Geradstetten)
	BAHNÜBERFÜHRUNG_2 = 'de:08119:3748:1:4'  # Rtg D.-Bonhoeffer-Str./Kelterwiesenbach (Remshalden Geradstetten)
	GERADST__BAHNÜBERFÜHRUNG = 'de:08119:3748'
	GERADST__BAHNÜBERFÜHRUNG_1 = 'de:08119:3748:1:3'  # Rtg Etyeker Platz (Remshalden Geradstetten)
	GERADST__BAHNÜBERFÜHRUNG_2 = 'de:08119:3748:1:4'  # Rtg D.-Bonhoeffer-Str./Kelterwiesenbach (Remshalden Geradstetten)
	GRUNBACH_KELTERSTRASSE = 'de:08119:3749'
	GRUNBACH_KELTERSTRASSE_1 = 'de:08119:3749:0:3'  # Rtg Reinhold-Maier-Str. (Remshalden Grunbach)
	GRUNBACH_KELTERSTRASSE_2 = 'de:08119:3749:0:4'  # Rtg Kirchplatz (Remshalden Grunbach)
	GRUNBACH_SCHORNDORFER_STR = 'de:08119:3752'
	GRUNBACH_SCHORNDORFER_STR_1 = 'de:08119:3752:0:3'  # Rtg Geradstetten D.-Bonhoeffer-Str. (Remshalden Grunbach)
	GRUNBACH_SCHORNDORFER_STR_2 = 'de:08119:3752:0:4'  # Rtg Grundschule/Reinhold-Maier-Str. (Remshalden Grunbach)
	SCHORNDORFER_STRASSE_1 = 'de:08119:3752:0:3'  # Rtg Geradstetten D.-Bonhoeffer-Str. (Remshalden Grunbach)
	SCHORNDORFER_STRASSE_2 = 'de:08119:3752:0:4'  # Rtg Grundschule/Reinhold-Maier-Str. (Remshalden Grunbach)
	SCHORNDORFER_STRASSE_3 = 'de:08116:4201:1:3'  # eRtg Zimmerbachstr./Keplerstr. (Esslingen am Neckar Oberesslingen)
	SCHORNDORFER_STRASSE_4 = 'de:08119:5111:0:3'  # Rtg Miedelsbach (Steinenberg) (Rudersberg Steinenberg)
	SCHORNDORFER_STRASSE_5 = 'de:08119:5111:0:4'  # eRtg Michelauer Str./Turnhalle/Ortsmitte (Rudersberg Steinenberg)
	SCHORNDORFER_STRASSE_6 = 'de:08116:6499:0:3'  # eRtg Reichenbachstraße (Reichenbach an der Fils Reichenbach (F))
	SCHORNDORFER_STRASSE_7 = 'de:08116:6499:0:4'  # eRtg Blumenstraße (Reichenbach an der Fils Reichenbach (F))
	SCHORNDORFER_STRASSE_8 = 'de:08119:6694:0:3'  # eRtg Hauptstraße (Urbach Urbach)
	SCHORNDORFER_STRASSE_9 = 'de:08119:6694:0:4'  # eRtg Wasenstraße (Urbach Urbach)
	GERADSTETTEN_D__BONHOEFFER_STR = 'de:08119:3753'
	GERADSTETTEN_D__BONHOEFFER_STR_1 = 'de:08119:3753:0:3'  # Rtg Rathaus (Remshalden Geradstetten)
	GERADSTETTEN_D__BONHOEFFER_STR_2 = 'de:08119:3753:0:4'  # Rtg Grunbach Schorndorfer Str. (Remshalden Geradstetten)
	DIETRICH_BONHOEFFER_STRASSE = 'de:08119:3753'
	DIETRICH_BONHOEFFER_STRASSE_1 = 'de:08119:3753:0:3'  # Rtg Rathaus (Remshalden Geradstetten)
	DIETRICH_BONHOEFFER_STRASSE_2 = 'de:08119:3753:0:4'  # Rtg Grunbach Schorndorfer Str. (Remshalden Geradstetten)
	WEILER_AM_BRONNBACH = 'de:08119:3754'
	WEILER_AM_BRONNBACH_1 = 'de:08119:3754:0:3'  # eRtg Lerchenstraße (Schorndorf Weiler (Schorndorf))
	WEILER_AM_BRONNBACH_2 = 'de:08119:3754:0:4'  # eRtg Kirche (Schorndorf Weiler (Schorndorf))
	AM_BRONNBACH = 'de:08119:3754'
	AM_BRONNBACH_1 = 'de:08119:3754:0:3'  # eRtg Lerchenstraße (Schorndorf Weiler (Schorndorf))
	AM_BRONNBACH_2 = 'de:08119:3754:0:4'  # eRtg Kirche (Schorndorf Weiler (Schorndorf))
	HOFGARTENSTR = 'de:08119:3755:0:3'  # Rtg Ost (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_HOFGARTENSTR = 'de:08119:3755:0:3'  # Rtg Ost (Kernen im Remstal Rommelshausen)
	WINNENDEN_ADELSBACH = 'de:08119:3758'
	WINNENDEN_ADELSBACH_1 = 'de:08119:3758:0:3'  # Rtg Hertm.weiler Heckenweg (Winnenden Winnenden)
	WINNENDEN_ADELSBACH_2 = 'de:08119:3758:0:4'  # Rtg Rotweg (Winnenden Winnenden)
	ADELSBACH = 'de:08119:3758'
	ADELSBACH_1 = 'de:08119:3758:0:3'  # Rtg Hertm.weiler Heckenweg (Winnenden Winnenden)
	ADELSBACH_2 = 'de:08119:3758:0:4'  # Rtg Rotweg (Winnenden Winnenden)
	SCHORNDORF_SCHLICHTENER_STRASSE = 'de:08119:3759:0:1'  # Rtg Künkelinhalle (Schorndorf Schorndorf)
	SCHLICHTENER_STRASSE = 'de:08119:3759:0:1'  # Rtg Künkelinhalle (Schorndorf Schorndorf)
	BURGUNDERSTRASSE_1 = 'de:08119:3761:0:3'  # Ankunft 222 (Weinstadt Beutelsbach)
	BURGUNDERSTRASSE_2 = 'de:08116:4011:0:3'  # Rtg Cannstatter Str. / Esslingen VHS (Esslingen am Neckar Mettingen)
	BURGUNDERSTRASSE_3 = 'de:08116:4011:0:4'  # Rtg. Mettingen Bf. (Schenkenbergstr.) (Esslingen am Neckar Mettingen)
	BEUTELSBACH_BURGUNDERSTRASSE = 'de:08119:3761:0:3'  # Ankunft 222 (Weinstadt Beutelsbach)
	BEUTELSBACH_URBANSTRASSE = 'de:08119:3762:0:4'  # Rtg Nordhaldenstr. (Weinstadt Beutelsbach)
	URBANSTRASSE = 'de:08119:3762:0:4'  # Rtg Nordhaldenstr. (Weinstadt Beutelsbach)
	NORDHALDENSTRASSE = 'de:08119:3763'
	NORDHALDENSTRASSE_1 = 'de:08119:3763:0:3'  # Rtg Weinsteige (Weinstadt Beutelsbach)
	NORDHALDENSTRASSE_2 = 'de:08119:3763:0:4'  # Rtg. Kaiserstr. (Weinstadt Beutelsbach)
	BEUTELSBACH_NORDHALDENSTRASSE = 'de:08119:3763'
	BEUTELSBACH_NORDHALDENSTRASSE_1 = 'de:08119:3763:0:3'  # Rtg Weinsteige (Weinstadt Beutelsbach)
	BEUTELSBACH_NORDHALDENSTRASSE_2 = 'de:08119:3763:0:4'  # Rtg. Kaiserstr. (Weinstadt Beutelsbach)
	BEUTELSBACH_KAISERSTRASSE = 'de:08119:3764'
	BEUTELSBACH_KAISERSTRASSE_1 = 'de:08119:3764:0:3'  # Rtg. Nordhaldenstr. (Weinstadt Beutelsbach)
	BEUTELSBACH_KAISERSTRASSE_2 = 'de:08119:3764:0:4'  # Rtg. Poststr. (Weinstadt Beutelsbach)
	BEUTELSBACH_POSTSTRASSE = 'de:08119:3765'
	BEUTELSBACH_POSTSTRASSE_1 = 'de:08119:3765:0:3'  # Rtg. Kaiserstr. (Weinstadt Beutelsbach)
	BEUTELSBACH_POSTSTRASSE_2 = 'de:08119:3765:0:4'  # Rtg. Beim Kreise (Weinstadt Beutelsbach)
	POSTSTRASSE = 'de:08119:3765'
	POSTSTRASSE_1 = 'de:08119:3765:0:3'  # Rtg. Kaiserstr. (Weinstadt Beutelsbach)
	POSTSTRASSE_2 = 'de:08119:3765:0:4'  # Rtg. Beim Kreise (Weinstadt Beutelsbach)
	ENDERSBACH_RATHAUS = 'de:08119:3768'
	ENDERSBACH_RATHAUS_1 = 'de:08119:3768:0:3'  # Rtg Ortsmitte (Weinstadt Endersbach)
	ENDERSBACH_RATHAUS_2 = 'de:08119:3768:0:4'  # Rtg Pestalozzistr. (Weinstadt Endersbach)
	PESTALOZZISTRASSE = 'de:08119:3769'
	PESTALOZZISTRASSE_1 = 'de:08119:3769:0:3'  # Rtg. Endersb. Rathaus (Weinstadt Endersbach)
	PESTALOZZISTRASSE_2 = 'de:08119:3769:0:4'  # Rtg. Endersbach Bf (Weinstadt Endersbach)
	ENDERSBACH_PESTALOZZISTRASSE = 'de:08119:3769'
	ENDERSBACH_PESTALOZZISTRASSE_1 = 'de:08119:3769:0:3'  # Rtg. Endersb. Rathaus (Weinstadt Endersbach)
	ENDERSBACH_PESTALOZZISTRASSE_2 = 'de:08119:3769:0:4'  # Rtg. Endersbach Bf (Weinstadt Endersbach)
	BEUTELSBACH_WEINSTEIGE = 'de:08119:3770:0:3'  # Rtg Burgunderstr. (Weinstadt Beutelsbach)
	BÜHLÄCKERSTRASSE = 'de:08119:3771'
	BÜHLÄCKERSTRASSE_1 = 'de:08119:3771:0:1'  # Rtg Frauenländerstr (Kernen im Remstal Stetten (i. R.))
	BÜHLÄCKERSTRASSE_2 = 'de:08119:3771:0:2'  # Rtg. Endersbacher Str. (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__BÜHLÄCKERSTR = 'de:08119:3771'
	STETTEN_I__R__BÜHLÄCKERSTR_1 = 'de:08119:3771:0:1'  # Rtg Frauenländerstr (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__BÜHLÄCKERSTR_2 = 'de:08119:3771:0:2'  # Rtg. Endersbacher Str. (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__FREIBAD = 'de:08119:3773'
	STETTEN_I__R__FREIBAD_1 = 'de:08119:3773:0:3'  # Rtg Grundschule (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__FREIBAD_2 = 'de:08119:3773:0:4'  # Rtg Frauenländerstr. (Kernen im Remstal Stetten (i. R.))
	ENGELBERG_WALDORFSCHULE = 'de:08119:3774:0:3'  # Linien 217/245 (Winterbach Winterbach)
	WINTERB__ENGELB__WALDORFSCHULE = 'de:08119:3774:0:3'  # Linien 217/245 (Winterbach Winterbach)
	ALTBACH_ALTBACHER_HOF = 'de:08116:3800:0:3'  # eRtg Feuerwehr (Altbach Altbach)
	ALTBACHER_HOF = 'de:08116:3800:0:3'  # eRtg Feuerwehr (Altbach Altbach)
	ALTBACH_IM_BRAUNKIEL = 'de:08116:3801:0:3'  # eRtg Altbacher Hof (Altbach Altbach)
	IM_BRAUNKIEL = 'de:08116:3801:0:3'  # eRtg Altbacher Hof (Altbach Altbach)
	LOSBURGSTRASSE = 'de:08116:3802:0:3'  # eRtg Stauferstraße (Altbach Altbach)
	ALTBACH_LOSBURGSTRASSE = 'de:08116:3802:0:3'  # eRtg Stauferstraße (Altbach Altbach)
	ALTBACH_RATHAUS = 'de:08116:3803:1:3'  # Rtg Haldenrainweg/Losburgstr. (Altbach Altbach)
	ALTBACH_GEMEINDEHALLE = 'de:08116:3804'
	ALTBACH_GEMEINDEHALLE_1 = 'de:08116:3804:0:3'  # eRtg Pfostenberg (Altbach Altbach)
	ALTBACH_GEMEINDEHALLE_2 = 'de:08116:3804:0:4'  # eRtg Haldenrainweg (Altbach Altbach)
	ESSLINGEN__N__CHARLOTTENPLATZ = 'de:08116:3805'
	ESSLINGEN__N__CHARLOTTENPLATZ_1 = 'de:08116:3805:0:3'  # eRtg Indexstraße/Bahnhof/Zell (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__CHARLOTTENPLATZ_2 = 'de:08116:3805:0:4'  # eRtg Schwimmbad (Esslingen am Neckar Esslingen (N))
	CHARLOTTENPLATZ_1 = 'de:08116:3805:0:3'  # eRtg Indexstraße/Bahnhof/Zell (Esslingen am Neckar Esslingen (N))
	CHARLOTTENPLATZ_2 = 'de:08116:3805:0:4'  # eRtg Schwimmbad (Esslingen am Neckar Esslingen (N))
	CHARLOTTENPLATZ_3 = 'de:08111:6075:1:1'  # Rtg Rathaus (Stuttgart Stuttgart)
	CHARLOTTENPLATZ_4 = 'de:08111:6075:2:2'  # Rtg Staatsgalerie (Stuttgart Stuttgart)
	CHARLOTTENPLATZ_5 = 'de:08111:6075:3:7'  # Rtg Olgaeck (Stuttgart Stuttgart)
	CHARLOTTENPLATZ_6 = 'de:08111:6075:4:8'  # Rtg Schlossplatz (Stuttgart Stuttgart)
	CHARLOTTENPLATZ_7 = 'de:08111:6075:5:1'  # Linie 44 Rtg Schlossplatz/Dorotheenstr. (Stuttgart Stuttgart)
	CHARLOTTENPLATZ_8 = 'de:08111:6075:7:3'  # Linie 43 Rtg Olgaeck (Stuttgart Stuttgart)
	CHARLOTTENPLATZ_9 = 'de:08111:6075:8:5'  # Linie 42 Rtg Schlossplatz (Stuttgart Stuttgart)
	CHARLOTTENPLATZ_10 = 'de:08111:6075:9:4'  # Linie 42 Rtg Olgaeck (Stuttgart Stuttgart)
	REICHENBACHER_STRASSE = 'de:08116:3806'
	REICHENBACHER_STRASSE_1 = 'de:08116:3806:0:3'  # eRtg Kreissparkasse (Baltmannsweiler Baltmannsweiler)
	REICHENBACHER_STRASSE_2 = 'de:08116:3806:0:4'  # Rtg Reichenbach (Baltmannsweiler Baltmannsweiler)
	BALTMANNSW__REICHENBACHER_STR = 'de:08116:3806'
	BALTMANNSW__REICHENBACHER_STR_1 = 'de:08116:3806:0:3'  # eRtg Kreissparkasse (Baltmannsweiler Baltmannsweiler)
	BALTMANNSW__REICHENBACHER_STR_2 = 'de:08116:3806:0:4'  # Rtg Reichenbach (Baltmannsweiler Baltmannsweiler)
	ALTE_HEUSTEIGE = 'de:08116:3807'
	ALTE_HEUSTEIGE_1 = 'de:08116:3807:0:3'  # eRtg Zell (Esslingen am Neckar Zell (Esslingen))
	ALTE_HEUSTEIGE_2 = 'de:08116:3807:0:4'  # eRtg Keplerstr./Gartenstadt/Weiherstr. (Esslingen am Neckar Zell (Esslingen))
	ZELL_ALTE_HEUSTEIGE = 'de:08116:3807'
	ZELL_ALTE_HEUSTEIGE_1 = 'de:08116:3807:0:3'  # eRtg Zell (Esslingen am Neckar Zell (Esslingen))
	ZELL_ALTE_HEUSTEIGE_2 = 'de:08116:3807:0:4'  # eRtg Keplerstr./Gartenstadt/Weiherstr. (Esslingen am Neckar Zell (Esslingen))
	BALTMANNSWEILER_RATHAUS = 'de:08116:3808:0:3'  # Rtg Kreissparkasse (Baltmannsweiler Baltmannsweiler)
	OBERESSLINGEN_INDEXSTR_BAHNHOF = 'de:08116:3810'
	OBERESSLINGEN_INDEXSTR_BAHNHOF_1 = 'de:08116:3810:0:3'  # Rtg Rosenau (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_INDEXSTR_BAHNHOF_2 = 'de:08116:3810:0:4'  # Rtg Herderstr. (Esslingen am Neckar Oberesslingen)
	INDEXSTRASSE_BAHNHOF = 'de:08116:3810'
	INDEXSTRASSE_BAHNHOF_1 = 'de:08116:3810:0:3'  # Rtg Rosenau (Esslingen am Neckar Oberesslingen)
	INDEXSTRASSE_BAHNHOF_2 = 'de:08116:3810:0:4'  # Rtg Herderstr. (Esslingen am Neckar Oberesslingen)
	THOMASHARDTER_STRASSE = 'de:08116:3812'
	THOMASHARDTER_STRASSE_1 = 'de:08116:3812:0:3'  # eRtg Bürgerzentrum (Lichtenwald Hegenlohe)
	THOMASHARDTER_STRASSE_2 = 'de:08116:3812:0:4'  # eRtg Kirche (Lichtenwald Hegenlohe)
	HEGENLOHE_THOMASHARDTER_STRASSE = 'de:08116:3812'
	HEGENLOHE_THOMASHARDTER_STRASSE_1 = 'de:08116:3812:0:3'  # eRtg Bürgerzentrum (Lichtenwald Hegenlohe)
	HEGENLOHE_THOMASHARDTER_STRASSE_2 = 'de:08116:3812:0:4'  # eRtg Kirche (Lichtenwald Hegenlohe)
	STAUFERSTRASSE_1 = 'de:08116:3816:0:3'  # eRtg Im Braunkiel (Altbach Altbach)
	STAUFERSTRASSE_2 = 'de:08119:5158:0:3'  # eRtg Alfred-Diebold-Weg (Waiblingen Waiblingen)
	STAUFERSTRASSE_3 = 'de:08119:5425:0:3'  # Rtg Fellbach Bf (Fellbach Schmiden)
	STAUFERSTRASSE_4 = 'de:08119:5425:0:4'  # Rtg Karolingerstr. (Fellbach Schmiden)
	ALTBACH_STAUFERSTRASSE = 'de:08116:3816:0:3'  # eRtg Im Braunkiel (Altbach Altbach)
	ALTBACH_FEUERWEHR = 'de:08116:3817'
	ALTBACH_FEUERWEHR_1 = 'de:08116:3817:0:3'  # eRtg Rathaus (Altbach Altbach)
	ALTBACH_FEUERWEHR_2 = 'de:08116:3817:0:4'  # eRtg Badstraße (Altbach Altbach)
	HALDENRAINWEG = 'de:08116:3818:0:3'  # eRtg Losburgstraße (Altbach Altbach)
	ALTBACH_HALDENRAINWEG = 'de:08116:3818:0:3'  # eRtg Losburgstraße (Altbach Altbach)
	NECKARTENZL__ROBERT_BOSCH_STR = 'de:08116:3819'
	NECKARTENZL__ROBERT_BOSCH_STR_1 = 'de:08116:3819:0:3'  # Rtg Altenried Rathaus (Neckartenzlingen Neckartenzlingen)
	NECKARTENZL__ROBERT_BOSCH_STR_2 = 'de:08116:3819:0:4'  # Rtg Spitzacker/Marktplatz (Neckartenzlingen Neckartenzlingen)
	KOHLBERG_TISCHARDTER_STRASSE = 'de:08116:3820'
	KOHLBERG_TISCHARDTER_STRASSE_1 = 'de:08116:3820:0:3'  # eRtg Tischardt Rathaus (Kohlberg Kohlberg)
	KOHLBERG_TISCHARDTER_STRASSE_2 = 'de:08116:3820:0:4'  # eRtg Kohlberg Rathaus (Kohlberg Kohlberg)
	KOHLBERG_METZINGER_STR = 'de:08116:3821'
	KOHLBERG_METZINGER_STR_1 = 'de:08116:3821:0:3'  # eRtg Linde (Kohlberg Kohlberg)
	KOHLBERG_METZINGER_STR_2 = 'de:08116:3821:0:4'  # eRtg Rathaus (Kohlberg Kohlberg)
	METZINGER_STRASSE_1 = 'de:08116:3821:0:3'  # eRtg Linde (Kohlberg Kohlberg)
	METZINGER_STRASSE_2 = 'de:08116:3821:0:4'  # eRtg Rathaus (Kohlberg Kohlberg)
	METZINGER_STRASSE_3 = 'de:08116:4305:0:3'  # eRtg Kroatenhof (Nürtingen Nürtingen)
	METZINGER_STRASSE_4 = 'de:08116:4305:0:4'  # eRtg Großbettlingen/Humpfentalstr/Verdis (Nürtingen Nürtingen)
	METZINGER_STRASSE_5 = 'de:08116:4309:0:3'  # eRtg Auwiesenschule (Neckartenzlingen Neckartenzlingen)
	METZINGER_STRASSE_6 = 'de:08116:4309:0:4'  # eRtg Marktplatz (Neckartenzlingen Neckartenzlingen)
	METZINGER_STRASSE_7 = 'de:08116:5798:0:3'  # Rtg Kreuzäcker/Reußensteinweg (Filderstadt Bonlanden)
	METZINGER_STRASSE_8 = 'de:08116:5798:0:4'  # Rtg Marktstr. (Filderstadt Bonlanden)
	METZINGER_STRASSE_9 = 'de:08116:6956:0:3'  # eRtg Albstraße (Großbettlingen Großbettlingen)
	NATURSCHUTZZENTRUM = 'de:08116:3822'
	NATURSCHUTZZENTRUM_1 = 'de:08116:3822:0:3'  # Rtg Torfgrube (Lenningen Schopfloch (Lenningen))
	NATURSCHUTZZENTRUM_2 = 'de:08116:3822:0:4'  # Rtg Schopfloch (Lenningen Schopfloch (Lenningen))
	SCHOPFLOCH_NATURSCHUTZZENTRUM = 'de:08116:3822'
	SCHOPFLOCH_NATURSCHUTZZENTRUM_1 = 'de:08116:3822:0:3'  # Rtg Torfgrube (Lenningen Schopfloch (Lenningen))
	SCHOPFLOCH_NATURSCHUTZZENTRUM_2 = 'de:08116:3822:0:4'  # Rtg Schopfloch (Lenningen Schopfloch (Lenningen))
	SCHOPFLOCH_UNTERM_GREUT = 'de:08116:3823:0:3'  # eRtg Alle Richtungen (Lenningen Schopfloch (Lenningen))
	UNTERM_GREUT = 'de:08116:3823:0:3'  # eRtg Alle Richtungen (Lenningen Schopfloch (Lenningen))
	SCHOPFLOCH_TORFGRUBE = 'de:08116:3824:0:3'  # eRtg Naturschutzzentrum (Lenningen Schopfloch (Lenningen))
	TORFGRUBE = 'de:08116:3824:0:3'  # eRtg Naturschutzzentrum (Lenningen Schopfloch (Lenningen))
	OBERBOIHINGEN_TACHENHÄUSERSTR = 'de:08116:3825:0:3'  # Rtg Stattmannstr./Kreissparkasse (Oberboihingen Oberboihingen)
	TACHENHÄUSERSTRASSE = 'de:08116:3825:0:3'  # Rtg Stattmannstr./Kreissparkasse (Oberboihingen Oberboihingen)
	WENDLINGEN__N__GOETHESTR = 'de:08116:3826:0:3'  # Rtg Kirchheimer Str. (Wendlingen am Neckar Wendlingen (N))
	WENDL__N__KIRCHHEIMER_STR = 'de:08116:3827:0:3'  # Rtg Eugenstr. (Wendlingen am Neckar Wendlingen (N))
	WENDL__N__WILHELM_RAABE_STR = 'de:08116:3829:0:3'  # Rtg Ludwig-Finkh-Str. (Wendlingen am Neckar Wendlingen (N))
	WILHELM_RAABE_STRASSE = 'de:08116:3829:0:3'  # Rtg Ludwig-Finkh-Str. (Wendlingen am Neckar Wendlingen (N))
	LUDWIG_FINCKH_STRASSE = 'de:08116:3830:0:3'  # Rtg Fabrikstr. (Wendlingen am Neckar Wendlingen (N))
	WENDL__N__LUDWIG_FINCKH_STR = 'de:08116:3830:0:3'  # Rtg Fabrikstr. (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N__FABRIKSTR = 'de:08116:3831:0:3'  # Rtg ZOB (Wendlingen am Neckar Wendlingen (N))
	FABRIKSTRASSE_1 = 'de:08116:3831:0:3'  # Rtg ZOB (Wendlingen am Neckar Wendlingen (N))
	FABRIKSTRASSE_2 = 'de:08116:4339:0:3'  # eRtg Haldenstraße (Kirchheim unter Teck Ötlingen)
	FABRIKSTRASSE_3 = 'de:08116:4339:0:4'  # eRtg Hegelstraße (Kirchheim unter Teck Ötlingen)
	FABRIKSTRASSE_4 = 'de:08116:4406:0:3'  # eRtg Mitte (Neidlingen Neidlingen)
	FABRIKSTRASSE_5 = 'de:08116:4406:0:4'  # eRtg Weilheim (Neidlingen Neidlingen)
	FABRIKSTRASSE_6 = 'de:08116:4412:0:3'  # eRtg Wilhelmstraße (Bissingen an der Teck Bissingen an der Teck (T))
	FABRIKSTRASSE_7 = 'de:08116:4412:0:4'  # eRtg Gartenstraße (Bissingen an der Teck Bissingen an der Teck (T))
	FABRIKSTRASSE_8 = 'de:08117:2036:0:H1'  # Rtg Bleicherei (Uhingen Uhingen)
	WENDLINGEN__N__KOLPINGSTR = 'de:08116:3832:0:3'  # Rtg Taläckerstr. (Wendlingen am Neckar Wendlingen (N))
	KOLPINGSTRASSE = 'de:08116:3832:0:3'  # Rtg Taläckerstr. (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N__TALÄCKERSTR = 'de:08116:3833:0:3'  # Rtg ZOB (Wendlingen am Neckar Wendlingen (N))
	TALÄCKERSTRASSE = 'de:08116:3833:0:3'  # Rtg ZOB (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N__BAHNHOFSTR = 'de:08116:3834:0:1'  # Rtg Schützenstr. (Wendlingen am Neckar Wendlingen (N))
	ALTBACH_BIRKENWEG = 'de:08116:3835:0:1'  # Rtg Altbacher Hof (Altbach Altbach)
	ZELL_STEINBEISSTRASSE = 'de:08116:3850:0:1'  # Rtg Alleen/Max-Planck/Röntgenstr. (Esslingen am Neckar Zell (Esslingen))
	ZELL_EGERT = 'de:08116:3851:0:3'  # Rtg Zell Bf (Esslingen am Neckar Zell (Esslingen))
	EGERT = 'de:08116:3851:0:3'  # Rtg Zell Bf (Esslingen am Neckar Zell (Esslingen))
	BILDUNGSZENTRUM_1 = 'de:08116:3893:0:3'  # eRtg Limburgschule/Bissinger Straße (Weilheim an der Teck Weilheim (T))
	BILDUNGSZENTRUM_2 = 'de:08116:3893:0:4'  # eRtg Kreissparkasse/Brunnenstraße (Weilheim an der Teck Weilheim (T))
	BILDUNGSZENTRUM_3 = 'de:08118:5860:0:3'  # eRtg alle Richtungen (Markgröningen Markgröningen)
	BILDUNGSZENTRUM_4 = 'de:08119:5995:0:3'  # eRtg Alle Richtungen (Weissach im Tal Cottenweiler)
	WEILHEIM__T__BILDUNGSZENTRUM = 'de:08116:3893'
	WEILHEIM__T__BILDUNGSZENTRUM_1 = 'de:08116:3893:0:3'  # eRtg Limburgschule/Bissinger Straße (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__BILDUNGSZENTRUM_2 = 'de:08116:3893:0:4'  # eRtg Kreissparkasse/Brunnenstraße (Weilheim an der Teck Weilheim (T))
	MARKTPLATZ_1 = 'de:08116:3894:0:3'  # Rtg Kirchheimer Str./Hegelstr/Neidlinge (Weilheim an der Teck Weilheim (T))
	MARKTPLATZ_2 = 'de:08116:3894:0:4'  # eRtg Schluchtweg (Weilheim an der Teck Weilheim (T))
	MARKTPLATZ_3 = 'de:08116:3894:0:5'  # Rtg Neidlinger Straße (Weilheim an der Teck Weilheim (T))
	MARKTPLATZ_4 = 'de:08116:4034:1:3'  # Rtg Ost (Esslingen am Neckar Esslingen (N))
	MARKTPLATZ_5 = 'de:08116:4034:1:4'  # Rtg West (Esslingen am Neckar Esslingen (N))
	MARKTPLATZ_6 = 'de:08116:4213:1:4'  # eRtg Kirchheim (T) ZOB/K.-W.-Halle (Kirchheim unter Teck Kirchheim (T))
	MARKTPLATZ_7 = 'de:08116:4486:0:3'  # Rtg Auwiesen/Rathaus/Metzinger Str. (Neckartenzlingen Neckartenzlingen)
	MARKTPLATZ_8 = 'de:08116:4486:0:4'  # Rtg Spitzacker (Neckartenzlingen Neckartenzlingen)
	MARKTPLATZ_9 = 'de:08115:4534:1:4'  # Südseite (Weissach Weissach (BB))
	MARKTPLATZ_10 = 'de:08115:4534:2:3'  # Marktseite (Weissach Weissach (BB))
	MARKTPLATZ_11 = 'de:08119:4905:1:3'  # Marktplatz (Sulzbach an der Murr Sulzbach (M))
	MARKTPLATZ_12 = 'de:08119:4905:1:4'  # Marktplatz (Sulzbach an der Murr Sulzbach (M))
	MARKTPLATZ_13 = 'de:08119:4905:1:5'  # Marktplatz (Sulzbach an der Murr Sulzbach (M))
	MARKTPLATZ_14 = 'de:08119:4905:1:6'  # Marktplatz (Sulzbach an der Murr Sulzbach (M))
	MARKTPLATZ_15 = 'de:08119:4925:1:3'  # eRtg Alte Straße (Murrhardt Fornsbach)
	MARKTPLATZ_16 = 'de:08119:4925:1:4'  # eRtg Kindergarten (Murrhardt Fornsbach)
	MARKTPLATZ_17 = 'de:08119:5099:0:3'  # eRtg alle Richtungen (Alfdorf Alfdorf)
	MARKTPLATZ_18 = 'de:08119:5119:0:3'  # Rtg Rudersberg Nord/Brühlstr. (Rudersberg Rudersberg)
	MARKTPLATZ_19 = 'de:08119:5119:0:4'  # Rtg Rudersberg Bf (Rudersberg Rudersberg)
	MARKTPLATZ_20 = 'de:08118:5738:0:3'  # eRtg Steige (Vaihingen an der Enz Horrheim)
	MARKTPLATZ_21 = 'de:08118:5738:0:4'  # eRtg Feuerwehrhaus (Vaihingen an der Enz Horrheim)
	MARKTPLATZ_22 = 'de:08117:1106:0:H1'  # Rtg Schw. Gmünd (Wäschenbeuren Wäschenbeuren)
	MARKTPLATZ_23 = 'de:08117:1106:0:R1'  # Rtg Göppingen (Wäschenbeuren Wäschenbeuren)
	MARKTPLATZ_24 = 'de:08117:3002:0:H1'  # Rtg Bleichstr. (Göppingen Göppingen)
	MARKTPLATZ_25 = 'de:08117:6004:0:H1'  # Rtg Süßen/Ottenbach (Salach Salach)
	MARKTPLATZ_26 = 'de:08117:6004:0:R1'  # Rtg Eislingen/Göppingen (Salach Salach)
	MARKTPLATZ_27 = 'de:08117:7014:0:H1'  # Rtg Schopflenberg (Uhingen Sparwiesen)
	MARKTPLATZ_28 = 'de:08117:7014:0:R1'  # Rtg Uhingen (Uhingen Sparwiesen)
	MARKTPLATZ_29 = 'de:08236:3513:0:1'  # Rtg Rutesheim/Hausen (Heimsheim Heimsheim)
	MARKTPLATZ_30 = 'de:08236:3513:0:2'  # Rtg See (Heimsheim Heimsheim)
	MARKTPLATZ_31 = 'de:08136:6512:0:3'  # eRtg Hintersteinenberg (Gschwend Gschwend)
	MARKTPLATZ_32 = 'de:08136:6512:0:4'  # eRtg Fa Kunz (Gschwend Gschwend)
	WEILHEIM__T__MARKTPLATZ = 'de:08116:3894'
	WEILHEIM__T__MARKTPLATZ_1 = 'de:08116:3894:0:3'  # Rtg Kirchheimer Str./Hegelstr/Neidlinge (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__MARKTPLATZ_2 = 'de:08116:3894:0:4'  # eRtg Schluchtweg (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__MARKTPLATZ_3 = 'de:08116:3894:0:5'  # Rtg Neidlinger Straße (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__WEINSTEIGE = 'de:08116:3895'
	WEILHEIM__T__WEINSTEIGE_1 = 'de:08116:3895:0:3'  # eRtg Egelsberg Teckstraße (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__WEINSTEIGE_2 = 'de:08116:3895:0:4'  # eRtg Limburgschule/Bissinger Strasse (Weilheim an der Teck Weilheim (T))
	LIMBURGSCHULE_BISSINGER_STRASSE = 'de:08116:3896:0:3'  # eRtg Bildungszentrum (Weilheim an der Teck Weilheim (T))
	WEILH__T__LIMB_SCH__BISS__STR = 'de:08116:3896:0:3'  # eRtg Bildungszentrum (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__MAIERHÖFE = 'de:08116:3897'
	WEILHEIM__T__MAIERHÖFE_1 = 'de:08116:3897:0:3'  # eRtg Jahnstraße (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__MAIERHÖFE_2 = 'de:08116:3897:0:4'  # eRtg Bahnhof (Weilheim an der Teck Weilheim (T))
	MAIERHÖFE = 'de:08116:3897'
	MAIERHÖFE_1 = 'de:08116:3897:0:3'  # eRtg Jahnstraße (Weilheim an der Teck Weilheim (T))
	MAIERHÖFE_2 = 'de:08116:3897:0:4'  # eRtg Bahnhof (Weilheim an der Teck Weilheim (T))
	BADSTRASSE_1 = 'de:08116:3898:0:3'  # Rtg Zell (Altbach Altbach)
	BADSTRASSE_2 = 'de:08116:3898:0:4'  # Rtg Rathaus (Altbach Altbach)
	BADSTRASSE_3 = 'de:08119:4982:0:3'  # eRtg Am Brenner/Traube (Auenwald Lippoldsweiler)
	BADSTRASSE_4 = 'de:08119:4982:0:4'  # eRtg Raiffeisenbank (Auenwald Lippoldsweiler)
	ALTBACH_BADSTRASSE = 'de:08116:3898'
	ALTBACH_BADSTRASSE_1 = 'de:08116:3898:0:3'  # Rtg Zell (Altbach Altbach)
	ALTBACH_BADSTRASSE_2 = 'de:08116:3898:0:4'  # Rtg Rathaus (Altbach Altbach)
	EGELSBERG_EGELSBERGSTRASSE = 'de:08116:3899'
	EGELSBERG_EGELSBERGSTRASSE_1 = 'de:08116:3899:0:3'  # eRtg Egelsberg Hochhaus (Weilheim an der Teck Weilheim (T))
	EGELSBERG_EGELSBERGSTRASSE_2 = 'de:08116:3899:0:4'  # eRtg Lange Morgen (Weilheim an der Teck Weilheim (T))
	WEILH__T__EGB__EGELSBGSTR = 'de:08116:3899'
	WEILH__T__EGB__EGELSBGSTR_1 = 'de:08116:3899:0:3'  # eRtg Egelsberg Hochhaus (Weilheim an der Teck Weilheim (T))
	WEILH__T__EGB__EGELSBGSTR_2 = 'de:08116:3899:0:4'  # eRtg Lange Morgen (Weilheim an der Teck Weilheim (T))
	EGELSBERG_TECKSTRASSE = 'de:08116:3900'
	EGELSBERG_TECKSTRASSE_1 = 'de:08116:3900:0:3'  # eRtg Egelsberg Hochhaus (Weilheim an der Teck Weilheim (T))
	EGELSBERG_TECKSTRASSE_2 = 'de:08116:3900:0:4'  # eRtg Bissingen Fabrikstraße (Weilheim an der Teck Weilheim (T))
	WEILH__T__EGB__TECKSTRASSE = 'de:08116:3900'
	WEILH__T__EGB__TECKSTRASSE_1 = 'de:08116:3900:0:3'  # eRtg Egelsberg Hochhaus (Weilheim an der Teck Weilheim (T))
	WEILH__T__EGB__TECKSTRASSE_2 = 'de:08116:3900:0:4'  # eRtg Bissingen Fabrikstraße (Weilheim an der Teck Weilheim (T))
	WEINBERG__TECKSTRASSE = 'de:08116:3901:0:4'  # eRtg Blumen-Wilhelmstraße (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__WEINBERG__TECKSTR = 'de:08116:3901:0:4'  # eRtg Blumen-Wilhelmstraße (Reichenbach an der Fils Reichenbach (F))
	LEINTEL = 'de:08116:3903:0:3'  # eRtg Steinäckerstraße / Schillerstr. (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__LEINTEL = 'de:08116:3903:0:3'  # eRtg Steinäckerstraße / Schillerstr. (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__STEINÄCKERSTR = 'de:08116:3904:0:3'  # eRtg Risshalde / Leintel (Reichenbach an der Fils Reichenbach (F))
	STEINÄCKERSTRASSE = 'de:08116:3904:0:3'  # eRtg Risshalde / Leintel (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F__SCHILLERSTR = 'de:08116:3905'
	REICHENBACH__F__SCHILLERSTR_1 = 'de:08116:3905:0:3'  # Rtg Schulzentrum (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F__SCHILLERSTR_2 = 'de:08116:3905:0:4'  # Rtg Stuttgarter-/Hauptstr. (Reichenbach an der Fils Reichenbach (F))
	ZIEGELHOF = 'de:08116:3906'
	ZIEGELHOF_1 = 'de:08116:3906:0:1'  # Rtg Lerchenweg (Hochdorf Hochdorf (ES))
	ZIEGELHOF_2 = 'de:08116:3906:0:2'  # Rtg Kulturz. Die Halle (Hochdorf Hochdorf (ES))
	HOCHD___ES__ZIEGELHOF = 'de:08116:3906'
	HOCHD___ES__ZIEGELHOF_1 = 'de:08116:3906:0:1'  # Rtg Lerchenweg (Hochdorf Hochdorf (ES))
	HOCHD___ES__ZIEGELHOF_2 = 'de:08116:3906:0:2'  # Rtg Kulturz. Die Halle (Hochdorf Hochdorf (ES))
	HOCHDORF_REUSSENSTEINWEG = 'de:08116:3908'
	HOCHDORF_REUSSENSTEINWEG_1 = 'de:08116:3908:0:1'  # Rtg Talbachbrücke (Hochdorf Hochdorf (ES))
	HOCHDORF_REUSSENSTEINWEG_2 = 'de:08116:3908:0:2'  # Rtg Reichenbach (Hochdorf Hochdorf (ES))
	HOCHDORF_MOZARTSTRASSE = 'de:08116:3910'
	HOCHDORF_MOZARTSTRASSE_1 = 'de:08116:3910:0:1'  # Rtg Ostring (Hochdorf Hochdorf (ES))
	HOCHDORF_MOZARTSTRASSE_2 = 'de:08116:3910:0:2'  # Rtg Roßwälder Str. (Hochdorf Hochdorf (ES))
	MOZARTSTRASSE_1 = 'de:08116:3910:0:1'  # Rtg Ostring (Hochdorf Hochdorf (ES))
	MOZARTSTRASSE_2 = 'de:08116:3910:0:2'  # Rtg Roßwälder Str. (Hochdorf Hochdorf (ES))
	MOZARTSTRASSE_3 = 'de:08119:6039:0:3'  # L.60 Rtg Lutherkirche (Fellbach Fellbach)
	MOZARTSTRASSE_4 = 'de:08119:6039:0:4'  # L.67 Rtg Lutherkirche (Fellbach Fellbach)
	MOZARTSTRASSE_5 = 'de:08117:1040:0:H1'  # Rtg Fahrzeugwerke (Eislingen/Fils Eislingen (F))
	MOZARTSTRASSE_6 = 'de:08117:1040:0:R1'  # Rtg Beundstr. (Eislingen/Fils Eislingen (F))
	MOZARTSTRASSE_7 = 'de:08117:4007:0:H1'  # Rtg Bartenb. Rotes Reusch (Göppingen Göppingen)
	HOCHDORF_OSTRING = 'de:08116:3911:0:2'  # Rtg Mozartstr. (Hochdorf Hochdorf (ES))
	OSTRING = 'de:08116:3911:0:2'  # Rtg Mozartstr. (Hochdorf Hochdorf (ES))
	IM_UNTEREN_ZEHEN = 'de:08116:3912'
	IM_UNTEREN_ZEHEN_1 = 'de:08116:3912:0:3'  # eRtg Grüner Kranz (Esslingen am Neckar Wiflingshausen)
	IM_UNTEREN_ZEHEN_2 = 'de:08116:3912:0:4'  # eRtg Talstraße (Esslingen am Neckar Wiflingshausen)
	WIFLINGSH__IM_UNT_ZEHEN = 'de:08116:3912'
	WIFLINGSH__IM_UNT_ZEHEN_1 = 'de:08116:3912:0:3'  # eRtg Grüner Kranz (Esslingen am Neckar Wiflingshausen)
	WIFLINGSH__IM_UNT_ZEHEN_2 = 'de:08116:3912:0:4'  # eRtg Talstraße (Esslingen am Neckar Wiflingshausen)
	SONNENWEG_1 = 'de:08116:3913:0:1'  # Rtg Hochdorfer Str. (Notzingen Notzingen)
	SONNENWEG_2 = 'de:08116:3913:0:2'  # Rtg Aspen (Notzingen Notzingen)
	SONNENWEG_3 = 'de:08136:7020:0:H1'  # Sonnenweg (Lorch Lorch)
	NOTZINGEN_SONNENWEG = 'de:08116:3913'
	NOTZINGEN_SONNENWEG_1 = 'de:08116:3913:0:1'  # Rtg Hochdorfer Str. (Notzingen Notzingen)
	NOTZINGEN_SONNENWEG_2 = 'de:08116:3913:0:2'  # Rtg Aspen (Notzingen Notzingen)
	ZOLLBERG_HOCHSTETTER_WEG = 'de:08116:3920'
	ZOLLBERG_HOCHSTETTER_WEG_1 = 'de:08116:3920:0:3'  # Rtg Zollhausweg (Esslingen am Neckar Zollberg)
	ZOLLBERG_HOCHSTETTER_WEG_2 = 'de:08116:3920:0:4'  # Rtg Braungartweg (Esslingen am Neckar Zollberg)
	HOCHSTETTER_WEG = 'de:08116:3920'
	HOCHSTETTER_WEG_1 = 'de:08116:3920:0:3'  # Rtg Zollhausweg (Esslingen am Neckar Zollberg)
	HOCHSTETTER_WEG_2 = 'de:08116:3920:0:4'  # Rtg Braungartweg (Esslingen am Neckar Zollberg)
	BEUREN_BRÜHL__FREILICHTMUSEUM = 'de:08116:3930:0:4'  # Rtg Bürgerhaus (Beuren Beuren)
	BRÜHL__FREILICHTMUSEUM = 'de:08116:3930:0:4'  # Rtg Bürgerhaus (Beuren Beuren)
	SCHOPFL__SCHLATTERHÖHE_L_1212 = 'de:08116:3932:0:3'  # eRtg Fuchslöcher (Lenningen Schopfloch (Lenningen))
	SCHLATTERHÖHE_L_1212 = 'de:08116:3932:0:3'  # eRtg Fuchslöcher (Lenningen Schopfloch (Lenningen))
	BEURENER_STEIGE = 'de:08116:3933'
	BEURENER_STEIGE_1 = 'de:08116:3933:0:3'  # eRtg Freilichtmuseum (Beuren Beuren)
	BEURENER_STEIGE_2 = 'de:08116:3933:0:4'  # eRtg Untere Straße (Beuren Beuren)
	BEUREN_BEURENER_STEIGE = 'de:08116:3933'
	BEUREN_BEURENER_STEIGE_1 = 'de:08116:3933:0:3'  # eRtg Freilichtmuseum (Beuren Beuren)
	BEUREN_BEURENER_STEIGE_2 = 'de:08116:3933:0:4'  # eRtg Untere Straße (Beuren Beuren)
	NECKARTAILFINGEN_SCHILLERSTR = 'de:08116:3937'
	NECKARTAILFINGEN_SCHILLERSTR_1 = 'de:08116:3937:0:3'  # eRtg Eichendorffstraße (Neckartailfingen Neckartailfingen)
	NECKARTAILFINGEN_SCHILLERSTR_2 = 'de:08116:3937:0:4'  # eRtg Neckartartailfingen Schule (Neckartailfingen Neckartailfingen)
	EICHENDORFFSTRASSE_1 = 'de:08116:3938:0:4'  # Rtg Schillerstr. (Neckartailfingen Neckartailfingen)
	EICHENDORFFSTRASSE_2 = 'de:08116:4174:1:3'  # eRtg Eichendorffschule (Esslingen am Neckar Zollberg)
	EICHENDORFFSTRASSE_3 = 'de:08116:4269:0:3'  # eRtg Bismarckstraße (Kirchheim unter Teck Kirchheim (T))
	EICHENDORFFSTRASSE_4 = 'de:08116:4269:0:4'  # eRtg Tannenbergstraße (Kirchheim unter Teck Kirchheim (T))
	EICHENDORFFSTRASSE_5 = 'de:08116:4446:0:3'  # eRtg Albstraße (Siedlerstube) (Nürtingen Nürtingen)
	EICHENDORFFSTRASSE_6 = 'de:08116:4446:0:4'  # eRtg Ziegeleistraße (Nürtingen Nürtingen)
	NECKARTAILF__EICHENDORFFSTR = 'de:08116:3938:0:4'  # Rtg Schillerstr. (Neckartailfingen Neckartailfingen)
	FREILICHTMUSEUM = 'de:08116:3939'
	FREILICHTMUSEUM_1 = 'de:08116:3939:0:3'  # Rtg Erken/Lenningen (Beuren Beuren)
	FREILICHTMUSEUM_2 = 'de:08116:3939:0:4'  # Rtg Beuren bis 11.09.16 (Beuren Beuren)
	FREILICHTMUSEUM_3 = 'de:08116:3939:0:5'  # Rtg Beuren ab 12.09.16 (Beuren Beuren)
	BEUREN_FREILICHTMUSEUM = 'de:08116:3939'
	BEUREN_FREILICHTMUSEUM_1 = 'de:08116:3939:0:3'  # Rtg Erken/Lenningen (Beuren Beuren)
	BEUREN_FREILICHTMUSEUM_2 = 'de:08116:3939:0:4'  # Rtg Beuren bis 11.09.16 (Beuren Beuren)
	BEUREN_FREILICHTMUSEUM_3 = 'de:08116:3939:0:5'  # Rtg Beuren ab 12.09.16 (Beuren Beuren)
	STUIFENSTRASSE_1 = 'de:08116:3940:0:3'  # eRtg Stumpenhof (Plochingen Plochingen)
	STUIFENSTRASSE_2 = 'de:08117:1225:0:H1'  # Rtg Lenglingen (Göppingen Maitis)
	PLOCHINGEN_STUIFENSTRASSE = 'de:08116:3940:0:3'  # eRtg Stumpenhof (Plochingen Plochingen)
	K_1206__BAHNHOF = 'de:08116:3941'
	K_1206__BAHNHOF_1 = 'de:08116:3941:1:3'  # eRtg Ulmer Straße (Reichenbach an der Fils Reichenbach (F))
	K_1206__BAHNHOF_2 = 'de:08116:3941:1:4'  # eRtg Weberei (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__K_1206__BAHNHOF = 'de:08116:3941'
	REICHENB__F__K_1206__BAHNHOF_1 = 'de:08116:3941:1:3'  # eRtg Ulmer Straße (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__K_1206__BAHNHOF_2 = 'de:08116:3941:1:4'  # eRtg Weberei (Reichenbach an der Fils Reichenbach (F))
	LINDORF_ZÄHRINGER_STRASSE = 'de:08116:3943'
	LINDORF_ZÄHRINGER_STRASSE_1 = 'de:08116:3943:0:3'  # eRtg Kruichling (Kirchheim unter Teck Lindorf)
	LINDORF_ZÄHRINGER_STRASSE_2 = 'de:08116:3943:0:4'  # eRtg Kreissparkasse (Kirchheim unter Teck Lindorf)
	ZÄHRINGER_STRASSE = 'de:08116:3943'
	ZÄHRINGER_STRASSE_1 = 'de:08116:3943:0:3'  # eRtg Kruichling (Kirchheim unter Teck Lindorf)
	ZÄHRINGER_STRASSE_2 = 'de:08116:3943:0:4'  # eRtg Kreissparkasse (Kirchheim unter Teck Lindorf)
	KIRCHHEIM__T__KRUICHLING = 'de:08116:3944'
	KIRCHHEIM__T__KRUICHLING_1 = 'de:08116:3944:0:3'  # Rtg Lindorf (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__KRUICHLING_2 = 'de:08116:3944:0:4'  # Rtg Kirchheim (Kirchheim unter Teck Kirchheim (T))
	KRUICHLING = 'de:08116:3944'
	KRUICHLING_1 = 'de:08116:3944:0:3'  # Rtg Lindorf (Kirchheim unter Teck Kirchheim (T))
	KRUICHLING_2 = 'de:08116:3944:0:4'  # Rtg Kirchheim (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__LINDORFER_WEG = 'de:08116:3945'
	KIRCHHEIM__T__LINDORFER_WEG_1 = 'de:08116:3945:0:3'  # eRtg Badwiesen (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__LINDORFER_WEG_2 = 'de:08116:3945:0:4'  # eRtg Kruichling (Kirchheim unter Teck Kirchheim (T))
	LINDORFER_WEG = 'de:08116:3945'
	LINDORFER_WEG_1 = 'de:08116:3945:0:3'  # eRtg Badwiesen (Kirchheim unter Teck Kirchheim (T))
	LINDORFER_WEG_2 = 'de:08116:3945:0:4'  # eRtg Kruichling (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__AUSTRASSE = 'de:08116:3947'
	KIRCHHEIM__T__AUSTRASSE_1 = 'de:08116:3947:0:3'  # eRtg Marktplatz (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__AUSTRASSE_2 = 'de:08116:3947:0:4'  # eRtg Saarstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__SAARSTRASSE = 'de:08116:3948'
	KIRCHHEIM__T__SAARSTRASSE_1 = 'de:08116:3948:0:3'  # eRtg Austraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__SAARSTRASSE_2 = 'de:08116:3948:0:4'  # eRtg Kitteneshalde (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__KITTENESHALDE = 'de:08116:3949'
	KIRCHHEIM__T__KITTENESHALDE_1 = 'de:08116:3949:0:3'  # eRtg Saarstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__KITTENESHALDE_2 = 'de:08116:3949:0:4'  # eRtg Alte Plochinger Steige (Kirchheim unter Teck Kirchheim (T))
	KITTENESHALDE = 'de:08116:3949'
	KITTENESHALDE_1 = 'de:08116:3949:0:3'  # eRtg Saarstraße (Kirchheim unter Teck Kirchheim (T))
	KITTENESHALDE_2 = 'de:08116:3949:0:4'  # eRtg Alte Plochinger Steige (Kirchheim unter Teck Kirchheim (T))
	KIRCHH__T__ALTE_PLOCH__STEIGE = 'de:08116:3950'
	KIRCHH__T__ALTE_PLOCH__STEIGE_1 = 'de:08116:3950:0:3'  # eRtg Kitteneshalde (Kirchheim unter Teck Kirchheim (T))
	KIRCHH__T__ALTE_PLOCH__STEIGE_2 = 'de:08116:3950:0:4'  # eRtg Im Münzen (Kirchheim unter Teck Kirchheim (T))
	KIRCHH__T__ALTE_PLOCH__STEIGE_3 = 'de:08116:3950:0:5'  # beide Rtg. Linie 162A (Kirchheim unter Teck Kirchheim (T))
	ALTE_PLOCHINGER_STEIGE = 'de:08116:3950'
	ALTE_PLOCHINGER_STEIGE_1 = 'de:08116:3950:0:3'  # eRtg Kitteneshalde (Kirchheim unter Teck Kirchheim (T))
	ALTE_PLOCHINGER_STEIGE_2 = 'de:08116:3950:0:4'  # eRtg Im Münzen (Kirchheim unter Teck Kirchheim (T))
	ALTE_PLOCHINGER_STEIGE_3 = 'de:08116:3950:0:5'  # beide Rtg. Linie 162A (Kirchheim unter Teck Kirchheim (T))
	IM_MÜNZEN = 'de:08116:3951:0:3'  # eRtg Alte Plochinger Steige (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__IM_MÜNZEN = 'de:08116:3951:0:3'  # eRtg Alte Plochinger Steige (Kirchheim unter Teck Kirchheim (T))
	BOHNAUHAUS = 'de:08116:3952'
	BOHNAUHAUS_1 = 'de:08116:3952:0:3'  # eRtg Bohnau (Kirchheim unter Teck Kirchheim (T))
	BOHNAUHAUS_2 = 'de:08116:3952:0:4'  # eRtg Gaiserplatz (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BOHNAUHAUS = 'de:08116:3952'
	KIRCHHEIM__T__BOHNAUHAUS_1 = 'de:08116:3952:0:3'  # eRtg Bohnau (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BOHNAUHAUS_2 = 'de:08116:3952:0:4'  # eRtg Gaiserplatz (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__OTTO_HAHN_STRASSE = 'de:08116:3953'
	KIRCHHEIM__T__OTTO_HAHN_STRASSE_1 = 'de:08116:3953:0:3'  # eRtg Willi-Bleicher-Straße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__OTTO_HAHN_STRASSE_2 = 'de:08116:3953:0:4'  # eRtg Bohnau (Kirchheim unter Teck Kirchheim (T))
	WILLI_BLEICHER_STRASSE = 'de:08116:3954:0:3'  # eRtg Einsteinstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHH__T__WILLI_BLEICHER_STR = 'de:08116:3954:0:3'  # eRtg Einsteinstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__EINSTEINSTRASSE = 'de:08116:3955'
	KIRCHHEIM__T__EINSTEINSTRASSE_1 = 'de:08116:3955:0:3'  # eRtg Fauslerstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__EINSTEINSTRASSE_2 = 'de:08116:3955:0:4'  # eRtg Heinkelstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__HEINKELSTRASSE = 'de:08116:3956:0:3'  # eRtg Otto-Hahn-Straße (Kirchheim unter Teck Kirchheim (T))
	HEINKELSTRASSE_1 = 'de:08116:3956:0:3'  # eRtg Otto-Hahn-Straße (Kirchheim unter Teck Kirchheim (T))
	HEINKELSTRASSE_2 = 'de:08115:4723:0:3'  # eRtg Hulb (Böblingen Böblingen)
	HEINKELSTRASSE_3 = 'de:08115:4723:0:4'  # eRtg Hanns-Klemm-Straße Ost (Böblingen Böblingen)
	HEINKELSTRASSE_4 = 'de:08118:7565:0:3'  # eRtg Kirschhaldenweg (Löchgau Löchgau)
	HEINKELSTRASSE_5 = 'de:08118:7565:0:4'  # eRtg Steinbachbrücke (Löchgau Löchgau)
	GAISERPLATZ = 'de:08116:3958'
	GAISERPLATZ_1 = 'de:08116:3958:0:3'  # eRtg Bohnauhaus (Kirchheim unter Teck Kirchheim (T))
	GAISERPLATZ_2 = 'de:08116:3958:0:4'  # eRtg Kirchheim (T) (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__GAISERPLATZ = 'de:08116:3958'
	KIRCHHEIM__T__GAISERPLATZ_1 = 'de:08116:3958:0:3'  # eRtg Bohnauhaus (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__GAISERPLATZ_2 = 'de:08116:3958:0:4'  # eRtg Kirchheim (T) (Kirchheim unter Teck Kirchheim (T))
	ÖTLINGEN_HALDENSTR = 'de:08116:3959:0:3'  # eRtg Halde (Kirchheim unter Teck Ötlingen)
	HALDENSTRASSE = 'de:08116:3959:0:3'  # eRtg Halde (Kirchheim unter Teck Ötlingen)
	HAHNWEIDSTRASSE = 'de:08116:3960'
	HAHNWEIDSTRASSE_1 = 'de:08116:3960:0:3'  # Rtg ZOB (Kirchheim unter Teck Kirchheim (T))
	HAHNWEIDSTRASSE_2 = 'de:08116:3960:0:4'  # Rtg Fußgängerzone/Martinskirche (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__HAHNWEIDSTRASSE = 'de:08116:3960'
	KIRCHHEIM__T__HAHNWEIDSTRASSE_1 = 'de:08116:3960:0:3'  # Rtg ZOB (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__HAHNWEIDSTRASSE_2 = 'de:08116:3960:0:4'  # Rtg Fußgängerzone/Martinskirche (Kirchheim unter Teck Kirchheim (T))
	ALTE_SCHIFFFAHRT = 'de:08116:3992:0:1'  # Rtg Mettingen Bahnhof (Esslingen am Neckar Mettingen)
	METTINGEN_ALTE_SCHIFFFAHRT = 'de:08116:3992:0:1'  # Rtg Mettingen Bahnhof (Esslingen am Neckar Mettingen)
	METTINGEN_BF__TROLLINGERWEG = 'de:08116:3993'
	METTINGEN_BF__TROLLINGERWEG_1 = 'de:08116:3993:0:1'  # Rtg Weinstraße (Esslingen am Neckar Mettingen)
	METTINGEN_BF__TROLLINGERWEG_2 = 'de:08116:3993:0:2'  # Rtg Burgunderstr. (Esslingen am Neckar Mettingen)
	BAHNHOF__TROLLINGERWEG = 'de:08116:3993'
	BAHNHOF__TROLLINGERWEG_1 = 'de:08116:3993:0:1'  # Rtg Weinstraße (Esslingen am Neckar Mettingen)
	BAHNHOF__TROLLINGERWEG_2 = 'de:08116:3993:0:2'  # Rtg Burgunderstr. (Esslingen am Neckar Mettingen)
	MANZELLER_WEG = 'de:08111:3995'
	MANZELLER_WEG_1 = 'de:08111:3995:0:3'  # Rtg Amstetter Str. (Stuttgart Stuttgart)
	MANZELLER_WEG_2 = 'de:08111:3995:0:4'  # Rtg Weil Wannenrain (Stuttgart Stuttgart)
	IN_DEN_STEGWIESEN = 'de:08111:3996:0:3'  # Rtg Hafenbahnstr. (Stuttgart Stuttgart)
	HAFENBAHNSTRASSE = 'de:08111:3997:1:3'  # Rtg Otto-Hirsch-Brücken (Stuttgart Stuttgart)
	HAFENBAHNSTR = 'de:08111:3997:1:3'  # Rtg Otto-Hirsch-Brücken (Stuttgart Stuttgart)
	ESSLINGEN__N__HOCHSCHULZENTRUM = 'de:08116:3998'
	ESSLINGEN__N__HOCHSCHULZENTRUM_1 = 'de:08116:3998:0:3'  # Rtg Klinikum (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__HOCHSCHULZENTRUM_2 = 'de:08116:3998:0:4'  # Rtg Lenzhalde (Esslingen am Neckar Esslingen (N))
	HOCHSCHULZENTRUM = 'de:08116:3998'
	HOCHSCHULZENTRUM_1 = 'de:08116:3998:0:3'  # Rtg Klinikum (Esslingen am Neckar Esslingen (N))
	HOCHSCHULZENTRUM_2 = 'de:08116:3998:0:4'  # Rtg Lenzhalde (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__LENZHALDE = 'de:08116:3999'
	ESSLINGEN__N__LENZHALDE_1 = 'de:08116:3999:0:3'  # Rtg Lenzhalde (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__LENZHALDE_2 = 'de:08116:3999:0:4'  # Rtg Am Schönen Rain (Esslingen am Neckar Esslingen (N))
	LENZHALDE_1 = 'de:08116:3999:0:3'  # Rtg Lenzhalde (Esslingen am Neckar Esslingen (N))
	LENZHALDE_2 = 'de:08116:3999:0:4'  # Rtg Am Schönen Rain (Esslingen am Neckar Esslingen (N))
	LENZHALDE_3 = 'de:08116:5012:0:3'  # Rtg Rathaus (Ostfildern Scharnhausen)
	LENZHALDE_4 = 'de:08116:5012:0:4'  # Rtg Sonnenhalde (Ostfildern Scharnhausen)
	LENZHALDE_5 = 'de:08118:6835:0:3'  # Rtg Aldinger Str. (Kornwestheim Kornwestheim)
	LENZHALDE_6 = 'de:08118:6835:0:4'  # Rtg Friedhof/Pfarrer (Kornwestheim Kornwestheim)
	ESSLINGEN__N__AM_SCHÖNEN_RAIN = 'de:08116:4000:1:3'  # Rtg Lenzhalde (Esslingen am Neckar Esslingen (N))
	AM_SCHÖNEN_RAIN = 'de:08116:4000:1:3'  # Rtg Lenzhalde (Esslingen am Neckar Esslingen (N))
	BRANDENBURGER_STRASSE = 'de:08116:4001'
	BRANDENBURGER_STRASSE_1 = 'de:08116:4001:0:3'  # eRtg Th.-Heuss-Gymn. / Kennenburg (Esslingen am Neckar Oberesslingen)
	BRANDENBURGER_STRASSE_2 = 'de:08116:4001:0:4'  # eRtg Ortseingang (Esslingen am Neckar Oberesslingen)
	OBERESSL__BRANDENBURGER_STR = 'de:08116:4001'
	OBERESSL__BRANDENBURGER_STR_1 = 'de:08116:4001:0:3'  # eRtg Th.-Heuss-Gymn. / Kennenburg (Esslingen am Neckar Oberesslingen)
	OBERESSL__BRANDENBURGER_STR_2 = 'de:08116:4001:0:4'  # eRtg Ortseingang (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_STEINHALDE = 'de:08116:4002'
	OBERESSLINGEN_STEINHALDE_1 = 'de:08116:4002:0:3'  # eRtg Kreuz/Herder/Schornd./Schwimmbad (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_STEINHALDE_2 = 'de:08116:4002:0:4'  # eRtg T-H-Gym./Endstation/Brandenburger S (Esslingen am Neckar Oberesslingen)
	STEINHALDE = 'de:08116:4002'
	STEINHALDE_1 = 'de:08116:4002:0:3'  # eRtg Kreuz/Herder/Schornd./Schwimmbad (Esslingen am Neckar Oberesslingen)
	STEINHALDE_2 = 'de:08116:4002:0:4'  # eRtg T-H-Gym./Endstation/Brandenburger S (Esslingen am Neckar Oberesslingen)
	LIEBERSBR__KIMMICHSWEILER_WEG = 'de:08116:4003:0:3'  # eRtg Eiserne Hand (Esslingen am Neckar Liebersbronn)
	KIMMICHSWEILER_WEG = 'de:08116:4003:0:3'  # eRtg Eiserne Hand (Esslingen am Neckar Liebersbronn)
	OBERESSL__PARACELSUSSTRASSE = 'de:08116:4004'
	OBERESSL__PARACELSUSSTRASSE_1 = 'de:08116:4004:0:3'  # Rtg Esslingen (Esslingen am Neckar Oberesslingen)
	OBERESSL__PARACELSUSSTRASSE_2 = 'de:08116:4004:0:4'  # Rtg Zell (Esslingen am Neckar Oberesslingen)
	PARACELSUSSTRASSE = 'de:08116:4004'
	PARACELSUSSTRASSE_1 = 'de:08116:4004:0:3'  # Rtg Esslingen (Esslingen am Neckar Oberesslingen)
	PARACELSUSSTRASSE_2 = 'de:08116:4004:0:4'  # Rtg Zell (Esslingen am Neckar Oberesslingen)
	BISMARCKSTRASSE_1 = 'de:08116:4005:0:3'  # Rtg Charlottenplatz (Esslingen am Neckar Esslingen (N))
	BISMARCKSTRASSE_2 = 'de:08116:4005:0:4'  # Rtg Rosenau (Esslingen am Neckar Esslingen (N))
	BISMARCKSTRASSE_3 = 'de:08116:4332:0:3'  # eRtg Konrad-Widerholt-Halle (Kirchheim unter Teck Kirchheim (T))
	BISMARCKSTRASSE_4 = 'de:08116:4332:0:4'  # eRtg Eichendorffstraße (Kirchheim unter Teck Kirchheim (T))
	ESSLINGEN__N__BISMARCKSTR = 'de:08116:4005'
	ESSLINGEN__N__BISMARCKSTR_1 = 'de:08116:4005:0:3'  # Rtg Charlottenplatz (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__BISMARCKSTR_2 = 'de:08116:4005:0:4'  # Rtg Rosenau (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__MERKEL_SCHES_BAD = 'de:08116:4006'
	ESSLINGEN__N__MERKEL_SCHES_BAD_1 = 'de:08116:4006:1:3'  # Rtg Charlottenplatz (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__MERKEL_SCHES_BAD_2 = 'de:08116:4006:1:4'  # Rtg stadteinwärts (Esslingen am Neckar Esslingen (N))
	MERKEL_SCHES_BAD = 'de:08116:4006'
	MERKEL_SCHES_BAD_1 = 'de:08116:4006:1:3'  # Rtg Charlottenplatz (Esslingen am Neckar Esslingen (N))
	MERKEL_SCHES_BAD_2 = 'de:08116:4006:1:4'  # Rtg stadteinwärts (Esslingen am Neckar Esslingen (N))
	SCHELZTOR = 'de:08116:4007'
	SCHELZTOR_1 = 'de:08116:4007:0:3'  # Rtg stadtauswärts (Esslingen am Neckar Esslingen (N))
	SCHELZTOR_2 = 'de:08116:4007:0:4'  # Rtg stadteinwärts (Esslingen am Neckar Esslingen (N))
	SCHELZTOR_3 = 'de:08116:4007:0:5'  # Rtg Schelztorstr. (Esslingen am Neckar Esslingen (N))
	SCHELZTOR_4 = 'de:08116:4007:0:6'  # Rtg Mettingen 101 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__SCHELZTOR = 'de:08116:4007'
	ESSLINGEN__N__SCHELZTOR_1 = 'de:08116:4007:0:3'  # Rtg stadtauswärts (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__SCHELZTOR_2 = 'de:08116:4007:0:4'  # Rtg stadteinwärts (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__SCHELZTOR_3 = 'de:08116:4007:0:5'  # Rtg Schelztorstr. (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__SCHELZTOR_4 = 'de:08116:4007:0:6'  # Rtg Mettingen 101 (Esslingen am Neckar Esslingen (N))
	ESSL__N__SCHLACHTHAUSBRÜCKE = 'de:08116:4008'
	ESSL__N__SCHLACHTHAUSBRÜCKE_1 = 'de:08116:4008:0:3'  # Rtg Mettingen (Esslingen am Neckar Esslingen (N))
	ESSL__N__SCHLACHTHAUSBRÜCKE_2 = 'de:08116:4008:0:4'  # Rtg Pliensauturm (Esslingen am Neckar Esslingen (N))
	SCHLACHTHAUSBRÜCKE = 'de:08116:4008'
	SCHLACHTHAUSBRÜCKE_1 = 'de:08116:4008:0:3'  # Rtg Mettingen (Esslingen am Neckar Esslingen (N))
	SCHLACHTHAUSBRÜCKE_2 = 'de:08116:4008:0:4'  # Rtg Pliensauturm (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__VOLKSHOCHSCHULE = 'de:08116:4009'
	ESSLINGEN__N__VOLKSHOCHSCHULE_1 = 'de:08116:4009:0:3'  # Rtg Mettingen (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__VOLKSHOCHSCHULE_2 = 'de:08116:4009:0:4'  # Rtg Schelztor (Esslingen am Neckar Esslingen (N))
	VOLKSHOCHSCHULE = 'de:08116:4009'
	VOLKSHOCHSCHULE_1 = 'de:08116:4009:0:3'  # Rtg Mettingen (Esslingen am Neckar Esslingen (N))
	VOLKSHOCHSCHULE_2 = 'de:08116:4009:0:4'  # Rtg Schelztor (Esslingen am Neckar Esslingen (N))
	METTINGEN_CANNSTATTER_STR = 'de:08116:4010'
	METTINGEN_CANNSTATTER_STR_1 = 'de:08116:4010:0:3'  # Rtg Mettingen Bf (Esslingen am Neckar Mettingen)
	METTINGEN_CANNSTATTER_STR_2 = 'de:08116:4010:0:4'  # Rtg Weil (Esslingen am Neckar Mettingen)
	CANNSTATTER_STRASSE_1 = 'de:08116:4010:0:3'  # Rtg Mettingen Bf (Esslingen am Neckar Mettingen)
	CANNSTATTER_STRASSE_2 = 'de:08116:4010:0:4'  # Rtg Weil (Esslingen am Neckar Mettingen)
	CANNSTATTER_STRASSE_3 = 'de:08118:5588:0:3'  # eRtg Hornbach/Mühlweg (Remseck am Neckar Aldingen)
	CANNSTATTER_STRASSE_4 = 'de:08118:5588:0:4'  # eRtg Schlosshof/Goethestraße (Remseck am Neckar Aldingen)
	METTINGEN_BURGUNDERSTR = 'de:08116:4011'
	METTINGEN_BURGUNDERSTR_1 = 'de:08116:4011:0:3'  # Rtg Cannstatter Str. / Esslingen VHS (Esslingen am Neckar Mettingen)
	METTINGEN_BURGUNDERSTR_2 = 'de:08116:4011:0:4'  # Rtg. Mettingen Bf. (Schenkenbergstr.) (Esslingen am Neckar Mettingen)
	METTINGEN_WEINSTRASSE = 'de:08116:4012'
	METTINGEN_WEINSTRASSE_1 = 'de:08116:4012:0:3'  # Rtg Mettingen (Esslingen am Neckar Mettingen)
	METTINGEN_WEINSTRASSE_2 = 'de:08116:4012:0:4'  # Rtg Obertürkheim (Esslingen am Neckar Mettingen)
	STADTGRENZE = 'de:08111:4013'
	STADTGRENZE_1 = 'de:08111:4013:0:3'  # Rtg Obertürkheim Bf (Stuttgart Stuttgart)
	STADTGRENZE_2 = 'de:08111:4013:0:4'  # Rtg Mettingen Weinstr. (Stuttgart Stuttgart)
	KÖNIGSALLEE = 'de:08116:4015'
	KÖNIGSALLEE_1 = 'de:08116:4015:0:3'  # Rtg Mettingen (Esslingen am Neckar Weil)
	KÖNIGSALLEE_2 = 'de:08116:4015:0:4'  # Rtg Pliensauvor./Esslingen (Esslingen am Neckar Weil)
	WEIL_KÖNIGSALLEE = 'de:08116:4015'
	WEIL_KÖNIGSALLEE_1 = 'de:08116:4015:0:3'  # Rtg Mettingen (Esslingen am Neckar Weil)
	WEIL_KÖNIGSALLEE_2 = 'de:08116:4015:0:4'  # Rtg Pliensauvor./Esslingen (Esslingen am Neckar Weil)
	WEIL_GESTÜTSWEG = 'de:08116:4016'
	WEIL_GESTÜTSWEG_1 = 'de:08116:4016:0:3'  # Rtg Mettingen/Hedelfingen (Esslingen am Neckar Weil)
	WEIL_GESTÜTSWEG_2 = 'de:08116:4016:0:4'  # Rtg Pliensauvorstadt (Esslingen am Neckar Weil)
	GESTÜTSWEG = 'de:08116:4016'
	GESTÜTSWEG_1 = 'de:08116:4016:0:3'  # Rtg Mettingen/Hedelfingen (Esslingen am Neckar Weil)
	GESTÜTSWEG_2 = 'de:08116:4016:0:4'  # Rtg Pliensauvorstadt (Esslingen am Neckar Weil)
	WEIL = 'de:08116:4017:0:3'  # Rtg Mettingen/Hedelfingen/Zollberg (Esslingen am Neckar Weil)
	ZELL_FRIEDHOF = 'de:08116:4018'
	ZELL_FRIEDHOF_1 = 'de:08116:4018:0:3'  # Rtg Egert Lindenplatz (Esslingen am Neckar Zell (Esslingen))
	ZELL_FRIEDHOF_2 = 'de:08116:4018:0:4'  # Rtg Zell Bf (Esslingen am Neckar Zell (Esslingen))
	ZELL_HANGELSTEIN_KIRCHE = 'de:08116:4020'
	ZELL_HANGELSTEIN_KIRCHE_1 = 'de:08116:4020:0:3'  # Rtg Albblick (Esslingen am Neckar Zell (Esslingen))
	ZELL_HANGELSTEIN_KIRCHE_2 = 'de:08116:4020:0:4'  # Rtg Bahnhof (Esslingen am Neckar Zell (Esslingen))
	HANGELSTEIN_KIRCHE = 'de:08116:4020'
	HANGELSTEIN_KIRCHE_1 = 'de:08116:4020:0:3'  # Rtg Albblick (Esslingen am Neckar Zell (Esslingen))
	HANGELSTEIN_KIRCHE_2 = 'de:08116:4020:0:4'  # Rtg Bahnhof (Esslingen am Neckar Zell (Esslingen))
	ZELL_ALBBLICK = 'de:08116:4021'
	ZELL_ALBBLICK_1 = 'de:08116:4021:0:3'  # Rtg Oberesslingen/alle Rtgn. (Esslingen am Neckar Zell (Esslingen))
	ZELL_ALBBLICK_2 = 'de:08116:4021:0:4'  # Rtg Johannesstr. (Esslingen am Neckar Zell (Esslingen))
	ALBBLICK_1 = 'de:08116:4021:0:3'  # Rtg Oberesslingen/alle Rtgn. (Esslingen am Neckar Zell (Esslingen))
	ALBBLICK_2 = 'de:08116:4021:0:4'  # Rtg Johannesstr. (Esslingen am Neckar Zell (Esslingen))
	ALBBLICK_3 = 'de:08111:6016:0:3'  # Rtg Am Ochsenwald (Stuttgart Stuttgart)
	ALBBLICK_4 = 'de:08111:6016:0:4'  # Rtg Pestalozzi-Schule (Stuttgart Stuttgart)
	HOCHSCHULE_1 = 'de:08116:4022:0:3'  # Rtg stadtauswärts (Esslingen am Neckar Esslingen (N))
	HOCHSCHULE_2 = 'de:08116:4022:0:4'  # Rtg stadteinwärts (Esslingen am Neckar Esslingen (N))
	HOCHSCHULE_3 = 'de:08117:3301:0:H1'  # Rtg Ulmer-/Heininger Str. (Göppingen Göppingen)
	HOCHSCHULE_4 = 'de:08117:3301:0:R1'  # Rtg Bahnhofssteg (Göppingen Göppingen)
	ESSLINGEN__N__HOCHSCHULE = 'de:08116:4022'
	ESSLINGEN__N__HOCHSCHULE_1 = 'de:08116:4022:0:3'  # Rtg stadtauswärts (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__HOCHSCHULE_2 = 'de:08116:4022:0:4'  # Rtg stadteinwärts (Esslingen am Neckar Esslingen (N))
	OBERESSLINGEN_STAUFFENBERGSTRASSE = 'de:08116:4023'
	OBERESSLINGEN_STAUFFENBERGSTRASSE_1 = 'de:08116:4023:0:3'  # Rtg Bismarckstr. (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_STAUFFENBERGSTRASSE_2 = 'de:08116:4023:0:4'  # Rtg Index (Esslingen am Neckar Oberesslingen)
	STAUFFENBERGSTRASSE_1 = 'de:08116:4023:0:3'  # Rtg Bismarckstr. (Esslingen am Neckar Oberesslingen)
	STAUFFENBERGSTRASSE_2 = 'de:08116:4023:0:4'  # Rtg Index (Esslingen am Neckar Oberesslingen)
	STAUFFENBERGSTRASSE_3 = 'de:08118:5442:0:3'  # eRtg Goerdelerstraße (Kornwestheim Kornwestheim)
	STAUFFENBERGSTRASSE_4 = 'de:08118:5442:0:4'  # eRtg Realschule (Kornwestheim Kornwestheim)
	OBERESSL__GARTENSTADT = 'de:08116:4024'
	OBERESSL__GARTENSTADT_1 = 'de:08116:4024:0:3'  # Rtg Zell (Esslingen am Neckar Oberesslingen)
	OBERESSL__GARTENSTADT_2 = 'de:08116:4024:0:4'  # Rtg Oberesslingen (Esslingen am Neckar Oberesslingen)
	WEIHERSTRASSE = 'de:08116:4026'
	WEIHERSTRASSE_1 = 'de:08116:4026:0:3'  # eRtg Zimmerbachstraße (Esslingen am Neckar Oberesslingen)
	WEIHERSTRASSE_2 = 'de:08116:4026:0:4'  # eRtg T-H-Gym/Gartenstadt (Esslingen am Neckar Oberesslingen)
	OBERESSL__WEIHERSTRASSE = 'de:08116:4026'
	OBERESSL__WEIHERSTRASSE_1 = 'de:08116:4026:0:3'  # eRtg Zimmerbachstraße (Esslingen am Neckar Oberesslingen)
	OBERESSL__WEIHERSTRASSE_2 = 'de:08116:4026:0:4'  # eRtg T-H-Gym/Gartenstadt (Esslingen am Neckar Oberesslingen)
	ZIMMERBACHSTRASSE = 'de:08116:4027'
	ZIMMERBACHSTRASSE_1 = 'de:08116:4027:0:3'  # eRtg Keplerstraße/Schorndorfer Str. (Esslingen am Neckar Oberesslingen)
	ZIMMERBACHSTRASSE_2 = 'de:08116:4027:0:4'  # eRtg Weiherstraße (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_ZIMMERBACHSTR = 'de:08116:4027'
	OBERESSLINGEN_ZIMMERBACHSTR_1 = 'de:08116:4027:0:3'  # eRtg Keplerstraße/Schorndorfer Str. (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_ZIMMERBACHSTR_2 = 'de:08116:4027:0:4'  # eRtg Weiherstraße (Esslingen am Neckar Oberesslingen)
	ESSLINGEN__N__ANNA_SCHIEBER_WEG = 'de:08116:4028'
	ESSLINGEN__N__ANNA_SCHIEBER_WEG_1 = 'de:08116:4028:1:3'  # Rtg stadtauswärts (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__ANNA_SCHIEBER_WEG_2 = 'de:08116:4028:1:4'  # Rtg stadteinwärts (Esslingen am Neckar Esslingen (N))
	ANNA_SCHIEBER_WEG = 'de:08116:4028'
	ANNA_SCHIEBER_WEG_1 = 'de:08116:4028:1:3'  # Rtg stadtauswärts (Esslingen am Neckar Esslingen (N))
	ANNA_SCHIEBER_WEG_2 = 'de:08116:4028:1:4'  # Rtg stadteinwärts (Esslingen am Neckar Esslingen (N))
	OBERESSLINGEN_KREUZSTR = 'de:08116:4029'
	OBERESSLINGEN_KREUZSTR_1 = 'de:08116:4029:0:3'  # eRtg Hirschlandhof/Krankenhaus (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_KREUZSTR_2 = 'de:08116:4029:0:4'  # eRtg Steinhalde/Schorndorfer Str./Index (Esslingen am Neckar Oberesslingen)
	ESSLINGEN__N__REUTLINGER_STR = 'de:08116:4031'
	ESSLINGEN__N__REUTLINGER_STR_1 = 'de:08116:4031:0:3'  # Rtg Esslingen (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__REUTLINGER_STR_2 = 'de:08116:4031:0:4'  # Rtg Zell (Esslingen am Neckar Esslingen (N))
	EBERSHALDENFRIEDHOF = 'de:08116:4032'
	EBERSHALDENFRIEDHOF_1 = 'de:08116:4032:0:3'  # Rtg stadtauswärts (Esslingen am Neckar Esslingen (N))
	EBERSHALDENFRIEDHOF_2 = 'de:08116:4032:0:4'  # Rtg stadteinwärts (Esslingen am Neckar Esslingen (N))
	ESSL__N__EBERSHALDENFRIEDHOF = 'de:08116:4032'
	ESSL__N__EBERSHALDENFRIEDHOF_1 = 'de:08116:4032:0:3'  # Rtg stadtauswärts (Esslingen am Neckar Esslingen (N))
	ESSL__N__EBERSHALDENFRIEDHOF_2 = 'de:08116:4032:0:4'  # Rtg stadteinwärts (Esslingen am Neckar Esslingen (N))
	NECKAR_FORUM = 'de:08116:4033'
	NECKAR_FORUM_1 = 'de:08116:4033:0:3'  # eRtg Katharinenstaffel/Eberhardtshaldenf (Esslingen am Neckar Esslingen (N))
	NECKAR_FORUM_2 = 'de:08116:4033:0:4'  # eRtg Kleiner Markt (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__NECKAR_FORUM = 'de:08116:4033'
	ESSLINGEN__N__NECKAR_FORUM_1 = 'de:08116:4033:0:3'  # eRtg Katharinenstaffel/Eberhardtshaldenf (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__NECKAR_FORUM_2 = 'de:08116:4033:0:4'  # eRtg Kleiner Markt (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__MARKTPLATZ = 'de:08116:4034'
	ESSLINGEN__N__MARKTPLATZ_1 = 'de:08116:4034:1:3'  # Rtg Ost (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__MARKTPLATZ_2 = 'de:08116:4034:1:4'  # Rtg West (Esslingen am Neckar Esslingen (N))
	PLIENSAUV__STUTTGARTER_STR = 'de:08116:4035'
	PLIENSAUV__STUTTGARTER_STR_1 = 'de:08116:4035:1:3'  # Rtg Weil/Nellingen/Schelztor (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUV__STUTTGARTER_STR_2 = 'de:08116:4035:1:4'  # Rtg Königsallee (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUVORSTADT_K__PFAFF_STR = 'de:08116:4036'
	PLIENSAUVORSTADT_K__PFAFF_STR_1 = 'de:08116:4036:0:3'  # Rtg Weil (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUVORSTADT_K__PFAFF_STR_2 = 'de:08116:4036:0:5'  # Rtg Weil neu (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUVORSTADT_K__PFAFF_STR_3 = 'de:08116:4036:0:6'  # Rtg Esslingen neu (Esslingen am Neckar Pliensauvorstadt)
	EBERHARD_BAUER_STRASSE = 'de:08116:4037'
	EBERHARD_BAUER_STRASSE_1 = 'de:08116:4037:0:4'  # Rtg Esslingen (Esslingen am Neckar Pliensauvorstadt)
	EBERHARD_BAUER_STRASSE_2 = 'de:08116:4037:0:5'  # Rtg Weil/Mettingen/Hedelfingen neu (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUV__E__BAUER_STRASSE = 'de:08116:4037'
	PLIENSAUV__E__BAUER_STRASSE_1 = 'de:08116:4037:0:4'  # Rtg Esslingen (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUV__E__BAUER_STRASSE_2 = 'de:08116:4037:0:5'  # Rtg Weil/Mettingen/Hedelfingen neu (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUV__PARKSTRASSE = 'de:08116:4038:0:3'  # Rtg Weil Stadion (Esslingen am Neckar Pliensauvorstadt)
	ESSLINGEN__N__SCHILLERPLATZ = 'de:08116:4040'
	ESSLINGEN__N__SCHILLERPLATZ_1 = 'de:08116:4040:1:3'  # Rtg Schwimmbad (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__SCHILLERPLATZ_2 = 'de:08116:4040:1:4'  # Rtg Neckarfreibad (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__NECKARFREIBAD = 'de:08116:4041'
	ESSLINGEN__N__NECKARFREIBAD_1 = 'de:08116:4041:0:3'  # Rtg Adenauerbrücke (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__NECKARFREIBAD_2 = 'de:08116:4041:0:4'  # Rtg Schillerplatz (Esslingen am Neckar Esslingen (N))
	NECKARFREIBAD = 'de:08116:4041'
	NECKARFREIBAD_1 = 'de:08116:4041:0:3'  # Rtg Adenauerbrücke (Esslingen am Neckar Esslingen (N))
	NECKARFREIBAD_2 = 'de:08116:4041:0:4'  # Rtg Schillerplatz (Esslingen am Neckar Esslingen (N))
	OBERESSL__SCHWERTMÜHLE = 'de:08116:4042'
	OBERESSL__SCHWERTMÜHLE_1 = 'de:08116:4042:0:3'  # Rtg Max-Planck-Str. (Esslingen am Neckar Oberesslingen)
	OBERESSL__SCHWERTMÜHLE_2 = 'de:08116:4042:0:4'  # Rtg Bf (Süd) (Esslingen am Neckar Oberesslingen)
	SCHWERTMÜHLE = 'de:08116:4042'
	SCHWERTMÜHLE_1 = 'de:08116:4042:0:3'  # Rtg Max-Planck-Str. (Esslingen am Neckar Oberesslingen)
	SCHWERTMÜHLE_2 = 'de:08116:4042:0:4'  # Rtg Bf (Süd) (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_MAX_PLANCK_STR = 'de:08116:4043'
	OBERESSLINGEN_MAX_PLANCK_STR_1 = 'de:08116:4043:0:3'  # Rtg Zeppelinstr./Sirnauer Brücke (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_MAX_PLANCK_STR_2 = 'de:08116:4043:0:4'  # Rtg Schwertmühle (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_ZEPPELINSTR = 'de:08116:4044'
	OBERESSLINGEN_ZEPPELINSTR_1 = 'de:08116:4044:0:3'  # Rtg Röntgenstr. (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_ZEPPELINSTR_2 = 'de:08116:4044:0:4'  # Rtg Max-Planck-Str. (Esslingen am Neckar Oberesslingen)
	ZEPPELINSTRASSE_1 = 'de:08116:4044:0:3'  # Rtg Röntgenstr. (Esslingen am Neckar Oberesslingen)
	ZEPPELINSTRASSE_2 = 'de:08116:4044:0:4'  # Rtg Max-Planck-Str. (Esslingen am Neckar Oberesslingen)
	ZEPPELINSTRASSE_3 = 'de:08116:5020:0:3'  # Rtg Rosenstr. (Ostfildern Kemnat)
	ZEPPELINSTRASSE_4 = 'de:08116:5020:0:4'  # Rtg Schönbergstr. (Ostfildern Kemnat)
	ZEPPELINSTRASSE_5 = 'de:08118:5439:0:3'  # Rtg Im Weizen (Kornwestheim Kornwestheim)
	ZEPPELINSTRASSE_6 = 'de:08118:5439:0:4'  # Rtg Johannesstr. (Kornwestheim Kornwestheim)
	ZEPPELINSTRASSE_7 = 'de:08116:6878:0:3'  # eRtg Plochingen (Deizisau Deizisau)
	OBERESSLINGEN_RÖNTGENSTR = 'de:08116:4045'
	OBERESSLINGEN_RÖNTGENSTR_1 = 'de:08116:4045:0:3'  # eRtg Alleenstraße (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_RÖNTGENSTR_2 = 'de:08116:4045:0:4'  # eRtg Zeppelinstraße (Esslingen am Neckar Oberesslingen)
	RÖNTGENSTRASSE = 'de:08116:4045'
	RÖNTGENSTRASSE_1 = 'de:08116:4045:0:3'  # eRtg Alleenstraße (Esslingen am Neckar Oberesslingen)
	RÖNTGENSTRASSE_2 = 'de:08116:4045:0:4'  # eRtg Zeppelinstraße (Esslingen am Neckar Oberesslingen)
	ZELL_ALLEENSTRASSE = 'de:08116:4047'
	ZELL_ALLEENSTRASSE_1 = 'de:08116:4047:0:3'  # eRtg Sirnauer Brücke (Esslingen am Neckar Zell (Esslingen))
	ZELL_ALLEENSTRASSE_2 = 'de:08116:4047:0:4'  # eRtg Schulzentrum/Röntgenstr/Kirche (Esslingen am Neckar Zell (Esslingen))
	ALLEENSTRASSE = 'de:08116:4047'
	ALLEENSTRASSE_1 = 'de:08116:4047:0:3'  # eRtg Sirnauer Brücke (Esslingen am Neckar Zell (Esslingen))
	ALLEENSTRASSE_2 = 'de:08116:4047:0:4'  # eRtg Schulzentrum/Röntgenstr/Kirche (Esslingen am Neckar Zell (Esslingen))
	SIRNAUER_HOF = 'de:08116:4050'
	SIRNAUER_HOF_1 = 'de:08116:4050:0:3'  # Rtg Betriebshof (Esslingen am Neckar Sirnau)
	SIRNAUER_HOF_2 = 'de:08116:4050:0:4'  # Rtg Sirnauer Brücke/Finkenweg (Esslingen am Neckar Sirnau)
	SIRNAU_SIRNAUER_HOF = 'de:08116:4050'
	SIRNAU_SIRNAUER_HOF_1 = 'de:08116:4050:0:3'  # Rtg Betriebshof (Esslingen am Neckar Sirnau)
	SIRNAU_SIRNAUER_HOF_2 = 'de:08116:4050:0:4'  # Rtg Sirnauer Brücke/Finkenweg (Esslingen am Neckar Sirnau)
	SIRNAU_SCHWALBENWEG = 'de:08116:4051'
	SIRNAU_SCHWALBENWEG_1 = 'de:08116:4051:0:3'  # eRtg Finkenweg (Esslingen am Neckar Sirnau)
	SIRNAU_SCHWALBENWEG_2 = 'de:08116:4051:0:4'  # eRtg Sirnauer Hof (Esslingen am Neckar Sirnau)
	SCHWALBENWEG_1 = 'de:08116:4051:0:3'  # eRtg Finkenweg (Esslingen am Neckar Sirnau)
	SCHWALBENWEG_2 = 'de:08116:4051:0:4'  # eRtg Sirnauer Hof (Esslingen am Neckar Sirnau)
	SCHWALBENWEG_3 = 'de:08119:5237:0:3'  # Rtg Wasserbergweg (Plüderhausen Plüderhausen)
	SCHWALBENWEG_4 = 'de:08119:5237:0:4'  # Rtg Gänswasen (Plüderhausen Plüderhausen)
	FINKENWEG_1 = 'de:08116:4052:0:3'  # eRtg See Adenauerbrücke (Esslingen am Neckar Sirnau)
	FINKENWEG_2 = 'de:08116:4052:0:4'  # eRtg Schwalbenweg (Esslingen am Neckar Sirnau)
	FINKENWEG_3 = 'de:08117:2019:0:H1'  # Rtg Im Morgen (Albershausen Albershausen)
	FINKENWEG_4 = 'de:08117:2019:0:R1'  # Rtg Frühlingstr. (Albershausen Albershausen)
	SIRNAU_FINKENWEG = 'de:08116:4052'
	SIRNAU_FINKENWEG_1 = 'de:08116:4052:0:3'  # eRtg See Adenauerbrücke (Esslingen am Neckar Sirnau)
	SIRNAU_FINKENWEG_2 = 'de:08116:4052:0:4'  # eRtg Schwalbenweg (Esslingen am Neckar Sirnau)
	SCHAFHAUSEN_DÄTZINGER_STRASSE = 'de:08115:4053'
	SCHAFHAUSEN_DÄTZINGER_STRASSE_1 = 'de:08115:4053:0:3'  # eRtg Calwer Straße (Weil der Stadt Schafhausen)
	SCHAFHAUSEN_DÄTZINGER_STRASSE_2 = 'de:08115:4053:0:4'  # eRtg Rathaus (Weil der Stadt Schafhausen)
	DÄTZINGER_STRASSE = 'de:08115:4053'
	DÄTZINGER_STRASSE_1 = 'de:08115:4053:0:3'  # eRtg Calwer Straße (Weil der Stadt Schafhausen)
	DÄTZINGER_STRASSE_2 = 'de:08115:4053:0:4'  # eRtg Rathaus (Weil der Stadt Schafhausen)
	DEIZISAU_ESSLINGER_STR = 'de:08116:4054'
	DEIZISAU_ESSLINGER_STR_1 = 'de:08116:4054:0:3'  # Rtg Esslingen (Deizisau Deizisau)
	DEIZISAU_ESSLINGER_STR_2 = 'de:08116:4054:0:4'  # Rtg Deizisau (Deizisau Deizisau)
	DEIZISAU_WERT = 'de:08116:4055:0:3'  # eRtg Alte Bergstraße (Deizisau Deizisau)
	WERT = 'de:08116:4055:0:3'  # eRtg Alte Bergstraße (Deizisau Deizisau)
	DEIZISAU_RATHAUS = 'de:08116:4057'
	DEIZISAU_RATHAUS_1 = 'de:08116:4057:1:3'  # eRtg Schule (Deizisau Deizisau)
	DEIZISAU_RATHAUS_2 = 'de:08116:4057:1:4'  # Rtg Alte Bergstraße (Deizisau Deizisau)
	DEIZISAU_RATHAUS_3 = 'de:08116:4057:1:5'  # Rtg Esslingen (Deizisau Deizisau)
	DEIZISAU_SCHULE = 'de:08116:4058'
	DEIZISAU_SCHULE_1 = 'de:08116:4058:0:3'  # eRtg Rathaus (Deizisau Deizisau)
	DEIZISAU_SCHULE_2 = 'de:08116:4058:0:4'  # eRtg Olga-/Keplerstr. (Deizisau Deizisau)
	PLOCHINGER_STR = 'de:08116:4060:0:3'  # eRtg Keplerstraße (Deizisau Deizisau)
	DEIZISAU_PLOCHINGER_STRASSE = 'de:08116:4060:0:3'  # eRtg Keplerstraße (Deizisau Deizisau)
	OBERESSLINGEN_HIRSCHLANDSTR = 'de:08116:4062'
	OBERESSLINGEN_HIRSCHLANDSTR_1 = 'de:08116:4062:1:3'  # eRtg Flandernstraße (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_HIRSCHLANDSTR_2 = 'de:08116:4062:1:4'  # eRtg Hirschlandhof (Esslingen am Neckar Oberesslingen)
	HIRSCHLANDSTRASSE = 'de:08116:4062'
	HIRSCHLANDSTRASSE_1 = 'de:08116:4062:1:3'  # eRtg Flandernstraße (Esslingen am Neckar Oberesslingen)
	HIRSCHLANDSTRASSE_2 = 'de:08116:4062:1:4'  # eRtg Hirschlandhof (Esslingen am Neckar Oberesslingen)
	LENAUDENKMAL = 'de:08116:4063:0:3'  # Rtg Hirrschlandkopf/Kliniken (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__LENAUDENKMAL = 'de:08116:4063:0:3'  # Rtg Hirrschlandkopf/Kliniken (Esslingen am Neckar Esslingen (N))
	SPORTPARK_WEIL = 'de:08116:4064'
	SPORTPARK_WEIL_1 = 'de:08116:4064:0:3'  # Rtg Mettingen/Hedelfingen (Esslingen am Neckar Weil)
	SPORTPARK_WEIL_2 = 'de:08116:4064:0:4'  # Rtg Pliensauvorstadt (Esslingen am Neckar Weil)
	WEIL_SPORTPARK_WEIL = 'de:08116:4064'
	WEIL_SPORTPARK_WEIL_1 = 'de:08116:4064:0:3'  # Rtg Mettingen/Hedelfingen (Esslingen am Neckar Weil)
	WEIL_SPORTPARK_WEIL_2 = 'de:08116:4064:0:4'  # Rtg Pliensauvorstadt (Esslingen am Neckar Weil)
	WEIL_WANNENRAIN = 'de:08116:4065'
	WEIL_WANNENRAIN_1 = 'de:08116:4065:0:3'  # Rtg Mettingen/Hedelfingen (Esslingen am Neckar Weil)
	WEIL_WANNENRAIN_2 = 'de:08116:4065:0:4'  # Rtg Pliensauvorstadt (Esslingen am Neckar Weil)
	WANNENRAIN = 'de:08116:4065'
	WANNENRAIN_1 = 'de:08116:4065:0:3'  # Rtg Mettingen/Hedelfingen (Esslingen am Neckar Weil)
	WANNENRAIN_2 = 'de:08116:4065:0:4'  # Rtg Pliensauvorstadt (Esslingen am Neckar Weil)
	MERCEDES_BENZ = 'de:08111:4066:0:3'  # Rtg Am Mittelkai (Stuttgart Stuttgart)
	AM_MITTELKAI = 'de:08111:4067:0:3'  # Rtg In den Stegwiesen (Stuttgart Stuttgart)
	AMSTETTER_STRASSE = 'de:08111:4068'
	AMSTETTER_STRASSE_1 = 'de:08111:4068:0:3'  # Rtg Hedelfingen (Stuttgart Stuttgart)
	AMSTETTER_STRASSE_2 = 'de:08111:4068:0:4'  # Rtg Manzeller Weg (Stuttgart Stuttgart)
	OBERHOF = 'de:08116:4070'
	OBERHOF_1 = 'de:08116:4070:0:3'  # eRtg Rotes Kreuz (Esslingen am Neckar Oberhof)
	OBERHOF_2 = 'de:08116:4070:0:4'  # eRtg Steinhalde/Kirche/THG (Esslingen am Neckar Oberhof)
	ROTES_KREUZ = 'de:08116:4071'
	ROTES_KREUZ_1 = 'de:08116:4071:0:3'  # Linie 106 Rtg Oberhof (Esslingen am Neckar Liebersbronn)
	ROTES_KREUZ_2 = 'de:08116:4071:0:4'  # Rtg Oberhof (Esslingen am Neckar Liebersbronn)
	ROTES_KREUZ_3 = 'de:08116:4071:0:5'  # Linie 106 Rtg Weißer Stein (Esslingen am Neckar Liebersbronn)
	ROTES_KREUZ_4 = 'de:08116:4071:0:6'  # Rtg Aichschieß Industriegebiet (Esslingen am Neckar Liebersbronn)
	LIEBERSBR__ROTES_KREUZ = 'de:08116:4071'
	LIEBERSBR__ROTES_KREUZ_1 = 'de:08116:4071:0:3'  # Linie 106 Rtg Oberhof (Esslingen am Neckar Liebersbronn)
	LIEBERSBR__ROTES_KREUZ_2 = 'de:08116:4071:0:4'  # Rtg Oberhof (Esslingen am Neckar Liebersbronn)
	LIEBERSBR__ROTES_KREUZ_3 = 'de:08116:4071:0:5'  # Linie 106 Rtg Weißer Stein (Esslingen am Neckar Liebersbronn)
	LIEBERSBR__ROTES_KREUZ_4 = 'de:08116:4071:0:6'  # Rtg Aichschieß Industriegebiet (Esslingen am Neckar Liebersbronn)
	ALTBACHER_WEG = 'de:08116:4072'
	ALTBACHER_WEG_1 = 'de:08116:4072:0:3'  # eRtg Weißer Stein (Esslingen am Neckar Liebersbronn)
	ALTBACHER_WEG_2 = 'de:08116:4072:0:4'  # eRtg Rotes Kreuz (Esslingen am Neckar Liebersbronn)
	LIEBERSBR__ALTBACHER_WEG = 'de:08116:4072'
	LIEBERSBR__ALTBACHER_WEG_1 = 'de:08116:4072:0:3'  # eRtg Weißer Stein (Esslingen am Neckar Liebersbronn)
	LIEBERSBR__ALTBACHER_WEG_2 = 'de:08116:4072:0:4'  # eRtg Rotes Kreuz (Esslingen am Neckar Liebersbronn)
	HIRSCHLANDHOF = 'de:08116:4073'
	HIRSCHLANDHOF_1 = 'de:08116:4073:0:3'  # eRtg Hirschlandstraße (Esslingen am Neckar Oberesslingen)
	HIRSCHLANDHOF_2 = 'de:08116:4073:0:4'  # eRtg Kreuzstraße (Esslingen am Neckar Oberesslingen)
	OBERESSL__HIRSCHLANDHOF = 'de:08116:4073'
	OBERESSL__HIRSCHLANDHOF_1 = 'de:08116:4073:0:3'  # eRtg Hirschlandstraße (Esslingen am Neckar Oberesslingen)
	OBERESSL__HIRSCHLANDHOF_2 = 'de:08116:4073:0:4'  # eRtg Kreuzstraße (Esslingen am Neckar Oberesslingen)
	WEISSER_STEIN = 'de:08116:4074'
	WEISSER_STEIN_1 = 'de:08116:4074:0:3'  # eRtg Kreissparkasse (Plochingen Plochingen)
	WEISSER_STEIN_2 = 'de:08116:4074:0:4'  # eRtg Kreuzung (Plochingen Plochingen)
	PLOCHINGEN_WEISSER_STEIN = 'de:08116:4074'
	PLOCHINGEN_WEISSER_STEIN_1 = 'de:08116:4074:0:3'  # eRtg Kreissparkasse (Plochingen Plochingen)
	PLOCHINGEN_WEISSER_STEIN_2 = 'de:08116:4074:0:4'  # eRtg Kreuzung (Plochingen Plochingen)
	BALTMANNSW__KREISSPARKASSE = 'de:08116:4075'
	BALTMANNSW__KREISSPARKASSE_1 = 'de:08116:4075:0:3'  # Rtg Reichenbacher Str. (Baltmannsweiler Baltmannsweiler)
	BALTMANNSW__KREISSPARKASSE_2 = 'de:08116:4075:0:4'  # Rtg Hohengehren/Weißer Stein (Baltmannsweiler Baltmannsweiler)
	KREISSPARKASSE_1 = 'de:08116:4075:0:3'  # Rtg Reichenbacher Str. (Baltmannsweiler Baltmannsweiler)
	KREISSPARKASSE_2 = 'de:08116:4075:0:4'  # Rtg Hohengehren/Weißer Stein (Baltmannsweiler Baltmannsweiler)
	KREISSPARKASSE_3 = 'de:08116:4226:0:3'  # eRtg Plochingen (Plochingen Plochingen)
	KREISSPARKASSE_4 = 'de:08116:4299:1:3'  # eRtg Lerchenstraße (Oberboihingen Oberboihingen)
	KREISSPARKASSE_5 = 'de:08116:4299:1:4'  # eRtg Stattmannstraße (Oberboihingen Oberboihingen)
	KREISSPARKASSE_6 = 'de:08117:4033:0:H1'  # Rtg Friedhof (Börtlingen Börtlingen)
	KREISSPARKASSE_7 = 'de:08117:4033:0:R1'  # Rtg Löwen (Börtlingen Börtlingen)
	BALTMANNSW__SCHURWALDHÖHE = 'de:08116:4076'
	BALTMANNSW__SCHURWALDHÖHE_1 = 'de:08116:4076:0:3'  # eRtg Tiefgarage/Waldhorn/Ortsmitte (Baltmannsweiler Baltmannsweiler)
	BALTMANNSW__SCHURWALDHÖHE_2 = 'de:08116:4076:0:4'  # eRtg Weißer Stein/Kreissparkasse (Baltmannsweiler Baltmannsweiler)
	SCHURWALDHÖHE = 'de:08116:4076'
	SCHURWALDHÖHE_1 = 'de:08116:4076:0:3'  # eRtg Tiefgarage/Waldhorn/Ortsmitte (Baltmannsweiler Baltmannsweiler)
	SCHURWALDHÖHE_2 = 'de:08116:4076:0:4'  # eRtg Weißer Stein/Kreissparkasse (Baltmannsweiler Baltmannsweiler)
	WALDHORN_1 = 'de:08116:4077:0:3'  # eRtg Ortsmitte (Baltmannsweiler Hohengehren)
	WALDHORN_2 = 'de:08116:4077:0:4'  # eRtg Schurwaldhöhe (Baltmannsweiler Hohengehren)
	WALDHORN_3 = 'de:08117:1031:0:H1'  # Rtg Eislingen/Salach (Eislingen/Fils Eislingen (F))
	WALDHORN_4 = 'de:08117:1031:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	HOHENGEHREN_WALDHORN = 'de:08116:4077'
	HOHENGEHREN_WALDHORN_1 = 'de:08116:4077:0:3'  # eRtg Ortsmitte (Baltmannsweiler Hohengehren)
	HOHENGEHREN_WALDHORN_2 = 'de:08116:4077:0:4'  # eRtg Schurwaldhöhe (Baltmannsweiler Hohengehren)
	HOHENGEHREN_KINDERGARTEN = 'de:08116:4078:0:4'  # eRtg Ortsmitte (Baltmannsweiler Hohengehren)
	KINDERGARTEN_1 = 'de:08116:4078:0:4'  # eRtg Ortsmitte (Baltmannsweiler Hohengehren)
	KINDERGARTEN_2 = 'de:08119:4926:0:3'  # eRtg Marktplatz (Murrhardt Fornsbach)
	KINDERGARTEN_3 = 'de:08119:4926:0:4'  # eRtg L 1066 (Murrhardt Fornsbach)
	KINDERGARTEN_4 = 'de:08118:5667:0:3'  # eRtg Kleinsachsenheimer Straße (Bietigheim-Bissingen Metterzimmern)
	KINDERGARTEN_5 = 'de:08118:5667:0:4'  # eRtg Bietigheimer Straße (Bietigheim-Bissingen Metterzimmern)
	KINDERGARTEN_6 = 'de:08119:6672:0:3'  # eRtg Hüttlen (Spiegelberg Jux)
	KINDERGARTEN_7 = 'de:08119:6672:0:4'  # eRtg Jux (Spiegelberg Jux)
	KINDERGARTEN_8 = 'de:08118:7661:0:3'  # Rtg Rtg Häfnerhaslach (Sachsenheim Ochsenbach)
	KINDERGARTEN_9 = 'de:08118:7661:0:4'  # Rtg Spielberg (Sachsenheim Ochsenbach)
	KINDERGARTEN_10 = 'de:08117:7405:0:R1'  # Rtg Kirche (Ebersbach an der Fils Roßwälden)
	KINDERGARTEN_11 = 'de:08117:7676:0:H1'  # Rtg Baiereck (Uhingen Baiereck)
	KINDERGARTEN_12 = 'de:08117:7676:0:R1'  # Rtg Nassach (Uhingen Baiereck)
	KINDERGARTEN_13 = 'de:08235:10124:0:3'  # Rtg Vollmaringer-/Birkenweg (Nagold Nagold)
	KINDERGARTEN_14 = 'de:08235:10124:0:4'  # Rtg Heckengäustr. (Nagold Nagold)
	TIEFGARAGE = 'de:08116:4079:1:4'  # eRtg Kindergarten (Baltmannsweiler Hohengehren)
	HOHENGEHREN_TIEFGARAGE = 'de:08116:4079:1:4'  # eRtg Kindergarten (Baltmannsweiler Hohengehren)
	GOLDBODEN = 'de:08119:4080'
	GOLDBODEN_1 = 'de:08119:4080:0:3'  # Rtg. Thomashardt/Schlichten (Winterbach Winterbach)
	GOLDBODEN_2 = 'de:08119:4080:0:4'  # Rtg Hohengehren (Winterbach Winterbach)
	GOLDBODEN_3 = 'de:08119:4080:0:5'  # Rtg. Winterbach Engelberg (262) (Winterbach Winterbach)
	GOLDBODEN_4 = 'de:08119:4080:0:6'  # Rtg. Winterb. Engelb.Waldorfschule (Winterbach Winterbach)
	WINTERBACH_GOLDBODEN = 'de:08119:4080'
	WINTERBACH_GOLDBODEN_1 = 'de:08119:4080:0:3'  # Rtg. Thomashardt/Schlichten (Winterbach Winterbach)
	WINTERBACH_GOLDBODEN_2 = 'de:08119:4080:0:4'  # Rtg Hohengehren (Winterbach Winterbach)
	WINTERBACH_GOLDBODEN_3 = 'de:08119:4080:0:5'  # Rtg. Winterbach Engelberg (262) (Winterbach Winterbach)
	WINTERBACH_GOLDBODEN_4 = 'de:08119:4080:0:6'  # Rtg. Winterb. Engelb.Waldorfschule (Winterbach Winterbach)
	WINTERBACH_MANOLZWEILER = 'de:08119:4081:0:3'  # eRtg alle Richtungen (Winterbach Winterbach)
	MANOLZWEILER = 'de:08119:4081:0:3'  # eRtg alle Richtungen (Winterbach Winterbach)
	WINTERBACH_ENGELBERG = 'de:08119:4082'
	WINTERBACH_ENGELBERG_1 = 'de:08119:4082:0:3'  # Rtg. Engelberg/Goldboden (Winterbach Winterbach)
	WINTERBACH_ENGELBERG_2 = 'de:08119:4082:0:4'  # Rtg Bahnunterführung (Winterbach Winterbach)
	WINTERBACH_ENGELBERG_3 = 'de:08119:4082:0:5'  # Rtg Manolzw./Goldboden (Winterbach Winterbach)
	WINTERBACH_BAHNUNTERF = 'de:08119:4083'
	WINTERBACH_BAHNUNTERF_1 = 'de:08119:4083:1:3'  # Rtg Engelberg (Winterbach Winterbach)
	WINTERBACH_BAHNUNTERF_2 = 'de:08119:4083:1:4'  # Rtg Pflegeheim/Remsbrücke (Winterbach Winterbach)
	BAHNUNTERFÜHRUNG = 'de:08119:4083'
	BAHNUNTERFÜHRUNG_1 = 'de:08119:4083:1:3'  # Rtg Engelberg (Winterbach Winterbach)
	BAHNUNTERFÜHRUNG_2 = 'de:08119:4083:1:4'  # Rtg Pflegeheim/Remsbrücke (Winterbach Winterbach)
	OBERESSL__SIRNAUER_BRÜCKE = 'de:08116:4084'
	OBERESSL__SIRNAUER_BRÜCKE_1 = 'de:08116:4084:0:3'  # Rtg Sirnauer Hof (Esslingen am Neckar Oberesslingen)
	OBERESSL__SIRNAUER_BRÜCKE_2 = 'de:08116:4084:0:4'  # Rtg Alleenstr./Max-Planck-Str (Esslingen am Neckar Oberesslingen)
	SIRNAUER_BRÜCKE = 'de:08116:4084'
	SIRNAUER_BRÜCKE_1 = 'de:08116:4084:0:3'  # Rtg Sirnauer Hof (Esslingen am Neckar Oberesslingen)
	SIRNAUER_BRÜCKE_2 = 'de:08116:4084:0:4'  # Rtg Alleenstr./Max-Planck-Str (Esslingen am Neckar Oberesslingen)
	KÖNGEN_UNTERDORFSTRASSE = 'de:08116:4086:0:3'  # eRtg Steinbruchstraße (Köngen Köngen)
	UNTERDORFSTRASSE = 'de:08116:4086:0:3'  # eRtg Steinbruchstraße (Köngen Köngen)
	KÖNGEN_RATHAUS = 'de:08116:4088:0:6'  # Ersatzhalt Li 151 (Köngen Köngen)
	KÖNGEN_LINDE = 'de:08116:4089:0:3'  # eRtg Betz (Köngen Köngen)
	BETZ = 'de:08116:4090:0:3'  # eRtg Alup (Köngen Köngen)
	KÖNGEN_BETZ = 'de:08116:4090:0:3'  # eRtg Alup (Köngen Köngen)
	ALUP = 'de:08116:4091:0:3'  # eRtg Wilhelm-Maier-Straße (Köngen Köngen)
	KÖNGEN_ALUP = 'de:08116:4091:0:3'  # eRtg Wilhelm-Maier-Straße (Köngen Köngen)
	PLOCHINGER_STRASSE = 'de:08116:4092:0:3'  # eRtg Unterdorfstraße (Köngen Köngen)
	KÖNGEN_PLOCHINGER_STR = 'de:08116:4092:0:3'  # eRtg Unterdorfstraße (Köngen Köngen)
	UNTERENSINGEN_LINDE = 'de:08116:4093'
	UNTERENSINGEN_LINDE_1 = 'de:08116:4093:0:3'  # eRtg Stern (Unterensingen Unterensingen)
	UNTERENSINGEN_LINDE_2 = 'de:08116:4093:0:4'  # eRtg Zizishausen (Unterensingen Unterensingen)
	KATHARINENSTAFFEL = 'de:08116:4094'
	KATHARINENSTAFFEL_1 = 'de:08116:4094:0:3'  # eRtg Panorama-Klinik/Lenaudenkmal (Esslingen am Neckar Esslingen (N))
	KATHARINENSTAFFEL_2 = 'de:08116:4094:0:4'  # eRtg Neckar Forum/Hochschule (Esslingen am Neckar Esslingen (N))
	ESSL__N__KATHARINENSTAFFEL = 'de:08116:4094'
	ESSL__N__KATHARINENSTAFFEL_1 = 'de:08116:4094:0:3'  # eRtg Panorama-Klinik/Lenaudenkmal (Esslingen am Neckar Esslingen (N))
	ESSL__N__KATHARINENSTAFFEL_2 = 'de:08116:4094:0:4'  # eRtg Neckar Forum/Hochschule (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__FINANZAMT = 'de:08116:4095:0:3'  # Rtg Neckar Forum (Esslingen am Neckar Esslingen (N))
	KENNENB__GERIATRISCHES_ZENTRUM = 'de:08116:4097:0:3'  # eRtg Kennenburg (Esslingen am Neckar Kennenburg)
	GERIATRISCHES_ZENTRUM = 'de:08116:4097:0:3'  # eRtg Kennenburg (Esslingen am Neckar Kennenburg)
	KENNENBURG = 'de:08116:4098'
	KENNENBURG_1 = 'de:08116:4098:0:3'  # Rtg Jägerhaus (Esslingen am Neckar Kennenburg)
	KENNENBURG_2 = 'de:08116:4098:0:4'  # Rtg ZOB (Esslingen am Neckar Kennenburg)
	KENNENBURG_3 = 'de:08116:4098:0:5'  # eRtg Geriatrisches Zentrum (Esslingen am Neckar Kennenburg)
	KOHLERWEG = 'de:08116:4099'
	KOHLERWEG_1 = 'de:08116:4099:0:3'  # eRtg Anhäuserstraße (Esslingen am Neckar Hegensberg)
	KOHLERWEG_2 = 'de:08116:4099:0:4'  # eRtg Kennenburg (Esslingen am Neckar Hegensberg)
	HEGENSBERG_KOHLERWEG = 'de:08116:4099'
	HEGENSBERG_KOHLERWEG_1 = 'de:08116:4099:0:3'  # eRtg Anhäuserstraße (Esslingen am Neckar Hegensberg)
	HEGENSBERG_KOHLERWEG_2 = 'de:08116:4099:0:4'  # eRtg Kennenburg (Esslingen am Neckar Hegensberg)
	ANHÄUSERSTRASSE = 'de:08116:4100'
	ANHÄUSERSTRASSE_1 = 'de:08116:4100:0:3'  # eRtg Kriegerdenkmal/Linde (Esslingen am Neckar Hegensberg)
	ANHÄUSERSTRASSE_2 = 'de:08116:4100:0:4'  # eRtg Kohlerweg (Esslingen am Neckar Hegensberg)
	HEGENSBERG_ANHÄUSERSTRASSE = 'de:08116:4100'
	HEGENSBERG_ANHÄUSERSTRASSE_1 = 'de:08116:4100:0:3'  # eRtg Kriegerdenkmal/Linde (Esslingen am Neckar Hegensberg)
	HEGENSBERG_ANHÄUSERSTRASSE_2 = 'de:08116:4100:0:4'  # eRtg Kohlerweg (Esslingen am Neckar Hegensberg)
	KRIEGERDENKMAL_1 = 'de:08116:4101:0:3'  # eRtg Linde/Kennenburg (Esslingen am Neckar Hegensberg)
	KRIEGERDENKMAL_2 = 'de:08116:4101:0:4'  # eRtg Kirche (Esslingen am Neckar Hegensberg)
	KRIEGERDENKMAL_3 = 'de:08116:4472:1:3'  # eRtg Raiffeisenstraße (Aichtal Grötzingen)
	HEGENSB__KRIEGERDENKMAL = 'de:08116:4101'
	HEGENSB__KRIEGERDENKMAL_1 = 'de:08116:4101:0:3'  # eRtg Linde/Kennenburg (Esslingen am Neckar Hegensberg)
	HEGENSB__KRIEGERDENKMAL_2 = 'de:08116:4101:0:4'  # eRtg Kirche (Esslingen am Neckar Hegensberg)
	LIEBERSBRONN_KIRCHE = 'de:08116:4102'
	LIEBERSBRONN_KIRCHE_1 = 'de:08116:4102:0:3'  # eRtg Kriegerdenkmal (Esslingen am Neckar Liebersbronn)
	LIEBERSBRONN_KIRCHE_2 = 'de:08116:4102:0:4'  # eRtg Neue Straße (Esslingen am Neckar Liebersbronn)
	NEUE_STRASSE_1 = 'de:08116:4103:0:3'  # Rtg Alte Steige (Esslingen am Neckar Liebersbronn)
	NEUE_STRASSE_2 = 'de:08116:4103:0:4'  # Rtg Kirche (Esslingen am Neckar Liebersbronn)
	NEUE_STRASSE_3 = 'de:08119:6844:0:3'  # eRtg Ankunft/Sachsenweiler (Backnang Steinbach)
	NEUE_STRASSE_4 = 'de:08119:6844:0:4'  # eRtg Lindenplatz (Backnang Steinbach)
	LIEBERSBRONN_NEUE_STRASSE = 'de:08116:4103'
	LIEBERSBRONN_NEUE_STRASSE_1 = 'de:08116:4103:0:3'  # Rtg Alte Steige (Esslingen am Neckar Liebersbronn)
	LIEBERSBRONN_NEUE_STRASSE_2 = 'de:08116:4103:0:4'  # Rtg Kirche (Esslingen am Neckar Liebersbronn)
	LIEBERSBRONN_ALTE_STEIGE = 'de:08116:4104'
	LIEBERSBRONN_ALTE_STEIGE_1 = 'de:08116:4104:0:3'  # eRtg Neue Straße (Esslingen am Neckar Liebersbronn)
	LIEBERSBRONN_ALTE_STEIGE_2 = 'de:08116:4104:0:4'  # eRtg Jägerhaus (Esslingen am Neckar Liebersbronn)
	LIEBERSBRONN_JÄGERHAUS = 'de:08116:4105:0:3'  # eRtg Alte Steige (Esslingen am Neckar Liebersbronn)
	JÄGERHAUS = 'de:08116:4105:0:3'  # eRtg Alte Steige (Esslingen am Neckar Liebersbronn)
	GEORGII_WALDSTADION = 'de:08116:4106'
	GEORGII_WALDSTADION_1 = 'de:08116:4106:0:3'  # Rtg Jägerhaus (Esslingen am Neckar Liebersbronn)
	GEORGII_WALDSTADION_2 = 'de:08116:4106:0:4'  # Rtg Dulkhäusle (Esslingen am Neckar Liebersbronn)
	LIEBERSBR__GEORGII_WALDSTADION = 'de:08116:4106'
	LIEBERSBR__GEORGII_WALDSTADION_1 = 'de:08116:4106:0:3'  # Rtg Jägerhaus (Esslingen am Neckar Liebersbronn)
	LIEBERSBR__GEORGII_WALDSTADION_2 = 'de:08116:4106:0:4'  # Rtg Dulkhäusle (Esslingen am Neckar Liebersbronn)
	WIFLINGSH__DULKHÄUSLE = 'de:08116:4107'
	WIFLINGSH__DULKHÄUSLE_1 = 'de:08116:4107:0:3'  # eRtg Georgii-Waldstadion (Esslingen am Neckar Wiflingshausen)
	WIFLINGSH__DULKHÄUSLE_2 = 'de:08116:4107:0:4'  # eRtg Waldackerweg (Esslingen am Neckar Wiflingshausen)
	DULKHÄUSLE = 'de:08116:4107'
	DULKHÄUSLE_1 = 'de:08116:4107:0:3'  # eRtg Georgii-Waldstadion (Esslingen am Neckar Wiflingshausen)
	DULKHÄUSLE_2 = 'de:08116:4107:0:4'  # eRtg Waldackerweg (Esslingen am Neckar Wiflingshausen)
	SULZGRIESER_STEIGE = 'de:08116:4109'
	SULZGRIESER_STEIGE_1 = 'de:08116:4109:0:3'  # eRtg Ebene (Esslingen am Neckar Esslingen (N))
	SULZGRIESER_STEIGE_2 = 'de:08116:4109:0:4'  # eRtg Kleiner Markt (Esslingen am Neckar Esslingen (N))
	ESSL__N__SULZGRIESER_STEIGE = 'de:08116:4109'
	ESSL__N__SULZGRIESER_STEIGE_1 = 'de:08116:4109:0:3'  # eRtg Ebene (Esslingen am Neckar Esslingen (N))
	ESSL__N__SULZGRIESER_STEIGE_2 = 'de:08116:4109:0:4'  # eRtg Kleiner Markt (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__EBENE = 'de:08116:4110'
	ESSLINGEN__N__EBENE_1 = 'de:08116:4110:0:3'  # eRtg Krummenacker (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__EBENE_2 = 'de:08116:4110:0:4'  # eRtg Sulzgrieser Steige (Esslingen am Neckar Esslingen (N))
	EBENE = 'de:08116:4110'
	EBENE_1 = 'de:08116:4110:0:3'  # eRtg Krummenacker (Esslingen am Neckar Esslingen (N))
	EBENE_2 = 'de:08116:4110:0:4'  # eRtg Sulzgrieser Steige (Esslingen am Neckar Esslingen (N))
	KRUMMENACKER = 'de:08116:4111'
	KRUMMENACKER_1 = 'de:08116:4111:0:3'  # eRtg Krone (Esslingen am Neckar Krummenacker)
	KRUMMENACKER_2 = 'de:08116:4111:0:4'  # eRtg Schulzentrum Nord/Ebene (Esslingen am Neckar Krummenacker)
	MAIENWALTERSTRASSE = 'de:08116:4112'
	MAIENWALTERSTRASSE_1 = 'de:08116:4112:0:3'  # eRtg Zentrum (Esslingen am Neckar Sulzgries)
	MAIENWALTERSTRASSE_2 = 'de:08116:4112:0:4'  # eRtg Krummenacker (Esslingen am Neckar Sulzgries)
	SULZGRIES_MAIENWALTERSTRASSE = 'de:08116:4112'
	SULZGRIES_MAIENWALTERSTRASSE_1 = 'de:08116:4112:0:3'  # eRtg Zentrum (Esslingen am Neckar Sulzgries)
	SULZGRIES_MAIENWALTERSTRASSE_2 = 'de:08116:4112:0:4'  # eRtg Krummenacker (Esslingen am Neckar Sulzgries)
	ZENTRUM = 'de:08116:4113'
	ZENTRUM_1 = 'de:08116:4113:0:3'  # Rtg Bürgerhaus RSKN/Bergstr. (Esslingen am Neckar Sulzgries)
	ZENTRUM_2 = 'de:08116:4113:0:4'  # Rtg Maienwalterstraße stadteinwärts (Esslingen am Neckar Sulzgries)
	SULZGRIES_ZENTRUM = 'de:08116:4113'
	SULZGRIES_ZENTRUM_1 = 'de:08116:4113:0:3'  # Rtg Bürgerhaus RSKN/Bergstr. (Esslingen am Neckar Sulzgries)
	SULZGRIES_ZENTRUM_2 = 'de:08116:4113:0:4'  # Rtg Maienwalterstraße stadteinwärts (Esslingen am Neckar Sulzgries)
	BÜRGERHAUS_RSKN = 'de:08116:4114'
	BÜRGERHAUS_RSKN_1 = 'de:08116:4114:0:3'  # eRtg Paradiesweg (Esslingen am Neckar Rüdern)
	BÜRGERHAUS_RSKN_2 = 'de:08116:4114:0:4'  # eRtg Zentrum/Schule (Esslingen am Neckar Rüdern)
	RÜDERN_BÜRGERHAUS_RSKN = 'de:08116:4114'
	RÜDERN_BÜRGERHAUS_RSKN_1 = 'de:08116:4114:0:3'  # eRtg Paradiesweg (Esslingen am Neckar Rüdern)
	RÜDERN_BÜRGERHAUS_RSKN_2 = 'de:08116:4114:0:4'  # eRtg Zentrum/Schule (Esslingen am Neckar Rüdern)
	RÜDERN_PARADIESWEG = 'de:08116:4115'
	RÜDERN_PARADIESWEG_1 = 'de:08116:4115:0:3'  # eRtg Glocke (Esslingen am Neckar Rüdern)
	RÜDERN_PARADIESWEG_2 = 'de:08116:4115:0:4'  # eRtg Bürgerhaus RSKN (Esslingen am Neckar Rüdern)
	PARADIESWEG = 'de:08116:4115'
	PARADIESWEG_1 = 'de:08116:4115:0:3'  # eRtg Glocke (Esslingen am Neckar Rüdern)
	PARADIESWEG_2 = 'de:08116:4115:0:4'  # eRtg Bürgerhaus RSKN (Esslingen am Neckar Rüdern)
	RÜDERN_HINTERER_HOLZWEG = 'de:08116:4116:0:3'  # eRtg Paradiesweg (Esslingen am Neckar Rüdern)
	HINTERER_HOLZWEG = 'de:08116:4116:0:3'  # eRtg Paradiesweg (Esslingen am Neckar Rüdern)
	SULZGRIES_BERGSTRASSE = 'de:08116:4118'
	SULZGRIES_BERGSTRASSE_1 = 'de:08116:4118:0:3'  # eRtg Kelter (Esslingen am Neckar Sulzgries)
	SULZGRIES_BERGSTRASSE_2 = 'de:08116:4118:0:4'  # eRtg Zentrum (Esslingen am Neckar Sulzgries)
	NECKARHALDE_HEIDESTRASSE = 'de:08116:4119'
	NECKARHALDE_HEIDESTRASSE_1 = 'de:08116:4119:0:3'  # eRtg Stahlackerweg (Esslingen am Neckar Neckarhalde)
	NECKARHALDE_HEIDESTRASSE_2 = 'de:08116:4119:0:4'  # eRtg Bergstr. (Esslingen am Neckar Neckarhalde)
	HEIDESTRASSE = 'de:08116:4119'
	HEIDESTRASSE_1 = 'de:08116:4119:0:3'  # eRtg Stahlackerweg (Esslingen am Neckar Neckarhalde)
	HEIDESTRASSE_2 = 'de:08116:4119:0:4'  # eRtg Bergstr. (Esslingen am Neckar Neckarhalde)
	STAHLACKERWEG = 'de:08116:4120'
	STAHLACKERWEG_1 = 'de:08116:4120:0:3'  # eRtg Endstation (Esslingen am Neckar Neckarhalde)
	STAHLACKERWEG_2 = 'de:08116:4120:0:4'  # eRtg Kelter (Esslingen am Neckar Neckarhalde)
	NECKARHALDE_STAHLACKERWEG = 'de:08116:4120'
	NECKARHALDE_STAHLACKERWEG_1 = 'de:08116:4120:0:3'  # eRtg Endstation (Esslingen am Neckar Neckarhalde)
	NECKARHALDE_STAHLACKERWEG_2 = 'de:08116:4120:0:4'  # eRtg Kelter (Esslingen am Neckar Neckarhalde)
	NECKARHALDE_HERRENWIESENWEG = 'de:08116:4121:0:3'  # eRtg Stahlackerweg (Esslingen am Neckar Neckarhalde)
	HERRENWIESENWEG = 'de:08116:4121:0:3'  # eRtg Stahlackerweg (Esslingen am Neckar Neckarhalde)
	ESSL__N__PANORAMA_KLINIK = 'de:08116:4122'
	ESSL__N__PANORAMA_KLINIK_1 = 'de:08116:4122:0:3'  # Rtg Burg (Esslingen am Neckar Esslingen (N))
	ESSL__N__PANORAMA_KLINIK_2 = 'de:08116:4122:0:4'  # Rtg Stadt (Esslingen am Neckar Esslingen (N))
	PANORAMA_KLINIK = 'de:08116:4122'
	PANORAMA_KLINIK_1 = 'de:08116:4122:0:3'  # Rtg Burg (Esslingen am Neckar Esslingen (N))
	PANORAMA_KLINIK_2 = 'de:08116:4122:0:4'  # Rtg Stadt (Esslingen am Neckar Esslingen (N))
	BURG = 'de:08116:4123'
	BURG_1 = 'de:08116:4123:0:3'  # eRtg Hohenkreuz/Lenaudenkmal (Esslingen am Neckar Esslingen (N))
	BURG_2 = 'de:08116:4123:0:4'  # eRtg Panorama-Klinik (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__BURG = 'de:08116:4123'
	ESSLINGEN__N__BURG_1 = 'de:08116:4123:0:3'  # eRtg Hohenkreuz/Lenaudenkmal (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__BURG_2 = 'de:08116:4123:0:4'  # eRtg Panorama-Klinik (Esslingen am Neckar Esslingen (N))
	HOHENKREUZ = 'de:08116:4124'
	HOHENKREUZ_1 = 'de:08116:4124:0:3'  # eRtg Wäldenbronn Kreuzung (Esslingen am Neckar Hohenkreuz)
	HOHENKREUZ_2 = 'de:08116:4124:0:4'  # eRtg Seracher Straße/Burg (Esslingen am Neckar Hohenkreuz)
	WÄLDENBRONN_KREUZUNG = 'de:08116:4125'
	WÄLDENBRONN_KREUZUNG_1 = 'de:08116:4125:0:5'  # Rtg Langer Weg/Kirchackerstr. (Esslingen am Neckar Hohenkreuz)
	WÄLDENBRONN_KREUZUNG_2 = 'de:08116:4125:0:6'  # Rtg Hohenkreuz (Esslingen am Neckar Hohenkreuz)
	WÄLDENBRONN_KREUZUNG_3 = 'de:08116:4125:0:7'  # Rtg Kirchackerstr. (Esslingen am Neckar Hohenkreuz)
	WÄLDENBRONN_KREUZUNG_4 = 'de:08116:4125:0:8'  # Rtg Bärenwiesen/Hohenkreuz/Langer Weg (Esslingen am Neckar Hohenkreuz)
	HKR__WÄLDENBRONN_KREUZUNG = 'de:08116:4125'
	HKR__WÄLDENBRONN_KREUZUNG_1 = 'de:08116:4125:0:5'  # Rtg Langer Weg/Kirchackerstr. (Esslingen am Neckar Hohenkreuz)
	HKR__WÄLDENBRONN_KREUZUNG_2 = 'de:08116:4125:0:6'  # Rtg Hohenkreuz (Esslingen am Neckar Hohenkreuz)
	HKR__WÄLDENBRONN_KREUZUNG_3 = 'de:08116:4125:0:7'  # Rtg Kirchackerstr. (Esslingen am Neckar Hohenkreuz)
	HKR__WÄLDENBRONN_KREUZUNG_4 = 'de:08116:4125:0:8'  # Rtg Bärenwiesen/Hohenkreuz/Langer Weg (Esslingen am Neckar Hohenkreuz)
	WÄLDENBR__LANGER_WEG = 'de:08116:4126'
	WÄLDENBR__LANGER_WEG_1 = 'de:08116:4126:0:3'  # eRtg Brunnen (Esslingen am Neckar Wäldenbronn)
	WÄLDENBR__LANGER_WEG_2 = 'de:08116:4126:0:4'  # eRtg Wäldenbronn Kreuzung (Esslingen am Neckar Wäldenbronn)
	LANGER_WEG = 'de:08116:4126'
	LANGER_WEG_1 = 'de:08116:4126:0:3'  # eRtg Brunnen (Esslingen am Neckar Wäldenbronn)
	LANGER_WEG_2 = 'de:08116:4126:0:4'  # eRtg Wäldenbronn Kreuzung (Esslingen am Neckar Wäldenbronn)
	BRUNNEN = 'de:08116:4127'
	BRUNNEN_1 = 'de:08116:4127:0:3'  # eRtg Brückle (Esslingen am Neckar Wäldenbronn)
	BRUNNEN_2 = 'de:08116:4127:0:4'  # eRtg Langer Weg (Esslingen am Neckar Wäldenbronn)
	WÄLDENBRONN_BRUNNEN = 'de:08116:4127'
	WÄLDENBRONN_BRUNNEN_1 = 'de:08116:4127:0:3'  # eRtg Brückle (Esslingen am Neckar Wäldenbronn)
	WÄLDENBRONN_BRUNNEN_2 = 'de:08116:4127:0:4'  # eRtg Langer Weg (Esslingen am Neckar Wäldenbronn)
	BRÜCKLE = 'de:08116:4128'
	BRÜCKLE_1 = 'de:08116:4128:0:3'  # eRtg Betriebshof (Esslingen am Neckar Wäldenbronn)
	BRÜCKLE_2 = 'de:08116:4128:0:4'  # eRtg Brunnen (Esslingen am Neckar Wäldenbronn)
	WÄLDENBRONN_BRÜCKLE = 'de:08116:4128'
	WÄLDENBRONN_BRÜCKLE_1 = 'de:08116:4128:0:3'  # eRtg Betriebshof (Esslingen am Neckar Wäldenbronn)
	WÄLDENBRONN_BRÜCKLE_2 = 'de:08116:4128:0:4'  # eRtg Brunnen (Esslingen am Neckar Wäldenbronn)
	WÄLDENBRONN_OBERTALWEG = 'de:08116:4129:1:4'  # eRtg Esslingen/Brückle (Esslingen am Neckar Wäldenbronn)
	OBERTALWEG = 'de:08116:4129:1:4'  # eRtg Esslingen/Brückle (Esslingen am Neckar Wäldenbronn)
	SANKT_BERNHARDT_FLANDERNSTR = 'de:08116:4130'
	SANKT_BERNHARDT_FLANDERNSTR_1 = 'de:08116:4130:0:3'  # eRtg Eugen-Bolz-Straße (Esslingen am Neckar St. Bernhardt)
	SANKT_BERNHARDT_FLANDERNSTR_2 = 'de:08116:4130:0:4'  # eRtg Hirschlandkopf (Esslingen am Neckar St. Bernhardt)
	FLANDERNSTRASSE = 'de:08116:4130'
	FLANDERNSTRASSE_1 = 'de:08116:4130:0:3'  # eRtg Eugen-Bolz-Straße (Esslingen am Neckar St. Bernhardt)
	FLANDERNSTRASSE_2 = 'de:08116:4130:0:4'  # eRtg Hirschlandkopf (Esslingen am Neckar St. Bernhardt)
	EUGEN_BOLZ_STRASSE = 'de:08116:4131'
	EUGEN_BOLZ_STRASSE_1 = 'de:08116:4131:0:3'  # eRtg Kirchackerstraße (Esslingen am Neckar St. Bernhardt)
	EUGEN_BOLZ_STRASSE_2 = 'de:08116:4131:0:4'  # eRtg Flandernstraße (Esslingen am Neckar St. Bernhardt)
	SANKT_BERNHARDT_E__BOLZ_STRASSE = 'de:08116:4131'
	SANKT_BERNHARDT_E__BOLZ_STRASSE_1 = 'de:08116:4131:0:3'  # eRtg Kirchackerstraße (Esslingen am Neckar St. Bernhardt)
	SANKT_BERNHARDT_E__BOLZ_STRASSE_2 = 'de:08116:4131:0:4'  # eRtg Flandernstraße (Esslingen am Neckar St. Bernhardt)
	KIRCHACKERSTRASSE = 'de:08116:4132'
	KIRCHACKERSTRASSE_1 = 'de:08116:4132:0:3'  # eRtg Wäldenbronn Kreuzung (Esslingen am Neckar St. Bernhardt)
	KIRCHACKERSTRASSE_2 = 'de:08116:4132:0:4'  # eRtg Eugen-Bolz-Straße (Esslingen am Neckar St. Bernhardt)
	SANKT_BERNHARDT_KIRCHACKERSTR = 'de:08116:4132'
	SANKT_BERNHARDT_KIRCHACKERSTR_1 = 'de:08116:4132:0:3'  # eRtg Wäldenbronn Kreuzung (Esslingen am Neckar St. Bernhardt)
	SANKT_BERNHARDT_KIRCHACKERSTR_2 = 'de:08116:4132:0:4'  # eRtg Eugen-Bolz-Straße (Esslingen am Neckar St. Bernhardt)
	BÄRENWIESEN = 'de:08116:4133'
	BÄRENWIESEN_1 = 'de:08116:4133:0:3'  # eRtg Seracher Straße (Esslingen am Neckar Serach)
	BÄRENWIESEN_2 = 'de:08116:4133:0:4'  # eRtg Wäldenbronn Kreuzung (Esslingen am Neckar Serach)
	SERACH_BÄRENWIESEN = 'de:08116:4133'
	SERACH_BÄRENWIESEN_1 = 'de:08116:4133:0:3'  # eRtg Seracher Straße (Esslingen am Neckar Serach)
	SERACH_BÄRENWIESEN_2 = 'de:08116:4133:0:4'  # eRtg Wäldenbronn Kreuzung (Esslingen am Neckar Serach)
	SERACHER_STRASSE = 'de:08116:4134'
	SERACHER_STRASSE_1 = 'de:08116:4134:0:3'  # Rtg Schulzentrum Nord (Esslingen am Neckar Serach)
	SERACHER_STRASSE_2 = 'de:08116:4134:0:4'  # eRtg Bärenwiesen (Esslingen am Neckar Serach)
	SERACH_SERACHER_STRASSE = 'de:08116:4134'
	SERACH_SERACHER_STRASSE_1 = 'de:08116:4134:0:3'  # Rtg Schulzentrum Nord (Esslingen am Neckar Serach)
	SERACH_SERACHER_STRASSE_2 = 'de:08116:4134:0:4'  # eRtg Bärenwiesen (Esslingen am Neckar Serach)
	SERACH_SCHULZENTR__NORD = 'de:08116:4135'
	SERACH_SCHULZENTR__NORD_1 = 'de:08116:4135:0:3'  # eRtg Krummenacker/Ebene (Esslingen am Neckar Serach)
	SERACH_SCHULZENTR__NORD_2 = 'de:08116:4135:0:4'  # eRtg Seracher Straße (Esslingen am Neckar Serach)
	SCHULZENTRUM_NORD = 'de:08116:4135'
	SCHULZENTRUM_NORD_1 = 'de:08116:4135:0:3'  # eRtg Krummenacker/Ebene (Esslingen am Neckar Serach)
	SCHULZENTRUM_NORD_2 = 'de:08116:4135:0:4'  # eRtg Seracher Straße (Esslingen am Neckar Serach)
	SANKT_BERNHARDT_ALTE_TALSTRASSE = 'de:08116:4136:0:3'  # Rtg Talstr. (Esslingen am Neckar St. Bernhardt)
	ALTE_TALSTRASSE = 'de:08116:4136:0:3'  # Rtg Talstr. (Esslingen am Neckar St. Bernhardt)
	SANKT_BERNHARDT_TALSTRASSE = 'de:08116:4137'
	SANKT_BERNHARDT_TALSTRASSE_1 = 'de:08116:4137:0:3'  # eRtg Im Unteren Zehen (Esslingen am Neckar St. Bernhardt)
	SANKT_BERNHARDT_TALSTRASSE_2 = 'de:08116:4137:0:4'  # eRtg Sängerkranz (Esslingen am Neckar St. Bernhardt)
	WIFLINGSH__VOLKER_BÖHRINGER_WEG = 'de:08116:4139'
	WIFLINGSH__VOLKER_BÖHRINGER_WEG_1 = 'de:08116:4139:0:3'  # eRtg Hubertusweg (Esslingen am Neckar Wiflingshausen)
	WIFLINGSH__VOLKER_BÖHRINGER_WEG_2 = 'de:08116:4139:0:4'  # eRtg Im Unteren Zehen (Esslingen am Neckar Wiflingshausen)
	VOLKER_BÖHRINGER_WEG = 'de:08116:4139'
	VOLKER_BÖHRINGER_WEG_1 = 'de:08116:4139:0:3'  # eRtg Hubertusweg (Esslingen am Neckar Wiflingshausen)
	VOLKER_BÖHRINGER_WEG_2 = 'de:08116:4139:0:4'  # eRtg Im Unteren Zehen (Esslingen am Neckar Wiflingshausen)
	WIFLINGSH__HUBERTUSWEG = 'de:08116:4140'
	WIFLINGSH__HUBERTUSWEG_1 = 'de:08116:4140:0:3'  # eRtg Waldackerweg (Esslingen am Neckar Wiflingshausen)
	WIFLINGSH__HUBERTUSWEG_2 = 'de:08116:4140:0:4'  # eRtg Grüner Kranz (Esslingen am Neckar Wiflingshausen)
	HUBERTUSWEG = 'de:08116:4140'
	HUBERTUSWEG_1 = 'de:08116:4140:0:3'  # eRtg Waldackerweg (Esslingen am Neckar Wiflingshausen)
	HUBERTUSWEG_2 = 'de:08116:4140:0:4'  # eRtg Grüner Kranz (Esslingen am Neckar Wiflingshausen)
	WALDACKERWEG = 'de:08116:4141'
	WALDACKERWEG_1 = 'de:08116:4141:0:3'  # eRtg Dulkhäusle (Esslingen am Neckar Wiflingshausen)
	WALDACKERWEG_2 = 'de:08116:4141:0:4'  # eRtg Hubertusweg (Esslingen am Neckar Wiflingshausen)
	WIFLINGSH__WALDACKERWEG = 'de:08116:4141'
	WIFLINGSH__WALDACKERWEG_1 = 'de:08116:4141:0:3'  # eRtg Dulkhäusle (Esslingen am Neckar Wiflingshausen)
	WIFLINGSH__WALDACKERWEG_2 = 'de:08116:4141:0:4'  # eRtg Hubertusweg (Esslingen am Neckar Wiflingshausen)
	OBERESSL__ADENAUERBR___OBEN = 'de:08116:4143'
	OBERESSL__ADENAUERBR___OBEN_1 = 'de:08116:4143:0:3'  # Rtg Esslingen (Esslingen am Neckar Oberesslingen)
	OBERESSL__ADENAUERBR___OBEN_2 = 'de:08116:4143:0:4'  # Rtg Berkheim/Sirnau (Esslingen am Neckar Oberesslingen)
	ADENAUERBRÜCKE__OBEN = 'de:08116:4143'
	ADENAUERBRÜCKE__OBEN_1 = 'de:08116:4143:0:3'  # Rtg Esslingen (Esslingen am Neckar Oberesslingen)
	ADENAUERBRÜCKE__OBEN_2 = 'de:08116:4143:0:4'  # Rtg Berkheim/Sirnau (Esslingen am Neckar Oberesslingen)
	SEE_1 = 'de:08116:4144:0:3'  # eRtg Gemeindezentrum (Esslingen am Neckar Berkheim)
	SEE_2 = 'de:08116:4144:0:4'  # eRtg Finkenweg (Esslingen am Neckar Berkheim)
	SEE_3 = 'de:08116:4416:1:3'  # eRtg Rathaus (Bissingen an der Teck Bissingen an der Teck (T))
	SEE_4 = 'de:08115:4790:0:3'  # eRtg Nord (Weil im Schönbuch Weil im Schönbuch)
	SEE_5 = 'de:08115:4790:0:4'  # eRtg Alle Ri außer Nord (Weil im Schönbuch Weil im Schönbuch)
	SEE_6 = 'de:08236:3510:0:1'  # Rtg Marktplatz (Heimsheim Heimsheim)
	SEE_7 = 'de:08236:3510:0:2'  # Rtg Leonberger Str. (Heimsheim Heimsheim)
	BERKHEIM_SEE = 'de:08116:4144'
	BERKHEIM_SEE_1 = 'de:08116:4144:0:3'  # eRtg Gemeindezentrum (Esslingen am Neckar Berkheim)
	BERKHEIM_SEE_2 = 'de:08116:4144:0:4'  # eRtg Finkenweg (Esslingen am Neckar Berkheim)
	WEIL_I__S__SCHAICHHOFSTR = 'de:08115:4145'
	WEIL_I__S__SCHAICHHOFSTR_1 = 'de:08115:4145:0:3'  # Rtg Schaichhofsiedlung (Weil im Schönbuch Weil im Schönbuch)
	WEIL_I__S__SCHAICHHOFSTR_2 = 'de:08115:4145:0:4'  # Rtg Troppel (Weil im Schönbuch Weil im Schönbuch)
	SCHAICHHOFSTRASSE = 'de:08115:4145'
	SCHAICHHOFSTRASSE_1 = 'de:08115:4145:0:3'  # Rtg Schaichhofsiedlung (Weil im Schönbuch Weil im Schönbuch)
	SCHAICHHOFSTRASSE_2 = 'de:08115:4145:0:4'  # Rtg Troppel (Weil im Schönbuch Weil im Schönbuch)
	BERKHEIM_FIRMA_FESTO = 'de:08116:4146'
	BERKHEIM_FIRMA_FESTO_1 = 'de:08116:4146:0:3'  # eRtg Berkheim Vesperstüble (Esslingen am Neckar Berkheim)
	BERKHEIM_FIRMA_FESTO_2 = 'de:08116:4146:0:4'  # eRtg Zollberg Mutzenreisstr. (Esslingen am Neckar Berkheim)
	FIRMA_FESTO = 'de:08116:4146'
	FIRMA_FESTO_1 = 'de:08116:4146:0:3'  # eRtg Berkheim Vesperstüble (Esslingen am Neckar Berkheim)
	FIRMA_FESTO_2 = 'de:08116:4146:0:4'  # eRtg Zollberg Mutzenreisstr. (Esslingen am Neckar Berkheim)
	BERKHEIM_NELLINGER_STRASSE = 'de:08116:4147'
	BERKHEIM_NELLINGER_STRASSE_1 = 'de:08116:4147:0:3'  # eRtg Gemeindezentrum (Esslingen am Neckar Berkheim)
	BERKHEIM_NELLINGER_STRASSE_2 = 'de:08116:4147:0:4'  # eRtg Firma Festo (Esslingen am Neckar Berkheim)
	NELLINGER_STRASSE = 'de:08116:4147'
	NELLINGER_STRASSE_1 = 'de:08116:4147:0:3'  # eRtg Gemeindezentrum (Esslingen am Neckar Berkheim)
	NELLINGER_STRASSE_2 = 'de:08116:4147:0:4'  # eRtg Firma Festo (Esslingen am Neckar Berkheim)
	BERKH__GEMEINDEZENTRUM = 'de:08116:4148'
	BERKH__GEMEINDEZENTRUM_1 = 'de:08116:4148:0:3'  # eRtg Am Wiesengrund (Esslingen am Neckar Berkheim)
	BERKH__GEMEINDEZENTRUM_2 = 'de:08116:4148:0:4'  # eRtg See (Esslingen am Neckar Berkheim)
	BERKH__AM_WIESENGRUND = 'de:08116:4149:1:3'  # eRtg Jakobstraße (Esslingen am Neckar Berkheim)
	AM_WIESENGRUND = 'de:08116:4149:1:3'  # eRtg Jakobstraße (Esslingen am Neckar Berkheim)
	BERKHEIM_JAKOBSTRASSE = 'de:08116:4151:0:3'  # eRtg Kronenstraße (Esslingen am Neckar Berkheim)
	JAKOBSTRASSE = 'de:08116:4151:0:3'  # eRtg Kronenstraße (Esslingen am Neckar Berkheim)
	BERKHEIM_KRONENSTRASSE = 'de:08116:4152:0:3'  # eRtg Gemeindezentrum (Esslingen am Neckar Berkheim)
	KRONENSTRASSE_1 = 'de:08116:4152:0:3'  # eRtg Gemeindezentrum (Esslingen am Neckar Berkheim)
	KRONENSTRASSE_2 = 'de:08116:4165:0:3'  # eRtg Holl (Aichwald Aichelberg (Aichwald))
	KIRCHHEIM__T__KIEBITZWEG = 'de:08116:4153'
	KIRCHHEIM__T__KIEBITZWEG_1 = 'de:08116:4153:0:3'  # Rtg Schlierb. Dreieck (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__KIEBITZWEG_2 = 'de:08116:4153:0:4'  # Rtg Am Wiesenrain (Kirchheim unter Teck Kirchheim (T))
	KIEBITZWEG = 'de:08116:4153'
	KIEBITZWEG_1 = 'de:08116:4153:0:3'  # Rtg Schlierb. Dreieck (Kirchheim unter Teck Kirchheim (T))
	KIEBITZWEG_2 = 'de:08116:4153:0:4'  # Rtg Am Wiesenrain (Kirchheim unter Teck Kirchheim (T))
	AICHSCHIESS_INDUSTRIEGEB = 'de:08116:4154'
	AICHSCHIESS_INDUSTRIEGEB_1 = 'de:08116:4154:0:3'  # eRtg Industriegebiet Nord/Kreuzung (Aichwald Aichschieß)
	AICHSCHIESS_INDUSTRIEGEB_2 = 'de:08116:4154:0:4'  # eRtg Rotes Kreuz (Aichwald Aichschieß)
	KREUZUNG_1 = 'de:08116:4155:0:3'  # Linie 114/115 Rtg Schanbach (Aichwald Aichschieß)
	KREUZUNG_2 = 'de:08116:4155:0:4'  # Linie 106 Rtg Esslingen u. 149 (Aichwald Aichschieß)
	KREUZUNG_3 = 'de:08116:4155:0:6'  # Linie 114/115 Rtg Esslingen (Aichwald Aichschieß)
	KREUZUNG_4 = 'de:08119:4974:0:3'  # eRtg Wasenfeld/Täferhalde (Backnang Heiningen (Backnang))
	KREUZUNG_5 = 'de:08119:4974:0:4'  # eRtg Ungeheuerhof Abzw (Backnang Heiningen (Backnang))
	KREUZUNG_6 = 'de:08117:2060:0:H1'  # Rtg Bürgerhaus (Aichelberg Aichelberg (GP))
	KREUZUNG_7 = 'de:08117:2060:0:R1'  # Rtg Zell (Aichelberg Aichelberg (GP))
	AICHSCHIESS_KREUZUNG = 'de:08116:4155'
	AICHSCHIESS_KREUZUNG_1 = 'de:08116:4155:0:3'  # Linie 114/115 Rtg Schanbach (Aichwald Aichschieß)
	AICHSCHIESS_KREUZUNG_2 = 'de:08116:4155:0:4'  # Linie 106 Rtg Esslingen u. 149 (Aichwald Aichschieß)
	AICHSCHIESS_KREUZUNG_3 = 'de:08116:4155:0:6'  # Linie 114/115 Rtg Esslingen (Aichwald Aichschieß)
	SCHANBACH_KREISVERKEHR = 'de:08116:4156'
	SCHANBACH_KREISVERKEHR_1 = 'de:08116:4156:0:3'  # eRtg Rössle/Schule/Siedlung/Krummhardt (Aichwald Schanbach)
	SCHANBACH_KREISVERKEHR_2 = 'de:08116:4156:0:4'  # eRtg Kreuzung/Aichschieß (Aichwald Schanbach)
	KREISVERKEHR_1 = 'de:08116:4156:0:3'  # eRtg Rössle/Schule/Siedlung/Krummhardt (Aichwald Schanbach)
	KREISVERKEHR_2 = 'de:08116:4156:0:4'  # eRtg Kreuzung/Aichschieß (Aichwald Schanbach)
	KREISVERKEHR_3 = 'de:08118:6490:0:3'  # Rtg Schöckingen (Ditzingen Hirschlanden)
	KREISVERKEHR_4 = 'de:08118:6490:0:4'  # Rtg Turnhalle (Ditzingen Hirschlanden)
	KREISVERKEHR_5 = 'de:08117:3322:0:H1'  # Rtg Kirche (Gammelshausen Gammelshausen)
	KREISVERKEHR_6 = 'de:08117:3322:0:R1'  # Rtg Dürnau (Gammelshausen Gammelshausen)
	KREISVERKEHR_7 = 'de:08117:3322:2:3'  # Rtg Heiningen (Gammelshausen Gammelshausen)
	KREISVERKEHR_8 = 'de:08117:3322:2:4'  # Rtg Dürnau Abzw. Kornberghalle (Gammelshausen Gammelshausen)
	SCHANBACH_SCHULE = 'de:08116:4157:0:3'  # eRtg Krummhardt Abzweig (Aichwald Schanbach)
	RÖSSLE = 'de:08116:4159'
	RÖSSLE_1 = 'de:08116:4159:0:3'  # Rtg Lobenroter Hof (Aichwald Schanbach)
	RÖSSLE_2 = 'de:08116:4159:0:4'  # Rtg Siedlung/Kreisverkehr (Aichwald Schanbach)
	SCHANBACH_RÖSSLE = 'de:08116:4159'
	SCHANBACH_RÖSSLE_1 = 'de:08116:4159:0:3'  # Rtg Lobenroter Hof (Aichwald Schanbach)
	SCHANBACH_RÖSSLE_2 = 'de:08116:4159:0:4'  # Rtg Siedlung/Kreisverkehr (Aichwald Schanbach)
	SCHANB__LOBENROTER_HOF = 'de:08116:4160:0:3'  # eRtg Lobenrot Ort (Aichwald Schanbach)
	LOBENROTER_HOF = 'de:08116:4160:0:3'  # eRtg Lobenrot Ort (Aichwald Schanbach)
	LOBENROT_ORT = 'de:08116:4161:0:3'  # eRtg Rössle (Aichwald Lobenrot)
	SCHANBACH_SIEDLUNG = 'de:08116:4162'
	SCHANBACH_SIEDLUNG_1 = 'de:08116:4162:0:3'  # Rtg Krummhardt (Aichwald Schanbach)
	SCHANBACH_SIEDLUNG_2 = 'de:08116:4162:0:4'  # Rtg Kindergarten/Kreuzung (Aichwald Schanbach)
	AICHSCHIESS_KRUMMHARDT_ABZW = 'de:08116:4163'
	AICHSCHIESS_KRUMMHARDT_ABZW_1 = 'de:08116:4163:0:3'  # eRtg Kronenstraße (Aichwald Aichschieß)
	AICHSCHIESS_KRUMMHARDT_ABZW_2 = 'de:08116:4163:0:4'  # eRtg Siedlung (Aichwald Aichschieß)
	AICHSCHIESS_KRUMMHARDT_ABZW_3 = 'de:08116:4163:0:5'  # eRtg Panoramastraße (Aichwald Aichschieß)
	KRUMMHARDT_ABZWEIG = 'de:08116:4163'
	KRUMMHARDT_ABZWEIG_1 = 'de:08116:4163:0:3'  # eRtg Kronenstraße (Aichwald Aichschieß)
	KRUMMHARDT_ABZWEIG_2 = 'de:08116:4163:0:4'  # eRtg Siedlung (Aichwald Aichschieß)
	KRUMMHARDT_ABZWEIG_3 = 'de:08116:4163:0:5'  # eRtg Panoramastraße (Aichwald Aichschieß)
	KRUMMHARDT_ORT = 'de:08116:4164:0:3'  # Rtg Aichelb. Kronenstr./BBA Schanbach (Aichwald Krummhardt)
	AICHELBERG_KRONENSTRASSE = 'de:08116:4165:0:3'  # eRtg Holl (Aichwald Aichelberg (Aichwald))
	AICHELBERG_RATHAUS = 'de:08116:4166:0:3'  # eRtg Krummhardt Ort (Aichwald Aichelberg (Aichwald))
	HOLL = 'de:08116:4167'
	HOLL_1 = 'de:08116:4167:0:3'  # Rtg Endersbach (Aichwald Aichelberg (Aichwald))
	HOLL_2 = 'de:08116:4167:0:4'  # Rtg Esslingen (Aichwald Aichelberg (Aichwald))
	AICHELBERG_HOLL = 'de:08116:4167'
	AICHELBERG_HOLL_1 = 'de:08116:4167:0:3'  # Rtg Endersbach (Aichwald Aichelberg (Aichwald))
	AICHELBERG_HOLL_2 = 'de:08116:4167:0:4'  # Rtg Esslingen (Aichwald Aichelberg (Aichwald))
	KEPLERSTRASSE_1 = 'de:08116:4168:0:3'  # eRtg Alte Heusteige (Esslingen am Neckar Oberesslingen)
	KEPLERSTRASSE_2 = 'de:08116:4168:0:4'  # eRtg Zimmerbachstraße (Esslingen am Neckar Oberesslingen)
	KEPLERSTRASSE_3 = 'de:08115:5935:0:3'  # eRtg Gewerbegebiet Steige (Rutesheim Rutesheim)
	KEPLERSTRASSE_4 = 'de:08116:6998:0:3'  # eRtg Olga-/Zeppelinstraße (Deizisau Deizisau)
	KEPLERSTRASSE_5 = 'de:08117:9003:0:H1'  # Rtg Jebenhausen (Göppingen Göppingen)
	KEPLERSTRASSE_6 = 'de:08117:9003:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	OBERESSL__KEPLERSTRASSE = 'de:08116:4168'
	OBERESSL__KEPLERSTRASSE_1 = 'de:08116:4168:0:3'  # eRtg Alte Heusteige (Esslingen am Neckar Oberesslingen)
	OBERESSL__KEPLERSTRASSE_2 = 'de:08116:4168:0:4'  # eRtg Zimmerbachstraße (Esslingen am Neckar Oberesslingen)
	ZELL_KÖRSCHSTRASSE = 'de:08116:4169'
	ZELL_KÖRSCHSTRASSE_1 = 'de:08116:4169:0:3'  # Rtg Zell (Esslingen am Neckar Zell (Esslingen))
	ZELL_KÖRSCHSTRASSE_2 = 'de:08116:4169:0:4'  # Rtg Berufsschule (Esslingen am Neckar Zell (Esslingen))
	KÖRSCHSTRASSE = 'de:08116:4169'
	KÖRSCHSTRASSE_1 = 'de:08116:4169:0:3'  # Rtg Zell (Esslingen am Neckar Zell (Esslingen))
	KÖRSCHSTRASSE_2 = 'de:08116:4169:0:4'  # Rtg Berufsschule (Esslingen am Neckar Zell (Esslingen))
	BERUFLICHES_SCHULZENTRUM = 'de:08116:4170'
	BERUFLICHES_SCHULZENTRUM_1 = 'de:08116:4170:0:4'  # Rtg Zentrum (Esslingen am Neckar Zell (Esslingen))
	BERUFLICHES_SCHULZENTRUM_2 = 'de:08116:4170:0:5'  # Rtg Alleenstraße (Esslingen am Neckar Zell (Esslingen))
	ZELL_BERUFLICHES_SCHULZENTRUM = 'de:08116:4170'
	ZELL_BERUFLICHES_SCHULZENTRUM_1 = 'de:08116:4170:0:4'  # Rtg Zentrum (Esslingen am Neckar Zell (Esslingen))
	ZELL_BERUFLICHES_SCHULZENTRUM_2 = 'de:08116:4170:0:5'  # Rtg Alleenstraße (Esslingen am Neckar Zell (Esslingen))
	KATZENKOPF = 'de:08116:4171'
	KATZENKOPF_1 = 'de:08116:4171:0:3'  # eRtg Kelter (Esslingen am Neckar Wäldenbronn)
	KATZENKOPF_2 = 'de:08116:4171:0:4'  # eRtg Betriebshof (Esslingen am Neckar Wäldenbronn)
	WÄLDENBRONN_KATZENKOPF = 'de:08116:4171'
	WÄLDENBRONN_KATZENKOPF_1 = 'de:08116:4171:0:3'  # eRtg Kelter (Esslingen am Neckar Wäldenbronn)
	WÄLDENBRONN_KATZENKOPF_2 = 'de:08116:4171:0:4'  # eRtg Betriebshof (Esslingen am Neckar Wäldenbronn)
	ZELL_JOHANNESSTRASSE = 'de:08116:4173:0:3'  # eRtg Zentrum (Esslingen am Neckar Zell (Esslingen))
	JOHANNESSTRASSE_1 = 'de:08116:4173:0:3'  # eRtg Zentrum (Esslingen am Neckar Zell (Esslingen))
	JOHANNESSTRASSE_2 = 'de:08119:5191:0:3'  # eRtg Rathaus/Seebad (Schorndorf Schorndorf)
	JOHANNESSTRASSE_3 = 'de:08119:5191:0:4'  # eRtg Künkelinhalle/Burggymnasium (Schorndorf Schorndorf)
	JOHANNESSTRASSE_4 = 'de:08118:5461:0:3'  # eRtg Zeppelinstraße (Kornwestheim Kornwestheim)
	JOHANNESSTRASSE_5 = 'de:08118:5461:0:4'  # eRtg Kornwestheim (Kornwestheim Kornwestheim)
	JOHANNESSTRASSE_6 = 'de:08115:7074:0:1'  # Rtg Silcherstr (Herrenberg Herrenberg)
	JOHANNESSTRASSE_7 = 'de:08115:7074:0:2'  # Rtg Hallenbad (Herrenberg Herrenberg)
	ZOLLBERG_EICHENDORFFSTR = 'de:08116:4174:1:3'  # eRtg Eichendorffschule (Esslingen am Neckar Zollberg)
	EICHENDORFFSCHULE = 'de:08116:4175'
	EICHENDORFFSCHULE_1 = 'de:08116:4175:0:3'  # Rtg Braungartweg (Esslingen am Neckar Zollberg)
	EICHENDORFFSCHULE_2 = 'de:08116:4175:0:4'  # Rtg Zollberg Eichendorffstr. (Esslingen am Neckar Zollberg)
	ZOLLB__EICHENDORFFSCHULE = 'de:08116:4175'
	ZOLLB__EICHENDORFFSCHULE_1 = 'de:08116:4175:0:3'  # Rtg Braungartweg (Esslingen am Neckar Zollberg)
	ZOLLB__EICHENDORFFSCHULE_2 = 'de:08116:4175:0:4'  # Rtg Zollberg Eichendorffstr. (Esslingen am Neckar Zollberg)
	BRAUNGARTWEG = 'de:08116:4176'
	BRAUNGARTWEG_1 = 'de:08116:4176:0:3'  # Rtg Hochstetter Weg (Esslingen am Neckar Zollberg)
	BRAUNGARTWEG_2 = 'de:08116:4176:0:4'  # Rtg Eichendorffschule (Esslingen am Neckar Zollberg)
	ZOLLBERG_BRAUNGARTWEG = 'de:08116:4176'
	ZOLLBERG_BRAUNGARTWEG_1 = 'de:08116:4176:0:3'  # Rtg Hochstetter Weg (Esslingen am Neckar Zollberg)
	ZOLLBERG_BRAUNGARTWEG_2 = 'de:08116:4176:0:4'  # Rtg Eichendorffschule (Esslingen am Neckar Zollberg)
	ZOLLBERG_ZOLLERNPLATZ = 'de:08116:4177'
	ZOLLBERG_ZOLLERNPLATZ_1 = 'de:08116:4177:0:3'  # Rtg Zollberg (Esslingen am Neckar Zollberg)
	ZOLLBERG_ZOLLERNPLATZ_2 = 'de:08116:4177:0:4'  # Rtg Jusiweg (Esslingen am Neckar Zollberg)
	ZOLLERNPLATZ = 'de:08116:4177'
	ZOLLERNPLATZ_1 = 'de:08116:4177:0:3'  # Rtg Zollberg (Esslingen am Neckar Zollberg)
	ZOLLERNPLATZ_2 = 'de:08116:4177:0:4'  # Rtg Jusiweg (Esslingen am Neckar Zollberg)
	ZOLLHAUSWEG = 'de:08116:4178:0:3'  # Rtg Jusiweg (Esslingen am Neckar Zollberg)
	ZOLLBERG_ZOLLHAUSWEG = 'de:08116:4178:0:3'  # Rtg Jusiweg (Esslingen am Neckar Zollberg)
	ZOLLBERG_REALSCHULE = 'de:08116:4179:0:4'  # Rtg Esslingen/Zollernplatz (Esslingen am Neckar Zollberg)
	JUSIWEG = 'de:08116:4180'
	JUSIWEG_1 = 'de:08116:4180:0:3'  # Rtg Realschule (Esslingen am Neckar Zollberg)
	JUSIWEG_2 = 'de:08116:4180:0:4'  # Rtg Hochstetter Weg/Zollhausweg (Esslingen am Neckar Zollberg)
	ZOLLBERG_JUSIWEG = 'de:08116:4180'
	ZOLLBERG_JUSIWEG_1 = 'de:08116:4180:0:3'  # Rtg Realschule (Esslingen am Neckar Zollberg)
	ZOLLBERG_JUSIWEG_2 = 'de:08116:4180:0:4'  # Rtg Hochstetter Weg/Zollhausweg (Esslingen am Neckar Zollberg)
	DACHTEL_BACKHAUS = 'de:08115:4181'
	DACHTEL_BACKHAUS_1 = 'de:08115:4181:0:3'  # eRtg Rath./Aidlinger Str./H.-Hesse-Str (Aidlingen Dachtel)
	DACHTEL_BACKHAUS_2 = 'de:08115:4181:0:4'  # eRtg Bergsiedlung (Aidlingen Dachtel)
	BACKHAUS_1 = 'de:08115:4181:0:3'  # eRtg Rath./Aidlinger Str./H.-Hesse-Str (Aidlingen Dachtel)
	BACKHAUS_2 = 'de:08115:4181:0:4'  # eRtg Bergsiedlung (Aidlingen Dachtel)
	BACKHAUS_3 = 'de:08119:4947:0:3'  # eRtg SOS Kinderdorf (Schorndorf Oberberken)
	BACKHAUS_4 = 'de:08119:5170:0:3'  # Rtg Prinz-Eugen-Platz (Weinstadt Großheppach)
	BACKHAUS_5 = 'de:08119:5170:0:4'  # Rtg Kleinheppacher Str. (Weinstadt Großheppach)
	BACKHAUS_6 = 'de:08117:3316:0:H1'  # Backhaus (Eschenbach Eschenbach)
	BACKHAUS_7 = 'de:08117:5015:0:H1'  # Rtg Uhlandstraße (Wangen Oberwälden)
	WEIL_I__S__SCHAICHHOFSIEDLUNG = 'de:08115:4185'
	WEIL_I__S__SCHAICHHOFSIEDLUNG_1 = 'de:08115:4185:0:3'  # eRtg Schaichhofstraße (Weil im Schönbuch Weil im Schönbuch)
	WEIL_I__S__SCHAICHHOFSIEDLUNG_2 = 'de:08115:4185:0:4'  # eRtg Schaichhof (Weil im Schönbuch Weil im Schönbuch)
	SCHAICHHOFSIEDLUNG = 'de:08115:4185'
	SCHAICHHOFSIEDLUNG_1 = 'de:08115:4185:0:3'  # eRtg Schaichhofstraße (Weil im Schönbuch Weil im Schönbuch)
	SCHAICHHOFSIEDLUNG_2 = 'de:08115:4185:0:4'  # eRtg Schaichhof (Weil im Schönbuch Weil im Schönbuch)
	STUTTGARTER_STRASSE_EICHHOLZ = 'de:08115:4186'
	STUTTGARTER_STRASSE_EICHHOLZ_1 = 'de:08115:4186:0:3'  # eRtg Waldfriedhof (Sindelfingen Maichingen)
	STUTTGARTER_STRASSE_EICHHOLZ_2 = 'de:08115:4186:0:4'  # eRtg Am Schattwald (Sindelfingen Maichingen)
	MAICH__STUTTG__STR__EICHHOLZ = 'de:08115:4186'
	MAICH__STUTTG__STR__EICHHOLZ_1 = 'de:08115:4186:0:3'  # eRtg Waldfriedhof (Sindelfingen Maichingen)
	MAICH__STUTTG__STR__EICHHOLZ_2 = 'de:08115:4186:0:4'  # eRtg Am Schattwald (Sindelfingen Maichingen)
	DACHTEL_BERGSIEDLUNG = 'de:08115:4189'
	DACHTEL_BERGSIEDLUNG_1 = 'de:08115:4189:0:3'  # Rtg Bergsiedlung/Keplerstr. (Aidlingen Dachtel)
	DACHTEL_BERGSIEDLUNG_2 = 'de:08115:4189:0:4'  # Rtg Backhaus (Aidlingen Dachtel)
	BERGSIEDLUNG = 'de:08115:4189'
	BERGSIEDLUNG_1 = 'de:08115:4189:0:3'  # Rtg Bergsiedlung/Keplerstr. (Aidlingen Dachtel)
	BERGSIEDLUNG_2 = 'de:08115:4189:0:4'  # Rtg Backhaus (Aidlingen Dachtel)
	DACHTEL_BERGSIEDL___KEPLERSTR = 'de:08115:4190'
	DACHTEL_BERGSIEDL___KEPLERSTR_1 = 'de:08115:4190:0:3'  # Rtg Gechingen (Aidlingen Dachtel)
	DACHTEL_BERGSIEDL___KEPLERSTR_2 = 'de:08115:4190:0:4'  # Rtg Dachtel (Aidlingen Dachtel)
	BERGSIEDLUNG__KEPLERSTRASSE = 'de:08115:4190'
	BERGSIEDLUNG__KEPLERSTRASSE_1 = 'de:08115:4190:0:3'  # Rtg Gechingen (Aidlingen Dachtel)
	BERGSIEDLUNG__KEPLERSTRASSE_2 = 'de:08115:4190:0:4'  # Rtg Dachtel (Aidlingen Dachtel)
	NEUH__F__F__SCHILLER_SCHULE = 'de:08116:4191'
	NEUH__F__F__SCHILLER_SCHULE_1 = 'de:08116:4191:0:1'  # Rtg Brühlsiedlung (Neuhausen auf den Fildern Neuhausen (F))
	NEUH__F__F__SCHILLER_SCHULE_2 = 'de:08116:4191:0:2'  # Rtg Schubertstr. (Neuhausen auf den Fildern Neuhausen (F))
	NEUH__F__F__SCHILLER_SCHULE_3 = 'de:08116:4191:0:3'  # Rtg Novizenweg (Neuhausen auf den Fildern Neuhausen (F))
	FRIEDRICH_SCHILLER_SCHULE = 'de:08116:4191'
	FRIEDRICH_SCHILLER_SCHULE_1 = 'de:08116:4191:0:1'  # Rtg Brühlsiedlung (Neuhausen auf den Fildern Neuhausen (F))
	FRIEDRICH_SCHILLER_SCHULE_2 = 'de:08116:4191:0:2'  # Rtg Schubertstr. (Neuhausen auf den Fildern Neuhausen (F))
	FRIEDRICH_SCHILLER_SCHULE_3 = 'de:08116:4191:0:3'  # Rtg Novizenweg (Neuhausen auf den Fildern Neuhausen (F))
	WEIL_I__S__TROPPEL = 'de:08115:4192'
	WEIL_I__S__TROPPEL_1 = 'de:08115:4192:1:1'  # R-Bahn (Weil im Schönbuch Weil im Schönbuch)
	WEIL_I__S__TROPPEL_2 = 'de:08115:4192:2:3'  # eRtg Hauptstraße (Weil im Schönbuch Weil im Schönbuch)
	WEIL_I__S__TROPPEL_3 = 'de:08115:4192:2:4'  # eRtg Schaichhofstraße (Weil im Schönbuch Weil im Schönbuch)
	TROPPEL = 'de:08115:4192'
	TROPPEL_1 = 'de:08115:4192:1:1'  # R-Bahn (Weil im Schönbuch Weil im Schönbuch)
	TROPPEL_2 = 'de:08115:4192:2:3'  # eRtg Hauptstraße (Weil im Schönbuch Weil im Schönbuch)
	TROPPEL_3 = 'de:08115:4192:2:4'  # eRtg Schaichhofstraße (Weil im Schönbuch Weil im Schönbuch)
	HOLZGERLINGEN = 'de:08115:4193'
	HOLZGERLINGEN_1 = 'de:08115:4193:1:1'  # R-Bahn (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_2 = 'de:08115:4193:1:2'  # Stumpfgleis (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_3 = 'de:08115:4193:2:3'  # eRtg Rathaus (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_4 = 'de:08115:4193:2:4'  # eRtg Stäuchle (Holzgerlingen Holzgerlingen)
	DÄTZINGEN_SCHULE = 'de:08115:4194:0:3'  # eRtg Kreuzstraße (Grafenau Dätzingen)
	DÖFFINGER_STRASSE = 'de:08115:4195'
	DÖFFINGER_STRASSE_1 = 'de:08115:4195:0:3'  # Pflieger (Sindelfingen Darmsheim)
	DÖFFINGER_STRASSE_2 = 'de:08115:4195:0:4'  # RBS/Hassler (Sindelfingen Darmsheim)
	DARMSHEIM_DÖFFINGER_STR = 'de:08115:4195'
	DARMSHEIM_DÖFFINGER_STR_1 = 'de:08115:4195:0:3'  # Pflieger (Sindelfingen Darmsheim)
	DARMSHEIM_DÖFFINGER_STR_2 = 'de:08115:4195:0:4'  # RBS/Hassler (Sindelfingen Darmsheim)
	OBERESSL__TH__HEUSS_GYMNASIUM = 'de:08116:4197'
	OBERESSL__TH__HEUSS_GYMNASIUM_1 = 'de:08116:4197:0:3'  # eRtg Weiherstraße/Oberhof (Esslingen am Neckar Oberesslingen)
	OBERESSL__TH__HEUSS_GYMNASIUM_2 = 'de:08116:4197:0:4'  # eRtg Brandenb. Str. (Esslingen am Neckar Oberesslingen)
	THEODOR_HEUSS_GYMNASIUM = 'de:08116:4197'
	THEODOR_HEUSS_GYMNASIUM_1 = 'de:08116:4197:0:3'  # eRtg Weiherstraße/Oberhof (Esslingen am Neckar Oberesslingen)
	THEODOR_HEUSS_GYMNASIUM_2 = 'de:08116:4197:0:4'  # eRtg Brandenb. Str. (Esslingen am Neckar Oberesslingen)
	HERDERSTRASSE = 'de:08116:4200'
	HERDERSTRASSE_1 = 'de:08116:4200:1:3'  # Rtg Steinhalde (Esslingen am Neckar Oberesslingen)
	HERDERSTRASSE_2 = 'de:08116:4200:1:4'  # Rtg Innenstadt (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_HERDERSTR = 'de:08116:4200'
	OBERESSLINGEN_HERDERSTR_1 = 'de:08116:4200:1:3'  # Rtg Steinhalde (Esslingen am Neckar Oberesslingen)
	OBERESSLINGEN_HERDERSTR_2 = 'de:08116:4200:1:4'  # Rtg Innenstadt (Esslingen am Neckar Oberesslingen)
	OBERESSL__SCHORNDORFER_STR = 'de:08116:4201:1:3'  # eRtg Zimmerbachstr./Keplerstr. (Esslingen am Neckar Oberesslingen)
	SULZGRIES_SCHULE = 'de:08116:4202'
	SULZGRIES_SCHULE_1 = 'de:08116:4202:0:1'  # Rtg Rüdern Bürgerhaus RSKN (Esslingen am Neckar Sulzgries)
	SULZGRIES_SCHULE_2 = 'de:08116:4202:0:3'  # eRtg Bergstraße (Esslingen am Neckar Sulzgries)
	HEGENSBERG_ORTSEINGANG = 'de:08116:4203'
	HEGENSBERG_ORTSEINGANG_1 = 'de:08116:4203:0:3'  # eRtg Brandenburger Straße (Esslingen am Neckar Hegensberg)
	HEGENSBERG_ORTSEINGANG_2 = 'de:08116:4203:0:4'  # eRtg Linde (Esslingen am Neckar Hegensberg)
	ORTSEINGANG = 'de:08116:4203'
	ORTSEINGANG_1 = 'de:08116:4203:0:3'  # eRtg Brandenburger Straße (Esslingen am Neckar Hegensberg)
	ORTSEINGANG_2 = 'de:08116:4203:0:4'  # eRtg Linde (Esslingen am Neckar Hegensberg)
	HEGENSBERG_HOFSTRASSE = 'de:08116:4204'
	HEGENSBERG_HOFSTRASSE_1 = 'de:08116:4204:0:3'  # eRtg Ortseingang (Esslingen am Neckar Hegensberg)
	HEGENSBERG_HOFSTRASSE_2 = 'de:08116:4204:0:4'  # eRtg Kriegerdenkmal (Esslingen am Neckar Hegensberg)
	HOFSTRASSE_1 = 'de:08116:4204:0:3'  # eRtg Ortseingang (Esslingen am Neckar Hegensberg)
	HOFSTRASSE_2 = 'de:08116:4204:0:4'  # eRtg Kriegerdenkmal (Esslingen am Neckar Hegensberg)
	HOFSTRASSE_3 = 'de:08115:4638:0:3'  # eRtg Döffinger Straße (Sindelfingen Darmsheim)
	BÖBLINGEN_SINDELFINGER_STR = 'de:08115:4205'
	BÖBLINGEN_SINDELFINGER_STR_1 = 'de:08115:4205:0:3'  # eRtg Goldberg-Gymnasium (Böblingen Böblingen)
	BÖBLINGEN_SINDELFINGER_STR_2 = 'de:08115:4205:0:4'  # eRtg Böblingen (Böblingen Böblingen)
	SINDELFINGER_STRASSE_1 = 'de:08115:4205:0:3'  # eRtg Goldberg-Gymnasium (Böblingen Böblingen)
	SINDELFINGER_STRASSE_2 = 'de:08115:4205:0:4'  # eRtg Böblingen (Böblingen Böblingen)
	SINDELFINGER_STRASSE_3 = 'de:08115:4642:0:3'  # eRtg Maichingen (Sindelfingen Maichingen)
	SINDELFINGER_STRASSE_4 = 'de:08115:4642:0:4'  # eRtg Mercedes-Benz Tor V (Sindelfingen Maichingen)
	HOLZGERLINGEN_ABZW = 'de:08115:4206'
	HOLZGERLINGEN_ABZW_1 = 'de:08115:4206:0:3'  # Rtg Böblingen (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_ABZW_2 = 'de:08115:4206:0:4'  # Rtg Schaichhof (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_HOHENZOLLERNSTR = 'de:08115:4208'
	HOLZGERLINGEN_HOHENZOLLERNSTR_1 = 'de:08115:4208:0:3'  # eRtg Lilienstr./Altenzentrum/Gymnasium (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_HOHENZOLLERNSTR_2 = 'de:08115:4208:0:4'  # eRtg Germanenstraße/Böblingen (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_RATHAUS = 'de:08115:4210'
	HOLZGERLINGEN_RATHAUS_1 = 'de:08115:4210:0:3'  # Rtg Germanenstraße (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_RATHAUS_2 = 'de:08115:4210:0:4'  # Rtg Holzgerlingen (Holzgerlingen Holzgerlingen)
	KIRCHHEIM__T = 'de:08116:4211'
	KIRCHHEIM__T_1 = 'de:08116:4211:1:2'  # Gleis 2 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T_2 = 'de:08116:4211:3:1'  # Bstg 1 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T_3 = 'de:08116:4211:3:2'  # Bstg 2 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T_4 = 'de:08116:4211:3:3'  # Bstg 3 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T_5 = 'de:08116:4211:4:35'  # Ausstieg 7 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T_6 = 'de:08116:4211:4:4'  # Bstg 4 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T_7 = 'de:08116:4211:4:5'  # Bstg 5 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T_8 = 'de:08116:4211:4:6'  # Bstg 6 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T_9 = 'de:08116:4211:4:7'  # Bstg 7 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T_10 = 'de:08116:4211:5:11'  # Bstg 11 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T_11 = 'de:08116:4211:5:8'  # 8 Linien 161, 163, 165 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T_12 = 'de:08116:4211:5:9'  # Bstg 9 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T_13 = 'de:08116:4211:6:1'  # Gleis 1 (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__MARKTPLATZ = 'de:08116:4213:1:4'  # eRtg Kirchheim (T) ZOB/K.-W.-Halle (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BRAHMSSTRASSE = 'de:08116:4215'
	KIRCHHEIM__T__BRAHMSSTRASSE_1 = 'de:08116:4215:0:3'  # eRtg Wellinger Straße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BRAHMSSTRASSE_2 = 'de:08116:4215:0:4'  # eRtg Hugo-Wolf-Weg (Kirchheim unter Teck Kirchheim (T))
	BRAHMSSTRASSE = 'de:08116:4215'
	BRAHMSSTRASSE_1 = 'de:08116:4215:0:3'  # eRtg Wellinger Straße (Kirchheim unter Teck Kirchheim (T))
	BRAHMSSTRASSE_2 = 'de:08116:4215:0:4'  # eRtg Hugo-Wolf-Weg (Kirchheim unter Teck Kirchheim (T))
	NOTZINGEN_KIRCHHEIMER_STR = 'de:08116:4216:0:3'  # eRtg Brahmsstraße (Notzingen Notzingen)
	NOTZ__WELLINGER_STRASSE = 'de:08116:4217:0:3'  # eRtg Rathaus (Notzingen Notzingen)
	WELLINGER_STRASSE = 'de:08116:4217:0:3'  # eRtg Rathaus (Notzingen Notzingen)
	NOTZINGEN_RATHAUS = 'de:08116:4218'
	NOTZINGEN_RATHAUS_1 = 'de:08116:4218:0:3'  # Rtg Notzingen Ost (Notzingen Notzingen)
	NOTZINGEN_RATHAUS_2 = 'de:08116:4218:0:4'  # Rtg Wernau/Kirchheim/Hochdorf (Notzingen Notzingen)
	NOTZINGEN_HOCHDORFER_STR = 'de:08116:4219'
	NOTZINGEN_HOCHDORFER_STR_1 = 'de:08116:4219:0:3'  # Rtg Kirchhheim (Notzingen Notzingen)
	NOTZINGEN_HOCHDORFER_STR_2 = 'de:08116:4219:0:4'  # Rtg Hochdorf/Wernau (Notzingen Notzingen)
	HOCHDORFER_STRASSE_1 = 'de:08116:4219:0:3'  # Rtg Kirchhheim (Notzingen Notzingen)
	HOCHDORFER_STRASSE_2 = 'de:08116:4219:0:4'  # Rtg Hochdorf/Wernau (Notzingen Notzingen)
	HOCHDORFER_STRASSE_3 = 'de:08119:5554:1:4'  # eRtg Hochberg Rotweg (Waiblingen Bittenfeld)
	HOCHDORFER_STRASSE_4 = 'de:08118:5623:1:3'  # Rtg Nussdorf (Eberdingen Eberdingen)
	HOCHDORFER_STRASSE_5 = 'de:08118:5623:1:4'  # Rtg Hochdorf (Eberdingen Eberdingen)
	TALBACHBRÜCKE = 'de:08116:4220:0:3'  # eRtg Weinbergstraße (Hochdorf Hochdorf (ES))
	HOCHDORF_TALBACHBRÜCKE = 'de:08116:4220:0:3'  # eRtg Weinbergstraße (Hochdorf Hochdorf (ES))
	ROSSWÄLDER_STRASSE = 'de:08116:4221:0:3'  # eRtg Rathaus (Hochdorf Hochdorf (ES))
	HOCHDORF_ROSSWÄLDER_STR = 'de:08116:4221:0:3'  # eRtg Rathaus (Hochdorf Hochdorf (ES))
	HOCHDORF_RATHAUS = 'de:08116:4222'
	HOCHDORF_RATHAUS_1 = 'de:08116:4222:0:1'  # Rtg Talbachbrücke (Hochdorf Hochdorf (ES))
	HOCHDORF_RATHAUS_2 = 'de:08116:4222:0:2'  # Rtg Aspen (Hochdorf Hochdorf (ES))
	HOCHDORF_RATHAUS_3 = 'de:08116:4222:0:3'  # Rtg Ulmer Str/Weberei (Hochdorf Hochdorf (ES))
	WEBEREI = 'de:08116:4223'
	WEBEREI_1 = 'de:08116:4223:0:3'  # eRtg K 1206 (Bahnhof) (Reichenbach an der Fils Reichenbach (F))
	WEBEREI_2 = 'de:08116:4223:0:4'  # eRtg Talbachbrücke (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F__WEBEREI = 'de:08116:4223'
	REICHENBACH__F__WEBEREI_1 = 'de:08116:4223:0:3'  # eRtg K 1206 (Bahnhof) (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F__WEBEREI_2 = 'de:08116:4223:0:4'  # eRtg Talbachbrücke (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F = 'de:08116:4224'
	REICHENBACH__F_1 = 'de:08116:4224:1:2'  # Gleis 2 (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F_2 = 'de:08116:4224:2:1'  # Gleis 1 (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F_3 = 'de:08116:4224:3:3'  # Rtg Stuttgarter/Hauptstr. (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F_4 = 'de:08116:4224:3:4'  # Rtg Ulmer/Blumenstr. (Reichenbach an der Fils Reichenbach (F))
	PLOCHINGEN_ULMER_STR = 'de:08116:4225'
	PLOCHINGEN_ULMER_STR_1 = 'de:08116:4225:0:3'  # eRtg Stuttgarter Straße (Plochingen Plochingen)
	PLOCHINGEN_ULMER_STR_2 = 'de:08116:4225:0:4'  # eRtg Kreissparkasse (Plochingen Plochingen)
	PLOCHINGEN_KREISSPARKASSE = 'de:08116:4226:0:3'  # eRtg Plochingen (Plochingen Plochingen)
	PLOCHINGEN_EISENBAHNSTRASSE = 'de:08116:4227:0:3'  # eRtg Ulmer Straße (Plochingen Plochingen)
	EISENBAHNSTRASSE = 'de:08116:4227:0:3'  # eRtg Ulmer Straße (Plochingen Plochingen)
	STADTPLATZ = 'de:08116:4228:0:3'  # Rtg Dorotheenweg (Wernau (Neckar) Wernau (N))
	WERNAU__N__STADTPLATZ = 'de:08116:4228:0:3'  # Rtg Dorotheenweg (Wernau (Neckar) Wernau (N))
	KAPELLE = 'de:08116:4229'
	KAPELLE_1 = 'de:08116:4229:0:3'  # eRtg Panoramastraße (Plochingen Plochingen)
	KAPELLE_2 = 'de:08116:4229:0:4'  # eRtgKreissparkasse (Plochingen Plochingen)
	PLOCHINGEN_KAPELLE = 'de:08116:4229'
	PLOCHINGEN_KAPELLE_1 = 'de:08116:4229:0:3'  # eRtg Panoramastraße (Plochingen Plochingen)
	PLOCHINGEN_KAPELLE_2 = 'de:08116:4229:0:4'  # eRtgKreissparkasse (Plochingen Plochingen)
	PANORAMASTRASSE_1 = 'de:08116:4230:0:3'  # eRtg Marquardtschule / Beethovenstraße (Plochingen Plochingen)
	PANORAMASTRASSE_2 = 'de:08116:4230:0:4'  # eRtg Kapelle (Plochingen Plochingen)
	PANORAMASTRASSE_3 = 'de:08118:5684:0:3'  # eRtg Waldschule (Bietigheim-Bissingen Bissingen (LB))
	PANORAMASTRASSE_4 = 'de:08118:5684:0:4'  # eRtg Karlstraße (Bietigheim-Bissingen Bissingen (LB))
	PANORAMASTRASSE_5 = 'de:08136:7018:0:H1'  # Panoramastraße (Lorch Lorch)
	PLOCHINGEN_PANORAMASTRASSE = 'de:08116:4230'
	PLOCHINGEN_PANORAMASTRASSE_1 = 'de:08116:4230:0:3'  # eRtg Marquardtschule / Beethovenstraße (Plochingen Plochingen)
	PLOCHINGEN_PANORAMASTRASSE_2 = 'de:08116:4230:0:4'  # eRtg Kapelle (Plochingen Plochingen)
	PLOCHINGEN_BEETHOVENSTRASSE = 'de:08116:4231'
	PLOCHINGEN_BEETHOVENSTRASSE_1 = 'de:08116:4231:0:3'  # eRtg Lettenäcker (Plochingen Plochingen)
	PLOCHINGEN_BEETHOVENSTRASSE_2 = 'de:08116:4231:0:4'  # eRtg Marquardtschule / Panoramastraße (Plochingen Plochingen)
	PLOCHINGEN_LETTENÄCKER = 'de:08116:4232:0:3'  # eRtg Stufenstraße (Plochingen Plochingen)
	LETTENÄCKER = 'de:08116:4232:0:3'  # eRtg Stufenstraße (Plochingen Plochingen)
	PLOCHINGEN_STUMPENHOF = 'de:08116:4233:0:3'  # eRtg Lettenäcker (Plochingen Plochingen)
	STUMPENHOF = 'de:08116:4233:0:3'  # eRtg Lettenäcker (Plochingen Plochingen)
	REICHENB__F__ORTSMITTE = 'de:08116:4234'
	REICHENB__F__ORTSMITTE_1 = 'de:08116:4234:1:3'  # Rtg Plochingen/Schulzentrum (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__ORTSMITTE_2 = 'de:08116:4234:1:4'  # Rtg Bahnhof/Ulmer Straße (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__SCHULZENTRUM = 'de:08116:4235'
	REICHENB__F__SCHULZENTRUM_1 = 'de:08116:4235:0:3'  # Rtg West (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__SCHULZENTRUM_2 = 'de:08116:4235:0:4'  # Rtg Ost (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__SCHÖNBLICK = 'de:08116:4236'
	REICHENB__F__SCHÖNBLICK_1 = 'de:08116:4236:0:3'  # eRtg Siegenbergplatz (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__SCHÖNBLICK_2 = 'de:08116:4236:0:4'  # eRtg Schulzentrum (Reichenbach an der Fils Reichenbach (F))
	SCHÖNBLICK_1 = 'de:08116:4236:0:3'  # eRtg Siegenbergplatz (Reichenbach an der Fils Reichenbach (F))
	SCHÖNBLICK_2 = 'de:08116:4236:0:4'  # eRtg Schulzentrum (Reichenbach an der Fils Reichenbach (F))
	SCHÖNBLICK_3 = 'de:08116:4473:0:3'  # 809 Rtg Froschegert/Harthausen (Aichtal Grötzingen)
	SCHÖNBLICK_4 = 'de:08116:4473:0:4'  # 809 Rtg Ortsmitte/Kriegerdenkmal (Aichtal Grötzingen)
	ALBSTRASSE_1 = 'de:08116:4237:0:3'  # eRtg Schönblick (Reichenbach an der Fils Reichenbach (F))
	ALBSTRASSE_2 = 'de:08118:5471:0:3'  # eRtg Heubergstraße (Kornwestheim Kornwestheim)
	ALBSTRASSE_3 = 'de:08116:5901:0:3'  # eRtg Herion (Großbettlingen Großbettlingen)
	ALBSTRASSE_4 = 'de:08116:5901:0:4'  # eRtg Metzinger Str. (Großbettlingen Großbettlingen)
	ALBSTRASSE_5 = 'de:08116:7081:0:3'  # eRtg Uracher Weg (Neuffen Neuffen)
	ALBSTRASSE_6 = 'de:08116:7081:0:4'  # eRtg Oberer Graben (Neuffen Neuffen)
	ALBSTRASSE_7 = 'de:08117:7013:0:H1'  # Rtg Schopflenberg (Uhingen Sparwiesen)
	ALBSTRASSE_8 = 'de:08117:7013:0:R1'  # Rtg Uhingen (Uhingen Sparwiesen)
	REICHENBACH__F__ALBSTRASSE = 'de:08116:4237:0:3'  # eRtg Schönblick (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F__HOCHHAUS = 'de:08116:4238:0:3'  # eRtg Blumen-Wilh. Str. (Reichenbach an der Fils Reichenbach (F))
	HOCHHAUS_1 = 'de:08116:4238:0:3'  # eRtg Blumen-Wilh. Str. (Reichenbach an der Fils Reichenbach (F))
	HOCHHAUS_2 = 'de:08136:7015:0:H1'  # Hochhaus (Lorch Lorch)
	REICHENBACH__F__RISSHALDE = 'de:08116:4240:0:3'  # eRtg Weinbergstraße (Reichenbach an der Fils Reichenbach (F))
	RISSHALDE = 'de:08116:4240:0:3'  # eRtg Weinbergstraße (Reichenbach an der Fils Reichenbach (F))
	WERNAU__N = 'de:08116:4241'
	WERNAU__N_1 = 'de:08116:4241:1:2'  # Gleis 2 (Wernau (Neckar) Wernau (N))
	WERNAU__N_2 = 'de:08116:4241:2:1'  # Gleis 1 (Wernau (Neckar) Wernau (N))
	WERNAU__N_3 = 'de:08116:4241:3:3'  # Rtg Kirche Sankt Erasmus (Wernau (Neckar) Wernau (N))
	DOROTHEENWEG = 'de:08116:4244:0:3'  # eRtg Marienstraße (Wernau (Neckar) Wernau (N))
	WERNAU__N__DOROTHEENWEG = 'de:08116:4244:0:3'  # eRtg Marienstraße (Wernau (Neckar) Wernau (N))
	WERNAU__N__MARIENSTRASSE = 'de:08116:4245:0:3'  # eRtg Bergstraße (Wernau (Neckar) Wernau (N))
	WERNAU__N__BERGSTRASSE = 'de:08116:4246:0:3'  # eRtg Bruckenstraße (Wernau (Neckar) Wernau (N))
	WERNAU__N__FRIEDR__LIST_STR = 'de:08116:4247'
	WERNAU__N__FRIEDR__LIST_STR_1 = 'de:08116:4247:0:3'  # eRtg Max-Eyth-Straße (Wernau (Neckar) Wernau (N))
	WERNAU__N__FRIEDR__LIST_STR_2 = 'de:08116:4247:0:4'  # Rtg Bruckenstr. (Wernau (Neckar) Wernau (N))
	WERNAU__N__MAX_EYTH_STR = 'de:08116:4248:0:3'  # eRtg Ziegelei (Wernau (Neckar) Wernau (N))
	BRÜHL_1 = 'de:08116:4249:0:3'  # Rtg Kirchheim (Wernau (Neckar) Wernau (N))
	BRÜHL_2 = 'de:08116:4249:0:4'  # Rtg Bruckenstr. (Wernau (Neckar) Wernau (N))
	BRÜHL_3 = 'de:08116:4347:0:3'  # eRtg Kreissparkasse (Kirchheim unter Teck Lindorf)
	BRÜHL_4 = 'de:08116:4347:0:4'  # eRtg Mörikeschule (Kirchheim unter Teck Lindorf)
	WERNAU__N__BRÜHL = 'de:08116:4249'
	WERNAU__N__BRÜHL_1 = 'de:08116:4249:0:3'  # Rtg Kirchheim (Wernau (Neckar) Wernau (N))
	WERNAU__N__BRÜHL_2 = 'de:08116:4249:0:4'  # Rtg Bruckenstr. (Wernau (Neckar) Wernau (N))
	WERNAU__N__REALSCHULE = 'de:08116:4251:0:3'  # eRtg Königsberger Straße (Wernau (Neckar) Wernau (N))
	PLOCHINGEN_PFOSTENBERG = 'de:08116:4255'
	PLOCHINGEN_PFOSTENBERG_1 = 'de:08116:4255:0:3'  # eRtg Esslinger Straße (Plochingen Plochingen)
	PLOCHINGEN_PFOSTENBERG_2 = 'de:08116:4255:0:4'  # Rtg Altbach Gemeindehalle (Plochingen Plochingen)
	PFOSTENBERG = 'de:08116:4255'
	PFOSTENBERG_1 = 'de:08116:4255:0:3'  # eRtg Esslinger Straße (Plochingen Plochingen)
	PFOSTENBERG_2 = 'de:08116:4255:0:4'  # Rtg Altbach Gemeindehalle (Plochingen Plochingen)
	WENDLINGEN__N = 'de:08116:4257'
	WENDLINGEN__N_1 = 'de:08116:4257:1:1'  # Gleis 1 (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N_2 = 'de:08116:4257:1:11'  # Gleis 11 (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N_3 = 'de:08116:4257:1:12'  # Gleis 12 (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N_4 = 'de:08116:4257:2:2'  # Gleis 2 (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N_5 = 'de:08116:4257:2:3'  # Gleis 3 (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N_6 = 'de:08116:4257:3:1'  # Nachttaxi (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N_7 = 'de:08116:4257:3:2'  # Steig 2 (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N_8 = 'de:08116:4257:3:3'  # Steig 3 (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N_9 = 'de:08116:4257:3:4'  # Steig 4 (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N_10 = 'de:08116:4257:3:5'  # Steig 5 (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N_11 = 'de:08116:4257:3:6'  # Steig 6 (Abfahrt Bahnhofstraße) (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N__STADTMITTE = 'de:08116:4258:0:3'  # eRtg ZOB (Wendlingen am Neckar Wendlingen (N))
	STADTMITTE_1 = 'de:08116:4258:0:3'  # eRtg ZOB (Wendlingen am Neckar Wendlingen (N))
	STADTMITTE_2 = 'de:08119:5391:0:3'  # Rtg Giselastr./Bürgerzentrum (Waiblingen Waiblingen)
	STADTMITTE_3 = 'de:08119:5391:0:4'  # Rtg Blumenstr. (Waiblingen Waiblingen)
	STADTMITTE_4 = 'de:08111:6056:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	STADTMITTE_5 = 'de:08111:6056:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	STADTMITTE_6 = 'de:08111:6056:2:1'  # Rtg Österreichischer Pl./Rathaus (Stuttgart Stuttgart)
	STADTMITTE_7 = 'de:08111:6056:3:2'  # Rtg Hohe Straße (Stuttgart Stuttgart)
	STADTMITTE_8 = 'de:08111:6056:4:2'  # Linie 43 Rtg Wilhelmsbau (Stuttgart Stuttgart)
	STADTMITTE_9 = 'de:08111:6056:4:3'  # Linie 92 Rtg Wilhelmsbau (Stuttgart Stuttgart)
	STADTMITTE_10 = 'de:08111:6056:5:1'  # Linie 43 Rtg Berliner Platz (Stuttgart Stuttgart)
	STADTMITTE_11 = 'de:08118:6766:1:3'  # eRtg Wilhelmsplatz (Asperg Asperg)
	STADTMITTE_12 = 'de:08118:6766:1:4'  # eRtg Weinstraße (Asperg Asperg)
	STADTMITTE_13 = 'de:08117:1029:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	STADTMITTE_14 = 'de:08117:1029:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	STADTMITTE_15 = 'de:08117:1029:2:3'  # Rtg Talstraße (Eislingen/Fils Eislingen (F))
	STADTMITTE_16 = 'de:08117:1029:2:4'  # Rtg Hbf Waldhorn (Eislingen/Fils Eislingen (F))
	STADTMITTE_17 = 'de:08117:8014:0:H1'  # Rtg Ebersbach Bahnhof (Ebersbach an der Fils Ebersbach (F))
	KARPATENWEG = 'de:08116:4263:0:3'  # eRtg Warth (Kirchheim unter Teck Ötlingen)
	ÖTLINGEN_KARPATENWEG = 'de:08116:4263:0:3'  # eRtg Warth (Kirchheim unter Teck Ötlingen)
	DETTINGEN__T__RÖMERSTR = 'de:08116:4264:1:3'  # Rtg Volksbank (Dettingen unter Teck Dettingen (T))
	KIRCHHEIM__T__MEDIUS_KLINIK = 'de:08116:4265'
	KIRCHHEIM__T__MEDIUS_KLINIK_1 = 'de:08116:4265:0:3'  # Rtg Innenstadt (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__MEDIUS_KLINIK_2 = 'de:08116:4265:0:4'  # Rtg Ötlingen (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__STADTHALLE = 'de:08116:4266'
	KIRCHHEIM__T__STADTHALLE_1 = 'de:08116:4266:1:3'  # eRtg Krankenhaus (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__STADTHALLE_2 = 'de:08116:4266:1:4'  # eRtg Bahnhof/ZOB (Kirchheim unter Teck Kirchheim (T))
	ROSSMARKT = 'de:08116:4267'
	ROSSMARKT_1 = 'de:08116:4267:1:3'  # eRtg Bahnhof/ZOB (Kirchheim unter Teck Kirchheim (T))
	ROSSMARKT_2 = 'de:08116:4267:1:4'  # eRtg K.-W.-Halle (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__ROSSMARKT = 'de:08116:4267'
	KIRCHHEIM__T__ROSSMARKT_1 = 'de:08116:4267:1:3'  # eRtg Bahnhof/ZOB (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__ROSSMARKT_2 = 'de:08116:4267:1:4'  # eRtg K.-W.-Halle (Kirchheim unter Teck Kirchheim (T))
	POLIZEIWACHE = 'de:08116:4268'
	POLIZEIWACHE_1 = 'de:08116:4268:1:3'  # eRtg Straßenmeisterei (Kirchheim unter Teck Kirchheim (T))
	POLIZEIWACHE_2 = 'de:08116:4268:1:4'  # eRtg Konrad-Widerholt-Halle (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__POLIZEIWACHE = 'de:08116:4268'
	KIRCHHEIM__T__POLIZEIWACHE_1 = 'de:08116:4268:1:3'  # eRtg Straßenmeisterei (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__POLIZEIWACHE_2 = 'de:08116:4268:1:4'  # eRtg Konrad-Widerholt-Halle (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__EICHENDORFFSTR = 'de:08116:4269'
	KIRCHHEIM__T__EICHENDORFFSTR_1 = 'de:08116:4269:0:3'  # eRtg Bismarckstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__EICHENDORFFSTR_2 = 'de:08116:4269:0:4'  # eRtg Tannenbergstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__STRASSENMEISTEREI = 'de:08116:4270'
	KIRCHHEIM__T__STRASSENMEISTEREI_1 = 'de:08116:4270:0:3'  # eRtg Robert-Bosch-Straße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__STRASSENMEISTEREI_2 = 'de:08116:4270:0:4'  # eRtg Polizeiwache (Kirchheim unter Teck Kirchheim (T))
	STRASSENMEISTEREI_1 = 'de:08116:4270:0:3'  # eRtg Robert-Bosch-Straße (Kirchheim unter Teck Kirchheim (T))
	STRASSENMEISTEREI_2 = 'de:08116:4270:0:4'  # eRtg Polizeiwache (Kirchheim unter Teck Kirchheim (T))
	STRASSENMEISTEREI_3 = 'de:08115:4855:0:3'  # eRtg Nebringen/Tailfingen (Herrenberg Herrenberg)
	STRASSENMEISTEREI_4 = 'de:08115:4855:0:4'  # eRtg Bahnhofstraße (Herrenberg Herrenberg)
	DETTINGEN__T__R__BOSCH_STRASSE = 'de:08116:4272'
	DETTINGEN__T__R__BOSCH_STRASSE_1 = 'de:08116:4272:0:3'  # eRtg Römerstraße (Dettingen unter Teck Dettingen (T))
	DETTINGEN__T__R__BOSCH_STRASSE_2 = 'de:08116:4272:0:4'  # eRtg Straßenmeisterei (Dettingen unter Teck Dettingen (T))
	HANFSTRASSE = 'de:08116:4273:1:3'  # eRtg Robert-Bosch-Straße (Dettingen unter Teck Dettingen (T))
	DETTINGEN__T__HANFSTRASSE = 'de:08116:4273:1:3'  # eRtg Robert-Bosch-Straße (Dettingen unter Teck Dettingen (T))
	DETTINGEN__T__VOLKSBANK = 'de:08116:4274'
	DETTINGEN__T__VOLKSBANK_1 = 'de:08116:4274:1:3'  # Rtg Hanfstraße (Dettingen unter Teck Dettingen (T))
	DETTINGEN__T__VOLKSBANK_2 = 'de:08116:4274:1:4'  # Rtg Obere Straße (Dettingen unter Teck Dettingen (T))
	VOLKSBANK_1 = 'de:08116:4274:1:3'  # Rtg Hanfstraße (Dettingen unter Teck Dettingen (T))
	VOLKSBANK_2 = 'de:08116:4274:1:4'  # Rtg Obere Straße (Dettingen unter Teck Dettingen (T))
	VOLKSBANK_3 = 'de:08118:5479:0:3'  # eRtg Stuttgarter Straße (Asperg Asperg)
	VOLKSBANK_4 = 'de:08117:3226:0:H1'  # Rtg Gosbach (Bad Ditzenbach Auendorf)
	VOLKSBANK_5 = 'de:08117:3226:0:R1'  # Rtg Fuchsecksattel (Bad Ditzenbach Auendorf)
	VOLKSBANK_6 = 'de:08117:7656:0:H1'  # Rtg. SOS Kinderdorf (Adelberg Adelberg)
	VOLKSBANK_7 = 'de:08117:7656:0:R1'  # Rtg Abzw. Kloster (Adelberg Adelberg)
	DETTINGEN__T__OBERE_STRASSE = 'de:08116:4275'
	DETTINGEN__T__OBERE_STRASSE_1 = 'de:08116:4275:0:3'  # eRtg Goldmorgen (Dettingen unter Teck Dettingen (T))
	DETTINGEN__T__OBERE_STRASSE_2 = 'de:08116:4275:0:4'  # eRtg Volksbank (Dettingen unter Teck Dettingen (T))
	OBERE_STRASSE = 'de:08116:4275'
	OBERE_STRASSE_1 = 'de:08116:4275:0:3'  # eRtg Goldmorgen (Dettingen unter Teck Dettingen (T))
	OBERE_STRASSE_2 = 'de:08116:4275:0:4'  # eRtg Volksbank (Dettingen unter Teck Dettingen (T))
	KATHOLISCHE_KIRCHE_1 = 'de:08116:4277:0:3'  # eRtg Industriegebiet (Dettingen unter Teck Dettingen (T))
	KATHOLISCHE_KIRCHE_2 = 'de:08116:4277:0:4'  # eRtg Schlossberghalle (Dettingen unter Teck Dettingen (T))
	KATHOLISCHE_KIRCHE_3 = 'de:08116:4314:0:3'  # eRtg Heerweg (Bempflingen Bempflingen)
	KATHOLISCHE_KIRCHE_4 = 'de:08116:4314:0:4'  # eRtg Lindenstraße (Bempflingen Bempflingen)
	KATHOLISCHE_KIRCHE_5 = 'de:08115:4725:0:3'  # Rtg Hermann-Hesse-Str. (Aidlingen Aidlingen)
	KATHOLISCHE_KIRCHE_6 = 'de:08115:4725:0:4'  # Rtg Rathaus (Aidlingen Aidlingen)
	KATHOLISCHE_KIRCHE_7 = 'de:08119:5171:0:4'  # Rtg Heckbachstr./Großh. Kleinh.Str. (Weinstadt Großheppach)
	KATHOLISCHE_KIRCHE_8 = 'de:08119:5429:1:3'  # eRtg Aldinger Straße (Waiblingen Hegnach)
	KATHOLISCHE_KIRCHE_9 = 'de:08118:5613:0:3'  # Rtg Beilstein (Oberstenfeld Oberstenfeld)
	KATHOLISCHE_KIRCHE_10 = 'de:08118:5613:0:4'  # Rtg Oberstenfeld (Oberstenfeld Oberstenfeld)
	KATHOLISCHE_KIRCHE_11 = 'de:08115:7015:0:3'  # Rtg Altes Rathaus Mötzinger Straße (Jettingen Unterjettingen)
	KATHOLISCHE_KIRCHE_12 = 'de:08115:7015:0:4'  # Rtg Sindlingen Öschelbronn (Jettingen Unterjettingen)
	KATHOLISCHE_KIRCHE_13 = 'de:08118:7085:0:3'  # eRtg Schwalbenhälde (Besigheim Besigheim)
	KATHOLISCHE_KIRCHE_14 = 'de:08118:7085:0:4'  # eRtg Enzplatz (Besigheim Besigheim)
	KATHOLISCHE_KIRCHE_15 = 'de:08117:12114:0:1'  # Rtg Schillerstraße (Zell unter Aichelberg Zell (A))
	KATHOLISCHE_KIRCHE_16 = 'de:08117:12114:0:2'  # Rtg Hattenh./Ohmden (Zell unter Aichelberg Zell (A))
	KATHOLISCHE_KIRCHE_17 = 'de:08117:2114:0:H1'  # Rtg Kirchheimer Straße (Zell unter Aichelberg Zell (A))
	KATHOLISCHE_KIRCHE_18 = 'de:08117:2114:0:R1'  # Rtg Schillerstr. (Zell unter Aichelberg Zell (A))
	KATHOLISCHE_KIRCHE_19 = 'de:08117:3239:0:H1'  # Rtg Mühlhausen i. T. (Gruibingen Gruibingen)
	DETTINGEN__T__KATH__KIRCHE = 'de:08116:4277'
	DETTINGEN__T__KATH__KIRCHE_1 = 'de:08116:4277:0:3'  # eRtg Industriegebiet (Dettingen unter Teck Dettingen (T))
	DETTINGEN__T__KATH__KIRCHE_2 = 'de:08116:4277:0:4'  # eRtg Schlossberghalle (Dettingen unter Teck Dettingen (T))
	DETTINGEN__T__GOLDMORGEN = 'de:08116:4278'
	DETTINGEN__T__GOLDMORGEN_1 = 'de:08116:4278:0:3'  # Rtg Volksbank (Dettingen unter Teck Dettingen (T))
	DETTINGEN__T__GOLDMORGEN_2 = 'de:08116:4278:0:4'  # Rtg Owen (Dettingen unter Teck Dettingen (T))
	GOLDMORGEN = 'de:08116:4278'
	GOLDMORGEN_1 = 'de:08116:4278:0:3'  # Rtg Volksbank (Dettingen unter Teck Dettingen (T))
	GOLDMORGEN_2 = 'de:08116:4278:0:4'  # Rtg Owen (Dettingen unter Teck Dettingen (T))
	OWEN_TECKHALLE = 'de:08116:4279'
	OWEN_TECKHALLE_1 = 'de:08116:4279:0:3'  # eRtg Beurener Straße (Owen Owen)
	OWEN_TECKHALLE_2 = 'de:08116:4279:0:4'  # eRtg Goldmorgen (Owen Owen)
	TECKHALLE = 'de:08116:4279'
	TECKHALLE_1 = 'de:08116:4279:0:3'  # eRtg Beurener Straße (Owen Owen)
	TECKHALLE_2 = 'de:08116:4279:0:4'  # eRtg Goldmorgen (Owen Owen)
	OWEN_GESCHICHTSHAUS = 'de:08116:4280'
	OWEN_GESCHICHTSHAUS_1 = 'de:08116:4280:1:3'  # Rtg Owen Bahnhof/Lenningen (Owen Owen)
	OWEN_GESCHICHTSHAUS_2 = 'de:08116:4280:1:4'  # Rtg Teckhalle (Owen Owen)
	GESCHICHTSHAUS = 'de:08116:4280'
	GESCHICHTSHAUS_1 = 'de:08116:4280:1:3'  # Rtg Owen Bahnhof/Lenningen (Owen Owen)
	GESCHICHTSHAUS_2 = 'de:08116:4280:1:4'  # Rtg Teckhalle (Owen Owen)
	REBENWEG = 'de:08116:4281'
	REBENWEG_1 = 'de:08116:4281:0:3'  # eRtg Brucken Untere Straße (Owen Owen)
	REBENWEG_2 = 'de:08116:4281:0:4'  # eRtg Beurener Straße (Owen Owen)
	OWEN_REBENWEG = 'de:08116:4281'
	OWEN_REBENWEG_1 = 'de:08116:4281:0:3'  # eRtg Brucken Untere Straße (Owen Owen)
	OWEN_REBENWEG_2 = 'de:08116:4281:0:4'  # eRtg Beurener Straße (Owen Owen)
	UNTERL__BRUCKEN_UNTERE_STRASSE = 'de:08116:4282'
	UNTERL__BRUCKEN_UNTERE_STRASSE_1 = 'de:08116:4282:1:3'  # Rtg Owen (Lenningen Unterlenningen)
	UNTERL__BRUCKEN_UNTERE_STRASSE_2 = 'de:08116:4282:1:4'  # Rtg Unterlenningen (Lenningen Unterlenningen)
	BRUCKEN_UNTERE_STRASSE = 'de:08116:4282'
	BRUCKEN_UNTERE_STRASSE_1 = 'de:08116:4282:1:3'  # Rtg Owen (Lenningen Unterlenningen)
	BRUCKEN_UNTERE_STRASSE_2 = 'de:08116:4282:1:4'  # Rtg Unterlenningen (Lenningen Unterlenningen)
	UNTERLENNINGEN_BAHNHOFSTRASSE = 'de:08116:4283'
	UNTERLENNINGEN_BAHNHOFSTRASSE_1 = 'de:08116:4283:1:3'  # Rtg Brucken (Lenningen Unterlenningen)
	UNTERLENNINGEN_BAHNHOFSTRASSE_2 = 'de:08116:4283:1:4'  # Rtg Oberlenningen (Lenningen Unterlenningen)
	UNTERLENNINGEN_RATHAUS = 'de:08116:4284'
	UNTERLENNINGEN_RATHAUS_1 = 'de:08116:4284:0:3'  # eRtg Hochwang Rathaus/Bahnhofstraße (Lenningen Unterlenningen)
	UNTERLENNINGEN_RATHAUS_2 = 'de:08116:4284:0:4'  # eRtg Oberlenningen (Lenningen Unterlenningen)
	SIRNAU_ECKENERSTRASSE = 'de:08116:4286'
	SIRNAU_ECKENERSTRASSE_1 = 'de:08116:4286:0:3'  # eRtg Wolf-Hirth-Straße (Esslingen am Neckar Sirnau)
	SIRNAU_ECKENERSTRASSE_2 = 'de:08116:4286:0:4'  # eRtg Sirnauer Hof (Esslingen am Neckar Sirnau)
	ECKENERSTRASSE = 'de:08116:4286'
	ECKENERSTRASSE_1 = 'de:08116:4286:0:3'  # eRtg Wolf-Hirth-Straße (Esslingen am Neckar Sirnau)
	ECKENERSTRASSE_2 = 'de:08116:4286:0:4'  # eRtg Sirnauer Hof (Esslingen am Neckar Sirnau)
	WENDLINGEN__N__INDUSTRIEGEBIET = 'de:08116:4287'
	WENDLINGEN__N__INDUSTRIEGEBIET_1 = 'de:08116:4287:0:3'  # Rtg Unterensingen (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N__INDUSTRIEGEBIET_2 = 'de:08116:4287:0:4'  # Rtg Wendlingen (Wendlingen am Neckar Wendlingen (N))
	KÖNGEN_WILHELM_MAIER_STRASSE = 'de:08116:4288:0:3'  # eRtg Wertstraße (Köngen Köngen)
	WILHELM_MAIER_STRASSE = 'de:08116:4288:0:3'  # eRtg Wertstraße (Köngen Köngen)
	UNTERENSINGEN_BLUMENSTRASSE = 'de:08116:4289'
	UNTERENSINGEN_BLUMENSTRASSE_1 = 'de:08116:4289:0:3'  # eRtg Industriegebiet (Unterensingen Unterensingen)
	UNTERENSINGEN_BLUMENSTRASSE_2 = 'de:08116:4289:0:4'  # eRtg Stern (Unterensingen Unterensingen)
	UNTERENSINGEN_STERN = 'de:08116:4290'
	UNTERENSINGEN_STERN_1 = 'de:08116:4290:0:3'  # eRtg Blumenstraße (Unterensingen Unterensingen)
	UNTERENSINGEN_STERN_2 = 'de:08116:4290:0:4'  # eRtg Linde (Unterensingen Unterensingen)
	WENDLINGEN__N__ALTER_SPORTPL = 'de:08116:4293:0:3'  # Rtg Austr. (Wendlingen am Neckar Wendlingen (N))
	ALTER_SPORTPLATZ = 'de:08116:4293:0:3'  # Rtg Austr. (Wendlingen am Neckar Wendlingen (N))
	KAPELLENSTRASSE_1 = 'de:08116:4295:0:3'  # eRtg Schulzentrum (Wendlingen am Neckar Wendlingen (N))
	KAPELLENSTRASSE_2 = 'de:08117:7610:0:H1'  # Rtg Stiegelwiese (Süßen Süßen)
	WENDLINGEN__N__KAPELLENSTRASSE = 'de:08116:4295:0:3'  # eRtg Schulzentrum (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N__SCHULZENTRUM = 'de:08116:4296:0:3'  # eRtg Schützenstr./Anne-Frank-Schule (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N__SCHÜTZENSTRASSE = 'de:08116:4297'
	WENDLINGEN__N__SCHÜTZENSTRASSE_1 = 'de:08116:4297:0:3'  # eRtg Stadtmitte/Taläckerstr. (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N__SCHÜTZENSTRASSE_2 = 'de:08116:4297:0:4'  # eRtg Lerchenstraße (Wendlingen am Neckar Wendlingen (N))
	SCHÜTZENSTRASSE_1 = 'de:08116:4297:0:3'  # eRtg Stadtmitte/Taläckerstr. (Wendlingen am Neckar Wendlingen (N))
	SCHÜTZENSTRASSE_2 = 'de:08116:4297:0:4'  # eRtg Lerchenstraße (Wendlingen am Neckar Wendlingen (N))
	SCHÜTZENSTRASSE_3 = 'de:08117:1057:0:H1'  # Linie 13/91/7688 (Göppingen Göppingen)
	SCHÜTZENSTRASSE_4 = 'de:08117:1057:0:H2'  # Linie 1/3/6/16 (Göppingen Göppingen)
	SCHÜTZENSTRASSE_5 = 'de:08117:1057:0:H3'  # Linie 4/11/94/31/32/33 (Göppingen Göppingen)
	LERCHENSTRASSE_1 = 'de:08116:4298:0:3'  # eRtg Kreissparkasse (Oberboihingen Oberboihingen)
	LERCHENSTRASSE_2 = 'de:08116:4298:0:4'  # eRtg Schützenstraße (Oberboihingen Oberboihingen)
	LERCHENSTRASSE_3 = 'de:08116:4362:0:3'  # eRtg Schluchtweg (Weilheim an der Teck Weilheim (T))
	LERCHENSTRASSE_4 = 'de:08119:5356:1:3'  # Rtg Klöpferbach (Aspach Großaspach)
	LERCHENSTRASSE_5 = 'de:08119:5356:1:4'  # Rtg Rossbergstaffel (Aspach Großaspach)
	OBERBOIHINGEN_LERCHENSTRASSE = 'de:08116:4298'
	OBERBOIHINGEN_LERCHENSTRASSE_1 = 'de:08116:4298:0:3'  # eRtg Kreissparkasse (Oberboihingen Oberboihingen)
	OBERBOIHINGEN_LERCHENSTRASSE_2 = 'de:08116:4298:0:4'  # eRtg Schützenstraße (Oberboihingen Oberboihingen)
	OBERBOIHINGEN_KREISSPARKASSE = 'de:08116:4299'
	OBERBOIHINGEN_KREISSPARKASSE_1 = 'de:08116:4299:1:3'  # eRtg Lerchenstraße (Oberboihingen Oberboihingen)
	OBERBOIHINGEN_KREISSPARKASSE_2 = 'de:08116:4299:1:4'  # eRtg Stattmannstraße (Oberboihingen Oberboihingen)
	OBERBOIHINGEN_STATTMANNSTRASSE = 'de:08116:4300'
	OBERBOIHINGEN_STATTMANNSTRASSE_1 = 'de:08116:4300:0:3'  # eRtg Kreissparkasse (Oberboihingen Oberboihingen)
	OBERBOIHINGEN_STATTMANNSTRASSE_2 = 'de:08116:4300:0:4'  # eRtg Inselbad (Oberboihingen Oberboihingen)
	STATTMANNSTRASSE = 'de:08116:4300'
	STATTMANNSTRASSE_1 = 'de:08116:4300:0:3'  # eRtg Kreissparkasse (Oberboihingen Oberboihingen)
	STATTMANNSTRASSE_2 = 'de:08116:4300:0:4'  # eRtg Inselbad (Oberboihingen Oberboihingen)
	INSELBAD = 'de:08116:4301'
	INSELBAD_1 = 'de:08116:4301:0:3'  # eRtg Stattmannstr./Unterensinger Str./We (Nürtingen Zizishausen)
	INSELBAD_2 = 'de:08116:4301:0:4'  # eRtg Weberstr./Stattmannstr./Schule (Nürtingen Zizishausen)
	ZIZISHAUSEN_INSELBAD = 'de:08116:4301'
	ZIZISHAUSEN_INSELBAD_1 = 'de:08116:4301:0:3'  # eRtg Stattmannstr./Unterensinger Str./We (Nürtingen Zizishausen)
	ZIZISHAUSEN_INSELBAD_2 = 'de:08116:4301:0:4'  # eRtg Weberstr./Stattmannstr./Schule (Nürtingen Zizishausen)
	NÜRTINGEN_REALSCHULE = 'de:08116:4302'
	NÜRTINGEN_REALSCHULE_1 = 'de:08116:4302:0:3'  # eRtg Nürtingen (Nürtingen Nürtingen)
	NÜRTINGEN_REALSCHULE_2 = 'de:08116:4302:0:4'  # Rtg Bauhof (Nürtingen Nürtingen)
	ZELLER_STRASSE_1 = 'de:08116:4303:0:3'  # eRtg Siedlung (Ohmden Ohmden)
	ZELLER_STRASSE_2 = 'de:08116:4303:0:4'  # eRtg Rathaus (Ohmden Ohmden)
	ZELLER_STRASSE_3 = 'de:08117:2050:0:H1'  # Rtg Graubachtal (Hattenhofen Hattenhofen)
	ZELLER_STRASSE_4 = 'de:08117:2050:0:R1'  # Rtg Zell (A) (Hattenhofen Hattenhofen)
	OHMDEN_ZELLER_STRASSE = 'de:08116:4303'
	OHMDEN_ZELLER_STRASSE_1 = 'de:08116:4303:0:3'  # eRtg Siedlung (Ohmden Ohmden)
	OHMDEN_ZELLER_STRASSE_2 = 'de:08116:4303:0:4'  # eRtg Rathaus (Ohmden Ohmden)
	NÜRTINGEN_SCHILLERPLATZ = 'de:08116:4304:0:4'  # Rtg Süd (Nürtingen Nürtingen)
	NÜRTINGEN_METZINGER_STRASSE = 'de:08116:4305'
	NÜRTINGEN_METZINGER_STRASSE_1 = 'de:08116:4305:0:3'  # eRtg Kroatenhof (Nürtingen Nürtingen)
	NÜRTINGEN_METZINGER_STRASSE_2 = 'de:08116:4305:0:4'  # eRtg Großbettlingen/Humpfentalstr/Verdis (Nürtingen Nürtingen)
	WEILHEIM__T__TOBELWASEN = 'de:08116:4306'
	WEILHEIM__T__TOBELWASEN_1 = 'de:08116:4306:0:3'  # Rtg ??? (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__TOBELWASEN_2 = 'de:08116:4306:0:4'  # Rtg ??? (Weilheim an der Teck Weilheim (T))
	TOBELWASEN = 'de:08116:4306'
	TOBELWASEN_1 = 'de:08116:4306:0:3'  # Rtg ??? (Weilheim an der Teck Weilheim (T))
	TOBELWASEN_2 = 'de:08116:4306:0:4'  # Rtg ??? (Weilheim an der Teck Weilheim (T))
	KIRCHHEIM__T__AM_WIESENRAIN = 'de:08116:4307'
	KIRCHHEIM__T__AM_WIESENRAIN_1 = 'de:08116:4307:0:3'  # eRtg Kiebitzweg (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__AM_WIESENRAIN_2 = 'de:08116:4307:0:4'  # eRtg Kiebitzweg (Kirchheim unter Teck Kirchheim (T))
	AM_WIESENRAIN = 'de:08116:4307'
	AM_WIESENRAIN_1 = 'de:08116:4307:0:3'  # eRtg Kiebitzweg (Kirchheim unter Teck Kirchheim (T))
	AM_WIESENRAIN_2 = 'de:08116:4307:0:4'  # eRtg Kiebitzweg (Kirchheim unter Teck Kirchheim (T))
	NECKARTENZLINGEN_EICHWASEN = 'de:08116:4308'
	NECKARTENZLINGEN_EICHWASEN_1 = 'de:08116:4308:0:3'  # Rtg Neckartenzlingen (Neckartenzlingen Neckartenzlingen)
	NECKARTENZLINGEN_EICHWASEN_2 = 'de:08116:4308:0:4'  # Rtg Altdorf (Neckartenzlingen Neckartenzlingen)
	EICHWASEN = 'de:08116:4308'
	EICHWASEN_1 = 'de:08116:4308:0:3'  # Rtg Neckartenzlingen (Neckartenzlingen Neckartenzlingen)
	EICHWASEN_2 = 'de:08116:4308:0:4'  # Rtg Altdorf (Neckartenzlingen Neckartenzlingen)
	NECKARTENZL__METZINGER_STRASSE = 'de:08116:4309'
	NECKARTENZL__METZINGER_STRASSE_1 = 'de:08116:4309:0:3'  # eRtg Auwiesenschule (Neckartenzlingen Neckartenzlingen)
	NECKARTENZL__METZINGER_STRASSE_2 = 'de:08116:4309:0:4'  # eRtg Marktplatz (Neckartenzlingen Neckartenzlingen)
	NECKARTENZL__AUWIESENSCHULE = 'de:08116:4310'
	NECKARTENZL__AUWIESENSCHULE_1 = 'de:08116:4310:0:3'  # Rtg Metzinger Str. (Neckartenzlingen Neckartenzlingen)
	NECKARTENZL__AUWIESENSCHULE_2 = 'de:08116:4310:0:4'  # Rtg Kleiderwerk Adler (Neckartenzlingen Neckartenzlingen)
	NECKARTENZL__AUWIESENSCHULE_3 = 'de:08116:4310:0:5'  # alle Anderen / SchulZOB (Neckartenzlingen Neckartenzlingen)
	AUWIESENSCHULE = 'de:08116:4310'
	AUWIESENSCHULE_1 = 'de:08116:4310:0:3'  # Rtg Metzinger Str. (Neckartenzlingen Neckartenzlingen)
	AUWIESENSCHULE_2 = 'de:08116:4310:0:4'  # Rtg Kleiderwerk Adler (Neckartenzlingen Neckartenzlingen)
	AUWIESENSCHULE_3 = 'de:08116:4310:0:5'  # alle Anderen / SchulZOB (Neckartenzlingen Neckartenzlingen)
	NECKARTENZL__KLEIDERWERK_ADLER = 'de:08116:4311'
	NECKARTENZL__KLEIDERWERK_ADLER_1 = 'de:08116:4311:0:3'  # eRtg Neckartenzlinger Straße (Neckartenzlingen Neckartenzlingen)
	NECKARTENZL__KLEIDERWERK_ADLER_2 = 'de:08116:4311:0:4'  # eRtg Auwiesenschule (Neckartenzlingen Neckartenzlingen)
	KLEIDERWERK_ADLER = 'de:08116:4311'
	KLEIDERWERK_ADLER_1 = 'de:08116:4311:0:3'  # eRtg Neckartenzlinger Straße (Neckartenzlingen Neckartenzlingen)
	KLEIDERWERK_ADLER_2 = 'de:08116:4311:0:4'  # eRtg Auwiesenschule (Neckartenzlingen Neckartenzlingen)
	GROSSBETTLINGEN_HEERWEG = 'de:08116:4312'
	GROSSBETTLINGEN_HEERWEG_1 = 'de:08116:4312:0:3'  # Rtg Rathaus (Großbettlingen Großbettlingen)
	GROSSBETTLINGEN_HEERWEG_2 = 'de:08116:4312:0:4'  # Rtg Bempfl. Kath. Kirch/Neckartenzl. (Großbettlingen Großbettlingen)
	HEERWEG = 'de:08116:4312'
	HEERWEG_1 = 'de:08116:4312:0:3'  # Rtg Rathaus (Großbettlingen Großbettlingen)
	HEERWEG_2 = 'de:08116:4312:0:4'  # Rtg Bempfl. Kath. Kirch/Neckartenzl. (Großbettlingen Großbettlingen)
	NECKARTENZLINGER_STRASSE = 'de:08116:4313'
	NECKARTENZLINGER_STRASSE_1 = 'de:08116:4313:0:3'  # Rtg Lindenstr. (Bempflingen Bempflingen)
	NECKARTENZLINGER_STRASSE_2 = 'de:08116:4313:0:4'  # Rtg Neckartenzlingen/Kleiderwerk Adler (Bempflingen Bempflingen)
	BEMPFL__NECKARTENZLINGER_STR = 'de:08116:4313'
	BEMPFL__NECKARTENZLINGER_STR_1 = 'de:08116:4313:0:3'  # Rtg Lindenstr. (Bempflingen Bempflingen)
	BEMPFL__NECKARTENZLINGER_STR_2 = 'de:08116:4313:0:4'  # Rtg Neckartenzlingen/Kleiderwerk Adler (Bempflingen Bempflingen)
	BEMPFLINGEN_KATHOLISCHE_KIRCHE = 'de:08116:4314'
	BEMPFLINGEN_KATHOLISCHE_KIRCHE_1 = 'de:08116:4314:0:3'  # eRtg Heerweg (Bempflingen Bempflingen)
	BEMPFLINGEN_KATHOLISCHE_KIRCHE_2 = 'de:08116:4314:0:4'  # eRtg Lindenstraße (Bempflingen Bempflingen)
	LINDENSTRASSE_1 = 'de:08116:4315:0:3'  # eRtg Grafenberger Straße/Kath. Kirche (Bempflingen Bempflingen)
	LINDENSTRASSE_2 = 'de:08116:4315:0:4'  # eRtg Neckartenzlinger Straße (Bempflingen Bempflingen)
	LINDENSTRASSE_3 = 'de:08118:5467:0:3'  # eRtg Sonatenweg (Kornwestheim Kornwestheim)
	LINDENSTRASSE_4 = 'de:08118:5467:0:4'  # eRtg Kornwestheim (Kornwestheim Kornwestheim)
	LINDENSTRASSE_5 = 'de:08118:6822:0:3'  # eRtg Friedhof (Remseck am Neckar Hochberg)
	LINDENSTRASSE_6 = 'de:08118:6822:0:4'  # eRtg Alexandrinenplatz (Remseck am Neckar Hochberg)
	LINDENSTRASSE_7 = 'de:08115:7120:0:1'  # Rtg Goldäcker (Steinenbronn Steinenbronn)
	LINDENSTRASSE_8 = 'de:08115:7120:0:2'  # Rtg Waldenbuch (Steinenbronn Steinenbronn)
	LINDENSTRASSE_9 = 'de:08118:7425:0:3'  # eRtg Südstraße (Bietigheim-Bissingen Bissingen (LB))
	LINDENSTRASSE_10 = 'de:08118:7425:0:4'  # eRtg Eichenweg (Bietigheim-Bissingen Bissingen (LB))
	LINDENSTRASSE_11 = 'de:08117:3378:0:H1'  # Rtg Friedhof (Heiningen Heiningen (GP))
	LINDENSTRASSE_12 = 'de:08117:3378:0:R1'  # Rtg Breitestr. (Heiningen Heiningen (GP))
	LINDENSTRASSE_13 = 'de:08117:3378:0:R2'  # Rtg Adler (Heiningen Heiningen (GP))
	LINDENSTRASSE_14 = 'de:08136:7822:0:1'  # Lindenstraße (Waldstetten Wißgoldingen)
	LINDENSTRASSE_15 = 'de:08425:2636:0:1'  # Ortsmitte (Merklingen Merklingen)
	LINDENSTRASSE_16 = 'de:08425:2636:0:2'  # Machtolsheim (Merklingen Merklingen)
	BEMPFLINGEN_LINDENSTRASSE = 'de:08116:4315'
	BEMPFLINGEN_LINDENSTRASSE_1 = 'de:08116:4315:0:3'  # eRtg Grafenberger Straße/Kath. Kirche (Bempflingen Bempflingen)
	BEMPFLINGEN_LINDENSTRASSE_2 = 'de:08116:4315:0:4'  # eRtg Neckartenzlinger Straße (Bempflingen Bempflingen)
	BEMPFLINGEN_RATHAUS = 'de:08116:4316'
	BEMPFLINGEN_RATHAUS_1 = 'de:08116:4316:1:3'  # eRtg Lindenstraße (Bempflingen Bempflingen)
	BEMPFLINGEN_RATHAUS_2 = 'de:08116:4316:1:4'  # eRtg Riederich Stuttg. Straße (Bempflingen Bempflingen)
	KIRCHHEIM__T__MARTINSKIRCHE = 'de:08116:4317:1:3'  # eRtg Teckstraße/Austraße/Ziegelwasen (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__TECKSTRASSE = 'de:08116:4319'
	KIRCHHEIM__T__TECKSTRASSE_1 = 'de:08116:4319:1:3'  # Rtg Freibad (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__TECKSTRASSE_2 = 'de:08116:4319:1:4'  # Rtg Bahnhof (Kirchheim unter Teck Kirchheim (T))
	TECKSTRASSE = 'de:08116:4319'
	TECKSTRASSE_1 = 'de:08116:4319:1:3'  # Rtg Freibad (Kirchheim unter Teck Kirchheim (T))
	TECKSTRASSE_2 = 'de:08116:4319:1:4'  # Rtg Bahnhof (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__STADION = 'de:08116:4320'
	KIRCHHEIM__T__STADION_1 = 'de:08116:4320:0:3'  # eRtg Fauslerstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__STADION_2 = 'de:08116:4320:0:4'  # eRtg Freibad/Kirchheim (Kirchheim unter Teck Kirchheim (T))
	KOHLBERG_HÖRNLESWEG = 'de:08116:4321'
	KOHLBERG_HÖRNLESWEG_1 = 'de:08116:4321:0:3'  # eRtg Etzelweg (Kohlberg Kohlberg)
	KOHLBERG_HÖRNLESWEG_2 = 'de:08116:4321:0:4'  # eRtg Schule (Kohlberg Kohlberg)
	HÖRNLESWEG = 'de:08116:4321'
	HÖRNLESWEG_1 = 'de:08116:4321:0:3'  # eRtg Etzelweg (Kohlberg Kohlberg)
	HÖRNLESWEG_2 = 'de:08116:4321:0:4'  # eRtg Schule (Kohlberg Kohlberg)
	JESINGEN_WARTEHALLE = 'de:08116:4322'
	JESINGEN_WARTEHALLE_1 = 'de:08116:4322:0:3'  # Rtg Rathaus (Kirchheim unter Teck Jesingen)
	JESINGEN_WARTEHALLE_2 = 'de:08116:4322:0:4'  # Rtg Fauslerstr. (Kirchheim unter Teck Jesingen)
	WARTEHALLE_1 = 'de:08116:4322:0:3'  # Rtg Rathaus (Kirchheim unter Teck Jesingen)
	WARTEHALLE_2 = 'de:08116:4322:0:4'  # Rtg Fauslerstr. (Kirchheim unter Teck Jesingen)
	WARTEHALLE_3 = 'de:08135:1138:0:1'  # Alle Richtungen (Gerstetten Gerstetten)
	JESINGEN_RATHAUS = 'de:08116:4323'
	JESINGEN_RATHAUS_1 = 'de:08116:4323:0:3'  # Rtg Holzmaden (Kirchheim unter Teck Jesingen)
	JESINGEN_RATHAUS_2 = 'de:08116:4323:0:4'  # Rtg Kirchheim von Holzmaden (Kirchheim unter Teck Jesingen)
	JESINGEN_RATHAUS_3 = 'de:08116:4323:0:5'  # Rtg Ohmden (Kirchheim unter Teck Jesingen)
	JESINGEN_RATHAUS_4 = 'de:08116:4323:0:6'  # Rtg Kirchheim von Ohmden (Kirchheim unter Teck Jesingen)
	HOLZMADEN_BF = 'de:08116:4324'
	HOLZMADEN_BF_1 = 'de:08116:4324:0:3'  # eRtg Maierhöfe (Holzmaden Holzmaden)
	HOLZMADEN_BF_2 = 'de:08116:4324:0:4'  # eRtg Rathaus (Holzmaden Holzmaden)
	WEILHEIM__T__SCHLUCHTWEG = 'de:08116:4325'
	WEILHEIM__T__SCHLUCHTWEG_1 = 'de:08116:4325:0:3'  # Rtg Jahnstr. (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__SCHLUCHTWEG_2 = 'de:08116:4325:0:4'  # Rtg KSK (Weilheim an der Teck Weilheim (T))
	SCHLUCHTWEG = 'de:08116:4325'
	SCHLUCHTWEG_1 = 'de:08116:4325:0:3'  # Rtg Jahnstr. (Weilheim an der Teck Weilheim (T))
	SCHLUCHTWEG_2 = 'de:08116:4325:0:4'  # Rtg KSK (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__BAHNHOFSTRASSE = 'de:08116:4326:0:3'  # Rtg ??? (Weilheim an der Teck Weilheim (T))
	ALTVATERWEG = 'de:08116:4328:1:3'  # eRtg Freiwaldaustraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__ALTVATERWEG = 'de:08116:4328:1:3'  # eRtg Freiwaldaustraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__FREIWALDAUSTRASSE = 'de:08116:4329:0:3'  # eRtg Spielplatz (Kirchheim unter Teck Kirchheim (T))
	FREIWALDAUSTRASSE = 'de:08116:4329:0:3'  # eRtg Spielplatz (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BULKESWEG = 'de:08116:4330'
	KIRCHHEIM__T__BULKESWEG_1 = 'de:08116:4330:0:3'  # eRtg Tannenbergstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BULKESWEG_2 = 'de:08116:4330:0:4'  # eRtg Spielplatz (Kirchheim unter Teck Kirchheim (T))
	BULKESWEG = 'de:08116:4330'
	BULKESWEG_1 = 'de:08116:4330:0:3'  # eRtg Tannenbergstraße (Kirchheim unter Teck Kirchheim (T))
	BULKESWEG_2 = 'de:08116:4330:0:4'  # eRtg Spielplatz (Kirchheim unter Teck Kirchheim (T))
	TANNENBERGSTRASSE = 'de:08116:4331'
	TANNENBERGSTRASSE_1 = 'de:08116:4331:0:3'  # eRtg Eichendorffstraße (Kirchheim unter Teck Kirchheim (T))
	TANNENBERGSTRASSE_2 = 'de:08116:4331:0:4'  # eRtg Bulkesweg/Bohnauhaus (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__TANNENBERGSTRASSE = 'de:08116:4331'
	KIRCHHEIM__T__TANNENBERGSTRASSE_1 = 'de:08116:4331:0:3'  # eRtg Eichendorffstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__TANNENBERGSTRASSE_2 = 'de:08116:4331:0:4'  # eRtg Bulkesweg/Bohnauhaus (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BISMARCKSTRASSE = 'de:08116:4332'
	KIRCHHEIM__T__BISMARCKSTRASSE_1 = 'de:08116:4332:0:3'  # eRtg Konrad-Widerholt-Halle (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BISMARCKSTRASSE_2 = 'de:08116:4332:0:4'  # eRtg Eichendorffstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__K__W__HALLE = 'de:08116:4333'
	KIRCHHEIM__T__K__W__HALLE_1 = 'de:08116:4333:1:3'  # eRtg Fußgängerzone (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__K__W__HALLE_2 = 'de:08116:4333:1:4'  # eRtg Bismarckstraße (Kirchheim unter Teck Kirchheim (T))
	KONRAD_WIDERHOLT_HALLE = 'de:08116:4333'
	KONRAD_WIDERHOLT_HALLE_1 = 'de:08116:4333:1:3'  # eRtg Fußgängerzone (Kirchheim unter Teck Kirchheim (T))
	KONRAD_WIDERHOLT_HALLE_2 = 'de:08116:4333:1:4'  # eRtg Bismarckstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__SPIELPLATZ = 'de:08116:4334'
	KIRCHHEIM__T__SPIELPLATZ_1 = 'de:08116:4334:0:3'  # eRtg Bulkesweg (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__SPIELPLATZ_2 = 'de:08116:4334:0:4'  # eRtg Altvaterweg (Kirchheim unter Teck Kirchheim (T))
	SPIELPLATZ = 'de:08116:4334'
	SPIELPLATZ_1 = 'de:08116:4334:0:3'  # eRtg Bulkesweg (Kirchheim unter Teck Kirchheim (T))
	SPIELPLATZ_2 = 'de:08116:4334:0:4'  # eRtg Altvaterweg (Kirchheim unter Teck Kirchheim (T))
	PARKPLATZ_SCHLOSSGYMNASIUM = 'de:08116:4335:0:3'  # eRtg Alle Richtungen (Kirchheim unter Teck Kirchheim (T))
	KIRCHH__T__PARKPL__SCHLOSSG = 'de:08116:4335:0:3'  # eRtg Alle Richtungen (Kirchheim unter Teck Kirchheim (T))
	HEGELSTRASSE = 'de:08116:4338'
	HEGELSTRASSE_1 = 'de:08116:4338:0:3'  # Rtg Fabrikstr. (Kirchheim unter Teck Kirchheim (T))
	HEGELSTRASSE_2 = 'de:08116:4338:0:4'  # Rtg Krankenhaus (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__HEGELSTRASSE = 'de:08116:4338'
	KIRCHHEIM__T__HEGELSTRASSE_1 = 'de:08116:4338:0:3'  # Rtg Fabrikstr. (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__HEGELSTRASSE_2 = 'de:08116:4338:0:4'  # Rtg Krankenhaus (Kirchheim unter Teck Kirchheim (T))
	ÖTLINGEN_FABRIKSTRASSE = 'de:08116:4339'
	ÖTLINGEN_FABRIKSTRASSE_1 = 'de:08116:4339:0:3'  # eRtg Haldenstraße (Kirchheim unter Teck Ötlingen)
	ÖTLINGEN_FABRIKSTRASSE_2 = 'de:08116:4339:0:4'  # eRtg Hegelstraße (Kirchheim unter Teck Ötlingen)
	ÖTLINGEN_KIRCHE = 'de:08116:4340:0:3'  # eRtg Mörikeschule (Kirchheim unter Teck Ötlingen)
	ÖTLINGEN_ORTSMITTE = 'de:08116:4341:0:3'  # eRtg Ötlingen (Kirchheim unter Teck Ötlingen)
	HALDE_1 = 'de:08116:4343:0:3'  # eRtg Karpatenweg (Kirchheim unter Teck Ötlingen)
	HALDE_2 = 'de:08116:4481:0:3'  # eRtg Wasserwerk (Nürtingen Neckarhausen)
	HALDE_3 = 'de:08116:4481:0:4'  # eRtg Rathaus (Nürtingen Neckarhausen)
	ÖTLINGEN_HALDE = 'de:08116:4343:0:3'  # eRtg Karpatenweg (Kirchheim unter Teck Ötlingen)
	WARTH = 'de:08116:4344:0:3'  # eRtg Ötlingen (Kirchheim unter Teck Ötlingen)
	ÖTLINGEN_WARTH = 'de:08116:4344:0:3'  # eRtg Ötlingen (Kirchheim unter Teck Ötlingen)
	ÖTLINGEN = 'de:08116:4345'
	ÖTLINGEN_1 = 'de:08116:4345:1:1'  # Gleis 1 (Kirchheim unter Teck Ötlingen)
	ÖTLINGEN_2 = 'de:08116:4345:3:3'  # eRtg Am Wasen (Kirchheim unter Teck Ötlingen)
	MÖRIKESCHULE = 'de:08116:4346'
	MÖRIKESCHULE_1 = 'de:08116:4346:0:3'  # Rtg Ötlingen (Kirchheim unter Teck Ötlingen)
	MÖRIKESCHULE_2 = 'de:08116:4346:0:4'  # Rtg Lindorf (Kirchheim unter Teck Ötlingen)
	ÖTLINGEN_MÖRIKESCHULE = 'de:08116:4346'
	ÖTLINGEN_MÖRIKESCHULE_1 = 'de:08116:4346:0:3'  # Rtg Ötlingen (Kirchheim unter Teck Ötlingen)
	ÖTLINGEN_MÖRIKESCHULE_2 = 'de:08116:4346:0:4'  # Rtg Lindorf (Kirchheim unter Teck Ötlingen)
	LINDORF_BRÜHL = 'de:08116:4347'
	LINDORF_BRÜHL_1 = 'de:08116:4347:0:3'  # eRtg Kreissparkasse (Kirchheim unter Teck Lindorf)
	LINDORF_BRÜHL_2 = 'de:08116:4347:0:4'  # eRtg Mörikeschule (Kirchheim unter Teck Lindorf)
	LINDORF_ORTSMITTE = 'de:08116:4349'
	LINDORF_ORTSMITTE_1 = 'de:08116:4349:1:3'  # eRtg Zähringer Straße (Kirchheim unter Teck Lindorf)
	LINDORF_ORTSMITTE_2 = 'de:08116:4349:1:4'  # eRtg Brühl (Kirchheim unter Teck Lindorf)
	KIRCHHEIM__T__SCHLOSSGYMNASIUM = 'de:08116:4350'
	KIRCHHEIM__T__SCHLOSSGYMNASIUM_1 = 'de:08116:4350:0:3'  # eRtg Habichtweg (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__SCHLOSSGYMNASIUM_2 = 'de:08116:4350:0:4'  # eRtg Freibad/K.-W.-Halle (Kirchheim unter Teck Kirchheim (T))
	SCHLOSSGYMNASIUM = 'de:08116:4350'
	SCHLOSSGYMNASIUM_1 = 'de:08116:4350:0:3'  # eRtg Habichtweg (Kirchheim unter Teck Kirchheim (T))
	SCHLOSSGYMNASIUM_2 = 'de:08116:4350:0:4'  # eRtg Freibad/K.-W.-Halle (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__HABICHTWEG = 'de:08116:4351'
	KIRCHHEIM__T__HABICHTWEG_1 = 'de:08116:4351:0:3'  # eRtg Milanweg (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__HABICHTWEG_2 = 'de:08116:4351:0:4'  # Endhaltestelle Linie 164 (Kirchheim unter Teck Kirchheim (T))
	HABICHTWEG = 'de:08116:4351'
	HABICHTWEG_1 = 'de:08116:4351:0:3'  # eRtg Milanweg (Kirchheim unter Teck Kirchheim (T))
	HABICHTWEG_2 = 'de:08116:4351:0:4'  # Endhaltestelle Linie 164 (Kirchheim unter Teck Kirchheim (T))
	MILANWEG = 'de:08116:4352'
	MILANWEG_1 = 'de:08116:4352:0:3'  # eRtg Benzenhofweg (Kirchheim unter Teck Kirchheim (T))
	MILANWEG_2 = 'de:08116:4352:0:4'  # eRtg Habichtweg (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__MILANWEG = 'de:08116:4352'
	KIRCHHEIM__T__MILANWEG_1 = 'de:08116:4352:0:3'  # eRtg Benzenhofweg (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__MILANWEG_2 = 'de:08116:4352:0:4'  # eRtg Habichtweg (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BENZENHOFWEG = 'de:08116:4353'
	KIRCHHEIM__T__BENZENHOFWEG_1 = 'de:08116:4353:0:3'  # eRtg Am Wiesenrain (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BENZENHOFWEG_2 = 'de:08116:4353:0:4'  # eRtg Milanweg (Kirchheim unter Teck Kirchheim (T))
	BENZENHOFWEG = 'de:08116:4353'
	BENZENHOFWEG_1 = 'de:08116:4353:0:3'  # eRtg Am Wiesenrain (Kirchheim unter Teck Kirchheim (T))
	BENZENHOFWEG_2 = 'de:08116:4353:0:4'  # eRtg Milanweg (Kirchheim unter Teck Kirchheim (T))
	SCHAFHOF_1 = 'de:08116:4354:0:4'  # eRtg Schlossgymnasium/Jesingen (Kirchheim unter Teck Kirchheim (T))
	SCHAFHOF_2 = 'de:08119:5064:0:3'  # eRtg Aichstrut (Welzheim Welzheim)
	SCHAFHOF_3 = 'de:08119:5064:0:4'  # eRtg Steinbeise (Welzheim Welzheim)
	KIRCHHEIM__T__SCHAFHOF = 'de:08116:4354:0:4'  # eRtg Schlossgymnasium/Jesingen (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__WALDFRIEDHOF = 'de:08116:4355:0:3'  # eRtg Wellinger Straße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__UHLANDGYMNASIUM = 'de:08116:4356:0:3'  # eRtg Kirchheim (T) (Kirchheim unter Teck Kirchheim (T))
	UHLANDGYMNASIUM = 'de:08116:4356:0:3'  # eRtg Kirchheim (T) (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__HOLBEINWEG = 'de:08116:4357:0:3'  # eRtg Rembrandtweg (Kirchheim unter Teck Kirchheim (T))
	HOLBEINWEG = 'de:08116:4357:0:3'  # eRtg Rembrandtweg (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__REMBRANDTWEG = 'de:08116:4358:0:3'  # eRtg Marktplatz (Kirchheim unter Teck Kirchheim (T))
	REMBRANDTWEG = 'de:08116:4358:0:3'  # eRtg Marktplatz (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BADWIESEN = 'de:08116:4361'
	KIRCHHEIM__T__BADWIESEN_1 = 'de:08116:4361:0:3'  # eRtg Kirchheim (T) (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BADWIESEN_2 = 'de:08116:4361:0:4'  # eRtg Lindorfer Weg (Kirchheim unter Teck Kirchheim (T))
	BADWIESEN = 'de:08116:4361'
	BADWIESEN_1 = 'de:08116:4361:0:3'  # eRtg Kirchheim (T) (Kirchheim unter Teck Kirchheim (T))
	BADWIESEN_2 = 'de:08116:4361:0:4'  # eRtg Lindorfer Weg (Kirchheim unter Teck Kirchheim (T))
	WEILHEIM__T__LERCHENSTRASSE = 'de:08116:4362:0:3'  # eRtg Schluchtweg (Weilheim an der Teck Weilheim (T))
	ZIEGELWASEN = 'de:08116:4363'
	ZIEGELWASEN_1 = 'de:08116:4363:0:3'  # Rtg Notzingen/Schlierbach (Kirchheim unter Teck Kirchheim (T))
	ZIEGELWASEN_2 = 'de:08116:4363:0:4'  # Rtg Kirchheim (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__ZIEGELWASEN = 'de:08116:4363'
	KIRCHHEIM__T__ZIEGELWASEN_1 = 'de:08116:4363:0:3'  # Rtg Notzingen/Schlierbach (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__ZIEGELWASEN_2 = 'de:08116:4363:0:4'  # Rtg Kirchheim (Kirchheim unter Teck Kirchheim (T))
	HUGO_WOLF_WEG = 'de:08116:4364'
	HUGO_WOLF_WEG_1 = 'de:08116:4364:0:3'  # eRtg Brahmsstraße (Kirchheim unter Teck Kirchheim (T))
	HUGO_WOLF_WEG_2 = 'de:08116:4364:0:4'  # eRtg Ziegelwasen (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__HUGO_WOLF_WEG = 'de:08116:4364'
	KIRCHHEIM__T__HUGO_WOLF_WEG_1 = 'de:08116:4364:0:3'  # eRtg Brahmsstraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__HUGO_WOLF_WEG_2 = 'de:08116:4364:0:4'  # eRtg Ziegelwasen (Kirchheim unter Teck Kirchheim (T))
	NOTZINGEN_WELLINGEN_KIRCHLE = 'de:08116:4365'
	NOTZINGEN_WELLINGEN_KIRCHLE_1 = 'de:08116:4365:0:3'  # Rtg Schlierbacher Str. (Notzingen Notzingen)
	NOTZINGEN_WELLINGEN_KIRCHLE_2 = 'de:08116:4365:0:4'  # Rtg Rathaus (Notzingen Notzingen)
	WELLINGEN_KIRCHLE = 'de:08116:4365'
	WELLINGEN_KIRCHLE_1 = 'de:08116:4365:0:3'  # Rtg Schlierbacher Str. (Notzingen Notzingen)
	WELLINGEN_KIRCHLE_2 = 'de:08116:4365:0:4'  # Rtg Rathaus (Notzingen Notzingen)
	NOTZ__WELL__SCHLIERBACHER_STR = 'de:08116:4366:0:3'  # eRtg Rathaus/Wellingen Kirchle (Notzingen Notzingen)
	WELLINGEN_SCHLIERBACHER_STR = 'de:08116:4366:0:3'  # eRtg Rathaus/Wellingen Kirchle (Notzingen Notzingen)
	ÖTLINGER_STRASSE = 'de:08116:4368:0:3'  # eRtg Wellinger Straße (Notzingen Notzingen)
	NOTZINGEN_ÖTLINGER_STRASSE = 'de:08116:4368:0:3'  # eRtg Wellinger Straße (Notzingen Notzingen)
	WERNAU__N__FREITAGSHOF = 'de:08116:4369'
	WERNAU__N__FREITAGSHOF_1 = 'de:08116:4369:0:3'  # eRtg Brühl (Wernau (Neckar) Wernau (N))
	WERNAU__N__FREITAGSHOF_2 = 'de:08116:4369:0:4'  # eRtg Ötlinger Straße (Wernau (Neckar) Wernau (N))
	FREITAGSHOF = 'de:08116:4369'
	FREITAGSHOF_1 = 'de:08116:4369:0:3'  # eRtg Brühl (Wernau (Neckar) Wernau (N))
	FREITAGSHOF_2 = 'de:08116:4369:0:4'  # eRtg Ötlinger Straße (Wernau (Neckar) Wernau (N))
	KIRCHHEIM__T__HANS_THOMA_WEG = 'de:08116:4371:0:3'  # eRtg Uhlandgymnasium (Kirchheim unter Teck Kirchheim (T))
	HANS_THOMA_WEG = 'de:08116:4371:0:3'  # eRtg Uhlandgymnasium (Kirchheim unter Teck Kirchheim (T))
	NÜRTINGEN_AMTSGERICHT = 'de:08116:4373'
	NÜRTINGEN_AMTSGERICHT_1 = 'de:08116:4373:0:3'  # eRtg Schillerplatz (Nürtingen Nürtingen)
	NÜRTINGEN_AMTSGERICHT_2 = 'de:08116:4373:0:4'  # eRtg Albstraße (Siedlerstube) (Nürtingen Nürtingen)
	AMTSGERICHT = 'de:08116:4373'
	AMTSGERICHT_1 = 'de:08116:4373:0:3'  # eRtg Schillerplatz (Nürtingen Nürtingen)
	AMTSGERICHT_2 = 'de:08116:4373:0:4'  # eRtg Albstraße (Siedlerstube) (Nürtingen Nürtingen)
	BAHNHOF__PLOCHINGER_STRASSE = 'de:08116:4375'
	BAHNHOF__PLOCHINGER_STRASSE_1 = 'de:08116:4375:1:3'  # Rtg Säerstr. (Nürtingen Nürtingen)
	BAHNHOF__PLOCHINGER_STRASSE_2 = 'de:08116:4375:1:4'  # Rtg Vendelaustr. (Nürtingen Nürtingen)
	NÜRTINGEN_BAHNHOF__PLOCHINGER_STRASSE = 'de:08116:4375'
	NÜRTINGEN_BAHNHOF__PLOCHINGER_STRASSE_1 = 'de:08116:4375:1:3'  # Rtg Säerstr. (Nürtingen Nürtingen)
	NÜRTINGEN_BAHNHOF__PLOCHINGER_STRASSE_2 = 'de:08116:4375:1:4'  # Rtg Vendelaustr. (Nürtingen Nürtingen)
	NÜRTINGEN_KIRCHHEIMER_STRASSE = 'de:08116:4376:0:3'  # eRtg Plochinger Straße (Nürtingen Nürtingen)
	OSTSIEDLUNG = 'de:08116:4378:0:3'  # eRtg Rümelinstraße (Nürtingen Nürtingen)
	NÜRTINGEN_OSTSIEDLUNG = 'de:08116:4378:0:3'  # eRtg Rümelinstraße (Nürtingen Nürtingen)
	WEST = 'de:08116:4379'
	WEST_1 = 'de:08116:4379:0:3'  # eRtg Ostsiedlung (Nürtingen Reudern)
	WEST_2 = 'de:08116:4379:0:4'  # eRtg Mitte (Nürtingen Reudern)
	REUDERN_WEST = 'de:08116:4379'
	REUDERN_WEST_1 = 'de:08116:4379:0:3'  # eRtg Ostsiedlung (Nürtingen Reudern)
	REUDERN_WEST_2 = 'de:08116:4379:0:4'  # eRtg Mitte (Nürtingen Reudern)
	REUDERN_MITTE = 'de:08116:4380'
	REUDERN_MITTE_1 = 'de:08116:4380:0:3'  # eRtg West (Nürtingen Reudern)
	REUDERN_MITTE_2 = 'de:08116:4380:0:4'  # eRtg Ost (Nürtingen Reudern)
	REUDERN_OST = 'de:08116:4381'
	REUDERN_OST_1 = 'de:08116:4381:0:3'  # Rtg Kirchheim (Nürtingen Reudern)
	REUDERN_OST_2 = 'de:08116:4381:0:4'  # Rtg Nürtingen (Nürtingen Reudern)
	OST = 'de:08116:4381'
	OST_1 = 'de:08116:4381:0:3'  # Rtg Kirchheim (Nürtingen Reudern)
	OST_2 = 'de:08116:4381:0:4'  # Rtg Nürtingen (Nürtingen Reudern)
	BÜRGERSEE = 'de:08116:4382'
	BÜRGERSEE_1 = 'de:08116:4382:0:3'  # eRtg Ost (Kirchheim unter Teck Kirchheim (T))
	BÜRGERSEE_2 = 'de:08116:4382:0:4'  # eRtg Nägelestal (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BÜRGERSEE = 'de:08116:4382'
	KIRCHHEIM__T__BÜRGERSEE_1 = 'de:08116:4382:0:3'  # eRtg Ost (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BÜRGERSEE_2 = 'de:08116:4382:0:4'  # eRtg Nägelestal (Kirchheim unter Teck Kirchheim (T))
	NÄGELESTAL = 'de:08116:4383'
	NÄGELESTAL_1 = 'de:08116:4383:0:3'  # eRtg Bürgersee (Kirchheim unter Teck Kirchheim (T))
	NÄGELESTAL_2 = 'de:08116:4383:0:4'  # eRtg Steingaustraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__NÄGELESTAL = 'de:08116:4383'
	KIRCHHEIM__T__NÄGELESTAL_1 = 'de:08116:4383:0:3'  # eRtg Bürgersee (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__NÄGELESTAL_2 = 'de:08116:4383:0:4'  # eRtg Steingaustraße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__STEINGAUSTRASSE = 'de:08116:4384:0:3'  # eRtg Stadthalle (Kirchheim unter Teck Kirchheim (T))
	STEINGAUSTRASSE = 'de:08116:4384:0:3'  # eRtg Stadthalle (Kirchheim unter Teck Kirchheim (T))
	HENRIETTENSTRASSE = 'de:08116:4385:0:3'  # eRtg Nägelestal (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__HENRIETTENSTRASSE = 'de:08116:4385:0:3'  # eRtg Nägelestal (Kirchheim unter Teck Kirchheim (T))
	ÖTLINGEN_AM_WASEN = 'de:08116:4386:0:3'  # eRtg Kirche/Apotheke (Kirchheim unter Teck Ötlingen)
	AM_WASEN = 'de:08116:4386:0:3'  # eRtg Kirche/Apotheke (Kirchheim unter Teck Ötlingen)
	JESINGEN_FAUSLERSTRASSE = 'de:08116:4387'
	JESINGEN_FAUSLERSTRASSE_1 = 'de:08116:4387:0:3'  # eRtg Wartehalle (Kirchheim unter Teck Jesingen)
	JESINGEN_FAUSLERSTRASSE_2 = 'de:08116:4387:0:4'  # eRtg Einsteinstraße (Kirchheim unter Teck Jesingen)
	FAUSLERSTRASSE = 'de:08116:4387'
	FAUSLERSTRASSE_1 = 'de:08116:4387:0:3'  # eRtg Wartehalle (Kirchheim unter Teck Jesingen)
	FAUSLERSTRASSE_2 = 'de:08116:4387:0:4'  # eRtg Einsteinstraße (Kirchheim unter Teck Jesingen)
	SIEGENBERGPLATZ = 'de:08116:4388:0:3'  # eRtg Neuffenstraße (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F__SIEGENBERGPL = 'de:08116:4388:0:3'  # eRtg Neuffenstraße (Reichenbach an der Fils Reichenbach (F))
	OHMDEN_FRIEDHOF = 'de:08116:4390'
	OHMDEN_FRIEDHOF_1 = 'de:08116:4390:0:3'  # eRtg Ohmden Rathaus (Ohmden Ohmden)
	OHMDEN_FRIEDHOF_2 = 'de:08116:4390:0:4'  # eRtg Jesingen Rathaus (Ohmden Ohmden)
	OHMDEN_RATHAUS = 'de:08116:4391'
	OHMDEN_RATHAUS_1 = 'de:08116:4391:0:3'  # eRtg Zeller Straße (Ohmden Ohmden)
	OHMDEN_RATHAUS_2 = 'de:08116:4391:0:4'  # eRtg Friedhof (Ohmden Ohmden)
	OHMDEN_SIEDLUNG = 'de:08116:4392:0:4'  # eRtg Zeller Straße (Ohmden Ohmden)
	HOLZMADEN_PLATZ = 'de:08116:4393:0:3'  # eRtg Weilheimer-/Brunnenstraße (Holzmaden Holzmaden)
	PLATZ = 'de:08116:4393:0:3'  # eRtg Weilheimer-/Brunnenstraße (Holzmaden Holzmaden)
	WEILHEIM__T__JAHNSTRASSE = 'de:08116:4394'
	WEILHEIM__T__JAHNSTRASSE_1 = 'de:08116:4394:0:3'  # Rtg Maierhöfe (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__JAHNSTRASSE_2 = 'de:08116:4394:0:4'  # Rtg Schluchtweg (Weilheim an der Teck Weilheim (T))
	WEILH__T__KIRCHH_STR__HEGELSTR = 'de:08116:4395'
	WEILH__T__KIRCHH_STR__HEGELSTR_1 = 'de:08116:4395:0:3'  # eRtg Lange Morgen (Weilheim an der Teck Weilheim (T))
	WEILH__T__KIRCHH_STR__HEGELSTR_2 = 'de:08116:4395:0:4'  # eRtg Kreissparkasse/Brunnenstraße (Weilheim an der Teck Weilheim (T))
	KIRCHHEIMER_STR__HEGELSTRASSE = 'de:08116:4395'
	KIRCHHEIMER_STR__HEGELSTRASSE_1 = 'de:08116:4395:0:3'  # eRtg Lange Morgen (Weilheim an der Teck Weilheim (T))
	KIRCHHEIMER_STR__HEGELSTRASSE_2 = 'de:08116:4395:0:4'  # eRtg Kreissparkasse/Brunnenstraße (Weilheim an der Teck Weilheim (T))
	LANGE_MORGEN = 'de:08116:4396'
	LANGE_MORGEN_1 = 'de:08116:4396:0:3'  # eRtg Egelsberg Egelsbergstraße (Weilheim an der Teck Weilheim (T))
	LANGE_MORGEN_2 = 'de:08116:4396:0:4'  # eRtg Kirchheimer Str./Hegelstraße (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__LANGE_MORGEN = 'de:08116:4396'
	WEILHEIM__T__LANGE_MORGEN_1 = 'de:08116:4396:0:3'  # eRtg Egelsberg Egelsbergstraße (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__LANGE_MORGEN_2 = 'de:08116:4396:0:4'  # eRtg Kirchheimer Str./Hegelstraße (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__NEIDLINGER_STRASSE = 'de:08116:4398'
	WEILHEIM__T__NEIDLINGER_STRASSE_1 = 'de:08116:4398:0:3'  # eRtg Abzweigung Hepsisau (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__NEIDLINGER_STRASSE_2 = 'de:08116:4398:0:4'  # eRtg Kreissparkasse/Brunnenstraße (Weilheim an der Teck Weilheim (T))
	NEIDLINGER_STRASSE = 'de:08116:4398'
	NEIDLINGER_STRASSE_1 = 'de:08116:4398:0:3'  # eRtg Abzweigung Hepsisau (Weilheim an der Teck Weilheim (T))
	NEIDLINGER_STRASSE_2 = 'de:08116:4398:0:4'  # eRtg Kreissparkasse/Brunnenstraße (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__ABZW__HEPSISAU = 'de:08116:4399'
	WEILHEIM__T__ABZW__HEPSISAU_1 = 'de:08116:4399:0:3'  # eRtg Fabrikstraße (Weilheim an der Teck Weilheim (T))
	WEILHEIM__T__ABZW__HEPSISAU_2 = 'de:08116:4399:0:4'  # eRtg Neidlinger Straße (Weilheim an der Teck Weilheim (T))
	ABZWEIGUNG_HEPSISAU = 'de:08116:4399'
	ABZWEIGUNG_HEPSISAU_1 = 'de:08116:4399:0:3'  # eRtg Fabrikstraße (Weilheim an der Teck Weilheim (T))
	ABZWEIGUNG_HEPSISAU_2 = 'de:08116:4399:0:4'  # eRtg Neidlinger Straße (Weilheim an der Teck Weilheim (T))
	ZAZENHAUSEN_KRONE = 'de:08111:4400'
	ZAZENHAUSEN_KRONE_1 = 'de:08111:4400:0:3'  # Rtg Zuffenhausen (Stuttgart Stuttgart)
	ZAZENHAUSEN_KRONE_2 = 'de:08111:4400:0:4'  # Rtg Mühlhausen (Stuttgart Stuttgart)
	ZUFFENHAUSEN_REIBEDANZ = 'de:08111:4401'
	ZUFFENHAUSEN_REIBEDANZ_1 = 'de:08111:4401:0:3'  # Rtg Zazenhausen (Stuttgart Stuttgart)
	ZUFFENHAUSEN_REIBEDANZ_2 = 'de:08111:4401:0:4'  # Rtg Zuffenhausen (Stuttgart Stuttgart)
	WERNER_UND_PFLEIDERER = 'de:08111:4402'
	WERNER_UND_PFLEIDERER_1 = 'de:08111:4402:0:3'  # eRtg Ohmstraße (Zuffenhausen Bf) (Stuttgart Stuttgart)
	WERNER_UND_PFLEIDERER_2 = 'de:08111:4402:0:4'  # eRtg Feuerbach Bosch (Stuttgart Stuttgart)
	FRICKENHAUSEN_ZIEGELEISTRASSE = 'de:08116:4403:1:4'  # Rtg Kelter (Frickenhausen Frickenhausen)
	ZIEGELEISTRASSE_1 = 'de:08116:4403:1:4'  # Rtg Kelter (Frickenhausen Frickenhausen)
	ZIEGELEISTRASSE_2 = 'de:08118:4522:0:3'  # Rtg Bahnhof/Stiegelplatz (Korntal-Münchingen Münchingen)
	ZIEGELEISTRASSE_3 = 'de:08118:4522:0:4'  # Rtg Danziger Platz (Korntal-Münchingen Münchingen)
	AUF_DEM_LERCHENBERG = 'de:08116:4404'
	AUF_DEM_LERCHENBERG_1 = 'de:08116:4404:0:3'  # Bucht (Nürtingen Nürtingen)
	AUF_DEM_LERCHENBERG_2 = 'de:08116:4404:0:4'  # Nord (Nürtingen Nürtingen)
	NÜRTINGEN_AUF_DEM_LERCHENBERG = 'de:08116:4404'
	NÜRTINGEN_AUF_DEM_LERCHENBERG_1 = 'de:08116:4404:0:3'  # Bucht (Nürtingen Nürtingen)
	NÜRTINGEN_AUF_DEM_LERCHENBERG_2 = 'de:08116:4404:0:4'  # Nord (Nürtingen Nürtingen)
	HEPSISAU_RATHAUS = 'de:08116:4405:0:3'  # eRtg Alle Richtungen (Weilheim an der Teck Hepsisau)
	NEIDLINGEN_FABRIKSTRASSE = 'de:08116:4406'
	NEIDLINGEN_FABRIKSTRASSE_1 = 'de:08116:4406:0:3'  # eRtg Mitte (Neidlingen Neidlingen)
	NEIDLINGEN_FABRIKSTRASSE_2 = 'de:08116:4406:0:4'  # eRtg Weilheim (Neidlingen Neidlingen)
	NEIDLINGEN_MITTE = 'de:08116:4407'
	NEIDLINGEN_MITTE_1 = 'de:08116:4407:0:3'  # eRtg Schlossgärten (Neidlingen Neidlingen)
	NEIDLINGEN_MITTE_2 = 'de:08116:4407:0:4'  # eRtg Fabrikstraße (Neidlingen Neidlingen)
	NEIDLINGEN_LEERWASEN = 'de:08116:4408:0:3'  # eRtg Mitte (Neidlingen Neidlingen)
	LEERWASEN = 'de:08116:4408:0:3'  # eRtg Mitte (Neidlingen Neidlingen)
	SCHLOSSGÄRTEN = 'de:08116:4409:0:4'  # eRtg Leerwasen (Neidlingen Neidlingen)
	NEIDLINGEN_SCHLOSSGÄRTEN = 'de:08116:4409:0:4'  # eRtg Leerwasen (Neidlingen Neidlingen)
	NABERN_INDUSTRIEGEBIET = 'de:08116:4410'
	NABERN_INDUSTRIEGEBIET_1 = 'de:08116:4410:0:3'  # eRtg Gartenstraße (Kirchheim unter Teck Nabern)
	NABERN_INDUSTRIEGEBIET_2 = 'de:08116:4410:0:4'  # eRtg Katholische Kirche (Kirchheim unter Teck Nabern)
	NABERN_GARTENSTRASSE = 'de:08116:4411'
	NABERN_GARTENSTRASSE_1 = 'de:08116:4411:0:3'  # Rtg Bissingen (Kirchheim unter Teck Nabern)
	NABERN_GARTENSTRASSE_2 = 'de:08116:4411:0:4'  # Rtg Dettingen (Kirchheim unter Teck Nabern)
	BISSINGEN__T__FABRIKSTRASSE = 'de:08116:4412'
	BISSINGEN__T__FABRIKSTRASSE_1 = 'de:08116:4412:0:3'  # eRtg Wilhelmstraße (Bissingen an der Teck Bissingen an der Teck (T))
	BISSINGEN__T__FABRIKSTRASSE_2 = 'de:08116:4412:0:4'  # eRtg Gartenstraße (Bissingen an der Teck Bissingen an der Teck (T))
	BISSINGEN__T__WILHELMSTRASSE = 'de:08116:4413'
	BISSINGEN__T__WILHELMSTRASSE_1 = 'de:08116:4413:0:3'  # eRtg Grundschule (Bissingen an der Teck Bissingen an der Teck (T))
	BISSINGEN__T__WILHELMSTRASSE_2 = 'de:08116:4413:0:4'  # eRtg Fabrikstraße (Bissingen an der Teck Bissingen an der Teck (T))
	BISSINGEN__T__RATHAUS = 'de:08116:4415'
	BISSINGEN__T__RATHAUS_1 = 'de:08116:4415:0:3'  # Rtg See (Bissingen an der Teck Bissingen an der Teck (T))
	BISSINGEN__T__RATHAUS_2 = 'de:08116:4415:0:4'  # Rtg Wilhelmstraße Grundschule (Bissingen an der Teck Bissingen an der Teck (T))
	BISSINGEN__T__SEE = 'de:08116:4416:1:3'  # eRtg Rathaus (Bissingen an der Teck Bissingen an der Teck (T))
	OCHSENWANG_RATHAUS = 'de:08116:4417:0:3'  # eRtg Rathaus (Bissingen an der Teck Ochsenwang)
	OBERLENNINGEN = 'de:08116:4418'
	OBERLENNINGEN_1 = 'de:08116:4418:1:1'  # Gleis 1 (Lenningen Oberlenningen)
	OBERLENNINGEN_2 = 'de:08116:4418:2:1'  # Bstg 1 (Lenningen Oberlenningen)
	OBERLENNINGEN_3 = 'de:08116:4418:2:2'  # Bstg 2 (Lenningen Oberlenningen)
	SCHLATTSTALL_ABZW = 'de:08116:4419'
	SCHLATTSTALL_ABZW_1 = 'de:08116:4419:0:3'  # eRtg Grünenbergstraße (Lenningen Schlattstall)
	SCHLATTSTALL_ABZW_2 = 'de:08116:4419:0:4'  # eRtg Etterstraße (Lenningen Schlattstall)
	GUTENBERG_GRÜNENBERGSTRASSE = 'de:08116:4420'
	GUTENBERG_GRÜNENBERGSTRASSE_1 = 'de:08116:4420:0:3'  # eRtg Post (Lenningen Gutenberg)
	GUTENBERG_GRÜNENBERGSTRASSE_2 = 'de:08116:4420:0:4'  # eRtg Abzweig (Lenningen Gutenberg)
	GRÜNENBERGSTRASSE = 'de:08116:4420'
	GRÜNENBERGSTRASSE_1 = 'de:08116:4420:0:3'  # eRtg Post (Lenningen Gutenberg)
	GRÜNENBERGSTRASSE_2 = 'de:08116:4420:0:4'  # eRtg Abzweig (Lenningen Gutenberg)
	GUTENBERG_POST = 'de:08116:4421'
	GUTENBERG_POST_1 = 'de:08116:4421:0:3'  # eRtg Schlatterhöhe B465 (Lenningen Gutenberg)
	GUTENBERG_POST_2 = 'de:08116:4421:0:4'  # eRtg Grünenbergstraße (Lenningen Gutenberg)
	SCHLATTERHÖHE_B465 = 'de:08116:4422'
	SCHLATTERHÖHE_B465_1 = 'de:08116:4422:0:3'  # Rtg Schopfloch/Gutenberg (Lenningen Schopfloch (Lenningen))
	SCHLATTERHÖHE_B465_2 = 'de:08116:4422:0:4'  # Rtg Schopfloch/Donnstetten (Lenningen Schopfloch (Lenningen))
	SCHOPFLOCH_SCHLATTERHÖHE_B465 = 'de:08116:4422'
	SCHOPFLOCH_SCHLATTERHÖHE_B465_1 = 'de:08116:4422:0:3'  # Rtg Schopfloch/Gutenberg (Lenningen Schopfloch (Lenningen))
	SCHOPFLOCH_SCHLATTERHÖHE_B465_2 = 'de:08116:4422:0:4'  # Rtg Schopfloch/Donnstetten (Lenningen Schopfloch (Lenningen))
	SCHOPFLOCH_KREISLERSTRASSE = 'de:08116:4423'
	SCHOPFLOCH_KREISLERSTRASSE_1 = 'de:08116:4423:0:3'  # eRtg Rathaus (Lenningen Schopfloch (Lenningen))
	SCHOPFLOCH_KREISLERSTRASSE_2 = 'de:08116:4423:0:4'  # eRtg Schlatterhöhe L 1212 (Lenningen Schopfloch (Lenningen))
	KREISLERSTRASSE = 'de:08116:4423'
	KREISLERSTRASSE_1 = 'de:08116:4423:0:3'  # eRtg Rathaus (Lenningen Schopfloch (Lenningen))
	KREISLERSTRASSE_2 = 'de:08116:4423:0:4'  # eRtg Schlatterhöhe L 1212 (Lenningen Schopfloch (Lenningen))
	SCHOPFLOCH_RATHAUS = 'de:08116:4424'
	SCHOPFLOCH_RATHAUS_1 = 'de:08116:4424:0:3'  # Rtg Reußensteinstraße (Lenningen Schopfloch (Lenningen))
	SCHOPFLOCH_RATHAUS_2 = 'de:08116:4424:0:4'  # Rtg Kreislerstraße (Lenningen Schopfloch (Lenningen))
	SCHOPFLOCH_REUSSENSTEINSTRASSE = 'de:08116:4425:0:3'  # eRtg alle Richtungen (Lenningen Schopfloch (Lenningen))
	REUSSENSTEINSTRASSE_1 = 'de:08116:4425:0:3'  # eRtg alle Richtungen (Lenningen Schopfloch (Lenningen))
	REUSSENSTEINSTRASSE_2 = 'de:08115:4685:0:3'  # eRtg Schulzentrum Murkenbach (Böblingen Böblingen)
	HERMANN_LÖNS_STRASSE_1 = 'de:08116:4427:0:3'  # eRtg Schelmenwasen (Nürtingen Nürtingen)
	HERMANN_LÖNS_STRASSE_2 = 'de:08116:4427:0:4'  # eRtg Achalmstraße (Nürtingen Nürtingen)
	HERMANN_LÖNS_STRASSE_3 = 'de:08118:6895:0:3'  # eRtg Golfplatz (Gerlingen Gerlingen)
	NÜRTINGEN_HERMANN_LÖNS_STRASSE = 'de:08116:4427'
	NÜRTINGEN_HERMANN_LÖNS_STRASSE_1 = 'de:08116:4427:0:3'  # eRtg Schelmenwasen (Nürtingen Nürtingen)
	NÜRTINGEN_HERMANN_LÖNS_STRASSE_2 = 'de:08116:4427:0:4'  # eRtg Achalmstraße (Nürtingen Nürtingen)
	ERKENBRECHTSWEILER_UNTERE_STR = 'de:08116:4428'
	ERKENBRECHTSWEILER_UNTERE_STR_1 = 'de:08116:4428:0:3'  # eRtg Beurener Steige/Grabenstetten (Erkenbrechtsweiler Erkenbrechtsweiler)
	ERKENBRECHTSWEILER_UNTERE_STR_2 = 'de:08116:4428:0:4'  # eRtg Hochwang Rathaus (Erkenbrechtsweiler Erkenbrechtsweiler)
	UNTERE_STRASSE = 'de:08116:4428'
	UNTERE_STRASSE_1 = 'de:08116:4428:0:3'  # eRtg Beurener Steige/Grabenstetten (Erkenbrechtsweiler Erkenbrechtsweiler)
	UNTERE_STRASSE_2 = 'de:08116:4428:0:4'  # eRtg Hochwang Rathaus (Erkenbrechtsweiler Erkenbrechtsweiler)
	FRICKENHAUSEN_SCHULE = 'de:08116:4429'
	FRICKENHAUSEN_SCHULE_1 = 'de:08116:4429:0:3'  # eRtg Hölzlachstraße (Frickenhausen Frickenhausen)
	FRICKENHAUSEN_SCHULE_2 = 'de:08116:4429:0:4'  # eRtg Reutte (Frickenhausen Frickenhausen)
	FRICKENHAUSEN_KELTER = 'de:08116:4430'
	FRICKENHAUSEN_KELTER_1 = 'de:08116:4430:1:3'  # Rtg Uhlandstr. (Frickenhausen Frickenhausen)
	FRICKENHAUSEN_KELTER_2 = 'de:08116:4430:1:4'  # Rtg Sternbergstr. (Frickenhausen Frickenhausen)
	HUMBOLDTSTRASSE = 'de:08116:4431'
	HUMBOLDTSTRASSE_1 = 'de:08116:4431:0:3'  # Rtg Frickenhausen (Frickenhausen Linsenhofen)
	HUMBOLDTSTRASSE_2 = 'de:08116:4431:0:4'  # Rtg Neuffen (Frickenhausen Linsenhofen)
	LINSENHOFEN_HUMBOLDTSTRASSE = 'de:08116:4431'
	LINSENHOFEN_HUMBOLDTSTRASSE_1 = 'de:08116:4431:0:3'  # Rtg Frickenhausen (Frickenhausen Linsenhofen)
	LINSENHOFEN_HUMBOLDTSTRASSE_2 = 'de:08116:4431:0:4'  # Rtg Neuffen (Frickenhausen Linsenhofen)
	LINSENHOFEN = 'de:08116:4433'
	LINSENHOFEN_1 = 'de:08116:4433:1:1'  # R-Bahn (Frickenhausen Linsenhofen)
	LINSENHOFEN_2 = 'de:08116:4433:2:2'  # Rtg Beuren (Frickenhausen Linsenhofen)
	LINSENHOFER_STRASSE = 'de:08116:4434:0:1'  # Rtg Gartenstraße (Beuren Beuren)
	BEUREN_LINSENHOFER_STRASSE = 'de:08116:4434:0:1'  # Rtg Gartenstraße (Beuren Beuren)
	BEUREN_PANORAMA_THERME = 'de:08116:4436:1:3'  # Rtg Balzholzer Str./Balzholz Rathaus (Beuren Beuren)
	PANORAMA_THERME = 'de:08116:4436:1:3'  # Rtg Balzholzer Str./Balzholz Rathaus (Beuren Beuren)
	BEUREN_GARTENSTRASSE = 'de:08116:4437'
	BEUREN_GARTENSTRASSE_1 = 'de:08116:4437:1:3'  # Rtg Balzholzer Str. Neuffen (Beuren Beuren)
	BEUREN_GARTENSTRASSE_2 = 'de:08116:4437:1:4'  # Rtg Bürgerhaus Erkenbrechtsweiler (Beuren Beuren)
	BEUREN_BALZHOLZ_RATHAUS = 'de:08116:4438'
	BEUREN_BALZHOLZ_RATHAUS_1 = 'de:08116:4438:0:3'  # eRtg Max-Planck-Straße (Beuren Beuren)
	BEUREN_BALZHOLZ_RATHAUS_2 = 'de:08116:4438:0:4'  # eRtg Balzholzer Straße (Beuren Beuren)
	BALZHOLZ_RATHAUS = 'de:08116:4438'
	BALZHOLZ_RATHAUS_1 = 'de:08116:4438:0:3'  # eRtg Max-Planck-Straße (Beuren Beuren)
	BALZHOLZ_RATHAUS_2 = 'de:08116:4438:0:4'  # eRtg Balzholzer Straße (Beuren Beuren)
	NEUFFEN_LINDENPLATZ = 'de:08116:4442'
	NEUFFEN_LINDENPLATZ_1 = 'de:08116:4442:0:3'  # eRtg Schule/Kohlberg (Neuffen Neuffen)
	NEUFFEN_LINDENPLATZ_2 = 'de:08116:4442:0:4'  # eRtg Neuffen (Neuffen Neuffen)
	NEUFFEN_MAX_PLANCK_STRASSE = 'de:08116:4443'
	NEUFFEN_MAX_PLANCK_STRASSE_1 = 'de:08116:4443:0:3'  # eRtg Oberer Graben (Neuffen Neuffen)
	NEUFFEN_MAX_PLANCK_STRASSE_2 = 'de:08116:4443:0:4'  # eRtg Balzholz Rathaus (Neuffen Neuffen)
	NÜRTINGEN_SÄERSTRASSE = 'de:08116:4444'
	NÜRTINGEN_SÄERSTRASSE_1 = 'de:08116:4444:0:3'  # eRtg Krankenhaus (Nürtingen Nürtingen)
	NÜRTINGEN_SÄERSTRASSE_2 = 'de:08116:4444:0:4'  # eRtg Plochinger Straße (Nürtingen Nürtingen)
	SÄERSTRASSE = 'de:08116:4444'
	SÄERSTRASSE_1 = 'de:08116:4444:0:3'  # eRtg Krankenhaus (Nürtingen Nürtingen)
	SÄERSTRASSE_2 = 'de:08116:4444:0:4'  # eRtg Plochinger Straße (Nürtingen Nürtingen)
	NÜRTINGEN_MEDIUS_KLINIK = 'de:08116:4445'
	NÜRTINGEN_MEDIUS_KLINIK_1 = 'de:08116:4445:0:3'  # Ankunft (Nürtingen Nürtingen)
	NÜRTINGEN_MEDIUS_KLINIK_2 = 'de:08116:4445:0:4'  # eRtg Säerstraße (Nürtingen Nürtingen)
	NÜRTINGEN_EICHENDORFFSTRASSE = 'de:08116:4446'
	NÜRTINGEN_EICHENDORFFSTRASSE_1 = 'de:08116:4446:0:3'  # eRtg Albstraße (Siedlerstube) (Nürtingen Nürtingen)
	NÜRTINGEN_EICHENDORFFSTRASSE_2 = 'de:08116:4446:0:4'  # eRtg Ziegeleistraße (Nürtingen Nürtingen)
	ROTENBERGPLATZ = 'de:08116:4447'
	ROTENBERGPLATZ_1 = 'de:08116:4447:0:3'  # eRtg Achalmstraße (Nürtingen Nürtingen)
	ROTENBERGPLATZ_2 = 'de:08116:4447:0:4'  # eRtg Albstraße (Siedlerstube) (Nürtingen Nürtingen)
	NÜRTINGEN_ROTENBERGPLATZ = 'de:08116:4447'
	NÜRTINGEN_ROTENBERGPLATZ_1 = 'de:08116:4447:0:3'  # eRtg Achalmstraße (Nürtingen Nürtingen)
	NÜRTINGEN_ROTENBERGPLATZ_2 = 'de:08116:4447:0:4'  # eRtg Albstraße (Siedlerstube) (Nürtingen Nürtingen)
	NÜRTINGEN_ACHALMSTRASSE = 'de:08116:4448'
	NÜRTINGEN_ACHALMSTRASSE_1 = 'de:08116:4448:0:3'  # eRtg Hermann-Löns-Straße (Nürtingen Nürtingen)
	NÜRTINGEN_ACHALMSTRASSE_2 = 'de:08116:4448:0:4'  # eRtg Rotenbergplatz (Nürtingen Nürtingen)
	NÜRTINGEN_WALDFRIEDHOF = 'de:08116:4449'
	NÜRTINGEN_WALDFRIEDHOF_1 = 'de:08116:4449:0:3'  # eRtg Schelmenwasen (Nürtingen Nürtingen)
	NÜRTINGEN_WALDFRIEDHOF_2 = 'de:08116:4449:0:4'  # eRtg Hermann-Löns-Straße (Nürtingen Nürtingen)
	NÜRTINGEN_SCHELMENWASEN = 'de:08116:4450'
	NÜRTINGEN_SCHELMENWASEN_1 = 'de:08116:4450:0:3'  # eRtg Steinachbrücke (Nürtingen Nürtingen)
	NÜRTINGEN_SCHELMENWASEN_2 = 'de:08116:4450:0:4'  # eRtg Hermann-Löns-Straße (Nürtingen Nürtingen)
	STEINACHBRÜCKE = 'de:08116:4451'
	STEINACHBRÜCKE_1 = 'de:08116:4451:1:3'  # eRtg Roßdorf Berliner Straße (Nürtingen Nürtingen)
	STEINACHBRÜCKE_2 = 'de:08116:4451:1:4'  # eRtg Schelmenwasen (Nürtingen Nürtingen)
	NÜRTINGEN_STEINACHBRÜCKE = 'de:08116:4451'
	NÜRTINGEN_STEINACHBRÜCKE_1 = 'de:08116:4451:1:3'  # eRtg Roßdorf Berliner Straße (Nürtingen Nürtingen)
	NÜRTINGEN_STEINACHBRÜCKE_2 = 'de:08116:4451:1:4'  # eRtg Schelmenwasen (Nürtingen Nürtingen)
	ROSSDORF_BERLINER_STRASSE = 'de:08116:4452'
	ROSSDORF_BERLINER_STRASSE_1 = 'de:08116:4452:0:3'  # eRtg Humpfentalstraße / Steinachbrücke (Nürtingen Nürtingen)
	ROSSDORF_BERLINER_STRASSE_2 = 'de:08116:4452:0:4'  # eRtg Roßdorf Schule (Nürtingen Nürtingen)
	NÜRT__ROSSDORF_BERLINER_STR = 'de:08116:4452'
	NÜRT__ROSSDORF_BERLINER_STR_1 = 'de:08116:4452:0:3'  # eRtg Humpfentalstraße / Steinachbrücke (Nürtingen Nürtingen)
	NÜRT__ROSSDORF_BERLINER_STR_2 = 'de:08116:4452:0:4'  # eRtg Roßdorf Schule (Nürtingen Nürtingen)
	NÜRT__ROSSDORF_DÜRERPLATZ = 'de:08116:4453:0:3'  # eRtg Roßdorf Schule (Nürtingen Nürtingen)
	ROSSDORF_DÜRERPLATZ = 'de:08116:4453:0:3'  # eRtg Roßdorf Schule (Nürtingen Nürtingen)
	ZIZISHAUSEN_FRIEDHOF = 'de:08116:4454'
	ZIZISHAUSEN_FRIEDHOF_1 = 'de:08116:4454:0:3'  # Rtg Unterensinger Str. (Nürtingen Zizishausen)
	ZIZISHAUSEN_FRIEDHOF_2 = 'de:08116:4454:0:4'  # Rtg Oberensingen (Nürtingen Zizishausen)
	UNTERENSINGER_STRASSE = 'de:08116:4456'
	UNTERENSINGER_STRASSE_1 = 'de:08116:4456:0:3'  # eRtg Linde (Nürtingen Zizishausen)
	UNTERENSINGER_STRASSE_2 = 'de:08116:4456:0:4'  # eRtg Inselbad/Friedhof (Nürtingen Zizishausen)
	ZIZISHAUSEN_UNTERENSINGER_STR = 'de:08116:4456'
	ZIZISHAUSEN_UNTERENSINGER_STR_1 = 'de:08116:4456:0:3'  # eRtg Linde (Nürtingen Zizishausen)
	ZIZISHAUSEN_UNTERENSINGER_STR_2 = 'de:08116:4456:0:4'  # eRtg Inselbad/Friedhof (Nürtingen Zizishausen)
	DAMMSTRASSE = 'de:08116:4459'
	DAMMSTRASSE_1 = 'de:08116:4459:0:3'  # eRtg Schillerplatz (Nürtingen Nürtingen)
	DAMMSTRASSE_2 = 'de:08116:4459:0:4'  # eRtg Eibenweg (Nürtingen Nürtingen)
	NÜRTINGEN_DAMMSTRASSE = 'de:08116:4459'
	NÜRTINGEN_DAMMSTRASSE_1 = 'de:08116:4459:0:3'  # eRtg Schillerplatz (Nürtingen Nürtingen)
	NÜRTINGEN_DAMMSTRASSE_2 = 'de:08116:4459:0:4'  # eRtg Eibenweg (Nürtingen Nürtingen)
	ENZENHARDTPLATZ = 'de:08116:4460'
	ENZENHARDTPLATZ_1 = 'de:08116:4460:0:3'  # eRtg Auf dem Lerchenberg (Nürtingen Nürtingen)
	ENZENHARDTPLATZ_2 = 'de:08116:4460:0:4'  # eRtg Stettiner Straße (Nürtingen Nürtingen)
	NÜRTINGEN_ENZENHARDTPLATZ = 'de:08116:4460'
	NÜRTINGEN_ENZENHARDTPLATZ_1 = 'de:08116:4460:0:3'  # eRtg Auf dem Lerchenberg (Nürtingen Nürtingen)
	NÜRTINGEN_ENZENHARDTPLATZ_2 = 'de:08116:4460:0:4'  # eRtg Stettiner Straße (Nürtingen Nürtingen)
	OSTPREUSSENWEG = 'de:08116:4461'
	OSTPREUSSENWEG_1 = 'de:08116:4461:0:3'  # eRtg Stettiner Straße (Nürtingen Nürtingen)
	OSTPREUSSENWEG_2 = 'de:08116:4461:0:4'  # eRtg Talstraße (Nürtingen Nürtingen)
	NÜRTINGEN_OSTPREUSSENWEG = 'de:08116:4461'
	NÜRTINGEN_OSTPREUSSENWEG_1 = 'de:08116:4461:0:3'  # eRtg Stettiner Straße (Nürtingen Nürtingen)
	NÜRTINGEN_OSTPREUSSENWEG_2 = 'de:08116:4461:0:4'  # eRtg Talstraße (Nürtingen Nürtingen)
	RAIDWANGEN_TALSTRASSE = 'de:08116:4462'
	RAIDWANGEN_TALSTRASSE_1 = 'de:08116:4462:0:3'  # eRtg Ostpreußenweg (Nürtingen Raidwangen)
	RAIDWANGEN_TALSTRASSE_2 = 'de:08116:4462:0:4'  # eRtg Nürtinger Straße (Nürtingen Raidwangen)
	GROSSBETTLINGEN_NÜRTINGER_STR = 'de:08116:4463'
	GROSSBETTLINGEN_NÜRTINGER_STR_1 = 'de:08116:4463:0:3'  # eRtg Nürtingen/Heerweg (Großbettlingen Großbettlingen)
	GROSSBETTLINGEN_NÜRTINGER_STR_2 = 'de:08116:4463:0:4'  # eRtg Bettlinger Forum (Großbettlingen Großbettlingen)
	NÜRTINGER_STRASSE_1 = 'de:08116:4463:0:3'  # eRtg Nürtingen/Heerweg (Großbettlingen Großbettlingen)
	NÜRTINGER_STRASSE_2 = 'de:08116:4463:0:4'  # eRtg Bettlinger Forum (Großbettlingen Großbettlingen)
	NÜRTINGER_STRASSE_3 = 'de:08116:4471:0:3'  # Rtg Aich (Aichtal Grötzingen)
	NÜRTINGER_STRASSE_4 = 'de:08116:4471:0:4'  # Rtg Nürtingen (Aichtal Grötzingen)
	NÜRTINGER_STRASSE_5 = 'de:08116:4482:0:3'  # Rtg Wasserwerk (Neckartailfingen Neckartailfingen)
	NÜRTINGER_STRASSE_6 = 'de:08116:4482:0:4'  # Rtg Rathaus (Neckartailfingen Neckartailfingen)
	NÜRTINGER_STRASSE_7 = 'de:08115:4671:0:3'  # eRtg Murkenbachweg (Böblingen Böblingen)
	KLEINBETTL__GRAFENBERGER_STR = 'de:08116:4464'
	KLEINBETTL__GRAFENBERGER_STR_1 = 'de:08116:4464:0:3'  # Rtg Rathaus (Bempflingen Kleinbettlingen)
	KLEINBETTL__GRAFENBERGER_STR_2 = 'de:08116:4464:0:4'  # Rtg Bempflingen (Bempflingen Kleinbettlingen)
	GRAFENBERGER_STRASSE = 'de:08116:4464'
	GRAFENBERGER_STRASSE_1 = 'de:08116:4464:0:3'  # Rtg Rathaus (Bempflingen Kleinbettlingen)
	GRAFENBERGER_STRASSE_2 = 'de:08116:4464:0:4'  # Rtg Bempflingen (Bempflingen Kleinbettlingen)
	KLEINBETTLINGEN_RATHAUS = 'de:08116:4465:0:3'  # eRtg Grafenberger-Metzinger Straße (Bempflingen Kleinbettlingen)
	TISCHARDT_RATHAUS = 'de:08116:4466'
	TISCHARDT_RATHAUS_1 = 'de:08116:4466:0:3'  # eRtg Frickenhäuser Straße (Frickenhausen Tischardt)
	TISCHARDT_RATHAUS_2 = 'de:08116:4466:0:4'  # eRtg Tischardter Straße (Frickenhausen Tischardt)
	FRICKENHAUSEN_REUTTE = 'de:08116:4467'
	FRICKENHAUSEN_REUTTE_1 = 'de:08116:4467:0:3'  # eRtg Schule (Frickenhausen Frickenhausen)
	FRICKENHAUSEN_REUTTE_2 = 'de:08116:4467:0:4'  # eRtg Frickenhäuser Straße (Frickenhausen Frickenhausen)
	REUTTE = 'de:08116:4467'
	REUTTE_1 = 'de:08116:4467:0:3'  # eRtg Schule (Frickenhausen Frickenhausen)
	REUTTE_2 = 'de:08116:4467:0:4'  # eRtg Frickenhäuser Straße (Frickenhausen Frickenhausen)
	GRÖTZINGEN_NÜRTINGER_STRASSE = 'de:08116:4471'
	GRÖTZINGEN_NÜRTINGER_STRASSE_1 = 'de:08116:4471:0:3'  # Rtg Aich (Aichtal Grötzingen)
	GRÖTZINGEN_NÜRTINGER_STRASSE_2 = 'de:08116:4471:0:4'  # Rtg Nürtingen (Aichtal Grötzingen)
	GRÖTZINGEN_KRIEGERDENKMAL = 'de:08116:4472:1:3'  # eRtg Raiffeisenstraße (Aichtal Grötzingen)
	GRÖTZINGEN_SCHÖNBLICK = 'de:08116:4473'
	GRÖTZINGEN_SCHÖNBLICK_1 = 'de:08116:4473:0:3'  # 809 Rtg Froschegert/Harthausen (Aichtal Grötzingen)
	GRÖTZINGEN_SCHÖNBLICK_2 = 'de:08116:4473:0:4'  # 809 Rtg Ortsmitte/Kriegerdenkmal (Aichtal Grötzingen)
	WENDLINGEN__N__NEUFFENSTRASSE = 'de:08116:4474:1:3'  # eRtg Kapellenstraße (Wendlingen am Neckar Wendlingen (N))
	NEUFFENSTRASSE_1 = 'de:08116:4474:1:3'  # eRtg Kapellenstraße (Wendlingen am Neckar Wendlingen (N))
	NEUFFENSTRASSE_2 = 'de:08116:6504:0:3'  # eRtg Hochhaus (Reichenbach an der Fils Reichenbach (F))
	BOMBACH = 'de:08116:4475'
	BOMBACH_1 = 'de:08116:4475:0:3'  # Rtg Neuenhaus (Aichtal Aich)
	BOMBACH_2 = 'de:08116:4475:0:4'  # Rtg Aich (Aichtal Aich)
	AICH_BOMBACH = 'de:08116:4475'
	AICH_BOMBACH_1 = 'de:08116:4475:0:3'  # Rtg Neuenhaus (Aichtal Aich)
	AICH_BOMBACH_2 = 'de:08116:4475:0:4'  # Rtg Aich (Aichtal Aich)
	GRÖRACH = 'de:08116:4476'
	GRÖRACH_1 = 'de:08116:4476:0:3'  # Rtg Neuenhaus (Aichtal Neuenhaus)
	GRÖRACH_2 = 'de:08116:4476:0:4'  # Rtg Aich (Aichtal Neuenhaus)
	NEUENHAUS_GRÖRACH = 'de:08116:4476'
	NEUENHAUS_GRÖRACH_1 = 'de:08116:4476:0:3'  # Rtg Neuenhaus (Aichtal Neuenhaus)
	NEUENHAUS_GRÖRACH_2 = 'de:08116:4476:0:4'  # Rtg Aich (Aichtal Neuenhaus)
	NEUENHAUS_HÄFNERSTRASSE = 'de:08116:4477'
	NEUENHAUS_HÄFNERSTRASSE_1 = 'de:08116:4477:0:3'  # eRtg Kirche (Aichtal Neuenhaus)
	NEUENHAUS_HÄFNERSTRASSE_2 = 'de:08116:4477:0:4'  # eRtg Grörach/Im Vogelsang (Aichtal Neuenhaus)
	HÄFNERSTRASSE = 'de:08116:4477'
	HÄFNERSTRASSE_1 = 'de:08116:4477:0:3'  # eRtg Kirche (Aichtal Neuenhaus)
	HÄFNERSTRASSE_2 = 'de:08116:4477:0:4'  # eRtg Grörach/Im Vogelsang (Aichtal Neuenhaus)
	GEISSHALDE = 'de:08115:4478'
	GEISSHALDE_1 = 'de:08115:4478:0:1'  # Rtg Backhaus (Aidlingen Deufringen)
	GEISSHALDE_2 = 'de:08115:4478:0:2'  # Rtg Rathaus (Aidlingen Deufringen)
	DEUFRINGEN_GEISSHALDE = 'de:08115:4478'
	DEUFRINGEN_GEISSHALDE_1 = 'de:08115:4478:0:1'  # Rtg Backhaus (Aidlingen Deufringen)
	DEUFRINGEN_GEISSHALDE_2 = 'de:08115:4478:0:2'  # Rtg Rathaus (Aidlingen Deufringen)
	VERDISTRASSE = 'de:08116:4479'
	VERDISTRASSE_1 = 'de:08116:4479:0:3'  # eRtg Rathaus (Nürtingen Neckarhausen)
	VERDISTRASSE_2 = 'de:08116:4479:0:4'  # eRtg Neckarbrücke (Nürtingen Neckarhausen)
	NECKARHAUSEN_VERDISTRASSE = 'de:08116:4479'
	NECKARHAUSEN_VERDISTRASSE_1 = 'de:08116:4479:0:3'  # eRtg Rathaus (Nürtingen Neckarhausen)
	NECKARHAUSEN_VERDISTRASSE_2 = 'de:08116:4479:0:4'  # eRtg Neckarbrücke (Nürtingen Neckarhausen)
	NECKARHAUSEN_RATHAUS = 'de:08116:4480'
	NECKARHAUSEN_RATHAUS_1 = 'de:08116:4480:0:3'  # eRtg Halde (Nürtingen Neckarhausen)
	NECKARHAUSEN_RATHAUS_2 = 'de:08116:4480:0:4'  # eRtg Verdistraße/Neckarhausener Straße (Nürtingen Neckarhausen)
	NECKARHAUSEN_HALDE = 'de:08116:4481'
	NECKARHAUSEN_HALDE_1 = 'de:08116:4481:0:3'  # eRtg Wasserwerk (Nürtingen Neckarhausen)
	NECKARHAUSEN_HALDE_2 = 'de:08116:4481:0:4'  # eRtg Rathaus (Nürtingen Neckarhausen)
	NECKARTAILF__NÜRTINGER_STR = 'de:08116:4482'
	NECKARTAILF__NÜRTINGER_STR_1 = 'de:08116:4482:0:3'  # Rtg Wasserwerk (Neckartailfingen Neckartailfingen)
	NECKARTAILF__NÜRTINGER_STR_2 = 'de:08116:4482:0:4'  # Rtg Rathaus (Neckartailfingen Neckartailfingen)
	NECKARTAILFINGEN_RATHAUS = 'de:08116:4483'
	NECKARTAILFINGEN_RATHAUS_1 = 'de:08116:4483:1:3'  # eRtg Nürtinger Straße (Neckartailfingen Neckartailfingen)
	NECKARTAILFINGEN_RATHAUS_2 = 'de:08116:4483:1:4'  # eRtg Tübinger Straße / Eichendorffstraße (Neckartailfingen Neckartailfingen)
	NECKARTAILFINGEN_SCHULE = 'de:08116:4484'
	NECKARTAILFINGEN_SCHULE_1 = 'de:08116:4484:0:3'  # eRtg Neckarbrücke/Schillerstr/Eichendorf (Neckartailfingen Neckartailfingen)
	NECKARTAILFINGEN_SCHULE_2 = 'de:08116:4484:0:4'  # eRtg Stuttgarter Str/Obere Liesäcker (Neckartailfingen Neckartailfingen)
	NECKARTENZLINGEN_SPITZACKER = 'de:08116:4485'
	NECKARTENZLINGEN_SPITZACKER_1 = 'de:08116:4485:1:3'  # alle Richtungen (Neckartenzlingen Neckartenzlingen)
	NECKARTENZLINGEN_SPITZACKER_2 = 'de:08116:4485:1:4'  # Rtg R.-Bosch-Str. (Neckartenzlingen Neckartenzlingen)
	SPITZACKER = 'de:08116:4485'
	SPITZACKER_1 = 'de:08116:4485:1:3'  # alle Richtungen (Neckartenzlingen Neckartenzlingen)
	SPITZACKER_2 = 'de:08116:4485:1:4'  # Rtg R.-Bosch-Str. (Neckartenzlingen Neckartenzlingen)
	NECKARTENZLINGEN_MARKTPLATZ = 'de:08116:4486'
	NECKARTENZLINGEN_MARKTPLATZ_1 = 'de:08116:4486:0:3'  # Rtg Auwiesen/Rathaus/Metzinger Str. (Neckartenzlingen Neckartenzlingen)
	NECKARTENZLINGEN_MARKTPLATZ_2 = 'de:08116:4486:0:4'  # Rtg Spitzacker (Neckartenzlingen Neckartenzlingen)
	NECKARTENZLINGEN_RATHAUS = 'de:08116:4487'
	NECKARTENZLINGEN_RATHAUS_1 = 'de:08116:4487:0:3'  # eRtg Marktplatz (Neckartenzlingen Neckartenzlingen)
	NECKARTENZLINGEN_RATHAUS_2 = 'de:08116:4487:0:4'  # eRtg Auwiesenschule (Neckartenzlingen Neckartenzlingen)
	ROTENBACH = 'de:08116:4488'
	ROTENBACH_1 = 'de:08116:4488:0:3'  # Rtg Neckartenzlingen (Neckartenzlingen Neckartenzlingen)
	ROTENBACH_2 = 'de:08116:4488:0:4'  # Rtg Altdorf (Neckartenzlingen Neckartenzlingen)
	NECKARTENZLINGEN_ROTENBACH = 'de:08116:4488'
	NECKARTENZLINGEN_ROTENBACH_1 = 'de:08116:4488:0:3'  # Rtg Neckartenzlingen (Neckartenzlingen Neckartenzlingen)
	NECKARTENZLINGEN_ROTENBACH_2 = 'de:08116:4488:0:4'  # Rtg Altdorf (Neckartenzlingen Neckartenzlingen)
	ALTDORF_STUTTGARTER_STRASSE = 'de:08116:4489'
	ALTDORF_STUTTGARTER_STRASSE_1 = 'de:08116:4489:0:3'  # Rtg Neckartailfingen (Altdorf Altdorf (ES))
	ALTDORF_STUTTGARTER_STRASSE_2 = 'de:08116:4489:0:4'  # Rtg Neckartenzlingen (Altdorf Altdorf (ES))
	BEMPFLINGEN_SCHULE = 'de:08116:4490:0:4'  # alle Richtungen (Bempflingen Bempflingen)
	KAPPISHÄUSERN_FEUERSEE = 'de:08116:4492:0:3'  # eRtg Rathaus (Neuffen Kappishäusern)
	FEUERSEE_1 = 'de:08116:4492:0:3'  # eRtg Rathaus (Neuffen Kappishäusern)
	FEUERSEE_2 = 'de:08111:6221:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	FEUERSEE_3 = 'de:08111:6221:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	FEUERSEE_4 = 'de:08111:6221:2:5'  # 41 Rtg Mar.-/Silberb.Str/92 Rtg Silber/R (Stuttgart Stuttgart)
	FEUERSEE_5 = 'de:08111:6221:3:6'  # Rtg Schloss-/Johannesstr. (Stuttgart Stuttgart)
	FEUERSEE_6 = 'de:08111:6221:4:3'  # Rtg Senefelderstr. (Stuttgart Stuttgart)
	FEUERSEE_7 = 'de:08111:6221:5:4'  # Rtg Marienstr./Rotebühlplatz (Stuttgart Stuttgart)
	KAPPISHÄUSERN_LINDE = 'de:08116:4493'
	KAPPISHÄUSERN_LINDE_1 = 'de:08116:4493:1:3'  # Rtg Metzingen (Neuffen Kappishäusern)
	KAPPISHÄUSERN_LINDE_2 = 'de:08116:4493:1:4'  # Rtg Neuffen/Kohlberg (Neuffen Kappishäusern)
	KAPPISHÄUSERN_LINDE_3 = 'de:08116:4493:1:5'  # Rtg Kappishäusern (Neuffen Kappishäusern)
	SCHORNDORF_REMS_MURR_KLINIK = 'de:08119:4497'
	SCHORNDORF_REMS_MURR_KLINIK_1 = 'de:08119:4497:0:3'  # Rtg Innenstadt/KammPlatz (Schorndorf Schorndorf)
	SCHORNDORF_REMS_MURR_KLINIK_2 = 'de:08119:4497:0:4'  # Rtg VierPappeln/Grauhalde (Schorndorf Schorndorf)
	REMS_MURR_KLINIK = 'de:08119:4497'
	REMS_MURR_KLINIK_1 = 'de:08119:4497:0:3'  # Rtg Innenstadt/KammPlatz (Schorndorf Schorndorf)
	REMS_MURR_KLINIK_2 = 'de:08119:4497:0:4'  # Rtg VierPappeln/Grauhalde (Schorndorf Schorndorf)
	VIER_PAPPELN = 'de:08119:4498'
	VIER_PAPPELN_1 = 'de:08119:4498:0:3'  # Rtg Schlichten (Schorndorf Schorndorf)
	VIER_PAPPELN_2 = 'de:08119:4498:0:4'  # Rtg Innenstadt (Schorndorf Schorndorf)
	SCHORNDORF_VIER_PAPPELN = 'de:08119:4498'
	SCHORNDORF_VIER_PAPPELN_1 = 'de:08119:4498:0:3'  # Rtg Schlichten (Schorndorf Schorndorf)
	SCHORNDORF_VIER_PAPPELN_2 = 'de:08119:4498:0:4'  # Rtg Innenstadt (Schorndorf Schorndorf)
	SCHLICHTEN_KIRCHE = 'de:08119:4501'
	SCHLICHTEN_KIRCHE_1 = 'de:08119:4501:0:3'  # Rtg Schlichten Strutstr. (Schorndorf Schlichten)
	SCHLICHTEN_KIRCHE_2 = 'de:08119:4501:0:4'  # Rtg Thomashardt Abzw. Baiereck (Schorndorf Schlichten)
	THOMASHARDT_ABZW__BAIERECK = 'de:08116:4502'
	THOMASHARDT_ABZW__BAIERECK_1 = 'de:08116:4502:0:3'  # Rtg Baiereck/Schlichten (Lichtenwald Thomashardt)
	THOMASHARDT_ABZW__BAIERECK_2 = 'de:08116:4502:0:4'  # Rtg Baiereck/Lichtenwald (Lichtenwald Thomashardt)
	ABZWEIGUNG_BAIERECK = 'de:08116:4502'
	ABZWEIGUNG_BAIERECK_1 = 'de:08116:4502:0:3'  # Rtg Baiereck/Schlichten (Lichtenwald Thomashardt)
	ABZWEIGUNG_BAIERECK_2 = 'de:08116:4502:0:4'  # Rtg Baiereck/Lichtenwald (Lichtenwald Thomashardt)
	THOMASHARDT_HEGENLOHER_STRASSE = 'de:08116:4503'
	THOMASHARDT_HEGENLOHER_STRASSE_1 = 'de:08116:4503:0:3'  # Rtg Hegenlohe (Lichtenwald Thomashardt)
	THOMASHARDT_HEGENLOHER_STRASSE_2 = 'de:08116:4503:0:4'  # Rtg Schlichten (Lichtenwald Thomashardt)
	HEGENLOHER_STRASSE = 'de:08116:4503'
	HEGENLOHER_STRASSE_1 = 'de:08116:4503:0:3'  # Rtg Hegenlohe (Lichtenwald Thomashardt)
	HEGENLOHER_STRASSE_2 = 'de:08116:4503:0:4'  # Rtg Schlichten (Lichtenwald Thomashardt)
	NACHBARSCHAFTSSCHULE_1 = 'de:08116:4504:0:3'  # eRtg Friedhof (Lichtenwald Thomashardt)
	NACHBARSCHAFTSSCHULE_2 = 'de:08116:4504:0:4'  # eRtg Bürgerzentrum (Lichtenwald Thomashardt)
	NACHBARSCHAFTSSCHULE_3 = 'de:08119:5277:0:3'  # eRtg Lannerstraße (Berglen Bretzenacker)
	NACHBARSCHAFTSSCHULE_4 = 'de:08119:5277:0:4'  # eRtg Erlenhof (Berglen Bretzenacker)
	THOMASH__NACHBARSCHAFTSSCHULE = 'de:08116:4504'
	THOMASH__NACHBARSCHAFTSSCHULE_1 = 'de:08116:4504:0:3'  # eRtg Friedhof (Lichtenwald Thomashardt)
	THOMASH__NACHBARSCHAFTSSCHULE_2 = 'de:08116:4504:0:4'  # eRtg Bürgerzentrum (Lichtenwald Thomashardt)
	HEGENLOHE_KIRCHE = 'de:08116:4505'
	HEGENLOHE_KIRCHE_1 = 'de:08116:4505:0:3'  # eRtg Thomashardter Straße (Lichtenwald Hegenlohe)
	HEGENLOHE_KIRCHE_2 = 'de:08116:4505:0:4'  # eRtg Bannmühle (Lichtenwald Hegenlohe)
	REICHENBACH__F__BANNMÜHLE = 'de:08116:4506'
	REICHENBACH__F__BANNMÜHLE_1 = 'de:08116:4506:0:3'  # eRtg Kirche (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F__BANNMÜHLE_2 = 'de:08116:4506:0:4'  # eRtg Reichenbachstraße (Reichenbach an der Fils Reichenbach (F))
	BANNMÜHLE = 'de:08116:4506'
	BANNMÜHLE_1 = 'de:08116:4506:0:3'  # eRtg Kirche (Reichenbach an der Fils Reichenbach (F))
	BANNMÜHLE_2 = 'de:08116:4506:0:4'  # eRtg Reichenbachstraße (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__REICHENBACHSTR = 'de:08116:4507'
	REICHENB__F__REICHENBACHSTR_1 = 'de:08116:4507:0:3'  # Rtg Lichtenwald (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__REICHENBACHSTR_2 = 'de:08116:4507:0:4'  # Rtg Reichenbach (Reichenbach an der Fils Reichenbach (F))
	REICHENBACHSTRASSE = 'de:08116:4507'
	REICHENBACHSTRASSE_1 = 'de:08116:4507:0:3'  # Rtg Lichtenwald (Reichenbach an der Fils Reichenbach (F))
	REICHENBACHSTRASSE_2 = 'de:08116:4507:0:4'  # Rtg Reichenbach (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F__BLUMENSTRASSE = 'de:08116:4508'
	REICHENBACH__F__BLUMENSTRASSE_1 = 'de:08116:4508:0:3'  # eRtg Schorndorfer Straße (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F__BLUMENSTRASSE_2 = 'de:08116:4508:0:4'  # eRtg Blumen-/Wilhelmstraße (Reichenbach an der Fils Reichenbach (F))
	MÖTZINGEN_RATHAUS = 'de:08115:4509'
	MÖTZINGEN_RATHAUS_1 = 'de:08115:4509:0:3'  # A Rtg Bondorf (Mötzingen Mötzingen)
	MÖTZINGEN_RATHAUS_2 = 'de:08115:4509:0:4'  # B Rtg Lindenhof/Goethestr. (Mötzingen Mötzingen)
	MÖTZINGEN_RATHAUS_3 = 'de:08115:4509:0:5'  # C Rtg Nagold (Mötzingen Mötzingen)
	BONDORF = 'de:08115:4510'
	BONDORF_1 = 'de:08115:4510:1:2'  # Gleis 2 (Bondorf Bondorf)
	BONDORF_2 = 'de:08115:4510:1:3'  # Gleis 3 (Bondorf Bondorf)
	BONDORF_3 = 'de:08115:4510:2:1'  # Gleis 1 (Bondorf Bondorf)
	BONDORF_4 = 'de:08115:4510:3:3'  # alle Rtg (Bondorf Bondorf)
	HERRENBERG = 'de:08115:4512'
	HERRENBERG_1 = 'de:08115:4512:1:2'  # Gleis 2 (Herrenberg Herrenberg)
	HERRENBERG_2 = 'de:08115:4512:1:3'  # Gleis 3 (Herrenberg Herrenberg)
	HERRENBERG_3 = 'de:08115:4512:2:1'  # Gleis 1 (Herrenberg Herrenberg)
	HERRENBERG_4 = 'de:08115:4512:3:4'  # Gleis 4 (Herrenberg Herrenberg)
	HERRENBERG_5 = 'de:08115:4512:4:101'  # Gleis 101 (Herrenberg Herrenberg)
	HERRENBERG_6 = 'de:08115:4512:4:102'  # Gleis 102 (Herrenberg Herrenberg)
	HERRENBERG_7 = 'de:08115:4512:5:12'  # SEV Bahnhofstraße (Herrenberg Herrenberg)
	HERRENBERG_8 = 'de:08115:4512:5:A'  # Bstg A (Herrenberg Herrenberg)
	HERRENBERG_9 = 'de:08115:4512:5:B'  # Bstg B (Herrenberg Herrenberg)
	HERRENBERG_10 = 'de:08115:4512:5:C'  # Bstg C (Herrenberg Herrenberg)
	HERRENBERG_11 = 'de:08115:4512:5:D'  # Bstg D (Herrenberg Herrenberg)
	HERRENBERG_12 = 'de:08115:4512:5:E'  # Bstg E (Herrenberg Herrenberg)
	HERRENBERG_13 = 'de:08115:4512:5:F'  # Bstg F (Herrenberg Herrenberg)
	HERRENBERG_14 = 'de:08115:4512:5:G'  # Bstg X (Herrenberg Herrenberg)
	HERRENBERG_BERLINER_STRASSE = 'de:08115:4513'
	HERRENBERG_BERLINER_STRASSE_1 = 'de:08115:4513:0:3'  # Rtg Schießtäle ZOB Kalkofenstr. (Herrenberg Herrenberg)
	HERRENBERG_BERLINER_STRASSE_2 = 'de:08115:4513:0:4'  # Rtg Schulmeisterbuche (Herrenberg Herrenberg)
	KALLENBERG_RASTHAUS = 'de:08118:4516'
	KALLENBERG_RASTHAUS_1 = 'de:08118:4516:0:3'  # Rtg Lackfabrik (Korntal-Münchingen Münchingen)
	KALLENBERG_RASTHAUS_2 = 'de:08118:4516:0:4'  # Rtg Gaststätte (Korntal-Münchingen Münchingen)
	MÜNCH__KALLENBERG_RASTHAUS = 'de:08118:4516'
	MÜNCH__KALLENBERG_RASTHAUS_1 = 'de:08118:4516:0:3'  # Rtg Lackfabrik (Korntal-Münchingen Münchingen)
	MÜNCH__KALLENBERG_RASTHAUS_2 = 'de:08118:4516:0:4'  # Rtg Gaststätte (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_MÜLLERHEIM = 'de:08118:4518'
	MÜNCHINGEN_MÜLLERHEIM_1 = 'de:08118:4518:0:3'  # Rtg Kriegbaum (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_MÜLLERHEIM_2 = 'de:08118:4518:0:4'  # Rtg Lackfabrik (Korntal-Münchingen Münchingen)
	MÜLLERHEIM = 'de:08118:4518'
	MÜLLERHEIM_1 = 'de:08118:4518:0:3'  # Rtg Kriegbaum (Korntal-Münchingen Münchingen)
	MÜLLERHEIM_2 = 'de:08118:4518:0:4'  # Rtg Lackfabrik (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_KORNWESTHEIMER_STR = 'de:08118:4519'
	MÜNCHINGEN_KORNWESTHEIMER_STR_1 = 'de:08118:4519:0:3'  # eRtg Sportzentrum (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_KORNWESTHEIMER_STR_2 = 'de:08118:4519:0:4'  # eRtg Müllerheim (Korntal-Münchingen Münchingen)
	SPORTZENTRUM_1 = 'de:08118:4520:0:3'  # eRtg Ziegeleistraße (Korntal-Münchingen Münchingen)
	SPORTZENTRUM_2 = 'de:08118:4520:0:4'  # eRtg Kornwestheimer Straße (Korntal-Münchingen Münchingen)
	SPORTZENTRUM_3 = 'de:08118:4520:0:5'  # Rtg Ziegeleistraße in Stuttgarter Str. (Korntal-Münchingen Münchingen)
	SPORTZENTRUM_4 = 'de:08118:4520:0:6'  # Rtg Lingwiesen (Korntal-Münchingen Münchingen)
	SPORTZENTRUM_5 = 'de:08116:7177:0:3'  # Rtg Musberg Seebruckenmühle (Leinfelden-Echterdingen Leinfelden)
	SPORTZENTRUM_6 = 'de:08116:7177:0:4'  # Rtg Echterdingen Kelterrain (Leinfelden-Echterdingen Leinfelden)
	MÜNCHINGEN_SPORTZENTRUM = 'de:08118:4520'
	MÜNCHINGEN_SPORTZENTRUM_1 = 'de:08118:4520:0:3'  # eRtg Ziegeleistraße (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_SPORTZENTRUM_2 = 'de:08118:4520:0:4'  # eRtg Kornwestheimer Straße (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_SPORTZENTRUM_3 = 'de:08118:4520:0:5'  # Rtg Ziegeleistraße in Stuttgarter Str. (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_SPORTZENTRUM_4 = 'de:08118:4520:0:6'  # Rtg Lingwiesen (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_DANZIGER_STRASSE = 'de:08118:4521:0:3'  # eRtg Sportzentrum (Korntal-Münchingen Münchingen)
	DANZIGER_STRASSE_1 = 'de:08118:4521:0:3'  # eRtg Sportzentrum (Korntal-Münchingen Münchingen)
	DANZIGER_STRASSE_2 = 'de:08115:4711:1:1'  # Rtg Dettenhausen (Böblingen Böblingen)
	DANZIGER_STRASSE_3 = 'de:08115:4711:1:2'  # Rtg Böblingen (Böblingen Böblingen)
	DANZIGER_STRASSE_4 = 'de:08118:5533:0:3'  # Rtg Danziger Str. (Ludwigsburg Ludwigsburg)
	DANZIGER_STRASSE_5 = 'de:08118:5533:0:4'  # Rtg Hindenburgstr. (Ludwigsburg Ludwigsburg)
	DANZIGER_STRASSE_6 = 'de:08119:6859:0:3'  # Rtg Zwingelh. Str. (Kirchberg an der Murr Kirchberg (M))
	DANZIGER_STRASSE_7 = 'de:08117:1021:0:H1'  # Rtg Vogelgarten (Eislingen/Fils Eislingen (F))
	DANZIGER_STRASSE_8 = 'de:08117:1021:0:R1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	MÜNCHINGEN_ZIEGELEISTRASSE = 'de:08118:4522'
	MÜNCHINGEN_ZIEGELEISTRASSE_1 = 'de:08118:4522:0:3'  # Rtg Bahnhof/Stiegelplatz (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_ZIEGELEISTRASSE_2 = 'de:08118:4522:0:4'  # Rtg Danziger Platz (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_STIEGELPLATZ = 'de:08118:4523'
	MÜNCHINGEN_STIEGELPLATZ_1 = 'de:08118:4523:1:3'  # Rtg Schwieber./Ditz. (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_STIEGELPLATZ_2 = 'de:08118:4523:1:4'  # Rtg Münchingen/Danziger (Korntal-Münchingen Münchingen)
	STIEGELPLATZ = 'de:08118:4523'
	STIEGELPLATZ_1 = 'de:08118:4523:1:3'  # Rtg Schwieber./Ditz. (Korntal-Münchingen Münchingen)
	STIEGELPLATZ_2 = 'de:08118:4523:1:4'  # Rtg Münchingen/Danziger (Korntal-Münchingen Münchingen)
	SÄGMÜHLE = 'de:08118:4524'
	SÄGMÜHLE_1 = 'de:08118:4524:0:3'  # eRtg Münchinger Straße (Hemmingen Hemmingen)
	SÄGMÜHLE_2 = 'de:08118:4524:0:4'  # eRtg Stiegelplatz (Hemmingen Hemmingen)
	HEMMINGEN_SÄGMÜHLE = 'de:08118:4524'
	HEMMINGEN_SÄGMÜHLE_1 = 'de:08118:4524:0:3'  # eRtg Münchinger Straße (Hemmingen Hemmingen)
	HEMMINGEN_SÄGMÜHLE_2 = 'de:08118:4524:0:4'  # eRtg Stiegelplatz (Hemmingen Hemmingen)
	SCHWIEBERDINGEN_H__ESSIG_STR = 'de:08118:4525'
	SCHWIEBERDINGEN_H__ESSIG_STR_1 = 'de:08118:4525:0:3'  # eRtg Mitte (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_H__ESSIG_STR_2 = 'de:08118:4525:0:4'  # eRtg Stettiner Straße (Schwieberdingen Schwieberdingen)
	HERMANN_ESSIG_STRASSE = 'de:08118:4525'
	HERMANN_ESSIG_STRASSE_1 = 'de:08118:4525:0:3'  # eRtg Mitte (Schwieberdingen Schwieberdingen)
	HERMANN_ESSIG_STRASSE_2 = 'de:08118:4525:0:4'  # eRtg Stettiner Straße (Schwieberdingen Schwieberdingen)
	HEMMINGEN_RATHAUS = 'de:08118:4526'
	HEMMINGEN_RATHAUS_1 = 'de:08118:4526:0:3'  # Rtg Hemmingen (Hemmingen Hemmingen)
	HEMMINGEN_RATHAUS_2 = 'de:08118:4526:0:4'  # Rtg Schwieberdingen (Hemmingen Hemmingen)
	HEMMINGEN = 'de:08118:4527'
	HEMMINGEN_1 = 'de:08118:4527:1:1'  # Beide Richtungen (Hemmingen Hemmingen)
	HEMMINGEN_2 = 'de:08118:4527:2:3'  # Endstation/Rtg Schauert/Bfstr/Höfingen (Hemmingen Hemmingen)
	HEMMINGEN_3 = 'de:08118:4527:2:4'  # Rtg Eisgasse/Rathaus/Schwieberdingen (Hemmingen Hemmingen)
	HEMMINGEN_SCHAUCHERT = 'de:08118:4528:0:4'  # eRtg Hemmingen (Hemmingen Hemmingen)
	SCHAUCHERT = 'de:08118:4528:0:4'  # eRtg Hemmingen (Hemmingen Hemmingen)
	HEIMERDINGEN_HEMMINGER_STRASSE = 'de:08118:4529'
	HEIMERDINGEN_HEMMINGER_STRASSE_1 = 'de:08118:4529:1:3'  # eRtg Weissacher Straße (Ditzingen Heimerdingen)
	HEIMERDINGEN_HEMMINGER_STRASSE_2 = 'de:08118:4529:1:4'  # eRtg Heimerdingen (Ditzingen Heimerdingen)
	HEIMERDINGEN_WEISSACHER_STRASSE = 'de:08118:4530'
	HEIMERDINGEN_WEISSACHER_STRASSE_1 = 'de:08118:4530:0:3'  # Rtg Weissach/Wiesenäcker (Ditzingen Heimerdingen)
	HEIMERDINGEN_WEISSACHER_STRASSE_2 = 'de:08118:4530:0:4'  # Rtg Hemmingen/Schöckingen (Ditzingen Heimerdingen)
	WEISSACHER_STRASSE = 'de:08118:4530'
	WEISSACHER_STRASSE_1 = 'de:08118:4530:0:3'  # Rtg Weissach/Wiesenäcker (Ditzingen Heimerdingen)
	WEISSACHER_STRASSE_2 = 'de:08118:4530:0:4'  # Rtg Hemmingen/Schöckingen (Ditzingen Heimerdingen)
	WIESENÄCKERSTRASSE = 'de:08118:4531:0:3'  # eRtg Ölmühle/Weissacher Straße (Ditzingen Heimerdingen)
	HEIMERDINGEN_WIESENÄCKERSTRASSE = 'de:08118:4531:0:3'  # eRtg Ölmühle/Weissacher Straße (Ditzingen Heimerdingen)
	ÖLMÜHLE = 'de:08115:4532'
	ÖLMÜHLE_1 = 'de:08115:4532:0:3'  # Rtg Weissach (Weissach Weissach (BB))
	ÖLMÜHLE_2 = 'de:08115:4532:0:4'  # Rtg Heimerdingen (Weissach Weissach (BB))
	WEISSACH_ÖLMÜHLE = 'de:08115:4532'
	WEISSACH_ÖLMÜHLE_1 = 'de:08115:4532:0:3'  # Rtg Weissach (Weissach Weissach (BB))
	WEISSACH_ÖLMÜHLE_2 = 'de:08115:4532:0:4'  # Rtg Heimerdingen (Weissach Weissach (BB))
	WEISSACH_BAHNHOFSTRASSE = 'de:08115:4533'
	WEISSACH_BAHNHOFSTRASSE_1 = 'de:08115:4533:1:3'  # Rtg Marktplatz (Weissach Weissach (BB))
	WEISSACH_BAHNHOFSTRASSE_2 = 'de:08115:4533:1:4'  # Rtg Heimerdingen/Wöhr (Weissach Weissach (BB))
	WEISSACH_MARKTPLATZ = 'de:08115:4534'
	WEISSACH_MARKTPLATZ_1 = 'de:08115:4534:1:4'  # Südseite (Weissach Weissach (BB))
	WEISSACH_MARKTPLATZ_2 = 'de:08115:4534:2:3'  # Marktseite (Weissach Weissach (BB))
	SCHÖCKINGER_STRASSE = 'de:08118:4535'
	SCHÖCKINGER_STRASSE_1 = 'de:08118:4535:0:3'  # Rtg Eisgasse (Hemmingen Hemmingen)
	SCHÖCKINGER_STRASSE_2 = 'de:08118:4535:0:4'  # Rtg Schöckingen (Hemmingen Hemmingen)
	HEMMINGEN_SCHÖCKINGER_STRASSE = 'de:08118:4535'
	HEMMINGEN_SCHÖCKINGER_STRASSE_1 = 'de:08118:4535:0:3'  # Rtg Eisgasse (Hemmingen Hemmingen)
	HEMMINGEN_SCHÖCKINGER_STRASSE_2 = 'de:08118:4535:0:4'  # Rtg Schöckingen (Hemmingen Hemmingen)
	HEIMERDINGEN = 'de:08118:4536'
	HEIMERDINGEN_1 = 'de:08118:4536:1:1'  # R-Bahn (Ditzingen Heimerdingen)
	HEIMERDINGEN_2 = 'de:08118:4536:2:3'  # Rtg Gewerbegebiet Nord (Ditzingen Heimerdingen)
	HEIMERDINGEN_3 = 'de:08118:4536:2:4'  # Rtg Gewerbegebiet Süd (Ditzingen Heimerdingen)
	WARMBRONN_ORTSMITTE = 'de:08115:4537:0:4'  # Rtg Hauptstr. 77 (Leonberg Warmbronn)
	HAUPTSTRASSE_77 = 'de:08115:4538'
	HAUPTSTRASSE_77_1 = 'de:08115:4538:0:3'  # Rtg Talackerstr. (Leonberg Warmbronn)
	HAUPTSTRASSE_77_2 = 'de:08115:4538:0:4'  # Rtg Lauerhalde (Leonberg Warmbronn)
	WARMBRONN_HAUPTSTRASSE_77 = 'de:08115:4538'
	WARMBRONN_HAUPTSTRASSE_77_1 = 'de:08115:4538:0:3'  # Rtg Talackerstr. (Leonberg Warmbronn)
	WARMBRONN_HAUPTSTRASSE_77_2 = 'de:08115:4538:0:4'  # Rtg Lauerhalde (Leonberg Warmbronn)
	WARMBRONN_LAUERHALDE = 'de:08115:4539'
	WARMBRONN_LAUERHALDE_1 = 'de:08115:4539:0:3'  # Rtg Hauptstr. (Leonberg Warmbronn)
	WARMBRONN_LAUERHALDE_2 = 'de:08115:4539:0:4'  # Rtg Eltingen (Leonberg Warmbronn)
	LAUERHALDE = 'de:08115:4539'
	LAUERHALDE_1 = 'de:08115:4539:0:3'  # Rtg Hauptstr. (Leonberg Warmbronn)
	LAUERHALDE_2 = 'de:08115:4539:0:4'  # Rtg Eltingen (Leonberg Warmbronn)
	ELTINGEN_HERTICHSTRASSE_33 = 'de:08115:4540'
	ELTINGEN_HERTICHSTRASSE_33_1 = 'de:08115:4540:0:3'  # Rtg Warmbronn (Leonberg Eltingen)
	ELTINGEN_HERTICHSTRASSE_33_2 = 'de:08115:4540:0:4'  # Rtg Leobad (Leonberg Eltingen)
	HERTICHSTRASSE_33 = 'de:08115:4540'
	HERTICHSTRASSE_33_1 = 'de:08115:4540:0:3'  # Rtg Warmbronn (Leonberg Eltingen)
	HERTICHSTRASSE_33_2 = 'de:08115:4540:0:4'  # Rtg Leobad (Leonberg Eltingen)
	ELTINGEN_NIEDERHOFEN = 'de:08115:4541'
	ELTINGEN_NIEDERHOFEN_1 = 'de:08115:4541:0:3'  # Rtg Warmbronn (Leonberg Eltingen)
	ELTINGEN_NIEDERHOFEN_2 = 'de:08115:4541:0:4'  # Rtg Elt. Brennerstr. (Leonberg Eltingen)
	NIEDERHOFEN = 'de:08115:4541'
	NIEDERHOFEN_1 = 'de:08115:4541:0:3'  # Rtg Warmbronn (Leonberg Eltingen)
	NIEDERHOFEN_2 = 'de:08115:4541:0:4'  # Rtg Elt. Brennerstr. (Leonberg Eltingen)
	ELTINGEN_BRENNERSTRASSE = 'de:08115:4543'
	ELTINGEN_BRENNERSTRASSE_1 = 'de:08115:4543:0:3'  # Rtg Gebersheimer Str. (Leonberg Eltingen)
	ELTINGEN_BRENNERSTRASSE_2 = 'de:08115:4543:0:4'  # Rtg Neuköllner Platz (Leonberg Eltingen)
	BRENNERSTRASSE = 'de:08115:4543'
	BRENNERSTRASSE_1 = 'de:08115:4543:0:3'  # Rtg Gebersheimer Str. (Leonberg Eltingen)
	BRENNERSTRASSE_2 = 'de:08115:4543:0:4'  # Rtg Neuköllner Platz (Leonberg Eltingen)
	MARIE_CURIE_SCHULE = 'de:08115:4546'
	MARIE_CURIE_SCHULE_1 = 'de:08115:4546:0:3'  # Rtg Neuköllner Pl (Warmbronn) (Leonberg Ramtel)
	MARIE_CURIE_SCHULE_2 = 'de:08115:4546:0:4'  # Rtg Gerlinger Str. (Leonberg) (Leonberg Ramtel)
	RAMTEL_MARIE_CURIE_SCHULE = 'de:08115:4546'
	RAMTEL_MARIE_CURIE_SCHULE_1 = 'de:08115:4546:0:3'  # Rtg Neuköllner Pl (Warmbronn) (Leonberg Ramtel)
	RAMTEL_MARIE_CURIE_SCHULE_2 = 'de:08115:4546:0:4'  # Rtg Gerlinger Str. (Leonberg) (Leonberg Ramtel)
	ALBERT_SCHWEITZER_GYMNASIUM = 'de:08115:4547'
	ALBERT_SCHWEITZER_GYMNASIUM_1 = 'de:08115:4547:1:3'  # Rtg Neuköllner Pl (Leonberg Leonberg)
	ALBERT_SCHWEITZER_GYMNASIUM_2 = 'de:08115:4547:1:4'  # Rtg Römerstr (Leonberg Leonberg)
	LEONB__A__SCHWEITZER_GYMNASIUM = 'de:08115:4547'
	LEONB__A__SCHWEITZER_GYMNASIUM_1 = 'de:08115:4547:1:3'  # Rtg Neuköllner Pl (Leonberg Leonberg)
	LEONB__A__SCHWEITZER_GYMNASIUM_2 = 'de:08115:4547:1:4'  # Rtg Römerstr (Leonberg Leonberg)
	ELT__GEBERSH___ROSEGGERSTR = 'de:08115:4548'
	ELT__GEBERSH___ROSEGGERSTR_1 = 'de:08115:4548:0:3'  # Rtg Niederhofen (Leonberg Eltingen)
	ELT__GEBERSH___ROSEGGERSTR_2 = 'de:08115:4548:0:4'  # Rtg Brennerstr. (Leonberg Eltingen)
	GEBERSHEIMER__ROSEGGERSTRASSE = 'de:08115:4548'
	GEBERSHEIMER__ROSEGGERSTRASSE_1 = 'de:08115:4548:0:3'  # Rtg Niederhofen (Leonberg Eltingen)
	GEBERSHEIMER__ROSEGGERSTRASSE_2 = 'de:08115:4548:0:4'  # Rtg Brennerstr. (Leonberg Eltingen)
	WARMBRONN_TALACKERSTRASSE = 'de:08115:4549:0:3'  # Rtg Büsnauer Str. (Leonberg Warmbronn)
	TALACKERSTRASSE = 'de:08115:4549:0:3'  # Rtg Büsnauer Str. (Leonberg Warmbronn)
	GEBERSHEIM_RATHAUS = 'de:08115:4550'
	GEBERSHEIM_RATHAUS_1 = 'de:08115:4550:0:3'  # Rtg Carl-Zeiss-Straße (Leonberg Gebersheim)
	GEBERSHEIM_RATHAUS_2 = 'de:08115:4550:0:4'  # Rtg Rutesheim Dieselstraße (Leonberg Gebersheim)
	SIND__MERCED__BENZ_TOR_III = 'de:08115:4551'
	SIND__MERCED__BENZ_TOR_III_1 = 'de:08115:4551:0:3'  # eRtg Mercedes-Benz Hst. F (Sindelfingen Sindelfingen)
	SIND__MERCED__BENZ_TOR_III_2 = 'de:08115:4551:0:4'  # Rtg Sindelfingen (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_TOR_III = 'de:08115:4551'
	MERCEDES_BENZ_TOR_III_1 = 'de:08115:4551:0:3'  # eRtg Mercedes-Benz Hst. F (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_TOR_III_2 = 'de:08115:4551:0:4'  # Rtg Sindelfingen (Sindelfingen Sindelfingen)
	GERL__EINKAUFSZENTRUM = 'de:08118:4552'
	GERL__EINKAUFSZENTRUM_1 = 'de:08118:4552:0:3'  # Rtg Margarethenweg (Gerlingen Gerlingen)
	GERL__EINKAUFSZENTRUM_2 = 'de:08118:4552:0:4'  # Rtg Gerlingen (Gerlingen Gerlingen)
	EINKAUFSZENTRUM_1 = 'de:08118:4552:0:3'  # Rtg Margarethenweg (Gerlingen Gerlingen)
	EINKAUFSZENTRUM_2 = 'de:08118:4552:0:4'  # Rtg Gerlingen (Gerlingen Gerlingen)
	EINKAUFSZENTRUM_3 = 'de:08115:4741:0:3'  # Rtg Kohlplatte/Eisberg (Jettingen Oberjettingen)
	EINKAUFSZENTRUM_4 = 'de:08115:4741:0:4'  # Rtg Sindlinger Str. (Jettingen Oberjettingen)
	EINKAUFSZENTRUM_5 = 'de:08117:3510:0:H1'  # Rtg Drackenstein (Bad Ditzenbach Gosbach)
	EINKAUFSZENTRUM_6 = 'de:08117:3510:0:R1'  # Rtg Lamm (Bad Ditzenbach Gosbach)
	EINKAUFSZENTRUM_7 = 'de:08235:10122:0:3'  # Rtg ZOB (Nagold Nagold)
	EINKAUFSZENTRUM_8 = 'de:08235:10122:0:4'  # Rtg IMMO (Nagold Nagold)
	GERLINGEN_LEONBERGER_STR = 'de:08118:4554'
	GERLINGEN_LEONBERGER_STR_1 = 'de:08118:4554:0:3'  # eRtg Schulstraße (Gerlingen Gerlingen)
	GERLINGEN_LEONBERGER_STR_2 = 'de:08118:4554:0:4'  # eRtg Ringstraße (Gerlingen Gerlingen)
	RUTESHEIM_DIESELSTR = 'de:08115:4556'
	RUTESHEIM_DIESELSTR_1 = 'de:08115:4556:0:3'  # Rtg Leonberg Gartenstadt/Schillerstr. (Rutesheim Rutesheim)
	RUTESHEIM_DIESELSTR_2 = 'de:08115:4556:0:4'  # Rtg Rathaus (Rutesheim Rutesheim)
	RUTESHEIM_BOSCH = 'de:08115:4557'
	RUTESHEIM_BOSCH_1 = 'de:08115:4557:0:3'  # Rtg Rutesheim Rathaus (Rutesheim Rutesheim)
	RUTESHEIM_BOSCH_2 = 'de:08115:4557:0:4'  # Rtg Rutesheim Bf (Rutesheim Rutesheim)
	RUTESHEIM_BOSCH_3 = 'de:08115:4557:0:5'  # Rtg Sporthalle (Rutesheim Rutesheim)
	BOSCH = 'de:08115:4557'
	BOSCH_1 = 'de:08115:4557:0:3'  # Rtg Rutesheim Rathaus (Rutesheim Rutesheim)
	BOSCH_2 = 'de:08115:4557:0:4'  # Rtg Rutesheim Bf (Rutesheim Rutesheim)
	BOSCH_3 = 'de:08115:4557:0:5'  # Rtg Sporthalle (Rutesheim Rutesheim)
	HEMMINGEN_BAHNHOFSTR = 'de:08118:4558'
	HEMMINGEN_BAHNHOFSTR_1 = 'de:08118:4558:1:3'  # eRtg Ankunft (Hemmingen Hemmingen)
	HEMMINGEN_BAHNHOFSTR_2 = 'de:08118:4558:1:4'  # eRtg Abfahrt (Hemmingen Hemmingen)
	DRESCHERSTRASSE = 'de:08115:4559'
	DRESCHERSTRASSE_1 = 'de:08115:4559:0:3'  # Rtg Perouse (Rutesheim Rutesheim)
	DRESCHERSTRASSE_2 = 'de:08115:4559:0:4'  # Rtg Rutesheim (Rutesheim Rutesheim)
	RUTESHEIM_DRESCHERSTR = 'de:08115:4559'
	RUTESHEIM_DRESCHERSTR_1 = 'de:08115:4559:0:3'  # Rtg Perouse (Rutesheim Rutesheim)
	RUTESHEIM_DRESCHERSTR_2 = 'de:08115:4559:0:4'  # Rtg Rutesheim (Rutesheim Rutesheim)
	MÜNCHINGEN_FRIEDHOF = 'de:08118:4560'
	MÜNCHINGEN_FRIEDHOF_1 = 'de:08118:4560:0:3'  # eRtg Ditzingen/Korntal (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_FRIEDHOF_2 = 'de:08118:4560:0:4'  # eRtg Stiegelplatz (Korntal-Münchingen Münchingen)
	ELTINGEN_GEISLINGER_STR__51 = 'de:08115:4561'
	ELTINGEN_GEISLINGER_STR__51_1 = 'de:08115:4561:0:3'  # Rtg Leonberg (Leonberg Eltingen)
	ELTINGEN_GEISLINGER_STR__51_2 = 'de:08115:4561:0:4'  # Endstation (Leonberg Eltingen)
	GEISLINGER_STRASSE_51 = 'de:08115:4561'
	GEISLINGER_STRASSE_51_1 = 'de:08115:4561:0:3'  # Rtg Leonberg (Leonberg Eltingen)
	GEISLINGER_STRASSE_51_2 = 'de:08115:4561:0:4'  # Endstation (Leonberg Eltingen)
	ELTINGEN_GEISLINGER_STR__21 = 'de:08115:4562'
	ELTINGEN_GEISLINGER_STR__21_1 = 'de:08115:4562:0:3'  # Rtg Stadtmitte (Leonberg Eltingen)
	ELTINGEN_GEISLINGER_STR__21_2 = 'de:08115:4562:0:4'  # Rtg Nr 51 (Leonberg Eltingen)
	GEISLINGER_STRASSE_21 = 'de:08115:4562'
	GEISLINGER_STRASSE_21_1 = 'de:08115:4562:0:3'  # Rtg Stadtmitte (Leonberg Eltingen)
	GEISLINGER_STRASSE_21_2 = 'de:08115:4562:0:4'  # Rtg Nr 51 (Leonberg Eltingen)
	ELTINGEN_GEISLINGER_STR__3 = 'de:08115:4563'
	ELTINGEN_GEISLINGER_STR__3_1 = 'de:08115:4563:0:3'  # Rtg Innenstadt (Leonberg Eltingen)
	ELTINGEN_GEISLINGER_STR__3_2 = 'de:08115:4563:0:4'  # Rtg Geislinger Straße 21 (Leonberg Eltingen)
	GEISLINGER_STRASSE_3 = 'de:08115:4563'
	GEISLINGER_STRASSE_3_1 = 'de:08115:4563:0:3'  # Rtg Innenstadt (Leonberg Eltingen)
	GEISLINGER_STRASSE_3_2 = 'de:08115:4563:0:4'  # Rtg Geislinger Straße 21 (Leonberg Eltingen)
	WEISSACH_PORSCHE = 'de:08115:4564'
	WEISSACH_PORSCHE_1 = 'de:08115:4564:0:2'  # Linien 634, 636 (Weissach Weissach (BB))
	WEISSACH_PORSCHE_2 = 'de:08115:4564:0:3'  # Linien 652, 653, 765 (Weissach Weissach (BB))
	PORSCHE_1 = 'de:08115:4564:0:2'  # Linien 634, 636 (Weissach Weissach (BB))
	PORSCHE_2 = 'de:08115:4564:0:3'  # Linien 652, 653, 765 (Weissach Weissach (BB))
	PORSCHE_3 = 'de:08118:7084:0:3'  # Endstation (Sachsenheim Großsachsenheim)
	FLACHT_SPORTPLATZ = 'de:08115:4565'
	FLACHT_SPORTPLATZ_1 = 'de:08115:4565:1:3'  # eRtg Neuenbühl (Weissach Flacht)
	FLACHT_SPORTPLATZ_2 = 'de:08115:4565:1:4'  # eRtg Ortsmitte (Weissach Flacht)
	SPORTPLATZ_1 = 'de:08115:4565:1:3'  # eRtg Neuenbühl (Weissach Flacht)
	SPORTPLATZ_2 = 'de:08115:4565:1:4'  # eRtg Ortsmitte (Weissach Flacht)
	SPORTPLATZ_3 = 'de:08118:5367:0:3'  # eRtg Egelsee (Marbach am Neckar Rielingshausen)
	SPORTPLATZ_4 = 'de:08118:5367:0:4'  # eRtg Hardtwaldhalle (Marbach am Neckar Rielingshausen)
	SPORTPLATZ_5 = 'de:08118:5494:1:3'  # eRtg Herrschaftsweg (Ludwigsburg Pflugfelden)
	SPORTPLATZ_6 = 'de:08116:6708:0:3'  # Rtg Schlaitdorf Rathaus (Altenriet Altenriet)
	SPORTPLATZ_7 = 'de:08118:6890:0:3'  # Rtg Kirchheim (Gemmrigheim Gemmrigheim)
	SPORTPLATZ_8 = 'de:08118:6890:0:4'  # Rtg Gemmrigheim (Gemmrigheim Gemmrigheim)
	SPORTPLATZ_9 = 'de:08118:7598:0:3'  # eRtg Post (Löchgau Löchgau)
	SPORTPLATZ_10 = 'de:08117:4914:0:H1'  # Rtg Göpp. Lerchenbg. (Göppingen Bartenbach (Göppingen))
	SPORTPLATZ_11 = 'de:08117:4914:0:R1'  # Rtg Hofhalde (Göppingen Bartenbach (Göppingen))
	SPORTPLATZ_12 = 'de:08117:7642:0:H1'  # Rtg Wißgoldingen (Donzdorf Winzingen)
	SPORTPLATZ_13 = 'de:08117:7642:0:R1'  # Rtg Reichenbach u.R. (Donzdorf Winzingen)
	FLACHT_ORTSMITTE = 'de:08115:4566'
	FLACHT_ORTSMITTE_1 = 'de:08115:4566:0:3'  # eRtg Sportplatz (Weissach Flacht)
	FLACHT_ORTSMITTE_2 = 'de:08115:4566:0:4'  # eRtg Hohweg (Weissach Flacht)
	HOHWEG = 'de:08115:4567'
	HOHWEG_1 = 'de:08115:4567:0:3'  # eRtg Flacht Ortsmitte (Weissach Flacht)
	HOHWEG_2 = 'de:08115:4567:0:4'  # eRtg Bachstraße (Weissach Flacht)
	FLACHT_HOHWEG = 'de:08115:4567'
	FLACHT_HOHWEG_1 = 'de:08115:4567:0:3'  # eRtg Flacht Ortsmitte (Weissach Flacht)
	FLACHT_HOHWEG_2 = 'de:08115:4567:0:4'  # eRtg Bachstraße (Weissach Flacht)
	WEISSACH_FLACHTER_STR = 'de:08115:4568'
	WEISSACH_FLACHTER_STR_1 = 'de:08115:4568:0:3'  # eRtg Strudelbachhalle (Weissach Weissach (BB))
	WEISSACH_FLACHTER_STR_2 = 'de:08115:4568:0:4'  # Rtg Flacht Hohweg (Weissach Weissach (BB))
	EISGASSE = 'de:08118:4570'
	EISGASSE_1 = 'de:08118:4570:0:3'  # Rtg Hemmingen Bf (Hemmingen Hemmingen)
	EISGASSE_2 = 'de:08118:4570:0:4'  # Rtg Schöckinger Str./Rathaus (Hemmingen Hemmingen)
	HEMMINGEN_EISGASSE = 'de:08118:4570'
	HEMMINGEN_EISGASSE_1 = 'de:08118:4570:0:3'  # Rtg Hemmingen Bf (Hemmingen Hemmingen)
	HEMMINGEN_EISGASSE_2 = 'de:08118:4570:0:4'  # Rtg Schöckinger Str./Rathaus (Hemmingen Hemmingen)
	WEISSACH_HINDENBURGSTR = 'de:08115:4571'
	WEISSACH_HINDENBURGSTR_1 = 'de:08115:4571:0:3'  # Rtg Leonberg (Weissach Weissach (BB))
	WEISSACH_HINDENBURGSTR_2 = 'de:08115:4571:0:4'  # Rtg Porsche (Weissach Weissach (BB))
	AM_SCHLOSSBERG = 'de:08115:4572'
	AM_SCHLOSSBERG_1 = 'de:08115:4572:0:3'  # Rtg Rathaus (Leonberg Höfingen)
	AM_SCHLOSSBERG_2 = 'de:08115:4572:0:4'  # Rtg Leonberg (Leonberg Höfingen)
	HÖFINGEN_AM_SCHLOSSBERG = 'de:08115:4572'
	HÖFINGEN_AM_SCHLOSSBERG_1 = 'de:08115:4572:0:3'  # Rtg Rathaus (Leonberg Höfingen)
	HÖFINGEN_AM_SCHLOSSBERG_2 = 'de:08115:4572:0:4'  # Rtg Leonberg (Leonberg Höfingen)
	WEIL_DER_STADT_STADTGARTEN = 'de:08115:4574'
	WEIL_DER_STADT_STADTGARTEN_1 = 'de:08115:4574:0:3'  # eRtg Weil der Stadt (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_STADTGARTEN_2 = 'de:08115:4574:0:4'  # eRtg Grabenstr./Merklinger Str./Gymn. (Weil der Stadt Weil der Stadt)
	STADTGARTEN = 'de:08115:4574'
	STADTGARTEN_1 = 'de:08115:4574:0:3'  # eRtg Weil der Stadt (Weil der Stadt Weil der Stadt)
	STADTGARTEN_2 = 'de:08115:4574:0:4'  # eRtg Grabenstr./Merklinger Str./Gymn. (Weil der Stadt Weil der Stadt)
	MERKLINGER_STRASSE_1 = 'de:08115:4575:0:3'  # eRtg Metzelwiesen (Weil der Stadt Weil der Stadt)
	MERKLINGER_STRASSE_2 = 'de:08115:4575:0:4'  # eRtg Stadtgarten (Weil der Stadt Weil der Stadt)
	MERKLINGER_STRASSE_3 = 'de:08425:2634:0:1'  # Rtg Merklingen (Laichingen Machtolsheim)
	MERKLINGER_STRASSE_4 = 'de:08425:2634:0:2'  # Rtg Bühlenhausen (Laichingen Machtolsheim)
	WEIL_DER_STADT_MERKLINGER_STR = 'de:08115:4575'
	WEIL_DER_STADT_MERKLINGER_STR_1 = 'de:08115:4575:0:3'  # eRtg Metzelwiesen (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_MERKLINGER_STR_2 = 'de:08115:4575:0:4'  # eRtg Stadtgarten (Weil der Stadt Weil der Stadt)
	NEUHAUSER_STRASSE = 'de:08115:4577:0:3'  # Rtg Rathaus (Weil der Stadt Münklingen)
	MÜNKLINGEN_NEUHAUSER_STR = 'de:08115:4577:0:3'  # Rtg Rathaus (Weil der Stadt Münklingen)
	NEUENBÜHL = 'de:08115:4578'
	NEUENBÜHL_1 = 'de:08115:4578:0:3'  # eRtg Hauptstraße (Kreisel) (Weissach Flacht)
	NEUENBÜHL_2 = 'de:08115:4578:0:4'  # eRtg Sportplatz (Weissach Flacht)
	FLACHT_NEUENBÜHL = 'de:08115:4578'
	FLACHT_NEUENBÜHL_1 = 'de:08115:4578:0:3'  # eRtg Hauptstraße (Kreisel) (Weissach Flacht)
	FLACHT_NEUENBÜHL_2 = 'de:08115:4578:0:4'  # eRtg Sportplatz (Weissach Flacht)
	SCHAFHAUSEN_HOHENZOLLERNSTR = 'de:08115:4579:0:3'  # eRtg Magstadter Straße (Weil der Stadt Schafhausen)
	WEIL_DER_STADT_GRABENSTR = 'de:08115:4581'
	WEIL_DER_STADT_GRABENSTR_1 = 'de:08115:4581:0:3'  # eRtg Stadtgarten (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_GRABENSTR_2 = 'de:08115:4581:0:4'  # eRtg Würmbrücke (Weil der Stadt Weil der Stadt)
	SCHAFHAUSEN_WÜRMBRÜCKE = 'de:08115:4582'
	SCHAFHAUSEN_WÜRMBRÜCKE_1 = 'de:08115:4582:0:3'  # eRtg Grabenstraße (Weil der Stadt Schafhausen)
	SCHAFHAUSEN_WÜRMBRÜCKE_2 = 'de:08115:4582:0:4'  # eRtg Kreuzstr./Magstadter Str. (Weil der Stadt Schafhausen)
	WÜRMBRÜCKE_1 = 'de:08115:4582:0:3'  # eRtg Grabenstraße (Weil der Stadt Schafhausen)
	WÜRMBRÜCKE_2 = 'de:08115:4582:0:4'  # eRtg Kreuzstr./Magstadter Str. (Weil der Stadt Schafhausen)
	WÜRMBRÜCKE_3 = 'de:08115:4756:0:3'  # eRtg Königsberger Straße (Ehningen Ehningen)
	WÜRMBRÜCKE_4 = 'de:08115:4756:0:4'  # eRtg Ehningen (Ehningen Ehningen)
	SCHAFHAUSEN_WEILER__BB = 'de:08115:4583:0:3'  # eRtg Hohenzollernstr/Magstadter Straße (Weil der Stadt Schafhausen)
	WEILER__BB = 'de:08115:4583:0:3'  # eRtg Hohenzollernstr/Magstadter Straße (Weil der Stadt Schafhausen)
	SCHAFHAUSEN_MAGSTADTER_STR = 'de:08115:4584'
	SCHAFHAUSEN_MAGSTADTER_STR_1 = 'de:08115:4584:0:3'  # eRtg Schafhausen Weiler/Rathaus (Weil der Stadt Schafhausen)
	SCHAFHAUSEN_MAGSTADTER_STR_2 = 'de:08115:4584:0:4'  # eRtg Würmbrücke (Weil der Stadt Schafhausen)
	MAGSTADTER_STRASSE = 'de:08115:4584'
	MAGSTADTER_STRASSE_1 = 'de:08115:4584:0:3'  # eRtg Schafhausen Weiler/Rathaus (Weil der Stadt Schafhausen)
	MAGSTADTER_STRASSE_2 = 'de:08115:4584:0:4'  # eRtg Würmbrücke (Weil der Stadt Schafhausen)
	DÄTZINGEN_RATHAUS = 'de:08115:4585'
	DÄTZINGEN_RATHAUS_1 = 'de:08115:4585:0:3'  # eRtg Dätzinger Weg (Grafenau Dätzingen)
	DÄTZINGEN_RATHAUS_2 = 'de:08115:4585:0:4'  # eRtg Schule (Grafenau Dätzingen)
	WEIL_DER_STADT_FRIEDHOF = 'de:08115:4586'
	WEIL_DER_STADT_FRIEDHOF_1 = 'de:08115:4586:0:1'  # Rtg Renningen Ihninger Hof (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_FRIEDHOF_2 = 'de:08115:4586:0:2'  # Rtg Bahnhof (Weil der Stadt Weil der Stadt)
	MERKLINGEN_HAUPTSTRASSE = 'de:08115:4587'
	MERKLINGEN_HAUPTSTRASSE_1 = 'de:08115:4587:0:3'  # eRtg Ortsmitte (Weil der Stadt Merklingen)
	MERKLINGEN_HAUPTSTRASSE_2 = 'de:08115:4587:0:4'  # eRtg Metzelwiesen (Weil der Stadt Merklingen)
	MERKLINGEN_ORTSMITTE_1 = 'de:08115:4588:1:3'  # eRtg Industriestraße (Weil der Stadt Merklingen)
	MERKLINGEN_ORTSMITTE_2 = 'de:08115:4588:1:4'  # eRtg Hauptstraße (Weil der Stadt Merklingen)
	MERKLINGEN_ORTSMITTE_3 = 'de:08425:2632:0:2'  # einwärts (Merklingen Merklingen)
	MERKLINGEN_BEETHOVENSTR = 'de:08115:4589'
	MERKLINGEN_BEETHOVENSTR_1 = 'de:08115:4589:0:3'  # Rtg Hausener Str. (Weil der Stadt Merklingen)
	MERKLINGEN_BEETHOVENSTR_2 = 'de:08115:4589:0:4'  # eRtg Ortsmitte (Weil der Stadt Merklingen)
	HAUSENER_STRASSE = 'de:08115:4590'
	HAUSENER_STRASSE_1 = 'de:08115:4590:0:3'  # eRtg Würmtalstraße (Weil der Stadt Merklingen)
	HAUSENER_STRASSE_2 = 'de:08115:4590:0:4'  # eRtg Ortsmitte (Weil der Stadt Merklingen)
	MERKLINGEN_HAUSENER_STR = 'de:08115:4590'
	MERKLINGEN_HAUSENER_STR_1 = 'de:08115:4590:0:3'  # eRtg Würmtalstraße (Weil der Stadt Merklingen)
	MERKLINGEN_HAUSENER_STR_2 = 'de:08115:4590:0:4'  # eRtg Ortsmitte (Weil der Stadt Merklingen)
	MÜNKLINGEN_IM_TAL = 'de:08115:4591'
	MÜNKLINGEN_IM_TAL_1 = 'de:08115:4591:0:3'  # Rtg Riemenmühle (Weil der Stadt Münklingen)
	MÜNKLINGEN_IM_TAL_2 = 'de:08115:4591:0:4'  # Rtg Rathaus/Neuh.Str. (Weil der Stadt Münklingen)
	IM_TAL = 'de:08115:4591'
	IM_TAL_1 = 'de:08115:4591:0:3'  # Rtg Riemenmühle (Weil der Stadt Münklingen)
	IM_TAL_2 = 'de:08115:4591:0:4'  # Rtg Rathaus/Neuh.Str. (Weil der Stadt Münklingen)
	MÜNKLINGEN_RATHAUS = 'de:08115:4592'
	MÜNKLINGEN_RATHAUS_1 = 'de:08115:4592:0:3'  # Rtg Im Tal (Weil der Stadt Münklingen)
	MÜNKLINGEN_RATHAUS_2 = 'de:08115:4592:0:4'  # Rtg Neuhauser Str. (Weil der Stadt Münklingen)
	WÜRMTALSTRASSE = 'de:08115:4593'
	WÜRMTALSTRASSE_1 = 'de:08115:4593:1:3'  # Rtg Münkl./Weil d. S. (Weil der Stadt Hausen (Weil der Stadt))
	WÜRMTALSTRASSE_2 = 'de:08115:4593:1:4'  # Rtg Heimsheim/Lehningen (Weil der Stadt Hausen (Weil der Stadt))
	HAUSEN__W__WÜRMTALSTRASSE = 'de:08115:4593'
	HAUSEN__W__WÜRMTALSTRASSE_1 = 'de:08115:4593:1:3'  # Rtg Münkl./Weil d. S. (Weil der Stadt Hausen (Weil der Stadt))
	HAUSEN__W__WÜRMTALSTRASSE_2 = 'de:08115:4593:1:4'  # Rtg Heimsheim/Lehningen (Weil der Stadt Hausen (Weil der Stadt))
	RINGSTRASSE_1 = 'de:08115:4596:0:3'  # Rtg Hutwiesenstraße Am Schattwald (Magstadt Magstadt)
	RINGSTRASSE_2 = 'de:08115:4596:0:4'  # Rtg Marktplatz Brühlstr 22 (Magstadt Magstadt)
	RINGSTRASSE_3 = 'de:08119:5210:0:3'  # eRtg Kelter/Stöckachschule (Winnenden Winnenden)
	RINGSTRASSE_4 = 'de:08118:5916:0:3'  # Rtg Gerlingen (Gerlingen Gerlingen)
	RINGSTRASSE_5 = 'de:08118:5916:0:4'  # Rtg Weissach (Gerlingen Gerlingen)
	RINGSTRASSE_6 = 'de:08117:6021:0:H1'  # Rtg Ottenbach (Salach Salach)
	RINGSTRASSE_7 = 'de:08117:6021:0:R1'  # Rtg Eislingen (Salach Salach)
	MAGSTADT_RINGSTRASSE = 'de:08115:4596'
	MAGSTADT_RINGSTRASSE_1 = 'de:08115:4596:0:3'  # Rtg Hutwiesenstraße Am Schattwald (Magstadt Magstadt)
	MAGSTADT_RINGSTRASSE_2 = 'de:08115:4596:0:4'  # Rtg Marktplatz Brühlstr 22 (Magstadt Magstadt)
	BRÜHLSTRASSE_22 = 'de:08115:4600'
	BRÜHLSTRASSE_22_1 = 'de:08115:4600:0:3'  # eRtg Ringstraße (Magstadt Magstadt)
	BRÜHLSTRASSE_22_2 = 'de:08115:4600:0:4'  # eRtg Kniebisstraße (Magstadt Magstadt)
	MAGSTADT_BRÜHLSTR__22 = 'de:08115:4600'
	MAGSTADT_BRÜHLSTR__22_1 = 'de:08115:4600:0:3'  # eRtg Ringstraße (Magstadt Magstadt)
	MAGSTADT_BRÜHLSTR__22_2 = 'de:08115:4600:0:4'  # eRtg Kniebisstraße (Magstadt Magstadt)
	WARMBRONN_BÜSNAUER_STR__6 = 'de:08115:4601:0:3'  # Rtg Frauenkreuz (Leonberg Warmbronn)
	BÜSNAUER_STRASSE_6 = 'de:08115:4601:0:3'  # Rtg Frauenkreuz (Leonberg Warmbronn)
	BÜSNAUER_STRASSE_41 = 'de:08115:4602'
	BÜSNAUER_STRASSE_41_1 = 'de:08115:4602:1:3'  # Rtg Sindelfingen/Stuttgart (Leonberg Warmbronn)
	BÜSNAUER_STRASSE_41_2 = 'de:08115:4602:1:4'  # Rtg Büsnauer Str. 6 (Leonberg Warmbronn)
	WARMBRONN_BÜSNAUER_STR__41 = 'de:08115:4602'
	WARMBRONN_BÜSNAUER_STR__41_1 = 'de:08115:4602:1:3'  # Rtg Sindelfingen/Stuttgart (Leonberg Warmbronn)
	WARMBRONN_BÜSNAUER_STR__41_2 = 'de:08115:4602:1:4'  # Rtg Büsnauer Str. 6 (Leonberg Warmbronn)
	EICHHOLZ = 'de:08115:4603:0:3'  # eRtg Friedrich-Ebert-Straße (Sindelfingen Sindelfingen)
	SINDELF__EICHHOLZ = 'de:08115:4603:0:3'  # eRtg Friedrich-Ebert-Straße (Sindelfingen Sindelfingen)
	FRIEDRICH_EBERT_STRASSE_1 = 'de:08115:4604:0:3'  # eRtg Eichholz (Sindelfingen Sindelfingen)
	FRIEDRICH_EBERT_STRASSE_2 = 'de:08115:4604:0:4'  # eRtg Eschenried (Sindelfingen Sindelfingen)
	FRIEDRICH_EBERT_STRASSE_3 = 'de:08118:7440:0:3'  # eRtg Grünwiesenstraße (Bietigheim-Bissingen Bietigheim)
	FRIEDRICH_EBERT_STRASSE_4 = 'de:08118:7440:0:4'  # eRtg Bolzstraße (Bietigheim-Bissingen Bietigheim)
	SINDELF__FRIEDRICH_EBERT_STR = 'de:08115:4604'
	SINDELF__FRIEDRICH_EBERT_STR_1 = 'de:08115:4604:0:3'  # eRtg Eichholz (Sindelfingen Sindelfingen)
	SINDELF__FRIEDRICH_EBERT_STR_2 = 'de:08115:4604:0:4'  # eRtg Eschenried (Sindelfingen Sindelfingen)
	ESCHENRIED = 'de:08115:4605'
	ESCHENRIED_1 = 'de:08115:4605:1:3'  # eRtg Friedrich-Ebert-Straße (Sindelfingen Sindelfingen)
	ESCHENRIED_2 = 'de:08115:4605:1:4'  # eRtg Feldbergstraße (Sindelfingen Sindelfingen)
	SINDELF__ESCHENRIED = 'de:08115:4605'
	SINDELF__ESCHENRIED_1 = 'de:08115:4605:1:3'  # eRtg Friedrich-Ebert-Straße (Sindelfingen Sindelfingen)
	SINDELF__ESCHENRIED_2 = 'de:08115:4605:1:4'  # eRtg Feldbergstraße (Sindelfingen Sindelfingen)
	KLOSTERGARTEN = 'de:08115:4607'
	KLOSTERGARTEN_1 = 'de:08115:4607:0:3'  # Rtg Rotbühl (Sindelfingen Sindelfingen)
	KLOSTERGARTEN_2 = 'de:08115:4607:0:4'  # Rtg Domo (Sindelfingen Sindelfingen)
	KLOSTERGARTEN_3 = 'de:08115:4607:0:5'  # Rtg Nord (Sindelfingen Sindelfingen)
	SINDELF__KLOSTERGARTEN = 'de:08115:4607'
	SINDELF__KLOSTERGARTEN_1 = 'de:08115:4607:0:3'  # Rtg Rotbühl (Sindelfingen Sindelfingen)
	SINDELF__KLOSTERGARTEN_2 = 'de:08115:4607:0:4'  # Rtg Domo (Sindelfingen Sindelfingen)
	SINDELF__KLOSTERGARTEN_3 = 'de:08115:4607:0:5'  # Rtg Nord (Sindelfingen Sindelfingen)
	DARMSHEIM_ROBERT_BOSCH_STR = 'de:08115:4608:1:3'  # eRtg Tessiner Str. (Sindelfingen Darmsheim)
	SINDELFINGEN_DOMO = 'de:08115:4609'
	SINDELFINGEN_DOMO_1 = 'de:08115:4609:0:3'  # eRtg Obere Vorstadt (Sindelfingen Sindelfingen)
	SINDELFINGEN_DOMO_2 = 'de:08115:4609:0:4'  # eRtg Stadion (Sindelfingen Sindelfingen)
	DOMO = 'de:08115:4609'
	DOMO_1 = 'de:08115:4609:0:3'  # eRtg Obere Vorstadt (Sindelfingen Sindelfingen)
	DOMO_2 = 'de:08115:4609:0:4'  # eRtg Stadion (Sindelfingen Sindelfingen)
	SINDELFINGEN_BACHSTR = 'de:08115:4610:0:4'  # eRtg Mercedes-Benz Tor III/Calwer Str. (Sindelfingen Sindelfingen)
	BACHSTRASSE_1 = 'de:08115:4610:0:4'  # eRtg Mercedes-Benz Tor III/Calwer Str. (Sindelfingen Sindelfingen)
	BACHSTRASSE_2 = 'de:08115:4795:0:3'  # eRtg Holzgerlinger Straße (Altdorf Altdorf (BB))
	BACHSTRASSE_3 = 'de:08118:5606:0:3'  # eRtg Aussiedlerhöfe (Großbottwar Großbottwar)
	BACHSTRASSE_4 = 'de:08118:5606:0:4'  # eRtg Rosenplatz (Großbottwar Großbottwar)
	BACHSTRASSE_5 = 'de:08115:5912:0:3'  # eRtg Marktplatz (Weissach Weissach (BB))
	BACHSTRASSE_6 = 'de:08117:3020:0:H1'  # Rtg Kirche (Schlat Schlat)
	CALWER_STRASSE_1 = 'de:08115:4611:0:3'  # eRtg ZOB (Sindelfingen Sindelfingen)
	CALWER_STRASSE_2 = 'de:08115:4611:0:4'  # eRtg Bachstr. (Sindelfingen Sindelfingen)
	CALWER_STRASSE_3 = 'de:08115:4620:0:3'  # eRtg Böblingen (Böblingen Böblingen)
	CALWER_STRASSE_4 = 'de:08115:4620:0:4'  # eRtg Dagersheim (Böblingen Böblingen)
	CALWER_STRASSE_5 = 'de:08115:4840:0:3'  # eRtg Tübinger Straße (Herrenberg Oberjesingen)
	CALWER_STRASSE_6 = 'de:08115:4840:0:4'  # eRtg Zaberstr. (Herrenberg Oberjesingen)
	SINDELFINGEN_CALWER_STR = 'de:08115:4611'
	SINDELFINGEN_CALWER_STR_1 = 'de:08115:4611:0:3'  # eRtg ZOB (Sindelfingen Sindelfingen)
	SINDELFINGEN_CALWER_STR_2 = 'de:08115:4611:0:4'  # eRtg Bachstr. (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_TOR_I = 'de:08115:4612'
	MERCEDES_BENZ_TOR_I_1 = 'de:08115:4612:0:3'  # eRtg Mercedes-Benz Hst. F (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_TOR_I_2 = 'de:08115:4612:0:4'  # eRtg Goldberg-Gymnasium (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_TOR_I = 'de:08115:4612'
	SINDELF__MERCEDES_BENZ_TOR_I_1 = 'de:08115:4612:0:3'  # eRtg Mercedes-Benz Hst. F (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_TOR_I_2 = 'de:08115:4612:0:4'  # eRtg Goldberg-Gymnasium (Sindelfingen Sindelfingen)
	GOLDBERG_WASSERTURM = 'de:08115:4614'
	GOLDBERG_WASSERTURM_1 = 'de:08115:4614:0:3'  # 704/709 (Sindelfingen Sindelfingen)
	GOLDBERG_WASSERTURM_2 = 'de:08115:4614:0:4'  # 711/N73 (Sindelfingen Sindelfingen)
	SINDELF__GOLDB__WASSERTURM = 'de:08115:4614'
	SINDELF__GOLDB__WASSERTURM_1 = 'de:08115:4614:0:3'  # 704/709 (Sindelfingen Sindelfingen)
	SINDELF__GOLDB__WASSERTURM_2 = 'de:08115:4614:0:4'  # 711/N73 (Sindelfingen Sindelfingen)
	SINDELFINGEN_LEIPZIGER_STR = 'de:08115:4615'
	SINDELFINGEN_LEIPZIGER_STR_1 = 'de:08115:4615:2:3'  # eRtg Eschenbrünnlestr. (Sindelfingen Sindelfingen)
	SINDELFINGEN_LEIPZIGER_STR_2 = 'de:08115:4615:2:4'  # eRtg Goldberg (Sindelfingen Sindelfingen)
	LEIPZIGER_STRASSE = 'de:08115:4615'
	LEIPZIGER_STRASSE_1 = 'de:08115:4615:2:3'  # eRtg Eschenbrünnlestr. (Sindelfingen Sindelfingen)
	LEIPZIGER_STRASSE_2 = 'de:08115:4615:2:4'  # eRtg Goldberg (Sindelfingen Sindelfingen)
	BÖBLINGEN_ABZW__RAUHER_KAPF = 'de:08115:4617'
	BÖBLINGEN_ABZW__RAUHER_KAPF_1 = 'de:08115:4617:0:3'  # eRtg Schönaicher First/Breitenst Str. (Böblingen Böblingen)
	BÖBLINGEN_ABZW__RAUHER_KAPF_2 = 'de:08115:4617:0:4'  # eRtg Kelterweg (Böblingen Böblingen)
	ABZWEIG_RAUHER_KAPF = 'de:08115:4617'
	ABZWEIG_RAUHER_KAPF_1 = 'de:08115:4617:0:3'  # eRtg Schönaicher First/Breitenst Str. (Böblingen Böblingen)
	ABZWEIG_RAUHER_KAPF_2 = 'de:08115:4617:0:4'  # eRtg Kelterweg (Böblingen Böblingen)
	BÖBLINGEN_TALSTRASSE = 'de:08115:4619:0:3'  # eRtg Hanns-Klemm-Straße Ost (Böblingen Böblingen)
	BÖBLINGEN_CALWER_STR = 'de:08115:4620'
	BÖBLINGEN_CALWER_STR_1 = 'de:08115:4620:0:3'  # eRtg Böblingen (Böblingen Böblingen)
	BÖBLINGEN_CALWER_STR_2 = 'de:08115:4620:0:4'  # eRtg Dagersheim (Böblingen Böblingen)
	CALWER_STRASSE__HULB = 'de:08115:4621'
	CALWER_STRASSE__HULB_1 = 'de:08115:4621:0:3'  # Rtg Dagersheim (Böblingen Böblingen)
	CALWER_STRASSE__HULB_2 = 'de:08115:4621:0:4'  # Rtg Böblingen (Böblingen Böblingen)
	BÖBLINGEN_CALWER_STR___HULB = 'de:08115:4621'
	BÖBLINGEN_CALWER_STR___HULB_1 = 'de:08115:4621:0:3'  # Rtg Dagersheim (Böblingen Böblingen)
	BÖBLINGEN_CALWER_STR___HULB_2 = 'de:08115:4621:0:4'  # Rtg Böblingen (Böblingen Böblingen)
	CALWER_BRÜCKE = 'de:08115:4624'
	CALWER_BRÜCKE_1 = 'de:08115:4624:0:3'  # Rtg Daimlersteg (Sindelfingen Sindelfingen)
	CALWER_BRÜCKE_2 = 'de:08115:4624:0:4'  # Rtg Darmsheim (Sindelfingen Sindelfingen)
	SINDELF__CALWER_BRÜCKE = 'de:08115:4624'
	SINDELF__CALWER_BRÜCKE_1 = 'de:08115:4624:0:3'  # Rtg Daimlersteg (Sindelfingen Sindelfingen)
	SINDELF__CALWER_BRÜCKE_2 = 'de:08115:4624:0:4'  # Rtg Darmsheim (Sindelfingen Sindelfingen)
	SINDELFINGEN_SOMMERHOFENSTR = 'de:08115:4625'
	SINDELFINGEN_SOMMERHOFENSTR_1 = 'de:08115:4625:0:3'  # eRtg Pfarrwiesengymnasium (Sindelfingen Sindelfingen)
	SINDELFINGEN_SOMMERHOFENSTR_2 = 'de:08115:4625:0:4'  # eRtg Badezentrum (Sindelfingen Sindelfingen)
	SOMMERHOFENSTRASSE = 'de:08115:4625'
	SOMMERHOFENSTRASSE_1 = 'de:08115:4625:0:3'  # eRtg Pfarrwiesengymnasium (Sindelfingen Sindelfingen)
	SOMMERHOFENSTRASSE_2 = 'de:08115:4625:0:4'  # eRtg Badezentrum (Sindelfingen Sindelfingen)
	BADEZENTRUM = 'de:08115:4626'
	BADEZENTRUM_1 = 'de:08115:4626:0:3'  # eRtg Sommerhofenstraße (Sindelfingen Sindelfingen)
	BADEZENTRUM_2 = 'de:08115:4626:0:4'  # eRtg Altenheim Burghalde (Sindelfingen Sindelfingen)
	SINDELF__BADEZENTRUM = 'de:08115:4626'
	SINDELF__BADEZENTRUM_1 = 'de:08115:4626:0:3'  # eRtg Sommerhofenstraße (Sindelfingen Sindelfingen)
	SINDELF__BADEZENTRUM_2 = 'de:08115:4626:0:4'  # eRtg Altenheim Burghalde (Sindelfingen Sindelfingen)
	SINDELF__PFLEGEHEIM = 'de:08115:4627'
	SINDELF__PFLEGEHEIM_1 = 'de:08115:4627:0:3'  # eRtg Krankenhaus (Sindelfingen Sindelfingen)
	SINDELF__PFLEGEHEIM_2 = 'de:08115:4627:0:4'  # eRtg Burghaldenfriedhof (Sindelfingen Sindelfingen)
	ALTENHEIM_BURGHALDE = 'de:08115:4628'
	ALTENHEIM_BURGHALDE_1 = 'de:08115:4628:0:3'  # eRtg Badezentrum (Sindelfingen Sindelfingen)
	ALTENHEIM_BURGHALDE_2 = 'de:08115:4628:0:4'  # eRtg Krankenhaus (Sindelfingen Sindelfingen)
	SINDELF__ALTENHEIM_BURGHALDE = 'de:08115:4628'
	SINDELF__ALTENHEIM_BURGHALDE_1 = 'de:08115:4628:0:3'  # eRtg Badezentrum (Sindelfingen Sindelfingen)
	SINDELF__ALTENHEIM_BURGHALDE_2 = 'de:08115:4628:0:4'  # eRtg Krankenhaus (Sindelfingen Sindelfingen)
	BURGHALDENFRIEDHOF = 'de:08115:4629'
	BURGHALDENFRIEDHOF_1 = 'de:08115:4629:0:3'  # eRtg Pflegeheim (Sindelfingen Sindelfingen)
	BURGHALDENFRIEDHOF_2 = 'de:08115:4629:0:4'  # eRtg Stadthalle (Sindelfingen Sindelfingen)
	SINDELF__BURGHALDENFRIEDHOF = 'de:08115:4629'
	SINDELF__BURGHALDENFRIEDHOF_1 = 'de:08115:4629:0:3'  # eRtg Pflegeheim (Sindelfingen Sindelfingen)
	SINDELF__BURGHALDENFRIEDHOF_2 = 'de:08115:4629:0:4'  # eRtg Stadthalle (Sindelfingen Sindelfingen)
	DAGERSH__HAUS_D__VEREINE = 'de:08115:4630'
	DAGERSH__HAUS_D__VEREINE_1 = 'de:08115:4630:0:3'  # eRtg Hauptstraße (Böblingen Dagersheim)
	DAGERSH__HAUS_D__VEREINE_2 = 'de:08115:4630:0:4'  # eRtg Calwer Straße (Hulb) (Böblingen Dagersheim)
	HAUS_DER_VEREINE = 'de:08115:4630'
	HAUS_DER_VEREINE_1 = 'de:08115:4630:0:3'  # eRtg Hauptstraße (Böblingen Dagersheim)
	HAUS_DER_VEREINE_2 = 'de:08115:4630:0:4'  # eRtg Calwer Straße (Hulb) (Böblingen Dagersheim)
	DAGERSHEIM_HAUPTSTRASSE = 'de:08115:4631'
	DAGERSHEIM_HAUPTSTRASSE_1 = 'de:08115:4631:0:3'  # eRtg Alter Friedhof (Böblingen Dagersheim)
	DAGERSHEIM_HAUPTSTRASSE_2 = 'de:08115:4631:0:4'  # eRtg Haus der Vereine (Böblingen Dagersheim)
	DAGERSHEIM_KIRCHE = 'de:08115:4632'
	DAGERSHEIM_KIRCHE_1 = 'de:08115:4632:1:3'  # eRtg Aidlinger Straße (Böblingen Dagersheim)
	DAGERSHEIM_KIRCHE_2 = 'de:08115:4632:1:4'  # eRtg Schützenweg (Böblingen Dagersheim)
	SCHÜTZENWEG = 'de:08115:4633'
	SCHÜTZENWEG_1 = 'de:08115:4633:0:3'  # eRtg Kirche (Böblingen Dagersheim)
	SCHÜTZENWEG_2 = 'de:08115:4633:0:4'  # eRtg Eichenpfädle (Böblingen Dagersheim)
	DAGERSHEIM_SCHÜTZENWEG = 'de:08115:4633'
	DAGERSHEIM_SCHÜTZENWEG_1 = 'de:08115:4633:0:3'  # eRtg Kirche (Böblingen Dagersheim)
	DAGERSHEIM_SCHÜTZENWEG_2 = 'de:08115:4633:0:4'  # eRtg Eichenpfädle (Böblingen Dagersheim)
	DAGERSH__GOETHESTRASSE = 'de:08115:4635:0:3'  # eRtg Schützenweg (Böblingen Dagersheim)
	SINDELF__GOLDB__REALSCHULE = 'de:08115:4636:0:3'  # eRtg Frankenstraße (Sindelfingen Sindelfingen)
	GOLDBERG_REALSCHULE = 'de:08115:4636:0:3'  # eRtg Frankenstraße (Sindelfingen Sindelfingen)
	ALTER_FRIEDHOF_1 = 'de:08115:4637:0:3'  # eRtg Robert-Bosch-Straße (Sindelfingen Darmsheim)
	ALTER_FRIEDHOF_2 = 'de:08115:4637:0:4'  # Rtg Döffinger Str. (Sindelfingen Darmsheim)
	ALTER_FRIEDHOF_3 = 'de:08115:4686:0:3'  # eRtg Schafgasse (Böblingen Böblingen)
	ALTER_FRIEDHOF_4 = 'de:08115:4686:0:4'  # Rtg Ost (Böblingen Böblingen)
	DARMSHEIM_ALTER_FRIEDHOF = 'de:08115:4637'
	DARMSHEIM_ALTER_FRIEDHOF_1 = 'de:08115:4637:0:3'  # eRtg Robert-Bosch-Straße (Sindelfingen Darmsheim)
	DARMSHEIM_ALTER_FRIEDHOF_2 = 'de:08115:4637:0:4'  # Rtg Döffinger Str. (Sindelfingen Darmsheim)
	DARMSHEIM_HOFSTRASSE = 'de:08115:4638:0:3'  # eRtg Döffinger Straße (Sindelfingen Darmsheim)
	TESSINER_STRASSE = 'de:08115:4640:0:3'  # eRtg Hofstraße (Sindelfingen Darmsheim)
	DARMSHEIM_TESSINER_STR = 'de:08115:4640:0:3'  # eRtg Hofstraße (Sindelfingen Darmsheim)
	SINDELFINGEN_SCHOSSHÖFE = 'de:08115:4641'
	SINDELFINGEN_SCHOSSHÖFE_1 = 'de:08115:4641:0:3'  # Rtg Calwer Brücke (Sindelfingen Sindelfingen)
	SINDELFINGEN_SCHOSSHÖFE_2 = 'de:08115:4641:0:4'  # Rtg Dagersheim (Sindelfingen Sindelfingen)
	SCHOSSHÖFE = 'de:08115:4641'
	SCHOSSHÖFE_1 = 'de:08115:4641:0:3'  # Rtg Calwer Brücke (Sindelfingen Sindelfingen)
	SCHOSSHÖFE_2 = 'de:08115:4641:0:4'  # Rtg Dagersheim (Sindelfingen Sindelfingen)
	MAICHINGEN_SINDELFINGER_STR = 'de:08115:4642'
	MAICHINGEN_SINDELFINGER_STR_1 = 'de:08115:4642:0:3'  # eRtg Maichingen (Sindelfingen Maichingen)
	MAICHINGEN_SINDELFINGER_STR_2 = 'de:08115:4642:0:4'  # eRtg Mercedes-Benz Tor V (Sindelfingen Maichingen)
	HERRENBERGER_STRASSE_1 = 'de:08115:4643:0:3'  # Rtg Hallenbad (Sindelfingen Maichingen)
	HERRENBERGER_STRASSE_2 = 'de:08115:4643:0:4'  # Rtg Sindelfinger Str. (Sindelfingen Maichingen)
	HERRENBERGER_STRASSE_3 = 'de:08115:4716:1:3'  # Rtg Kreistierheim (Böblingen Böblingen)
	HERRENBERGER_STRASSE_4 = 'de:08115:4716:1:4'  # Rtg Kremser Str. (Böblingen Böblingen)
	HERRENBERGER_STRASSE_5 = 'de:08115:4832:0:3'  # Rtg Rathaus (Herrenberg Kayh)
	HERRENBERGER_STRASSE_6 = 'de:08115:4832:0:4'  # Rtg Mönchberg Abzweig (Herrenberg Kayh)
	HERRENBERGER_STRASSE_7 = 'de:08115:7016:0:3'  # Rtg Gemeindezentrum Sindlinger Straße (Jettingen Oberjettingen)
	HERRENBERGER_STRASSE_8 = 'de:08115:7016:0:4'  # Rtg Öfele Herrenberg (Jettingen Oberjettingen)
	MAICHINGEN_HERRENBERGER_STR = 'de:08115:4643'
	MAICHINGEN_HERRENBERGER_STR_1 = 'de:08115:4643:0:3'  # Rtg Hallenbad (Sindelfingen Maichingen)
	MAICHINGEN_HERRENBERGER_STR_2 = 'de:08115:4643:0:4'  # Rtg Sindelfinger Str. (Sindelfingen Maichingen)
	GUTTENBRUNNSTRASSE = 'de:08115:4644'
	GUTTENBRUNNSTRASSE_1 = 'de:08115:4644:0:3'  # Rtg Gottscheer Straße (Sindelfingen Sindelfingen)
	GUTTENBRUNNSTRASSE_2 = 'de:08115:4644:0:4'  # Rtg Maichingen (Sindelfingen Sindelfingen)
	SINDELFINGEN_GUTTENBRUNNSTR = 'de:08115:4644'
	SINDELFINGEN_GUTTENBRUNNSTR_1 = 'de:08115:4644:0:3'  # Rtg Gottscheer Straße (Sindelfingen Sindelfingen)
	SINDELFINGEN_GUTTENBRUNNSTR_2 = 'de:08115:4644:0:4'  # Rtg Maichingen (Sindelfingen Sindelfingen)
	SINDELFINGEN_ESSEGGER_STR = 'de:08115:4645'
	SINDELFINGEN_ESSEGGER_STR_1 = 'de:08115:4645:1:3'  # eRtg Wolfstraße/ZOB (Sindelfingen Sindelfingen)
	SINDELFINGEN_ESSEGGER_STR_2 = 'de:08115:4645:1:4'  # eRtg Guttenbrunnstraße (Sindelfingen Sindelfingen)
	ESSEGGER_STRASSE = 'de:08115:4645'
	ESSEGGER_STRASSE_1 = 'de:08115:4645:1:3'  # eRtg Wolfstraße/ZOB (Sindelfingen Sindelfingen)
	ESSEGGER_STRASSE_2 = 'de:08115:4645:1:4'  # eRtg Guttenbrunnstraße (Sindelfingen Sindelfingen)
	TULPENSTRASSE = 'de:08115:4647'
	TULPENSTRASSE_1 = 'de:08115:4647:0:3'  # eRtg Stadion (Sindelfingen Sindelfingen)
	TULPENSTRASSE_2 = 'de:08115:4647:0:4'  # eRtg Essegger Straße (Sindelfingen Sindelfingen)
	SINDELFINGEN_TULPENSTR = 'de:08115:4647'
	SINDELFINGEN_TULPENSTR_1 = 'de:08115:4647:0:3'  # eRtg Stadion (Sindelfingen Sindelfingen)
	SINDELFINGEN_TULPENSTR_2 = 'de:08115:4647:0:4'  # eRtg Essegger Straße (Sindelfingen Sindelfingen)
	WOLFSTRASSE_ALTENHEIM = 'de:08115:4649'
	WOLFSTRASSE_ALTENHEIM_1 = 'de:08115:4649:0:3'  # Rtg Essegger Str. (Sindelfingen Sindelfingen)
	WOLFSTRASSE_ALTENHEIM_2 = 'de:08115:4649:0:4'  # Rtg Obere Vorstadt (Sindelfingen Sindelfingen)
	SINDELF__WOLFSTR__ALTENHEIM = 'de:08115:4649'
	SINDELF__WOLFSTR__ALTENHEIM_1 = 'de:08115:4649:0:3'  # Rtg Essegger Str. (Sindelfingen Sindelfingen)
	SINDELF__WOLFSTR__ALTENHEIM_2 = 'de:08115:4649:0:4'  # Rtg Obere Vorstadt (Sindelfingen Sindelfingen)
	ESCHENBRÜNNLESTRASSE = 'de:08115:4652'
	ESCHENBRÜNNLESTRASSE_1 = 'de:08115:4652:0:3'  # eRtg Hofmeister/Schauwerk (Sindelfingen Sindelfingen)
	ESCHENBRÜNNLESTRASSE_2 = 'de:08115:4652:0:4'  # eRtg Leipziger Straße (Sindelfingen Sindelfingen)
	SINDELFINGEN_ESCHENBRÜNNLESTR = 'de:08115:4652'
	SINDELFINGEN_ESCHENBRÜNNLESTR_1 = 'de:08115:4652:0:3'  # eRtg Hofmeister/Schauwerk (Sindelfingen Sindelfingen)
	SINDELFINGEN_ESCHENBRÜNNLESTR_2 = 'de:08115:4652:0:4'  # eRtg Leipziger Straße (Sindelfingen Sindelfingen)
	SINDELF__HOFMEISTER_SCHAUWERK = 'de:08115:4653'
	SINDELF__HOFMEISTER_SCHAUWERK_1 = 'de:08115:4653:0:3'  # eRtg Breuningerland (Sindelfingen Sindelfingen)
	SINDELF__HOFMEISTER_SCHAUWERK_2 = 'de:08115:4653:0:4'  # eRtg Eschenbrünnlestraße (Sindelfingen Sindelfingen)
	HOFMEISTER_SCHAUWERK = 'de:08115:4653'
	HOFMEISTER_SCHAUWERK_1 = 'de:08115:4653:0:3'  # eRtg Breuningerland (Sindelfingen Sindelfingen)
	HOFMEISTER_SCHAUWERK_2 = 'de:08115:4653:0:4'  # eRtg Eschenbrünnlestraße (Sindelfingen Sindelfingen)
	TEXTILZENTRUM = 'de:08115:4654'
	TEXTILZENTRUM_1 = 'de:08115:4654:0:3'  # eRtg Viehweide (Sindelfingen Sindelfingen)
	TEXTILZENTRUM_2 = 'de:08115:4654:0:4'  # eRtg Breuningerland (Sindelfingen Sindelfingen)
	SINDELF__TEXTILZENTRUM = 'de:08115:4654'
	SINDELF__TEXTILZENTRUM_1 = 'de:08115:4654:0:3'  # eRtg Viehweide (Sindelfingen Sindelfingen)
	SINDELF__TEXTILZENTRUM_2 = 'de:08115:4654:0:4'  # eRtg Breuningerland (Sindelfingen Sindelfingen)
	DAGERSH__EICHENPFÄDLE = 'de:08115:4655'
	DAGERSH__EICHENPFÄDLE_1 = 'de:08115:4655:0:3'  # eRtg Schützenweg (Böblingen Dagersheim)
	DAGERSH__EICHENPFÄDLE_2 = 'de:08115:4655:0:4'  # eRtg Schoßhöfe (Böblingen Dagersheim)
	EICHENPFÄDLE = 'de:08115:4655'
	EICHENPFÄDLE_1 = 'de:08115:4655:0:3'  # eRtg Schützenweg (Böblingen Dagersheim)
	EICHENPFÄDLE_2 = 'de:08115:4655:0:4'  # eRtg Schoßhöfe (Böblingen Dagersheim)
	SINDELF__LANGE_ANWANDEN = 'de:08115:4656'
	SINDELF__LANGE_ANWANDEN_1 = 'de:08115:4656:0:3'  # eRtg Goldmühlestraße (Sindelfingen Sindelfingen)
	SINDELF__LANGE_ANWANDEN_2 = 'de:08115:4656:0:4'  # eRtg ZOB (Sindelfingen Sindelfingen)
	LANGE_ANWANDEN = 'de:08115:4656'
	LANGE_ANWANDEN_1 = 'de:08115:4656:0:3'  # eRtg Goldmühlestraße (Sindelfingen Sindelfingen)
	LANGE_ANWANDEN_2 = 'de:08115:4656:0:4'  # eRtg ZOB (Sindelfingen Sindelfingen)
	GOLDMÜHLESTRASSE = 'de:08115:4657'
	GOLDMÜHLESTRASSE_1 = 'de:08115:4657:0:3'  # eRtg Nüßstraße (Sindelfingen Sindelfingen)
	GOLDMÜHLESTRASSE_2 = 'de:08115:4657:0:4'  # eRtg Lange Anwanden (Sindelfingen Sindelfingen)
	SINDELFINGEN_GOLDMÜHLESTR = 'de:08115:4657'
	SINDELFINGEN_GOLDMÜHLESTR_1 = 'de:08115:4657:0:3'  # eRtg Nüßstraße (Sindelfingen Sindelfingen)
	SINDELFINGEN_GOLDMÜHLESTR_2 = 'de:08115:4657:0:4'  # eRtg Lange Anwanden (Sindelfingen Sindelfingen)
	NÜSSSTRASSE = 'de:08115:4658'
	NÜSSSTRASSE_1 = 'de:08115:4658:0:3'  # eRtg Goldberg Wasserturm (Sindelfingen Sindelfingen)
	NÜSSSTRASSE_2 = 'de:08115:4658:0:4'  # eRtg Goldmühlestraße (Sindelfingen Sindelfingen)
	SINDELFINGEN_NÜSSSTRASSE = 'de:08115:4658'
	SINDELFINGEN_NÜSSSTRASSE_1 = 'de:08115:4658:0:3'  # eRtg Goldberg Wasserturm (Sindelfingen Sindelfingen)
	SINDELFINGEN_NÜSSSTRASSE_2 = 'de:08115:4658:0:4'  # eRtg Goldmühlestraße (Sindelfingen Sindelfingen)
	SINDELFINGEN_GOTTSCHEER_STR = 'de:08115:4659'
	SINDELFINGEN_GOTTSCHEER_STR_1 = 'de:08115:4659:0:3'  # eRtg Essegger Straße (Sindelfingen Sindelfingen)
	SINDELFINGEN_GOTTSCHEER_STR_2 = 'de:08115:4659:0:4'  # Rtg Maichingen Unterr. Gymn. (Sindelfingen Sindelfingen)
	GOTTSCHEER_STRASSE = 'de:08115:4659'
	GOTTSCHEER_STRASSE_1 = 'de:08115:4659:0:3'  # eRtg Essegger Straße (Sindelfingen Sindelfingen)
	GOTTSCHEER_STRASSE_2 = 'de:08115:4659:0:4'  # Rtg Maichingen Unterr. Gymn. (Sindelfingen Sindelfingen)
	SINDELFINGEN_KÖNIGSKNOLL = 'de:08115:4660'
	SINDELFINGEN_KÖNIGSKNOLL_1 = 'de:08115:4660:0:3'  # Rtg Königsknoll (Sindelfingen Sindelfingen)
	SINDELFINGEN_KÖNIGSKNOLL_2 = 'de:08115:4660:0:4'  # Rtg Sonnenbergstr./Drosselweg (Sindelfingen Sindelfingen)
	KÖNIGSKNOLL = 'de:08115:4660'
	KÖNIGSKNOLL_1 = 'de:08115:4660:0:3'  # Rtg Königsknoll (Sindelfingen Sindelfingen)
	KÖNIGSKNOLL_2 = 'de:08115:4660:0:4'  # Rtg Sonnenbergstr./Drosselweg (Sindelfingen Sindelfingen)
	SINDELFINGEN_SONNENBERGSTR = 'de:08115:4661:0:3'  # eRtg Wilhelm-Haspel-Straße (Sindelfingen Sindelfingen)
	SONNENBERGSTRASSE = 'de:08115:4661:0:3'  # eRtg Wilhelm-Haspel-Straße (Sindelfingen Sindelfingen)
	WILHELM_HASPEL_STRASSE = 'de:08115:4662'
	WILHELM_HASPEL_STRASSE_1 = 'de:08115:4662:0:3'  # eRtg Viehweide (Sindelfingen Sindelfingen)
	WILHELM_HASPEL_STRASSE_2 = 'de:08115:4662:0:4'  # eRtg Eichholz (Sindelfingen Sindelfingen)
	SINDELFINGEN_W__HASPEL_STR = 'de:08115:4662'
	SINDELFINGEN_W__HASPEL_STR_1 = 'de:08115:4662:0:3'  # eRtg Viehweide (Sindelfingen Sindelfingen)
	SINDELFINGEN_W__HASPEL_STR_2 = 'de:08115:4662:0:4'  # eRtg Eichholz (Sindelfingen Sindelfingen)
	SINDELFINGEN_DRK = 'de:08115:4663'
	SINDELFINGEN_DRK_1 = 'de:08115:4663:0:3'  # eRtg Goldberg-Gymnasium (Sindelfingen Sindelfingen)
	SINDELFINGEN_DRK_2 = 'de:08115:4663:0:4'  # eRtg Goldberg (Sindelfingen Sindelfingen)
	SINDELFINGEN_DRK_3 = 'de:08115:4663:0:6'  # Rtg Süd Ostseite (Sindelfingen Sindelfingen)
	DRK = 'de:08115:4663'
	DRK_1 = 'de:08115:4663:0:3'  # eRtg Goldberg-Gymnasium (Sindelfingen Sindelfingen)
	DRK_2 = 'de:08115:4663:0:4'  # eRtg Goldberg (Sindelfingen Sindelfingen)
	DRK_3 = 'de:08115:4663:0:6'  # Rtg Süd Ostseite (Sindelfingen Sindelfingen)
	REITERHOF_1 = 'de:08115:4664:0:3'  # eRtg Textilzentrum (Sindelfingen Sindelfingen)
	REITERHOF_2 = 'de:08115:4691:0:3'  # eRtg Heusteigstraße (Böblingen Böblingen)
	REITERHOF_3 = 'de:08115:4691:0:4'  # eRtg Freudenstädter Straße (Böblingen Böblingen)
	SINDELFINGEN_REITERHOF = 'de:08115:4664:0:3'  # eRtg Textilzentrum (Sindelfingen Sindelfingen)
	VIEHWEIDE = 'de:08115:4665'
	VIEHWEIDE_1 = 'de:08115:4665:1:3'  # eRtg (Sindelfingen Sindelfingen)
	VIEHWEIDE_2 = 'de:08115:4665:1:4'  # eRtg Reiterhof (Sindelfingen Sindelfingen)
	SINDELF__VIEHWEIDE = 'de:08115:4665'
	SINDELF__VIEHWEIDE_1 = 'de:08115:4665:1:3'  # eRtg (Sindelfingen Sindelfingen)
	SINDELF__VIEHWEIDE_2 = 'de:08115:4665:1:4'  # eRtg Reiterhof (Sindelfingen Sindelfingen)
	SINDELF__KRANKENHAUS = 'de:08115:4666:0:3'  # eRtg Altenheim Burghalde (Sindelfingen Sindelfingen)
	BÖBLINGEN_SCHWABSTRASSE = 'de:08115:4667'
	BÖBLINGEN_SCHWABSTRASSE_1 = 'de:08115:4667:0:3'  # eRtg Feldbergstraße (Böblingen Böblingen)
	BÖBLINGEN_SCHWABSTRASSE_2 = 'de:08115:4667:0:4'  # eRtg Postplatz (Böblingen Böblingen)
	SCHWABSTRASSE_1 = 'de:08115:4667:0:3'  # eRtg Feldbergstraße (Böblingen Böblingen)
	SCHWABSTRASSE_2 = 'de:08115:4667:0:4'  # eRtg Postplatz (Böblingen Böblingen)
	SCHWABSTRASSE_3 = 'de:08111:6052:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	SCHWABSTRASSE_4 = 'de:08111:6052:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	SCHWABSTRASSE_5 = 'de:08111:6052:2:1'  # Rtg Schwab-/Reinsburgstr. (Stuttgart Stuttgart)
	SCHWABSTRASSE_6 = 'de:08111:6052:3:2'  # Rtg Bismarckplatz (Stuttgart Stuttgart)
	SCHWABSTRASSE_7 = 'de:08111:6052:4:4'  # Rtg Seyfferstr. (Stuttgart Stuttgart)
	SCHWABSTRASSE_8 = 'de:08111:6052:5:3'  # Rtg Senefelderstr. (Stuttgart Stuttgart)
	SCHWABSTRASSE_9 = 'de:08119:7095:0:3'  # eRtg Friedhof (Waiblingen Waiblingen)
	SCHWABSTRASSE_10 = 'de:08119:7095:0:4'  # eRtg Emil-Münz-Straße (Waiblingen Waiblingen)
	BÖBLINGEN_FELDBERGSTR = 'de:08115:4668:0:3'  # eRtg Eichendorff-Schule (Böblingen Böblingen)
	EICHENDORFF_SCHULE = 'de:08115:4669'
	EICHENDORFF_SCHULE_1 = 'de:08115:4669:0:3'  # eRtg Herdweg (Postamt) (Böblingen Böblingen)
	EICHENDORFF_SCHULE_2 = 'de:08115:4669:0:4'  # Rtg Ost (Böblingen Böblingen)
	BÖBLINGEN_EICHENDORFF_SCHULE = 'de:08115:4669'
	BÖBLINGEN_EICHENDORFF_SCHULE_1 = 'de:08115:4669:0:3'  # eRtg Herdweg (Postamt) (Böblingen Böblingen)
	BÖBLINGEN_EICHENDORFF_SCHULE_2 = 'de:08115:4669:0:4'  # Rtg Ost (Böblingen Böblingen)
	BÖBL__SCHULZENTRUM_MURKENBACH = 'de:08115:4670'
	BÖBL__SCHULZENTRUM_MURKENBACH_1 = 'de:08115:4670:0:3'  # eRtg Albert-Einstein-Gymnasium (Böblingen Böblingen)
	BÖBL__SCHULZENTRUM_MURKENBACH_2 = 'de:08115:4670:0:4'  # eRtg Heusteigstraße (Böblingen Böblingen)
	SCHULZENTRUM_MURKENBACH = 'de:08115:4670'
	SCHULZENTRUM_MURKENBACH_1 = 'de:08115:4670:0:3'  # eRtg Albert-Einstein-Gymnasium (Böblingen Böblingen)
	SCHULZENTRUM_MURKENBACH_2 = 'de:08115:4670:0:4'  # eRtg Heusteigstraße (Böblingen Böblingen)
	BÖBLINGEN_NÜRTINGER_STRASSE = 'de:08115:4671:0:3'  # eRtg Murkenbachweg (Böblingen Böblingen)
	BÖBLINGEN_SCHÖNAICHER_FIRST = 'de:08115:4672'
	BÖBLINGEN_SCHÖNAICHER_FIRST_1 = 'de:08115:4672:1:3'  # eRtg Taunusstraße (Böblingen Böblingen)
	BÖBLINGEN_SCHÖNAICHER_FIRST_2 = 'de:08115:4672:1:4'  # eRtg Hallenbad (Böblingen Böblingen)
	SCHÖNAICHER_FIRST = 'de:08115:4672'
	SCHÖNAICHER_FIRST_1 = 'de:08115:4672:1:3'  # eRtg Taunusstraße (Böblingen Böblingen)
	SCHÖNAICHER_FIRST_2 = 'de:08115:4672:1:4'  # eRtg Hallenbad (Böblingen Böblingen)
	BÖBLINGEN_TAUNUSSTRASSE = 'de:08115:4673'
	BÖBLINGEN_TAUNUSSTRASSE_1 = 'de:08115:4673:0:3'  # eRtg Rauher Kapf (Böblingen Böblingen)
	BÖBLINGEN_TAUNUSSTRASSE_2 = 'de:08115:4673:0:4'  # eRtg Schönaicher First (Böblingen Böblingen)
	TAUNUSSTRASSE = 'de:08115:4673'
	TAUNUSSTRASSE_1 = 'de:08115:4673:0:3'  # eRtg Rauher Kapf (Böblingen Böblingen)
	TAUNUSSTRASSE_2 = 'de:08115:4673:0:4'  # eRtg Schönaicher First (Böblingen Böblingen)
	BÖBLINGEN_RAUHER_KAPF = 'de:08115:4674:0:3'  # eRtg Taunusstr. (Böblingen Böblingen)
	RAUHER_KAPF = 'de:08115:4674:0:3'  # eRtg Taunusstr. (Böblingen Böblingen)
	BÖBLINGEN_WALDBURGSTR = 'de:08115:4675'
	BÖBLINGEN_WALDBURGSTR_1 = 'de:08115:4675:0:3'  # eRtg Elsa-Brändström-Straße (Böblingen Böblingen)
	BÖBLINGEN_WALDBURGSTR_2 = 'de:08115:4675:0:4'  # Rtg Südwest (Böblingen Böblingen)
	BÖBLINGEN_E__BRÄNDSTRÖM_STR = 'de:08115:4676'
	BÖBLINGEN_E__BRÄNDSTRÖM_STR_1 = 'de:08115:4676:0:3'  # eRtg Kreiskrankenhaus (Böblingen Böblingen)
	BÖBLINGEN_E__BRÄNDSTRÖM_STR_2 = 'de:08115:4676:0:4'  # Rtg Südwest (Böblingen Böblingen)
	ELSA_BRÄNDSTRÖM_STRASSE_1 = 'de:08115:4676:0:3'  # eRtg Kreiskrankenhaus (Böblingen Böblingen)
	ELSA_BRÄNDSTRÖM_STRASSE_2 = 'de:08115:4676:0:4'  # Rtg Südwest (Böblingen Böblingen)
	ELSA_BRÄNDSTRÖM_STRASSE_3 = 'de:08115:6817:0:3'  # Endstation/Rtg Lachentor/Mercedesstr. (Leonberg Höfingen)
	BÖBLINGEN_KREISKRANKENHAUS = 'de:08115:4677'
	BÖBLINGEN_KREISKRANKENHAUS_1 = 'de:08115:4677:1:3'  # eRtg Thermalbad (Stuttgarter Str.) (Böblingen Böblingen)
	BÖBLINGEN_KREISKRANKENHAUS_2 = 'de:08115:4677:1:4'  # Rtg West (Böblingen Böblingen)
	KREISKRANKENHAUS = 'de:08115:4677'
	KREISKRANKENHAUS_1 = 'de:08115:4677:1:3'  # eRtg Thermalbad (Stuttgarter Str.) (Böblingen Böblingen)
	KREISKRANKENHAUS_2 = 'de:08115:4677:1:4'  # Rtg West (Böblingen Böblingen)
	ALBERT_EINSTEIN_GYMNASIUM = 'de:08115:4680:0:3'  # eRtg Schulzentrum Murkenbach (Böblingen Böblingen)
	BÖBL__ALBERT_EINSTEIN_GYMN = 'de:08115:4680:0:3'  # eRtg Schulzentrum Murkenbach (Böblingen Böblingen)
	BÖBLINGEN_SILBERWEG = 'de:08115:4683'
	BÖBLINGEN_SILBERWEG_1 = 'de:08115:4683:0:3'  # eRtg Goldberg (Böblingen Böblingen)
	BÖBLINGEN_SILBERWEG_2 = 'de:08115:4683:0:4'  # eRtg Stuttgarter Straße (Böblingen Böblingen)
	SILBERWEG = 'de:08115:4683'
	SILBERWEG_1 = 'de:08115:4683:0:3'  # eRtg Goldberg (Böblingen Böblingen)
	SILBERWEG_2 = 'de:08115:4683:0:4'  # eRtg Stuttgarter Straße (Böblingen Böblingen)
	SCHAFGASSE = 'de:08115:4684'
	SCHAFGASSE_1 = 'de:08115:4684:0:3'  # eRtg Reußensteinstr. (Böblingen Böblingen)
	SCHAFGASSE_2 = 'de:08115:4684:0:4'  # eRtg Klaffensteinstr. (Böblingen Böblingen)
	BÖBLINGEN_SCHAFGASSE = 'de:08115:4684'
	BÖBLINGEN_SCHAFGASSE_1 = 'de:08115:4684:0:3'  # eRtg Reußensteinstr. (Böblingen Böblingen)
	BÖBLINGEN_SCHAFGASSE_2 = 'de:08115:4684:0:4'  # eRtg Klaffensteinstr. (Böblingen Böblingen)
	BÖBLINGEN_REUSSENSTEINSTR = 'de:08115:4685:0:3'  # eRtg Schulzentrum Murkenbach (Böblingen Böblingen)
	BÖBLINGEN_ALTER_FRIEDHOF = 'de:08115:4686'
	BÖBLINGEN_ALTER_FRIEDHOF_1 = 'de:08115:4686:0:3'  # eRtg Schafgasse (Böblingen Böblingen)
	BÖBLINGEN_ALTER_FRIEDHOF_2 = 'de:08115:4686:0:4'  # Rtg Ost (Böblingen Böblingen)
	BÖBLINGEN_GEMEINDEHAUS = 'de:08115:4687:0:3'  # eRtg Murkenbachweg (Böblingen Böblingen)
	GEMEINDEHAUS_1 = 'de:08115:4687:0:3'  # eRtg Murkenbachweg (Böblingen Böblingen)
	GEMEINDEHAUS_2 = 'de:08119:5186:0:3'  # Rtg alle Richtungen (Remshalden Buoch)
	GEMEINDEHAUS_3 = 'de:08117:3315:0:H1'  # Rtg Grundschule (Eschenbach Eschenbach)
	BÖBLINGEN_MURKENBACHWEG = 'de:08115:4688:1:3'  # eRtg Tannenberg (Böblingen Böblingen)
	MURKENBACHWEG = 'de:08115:4688:1:3'  # eRtg Tannenberg (Böblingen Böblingen)
	TANNENBERG = 'de:08115:4689:1:3'  # eRtg Ganssee (Böblingen Böblingen)
	BÖBLINGEN_TANNENBERG = 'de:08115:4689:1:3'  # eRtg Ganssee (Böblingen Böblingen)
	BÖBLINGEN_HERDWEG__POSTAMT = 'de:08115:4690'
	BÖBLINGEN_HERDWEG__POSTAMT_1 = 'de:08115:4690:1:3'  # eRtg Alter Friedhof (Böblingen Böblingen)
	BÖBLINGEN_HERDWEG__POSTAMT_2 = 'de:08115:4690:1:4'  # Rtg Ost (Böblingen Böblingen)
	HERDWEG__POSTAMT = 'de:08115:4690'
	HERDWEG__POSTAMT_1 = 'de:08115:4690:1:3'  # eRtg Alter Friedhof (Böblingen Böblingen)
	HERDWEG__POSTAMT_2 = 'de:08115:4690:1:4'  # Rtg Ost (Böblingen Böblingen)
	BÖBLINGEN_REITERHOF = 'de:08115:4691'
	BÖBLINGEN_REITERHOF_1 = 'de:08115:4691:0:3'  # eRtg Heusteigstraße (Böblingen Böblingen)
	BÖBLINGEN_REITERHOF_2 = 'de:08115:4691:0:4'  # eRtg Freudenstädter Straße (Böblingen Böblingen)
	BÖBLINGEN_HALLENBAD = 'de:08115:4692'
	BÖBLINGEN_HALLENBAD_1 = 'de:08115:4692:1:3'  # eRtg Schönaicher First (Böblingen Böblingen)
	BÖBLINGEN_HALLENBAD_2 = 'de:08115:4692:1:4'  # eRtg Sporthalle (Böblingen Böblingen)
	BÖBL__SCHÖNBUCHSTR___SPORTH = 'de:08115:4693:0:3'  # eRtg Reiterhof (Böblingen Böblingen)
	SCHÖNBUCHSTRASSE__SPORTH = 'de:08115:4693:0:3'  # eRtg Reiterhof (Böblingen Böblingen)
	BÖBLINGEN_SCHÖNBUCHSTR = 'de:08115:4694:0:4'  # Rtg Holzgerlingen (Böblingen Böblingen)
	BÖBLINGEN_DIEZENHALDENWEG = 'de:08115:4695:0:3'  # eRtg Tübinger Straße (Böblingen Böblingen)
	DIEZENHALDENWEG = 'de:08115:4695:0:3'  # eRtg Tübinger Straße (Böblingen Böblingen)
	MAURENER_WEG = 'de:08115:4696'
	MAURENER_WEG_1 = 'de:08115:4696:0:3'  # Rtg Amsterdamer Str. (Böblingen Böblingen)
	MAURENER_WEG_2 = 'de:08115:4696:0:4'  # Rtg Berliner Str. (Böblingen Böblingen)
	MAURENER_WEG_3 = 'de:08115:4696:0:5'  # Rtg Süd Nordseite (Böblingen Böblingen)
	BÖBLINGEN_MAURENER_WEG = 'de:08115:4696'
	BÖBLINGEN_MAURENER_WEG_1 = 'de:08115:4696:0:3'  # Rtg Amsterdamer Str. (Böblingen Böblingen)
	BÖBLINGEN_MAURENER_WEG_2 = 'de:08115:4696:0:4'  # Rtg Berliner Str. (Böblingen Böblingen)
	BÖBLINGEN_MAURENER_WEG_3 = 'de:08115:4696:0:5'  # Rtg Süd Nordseite (Böblingen Böblingen)
	BÖBLINGEN_AMSTERDAMER_STR = 'de:08115:4697:0:4'  # Rtg Ochsentrog/Geleener/Maurenerstr. (Böblingen Böblingen)
	AMSTERDAMER_STRASSE = 'de:08115:4697:0:4'  # Rtg Ochsentrog/Geleener/Maurenerstr. (Böblingen Böblingen)
	GELEENER_STRASSE_KINDERGARTEN = 'de:08115:4698'
	GELEENER_STRASSE_KINDERGARTEN_1 = 'de:08115:4698:0:3'  # eRtg Geleener Straße (Böblingen Böblingen)
	GELEENER_STRASSE_KINDERGARTEN_2 = 'de:08115:4698:0:4'  # Rtg Süd (Böblingen Böblingen)
	BÖBL__GELEENER_STR__KINDERG = 'de:08115:4698'
	BÖBL__GELEENER_STR__KINDERG_1 = 'de:08115:4698:0:3'  # eRtg Geleener Straße (Böblingen Böblingen)
	BÖBL__GELEENER_STR__KINDERG_2 = 'de:08115:4698:0:4'  # Rtg Süd (Böblingen Böblingen)
	GELEENER_STRASSE = 'de:08115:4699'
	GELEENER_STRASSE_1 = 'de:08115:4699:0:3'  # eRtg Maurener Weg (Böblingen Böblingen)
	GELEENER_STRASSE_2 = 'de:08115:4699:0:4'  # Rtg Südwest (Böblingen Böblingen)
	BÖBLINGEN_GELEENER_STR = 'de:08115:4699'
	BÖBLINGEN_GELEENER_STR_1 = 'de:08115:4699:0:3'  # eRtg Maurener Weg (Böblingen Böblingen)
	BÖBLINGEN_GELEENER_STR_2 = 'de:08115:4699:0:4'  # Rtg Südwest (Böblingen Böblingen)
	BÖBLINGEN_BERLINER_STRASSE = 'de:08115:4700'
	BÖBLINGEN_BERLINER_STRASSE_1 = 'de:08115:4700:0:3'  # eRtg Königsberger Straße (Böblingen Böblingen)
	BÖBLINGEN_BERLINER_STRASSE_2 = 'de:08115:4700:0:4'  # eRtg Maurener Weg (Böblingen Böblingen)
	BÖBLINGEN_STEINBEISSTR = 'de:08115:4702'
	BÖBLINGEN_STEINBEISSTR_1 = 'de:08115:4702:0:3'  # Rtg Berliner Str. (Böblingen Böblingen)
	BÖBLINGEN_STEINBEISSTR_2 = 'de:08115:4702:0:5'  # Rtg Nord Nordseite (Böblingen Böblingen)
	EUGEN_BOLZ_STRASSE_OST = 'de:08115:4703:0:4'  # Rtg Reiterhof (Böblingen Böblingen)
	BÖBLINGEN_EUGEN_BOLZ_STR__OST = 'de:08115:4703:0:4'  # Rtg Reiterhof (Böblingen Böblingen)
	BÖBLINGEN_FREUDENSTÄDTER_STR = 'de:08115:4704:0:3'  # eRtg Freiburger Allee (Böblingen Böblingen)
	FREUDENSTÄDTER_STRASSE = 'de:08115:4704:0:3'  # eRtg Freiburger Allee (Böblingen Böblingen)
	BÖBLINGEN_FREIBURGER_ALLEE = 'de:08115:4705'
	BÖBLINGEN_FREIBURGER_ALLEE_1 = 'de:08115:4705:0:3'  # eRtg Reiterhof (Böblingen Böblingen)
	BÖBLINGEN_FREIBURGER_ALLEE_2 = 'de:08115:4705:0:4'  # eRtg Diezenhalde (Zentrum) Süd (Böblingen Böblingen)
	FREIBURGER_ALLEE = 'de:08115:4705'
	FREIBURGER_ALLEE_1 = 'de:08115:4705:0:3'  # eRtg Reiterhof (Böblingen Böblingen)
	FREIBURGER_ALLEE_2 = 'de:08115:4705:0:4'  # eRtg Diezenhalde (Zentrum) Süd (Böblingen Böblingen)
	BÖBLINGEN_KREMSER_STRASSE = 'de:08115:4707'
	BÖBLINGEN_KREMSER_STRASSE_1 = 'de:08115:4707:0:3'  # Rtg Herrenberger Str. (Böblingen Böblingen)
	BÖBLINGEN_KREMSER_STRASSE_2 = 'de:08115:4707:0:4'  # Rtg Breslauer Str. (Böblingen Böblingen)
	KREMSER_STRASSE = 'de:08115:4707'
	KREMSER_STRASSE_1 = 'de:08115:4707:0:3'  # Rtg Herrenberger Str. (Böblingen Böblingen)
	KREMSER_STRASSE_2 = 'de:08115:4707:0:4'  # Rtg Breslauer Str. (Böblingen Böblingen)
	BÖBLINGEN_BÜHLER_STRASSE = 'de:08115:4710'
	BÖBLINGEN_BÜHLER_STRASSE_1 = 'de:08115:4710:0:3'  # eRtg Waldfriedhof (Böblingen Böblingen)
	BÖBLINGEN_BÜHLER_STRASSE_2 = 'de:08115:4710:0:4'  # eRtg Keltenburgstraße (Böblingen Böblingen)
	BÜHLER_STRASSE = 'de:08115:4710'
	BÜHLER_STRASSE_1 = 'de:08115:4710:0:3'  # eRtg Waldfriedhof (Böblingen Böblingen)
	BÜHLER_STRASSE_2 = 'de:08115:4710:0:4'  # eRtg Keltenburgstraße (Böblingen Böblingen)
	BÖBLINGEN_DANZIGER_STRASSE = 'de:08115:4711'
	BÖBLINGEN_DANZIGER_STRASSE_1 = 'de:08115:4711:1:1'  # Rtg Dettenhausen (Böblingen Böblingen)
	BÖBLINGEN_DANZIGER_STRASSE_2 = 'de:08115:4711:1:2'  # Rtg Böblingen (Böblingen Böblingen)
	BÖBLINGEN_HEUSTEIGSTRASSE = 'de:08115:4712'
	BÖBLINGEN_HEUSTEIGSTRASSE_1 = 'de:08115:4712:1:1'  # R-Bahn (Böblingen Böblingen)
	BÖBLINGEN_HEUSTEIGSTRASSE_2 = 'de:08115:4712:2:3'  # eRtg Schulzentrum Murkenbach (Böblingen Böblingen)
	BÖBLINGEN_HEUSTEIGSTRASSE_3 = 'de:08115:4712:2:4'  # eRtg Reiterhof (Böblingen Böblingen)
	HEUSTEIGSTRASSE = 'de:08115:4712'
	HEUSTEIGSTRASSE_1 = 'de:08115:4712:1:1'  # R-Bahn (Böblingen Böblingen)
	HEUSTEIGSTRASSE_2 = 'de:08115:4712:2:3'  # eRtg Schulzentrum Murkenbach (Böblingen Böblingen)
	HEUSTEIGSTRASSE_3 = 'de:08115:4712:2:4'  # eRtg Reiterhof (Böblingen Böblingen)
	ZIMMERSCHLAG = 'de:08115:4713'
	ZIMMERSCHLAG_1 = 'de:08115:4713:1:1'  # Rtg Dettenhausen (Böblingen Böblingen)
	ZIMMERSCHLAG_2 = 'de:08115:4713:2:2'  # Rtg Böblingen (Böblingen Böblingen)
	BÖBLINGEN_ZIMMERSCHLAG = 'de:08115:4713'
	BÖBLINGEN_ZIMMERSCHLAG_1 = 'de:08115:4713:1:1'  # Rtg Dettenhausen (Böblingen Böblingen)
	BÖBLINGEN_ZIMMERSCHLAG_2 = 'de:08115:4713:2:2'  # Rtg Böblingen (Böblingen Böblingen)
	OCHSENTROG = 'de:08115:4714'
	OCHSENTROG_1 = 'de:08115:4714:0:3'  # eRtg Amsterdamer Straße (Böblingen Böblingen)
	OCHSENTROG_2 = 'de:08115:4714:0:4'  # eRtg Waldfriedhof (Böblingen Böblingen)
	BÖBLINGEN_OCHSENTROG = 'de:08115:4714'
	BÖBLINGEN_OCHSENTROG_1 = 'de:08115:4714:0:3'  # eRtg Amsterdamer Straße (Böblingen Böblingen)
	BÖBLINGEN_OCHSENTROG_2 = 'de:08115:4714:0:4'  # eRtg Waldfriedhof (Böblingen Böblingen)
	BÖBLINGEN_WALDFRIEDHOF = 'de:08115:4715'
	BÖBLINGEN_WALDFRIEDHOF_1 = 'de:08115:4715:0:3'  # eRtg Ochsentrog (Böblingen Böblingen)
	BÖBLINGEN_WALDFRIEDHOF_2 = 'de:08115:4715:0:4'  # eRtg Bühler Straße (Böblingen Böblingen)
	BÖBLINGEN_HERRENBERGER_STRASSE = 'de:08115:4716'
	BÖBLINGEN_HERRENBERGER_STRASSE_1 = 'de:08115:4716:1:3'  # Rtg Kreistierheim (Böblingen Böblingen)
	BÖBLINGEN_HERRENBERGER_STRASSE_2 = 'de:08115:4716:1:4'  # Rtg Kremser Str. (Böblingen Böblingen)
	KREISTIERHEIM = 'de:08115:4717'
	KREISTIERHEIM_1 = 'de:08115:4717:0:3'  # eRtg Schickardstraße (Böblingen Böblingen)
	KREISTIERHEIM_2 = 'de:08115:4717:0:4'  # eRtg Herrenberger Straße (Böblingen Böblingen)
	BÖBLINGEN_KREISTIERHEIM = 'de:08115:4717'
	BÖBLINGEN_KREISTIERHEIM_1 = 'de:08115:4717:0:3'  # eRtg Schickardstraße (Böblingen Böblingen)
	BÖBLINGEN_KREISTIERHEIM_2 = 'de:08115:4717:0:4'  # eRtg Herrenberger Straße (Böblingen Böblingen)
	SCHICKARDSTRASSE = 'de:08115:4718'
	SCHICKARDSTRASSE_1 = 'de:08115:4718:0:3'  # eRtg Technologiepark (Böblingen Böblingen)
	SCHICKARDSTRASSE_2 = 'de:08115:4718:0:4'  # eRtg Kreistierheim (Böblingen Böblingen)
	BÖBLINGEN_SCHICKARDSTRASSE = 'de:08115:4718'
	BÖBLINGEN_SCHICKARDSTRASSE_1 = 'de:08115:4718:0:3'  # eRtg Technologiepark (Böblingen Böblingen)
	BÖBLINGEN_SCHICKARDSTRASSE_2 = 'de:08115:4718:0:4'  # eRtg Kreistierheim (Böblingen Böblingen)
	HANNS_KLEMM_STRASSE_WEST = 'de:08115:4719'
	HANNS_KLEMM_STRASSE_WEST_1 = 'de:08115:4719:0:3'  # eRtg Hulb (Böblingen Böblingen)
	HANNS_KLEMM_STRASSE_WEST_2 = 'de:08115:4719:0:4'  # eRtg Technologiepark (Böblingen Böblingen)
	BÖBL__HANNS_KLEMM_STR__WEST = 'de:08115:4719'
	BÖBL__HANNS_KLEMM_STR__WEST_1 = 'de:08115:4719:0:3'  # eRtg Hulb (Böblingen Böblingen)
	BÖBL__HANNS_KLEMM_STR__WEST_2 = 'de:08115:4719:0:4'  # eRtg Technologiepark (Böblingen Böblingen)
	BÖBLINGEN_DORNIERSTRASSE = 'de:08115:4721'
	BÖBLINGEN_DORNIERSTRASSE_1 = 'de:08115:4721:0:3'  # eRtg Junkersstraße (Böblingen Böblingen)
	BÖBLINGEN_DORNIERSTRASSE_2 = 'de:08115:4721:0:4'  # eRtg Hulb (Böblingen Böblingen)
	BÖBLINGEN_OTTO_LILIENTHAL_STR = 'de:08115:4722'
	BÖBLINGEN_OTTO_LILIENTHAL_STR_1 = 'de:08115:4722:0:3'  # eRtg Heinkelstraße (Böblingen Böblingen)
	BÖBLINGEN_OTTO_LILIENTHAL_STR_2 = 'de:08115:4722:0:4'  # eRtg Dornierstr. (Böblingen Böblingen)
	OTTO_LILIENTHAL_STRASSE = 'de:08115:4722'
	OTTO_LILIENTHAL_STRASSE_1 = 'de:08115:4722:0:3'  # eRtg Heinkelstraße (Böblingen Böblingen)
	OTTO_LILIENTHAL_STRASSE_2 = 'de:08115:4722:0:4'  # eRtg Dornierstr. (Böblingen Böblingen)
	BÖBLINGEN_HEINKELSTRASSE = 'de:08115:4723'
	BÖBLINGEN_HEINKELSTRASSE_1 = 'de:08115:4723:0:3'  # eRtg Hulb (Böblingen Böblingen)
	BÖBLINGEN_HEINKELSTRASSE_2 = 'de:08115:4723:0:4'  # eRtg Hanns-Klemm-Straße Ost (Böblingen Böblingen)
	HANNS_KLEMM_STRASSE_OST = 'de:08115:4724'
	HANNS_KLEMM_STRASSE_OST_1 = 'de:08115:4724:0:3'  # eRtg Heinkelstraße (Böblingen Böblingen)
	HANNS_KLEMM_STRASSE_OST_2 = 'de:08115:4724:0:4'  # eRtg Danziger Straße (Böblingen Böblingen)
	BÖBL__HANNS_KLEMM_STR__OST = 'de:08115:4724'
	BÖBL__HANNS_KLEMM_STR__OST_1 = 'de:08115:4724:0:3'  # eRtg Heinkelstraße (Böblingen Böblingen)
	BÖBL__HANNS_KLEMM_STR__OST_2 = 'de:08115:4724:0:4'  # eRtg Danziger Straße (Böblingen Böblingen)
	AIDL__KATHOLISCHE_KIRCHE = 'de:08115:4725'
	AIDL__KATHOLISCHE_KIRCHE_1 = 'de:08115:4725:0:3'  # Rtg Hermann-Hesse-Str. (Aidlingen Aidlingen)
	AIDL__KATHOLISCHE_KIRCHE_2 = 'de:08115:4725:0:4'  # Rtg Rathaus (Aidlingen Aidlingen)
	GÄRTRINGEN_RATHAUS = 'de:08115:4727'
	GÄRTRINGEN_RATHAUS_1 = 'de:08115:4727:1:3'  # Rtg Feuerwehrgerätehaus (Gärtringen Gärtringen)
	GÄRTRINGEN_RATHAUS_2 = 'de:08115:4727:1:4'  # Rtg Gärtringen (Gärtringen Gärtringen)
	ROHRAU_RATHAUS = 'de:08115:4729:0:3'  # eRtg Jennerstraße/Gärtringen (Gärtringen Rohrau)
	NUFRINGEN_MITTE = 'de:08115:4735'
	NUFRINGEN_MITTE_1 = 'de:08115:4735:1:3'  # Rtg Gärtringen (Nufringen Nufringen)
	NUFRINGEN_MITTE_2 = 'de:08115:4735:1:4'  # Rtg Herrenberg (Nufringen Nufringen)
	KUPPINGER_STRASSE = 'de:08115:4736'
	KUPPINGER_STRASSE_1 = 'de:08115:4736:0:3'  # Rtg Rathaus (Nufringen Nufringen)
	KUPPINGER_STRASSE_2 = 'de:08115:4736:0:4'  # Rtg Hbg. Polizeihochschule (Nufringen Nufringen)
	NUFRINGEN_KUPPINGER_STR = 'de:08115:4736'
	NUFRINGEN_KUPPINGER_STR_1 = 'de:08115:4736:0:3'  # Rtg Rathaus (Nufringen Nufringen)
	NUFRINGEN_KUPPINGER_STR_2 = 'de:08115:4736:0:4'  # Rtg Hbg. Polizeihochschule (Nufringen Nufringen)
	HERRENBERG_POLIZEIHOCHSCHULE = 'de:08115:4737'
	HERRENBERG_POLIZEIHOCHSCHULE_1 = 'de:08115:4737:0:3'  # Rtg Nufringen (Herrenberg Herrenberg)
	HERRENBERG_POLIZEIHOCHSCHULE_2 = 'de:08115:4737:0:4'  # Rtg Herrenberg (Herrenberg Herrenberg)
	HERRENBERG_POLIZEIHOCHSCHULE_3 = 'de:08115:4737:0:6'  # Taktverdichtung Rtg Herrenberg (Herrenberg Herrenberg)
	POLIZEIHOCHSCHULE = 'de:08115:4737'
	POLIZEIHOCHSCHULE_1 = 'de:08115:4737:0:3'  # Rtg Nufringen (Herrenberg Herrenberg)
	POLIZEIHOCHSCHULE_2 = 'de:08115:4737:0:4'  # Rtg Herrenberg (Herrenberg Herrenberg)
	POLIZEIHOCHSCHULE_3 = 'de:08115:4737:0:6'  # Taktverdichtung Rtg Herrenberg (Herrenberg Herrenberg)
	HERRENBERG_BRONNTOR = 'de:08115:4739'
	HERRENBERG_BRONNTOR_1 = 'de:08115:4739:0:3'  # eRtg Benzstraße (Herrenberg Herrenberg)
	HERRENBERG_BRONNTOR_2 = 'de:08115:4739:0:4'  # eRtg ZOB Bahnhofstr. (Herrenberg Herrenberg)
	BRONNTOR = 'de:08115:4739'
	BRONNTOR_1 = 'de:08115:4739:0:3'  # eRtg Benzstraße (Herrenberg Herrenberg)
	BRONNTOR_2 = 'de:08115:4739:0:4'  # eRtg ZOB Bahnhofstr. (Herrenberg Herrenberg)
	OBERJETTINGEN_EINKAUFSZENTRUM = 'de:08115:4741'
	OBERJETTINGEN_EINKAUFSZENTRUM_1 = 'de:08115:4741:0:3'  # Rtg Kohlplatte/Eisberg (Jettingen Oberjettingen)
	OBERJETTINGEN_EINKAUFSZENTRUM_2 = 'de:08115:4741:0:4'  # Rtg Sindlinger Str. (Jettingen Oberjettingen)
	SINDLINGER_STRASSE_1 = 'de:08115:4742:0:3'  # Rtg Schelmenäcker (Herrenberg Haslach)
	SINDLINGER_STRASSE_2 = 'de:08115:4742:0:4'  # Rtg Kirche (Herrenberg Haslach)
	SINDLINGER_STRASSE_3 = 'de:08115:4861:0:3'  # Rtg Gemeindezentrum (Jettingen Oberjettingen)
	SINDLINGER_STRASSE_4 = 'de:08115:4861:0:4'  # Rtg Herrenberger Straße (Jettingen Oberjettingen)
	SINDLINGER_STRASSE_5 = 'de:08115:6745:0:1'  # Rtg Öschelbronn (Gäufelden Nebringen)
	SINDLINGER_STRASSE_6 = 'de:08115:6745:0:2'  # Rtg Herrenberg (Gäufelden Nebringen)
	HASLACH_SINDLINGER_STR = 'de:08115:4742'
	HASLACH_SINDLINGER_STR_1 = 'de:08115:4742:0:3'  # Rtg Schelmenäcker (Herrenberg Haslach)
	HASLACH_SINDLINGER_STR_2 = 'de:08115:4742:0:4'  # Rtg Kirche (Herrenberg Haslach)
	HASLACH_KIRCHE = 'de:08115:4743'
	HASLACH_KIRCHE_1 = 'de:08115:4743:0:3'  # Rtg Sindlinger Str. (Herrenberg Haslach)
	HASLACH_KIRCHE_2 = 'de:08115:4743:0:4'  # Rtg Randenstr. (Herrenberg Haslach)
	RANDENSTRASSE = 'de:08115:4744'
	RANDENSTRASSE_1 = 'de:08115:4744:0:3'  # Rtg Schulmeisterbuche (Herrenberg Haslach)
	RANDENSTRASSE_2 = 'de:08115:4744:0:4'  # Rtg Kirche (Herrenberg Haslach)
	HASLACH_RANDENSTRASSE = 'de:08115:4744'
	HASLACH_RANDENSTRASSE_1 = 'de:08115:4744:0:3'  # Rtg Schulmeisterbuche (Herrenberg Haslach)
	HASLACH_RANDENSTRASSE_2 = 'de:08115:4744:0:4'  # Rtg Kirche (Herrenberg Haslach)
	POSTAMT_1 = 'de:08115:4745:1:3'  # Rtg Gäufelden Bf Bettäcker (Gäufelden Nebringen)
	POSTAMT_2 = 'de:08115:4745:1:4'  # Rtg Herrenberg Straßenmeisterei (Gäufelden Nebringen)
	POSTAMT_3 = 'de:08117:61:0:H1'  # eRtg Reußenstein (Wiesensteig Wiesensteig)
	NEBRINGEN_POSTAMT = 'de:08115:4745'
	NEBRINGEN_POSTAMT_1 = 'de:08115:4745:1:3'  # Rtg Gäufelden Bf Bettäcker (Gäufelden Nebringen)
	NEBRINGEN_POSTAMT_2 = 'de:08115:4745:1:4'  # Rtg Herrenberg Straßenmeisterei (Gäufelden Nebringen)
	ÖSCHELBRONN_STUTTG__STR = 'de:08115:4746'
	ÖSCHELBRONN_STUTTG__STR_1 = 'de:08115:4746:0:3'  # Rtg Rathaus (Gäufelden Öschelbronn (Gäufelden))
	ÖSCHELBRONN_STUTTG__STR_2 = 'de:08115:4746:0:4'  # Rtg Nebringen (Gäufelden Öschelbronn (Gäufelden))
	ÖSCHELBRONN_RATHAUS = 'de:08115:4747'
	ÖSCHELBRONN_RATHAUS_1 = 'de:08115:4747:0:3'  # Rtg Post (Gäufelden Öschelbronn (Gäufelden))
	ÖSCHELBRONN_RATHAUS_2 = 'de:08115:4747:0:4'  # Rtg Stuttgarter Straße (Gäufelden Öschelbronn (Gäufelden))
	ÖSCHELBRONN_RATHAUS_3 = 'de:08115:4747:0:5'  # Ersatz Rtg. Stgt. Str. (Gäufelden Öschelbronn (Gäufelden))
	ÖSCHELBRONN_POST = 'de:08115:4748'
	ÖSCHELBRONN_POST_1 = 'de:08115:4748:0:3'  # Rtg Unterjettingen (Gäufelden Öschelbronn (Gäufelden))
	ÖSCHELBRONN_POST_2 = 'de:08115:4748:0:4'  # Rtg Rathaus (Gäufelden Öschelbronn (Gäufelden))
	WEILDORF = 'de:08115:4749:0:3'  # eRtg Stuttgarter Straße (Bondorf Bondorf)
	BONDORF_WEILDORF = 'de:08115:4749:0:3'  # eRtg Stuttgarter Straße (Bondorf Bondorf)
	EHNINGEN_MAURENER_STRASSE = 'de:08115:4753:0:4'  # Rtg IBM (Ehningen Ehningen)
	MAURENER_STRASSE = 'de:08115:4753:0:4'  # Rtg IBM (Ehningen Ehningen)
	EHNINGEN_WÜRMBRÜCKE = 'de:08115:4756'
	EHNINGEN_WÜRMBRÜCKE_1 = 'de:08115:4756:0:3'  # eRtg Königsberger Straße (Ehningen Ehningen)
	EHNINGEN_WÜRMBRÜCKE_2 = 'de:08115:4756:0:4'  # eRtg Ehningen (Ehningen Ehningen)
	EHNINGEN_KÖNIGSB__STR = 'de:08115:4757'
	EHNINGEN_KÖNIGSB__STR_1 = 'de:08115:4757:0:3'  # eRtg Im Letten (Ehningen Ehningen)
	EHNINGEN_KÖNIGSB__STR_2 = 'de:08115:4757:0:4'  # eRtg Würmbrücke (Ehningen Ehningen)
	KÖNIGSBERGER_STRASSE_1 = 'de:08115:4757:0:3'  # eRtg Im Letten (Ehningen Ehningen)
	KÖNIGSBERGER_STRASSE_2 = 'de:08115:4757:0:4'  # eRtg Würmbrücke (Ehningen Ehningen)
	KÖNIGSBERGER_STRASSE_3 = 'de:08116:6724:0:3'  # eRtg Adlerstraße (Wernau (Neckar) Wernau (N))
	KÖNIGSBERGER_STRASSE_4 = 'de:08115:7038:0:3'  # eRtg Sankt Martin (Herrenberg Herrenberg)
	HERRENBERG_SCHULMEISTERBUCHE = 'de:08115:4758'
	HERRENBERG_SCHULMEISTERBUCHE_1 = 'de:08115:4758:0:3'  # Rtg Jettingen (Herrenberg Herrenberg)
	HERRENBERG_SCHULMEISTERBUCHE_2 = 'de:08115:4758:0:4'  # Rtg Haslach/Herrenberg (Herrenberg Herrenberg)
	SCHULMEISTERBUCHE = 'de:08115:4758'
	SCHULMEISTERBUCHE_1 = 'de:08115:4758:0:3'  # Rtg Jettingen (Herrenberg Herrenberg)
	SCHULMEISTERBUCHE_2 = 'de:08115:4758:0:4'  # Rtg Haslach/Herrenberg (Herrenberg Herrenberg)
	ÖFELE = 'de:08115:4759:0:3'  # Rtg Herrenberger Straße (Jettingen Oberjettingen)
	OBERJETTINGEN_ÖFELE = 'de:08115:4759:0:3'  # Rtg Herrenberger Straße (Jettingen Oberjettingen)
	BONDORF_HOCHWIESENSTR = 'de:08115:4760'
	BONDORF_HOCHWIESENSTR_1 = 'de:08115:4760:0:3'  # eRtg Nebringer Straße (Bondorf Bondorf)
	BONDORF_HOCHWIESENSTR_2 = 'de:08115:4760:0:4'  # eRtg Bahnbrücke (Bondorf Bondorf)
	HOCHWIESENSTRASSE = 'de:08115:4760'
	HOCHWIESENSTRASSE_1 = 'de:08115:4760:0:3'  # eRtg Nebringer Straße (Bondorf Bondorf)
	HOCHWIESENSTRASSE_2 = 'de:08115:4760:0:4'  # eRtg Bahnbrücke (Bondorf Bondorf)
	BAHNBRÜCKE = 'de:08115:4761'
	BAHNBRÜCKE_1 = 'de:08115:4761:1:3'  # eRtg Hochwiesenstraße (Bondorf Bondorf)
	BAHNBRÜCKE_2 = 'de:08115:4761:1:4'  # eRtg Mötzingen (Bondorf Bondorf)
	BONDORF_BAHNBRÜCKE = 'de:08115:4761'
	BONDORF_BAHNBRÜCKE_1 = 'de:08115:4761:1:3'  # eRtg Hochwiesenstraße (Bondorf Bondorf)
	BONDORF_BAHNBRÜCKE_2 = 'de:08115:4761:1:4'  # eRtg Mötzingen (Bondorf Bondorf)
	HOLZGERLINGEN_SCHÖNBERG = 'de:08115:4763'
	HOLZGERLINGEN_SCHÖNBERG_1 = 'de:08115:4763:0:3'  # eRtg Mittlere Mühle (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_SCHÖNBERG_2 = 'de:08115:4763:0:4'  # eRtg Birkenstraße (Holzgerlingen Holzgerlingen)
	AIDLINGER_STRASSE_1 = 'de:08115:4764:1:3'  # eRtg Diakon.mutterhaus (Böblingen Dagersheim)
	AIDLINGER_STRASSE_2 = 'de:08115:4764:1:4'  # Rtg. Hauptstraße (Böblingen Dagersheim)
	AIDLINGER_STRASSE_3 = 'de:08115:4779:0:3'  # eRtg Rathaus (Aidlingen Deufringen)
	AIDLINGER_STRASSE_4 = 'de:08115:4779:0:4'  # eRtg Hermann-Hesse-Straße (Aidlingen Deufringen)
	DAGERSHEIM_AIDLINGER_STR = 'de:08115:4764'
	DAGERSHEIM_AIDLINGER_STR_1 = 'de:08115:4764:1:3'  # eRtg Diakon.mutterhaus (Böblingen Dagersheim)
	DAGERSHEIM_AIDLINGER_STR_2 = 'de:08115:4764:1:4'  # Rtg. Hauptstraße (Böblingen Dagersheim)
	DIAKONISSENMUTTERHAUS = 'de:08115:4765'
	DIAKONISSENMUTTERHAUS_1 = 'de:08115:4765:0:3'  # Rtg Aidlingen (Aidlingen Aidlingen)
	DIAKONISSENMUTTERHAUS_2 = 'de:08115:4765:0:4'  # Rtg Dagersheim (Aidlingen Aidlingen)
	AIDL__DIAKON_MUTTERHAUS = 'de:08115:4765'
	AIDL__DIAKON_MUTTERHAUS_1 = 'de:08115:4765:0:3'  # Rtg Aidlingen (Aidlingen Aidlingen)
	AIDL__DIAKON_MUTTERHAUS_2 = 'de:08115:4765:0:4'  # Rtg Dagersheim (Aidlingen Aidlingen)
	AIDLINGEN_FURTHMÜHLE = 'de:08115:4766'
	AIDLINGEN_FURTHMÜHLE_1 = 'de:08115:4766:0:3'  # Rtg Aidlingen Stadt (Aidlingen Aidlingen)
	AIDLINGEN_FURTHMÜHLE_2 = 'de:08115:4766:0:4'  # Rtg Diakonie (Aidlingen Aidlingen)
	FURTHMÜHLE = 'de:08115:4766'
	FURTHMÜHLE_1 = 'de:08115:4766:0:3'  # Rtg Aidlingen Stadt (Aidlingen Aidlingen)
	FURTHMÜHLE_2 = 'de:08115:4766:0:4'  # Rtg Diakonie (Aidlingen Aidlingen)
	LÖCHLESBERG = 'de:08115:4768'
	LÖCHLESBERG_1 = 'de:08115:4768:0:3'  # eRtg Kreuzstraße/Döffinger Str./BB (Grafenau Döffingen)
	LÖCHLESBERG_2 = 'de:08115:4768:0:4'  # eRtg Kapellenberg (Grafenau Döffingen)
	DÖFFINGEN_LÖCHLESBERG = 'de:08115:4768'
	DÖFFINGEN_LÖCHLESBERG_1 = 'de:08115:4768:0:3'  # eRtg Kreuzstraße/Döffinger Str./BB (Grafenau Döffingen)
	DÖFFINGEN_LÖCHLESBERG_2 = 'de:08115:4768:0:4'  # eRtg Kapellenberg (Grafenau Döffingen)
	DÖFFINGEN_KAPELLENBERG = 'de:08115:4769'
	DÖFFINGEN_KAPELLENBERG_1 = 'de:08115:4769:0:3'  # Rtg Rathaus (Grafenau Döffingen)
	DÖFFINGEN_KAPELLENBERG_2 = 'de:08115:4769:0:4'  # Rtg Löchlesberg (Grafenau Döffingen)
	KAPELLENBERG = 'de:08115:4769'
	KAPELLENBERG_1 = 'de:08115:4769:0:3'  # Rtg Rathaus (Grafenau Döffingen)
	KAPELLENBERG_2 = 'de:08115:4769:0:4'  # Rtg Löchlesberg (Grafenau Döffingen)
	DÖFFINGEN_RATHAUS = 'de:08115:4770'
	DÖFFINGEN_RATHAUS_1 = 'de:08115:4770:0:3'  # eRtg Kapellenberg (Grafenau Döffingen)
	DÖFFINGEN_RATHAUS_2 = 'de:08115:4770:0:4'  # eRtg Ulrichweg/Seeger (Grafenau Döffingen)
	SEEGER = 'de:08115:4771'
	SEEGER_1 = 'de:08115:4771:0:3'  # eRtg Alter Friedhof (Grafenau Döffingen)
	SEEGER_2 = 'de:08115:4771:0:4'  # eRtg Abzweig (Grafenau Döffingen)
	DÖFFINGEN_SEEGER = 'de:08115:4771'
	DÖFFINGEN_SEEGER_1 = 'de:08115:4771:0:3'  # eRtg Alter Friedhof (Grafenau Döffingen)
	DÖFFINGEN_SEEGER_2 = 'de:08115:4771:0:4'  # eRtg Abzweig (Grafenau Döffingen)
	ALTDORF_WÜRMSTRASSE = 'de:08115:4772:0:3'  # eRtg Bachstraße/Th-Heuss-Str (Altdorf Altdorf (BB))
	WÜRMSTRASSE = 'de:08115:4772:0:3'  # eRtg Bachstraße/Th-Heuss-Str (Altdorf Altdorf (BB))
	DÄTZINGEN_KREUZSTRASSE = 'de:08115:4773'
	DÄTZINGEN_KREUZSTRASSE_1 = 'de:08115:4773:0:3'  # Rtg Rathaus (Grafenau Dätzingen)
	DÄTZINGEN_KREUZSTRASSE_2 = 'de:08115:4773:0:4'  # Rtg Döffingen (Grafenau Dätzingen)
	AIDLINGEN_LEHENWEILER_ABZW = 'de:08115:4774'
	AIDLINGEN_LEHENWEILER_ABZW_1 = 'de:08115:4774:0:3'  # eRtg Schafhausener Straße (Aidlingen Aidlingen)
	AIDLINGEN_LEHENWEILER_ABZW_2 = 'de:08115:4774:0:4'  # eRtg Lehenweiler/Dätzingen Abzw (Aidlingen Aidlingen)
	LEHENWEILER_ABZWEIG = 'de:08115:4774'
	LEHENWEILER_ABZWEIG_1 = 'de:08115:4774:0:3'  # eRtg Schafhausener Straße (Aidlingen Aidlingen)
	LEHENWEILER_ABZWEIG_2 = 'de:08115:4774:0:4'  # eRtg Lehenweiler/Dätzingen Abzw (Aidlingen Aidlingen)
	LEHENWEILER = 'de:08115:4775:0:3'  # eRtg Alle Richtungen (Aidlingen Aidlingen)
	AIDLINGEN_LEHENWEILER = 'de:08115:4775:0:3'  # eRtg Alle Richtungen (Aidlingen Aidlingen)
	SCHAFHAUSER_STRASSE = 'de:08115:4776'
	SCHAFHAUSER_STRASSE_1 = 'de:08115:4776:0:3'  # Rtg Lehenweiler/Grafenau (Aidlingen Aidlingen)
	SCHAFHAUSER_STRASSE_2 = 'de:08115:4776:0:4'  # Rtg Aidlingen (Aidlingen Aidlingen)
	AIDLINGEN_SCHAFHAUSER_STR = 'de:08115:4776'
	AIDLINGEN_SCHAFHAUSER_STR_1 = 'de:08115:4776:0:3'  # Rtg Lehenweiler/Grafenau (Aidlingen Aidlingen)
	AIDLINGEN_SCHAFHAUSER_STR_2 = 'de:08115:4776:0:4'  # Rtg Aidlingen (Aidlingen Aidlingen)
	AIDLINGEN_RATHAUS = 'de:08115:4777'
	AIDLINGEN_RATHAUS_1 = 'de:08115:4777:0:3'  # eRtg Katholische Kirche (Aidlingen Aidlingen)
	AIDLINGEN_RATHAUS_2 = 'de:08115:4777:0:4'  # eRtg Furthmühle (Aidlingen Aidlingen)
	HERMANN_HESSE_STRASSE_1 = 'de:08115:4778:0:3'  # eRtg Kath. Kirche/Rathaus (Aidlingen Aidlingen)
	HERMANN_HESSE_STRASSE_2 = 'de:08115:4778:0:4'  # eRtg Deufringen Aidlinger Straße (Aidlingen Aidlingen)
	HERMANN_HESSE_STRASSE_3 = 'de:08118:6848:0:4'  # eRtg Otto-Hahn-Straße (Ludwigsburg Neckarweihingen)
	AIDLINGEN_HERMANN_HESSE_STR = 'de:08115:4778'
	AIDLINGEN_HERMANN_HESSE_STR_1 = 'de:08115:4778:0:3'  # eRtg Kath. Kirche/Rathaus (Aidlingen Aidlingen)
	AIDLINGEN_HERMANN_HESSE_STR_2 = 'de:08115:4778:0:4'  # eRtg Deufringen Aidlinger Straße (Aidlingen Aidlingen)
	DEUFRINGEN_AIDLINGER_STR = 'de:08115:4779'
	DEUFRINGEN_AIDLINGER_STR_1 = 'de:08115:4779:0:3'  # eRtg Rathaus (Aidlingen Deufringen)
	DEUFRINGEN_AIDLINGER_STR_2 = 'de:08115:4779:0:4'  # eRtg Hermann-Hesse-Straße (Aidlingen Deufringen)
	DEUFRINGEN_RATHAUS = 'de:08115:4780'
	DEUFRINGEN_RATHAUS_1 = 'de:08115:4780:0:3'  # eRtg Bergsiedlung/Backhaus (Aidlingen Deufringen)
	DEUFRINGEN_RATHAUS_2 = 'de:08115:4780:0:4'  # eRtg Aidlinger Straße (Aidlingen Deufringen)
	WEIL_IM_SCHÖNBUCH_RÖTE = 'de:08115:4781'
	WEIL_IM_SCHÖNBUCH_RÖTE_1 = 'de:08115:4781:1:1'  # R-Bahn (Weil im Schönbuch Weil im Schönbuch)
	WEIL_IM_SCHÖNBUCH_RÖTE_2 = 'de:08115:4781:2:3'  # eRtg Troppel (Weil im Schönbuch Weil im Schönbuch)
	WEIL_IM_SCHÖNBUCH_RÖTE_3 = 'de:08115:4781:2:4'  # eRtg Rathaus (Weil im Schönbuch Weil im Schönbuch)
	RÖTE_1 = 'de:08115:4781:1:1'  # R-Bahn (Weil im Schönbuch Weil im Schönbuch)
	RÖTE_2 = 'de:08115:4781:2:3'  # eRtg Troppel (Weil im Schönbuch Weil im Schönbuch)
	RÖTE_3 = 'de:08115:4781:2:4'  # eRtg Rathaus (Weil im Schönbuch Weil im Schönbuch)
	RÖTE_4 = 'de:08115:6756:0:3'  # eRtg Mittenbühl (Grafenau Döffingen)
	RÖTE_5 = 'de:08115:6756:0:4'  # eRtg Hallenbad (Grafenau Döffingen)
	MITTLERE_MÜHLE = 'de:08115:4785'
	MITTLERE_MÜHLE_1 = 'de:08115:4785:0:3'  # eRtg Seestraße/Breitenstein (Holzgerlingen Holzgerlingen)
	MITTLERE_MÜHLE_2 = 'de:08115:4785:0:4'  # eRtg Schönberg (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_MITTLERE_MÜHLE = 'de:08115:4785'
	HOLZGERLINGEN_MITTLERE_MÜHLE_1 = 'de:08115:4785:0:3'  # eRtg Seestraße/Breitenstein (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_MITTLERE_MÜHLE_2 = 'de:08115:4785:0:4'  # eRtg Schönberg (Holzgerlingen Holzgerlingen)
	BUCH = 'de:08115:4786:1:1'  # R-Bahn (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_BUCH = 'de:08115:4786:1:1'  # R-Bahn (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_BÜHLENSTR = 'de:08115:4787:0:4'  # Rtg alle Richtungen (Holzgerlingen Holzgerlingen)
	BÜHLENSTRASSE = 'de:08115:4787:0:4'  # Rtg alle Richtungen (Holzgerlingen Holzgerlingen)
	BREITENSTEIN_RATHAUS = 'de:08115:4788:0:3'  # eRtg Alle Richtungen (Weil im Schönbuch Breitenstein)
	NEUWEILER_ABZW = 'de:08115:4789'
	NEUWEILER_ABZW_1 = 'de:08115:4789:0:3'  # eRtg Hennersdorfer Str/Rathaus (Weil im Schönbuch Neuweiler)
	NEUWEILER_ABZW_2 = 'de:08115:4789:0:4'  # eRtg Nord (Weil im Schönbuch Neuweiler)
	WEIL_I__S__SEE = 'de:08115:4790'
	WEIL_I__S__SEE_1 = 'de:08115:4790:0:3'  # eRtg Nord (Weil im Schönbuch Weil im Schönbuch)
	WEIL_I__S__SEE_2 = 'de:08115:4790:0:4'  # eRtg Alle Ri außer Nord (Weil im Schönbuch Weil im Schönbuch)
	ALTDORF_HOLZGERLINGER_STR = 'de:08115:4793'
	ALTDORF_HOLZGERLINGER_STR_1 = 'de:08115:4793:0:3'  # eRtg Holzgerlingen (Altdorf Altdorf (BB))
	ALTDORF_HOLZGERLINGER_STR_2 = 'de:08115:4793:0:4'  # eRtg Alter Friedhof (Altdorf Altdorf (BB))
	HOLZGERLINGER_STRASSE = 'de:08115:4793'
	HOLZGERLINGER_STRASSE_1 = 'de:08115:4793:0:3'  # eRtg Holzgerlingen (Altdorf Altdorf (BB))
	HOLZGERLINGER_STRASSE_2 = 'de:08115:4793:0:4'  # eRtg Alter Friedhof (Altdorf Altdorf (BB))
	ALTDORF_FRIEDHOF = 'de:08115:4794:0:3'  # eRtg Würmstraße/Theodor-Heuss-Straße (Altdorf Altdorf (BB))
	ALTDORF_BACHSTRASSE = 'de:08115:4795:0:3'  # eRtg Holzgerlinger Straße (Altdorf Altdorf (BB))
	THEODOR_HEUSS_STRASSE_1 = 'de:08115:4796:0:3'  # Rtg Ehningen (Hildrizhausen Hildrizhausen)
	THEODOR_HEUSS_STRASSE_2 = 'de:08115:4796:0:4'  # Rtg Altdorf (Hildrizhausen Hildrizhausen)
	THEODOR_HEUSS_STRASSE_3 = 'de:08118:6513:0:3'  # eRtg Roter Baum (Oberriexingen Oberriexingen)
	HILDRIZHAUSEN_TH__HEUSS_STR = 'de:08115:4796'
	HILDRIZHAUSEN_TH__HEUSS_STR_1 = 'de:08115:4796:0:3'  # Rtg Ehningen (Hildrizhausen Hildrizhausen)
	HILDRIZHAUSEN_TH__HEUSS_STR_2 = 'de:08115:4796:0:4'  # Rtg Altdorf (Hildrizhausen Hildrizhausen)
	ALTDORFER_STRASSE = 'de:08115:4797'
	ALTDORFER_STRASSE_1 = 'de:08115:4797:0:3'  # Rtg Ehningen (Hildrizhausen Hildrizhausen)
	ALTDORFER_STRASSE_2 = 'de:08115:4797:0:4'  # Rtg Altdorf (Hildrizhausen Hildrizhausen)
	HILDRIZHAUSEN_ALTDORFER_STR = 'de:08115:4797'
	HILDRIZHAUSEN_ALTDORFER_STR_1 = 'de:08115:4797:0:3'  # Rtg Ehningen (Hildrizhausen Hildrizhausen)
	HILDRIZHAUSEN_ALTDORFER_STR_2 = 'de:08115:4797:0:4'  # Rtg Altdorf (Hildrizhausen Hildrizhausen)
	HILDRIZHAUSEN_ALTES_RATHAUS = 'de:08115:4798'
	HILDRIZHAUSEN_ALTES_RATHAUS_1 = 'de:08115:4798:0:3'  # eRtg Altdorfer Straße (Hildrizhausen Hildrizhausen)
	HILDRIZHAUSEN_ALTES_RATHAUS_2 = 'de:08115:4798:0:4'  # eRtg IBM (Hildrizhausen Hildrizhausen)
	HERRENBERG_WALDFRIEDHOF = 'de:08115:4800:0:3'  # eRtg Grafenweg (Herrenberg Herrenberg)
	HERRENBERG_SENIORENZENTRUM = 'de:08115:4801'
	HERRENBERG_SENIORENZENTRUM_1 = 'de:08115:4801:0:3'  # eRtg Grafenweg (Herrenberg Herrenberg)
	HERRENBERG_SENIORENZENTRUM_2 = 'de:08115:4801:0:4'  # eRtg Hildrizhauser Str. (Herrenberg Herrenberg)
	HERRENBERG_HINDENBURGSTR = 'de:08115:4802'
	HERRENBERG_HINDENBURGSTR_1 = 'de:08115:4802:0:3'  # Rtg Altstadtgarage Kreiskrankenhaus (Herrenberg Herrenberg)
	HERRENBERG_HINDENBURGSTR_2 = 'de:08115:4802:0:4'  # Rtg ZOB (Herrenberg Herrenberg)
	SINDELFINGEN_IKEA = 'de:08115:4803'
	SINDELFINGEN_IKEA_1 = 'de:08115:4803:0:3'  # Rtg Sindelfingen ZOB (Sindelfingen Sindelfingen)
	SINDELFINGEN_IKEA_2 = 'de:08115:4803:0:4'  # Rtg R.-Diesel-Str. (Sindelfingen Sindelfingen)
	SCHÖNAICH_KELTERWEG = 'de:08115:4804'
	SCHÖNAICH_KELTERWEG_1 = 'de:08115:4804:0:3'  # eRtg Elsenhalde (Schönaich Schönaich)
	SCHÖNAICH_KELTERWEG_2 = 'de:08115:4804:0:4'  # eRtg Rauer Kapf (Schönaich Schönaich)
	SCHÖNAICH_ELSENHALDE = 'de:08115:4805'
	SCHÖNAICH_ELSENHALDE_1 = 'de:08115:4805:0:3'  # eRtg Daimlerstraße (Schönaich Schönaich)
	SCHÖNAICH_ELSENHALDE_2 = 'de:08115:4805:0:4'  # eRtg Bahnhofstraße (Schönaich Schönaich)
	ELSENHALDE = 'de:08115:4805'
	ELSENHALDE_1 = 'de:08115:4805:0:3'  # eRtg Daimlerstraße (Schönaich Schönaich)
	ELSENHALDE_2 = 'de:08115:4805:0:4'  # eRtg Bahnhofstraße (Schönaich Schönaich)
	SCHÖNAICH_BAHNHOFSTRASSE = 'de:08115:4806'
	SCHÖNAICH_BAHNHOFSTRASSE_1 = 'de:08115:4806:0:3'  # Rtg Elsenhalde (Schönaich Schönaich)
	SCHÖNAICH_BAHNHOFSTRASSE_2 = 'de:08115:4806:0:4'  # Rtg Neues Rathaus (Schönaich Schönaich)
	EICHE = 'de:08115:4808'
	EICHE_1 = 'de:08115:4808:0:3'  # eRtg Seestraße (Schönaich Schönaich)
	EICHE_2 = 'de:08115:4808:0:4'  # eRtg Bahnhofstraße (Schönaich Schönaich)
	SCHÖNAICH_EICHE = 'de:08115:4808'
	SCHÖNAICH_EICHE_1 = 'de:08115:4808:0:3'  # eRtg Seestraße (Schönaich Schönaich)
	SCHÖNAICH_EICHE_2 = 'de:08115:4808:0:4'  # eRtg Bahnhofstraße (Schönaich Schönaich)
	STEINBASS = 'de:08115:4811'
	STEINBASS_1 = 'de:08115:4811:0:3'  # eRtg Roter Berg (Schönaich Schönaich)
	STEINBASS_2 = 'de:08115:4811:0:4'  # eRtg Lessingstraße (Schönaich Schönaich)
	SCHÖNAICH_STEINBASS = 'de:08115:4811'
	SCHÖNAICH_STEINBASS_1 = 'de:08115:4811:0:3'  # eRtg Roter Berg (Schönaich Schönaich)
	SCHÖNAICH_STEINBASS_2 = 'de:08115:4811:0:4'  # eRtg Lessingstraße (Schönaich Schönaich)
	ROTER_BERG = 'de:08115:4812'
	ROTER_BERG_1 = 'de:08115:4812:0:3'  # eRtg Sulzbachtal (Schönaich Schönaich)
	ROTER_BERG_2 = 'de:08115:4812:0:4'  # eRtg Steinbass (Schönaich Schönaich)
	SCHÖNAICH_ROTER_BERG = 'de:08115:4812'
	SCHÖNAICH_ROTER_BERG_1 = 'de:08115:4812:0:3'  # eRtg Sulzbachtal (Schönaich Schönaich)
	SCHÖNAICH_ROTER_BERG_2 = 'de:08115:4812:0:4'  # eRtg Steinbass (Schönaich Schönaich)
	SCHÖNAICH_SULZBACHTAL = 'de:08115:4813'
	SCHÖNAICH_SULZBACHTAL_1 = 'de:08115:4813:0:3'  # eRtg Kirche (Schönaich Schönaich)
	SCHÖNAICH_SULZBACHTAL_2 = 'de:08115:4813:0:4'  # eRtg Roter Berg (Schönaich Schönaich)
	SULZBACHTAL = 'de:08115:4813'
	SULZBACHTAL_1 = 'de:08115:4813:0:3'  # eRtg Kirche (Schönaich Schönaich)
	SULZBACHTAL_2 = 'de:08115:4813:0:4'  # eRtg Roter Berg (Schönaich Schönaich)
	SCHÖNAICH_LESSINGSTRASSE = 'de:08115:4814'
	SCHÖNAICH_LESSINGSTRASSE_1 = 'de:08115:4814:0:3'  # eRtg Steinbass (Schönaich Schönaich)
	SCHÖNAICH_LESSINGSTRASSE_2 = 'de:08115:4814:0:4'  # eRtg Schillerstraße (Schönaich Schönaich)
	LESSINGSTRASSE_1 = 'de:08115:4814:0:3'  # eRtg Steinbass (Schönaich Schönaich)
	LESSINGSTRASSE_2 = 'de:08115:4814:0:4'  # eRtg Schillerstraße (Schönaich Schönaich)
	LESSINGSTRASSE_3 = 'de:08115:5835:0:3'  # eRtg Bosch (Rutesheim Rutesheim)
	LESSINGSTRASSE_4 = 'de:08115:5835:0:4'  # eRtg Heuweg (Rutesheim Rutesheim)
	LESSINGSTRASSE_5 = 'de:08117:9002:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	ZEHNTSCHEUER_1 = 'de:08115:4816:0:3'  # eRtg Bühläcker (Schönaich Schönaich)
	ZEHNTSCHEUER_2 = 'de:08115:4816:0:4'  # eRtg Evangelische Kirche (Schönaich Schönaich)
	ZEHNTSCHEUER_3 = 'de:08118:5559:0:3'  # eRtg Steinheimer Straße (Ludwigsburg Poppenweiler)
	ZEHNTSCHEUER_4 = 'de:08118:5559:0:4'  # eRtg Friedhof (Ludwigsburg Poppenweiler)
	SCHÖNAICH_ZEHNTSCHEUER = 'de:08115:4816'
	SCHÖNAICH_ZEHNTSCHEUER_1 = 'de:08115:4816:0:3'  # eRtg Bühläcker (Schönaich Schönaich)
	SCHÖNAICH_ZEHNTSCHEUER_2 = 'de:08115:4816:0:4'  # eRtg Evangelische Kirche (Schönaich Schönaich)
	SCHÖNAICH_BÜHLÄCKER = 'de:08115:4817'
	SCHÖNAICH_BÜHLÄCKER_1 = 'de:08115:4817:0:3'  # eRtg Wolfenmühle (Schönaich Schönaich)
	SCHÖNAICH_BÜHLÄCKER_2 = 'de:08115:4817:0:4'  # eRtg Zehntscheuer (Schönaich Schönaich)
	BÜHLÄCKER = 'de:08115:4817'
	BÜHLÄCKER_1 = 'de:08115:4817:0:3'  # eRtg Wolfenmühle (Schönaich Schönaich)
	BÜHLÄCKER_2 = 'de:08115:4817:0:4'  # eRtg Zehntscheuer (Schönaich Schönaich)
	HENNERSDORFER_STRASSE = 'de:08115:4819'
	HENNERSDORFER_STRASSE_1 = 'de:08115:4819:0:3'  # Rtg Schönaich (Weil im Schönbuch Neuweiler)
	HENNERSDORFER_STRASSE_2 = 'de:08115:4819:0:4'  # Rtg Weil (Weil im Schönbuch Neuweiler)
	NEUWEILER_HENNERSDORFER_STR = 'de:08115:4819'
	NEUWEILER_HENNERSDORFER_STR_1 = 'de:08115:4819:0:3'  # Rtg Schönaich (Weil im Schönbuch Neuweiler)
	NEUWEILER_HENNERSDORFER_STR_2 = 'de:08115:4819:0:4'  # Rtg Weil (Weil im Schönbuch Neuweiler)
	NEUWEILER_RATHAUS = 'de:08115:4820:0:3'  # eRtg Hennersdorfer Straße (Weil im Schönbuch Neuweiler)
	NEUWEILER_BEETHOVENSTR = 'de:08115:4821:0:3'  # eRtg Wolfenmühle (Weil im Schönbuch Neuweiler)
	APOTHEKE_1 = 'de:08116:4822:1:3'  # eRtg Fabrikstraße (Kirchheim unter Teck Ötlingen)
	APOTHEKE_2 = 'de:08125:4673:0:1'  # eRtg Breslauer Straße/Kath. Kirche (Beilstein Beilstein)
	APOTHEKE_3 = 'de:08125:4673:0:2'  # eRtg Langhans (Beilstein Beilstein)
	APOTHEKE_4 = 'de:08125:4673:0:5'  # Nachtbus (Beilstein Beilstein)
	ÖTLINGEN_APOTHEKE = 'de:08116:4822:1:3'  # eRtg Fabrikstraße (Kirchheim unter Teck Ötlingen)
	KIRCHHEIM__T__BOHNAU = 'de:08116:4823'
	KIRCHHEIM__T__BOHNAU_1 = 'de:08116:4823:0:3'  # eRtg Otto-Hahn-Straße (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__BOHNAU_2 = 'de:08116:4823:0:4'  # eRtg Bohnauhaus (Kirchheim unter Teck Kirchheim (T))
	BOHNAU = 'de:08116:4823'
	BOHNAU_1 = 'de:08116:4823:0:3'  # eRtg Otto-Hahn-Straße (Kirchheim unter Teck Kirchheim (T))
	BOHNAU_2 = 'de:08116:4823:0:4'  # eRtg Bohnauhaus (Kirchheim unter Teck Kirchheim (T))
	SCHÖNAICH_STADION = 'de:08115:4824:0:3'  # eRtg Elsenhalde (Schönaich Schönaich)
	SCHÖNAICH_NEUES_RATHAUS = 'de:08115:4828:0:4'  # eRtg Evangelische Kirche (Schönaich Schönaich)
	NEUES_RATHAUS = 'de:08115:4828:0:4'  # eRtg Evangelische Kirche (Schönaich Schönaich)
	SCHÖNAICH_SCHWABENSTRASSE = 'de:08115:4829'
	SCHÖNAICH_SCHWABENSTRASSE_1 = 'de:08115:4829:0:3'  # eRtg Evangelische Kirche (Schönaich Schönaich)
	SCHÖNAICH_SCHWABENSTRASSE_2 = 'de:08115:4829:0:4'  # eRtg Seestraße (Schönaich Schönaich)
	SCHWABENSTRASSE = 'de:08115:4829'
	SCHWABENSTRASSE_1 = 'de:08115:4829:0:3'  # eRtg Evangelische Kirche (Schönaich Schönaich)
	SCHWABENSTRASSE_2 = 'de:08115:4829:0:4'  # eRtg Seestraße (Schönaich Schönaich)
	KAYH_OBERE_ÄCKER = 'de:08115:4830'
	KAYH_OBERE_ÄCKER_1 = 'de:08115:4830:0:3'  # Rtg Breitenholz Lamm (Herrenberg Kayh)
	KAYH_OBERE_ÄCKER_2 = 'de:08115:4830:0:4'  # Rtg Rathaus (Herrenberg Kayh)
	OBERE_ÄCKER = 'de:08115:4830'
	OBERE_ÄCKER_1 = 'de:08115:4830:0:3'  # Rtg Breitenholz Lamm (Herrenberg Kayh)
	OBERE_ÄCKER_2 = 'de:08115:4830:0:4'  # Rtg Rathaus (Herrenberg Kayh)
	KAYH_RATHAUS = 'de:08115:4831'
	KAYH_RATHAUS_1 = 'de:08115:4831:0:3'  # Rtg Obere Äcker (Herrenberg Kayh)
	KAYH_RATHAUS_2 = 'de:08115:4831:0:4'  # Rtg Herrenberger Straße (Herrenberg Kayh)
	KAYH_HERRENBERGER_STR = 'de:08115:4832'
	KAYH_HERRENBERGER_STR_1 = 'de:08115:4832:0:3'  # Rtg Rathaus (Herrenberg Kayh)
	KAYH_HERRENBERGER_STR_2 = 'de:08115:4832:0:4'  # Rtg Mönchberg Abzweig (Herrenberg Kayh)
	SCHULSTRASSE_1 = 'de:08115:4834:0:3'  # westlich (Gäufelden Tailfingen)
	SCHULSTRASSE_2 = 'de:08115:4834:0:4'  # Rtg Rathaus (Gäufelden Tailfingen)
	SCHULSTRASSE_3 = 'de:08119:5763:0:3'  # eRtg Alle Richtungen (Plüderhausen Plüderhausen)
	SCHULSTRASSE_4 = 'de:08119:6873:0:3'  # eRtg Hauptschule (Kirchberg an der Murr Kirchberg (M))
	SCHULSTRASSE_5 = 'de:08117:2039:0:H1'  # Rtg Volksbank (Uhingen Uhingen)
	SCHULSTRASSE_6 = 'de:08117:2039:0:R1'  # Rtg Allgaier (Uhingen Uhingen)
	SCHULSTRASSE_7 = 'de:08117:5010:0:H1'  # Rtg Eichbühl (Wangen Wangen (GP))
	SCHULSTRASSE_8 = 'de:08117:5010:0:R1'  # Rtg Holzhäuser Str. (Wangen Wangen (GP))
	TAILFINGEN_SCHULSTRASSE = 'de:08115:4834'
	TAILFINGEN_SCHULSTRASSE_1 = 'de:08115:4834:0:3'  # westlich (Gäufelden Tailfingen)
	TAILFINGEN_SCHULSTRASSE_2 = 'de:08115:4834:0:4'  # Rtg Rathaus (Gäufelden Tailfingen)
	GÜLTSTEIN_KIRCHE = 'de:08115:4835'
	GÜLTSTEIN_KIRCHE_1 = 'de:08115:4835:0:3'  # Rtg Erholungsheim (Herrenberg Gültstein)
	GÜLTSTEIN_KIRCHE_2 = 'de:08115:4835:0:4'  # Rtg Tailfingen (Herrenberg Gültstein)
	ERHOLUNGSHEIM = 'de:08115:4836'
	ERHOLUNGSHEIM_1 = 'de:08115:4836:1:3'  # Rtg Heerstr. (Herrenberg Gültstein)
	ERHOLUNGSHEIM_2 = 'de:08115:4836:1:4'  # Rtg Kirche (Herrenberg Gültstein)
	GÜLTSTEIN_ERHOLUNGSHEIM = 'de:08115:4836'
	GÜLTSTEIN_ERHOLUNGSHEIM_1 = 'de:08115:4836:1:3'  # Rtg Heerstr. (Herrenberg Gültstein)
	GÜLTSTEIN_ERHOLUNGSHEIM_2 = 'de:08115:4836:1:4'  # Rtg Kirche (Herrenberg Gültstein)
	HERRENB__KREISKRANKENH___B28 = 'de:08115:4837'
	HERRENB__KREISKRANKENH___B28_1 = 'de:08115:4837:0:3'  # Rtg Gülstein Mönchberg Kayh (Herrenberg Herrenberg)
	HERRENB__KREISKRANKENH___B28_2 = 'de:08115:4837:0:4'  # Rtg Hindenburgstraße (Herrenberg Herrenberg)
	KREISKRANKENHAUS__B28 = 'de:08115:4837'
	KREISKRANKENHAUS__B28_1 = 'de:08115:4837:0:3'  # Rtg Gülstein Mönchberg Kayh (Herrenberg Herrenberg)
	KREISKRANKENHAUS__B28_2 = 'de:08115:4837:0:4'  # Rtg Hindenburgstraße (Herrenberg Herrenberg)
	DECKENPFRONN_RATHAUS = 'de:08115:4838'
	DECKENPFRONN_RATHAUS_1 = 'de:08115:4838:0:3'  # eRtg Brühlstraße (Deckenpfronn Deckenpfronn)
	DECKENPFRONN_RATHAUS_2 = 'de:08115:4838:0:4'  # eRtg Gültingen/Lerchenberg (Deckenpfronn Deckenpfronn)
	DECKENPFRONN_TÜBINGER_STR = 'de:08115:4839'
	DECKENPFRONN_TÜBINGER_STR_1 = 'de:08115:4839:0:3'  # eRtg Rathaus (Deckenpfronn Deckenpfronn)
	DECKENPFRONN_TÜBINGER_STR_2 = 'de:08115:4839:0:4'  # eRtg Siemensstr. (Deckenpfronn Deckenpfronn)
	OBERJESINGEN_CALWER_STR = 'de:08115:4840'
	OBERJESINGEN_CALWER_STR_1 = 'de:08115:4840:0:3'  # eRtg Tübinger Straße (Herrenberg Oberjesingen)
	OBERJESINGEN_CALWER_STR_2 = 'de:08115:4840:0:4'  # eRtg Zaberstr. (Herrenberg Oberjesingen)
	OBERJESINGEN_ZABERSTRASSE = 'de:08115:4842'
	OBERJESINGEN_ZABERSTRASSE_1 = 'de:08115:4842:0:3'  # eRtg Grundschule/Calwer Str. (Herrenberg Oberjesingen)
	OBERJESINGEN_ZABERSTRASSE_2 = 'de:08115:4842:0:4'  # eRtg A.-Haut-Gasse (Herrenberg Oberjesingen)
	ZABERSTRASSE = 'de:08115:4842'
	ZABERSTRASSE_1 = 'de:08115:4842:0:3'  # eRtg Grundschule/Calwer Str. (Herrenberg Oberjesingen)
	ZABERSTRASSE_2 = 'de:08115:4842:0:4'  # eRtg A.-Haut-Gasse (Herrenberg Oberjesingen)
	KUPPINGEN_A__HAUT_GASSE = 'de:08115:4844'
	KUPPINGEN_A__HAUT_GASSE_1 = 'de:08115:4844:0:3'  # Rtg Oberjesingen (Herrenberg Kuppingen)
	KUPPINGEN_A__HAUT_GASSE_2 = 'de:08115:4844:0:4'  # Rtg Affstätt (Herrenberg Kuppingen)
	ALBRECHT_HAUT_GASSE = 'de:08115:4844'
	ALBRECHT_HAUT_GASSE_1 = 'de:08115:4844:0:3'  # Rtg Oberjesingen (Herrenberg Kuppingen)
	ALBRECHT_HAUT_GASSE_2 = 'de:08115:4844:0:4'  # Rtg Affstätt (Herrenberg Kuppingen)
	KUPPINGEN_JENNERSTR = 'de:08115:4845'
	KUPPINGEN_JENNERSTR_1 = 'de:08115:4845:0:3'  # Rtg Albrecht-Haut-Gasse (Herrenberg Kuppingen)
	KUPPINGEN_JENNERSTR_2 = 'de:08115:4845:0:4'  # Rtg Affstätt Kirche (Herrenberg Kuppingen)
	JENNERSTRASSE = 'de:08115:4845'
	JENNERSTRASSE_1 = 'de:08115:4845:0:3'  # Rtg Albrecht-Haut-Gasse (Herrenberg Kuppingen)
	JENNERSTRASSE_2 = 'de:08115:4845:0:4'  # Rtg Affstätt Kirche (Herrenberg Kuppingen)
	AFFSTÄTT_KIRCHE = 'de:08115:4846:0:3'  # eRtg Nelkenstraße (Herrenberg Affstätt)
	AFFSTÄTT_RATHAUS = 'de:08115:4847:0:3'  # eRtg Jennerstraße (Herrenberg Affstätt)
	MÜHLWEG_1 = 'de:08115:4848:0:3'  # Rtg Affstätt Rathaus (Herrenberg Herrenberg)
	MÜHLWEG_2 = 'de:08115:4848:0:4'  # Rtg ZOB Kalkofenstr (Herrenberg Herrenberg)
	MÜHLWEG_3 = 'de:08119:5951:0:3'  # Rtg Beinstein Endersb. Str. (Waiblingen Beinstein)
	MÜHLWEG_4 = 'de:08119:5951:0:4'  # Rtg Waiblingen/Nachtbus Rtg Stetten (Waiblingen Beinstein)
	MÜHLWEG_5 = 'de:08118:6832:1:3'  # eRtg Schlosshof (Remseck am Neckar Aldingen)
	MÜHLWEG_6 = 'de:08118:6832:1:4'  # eRtg Goethestraße (Remseck am Neckar Aldingen)
	HERRENBERG_MÜHLWEG = 'de:08115:4848'
	HERRENBERG_MÜHLWEG_1 = 'de:08115:4848:0:3'  # Rtg Affstätt Rathaus (Herrenberg Herrenberg)
	HERRENBERG_MÜHLWEG_2 = 'de:08115:4848:0:4'  # Rtg ZOB Kalkofenstr (Herrenberg Herrenberg)
	HERRENBERG_SILCHERSTRASSE = 'de:08115:4850'
	HERRENBERG_SILCHERSTRASSE_1 = 'de:08115:4850:0:3'  # Rtg Nebringen (Herrenberg Herrenberg)
	HERRENBERG_SILCHERSTRASSE_2 = 'de:08115:4850:0:4'  # Rtg Bahnhof (Herrenberg Herrenberg)
	VOGT_HESS_SCHULE = 'de:08115:4851:0:3'  # eRtg Berliner Straße (Herrenberg Herrenberg)
	HERRENBERG_VOGT_HESS_SCHULE = 'de:08115:4851:0:3'  # eRtg Berliner Straße (Herrenberg Herrenberg)
	MÖNCHBERG_ABZW = 'de:08115:4853'
	MÖNCHBERG_ABZW_1 = 'de:08115:4853:0:3'  # Rtg Kayh Herrenberger Straße (Herrenberg Mönchberg)
	MÖNCHBERG_ABZW_2 = 'de:08115:4853:0:4'  # Rtg Weingartenstraße (Herrenberg Mönchberg)
	AFFSTÄTT_NELKENSTR = 'de:08115:4854:0:3'  # eRtg Mühlweg (Herrenberg Affstätt)
	NELKENSTRASSE = 'de:08115:4854:0:3'  # eRtg Mühlweg (Herrenberg Affstätt)
	HERRENBERG_STRASSENMEISTEREI = 'de:08115:4855'
	HERRENBERG_STRASSENMEISTEREI_1 = 'de:08115:4855:0:3'  # eRtg Nebringen/Tailfingen (Herrenberg Herrenberg)
	HERRENBERG_STRASSENMEISTEREI_2 = 'de:08115:4855:0:4'  # eRtg Bahnhofstraße (Herrenberg Herrenberg)
	SCHÖNAICH_SEESTRASSE = 'de:08115:4856'
	SCHÖNAICH_SEESTRASSE_1 = 'de:08115:4856:0:3'  # eRtg Schwabenstraße (Schönaich Schönaich)
	SCHÖNAICH_SEESTRASSE_2 = 'de:08115:4856:0:4'  # eRtg Mittlere Mühle (Schönaich Schönaich)
	BIRKENSTRASSE = 'de:08115:4857'
	BIRKENSTRASSE_1 = 'de:08115:4857:0:3'  # eRtg Schönberg (Holzgerlingen Holzgerlingen)
	BIRKENSTRASSE_2 = 'de:08115:4857:0:4'  # eRtg Bühlenstraße (Holzgerlingen Holzgerlingen)
	HOLZGERL__BIRKENSTRASSE = 'de:08115:4857'
	HOLZGERL__BIRKENSTRASSE_1 = 'de:08115:4857:0:3'  # eRtg Schönberg (Holzgerlingen Holzgerlingen)
	HOLZGERL__BIRKENSTRASSE_2 = 'de:08115:4857:0:4'  # eRtg Bühlenstraße (Holzgerlingen Holzgerlingen)
	SINDLINGEN_ORTSMITTE = 'de:08115:4858'
	SINDLINGEN_ORTSMITTE_1 = 'de:08115:4858:0:3'  # Rtg Unterjettingen (Jettingen Unterjettingen)
	SINDLINGEN_ORTSMITTE_2 = 'de:08115:4858:0:4'  # Rtg Sindlingen Aussiedlerhöfe (Jettingen Unterjettingen)
	UNTERJETT_SINDLINGEN_ORTSMITTE = 'de:08115:4858'
	UNTERJETT_SINDLINGEN_ORTSMITTE_1 = 'de:08115:4858:0:3'  # Rtg Unterjettingen (Jettingen Unterjettingen)
	UNTERJETT_SINDLINGEN_ORTSMITTE_2 = 'de:08115:4858:0:4'  # Rtg Sindlingen Aussiedlerhöfe (Jettingen Unterjettingen)
	UNTERJETT__EV__KIRCHE = 'de:08115:4859'
	UNTERJETT__EV__KIRCHE_1 = 'de:08115:4859:0:3'  # Rtg Altes Rathaus (Jettingen Unterjettingen)
	UNTERJETT__EV__KIRCHE_2 = 'de:08115:4859:0:4'  # Rtg Gemeindezentrum (Jettingen Unterjettingen)
	OBERJETT__GEMEINDEZENTRUM = 'de:08115:4860'
	OBERJETT__GEMEINDEZENTRUM_1 = 'de:08115:4860:0:3'  # Rtg Ev. Kirche Altes Rathaus (Jettingen Oberjettingen)
	OBERJETT__GEMEINDEZENTRUM_2 = 'de:08115:4860:0:4'  # Rtg Sindlinger/Herrenberger Str. (Jettingen Oberjettingen)
	OBERJETTINGEN_SINDLINGER_STR = 'de:08115:4861'
	OBERJETTINGEN_SINDLINGER_STR_1 = 'de:08115:4861:0:3'  # Rtg Gemeindezentrum (Jettingen Oberjettingen)
	OBERJETTINGEN_SINDLINGER_STR_2 = 'de:08115:4861:0:4'  # Rtg Herrenberger Straße (Jettingen Oberjettingen)
	TAILFINGEN_RATHAUS = 'de:08115:4862'
	TAILFINGEN_RATHAUS_1 = 'de:08115:4862:0:3'  # Rtg Altingen Kirche (Gäufelden Tailfingen)
	TAILFINGEN_RATHAUS_2 = 'de:08115:4862:0:4'  # Rtg Maueräcker (Gäufelden Tailfingen)
	UNTERJETT__ALTES_RATHAUS = 'de:08115:4863'
	UNTERJETT__ALTES_RATHAUS_1 = 'de:08115:4863:0:3'  # Rtg Ev. Kirche Oberj. Gemeindezentrum (Jettingen Unterjettingen)
	UNTERJETT__ALTES_RATHAUS_2 = 'de:08115:4863:0:4'  # Rtg Kohlplatte / Kath. Kirche (Jettingen Unterjettingen)
	IMENTAL = 'de:08115:4864'
	IMENTAL_1 = 'de:08115:4864:0:3'  # Rtg Eisberg B28 (Jettingen Unterjettingen)
	IMENTAL_2 = 'de:08115:4864:0:4'  # Rtg Kohlplatte (Jettingen Unterjettingen)
	UNTERJETTINGEN_IMENTAL = 'de:08115:4864'
	UNTERJETTINGEN_IMENTAL_1 = 'de:08115:4864:0:3'  # Rtg Eisberg B28 (Jettingen Unterjettingen)
	UNTERJETTINGEN_IMENTAL_2 = 'de:08115:4864:0:4'  # Rtg Kohlplatte (Jettingen Unterjettingen)
	UNTERJETTINGEN_MÖTZINGER_STR = 'de:08115:4865'
	UNTERJETTINGEN_MÖTZINGER_STR_1 = 'de:08115:4865:0:3'  # Rtg Öschelbronner Str./Immental (Jettingen Unterjettingen)
	UNTERJETTINGEN_MÖTZINGER_STR_2 = 'de:08115:4865:0:4'  # Rtg Lindenhof (Jettingen Unterjettingen)
	MÖTZINGER_STRASSE_1 = 'de:08115:4865:0:3'  # Rtg Öschelbronner Str./Immental (Jettingen Unterjettingen)
	MÖTZINGER_STRASSE_2 = 'de:08115:4865:0:4'  # Rtg Lindenhof (Jettingen Unterjettingen)
	MÖTZINGER_STRASSE_3 = 'de:08115:7928:0:3'  # Rtg Stuttgarter Str. (Gäufelden Öschelbronn (Gäufelden))
	MÖTZINGER_STRASSE_4 = 'de:08115:7928:0:4'  # Rtg Radsporthalle (Gäufelden Öschelbronn (Gäufelden))
	MÖTZINGER_STRASSE_5 = 'de:08235:10072:0:3'  # Rtg Mötzingen (Nagold Nagold)
	MÖTZINGER_STRASSE_6 = 'de:08235:10072:0:4'  # Rtg Waldlust (Nagold Nagold)
	MÖNCHBERG_WEINGARTENSTR = 'de:08115:4867:0:3'  # alle Richtungen (Herrenberg Mönchberg)
	WEINGARTENSTRASSE_1 = 'de:08115:4867:0:3'  # alle Richtungen (Herrenberg Mönchberg)
	WEINGARTENSTRASSE_2 = 'de:08117:3324:0:H1'  # Rtg Eschenbacher Str. (Gammelshausen Gammelshausen)
	KUPPINGEN_JETTINGER_STR = 'de:08115:4868'
	KUPPINGEN_JETTINGER_STR_1 = 'de:08115:4868:0:3'  # Rtg Sulz Weinbergstraße (Herrenberg Kuppingen)
	KUPPINGEN_JETTINGER_STR_2 = 'de:08115:4868:0:4'  # Rtg Jennerstraße (Herrenberg Kuppingen)
	JETTINGER_STRASSE = 'de:08115:4868'
	JETTINGER_STRASSE_1 = 'de:08115:4868:0:3'  # Rtg Sulz Weinbergstraße (Herrenberg Kuppingen)
	JETTINGER_STRASSE_2 = 'de:08115:4868:0:4'  # Rtg Jennerstraße (Herrenberg Kuppingen)
	LILIENSTRASSE_1 = 'de:08115:4872:0:3'  # eRtg Bühlenstraße (Holzgerlingen Holzgerlingen)
	LILIENSTRASSE_2 = 'de:08115:4872:0:4'  # eRtg Hohenzollernstraße (Holzgerlingen Holzgerlingen)
	LILIENSTRASSE_3 = 'de:08115:5850:0:3'  # Rtg Rathaus (Renningen Malmsheim)
	LILIENSTRASSE_4 = 'de:08115:5850:0:4'  # Rtg Renn. Schnallenäcker (Renningen Malmsheim)
	LILIENSTRASSE_5 = 'de:08119:6954:0:3'  # Rtg Unter den Arkaden/Karlstr. (Kernen im Remstal Rommelshausen)
	LILIENSTRASSE_6 = 'de:08119:6954:0:4'  # Rtg Rommelshausen Bf (Kernen im Remstal Rommelshausen)
	HOLZGERLINGEN_LILIENSTRASSE = 'de:08115:4872'
	HOLZGERLINGEN_LILIENSTRASSE_1 = 'de:08115:4872:0:3'  # eRtg Bühlenstraße (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_LILIENSTRASSE_2 = 'de:08115:4872:0:4'  # eRtg Hohenzollernstraße (Holzgerlingen Holzgerlingen)
	WAIBLINGEN_STAUFERSCHULE = 'de:08119:4874'
	WAIBLINGEN_STAUFERSCHULE_1 = 'de:08119:4874:0:3'  # eRtg Waiblingen Bf (Waiblingen Waiblingen)
	WAIBLINGEN_STAUFERSCHULE_2 = 'de:08119:4874:0:4'  # Rtg Untere Mayenner Str/Seniorenzentrum (Waiblingen Waiblingen)
	STAUFERSCHULE_1 = 'de:08119:4874:0:3'  # eRtg Waiblingen Bf (Waiblingen Waiblingen)
	STAUFERSCHULE_2 = 'de:08119:4874:0:4'  # Rtg Untere Mayenner Str/Seniorenzentrum (Waiblingen Waiblingen)
	STAUFERSCHULE_3 = 'de:08117:9911:0:H1'  # Rtg Wäscherhofstr. (Wäschenbeuren Wäschenbeuren)
	STAUFERSCHULE_4 = 'de:08136:7049:0:1'  # Rtg Sonne (Lorch Lorch)
	FRONACKERSTRASSE = 'de:08119:4875'
	FRONACKERSTRASSE_1 = 'de:08119:4875:1:3'  # Rtg Christofstr./Ludwigsburger Straße (Waiblingen Waiblingen)
	FRONACKERSTRASSE_2 = 'de:08119:4875:1:4'  # Rtg Waiblingen Bf (Waiblingen Waiblingen)
	WAIBLINGEN_FRONACKERSTR = 'de:08119:4875'
	WAIBLINGEN_FRONACKERSTR_1 = 'de:08119:4875:1:3'  # Rtg Christofstr./Ludwigsburger Straße (Waiblingen Waiblingen)
	WAIBLINGEN_FRONACKERSTR_2 = 'de:08119:4875:1:4'  # Rtg Waiblingen Bf (Waiblingen Waiblingen)
	WAIBLINGEN_LUDWIGSBURGER_STR = 'de:08119:4876'
	WAIBLINGEN_LUDWIGSBURGER_STR_1 = 'de:08119:4876:0:3'  # Rtg Fröbelstraße/Talstr. (Waiblingen Waiblingen)
	WAIBLINGEN_LUDWIGSBURGER_STR_2 = 'de:08119:4876:0:4'  # eRtg Christofstraße (Waiblingen Waiblingen)
	WAIBLINGEN_HEGNACHER_HÖHE = 'de:08119:4877'
	WAIBLINGEN_HEGNACHER_HÖHE_1 = 'de:08119:4877:0:3'  # Rtg Hegnach Neckarstr. (Waiblingen Waiblingen)
	WAIBLINGEN_HEGNACHER_HÖHE_2 = 'de:08119:4877:0:4'  # Rtg Waiblingen Talstr. (Waiblingen Waiblingen)
	HEGNACHER_HÖHE = 'de:08119:4877'
	HEGNACHER_HÖHE_1 = 'de:08119:4877:0:3'  # Rtg Hegnach Neckarstr. (Waiblingen Waiblingen)
	HEGNACHER_HÖHE_2 = 'de:08119:4877:0:4'  # Rtg Waiblingen Talstr. (Waiblingen Waiblingen)
	NECKARSTRASSE_1 = 'de:08119:4878:0:3'  # Rtg Hegnach Aldinger Str. (Waiblingen Hegnach)
	NECKARSTRASSE_2 = 'de:08119:4878:0:4'  # Rtg Waiblingen Hegnacher Str. (Waiblingen Hegnach)
	NECKARSTRASSE_3 = 'de:08118:5444:0:3'  # eRtg Enzstraße (Kornwestheim Kornwestheim)
	NECKARSTRASSE_4 = 'de:08118:5444:0:4'  # eRtg Goerdelerstraße (Kornwestheim Kornwestheim)
	NECKARSTRASSE_5 = 'de:08118:5478:0:3'  # eRtg Saarstraße (Asperg Asperg)
	NECKARSTRASSE_6 = 'de:08118:5478:0:4'  # eRtg Im Waldeck (Asperg Asperg)
	NECKARSTRASSE_7 = 'de:08118:5514:0:3'  # eRtg Comburgstraße (Ludwigsburg Ludwigsburg)
	NECKARSTRASSE_8 = 'de:08118:5514:0:4'  # eRtg Schorndorfer Tor (Ludwigsburg Ludwigsburg)
	HEGNACH_NECKARSTRASSE = 'de:08119:4878'
	HEGNACH_NECKARSTRASSE_1 = 'de:08119:4878:0:3'  # Rtg Hegnach Aldinger Str. (Waiblingen Hegnach)
	HEGNACH_NECKARSTRASSE_2 = 'de:08119:4878:0:4'  # Rtg Waiblingen Hegnacher Str. (Waiblingen Hegnach)
	HEGNACH_RATHAUS = 'de:08119:4879:0:3'  # eRtg Katholische Kirche (Waiblingen Hegnach)
	HOLZGERL__GERMANENSTRASSE = 'de:08115:4880'
	HOLZGERL__GERMANENSTRASSE_1 = 'de:08115:4880:0:3'  # eRtg Hohenzollernstraße/Böblingen (Holzgerlingen Holzgerlingen)
	HOLZGERL__GERMANENSTRASSE_2 = 'de:08115:4880:0:4'  # eRtg Rathaus (Holzgerlingen Holzgerlingen)
	GERMANENSTRASSE = 'de:08115:4880'
	GERMANENSTRASSE_1 = 'de:08115:4880:0:3'  # eRtg Hohenzollernstraße/Böblingen (Holzgerlingen Holzgerlingen)
	GERMANENSTRASSE_2 = 'de:08115:4880:0:4'  # eRtg Rathaus (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_STÄUCHLE = 'de:08115:4881'
	HOLZGERLINGEN_STÄUCHLE_1 = 'de:08115:4881:0:3'  # Rtg Bahnhof (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_STÄUCHLE_2 = 'de:08115:4881:0:4'  # Rtg Talstr. (Holzgerlingen Holzgerlingen)
	STÄUCHLE = 'de:08115:4881'
	STÄUCHLE_1 = 'de:08115:4881:0:3'  # Rtg Bahnhof (Holzgerlingen Holzgerlingen)
	STÄUCHLE_2 = 'de:08115:4881:0:4'  # Rtg Talstr. (Holzgerlingen Holzgerlingen)
	HEGNACH_HAUPTSTR = 'de:08119:4882:0:3'  # eRtg Rathaus (Waiblingen Hegnach)
	NECKARREMS_DORFSTRASSE = 'de:08118:4883'
	NECKARREMS_DORFSTRASSE_1 = 'de:08118:4883:0:3'  # Rtg Rathaus (Remseck am Neckar Neckarrems)
	NECKARREMS_DORFSTRASSE_2 = 'de:08118:4883:0:4'  # Rtg Kelterschule (Remseck am Neckar Neckarrems)
	DORFSTRASSE_1 = 'de:08118:4883:0:3'  # Rtg Rathaus (Remseck am Neckar Neckarrems)
	DORFSTRASSE_2 = 'de:08118:4883:0:4'  # Rtg Kelterschule (Remseck am Neckar Neckarrems)
	DORFSTRASSE_3 = 'de:08118:5496:0:3'  # eRtg Robert-Bosch-Straße (Ludwigsburg Pflugfelden)
	DORFSTRASSE_4 = 'de:08117:7403:0:H1'  # Rtg Schlierbach (Ebersbach an der Fils Roßwälden)
	DORFSTRASSE_5 = 'de:08117:7403:0:R1'  # Rtg Ebersbach (Ebersbach an der Fils Roßwälden)
	NECKARREMS_NECKARHALDE = 'de:08118:4885'
	NECKARREMS_NECKARHALDE_1 = 'de:08118:4885:0:3'  # eRtg Remstalstraße/Hochberg (Remseck am Neckar Neckarrems)
	NECKARREMS_NECKARHALDE_2 = 'de:08118:4885:0:4'  # eRtg Rathaus (Remseck am Neckar Neckarrems)
	ALDINGEN_NECKARSCHULE = 'de:08118:4886:1:4'  # eRtg Neckarstraße (Remseck am Neckar Aldingen)
	NECKARSCHULE = 'de:08118:4886:1:4'  # eRtg Neckarstraße (Remseck am Neckar Aldingen)
	REMSECK_RATHAUS_STADTHALLE = 'de:08118:4887'
	REMSECK_RATHAUS_STADTHALLE_1 = 'de:08118:4887:0:3'  # eRtg Steige (Remseck am Neckar Neckarrems)
	REMSECK_RATHAUS_STADTHALLE_2 = 'de:08118:4887:0:4'  # eRtg Wasenstraße/Stadtbahn (Remseck am Neckar Neckarrems)
	RATHAUS_STADTHALLE = 'de:08118:4887'
	RATHAUS_STADTHALLE_1 = 'de:08118:4887:0:3'  # eRtg Steige (Remseck am Neckar Neckarrems)
	RATHAUS_STADTHALLE_2 = 'de:08118:4887:0:4'  # eRtg Wasenstraße/Stadtbahn (Remseck am Neckar Neckarrems)
	SCHLOSSHOF_1 = 'de:08118:4888:1:3'  # eRtg Cannstatter Straße (Remseck am Neckar Aldingen)
	SCHLOSSHOF_2 = 'de:08118:4888:1:4'  # eRtg Neckarstraße (Remseck am Neckar Aldingen)
	SCHLOSSHOF_3 = 'de:08119:5304:0:3'  # eRtg Mettelberg (Murrhardt Fornsbach)
	ALDINGEN_SCHLOSSHOF = 'de:08118:4888'
	ALDINGEN_SCHLOSSHOF_1 = 'de:08118:4888:1:3'  # eRtg Cannstatter Straße (Remseck am Neckar Aldingen)
	ALDINGEN_SCHLOSSHOF_2 = 'de:08118:4888:1:4'  # eRtg Neckarstraße (Remseck am Neckar Aldingen)
	NECKARGRÖNINGEN_WASENSTR = 'de:08118:4889'
	NECKARGRÖNINGEN_WASENSTR_1 = 'de:08118:4889:1:3'  # eRtg Stadtbahn/Rathaus/Waiblingen (Remseck am Neckar Neckargröningen)
	NECKARGRÖNINGEN_WASENSTR_2 = 'de:08118:4889:1:4'  # eRtg Lamm (Remseck am Neckar Neckargröningen)
	LAMM_1 = 'de:08118:4890:1:3'  # eRtg Wasenstraße (Remseck am Neckar Neckargröningen)
	LAMM_2 = 'de:08118:4890:1:4'  # eRtg Bank (Remseck am Neckar Neckargröningen)
	LAMM_3 = 'de:08117:3504:0:H1'  # Rtg Einkaufszentrum (Bad Ditzenbach Gosbach)
	LAMM_4 = 'de:08117:3504:0:R1'  # Rtg Hirsch (Bad Ditzenbach Gosbach)
	LAMM_5 = 'de:08117:7675:0:R1'  # Rtg Sonne (Böhmenkirch Treffelhausen)
	NECKARGRÖNINGEN_LAMM = 'de:08118:4890'
	NECKARGRÖNINGEN_LAMM_1 = 'de:08118:4890:1:3'  # eRtg Wasenstraße (Remseck am Neckar Neckargröningen)
	NECKARGRÖNINGEN_LAMM_2 = 'de:08118:4890:1:4'  # eRtg Bank (Remseck am Neckar Neckargröningen)
	NECKARGRÖNINGEN_BANK = 'de:08118:4891'
	NECKARGRÖNINGEN_BANK_1 = 'de:08118:4891:0:3'  # Rtg Lamm (Remseck am Neckar Neckargröningen)
	NECKARGRÖNINGEN_BANK_2 = 'de:08118:4891:0:4'  # Rtg Schießtal/Neckarstr./Ostfriedhof (Remseck am Neckar Neckargröningen)
	BANK = 'de:08118:4891'
	BANK_1 = 'de:08118:4891:0:3'  # Rtg Lamm (Remseck am Neckar Neckargröningen)
	BANK_2 = 'de:08118:4891:0:4'  # Rtg Schießtal/Neckarstr./Ostfriedhof (Remseck am Neckar Neckargröningen)
	WESTFALENSTRASSE = 'de:08118:4892'
	WESTFALENSTRASSE_1 = 'de:08118:4892:0:3'  # eRtg Mauserstraße/Hirschstraße (Ludwigsburg Oßweil)
	WESTFALENSTRASSE_2 = 'de:08118:4892:0:4'  # eRtg Neckarweihinger Straße (Ludwigsburg Oßweil)
	OSSWEIL_WESTFALENSTR = 'de:08118:4892'
	OSSWEIL_WESTFALENSTR_1 = 'de:08118:4892:0:3'  # eRtg Mauserstraße/Hirschstraße (Ludwigsburg Oßweil)
	OSSWEIL_WESTFALENSTR_2 = 'de:08118:4892:0:4'  # eRtg Neckarweihinger Straße (Ludwigsburg Oßweil)
	LUDWIGSBURG_COMBURGSTR = 'de:08118:4893'
	LUDWIGSBURG_COMBURGSTR_1 = 'de:08118:4893:0:3'  # Rtg Friesenstr. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_COMBURGSTR_2 = 'de:08118:4893:0:4'  # Rtg Neckarstr. (Ludwigsburg Ludwigsburg)
	COMBURGSTRASSE = 'de:08118:4893'
	COMBURGSTRASSE_1 = 'de:08118:4893:0:3'  # Rtg Friesenstr. (Ludwigsburg Ludwigsburg)
	COMBURGSTRASSE_2 = 'de:08118:4893:0:4'  # Rtg Neckarstr. (Ludwigsburg Ludwigsburg)
	SCHORNDORFER_TOR = 'de:08118:4894'
	SCHORNDORFER_TOR_1 = 'de:08118:4894:0:3'  # Rtg Blühendes Barock (Ludwigsburg Ludwigsburg)
	SCHORNDORFER_TOR_2 = 'de:08118:4894:0:4'  # Rtg Klinikum/Neckarstr. (Ludwigsburg Ludwigsburg)
	LUDW__SCHORNDORFER_TOR = 'de:08118:4894'
	LUDW__SCHORNDORFER_TOR_1 = 'de:08118:4894:0:3'  # Rtg Blühendes Barock (Ludwigsburg Ludwigsburg)
	LUDW__SCHORNDORFER_TOR_2 = 'de:08118:4894:0:4'  # Rtg Klinikum/Neckarstr. (Ludwigsburg Ludwigsburg)
	WINNENDEN_H__SCHWAB_HALLE = 'de:08119:4896'
	WINNENDEN_H__SCHWAB_HALLE_1 = 'de:08119:4896:1:3'  # Rtg Kelter (Winnenden Winnenden)
	WINNENDEN_H__SCHWAB_HALLE_2 = 'de:08119:4896:1:4'  # Rtg Wunnebad (Winnenden Winnenden)
	HERMANN_SCHWAB_HALLE = 'de:08119:4896'
	HERMANN_SCHWAB_HALLE_1 = 'de:08119:4896:1:3'  # Rtg Kelter (Winnenden Winnenden)
	HERMANN_SCHWAB_HALLE_2 = 'de:08119:4896:1:4'  # Rtg Wunnebad (Winnenden Winnenden)
	STRÜMPF__KATHARINENHOF = 'de:08119:4898'
	STRÜMPF__KATHARINENHOF_1 = 'de:08119:4898:0:3'  # eRtg Post (Backnang Strümpfelbach (Backnang))
	STRÜMPF__KATHARINENHOF_2 = 'de:08119:4898:0:4'  # eRtg Strümpfelbach (Backnang Strümpfelbach (Backnang))
	KATHARINENHOF = 'de:08119:4898'
	KATHARINENHOF_1 = 'de:08119:4898:0:3'  # eRtg Post (Backnang Strümpfelbach (Backnang))
	KATHARINENHOF_2 = 'de:08119:4898:0:4'  # eRtg Strümpfelbach (Backnang Strümpfelbach (Backnang))
	OPPENWEILER_POST = 'de:08119:4899:0:3'  # eRtg Ortsmitte (Oppenweiler Oppenweiler)
	OPPENWEILER_ORTSMITTE = 'de:08119:4901'
	OPPENWEILER_ORTSMITTE_1 = 'de:08119:4901:1:3'  # eRtg Murrtalschule (Oppenweiler Oppenweiler)
	OPPENWEILER_ORTSMITTE_2 = 'de:08119:4901:1:4'  # eRtg Bühlfeld (Oppenweiler Oppenweiler)
	OPPENWEILER_REICHENBERG = 'de:08119:4902'
	OPPENWEILER_REICHENBERG_1 = 'de:08119:4902:0:3'  # Rtg Murrhardt (Oppenweiler Oppenweiler)
	OPPENWEILER_REICHENBERG_2 = 'de:08119:4902:0:4'  # Rtg Oppenweiler (Oppenweiler Oppenweiler)
	REICHENBERG = 'de:08119:4902'
	REICHENBERG_1 = 'de:08119:4902:0:3'  # Rtg Murrhardt (Oppenweiler Oppenweiler)
	REICHENBERG_2 = 'de:08119:4902:0:4'  # Rtg Oppenweiler (Oppenweiler Oppenweiler)
	ELLENWEILER = 'de:08119:4903'
	ELLENWEILER_1 = 'de:08119:4903:0:3'  # Rtg Murrhardt (Oppenweiler Oppenweiler)
	ELLENWEILER_2 = 'de:08119:4903:0:4'  # Rtg Oppenweiler (Oppenweiler Oppenweiler)
	OPPENWEILER_ELLENWEILER = 'de:08119:4903'
	OPPENWEILER_ELLENWEILER_1 = 'de:08119:4903:0:3'  # Rtg Murrhardt (Oppenweiler Oppenweiler)
	OPPENWEILER_ELLENWEILER_2 = 'de:08119:4903:0:4'  # Rtg Oppenweiler (Oppenweiler Oppenweiler)
	SCHLOSS_LAUTERECK = 'de:08119:4904'
	SCHLOSS_LAUTERECK_1 = 'de:08119:4904:0:3'  # eRtg Marktplatz/Bf (Sulzbach an der Murr Sulzbach (M))
	SCHLOSS_LAUTERECK_2 = 'de:08119:4904:0:4'  # eRtg Ellenweiler/Lautertal (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__SCHLOSS_LAUTERECK = 'de:08119:4904'
	SULZB__M__SCHLOSS_LAUTERECK_1 = 'de:08119:4904:0:3'  # eRtg Marktplatz/Bf (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__SCHLOSS_LAUTERECK_2 = 'de:08119:4904:0:4'  # eRtg Ellenweiler/Lautertal (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__MARKTPLATZ = 'de:08119:4905'
	SULZBACH__M__MARKTPLATZ_1 = 'de:08119:4905:1:3'  # Marktplatz (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__MARKTPLATZ_2 = 'de:08119:4905:1:4'  # Marktplatz (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__MARKTPLATZ_3 = 'de:08119:4905:1:5'  # Marktplatz (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__MARKTPLATZ_4 = 'de:08119:4905:1:6'  # Marktplatz (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__BERWINKEL = 'de:08119:4906'
	SULZB__M__BERWINKEL_1 = 'de:08119:4906:0:3'  # eRtg Marktplatz (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__BERWINKEL_2 = 'de:08119:4906:0:4'  # eRtg Erlach Abzweig (Sulzbach an der Murr Sulzbach (M))
	BERWINKEL = 'de:08119:4906'
	BERWINKEL_1 = 'de:08119:4906:0:3'  # eRtg Marktplatz (Sulzbach an der Murr Sulzbach (M))
	BERWINKEL_2 = 'de:08119:4906:0:4'  # eRtg Erlach Abzweig (Sulzbach an der Murr Sulzbach (M))
	GROSSERL__ABZW__NEUFÜRSTENHÜTTE = 'de:08119:4908'
	GROSSERL__ABZW__NEUFÜRSTENHÜTTE_1 = 'de:08119:4908:1:3'  # Rtg Mainhardt (Großerlach Großerlach)
	GROSSERL__ABZW__NEUFÜRSTENHÜTTE_2 = 'de:08119:4908:1:4'  # Rtg Backnang (Großerlach Großerlach)
	GROSSERL__ABZW__NEUFÜRSTENHÜTTE_3 = 'de:08119:4908:1:5'  # eRtg Mainhardt (Großerlach Großerlach)
	GROSSERL__ABZW__NEUFÜRSTENHÜTTE_4 = 'de:08119:4908:1:6'  # Rtg Großerlach Rathaus (Großerlach Großerlach)
	ABZWEIG_NEUFÜRSTENHÜTTE = 'de:08119:4908'
	ABZWEIG_NEUFÜRSTENHÜTTE_1 = 'de:08119:4908:1:3'  # Rtg Mainhardt (Großerlach Großerlach)
	ABZWEIG_NEUFÜRSTENHÜTTE_2 = 'de:08119:4908:1:4'  # Rtg Backnang (Großerlach Großerlach)
	ABZWEIG_NEUFÜRSTENHÜTTE_3 = 'de:08119:4908:1:5'  # eRtg Mainhardt (Großerlach Großerlach)
	ABZWEIG_NEUFÜRSTENHÜTTE_4 = 'de:08119:4908:1:6'  # Rtg Großerlach Rathaus (Großerlach Großerlach)
	MURRHARDTER_STRASSE = 'de:08119:4910'
	MURRHARDTER_STRASSE_1 = 'de:08119:4910:0:3'  # Rtg Murrhardt (Sulzbach an der Murr Sulzbach (M))
	MURRHARDTER_STRASSE_2 = 'de:08119:4910:0:4'  # Rtg Backnang (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__MURRHARDTER_STR = 'de:08119:4910'
	SULZBACH__M__MURRHARDTER_STR_1 = 'de:08119:4910:0:3'  # Rtg Murrhardt (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__MURRHARDTER_STR_2 = 'de:08119:4910:0:4'  # Rtg Backnang (Sulzbach an der Murr Sulzbach (M))
	HUMMELBÜHL = 'de:08119:4911'
	HUMMELBÜHL_1 = 'de:08119:4911:0:3'  # eRtg Murrhardter Straße (Sulzbach an der Murr Sulzbach (M))
	HUMMELBÜHL_2 = 'de:08119:4911:0:4'  # eRtg Bartenbach (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__HUMMELBÜHL = 'de:08119:4911'
	SULZBACH__M__HUMMELBÜHL_1 = 'de:08119:4911:0:3'  # eRtg Murrhardter Straße (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__HUMMELBÜHL_2 = 'de:08119:4911:0:4'  # eRtg Bartenbach (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__BARTENBACH = 'de:08119:4912'
	SULZB__M__BARTENBACH_1 = 'de:08119:4912:0:3'  # Rtg Murrhardt (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__BARTENBACH_2 = 'de:08119:4912:0:4'  # Rtg Sulzbach (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__BARTENBACH_3 = 'de:08119:4912:0:5'  # Linie 387 (Sulzbach an der Murr Sulzbach (M))
	BARTENBACH = 'de:08119:4912'
	BARTENBACH_1 = 'de:08119:4912:0:3'  # Rtg Murrhardt (Sulzbach an der Murr Sulzbach (M))
	BARTENBACH_2 = 'de:08119:4912:0:4'  # Rtg Sulzbach (Sulzbach an der Murr Sulzbach (M))
	BARTENBACH_3 = 'de:08119:4912:0:5'  # Linie 387 (Sulzbach an der Murr Sulzbach (M))
	MURRHARDT_ÄRZTEHAUS = 'de:08119:4913'
	MURRHARDT_ÄRZTEHAUS_1 = 'de:08119:4913:0:3'  # eRtg Murrhardt (Murrhardt Murrhardt)
	MURRHARDT_ÄRZTEHAUS_2 = 'de:08119:4913:0:4'  # eRtg Fornsbach (Murrhardt Murrhardt)
	ÄRZTEHAUS = 'de:08119:4913'
	ÄRZTEHAUS_1 = 'de:08119:4913:0:3'  # eRtg Murrhardt (Murrhardt Murrhardt)
	ÄRZTEHAUS_2 = 'de:08119:4913:0:4'  # eRtg Fornsbach (Murrhardt Murrhardt)
	MURRH__BODELSCHWINGHSCHULE = 'de:08119:4914'
	MURRH__BODELSCHWINGHSCHULE_1 = 'de:08119:4914:0:3'  # eRtg Hörschbachschule (Murrhardt Murrhardt)
	MURRH__BODELSCHWINGHSCHULE_2 = 'de:08119:4914:0:4'  # eRtg Harbach (Murrhardt Murrhardt)
	BODELSCHWINGHSCHULE = 'de:08119:4914'
	BODELSCHWINGHSCHULE_1 = 'de:08119:4914:0:3'  # eRtg Hörschbachschule (Murrhardt Murrhardt)
	BODELSCHWINGHSCHULE_2 = 'de:08119:4914:0:4'  # eRtg Harbach (Murrhardt Murrhardt)
	MURRH__HÖRSCHBACHSCHULE = 'de:08119:4915'
	MURRH__HÖRSCHBACHSCHULE_1 = 'de:08119:4915:0:3'  # Rtg Ost (Murrhardt Murrhardt)
	MURRH__HÖRSCHBACHSCHULE_2 = 'de:08119:4915:0:4'  # Rtg West (Murrhardt Murrhardt)
	HÖRSCHBACHSCHULE = 'de:08119:4915'
	HÖRSCHBACHSCHULE_1 = 'de:08119:4915:0:3'  # Rtg Ost (Murrhardt Murrhardt)
	HÖRSCHBACHSCHULE_2 = 'de:08119:4915:0:4'  # Rtg West (Murrhardt Murrhardt)
	MURRHARDT = 'de:08119:4916'
	MURRHARDT_1 = 'de:08119:4916:1:3'  # Gleis 3 (Murrhardt Murrhardt)
	MURRHARDT_2 = 'de:08119:4916:2:2'  # Gleis 2 (Murrhardt Murrhardt)
	MURRHARDT_3 = 'de:08119:4916:3:1'  # Bstg 1 (Murrhardt Murrhardt)
	MURRHARDT_4 = 'de:08119:4916:3:2'  # Bstg 2 (Murrhardt Murrhardt)
	ERICH_SCHUMM_STRASSE = 'de:08119:4917'
	ERICH_SCHUMM_STRASSE_1 = 'de:08119:4917:0:3'  # eRtg Gymnasium/Siegelsberg (Murrhardt Murrhardt)
	ERICH_SCHUMM_STRASSE_2 = 'de:08119:4917:0:4'  # eRtg Murrhardt (Murrhardt Murrhardt)
	MURRH__ERICH_SCHUMM_STRASSE = 'de:08119:4917'
	MURRH__ERICH_SCHUMM_STRASSE_1 = 'de:08119:4917:0:3'  # eRtg Gymnasium/Siegelsberg (Murrhardt Murrhardt)
	MURRH__ERICH_SCHUMM_STRASSE_2 = 'de:08119:4917:0:4'  # eRtg Murrhardt (Murrhardt Murrhardt)
	MURRHARDT_GYMNASIUM = 'de:08119:4918:0:3'  # eRtg Murrhardt (Murrhardt Murrhardt)
	MURRHARDT_SIEGELSBERG = 'de:08119:4919:0:3'  # eRtg Alle Richtungen (Murrhardt Murrhardt)
	SIEGELSBERG = 'de:08119:4919:0:3'  # eRtg Alle Richtungen (Murrhardt Murrhardt)
	ALM = 'de:08119:4922'
	ALM_1 = 'de:08119:4922:0:3'  # eRtg Hausen (Murrhardt Murrhardt)
	ALM_2 = 'de:08119:4922:0:4'  # eRtg Theodor-Heuss-Straße (Murrhardt Murrhardt)
	MURRHARDT_ALM = 'de:08119:4922'
	MURRHARDT_ALM_1 = 'de:08119:4922:0:3'  # eRtg Hausen (Murrhardt Murrhardt)
	MURRHARDT_ALM_2 = 'de:08119:4922:0:4'  # eRtg Theodor-Heuss-Straße (Murrhardt Murrhardt)
	HAUSEN = 'de:08119:4923'
	HAUSEN_1 = 'de:08119:4923:0:3'  # Rtg Fornsbach (Murrhardt Murrhardt)
	HAUSEN_2 = 'de:08119:4923:0:4'  # Rtg Murrhardt (Murrhardt Murrhardt)
	MURRHARDT_HAUSEN = 'de:08119:4923'
	MURRHARDT_HAUSEN_1 = 'de:08119:4923:0:3'  # Rtg Fornsbach (Murrhardt Murrhardt)
	MURRHARDT_HAUSEN_2 = 'de:08119:4923:0:4'  # Rtg Murrhardt (Murrhardt Murrhardt)
	MURRH__EISENSCHMIEDMÜHLE = 'de:08119:4924'
	MURRH__EISENSCHMIEDMÜHLE_1 = 'de:08119:4924:0:3'  # eRtg Waldsee (Murrhardt Murrhardt)
	MURRH__EISENSCHMIEDMÜHLE_2 = 'de:08119:4924:0:4'  # eRtg Hausen (Murrhardt Murrhardt)
	EISENSCHMIEDMÜHLE = 'de:08119:4924'
	EISENSCHMIEDMÜHLE_1 = 'de:08119:4924:0:3'  # eRtg Waldsee (Murrhardt Murrhardt)
	EISENSCHMIEDMÜHLE_2 = 'de:08119:4924:0:4'  # eRtg Hausen (Murrhardt Murrhardt)
	FORNSBACH_MARKTPLATZ = 'de:08119:4925'
	FORNSBACH_MARKTPLATZ_1 = 'de:08119:4925:1:3'  # eRtg Alte Straße (Murrhardt Fornsbach)
	FORNSBACH_MARKTPLATZ_2 = 'de:08119:4925:1:4'  # eRtg Kindergarten (Murrhardt Fornsbach)
	FORNSBACH_KINDERGARTEN = 'de:08119:4926'
	FORNSBACH_KINDERGARTEN_1 = 'de:08119:4926:0:3'  # eRtg Marktplatz (Murrhardt Fornsbach)
	FORNSBACH_KINDERGARTEN_2 = 'de:08119:4926:0:4'  # eRtg L 1066 (Murrhardt Fornsbach)
	FORNSBACH_ALTE_STRASSE = 'de:08119:4927:0:3'  # eRtg Murrhardt/Kirchenkirnbg./Hammerschm (Murrhardt Fornsbach)
	ALTE_STRASSE = 'de:08119:4927:0:3'  # eRtg Murrhardt/Kirchenkirnbg./Hammerschm (Murrhardt Fornsbach)
	STETTEN_I__R__KELTER = 'de:08119:4928:0:4'  # eRtg Evangelische Kirche (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__EV__KIRCHE = 'de:08119:4929:0:3'  # eRtg Stettener Bank (Kernen im Remstal Stetten (i. R.))
	KEGELPLATZ = 'de:08119:4930:0:3'  # Rtg Kelter/Esslinger Str. (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__KEGELPLATZ = 'de:08119:4930:0:3'  # Rtg Kelter/Esslinger Str. (Kernen im Remstal Stetten (i. R.))
	KLOSTERSTRASSE = 'de:08119:4931:0:3'  # Rtg Diakonie/Kegelplatz (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__KLOSTERSTR = 'de:08119:4931:0:3'  # Rtg Diakonie/Kegelplatz (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__ZUM_KL__FELDLE = 'de:08119:4932:0:3'  # Rtg Kegelplatz/Lange Str. (Kernen im Remstal Stetten (i. R.))
	ZUM_KLEINEN_FELDLE = 'de:08119:4932:0:3'  # Rtg Kegelplatz/Lange Str. (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__DIAKONIE = 'de:08119:4933'
	STETTEN_I__R__DIAKONIE_1 = 'de:08119:4933:0:3'  # eRtg Zum Kleinen Feldle (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__DIAKONIE_2 = 'de:08119:4933:0:4'  # eRtg Endersbacher Straße (Kernen im Remstal Stetten (i. R.))
	DIAKONIE = 'de:08119:4933'
	DIAKONIE_1 = 'de:08119:4933:0:3'  # eRtg Zum Kleinen Feldle (Kernen im Remstal Stetten (i. R.))
	DIAKONIE_2 = 'de:08119:4933:0:4'  # eRtg Endersbacher Straße (Kernen im Remstal Stetten (i. R.))
	ENDERSBACHER_STRASSE_1 = 'de:08119:4934:0:3'  # eRtg Diakonie (Kernen im Remstal Stetten (i. R.))
	ENDERSBACHER_STRASSE_2 = 'de:08119:4934:0:4'  # eRtg Hangweide (Kernen im Remstal Stetten (i. R.))
	ENDERSBACHER_STRASSE_3 = 'de:08119:4934:0:5'  # Linie 219 Rtg Bühläckerstr (Kernen im Remstal Stetten (i. R.))
	ENDERSBACHER_STRASSE_4 = 'de:08119:4934:0:6'  # Linie 219 Rtg Endersbach (Kernen im Remstal Stetten (i. R.))
	ENDERSBACHER_STRASSE_5 = 'de:08119:5165:0:3'  # Rtg Rathaus (Waiblingen Beinstein)
	ENDERSBACHER_STRASSE_6 = 'de:08119:5165:0:4'  # Rtg Kalkofen (Waiblingen Beinstein)
	STETTEN_I__R__ENDERSB__STR = 'de:08119:4934'
	STETTEN_I__R__ENDERSB__STR_1 = 'de:08119:4934:0:3'  # eRtg Diakonie (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__ENDERSB__STR_2 = 'de:08119:4934:0:4'  # eRtg Hangweide (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__ENDERSB__STR_3 = 'de:08119:4934:0:5'  # Linie 219 Rtg Bühläckerstr (Kernen im Remstal Stetten (i. R.))
	STETTEN_I__R__ENDERSB__STR_4 = 'de:08119:4934:0:6'  # Linie 219 Rtg Endersbach (Kernen im Remstal Stetten (i. R.))
	ROMMELSH__HANGWEIDE = 'de:08119:4935'
	ROMMELSH__HANGWEIDE_1 = 'de:08119:4935:0:3'  # Rtg Stetten Endersbacher Str. (Kernen im Remstal Rommelshausen)
	ROMMELSH__HANGWEIDE_2 = 'de:08119:4935:0:4'  # eRtg Rumold-Realschule (Kernen im Remstal Rommelshausen)
	HANGWEIDE = 'de:08119:4935'
	HANGWEIDE_1 = 'de:08119:4935:0:3'  # Rtg Stetten Endersbacher Str. (Kernen im Remstal Rommelshausen)
	HANGWEIDE_2 = 'de:08119:4935:0:4'  # eRtg Rumold-Realschule (Kernen im Remstal Rommelshausen)
	RUMOLD_REALSCHULE = 'de:08119:4936'
	RUMOLD_REALSCHULE_1 = 'de:08119:4936:1:3'  # Rtg Hangweide (Kernen im Remstal Rommelshausen)
	RUMOLD_REALSCHULE_2 = 'de:08119:4936:1:4'  # Rtg Karlstr. (Kernen im Remstal Rommelshausen)
	ROMMELSH__RUMOLD_REALSCHULE = 'de:08119:4936'
	ROMMELSH__RUMOLD_REALSCHULE_1 = 'de:08119:4936:1:3'  # Rtg Hangweide (Kernen im Remstal Rommelshausen)
	ROMMELSH__RUMOLD_REALSCHULE_2 = 'de:08119:4936:1:4'  # Rtg Karlstr. (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_SCHAFSTR = 'de:08119:4938:0:3'  # eRtg Rumold-Realschule (Kernen im Remstal Rommelshausen)
	SCHAFSTRASSE = 'de:08119:4938:0:3'  # eRtg Rumold-Realschule (Kernen im Remstal Rommelshausen)
	ROMMELSH__UNTER_DEN_ARKADEN = 'de:08119:4939:0:3'  # eRtg Seestraße (Kernen im Remstal Rommelshausen)
	UNTER_DEN_ARKADEN = 'de:08119:4939:0:3'  # eRtg Seestraße (Kernen im Remstal Rommelshausen)
	WAIBLINGEN_SOMMERHALDE = 'de:08119:4940'
	WAIBLINGEN_SOMMERHALDE_1 = 'de:08119:4940:0:3'  # Rtg Rommelshausen Bf (Waiblingen Waiblingen)
	WAIBLINGEN_SOMMERHALDE_2 = 'de:08119:4940:0:4'  # Rtg Alte Bundesstr./Neue Rommelshr Str. (Waiblingen Waiblingen)
	SOMMERHALDE = 'de:08119:4940'
	SOMMERHALDE_1 = 'de:08119:4940:0:3'  # Rtg Rommelshausen Bf (Waiblingen Waiblingen)
	SOMMERHALDE_2 = 'de:08119:4940:0:4'  # Rtg Alte Bundesstr./Neue Rommelshr Str. (Waiblingen Waiblingen)
	LÖWENKELLER = 'de:08119:4943:0:3'  # eRtg SOS Kinderdorf (Schorndorf Schorndorf)
	SCHORNDORF_LÖWENKELLER = 'de:08119:4943:0:3'  # eRtg SOS Kinderdorf (Schorndorf Schorndorf)
	MURRHARDT_LISZTSTRASSE = 'de:08119:4945:0:3'  # eRtg Alm/Hausen (Murrhardt Murrhardt)
	LISZTSTRASSE = 'de:08119:4945:0:3'  # eRtg Alm/Hausen (Murrhardt Murrhardt)
	HÖHENWEG = 'de:08119:4946:0:4'  # eRtg Schurwaldhalle/Backhaus (Schorndorf Oberberken)
	OBERBERKEN_HÖHENWEG = 'de:08119:4946:0:4'  # eRtg Schurwaldhalle/Backhaus (Schorndorf Oberberken)
	OBERBERKEN_BACKHAUS = 'de:08119:4947:0:3'  # eRtg SOS Kinderdorf (Schorndorf Oberberken)
	OBERB__UNTERB__HERRENBACHSTR = 'de:08119:4948:0:3'  # eRtg Schurwaldhalle (Schorndorf Oberberken)
	UNTERBERKEN_HERRENBACHSTRASSE = 'de:08119:4948:0:3'  # eRtg Schurwaldhalle (Schorndorf Oberberken)
	GAILDORFER_STRASSE = 'de:08119:4949'
	GAILDORFER_STRASSE_1 = 'de:08119:4949:0:3'  # Rtg Oppenweiler (Backnang Backnang)
	GAILDORFER_STRASSE_2 = 'de:08119:4949:0:4'  # Rtg Backnang (Backnang Backnang)
	BACKNANG_GAILDORFER_STR = 'de:08119:4949'
	BACKNANG_GAILDORFER_STR_1 = 'de:08119:4949:0:3'  # Rtg Oppenweiler (Backnang Backnang)
	BACKNANG_GAILDORFER_STR_2 = 'de:08119:4949:0:4'  # Rtg Backnang (Backnang Backnang)
	BÜHLFELD = 'de:08119:4950:0:3'  # eRtg Katharinenhof (Oppenweiler Oppenweiler)
	OPPENWEILER_BÜHLFELD = 'de:08119:4950:0:3'  # eRtg Katharinenhof (Oppenweiler Oppenweiler)
	MURRTALSCHULE = 'de:08119:4951'
	MURRTALSCHULE_1 = 'de:08119:4951:0:3'  # Rtg Reichenberg (Oppenweiler Oppenweiler)
	MURRTALSCHULE_2 = 'de:08119:4951:0:4'  # Rtg Ortsmitte (Oppenweiler Oppenweiler)
	OPPENWEILER_MURRTALSCHULE = 'de:08119:4951'
	OPPENWEILER_MURRTALSCHULE_1 = 'de:08119:4951:0:3'  # Rtg Reichenberg (Oppenweiler Oppenweiler)
	OPPENWEILER_MURRTALSCHULE_2 = 'de:08119:4951:0:4'  # Rtg Ortsmitte (Oppenweiler Oppenweiler)
	SULZBACH__M = 'de:08119:4952'
	SULZBACH__M_1 = 'de:08119:4952:1:2'  # Gleis 2 (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M_2 = 'de:08119:4952:2:3'  # eRtg Alle Richtungen (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M_3 = 'de:08119:4952:3:1'  # Gleis 1 (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__LAUTERTAL = 'de:08119:4953'
	SULZBACH__M__LAUTERTAL_1 = 'de:08119:4953:0:3'  # eRtg Lautern (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__LAUTERTAL_2 = 'de:08119:4953:0:4'  # eRtg Marktplatz (Sulzbach an der Murr Sulzbach (M))
	LAUTERTAL_1 = 'de:08119:4953:0:3'  # eRtg Lautern (Sulzbach an der Murr Sulzbach (M))
	LAUTERTAL_2 = 'de:08119:4953:0:4'  # eRtg Marktplatz (Sulzbach an der Murr Sulzbach (M))
	LAUTERTAL_3 = 'de:08125:1991:0:1'  # eRtg Neulautern (Wüstenrot Wüstenrot)
	LAUTERTAL_4 = 'de:08125:1991:0:2'  # eRtg Gieshof (Wüstenrot Wüstenrot)
	SPIEGELBERG_DAUERNBERG_ABZW = 'de:08119:4954'
	SPIEGELBERG_DAUERNBERG_ABZW_1 = 'de:08119:4954:0:3'  # eRtg Sulzbacher Straße (Spiegelberg Spiegelberg)
	SPIEGELBERG_DAUERNBERG_ABZW_2 = 'de:08119:4954:0:4'  # eRtg Bernhalden (Spiegelberg Spiegelberg)
	DAUERNBERG_ABZWEIG = 'de:08119:4954'
	DAUERNBERG_ABZWEIG_1 = 'de:08119:4954:0:3'  # eRtg Sulzbacher Straße (Spiegelberg Spiegelberg)
	DAUERNBERG_ABZWEIG_2 = 'de:08119:4954:0:4'  # eRtg Bernhalden (Spiegelberg Spiegelberg)
	SULZB__M__SIEBERSBACH = 'de:08119:4955'
	SULZB__M__SIEBERSBACH_1 = 'de:08119:4955:0:3'  # Rtg Spiegelberg (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__SIEBERSBACH_2 = 'de:08119:4955:0:4'  # Rtg Sulzbach (Sulzbach an der Murr Sulzbach (M))
	SIEBERSBACH = 'de:08119:4955'
	SIEBERSBACH_1 = 'de:08119:4955:0:3'  # Rtg Spiegelberg (Sulzbach an der Murr Sulzbach (M))
	SIEBERSBACH_2 = 'de:08119:4955:0:4'  # Rtg Sulzbach (Sulzbach an der Murr Sulzbach (M))
	BERNHALDEN = 'de:08119:4956'
	BERNHALDEN_1 = 'de:08119:4956:0:3'  # Rtg Spiegelberg (Oppenweiler Oppenweiler)
	BERNHALDEN_2 = 'de:08119:4956:0:4'  # Rtg Sulzbach (Oppenweiler Oppenweiler)
	BERNHALDEN_3 = 'de:08119:4956:0:5'  # Rtg Jux (Oppenweiler Oppenweiler)
	OPPENW__BERNHALDEN = 'de:08119:4956'
	OPPENW__BERNHALDEN_1 = 'de:08119:4956:0:3'  # Rtg Spiegelberg (Oppenweiler Oppenweiler)
	OPPENW__BERNHALDEN_2 = 'de:08119:4956:0:4'  # Rtg Sulzbach (Oppenweiler Oppenweiler)
	OPPENW__BERNHALDEN_3 = 'de:08119:4956:0:5'  # Rtg Jux (Oppenweiler Oppenweiler)
	DAUERNBERG = 'de:08119:4957:0:3'  # eRtg Alle Richtungen (Spiegelberg Spiegelberg)
	SPIEGELB__DAUERNBERG = 'de:08119:4957:0:3'  # eRtg Alle Richtungen (Spiegelberg Spiegelberg)
	SULZBACHER_STRASSE = 'de:08119:4958'
	SULZBACHER_STRASSE_1 = 'de:08119:4958:0:3'  # eRtg Löwensteiner/Großhöchberger Str. (Spiegelberg Spiegelberg)
	SULZBACHER_STRASSE_2 = 'de:08119:4958:0:4'  # eRtg Dauernberg Abzw (Spiegelberg Spiegelberg)
	SPIEGELBERG_SULZBACHER_STR = 'de:08119:4958'
	SPIEGELBERG_SULZBACHER_STR_1 = 'de:08119:4958:0:3'  # eRtg Löwensteiner/Großhöchberger Str. (Spiegelberg Spiegelberg)
	SPIEGELBERG_SULZBACHER_STR_2 = 'de:08119:4958:0:4'  # eRtg Dauernberg Abzw (Spiegelberg Spiegelberg)
	LÖWENSTEINER_STRASSE = 'de:08119:4959'
	LÖWENSTEINER_STRASSE_1 = 'de:08119:4959:0:3'  # eRtg Gieshof (Spiegelberg Spiegelberg)
	LÖWENSTEINER_STRASSE_2 = 'de:08119:4959:0:4'  # eRtg Sulzbacher Straße (Spiegelberg Spiegelberg)
	SPIEGELBERG_LÖWENSTEINER_STR = 'de:08119:4959'
	SPIEGELBERG_LÖWENSTEINER_STR_1 = 'de:08119:4959:0:3'  # eRtg Gieshof (Spiegelberg Spiegelberg)
	SPIEGELBERG_LÖWENSTEINER_STR_2 = 'de:08119:4959:0:4'  # eRtg Sulzbacher Straße (Spiegelberg Spiegelberg)
	GIESHOF = 'de:08119:4960'
	GIESHOF_1 = 'de:08119:4960:0:3'  # eRtg Lautertal (Spiegelberg Spiegelberg)
	GIESHOF_2 = 'de:08119:4960:0:4'  # eRtg Löwensteiner Straße (Spiegelberg Spiegelberg)
	SPIEGELBERG_GIESHOF = 'de:08119:4960'
	SPIEGELBERG_GIESHOF_1 = 'de:08119:4960:0:3'  # eRtg Lautertal (Spiegelberg Spiegelberg)
	SPIEGELBERG_GIESHOF_2 = 'de:08119:4960:0:4'  # eRtg Löwensteiner Straße (Spiegelberg Spiegelberg)
	SPIEGELB__GROSSHÖCHBERGER_STR = 'de:08119:4961'
	SPIEGELB__GROSSHÖCHBERGER_STR_1 = 'de:08119:4961:0:3'  # eRtg Schule (Spiegelberg Spiegelberg)
	SPIEGELB__GROSSHÖCHBERGER_STR_2 = 'de:08119:4961:0:4'  # eRtg Sulzbacher Straße (Spiegelberg Spiegelberg)
	GROSSHÖCHBERGER_STRASSE = 'de:08119:4961'
	GROSSHÖCHBERGER_STRASSE_1 = 'de:08119:4961:0:3'  # eRtg Schule (Spiegelberg Spiegelberg)
	GROSSHÖCHBERGER_STRASSE_2 = 'de:08119:4961:0:4'  # eRtg Sulzbacher Straße (Spiegelberg Spiegelberg)
	GROSSERLACH_ERLACH_ABZW = 'de:08119:4962'
	GROSSERLACH_ERLACH_ABZW_1 = 'de:08119:4962:0:3'  # eRtg Berwinkel (Großerlach Großerlach)
	GROSSERLACH_ERLACH_ABZW_2 = 'de:08119:4962:0:4'  # eRtg Abzweig Neufürstenhütte (Großerlach Großerlach)
	ERLACH_ABZWEIG = 'de:08119:4962'
	ERLACH_ABZWEIG_1 = 'de:08119:4962:0:3'  # eRtg Berwinkel (Großerlach Großerlach)
	ERLACH_ABZWEIG_2 = 'de:08119:4962:0:4'  # eRtg Abzweig Neufürstenhütte (Großerlach Großerlach)
	MITTELFISCHBACH_ABZWEIGUNG = 'de:08119:4964'
	MITTELFISCHBACH_ABZWEIGUNG_1 = 'de:08119:4964:0:3'  # eRtg Oberfischbach (Großerlach Großerlach)
	MITTELFISCHBACH_ABZWEIGUNG_2 = 'de:08119:4964:0:4'  # eRtg Abzweig Neufürstenhütte (Großerlach Großerlach)
	GROSSERL__MITTELFISCHB__ABZW = 'de:08119:4964'
	GROSSERL__MITTELFISCHB__ABZW_1 = 'de:08119:4964:0:3'  # eRtg Oberfischbach (Großerlach Großerlach)
	GROSSERL__MITTELFISCHB__ABZW_2 = 'de:08119:4964:0:4'  # eRtg Abzweig Neufürstenhütte (Großerlach Großerlach)
	GROSSERLACH_RATHAUS = 'de:08119:4965'
	GROSSERLACH_RATHAUS_1 = 'de:08119:4965:0:3'  # eRtg Abzweig Neufürstenhütte (Großerlach Großerlach)
	GROSSERLACH_RATHAUS_2 = 'de:08119:4965:0:4'  # eRtg Mainhardt (Großerlach Großerlach)
	GROSSERL__OBERFISCHBACH = 'de:08119:4966'
	GROSSERL__OBERFISCHBACH_1 = 'de:08119:4966:0:3'  # eRtg Kleinerlach (Großerlach Großerlach)
	GROSSERL__OBERFISCHBACH_2 = 'de:08119:4966:0:4'  # eRtg Mittelfischbach Abzweigung (Großerlach Großerlach)
	OBERFISCHBACH = 'de:08119:4966'
	OBERFISCHBACH_1 = 'de:08119:4966:0:3'  # eRtg Kleinerlach (Großerlach Großerlach)
	OBERFISCHBACH_2 = 'de:08119:4966:0:4'  # eRtg Mittelfischbach Abzweigung (Großerlach Großerlach)
	GROSSERLACH_KLEINERLACH = 'de:08119:4967'
	GROSSERLACH_KLEINERLACH_1 = 'de:08119:4967:0:3'  # eRtg Neufürstenhütte (Großerlach Großerlach)
	GROSSERLACH_KLEINERLACH_2 = 'de:08119:4967:0:4'  # eRtg Oberfischbach (Großerlach Großerlach)
	KLEINERLACH = 'de:08119:4967'
	KLEINERLACH_1 = 'de:08119:4967:0:3'  # eRtg Neufürstenhütte (Großerlach Großerlach)
	KLEINERLACH_2 = 'de:08119:4967:0:4'  # eRtg Oberfischbach (Großerlach Großerlach)
	NEUFÜRSTENHÜTTE = 'de:08119:4968'
	NEUFÜRSTENHÜTTE_1 = 'de:08119:4968:0:3'  # eRtg Altfürstenhütte (Großerlach Großerlach)
	NEUFÜRSTENHÜTTE_2 = 'de:08119:4968:0:4'  # eRtg Kleinerlach (Großerlach Großerlach)
	GROSSERL__NEUFÜRSTENHÜTTE = 'de:08119:4968'
	GROSSERL__NEUFÜRSTENHÜTTE_1 = 'de:08119:4968:0:3'  # eRtg Altfürstenhütte (Großerlach Großerlach)
	GROSSERL__NEUFÜRSTENHÜTTE_2 = 'de:08119:4968:0:4'  # eRtg Kleinerlach (Großerlach Großerlach)
	ALTFÜRSTENHÜTTE = 'de:08119:4969'
	ALTFÜRSTENHÜTTE_1 = 'de:08119:4969:0:3'  # eRtg Böhringsweiler (Großerlach Großerlach)
	ALTFÜRSTENHÜTTE_2 = 'de:08119:4969:0:4'  # eRtg Neufürstenhütte (Großerlach Großerlach)
	GROSSERL__ALTFÜRSTENHÜTTE = 'de:08119:4969'
	GROSSERL__ALTFÜRSTENHÜTTE_1 = 'de:08119:4969:0:3'  # eRtg Böhringsweiler (Großerlach Großerlach)
	GROSSERL__ALTFÜRSTENHÜTTE_2 = 'de:08119:4969:0:4'  # eRtg Neufürstenhütte (Großerlach Großerlach)
	BÖHRINGSWEILER = 'de:08119:4970'
	BÖHRINGSWEILER_1 = 'de:08119:4970:0:3'  # eRtg Abzw. Hohenegarten (Großerlach Großerlach)
	BÖHRINGSWEILER_2 = 'de:08119:4970:0:4'  # eRtg Altfürstenhütte (Großerlach Großerlach)
	GROSSERL__BÖHRINGSWEILER = 'de:08119:4970'
	GROSSERL__BÖHRINGSWEILER_1 = 'de:08119:4970:0:3'  # eRtg Abzw. Hohenegarten (Großerlach Großerlach)
	GROSSERL__BÖHRINGSWEILER_2 = 'de:08119:4970:0:4'  # eRtg Altfürstenhütte (Großerlach Großerlach)
	BACKNANG_GESUNDHEITSZENTRUM = 'de:08119:4971'
	BACKNANG_GESUNDHEITSZENTRUM_1 = 'de:08119:4971:0:3'  # eRtg Industriestraße (Backnang Backnang)
	BACKNANG_GESUNDHEITSZENTRUM_2 = 'de:08119:4971:0:4'  # eRtg Johanneskirche (Backnang Backnang)
	GESUNDHEITSZENTRUM = 'de:08119:4971'
	GESUNDHEITSZENTRUM_1 = 'de:08119:4971:0:3'  # eRtg Industriestraße (Backnang Backnang)
	GESUNDHEITSZENTRUM_2 = 'de:08119:4971:0:4'  # eRtg Johanneskirche (Backnang Backnang)
	BACKNANG_INDUSTRIESTRASSE = 'de:08119:4972'
	BACKNANG_INDUSTRIESTRASSE_1 = 'de:08119:4972:0:3'  # Rtg Backnang (Backnang Backnang)
	BACKNANG_INDUSTRIESTRASSE_2 = 'de:08119:4972:0:4'  # Rtg Weissach (Backnang Backnang)
	UNGEHEUERHOF_ABZWEIG = 'de:08119:4973'
	UNGEHEUERHOF_ABZWEIG_1 = 'de:08119:4973:0:3'  # eRtg Ungeheuerhof/Heiningen Kreuzung (Backnang Backnang)
	UNGEHEUERHOF_ABZWEIG_2 = 'de:08119:4973:0:4'  # eRtg Industriestraße (Backnang Backnang)
	BACKNANG_UNGEHEUERHOF_ABZW = 'de:08119:4973'
	BACKNANG_UNGEHEUERHOF_ABZW_1 = 'de:08119:4973:0:3'  # eRtg Ungeheuerhof/Heiningen Kreuzung (Backnang Backnang)
	BACKNANG_UNGEHEUERHOF_ABZW_2 = 'de:08119:4973:0:4'  # eRtg Industriestraße (Backnang Backnang)
	HEININGEN_KREUZUNG = 'de:08119:4974'
	HEININGEN_KREUZUNG_1 = 'de:08119:4974:0:3'  # eRtg Wasenfeld/Täferhalde (Backnang Heiningen (Backnang))
	HEININGEN_KREUZUNG_2 = 'de:08119:4974:0:4'  # eRtg Ungeheuerhof Abzw (Backnang Heiningen (Backnang))
	BACKNANG_UNGEHEUERHOF = 'de:08119:4975'
	BACKNANG_UNGEHEUERHOF_1 = 'de:08119:4975:0:3'  # eRtg Beethovenstraße/Allmersbach i.T. (Backnang Backnang)
	BACKNANG_UNGEHEUERHOF_2 = 'de:08119:4975:0:4'  # eRtg Ungeheuerhof Abzweig (Backnang Backnang)
	UNGEHEUERHOF = 'de:08119:4975'
	UNGEHEUERHOF_1 = 'de:08119:4975:0:3'  # eRtg Beethovenstraße/Allmersbach i.T. (Backnang Backnang)
	UNGEHEUERHOF_2 = 'de:08119:4975:0:4'  # eRtg Ungeheuerhof Abzweig (Backnang Backnang)
	UNTERBRÜDEN_RATHAUS = 'de:08119:4976:0:3'  # eRtg Friedhof (Auenwald Unterbrüden)
	OBERWEISSACH_WEISSACHSTR = 'de:08119:4978'
	OBERWEISSACH_WEISSACHSTR_1 = 'de:08119:4978:0:3'  # eRtg Hohnw. Rathausstraße/Ebniseestraße (Weissach im Tal Oberweissach)
	OBERWEISSACH_WEISSACHSTR_2 = 'de:08119:4978:0:4'  # eRtg Aichholzhof (Weissach im Tal Oberweissach)
	WEISSACHSTRASSE = 'de:08119:4978'
	WEISSACHSTRASSE_1 = 'de:08119:4978:0:3'  # eRtg Hohnw. Rathausstraße/Ebniseestraße (Weissach im Tal Oberweissach)
	WEISSACHSTRASSE_2 = 'de:08119:4978:0:4'  # eRtg Aichholzhof (Weissach im Tal Oberweissach)
	LIPPOLDSW__HOHNW__RATHAUSSTR = 'de:08119:4979'
	LIPPOLDSW__HOHNW__RATHAUSSTR_1 = 'de:08119:4979:0:3'  # eRtg Hohnweiler Rathaus (Auenwald Lippoldsweiler)
	LIPPOLDSW__HOHNW__RATHAUSSTR_2 = 'de:08119:4979:0:4'  # eRtg Däfern (Auenwald Lippoldsweiler)
	HOHNWEILER_RATHAUSSTRASSE = 'de:08119:4979'
	HOHNWEILER_RATHAUSSTRASSE_1 = 'de:08119:4979:0:3'  # eRtg Hohnweiler Rathaus (Auenwald Lippoldsweiler)
	HOHNWEILER_RATHAUSSTRASSE_2 = 'de:08119:4979:0:4'  # eRtg Däfern (Auenwald Lippoldsweiler)
	DÄFERN = 'de:08119:4980:0:3'  # eRtg Alle Richtungen (Auenwald Lippoldsweiler)
	LIPPOLDSW__DÄFERN = 'de:08119:4980:0:3'  # eRtg Alle Richtungen (Auenwald Lippoldsweiler)
	HOHNWEILER_RATHAUS = 'de:08119:4981'
	HOHNWEILER_RATHAUS_1 = 'de:08119:4981:0:3'  # eRtg Raiffeisenbank/Däfern (Auenwald Lippoldsweiler)
	HOHNWEILER_RATHAUS_2 = 'de:08119:4981:0:4'  # eRtg Hohnweiler Rathausstraße (Auenwald Lippoldsweiler)
	LIPPOLDSW__HOHNW__RATHAUS = 'de:08119:4981'
	LIPPOLDSW__HOHNW__RATHAUS_1 = 'de:08119:4981:0:3'  # eRtg Raiffeisenbank/Däfern (Auenwald Lippoldsweiler)
	LIPPOLDSW__HOHNW__RATHAUS_2 = 'de:08119:4981:0:4'  # eRtg Hohnweiler Rathausstraße (Auenwald Lippoldsweiler)
	LIPPOLDSW__BADSTRASSE = 'de:08119:4982'
	LIPPOLDSW__BADSTRASSE_1 = 'de:08119:4982:0:3'  # eRtg Am Brenner/Traube (Auenwald Lippoldsweiler)
	LIPPOLDSW__BADSTRASSE_2 = 'de:08119:4982:0:4'  # eRtg Raiffeisenbank (Auenwald Lippoldsweiler)
	NATURFREUNDEHAUS_1 = 'de:08119:4984:0:3'  # eRtg Am Brenner (Althütte Sechselberg)
	NATURFREUNDEHAUS_2 = 'de:08119:7818:0:4'  # eRtg Tannhof (Welzheim Welzheim)
	SECHSELBERG_NATURFREUNDEHAUS = 'de:08119:4984:0:3'  # eRtg Am Brenner (Althütte Sechselberg)
	DEUTSCHES_ROTES_KREUZ = 'de:08119:4985'
	DEUTSCHES_ROTES_KREUZ_1 = 'de:08119:4985:0:3'  # eRtg Schule (Althütte Sechselberg)
	DEUTSCHES_ROTES_KREUZ_2 = 'de:08119:4985:0:4'  # eRtg Naturfreundehaus (Althütte Sechselberg)
	SECHSELB__DT__ROTES_KREUZ = 'de:08119:4985'
	SECHSELB__DT__ROTES_KREUZ_1 = 'de:08119:4985:0:3'  # eRtg Schule (Althütte Sechselberg)
	SECHSELB__DT__ROTES_KREUZ_2 = 'de:08119:4985:0:4'  # eRtg Naturfreundehaus (Althütte Sechselberg)
	SECHSELBERG_SCHULE = 'de:08119:4986:0:3'  # eRtg Alle Richtungen (Althütte Sechselberg)
	WALDENWEILER = 'de:08119:4987'
	WALDENWEILER_1 = 'de:08119:4987:0:3'  # eRtg Rathaus (Althütte Sechselberg)
	WALDENWEILER_2 = 'de:08119:4987:0:4'  # eRtg Schule (Althütte Sechselberg)
	SECHSELBERG_WALDENWEILER = 'de:08119:4987'
	SECHSELBERG_WALDENWEILER_1 = 'de:08119:4987:0:3'  # eRtg Rathaus (Althütte Sechselberg)
	SECHSELBERG_WALDENWEILER_2 = 'de:08119:4987:0:4'  # eRtg Schule (Althütte Sechselberg)
	OBERWEISSACH_EBNISEESTR = 'de:08119:4988'
	OBERWEISSACH_EBNISEESTR_1 = 'de:08119:4988:0:3'  # eRtg Bruch (Weissach im Tal Oberweissach)
	OBERWEISSACH_EBNISEESTR_2 = 'de:08119:4988:0:4'  # eRtg Weissachstraße (Weissach im Tal Oberweissach)
	BRUCH_1 = 'de:08119:4989:0:3'  # eRtg Kelter (Weissach im Tal Bruch (Weissach))
	BRUCH_2 = 'de:08119:4989:0:4'  # eRtg Ebniseestraße (Weissach im Tal Bruch (Weissach))
	BRUCH_3 = 'de:08119:5077:0:3'  # Rtg Mönchhof (Kaisersbach Kaisersbach)
	BRUCH_4 = 'de:08119:5077:0:4'  # Rtg Hauptstr. (Kaisersbach Kaisersbach)
	BRUCH_KELTER = 'de:08119:4990'
	BRUCH_KELTER_1 = 'de:08119:4990:0:3'  # eRtg Däfern/Lutzenberg (Weissach im Tal Bruch (Weissach))
	BRUCH_KELTER_2 = 'de:08119:4990:0:4'  # eRtg Bruch (Weissach im Tal Bruch (Weissach))
	LUTZENBERG = 'de:08119:4991'
	LUTZENBERG_1 = 'de:08119:4991:0:3'  # Rtg Bruch/Alth. Kallenberg (Althütte Althütte)
	LUTZENBERG_2 = 'de:08119:4991:0:4'  # Rtg Althütte Rathaus (Althütte Althütte)
	ALTHÜTTE_LUTZENBERG = 'de:08119:4991'
	ALTHÜTTE_LUTZENBERG_1 = 'de:08119:4991:0:3'  # Rtg Bruch/Alth. Kallenberg (Althütte Althütte)
	ALTHÜTTE_LUTZENBERG_2 = 'de:08119:4991:0:4'  # Rtg Althütte Rathaus (Althütte Althütte)
	KALLENBERG = 'de:08119:4992:0:3'  # eRtg Alle Richtungen (Althütte Althütte)
	ALTHÜTTE_KALLENBERG = 'de:08119:4992:0:3'  # eRtg Alle Richtungen (Althütte Althütte)
	RUDERSBG__MANNENBG__IM_HAU = 'de:08119:4993:0:4'  # eRtg Mannenberg (Rudersberg Rudersberg)
	MANNENBERG_IM_HAU = 'de:08119:4993:0:4'  # eRtg Mannenberg (Rudersberg Rudersberg)
	RUDERSBERG_MANNENBERG = 'de:08119:4994'
	RUDERSBERG_MANNENBERG_1 = 'de:08119:4994:0:3'  # eRtg Im Hau (Rudersberg Rudersberg)
	RUDERSBERG_MANNENBERG_2 = 'de:08119:4994:0:4'  # eRtg Klaffenbach/Oberndorf (Rudersberg Rudersberg)
	MANNENBERG = 'de:08119:4994'
	MANNENBERG_1 = 'de:08119:4994:0:3'  # eRtg Im Hau (Rudersberg Rudersberg)
	MANNENBERG_2 = 'de:08119:4994:0:4'  # eRtg Klaffenbach/Oberndorf (Rudersberg Rudersberg)
	ALTHÜTTE_RATHAUS = 'de:08119:4995'
	ALTHÜTTE_RATHAUS_1 = 'de:08119:4995:1:3'  # Rtg Ebniseestr. (Althütte Althütte)
	ALTHÜTTE_RATHAUS_2 = 'de:08119:4995:1:4'  # Rtg Ortsmitte (Althütte Althütte)
	ALTHÜTTE_ORTSMITTE = 'de:08119:4996'
	ALTHÜTTE_ORTSMITTE_1 = 'de:08119:4996:0:3'  # Rtg Rathaus (Althütte Althütte)
	ALTHÜTTE_ORTSMITTE_2 = 'de:08119:4996:0:4'  # Rtg Strohhof (Althütte Althütte)
	STROHHOF = 'de:08119:4997'
	STROHHOF_1 = 'de:08119:4997:0:3'  # Rtg Klaffenbach Althütter Str. (Althütte Althütte)
	STROHHOF_2 = 'de:08119:4997:0:4'  # Rtg Althütte Ortsmitte (Althütte Althütte)
	ALTHÜTTE_STROHHOF = 'de:08119:4997'
	ALTHÜTTE_STROHHOF_1 = 'de:08119:4997:0:3'  # Rtg Klaffenbach Althütter Str. (Althütte Althütte)
	ALTHÜTTE_STROHHOF_2 = 'de:08119:4997:0:4'  # Rtg Althütte Ortsmitte (Althütte Althütte)
	RUDERSBERG_KLAFFENBACH = 'de:08119:4998'
	RUDERSBERG_KLAFFENBACH_1 = 'de:08119:4998:0:3'  # eRtg Abzweig Steinbach (Rudersberg Rudersberg)
	RUDERSBERG_KLAFFENBACH_2 = 'de:08119:4998:0:4'  # eRtg Grauhaldenhof (Rudersberg Rudersberg)
	KLAFFENBACH = 'de:08119:4998'
	KLAFFENBACH_1 = 'de:08119:4998:0:3'  # eRtg Abzweig Steinbach (Rudersberg Rudersberg)
	KLAFFENBACH_2 = 'de:08119:4998:0:4'  # eRtg Grauhaldenhof (Rudersberg Rudersberg)
	GRAUHALDENHOF = 'de:08119:4999'
	GRAUHALDENHOF_1 = 'de:08119:4999:0:3'  # eRtg Klaffenbach (Rudersberg Rudersberg)
	GRAUHALDENHOF_2 = 'de:08119:4999:0:4'  # eRtg Oberndorf Brunnenplatz (Rudersberg Rudersberg)
	RUDERSBERG_GRAUHALDENHOF = 'de:08119:4999'
	RUDERSBERG_GRAUHALDENHOF_1 = 'de:08119:4999:0:3'  # eRtg Klaffenbach (Rudersberg Rudersberg)
	RUDERSBERG_GRAUHALDENHOF_2 = 'de:08119:4999:0:4'  # eRtg Oberndorf Brunnenplatz (Rudersberg Rudersberg)
	NEUENHAUS_KIRCHE = 'de:08116:5000'
	NEUENHAUS_KIRCHE_1 = 'de:08116:5000:0:3'  # eRtg Ort (Aichtal Neuenhaus)
	NEUENHAUS_KIRCHE_2 = 'de:08116:5000:0:4'  # eRtg Grörach/Häfnerstraße (Aichtal Neuenhaus)
	GRÖTZINGEN_ORTSMITTE = 'de:08116:5001'
	GRÖTZINGEN_ORTSMITTE_1 = 'de:08116:5001:1:3'  # Rtg Raiffeisenstr. (Aichtal Grötzingen)
	GRÖTZINGEN_ORTSMITTE_2 = 'de:08116:5001:1:4'  # Rtg Schönblick (Aichtal Grötzingen)
	NELLINGEN_NELLINGER_LINDE = 'de:08116:5003'
	NELLINGEN_NELLINGER_LINDE_1 = 'de:08116:5003:0:3'  # Rtg Zollberg Mutzenreisstr. (Ostfildern Nellingen)
	NELLINGEN_NELLINGER_LINDE_2 = 'de:08116:5003:0:4'  # Rtg Nussweg/Steinen (Ostfildern Nellingen)
	NELLINGER_LINDE = 'de:08116:5003'
	NELLINGER_LINDE_1 = 'de:08116:5003:0:3'  # Rtg Zollberg Mutzenreisstr. (Ostfildern Nellingen)
	NELLINGER_LINDE_2 = 'de:08116:5003:0:4'  # Rtg Nussweg/Steinen (Ostfildern Nellingen)
	DENKENDORFER_STRASSE = 'de:08116:5005'
	DENKENDORFER_STRASSE_1 = 'de:08116:5005:0:3'  # Rtg Kaiserstr. (Ostfildern Nellingen)
	DENKENDORFER_STRASSE_2 = 'de:08116:5005:0:4'  # Rtg Denkendorf Stadion (Ostfildern Nellingen)
	NELLINGEN_DENKENDORFER_STR = 'de:08116:5005'
	NELLINGEN_DENKENDORFER_STR_1 = 'de:08116:5005:0:3'  # Rtg Kaiserstr. (Ostfildern Nellingen)
	NELLINGEN_DENKENDORFER_STR_2 = 'de:08116:5005:0:4'  # Rtg Denkendorf Stadion (Ostfildern Nellingen)
	DENKENDORF_STADION = 'de:08116:5006'
	DENKENDORF_STADION_1 = 'de:08116:5006:0:3'  # Rtg Esslinger Str. (Denkendorf Denkendorf)
	DENKENDORF_STADION_2 = 'de:08116:5006:0:4'  # Rtg Nellingen Denkendorfer Str. (Denkendorf Denkendorf)
	DENKENDORF_ESSLINGER_STR = 'de:08116:5007'
	DENKENDORF_ESSLINGER_STR_1 = 'de:08116:5007:0:3'  # 119 Rtg Weingartenstr. (Denkendorf Denkendorf)
	DENKENDORF_ESSLINGER_STR_2 = 'de:08116:5007:0:4'  # Rtg Nellingen (Denkendorf Denkendorf)
	DENKENDORF_ESSLINGER_STR_3 = 'de:08116:5007:0:5'  # 121 Rtg Weingartstr. (Denkendorf Denkendorf)
	DENKENDORF_RATHAUS = 'de:08116:5008:0:3'  # Rtg Neuhäuser Str. (Denkendorf Denkendorf)
	DENKENDORF_NEUHÄUSER_STR = 'de:08116:5009'
	DENKENDORF_NEUHÄUSER_STR_1 = 'de:08116:5009:0:3'  # Rtg Neuhausen Traube (Denkendorf Denkendorf)
	DENKENDORF_NEUHÄUSER_STR_2 = 'de:08116:5009:0:4'  # Rtg Vogelsangstr. (Denkendorf Denkendorf)
	DENKENDORF_VOGELSANGSTRASSE = 'de:08116:5010:0:3'  # Bus-Li 119/121> ESBF/DDES (Denkendorf Denkendorf)
	VOGELSANGSTRASSE = 'de:08116:5010:0:3'  # Bus-Li 119/121> ESBF/DDES (Denkendorf Denkendorf)
	SCHARNH__SONNENHALDE = 'de:08116:5011'
	SCHARNH__SONNENHALDE_1 = 'de:08116:5011:0:3'  # Rtg Sonnenhalde (Ostfildern Scharnhausen)
	SCHARNH__SONNENHALDE_2 = 'de:08116:5011:0:4'  # Rtg Kreuzbrunnen (Ostfildern Scharnhausen)
	SCHARNHAUSEN_LENZHALDE = 'de:08116:5012'
	SCHARNHAUSEN_LENZHALDE_1 = 'de:08116:5012:0:3'  # Rtg Rathaus (Ostfildern Scharnhausen)
	SCHARNHAUSEN_LENZHALDE_2 = 'de:08116:5012:0:4'  # Rtg Sonnenhalde (Ostfildern Scharnhausen)
	ROTHWEG = 'de:08116:5013'
	ROTHWEG_1 = 'de:08116:5013:0:3'  # Rtg Vogelsangstr. (Denkendorf Denkendorf)
	ROTHWEG_2 = 'de:08116:5013:0:4'  # Rtg Forschung (Denkendorf Denkendorf)
	DENKENDORF_ROTHWEG = 'de:08116:5013'
	DENKENDORF_ROTHWEG_1 = 'de:08116:5013:0:3'  # Rtg Vogelsangstr. (Denkendorf Denkendorf)
	DENKENDORF_ROTHWEG_2 = 'de:08116:5013:0:4'  # Rtg Forschung (Denkendorf Denkendorf)
	DENKENDORF_FORSCHUNG = 'de:08116:5014'
	DENKENDORF_FORSCHUNG_1 = 'de:08116:5014:0:3'  # Rtg Rothweg (Denkendorf Denkendorf)
	DENKENDORF_FORSCHUNG_2 = 'de:08116:5014:0:4'  # Rtg Deizisau Körschtal (Denkendorf Denkendorf)
	FORSCHUNG = 'de:08116:5014'
	FORSCHUNG_1 = 'de:08116:5014:0:3'  # Rtg Rothweg (Denkendorf Denkendorf)
	FORSCHUNG_2 = 'de:08116:5014:0:4'  # Rtg Deizisau Körschtal (Denkendorf Denkendorf)
	KÖRSCHTAL = 'de:08116:5016'
	KÖRSCHTAL_1 = 'de:08116:5016:0:3'  # Rtg Denkendorf Forschung (Deizisau Deizisau)
	KÖRSCHTAL_2 = 'de:08116:5016:0:4'  # Sirnau Dornierstr. (Deizisau Deizisau)
	DEIZISAU_KÖRSCHTAL = 'de:08116:5016'
	DEIZISAU_KÖRSCHTAL_1 = 'de:08116:5016:0:3'  # Rtg Denkendorf Forschung (Deizisau Deizisau)
	DEIZISAU_KÖRSCHTAL_2 = 'de:08116:5016:0:4'  # Sirnau Dornierstr. (Deizisau Deizisau)
	KEMNAT_ALTES_RATHAUS = 'de:08116:5017'
	KEMNAT_ALTES_RATHAUS_1 = 'de:08116:5017:0:3'  # Rtg Reutlinger Str. (Ostfildern Kemnat)
	KEMNAT_ALTES_RATHAUS_2 = 'de:08116:5017:0:4'  # Rtg Rosenstr. (Ostfildern Kemnat)
	KEMNAT_ROSENSTRASSE = 'de:08116:5018'
	KEMNAT_ROSENSTRASSE_1 = 'de:08116:5018:0:3'  # Rtg Kemnat Altes Rathaus (Ostfildern Kemnat)
	KEMNAT_ROSENSTRASSE_2 = 'de:08116:5018:0:4'  # Rtg Kemnat Zeppelinstr. (Ostfildern Kemnat)
	KEMNAT_REUTLINGER_STR = 'de:08116:5019'
	KEMNAT_REUTLINGER_STR_1 = 'de:08116:5019:0:3'  # Rtg Scharnhausen Nagelstr. (Ostfildern Kemnat)
	KEMNAT_REUTLINGER_STR_2 = 'de:08116:5019:0:4'  # Rtg Altes Rathaus (Ostfildern Kemnat)
	KEMNAT_ZEPPELINSTRASSE = 'de:08116:5020'
	KEMNAT_ZEPPELINSTRASSE_1 = 'de:08116:5020:0:3'  # Rtg Rosenstr. (Ostfildern Kemnat)
	KEMNAT_ZEPPELINSTRASSE_2 = 'de:08116:5020:0:4'  # Rtg Schönbergstr. (Ostfildern Kemnat)
	NELLINGEN_SCHULZENTRUM = 'de:08116:5021:1:3'  # Rtg Riegelstraße/Kreuzbrunnen (Ostfildern Nellingen)
	RUIT_HORBSTRASSE = 'de:08116:5022'
	RUIT_HORBSTRASSE_1 = 'de:08116:5022:0:3'  # Rtg Heumaden (Ostfildern Ruit)
	RUIT_HORBSTRASSE_2 = 'de:08116:5022:0:4'  # Rtg Ruit (Ostfildern Ruit)
	HORBSTRASSE = 'de:08116:5022'
	HORBSTRASSE_1 = 'de:08116:5022:0:3'  # Rtg Heumaden (Ostfildern Ruit)
	HORBSTRASSE_2 = 'de:08116:5022:0:4'  # Rtg Ruit (Ostfildern Ruit)
	DENKENDORF_HINDENBURGSTR = 'de:08116:5023:0:3'  # Rtg Esslinger Str. (Denkendorf Denkendorf)
	OBERER_EISBERGWEG = 'de:08116:5024'
	OBERER_EISBERGWEG_1 = 'de:08116:5024:0:3'  # Rtg Pliensauv. Im Holder (Esslingen am Neckar Zollberg)
	OBERER_EISBERGWEG_2 = 'de:08116:5024:0:4'  # Rtg Zollberg (Esslingen am Neckar Zollberg)
	ZOLLBERG_OB__EISBERGWEG = 'de:08116:5024'
	ZOLLBERG_OB__EISBERGWEG_1 = 'de:08116:5024:0:3'  # Rtg Pliensauv. Im Holder (Esslingen am Neckar Zollberg)
	ZOLLBERG_OB__EISBERGWEG_2 = 'de:08116:5024:0:4'  # Rtg Zollberg (Esslingen am Neckar Zollberg)
	OBERNDORF_BRUNNENPLATZ = 'de:08119:5025'
	OBERNDORF_BRUNNENPLATZ_1 = 'de:08119:5025:0:3'  # Rtg Grauhaldenhof (Rudersberg Rudersberg)
	OBERNDORF_BRUNNENPLATZ_2 = 'de:08119:5025:0:4'  # Rtg Rudersberg Nord (Rudersberg Rudersberg)
	RUDERSB__OBERND__BRUNNENPLATZ = 'de:08119:5025'
	RUDERSB__OBERND__BRUNNENPLATZ_1 = 'de:08119:5025:0:3'  # Rtg Grauhaldenhof (Rudersberg Rudersberg)
	RUDERSB__OBERND__BRUNNENPLATZ_2 = 'de:08119:5025:0:4'  # Rtg Rudersberg Nord (Rudersberg Rudersberg)
	ABZWEIG_GREUTHOF = 'de:08119:5026:0:3'  # eRtg Kapf Mohnwiesenweg (Alfdorf Vordersteinenberg)
	VORDERST__ABZW__GREUTHOF = 'de:08119:5026:0:3'  # eRtg Kapf Mohnwiesenweg (Alfdorf Vordersteinenberg)
	ALTHÜTTE_VOGGENHOF_ABZW = 'de:08119:5027'
	ALTHÜTTE_VOGGENHOF_ABZW_1 = 'de:08119:5027:0:3'  # eRtg Ebniseestraße (Althütte Althütte)
	ALTHÜTTE_VOGGENHOF_ABZW_2 = 'de:08119:5027:0:4'  # eRtg Ebni (Althütte Althütte)
	VOGGENHOF_ABZWEIG = 'de:08119:5027'
	VOGGENHOF_ABZWEIG_1 = 'de:08119:5027:0:3'  # eRtg Ebniseestraße (Althütte Althütte)
	VOGGENHOF_ABZWEIG_2 = 'de:08119:5027:0:4'  # eRtg Ebni (Althütte Althütte)
	KAISERSBACH_EBNI = 'de:08119:5028'
	KAISERSBACH_EBNI_1 = 'de:08119:5028:0:3'  # Rtg Ebnisee (Kaisersbach Kaisersbach)
	KAISERSBACH_EBNI_2 = 'de:08119:5028:0:4'  # Rtg Voggenhof (Kaisersbach Kaisersbach)
	EBNI = 'de:08119:5028'
	EBNI_1 = 'de:08119:5028:0:3'  # Rtg Ebnisee (Kaisersbach Kaisersbach)
	EBNI_2 = 'de:08119:5028:0:4'  # Rtg Voggenhof (Kaisersbach Kaisersbach)
	EBNISEE = 'de:08119:5029'
	EBNISEE_1 = 'de:08119:5029:1:3'  # Rtg Eulenhof (Kaisersbach Kaisersbach)
	EBNISEE_2 = 'de:08119:5029:1:4'  # Rtg Ebni (Kaisersbach Kaisersbach)
	KAISERSBACH_EBNISEE = 'de:08119:5029'
	KAISERSBACH_EBNISEE_1 = 'de:08119:5029:1:3'  # Rtg Eulenhof (Kaisersbach Kaisersbach)
	KAISERSBACH_EBNISEE_2 = 'de:08119:5029:1:4'  # Rtg Ebni (Kaisersbach Kaisersbach)
	UNTERWEISSACH_LINDENPLATZ = 'de:08119:5030'
	UNTERWEISSACH_LINDENPLATZ_1 = 'de:08119:5030:0:3'  # eRtg Rathaus/Welzheimer Straße (Weissach im Tal Unterweissach)
	UNTERWEISSACH_LINDENPLATZ_2 = 'de:08119:5030:0:4'  # eRtg Täferhalde/Edeka (Weissach im Tal Unterweissach)
	HEININGEN_RATHAUS = 'de:08119:5031'
	HEININGEN_RATHAUS_1 = 'de:08119:5031:0:3'  # eRtg Schozachstraße (Backnang Heiningen (Backnang))
	HEININGEN_RATHAUS_2 = 'de:08119:5031:0:4'  # eRtg Marbacher Straße (Backnang Heiningen (Backnang))
	ALLMERSB_I_T__ORTSMITTE = 'de:08119:5032'
	ALLMERSB_I_T__ORTSMITTE_1 = 'de:08119:5032:0:3'  # eRtg Schule (Allmersbach im Tal Allmersbach im Tal)
	ALLMERSB_I_T__ORTSMITTE_2 = 'de:08119:5032:0:4'  # eRtg Wasenfeld (Allmersbach im Tal Allmersbach im Tal)
	KÖNIGSBRONNHOF = 'de:08119:5033'
	KÖNIGSBRONNHOF_1 = 'de:08119:5033:0:3'  # eRtg Seelach (Rudersberg Rudersberg)
	KÖNIGSBRONNHOF_2 = 'de:08119:5033:0:4'  # eRtg Ortsmitte/Kallenberg (Rudersberg Rudersberg)
	RUDERSB__KÖNIGSBRONNHOF = 'de:08119:5033'
	RUDERSB__KÖNIGSBRONNHOF_1 = 'de:08119:5033:0:3'  # eRtg Seelach (Rudersberg Rudersberg)
	RUDERSB__KÖNIGSBRONNHOF_2 = 'de:08119:5033:0:4'  # eRtg Ortsmitte/Kallenberg (Rudersberg Rudersberg)
	SEELACH = 'de:08119:5034'
	SEELACH_1 = 'de:08119:5034:0:3'  # eRtg Backnanger Straße (Rudersberg Rudersberg)
	SEELACH_2 = 'de:08119:5034:0:4'  # eRtg Königsbronnhof (Rudersberg Rudersberg)
	RUDERSBERG_SEELACH = 'de:08119:5034'
	RUDERSBERG_SEELACH_1 = 'de:08119:5034:0:3'  # eRtg Backnanger Straße (Rudersberg Rudersberg)
	RUDERSBERG_SEELACH_2 = 'de:08119:5034:0:4'  # eRtg Königsbronnhof (Rudersberg Rudersberg)
	RUDERSBERG_BACKNANGER_STR = 'de:08119:5035'
	RUDERSBERG_BACKNANGER_STR_1 = 'de:08119:5035:0:3'  # eRtg Marktplatz/Rudersberg (Rudersberg Rudersberg)
	RUDERSBERG_BACKNANGER_STR_2 = 'de:08119:5035:0:4'  # eRtg Seelach (Rudersberg Rudersberg)
	RUDERSBERG = 'de:08119:5036'
	RUDERSBERG_1 = 'de:08119:5036:1:1'  # Beide Richtungen (Rudersberg Rudersberg)
	RUDERSBERG_2 = 'de:08119:5036:2:3'  # eRtg Marktplatz/Backn. Straße (Rudersberg Rudersberg)
	RUDERSBERG_3 = 'de:08119:5036:2:4'  # eRtg Schlechtbach (Rudersberg Rudersberg)
	ALLMERSBACH_I__T__SCHULE = 'de:08119:5037:0:3'  # eRtg Rathaus (Allmersbach im Tal Allmersbach im Tal)
	HEUTENSBACH_RATHAUS = 'de:08119:5038:0:3'  # eRtg Heutensbacher Straße (Allmersbach im Tal Heutensbach)
	LAUTERN = 'de:08119:5042'
	LAUTERN_1 = 'de:08119:5042:0:3'  # Rtg Spiegelberg (Sulzbach an der Murr Sulzbach (M))
	LAUTERN_2 = 'de:08119:5042:0:4'  # Rtg Sulzbach (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__LAUTERN = 'de:08119:5042'
	SULZBACH__M__LAUTERN_1 = 'de:08119:5042:0:3'  # Rtg Spiegelberg (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__LAUTERN_2 = 'de:08119:5042:0:4'  # Rtg Sulzbach (Sulzbach an der Murr Sulzbach (M))
	SPIEGELBERG_SCHULE = 'de:08119:5044:0:3'  # eRtg Alle Richtungen (Spiegelberg Spiegelberg)
	SPIEGELB__GROSSHÖCHBERG = 'de:08119:5045:0:3'  # eRtg Alle Richtungen (Spiegelberg Spiegelberg)
	GROSSHÖCHBERG = 'de:08119:5045:0:3'  # eRtg Alle Richtungen (Spiegelberg Spiegelberg)
	JUX = 'de:08119:5046'
	JUX_1 = 'de:08119:5046:0:3'  # eRtg Löwensteiner Straße (Spiegelberg Jux)
	JUX_2 = 'de:08119:5046:0:4'  # eRtg Kindergarten/Hüttlen (Spiegelberg Jux)
	SPIEGELBERG_HÜTTLEN = 'de:08119:5047'
	SPIEGELBERG_HÜTTLEN_1 = 'de:08119:5047:0:3'  # eRtg Jux (Spiegelberg Spiegelberg)
	SPIEGELBERG_HÜTTLEN_2 = 'de:08119:5047:0:4'  # eRtg Nassach/Parkplatz Zollstock (Spiegelberg Spiegelberg)
	HÜTTLEN = 'de:08119:5047'
	HÜTTLEN_1 = 'de:08119:5047:0:3'  # eRtg Jux (Spiegelberg Spiegelberg)
	HÜTTLEN_2 = 'de:08119:5047:0:4'  # eRtg Nassach/Parkplatz Zollstock (Spiegelberg Spiegelberg)
	NASSACH = 'de:08119:5048'
	NASSACH_1 = 'de:08119:5048:0:3'  # eRtg P. Zollstock/Hüttlen (Spiegelberg Nassach)
	NASSACH_2 = 'de:08119:5048:0:4'  # eRtg Bernhalden/Kurzach (Spiegelberg Nassach)
	NASSACH_KURZACH = 'de:08119:5049:0:3'  # eRtg Alle Richtungen (Spiegelberg Nassach)
	KURZACH = 'de:08119:5049:0:3'  # eRtg Alle Richtungen (Spiegelberg Nassach)
	MITTLERE_BRÜCKE = 'de:08119:5050'
	MITTLERE_BRÜCKE_1 = 'de:08119:5050:0:3'  # eRtg Schornbacher Weg (Schorndorf Schorndorf)
	MITTLERE_BRÜCKE_2 = 'de:08119:5050:0:4'  # Rtg ZOB/Am Mühlbach (neu) (Schorndorf Schorndorf)
	SCHORNDORF_MITTLERE_BRÜCKE = 'de:08119:5050'
	SCHORNDORF_MITTLERE_BRÜCKE_1 = 'de:08119:5050:0:3'  # eRtg Schornbacher Weg (Schorndorf Schorndorf)
	SCHORNDORF_MITTLERE_BRÜCKE_2 = 'de:08119:5050:0:4'  # Rtg ZOB/Am Mühlbach (neu) (Schorndorf Schorndorf)
	HAUBERSBRONN = 'de:08119:5051:1:1'  # R-Bahn (Schorndorf Haubersbronn)
	WELZHEIM_STEINBRUCK = 'de:08119:5053'
	WELZHEIM_STEINBRUCK_1 = 'de:08119:5053:0:3'  # eRtg Eselshalden (Welzheim Welzheim)
	WELZHEIM_STEINBRUCK_2 = 'de:08119:5053:0:4'  # eRtg Welzheimer Straße (Welzheim Welzheim)
	STEINBRUCK = 'de:08119:5053'
	STEINBRUCK_1 = 'de:08119:5053:0:3'  # eRtg Eselshalden (Welzheim Welzheim)
	STEINBRUCK_2 = 'de:08119:5053:0:4'  # eRtg Welzheimer Straße (Welzheim Welzheim)
	ESELSHALDEN = 'de:08119:5054'
	ESELSHALDEN_1 = 'de:08119:5054:0:3'  # eRtg Bausche (Welzheim Welzheim)
	ESELSHALDEN_2 = 'de:08119:5054:0:4'  # eRtg Steinbruck (Welzheim Welzheim)
	WELZHEIM_ESELSHALDEN = 'de:08119:5054'
	WELZHEIM_ESELSHALDEN_1 = 'de:08119:5054:0:3'  # eRtg Bausche (Welzheim Welzheim)
	WELZHEIM_ESELSHALDEN_2 = 'de:08119:5054:0:4'  # eRtg Steinbruck (Welzheim Welzheim)
	BAUSCHE = 'de:08119:5055'
	BAUSCHE_1 = 'de:08119:5055:0:3'  # eRtg Reizenwiesen (Welzheim Welzheim)
	BAUSCHE_2 = 'de:08119:5055:0:4'  # eRtg Eselshalden (Welzheim Welzheim)
	WELZHEIM_BAUSCHE = 'de:08119:5055'
	WELZHEIM_BAUSCHE_1 = 'de:08119:5055:0:3'  # eRtg Reizenwiesen (Welzheim Welzheim)
	WELZHEIM_BAUSCHE_2 = 'de:08119:5055:0:4'  # eRtg Eselshalden (Welzheim Welzheim)
	BREITENFÜRST_SCHULHAUS = 'de:08119:5056'
	BREITENFÜRST_SCHULHAUS_1 = 'de:08119:5056:0:3'  # eRtg Breitenf. Reinhold-Maier-Platz (Welzheim Welzheim)
	BREITENFÜRST_SCHULHAUS_2 = 'de:08119:5056:0:4'  # eRtg Reizenwiesen (Welzheim Welzheim)
	HINTERBÜCHELBERG = 'de:08119:5057'
	HINTERBÜCHELBERG_1 = 'de:08119:5057:0:3'  # eRtg Karnsberg (Murrhardt Murrhardt)
	HINTERBÜCHELBERG_2 = 'de:08119:5057:0:4'  # Rtg Waltersberg (Murrhardt Murrhardt)
	MURRHARDT_HINTERBÜCHELBERG = 'de:08119:5057'
	MURRHARDT_HINTERBÜCHELBERG_1 = 'de:08119:5057:0:3'  # eRtg Karnsberg (Murrhardt Murrhardt)
	MURRHARDT_HINTERBÜCHELBERG_2 = 'de:08119:5057:0:4'  # Rtg Waltersberg (Murrhardt Murrhardt)
	BETHEL = 'de:08119:5058'
	BETHEL_1 = 'de:08119:5058:0:3'  # Nord (Welzheim Welzheim)
	BETHEL_2 = 'de:08119:5058:0:4'  # Süd (Welzheim Welzheim)
	WELZHEIM_BETHEL = 'de:08119:5058'
	WELZHEIM_BETHEL_1 = 'de:08119:5058:0:3'  # Nord (Welzheim Welzheim)
	WELZHEIM_BETHEL_2 = 'de:08119:5058:0:4'  # Süd (Welzheim Welzheim)
	MURRHARDT_KARNSBERG = 'de:08119:5059'
	MURRHARDT_KARNSBERG_1 = 'de:08119:5059:0:3'  # eRtg Hinterbüchelberg (Murrhardt Murrhardt)
	MURRHARDT_KARNSBERG_2 = 'de:08119:5059:0:4'  # eRtg Siegelsberg (Murrhardt Murrhardt)
	KARNSBERG = 'de:08119:5059'
	KARNSBERG_1 = 'de:08119:5059:0:3'  # eRtg Hinterbüchelberg (Murrhardt Murrhardt)
	KARNSBERG_2 = 'de:08119:5059:0:4'  # eRtg Siegelsberg (Murrhardt Murrhardt)
	BUSBAHNHOF = 'de:08119:5060'
	BUSBAHNHOF_1 = 'de:08119:5060:1:1'  # Rtg Schorndorf (Welzheim Welzheim)
	BUSBAHNHOF_2 = 'de:08119:5060:1:2'  # Rtg Schorndorf (263) (Welzheim Welzheim)
	BUSBAHNHOF_3 = 'de:08119:5060:1:3'  # Rtg Kirchplatz/Friedhof/Schwäbisch Gmünd (Welzheim Welzheim)
	BUSBAHNHOF_4 = 'de:08119:5060:1:4'  # Rtg Welzheim Firma-Bauknecht/Althütte (Welzheim Welzheim)
	BUSBAHNHOF_5 = 'de:08119:5060:1:5'  # Rtg Walkersbach/Rudersberg (Welzheim Welzheim)
	BUSBAHNHOF_6 = 'de:08119:5060:1:6'  # Rtg Cronhütte/Rienharz (Welzheim Welzheim)
	BUSBAHNHOF_7 = 'de:08119:5060:1:A'  # Ankunftssteig (Welzheim Welzheim)
	WELZHEIM_BUSBAHNHOF = 'de:08119:5060'
	WELZHEIM_BUSBAHNHOF_1 = 'de:08119:5060:1:1'  # Rtg Schorndorf (Welzheim Welzheim)
	WELZHEIM_BUSBAHNHOF_2 = 'de:08119:5060:1:2'  # Rtg Schorndorf (263) (Welzheim Welzheim)
	WELZHEIM_BUSBAHNHOF_3 = 'de:08119:5060:1:3'  # Rtg Kirchplatz/Friedhof/Schwäbisch Gmünd (Welzheim Welzheim)
	WELZHEIM_BUSBAHNHOF_4 = 'de:08119:5060:1:4'  # Rtg Welzheim Firma-Bauknecht/Althütte (Welzheim Welzheim)
	WELZHEIM_BUSBAHNHOF_5 = 'de:08119:5060:1:5'  # Rtg Walkersbach/Rudersberg (Welzheim Welzheim)
	WELZHEIM_BUSBAHNHOF_6 = 'de:08119:5060:1:6'  # Rtg Cronhütte/Rienharz (Welzheim Welzheim)
	WELZHEIM_BUSBAHNHOF_7 = 'de:08119:5060:1:A'  # Ankunftssteig (Welzheim Welzheim)
	WELZHEIM_KIRCHPLATZ = 'de:08119:5061'
	WELZHEIM_KIRCHPLATZ_1 = 'de:08119:5061:0:3'  # eRtg Busbahnhof (Welzheim Welzheim)
	WELZHEIM_KIRCHPLATZ_2 = 'de:08119:5061:0:4'  # eRtg Gschwender Str/Schulz. (Welzheim Welzheim)
	KIRCHPLATZ_1 = 'de:08119:5061:0:3'  # eRtg Busbahnhof (Welzheim Welzheim)
	KIRCHPLATZ_2 = 'de:08119:5061:0:4'  # eRtg Gschwender Str/Schulz. (Welzheim Welzheim)
	KIRCHPLATZ_3 = 'de:08119:5379:0:3'  # eRtg Rosenstraße (Auenwald Oberbrüden)
	KIRCHPLATZ_4 = 'de:08119:5379:0:4'  # eRtg Mittelbrüden (Auenwald Oberbrüden)
	KIRCHPLATZ_5 = 'de:08119:5805:0:3'  # Rtg. Kelterstr./Grunbach (Remshalden Grunbach)
	KIRCHPLATZ_6 = 'de:08119:5805:0:4'  # Rtg Buoch (Nordic Walking Park) (Remshalden Grunbach)
	KIRCHPLATZ_7 = 'de:08117:3334:0:R1'  # Rtg Dürnau (Bad Boll Bad Boll)
	MURRHARDT_STEINBERG = 'de:08119:5062'
	MURRHARDT_STEINBERG_1 = 'de:08119:5062:0:3'  # eRtg Steinberg Halde (Murrhardt Murrhardt)
	MURRHARDT_STEINBERG_2 = 'de:08119:5062:0:4'  # eRtg Steinberg Fritzhof (Murrhardt Murrhardt)
	STEINBERG = 'de:08119:5062'
	STEINBERG_1 = 'de:08119:5062:0:3'  # eRtg Steinberg Halde (Murrhardt Murrhardt)
	STEINBERG_2 = 'de:08119:5062:0:4'  # eRtg Steinberg Fritzhof (Murrhardt Murrhardt)
	GOTTLOB_BAUKNECHT_PLATZ = 'de:08119:5063'
	GOTTLOB_BAUKNECHT_PLATZ_1 = 'de:08119:5063:0:3'  # eRtg Oberer Wasen (Welzheim Welzheim)
	GOTTLOB_BAUKNECHT_PLATZ_2 = 'de:08119:5063:0:4'  # eRtg Kirchplatz (Welzheim Welzheim)
	WELZH__GOTTL__BAUKNECHT_PLATZ = 'de:08119:5063'
	WELZH__GOTTL__BAUKNECHT_PLATZ_1 = 'de:08119:5063:0:3'  # eRtg Oberer Wasen (Welzheim Welzheim)
	WELZH__GOTTL__BAUKNECHT_PLATZ_2 = 'de:08119:5063:0:4'  # eRtg Kirchplatz (Welzheim Welzheim)
	WELZHEIM_SCHAFHOF = 'de:08119:5064'
	WELZHEIM_SCHAFHOF_1 = 'de:08119:5064:0:3'  # eRtg Aichstrut (Welzheim Welzheim)
	WELZHEIM_SCHAFHOF_2 = 'de:08119:5064:0:4'  # eRtg Steinbeise (Welzheim Welzheim)
	AICHSTRUT = 'de:08119:5065'
	AICHSTRUT_1 = 'de:08119:5065:0:3'  # Rtg Schafhof (Welzheim Welzheim)
	AICHSTRUT_2 = 'de:08119:5065:0:4'  # Rtg Gmeinweiler (Welzheim Welzheim)
	WELZHEIM_AICHSTRUT = 'de:08119:5065'
	WELZHEIM_AICHSTRUT_1 = 'de:08119:5065:0:3'  # Rtg Schafhof (Welzheim Welzheim)
	WELZHEIM_AICHSTRUT_2 = 'de:08119:5065:0:4'  # Rtg Gmeinweiler (Welzheim Welzheim)
	GMEINWEILER = 'de:08119:5066'
	GMEINWEILER_1 = 'de:08119:5066:0:3'  # eRtg Gebenweiler (Kaisersbach Kaisersbach)
	GMEINWEILER_2 = 'de:08119:5066:0:4'  # eRtg Aichstrut (Kaisersbach Kaisersbach)
	KAISERSBACH_GMEINWEILER = 'de:08119:5066'
	KAISERSBACH_GMEINWEILER_1 = 'de:08119:5066:0:3'  # eRtg Gebenweiler (Kaisersbach Kaisersbach)
	KAISERSBACH_GMEINWEILER_2 = 'de:08119:5066:0:4'  # eRtg Aichstrut (Kaisersbach Kaisersbach)
	GEBENWEILER = 'de:08119:5067'
	GEBENWEILER_1 = 'de:08119:5067:0:3'  # eRtg Grairich (Kaisersbach Kaisersbach)
	GEBENWEILER_2 = 'de:08119:5067:0:4'  # eRtg Gmeinweiler (Kaisersbach Kaisersbach)
	KAISERSBACH_GEBENWEILER = 'de:08119:5067'
	KAISERSBACH_GEBENWEILER_1 = 'de:08119:5067:0:3'  # eRtg Grairich (Kaisersbach Kaisersbach)
	KAISERSBACH_GEBENWEILER_2 = 'de:08119:5067:0:4'  # eRtg Gmeinweiler (Kaisersbach Kaisersbach)
	KAISERSBACH_GRAIRICH = 'de:08119:5068'
	KAISERSBACH_GRAIRICH_1 = 'de:08119:5068:0:3'  # eRtg Forststraße (Kaisersbach Kaisersbach)
	KAISERSBACH_GRAIRICH_2 = 'de:08119:5068:0:4'  # eRtg Gebenweiler (Kaisersbach Kaisersbach)
	GRAIRICH = 'de:08119:5068'
	GRAIRICH_1 = 'de:08119:5068:0:3'  # eRtg Forststraße (Kaisersbach Kaisersbach)
	GRAIRICH_2 = 'de:08119:5068:0:4'  # eRtg Gebenweiler (Kaisersbach Kaisersbach)
	KAISERSBACH_KREUZSTRASSE = 'de:08119:5071'
	KAISERSBACH_KREUZSTRASSE_1 = 'de:08119:5071:0:3'  # eRtg Eulenhof (Kaisersbach Kaisersbach)
	KAISERSBACH_KREUZSTRASSE_2 = 'de:08119:5071:0:4'  # eRtg Gausmannsweiler (Kaisersbach Kaisersbach)
	WELZHEIM_EIERHOF = 'de:08119:5072'
	WELZHEIM_EIERHOF_1 = 'de:08119:5072:0:3'  # eRtg Seiboldsweiler (Welzheim Welzheim)
	WELZHEIM_EIERHOF_2 = 'de:08119:5072:0:4'  # eRtg Oberer Wasen (Welzheim Welzheim)
	EIERHOF = 'de:08119:5072'
	EIERHOF_1 = 'de:08119:5072:0:3'  # eRtg Seiboldsweiler (Welzheim Welzheim)
	EIERHOF_2 = 'de:08119:5072:0:4'  # eRtg Oberer Wasen (Welzheim Welzheim)
	SEIBOLDSWEILER = 'de:08119:5073'
	SEIBOLDSWEILER_1 = 'de:08119:5073:0:3'  # eRtg Eckartsweiler (Welzheim Welzheim)
	SEIBOLDSWEILER_2 = 'de:08119:5073:0:4'  # eRtg Eierhof (Welzheim Welzheim)
	WELZHEIM_SEIBOLDSWEILER = 'de:08119:5073'
	WELZHEIM_SEIBOLDSWEILER_1 = 'de:08119:5073:0:3'  # eRtg Eckartsweiler (Welzheim Welzheim)
	WELZHEIM_SEIBOLDSWEILER_2 = 'de:08119:5073:0:4'  # eRtg Eierhof (Welzheim Welzheim)
	ECKARTSWEILER = 'de:08119:5074'
	ECKARTSWEILER_1 = 'de:08119:5074:0:3'  # eRtg Gausmannsweiler (Welzheim Welzheim)
	ECKARTSWEILER_2 = 'de:08119:5074:0:4'  # eRtg Seiboldsweiler (Welzheim Welzheim)
	WELZHEIM_ECKARTSWEILER = 'de:08119:5074'
	WELZHEIM_ECKARTSWEILER_1 = 'de:08119:5074:0:3'  # eRtg Gausmannsweiler (Welzheim Welzheim)
	WELZHEIM_ECKARTSWEILER_2 = 'de:08119:5074:0:4'  # eRtg Seiboldsweiler (Welzheim Welzheim)
	WELZH__GAUSMANNSWEILER = 'de:08119:5075'
	WELZH__GAUSMANNSWEILER_1 = 'de:08119:5075:0:3'  # eRtg Kreuzstraße (Welzheim Welzheim)
	WELZH__GAUSMANNSWEILER_2 = 'de:08119:5075:0:4'  # eRtg Eckartsweiler (Welzheim Welzheim)
	GAUSMANNSWEILER = 'de:08119:5075'
	GAUSMANNSWEILER_1 = 'de:08119:5075:0:3'  # eRtg Kreuzstraße (Welzheim Welzheim)
	GAUSMANNSWEILER_2 = 'de:08119:5075:0:4'  # eRtg Eckartsweiler (Welzheim Welzheim)
	KAISERSBACH_HAUPTSTRASSE = 'de:08119:5076'
	KAISERSBACH_HAUPTSTRASSE_1 = 'de:08119:5076:0:3'  # Rtg Heumaden/Forststr. (Kaisersbach Kaisersbach)
	KAISERSBACH_HAUPTSTRASSE_2 = 'de:08119:5076:0:4'  # Rtg Bruch (Kaisersbach Kaisersbach)
	KAISERSBACH_BRUCH = 'de:08119:5077'
	KAISERSBACH_BRUCH_1 = 'de:08119:5077:0:3'  # Rtg Mönchhof (Kaisersbach Kaisersbach)
	KAISERSBACH_BRUCH_2 = 'de:08119:5077:0:4'  # Rtg Hauptstr. (Kaisersbach Kaisersbach)
	MÖNCHHOF = 'de:08119:5078'
	MÖNCHHOF_1 = 'de:08119:5078:0:3'  # eRtg Bruch (Kaisersbach Kaisersbach)
	MÖNCHHOF_2 = 'de:08119:5078:0:4'  # eRtg Abzweig Altersberg (Kaisersbach Kaisersbach)
	KAISERSBACH_MÖNCHHOF = 'de:08119:5078'
	KAISERSBACH_MÖNCHHOF_1 = 'de:08119:5078:0:3'  # eRtg Bruch (Kaisersbach Kaisersbach)
	KAISERSBACH_MÖNCHHOF_2 = 'de:08119:5078:0:4'  # eRtg Abzweig Altersberg (Kaisersbach Kaisersbach)
	MURRH__STEINB__FRITZHOF = 'de:08119:5079'
	MURRH__STEINB__FRITZHOF_1 = 'de:08119:5079:0:3'  # eRtg Steinberg (Murrhardt Murrhardt)
	MURRH__STEINB__FRITZHOF_2 = 'de:08119:5079:0:4'  # eRtg Gymnasium (Murrhardt Murrhardt)
	STEINBERG_FRITZHOF = 'de:08119:5079'
	STEINBERG_FRITZHOF_1 = 'de:08119:5079:0:3'  # eRtg Steinberg (Murrhardt Murrhardt)
	STEINBERG_FRITZHOF_2 = 'de:08119:5079:0:4'  # eRtg Gymnasium (Murrhardt Murrhardt)
	KIRCHENKIRNBERG_BOHNREUTE = 'de:08119:5080'
	KIRCHENKIRNBERG_BOHNREUTE_1 = 'de:08119:5080:0:3'  # eRtg Abzweig Altersberg (Murrhardt Kirchenkirnberg)
	KIRCHENKIRNBERG_BOHNREUTE_2 = 'de:08119:5080:0:4'  # eRtg Gemeindehalle/Kirche (Murrhardt Kirchenkirnberg)
	BOHNREUTE = 'de:08119:5080'
	BOHNREUTE_1 = 'de:08119:5080:0:3'  # eRtg Abzweig Altersberg (Murrhardt Kirchenkirnberg)
	BOHNREUTE_2 = 'de:08119:5080:0:4'  # eRtg Gemeindehalle/Kirche (Murrhardt Kirchenkirnberg)
	KIRCHENK__GEMEINDEHALLE = 'de:08119:5081'
	KIRCHENK__GEMEINDEHALLE_1 = 'de:08119:5081:0:3'  # eRtg Bohnreute (Murrhardt Kirchenkirnberg)
	KIRCHENK__GEMEINDEHALLE_2 = 'de:08119:5081:0:4'  # eRtg Tiefenmad (Murrhardt Kirchenkirnberg)
	KIRCHENKIRNBERG_TIEFENMAD = 'de:08119:5083'
	KIRCHENKIRNBERG_TIEFENMAD_1 = 'de:08119:5083:0:3'  # eRtg Gemeindehalle (Murrhardt Kirchenkirnberg)
	KIRCHENKIRNBERG_TIEFENMAD_2 = 'de:08119:5083:0:4'  # eRtg Oberneustetten (Murrhardt Kirchenkirnberg)
	TIEFENMAD = 'de:08119:5083'
	TIEFENMAD_1 = 'de:08119:5083:0:3'  # eRtg Gemeindehalle (Murrhardt Kirchenkirnberg)
	TIEFENMAD_2 = 'de:08119:5083:0:4'  # eRtg Oberneustetten (Murrhardt Kirchenkirnberg)
	OBERNEUSTETTEN = 'de:08119:5084'
	OBERNEUSTETTEN_1 = 'de:08119:5084:0:3'  # Rtg Murrhardt (Murrhardt Kirchenkirnberg)
	OBERNEUSTETTEN_2 = 'de:08119:5084:0:4'  # Rtg Tiefenmad (Murrhardt Kirchenkirnberg)
	KIRCHENK__OBERNEUSTETTEN = 'de:08119:5084'
	KIRCHENK__OBERNEUSTETTEN_1 = 'de:08119:5084:0:3'  # Rtg Murrhardt (Murrhardt Kirchenkirnberg)
	KIRCHENK__OBERNEUSTETTEN_2 = 'de:08119:5084:0:4'  # Rtg Tiefenmad (Murrhardt Kirchenkirnberg)
	UNTERNEUSTETTEN = 'de:08119:5085'
	UNTERNEUSTETTEN_1 = 'de:08119:5085:0:3'  # eRtg Oberneustetten (Murrhardt Kirchenkirnberg)
	UNTERNEUSTETTEN_2 = 'de:08119:5085:0:4'  # eRtg Göckelhof Kreuzung (Murrhardt Kirchenkirnberg)
	KIRCHENK__UNTERNEUSTETTEN = 'de:08119:5085'
	KIRCHENK__UNTERNEUSTETTEN_1 = 'de:08119:5085:0:3'  # eRtg Oberneustetten (Murrhardt Kirchenkirnberg)
	KIRCHENK__UNTERNEUSTETTEN_2 = 'de:08119:5085:0:4'  # eRtg Göckelhof Kreuzung (Murrhardt Kirchenkirnberg)
	GÖCKELHOF_KREUZUNG = 'de:08119:5086'
	GÖCKELHOF_KREUZUNG_1 = 'de:08119:5086:0:3'  # eRtg Unterneustetten (Murrhardt Fornsbach)
	GÖCKELHOF_KREUZUNG_2 = 'de:08119:5086:0:4'  # eRtg Kindergarten/Waldsee (Murrhardt Fornsbach)
	FORNSB__GÖCKELHOF_KREUZUNG = 'de:08119:5086'
	FORNSB__GÖCKELHOF_KREUZUNG_1 = 'de:08119:5086:0:3'  # eRtg Unterneustetten (Murrhardt Fornsbach)
	FORNSB__GÖCKELHOF_KREUZUNG_2 = 'de:08119:5086:0:4'  # eRtg Kindergarten/Waldsee (Murrhardt Fornsbach)
	WELZH__BREIT__LORCHER_STR = 'de:08119:5087'
	WELZH__BREIT__LORCHER_STR_1 = 'de:08119:5087:0:3'  # eRtg Breitenf. Reinhold-Maier-Platz (Welzheim Welzheim)
	WELZH__BREIT__LORCHER_STR_2 = 'de:08119:5087:0:4'  # eRtg Neuhof (Welzheim Welzheim)
	BREITENFÜRST_LORCHER_STRASSE = 'de:08119:5087'
	BREITENFÜRST_LORCHER_STRASSE_1 = 'de:08119:5087:0:3'  # eRtg Breitenf. Reinhold-Maier-Platz (Welzheim Welzheim)
	BREITENFÜRST_LORCHER_STRASSE_2 = 'de:08119:5087:0:4'  # eRtg Neuhof (Welzheim Welzheim)
	WELZHEIM_NEUHOF = 'de:08119:5088'
	WELZHEIM_NEUHOF_1 = 'de:08119:5088:0:3'  # eRtg Breitenfürst Lorcher Straße (Welzheim Welzheim)
	WELZHEIM_NEUHOF_2 = 'de:08119:5088:0:4'  # eRtg Haghof (Welzheim Welzheim)
	PFAHLBRONN_HAGHOF = 'de:08119:5089'
	PFAHLBRONN_HAGHOF_1 = 'de:08119:5089:0:3'  # eRtg Neuhof (Alfdorf Pfahlbronn)
	PFAHLBRONN_HAGHOF_2 = 'de:08119:5089:0:4'  # eRtg Petershalde (Alfdorf Pfahlbronn)
	HAGHOF = 'de:08119:5089'
	HAGHOF_1 = 'de:08119:5089:0:3'  # eRtg Neuhof (Alfdorf Pfahlbronn)
	HAGHOF_2 = 'de:08119:5089:0:4'  # eRtg Petershalde (Alfdorf Pfahlbronn)
	PFAHLBRONN_PETERSHALDE = 'de:08119:5090'
	PFAHLBRONN_PETERSHALDE_1 = 'de:08119:5090:0:3'  # eRtg Haghof (Alfdorf Pfahlbronn)
	PFAHLBRONN_PETERSHALDE_2 = 'de:08119:5090:0:4'  # eRtg Haubenwasen (Alfdorf Pfahlbronn)
	PETERSHALDE = 'de:08119:5090'
	PETERSHALDE_1 = 'de:08119:5090:0:3'  # eRtg Haghof (Alfdorf Pfahlbronn)
	PETERSHALDE_2 = 'de:08119:5090:0:4'  # eRtg Haubenwasen (Alfdorf Pfahlbronn)
	PFAHLBRONN_WELZHEIMER_STR = 'de:08119:5091'
	PFAHLBRONN_WELZHEIMER_STR_1 = 'de:08119:5091:0:3'  # eRtg Haubenwasen (Alfdorf Pfahlbronn)
	PFAHLBRONN_WELZHEIMER_STR_2 = 'de:08119:5091:0:4'  # eRtg Lorcher Straße (Alfdorf Pfahlbronn)
	LORCHER_STRASSE = 'de:08119:5092'
	LORCHER_STRASSE_1 = 'de:08119:5092:0:3'  # Rtg Welzheimer Str. (Alfdorf Pfahlbronn)
	LORCHER_STRASSE_2 = 'de:08119:5092:0:4'  # eRtg Untere Bühlstraße (Alfdorf Pfahlbronn)
	PFAHLBRONN_LORCHER_STR = 'de:08119:5092'
	PFAHLBRONN_LORCHER_STR_1 = 'de:08119:5092:0:3'  # Rtg Welzheimer Str. (Alfdorf Pfahlbronn)
	PFAHLBRONN_LORCHER_STR_2 = 'de:08119:5092:0:4'  # eRtg Untere Bühlstraße (Alfdorf Pfahlbronn)
	ABZWEIG_ALTERSBERG = 'de:08119:5093'
	ABZWEIG_ALTERSBERG_1 = 'de:08119:5093:0:3'  # eRtg Mönchhof (Kaisersbach Kaisersbach)
	ABZWEIG_ALTERSBERG_2 = 'de:08119:5093:0:4'  # eRtg Bohnreute (Kaisersbach Kaisersbach)
	KAISERSBACH_ABZW__ALTERSBERG = 'de:08119:5093'
	KAISERSBACH_ABZW__ALTERSBERG_1 = 'de:08119:5093:0:3'  # eRtg Mönchhof (Kaisersbach Kaisersbach)
	KAISERSBACH_ABZW__ALTERSBERG_2 = 'de:08119:5093:0:4'  # eRtg Bohnreute (Kaisersbach Kaisersbach)
	VORDERSTEINENBERG = 'de:08119:5095'
	VORDERSTEINENBERG_1 = 'de:08119:5095:0:3'  # Rtg Kapf (Alfdorf Vordersteinenberg)
	VORDERSTEINENBERG_2 = 'de:08119:5095:0:4'  # Rtg Hintersteinenberg (Alfdorf Vordersteinenberg)
	VORDERST__KAPF_MOHNWIESENWEG = 'de:08119:5096'
	VORDERST__KAPF_MOHNWIESENWEG_1 = 'de:08119:5096:0:3'  # eRtg Vordersteinenberg (Alfdorf Vordersteinenberg)
	VORDERST__KAPF_MOHNWIESENWEG_2 = 'de:08119:5096:0:4'  # eRtg Kapf Kapfhofweg (Alfdorf Vordersteinenberg)
	KAPF_MOHNWIESENWEG = 'de:08119:5096'
	KAPF_MOHNWIESENWEG_1 = 'de:08119:5096:0:3'  # eRtg Vordersteinenberg (Alfdorf Vordersteinenberg)
	KAPF_MOHNWIESENWEG_2 = 'de:08119:5096:0:4'  # eRtg Kapf Kapfhofweg (Alfdorf Vordersteinenberg)
	KAPF_KAPFHOFWEG = 'de:08119:5097'
	KAPF_KAPFHOFWEG_1 = 'de:08119:5097:0:3'  # Rtg Strübelmühle (Alfdorf Vordersteinenberg)
	KAPF_KAPFHOFWEG_2 = 'de:08119:5097:0:4'  # Rtg Mohnwiesenweg (Alfdorf Vordersteinenberg)
	VORDERST__KAPF_KAPFHOFWEG = 'de:08119:5097'
	VORDERST__KAPF_KAPFHOFWEG_1 = 'de:08119:5097:0:3'  # Rtg Strübelmühle (Alfdorf Vordersteinenberg)
	VORDERST__KAPF_KAPFHOFWEG_2 = 'de:08119:5097:0:4'  # Rtg Mohnwiesenweg (Alfdorf Vordersteinenberg)
	STRÜBELMÜHLE = 'de:08119:5098'
	STRÜBELMÜHLE_1 = 'de:08119:5098:0:3'  # eRtg Kapf Kapfhofweg (Alfdorf Alfdorf)
	STRÜBELMÜHLE_2 = 'de:08119:5098:0:4'  # eRtg Döllenstraße (Alfdorf Alfdorf)
	ALFDORF_STRÜBELMÜHLE = 'de:08119:5098'
	ALFDORF_STRÜBELMÜHLE_1 = 'de:08119:5098:0:3'  # eRtg Kapf Kapfhofweg (Alfdorf Alfdorf)
	ALFDORF_STRÜBELMÜHLE_2 = 'de:08119:5098:0:4'  # eRtg Döllenstraße (Alfdorf Alfdorf)
	ALFDORF_MARKTPLATZ = 'de:08119:5099:0:3'  # eRtg alle Richtungen (Alfdorf Alfdorf)
	UNTERE_BÜHLSTRASSE = 'de:08119:5100'
	UNTERE_BÜHLSTRASSE_1 = 'de:08119:5100:0:3'  # eRtg Lorcher Straße (Alfdorf Alfdorf)
	UNTERE_BÜHLSTRASSE_2 = 'de:08119:5100:0:4'  # eRtg Marktplatz (Alfdorf Alfdorf)
	ALFD__UNTERE_BÜHLSTRASSE = 'de:08119:5100'
	ALFD__UNTERE_BÜHLSTRASSE_1 = 'de:08119:5100:0:3'  # eRtg Lorcher Straße (Alfdorf Alfdorf)
	ALFD__UNTERE_BÜHLSTRASSE_2 = 'de:08119:5100:0:4'  # eRtg Marktplatz (Alfdorf Alfdorf)
	BRECH = 'de:08119:5101'
	BRECH_1 = 'de:08119:5101:0:3'  # Rtg Lorch (Alfdorf Pfahlbronn)
	BRECH_2 = 'de:08119:5101:0:4'  # Rtg Alfdorf (Alfdorf Pfahlbronn)
	PFAHLBRONN_BRECH = 'de:08119:5101'
	PFAHLBRONN_BRECH_1 = 'de:08119:5101:0:3'  # Rtg Lorch (Alfdorf Pfahlbronn)
	PFAHLBRONN_BRECH_2 = 'de:08119:5101:0:4'  # Rtg Alfdorf (Alfdorf Pfahlbronn)
	BRECH_KAISERLINDE = 'de:08119:5102'
	BRECH_KAISERLINDE_1 = 'de:08119:5102:0:3'  # eRtg Skilift (Alfdorf Pfahlbronn)
	BRECH_KAISERLINDE_2 = 'de:08119:5102:0:4'  # eRtg Brech (Alfdorf Pfahlbronn)
	PFAHLBR__BRECH_KAISERLINDE = 'de:08119:5102'
	PFAHLBR__BRECH_KAISERLINDE_1 = 'de:08119:5102:0:3'  # eRtg Skilift (Alfdorf Pfahlbronn)
	PFAHLBR__BRECH_KAISERLINDE_2 = 'de:08119:5102:0:4'  # eRtg Brech (Alfdorf Pfahlbronn)
	UNTERE_SCHLOSSSTRASSE = 'de:08119:5103'
	UNTERE_SCHLOSSSTRASSE_1 = 'de:08119:5103:0:3'  # eRtg Marktplatz (Alfdorf Alfdorf)
	UNTERE_SCHLOSSSTRASSE_2 = 'de:08119:5103:0:4'  # eRtg Bonholz Abzweig (Alfdorf Alfdorf)
	ALFD__UNTERE_SCHLOSSSTRASSE = 'de:08119:5103'
	ALFD__UNTERE_SCHLOSSSTRASSE_1 = 'de:08119:5103:0:3'  # eRtg Marktplatz (Alfdorf Alfdorf)
	ALFD__UNTERE_SCHLOSSSTRASSE_2 = 'de:08119:5103:0:4'  # eRtg Bonholz Abzweig (Alfdorf Alfdorf)
	BONHOLZ_ABZWEIG = 'de:08119:5104'
	BONHOLZ_ABZWEIG_1 = 'de:08119:5104:0:3'  # Rtg Adelstetten (Alfdorf Alfdorf)
	BONHOLZ_ABZWEIG_2 = 'de:08119:5104:0:4'  # Rtg Alfdorf (Alfdorf Alfdorf)
	ALFDORF_BONHOLZ_ABZW = 'de:08119:5104'
	ALFDORF_BONHOLZ_ABZW_1 = 'de:08119:5104:0:3'  # Rtg Adelstetten (Alfdorf Alfdorf)
	ALFDORF_BONHOLZ_ABZW_2 = 'de:08119:5104:0:4'  # Rtg Alfdorf (Alfdorf Alfdorf)
	ADELSTETTEN = 'de:08119:5105'
	ADELSTETTEN_1 = 'de:08119:5105:0:3'  # eRtg Bonholz Abzweig (Alfdorf Pfahlbronn)
	ADELSTETTEN_2 = 'de:08119:5105:0:4'  # eRtg Pfersbach Rössle (Alfdorf Pfahlbronn)
	PFAHLBRONN_ADELSTETTEN = 'de:08119:5105'
	PFAHLBRONN_ADELSTETTEN_1 = 'de:08119:5105:0:3'  # eRtg Bonholz Abzweig (Alfdorf Pfahlbronn)
	PFAHLBRONN_ADELSTETTEN_2 = 'de:08119:5105:0:4'  # eRtg Pfersbach Rössle (Alfdorf Pfahlbronn)
	GRAB_HOHENBRACH = 'de:08119:5106'
	GRAB_HOHENBRACH_1 = 'de:08119:5106:0:3'  # eRtg Erlach (Großerlach Grab)
	GRAB_HOHENBRACH_2 = 'de:08119:5106:0:4'  # eRtg Grab (Großerlach Grab)
	HOHENBRACH = 'de:08119:5106'
	HOHENBRACH_1 = 'de:08119:5106:0:3'  # eRtg Erlach (Großerlach Grab)
	HOHENBRACH_2 = 'de:08119:5106:0:4'  # eRtg Grab (Großerlach Grab)
	HAUBERSBRONN_RATHAUS = 'de:08119:5107'
	HAUBERSBRONN_RATHAUS_1 = 'de:08119:5107:1:3'  # Rtg Mitte/Miedelsbacher Str. (Schorndorf Haubersbronn)
	HAUBERSBRONN_RATHAUS_2 = 'de:08119:5107:1:4'  # eRtg Haubersbronn/Schorndorf (Schorndorf Haubersbronn)
	HAUBERSBR__MIEDELSB__STR = 'de:08119:5108'
	HAUBERSBR__MIEDELSB__STR_1 = 'de:08119:5108:0:3'  # Rtg Im Hiller (Schorndorf Haubersbronn)
	HAUBERSBR__MIEDELSB__STR_2 = 'de:08119:5108:0:4'  # Rtg Haubersbronn Mitte/Rathaus (Schorndorf Haubersbronn)
	MIEDELSBACHER_STRASSE = 'de:08119:5108'
	MIEDELSBACHER_STRASSE_1 = 'de:08119:5108:0:3'  # Rtg Im Hiller (Schorndorf Haubersbronn)
	MIEDELSBACHER_STRASSE_2 = 'de:08119:5108:0:4'  # Rtg Haubersbronn Mitte/Rathaus (Schorndorf Haubersbronn)
	MIEDELSBACH_IM_HILLER = 'de:08119:5109'
	MIEDELSBACH_IM_HILLER_1 = 'de:08119:5109:0:3'  # Rtg Rathaus/Tannbachbrücke (Schorndorf Miedelsbach)
	MIEDELSBACH_IM_HILLER_2 = 'de:08119:5109:0:4'  # Rtg. Haubersbronn (Schorndorf Miedelsbach)
	IM_HILLER = 'de:08119:5109'
	IM_HILLER_1 = 'de:08119:5109:0:3'  # Rtg Rathaus/Tannbachbrücke (Schorndorf Miedelsbach)
	IM_HILLER_2 = 'de:08119:5109:0:4'  # Rtg. Haubersbronn (Schorndorf Miedelsbach)
	STEINENBERG_SCHORNDORFER_STR = 'de:08119:5111'
	STEINENBERG_SCHORNDORFER_STR_1 = 'de:08119:5111:0:3'  # Rtg Miedelsbach (Steinenberg) (Rudersberg Steinenberg)
	STEINENBERG_SCHORNDORFER_STR_2 = 'de:08119:5111:0:4'  # eRtg Michelauer Str./Turnhalle/Ortsmitte (Rudersberg Steinenberg)
	MICHELAUER_STRASSE = 'de:08119:5112'
	MICHELAUER_STRASSE_1 = 'de:08119:5112:0:3'  # Rtg.Steinenb. Schornd.Str. (Rudersberg Steinenberg)
	MICHELAUER_STRASSE_2 = 'de:08119:5112:0:4'  # Rtg. Michelau Ortsmitte (Rudersberg Steinenberg)
	STEINENBERG_MICHELAUER_STR = 'de:08119:5112'
	STEINENBERG_MICHELAUER_STR_1 = 'de:08119:5112:0:3'  # Rtg.Steinenb. Schornd.Str. (Rudersberg Steinenberg)
	STEINENBERG_MICHELAUER_STR_2 = 'de:08119:5112:0:4'  # Rtg. Michelau Ortsmitte (Rudersberg Steinenberg)
	MICHELAU_ORTSMITTE = 'de:08119:5113'
	MICHELAU_ORTSMITTE_1 = 'de:08119:5113:1:3'  # Rtg Michelau Bf/Schlechb./Asperglen (Rudersberg Schlechtbach)
	MICHELAU_ORTSMITTE_2 = 'de:08119:5113:1:4'  # Rtg Steinenberg Turnhalle/Michelauer Str (Rudersberg Schlechtbach)
	SCHLECHTB__MICHELAU_ORTSMITTE = 'de:08119:5113'
	SCHLECHTB__MICHELAU_ORTSMITTE_1 = 'de:08119:5113:1:3'  # Rtg Michelau Bf/Schlechb./Asperglen (Rudersberg Schlechtbach)
	SCHLECHTB__MICHELAU_ORTSMITTE_2 = 'de:08119:5113:1:4'  # Rtg Steinenberg Turnhalle/Michelauer Str (Rudersberg Schlechtbach)
	ASPERGLEN_KREHWINKEL = 'de:08119:5114'
	ASPERGLEN_KREHWINKEL_1 = 'de:08119:5114:0:3'  # Rtg. Asperglen (Rudersberg Asperglen)
	ASPERGLEN_KREHWINKEL_2 = 'de:08119:5114:0:4'  # Rtg. Necklinsberg (Rudersberg Asperglen)
	KREHWINKEL = 'de:08119:5114'
	KREHWINKEL_1 = 'de:08119:5114:0:3'  # Rtg. Asperglen (Rudersberg Asperglen)
	KREHWINKEL_2 = 'de:08119:5114:0:4'  # Rtg. Necklinsberg (Rudersberg Asperglen)
	ASPERGLEN = 'de:08119:5115'
	ASPERGLEN_1 = 'de:08119:5115:0:3'  # Rtg Krehwinkel (Rudersberg Asperglen)
	ASPERGLEN_2 = 'de:08119:5115:0:4'  # Rtg Steinenberg (Rudersberg Asperglen)
	SCHLECHTBACH_LINDENTAL = 'de:08119:5117:0:3'  # eRtg Alle Richtungen (Rudersberg Schlechtbach)
	LINDENTAL = 'de:08119:5117:0:3'  # eRtg Alle Richtungen (Rudersberg Schlechtbach)
	SCHLECHTBACH_ORTSMITTE = 'de:08119:5118'
	SCHLECHTBACH_ORTSMITTE_1 = 'de:08119:5118:1:3'  # Rtg Rudersberg (Rudersberg Schlechtbach)
	SCHLECHTBACH_ORTSMITTE_2 = 'de:08119:5118:1:4'  # Rtg Lindentaler Str./Michelau (Rudersberg Schlechtbach)
	RUDERSBERG_MARKTPLATZ = 'de:08119:5119'
	RUDERSBERG_MARKTPLATZ_1 = 'de:08119:5119:0:3'  # Rtg Rudersberg Nord/Brühlstr. (Rudersberg Rudersberg)
	RUDERSBERG_MARKTPLATZ_2 = 'de:08119:5119:0:4'  # Rtg Rudersberg Bf (Rudersberg Rudersberg)
	SCHLECHTB__KELTERSIEDLUNG = 'de:08119:5120'
	SCHLECHTB__KELTERSIEDLUNG_1 = 'de:08119:5120:0:3'  # eRtg Lindental (Rudersberg Schlechtbach)
	SCHLECHTB__KELTERSIEDLUNG_2 = 'de:08119:5120:0:4'  # eRtg Schlechtbach (Rudersberg Schlechtbach)
	KELTERSIEDLUNG = 'de:08119:5120'
	KELTERSIEDLUNG_1 = 'de:08119:5120:0:3'  # eRtg Lindental (Rudersberg Schlechtbach)
	KELTERSIEDLUNG_2 = 'de:08119:5120:0:4'  # eRtg Schlechtbach (Rudersberg Schlechtbach)
	RUDERSBERG_SCHULZENTRUM = 'de:08119:5121:0:3'  # eRtg Alle Richtungen (Rudersberg Rudersberg)
	STEINENBERG_TURNHALLE = 'de:08119:5122:0:3'  # eRtg Michelau (Rudersberg Steinenberg)
	OBERER_WASEN = 'de:08119:5123'
	OBERER_WASEN_1 = 'de:08119:5123:0:3'  # eRtg Steinbeise (Welzheim Welzheim)
	OBERER_WASEN_2 = 'de:08119:5123:0:4'  # eRtg Gottlob-Bauknecht-Platz (Welzheim Welzheim)
	WELZHEIM_OBERER_WASEN = 'de:08119:5123'
	WELZHEIM_OBERER_WASEN_1 = 'de:08119:5123:0:3'  # eRtg Steinbeise (Welzheim Welzheim)
	WELZHEIM_OBERER_WASEN_2 = 'de:08119:5123:0:4'  # eRtg Gottlob-Bauknecht-Platz (Welzheim Welzheim)
	ABZWEIG_STEINBACH = 'de:08119:5124'
	ABZWEIG_STEINBACH_1 = 'de:08119:5124:0:3'  # eRtg Abzweig Schmalenberg (Rudersberg Rudersberg)
	ABZWEIG_STEINBACH_2 = 'de:08119:5124:0:4'  # eRtg Klaffenbach (Rudersberg Rudersberg)
	RUDERSBERG_ABZW__STEINBACH = 'de:08119:5124'
	RUDERSBERG_ABZW__STEINBACH_1 = 'de:08119:5124:0:3'  # eRtg Abzweig Schmalenberg (Rudersberg Rudersberg)
	RUDERSBERG_ABZW__STEINBACH_2 = 'de:08119:5124:0:4'  # eRtg Klaffenbach (Rudersberg Rudersberg)
	ABZWEIG_SCHMALENBERG = 'de:08119:5125'
	ABZWEIG_SCHMALENBERG_1 = 'de:08119:5125:0:3'  # eRtg Klingenmühle (Rudersberg Rudersberg)
	ABZWEIG_SCHMALENBERG_2 = 'de:08119:5125:0:4'  # eRtg Abzweig Steinbach (Rudersberg Rudersberg)
	RUD__ABZW__SCHMALENBERG = 'de:08119:5125'
	RUD__ABZW__SCHMALENBERG_1 = 'de:08119:5125:0:3'  # eRtg Klingenmühle (Rudersberg Rudersberg)
	RUD__ABZW__SCHMALENBERG_2 = 'de:08119:5125:0:4'  # eRtg Abzweig Steinbach (Rudersberg Rudersberg)
	WELZHEIM_LAUFENMÜHLE = 'de:08119:5127'
	WELZHEIM_LAUFENMÜHLE_1 = 'de:08119:5127:2:3'  # Rtg Welzheim Amselweg (Welzheim Welzheim)
	WELZHEIM_LAUFENMÜHLE_2 = 'de:08119:5127:2:4'  # Rtg Klingenmühle (Welzheim Welzheim)
	LAUFENMÜHLE = 'de:08119:5127'
	LAUFENMÜHLE_1 = 'de:08119:5127:2:3'  # Rtg Welzheim Amselweg (Welzheim Welzheim)
	LAUFENMÜHLE_2 = 'de:08119:5127:2:4'  # Rtg Klingenmühle (Welzheim Welzheim)
	WELZHEIM_AMSELWEG = 'de:08119:5128'
	WELZHEIM_AMSELWEG_1 = 'de:08119:5128:0:3'  # eRtg Friedhof (Welzheim Welzheim)
	WELZHEIM_AMSELWEG_2 = 'de:08119:5128:0:4'  # eRtg Laufenmühle (Welzheim Welzheim)
	AMSELWEG_1 = 'de:08119:5128:0:3'  # eRtg Friedhof (Welzheim Welzheim)
	AMSELWEG_2 = 'de:08119:5128:0:4'  # eRtg Laufenmühle (Welzheim Welzheim)
	AMSELWEG_3 = 'de:08118:6811:0:3'  # eRtg Kurze Straße (Gemmrigheim Gemmrigheim)
	AMSELWEG_4 = 'de:08115:7040:0:3'  # eRtg Finkenweg (Herrenberg Herrenberg)
	WELZHEIM_FRIEDHOF = 'de:08119:5129'
	WELZHEIM_FRIEDHOF_1 = 'de:08119:5129:0:3'  # eRtg Kirchplatz (Welzheim Welzheim)
	WELZHEIM_FRIEDHOF_2 = 'de:08119:5129:0:4'  # eRtg Amselweg (Welzheim Welzheim)
	STEINBEISE = 'de:08119:5130'
	STEINBEISE_1 = 'de:08119:5130:0:3'  # eRtg Schafhof (Welzheim Welzheim)
	STEINBEISE_2 = 'de:08119:5130:0:4'  # eRtg Gschwender Straße (Welzheim Welzheim)
	WELZHEIM_STEINBEISE = 'de:08119:5130'
	WELZHEIM_STEINBEISE_1 = 'de:08119:5130:0:3'  # eRtg Schafhof (Welzheim Welzheim)
	WELZHEIM_STEINBEISE_2 = 'de:08119:5130:0:4'  # eRtg Gschwender Straße (Welzheim Welzheim)
	WELZHEIM_SCHULZENTRUM = 'de:08119:5131:0:3'  # eRtg Alle Richtungen (Welzheim Welzheim)
	WELZH__BREITENF__R__MAIER_PL = 'de:08119:5132'
	WELZH__BREITENF__R__MAIER_PL_1 = 'de:08119:5132:0:3'  # eRtg Bethel (Welzheim Welzheim)
	WELZH__BREITENF__R__MAIER_PL_2 = 'de:08119:5132:0:4'  # eRtg Breitenfürst Schulhaus (Welzheim Welzheim)
	BREITENF__REINHOLD_MAIER_PLATZ = 'de:08119:5132'
	BREITENF__REINHOLD_MAIER_PLATZ_1 = 'de:08119:5132:0:3'  # eRtg Bethel (Welzheim Welzheim)
	BREITENF__REINHOLD_MAIER_PLATZ_2 = 'de:08119:5132:0:4'  # eRtg Breitenfürst Schulhaus (Welzheim Welzheim)
	URBACH_HAUPTSTRASSE = 'de:08119:5134'
	URBACH_HAUPTSTRASSE_1 = 'de:08119:5134:1:3'  # Rtg Plüderhausen (Urbach Urbach)
	URBACH_HAUPTSTRASSE_2 = 'de:08119:5134:1:4'  # Rtg Schorndorf (Urbach Urbach)
	ROBERT_FRANCK_STRASSE = 'de:08119:5136'
	ROBERT_FRANCK_STRASSE_1 = 'de:08119:5136:0:3'  # eRtg Murrhärle (Murrhardt Murrhardt)
	ROBERT_FRANCK_STRASSE_2 = 'de:08119:5136:0:4'  # eRtg Murrhardt (Murrhardt Murrhardt)
	MURRH__ROBERT_FRANCK_STRASSE = 'de:08119:5136'
	MURRH__ROBERT_FRANCK_STRASSE_1 = 'de:08119:5136:0:3'  # eRtg Murrhärle (Murrhardt Murrhardt)
	MURRH__ROBERT_FRANCK_STRASSE_2 = 'de:08119:5136:0:4'  # eRtg Murrhardt (Murrhardt Murrhardt)
	MURRHARDT_MURRHÄRLE = 'de:08119:5138'
	MURRHARDT_MURRHÄRLE_1 = 'de:08119:5138:0:3'  # eRtg Kieselhof (Murrhardt Murrhardt)
	MURRHARDT_MURRHÄRLE_2 = 'de:08119:5138:0:4'  # eRtg Robert-Franck-Straße (Murrhardt Murrhardt)
	MURRHÄRLE = 'de:08119:5138'
	MURRHÄRLE_1 = 'de:08119:5138:0:3'  # eRtg Kieselhof (Murrhardt Murrhardt)
	MURRHÄRLE_2 = 'de:08119:5138:0:4'  # eRtg Robert-Franck-Straße (Murrhardt Murrhardt)
	KIESELHOF = 'de:08119:5139'
	KIESELHOF_1 = 'de:08119:5139:0:3'  # eRtg Hördthof (Murrhardt Murrhardt)
	KIESELHOF_2 = 'de:08119:5139:0:4'  # eRtg Murrhärle (Murrhardt Murrhardt)
	MURRHARDT_KIESELHOF = 'de:08119:5139'
	MURRHARDT_KIESELHOF_1 = 'de:08119:5139:0:3'  # eRtg Hördthof (Murrhardt Murrhardt)
	MURRHARDT_KIESELHOF_2 = 'de:08119:5139:0:4'  # eRtg Murrhärle (Murrhardt Murrhardt)
	MURRHARDT_HÖRDTHOF = 'de:08119:5140'
	MURRHARDT_HÖRDTHOF_1 = 'de:08119:5140:0:3'  # eRtg Frankenweiler (Murrhardt Murrhardt)
	MURRHARDT_HÖRDTHOF_2 = 'de:08119:5140:0:4'  # eRtg Kieselhof (Murrhardt Murrhardt)
	HÖRDTHOF = 'de:08119:5140'
	HÖRDTHOF_1 = 'de:08119:5140:0:3'  # eRtg Frankenweiler (Murrhardt Murrhardt)
	HÖRDTHOF_2 = 'de:08119:5140:0:4'  # eRtg Kieselhof (Murrhardt Murrhardt)
	GRAB_FRANKENWEILER = 'de:08119:5141'
	GRAB_FRANKENWEILER_1 = 'de:08119:5141:0:3'  # eRtg Trauzenbach (Großerlach Grab)
	GRAB_FRANKENWEILER_2 = 'de:08119:5141:0:4'  # eRtg Hördthof (Großerlach Grab)
	FRANKENWEILER = 'de:08119:5141'
	FRANKENWEILER_1 = 'de:08119:5141:0:3'  # eRtg Trauzenbach (Großerlach Grab)
	FRANKENWEILER_2 = 'de:08119:5141:0:4'  # eRtg Hördthof (Großerlach Grab)
	GRAB_TRAUZENBACH = 'de:08119:5142'
	GRAB_TRAUZENBACH_1 = 'de:08119:5142:0:3'  # eRtg Grab (Großerlach Grab)
	GRAB_TRAUZENBACH_2 = 'de:08119:5142:0:4'  # eRtg Frankenweiler (Großerlach Grab)
	TRAUZENBACH = 'de:08119:5142'
	TRAUZENBACH_1 = 'de:08119:5142:0:3'  # eRtg Grab (Großerlach Grab)
	TRAUZENBACH_2 = 'de:08119:5142:0:4'  # eRtg Frankenweiler (Großerlach Grab)
	LIMESWEG_1 = 'de:08119:5143:0:3'  # eRtg Alle Richtungen (Großerlach Grab)
	LIMESWEG_2 = 'de:08117:1216:0:H1'  # Rtg Wäschenbeuren (Göppingen Maitis)
	GRAB_LIMESWEG = 'de:08119:5143:0:3'  # eRtg Alle Richtungen (Großerlach Grab)
	WAIBLINGEN_BLUMENSTRASSE = 'de:08119:5144'
	WAIBLINGEN_BLUMENSTRASSE_1 = 'de:08119:5144:1:3'  # Rtg Stadtmitte (Waiblingen Waiblingen)
	WAIBLINGEN_BLUMENSTRASSE_2 = 'de:08119:5144:1:4'  # Rtg Obere Bf-str. (Waiblingen Waiblingen)
	WAIBL__BÜRGERZ___REMSBRÜCKE = 'de:08119:5145'
	WAIBL__BÜRGERZ___REMSBRÜCKE_1 = 'de:08119:5145:1:3'  # Rtg Schwanen/Galerie (Waiblingen Waiblingen)
	WAIBL__BÜRGERZ___REMSBRÜCKE_2 = 'de:08119:5145:1:4'  # Rtg Bürgerzentrum (Hallenbad)/Rundsporth (Waiblingen Waiblingen)
	BÜRGERZENTRUM__REMSBRÜCKE = 'de:08119:5145'
	BÜRGERZENTRUM__REMSBRÜCKE_1 = 'de:08119:5145:1:3'  # Rtg Schwanen/Galerie (Waiblingen Waiblingen)
	BÜRGERZENTRUM__REMSBRÜCKE_2 = 'de:08119:5145:1:4'  # Rtg Bürgerzentrum (Hallenbad)/Rundsporth (Waiblingen Waiblingen)
	WAIBLINGEN_NEUSTÄDTER_STRASSE = 'de:08119:5146:0:4'  # Rtg stadteinwärts (Waiblingen Waiblingen)
	NEUSTÄDTER_STRASSE = 'de:08119:5146:0:4'  # Rtg stadteinwärts (Waiblingen Waiblingen)
	ALTE_WAIBLINGER_STRASSE = 'de:08119:5147'
	ALTE_WAIBLINGER_STRASSE_1 = 'de:08119:5147:0:3'  # Rtg Alte Schule (Waiblingen Neustadt)
	ALTE_WAIBLINGER_STRASSE_2 = 'de:08119:5147:0:4'  # Rtg Waiblingen Neustädter Str. (Waiblingen Neustadt)
	NEUSTADT_A__WAIBLINGER_STR = 'de:08119:5147'
	NEUSTADT_A__WAIBLINGER_STR_1 = 'de:08119:5147:0:3'  # Rtg Alte Schule (Waiblingen Neustadt)
	NEUSTADT_A__WAIBLINGER_STR_2 = 'de:08119:5147:0:4'  # Rtg Waiblingen Neustädter Str. (Waiblingen Neustadt)
	NEUSTADT_RATHAUS = 'de:08119:5148'
	NEUSTADT_RATHAUS_1 = 'de:08119:5148:0:3'  # Rtg Torstr. (Waiblingen Neustadt)
	NEUSTADT_RATHAUS_2 = 'de:08119:5148:0:4'  # Rtg Alte Schule (Waiblingen Neustadt)
	FRIEDENSSCHULE = 'de:08119:5149:0:3'  # Rtg Alle Richtungen (Waiblingen Neustadt)
	NEUSTADT_FRIEDENSSCHULE = 'de:08119:5149:0:3'  # Rtg Alle Richtungen (Waiblingen Neustadt)
	HOHENACKER_BILDÄCKERSTR = 'de:08119:5150'
	HOHENACKER_BILDÄCKERSTR_1 = 'de:08119:5150:0:3'  # Rtg Bittenfeld/Ortsmitte (Waiblingen Hohenacker)
	HOHENACKER_BILDÄCKERSTR_2 = 'de:08119:5150:0:4'  # Rtg Kriegsberg (Waiblingen Hohenacker)
	BILDÄCKERSTRASSE = 'de:08119:5150'
	BILDÄCKERSTRASSE_1 = 'de:08119:5150:0:3'  # Rtg Bittenfeld/Ortsmitte (Waiblingen Hohenacker)
	BILDÄCKERSTRASSE_2 = 'de:08119:5150:0:4'  # Rtg Kriegsberg (Waiblingen Hohenacker)
	HOHENACKER_ORTSMITTE = 'de:08119:5151'
	HOHENACKER_ORTSMITTE_1 = 'de:08119:5151:0:3'  # Rtg Hegnacher Str. (Waiblingen Hohenacker)
	HOHENACKER_ORTSMITTE_2 = 'de:08119:5151:0:4'  # Rtg Bildäckerstr. (Waiblingen Hohenacker)
	HOHENACKER_BITTENF__STR = 'de:08119:5152'
	HOHENACKER_BITTENF__STR_1 = 'de:08119:5152:0:3'  # Rtg Bittenfeld Rienzhofer Mühle (Waiblingen Hohenacker)
	HOHENACKER_BITTENF__STR_2 = 'de:08119:5152:0:4'  # Rtg Hegnacher Str. (Waiblingen Hohenacker)
	BITTENFELDER_STRASSE = 'de:08119:5152'
	BITTENFELDER_STRASSE_1 = 'de:08119:5152:0:3'  # Rtg Bittenfeld Rienzhofer Mühle (Waiblingen Hohenacker)
	BITTENFELDER_STRASSE_2 = 'de:08119:5152:0:4'  # Rtg Hegnacher Str. (Waiblingen Hohenacker)
	RIENZHOFER_MÜHLE = 'de:08119:5154'
	RIENZHOFER_MÜHLE_1 = 'de:08119:5154:0:3'  # Rtg Am Zipfelbach (Waiblingen Bittenfeld)
	RIENZHOFER_MÜHLE_2 = 'de:08119:5154:0:4'  # Rtg Hohenacker Bittenf. Str. (Waiblingen Bittenfeld)
	BITTENFELD_RIENZH__MÜHLE = 'de:08119:5154'
	BITTENFELD_RIENZH__MÜHLE_1 = 'de:08119:5154:0:3'  # Rtg Am Zipfelbach (Waiblingen Bittenfeld)
	BITTENFELD_RIENZH__MÜHLE_2 = 'de:08119:5154:0:4'  # Rtg Hohenacker Bittenf. Str. (Waiblingen Bittenfeld)
	AM_ZIPFELBACH = 'de:08119:5155'
	AM_ZIPFELBACH_1 = 'de:08119:5155:1:3'  # Rtg Schillerschule/Altes Rathaus (Waiblingen Bittenfeld)
	AM_ZIPFELBACH_2 = 'de:08119:5155:1:4'  # Rtg Rienzhofer Mühle (Waiblingen Bittenfeld)
	BITTENFELD_AM_ZIPFELBACH = 'de:08119:5155'
	BITTENFELD_AM_ZIPFELBACH_1 = 'de:08119:5155:1:3'  # Rtg Schillerschule/Altes Rathaus (Waiblingen Bittenfeld)
	BITTENFELD_AM_ZIPFELBACH_2 = 'de:08119:5155:1:4'  # Rtg Rienzhofer Mühle (Waiblingen Bittenfeld)
	BITTENFELD_ALTES_RATHAUS = 'de:08119:5156'
	BITTENFELD_ALTES_RATHAUS_1 = 'de:08119:5156:0:3'  # Rtg Klingenstr. (Waiblingen Bittenfeld)
	BITTENFELD_ALTES_RATHAUS_2 = 'de:08119:5156:0:4'  # Rtg Am Zipfelbach (Waiblingen Bittenfeld)
	BITTENFELD_KLINGENSTRASSE = 'de:08119:5157:0:3'  # Endstation bzw. Start (Waiblingen Bittenfeld)
	WAIBLINGEN_STAUFERSTRASSE = 'de:08119:5158:0:3'  # eRtg Alfred-Diebold-Weg (Waiblingen Waiblingen)
	WAIBLINGEN_AM_KÄTZENBACH = 'de:08119:5160'
	WAIBLINGEN_AM_KÄTZENBACH_1 = 'de:08119:5160:0:3'  # Rtg Rundsporthalle (Waiblingen Waiblingen)
	WAIBLINGEN_AM_KÄTZENBACH_2 = 'de:08119:5160:0:4'  # Rtg Beinsteiner Str./Bürgerzentrum (Waiblingen Waiblingen)
	AM_KÄTZENBACH = 'de:08119:5160'
	AM_KÄTZENBACH_1 = 'de:08119:5160:0:3'  # Rtg Rundsporthalle (Waiblingen Waiblingen)
	AM_KÄTZENBACH_2 = 'de:08119:5160:0:4'  # Rtg Beinsteiner Str./Bürgerzentrum (Waiblingen Waiblingen)
	WAIBLINGEN_RUNDSPORTHALLE = 'de:08119:5161'
	WAIBLINGEN_RUNDSPORTHALLE_1 = 'de:08119:5161:0:3'  # Rtg Beinstein (Waiblingen Waiblingen)
	WAIBLINGEN_RUNDSPORTHALLE_2 = 'de:08119:5161:0:4'  # Rtg Am Kätzenbach (Waiblingen Waiblingen)
	BEINSTEIN_WAIBLINGER_STR = 'de:08119:5162'
	BEINSTEIN_WAIBLINGER_STR_1 = 'de:08119:5162:0:3'  # Rtg Beinstein (Waiblingen Beinstein)
	BEINSTEIN_WAIBLINGER_STR_2 = 'de:08119:5162:0:4'  # Rtg Rundsporthalle (Waiblingen Beinstein)
	WAIBLINGER_STRASSE = 'de:08119:5162'
	WAIBLINGER_STRASSE_1 = 'de:08119:5162:0:3'  # Rtg Beinstein (Waiblingen Beinstein)
	WAIBLINGER_STRASSE_2 = 'de:08119:5162:0:4'  # Rtg Rundsporthalle (Waiblingen Beinstein)
	GALGENBERGBRÜCKE = 'de:08119:5163:0:3'  # eRtg Stauferstraße (Waiblingen Waiblingen)
	WAIBLINGEN_GALGENBERGBRÜCKE = 'de:08119:5163:0:3'  # eRtg Stauferstraße (Waiblingen Waiblingen)
	BEINSTEIN_RATHAUS = 'de:08119:5164'
	BEINSTEIN_RATHAUS_1 = 'de:08119:5164:0:3'  # Rtg Mühlweg (Waiblingen Beinstein)
	BEINSTEIN_RATHAUS_2 = 'de:08119:5164:0:4'  # Rtg Endersbacher/Kleinhepp. Str. (Waiblingen Beinstein)
	BEINSTEIN_ENDERSBACHER_STR = 'de:08119:5165'
	BEINSTEIN_ENDERSBACHER_STR_1 = 'de:08119:5165:0:3'  # Rtg Rathaus (Waiblingen Beinstein)
	BEINSTEIN_ENDERSBACHER_STR_2 = 'de:08119:5165:0:4'  # Rtg Kalkofen (Waiblingen Beinstein)
	STETTENER_STRASSE = 'de:08119:5166'
	STETTENER_STRASSE_1 = 'de:08119:5166:1:3'  # Rtg Endersbach Ortsmitte (Weinstadt Endersbach)
	STETTENER_STRASSE_2 = 'de:08119:5166:1:4'  # Rtg Waiblingen Freibad (Weinstadt Endersbach)
	ENDERSBACH_STETTENER_STR = 'de:08119:5166'
	ENDERSBACH_STETTENER_STR_1 = 'de:08119:5166:1:3'  # Rtg Endersbach Ortsmitte (Weinstadt Endersbach)
	ENDERSBACH_STETTENER_STR_2 = 'de:08119:5166:1:4'  # Rtg Waiblingen Freibad (Weinstadt Endersbach)
	ENDERSBACH_VIADUKT = 'de:08119:5168:0:3'  # Rtg Jahnhalle/Bahnhofstr. (Weinstadt Endersbach)
	VIADUKT = 'de:08119:5168:0:3'  # Rtg Jahnhalle/Bahnhofstr. (Weinstadt Endersbach)
	PRINZ_EUGEN_PLATZ = 'de:08119:5169'
	PRINZ_EUGEN_PLATZ_1 = 'de:08119:5169:0:3'  # Rtg Beutelsbach/Stgt-/Weststr. (Weinstadt Großheppach)
	PRINZ_EUGEN_PLATZ_2 = 'de:08119:5169:0:4'  # Rtg Backhaus (Weinstadt Großheppach)
	GROSSH__PRINZ_EUGEN_PLATZ = 'de:08119:5169'
	GROSSH__PRINZ_EUGEN_PLATZ_1 = 'de:08119:5169:0:3'  # Rtg Beutelsbach/Stgt-/Weststr. (Weinstadt Großheppach)
	GROSSH__PRINZ_EUGEN_PLATZ_2 = 'de:08119:5169:0:4'  # Rtg Backhaus (Weinstadt Großheppach)
	GROSSHEPPACH_BACKHAUS = 'de:08119:5170'
	GROSSHEPPACH_BACKHAUS_1 = 'de:08119:5170:0:3'  # Rtg Prinz-Eugen-Platz (Weinstadt Großheppach)
	GROSSHEPPACH_BACKHAUS_2 = 'de:08119:5170:0:4'  # Rtg Kleinheppacher Str. (Weinstadt Großheppach)
	GROSSHEPPACH_KATH__KIRCHE = 'de:08119:5171:0:4'  # Rtg Heckbachstr./Großh. Kleinh.Str. (Weinstadt Großheppach)
	GROSSH__KLEINHEPPACHER_STR = 'de:08119:5172:0:3'  # Rtg Backhaus (Weinstadt Großheppach)
	KLEINHEPPACHER_STRASSE_1 = 'de:08119:5172:0:3'  # Rtg Backhaus (Weinstadt Großheppach)
	KLEINHEPPACHER_STRASSE_2 = 'de:08119:6636:0:3'  # Rtg Rossäcker (Waiblingen Beinstein)
	KLEINHEPPACHER_STRASSE_3 = 'de:08119:6636:0:4'  # Rtg Rathaus/Endersbacher Str. (Waiblingen Beinstein)
	OTTO_MÜHLSCHLEGEL_HAUS = 'de:08119:5174'
	OTTO_MÜHLSCHLEGEL_HAUS_1 = 'de:08119:5174:0:3'  # Rtg Strümpfelbach Seemühle (Weinstadt Endersbach)
	OTTO_MÜHLSCHLEGEL_HAUS_2 = 'de:08119:5174:0:4'  # Rtg Jahnhalle (Weinstadt Endersbach)
	ENDERSB__O__MÜHLSCHLEGEL_HAUS = 'de:08119:5174'
	ENDERSB__O__MÜHLSCHLEGEL_HAUS_1 = 'de:08119:5174:0:3'  # Rtg Strümpfelbach Seemühle (Weinstadt Endersbach)
	ENDERSB__O__MÜHLSCHLEGEL_HAUS_2 = 'de:08119:5174:0:4'  # Rtg Jahnhalle (Weinstadt Endersbach)
	STRÜMPFELBACH_SEEMÜHLE = 'de:08119:5175'
	STRÜMPFELBACH_SEEMÜHLE_1 = 'de:08119:5175:0:3'  # Rtg Hundsäcker (Weinstadt Strümpfelbach (Weinstadt))
	STRÜMPFELBACH_SEEMÜHLE_2 = 'de:08119:5175:0:4'  # Rtg Endersbach (Otto-Mühlschlegel-Haus) (Weinstadt Strümpfelbach (Weinstadt))
	SEEMÜHLE_1 = 'de:08119:5175:0:3'  # Rtg Hundsäcker (Weinstadt Strümpfelbach (Weinstadt))
	SEEMÜHLE_2 = 'de:08119:5175:0:4'  # Rtg Endersbach (Otto-Mühlschlegel-Haus) (Weinstadt Strümpfelbach (Weinstadt))
	SEEMÜHLE_3 = 'de:08118:6849:0:3'  # eRtg Roßwag Sporthalle (Vaihingen an der Enz Roßwag)
	SEEMÜHLE_4 = 'de:08118:6849:0:4'  # eRtg Vaihingen (E) (Vaihingen an der Enz Roßwag)
	STRÜMPFELBACH_TRAUBE = 'de:08119:5176'
	STRÜMPFELBACH_TRAUBE_1 = 'de:08119:5176:0:3'  # Rtg Kelter (Weinstadt Strümpfelbach (Weinstadt))
	STRÜMPFELBACH_TRAUBE_2 = 'de:08119:5176:0:4'  # Rtg Hundsäcker (Weinstadt Strümpfelbach (Weinstadt))
	STRÜMPFELBACH_KELTER = 'de:08119:5177'
	STRÜMPFELBACH_KELTER_1 = 'de:08119:5177:0:3'  # Rtg Rathaus (Weinstadt Strümpfelbach (Weinstadt))
	STRÜMPFELBACH_KELTER_2 = 'de:08119:5177:0:4'  # Rtg Traube (Weinstadt Strümpfelbach (Weinstadt))
	STRÜMPFELBACH_RATHAUS = 'de:08119:5178:0:3'  # End- und Startpunkt (Weinstadt Strümpfelbach (Weinstadt))
	BEUTELSBACH_RATHAUS = 'de:08119:5179'
	BEUTELSBACH_RATHAUS_1 = 'de:08119:5179:0:3'  # eRtg Ulrichstraße (Weinstadt Beutelsbach)
	BEUTELSBACH_RATHAUS_2 = 'de:08119:5179:0:4'  # Rtg Benzach/Beutelsbach Bf (Weinstadt Beutelsbach)
	HUNDSÄCKER = 'de:08119:5180'
	HUNDSÄCKER_1 = 'de:08119:5180:0:3'  # Rtg Traube (Weinstadt Strümpfelbach (Weinstadt))
	HUNDSÄCKER_2 = 'de:08119:5180:0:4'  # Rtg Seemühle (Weinstadt Strümpfelbach (Weinstadt))
	STRÜMPFELBACH_HUNDSÄCKER = 'de:08119:5180'
	STRÜMPFELBACH_HUNDSÄCKER_1 = 'de:08119:5180:0:3'  # Rtg Traube (Weinstadt Strümpfelbach (Weinstadt))
	STRÜMPFELBACH_HUNDSÄCKER_2 = 'de:08119:5180:0:4'  # Rtg Seemühle (Weinstadt Strümpfelbach (Weinstadt))
	GRUNDSCHULE_1 = 'de:08119:5183:0:3'  # alle Richtungen (Remshalden Grunbach)
	GRUNDSCHULE_2 = 'de:08118:5653:1:3'  # eRtg Vaihingen (E) Bahnhof (Vaihingen an der Enz Kleinglattbach)
	GRUNDSCHULE_3 = 'de:08118:5653:1:4'  # eRtg Osterwiesenweg (Vaihingen an der Enz Kleinglattbach)
	GRUNDSCHULE_4 = 'de:08117:10121:0:3'  # eRtg alle Richtungen (Uhingen Sparwiesen)
	GRUNDSCHULE_5 = 'de:08117:3233:0:H1'  # Rtg Brückenweg (Gruibingen Gruibingen)
	GRUNDSCHULE_6 = 'de:08117:3317:0:H1'  # Rtg Heiningen (Eschenbach Eschenbach)
	GRUNDSCHULE_7 = 'de:08117:3505:0:H1'  # Rtg Gosbach Auendorf (Bad Ditzenbach Bad Ditzenbach)
	GRUNDSCHULE_8 = 'de:08117:3505:0:R1'  # Rtg Kurklinik (Bad Ditzenbach Bad Ditzenbach)
	GRUNDSCHULE_9 = 'de:08117:4018:0:H1'  # Rtg Feuerwehr (Rechberghausen Rechberghausen)
	GRUNDSCHULE_10 = 'de:08117:4018:0:R1'  # Rtg Hauffstraße (Rechberghausen Rechberghausen)
	GRUNBACH_GRUNDSCHULE = 'de:08119:5183:0:3'  # alle Richtungen (Remshalden Grunbach)
	BUOCH_GEMEINDEHAUS = 'de:08119:5186:0:3'  # Rtg alle Richtungen (Remshalden Buoch)
	BUOCH_FREIZEITZENTRUM = 'de:08119:5187'
	BUOCH_FREIZEITZENTRUM_1 = 'de:08119:5187:0:3'  # eRtg Gemeindehaus (Remshalden Buoch)
	BUOCH_FREIZEITZENTRUM_2 = 'de:08119:5187:0:4'  # eRtg Buocher Straße (Remshalden Buoch)
	FREIZEITZENTRUM = 'de:08119:5187'
	FREIZEITZENTRUM_1 = 'de:08119:5187:0:3'  # eRtg Gemeindehaus (Remshalden Buoch)
	FREIZEITZENTRUM_2 = 'de:08119:5187:0:4'  # eRtg Buocher Straße (Remshalden Buoch)
	JAHNHALLE = 'de:08119:5188'
	JAHNHALLE_1 = 'de:08119:5188:0:3'  # Rtg Otto-Mühlschlegel-Haus (Weinstadt Endersbach)
	JAHNHALLE_2 = 'de:08119:5188:0:4'  # Rtg Ortsmitte (Weinstadt Endersbach)
	ENDERSBACH_JAHNHALLE = 'de:08119:5188'
	ENDERSBACH_JAHNHALLE_1 = 'de:08119:5188:0:3'  # Rtg Otto-Mühlschlegel-Haus (Weinstadt Endersbach)
	ENDERSBACH_JAHNHALLE_2 = 'de:08119:5188:0:4'  # Rtg Ortsmitte (Weinstadt Endersbach)
	RISSLERINSTRASSE = 'de:08119:5190:0:3'  # Rtg Schillerplatz (Schorndorf Schorndorf)
	SCHORNDORF_RISSLERINSTR = 'de:08119:5190:0:3'  # Rtg Schillerplatz (Schorndorf Schorndorf)
	SCHORNDORF_JOHANNESSTR = 'de:08119:5191'
	SCHORNDORF_JOHANNESSTR_1 = 'de:08119:5191:0:3'  # eRtg Rathaus/Seebad (Schorndorf Schorndorf)
	SCHORNDORF_JOHANNESSTR_2 = 'de:08119:5191:0:4'  # eRtg Künkelinhalle/Burggymnasium (Schorndorf Schorndorf)
	WEILER_RATHAUS = 'de:08119:5193'
	WEILER_RATHAUS_1 = 'de:08119:5193:1:3'  # eRtg Kirche/Pfarrstraße/Pflegeheim (Schorndorf Weiler (Schorndorf))
	WEILER_RATHAUS_2 = 'de:08119:5193:1:4'  # eRtg Dürrbach / Rehfeld (Schorndorf Weiler (Schorndorf))
	HEBSACK_ABZW = 'de:08119:5197'
	HEBSACK_ABZW_1 = 'de:08119:5197:0:3'  # eRtg Rohrbronn (Remshalden Hebsack)
	HEBSACK_ABZW_2 = 'de:08119:5197:0:4'  # eRtg Remsbrücke (Remshalden Hebsack)
	ROHRBRONN = 'de:08119:5198'
	ROHRBRONN_1 = 'de:08119:5198:0:3'  # Rtg Hößlinswart (Remshalden Rohrbronn)
	ROHRBRONN_2 = 'de:08119:5198:0:4'  # Rtg Hebsack Abzweig/Winterb. Remsbrücke (Remshalden Rohrbronn)
	HÖSSLINSWART = 'de:08119:5199'
	HÖSSLINSWART_1 = 'de:08119:5199:1:3'  # eRtg Steinach (Berglen Hößlinswart)
	HÖSSLINSWART_2 = 'de:08119:5199:1:4'  # eRtg Winterbach (Berglen Hößlinswart)
	ERLENHOF = 'de:08119:5201'
	ERLENHOF_1 = 'de:08119:5201:1:1'  # Rtg Steinach Buchenstr./Bretz. Nb-schule (Berglen Ödernhardt)
	ERLENHOF_2 = 'de:08119:5201:1:2'  # Rtg Lehnenberg Krzg. (Berglen Ödernhardt)
	ERLENHOF_3 = 'de:08119:5201:1:A'  # Ankunftssteig (Berglen Ödernhardt)
	ÖDERNHARDT_ERLENHOF = 'de:08119:5201'
	ÖDERNHARDT_ERLENHOF_1 = 'de:08119:5201:1:1'  # Rtg Steinach Buchenstr./Bretz. Nb-schule (Berglen Ödernhardt)
	ÖDERNHARDT_ERLENHOF_2 = 'de:08119:5201:1:2'  # Rtg Lehnenberg Krzg. (Berglen Ödernhardt)
	ÖDERNHARDT_ERLENHOF_3 = 'de:08119:5201:1:A'  # Ankunftssteig (Berglen Ödernhardt)
	LEHNENBERG_KREUZUNG = 'de:08119:5202'
	LEHNENBERG_KREUZUNG_1 = 'de:08119:5202:0:3'  # eRtg Erlenhof (Berglen Reichenbach (Berglen))
	LEHNENBERG_KREUZUNG_2 = 'de:08119:5202:0:4'  # eRtg Industriegebiet (Berglen Reichenbach (Berglen))
	REICHENB__LEHNENB__KREUZUNG = 'de:08119:5202'
	REICHENB__LEHNENB__KREUZUNG_1 = 'de:08119:5202:0:3'  # eRtg Erlenhof (Berglen Reichenbach (Berglen))
	REICHENB__LEHNENB__KREUZUNG_2 = 'de:08119:5202:0:4'  # eRtg Industriegebiet (Berglen Reichenbach (Berglen))
	BIRKMANNSW__TALAUE = 'de:08119:5203'
	BIRKMANNSW__TALAUE_1 = 'de:08119:5203:0:3'  # eRtg Ortsmitte (Winnenden Birkmannsweiler)
	BIRKMANNSW__TALAUE_2 = 'de:08119:5203:0:4'  # eRtg Winnenden/Höfen (Winnenden Birkmannsweiler)
	TALAUE = 'de:08119:5203'
	TALAUE_1 = 'de:08119:5203:0:3'  # eRtg Ortsmitte (Winnenden Birkmannsweiler)
	TALAUE_2 = 'de:08119:5203:0:4'  # eRtg Winnenden/Höfen (Winnenden Birkmannsweiler)
	BIRKMANNSW__ORTSMITTE = 'de:08119:5204'
	BIRKMANNSW__ORTSMITTE_1 = 'de:08119:5204:0:3'  # eRtg Industriegebiet (Winnenden Birkmannsweiler)
	BIRKMANNSW__ORTSMITTE_2 = 'de:08119:5204:0:4'  # eRtg Talaue (Winnenden Birkmannsweiler)
	HÖFEN_SEEHALDENWEG = 'de:08119:5205'
	HÖFEN_SEEHALDENWEG_1 = 'de:08119:5205:0:3'  # eRtg Ortsmitte (Winnenden Höfen)
	HÖFEN_SEEHALDENWEG_2 = 'de:08119:5205:0:4'  # eRtg Wolfsklingenweg/Burkhardtshof (Winnenden Höfen)
	SEEHALDENWEG = 'de:08119:5205'
	SEEHALDENWEG_1 = 'de:08119:5205:0:3'  # eRtg Ortsmitte (Winnenden Höfen)
	SEEHALDENWEG_2 = 'de:08119:5205:0:4'  # eRtg Wolfsklingenweg/Burkhardtshof (Winnenden Höfen)
	WINNENDEN_FRIEDHOF = 'de:08119:5207:0:3'  # eRtg Wallstraße/BZII (Winnenden Winnenden)
	WALLSTRASSE = 'de:08119:5209:0:3'  # eRtg Winnenden (Winnenden Winnenden)
	WINNENDEN_WALLSTRASSE = 'de:08119:5209:0:3'  # eRtg Winnenden (Winnenden Winnenden)
	WINNENDEN_RINGSTRASSE = 'de:08119:5210:0:3'  # eRtg Kelter/Stöckachschule (Winnenden Winnenden)
	SCHORNDORF_HAYDNSTR__BÄDER = 'de:08119:5211'
	SCHORNDORF_HAYDNSTR__BÄDER_1 = 'de:08119:5211:0:3'  # eRtg Gottlob-Kamm-Platz (Schorndorf Schorndorf)
	SCHORNDORF_HAYDNSTR__BÄDER_2 = 'de:08119:5211:0:4'  # eRtg Schlossstraße (Schorndorf Schorndorf)
	HAYDNSTRASSE_BÄDER = 'de:08119:5211'
	HAYDNSTRASSE_BÄDER_1 = 'de:08119:5211:0:3'  # eRtg Gottlob-Kamm-Platz (Schorndorf Schorndorf)
	HAYDNSTRASSE_BÄDER_2 = 'de:08119:5211:0:4'  # eRtg Schlossstraße (Schorndorf Schorndorf)
	OSKAR_FRECH_SEEBAD = 'de:08119:5212'
	OSKAR_FRECH_SEEBAD_1 = 'de:08119:5212:0:3'  # eRtg Reithalle (Schorndorf Schorndorf)
	OSKAR_FRECH_SEEBAD_2 = 'de:08119:5212:0:4'  # eRtg Johannesstraße (Schorndorf Schorndorf)
	SCHORNDORF_OSKAR_FRECH_SEEBAD = 'de:08119:5212'
	SCHORNDORF_OSKAR_FRECH_SEEBAD_1 = 'de:08119:5212:0:3'  # eRtg Reithalle (Schorndorf Schorndorf)
	SCHORNDORF_OSKAR_FRECH_SEEBAD_2 = 'de:08119:5212:0:4'  # eRtg Johannesstraße (Schorndorf Schorndorf)
	SPITTLER_STIFT = 'de:08119:5214:0:3'  # eRtg Röhrachweg (Schorndorf Schorndorf)
	SCHORNDORF_SPITTLER_STIFT = 'de:08119:5214:0:3'  # eRtg Röhrachweg (Schorndorf Schorndorf)
	SCHORNDORF_WIESLAUFWEG = 'de:08119:5215:0:3'  # eRtg Schorndorf/Kondrad-Hausmann-Weg (Schorndorf Schorndorf)
	WIESLAUFWEG = 'de:08119:5215:0:3'  # eRtg Schorndorf/Kondrad-Hausmann-Weg (Schorndorf Schorndorf)
	SCHORNDORF_SCHILLERPLATZ = 'de:08119:5216'
	SCHORNDORF_SCHILLERPLATZ_1 = 'de:08119:5216:0:3'  # eRtg Burggymnasium (Schorndorf Schorndorf)
	SCHORNDORF_SCHILLERPLATZ_2 = 'de:08119:5216:0:4'  # eRtg Jakob-Raible-Anlage (Schorndorf Schorndorf)
	HÖFEN_BAACHER_STRASSE = 'de:08119:5217'
	HÖFEN_BAACHER_STRASSE_1 = 'de:08119:5217:0:3'  # eRtg Baach Ortsmitte (Winnenden Höfen)
	HÖFEN_BAACHER_STRASSE_2 = 'de:08119:5217:0:4'  # eRtg Höfen Ortsmitte (Winnenden Höfen)
	BAACHER_STRASSE = 'de:08119:5217'
	BAACHER_STRASSE_1 = 'de:08119:5217:0:3'  # eRtg Baach Ortsmitte (Winnenden Höfen)
	BAACHER_STRASSE_2 = 'de:08119:5217:0:4'  # eRtg Höfen Ortsmitte (Winnenden Höfen)
	WINN__HERMANN_LÖNS_WEG = 'de:08119:5218'
	WINN__HERMANN_LÖNS_WEG_1 = 'de:08119:5218:0:3'  # eRtg Haselsteinstraße (Winnenden Winnenden)
	WINN__HERMANN_LÖNS_WEG_2 = 'de:08119:5218:0:4'  # eRtg Kelter (Winnenden Winnenden)
	HERMANN_LÖNS_WEG = 'de:08119:5218'
	HERMANN_LÖNS_WEG_1 = 'de:08119:5218:0:3'  # eRtg Haselsteinstraße (Winnenden Winnenden)
	HERMANN_LÖNS_WEG_2 = 'de:08119:5218:0:4'  # eRtg Kelter (Winnenden Winnenden)
	JAKOB_RAIBLE_ANLAGE = 'de:08119:5219:0:4'  # eRtg Mönchsbrücke (Schorndorf Schorndorf)
	SCHORNDORF_JAKOB_RAIBLE_ANLAGE = 'de:08119:5219:0:4'  # eRtg Mönchsbrücke (Schorndorf Schorndorf)
	MÖNCHSBRÜCKE = 'de:08119:5220'
	MÖNCHSBRÜCKE_1 = 'de:08119:5220:0:3'  # eRtg Jakob-Raible-Anlage (Schorndorf Schorndorf)
	MÖNCHSBRÜCKE_2 = 'de:08119:5220:0:4'  # eRtg Grauhalde (Schorndorf Schorndorf)
	SCHORNDORF_MÖNCHSBRÜCKE = 'de:08119:5220'
	SCHORNDORF_MÖNCHSBRÜCKE_1 = 'de:08119:5220:0:3'  # eRtg Jakob-Raible-Anlage (Schorndorf Schorndorf)
	SCHORNDORF_MÖNCHSBRÜCKE_2 = 'de:08119:5220:0:4'  # eRtg Grauhalde (Schorndorf Schorndorf)
	SCHORNDORF_BUCHENWEG = 'de:08119:5222:0:4'  # Rtg Lindenweg (Schorndorf Schorndorf)
	HUNGERBÜHLSTRASSE = 'de:08119:5223:0:3'  # eRtg Neuer Friedhof (Schorndorf Schorndorf)
	SCHORNDORF_HUNGERBÜHLSTRASSE = 'de:08119:5223:0:3'  # eRtg Neuer Friedhof (Schorndorf Schorndorf)
	SCHORNDORF_GRABENSTR___BF = 'de:08119:5224'
	SCHORNDORF_GRABENSTR___BF_1 = 'de:08119:5224:1:3'  # Ein-/Ausstieg/Am Mühlbach (neu) (Schorndorf Schorndorf)
	SCHORNDORF_GRABENSTR___BF_2 = 'de:08119:5224:1:4'  # eRtg ZOB (Schorndorf Schorndorf)
	GRABENSTRASSE__BAHNHOF = 'de:08119:5224'
	GRABENSTRASSE__BAHNHOF_1 = 'de:08119:5224:1:3'  # Ein-/Ausstieg/Am Mühlbach (neu) (Schorndorf Schorndorf)
	GRABENSTRASSE__BAHNHOF_2 = 'de:08119:5224:1:4'  # eRtg ZOB (Schorndorf Schorndorf)
	SCHORNDORF_KLEINGÄRTEN = 'de:08119:5225'
	SCHORNDORF_KLEINGÄRTEN_1 = 'de:08119:5225:0:3'  # Rtg R.-Bosch-Str. (Schorndorf Schorndorf)
	SCHORNDORF_KLEINGÄRTEN_2 = 'de:08119:5225:0:4'  # eRtg Hohenstaufenstraße (Schorndorf Schorndorf)
	KLEINGÄRTEN = 'de:08119:5225'
	KLEINGÄRTEN_1 = 'de:08119:5225:0:3'  # Rtg R.-Bosch-Str. (Schorndorf Schorndorf)
	KLEINGÄRTEN_2 = 'de:08119:5225:0:4'  # eRtg Hohenstaufenstraße (Schorndorf Schorndorf)
	URBACH_WASENSTRASSE = 'de:08119:5226'
	URBACH_WASENSTRASSE_1 = 'de:08119:5226:0:3'  # West (Urbach Urbach)
	URBACH_WASENSTRASSE_2 = 'de:08119:5226:0:4'  # Ost (Urbach Urbach)
	URBACH_WITTUMHALLE = 'de:08119:5227'
	URBACH_WITTUMHALLE_1 = 'de:08119:5227:0:3'  # eRtg Schloss (Urbach Urbach)
	URBACH_WITTUMHALLE_2 = 'de:08119:5227:0:4'  # eRtg Wasenstraße (Urbach Urbach)
	WITTUMHALLE = 'de:08119:5227'
	WITTUMHALLE_1 = 'de:08119:5227:0:3'  # eRtg Schloss (Urbach Urbach)
	WITTUMHALLE_2 = 'de:08119:5227:0:4'  # eRtg Wasenstraße (Urbach Urbach)
	PLÜDERH__AHORNSTRASSE = 'de:08119:5234'
	PLÜDERH__AHORNSTRASSE_1 = 'de:08119:5234:1:3'  # Rtg Aichenbachhof/Gänswasen (Plüderhausen Plüderhausen)
	PLÜDERH__AHORNSTRASSE_2 = 'de:08119:5234:1:4'  # Rtg Birkenallee (Plüderhausen Plüderhausen)
	AHORNSTRASSE = 'de:08119:5234'
	AHORNSTRASSE_1 = 'de:08119:5234:1:3'  # Rtg Aichenbachhof/Gänswasen (Plüderhausen Plüderhausen)
	AHORNSTRASSE_2 = 'de:08119:5234:1:4'  # Rtg Birkenallee (Plüderhausen Plüderhausen)
	PLÜDERH__GÄNSWASEN = 'de:08119:5235'
	PLÜDERH__GÄNSWASEN_1 = 'de:08119:5235:0:3'  # Rtg Schwalbenweg (Plüderhausen Plüderhausen)
	PLÜDERH__GÄNSWASEN_2 = 'de:08119:5235:0:4'  # Rtg Ahornstraße (Plüderhausen Plüderhausen)
	GÄNSWASEN = 'de:08119:5235'
	GÄNSWASEN_1 = 'de:08119:5235:0:3'  # Rtg Schwalbenweg (Plüderhausen Plüderhausen)
	GÄNSWASEN_2 = 'de:08119:5235:0:4'  # Rtg Ahornstraße (Plüderhausen Plüderhausen)
	PLÜDERHAUSEN_HEUSEE = 'de:08119:5236:0:3'  # eRtg Ahornstraße (Plüderhausen Plüderhausen)
	HEUSEE = 'de:08119:5236:0:3'  # eRtg Ahornstraße (Plüderhausen Plüderhausen)
	PLÜDERH__SCHWALBENWEG = 'de:08119:5237'
	PLÜDERH__SCHWALBENWEG_1 = 'de:08119:5237:0:3'  # Rtg Wasserbergweg (Plüderhausen Plüderhausen)
	PLÜDERH__SCHWALBENWEG_2 = 'de:08119:5237:0:4'  # Rtg Gänswasen (Plüderhausen Plüderhausen)
	WASSERBERGWEG = 'de:08119:5239'
	WASSERBERGWEG_1 = 'de:08119:5239:0:3'  # Rtg Margaretenstr. (Plüderhausen Plüderhausen)
	WASSERBERGWEG_2 = 'de:08119:5239:0:4'  # Rtg Schwalbenweg (Plüderhausen Plüderhausen)
	PLÜDERH__WASSERBERGWEG = 'de:08119:5239'
	PLÜDERH__WASSERBERGWEG_1 = 'de:08119:5239:0:3'  # Rtg Margaretenstr. (Plüderhausen Plüderhausen)
	PLÜDERH__WASSERBERGWEG_2 = 'de:08119:5239:0:4'  # Rtg Schwalbenweg (Plüderhausen Plüderhausen)
	PLÜDERHAUSEN_MARGARETENSTR = 'de:08119:5240'
	PLÜDERHAUSEN_MARGARETENSTR_1 = 'de:08119:5240:0:3'  # Rtg Adlerbrücke (Plüderhausen Plüderhausen)
	PLÜDERHAUSEN_MARGARETENSTR_2 = 'de:08119:5240:0:4'  # Rtg Wasserbergweg (Plüderhausen Plüderhausen)
	MARGARETENSTRASSE = 'de:08119:5240'
	MARGARETENSTRASSE_1 = 'de:08119:5240:0:3'  # Rtg Adlerbrücke (Plüderhausen Plüderhausen)
	MARGARETENSTRASSE_2 = 'de:08119:5240:0:4'  # Rtg Wasserbergweg (Plüderhausen Plüderhausen)
	PLÜDERH__AICHENBACHHOF = 'de:08119:5241:0:3'  # eRtg Gänswasen (Plüderhausen Plüderhausen)
	AICHENBACHHOF = 'de:08119:5241:0:3'  # eRtg Gänswasen (Plüderhausen Plüderhausen)
	ADLERBRÜCKE = 'de:08119:5242'
	ADLERBRÜCKE_1 = 'de:08119:5242:0:3'  # Rtg Hohbergschule (Plüderhausen Plüderhausen)
	ADLERBRÜCKE_2 = 'de:08119:5242:0:4'  # Rtg Margaretenstr. (Plüderhausen Plüderhausen)
	PLÜDERH__ADLERBRÜCKE = 'de:08119:5242'
	PLÜDERH__ADLERBRÜCKE_1 = 'de:08119:5242:0:3'  # Rtg Hohbergschule (Plüderhausen Plüderhausen)
	PLÜDERH__ADLERBRÜCKE_2 = 'de:08119:5242:0:4'  # Rtg Margaretenstr. (Plüderhausen Plüderhausen)
	HOHBERGSCHULE = 'de:08119:5243:0:3'  # Rtg Königsbau (Plüderhausen Plüderhausen)
	PLÜDERH__HOHBERGSCHULE = 'de:08119:5243:0:3'  # Rtg Königsbau (Plüderhausen Plüderhausen)
	PLÜDERH__BAHNHOFSTRASSE = 'de:08119:5245:1:3'  # eRtg Birkenallee (Bahnübergang) (Plüderhausen Plüderhausen)
	SCHWANFELD = 'de:08119:5247'
	SCHWANFELD_1 = 'de:08119:5247:0:3'  # eRtg Reesenbänkle (Plüderhausen Plüderhausen)
	SCHWANFELD_2 = 'de:08119:5247:0:4'  # eRtg Königsbau (Plüderhausen Plüderhausen)
	PLÜDERH__SCHWANFELD = 'de:08119:5247'
	PLÜDERH__SCHWANFELD_1 = 'de:08119:5247:0:3'  # eRtg Reesenbänkle (Plüderhausen Plüderhausen)
	PLÜDERH__SCHWANFELD_2 = 'de:08119:5247:0:4'  # eRtg Königsbau (Plüderhausen Plüderhausen)
	PLÜDERH__REESENBÄNKLE = 'de:08119:5248'
	PLÜDERH__REESENBÄNKLE_1 = 'de:08119:5248:1:3'  # eRtg Münzenhalde (Plüderhausen Plüderhausen)
	PLÜDERH__REESENBÄNKLE_2 = 'de:08119:5248:1:4'  # eRtg Schwanfeld (Plüderhausen Plüderhausen)
	REESENBÄNKLE = 'de:08119:5248'
	REESENBÄNKLE_1 = 'de:08119:5248:1:3'  # eRtg Münzenhalde (Plüderhausen Plüderhausen)
	REESENBÄNKLE_2 = 'de:08119:5248:1:4'  # eRtg Schwanfeld (Plüderhausen Plüderhausen)
	PLÜDERHAUSEN_MÜNZENHALDE = 'de:08119:5249'
	PLÜDERHAUSEN_MÜNZENHALDE_1 = 'de:08119:5249:0:3'  # eRtg Hohbaum (Plüderhausen Plüderhausen)
	PLÜDERHAUSEN_MÜNZENHALDE_2 = 'de:08119:5249:0:4'  # Rtg Reesenbänkle (Plüderhausen Plüderhausen)
	MÜNZENHALDE = 'de:08119:5249'
	MÜNZENHALDE_1 = 'de:08119:5249:0:3'  # eRtg Hohbaum (Plüderhausen Plüderhausen)
	MÜNZENHALDE_2 = 'de:08119:5249:0:4'  # Rtg Reesenbänkle (Plüderhausen Plüderhausen)
	PLÜDERH__HOHBAUM = 'de:08119:5250'
	PLÜDERH__HOHBAUM_1 = 'de:08119:5250:0:3'  # Rtg Birkenallee (Plüderhausen Plüderhausen)
	PLÜDERH__HOHBAUM_2 = 'de:08119:5250:0:4'  # Rtg Münzenhalde (Plüderhausen Plüderhausen)
	HOHBAUM = 'de:08119:5250'
	HOHBAUM_1 = 'de:08119:5250:0:3'  # Rtg Birkenallee (Plüderhausen Plüderhausen)
	HOHBAUM_2 = 'de:08119:5250:0:4'  # Rtg Münzenhalde (Plüderhausen Plüderhausen)
	KÖNIGSBAU = 'de:08119:5251'
	KÖNIGSBAU_1 = 'de:08119:5251:0:3'  # eRtg Hohbergschule (Plüderhausen Plüderhausen)
	KÖNIGSBAU_2 = 'de:08119:5251:0:4'  # Rtg Schwanfeld (Plüderhausen Plüderhausen)
	PLÜDERHAUSEN_KÖNIGSBAU = 'de:08119:5251'
	PLÜDERHAUSEN_KÖNIGSBAU_1 = 'de:08119:5251:0:3'  # eRtg Hohbergschule (Plüderhausen Plüderhausen)
	PLÜDERHAUSEN_KÖNIGSBAU_2 = 'de:08119:5251:0:4'  # Rtg Schwanfeld (Plüderhausen Plüderhausen)
	POSTWEG = 'de:08119:5252:1:3'  # eRtg Schwanfeld/Staufenapotheke (Plüderhausen Plüderhausen)
	PLÜDERHAUSEN_POSTWEG = 'de:08119:5252:1:3'  # eRtg Schwanfeld/Staufenapotheke (Plüderhausen Plüderhausen)
	SCHORNDORF_MITTLERE_UFERSTR = 'de:08119:5254:0:4'  # eRtg Stuttgarter Straße (Schorndorf Schorndorf)
	MITTLERE_UFERSTRASSE = 'de:08119:5254:0:4'  # eRtg Stuttgarter Straße (Schorndorf Schorndorf)
	SCHORNDORF_SCHORNBACHER_WEG = 'de:08119:5255'
	SCHORNDORF_SCHORNBACHER_WEG_1 = 'de:08119:5255:0:3'  # eRtg Holzbergweg (Schorndorf Schorndorf)
	SCHORNDORF_SCHORNBACHER_WEG_2 = 'de:08119:5255:0:4'  # eRtg Mittlere Brücke (Schorndorf Schorndorf)
	SCHORNBACHER_WEG = 'de:08119:5255'
	SCHORNBACHER_WEG_1 = 'de:08119:5255:0:3'  # eRtg Holzbergweg (Schorndorf Schorndorf)
	SCHORNBACHER_WEG_2 = 'de:08119:5255:0:4'  # eRtg Mittlere Brücke (Schorndorf Schorndorf)
	SCHORNDORF_HOLZBERGWEG = 'de:08119:5256'
	SCHORNDORF_HOLZBERGWEG_1 = 'de:08119:5256:0:3'  # eRtg Rohr (Schorndorf Schorndorf)
	SCHORNDORF_HOLZBERGWEG_2 = 'de:08119:5256:0:4'  # eRtg Schornbacher Weg (Schorndorf Schorndorf)
	HOLZBERGWEG = 'de:08119:5256'
	HOLZBERGWEG_1 = 'de:08119:5256:0:3'  # eRtg Rohr (Schorndorf Schorndorf)
	HOLZBERGWEG_2 = 'de:08119:5256:0:4'  # eRtg Schornbacher Weg (Schorndorf Schorndorf)
	SCHLIPFWEG = 'de:08119:5259'
	SCHLIPFWEG_1 = 'de:08119:5259:0:3'  # eRtg Rohr (Schorndorf Schorndorf)
	SCHLIPFWEG_2 = 'de:08119:5259:0:4'  # eRtg Welzheimer Straße (Schorndorf Schorndorf)
	SCHORNDORF_SCHLIPFWEG = 'de:08119:5259'
	SCHORNDORF_SCHLIPFWEG_1 = 'de:08119:5259:0:3'  # eRtg Rohr (Schorndorf Schorndorf)
	SCHORNDORF_SCHLIPFWEG_2 = 'de:08119:5259:0:4'  # eRtg Welzheimer Straße (Schorndorf Schorndorf)
	SCHORNBACH_POST = 'de:08119:5260'
	SCHORNBACH_POST_1 = 'de:08119:5260:0:3'  # Rtg Buhlbronn (Schorndorf Schornbach)
	SCHORNBACH_POST_2 = 'de:08119:5260:0:4'  # Rtg Schorndorf (Schorndorf Schornbach)
	ROHR_1 = 'de:08119:5261:0:3'  # eRtg Post (Schorndorf Schornbach)
	ROHR_2 = 'de:08119:5261:0:4'  # eRtg Holzbergweg (Schorndorf Schornbach)
	ROHR_3 = 'de:08111:6001:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	ROHR_4 = 'de:08111:6001:1:3'  # Gleis 3 (Stuttgart Stuttgart)
	ROHR_5 = 'de:08111:6001:2:3'  # Rtg Osterbronnstr. (Stuttgart Stuttgart)
	ROHR_6 = 'de:08111:6001:2:4'  # Rtg Rohr Mitte (Stuttgart Stuttgart)
	SCHORNBACH_ROHR = 'de:08119:5261'
	SCHORNBACH_ROHR_1 = 'de:08119:5261:0:3'  # eRtg Post (Schorndorf Schornbach)
	SCHORNBACH_ROHR_2 = 'de:08119:5261:0:4'  # eRtg Holzbergweg (Schorndorf Schornbach)
	BUHLBRONN = 'de:08119:5262'
	BUHLBRONN_1 = 'de:08119:5262:0:3'  # eRtg Streich (Schorndorf Buhlbronn)
	BUHLBRONN_2 = 'de:08119:5262:0:4'  # eRtg Post (Schorndorf Buhlbronn)
	STREICH = 'de:08119:5263'
	STREICH_1 = 'de:08119:5263:0:3'  # eRtg Birkenweißbuch (Berglen Vorderweißbuch)
	STREICH_2 = 'de:08119:5263:0:4'  # eRtg Buhlbronn (Berglen Vorderweißbuch)
	VORDERW__STREICH = 'de:08119:5263'
	VORDERW__STREICH_1 = 'de:08119:5263:0:3'  # eRtg Birkenweißbuch (Berglen Vorderweißbuch)
	VORDERW__STREICH_2 = 'de:08119:5263:0:4'  # eRtg Buhlbronn (Berglen Vorderweißbuch)
	VORDERWEISSBUCH = 'de:08119:5264'
	VORDERWEISSBUCH_1 = 'de:08119:5264:0:3'  # eRtg Necklinsberg Kreuzung (Berglen Vorderweißbuch)
	VORDERWEISSBUCH_2 = 'de:08119:5264:0:4'  # eRtg Birkenweißbuch (Berglen Vorderweißbuch)
	VORDERW__BIRKENWEISSBUCH = 'de:08119:5265'
	VORDERW__BIRKENWEISSBUCH_1 = 'de:08119:5265:0:3'  # eRtg Vorderweißbuch (Berglen Vorderweißbuch)
	VORDERW__BIRKENWEISSBUCH_2 = 'de:08119:5265:0:4'  # eRtg Streich (Berglen Vorderweißbuch)
	BIRKENWEISSBUCH = 'de:08119:5265'
	BIRKENWEISSBUCH_1 = 'de:08119:5265:0:3'  # eRtg Vorderweißbuch (Berglen Vorderweißbuch)
	BIRKENWEISSBUCH_2 = 'de:08119:5265:0:4'  # eRtg Streich (Berglen Vorderweißbuch)
	NECKLINSBERG_KREUZUNG = 'de:08119:5266'
	NECKLINSBERG_KREUZUNG_1 = 'de:08119:5266:0:3'  # eRtg Necklinsberg (Rudersberg Asperglen)
	NECKLINSBERG_KREUZUNG_2 = 'de:08119:5266:0:4'  # eRtg Vorderweißbuch (Rudersberg Asperglen)
	ASPERGLEN_NECKLINSB__KREUZUNG = 'de:08119:5266'
	ASPERGLEN_NECKLINSB__KREUZUNG_1 = 'de:08119:5266:0:3'  # eRtg Necklinsberg (Rudersberg Asperglen)
	ASPERGLEN_NECKLINSB__KREUZUNG_2 = 'de:08119:5266:0:4'  # eRtg Vorderweißbuch (Rudersberg Asperglen)
	NECKLINSBERG = 'de:08119:5267:0:3'  # Rtg alle Richtungen (Rudersberg Asperglen)
	ASPERGL__NECKLINSBERG = 'de:08119:5267:0:3'  # Rtg alle Richtungen (Rudersberg Asperglen)
	OPPELSBOHM_ORTSMITTE = 'de:08119:5268'
	OPPELSBOHM_ORTSMITTE_1 = 'de:08119:5268:1:3'  # eRtg Alle Richtungen (Berglen Oppelsbohm)
	OPPELSBOHM_ORTSMITTE_2 = 'de:08119:5268:1:4'  # Rtg. Kaisersbach (Berglen Oppelsbohm)
	RETTERSBURG = 'de:08119:5269'
	RETTERSBURG_1 = 'de:08119:5269:0:3'  # eRtg Ortsmitte (Berglen Rettersburg)
	RETTERSBURG_2 = 'de:08119:5269:0:4'  # eRtg Öschelbronn (Berglen Rettersburg)
	ÖSCHELBRONN = 'de:08119:5270'
	ÖSCHELBRONN_1 = 'de:08119:5270:0:3'  # eRtg Rettersburg (Berglen Öschelbronn (Berglen))
	ÖSCHELBRONN_2 = 'de:08119:5270:0:4'  # eRtg Stöckenhof (Berglen Öschelbronn (Berglen))
	STÖCKENHOF = 'de:08119:5271'
	STÖCKENHOF_1 = 'de:08119:5271:0:3'  # eRtg Öschelbronn (Berglen Öschelbronn (Berglen))
	STÖCKENHOF_2 = 'de:08119:5271:0:4'  # eRtg Bürg (Berglen Öschelbronn (Berglen))
	ÖSCHELBRONN_STÖCKENHOF = 'de:08119:5271'
	ÖSCHELBRONN_STÖCKENHOF_1 = 'de:08119:5271:0:3'  # eRtg Öschelbronn (Berglen Öschelbronn (Berglen))
	ÖSCHELBRONN_STÖCKENHOF_2 = 'de:08119:5271:0:4'  # eRtg Bürg (Berglen Öschelbronn (Berglen))
	BÜRG = 'de:08119:5272'
	BÜRG_1 = 'de:08119:5272:0:3'  # eRtg Stöckenhof (Winnenden Bürg)
	BÜRG_2 = 'de:08119:5272:0:4'  # eRtg Schulerhof (Winnenden Bürg)
	HÖFEN_GEMEINDEHALLE = 'de:08119:5273'
	HÖFEN_GEMEINDEHALLE_1 = 'de:08119:5273:0:3'  # eRtg Schulerhof (Winnenden Höfen)
	HÖFEN_GEMEINDEHALLE_2 = 'de:08119:5273:0:4'  # eRtg Ortsmitte (Winnenden Höfen)
	BAACH_ORTSMITTE = 'de:08119:5274'
	BAACH_ORTSMITTE_1 = 'de:08119:5274:0:3'  # eRtg Schulerhof (Winnenden Baach (Winnenden))
	BAACH_ORTSMITTE_2 = 'de:08119:5274:0:4'  # eRtg Höfen (Winnenden Baach (Winnenden))
	HÖFEN_ORTSMITTE = 'de:08119:5275'
	HÖFEN_ORTSMITTE_1 = 'de:08119:5275:0:3'  # eRtg Baach/Hertmannsweiler (Winnenden Höfen)
	HÖFEN_ORTSMITTE_2 = 'de:08119:5275:0:4'  # eRtg Seehaldenweg (Winnenden Höfen)
	LANNERSTRASSE = 'de:08119:5276'
	LANNERSTRASSE_1 = 'de:08119:5276:0:3'  # eRtg Ortsmitte (Berglen Oppelsbohm)
	LANNERSTRASSE_2 = 'de:08119:5276:0:4'  # eRtg Nachbarschaftsschule (Berglen Oppelsbohm)
	OPPELSBOHM_LANNERSTRASSE = 'de:08119:5276'
	OPPELSBOHM_LANNERSTRASSE_1 = 'de:08119:5276:0:3'  # eRtg Ortsmitte (Berglen Oppelsbohm)
	OPPELSBOHM_LANNERSTRASSE_2 = 'de:08119:5276:0:4'  # eRtg Nachbarschaftsschule (Berglen Oppelsbohm)
	BRETZ__NACHBARSCH_SCHULE = 'de:08119:5277'
	BRETZ__NACHBARSCH_SCHULE_1 = 'de:08119:5277:0:3'  # eRtg Lannerstraße (Berglen Bretzenacker)
	BRETZ__NACHBARSCH_SCHULE_2 = 'de:08119:5277:0:4'  # eRtg Erlenhof (Berglen Bretzenacker)
	ÖDERNHARDT = 'de:08119:5278:0:3'  # eRtg Erlenhof (Berglen Ödernhardt)
	KOTTWEIL = 'de:08119:5279:0:3'  # eRtg Erlenhof (Berglen Steinach (Berglen))
	STEINACH_KOTTWEIL = 'de:08119:5279:0:3'  # eRtg Erlenhof (Berglen Steinach (Berglen))
	CRONHÜTTE = 'de:08119:5280'
	CRONHÜTTE_1 = 'de:08119:5280:0:3'  # eRtg Täle (Kaisersbach Kaisersbach)
	CRONHÜTTE_2 = 'de:08119:5280:0:4'  # eRtg Hellershof (Kaisersbach Kaisersbach)
	KAISERSBACH_CRONHÜTTE = 'de:08119:5280'
	KAISERSBACH_CRONHÜTTE_1 = 'de:08119:5280:0:3'  # eRtg Täle (Kaisersbach Kaisersbach)
	KAISERSBACH_CRONHÜTTE_2 = 'de:08119:5280:0:4'  # eRtg Hellershof (Kaisersbach Kaisersbach)
	HELLERSHOF = 'de:08119:5281'
	HELLERSHOF_1 = 'de:08119:5281:0:3'  # eRtg Cronhütte (Alfdorf Vordersteinenberg)
	HELLERSHOF_2 = 'de:08119:5281:0:4'  # eRtg Schadberg (Alfdorf Vordersteinenberg)
	VORDERST__HELLERSHOF = 'de:08119:5281'
	VORDERST__HELLERSHOF_1 = 'de:08119:5281:0:3'  # eRtg Cronhütte (Alfdorf Vordersteinenberg)
	VORDERST__HELLERSHOF_2 = 'de:08119:5281:0:4'  # eRtg Schadberg (Alfdorf Vordersteinenberg)
	KAISERSB__OB__SCHADBERG = 'de:08119:5282'
	KAISERSB__OB__SCHADBERG_1 = 'de:08119:5282:0:3'  # eRtg Schadberg (Kaisersbach Kaisersbach)
	KAISERSB__OB__SCHADBERG_2 = 'de:08119:5282:0:4'  # eRtg Steinbeise (Kaisersbach Kaisersbach)
	OBERER_SCHADBERG = 'de:08119:5282'
	OBERER_SCHADBERG_1 = 'de:08119:5282:0:3'  # eRtg Schadberg (Kaisersbach Kaisersbach)
	OBERER_SCHADBERG_2 = 'de:08119:5282:0:4'  # eRtg Steinbeise (Kaisersbach Kaisersbach)
	KAISERSB__SCHADBERG = 'de:08119:5283'
	KAISERSB__SCHADBERG_1 = 'de:08119:5283:0:3'  # eRtg Hellershof Kreuzung (Kaisersbach Kaisersbach)
	KAISERSB__SCHADBERG_2 = 'de:08119:5283:0:4'  # eRtg Oberer Schadberg (Kaisersbach Kaisersbach)
	SCHADBERG = 'de:08119:5283'
	SCHADBERG_1 = 'de:08119:5283:0:3'  # eRtg Hellershof Kreuzung (Kaisersbach Kaisersbach)
	SCHADBERG_2 = 'de:08119:5283:0:4'  # eRtg Oberer Schadberg (Kaisersbach Kaisersbach)
	KLINGENMÜHLE = 'de:08119:5284'
	KLINGENMÜHLE_1 = 'de:08119:5284:0:3'  # eRtg Laufenmühle (Welzheim Welzheim)
	KLINGENMÜHLE_2 = 'de:08119:5284:0:4'  # eRtg Abzweig Schmalenberg (Welzheim Welzheim)
	WELZHEIM_KLINGENMÜHLE = 'de:08119:5284'
	WELZHEIM_KLINGENMÜHLE_1 = 'de:08119:5284:0:3'  # eRtg Laufenmühle (Welzheim Welzheim)
	WELZHEIM_KLINGENMÜHLE_2 = 'de:08119:5284:0:4'  # eRtg Abzweig Schmalenberg (Welzheim Welzheim)
	STÖCKACHSCHULE = 'de:08119:5286:0:3'  # eRtg Wolfsklingenweg (Winnenden Winnenden)
	WINNENDEN_STÖCKACHSCHULE = 'de:08119:5286:0:3'  # eRtg Wolfsklingenweg (Winnenden Winnenden)
	UNTERW__BEETHOVENSTRASSE = 'de:08119:5287'
	UNTERW__BEETHOVENSTRASSE_1 = 'de:08119:5287:0:3'  # eRtg Täferhalde (Weissach im Tal Unterweissach)
	UNTERW__BEETHOVENSTRASSE_2 = 'de:08119:5287:0:4'  # eRtg Ungeheuerhof (Weissach im Tal Unterweissach)
	UNTERWEISSACH_EDEKA = 'de:08119:5288'
	UNTERWEISSACH_EDEKA_1 = 'de:08119:5288:0:3'  # eRtg Aichholzhof/Stockäcker (Weissach im Tal Unterweissach)
	UNTERWEISSACH_EDEKA_2 = 'de:08119:5288:0:4'  # eRtg Welzheimer Str./Lindenplatz (Weissach im Tal Unterweissach)
	EDEKA = 'de:08119:5288'
	EDEKA_1 = 'de:08119:5288:0:3'  # eRtg Aichholzhof/Stockäcker (Weissach im Tal Unterweissach)
	EDEKA_2 = 'de:08119:5288:0:4'  # eRtg Welzheimer Str./Lindenplatz (Weissach im Tal Unterweissach)
	REICHENB__LEHNENBERG = 'de:08119:5290'
	REICHENB__LEHNENBERG_1 = 'de:08119:5290:0:5'  # Rtg Spechtshof (Berglen Reichenbach (Berglen))
	REICHENB__LEHNENBERG_2 = 'de:08119:5290:0:6'  # Rtg Breuningsweiler (Berglen Reichenbach (Berglen))
	LEHNENBERG = 'de:08119:5290'
	LEHNENBERG_1 = 'de:08119:5290:0:5'  # Rtg Spechtshof (Berglen Reichenbach (Berglen))
	LEHNENBERG_2 = 'de:08119:5290:0:6'  # Rtg Breuningsweiler (Berglen Reichenbach (Berglen))
	SPECHTSHOF = 'de:08119:5291'
	SPECHTSHOF_1 = 'de:08119:5291:0:3'  # eRtg Reichenbach (Berglen Reichenbach (Berglen))
	SPECHTSHOF_2 = 'de:08119:5291:0:4'  # eRtg Lehnenberg (Berglen Reichenbach (Berglen))
	REICHENB__SPECHTSHOF = 'de:08119:5291'
	REICHENB__SPECHTSHOF_1 = 'de:08119:5291:0:3'  # eRtg Reichenbach (Berglen Reichenbach (Berglen))
	REICHENB__SPECHTSHOF_2 = 'de:08119:5291:0:4'  # eRtg Lehnenberg (Berglen Reichenbach (Berglen))
	REICHENBACH = 'de:08119:5292'
	REICHENBACH_1 = 'de:08119:5292:0:3'  # eRtg Steinach (Berglen Reichenbach (Berglen))
	REICHENBACH_2 = 'de:08119:5292:0:4'  # eRtg Spechtshof (Berglen Reichenbach (Berglen))
	STEINACH_BUCHENSTRASSE = 'de:08119:5293'
	STEINACH_BUCHENSTRASSE_1 = 'de:08119:5293:0:3'  # eRtg Hößlinswart/Erlenhof (Berglen Steinach (Berglen))
	STEINACH_BUCHENSTRASSE_2 = 'de:08119:5293:0:4'  # Rtg. Hößlinswart (Berglen Steinach (Berglen))
	STEINACH_BUCHENSTRASSE_3 = 'de:08119:5293:0:5'  # Neuer Steig Rtg. Erlenhof (Berglen Steinach (Berglen))
	BUCHENSTRASSE_1 = 'de:08119:5293:0:3'  # eRtg Hößlinswart/Erlenhof (Berglen Steinach (Berglen))
	BUCHENSTRASSE_2 = 'de:08119:5293:0:4'  # Rtg. Hößlinswart (Berglen Steinach (Berglen))
	BUCHENSTRASSE_3 = 'de:08119:5293:0:5'  # Neuer Steig Rtg. Erlenhof (Berglen Steinach (Berglen))
	BUCHENSTRASSE_4 = 'de:08117:5004:0:H1'  # Rtg Haier Waldorfsch. (Göppingen Faurndau)
	MURRHARDT_STADTHALLE = 'de:08119:5294:1:3'  # eRtg Alle Richtungen (Murrhardt Murrhardt)
	MURRHARDT_HAMMERSCHMIEDE = 'de:08119:5295:0:3'  # eRtg Klingen (Murrhardt Murrhardt)
	HAMMERSCHMIEDE = 'de:08119:5295:0:3'  # eRtg Klingen (Murrhardt Murrhardt)
	KLINGEN = 'de:08119:5296:0:3'  # eRtg Sauerhöfle (Murrhardt Murrhardt)
	MURRHARDT_KLINGEN = 'de:08119:5296:0:3'  # eRtg Sauerhöfle (Murrhardt Murrhardt)
	SAUERHÖFLE = 'de:08119:5297:0:3'  # eRtg Käsbach (Murrhardt Murrhardt)
	MURRHARDT_SAUERHÖFLE = 'de:08119:5297:0:3'  # eRtg Käsbach (Murrhardt Murrhardt)
	MURRHARDT_KÄSBACH = 'de:08119:5298:0:3'  # eRtg Vorderwestermurr (Murrhardt Murrhardt)
	KÄSBACH = 'de:08119:5298:0:3'  # eRtg Vorderwestermurr (Murrhardt Murrhardt)
	MURRH__VORDERWESTERMURR = 'de:08119:5299:0:3'  # eRtg Fautspach (Murrhardt Murrhardt)
	VORDERWESTERMURR = 'de:08119:5299:0:3'  # eRtg Fautspach (Murrhardt Murrhardt)
	FAUTSPACH = 'de:08119:5301:0:3'  # eRtg Sechselberg/Hinterwestermurr (Althütte Sechselberg)
	SECHSELBERG_FAUTSPACH = 'de:08119:5301:0:3'  # eRtg Sechselberg/Hinterwestermurr (Althütte Sechselberg)
	FORNSB__HINTERWESTERMURR = 'de:08119:5303:0:3'  # eRtg Schlosshof (Murrhardt Fornsbach)
	HINTERWESTERMURR = 'de:08119:5303:0:3'  # eRtg Schlosshof (Murrhardt Fornsbach)
	FORNSBACH_SCHLOSSHOF = 'de:08119:5304:0:3'  # eRtg Mettelberg (Murrhardt Fornsbach)
	FORNSBACH_METTELBERG = 'de:08119:5305:0:3'  # eRtg Schlossmühle (Murrhardt Fornsbach)
	METTELBERG = 'de:08119:5305:0:3'  # eRtg Schlossmühle (Murrhardt Fornsbach)
	WINNENDEN_KELTER = 'de:08119:5306'
	WINNENDEN_KELTER_1 = 'de:08119:5306:0:3'  # Rtg Hermann-Löns-Weg/Herm.-Schwab-Halle (Winnenden Winnenden)
	WINNENDEN_KELTER_2 = 'de:08119:5306:0:4'  # Rtg Wallstr./Stöckachschule (Winnenden Winnenden)
	WINNENDEN_BILDUNGSZ__II = 'de:08119:5307:1:3'  # Alle Richtungen (Winnenden Winnenden)
	BILDUNGSZENTRUM_II = 'de:08119:5307:1:3'  # Alle Richtungen (Winnenden Winnenden)
	WINNENDEN_LANGE_WEIDEN = 'de:08119:5308'
	WINNENDEN_LANGE_WEIDEN_1 = 'de:08119:5308:0:4'  # eRtg Rems-Murr-Klinik (Winnenden Winnenden)
	WINNENDEN_LANGE_WEIDEN_2 = 'de:08119:5308:0:5'  # Rtg Brückenstr. (Winnenden Winnenden)
	LANGE_WEIDEN = 'de:08119:5308'
	LANGE_WEIDEN_1 = 'de:08119:5308:0:4'  # eRtg Rems-Murr-Klinik (Winnenden Winnenden)
	LANGE_WEIDEN_2 = 'de:08119:5308:0:5'  # Rtg Brückenstr. (Winnenden Winnenden)
	SCHELMENHOLZ_FORCHENWALDSTR = 'de:08119:5309'
	SCHELMENHOLZ_FORCHENWALDSTR_1 = 'de:08119:5309:0:3'  # eRtg Th.-Heuss-Platz (Winnenden Schelmenholz)
	SCHELMENHOLZ_FORCHENWALDSTR_2 = 'de:08119:5309:0:4'  # eRtg Eschenweg (Winnenden Schelmenholz)
	FORCHENWALDSTRASSE = 'de:08119:5309'
	FORCHENWALDSTRASSE_1 = 'de:08119:5309:0:3'  # eRtg Th.-Heuss-Platz (Winnenden Schelmenholz)
	FORCHENWALDSTRASSE_2 = 'de:08119:5309:0:4'  # eRtg Eschenweg (Winnenden Schelmenholz)
	HANWEILER_BURGUNDERWEG = 'de:08119:5310'
	HANWEILER_BURGUNDERWEG_1 = 'de:08119:5310:0:3'  # eRtg Backhäusle (Winnenden Hanweiler)
	HANWEILER_BURGUNDERWEG_2 = 'de:08119:5310:0:4'  # eRtg Eschenweg (Winnenden Hanweiler)
	BURGUNDERWEG = 'de:08119:5310'
	BURGUNDERWEG_1 = 'de:08119:5310:0:3'  # eRtg Backhäusle (Winnenden Hanweiler)
	BURGUNDERWEG_2 = 'de:08119:5310:0:4'  # eRtg Eschenweg (Winnenden Hanweiler)
	BACKHÄUSLE = 'de:08119:5311:1:3'  # Alle Richtungen (Winnenden Hanweiler)
	HANWEILER_BACKHÄUSLE = 'de:08119:5311:1:3'  # Alle Richtungen (Winnenden Hanweiler)
	MURRHARDT_KAISER_LUDWIG_STR = 'de:08119:5312'
	MURRHARDT_KAISER_LUDWIG_STR_1 = 'de:08119:5312:0:3'  # eRtg Bahnhof/Stadthalle (Murrhardt Murrhardt)
	MURRHARDT_KAISER_LUDWIG_STR_2 = 'de:08119:5312:0:4'  # eRtg Hörschbachschule (Murrhardt Murrhardt)
	KAISER_LUDWIG_STRASSE = 'de:08119:5312'
	KAISER_LUDWIG_STRASSE_1 = 'de:08119:5312:0:3'  # eRtg Bahnhof/Stadthalle (Murrhardt Murrhardt)
	KAISER_LUDWIG_STRASSE_2 = 'de:08119:5312:0:4'  # eRtg Hörschbachschule (Murrhardt Murrhardt)
	LEUTENBACH_RATHAUS = 'de:08119:5313'
	LEUTENBACH_RATHAUS_1 = 'de:08119:5313:0:3'  # eRtg Winnenden (Leutenbach Leutenbach)
	LEUTENBACH_RATHAUS_2 = 'de:08119:5313:0:4'  # eRtg Hauptstraße (Leutenbach Leutenbach)
	LEUTENBACH_HAUPTSTRASSE = 'de:08119:5314'
	LEUTENBACH_HAUPTSTRASSE_1 = 'de:08119:5314:0:3'  # Rtg Weiler (Leutenbach Leutenbach)
	LEUTENBACH_HAUPTSTRASSE_2 = 'de:08119:5314:0:4'  # Rtg Winnenden (Leutenbach Leutenbach)
	WEILER_ZUM_STEIN_STUTTG__STR = 'de:08119:5315'
	WEILER_ZUM_STEIN_STUTTG__STR_1 = 'de:08119:5315:0:3'  # eRtg Hauptstraße (Leutenbach Weiler zum Stein)
	WEILER_ZUM_STEIN_STUTTG__STR_2 = 'de:08119:5315:0:4'  # eRtg Rathaus (Leutenbach Weiler zum Stein)
	LEUTENBACH_SCHULE = 'de:08119:5319'
	LEUTENBACH_SCHULE_1 = 'de:08119:5319:0:3'  # Rtg Rathaus/Nellmersb. Leutenb.Str. (Leutenbach Leutenbach)
	LEUTENBACH_SCHULE_2 = 'de:08119:5319:0:4'  # Rtg Rathaus (Leutenbach Leutenbach)
	LEUTENBACHER_STRASSE = 'de:08119:5320'
	LEUTENBACHER_STRASSE_1 = 'de:08119:5320:0:3'  # eRtg Bahnhofstraße (Leutenbach Nellmersbach)
	LEUTENBACHER_STRASSE_2 = 'de:08119:5320:0:4'  # eRtg Hauptschule (Leutenbach Nellmersbach)
	NELLMERSBACH_LEUTENBACHER_STR = 'de:08119:5320'
	NELLMERSBACH_LEUTENBACHER_STR_1 = 'de:08119:5320:0:3'  # eRtg Bahnhofstraße (Leutenbach Nellmersbach)
	NELLMERSBACH_LEUTENBACHER_STR_2 = 'de:08119:5320:0:4'  # eRtg Hauptschule (Leutenbach Nellmersbach)
	NELLMERSBACH_BAHNHOFSTR = 'de:08119:5321'
	NELLMERSBACH_BAHNHOFSTR_1 = 'de:08119:5321:1:3'  # Rtg B14 (Leutenbach Nellmersbach)
	NELLMERSBACH_BAHNHOFSTR_2 = 'de:08119:5321:1:4'  # Rtg Leutenbacher Str. (Leutenbach Nellmersbach)
	BACKNANG_WALDFRIEDHOF = 'de:08119:5322'
	BACKNANG_WALDFRIEDHOF_1 = 'de:08119:5322:0:3'  # eRtg Eichwald (Backnang Backnang)
	BACKNANG_WALDFRIEDHOF_2 = 'de:08119:5322:0:4'  # eRtg Seehof (Backnang Backnang)
	BACKNANG_ETZWIESENBRÜCKE = 'de:08119:5323'
	BACKNANG_ETZWIESENBRÜCKE_1 = 'de:08119:5323:0:3'  # Rtg Bahnhof (Backnang Backnang)
	BACKNANG_ETZWIESENBRÜCKE_2 = 'de:08119:5323:0:4'  # Rtg Gerberstr. (Backnang Backnang)
	ETZWIESENBRÜCKE = 'de:08119:5323'
	ETZWIESENBRÜCKE_1 = 'de:08119:5323:0:3'  # Rtg Bahnhof (Backnang Backnang)
	ETZWIESENBRÜCKE_2 = 'de:08119:5323:0:4'  # Rtg Gerberstr. (Backnang Backnang)
	BACKNANG_RATHAUS = 'de:08119:5324:0:3'  # eRtg Biegel (Backnang Backnang)
	PLATTENWALD = 'de:08119:5325'
	PLATTENWALD_1 = 'de:08119:5325:0:3'  # eRtg Weidenweg (Backnang Backnang)
	PLATTENWALD_2 = 'de:08119:5325:0:4'  # eRtg Föhrenbühl (Backnang Backnang)
	BACKNANG_PLATTENWALD = 'de:08119:5325'
	BACKNANG_PLATTENWALD_1 = 'de:08119:5325:0:3'  # eRtg Weidenweg (Backnang Backnang)
	BACKNANG_PLATTENWALD_2 = 'de:08119:5325:0:4'  # eRtg Föhrenbühl (Backnang Backnang)
	BACKNANG_ASPACHER_BRÜCKE = 'de:08119:5329:0:4'  # Rtg Etzwiesen/Schöntal/Seminar (Backnang Backnang)
	ASPACHER_BRÜCKE = 'de:08119:5329:0:4'  # Rtg Etzwiesen/Schöntal/Seminar (Backnang Backnang)
	BERLINER_RING = 'de:08119:5330'
	BERLINER_RING_1 = 'de:08119:5330:0:3'  # eRtg Strümpfelb./Seehofweg/Dresdner Ring (Backnang Backnang)
	BERLINER_RING_2 = 'de:08119:5330:0:4'  # eRtg Im Wiesengrund (Backnang Backnang)
	BACKNANG_BERLINER_RING = 'de:08119:5330'
	BACKNANG_BERLINER_RING_1 = 'de:08119:5330:0:3'  # eRtg Strümpfelb./Seehofweg/Dresdner Ring (Backnang Backnang)
	BACKNANG_BERLINER_RING_2 = 'de:08119:5330:0:4'  # eRtg Im Wiesengrund (Backnang Backnang)
	SEEHOFWEG = 'de:08119:5331'
	SEEHOFWEG_1 = 'de:08119:5331:0:3'  # Rtg Seehof/Plaisirschule (Backnang Backnang)
	SEEHOFWEG_2 = 'de:08119:5331:0:4'  # Rtg Berliner Ring (Backnang Backnang)
	SEEHOFWEG_3 = 'de:08119:5331:0:5'  # Rtg Berliner Ring (Backnang Backnang)
	BACKNANG_SEEHOFWEG = 'de:08119:5331'
	BACKNANG_SEEHOFWEG_1 = 'de:08119:5331:0:3'  # Rtg Seehof/Plaisirschule (Backnang Backnang)
	BACKNANG_SEEHOFWEG_2 = 'de:08119:5331:0:4'  # Rtg Berliner Ring (Backnang Backnang)
	BACKNANG_SEEHOFWEG_3 = 'de:08119:5331:0:5'  # Rtg Berliner Ring (Backnang Backnang)
	PLAISIRSCHULE = 'de:08119:5332'
	PLAISIRSCHULE_1 = 'de:08119:5332:0:3'  # eRtg Potsdamer Ring (Backnang Backnang)
	PLAISIRSCHULE_2 = 'de:08119:5332:0:4'  # eRtg Seehofweg (Backnang Backnang)
	BACKNANG_PLAISIRSCHULE = 'de:08119:5332'
	BACKNANG_PLAISIRSCHULE_1 = 'de:08119:5332:0:3'  # eRtg Potsdamer Ring (Backnang Backnang)
	BACKNANG_PLAISIRSCHULE_2 = 'de:08119:5332:0:4'  # eRtg Seehofweg (Backnang Backnang)
	POTSDAMER_RING = 'de:08119:5333'
	POTSDAMER_RING_1 = 'de:08119:5333:0:3'  # eRtg Stettiner Ring (Backnang Backnang)
	POTSDAMER_RING_2 = 'de:08119:5333:0:4'  # eRtg Plaisirschule (Backnang Backnang)
	BACKNANG_POTSDAMER_RING = 'de:08119:5333'
	BACKNANG_POTSDAMER_RING_1 = 'de:08119:5333:0:3'  # eRtg Stettiner Ring (Backnang Backnang)
	BACKNANG_POTSDAMER_RING_2 = 'de:08119:5333:0:4'  # eRtg Plaisirschule (Backnang Backnang)
	BACKNANG_STETTINER_RING = 'de:08119:5334'
	BACKNANG_STETTINER_RING_1 = 'de:08119:5334:0:3'  # eRtg Hintere Tos (Backnang Backnang)
	BACKNANG_STETTINER_RING_2 = 'de:08119:5334:0:4'  # eRtg Potsdamer Ring (Backnang Backnang)
	STETTINER_RING = 'de:08119:5334'
	STETTINER_RING_1 = 'de:08119:5334:0:3'  # eRtg Hintere Tos (Backnang Backnang)
	STETTINER_RING_2 = 'de:08119:5334:0:4'  # eRtg Potsdamer Ring (Backnang Backnang)
	GERBERSTRASSE = 'de:08119:5335:0:3'  # eRtg Schöntaler Straße/Seminar (Backnang Backnang)
	BACKNANG_GERBERSTRASSE = 'de:08119:5335:0:3'  # eRtg Schöntaler Straße/Seminar (Backnang Backnang)
	BACKNANG_FEUERWEHR = 'de:08119:5336'
	BACKNANG_FEUERWEHR_1 = 'de:08119:5336:0:3'  # eRtg Gartenstraße (Backnang Backnang)
	BACKNANG_FEUERWEHR_2 = 'de:08119:5336:0:4'  # eRtg Biegel (Backnang Backnang)
	EINTRACHT = 'de:08119:5337'
	EINTRACHT_1 = 'de:08119:5337:0:3'  # eRtg Gertrud-Bäumer-Weg (Backnang Backnang)
	EINTRACHT_2 = 'de:08119:5337:0:4'  # eRtg Gartenstraße (Backnang Backnang)
	BACKNANG_EINTRACHT = 'de:08119:5337'
	BACKNANG_EINTRACHT_1 = 'de:08119:5337:0:3'  # eRtg Gertrud-Bäumer-Weg (Backnang Backnang)
	BACKNANG_EINTRACHT_2 = 'de:08119:5337:0:4'  # eRtg Gartenstraße (Backnang Backnang)
	BACKNANG_SOMMERRAIN = 'de:08119:5338'
	BACKNANG_SOMMERRAIN_1 = 'de:08119:5338:1:3'  # eRtg Föhrenbühl/Steinbach (Backnang Backnang)
	BACKNANG_SOMMERRAIN_2 = 'de:08119:5338:1:4'  # eRtg Getrud-Bäumer-Weg/Hintere Tos (Backnang Backnang)
	BACKNANG_TAUSGYMNASIUM = 'de:08119:5339:0:3'  # eRtg Alle Richtungen (Backnang Backnang)
	TAUSGYMNASIUM = 'de:08119:5339:0:3'  # eRtg Alle Richtungen (Backnang Backnang)
	WEIDENWEG = 'de:08119:5340'
	WEIDENWEG_1 = 'de:08119:5340:0:3'  # eRtg Tausgymnasium (Backnang Backnang)
	WEIDENWEG_2 = 'de:08119:5340:0:4'  # eRtg Plattenwald (Backnang Backnang)
	BACKNANG_WEIDENWEG = 'de:08119:5340'
	BACKNANG_WEIDENWEG_1 = 'de:08119:5340:0:3'  # eRtg Tausgymnasium (Backnang Backnang)
	BACKNANG_WEIDENWEG_2 = 'de:08119:5340:0:4'  # eRtg Plattenwald (Backnang Backnang)
	FÖHRENBÜHL = 'de:08119:5342'
	FÖHRENBÜHL_1 = 'de:08119:5342:0:3'  # eRtg Plattenwald (Backnang Backnang)
	FÖHRENBÜHL_2 = 'de:08119:5342:0:4'  # eRtg Sommerrain (Backnang Backnang)
	BACKNANG_FÖHRENBÜHL = 'de:08119:5342'
	BACKNANG_FÖHRENBÜHL_1 = 'de:08119:5342:0:3'  # eRtg Plattenwald (Backnang Backnang)
	BACKNANG_FÖHRENBÜHL_2 = 'de:08119:5342:0:4'  # eRtg Sommerrain (Backnang Backnang)
	STEINBACH_DORFHALLE = 'de:08119:5343:1:3'  # Lindenplatz (Backnang Steinbach)
	DORFHALLE = 'de:08119:5343:1:3'  # Lindenplatz (Backnang Steinbach)
	BACKNANG_BURGPLATZ = 'de:08119:5345'
	BACKNANG_BURGPLATZ_1 = 'de:08119:5345:0:3'  # eRtg Obere Bahnhofstraße (Backnang Backnang)
	BACKNANG_BURGPLATZ_2 = 'de:08119:5345:0:4'  # eRtg Hasenhälde (Backnang Backnang)
	BURGPLATZ_1 = 'de:08119:5345:0:3'  # eRtg Obere Bahnhofstraße (Backnang Backnang)
	BURGPLATZ_2 = 'de:08119:5345:0:4'  # eRtg Hasenhälde (Backnang Backnang)
	BURGPLATZ_3 = 'de:08118:5700:1:3'  # Rtg Freibad (Bönnigheim Bönnigheim)
	BURGPLATZ_4 = 'de:08118:5700:1:4'  # eRtg Schulzentrum (Bönnigheim Bönnigheim)
	MARIENHEIM = 'de:08119:5346'
	MARIENHEIM_1 = 'de:08119:5346:0:4'  # eRtg Hof (Backnang Backnang)
	MARIENHEIM_2 = 'de:08119:5346:0:5'  # Ersatz Rtg. Hasenhälde (Backnang Backnang)
	BACKNANG_MARIENHEIM = 'de:08119:5346'
	BACKNANG_MARIENHEIM_1 = 'de:08119:5346:0:4'  # eRtg Hof (Backnang Backnang)
	BACKNANG_MARIENHEIM_2 = 'de:08119:5346:0:5'  # Ersatz Rtg. Hasenhälde (Backnang Backnang)
	SACHSENWEILER_HOF = 'de:08119:5347'
	SACHSENWEILER_HOF_1 = 'de:08119:5347:0:3'  # eRtg Marienheim (Backnang Sachsenweiler)
	SACHSENWEILER_HOF_2 = 'de:08119:5347:0:4'  # eRtg Am Dresselbach (Backnang Sachsenweiler)
	AM_DRESSELBACH = 'de:08119:5348:0:3'  # eRtg Evangelische Kirche (Backnang Sachsenweiler)
	SACHSENW__AM_DRESSELBACH = 'de:08119:5348:0:3'  # eRtg Evangelische Kirche (Backnang Sachsenweiler)
	EV__KIRCHE = 'de:08119:5349:0:3'  # eRtg Hof (Backnang Sachsenweiler)
	SACHSENWEILER_EV__KIRCHE = 'de:08119:5349:0:3'  # eRtg Hof (Backnang Sachsenweiler)
	BACKNANG_SEMINAR = 'de:08119:5350'
	BACKNANG_SEMINAR_1 = 'de:08119:5350:0:3'  # eRtg Rossbergstaffel (Backnang Backnang)
	BACKNANG_SEMINAR_2 = 'de:08119:5350:0:4'  # eRtg Aspacher Brücke/Etzwiesenbrücke (Backnang Backnang)
	SEMINAR = 'de:08119:5350'
	SEMINAR_1 = 'de:08119:5350:0:3'  # eRtg Rossbergstaffel (Backnang Backnang)
	SEMINAR_2 = 'de:08119:5350:0:4'  # eRtg Aspacher Brücke/Etzwiesenbrücke (Backnang Backnang)
	BACKNANG_LICHTENSTEINSTR = 'de:08119:5351'
	BACKNANG_LICHTENSTEINSTR_1 = 'de:08119:5351:0:3'  # eRtg Mittelschöntal (Backnang Backnang)
	BACKNANG_LICHTENSTEINSTR_2 = 'de:08119:5351:0:4'  # eRtg Rötlensweg (Backnang Backnang)
	LICHTENSTEINSTRASSE = 'de:08119:5351'
	LICHTENSTEINSTRASSE_1 = 'de:08119:5351:0:3'  # eRtg Mittelschöntal (Backnang Backnang)
	LICHTENSTEINSTRASSE_2 = 'de:08119:5351:0:4'  # eRtg Rötlensweg (Backnang Backnang)
	BACKNANG_OBERSCHÖNTAL_ABZW = 'de:08119:5352:0:3'  # eRtg Oberschöntal (Backnang Backnang)
	OBERSCHÖNTAL_ABZWEIG = 'de:08119:5352:0:3'  # eRtg Oberschöntal (Backnang Backnang)
	UNTERSCHÖNTAL = 'de:08119:5353:0:3'  # eRtg Oberschöntal Abzweig (Backnang Backnang)
	BACKNANG_UNTERSCHÖNTAL = 'de:08119:5353:0:3'  # eRtg Oberschöntal Abzweig (Backnang Backnang)
	BACKNANG_OBERSCHÖNTAL = 'de:08119:5354:0:3'  # eRtg Lichtensteinstraße (Backnang Backnang)
	OBERSCHÖNTAL = 'de:08119:5354:0:3'  # eRtg Lichtensteinstraße (Backnang Backnang)
	ROSSBERGSTAFFEL = 'de:08119:5355'
	ROSSBERGSTAFFEL_1 = 'de:08119:5355:0:3'  # eRtg Seminar (Backnang Backnang)
	ROSSBERGSTAFFEL_2 = 'de:08119:5355:0:4'  # eRtg Aspach/Wasserturm (Backnang Backnang)
	BACKNANG_ROSSBERGSTAFFEL = 'de:08119:5355'
	BACKNANG_ROSSBERGSTAFFEL_1 = 'de:08119:5355:0:3'  # eRtg Seminar (Backnang Backnang)
	BACKNANG_ROSSBERGSTAFFEL_2 = 'de:08119:5355:0:4'  # eRtg Aspach/Wasserturm (Backnang Backnang)
	GROSSASPACH_LERCHENSTR = 'de:08119:5356'
	GROSSASPACH_LERCHENSTR_1 = 'de:08119:5356:1:3'  # Rtg Klöpferbach (Aspach Großaspach)
	GROSSASPACH_LERCHENSTR_2 = 'de:08119:5356:1:4'  # Rtg Rossbergstaffel (Aspach Großaspach)
	GROSSASPACH_KLÖPFERBACH = 'de:08119:5357'
	GROSSASPACH_KLÖPFERBACH_1 = 'de:08119:5357:1:3'  # Rtg Fr.-List-Str./Marbacher Str. (Aspach Großaspach)
	GROSSASPACH_KLÖPFERBACH_2 = 'de:08119:5357:1:4'  # Rtg Lerchenstr. (Aspach Großaspach)
	KLÖPFERBACH = 'de:08119:5357'
	KLÖPFERBACH_1 = 'de:08119:5357:1:3'  # Rtg Fr.-List-Str./Marbacher Str. (Aspach Großaspach)
	KLÖPFERBACH_2 = 'de:08119:5357:1:4'  # Rtg Lerchenstr. (Aspach Großaspach)
	HOHROT = 'de:08119:5358'
	HOHROT_1 = 'de:08119:5358:0:3'  # eRtg Rietenau (Aspach Großaspach)
	HOHROT_2 = 'de:08119:5358:0:4'  # eRtg Friedrich-List-Straße (Aspach Großaspach)
	GROSSASPACH_HOHROT = 'de:08119:5358'
	GROSSASPACH_HOHROT_1 = 'de:08119:5358:0:3'  # eRtg Rietenau (Aspach Großaspach)
	GROSSASPACH_HOHROT_2 = 'de:08119:5358:0:4'  # eRtg Friedrich-List-Straße (Aspach Großaspach)
	RIETENAU = 'de:08119:5359'
	RIETENAU_1 = 'de:08119:5359:0:3'  # Rtg Allmersbach (Aspach Rietenau)
	RIETENAU_2 = 'de:08119:5359:0:4'  # Rtg Aspach (Aspach Rietenau)
	ALLMERSBACH_AM_WEINBERG = 'de:08119:5360'
	ALLMERSBACH_AM_WEINBERG_1 = 'de:08119:5360:0:3'  # eRtg Röhrach (Aspach Allmersbach am Weinberg (Aspach))
	ALLMERSBACH_AM_WEINBERG_2 = 'de:08119:5360:0:4'  # eRtg Rietenau (Aspach Allmersbach am Weinberg (Aspach))
	ALLMERSBACH_A_W__RÖHRACH = 'de:08119:5361'
	ALLMERSBACH_A_W__RÖHRACH_1 = 'de:08119:5361:0:3'  # eRtg Schafhaus (Aspach Allmersbach am Weinberg (Aspach))
	ALLMERSBACH_A_W__RÖHRACH_2 = 'de:08119:5361:0:4'  # eRtg Allmersbach am Weinberg (Aspach Allmersbach am Weinberg (Aspach))
	RÖHRACH = 'de:08119:5361'
	RÖHRACH_1 = 'de:08119:5361:0:3'  # eRtg Schafhaus (Aspach Allmersbach am Weinberg (Aspach))
	RÖHRACH_2 = 'de:08119:5361:0:4'  # eRtg Allmersbach am Weinberg (Aspach Allmersbach am Weinberg (Aspach))
	SCHAFHAUS = 'de:08119:5362'
	SCHAFHAUS_1 = 'de:08119:5362:0:3'  # eRtg Ortsmitte (Aspach Kleinaspach)
	SCHAFHAUS_2 = 'de:08119:5362:0:4'  # eRtg Röhrach (Aspach Kleinaspach)
	KLEINASPACH_SCHAFHAUS = 'de:08119:5362'
	KLEINASPACH_SCHAFHAUS_1 = 'de:08119:5362:0:3'  # eRtg Ortsmitte (Aspach Kleinaspach)
	KLEINASPACH_SCHAFHAUS_2 = 'de:08119:5362:0:4'  # eRtg Röhrach (Aspach Kleinaspach)
	RIELINGSHÄUSER_STRASSE = 'de:08118:5363'
	RIELINGSHÄUSER_STRASSE_1 = 'de:08118:5363:0:3'  # eRtg Marbach (N) (Marbach am Neckar Marbach (N))
	RIELINGSHÄUSER_STRASSE_2 = 'de:08118:5363:0:4'  # eRtg Huober (Marbach am Neckar Marbach (N))
	MARB__N__RIELINGSHÄUSER_STR = 'de:08118:5363'
	MARB__N__RIELINGSHÄUSER_STR_1 = 'de:08118:5363:0:3'  # eRtg Marbach (N) (Marbach am Neckar Marbach (N))
	MARB__N__RIELINGSHÄUSER_STR_2 = 'de:08118:5363:0:4'  # eRtg Huober (Marbach am Neckar Marbach (N))
	GROSSASPACH_FÜRSTENHOF = 'de:08119:5364'
	GROSSASPACH_FÜRSTENHOF_1 = 'de:08119:5364:0:3'  # eRtg Zwingelhausen Abzweig (Aspach Großaspach)
	GROSSASPACH_FÜRSTENHOF_2 = 'de:08119:5364:0:4'  # eRtg Lerchenstraße (Aspach Großaspach)
	FÜRSTENHOF = 'de:08119:5364'
	FÜRSTENHOF_1 = 'de:08119:5364:0:3'  # eRtg Zwingelhausen Abzweig (Aspach Großaspach)
	FÜRSTENHOF_2 = 'de:08119:5364:0:4'  # eRtg Lerchenstraße (Aspach Großaspach)
	GROSSASPACH_ZWINGELHAUSEN_ABZW = 'de:08119:5365'
	GROSSASPACH_ZWINGELHAUSEN_ABZW_1 = 'de:08119:5365:0:3'  # eRtg Frühmeßhof (Aspach Großaspach)
	GROSSASPACH_ZWINGELHAUSEN_ABZW_2 = 'de:08119:5365:0:4'  # eRtg Fürstenhof (Aspach Großaspach)
	ZWINGELHAUSEN_ABZWEIG = 'de:08119:5365'
	ZWINGELHAUSEN_ABZWEIG_1 = 'de:08119:5365:0:3'  # eRtg Frühmeßhof (Aspach Großaspach)
	ZWINGELHAUSEN_ABZWEIG_2 = 'de:08119:5365:0:4'  # eRtg Fürstenhof (Aspach Großaspach)
	KIRCHBERG__M__FRÜHMESSHOF = 'de:08119:5366'
	KIRCHBERG__M__FRÜHMESSHOF_1 = 'de:08119:5366:0:3'  # eRtg Egelsee (Kirchberg an der Murr Kirchberg (M))
	KIRCHBERG__M__FRÜHMESSHOF_2 = 'de:08119:5366:0:4'  # eRtg Zwingelhausen Abzweig (Kirchberg an der Murr Kirchberg (M))
	FRÜHMESSHOF = 'de:08119:5366'
	FRÜHMESSHOF_1 = 'de:08119:5366:0:3'  # eRtg Egelsee (Kirchberg an der Murr Kirchberg (M))
	FRÜHMESSHOF_2 = 'de:08119:5366:0:4'  # eRtg Zwingelhausen Abzweig (Kirchberg an der Murr Kirchberg (M))
	RIELINGSHAUSEN_SPORTPLATZ = 'de:08118:5367'
	RIELINGSHAUSEN_SPORTPLATZ_1 = 'de:08118:5367:0:3'  # eRtg Egelsee (Marbach am Neckar Rielingshausen)
	RIELINGSHAUSEN_SPORTPLATZ_2 = 'de:08118:5367:0:4'  # eRtg Hardtwaldhalle (Marbach am Neckar Rielingshausen)
	EGELSEE_1 = 'de:08118:5368:0:3'  # Rtg Marbach (Marbach am Neckar Rielingshausen)
	EGELSEE_2 = 'de:08118:5368:0:4'  # Rtg Backnang (Marbach am Neckar Rielingshausen)
	EGELSEE_3 = 'de:08236:3507:0:1'  # Rtg Süden (Heimsheim Heimsheim)
	EGELSEE_4 = 'de:08236:3507:0:2'  # Rtg Dieb (Heimsheim Heimsheim)
	RIELINGSHAUSEN_EGELSEE = 'de:08118:5368'
	RIELINGSHAUSEN_EGELSEE_1 = 'de:08118:5368:0:3'  # Rtg Marbach (Marbach am Neckar Rielingshausen)
	RIELINGSHAUSEN_EGELSEE_2 = 'de:08118:5368:0:4'  # Rtg Backnang (Marbach am Neckar Rielingshausen)
	RIELINGSHAUSEN_HAUPTSTRASSE = 'de:08118:5369'
	RIELINGSHAUSEN_HAUPTSTRASSE_1 = 'de:08118:5369:0:3'  # eRtg Schweißbrücke (Marbach am Neckar Rielingshausen)
	RIELINGSHAUSEN_HAUPTSTRASSE_2 = 'de:08118:5369:0:4'  # eRtg Egelsee (Marbach am Neckar Rielingshausen)
	ERDMANNH__SCHWEISSBRÜCKE = 'de:08118:5370'
	ERDMANNH__SCHWEISSBRÜCKE_1 = 'de:08118:5370:1:3'  # Rtg Marbach (Erdmannhausen Erdmannhausen)
	ERDMANNH__SCHWEISSBRÜCKE_2 = 'de:08118:5370:1:4'  # Rtg Backnang (Erdmannhausen Erdmannhausen)
	SCHWEISSBRÜCKE = 'de:08118:5370'
	SCHWEISSBRÜCKE_1 = 'de:08118:5370:1:3'  # Rtg Marbach (Erdmannhausen Erdmannhausen)
	SCHWEISSBRÜCKE_2 = 'de:08118:5370:1:4'  # Rtg Backnang (Erdmannhausen Erdmannhausen)
	HUOBER = 'de:08118:5371'
	HUOBER_1 = 'de:08118:5371:0:3'  # eRtg Rielingshäuser Straße (Erdmannhausen Erdmannhausen)
	HUOBER_2 = 'de:08118:5371:0:4'  # eRtg Schweißbrücke (Erdmannhausen Erdmannhausen)
	ERDMANNHAUSEN_HUOBER = 'de:08118:5371'
	ERDMANNHAUSEN_HUOBER_1 = 'de:08118:5371:0:3'  # eRtg Rielingshäuser Straße (Erdmannhausen Erdmannhausen)
	ERDMANNHAUSEN_HUOBER_2 = 'de:08118:5371:0:4'  # eRtg Schweißbrücke (Erdmannhausen Erdmannhausen)
	GÜNTTERSTRASSE = 'de:08118:5372'
	GÜNTTERSTRASSE_1 = 'de:08118:5372:0:3'  # eRtg König-Wilhelm-Platz (Marbach am Neckar Marbach (N))
	GÜNTTERSTRASSE_2 = 'de:08118:5372:0:4'  # eRtg Marbach (N) (Marbach am Neckar Marbach (N))
	MARBACH__N__GÜNTTERSTR = 'de:08118:5372'
	MARBACH__N__GÜNTTERSTR_1 = 'de:08118:5372:0:3'  # eRtg König-Wilhelm-Platz (Marbach am Neckar Marbach (N))
	MARBACH__N__GÜNTTERSTR_2 = 'de:08118:5372:0:4'  # eRtg Marbach (N) (Marbach am Neckar Marbach (N))
	UNTERWEISSACH_WELZHEIMER_STR = 'de:08119:5374'
	UNTERWEISSACH_WELZHEIMER_STR_1 = 'de:08119:5374:0:3'  # eRtg Edeka (Weissach im Tal Unterweissach)
	UNTERWEISSACH_WELZHEIMER_STR_2 = 'de:08119:5374:0:4'  # eRtg Rathaus/Lindenplatz (Weissach im Tal Unterweissach)
	UNTERWEISSACH_RATHAUS = 'de:08119:5375:0:3'  # eRtg Auenwaldhalle (Weissach im Tal Unterweissach)
	HÜGELSTRASSE = 'de:08119:5377'
	HÜGELSTRASSE_1 = 'de:08119:5377:0:3'  # eRtg Mittelbrüden Im Anwänder (Auenwald Unterbrüden)
	HÜGELSTRASSE_2 = 'de:08119:5377:0:4'  # eRtg Rathaus (Auenwald Unterbrüden)
	UNTERBRÜDEN_HÜGELSTRASSE = 'de:08119:5377'
	UNTERBRÜDEN_HÜGELSTRASSE_1 = 'de:08119:5377:0:3'  # eRtg Mittelbrüden Im Anwänder (Auenwald Unterbrüden)
	UNTERBRÜDEN_HÜGELSTRASSE_2 = 'de:08119:5377:0:4'  # eRtg Rathaus (Auenwald Unterbrüden)
	OBERBRÜDEN_MITTELBRÜDEN = 'de:08119:5378'
	OBERBRÜDEN_MITTELBRÜDEN_1 = 'de:08119:5378:0:3'  # Rtg Oberbrüden (Auenwald Oberbrüden)
	OBERBRÜDEN_MITTELBRÜDEN_2 = 'de:08119:5378:0:4'  # Rtg Unterbrüden (Auenwald Oberbrüden)
	MITTELBRÜDEN = 'de:08119:5378'
	MITTELBRÜDEN_1 = 'de:08119:5378:0:3'  # Rtg Oberbrüden (Auenwald Oberbrüden)
	MITTELBRÜDEN_2 = 'de:08119:5378:0:4'  # Rtg Unterbrüden (Auenwald Oberbrüden)
	OBERBRÜDEN_KIRCHPLATZ = 'de:08119:5379'
	OBERBRÜDEN_KIRCHPLATZ_1 = 'de:08119:5379:0:3'  # eRtg Rosenstraße (Auenwald Oberbrüden)
	OBERBRÜDEN_KIRCHPLATZ_2 = 'de:08119:5379:0:4'  # eRtg Mittelbrüden (Auenwald Oberbrüden)
	OBERBRÜDEN_ROSENSTRASSE = 'de:08119:5380:1:3'  # eRtg Alle Richtungen (Auenwald Oberbrüden)
	UNTERBRÜDEN_FRIEDHOF = 'de:08119:5381'
	UNTERBRÜDEN_FRIEDHOF_1 = 'de:08119:5381:0:3'  # eRtg Lippoldsweiler (Friedhofseite) (Auenwald Unterbrüden)
	UNTERBRÜDEN_FRIEDHOF_2 = 'de:08119:5381:0:4'  # eRtg Auenwaldhalle/Hügelstraße (Auenwald Unterbrüden)
	LIPPOLDSWEILER_TRAUBE = 'de:08119:5382'
	LIPPOLDSWEILER_TRAUBE_1 = 'de:08119:5382:0:3'  # Rtg Unterbr. Friedhof (Auenwald Lippoldsweiler)
	LIPPOLDSWEILER_TRAUBE_2 = 'de:08119:5382:0:4'  # Rtg Naturfreundehaus/Am Brenner (Auenwald Lippoldsweiler)
	AUENWALDHALLE = 'de:08119:5383'
	AUENWALDHALLE_1 = 'de:08119:5383:0:3'  # eRtg Rathaus (Auenwald Unterbrüden)
	AUENWALDHALLE_2 = 'de:08119:5383:0:4'  # eRtg Heimatmuseum (Auenwald Unterbrüden)
	UNTERBRÜDEN_AUENWALDHALLE = 'de:08119:5383'
	UNTERBRÜDEN_AUENWALDHALLE_1 = 'de:08119:5383:0:3'  # eRtg Rathaus (Auenwald Unterbrüden)
	UNTERBRÜDEN_AUENWALDHALLE_2 = 'de:08119:5383:0:4'  # eRtg Heimatmuseum (Auenwald Unterbrüden)
	AM_BRENNER = 'de:08119:5384'
	AM_BRENNER_1 = 'de:08119:5384:0:3'  # eRtg Feuerwehrgerätehaus/Unterbrüden (Auenwald Lippoldsweiler)
	AM_BRENNER_2 = 'de:08119:5384:0:4'  # eRtg Badstraße (Auenwald Lippoldsweiler)
	LIPPOLDSW__AM_BRENNER = 'de:08119:5384'
	LIPPOLDSW__AM_BRENNER_1 = 'de:08119:5384:0:3'  # eRtg Feuerwehrgerätehaus/Unterbrüden (Auenwald Lippoldsweiler)
	LIPPOLDSW__AM_BRENNER_2 = 'de:08119:5384:0:4'  # eRtg Badstraße (Auenwald Lippoldsweiler)
	LIPPOLDSW__RAIFFEISENBANK = 'de:08119:5385'
	LIPPOLDSW__RAIFFEISENBANK_1 = 'de:08119:5385:0:3'  # eRtg Badstraße (Auenwald Lippoldsweiler)
	LIPPOLDSW__RAIFFEISENBANK_2 = 'de:08119:5385:0:4'  # eRtg Rathaus (Auenwald Lippoldsweiler)
	RAIFFEISENBANK = 'de:08119:5385'
	RAIFFEISENBANK_1 = 'de:08119:5385:0:3'  # eRtg Badstraße (Auenwald Lippoldsweiler)
	RAIFFEISENBANK_2 = 'de:08119:5385:0:4'  # eRtg Rathaus (Auenwald Lippoldsweiler)
	WAIBLINGEN_MARKTGASSE = 'de:08119:5386:0:3'  # eRtg Schmidener Straße (Waiblingen Waiblingen)
	MARKTGASSE = 'de:08119:5386:0:3'  # eRtg Schmidener Straße (Waiblingen Waiblingen)
	HOCHWACHTTURM = 'de:08119:5387:0:3'  # eRtg Untere Lindenstraße (Waiblingen Waiblingen)
	WAIBLINGEN_HOCHWACHTTURM = 'de:08119:5387:0:3'  # eRtg Untere Lindenstraße (Waiblingen Waiblingen)
	HEIMATMUSEUM = 'de:08119:5388:0:3'  # eRtg Lindenplatz (Weissach im Tal Unterweissach)
	UNTERW__HEIMATMUSEUM = 'de:08119:5388:0:3'  # eRtg Lindenplatz (Weissach im Tal Unterweissach)
	DANZIGER_PLATZ = 'de:08119:5389'
	DANZIGER_PLATZ_1 = 'de:08119:5389:0:3'  # Rtg Gänsäckerstr. (Waiblingen Waiblingen)
	DANZIGER_PLATZ_2 = 'de:08119:5389:0:4'  # Rtg Jesistraße/Sommerhalde (Waiblingen Waiblingen)
	WAIBLINGEN_DANZIGER_PLATZ = 'de:08119:5389'
	WAIBLINGEN_DANZIGER_PLATZ_1 = 'de:08119:5389:0:3'  # Rtg Gänsäckerstr. (Waiblingen Waiblingen)
	WAIBLINGEN_DANZIGER_PLATZ_2 = 'de:08119:5389:0:4'  # Rtg Jesistraße/Sommerhalde (Waiblingen Waiblingen)
	WAIBLINGEN_STADTMITTE = 'de:08119:5391'
	WAIBLINGEN_STADTMITTE_1 = 'de:08119:5391:0:3'  # Rtg Giselastr./Bürgerzentrum (Waiblingen Waiblingen)
	WAIBLINGEN_STADTMITTE_2 = 'de:08119:5391:0:4'  # Rtg Blumenstr. (Waiblingen Waiblingen)
	ENDERSBACH_ORTSMITTE = 'de:08119:5393'
	ENDERSBACH_ORTSMITTE_1 = 'de:08119:5393:1:3'  # Rtg. Kalkofen (Weinstadt Endersbach)
	ENDERSBACH_ORTSMITTE_2 = 'de:08119:5393:1:4'  # Rtg Endersbach Bf (gibt es nur Rück) (Weinstadt Endersbach)
	BEUTELSBACH_BENZACH = 'de:08119:5396'
	BEUTELSBACH_BENZACH_1 = 'de:08119:5396:0:3'  # Rtg Endersbach Bf (Weinstadt Beutelsbach)
	BEUTELSBACH_BENZACH_2 = 'de:08119:5396:0:4'  # Rtg Benzach Bildungszentrum (Weinstadt Beutelsbach)
	BENZACH = 'de:08119:5396'
	BENZACH_1 = 'de:08119:5396:0:3'  # Rtg Endersbach Bf (Weinstadt Beutelsbach)
	BENZACH_2 = 'de:08119:5396:0:4'  # Rtg Benzach Bildungszentrum (Weinstadt Beutelsbach)
	BEUTELSB__BENZ__BI_ZENTRUM = 'de:08119:5397:0:3'  # Rtg Endersbach Bf (Weinstadt Beutelsbach)
	BENZACH_BILDUNGSZENTRUM = 'de:08119:5397:0:3'  # Rtg Endersbach Bf (Weinstadt Beutelsbach)
	BEUTELSB__ULRICHSTRASSE = 'de:08119:5398'
	BEUTELSB__ULRICHSTRASSE_1 = 'de:08119:5398:0:3'  # Rtg Rathaus (Weinstadt Beutelsbach)
	BEUTELSB__ULRICHSTRASSE_2 = 'de:08119:5398:0:4'  # Rtg Schurwaldstr. (Weinstadt Beutelsbach)
	SCHURWALDSTRASSE = 'de:08119:5399'
	SCHURWALDSTRASSE_1 = 'de:08119:5399:0:3'  # Rtg Ulrichstr. (Weinstadt Beutelsbach)
	SCHURWALDSTRASSE_2 = 'de:08119:5399:0:4'  # Rtg Schnait Kelter (Weinstadt Beutelsbach)
	BEUTELSBACH_SCHURWALDSTRASSE = 'de:08119:5399'
	BEUTELSBACH_SCHURWALDSTRASSE_1 = 'de:08119:5399:0:3'  # Rtg Ulrichstr. (Weinstadt Beutelsbach)
	BEUTELSBACH_SCHURWALDSTRASSE_2 = 'de:08119:5399:0:4'  # Rtg Schnait Kelter (Weinstadt Beutelsbach)
	SCHNAIT_KELTER = 'de:08119:5400:0:3'  # Rtg Lützestr./Buchhaldenstr. (Weinstadt Schnait)
	SCHNAIT_LÜTZESTRASSE = 'de:08119:5401:0:3'  # eRtg Buchhaldenstraße (Weinstadt Schnait)
	LÜTZESTRASSE = 'de:08119:5401:0:3'  # eRtg Buchhaldenstraße (Weinstadt Schnait)
	SCHNAIT_SILCHERSTRASSE = 'de:08119:5402:0:3'  # eRtg Ulrichstraße (Weinstadt Schnait)
	BUCHHALDENSTRASSE = 'de:08119:5403'
	BUCHHALDENSTRASSE_1 = 'de:08119:5403:0:3'  # eRtg Wiesentalstraße (Weinstadt Schnait)
	BUCHHALDENSTRASSE_2 = 'de:08119:5403:0:4'  # Rtg Lützestr. (Weinstadt Schnait)
	SCHNAIT_BUCHHALDENSTR = 'de:08119:5403'
	SCHNAIT_BUCHHALDENSTR_1 = 'de:08119:5403:0:3'  # eRtg Wiesentalstraße (Weinstadt Schnait)
	SCHNAIT_BUCHHALDENSTR_2 = 'de:08119:5403:0:4'  # Rtg Lützestr. (Weinstadt Schnait)
	WAIBLINGEN_FREIBAD = 'de:08119:5404'
	WAIBLINGEN_FREIBAD_1 = 'de:08119:5404:0:3'  # Rtg Stettener Str./WNer Str./Mercedesstr (Waiblingen Waiblingen)
	WAIBLINGEN_FREIBAD_2 = 'de:08119:5404:0:4'  # eRtg Stadtwerke (Waiblingen Waiblingen)
	WAIBLINGEN_SALIERSTRASSE = 'de:08119:5405:1:3'  # Mikrozentrum (Waiblingen Waiblingen)
	SALIERSTRASSE = 'de:08119:5405:1:3'  # Mikrozentrum (Waiblingen Waiblingen)
	WAIBLINGEN_MIKROZENTRUM = 'de:08119:5406:0:3'  # Rtg Salierschule/Fuchsgrube (Waiblingen Waiblingen)
	MIKROZENTRUM = 'de:08119:5406:0:3'  # Rtg Salierschule/Fuchsgrube (Waiblingen Waiblingen)
	FUCHSGRUBE = 'de:08119:5407:0:3'  # Rtg Meisenweg/Nachtigallenweg (Waiblingen Waiblingen)
	WAIBLINGEN_FUCHSGRUBE = 'de:08119:5407:0:3'  # Rtg Meisenweg/Nachtigallenweg (Waiblingen Waiblingen)
	KORBER_STEIGE = 'de:08119:5408'
	KORBER_STEIGE_1 = 'de:08119:5408:1:3'  # Rtg Sämann (Waiblingen Waiblingen)
	KORBER_STEIGE_2 = 'de:08119:5408:1:4'  # Rtg Bürgerzentrum (Hallenbad) (Waiblingen Waiblingen)
	WAIBLINGEN_KORBER_STEIGE = 'de:08119:5408'
	WAIBLINGEN_KORBER_STEIGE_1 = 'de:08119:5408:1:3'  # Rtg Sämann (Waiblingen Waiblingen)
	WAIBLINGEN_KORBER_STEIGE_2 = 'de:08119:5408:1:4'  # Rtg Bürgerzentrum (Hallenbad) (Waiblingen Waiblingen)
	WAIBLINGEN_WINNENDER_STR = 'de:08119:5409'
	WAIBLINGEN_WINNENDER_STR_1 = 'de:08119:5409:0:3'  # Rtg Buchenweg/Galgenbergbrücke (Waiblingen Waiblingen)
	WAIBLINGEN_WINNENDER_STR_2 = 'de:08119:5409:0:4'  # Rtg Bürgerzentrum (Hallenbad) (Waiblingen Waiblingen)
	WINNENDER_STRASSE_1 = 'de:08119:5409:0:3'  # Rtg Buchenweg/Galgenbergbrücke (Waiblingen Waiblingen)
	WINNENDER_STRASSE_2 = 'de:08119:5409:0:4'  # Rtg Bürgerzentrum (Hallenbad) (Waiblingen Waiblingen)
	WINNENDER_STRASSE_3 = 'de:08119:5428:0:3'  # Rtg Winnenden Bf (Schwaikheim Schwaikheim)
	WINNENDER_STRASSE_4 = 'de:08119:5428:0:4'  # eRtg Rathaus (Schwaikheim Schwaikheim)
	WINNENDER_STRASSE_5 = 'de:08119:6911:0:3'  # eRtg Winnenden (Leutenbach Leutenbach)
	WINNENDER_STRASSE_6 = 'de:08119:6911:0:4'  # eRtg Rathaus (Leutenbach Leutenbach)
	WAIBLINGEN_RATHAUS = 'de:08119:5410:0:3'  # eRtg Schwanen (Waiblingen Waiblingen)
	QUERSPANGE = 'de:08119:5411:0:3'  # Rtg Hochwachtturm/Schmidener Str. (Waiblingen Waiblingen)
	WAIBLINGEN_QUERSPANGE = 'de:08119:5411:0:3'  # Rtg Hochwachtturm/Schmidener Str. (Waiblingen Waiblingen)
	IM_SÄMANN = 'de:08119:5412'
	IM_SÄMANN_1 = 'de:08119:5412:0:3'  # Rtg Nachtigallenweg (Waiblingen Waiblingen)
	IM_SÄMANN_2 = 'de:08119:5412:0:4'  # Rtg Korber Steige (Waiblingen Waiblingen)
	WAIBLINGEN_IM_SÄMANN = 'de:08119:5412'
	WAIBLINGEN_IM_SÄMANN_1 = 'de:08119:5412:0:3'  # Rtg Nachtigallenweg (Waiblingen Waiblingen)
	WAIBLINGEN_IM_SÄMANN_2 = 'de:08119:5412:0:4'  # Rtg Korber Steige (Waiblingen Waiblingen)
	WAIBLINGEN_NACHTIGALLENWEG = 'de:08119:5413'
	WAIBLINGEN_NACHTIGALLENWEG_1 = 'de:08119:5413:0:3'  # Rtg Finkenberg (Waiblingen Waiblingen)
	WAIBLINGEN_NACHTIGALLENWEG_2 = 'de:08119:5413:0:4'  # Rtg Im Sämann (Waiblingen Waiblingen)
	NACHTIGALLENWEG = 'de:08119:5413'
	NACHTIGALLENWEG_1 = 'de:08119:5413:0:3'  # Rtg Finkenberg (Waiblingen Waiblingen)
	NACHTIGALLENWEG_2 = 'de:08119:5413:0:4'  # Rtg Im Sämann (Waiblingen Waiblingen)
	ALFRED_LEIKAM_GARTEN = 'de:08119:5414'
	ALFRED_LEIKAM_GARTEN_1 = 'de:08119:5414:0:3'  # Rtg Seeplatz (Korb Korb)
	ALFRED_LEIKAM_GARTEN_2 = 'de:08119:5414:0:4'  # Rtg Bajastr. (Korb Korb)
	KORB_ALFRED_LEIKAM_GARTEN = 'de:08119:5414'
	KORB_ALFRED_LEIKAM_GARTEN_1 = 'de:08119:5414:0:3'  # Rtg Seeplatz (Korb Korb)
	KORB_ALFRED_LEIKAM_GARTEN_2 = 'de:08119:5414:0:4'  # Rtg Bajastr. (Korb Korb)
	SEEPLATZ = 'de:08119:5415'
	SEEPLATZ_1 = 'de:08119:5415:1:3'  # Rtg Steinreinach/Manslerstraße (Korb Korb)
	SEEPLATZ_2 = 'de:08119:5415:1:4'  # Rtg Alfred-Leikam-Garten (Korb Korb)
	KORB_SEEPLATZ = 'de:08119:5415'
	KORB_SEEPLATZ_1 = 'de:08119:5415:1:3'  # Rtg Steinreinach/Manslerstraße (Korb Korb)
	KORB_SEEPLATZ_2 = 'de:08119:5415:1:4'  # Rtg Alfred-Leikam-Garten (Korb Korb)
	KORB_STEINREINACH = 'de:08119:5416'
	KORB_STEINREINACH_1 = 'de:08119:5416:0:3'  # Rtg Bühl/Daimlerstr. (Korb Korb)
	KORB_STEINREINACH_2 = 'de:08119:5416:0:4'  # Rtg Seeplatz (Korb Korb)
	STEINREINACH = 'de:08119:5416'
	STEINREINACH_1 = 'de:08119:5416:0:3'  # Rtg Bühl/Daimlerstr. (Korb Korb)
	STEINREINACH_2 = 'de:08119:5416:0:4'  # Rtg Seeplatz (Korb Korb)
	MANSLER_STRASSE = 'de:08119:5417'
	MANSLER_STRASSE_1 = 'de:08119:5417:0:3'  # eRtg Schwaikheim (Korb Korb)
	MANSLER_STRASSE_2 = 'de:08119:5417:0:4'  # eRtg Seeplatz (Korb Korb)
	KORB_MANSLER_STRASSE = 'de:08119:5417'
	KORB_MANSLER_STRASSE_1 = 'de:08119:5417:0:3'  # eRtg Schwaikheim (Korb Korb)
	KORB_MANSLER_STRASSE_2 = 'de:08119:5417:0:4'  # eRtg Seeplatz (Korb Korb)
	SCHWAIKHEIM_CAFE_ROMMEL = 'de:08119:5418'
	SCHWAIKHEIM_CAFE_ROMMEL_1 = 'de:08119:5418:0:3'  # Rtg Rathaus/Bismarckstr. (Schwaikheim Schwaikheim)
	SCHWAIKHEIM_CAFE_ROMMEL_2 = 'de:08119:5418:0:4'  # Rtg Schwaikheim Bf (Schwaikheim Schwaikheim)
	CAFE_ROMMEL = 'de:08119:5418'
	CAFE_ROMMEL_1 = 'de:08119:5418:0:3'  # Rtg Rathaus/Bismarckstr. (Schwaikheim Schwaikheim)
	CAFE_ROMMEL_2 = 'de:08119:5418:0:4'  # Rtg Schwaikheim Bf (Schwaikheim Schwaikheim)
	SCHWAIKHEIM_RATHAUS = 'de:08119:5419'
	SCHWAIKHEIM_RATHAUS_1 = 'de:08119:5419:0:3'  # eRtg Winnender Straße (Schwaikheim Schwaikheim)
	SCHWAIKHEIM_RATHAUS_2 = 'de:08119:5419:0:4'  # Rtg Cafe Rommel/Bismarckstr. (Schwaikheim Schwaikheim)
	BREUNINGSWEILER_HASELSTEINSTR = 'de:08119:5422'
	BREUNINGSWEILER_HASELSTEINSTR_1 = 'de:08119:5422:0:3'  # eRtg Buocher Straße (Winnenden Breuningsweiler)
	BREUNINGSWEILER_HASELSTEINSTR_2 = 'de:08119:5422:0:4'  # eRtg Hermann-Löns-Weg (Winnenden Breuningsweiler)
	HASELSTEINSTRASSE = 'de:08119:5422'
	HASELSTEINSTRASSE_1 = 'de:08119:5422:0:3'  # eRtg Buocher Straße (Winnenden Breuningsweiler)
	HASELSTEINSTRASSE_2 = 'de:08119:5422:0:4'  # eRtg Hermann-Löns-Weg (Winnenden Breuningsweiler)
	BREUNINGSWEILER_BUOCHER_STR = 'de:08119:5423'
	BREUNINGSWEILER_BUOCHER_STR_1 = 'de:08119:5423:0:3'  # eRtg Freizeitzentrum/Lehnenberg (Winnenden Breuningsweiler)
	BREUNINGSWEILER_BUOCHER_STR_2 = 'de:08119:5423:0:4'  # eRtg Haselsteinstraße (Winnenden Breuningsweiler)
	BUOCHER_STRASSE = 'de:08119:5423'
	BUOCHER_STRASSE_1 = 'de:08119:5423:0:3'  # eRtg Freizeitzentrum/Lehnenberg (Winnenden Breuningsweiler)
	BUOCHER_STRASSE_2 = 'de:08119:5423:0:4'  # eRtg Haselsteinstraße (Winnenden Breuningsweiler)
	SCHMIDEN_STAUFERSTR = 'de:08119:5425'
	SCHMIDEN_STAUFERSTR_1 = 'de:08119:5425:0:3'  # Rtg Fellbach Bf (Fellbach Schmiden)
	SCHMIDEN_STAUFERSTR_2 = 'de:08119:5425:0:4'  # Rtg Karolingerstr. (Fellbach Schmiden)
	SCHMIDEN_KAROLINGERSTRASSE = 'de:08119:5426'
	SCHMIDEN_KAROLINGERSTRASSE_1 = 'de:08119:5426:0:3'  # Rtg Stauferstr. (Fellbach Schmiden)
	SCHMIDEN_KAROLINGERSTRASSE_2 = 'de:08119:5426:0:4'  # Rtg Gutenbergstr. (Fellbach Schmiden)
	KAROLINGERSTRASSE = 'de:08119:5426'
	KAROLINGERSTRASSE_1 = 'de:08119:5426:0:3'  # Rtg Stauferstr. (Fellbach Schmiden)
	KAROLINGERSTRASSE_2 = 'de:08119:5426:0:4'  # Rtg Gutenbergstr. (Fellbach Schmiden)
	GUTENBERGSTRASSE = 'de:08119:5427'
	GUTENBERGSTRASSE_1 = 'de:08119:5427:0:3'  # Rtg Karolingerstr. (Fellbach Schmiden)
	GUTENBERGSTRASSE_2 = 'de:08119:5427:0:4'  # Rtg Jakob-/Butterstr. (Fellbach Schmiden)
	SCHMIDEN_GUTENBERGSTR = 'de:08119:5427'
	SCHMIDEN_GUTENBERGSTR_1 = 'de:08119:5427:0:3'  # Rtg Karolingerstr. (Fellbach Schmiden)
	SCHMIDEN_GUTENBERGSTR_2 = 'de:08119:5427:0:4'  # Rtg Jakob-/Butterstr. (Fellbach Schmiden)
	SCHWAIKHEIM_WINNENDER_STR = 'de:08119:5428'
	SCHWAIKHEIM_WINNENDER_STR_1 = 'de:08119:5428:0:3'  # Rtg Winnenden Bf (Schwaikheim Schwaikheim)
	SCHWAIKHEIM_WINNENDER_STR_2 = 'de:08119:5428:0:4'  # eRtg Rathaus (Schwaikheim Schwaikheim)
	HEGNACH_KATH__KIRCHE = 'de:08119:5429:1:3'  # eRtg Aldinger Straße (Waiblingen Hegnach)
	FELLBACHER_STRASSE_1 = 'de:08119:5431:0:3'  # Rtg Karlstr./Unter den Arkaden (Kernen im Remstal Rommelshausen)
	FELLBACHER_STRASSE_2 = 'de:08119:5431:0:4'  # Rtg Fellbach Laubsängerweg (Kernen im Remstal Rommelshausen)
	FELLBACHER_STRASSE_3 = 'de:08118:5536:0:3'  # Rtg Mehrzweckhalle (Ludwigsburg Oßweil)
	FELLBACHER_STRASSE_4 = 'de:08118:5536:0:4'  # Rtg Brünner Str. (Ludwigsburg Oßweil)
	ROMMELSHAUSEN_FELLBACHER_STR = 'de:08119:5431'
	ROMMELSHAUSEN_FELLBACHER_STR_1 = 'de:08119:5431:0:3'  # Rtg Karlstr./Unter den Arkaden (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_FELLBACHER_STR_2 = 'de:08119:5431:0:4'  # Rtg Fellbach Laubsängerweg (Kernen im Remstal Rommelshausen)
	FELLBACH_ALTENHEIM = 'de:08119:5432:1:3'  # Rtg Gartenstr. (Fellbach Fellbach)
	FELLBACH_ROMMELSHAUSER_STRASSE = 'de:08119:5433'
	FELLBACH_ROMMELSHAUSER_STRASSE_1 = 'de:08119:5433:0:3'  # eRtg Fellbacher Straße (Fellbach Fellbach)
	FELLBACH_ROMMELSHAUSER_STRASSE_2 = 'de:08119:5433:0:4'  # eRtg Kappelbergstraße (Fellbach Fellbach)
	ROMMELSHAUSER_STRASSE = 'de:08119:5433'
	ROMMELSHAUSER_STRASSE_1 = 'de:08119:5433:0:3'  # eRtg Fellbacher Straße (Fellbach Fellbach)
	ROMMELSHAUSER_STRASSE_2 = 'de:08119:5433:0:4'  # eRtg Kappelbergstraße (Fellbach Fellbach)
	WAIBLINGER__SCHORNDORFER_STR = 'de:08119:5434:0:3'  # Rtg Bühlstr./R-M-C/Beim Wasserturm (Fellbach Fellbach)
	FELLB__WAIBL___SCHORND__STR = 'de:08119:5434:0:3'  # Rtg Bühlstr./R-M-C/Beim Wasserturm (Fellbach Fellbach)
	FELLB__FR__SCHILLER_GYMNASIUM = 'de:08119:5436:0:3'  # eRtg Pauluskirche (Fellbach Fellbach)
	FRIEDRICH_SCHILLER_GYMNASIUM = 'de:08119:5436:0:3'  # eRtg Pauluskirche (Fellbach Fellbach)
	KORNWESTHEIM_VILLENEUVESTRASSE = 'de:08118:5438'
	KORNWESTHEIM_VILLENEUVESTRASSE_1 = 'de:08118:5438:0:3'  # eRtg Pflugfelder Straße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_VILLENEUVESTRASSE_2 = 'de:08118:5438:0:4'  # eRtg Maybachstraße (Kornwestheim Kornwestheim)
	VILLENEUVESTRASSE = 'de:08118:5438'
	VILLENEUVESTRASSE_1 = 'de:08118:5438:0:3'  # eRtg Pflugfelder Straße (Kornwestheim Kornwestheim)
	VILLENEUVESTRASSE_2 = 'de:08118:5438:0:4'  # eRtg Maybachstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_ZEPPELINSTRASSE = 'de:08118:5439'
	KORNWESTHEIM_ZEPPELINSTRASSE_1 = 'de:08118:5439:0:3'  # Rtg Im Weizen (Kornwestheim Kornwestheim)
	KORNWESTHEIM_ZEPPELINSTRASSE_2 = 'de:08118:5439:0:4'  # Rtg Johannesstr. (Kornwestheim Kornwestheim)
	KORNWESTHEIM_HALLENBAD = 'de:08118:5440'
	KORNWESTHEIM_HALLENBAD_1 = 'de:08118:5440:0:3'  # eRtg Realschule (Kornwestheim Kornwestheim)
	KORNWESTHEIM_HALLENBAD_2 = 'de:08118:5440:0:4'  # eRtg Im Weizen (Kornwestheim Kornwestheim)
	KORNWESTHEIM_REALSCHULE = 'de:08118:5441'
	KORNWESTHEIM_REALSCHULE_1 = 'de:08118:5441:0:3'  # eRtg Stauffenbergstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_REALSCHULE_2 = 'de:08118:5441:0:4'  # eRtg Hallenbad (Kornwestheim Kornwestheim)
	KORNWESTHEIM_STAUFFENBERGSTR = 'de:08118:5442'
	KORNWESTHEIM_STAUFFENBERGSTR_1 = 'de:08118:5442:0:3'  # eRtg Goerdelerstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_STAUFFENBERGSTR_2 = 'de:08118:5442:0:4'  # eRtg Realschule (Kornwestheim Kornwestheim)
	KORNWESTHEIM_GOERDELERSTRASSE = 'de:08118:5443'
	KORNWESTHEIM_GOERDELERSTRASSE_1 = 'de:08118:5443:0:3'  # eRtg Neckarstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_GOERDELERSTRASSE_2 = 'de:08118:5443:0:4'  # eRtg Stauffenbergstraße (Kornwestheim Kornwestheim)
	GOERDELERSTRASSE = 'de:08118:5443'
	GOERDELERSTRASSE_1 = 'de:08118:5443:0:3'  # eRtg Neckarstraße (Kornwestheim Kornwestheim)
	GOERDELERSTRASSE_2 = 'de:08118:5443:0:4'  # eRtg Stauffenbergstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_NECKARSTRASSE = 'de:08118:5444'
	KORNWESTHEIM_NECKARSTRASSE_1 = 'de:08118:5444:0:3'  # eRtg Enzstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_NECKARSTRASSE_2 = 'de:08118:5444:0:4'  # eRtg Goerdelerstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_ENZSTRASSE = 'de:08118:5445'
	KORNWESTHEIM_ENZSTRASSE_1 = 'de:08118:5445:0:3'  # eRtg Talstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_ENZSTRASSE_2 = 'de:08118:5445:0:4'  # eRtg Neckarstraße (Kornwestheim Kornwestheim)
	ENZSTRASSE_1 = 'de:08118:5445:0:3'  # eRtg Talstraße (Kornwestheim Kornwestheim)
	ENZSTRASSE_2 = 'de:08118:5445:0:4'  # eRtg Neckarstraße (Kornwestheim Kornwestheim)
	ENZSTRASSE_3 = 'de:08118:7477:0:3'  # Rtg Am Türmle (Bietigheim-Bissingen Untermberg)
	ENZSTRASSE_4 = 'de:08118:7477:0:4'  # Rtg Bissinger Str. (Bietigheim-Bissingen Untermberg)
	KORNWESTHEIM_ALDINGER_STRASSE = 'de:08118:5446'
	KORNWESTHEIM_ALDINGER_STRASSE_1 = 'de:08118:5446:0:3'  # Rtg Talstr. (Kornwestheim Kornwestheim)
	KORNWESTHEIM_ALDINGER_STRASSE_2 = 'de:08118:5446:0:4'  # Rtg Lenzhalde (Kornwestheim Kornwestheim)
	ALDINGER_STRASSE_1 = 'de:08118:5446:0:3'  # Rtg Talstr. (Kornwestheim Kornwestheim)
	ALDINGER_STRASSE_2 = 'de:08118:5446:0:4'  # Rtg Lenzhalde (Kornwestheim Kornwestheim)
	ALDINGER_STRASSE_3 = 'de:08118:5522:0:3'  # Rtg Am Sonnenberg (Ludwigsburg Ludwigsburg)
	ALDINGER_STRASSE_4 = 'de:08118:5522:0:4'  # Rtg R.-Franck-Str./Rotbäumlesfeld (Ludwigsburg Ludwigsburg)
	ALDINGER_STRASSE_5 = 'de:08119:5546:0:4'  # eRtg Neckarstraße/LB (Waiblingen Hegnach)
	KORNWESTHEIM_FRIEDHOF = 'de:08118:5447:0:3'  # eRtg Lenzhalde (Kornwestheim Kornwestheim)
	KORNWESTHEIM_KIRCHTALSTRASSE = 'de:08118:5448:0:3'  # Rtg Im Wiesengrund (Kornwestheim Kornwestheim)
	KORNWESTHEIM_IM_WIESENGRUND = 'de:08118:5450:0:3'  # eRtg Mühlhäuser Straße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_MÜHLHÄUSER_STRASSE = 'de:08118:5451:0:3'  # eRtg Im Weizen (Kornwestheim Kornwestheim)
	MÜHLHÄUSER_STRASSE_1 = 'de:08118:5451:0:3'  # eRtg Im Weizen (Kornwestheim Kornwestheim)
	MÜHLHÄUSER_STRASSE_2 = 'de:08118:6850:0:3'  # Rtg Sporthalle (Vaihingen an der Enz Roßwag)
	MÜHLHÄUSER_STRASSE_3 = 'de:08118:7556:1:3'  # Rtg Friesenstraße (Ludwigsburg Oßweil)
	KORNWESTHEIM_BOLZSTRASSE = 'de:08118:5452'
	KORNWESTHEIM_BOLZSTRASSE_1 = 'de:08118:5452:0:3'  # eRtg Max-Planck-Straße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_BOLZSTRASSE_2 = 'de:08118:5452:0:4'  # eRtg Lindenstraße (Kornwestheim Kornwestheim)
	BOLZSTRASSE_1 = 'de:08118:5452:0:3'  # eRtg Max-Planck-Straße (Kornwestheim Kornwestheim)
	BOLZSTRASSE_2 = 'de:08118:5452:0:4'  # eRtg Lindenstraße (Kornwestheim Kornwestheim)
	BOLZSTRASSE_3 = 'de:08118:5658:0:3'  # Rtg KH (Bietigheim-Bissingen Bietigheim)
	BOLZSTRASSE_4 = 'de:08118:5658:0:4'  # Rtg Schillerplatz (Bietigheim-Bissingen Bietigheim)
	REMSTALSTRASSE = 'de:08118:5453'
	REMSTALSTRASSE_1 = 'de:08118:5453:0:3'  # eRtg Hauptstraße (Remseck am Neckar Neckarrems)
	REMSTALSTRASSE_2 = 'de:08118:5453:0:4'  # eRtg Rathaus (Remseck am Neckar Neckarrems)
	NECKARREMS_REMSTALSTR = 'de:08118:5453'
	NECKARREMS_REMSTALSTR_1 = 'de:08118:5453:0:3'  # eRtg Hauptstraße (Remseck am Neckar Neckarrems)
	NECKARREMS_REMSTALSTR_2 = 'de:08118:5453:0:4'  # eRtg Rathaus (Remseck am Neckar Neckarrems)
	RESIDENZSCHLOSS = 'de:08118:5454'
	RESIDENZSCHLOSS_1 = 'de:08118:5454:0:3'  # Rtg Favorite (Ludwigsburg Ludwigsburg)
	RESIDENZSCHLOSS_2 = 'de:08118:5454:0:4'  # Rtg Rathaus (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_RESIDENZSCHLOSS = 'de:08118:5454'
	LUDWIGSBURG_RESIDENZSCHLOSS_1 = 'de:08118:5454:0:3'  # Rtg Favorite (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_RESIDENZSCHLOSS_2 = 'de:08118:5454:0:4'  # Rtg Rathaus (Ludwigsburg Ludwigsburg)
	KÖHLSTRASSE = 'de:08118:5455'
	KÖHLSTRASSE_1 = 'de:08118:5455:0:3'  # eRtg Max-Eyth-Straße (Ludwigsburg Ludwigsburg)
	KÖHLSTRASSE_2 = 'de:08118:5455:0:4'  # eRtg Solitudeallee (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_KÖHLSTRASSE = 'de:08118:5455'
	LUDWIGSBURG_KÖHLSTRASSE_1 = 'de:08118:5455:0:3'  # eRtg Max-Eyth-Straße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_KÖHLSTRASSE_2 = 'de:08118:5455:0:4'  # eRtg Solitudeallee (Ludwigsburg Ludwigsburg)
	KORNWESTHEIM_MAX_EYTH_STRASSE = 'de:08118:5456'
	KORNWESTHEIM_MAX_EYTH_STRASSE_1 = 'de:08118:5456:0:3'  # Rtg Ludwigsburg (Kornwestheim Kornwestheim)
	KORNWESTHEIM_MAX_EYTH_STRASSE_2 = 'de:08118:5456:0:4'  # Rtg Maybachstr. (Kornwestheim Kornwestheim)
	KARLSRUHER_ALLEE = 'de:08118:5457:0:3'  # eRtg Sportplatz (Ludwigsburg Pflugfelden)
	PFLUGFELDEN_KARLSRUHER_ALLEE = 'de:08118:5457:0:3'  # eRtg Sportplatz (Ludwigsburg Pflugfelden)
	KORNWESTH__LUDWIGSBURGER_STR = 'de:08118:5459'
	KORNWESTH__LUDWIGSBURGER_STR_1 = 'de:08118:5459:0:3'  # Rtg Pflugfelder Str. (Kornwestheim Kornwestheim)
	KORNWESTH__LUDWIGSBURGER_STR_2 = 'de:08118:5459:0:4'  # Rtg An der Wette (Kornwestheim Kornwestheim)
	KORNWESTHEIM_AN_DER_WETTE = 'de:08118:5460'
	KORNWESTHEIM_AN_DER_WETTE_1 = 'de:08118:5460:1:3'  # eRtg Johannesstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_AN_DER_WETTE_2 = 'de:08118:5460:1:4'  # eRtg Ludwigsburger Str./Im Wiesengrund (Kornwestheim Kornwestheim)
	AN_DER_WETTE = 'de:08118:5460'
	AN_DER_WETTE_1 = 'de:08118:5460:1:3'  # eRtg Johannesstraße (Kornwestheim Kornwestheim)
	AN_DER_WETTE_2 = 'de:08118:5460:1:4'  # eRtg Ludwigsburger Str./Im Wiesengrund (Kornwestheim Kornwestheim)
	KORNWESTHEIM_JOHANNESSTRASSE = 'de:08118:5461'
	KORNWESTHEIM_JOHANNESSTRASSE_1 = 'de:08118:5461:0:3'  # eRtg Zeppelinstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_JOHANNESSTRASSE_2 = 'de:08118:5461:0:4'  # eRtg Kornwestheim (Kornwestheim Kornwestheim)
	KORNWESTHEIM_RATHAUS = 'de:08118:5462:0:3'  # eRtg Johannesstraße (Kornwestheim Kornwestheim)
	RECHBERGSTRASSE_1 = 'de:08118:5463:0:4'  # eRtg Ernst-Sigle-Gymnasium (Kornwestheim Kornwestheim)
	RECHBERGSTRASSE_2 = 'de:08136:7038:0:3'  # Rtg. Weitmars Talstr. (Lorch Weitmars)
	KORNWESTHEIM_RECHBERGSTRASSE = 'de:08118:5463:0:4'  # eRtg Ernst-Sigle-Gymnasium (Kornwestheim Kornwestheim)
	KORNWESTHEIM_LUDWIG_HERR_STR = 'de:08118:5464:0:3'  # eRtg Rechbergstraße (Kornwestheim Kornwestheim)
	LUDWIG_HERR_STRASSE = 'de:08118:5464:0:3'  # eRtg Rechbergstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_HORNBERGSTRASSE = 'de:08118:5465:0:3'  # eRtg Ludwig-Herr-Straße (Kornwestheim Kornwestheim)
	HORNBERGSTRASSE = 'de:08118:5465:0:3'  # eRtg Ludwig-Herr-Straße (Kornwestheim Kornwestheim)
	LUDWIGSBURG_STADTWERKE = 'de:08118:5466'
	LUDWIGSBURG_STADTWERKE_1 = 'de:08118:5466:0:3'  # eRtg Gänsfußallee (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_STADTWERKE_2 = 'de:08118:5466:0:4'  # eRtg ZOB (Ludwigsburg Ludwigsburg)
	KORNWESTHEIM_LINDENSTRASSE = 'de:08118:5467'
	KORNWESTHEIM_LINDENSTRASSE_1 = 'de:08118:5467:0:3'  # eRtg Sonatenweg (Kornwestheim Kornwestheim)
	KORNWESTHEIM_LINDENSTRASSE_2 = 'de:08118:5467:0:4'  # eRtg Kornwestheim (Kornwestheim Kornwestheim)
	KORNWESTHEIM_MAYBACHSTRASSE = 'de:08118:5468'
	KORNWESTHEIM_MAYBACHSTRASSE_1 = 'de:08118:5468:0:3'  # eRtg Villeneuvestraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_MAYBACHSTRASSE_2 = 'de:08118:5468:0:4'  # eRtg Max-Eyth-Straße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_STADION = 'de:08118:5469:0:3'  # eRtg Eichenweg (Kornwestheim Kornwestheim)
	EICHENWEG_1 = 'de:08118:5470:0:3'  # eRtg Albstraße (Kornwestheim Kornwestheim)
	EICHENWEG_2 = 'de:08115:7066:0:3'  # eRtg Im Steingraben (Herrenberg Herrenberg)
	EICHENWEG_3 = 'de:08118:7426:0:3'  # eRtg Lindenstraße (Bietigheim-Bissingen Bissingen (LB))
	EICHENWEG_4 = 'de:08118:7426:0:4'  # eRtg Entenäcker (Bietigheim-Bissingen Bissingen (LB))
	KORNWESTHEIM_EICHENWEG = 'de:08118:5470:0:3'  # eRtg Albstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_ALBSTRASSE = 'de:08118:5471:0:3'  # eRtg Heubergstraße (Kornwestheim Kornwestheim)
	RUHRSTRASSE = 'de:08118:5472'
	RUHRSTRASSE_1 = 'de:08118:5472:0:5'  # Rtg Hohenzollernplatz (Ludwigsburg Ludwigsburg)
	RUHRSTRASSE_2 = 'de:08118:5472:0:6'  # Rtg Friedenstr. 5 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_RUHRSTRASSE = 'de:08118:5472'
	LUDWIGSBURG_RUHRSTRASSE_1 = 'de:08118:5472:0:5'  # Rtg Hohenzollernplatz (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_RUHRSTRASSE_2 = 'de:08118:5472:0:6'  # Rtg Friedenstr. 5 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_GROENERSTR = 'de:08118:5474:0:3'  # eRtg Hoferstraße (Ludwigsburg Ludwigsburg)
	GROENERSTRASSE = 'de:08118:5474:0:3'  # eRtg Hoferstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_MANN___HUMMEL = 'de:08118:5475:0:3'  # Rtg Grönerstr. (Ludwigsburg Ludwigsburg)
	MANN___HUMMEL = 'de:08118:5475:0:3'  # Rtg Grönerstr. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_MÖRIKESTR = 'de:08118:5476'
	LUDWIGSBURG_MÖRIKESTR_1 = 'de:08118:5476:0:3'  # eRtg Osterholz (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_MÖRIKESTR_2 = 'de:08118:5476:0:4'  # eRtg Im Waldeck (Ludwigsburg Ludwigsburg)
	BUCH_BERLINER_STRASSE = 'de:08118:5477'
	BUCH_BERLINER_STRASSE_1 = 'de:08118:5477:1:3'  # eRtg Buch Sucystraße (Bietigheim-Bissingen Bietigheim)
	BUCH_BERLINER_STRASSE_2 = 'de:08118:5477:1:4'  # eRtg Breuningerland (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_BUCH_BERLINER_STR = 'de:08118:5477'
	BIETIGHEIM_BUCH_BERLINER_STR_1 = 'de:08118:5477:1:3'  # eRtg Buch Sucystraße (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_BUCH_BERLINER_STR_2 = 'de:08118:5477:1:4'  # eRtg Breuningerland (Bietigheim-Bissingen Bietigheim)
	ASPERG_NECKARSTRASSE = 'de:08118:5478'
	ASPERG_NECKARSTRASSE_1 = 'de:08118:5478:0:3'  # eRtg Saarstraße (Asperg Asperg)
	ASPERG_NECKARSTRASSE_2 = 'de:08118:5478:0:4'  # eRtg Im Waldeck (Asperg Asperg)
	ASPERG_VOLKSBANK = 'de:08118:5479:0:3'  # eRtg Stuttgarter Straße (Asperg Asperg)
	ASPERG_MARKGRÖNINGER_STR = 'de:08118:5481'
	ASPERG_MARKGRÖNINGER_STR_1 = 'de:08118:5481:0:3'  # eRtg Behindertenheim (Asperg Asperg)
	ASPERG_MARKGRÖNINGER_STR_2 = 'de:08118:5481:0:4'  # eRtg Weinstraße (Asperg Asperg)
	OBERE_MARBACHER_STRASSE = 'de:08118:5483'
	OBERE_MARBACHER_STRASSE_1 = 'de:08118:5483:0:3'  # Rtg  Untere Marbacher Str. (Ludwigsburg Ludwigsburg)
	OBERE_MARBACHER_STRASSE_2 = 'de:08118:5483:0:4'  # Rtg Schloß Favorite (Ludwigsburg Ludwigsburg)
	LUDW__OBERE_MARBACHER_STR = 'de:08118:5483'
	LUDW__OBERE_MARBACHER_STR_1 = 'de:08118:5483:0:3'  # Rtg  Untere Marbacher Str. (Ludwigsburg Ludwigsburg)
	LUDW__OBERE_MARBACHER_STR_2 = 'de:08118:5483:0:4'  # Rtg Schloß Favorite (Ludwigsburg Ludwigsburg)
	LUDW__UNTERE_MARBACHER_STR = 'de:08118:5484'
	LUDW__UNTERE_MARBACHER_STR_1 = 'de:08118:5484:0:3'  # Rtg Neckarbrücke (Ludwigsburg Ludwigsburg)
	LUDW__UNTERE_MARBACHER_STR_2 = 'de:08118:5484:0:4'  # Rtg Obere Marbacher Str. (Ludwigsburg Ludwigsburg)
	UNTERE_MARBACHER_STRASSE = 'de:08118:5484'
	UNTERE_MARBACHER_STRASSE_1 = 'de:08118:5484:0:3'  # Rtg Neckarbrücke (Ludwigsburg Ludwigsburg)
	UNTERE_MARBACHER_STRASSE_2 = 'de:08118:5484:0:4'  # Rtg Obere Marbacher Str. (Ludwigsburg Ludwigsburg)
	KORNWESTHEIM_KARL_JOOS_STRASSE = 'de:08118:5485:0:3'  # eRtg Hauffstraße (Kornwestheim Kornwestheim)
	KARL_JOOS_STRASSE = 'de:08118:5485:0:3'  # eRtg Hauffstraße (Kornwestheim Kornwestheim)
	NECKARWEIH__HAUPTSTR__99 = 'de:08118:5486'
	NECKARWEIH__HAUPTSTR__99_1 = 'de:08118:5486:0:3'  # nach Nord (Ludwigsburg Neckarweihingen)
	NECKARWEIH__HAUPTSTR__99_2 = 'de:08118:5486:0:4'  # nach Süd (Ludwigsburg Neckarweihingen)
	HAUPTSTRASSE_99 = 'de:08118:5486'
	HAUPTSTRASSE_99_1 = 'de:08118:5486:0:3'  # nach Nord (Ludwigsburg Neckarweihingen)
	HAUPTSTRASSE_99_2 = 'de:08118:5486:0:4'  # nach Süd (Ludwigsburg Neckarweihingen)
	HOHENRAINSTRASSE = 'de:08118:5487:0:3'  # eRtg Wendeplatte (Ludwigsburg Neckarweihingen)
	NECKARWEIH__HOHENRAINSTR = 'de:08118:5487:0:3'  # eRtg Wendeplatte (Ludwigsburg Neckarweihingen)
	LUDW__FRIEDENSTRASSE_5 = 'de:08118:5488'
	LUDW__FRIEDENSTRASSE_5_1 = 'de:08118:5488:0:3'  # Rtg Solitudeallee (Ludwigsburg Ludwigsburg)
	LUDW__FRIEDENSTRASSE_5_2 = 'de:08118:5488:0:4'  # Rtg Friedensstr. 52 (Ludwigsburg Ludwigsburg)
	FRIEDENSTRASSE_5 = 'de:08118:5488'
	FRIEDENSTRASSE_5_1 = 'de:08118:5488:0:3'  # Rtg Solitudeallee (Ludwigsburg Ludwigsburg)
	FRIEDENSTRASSE_5_2 = 'de:08118:5488:0:4'  # Rtg Friedensstr. 52 (Ludwigsburg Ludwigsburg)
	FRIEDENSTRASSE_52 = 'de:08118:5489'
	FRIEDENSTRASSE_52_1 = 'de:08118:5489:0:3'  # nach West (Ludwigsburg Ludwigsburg)
	FRIEDENSTRASSE_52_2 = 'de:08118:5489:0:4'  # nach Ost (Ludwigsburg Ludwigsburg)
	LUDW__FRIEDENSTRASSE_52 = 'de:08118:5489'
	LUDW__FRIEDENSTRASSE_52_1 = 'de:08118:5489:0:3'  # nach West (Ludwigsburg Ludwigsburg)
	LUDW__FRIEDENSTRASSE_52_2 = 'de:08118:5489:0:4'  # nach Ost (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_DAIMLERSTR = 'de:08118:5490'
	LUDWIGSBURG_DAIMLERSTR_1 = 'de:08118:5490:0:3'  # eRtg Friedenstraße 52 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_DAIMLERSTR_2 = 'de:08118:5490:0:4'  # eRtg Steinbeisstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_STEINBEISSTR = 'de:08118:5491'
	LUDWIGSBURG_STEINBEISSTR_1 = 'de:08118:5491:0:3'  # eRtg Daimlerstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_STEINBEISSTR_2 = 'de:08118:5491:0:4'  # eRtg Robert-Bosch-Straße (Ludwigsburg Ludwigsburg)
	PFLUGFELDEN_ROBERT_BOSCH_STR = 'de:08118:5492'
	PFLUGFELDEN_ROBERT_BOSCH_STR_1 = 'de:08118:5492:0:3'  # eRtg Steinbeisstraße (Ludwigsburg Pflugfelden)
	PFLUGFELDEN_ROBERT_BOSCH_STR_2 = 'de:08118:5492:0:4'  # eRtg Möglinger Straße (Ludwigsburg Pflugfelden)
	PFLUGFELDEN_SPORTPLATZ = 'de:08118:5494:1:3'  # eRtg Herrschaftsweg (Ludwigsburg Pflugfelden)
	HERRSCHAFTSWEG = 'de:08118:5495:0:3'  # eRtg Dorfstraße (Ludwigsburg Pflugfelden)
	PFLUGF__HERRSCHAFTSWEG = 'de:08118:5495:0:3'  # eRtg Dorfstraße (Ludwigsburg Pflugfelden)
	PFLUGFELDEN_DORFSTRASSE = 'de:08118:5496:0:3'  # eRtg Robert-Bosch-Straße (Ludwigsburg Pflugfelden)
	LUDW__BLÜHENDES_BAROCK = 'de:08118:5497'
	LUDW__BLÜHENDES_BAROCK_1 = 'de:08118:5497:0:3'  # eRtg Fasanenstr./Schorndorfer Tor (Ludwigsburg Ludwigsburg)
	LUDW__BLÜHENDES_BAROCK_2 = 'de:08118:5497:0:4'  # eRtg Rathaus (Ludwigsburg Ludwigsburg)
	BLÜHENDES_BAROCK = 'de:08118:5497'
	BLÜHENDES_BAROCK_1 = 'de:08118:5497:0:3'  # eRtg Fasanenstr./Schorndorfer Tor (Ludwigsburg Ludwigsburg)
	BLÜHENDES_BAROCK_2 = 'de:08118:5497:0:4'  # eRtg Rathaus (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_KLINIKUM = 'de:08118:5498'
	LUDWIGSBURG_KLINIKUM_1 = 'de:08118:5498:0:3'  # Rtg Friedhof (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_KLINIKUM_2 = 'de:08118:5498:0:4'  # Rtg Schorndorfer Tor (Ludwigsburg Ludwigsburg)
	KLINIKUM = 'de:08118:5498'
	KLINIKUM_1 = 'de:08118:5498:0:3'  # Rtg Friedhof (Ludwigsburg Ludwigsburg)
	KLINIKUM_2 = 'de:08118:5498:0:4'  # Rtg Schorndorfer Tor (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_NEUER_FRIEDHOF = 'de:08118:5499'
	LUDWIGSBURG_NEUER_FRIEDHOF_1 = 'de:08118:5499:0:3'  # eRtg Harteneckstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_NEUER_FRIEDHOF_2 = 'de:08118:5499:0:4'  # eRtg Klinikum (Ludwigsburg Ludwigsburg)
	NEUER_FRIEDHOF_1 = 'de:08118:5499:0:3'  # eRtg Harteneckstraße (Ludwigsburg Ludwigsburg)
	NEUER_FRIEDHOF_2 = 'de:08118:5499:0:4'  # eRtg Klinikum (Ludwigsburg Ludwigsburg)
	NEUER_FRIEDHOF_3 = 'de:08119:5975:0:3'  # eRtg Auerhahnstraße (Schorndorf Schorndorf)
	NEUER_FRIEDHOF_4 = 'de:08118:7422:0:3'  # Ankunft (Bietigheim-Bissingen Bissingen (LB))
	NEUER_FRIEDHOF_5 = 'de:08118:7422:0:4'  # eRtg Zahnklinik (Bietigheim-Bissingen Bissingen (LB))
	LUDWIGSBURG_HARTENECKSTR = 'de:08118:5500'
	LUDWIGSBURG_HARTENECKSTR_1 = 'de:08118:5500:0:3'  # Rtg Kornbeckstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_HARTENECKSTR_2 = 'de:08118:5500:0:4'  # Rtg Neuer Friedhof (Ludwigsburg Ludwigsburg)
	KORNBECKSTRASSE = 'de:08118:5501'
	KORNBECKSTRASSE_1 = 'de:08118:5501:0:3'  # eRtg Schlösslesfeld (Ludwigsburg Ludwigsburg)
	KORNBECKSTRASSE_2 = 'de:08118:5501:0:4'  # eRtg Mainzer Allee (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_KORNBECKSTR = 'de:08118:5501'
	LUDWIGSBURG_KORNBECKSTR_1 = 'de:08118:5501:0:3'  # eRtg Schlösslesfeld (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_KORNBECKSTR_2 = 'de:08118:5501:0:4'  # eRtg Mainzer Allee (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_SCHLÖSSLESFELD = 'de:08118:5502:0:4'  # eRtg Kornbeckstraße (Ludwigsburg Ludwigsburg)
	SCHLÖSSLESFELD = 'de:08118:5502:0:4'  # eRtg Kornbeckstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_ABELSTRASSE = 'de:08118:5503'
	LUDWIGSBURG_ABELSTRASSE_1 = 'de:08118:5503:0:3'  # Rtg Talstr. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_ABELSTRASSE_2 = 'de:08118:5503:0:4'  # Rtg Uhlandstr. (Ludwigsburg Ludwigsburg)
	ABELSTRASSE = 'de:08118:5503'
	ABELSTRASSE_1 = 'de:08118:5503:0:3'  # Rtg Talstr. (Ludwigsburg Ludwigsburg)
	ABELSTRASSE_2 = 'de:08118:5503:0:4'  # Rtg Uhlandstr. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_TALSTRASSE = 'de:08118:5504'
	LUDWIGSBURG_TALSTRASSE_1 = 'de:08118:5504:0:3'  # Rtg Bietigheimer Str. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_TALSTRASSE_2 = 'de:08118:5504:0:4'  # Rtg Abelstr. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_TALSTRASSE_3 = 'de:08118:5504:0:5'  # Rtg Heilbronner Str. (Ludwigsburg Ludwigsburg)
	BIETIGHEIMER_STRASSE_1 = 'de:08118:5505:0:4'  # Rtg Talstr. (Ludwigsburg Ludwigsburg)
	BIETIGHEIMER_STRASSE_2 = 'de:08118:5566:1:4'  # Rtg Poststrässle (Ingersheim Großingersheim)
	BIETIGHEIMER_STRASSE_3 = 'de:08118:5566:2:3'  # Rtg Tiefengässle (Ingersheim Großingersheim)
	BIETIGHEIMER_STRASSE_4 = 'de:08118:5669:0:3'  # eRtg Neuweiler Straße (Sachsenheim Kleinsachsenheim)
	BIETIGHEIMER_STRASSE_5 = 'de:08118:5669:0:4'  # eRtg Zeil (Sachsenheim Kleinsachsenheim)
	BIETIGHEIMER_STRASSE_6 = 'de:08118:6719:0:3'  # eRtg Helenenburgweg (Bietigheim-Bissingen Metterzimmern)
	BIETIGHEIMER_STRASSE_7 = 'de:08118:6999:0:3'  # eRtg Ortsmitte (Murr Murr)
	LUDW__BIETIGHEIMER_STRASSE = 'de:08118:5505:0:4'  # Rtg Talstr. (Ludwigsburg Ludwigsburg)
	SCHLOSS_FAVORITE = 'de:08118:5506'
	SCHLOSS_FAVORITE_1 = 'de:08118:5506:0:3'  # Rtg Residenzschloss (Ludwigsburg Ludwigsburg)
	SCHLOSS_FAVORITE_2 = 'de:08118:5506:0:4'  # Rtg Reichertshalde/Obere Marbacher Str. (Ludwigsburg Ludwigsburg)
	LUDW__SCHLOSS_FAVORITE = 'de:08118:5506'
	LUDW__SCHLOSS_FAVORITE_1 = 'de:08118:5506:0:3'  # Rtg Residenzschloss (Ludwigsburg Ludwigsburg)
	LUDW__SCHLOSS_FAVORITE_2 = 'de:08118:5506:0:4'  # Rtg Reichertshalde/Obere Marbacher Str. (Ludwigsburg Ludwigsburg)
	EGLOSHEIM_ROSENACKERWEG = 'de:08118:5507'
	EGLOSHEIM_ROSENACKERWEG_1 = 'de:08118:5507:0:3'  # Rtg Golfplatz/Straßenäcker (Ludwigsburg Eglosheim)
	EGLOSHEIM_ROSENACKERWEG_2 = 'de:08118:5507:0:4'  # Rtg Päd. Hochschule/ZOB (Ludwigsburg Eglosheim)
	ROSENACKERWEG = 'de:08118:5507'
	ROSENACKERWEG_1 = 'de:08118:5507:0:3'  # Rtg Golfplatz/Straßenäcker (Ludwigsburg Eglosheim)
	ROSENACKERWEG_2 = 'de:08118:5507:0:4'  # Rtg Päd. Hochschule/ZOB (Ludwigsburg Eglosheim)
	EGLOSHEIM_GOLFPLATZ = 'de:08118:5508'
	EGLOSHEIM_GOLFPLATZ_1 = 'de:08118:5508:0:3'  # Rtg Ikea (Ludwigsburg Eglosheim)
	EGLOSHEIM_GOLFPLATZ_2 = 'de:08118:5508:0:4'  # Rtg Ludwigsburg (Ludwigsburg Eglosheim)
	LUDWIGSBURG_MARTIN_LUTHER_STR = 'de:08118:5510'
	LUDWIGSBURG_MARTIN_LUTHER_STR_1 = 'de:08118:5510:0:3'  # Rtg ZOB (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_MARTIN_LUTHER_STR_2 = 'de:08118:5510:0:4'  # Rtg Kurfürstenstraße/Albrechstr. (Ludwigsburg Ludwigsburg)
	KATHARINENSTRASSE_1 = 'de:08118:5511:1:3'  # eRtg Tammer Straße (Ludwigsburg Eglosheim)
	KATHARINENSTRASSE_2 = 'de:08118:6976:0:3'  # eRtg Hildenstraße (Vaihingen an der Enz Gündelbach)
	EGLOSHEIM_KATHARINENSTR = 'de:08118:5511:1:3'  # eRtg Tammer Straße (Ludwigsburg Eglosheim)
	EGLOSHEIM_TAMMER_STRASSE = 'de:08118:5512'
	EGLOSHEIM_TAMMER_STRASSE_1 = 'de:08118:5512:0:3'  # eRtg Frankfurter Straße (Ludwigsburg Eglosheim)
	EGLOSHEIM_TAMMER_STRASSE_2 = 'de:08118:5512:0:4'  # eRtg Falkenweg (Ludwigsburg Eglosheim)
	EGLOSHEIM_FALKENWEG = 'de:08118:5513:1:3'  # eRtg Hirschbergstraße (Ludwigsburg Eglosheim)
	FALKENWEG = 'de:08118:5513:1:3'  # eRtg Hirschbergstraße (Ludwigsburg Eglosheim)
	LUDWIGSBURG_NECKARSTR = 'de:08118:5514'
	LUDWIGSBURG_NECKARSTR_1 = 'de:08118:5514:0:3'  # eRtg Comburgstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_NECKARSTR_2 = 'de:08118:5514:0:4'  # eRtg Schorndorfer Tor (Ludwigsburg Ludwigsburg)
	FRIESENSTRASSE = 'de:08118:5515'
	FRIESENSTRASSE_1 = 'de:08118:5515:0:3'  # eRtg Westfalenstraße (Ludwigsburg Oßweil)
	FRIESENSTRASSE_2 = 'de:08118:5515:0:4'  # eRtg Comburgstraße (Ludwigsburg Oßweil)
	OSSWEIL_FRIESENSTRASSE = 'de:08118:5515'
	OSSWEIL_FRIESENSTRASSE_1 = 'de:08118:5515:0:3'  # eRtg Westfalenstraße (Ludwigsburg Oßweil)
	OSSWEIL_FRIESENSTRASSE_2 = 'de:08118:5515:0:4'  # eRtg Comburgstraße (Ludwigsburg Oßweil)
	WETTEMARKT = 'de:08118:5516:0:3'  # eRtg Friesenstraße (Ludwigsburg Oßweil)
	OSSWEIL_WETTEMARKT = 'de:08118:5516:0:3'  # eRtg Friesenstraße (Ludwigsburg Oßweil)
	LUDW__HOHENZOLLERNPLATZ = 'de:08118:5517'
	LUDW__HOHENZOLLERNPLATZ_1 = 'de:08118:5517:0:3'  # eRtg Ludwigsburg (Ludwigsburg Ludwigsburg)
	LUDW__HOHENZOLLERNPLATZ_2 = 'de:08118:5517:0:4'  # eRtg Tennisplatz (Ludwigsburg Ludwigsburg)
	HOHENZOLLERNPLATZ = 'de:08118:5517'
	HOHENZOLLERNPLATZ_1 = 'de:08118:5517:0:3'  # eRtg Ludwigsburg (Ludwigsburg Ludwigsburg)
	HOHENZOLLERNPLATZ_2 = 'de:08118:5517:0:4'  # eRtg Tennisplatz (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_WÜSTENROT = 'de:08118:5518'
	LUDWIGSBURG_WÜSTENROT_1 = 'de:08118:5518:0:3'  # Rtg Karlshöhe (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_WÜSTENROT_2 = 'de:08118:5518:0:4'  # Rtg Hohenzollernplatz (Ludwigsburg Ludwigsburg)
	WÜSTENROT = 'de:08118:5518'
	WÜSTENROT_1 = 'de:08118:5518:0:3'  # Rtg Karlshöhe (Ludwigsburg Ludwigsburg)
	WÜSTENROT_2 = 'de:08118:5518:0:4'  # Rtg Hohenzollernplatz (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_KARLSHÖHE = 'de:08118:5519'
	LUDWIGSBURG_KARLSHÖHE_1 = 'de:08118:5519:0:3'  # eRtg Wüstenrot (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_KARLSHÖHE_2 = 'de:08118:5519:0:4'  # eRtg Tennisplatz (Ludwigsburg Ludwigsburg)
	KARLSHÖHE = 'de:08118:5519'
	KARLSHÖHE_1 = 'de:08118:5519:0:3'  # eRtg Wüstenrot (Ludwigsburg Ludwigsburg)
	KARLSHÖHE_2 = 'de:08118:5519:0:4'  # eRtg Tennisplatz (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_TENNISPLATZ = 'de:08118:5520'
	LUDWIGSBURG_TENNISPLATZ_1 = 'de:08118:5520:0:3'  # eRtg Karlshöhe (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_TENNISPLATZ_2 = 'de:08118:5520:0:4'  # eRtg Robert-Franck-Allee (Ludwigsburg Ludwigsburg)
	LUDW__ROBERT_FRANCK_ALLEE = 'de:08118:5521'
	LUDW__ROBERT_FRANCK_ALLEE_1 = 'de:08118:5521:0:3'  # Rtg Aldinger Str. (Ludwigsburg Ludwigsburg)
	LUDW__ROBERT_FRANCK_ALLEE_2 = 'de:08118:5521:0:4'  # Rtg Tennisplatz (Ludwigsburg Ludwigsburg)
	ROBERT_FRANCK_ALLEE = 'de:08118:5521'
	ROBERT_FRANCK_ALLEE_1 = 'de:08118:5521:0:3'  # Rtg Aldinger Str. (Ludwigsburg Ludwigsburg)
	ROBERT_FRANCK_ALLEE_2 = 'de:08118:5521:0:4'  # Rtg Tennisplatz (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_ALDINGER_STR = 'de:08118:5522'
	LUDWIGSBURG_ALDINGER_STR_1 = 'de:08118:5522:0:3'  # Rtg Am Sonnenberg (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_ALDINGER_STR_2 = 'de:08118:5522:0:4'  # Rtg R.-Franck-Str./Rotbäumlesfeld (Ludwigsburg Ludwigsburg)
	GRÜNBÜHL_NETZESTRASSE = 'de:08118:5523'
	GRÜNBÜHL_NETZESTRASSE_1 = 'de:08118:5523:0:3'  # eRtg Am Sonnenberg (Ludwigsburg Grünbühl)
	GRÜNBÜHL_NETZESTRASSE_2 = 'de:08118:5523:0:4'  # eRtg Berufsschule (Ludwigsburg Grünbühl)
	NETZESTRASSE = 'de:08118:5523'
	NETZESTRASSE_1 = 'de:08118:5523:0:3'  # eRtg Am Sonnenberg (Ludwigsburg Grünbühl)
	NETZESTRASSE_2 = 'de:08118:5523:0:4'  # eRtg Berufsschule (Ludwigsburg Grünbühl)
	REICHERTSHALDE = 'de:08118:5524'
	REICHERTSHALDE_1 = 'de:08118:5524:0:3'  # Rtg Bottwartalstr. (Ludwigsburg Ludwigsburg)
	REICHERTSHALDE_2 = 'de:08118:5524:0:4'  # Rtg Favorite (Ludwigsburg Ludwigsburg)
	LUDW__REICHERTSHALDE = 'de:08118:5524'
	LUDW__REICHERTSHALDE_1 = 'de:08118:5524:0:3'  # Rtg Bottwartalstr. (Ludwigsburg Ludwigsburg)
	LUDW__REICHERTSHALDE_2 = 'de:08118:5524:0:4'  # Rtg Favorite (Ludwigsburg Ludwigsburg)
	HOHENECK_LUDWIGSBURGER_STR = 'de:08118:5525'
	HOHENECK_LUDWIGSBURGER_STR_1 = 'de:08118:5525:0:3'  # eRtg Ebelstraße (Ludwigsburg Hoheneck)
	HOHENECK_LUDWIGSBURGER_STR_2 = 'de:08118:5525:0:4'  # eRtg Bottwartalstraße (Ludwigsburg Hoheneck)
	HOHENECK_EBELSTRASSE = 'de:08118:5526'
	HOHENECK_EBELSTRASSE_1 = 'de:08118:5526:0:3'  # eRtg Uferstraße (Ludwigsburg Hoheneck)
	HOHENECK_EBELSTRASSE_2 = 'de:08118:5526:0:4'  # eRtg Ludwigsburger Straße (Ludwigsburg Hoheneck)
	EBELSTRASSE = 'de:08118:5526'
	EBELSTRASSE_1 = 'de:08118:5526:0:3'  # eRtg Uferstraße (Ludwigsburg Hoheneck)
	EBELSTRASSE_2 = 'de:08118:5526:0:4'  # eRtg Ludwigsburger Straße (Ludwigsburg Hoheneck)
	HOHENECK_UFERSTRASSE = 'de:08118:5527'
	HOHENECK_UFERSTRASSE_1 = 'de:08118:5527:0:3'  # eRtg Heilbad (Ludwigsburg Hoheneck)
	HOHENECK_UFERSTRASSE_2 = 'de:08118:5527:0:4'  # eRtg Ebelstraße (Ludwigsburg Hoheneck)
	UFERSTRASSE = 'de:08118:5527'
	UFERSTRASSE_1 = 'de:08118:5527:0:3'  # eRtg Heilbad (Ludwigsburg Hoheneck)
	UFERSTRASSE_2 = 'de:08118:5527:0:4'  # eRtg Ebelstraße (Ludwigsburg Hoheneck)
	HEILBAD = 'de:08118:5528'
	HEILBAD_1 = 'de:08118:5528:1:3'  # Rtg Uferstr. (Ludwigsburg Hoheneck)
	HEILBAD_2 = 'de:08118:5528:1:4'  # Rtg Neckarbrücke (Ludwigsburg Hoheneck)
	HOHENECK_HEILBAD = 'de:08118:5528'
	HOHENECK_HEILBAD_1 = 'de:08118:5528:1:3'  # Rtg Uferstr. (Ludwigsburg Hoheneck)
	HOHENECK_HEILBAD_2 = 'de:08118:5528:1:4'  # Rtg Neckarbrücke (Ludwigsburg Hoheneck)
	LUDW__AGENTUR_FÜR_ARBEIT = 'de:08118:5529'
	LUDW__AGENTUR_FÜR_ARBEIT_1 = 'de:08118:5529:0:3'  # eRtg Landratsamt (Ludwigsburg Ludwigsburg)
	LUDW__AGENTUR_FÜR_ARBEIT_2 = 'de:08118:5529:0:4'  # eRtg Karlsplatz (Ludwigsburg Ludwigsburg)
	LANDRATSAMT_1 = 'de:08118:5530:0:3'  # Rtg Oststr. (Ludwigsburg Ludwigsburg)
	LANDRATSAMT_2 = 'de:08118:5530:0:4'  # Rtg Arbeitsamt (Ludwigsburg Ludwigsburg)
	LANDRATSAMT_3 = 'de:08117:4902:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	LUDWIGSBURG_LANDRATSAMT = 'de:08118:5530'
	LUDWIGSBURG_LANDRATSAMT_1 = 'de:08118:5530:0:3'  # Rtg Oststr. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_LANDRATSAMT_2 = 'de:08118:5530:0:4'  # Rtg Arbeitsamt (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_OSTSTRASSE = 'de:08118:5531'
	LUDWIGSBURG_OSTSTRASSE_1 = 'de:08118:5531:0:3'  # eRtg Danziger Straße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_OSTSTRASSE_2 = 'de:08118:5531:0:4'  # eRtg Landratsamt (Ludwigsburg Ludwigsburg)
	OSTSTRASSE = 'de:08118:5531'
	OSTSTRASSE_1 = 'de:08118:5531:0:3'  # eRtg Danziger Straße (Ludwigsburg Ludwigsburg)
	OSTSTRASSE_2 = 'de:08118:5531:0:4'  # eRtg Landratsamt (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_OSTERHOLZ = 'de:08118:5532'
	LUDWIGSBURG_OSTERHOLZ_1 = 'de:08118:5532:0:3'  # eRtg Schlieffenstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_OSTERHOLZ_2 = 'de:08118:5532:0:4'  # eRtg Mörikestraße (Ludwigsburg Ludwigsburg)
	OSTERHOLZ = 'de:08118:5532'
	OSTERHOLZ_1 = 'de:08118:5532:0:3'  # eRtg Schlieffenstraße (Ludwigsburg Ludwigsburg)
	OSTERHOLZ_2 = 'de:08118:5532:0:4'  # eRtg Mörikestraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_DANZIGER_STR = 'de:08118:5533'
	LUDWIGSBURG_DANZIGER_STR_1 = 'de:08118:5533:0:3'  # Rtg Danziger Str. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_DANZIGER_STR_2 = 'de:08118:5533:0:4'  # Rtg Hindenburgstr. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_BRÜNNER_STR = 'de:08118:5534'
	LUDWIGSBURG_BRÜNNER_STR_1 = 'de:08118:5534:0:3'  # Rtg Fellbacher Str. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_BRÜNNER_STR_2 = 'de:08118:5534:0:4'  # Rtg Danziger Str. (Ludwigsburg Ludwigsburg)
	BRÜNNER_STRASSE = 'de:08118:5534'
	BRÜNNER_STRASSE_1 = 'de:08118:5534:0:3'  # Rtg Fellbacher Str. (Ludwigsburg Ludwigsburg)
	BRÜNNER_STRASSE_2 = 'de:08118:5534:0:4'  # Rtg Danziger Str. (Ludwigsburg Ludwigsburg)
	EGLOSHEIM_BESIGHEIMER_STR = 'de:08118:5535'
	EGLOSHEIM_BESIGHEIMER_STR_1 = 'de:08118:5535:1:3'  # Rtg Rosenackerweg (Ludwigsburg Eglosheim)
	EGLOSHEIM_BESIGHEIMER_STR_2 = 'de:08118:5535:1:4'  # Rtg Breuningerland (Ludwigsburg Eglosheim)
	BESIGHEIMER_STRASSE_1 = 'de:08118:5535:1:3'  # Rtg Rosenackerweg (Ludwigsburg Eglosheim)
	BESIGHEIMER_STRASSE_2 = 'de:08118:5535:1:4'  # Rtg Breuningerland (Ludwigsburg Eglosheim)
	BESIGHEIMER_STRASSE_3 = 'de:08118:5703:0:3'  # eRtg Steinbachbrücke (Löchgau Löchgau)
	BESIGHEIMER_STRASSE_4 = 'de:08118:5703:0:4'  # eRtg Wette (Löchgau Löchgau)
	BESIGHEIMER_STRASSE_5 = 'de:08118:5706:0:3'  # eRtg Erligheim (Freudental Freudental)
	BESIGHEIMER_STRASSE_6 = 'de:08118:5706:0:4'  # eRtg Freudentaler Straße (Freudental Freudental)
	BESIGHEIMER_STRASSE_7 = 'de:08118:6524:0:3'  # Rtg Besigheim/Kirchheim (Besigheim Ottmarsheim)
	OSSWEIL_FELLBACHER_STR = 'de:08118:5536'
	OSSWEIL_FELLBACHER_STR_1 = 'de:08118:5536:0:3'  # Rtg Mehrzweckhalle (Ludwigsburg Oßweil)
	OSSWEIL_FELLBACHER_STR_2 = 'de:08118:5536:0:4'  # Rtg Brünner Str. (Ludwigsburg Oßweil)
	LUDWIGSBURG_ALBRECHTSTR = 'de:08118:5537:0:3'  # eRtg Gänsfußallee (Ludwigsburg Ludwigsburg)
	ALBRECHTSTRASSE = 'de:08118:5537:0:3'  # eRtg Gänsfußallee (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_HOFERSTRASSE = 'de:08118:5538:0:4'  # Rtg Brenzstr. (Ludwigsburg Ludwigsburg)
	HOFERSTRASSE = 'de:08118:5538:0:4'  # Rtg Brenzstr. (Ludwigsburg Ludwigsburg)
	JENISCHSTRASSE = 'de:08118:5539'
	JENISCHSTRASSE_1 = 'de:08118:5539:0:3'  # eRtg Osterholzallee (Ludwigsburg Ludwigsburg)
	JENISCHSTRASSE_2 = 'de:08118:5539:0:4'  # eRtg Gänsfußallee (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_JENISCHSTR = 'de:08118:5539'
	LUDWIGSBURG_JENISCHSTR_1 = 'de:08118:5539:0:3'  # eRtg Osterholzallee (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_JENISCHSTR_2 = 'de:08118:5539:0:4'  # eRtg Gänsfußallee (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_OSTERHOLZALLEE = 'de:08118:5540'
	LUDWIGSBURG_OSTERHOLZALLEE_1 = 'de:08118:5540:0:5'  # Rtg Weststadt-Kirche (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_OSTERHOLZALLEE_2 = 'de:08118:5540:0:6'  # Rtg Gänsfußallee (Ludwigsburg Ludwigsburg)
	OSTERHOLZALLEE = 'de:08118:5540'
	OSTERHOLZALLEE_1 = 'de:08118:5540:0:5'  # Rtg Weststadt-Kirche (Ludwigsburg Ludwigsburg)
	OSTERHOLZALLEE_2 = 'de:08118:5540:0:6'  # Rtg Gänsfußallee (Ludwigsburg Ludwigsburg)
	WESTSTADT_KIRCHE = 'de:08118:5541'
	WESTSTADT_KIRCHE_1 = 'de:08118:5541:0:3'  # eRtg Weimarstraße (Ludwigsburg Ludwigsburg)
	WESTSTADT_KIRCHE_2 = 'de:08118:5541:0:4'  # eRtg Osterholzallee (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_WESTSTADT_KIRCHE = 'de:08118:5541'
	LUDWIGSBURG_WESTSTADT_KIRCHE_1 = 'de:08118:5541:0:3'  # eRtg Weimarstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_WESTSTADT_KIRCHE_2 = 'de:08118:5541:0:4'  # eRtg Osterholzallee (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_WEIMARSTRASSE = 'de:08118:5542'
	LUDWIGSBURG_WEIMARSTRASSE_1 = 'de:08118:5542:0:3'  # eRtg August-Bebel-Straße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_WEIMARSTRASSE_2 = 'de:08118:5542:0:4'  # eRtg Weststadt-Kirche (Ludwigsburg Ludwigsburg)
	LUDW__PÄDAGOGISCHE_HOCHSCH = 'de:08118:5543'
	LUDW__PÄDAGOGISCHE_HOCHSCH_1 = 'de:08118:5543:1:3'  # eRtg Rosenackerweg (Ludwigsburg Ludwigsburg)
	LUDW__PÄDAGOGISCHE_HOCHSCH_2 = 'de:08118:5543:1:4'  # eRtg Heilbronner Straße (Ludwigsburg Ludwigsburg)
	PÄDAGOGISCHE_HOCHSCHULE = 'de:08118:5543'
	PÄDAGOGISCHE_HOCHSCHULE_1 = 'de:08118:5543:1:3'  # eRtg Rosenackerweg (Ludwigsburg Ludwigsburg)
	PÄDAGOGISCHE_HOCHSCHULE_2 = 'de:08118:5543:1:4'  # eRtg Heilbronner Straße (Ludwigsburg Ludwigsburg)
	GROSSINGERSHEIM_SCHULE = 'de:08118:5544:0:3'  # eRtg Brühlstraße (Ingersheim Großingersheim)
	EGLOSHEIM_STRASSENÄCKER = 'de:08118:5545:0:4'  # eRtg Bissinger Straße (Ludwigsburg Eglosheim)
	STRASSENÄCKER = 'de:08118:5545:0:4'  # eRtg Bissinger Straße (Ludwigsburg Eglosheim)
	HEGNACH_ALDINGER_STRASSE = 'de:08119:5546:0:4'  # eRtg Neckarstraße/LB (Waiblingen Hegnach)
	HOHENHASL__LANGMANTEL = 'de:08118:5547'
	HOHENHASL__LANGMANTEL_1 = 'de:08118:5547:0:3'  # eRtg Staffel (Sachsenheim Hohenhaslach)
	HOHENHASL__LANGMANTEL_2 = 'de:08118:5547:0:4'  # eRtg Metterbrücke (Sachsenheim Hohenhaslach)
	LANGMANTEL = 'de:08118:5547'
	LANGMANTEL_1 = 'de:08118:5547:0:3'  # eRtg Staffel (Sachsenheim Hohenhaslach)
	LANGMANTEL_2 = 'de:08118:5547:0:4'  # eRtg Metterbrücke (Sachsenheim Hohenhaslach)
	HOCHDORF_WENDEPLATTE = 'de:08118:5548:0:3'  # eRtg Sonne (Remseck am Neckar Hochdorf (Remseck))
	NECKARGRÖNINGEN_SCHIESSTAL = 'de:08118:5549'
	NECKARGRÖNINGEN_SCHIESSTAL_1 = 'de:08118:5549:0:3'  # Rtg Poppenweiler Hochberg (Remseck am Neckar Neckargröningen)
	NECKARGRÖNINGEN_SCHIESSTAL_2 = 'de:08118:5549:0:4'  # Rtg Neckargröningen Stadtbahn (Remseck am Neckar Neckargröningen)
	SCHIESSTAL = 'de:08118:5549'
	SCHIESSTAL_1 = 'de:08118:5549:0:3'  # Rtg Poppenweiler Hochberg (Remseck am Neckar Neckargröningen)
	SCHIESSTAL_2 = 'de:08118:5549:0:4'  # Rtg Neckargröningen Stadtbahn (Remseck am Neckar Neckargröningen)
	HOCHBERG_ADLERPLATZ = 'de:08118:5550'
	HOCHBERG_ADLERPLATZ_1 = 'de:08118:5550:0:3'  # eRtg Alexandrinenplatz (Remseck am Neckar Hochberg)
	HOCHBERG_ADLERPLATZ_2 = 'de:08118:5550:0:4'  # eRtg Neckarzentrum (Remseck am Neckar Hochberg)
	ADLERPLATZ = 'de:08118:5550'
	ADLERPLATZ_1 = 'de:08118:5550:0:3'  # eRtg Alexandrinenplatz (Remseck am Neckar Hochberg)
	ADLERPLATZ_2 = 'de:08118:5550:0:4'  # eRtg Neckarzentrum (Remseck am Neckar Hochberg)
	HOCHBERG_ALEXANDRINENPL = 'de:08118:5551'
	HOCHBERG_ALEXANDRINENPL_1 = 'de:08118:5551:1:3'  # eRtg Rotweg (Remseck am Neckar Hochberg)
	HOCHBERG_ALEXANDRINENPL_2 = 'de:08118:5551:1:4'  # eRtg Adlerplatz/Lindenstraße (Remseck am Neckar Hochberg)
	ALEXANDRINENPLATZ = 'de:08118:5551'
	ALEXANDRINENPLATZ_1 = 'de:08118:5551:1:3'  # eRtg Rotweg (Remseck am Neckar Hochberg)
	ALEXANDRINENPLATZ_2 = 'de:08118:5551:1:4'  # eRtg Adlerplatz/Lindenstraße (Remseck am Neckar Hochberg)
	HOCHDORF_HOCHBERGER_STR = 'de:08118:5552'
	HOCHDORF_HOCHBERGER_STR_1 = 'de:08118:5552:0:3'  # eRtg Wilhelmsplatz (Remseck am Neckar Hochdorf (Remseck))
	HOCHDORF_HOCHBERGER_STR_2 = 'de:08118:5552:0:4'  # eRtg Rotweg (Remseck am Neckar Hochdorf (Remseck))
	HOCHBERGER_STRASSE_1 = 'de:08118:5552:0:3'  # eRtg Wilhelmsplatz (Remseck am Neckar Hochdorf (Remseck))
	HOCHBERGER_STRASSE_2 = 'de:08118:5552:0:4'  # eRtg Rotweg (Remseck am Neckar Hochdorf (Remseck))
	HOCHBERGER_STRASSE_3 = 'de:08119:5553:1:3'  # eRtg Hochdorf (Waiblingen Bittenfeld)
	HOCHBERGER_STRASSE_4 = 'de:08118:6820:0:3'  # Rtg Neckarweihingen (Ludwigsburg Poppenweiler)
	HOCHBERGER_STRASSE_5 = 'de:08118:6820:0:4'  # Rtg Hochberg / Neckargröningen (Ludwigsburg Poppenweiler)
	BITTENFELD_HOCHBERGER_STR = 'de:08119:5553:1:3'  # eRtg Hochdorf (Waiblingen Bittenfeld)
	BITTENFELD_HOCHDORFER_STR = 'de:08119:5554:1:4'  # eRtg Hochberg Rotweg (Waiblingen Bittenfeld)
	HOCHDORF_POPPENWEILER_STR = 'de:08118:5556'
	HOCHDORF_POPPENWEILER_STR_1 = 'de:08118:5556:0:3'  # Rtg Friedhof (Remseck am Neckar Hochdorf (Remseck))
	HOCHDORF_POPPENWEILER_STR_2 = 'de:08118:5556:0:4'  # Rtg Wilhelmsplatz (Remseck am Neckar Hochdorf (Remseck))
	POPPENWEILER_STRASSE = 'de:08118:5556'
	POPPENWEILER_STRASSE_1 = 'de:08118:5556:0:3'  # Rtg Friedhof (Remseck am Neckar Hochdorf (Remseck))
	POPPENWEILER_STRASSE_2 = 'de:08118:5556:0:4'  # Rtg Wilhelmsplatz (Remseck am Neckar Hochdorf (Remseck))
	STEINHEIMER_STRASSE_1 = 'de:08118:5557:1:3'  # eRtg Burghaldenstraße (Ludwigsburg Poppenweiler)
	STEINHEIMER_STRASSE_2 = 'de:08118:5557:1:4'  # eRtg Zehntscheuer (Ludwigsburg Poppenweiler)
	STEINHEIMER_STRASSE_3 = 'de:08118:7407:0:3'  # eRtg Gustav-Rau-Straße (Bietigheim-Bissingen Bietigheim)
	STEINHEIMER_STRASSE_4 = 'de:08118:7407:0:4'  # Rtg Höpfigheimer Str. (Bietigheim-Bissingen Bietigheim)
	POPPENWEILER_STEINHEIMER_STR = 'de:08118:5557'
	POPPENWEILER_STEINHEIMER_STR_1 = 'de:08118:5557:1:3'  # eRtg Burghaldenstraße (Ludwigsburg Poppenweiler)
	POPPENWEILER_STEINHEIMER_STR_2 = 'de:08118:5557:1:4'  # eRtg Zehntscheuer (Ludwigsburg Poppenweiler)
	POPPENWEILER_FRIEDHOF = 'de:08118:5558'
	POPPENWEILER_FRIEDHOF_1 = 'de:08118:5558:0:3'  # eRtg Zehntscheuer (Ludwigsburg Poppenweiler)
	POPPENWEILER_FRIEDHOF_2 = 'de:08118:5558:0:4'  # eRtg Poppenweiler Straße (Ludwigsburg Poppenweiler)
	POPPENWEILER_ZEHNTSCHEUER = 'de:08118:5559'
	POPPENWEILER_ZEHNTSCHEUER_1 = 'de:08118:5559:0:3'  # eRtg Steinheimer Straße (Ludwigsburg Poppenweiler)
	POPPENWEILER_ZEHNTSCHEUER_2 = 'de:08118:5559:0:4'  # eRtg Friedhof (Ludwigsburg Poppenweiler)
	PLEIDELSHEIM_R__BOSCH_STR = 'de:08118:5560'
	PLEIDELSHEIM_R__BOSCH_STR_1 = 'de:08118:5560:0:3'  # eRtg Hohenzollernstraße (Pleidelsheim Pleidelsheim)
	PLEIDELSHEIM_R__BOSCH_STR_2 = 'de:08118:5560:0:4'  # eRtg Beihinger Platz (Pleidelsheim Pleidelsheim)
	BURGHALDENSTRASSE = 'de:08118:5561'
	BURGHALDENSTRASSE_1 = 'de:08118:5561:1:3'  # Bus 430 (Ludwigsburg Poppenweiler)
	BURGHALDENSTRASSE_2 = 'de:08118:5561:1:4'  # Bus 433 & Nachtbus (Ludwigsburg Poppenweiler)
	POPPENWEILER_BURGHALDENSTR = 'de:08118:5561'
	POPPENWEILER_BURGHALDENSTR_1 = 'de:08118:5561:1:3'  # Bus 430 (Ludwigsburg Poppenweiler)
	POPPENWEILER_BURGHALDENSTR_2 = 'de:08118:5561:1:4'  # Bus 433 & Nachtbus (Ludwigsburg Poppenweiler)
	ENZPLATZ = 'de:08118:5563'
	ENZPLATZ_1 = 'de:08118:5563:0:3'  # Rtg Bahnhof/Kelterplatz (Besigheim Besigheim)
	ENZPLATZ_2 = 'de:08118:5563:0:4'  # Rtg Martinshaus (Besigheim Besigheim)
	BESIGHEIM_ENZPLATZ = 'de:08118:5563'
	BESIGHEIM_ENZPLATZ_1 = 'de:08118:5563:0:3'  # Rtg Bahnhof/Kelterplatz (Besigheim Besigheim)
	BESIGHEIM_ENZPLATZ_2 = 'de:08118:5563:0:4'  # Rtg Martinshaus (Besigheim Besigheim)
	FREIBERG__N__MÖNCHSBERGSTR = 'de:08118:5565'
	FREIBERG__N__MÖNCHSBERGSTR_1 = 'de:08118:5565:0:3'  # eRtg Geisinger Dorfplatz (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__MÖNCHSBERGSTR_2 = 'de:08118:5565:0:4'  # eRtg Parkäcker (Freiberg am Neckar Freiberg (N))
	MÖNCHSBERGSTRASSE = 'de:08118:5565'
	MÖNCHSBERGSTRASSE_1 = 'de:08118:5565:0:3'  # eRtg Geisinger Dorfplatz (Freiberg am Neckar Freiberg (N))
	MÖNCHSBERGSTRASSE_2 = 'de:08118:5565:0:4'  # eRtg Parkäcker (Freiberg am Neckar Freiberg (N))
	GROSSINGERSH__BIETIGHEIMER_STR = 'de:08118:5566'
	GROSSINGERSH__BIETIGHEIMER_STR_1 = 'de:08118:5566:1:4'  # Rtg Poststrässle (Ingersheim Großingersheim)
	GROSSINGERSH__BIETIGHEIMER_STR_2 = 'de:08118:5566:2:3'  # Rtg Tiefengässle (Ingersheim Großingersheim)
	GROSSING__BRÜHLSTRASSE = 'de:08118:5567'
	GROSSING__BRÜHLSTRASSE_1 = 'de:08118:5567:0:3'  # eRtg Tiefengasse (Ingersheim Großingersheim)
	GROSSING__BRÜHLSTRASSE_2 = 'de:08118:5567:0:4'  # eRtg Schlossstraße (Ingersheim Großingersheim)
	KLEININGERSHEIM_BLUMENSTR = 'de:08118:5568:1:3'  # alle Richtungen (Ingersheim Kleiningersheim)
	PLEIDELSHEIM_BEIHINGER_STR = 'de:08118:5569'
	PLEIDELSHEIM_BEIHINGER_STR_1 = 'de:08118:5569:0:3'  # eRtg Wiegehalle (Pleidelsheim Pleidelsheim)
	PLEIDELSHEIM_BEIHINGER_STR_2 = 'de:08118:5569:0:4'  # eRtg Hohenzollernstraße (Pleidelsheim Pleidelsheim)
	PLEIDELSHEIM_HAUPTSTRASSE = 'de:08118:5570'
	PLEIDELSHEIM_HAUPTSTRASSE_1 = 'de:08118:5570:1:3'  # eRtg Wiegehalle/Riedbachstraße/Beihinger (Pleidelsheim Pleidelsheim)
	PLEIDELSHEIM_HAUPTSTRASSE_2 = 'de:08118:5570:1:4'  # eRtg Am Friedhof (Pleidelsheim Pleidelsheim)
	PLEIDELSHEIM_WIEGEHALLE = 'de:08118:5571'
	PLEIDELSHEIM_WIEGEHALLE_1 = 'de:08118:5571:1:3'  # eRtg SÜWAG (Pleidelsheim Pleidelsheim)
	PLEIDELSHEIM_WIEGEHALLE_2 = 'de:08118:5571:1:4'  # eRtg Hauptstraße/Beihinger Straße (Pleidelsheim Pleidelsheim)
	WIEGEHALLE = 'de:08118:5571'
	WIEGEHALLE_1 = 'de:08118:5571:1:3'  # eRtg SÜWAG (Pleidelsheim Pleidelsheim)
	WIEGEHALLE_2 = 'de:08118:5571:1:4'  # eRtg Hauptstraße/Beihinger Straße (Pleidelsheim Pleidelsheim)
	SÜWAG = 'de:08118:5572'
	SÜWAG_1 = 'de:08118:5572:0:3'  # eRtg Endwiesenstraße (Pleidelsheim Pleidelsheim)
	SÜWAG_2 = 'de:08118:5572:0:4'  # eRtg Wiegehalle (Pleidelsheim Pleidelsheim)
	PLEIDELSHEIM_SÜWAG = 'de:08118:5572'
	PLEIDELSHEIM_SÜWAG_1 = 'de:08118:5572:0:3'  # eRtg Endwiesenstraße (Pleidelsheim Pleidelsheim)
	PLEIDELSHEIM_SÜWAG_2 = 'de:08118:5572:0:4'  # eRtg Wiegehalle (Pleidelsheim Pleidelsheim)
	HÖPFIGHEIM_HAUPTSTRASSE = 'de:08118:5573'
	HÖPFIGHEIM_HAUPTSTRASSE_1 = 'de:08118:5573:0:3'  # Rtg Talstr. (Steinheim an der Murr Höpfigheim)
	HÖPFIGHEIM_HAUPTSTRASSE_2 = 'de:08118:5573:0:4'  # Rtg Endwiesenstr. (Steinheim an der Murr Höpfigheim)
	MUNDELSHEIM_LANGE_STR = 'de:08118:5574'
	MUNDELSHEIM_LANGE_STR_1 = 'de:08118:5574:0:3'  # eRtg Hessigheimer Straße (Mundelsheim Mundelsheim)
	MUNDELSHEIM_LANGE_STR_2 = 'de:08118:5574:0:4'  # eRtg SÜWAG (Mundelsheim Mundelsheim)
	MUNDELSHEIM_HESSIGHEIMER_STR = 'de:08118:5575'
	MUNDELSHEIM_HESSIGHEIMER_STR_1 = 'de:08118:5575:1:3'  # eRtg Hessigh. Gartenstraße (Mundelsheim Mundelsheim)
	MUNDELSHEIM_HESSIGHEIMER_STR_2 = 'de:08118:5575:1:4'  # eRtg Lange Straße (Mundelsheim Mundelsheim)
	HESSIGHEIMER_STRASSE = 'de:08118:5575'
	HESSIGHEIMER_STRASSE_1 = 'de:08118:5575:1:3'  # eRtg Hessigh. Gartenstraße (Mundelsheim Mundelsheim)
	HESSIGHEIMER_STRASSE_2 = 'de:08118:5575:1:4'  # eRtg Lange Straße (Mundelsheim Mundelsheim)
	HESSIGHEIM_GARTENSTRASSE = 'de:08118:5576'
	HESSIGHEIM_GARTENSTRASSE_1 = 'de:08118:5576:1:3'  # eRtg Sattlerei Eisele (Hessigheim Hessigheim)
	HESSIGHEIM_GARTENSTRASSE_2 = 'de:08118:5576:1:4'  # eRtg Mundelsh. Hessigheimer Str. (Hessigheim Hessigheim)
	HESSIGH__SATTLEREI_EISELE = 'de:08118:5578'
	HESSIGH__SATTLEREI_EISELE_1 = 'de:08118:5578:0:3'  # eRtg Neusatz (Hessigheim Hessigheim)
	HESSIGH__SATTLEREI_EISELE_2 = 'de:08118:5578:0:4'  # eRtg Gartenstraße (Hessigheim Hessigheim)
	SATTLEREI_EISELE = 'de:08118:5578'
	SATTLEREI_EISELE_1 = 'de:08118:5578:0:3'  # eRtg Neusatz (Hessigheim Hessigheim)
	SATTLEREI_EISELE_2 = 'de:08118:5578:0:4'  # eRtg Gartenstraße (Hessigheim Hessigheim)
	OTTMARSHEIM_RATHAUS = 'de:08118:5579'
	OTTMARSHEIM_RATHAUS_1 = 'de:08118:5579:0:3'  # Rtg Ilsfelder Str. (Besigheim Ottmarsheim)
	OTTMARSHEIM_RATHAUS_2 = 'de:08118:5579:0:4'  # Rtg Besigheim (Besigheim Ottmarsheim)
	BESIGHEIM_NEUSATZ = 'de:08118:5580'
	BESIGHEIM_NEUSATZ_1 = 'de:08118:5580:0:3'  # eRtg Rathaus/Sattlerei/Gartenstr. (Besigheim Besigheim)
	BESIGHEIM_NEUSATZ_2 = 'de:08118:5580:0:4'  # eRtg Oberamteigasse/Jahnstr. (Besigheim Besigheim)
	NEUSATZ = 'de:08118:5580'
	NEUSATZ_1 = 'de:08118:5580:0:3'  # eRtg Rathaus/Sattlerei/Gartenstr. (Besigheim Besigheim)
	NEUSATZ_2 = 'de:08118:5580:0:4'  # eRtg Oberamteigasse/Jahnstr. (Besigheim Besigheim)
	BESIGHEIM_JAHNSTRASSE = 'de:08118:5581'
	BESIGHEIM_JAHNSTRASSE_1 = 'de:08118:5581:1:3'  # eRtg Auf dem Kies (Besigheim Besigheim)
	BESIGHEIM_JAHNSTRASSE_2 = 'de:08118:5581:1:4'  # eRtg Freibad/FitKom (Besigheim Besigheim)
	BESIGHEIM_JAHNSTRASSE_3 = 'de:08118:5581:1:5'  # eRtg Schülerbussteig Rtg Auf dem Kiess (Besigheim Besigheim)
	KELTERPLATZ_1 = 'de:08118:5582:0:3'  # eRtg Besigheim (Besigheim Besigheim)
	KELTERPLATZ_2 = 'de:08118:5582:0:4'  # eRtg Oberamteigasse (Besigheim Besigheim)
	KELTERPLATZ_3 = 'de:08119:7458:0:03'  # alle Richtungen (Auenwald Oberbrüden)
	BESIGHEIM_KELTERPLATZ = 'de:08118:5582'
	BESIGHEIM_KELTERPLATZ_1 = 'de:08118:5582:0:3'  # eRtg Besigheim (Besigheim Besigheim)
	BESIGHEIM_KELTERPLATZ_2 = 'de:08118:5582:0:4'  # eRtg Oberamteigasse (Besigheim Besigheim)
	BESIGHEIM = 'de:08118:5583'
	BESIGHEIM_1 = 'de:08118:5583:1:1'  # Gleis 1 (Besigheim Besigheim)
	BESIGHEIM_2 = 'de:08118:5583:2:2'  # Gleis 2 (Besigheim Besigheim)
	BESIGHEIM_3 = 'de:08118:5583:3:1'  # Bstg 1 (Besigheim Besigheim)
	BESIGHEIM_4 = 'de:08118:5583:3:2'  # Bstg 2 (Besigheim Besigheim)
	BESIGHEIM_5 = 'de:08118:5583:3:3'  # Bstg 3 (Besigheim Besigheim)
	BESIGHEIM_6 = 'de:08118:5583:3:4'  # Bstg 4 (Besigheim Besigheim)
	BISSINGER_STRASSE_1 = 'de:08118:5584:1:3'  # Rtg Straßenäcker (West) (Ludwigsburg Eglosheim)
	BISSINGER_STRASSE_2 = 'de:08118:5584:1:4'  # Rtg Frankfurter Straße (Ost) (Ludwigsburg Eglosheim)
	BISSINGER_STRASSE_3 = 'de:08118:5645:0:3'  # eRtg Am Türmle/Haupstraße (Bietigheim-Bissingen Untermberg)
	BISSINGER_STRASSE_4 = 'de:08118:5645:0:4'  # eRtg Rathaus/Hallenbad (Bietigheim-Bissingen Untermberg)
	BISSINGER_STRASSE_5 = 'de:08116:7821:0:3'  # Rtg Schopfloch (Bissingen an der Teck Ochsenwang)
	BISSINGER_STRASSE_6 = 'de:08116:7821:0:4'  # Rtg Bissingen Rathaus (Bissingen an der Teck Ochsenwang)
	EGLOSHEIM_BISSINGER_STRASSE = 'de:08118:5584'
	EGLOSHEIM_BISSINGER_STRASSE_1 = 'de:08118:5584:1:3'  # Rtg Straßenäcker (West) (Ludwigsburg Eglosheim)
	EGLOSHEIM_BISSINGER_STRASSE_2 = 'de:08118:5584:1:4'  # Rtg Frankfurter Straße (Ost) (Ludwigsburg Eglosheim)
	KOHLENWEG = 'de:08118:5586:0:4'  # Rtg Süd (Remseck am Neckar Aldingen)
	ALDINGEN_KOHLENWEG = 'de:08118:5586:0:4'  # Rtg Süd (Remseck am Neckar Aldingen)
	KERNERSTEG = 'de:08118:5587:1:3'  # eRtg Pattonville (Remseck am Neckar Aldingen)
	ALDINGEN_KERNERSTEG = 'de:08118:5587:1:3'  # eRtg Pattonville (Remseck am Neckar Aldingen)
	ALDINGEN_CANNSTATTER_STR = 'de:08118:5588'
	ALDINGEN_CANNSTATTER_STR_1 = 'de:08118:5588:0:3'  # eRtg Hornbach/Mühlweg (Remseck am Neckar Aldingen)
	ALDINGEN_CANNSTATTER_STR_2 = 'de:08118:5588:0:4'  # eRtg Schlosshof/Goethestraße (Remseck am Neckar Aldingen)
	ALDINGEN_HAUS_DER_BÜRGER = 'de:08118:5589'
	ALDINGEN_HAUS_DER_BÜRGER_1 = 'de:08118:5589:0:3'  # eRtg Kernersteg (Remseck am Neckar Aldingen)
	ALDINGEN_HAUS_DER_BÜRGER_2 = 'de:08118:5589:0:4'  # eRtg Haus der Feuerwehr (Remseck am Neckar Aldingen)
	HAUS_DER_BÜRGER = 'de:08118:5589'
	HAUS_DER_BÜRGER_1 = 'de:08118:5589:0:3'  # eRtg Kernersteg (Remseck am Neckar Aldingen)
	HAUS_DER_BÜRGER_2 = 'de:08118:5589:0:4'  # eRtg Haus der Feuerwehr (Remseck am Neckar Aldingen)
	LISE_MEITNER_GYMNASIUM = 'de:08118:5590'
	LISE_MEITNER_GYMNASIUM_1 = 'de:08118:5590:0:3'  # eRtg Haus der Jugend (Remseck am Neckar Aldingen)
	LISE_MEITNER_GYMNASIUM_2 = 'de:08118:5590:0:4'  # eRtg Kindergarten Leonberger Straße (Remseck am Neckar Aldingen)
	ALDINGEN_L__MEITNER_GYMNASIUM = 'de:08118:5590'
	ALDINGEN_L__MEITNER_GYMNASIUM_1 = 'de:08118:5590:0:3'  # eRtg Haus der Jugend (Remseck am Neckar Aldingen)
	ALDINGEN_L__MEITNER_GYMNASIUM_2 = 'de:08118:5590:0:4'  # eRtg Kindergarten Leonberger Straße (Remseck am Neckar Aldingen)
	NECKARREMS_KELTERSCHULE = 'de:08118:5591'
	NECKARREMS_KELTERSCHULE_1 = 'de:08118:5591:0:3'  # eRtg Dorfstraße (Remseck am Neckar Neckarrems)
	NECKARREMS_KELTERSCHULE_2 = 'de:08118:5591:0:4'  # eRtg Neckarhalde (Remseck am Neckar Neckarrems)
	KELTERSCHULE = 'de:08118:5591'
	KELTERSCHULE_1 = 'de:08118:5591:0:3'  # eRtg Dorfstraße (Remseck am Neckar Neckarrems)
	KELTERSCHULE_2 = 'de:08118:5591:0:4'  # eRtg Neckarhalde (Remseck am Neckar Neckarrems)
	HOCHDORF_SONNE = 'de:08118:5592'
	HOCHDORF_SONNE_1 = 'de:08118:5592:1:3'  # eRtg Wilhelmsplatz (Remseck am Neckar Hochdorf (Remseck))
	HOCHDORF_SONNE_2 = 'de:08118:5592:1:4'  # Ankunft (Remseck am Neckar Hochdorf (Remseck))
	SONNE_1 = 'de:08118:5592:1:3'  # eRtg Wilhelmsplatz (Remseck am Neckar Hochdorf (Remseck))
	SONNE_2 = 'de:08118:5592:1:4'  # Ankunft (Remseck am Neckar Hochdorf (Remseck))
	SONNE_3 = 'de:08117:317:0:H1'  # Rtg Lamm (Böhmenkirch Treffelhausen)
	SONNE_4 = 'de:08117:317:0:R1'  # Rtg Schnittlingen (Böhmenkirch Treffelhausen)
	SONNE_5 = 'de:08136:7014:0:H1'  # vom Bhf. Lorch (Lorch Lorch)
	SONNE_6 = 'de:08136:7014:0:R1'  # Rtg Lorch Bahnhof (Lorch Lorch)
	OBERSTENFELD_BEILSTEINER_STR = 'de:08118:5593'
	OBERSTENFELD_BEILSTEINER_STR_1 = 'de:08118:5593:0:3'  # eRtg Schule (Oberstenfeld Oberstenfeld)
	OBERSTENFELD_BEILSTEINER_STR_2 = 'de:08118:5593:0:4'  # eRtg Lichtenberger Straße (Oberstenfeld Oberstenfeld)
	BEILSTEINER_STRASSE_1 = 'de:08118:5593:0:3'  # eRtg Schule (Oberstenfeld Oberstenfeld)
	BEILSTEINER_STRASSE_2 = 'de:08118:5593:0:4'  # eRtg Lichtenberger Straße (Oberstenfeld Oberstenfeld)
	BEILSTEINER_STRASSE_3 = 'de:08111:6267:0:3'  # Rtg Johanneskirche (Stuttgart Stuttgart)
	LICHTENBERGER_STRASSE = 'de:08118:5594'
	LICHTENBERGER_STRASSE_1 = 'de:08118:5594:0:3'  # eRtg Rathaus (Oberstenfeld Oberstenfeld)
	LICHTENBERGER_STRASSE_2 = 'de:08118:5594:0:4'  # eRtg Kreuzstraße (Oberstenfeld Oberstenfeld)
	OBERSTENF__LICHTENBERGER_STR = 'de:08118:5594'
	OBERSTENF__LICHTENBERGER_STR_1 = 'de:08118:5594:0:3'  # eRtg Rathaus (Oberstenfeld Oberstenfeld)
	OBERSTENF__LICHTENBERGER_STR_2 = 'de:08118:5594:0:4'  # eRtg Kreuzstraße (Oberstenfeld Oberstenfeld)
	GRONAU_PREV__FEUERWEHR = 'de:08118:5596:0:3'  # eRtg Prevorst Löwen (Oberstenfeld Gronau)
	PREVORST_FEUERWEHR = 'de:08118:5596:0:3'  # eRtg Prevorst Löwen (Oberstenfeld Gronau)
	MURR_BLATTERT = 'de:08118:5597'
	MURR_BLATTERT_1 = 'de:08118:5597:0:3'  # Rtg Ortmitte (Murr Murr)
	MURR_BLATTERT_2 = 'de:08118:5597:0:4'  # Rtg Marbach/Pleidelsheim (Murr Murr)
	BLATTERT = 'de:08118:5597'
	BLATTERT_1 = 'de:08118:5597:0:3'  # Rtg Ortmitte (Murr Murr)
	BLATTERT_2 = 'de:08118:5597:0:4'  # Rtg Marbach/Pleidelsheim (Murr Murr)
	MURR_ORTSMITTE = 'de:08118:5598'
	MURR_ORTSMITTE_1 = 'de:08118:5598:0:3'  # Rtg Hartlinde (Murr Murr)
	MURR_ORTSMITTE_2 = 'de:08118:5598:0:4'  # Rtg Blattert (Murr Murr)
	HÖPFIGHEIMER_STRASSE_1 = 'de:08118:5599:0:3'  # eRtg Bahnhof (Steinheim an der Murr Steinheim (M))
	HÖPFIGHEIMER_STRASSE_2 = 'de:08118:5599:0:4'  # eRtg Murrer Straße (Steinheim an der Murr Steinheim (M))
	HÖPFIGHEIMER_STRASSE_3 = 'de:08118:7414:0:3'  # eRtg Pleidelsheimer Straße (Bietigheim-Bissingen Bietigheim)
	STEINH__M__HÖPFIGHEIMER_STR = 'de:08118:5599'
	STEINH__M__HÖPFIGHEIMER_STR_1 = 'de:08118:5599:0:3'  # eRtg Bahnhof (Steinheim an der Murr Steinheim (M))
	STEINH__M__HÖPFIGHEIMER_STR_2 = 'de:08118:5599:0:4'  # eRtg Murrer Straße (Steinheim an der Murr Steinheim (M))
	MURR_HARDTLINDE = 'de:08118:5600'
	MURR_HARDTLINDE_1 = 'de:08118:5600:0:3'  # eRtg Ortsmitte (Murr Murr)
	MURR_HARDTLINDE_2 = 'de:08118:5600:0:4'  # eRtg Murrer Straße (Murr Murr)
	HARDTLINDE = 'de:08118:5600'
	HARDTLINDE_1 = 'de:08118:5600:0:3'  # eRtg Ortsmitte (Murr Murr)
	HARDTLINDE_2 = 'de:08118:5600:0:4'  # eRtg Murrer Straße (Murr Murr)
	STEINHEIM__M__MURRER_STR = 'de:08118:5601'
	STEINHEIM__M__MURRER_STR_1 = 'de:08118:5601:0:3'  # eRtg Hardtlinde (Steinheim an der Murr Steinheim (M))
	STEINHEIM__M__MURRER_STR_2 = 'de:08118:5601:0:4'  # eRtg Bahnhof (Steinheim an der Murr Steinheim (M))
	MURRER_STRASSE = 'de:08118:5601'
	MURRER_STRASSE_1 = 'de:08118:5601:0:3'  # eRtg Hardtlinde (Steinheim an der Murr Steinheim (M))
	MURRER_STRASSE_2 = 'de:08118:5601:0:4'  # eRtg Bahnhof (Steinheim an der Murr Steinheim (M))
	STEINHEIM__M__BF = 'de:08118:5602'
	STEINHEIM__M__BF_1 = 'de:08118:5602:0:3'  # Rtg Murrer Str. (Steinheim an der Murr Steinheim (M))
	STEINHEIM__M__BF_2 = 'de:08118:5602:0:4'  # Rtg Kelter (Steinheim an der Murr Steinheim (M))
	STEINHEIM__M__KELTER = 'de:08118:5603'
	STEINHEIM__M__KELTER_1 = 'de:08118:5603:0:3'  # eRtg Bahnhof (Steinheim an der Murr Steinheim (M))
	STEINHEIM__M__KELTER_2 = 'de:08118:5603:0:4'  # eRtg Kleinbottwarer Straße (Steinheim an der Murr Steinheim (M))
	KLEINBOTTWAR = 'de:08118:5604'
	KLEINBOTTWAR_1 = 'de:08118:5604:1:3'  # Rtg Großbottwar (Steinheim an der Murr Kleinbottwar)
	KLEINBOTTWAR_2 = 'de:08118:5604:1:4'  # Rtg Steinheim (Steinheim an der Murr Kleinbottwar)
	GROSSBOTTWAR_WALDSTRASSE = 'de:08118:5605'
	GROSSBOTTWAR_WALDSTRASSE_1 = 'de:08118:5605:0:3'  # eRtg KleinbottwarSeestraße/Marbach (Großbottwar Großbottwar)
	GROSSBOTTWAR_WALDSTRASSE_2 = 'de:08118:5605:0:4'  # eRtg Bachstraße/Rosenplatz (Großbottwar Großbottwar)
	GROSSBOTTWAR_BACHSTRASSE = 'de:08118:5606'
	GROSSBOTTWAR_BACHSTRASSE_1 = 'de:08118:5606:0:3'  # eRtg Aussiedlerhöfe (Großbottwar Großbottwar)
	GROSSBOTTWAR_BACHSTRASSE_2 = 'de:08118:5606:0:4'  # eRtg Rosenplatz (Großbottwar Großbottwar)
	WINZERHAUSEN_RAIFFEISENSTR = 'de:08118:5607:0:3'  # eRtg Aussiedlerhöfe (Großbottwar Winzerhausen)
	GROSSBOTTWAR_ROSENPLATZ = 'de:08118:5608'
	GROSSBOTTWAR_ROSENPLATZ_1 = 'de:08118:5608:1:3'  # eRtg Waldstraße (Großbottwar Großbottwar)
	GROSSBOTTWAR_ROSENPLATZ_2 = 'de:08118:5608:1:4'  # eRtg Kelter (Großbottwar Großbottwar)
	ROSENPLATZ = 'de:08118:5608'
	ROSENPLATZ_1 = 'de:08118:5608:1:3'  # eRtg Waldstraße (Großbottwar Großbottwar)
	ROSENPLATZ_2 = 'de:08118:5608:1:4'  # eRtg Kelter (Großbottwar Großbottwar)
	GROSSBOTTWAR_KELTER = 'de:08118:5609'
	GROSSBOTTWAR_KELTER_1 = 'de:08118:5609:0:3'  # eRtg Rosenplatz (Großbottwar Großbottwar)
	GROSSBOTTWAR_KELTER_2 = 'de:08118:5609:0:4'  # eRtg Flickenwiesen (Großbottwar Großbottwar)
	GROSSBOTTWAR_SAUSERHOF = 'de:08118:5610'
	GROSSBOTTWAR_SAUSERHOF_1 = 'de:08118:5610:0:3'  # Rtg Oberstenfeld (Großbottwar Großbottwar)
	GROSSBOTTWAR_SAUSERHOF_2 = 'de:08118:5610:0:4'  # Rtg Großbottwar (Großbottwar Großbottwar)
	SAUSERHOF = 'de:08118:5610'
	SAUSERHOF_1 = 'de:08118:5610:0:3'  # Rtg Oberstenfeld (Großbottwar Großbottwar)
	SAUSERHOF_2 = 'de:08118:5610:0:4'  # Rtg Großbottwar (Großbottwar Großbottwar)
	OBERSTENFELD_NUSSBAUMWEG = 'de:08118:5611'
	OBERSTENFELD_NUSSBAUMWEG_1 = 'de:08118:5611:0:3'  # Rtg Rathaus (Oberstenfeld Oberstenfeld)
	OBERSTENFELD_NUSSBAUMWEG_2 = 'de:08118:5611:0:4'  # Rtg Sauserhof (Oberstenfeld Oberstenfeld)
	NUSSBAUMWEG = 'de:08118:5611'
	NUSSBAUMWEG_1 = 'de:08118:5611:0:3'  # Rtg Rathaus (Oberstenfeld Oberstenfeld)
	NUSSBAUMWEG_2 = 'de:08118:5611:0:4'  # Rtg Sauserhof (Oberstenfeld Oberstenfeld)
	OBERSTENFELD_RATHAUS = 'de:08118:5612'
	OBERSTENFELD_RATHAUS_1 = 'de:08118:5612:0:3'  # eRtg Nussbaumweg (Oberstenfeld Oberstenfeld)
	OBERSTENFELD_RATHAUS_2 = 'de:08118:5612:0:4'  # eRtg Lichtenberger Straße/Kath. Kirche (Oberstenfeld Oberstenfeld)
	OBERSTENF__KATH__KIRCHE = 'de:08118:5613'
	OBERSTENF__KATH__KIRCHE_1 = 'de:08118:5613:0:3'  # Rtg Beilstein (Oberstenfeld Oberstenfeld)
	OBERSTENF__KATH__KIRCHE_2 = 'de:08118:5613:0:4'  # Rtg Oberstenfeld (Oberstenfeld Oberstenfeld)
	OBERSTENFELD_KREUZSTR = 'de:08118:5614'
	OBERSTENFELD_KREUZSTR_1 = 'de:08118:5614:0:3'  # eRtg Lichtenberger Straße (Oberstenfeld Oberstenfeld)
	OBERSTENFELD_KREUZSTR_2 = 'de:08118:5614:0:4'  # eRtg Schule (Oberstenfeld Oberstenfeld)
	GRONAU_SCHULE = 'de:08118:5615'
	GRONAU_SCHULE_1 = 'de:08118:5615:0:3'  # Rtg Oberstenfeld (Oberstenfeld Gronau)
	GRONAU_SCHULE_2 = 'de:08118:5615:0:4'  # Rtg Beilstein/Gronau (Oberstenfeld Gronau)
	BIETIGHEIM_KREISBERUFSSCHULE = 'de:08118:5616:0:3'  # eRtg Auwiesenbrücke (Bietigheim-Bissingen Bietigheim)
	DLW = 'de:08118:5617'
	DLW_1 = 'de:08118:5617:0:3'  # Rtg Bahnhof (Bietigheim-Bissingen Bietigheim)
	DLW_2 = 'de:08118:5617:0:4'  # Rtg KH/Auwiesen (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_DLW = 'de:08118:5617'
	BIETIGHEIM_DLW_1 = 'de:08118:5617:0:3'  # Rtg Bahnhof (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_DLW_2 = 'de:08118:5617:0:4'  # Rtg KH/Auwiesen (Bietigheim-Bissingen Bietigheim)
	GROSSB__AUSSIEDLERHÖFE = 'de:08118:5619'
	GROSSB__AUSSIEDLERHÖFE_1 = 'de:08118:5619:0:3'  # eRtg Raiffeisenstraße (Großbottwar Großbottwar)
	GROSSB__AUSSIEDLERHÖFE_2 = 'de:08118:5619:0:4'  # eRtg In den Frauengärten/Bachstraße (Großbottwar Großbottwar)
	HOLZWEILERHOF = 'de:08118:5620:0:3'  # eRtg Raiffeisenstraße (Großbottwar Winzerhausen)
	WINZERH__HOLZWEILERHOF = 'de:08118:5620:0:3'  # eRtg Raiffeisenstraße (Großbottwar Winzerhausen)
	HOCHDORF__E__RATHAUS = 'de:08118:5621'
	HOCHDORF__E__RATHAUS_1 = 'de:08118:5621:0:3'  # eRtg Industriegebiet/Keltenmuseum (Eberdingen Hochdorf (Eberdingen))
	HOCHDORF__E__RATHAUS_2 = 'de:08118:5621:0:4'  # eRtg Pulverdingen B10 (Eberdingen Hochdorf (Eberdingen))
	EBERDINGEN_HOCHDORFER_STR = 'de:08118:5623'
	EBERDINGEN_HOCHDORFER_STR_1 = 'de:08118:5623:1:3'  # Rtg Nussdorf (Eberdingen Eberdingen)
	EBERDINGEN_HOCHDORFER_STR_2 = 'de:08118:5623:1:4'  # Rtg Hochdorf (Eberdingen Eberdingen)
	ROTWEG = 'de:08118:5624'
	ROTWEG_1 = 'de:08118:5624:1:3'  # eRtg Hochdorf/Bittenfeld (Remseck am Neckar Hochberg)
	ROTWEG_2 = 'de:08118:5624:1:4'  # eRtg Alexandrinenplatz (Remseck am Neckar Hochberg)
	HOCHBERG_ROTWEG = 'de:08118:5624'
	HOCHBERG_ROTWEG_1 = 'de:08118:5624:1:3'  # eRtg Hochdorf/Bittenfeld (Remseck am Neckar Hochberg)
	HOCHBERG_ROTWEG_2 = 'de:08118:5624:1:4'  # eRtg Alexandrinenplatz (Remseck am Neckar Hochberg)
	NUSSDORF_MARTINSTRASSE = 'de:08118:5625'
	NUSSDORF_MARTINSTRASSE_1 = 'de:08118:5625:0:3'  # eRtg Vaihinger Straße/B10 (Eberdingen Nussdorf)
	NUSSDORF_MARTINSTRASSE_2 = 'de:08118:5625:0:4'  # eRtg Nußdorfer Straße (Eberdingen Nussdorf)
	MARTINSTRASSE = 'de:08118:5625'
	MARTINSTRASSE_1 = 'de:08118:5625:0:3'  # eRtg Vaihinger Straße/B10 (Eberdingen Nussdorf)
	MARTINSTRASSE_2 = 'de:08118:5625:0:4'  # eRtg Nußdorfer Straße (Eberdingen Nussdorf)
	SCHWIEBERD__HARDTHOF = 'de:08118:5626'
	SCHWIEBERD__HARDTHOF_1 = 'de:08118:5626:0:3'  # eRtg Pulverdingen B10 (Schwieberdingen Schwieberdingen)
	SCHWIEBERD__HARDTHOF_2 = 'de:08118:5626:0:4'  # eRtg Glemstal (Schwieberdingen Schwieberdingen)
	HARDTHOF = 'de:08118:5626'
	HARDTHOF_1 = 'de:08118:5626:0:3'  # eRtg Pulverdingen B10 (Schwieberdingen Schwieberdingen)
	HARDTHOF_2 = 'de:08118:5626:0:4'  # eRtg Glemstal (Schwieberdingen Schwieberdingen)
	KALLENBERG_LACKFABRIK = 'de:08118:5627'
	KALLENBERG_LACKFABRIK_1 = 'de:08118:5627:0:3'  # Rtg Müllerheim (Korntal-Münchingen Münchingen)
	KALLENBERG_LACKFABRIK_2 = 'de:08118:5627:0:4'  # Rtg Rasthaus (Korntal-Münchingen Münchingen)
	MÜNCH__KALLENBERG_LACKFABRIK = 'de:08118:5627'
	MÜNCH__KALLENBERG_LACKFABRIK_1 = 'de:08118:5627:0:3'  # Rtg Müllerheim (Korntal-Münchingen Münchingen)
	MÜNCH__KALLENBERG_LACKFABRIK_2 = 'de:08118:5627:0:4'  # Rtg Rasthaus (Korntal-Münchingen Münchingen)
	ENZW__PULVERDINGEN_ORT = 'de:08118:5628'
	ENZW__PULVERDINGEN_ORT_1 = 'de:08118:5628:0:3'  # eRtg Enzweihingen B10 (Vaihingen an der Enz Enzweihingen)
	ENZW__PULVERDINGEN_ORT_2 = 'de:08118:5628:0:4'  # eRtg Pulverdingen B10 (Vaihingen an der Enz Enzweihingen)
	PULVERDINGEN_ORT = 'de:08118:5628'
	PULVERDINGEN_ORT_1 = 'de:08118:5628:0:3'  # eRtg Enzweihingen B10 (Vaihingen an der Enz Enzweihingen)
	PULVERDINGEN_ORT_2 = 'de:08118:5628:0:4'  # eRtg Pulverdingen B10 (Vaihingen an der Enz Enzweihingen)
	NECKARZENTRUM = 'de:08118:5629:0:3'  # eRtg Schießtal und Adlerplatz (Remseck am Neckar Hochberg)
	HOCHBERG_NECKARZENTRUM = 'de:08118:5629:0:3'  # eRtg Schießtal und Adlerplatz (Remseck am Neckar Hochberg)
	HOHENHASLACH_SCHULE = 'de:08118:5630:0:3'  # eRtg Allmandklinge (Sachsenheim Hohenhaslach)
	VAIH__E__GRABENSTR = 'de:08118:5631'
	VAIH__E__GRABENSTR_1 = 'de:08118:5631:1:1'  # Bstg 1 (Vaihingen an der Enz Vaihingen (E))
	VAIH__E__GRABENSTR_2 = 'de:08118:5631:1:2'  # Bstg 2 (Vaihingen an der Enz Vaihingen (E))
	VAIH__E__GRABENSTR_3 = 'de:08118:5631:1:3'  # Bstg 3 (Vaihingen an der Enz Vaihingen (E))
	VAIH__E__GRABENSTR_4 = 'de:08118:5631:1:4'  # Bstg 4 (Vaihingen an der Enz Vaihingen (E))
	VAIH__E__GRABENSTR_5 = 'de:08118:5631:1:6'  # Bstg 6 (Vaihingen an der Enz Vaihingen (E))
	VAIH__E__GRABENSTR_6 = 'de:08118:5631:1:7'  # Bstg 7 (Vaihingen an der Enz Vaihingen (E))
	GRABENSTR___STADTM = 'de:08118:5631'
	GRABENSTR___STADTM_1 = 'de:08118:5631:1:1'  # Bstg 1 (Vaihingen an der Enz Vaihingen (E))
	GRABENSTR___STADTM_2 = 'de:08118:5631:1:2'  # Bstg 2 (Vaihingen an der Enz Vaihingen (E))
	GRABENSTR___STADTM_3 = 'de:08118:5631:1:3'  # Bstg 3 (Vaihingen an der Enz Vaihingen (E))
	GRABENSTR___STADTM_4 = 'de:08118:5631:1:4'  # Bstg 4 (Vaihingen an der Enz Vaihingen (E))
	GRABENSTR___STADTM_5 = 'de:08118:5631:1:6'  # Bstg 6 (Vaihingen an der Enz Vaihingen (E))
	GRABENSTR___STADTM_6 = 'de:08118:5631:1:7'  # Bstg 7 (Vaihingen an der Enz Vaihingen (E))
	MÜNCHINGEN = 'de:08118:5632'
	MÜNCHINGEN_1 = 'de:08118:5632:1:1'  # Rtg Korntal (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_2 = 'de:08118:5632:1:2'  # Rtg Weissach (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_3 = 'de:08118:5632:2:3'  # Bus (Korntal-Münchingen Münchingen)
	ASPERG_IM_WALDECK = 'de:08118:5634'
	ASPERG_IM_WALDECK_1 = 'de:08118:5634:0:3'  # eRtg Neckarstraße (Asperg Asperg)
	ASPERG_IM_WALDECK_2 = 'de:08118:5634:0:4'  # eRtg Waldeck (Asperg Asperg)
	IM_WALDECK = 'de:08118:5634'
	IM_WALDECK_1 = 'de:08118:5634:0:3'  # eRtg Neckarstraße (Asperg Asperg)
	IM_WALDECK_2 = 'de:08118:5634:0:4'  # eRtg Waldeck (Asperg Asperg)
	UNTERRIEX__INDUSTRIEGEBIET = 'de:08118:5635'
	UNTERRIEX__INDUSTRIEGEBIET_1 = 'de:08118:5635:0:3'  # Rtg Hauptstr. (Markgröningen Unterriexingen)
	UNTERRIEX__INDUSTRIEGEBIET_2 = 'de:08118:5635:0:4'  # Rtg Markgröningen (Markgröningen Unterriexingen)
	UNTERRIEXINGEN_HAUPTSTR = 'de:08118:5636'
	UNTERRIEXINGEN_HAUPTSTR_1 = 'de:08118:5636:0:3'  # eRtg Industriegebiet (Markgröningen Unterriexingen)
	UNTERRIEXINGEN_HAUPTSTR_2 = 'de:08118:5636:0:4'  # eRtg Am Türmle/Glemsbrücke (Markgröningen Unterriexingen)
	UNTERRIEX__GLEMSBRÜCKE = 'de:08118:5637'
	UNTERRIEX__GLEMSBRÜCKE_1 = 'de:08118:5637:0:3'  # eRtg Mühlstraße (Markgröningen Unterriexingen)
	UNTERRIEX__GLEMSBRÜCKE_2 = 'de:08118:5637:0:4'  # eRtg Hauptstraße (Markgröningen Unterriexingen)
	GLEMSBRÜCKE = 'de:08118:5637'
	GLEMSBRÜCKE_1 = 'de:08118:5637:0:3'  # eRtg Mühlstraße (Markgröningen Unterriexingen)
	GLEMSBRÜCKE_2 = 'de:08118:5637:0:4'  # eRtg Hauptstraße (Markgröningen Unterriexingen)
	BILDUNGSCAMPUS = 'de:08118:5638'
	BILDUNGSCAMPUS_1 = 'de:08118:5638:0:3'  # eRtg Oberriexinger Straße (Sachsenheim Großsachsenheim)
	BILDUNGSCAMPUS_2 = 'de:08118:5638:0:4'  # eRtg Hauptstraße (Sachsenheim Großsachsenheim)
	GROSSSACHSH__BILDUNGSCAMPUS = 'de:08118:5638'
	GROSSSACHSH__BILDUNGSCAMPUS_1 = 'de:08118:5638:0:3'  # eRtg Oberriexinger Straße (Sachsenheim Großsachsenheim)
	GROSSSACHSH__BILDUNGSCAMPUS_2 = 'de:08118:5638:0:4'  # eRtg Hauptstraße (Sachsenheim Großsachsenheim)
	SACHSENHEIM = 'de:08118:5639'
	SACHSENHEIM_1 = 'de:08118:5639:1:2'  # Gleis 2 (Sachsenheim Großsachsenheim)
	SACHSENHEIM_2 = 'de:08118:5639:1:3'  # Gleis 3 (Sachsenheim Großsachsenheim)
	SACHSENHEIM_3 = 'de:08118:5639:2:1'  # Bstg 1 (Sachsenheim Großsachsenheim)
	SACHSENHEIM_4 = 'de:08118:5639:2:2'  # Bstg 2 (Sachsenheim Großsachsenheim)
	SACHSENHEIM_5 = 'de:08118:5639:2:3'  # Bstg 3 (Sachsenheim Großsachsenheim)
	SACHSENHEIM_6 = 'de:08118:5639:2:4'  # Bstg 4 (Sachsenheim Großsachsenheim)
	SACHSENHEIM_7 = 'de:08118:5639:2:5'  # Bstg 5 (Sachsenheim Großsachsenheim)
	SACHSENHEIMER_STRASSE = 'de:08118:5640'
	SACHSENHEIMER_STRASSE_1 = 'de:08118:5640:0:3'  # eRtg Oberriexinger Straße (Oberriexingen Oberriexingen)
	SACHSENHEIMER_STRASSE_2 = 'de:08118:5640:0:4'  # eRtg Mühlstraße (Oberriexingen Oberriexingen)
	OBERRIEX__SACHSENHEIMER_STR = 'de:08118:5640'
	OBERRIEX__SACHSENHEIMER_STR_1 = 'de:08118:5640:0:3'  # eRtg Oberriexinger Straße (Oberriexingen Oberriexingen)
	OBERRIEX__SACHSENHEIMER_STR_2 = 'de:08118:5640:0:4'  # eRtg Mühlstraße (Oberriexingen Oberriexingen)
	OBERRIEXINGEN_MÜHLSTR = 'de:08118:5641:0:3'  # eRtg Theodor-Heuss-Straße (Oberriexingen Oberriexingen)
	BISSINGEN_LUDWIGSBURGER_STR = 'de:08118:5643'
	BISSINGEN_LUDWIGSBURGER_STR_1 = 'de:08118:5643:0:3'  # eRtg Rathaus (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_LUDWIGSBURGER_STR_2 = 'de:08118:5643:0:4'  # eRtg Tamm (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_RATHAUS = 'de:08118:5644'
	BISSINGEN_RATHAUS_1 = 'de:08118:5644:0:3'  # Rtg Schillerschule (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_RATHAUS_2 = 'de:08118:5644:0:4'  # Rtg Gerok-/Ludwigsburger Str. (Bietigheim-Bissingen Bissingen (LB))
	UNTERMBERG_BISSINGER_STR = 'de:08118:5645'
	UNTERMBERG_BISSINGER_STR_1 = 'de:08118:5645:0:3'  # eRtg Am Türmle/Haupstraße (Bietigheim-Bissingen Untermberg)
	UNTERMBERG_BISSINGER_STR_2 = 'de:08118:5645:0:4'  # eRtg Rathaus/Hallenbad (Bietigheim-Bissingen Untermberg)
	BIETIGH__KAMMGARNSPINNEREI = 'de:08118:5646'
	BIETIGH__KAMMGARNSPINNEREI_1 = 'de:08118:5646:0:3'  # eRtg Pfarrstraße (Bietigheim-Bissingen Bietigheim)
	BIETIGH__KAMMGARNSPINNEREI_2 = 'de:08118:5646:0:4'  # Ankunft (Bietigheim-Bissingen Bietigheim)
	KAMMGARNSPINNEREI = 'de:08118:5646'
	KAMMGARNSPINNEREI_1 = 'de:08118:5646:0:3'  # eRtg Pfarrstraße (Bietigheim-Bissingen Bietigheim)
	KAMMGARNSPINNEREI_2 = 'de:08118:5646:0:4'  # Ankunft (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_ELLENTALGYMNASIEN = 'de:08118:5647'
	BIETIGHEIM_ELLENTALGYMNASIEN_1 = 'de:08118:5647:1:1'  # Position 1 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_ELLENTALGYMNASIEN_2 = 'de:08118:5647:1:2'  # Position 2 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_ELLENTALGYMNASIEN_3 = 'de:08118:5647:1:4'  # Position 4 (Bietigheim-Bissingen Bietigheim)
	ELLENTALGYMNASIEN = 'de:08118:5647'
	ELLENTALGYMNASIEN_1 = 'de:08118:5647:1:1'  # Position 1 (Bietigheim-Bissingen Bietigheim)
	ELLENTALGYMNASIEN_2 = 'de:08118:5647:1:2'  # Position 2 (Bietigheim-Bissingen Bietigheim)
	ELLENTALGYMNASIEN_3 = 'de:08118:5647:1:4'  # Position 4 (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_AUWIESENBRÜCKE = 'de:08118:5648'
	BIETIGHEIM_AUWIESENBRÜCKE_1 = 'de:08118:5648:0:3'  # Rtg Schillerplatz/Kronen/Nord (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_AUWIESENBRÜCKE_2 = 'de:08118:5648:0:4'  # Rtg DLW/Bahnhof/KH (Bietigheim-Bissingen Bietigheim)
	AUWIESENBRÜCKE = 'de:08118:5648'
	AUWIESENBRÜCKE_1 = 'de:08118:5648:0:3'  # Rtg Schillerplatz/Kronen/Nord (Bietigheim-Bissingen Bietigheim)
	AUWIESENBRÜCKE_2 = 'de:08118:5648:0:4'  # Rtg DLW/Bahnhof/KH (Bietigheim-Bissingen Bietigheim)
	GROSSSACHSENHEIM_LUDWIGSB__STR = 'de:08118:5649'
	GROSSSACHSENHEIM_LUDWIGSB__STR_1 = 'de:08118:5649:0:3'  # Rtg Zimmerer Pfad (Sachsenheim Großsachsenheim)
	GROSSSACHSENHEIM_LUDWIGSB__STR_2 = 'de:08118:5649:0:4'  # Rtg Hauptstr./Sachsenheim (Sachsenheim Großsachsenheim)
	GROSSSACHSH__METTERBRÜCKE = 'de:08118:5650'
	GROSSSACHSH__METTERBRÜCKE_1 = 'de:08118:5650:1:3'  # eRtg Fessler Mühle (Sachsenheim Großsachsenheim)
	GROSSSACHSH__METTERBRÜCKE_2 = 'de:08118:5650:1:4'  # eRtg Sachsenheim (Sachsenheim Großsachsenheim)
	METTERBRÜCKE = 'de:08118:5650'
	METTERBRÜCKE_1 = 'de:08118:5650:1:3'  # eRtg Fessler Mühle (Sachsenheim Großsachsenheim)
	METTERBRÜCKE_2 = 'de:08118:5650:1:4'  # eRtg Sachsenheim (Sachsenheim Großsachsenheim)
	SERSHEIM_VAIHINGER_STR = 'de:08118:5651:1:3'  # eRtg Sedanstraße/Industriestraße (Sersheim Sersheim)
	KLEINGLATTB__GRUNDSCHULE = 'de:08118:5653'
	KLEINGLATTB__GRUNDSCHULE_1 = 'de:08118:5653:1:3'  # eRtg Vaihingen (E) Bahnhof (Vaihingen an der Enz Kleinglattbach)
	KLEINGLATTB__GRUNDSCHULE_2 = 'de:08118:5653:1:4'  # eRtg Osterwiesenweg (Vaihingen an der Enz Kleinglattbach)
	ENSINGEN_KLEINGLATTB__STR = 'de:08118:5654'
	ENSINGEN_KLEINGLATTB__STR_1 = 'de:08118:5654:0:3'  # Rtg Feuerwehr/Vaihingen (Vaihingen an der Enz Ensingen)
	ENSINGEN_KLEINGLATTB__STR_2 = 'de:08118:5654:0:4'  # Rtg Horrheim/Kleinglattbach (Vaihingen an der Enz Ensingen)
	KLEINGLATTBACHER_STRASSE = 'de:08118:5654'
	KLEINGLATTBACHER_STRASSE_1 = 'de:08118:5654:0:3'  # Rtg Feuerwehr/Vaihingen (Vaihingen an der Enz Ensingen)
	KLEINGLATTBACHER_STRASSE_2 = 'de:08118:5654:0:4'  # Rtg Horrheim/Kleinglattbach (Vaihingen an der Enz Ensingen)
	BIETIGHEIM_KRANKENHAUS = 'de:08118:5656'
	BIETIGHEIM_KRANKENHAUS_1 = 'de:08118:5656:0:3'  # eRtg Bolzstraße (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_KRANKENHAUS_2 = 'de:08118:5656:0:4'  # eRtg DLW/Auwiesenbrücke (Bietigheim-Bissingen Bietigheim)
	ALDINGEN_BERLINER_STRASSE = 'de:08118:5657'
	ALDINGEN_BERLINER_STRASSE_1 = 'de:08118:5657:0:3'  # eRtg Goethestraße (Remseck am Neckar Aldingen)
	ALDINGEN_BERLINER_STRASSE_2 = 'de:08118:5657:0:4'  # Rtg Pattonv. Realschule (Remseck am Neckar Aldingen)
	BIETIGHEIM_BOLZSTRASSE = 'de:08118:5658'
	BIETIGHEIM_BOLZSTRASSE_1 = 'de:08118:5658:0:3'  # Rtg KH (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_BOLZSTRASSE_2 = 'de:08118:5658:0:4'  # Rtg Schillerplatz (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_KRONENZENTRUM = 'de:08118:5659'
	BIETIGHEIM_KRONENZENTRUM_1 = 'de:08118:5659:0:3'  # eRtg DLW/Pfarrstr. (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_KRONENZENTRUM_2 = 'de:08118:5659:0:4'  # eRtg Bolzstraße/Schillerplatz/Enzbrücke (Bietigheim-Bissingen Bietigheim)
	KRONENZENTRUM = 'de:08118:5659'
	KRONENZENTRUM_1 = 'de:08118:5659:0:3'  # eRtg DLW/Pfarrstr. (Bietigheim-Bissingen Bietigheim)
	KRONENZENTRUM_2 = 'de:08118:5659:0:4'  # eRtg Bolzstraße/Schillerplatz/Enzbrücke (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_LÖCHGAUER_STR = 'de:08118:5660'
	BIETIGHEIM_LÖCHGAUER_STR_1 = 'de:08118:5660:0:3'  # Rtg A.-Visconti-Str. (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_LÖCHGAUER_STR_2 = 'de:08118:5660:0:4'  # Rtg Schillerplatz/Pfarrstr. (Bietigheim-Bissingen Bietigheim)
	LÖCHGAUER_STRASSE = 'de:08118:5660'
	LÖCHGAUER_STRASSE_1 = 'de:08118:5660:0:3'  # Rtg A.-Visconti-Str. (Bietigheim-Bissingen Bietigheim)
	LÖCHGAUER_STRASSE_2 = 'de:08118:5660:0:4'  # Rtg Schillerplatz/Pfarrstr. (Bietigheim-Bissingen Bietigheim)
	BIETIGH__HILLER__LÖCHGAUER_STR = 'de:08118:5661'
	BIETIGH__HILLER__LÖCHGAUER_STR_1 = 'de:08118:5661:0:3'  # eRtg Hillerplatz (Bietigheim-Bissingen Bietigheim)
	BIETIGH__HILLER__LÖCHGAUER_STR_2 = 'de:08118:5661:0:4'  # Rtg Pfarrstr. (Bietigheim-Bissingen Bietigheim)
	HILLER__LÖCHGAUER_STRASSE = 'de:08118:5661'
	HILLER__LÖCHGAUER_STRASSE_1 = 'de:08118:5661:0:3'  # eRtg Hillerplatz (Bietigheim-Bissingen Bietigheim)
	HILLER__LÖCHGAUER_STRASSE_2 = 'de:08118:5661:0:4'  # Rtg Pfarrstr. (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_PFARRSTR = 'de:08118:5662'
	BIETIGHEIM_PFARRSTR_1 = 'de:08118:5662:0:3'  # eRtg Hillerplatz (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_PFARRSTR_2 = 'de:08118:5662:0:4'  # eRtg Kronenzentrum (Bietigheim-Bissingen Bietigheim)
	PFARRSTRASSE = 'de:08118:5662'
	PFARRSTRASSE_1 = 'de:08118:5662:0:3'  # eRtg Hillerplatz (Bietigheim-Bissingen Bietigheim)
	PFARRSTRASSE_2 = 'de:08118:5662:0:4'  # eRtg Kronenzentrum (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_HILLERPLATZ = 'de:08118:5664'
	BIETIGHEIM_HILLERPLATZ_1 = 'de:08118:5664:0:3'  # Rtg Helenenburgweg (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_HILLERPLATZ_2 = 'de:08118:5664:0:4'  # Rtg Löchgauer Str. (Bietigheim-Bissingen Bietigheim)
	HILLERPLATZ = 'de:08118:5664'
	HILLERPLATZ_1 = 'de:08118:5664:0:3'  # Rtg Helenenburgweg (Bietigheim-Bissingen Bietigheim)
	HILLERPLATZ_2 = 'de:08118:5664:0:4'  # Rtg Löchgauer Str. (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_HELENENBURGWEG = 'de:08118:5666'
	BIETIGHEIM_HELENENBURGWEG_1 = 'de:08118:5666:0:3'  # Rtg Kindergarten (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_HELENENBURGWEG_2 = 'de:08118:5666:0:4'  # Rtg Hillerstr. (Bietigheim-Bissingen Bietigheim)
	HELENENBURGWEG = 'de:08118:5666'
	HELENENBURGWEG_1 = 'de:08118:5666:0:3'  # Rtg Kindergarten (Bietigheim-Bissingen Bietigheim)
	HELENENBURGWEG_2 = 'de:08118:5666:0:4'  # Rtg Hillerstr. (Bietigheim-Bissingen Bietigheim)
	METTERZ__KINDERGARTEN = 'de:08118:5667'
	METTERZ__KINDERGARTEN_1 = 'de:08118:5667:0:3'  # eRtg Kleinsachsenheimer Straße (Bietigheim-Bissingen Metterzimmern)
	METTERZ__KINDERGARTEN_2 = 'de:08118:5667:0:4'  # eRtg Bietigheimer Straße (Bietigheim-Bissingen Metterzimmern)
	KLEINSACHSENHEIMER_STRASSE_1 = 'de:08118:5668:0:3'  # Rtg Kindergarten (Bietigheim-Bissingen Metterzimmern)
	KLEINSACHSENHEIMER_STRASSE_2 = 'de:08118:5668:0:4'  # Rtg Sachsenheim (Bietigheim-Bissingen Metterzimmern)
	KLEINSACHSENHEIMER_STRASSE_3 = 'de:08118:5674:0:3'  # eRtg Sachsenheim (Sachsenheim Großsachsenheim)
	KLEINSACHSENHEIMER_STRASSE_4 = 'de:08118:5674:0:4'  # eRtg Großsachsenheimer Straße (Sachsenheim Großsachsenheim)
	METTERZ__KLEINSACHSHEIMER_STR = 'de:08118:5668'
	METTERZ__KLEINSACHSHEIMER_STR_1 = 'de:08118:5668:0:3'  # Rtg Kindergarten (Bietigheim-Bissingen Metterzimmern)
	METTERZ__KLEINSACHSHEIMER_STR_2 = 'de:08118:5668:0:4'  # Rtg Sachsenheim (Bietigheim-Bissingen Metterzimmern)
	KLEINSACHSH__BIETIGH__STR = 'de:08118:5669'
	KLEINSACHSH__BIETIGH__STR_1 = 'de:08118:5669:0:3'  # eRtg Neuweiler Straße (Sachsenheim Kleinsachsenheim)
	KLEINSACHSH__BIETIGH__STR_2 = 'de:08118:5669:0:4'  # eRtg Zeil (Sachsenheim Kleinsachsenheim)
	GROSSSACHSENHEIMER_STRASSE = 'de:08118:5671:0:3'  # eRtg Wilhelmstraße (Sachsenheim Kleinsachsenheim)
	KLEINSACHSH__GROSSSACHSH__STR = 'de:08118:5671:0:3'  # eRtg Wilhelmstraße (Sachsenheim Kleinsachsenheim)
	KLEINSACHSHEIM_NEUWEILER_STR = 'de:08118:5672:0:3'  # eRtg Kleinsachsenheimer Straße (Sachsenheim Kleinsachsenheim)
	NEUWEILER_STRASSE = 'de:08118:5672:0:3'  # eRtg Kleinsachsenheimer Straße (Sachsenheim Kleinsachsenheim)
	GROSSSACHSH__KLEINSACHSH__STR = 'de:08118:5674'
	GROSSSACHSH__KLEINSACHSH__STR_1 = 'de:08118:5674:0:3'  # eRtg Sachsenheim (Sachsenheim Großsachsenheim)
	GROSSSACHSH__KLEINSACHSH__STR_2 = 'de:08118:5674:0:4'  # eRtg Großsachsenheimer Straße (Sachsenheim Großsachsenheim)
	BIETIGH__BUCH_GRÖN__WEG = 'de:08118:5675'
	BIETIGH__BUCH_GRÖN__WEG_1 = 'de:08118:5675:0:3'  # eRtg Bietigheim (Bietigheim-Bissingen Bietigheim)
	BIETIGH__BUCH_GRÖN__WEG_2 = 'de:08118:5675:0:4'  # eRtg Buch Sonderschule (Bietigheim-Bissingen Bietigheim)
	BUCH_GRÖNINGER_WEG = 'de:08118:5675'
	BUCH_GRÖNINGER_WEG_1 = 'de:08118:5675:0:3'  # eRtg Bietigheim (Bietigheim-Bissingen Bietigheim)
	BUCH_GRÖNINGER_WEG_2 = 'de:08118:5675:0:4'  # eRtg Buch Sonderschule (Bietigheim-Bissingen Bietigheim)
	BUCH_DRESDNER_STRASSE = 'de:08118:5676'
	BUCH_DRESDNER_STRASSE_1 = 'de:08118:5676:0:3'  # eRtg Buch Gröninger Weg (Bietigheim-Bissingen Bietigheim)
	BUCH_DRESDNER_STRASSE_2 = 'de:08118:5676:0:4'  # eRtg Buch Buchschule (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_BUCH_DRESDNER_STR = 'de:08118:5676'
	BIETIGHEIM_BUCH_DRESDNER_STR_1 = 'de:08118:5676:0:3'  # eRtg Buch Gröninger Weg (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_BUCH_DRESDNER_STR_2 = 'de:08118:5676:0:4'  # eRtg Buch Buchschule (Bietigheim-Bissingen Bietigheim)
	GROSSSACHSENHEIM_HAUPTSTR = 'de:08118:5677'
	GROSSSACHSENHEIM_HAUPTSTR_1 = 'de:08118:5677:1:3'  # Rtg Ludwigsburger Str. (Sachsenheim Großsachsenheim)
	GROSSSACHSENHEIM_HAUPTSTR_2 = 'de:08118:5677:1:4'  # Rtg Bahnhof (Sachsenheim Großsachsenheim)
	BIETIGH__BUCH_BUCHSCHULE = 'de:08118:5678'
	BIETIGH__BUCH_BUCHSCHULE_1 = 'de:08118:5678:0:3'  # eRtg Buch Sonderschule (Bietigheim-Bissingen Bietigheim)
	BIETIGH__BUCH_BUCHSCHULE_2 = 'de:08118:5678:0:4'  # eRtg Buch Breslauer Straße (Bietigheim-Bissingen Bietigheim)
	BUCH_BUCHSCHULE = 'de:08118:5678'
	BUCH_BUCHSCHULE_1 = 'de:08118:5678:0:3'  # eRtg Buch Sonderschule (Bietigheim-Bissingen Bietigheim)
	BUCH_BUCHSCHULE_2 = 'de:08118:5678:0:4'  # eRtg Buch Breslauer Straße (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_BUCH_BRESLAUER_STR = 'de:08118:5679'
	BIETIGHEIM_BUCH_BRESLAUER_STR_1 = 'de:08118:5679:0:3'  # eRtg Buch Buchschule (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_BUCH_BRESLAUER_STR_2 = 'de:08118:5679:0:4'  # eRtg Buch Posener Straße (Bietigheim-Bissingen Bietigheim)
	BUCH_BRESLAUER_STRASSE = 'de:08118:5679'
	BUCH_BRESLAUER_STRASSE_1 = 'de:08118:5679:0:3'  # eRtg Buch Buchschule (Bietigheim-Bissingen Bietigheim)
	BUCH_BRESLAUER_STRASSE_2 = 'de:08118:5679:0:4'  # eRtg Buch Posener Straße (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_BUCH_POSENER_STR = 'de:08118:5680'
	BIETIGHEIM_BUCH_POSENER_STR_1 = 'de:08118:5680:0:3'  # eRtg Buch Breslauer Straße (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_BUCH_POSENER_STR_2 = 'de:08118:5680:0:4'  # eRtg Buch Sucystraße (Bietigheim-Bissingen Bietigheim)
	BUCH_POSENER_STRASSE = 'de:08118:5680'
	BUCH_POSENER_STRASSE_1 = 'de:08118:5680:0:3'  # eRtg Buch Breslauer Straße (Bietigheim-Bissingen Bietigheim)
	BUCH_POSENER_STRASSE_2 = 'de:08118:5680:0:4'  # eRtg Buch Sucystraße (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_BUCH_SUCYSTR = 'de:08118:5681:0:3'  # eRtg Buch Posener Straße (Bietigheim-Bissingen Bietigheim)
	BUCH_SUCYSTRASSE = 'de:08118:5681:0:3'  # eRtg Buch Posener Straße (Bietigheim-Bissingen Bietigheim)
	BISSINGEN_KARLSTRASSE = 'de:08118:5683'
	BISSINGEN_KARLSTRASSE_1 = 'de:08118:5683:0:3'  # Rtg Panoramastr. (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_KARLSTRASSE_2 = 'de:08118:5683:0:4'  # Rtg Bahnhof (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_PANORAMASTR = 'de:08118:5684'
	BISSINGEN_PANORAMASTR_1 = 'de:08118:5684:0:3'  # eRtg Waldschule (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_PANORAMASTR_2 = 'de:08118:5684:0:4'  # eRtg Karlstraße (Bietigheim-Bissingen Bissingen (LB))
	WALDSCHULE = 'de:08118:5685'
	WALDSCHULE_1 = 'de:08118:5685:0:3'  # eRtg Grotzstraße (Bietigheim-Bissingen Bissingen (LB))
	WALDSCHULE_2 = 'de:08118:5685:0:4'  # eRtg Panoramastraße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_WALDSCHULE = 'de:08118:5685'
	BISSINGEN_WALDSCHULE_1 = 'de:08118:5685:0:3'  # eRtg Grotzstraße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_WALDSCHULE_2 = 'de:08118:5685:0:4'  # eRtg Panoramastraße (Bietigheim-Bissingen Bissingen (LB))
	GROTZSTRASSE = 'de:08118:5686'
	GROTZSTRASSE_1 = 'de:08118:5686:0:3'  # eRtg Rathaus (Bietigheim-Bissingen Bissingen (LB))
	GROTZSTRASSE_2 = 'de:08118:5686:0:4'  # eRtg Waldschule (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_GROTZSTRASSE = 'de:08118:5686'
	BISSINGEN_GROTZSTRASSE_1 = 'de:08118:5686:0:3'  # eRtg Rathaus (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_GROTZSTRASSE_2 = 'de:08118:5686:0:4'  # eRtg Waldschule (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_SCHUBARTSTR = 'de:08118:5687'
	BISSINGEN_SCHUBARTSTR_1 = 'de:08118:5687:0:3'  # Rtg Zahnklinik (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_SCHUBARTSTR_2 = 'de:08118:5687:0:4'  # Rtg Friedhof (Bietigheim-Bissingen Bissingen (LB))
	SCHUBARTSTRASSE_1 = 'de:08118:5687:0:3'  # Rtg Zahnklinik (Bietigheim-Bissingen Bissingen (LB))
	SCHUBARTSTRASSE_2 = 'de:08118:5687:0:4'  # Rtg Friedhof (Bietigheim-Bissingen Bissingen (LB))
	SCHUBARTSTRASSE_3 = 'de:08117:7015:0:H1'  # Rtg Ludwigstr. (Göppingen Faurndau)
	SCHUBARTSTRASSE_4 = 'de:08117:7015:0:R1'  # Rtg Göppingen (Göppingen Faurndau)
	ZAHNKLINIK = 'de:08118:5689'
	ZAHNKLINIK_1 = 'de:08118:5689:0:3'  # eRtg Neuer Friedhof (Bietigheim-Bissingen Bissingen (LB))
	ZAHNKLINIK_2 = 'de:08118:5689:0:4'  # eRtg Kelterstraße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_ZAHNKLINIK = 'de:08118:5689'
	BISSINGEN_ZAHNKLINIK_1 = 'de:08118:5689:0:3'  # eRtg Neuer Friedhof (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_ZAHNKLINIK_2 = 'de:08118:5689:0:4'  # eRtg Kelterstraße (Bietigheim-Bissingen Bissingen (LB))
	ANTONIA_VISCONTI_STRASSE = 'de:08118:5690'
	ANTONIA_VISCONTI_STRASSE_1 = 'de:08118:5690:0:3'  # eRtg Hillerplatz (Bietigheim-Bissingen Bietigheim)
	ANTONIA_VISCONTI_STRASSE_2 = 'de:08118:5690:0:4'  # eRtg Waldhof (Bietigheim-Bissingen Bietigheim)
	BIETIGH__ANTONIA_VISCONTI_STR = 'de:08118:5690'
	BIETIGH__ANTONIA_VISCONTI_STR_1 = 'de:08118:5690:0:3'  # eRtg Hillerplatz (Bietigheim-Bissingen Bietigheim)
	BIETIGH__ANTONIA_VISCONTI_STR_2 = 'de:08118:5690:0:4'  # eRtg Waldhof (Bietigheim-Bissingen Bietigheim)
	UNTERMBERG_AM_TÜRMLE = 'de:08118:5691'
	UNTERMBERG_AM_TÜRMLE_1 = 'de:08118:5691:1:3'  # eRtg Hauptstraße (Bietigheim-Bissingen Untermberg)
	UNTERMBERG_AM_TÜRMLE_2 = 'de:08118:5691:1:4'  # eRtg Bissinger Straße (Bietigheim-Bissingen Untermberg)
	AM_TÜRMLE = 'de:08118:5691'
	AM_TÜRMLE_1 = 'de:08118:5691:1:3'  # eRtg Hauptstraße (Bietigheim-Bissingen Untermberg)
	AM_TÜRMLE_2 = 'de:08118:5691:1:4'  # eRtg Bissinger Straße (Bietigheim-Bissingen Untermberg)
	BIETIGHEIM_WALDHOF = 'de:08118:5692'
	BIETIGHEIM_WALDHOF_1 = 'de:08118:5692:0:3'  # eRtg Weißenhof Ost (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_WALDHOF_2 = 'de:08118:5692:0:4'  # eRtg Antonia-Visconti-Straße (Bietigheim-Bissingen Bietigheim)
	WALDHOF = 'de:08118:5692'
	WALDHOF_1 = 'de:08118:5692:0:3'  # eRtg Weißenhof Ost (Bietigheim-Bissingen Bietigheim)
	WALDHOF_2 = 'de:08118:5692:0:4'  # eRtg Antonia-Visconti-Straße (Bietigheim-Bissingen Bietigheim)
	WEISSENHOF_OST = 'de:08118:5693'
	WEISSENHOF_OST_1 = 'de:08118:5693:0:3'  # eRtg Wette (Löchgau Löchgau)
	WEISSENHOF_OST_2 = 'de:08118:5693:0:4'  # eRtg Waldhof (Löchgau Löchgau)
	LÖCHGAU_WEISSENHOF_OST = 'de:08118:5693'
	LÖCHGAU_WEISSENHOF_OST_1 = 'de:08118:5693:0:3'  # eRtg Wette (Löchgau Löchgau)
	LÖCHGAU_WEISSENHOF_OST_2 = 'de:08118:5693:0:4'  # eRtg Waldhof (Löchgau Löchgau)
	BIETIGHEIM_ENZBRÜCKE = 'de:08118:5694:0:3'  # eRtg Auwiesenbrücke (Bietigheim-Bissingen Bietigheim)
	ENZBRÜCKE_1 = 'de:08118:5694:0:3'  # eRtg Auwiesenbrücke (Bietigheim-Bissingen Bietigheim)
	ENZBRÜCKE_2 = 'de:08118:5751:0:3'  # eRtg Abzweig (Vaihingen an der Enz Vaihingen (E))
	ENZBRÜCKE_3 = 'de:08118:5751:0:4'  # eRtg Grabenstr. (Stadtm.) (Vaihingen an der Enz Vaihingen (E))
	ERLIGHEIM = 'de:08118:5695'
	ERLIGHEIM_1 = 'de:08118:5695:0:3'  # eRtg Sonnenhalde/Schulzentrum (Erligheim Erligheim)
	ERLIGHEIM_2 = 'de:08118:5695:0:4'  # eRtg Erligheimer Straße/Besigheimer Str. (Erligheim Erligheim)
	HOFENER_STRASSE = 'de:08118:5697:0:3'  # eRtg Hauptstraße (Bönnigheim Bönnigheim)
	BÖNNIGHEIM_HOFENER_STR = 'de:08118:5697:0:3'  # eRtg Hauptstraße (Bönnigheim Bönnigheim)
	AM_SCHLOSS = 'de:08118:5698:0:3'  # eRtg Burgplatz (Bönnigheim Bönnigheim)
	BÖNNIGHEIM_AM_SCHLOSS = 'de:08118:5698:0:3'  # eRtg Burgplatz (Bönnigheim Bönnigheim)
	BÖNNIGHEIM_BURGPLATZ = 'de:08118:5700'
	BÖNNIGHEIM_BURGPLATZ_1 = 'de:08118:5700:1:3'  # Rtg Freibad (Bönnigheim Bönnigheim)
	BÖNNIGHEIM_BURGPLATZ_2 = 'de:08118:5700:1:4'  # eRtg Schulzentrum (Bönnigheim Bönnigheim)
	BIETIGHEIM_BADEPARK = 'de:08118:5701'
	BIETIGHEIM_BADEPARK_1 = 'de:08118:5701:0:3'  # Rtg Fischerpfad (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_BADEPARK_2 = 'de:08118:5701:0:4'  # Rtg Hallenbad (Bietigheim-Bissingen Bietigheim)
	BADEPARK = 'de:08118:5701'
	BADEPARK_1 = 'de:08118:5701:0:3'  # Rtg Fischerpfad (Bietigheim-Bissingen Bietigheim)
	BADEPARK_2 = 'de:08118:5701:0:4'  # Rtg Hallenbad (Bietigheim-Bissingen Bietigheim)
	LÖCHGAU_BESIGHEIMER_STR = 'de:08118:5703'
	LÖCHGAU_BESIGHEIMER_STR_1 = 'de:08118:5703:0:3'  # eRtg Steinbachbrücke (Löchgau Löchgau)
	LÖCHGAU_BESIGHEIMER_STR_2 = 'de:08118:5703:0:4'  # eRtg Wette (Löchgau Löchgau)
	LÖCHGAU_WETTE = 'de:08118:5704'
	LÖCHGAU_WETTE_1 = 'de:08118:5704:0:3'  # eRtg Erligheimer Straße/ Löchgau Freuden (Löchgau Löchgau)
	LÖCHGAU_WETTE_2 = 'de:08118:5704:0:4'  # eRtg Weißenhof Ost/Besigh Str/Steinachbr (Löchgau Löchgau)
	WETTE = 'de:08118:5704'
	WETTE_1 = 'de:08118:5704:0:3'  # eRtg Erligheimer Straße/ Löchgau Freuden (Löchgau Löchgau)
	WETTE_2 = 'de:08118:5704:0:4'  # eRtg Weißenhof Ost/Besigh Str/Steinachbr (Löchgau Löchgau)
	FREUDENTALER_STRASSE = 'de:08118:5705'
	FREUDENTALER_STRASSE_1 = 'de:08118:5705:0:3'  # eRtg Besigheimer Straße (Löchgau Löchgau)
	FREUDENTALER_STRASSE_2 = 'de:08118:5705:0:4'  # eRtg Wette (Löchgau Löchgau)
	LÖCHGAU_FREUDENTALER_STR = 'de:08118:5705'
	LÖCHGAU_FREUDENTALER_STR_1 = 'de:08118:5705:0:3'  # eRtg Besigheimer Straße (Löchgau Löchgau)
	LÖCHGAU_FREUDENTALER_STR_2 = 'de:08118:5705:0:4'  # eRtg Wette (Löchgau Löchgau)
	FREUDENTAL_BESIGH__STR = 'de:08118:5706'
	FREUDENTAL_BESIGH__STR_1 = 'de:08118:5706:0:3'  # eRtg Erligheim (Freudental Freudental)
	FREUDENTAL_BESIGH__STR_2 = 'de:08118:5706:0:4'  # eRtg Freudentaler Straße (Freudental Freudental)
	FREUDENTAL_POST = 'de:08118:5707'
	FREUDENTAL_POST_1 = 'de:08118:5707:1:3'  # eRtg Weißenhof Ost (Freudental Freudental)
	FREUDENTAL_POST_2 = 'de:08118:5707:1:4'  # eRtg Wildparadies Tripsdrill/Staffel (Freudental Freudental)
	FREUDENTAL_GEWERBEGEBIET = 'de:08118:5708'
	FREUDENTAL_GEWERBEGEBIET_1 = 'de:08118:5708:0:3'  # eRtg Post (Freudental Freudental)
	FREUDENTAL_GEWERBEGEBIET_2 = 'de:08118:5708:0:4'  # eRtg Hohenhaslach Staffel (Freudental Freudental)
	STAFFEL = 'de:08118:5709'
	STAFFEL_1 = 'de:08118:5709:0:3'  # eRtg Post (Sachsenheim Hohenhaslach)
	STAFFEL_2 = 'de:08118:5709:0:4'  # eRtg Steige (Sachsenheim Hohenhaslach)
	HOHENHASLACH_STAFFEL = 'de:08118:5709'
	HOHENHASLACH_STAFFEL_1 = 'de:08118:5709:0:3'  # eRtg Post (Sachsenheim Hohenhaslach)
	HOHENHASLACH_STAFFEL_2 = 'de:08118:5709:0:4'  # eRtg Steige (Sachsenheim Hohenhaslach)
	HOHENHASLACH_STEIGE = 'de:08118:5710'
	HOHENHASLACH_STEIGE_1 = 'de:08118:5710:1:3'  # Rtg Staffel (Sachsenheim Hohenhaslach)
	HOHENHASLACH_STEIGE_2 = 'de:08118:5710:1:4'  # Rtg Allmandklinge/Schule (Sachsenheim Hohenhaslach)
	OCHSEN = 'de:08118:5711'
	OCHSEN_1 = 'de:08118:5711:0:3'  # eRtg Kirche (Sachsenheim Spielberg)
	OCHSEN_2 = 'de:08118:5711:0:4'  # eRtg Brunnenwiesen (Sachsenheim Spielberg)
	SPIELBERG_OCHSEN = 'de:08118:5711'
	SPIELBERG_OCHSEN_1 = 'de:08118:5711:0:3'  # eRtg Kirche (Sachsenheim Spielberg)
	SPIELBERG_OCHSEN_2 = 'de:08118:5711:0:4'  # eRtg Brunnenwiesen (Sachsenheim Spielberg)
	OCHSENBACH_KIRCHE = 'de:08118:5712'
	OCHSENBACH_KIRCHE_1 = 'de:08118:5712:0:3'  # eRtg Buchenrain (Sachsenheim Ochsenbach)
	OCHSENBACH_KIRCHE_2 = 'de:08118:5712:0:4'  # eRtg Ochsen (Sachsenheim Ochsenbach)
	OCHSENBACH_KIRBACHHOF = 'de:08118:5713'
	OCHSENBACH_KIRBACHHOF_1 = 'de:08118:5713:0:3'  # Rtg Häfnerhaslach (Sachsenheim Ochsenbach)
	OCHSENBACH_KIRBACHHOF_2 = 'de:08118:5713:0:4'  # Rtg Buchenrain (Sachsenheim Ochsenbach)
	KIRBACHHOF = 'de:08118:5713'
	KIRBACHHOF_1 = 'de:08118:5713:0:3'  # Rtg Häfnerhaslach (Sachsenheim Ochsenbach)
	KIRBACHHOF_2 = 'de:08118:5713:0:4'  # Rtg Buchenrain (Sachsenheim Ochsenbach)
	HÄFNERHASL__DORFBRUNNEN = 'de:08118:5714:1:3'  # eRtg Alle Richtungen (Sachsenheim Häfnerhaslach)
	DORFBRUNNEN = 'de:08118:5714:1:3'  # eRtg Alle Richtungen (Sachsenheim Häfnerhaslach)
	AUF_DEM_KIES = 'de:08118:5715'
	AUF_DEM_KIES_1 = 'de:08118:5715:1:3'  # eRtg Oberamteigasse (Besigheim Besigheim)
	AUF_DEM_KIES_2 = 'de:08118:5715:1:4'  # eRtg Jahnstraße (Besigheim Besigheim)
	BESIGHEIM_AUF_DEM_KIES = 'de:08118:5715'
	BESIGHEIM_AUF_DEM_KIES_1 = 'de:08118:5715:1:3'  # eRtg Oberamteigasse (Besigheim Besigheim)
	BESIGHEIM_AUF_DEM_KIES_2 = 'de:08118:5715:1:4'  # eRtg Jahnstraße (Besigheim Besigheim)
	FRIEDRICH_BREINING_STRASSE = 'de:08118:5716'
	FRIEDRICH_BREINING_STRASSE_1 = 'de:08118:5716:0:3'  # eRtg Elserring (Besigheim Besigheim)
	FRIEDRICH_BREINING_STRASSE_2 = 'de:08118:5716:0:4'  # eRtg Ulrichstraße (Besigheim Besigheim)
	BESIGHEIM_FR__BREINING_STR = 'de:08118:5716'
	BESIGHEIM_FR__BREINING_STR_1 = 'de:08118:5716:0:3'  # eRtg Elserring (Besigheim Besigheim)
	BESIGHEIM_FR__BREINING_STR_2 = 'de:08118:5716:0:4'  # eRtg Ulrichstraße (Besigheim Besigheim)
	BESIGHEIM_FRIEDHOF = 'de:08118:5718:0:3'  # eRtg Wasserturm (Besigheim Besigheim)
	BESIGHEIM_HUSARENHOF = 'de:08118:5719:0:3'  # eRtg Friedhof (Besigheim Besigheim)
	HUSARENHOF = 'de:08118:5719:0:3'  # eRtg Friedhof (Besigheim Besigheim)
	FÄHRGERGASSE = 'de:08118:5720'
	FÄHRGERGASSE_1 = 'de:08118:5720:0:3'  # Rtg Ottmarsheim (Gemmrigheim Gemmrigheim)
	FÄHRGERGASSE_2 = 'de:08118:5720:0:4'  # Rtg Besigheim (Gemmrigheim Gemmrigheim)
	GEMMRIGHEIM_FÄHRGERGASSE = 'de:08118:5720'
	GEMMRIGHEIM_FÄHRGERGASSE_1 = 'de:08118:5720:0:3'  # Rtg Ottmarsheim (Gemmrigheim Gemmrigheim)
	GEMMRIGHEIM_FÄHRGERGASSE_2 = 'de:08118:5720:0:4'  # Rtg Besigheim (Gemmrigheim Gemmrigheim)
	NIEDERE_KLINGE = 'de:08118:5721'
	NIEDERE_KLINGE_1 = 'de:08118:5721:0:3'  # eRtg Kurze Straße (Gemmrigheim Gemmrigheim)
	NIEDERE_KLINGE_2 = 'de:08118:5721:0:4'  # eRtg Carl-Benz-Straße (Gemmrigheim Gemmrigheim)
	GEMMRIGH__NIEDERE_KLINGE = 'de:08118:5721'
	GEMMRIGH__NIEDERE_KLINGE_1 = 'de:08118:5721:0:3'  # eRtg Kurze Straße (Gemmrigheim Gemmrigheim)
	GEMMRIGH__NIEDERE_KLINGE_2 = 'de:08118:5721:0:4'  # eRtg Carl-Benz-Straße (Gemmrigheim Gemmrigheim)
	PAPIERFABRIK = 'de:08118:5722'
	PAPIERFABRIK_1 = 'de:08118:5722:0:3'  # eRtg Fährgergasse/Kurze Str. (Gemmrigheim Gemmrigheim)
	PAPIERFABRIK_2 = 'de:08118:5722:0:4'  # eRtg Sportplatz (Gemmrigheim Gemmrigheim)
	GEMMRIGHEIM_PAPIERFABRIK = 'de:08118:5722'
	GEMMRIGHEIM_PAPIERFABRIK_1 = 'de:08118:5722:0:3'  # eRtg Fährgergasse/Kurze Str. (Gemmrigheim Gemmrigheim)
	GEMMRIGHEIM_PAPIERFABRIK_2 = 'de:08118:5722:0:4'  # eRtg Sportplatz (Gemmrigheim Gemmrigheim)
	KIRCHHEIM__N = 'de:08118:5723'
	KIRCHHEIM__N_1 = 'de:08118:5723:1:1'  # Gleis 1 (Kirchheim am Neckar Kirchheim (N))
	KIRCHHEIM__N_2 = 'de:08118:5723:1:2'  # Gleis 2 (Kirchheim am Neckar Kirchheim (N))
	KIRCHHEIM__N_3 = 'de:08118:5723:2:3'  # Rtg Bönnigheim (Kirchheim am Neckar Kirchheim (N))
	KIRCHHEIM__N_4 = 'de:08118:5723:2:4'  # Rtg Besigheim/Neckarwestheim (Kirchheim am Neckar Kirchheim (N))
	KIRCHHEIM__N__SCHILLERPL = 'de:08118:5724'
	KIRCHHEIM__N__SCHILLERPL_1 = 'de:08118:5724:0:3'  # eRtg Gemeindehalle (Kirchheim am Neckar Kirchheim (N))
	KIRCHHEIM__N__SCHILLERPL_2 = 'de:08118:5724:0:4'  # eRtg Kirchheim (N) (Kirchheim am Neckar Kirchheim (N))
	HOHENSTEIN_POST = 'de:08118:5725'
	HOHENSTEIN_POST_1 = 'de:08118:5725:0:3'  # eRtg Gemeindehalle (Bönnigheim Hohenstein)
	HOHENSTEIN_POST_2 = 'de:08118:5725:0:4'  # Rtg Bönnigh. Burgermühle (Bönnigheim Hohenstein)
	BÖNNIGHEIM_KIRCHHEIMER_STR = 'de:08118:5726'
	BÖNNIGHEIM_KIRCHHEIMER_STR_1 = 'de:08118:5726:0:3'  # eRtg Burgermühle (Bönnigheim Bönnigheim)
	BÖNNIGHEIM_KIRCHHEIMER_STR_2 = 'de:08118:5726:0:4'  # eRtg Schulzentrum (Bönnigheim Bönnigheim)
	BÖNNIGHEIM_SCHULZENTRUM = 'de:08118:5727'
	BÖNNIGHEIM_SCHULZENTRUM_1 = 'de:08118:5727:0:3'  # Rtg Burgplatz/Kirchheimer Str. (Bönnigheim Bönnigheim)
	BÖNNIGHEIM_SCHULZENTRUM_2 = 'de:08118:5727:0:4'  # Rtg Freudental Besigh. Str. (Bönnigheim Bönnigheim)
	BÖNNIGHEIM_SONNENBERG = 'de:08118:5728'
	BÖNNIGHEIM_SONNENBERG_1 = 'de:08118:5728:0:3'  # Rtg Cleebronn Abzw. Tripsdrill (Bönnigheim Bönnigheim)
	BÖNNIGHEIM_SONNENBERG_2 = 'de:08118:5728:0:4'  # Rtg Freibad (Bönnigheim Bönnigheim)
	SONNENBERG_1 = 'de:08118:5728:0:3'  # Rtg Cleebronn Abzw. Tripsdrill (Bönnigheim Bönnigheim)
	SONNENBERG_2 = 'de:08118:5728:0:4'  # Rtg Freibad (Bönnigheim Bönnigheim)
	SONNENBERG_3 = 'de:08111:6167:1:1'  # Rtg Riedsee (Stuttgart Stuttgart)
	SONNENBERG_4 = 'de:08111:6167:1:2'  # Rtg Peregrinastr. (Stuttgart Stuttgart)
	SONNENBERG_5 = 'de:08111:6167:3:3'  # N9 (Stuttgart Stuttgart)
	BÖNNIGHEIM_FREIBAD = 'de:08118:5729'
	BÖNNIGHEIM_FREIBAD_1 = 'de:08118:5729:0:3'  # eRtg Tripsdrill Erlebnispark (Bönnigheim Bönnigheim)
	BÖNNIGHEIM_FREIBAD_2 = 'de:08118:5729:0:4'  # eRtg Freibad (Bönnigheim Bönnigheim)
	BESIGHEIM_HARTWEG = 'de:08118:5730'
	BESIGHEIM_HARTWEG_1 = 'de:08118:5730:0:3'  # eRtg Paul-Gerhardt-Haus (Besigheim Besigheim)
	BESIGHEIM_HARTWEG_2 = 'de:08118:5730:0:5'  # Nachtbus (Besigheim Besigheim)
	HARTWEG = 'de:08118:5730'
	HARTWEG_1 = 'de:08118:5730:0:3'  # eRtg Paul-Gerhardt-Haus (Besigheim Besigheim)
	HARTWEG_2 = 'de:08118:5730:0:5'  # Nachtbus (Besigheim Besigheim)
	IM_SEE = 'de:08118:5731:1:3'  # eRtg alle Richtungen (Vaihingen an der Enz Kleinglattbach)
	KLEINGLATTBACH_IM_SEE = 'de:08118:5731:1:3'  # eRtg alle Richtungen (Vaihingen an der Enz Kleinglattbach)
	WASSERTURM = 'de:08118:5732:0:3'  # eRtg Hartweg (Besigheim Besigheim)
	BESIGHEIM_WASSERTURM = 'de:08118:5732:0:3'  # eRtg Hartweg (Besigheim Besigheim)
	VAIHINGEN__E__STADTHALLE = 'de:08118:5733'
	VAIHINGEN__E__STADTHALLE_1 = 'de:08118:5733:0:3'  # Rtg Neue Bahnhofstr. u. Gym (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__STADTHALLE_2 = 'de:08118:5733:0:4'  # Linie 578 (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__STADTHALLE_3 = 'de:08118:5733:0:5'  # Rtg Grabenstr. (Vaihingen an der Enz Vaihingen (E))
	HÄFNERHASL__ALTE_MÜHLE = 'de:08118:5735:0:3'  # eRtg Kirbachhof (Sachsenheim Häfnerhaslach)
	ALTE_MÜHLE = 'de:08118:5735:0:3'  # eRtg Kirbachhof (Sachsenheim Häfnerhaslach)
	HORRHEIM_FEUERWEHRHAUS = 'de:08118:5737'
	HORRHEIM_FEUERWEHRHAUS_1 = 'de:08118:5737:0:3'  # eRtg Marktplatz (Vaihingen an der Enz Horrheim)
	HORRHEIM_FEUERWEHRHAUS_2 = 'de:08118:5737:0:4'  # eRtg Altes Schulhaus/Nordbahnhof (Vaihingen an der Enz Horrheim)
	FEUERWEHRHAUS_1 = 'de:08118:5737:0:3'  # eRtg Marktplatz (Vaihingen an der Enz Horrheim)
	FEUERWEHRHAUS_2 = 'de:08118:5737:0:4'  # eRtg Altes Schulhaus/Nordbahnhof (Vaihingen an der Enz Horrheim)
	FEUERWEHRHAUS_3 = 'de:08117:3318:0:H1'  # Rtg Gewerbepark/Heiningen (Eschenbach Eschenbach)
	HORRHEIM_MARKTPLATZ = 'de:08118:5738'
	HORRHEIM_MARKTPLATZ_1 = 'de:08118:5738:0:3'  # eRtg Steige (Vaihingen an der Enz Horrheim)
	HORRHEIM_MARKTPLATZ_2 = 'de:08118:5738:0:4'  # eRtg Feuerwehrhaus (Vaihingen an der Enz Horrheim)
	MAULBRONNER_STRASSE = 'de:08118:5739'
	MAULBRONNER_STRASSE_1 = 'de:08118:5739:0:3'  # Rtg Gündelbach (Vaihingen an der Enz Horrheim)
	MAULBRONNER_STRASSE_2 = 'de:08118:5739:0:4'  # Rtg Marktplatz (Vaihingen an der Enz Horrheim)
	HORRHEIM_MAULBRONNER_STR = 'de:08118:5739'
	HORRHEIM_MAULBRONNER_STR_1 = 'de:08118:5739:0:3'  # Rtg Gündelbach (Vaihingen an der Enz Horrheim)
	HORRHEIM_MAULBRONNER_STR_2 = 'de:08118:5739:0:4'  # Rtg Marktplatz (Vaihingen an der Enz Horrheim)
	PFARRHAUS = 'de:08118:5740:0:3'  # eRtg Maulbronner Straße (Vaihingen an der Enz Gündelbach)
	GÜNDELBACH_PFARRHAUS = 'de:08118:5740:0:3'  # eRtg Maulbronner Straße (Vaihingen an der Enz Gündelbach)
	GROSSSACHSH__REALSCHULE = 'de:08118:5742:0:3'  # Rtg Burgfeldschule (Sachsenheim Großsachsenheim)
	PULVERDINGEN_B10 = 'de:08118:5743'
	PULVERDINGEN_B10_1 = 'de:08118:5743:0:3'  # eRtg Enzweihingen B10 (Vaihingen an der Enz Enzweihingen)
	PULVERDINGEN_B10_2 = 'de:08118:5743:0:4'  # eRtg Hardthof (Vaihingen an der Enz Enzweihingen)
	ENZW__PULVERDINGEN_B10 = 'de:08118:5743'
	ENZW__PULVERDINGEN_B10_1 = 'de:08118:5743:0:3'  # eRtg Enzweihingen B10 (Vaihingen an der Enz Enzweihingen)
	ENZW__PULVERDINGEN_B10_2 = 'de:08118:5743:0:4'  # eRtg Hardthof (Vaihingen an der Enz Enzweihingen)
	ENZWEIHINGEN_B10 = 'de:08118:5744'
	ENZWEIHINGEN_B10_1 = 'de:08118:5744:1:3'  # eRtg Vaihingen (E) (Vaihingen an der Enz Enzweihingen)
	ENZWEIHINGEN_B10_2 = 'de:08118:5744:1:4'  # eRtg Oberriexingen/Pulverd/Beerhaldenstr (Vaihingen an der Enz Enzweihingen)
	VAIHINGER_ECK = 'de:08118:5745'
	VAIHINGER_ECK_1 = 'de:08118:5745:0:3'  # Rtg Salzäcker (Vaihingen an der Enz Vaihingen (E))
	VAIHINGER_ECK_2 = 'de:08118:5745:0:4'  # Rtg Enzweihingen (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__VAIHINGER_ECK = 'de:08118:5745'
	VAIHINGEN__E__VAIHINGER_ECK_1 = 'de:08118:5745:0:3'  # Rtg Salzäcker (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__VAIHINGER_ECK_2 = 'de:08118:5745:0:4'  # Rtg Enzweihingen (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__AUSTRASSE = 'de:08118:5746:0:3'  # eRtg Salzäcker (Vaihingen an der Enz Vaihingen (E))
	RIET_RIETER_TAL = 'de:08118:5747'
	RIET_RIETER_TAL_1 = 'de:08118:5747:0:3'  # eRtg Enzweihingen B10 (Vaihingen an der Enz Riet)
	RIET_RIETER_TAL_2 = 'de:08118:5747:0:4'  # eRtg Dorfplatz (Vaihingen an der Enz Riet)
	RIETER_TAL = 'de:08118:5747'
	RIETER_TAL_1 = 'de:08118:5747:0:3'  # eRtg Enzweihingen B10 (Vaihingen an der Enz Riet)
	RIETER_TAL_2 = 'de:08118:5747:0:4'  # eRtg Dorfplatz (Vaihingen an der Enz Riet)
	DORFPLATZ_1 = 'de:08118:5748:1:3'  # Rtg Rieter Tal / B10 Vaihingen (Vaihingen an der Enz Riet)
	DORFPLATZ_2 = 'de:08118:5748:1:4'  # Rtg Hochdorf (Vaihingen an der Enz Riet)
	DORFPLATZ_3 = 'de:08135:1147:0:1'  # Rtg Geislingen (Gerstetten Gussenstadt)
	DORFPLATZ_4 = 'de:08135:1147:0:2'  # Rtg Gerstetten (Gerstetten Gussenstadt)
	RIET_DORFPLATZ = 'de:08118:5748'
	RIET_DORFPLATZ_1 = 'de:08118:5748:1:3'  # Rtg Rieter Tal / B10 Vaihingen (Vaihingen an der Enz Riet)
	RIET_DORFPLATZ_2 = 'de:08118:5748:1:4'  # Rtg Hochdorf (Vaihingen an der Enz Riet)
	VAIHINGEN__E__ENZBRÜCKE = 'de:08118:5751'
	VAIHINGEN__E__ENZBRÜCKE_1 = 'de:08118:5751:0:3'  # eRtg Abzweig (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__ENZBRÜCKE_2 = 'de:08118:5751:0:4'  # eRtg Grabenstr. (Stadtm.) (Vaihingen an der Enz Vaihingen (E))
	AURICH_ABZW = 'de:08118:5752'
	AURICH_ABZW_1 = 'de:08118:5752:0:3'  # eRtg Reitsteige (Vaihingen an der Enz Aurich)
	AURICH_ABZW_2 = 'de:08118:5752:0:4'  # eRtg Enzbrücke (Vaihingen an der Enz Aurich)
	AURICH_ORTSMITTE = 'de:08118:5753'
	AURICH_ORTSMITTE_1 = 'de:08118:5753:0:3'  # eRtg Nußdorfer Straße (Vaihingen an der Enz Aurich)
	AURICH_ORTSMITTE_2 = 'de:08118:5753:0:4'  # eRtg Reitsteige (Vaihingen an der Enz Aurich)
	TALHÖFE = 'de:08118:5756'
	TALHÖFE_1 = 'de:08118:5756:0:3'  # eRtg Enzweihingen B10 (Vaihingen an der Enz Enzweihingen)
	TALHÖFE_2 = 'de:08118:5756:0:4'  # eRtg Mühlstraße (Vaihingen an der Enz Enzweihingen)
	ENZWEIHINGEN_TALHÖFE = 'de:08118:5756'
	ENZWEIHINGEN_TALHÖFE_1 = 'de:08118:5756:0:3'  # eRtg Enzweihingen B10 (Vaihingen an der Enz Enzweihingen)
	ENZWEIHINGEN_TALHÖFE_2 = 'de:08118:5756:0:4'  # eRtg Mühlstraße (Vaihingen an der Enz Enzweihingen)
	VAIHINGEN__E__SALZÄCKER = 'de:08118:5757'
	VAIHINGEN__E__SALZÄCKER_1 = 'de:08118:5757:0:3'  # Rtg Stuttgarter Str. (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__SALZÄCKER_2 = 'de:08118:5757:0:4'  # Rtg Vaihinger Eck (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__SALZÄCKER_3 = 'de:08118:5757:0:5'  # Hans-Krieg-Str. Rtg Kehlstr. (Vaihingen an der Enz Vaihingen (E))
	KLEINGLATTBACH_SCHILLERSTR = 'de:08118:5759'
	KLEINGLATTBACH_SCHILLERSTR_1 = 'de:08118:5759:0:3'  # eRtg Stromberggymnasium (Vaihingen an der Enz Kleinglattbach)
	KLEINGLATTBACH_SCHILLERSTR_2 = 'de:08118:5759:0:4'  # eRtg Weizenstraße (Vaihingen an der Enz Kleinglattbach)
	KLEINSACHSENHEIM_ZEIL = 'de:08118:5760'
	KLEINSACHSENHEIM_ZEIL_1 = 'de:08118:5760:0:3'  # eRtg Bietigheimer Straße (Sachsenheim Kleinsachsenheim)
	KLEINSACHSENHEIM_ZEIL_2 = 'de:08118:5760:0:4'  # eRtg Kleinsachsenheimer Straße (Sachsenheim Kleinsachsenheim)
	ZEIL = 'de:08118:5760'
	ZEIL_1 = 'de:08118:5760:0:3'  # eRtg Bietigheimer Straße (Sachsenheim Kleinsachsenheim)
	ZEIL_2 = 'de:08118:5760:0:4'  # eRtg Kleinsachsenheimer Straße (Sachsenheim Kleinsachsenheim)
	PLÜDERHAUSEN = 'de:08119:5761'
	PLÜDERHAUSEN_1 = 'de:08119:5761:1:2'  # Gleis 2 (Plüderhausen Plüderhausen)
	PLÜDERHAUSEN_2 = 'de:08119:5761:1:3'  # Gleis 3 (Plüderhausen Plüderhausen)
	PLÜDERHAUSEN_SCHULSTR = 'de:08119:5763:0:3'  # eRtg Alle Richtungen (Plüderhausen Plüderhausen)
	PLÜDERH__WALKERSBACH = 'de:08119:5765'
	PLÜDERH__WALKERSBACH_1 = 'de:08119:5765:0:3'  # eRtg Lorch (Plüderhausen Plüderhausen)
	PLÜDERH__WALKERSBACH_2 = 'de:08119:5765:0:4'  # eRtg Busbahnhof (Plüderhausen Plüderhausen)
	WALKERSBACH = 'de:08119:5765'
	WALKERSBACH_1 = 'de:08119:5765:0:3'  # eRtg Lorch (Plüderhausen Plüderhausen)
	WALKERSBACH_2 = 'de:08119:5765:0:4'  # eRtg Busbahnhof (Plüderhausen Plüderhausen)
	NORDBAHNHOF_1 = 'de:08118:5766:1:3'  # eRtg Osterwiesenstraße (Vaihingen an der Enz Kleinglattbach)
	NORDBAHNHOF_2 = 'de:08118:5766:1:4'  # eRtg Sersh. Industriestraße/Horrheim (Vaihingen an der Enz Kleinglattbach)
	NORDBAHNHOF_3 = 'de:08111:6295:1:3'  # Gleis 3 (Stuttgart Stuttgart)
	NORDBAHNHOF_4 = 'de:08111:6295:1:4'  # Gleis 4 (Stuttgart Stuttgart)
	NORDBAHNHOF_5 = 'de:08111:6295:2:1'  # Rtg Mittnachtstr. (Stuttgart Stuttgart)
	NORDBAHNHOF_6 = 'de:08111:6295:2:2'  # Rtg Löwentor (Stuttgart Stuttgart)
	NORDBAHNHOF_7 = 'de:08111:6295:3:5'  # NBus Rtg Rosensteinstr (Stuttgart Stuttgart)
	KLEINGLATTB__NORDBAHNHOF = 'de:08118:5766'
	KLEINGLATTB__NORDBAHNHOF_1 = 'de:08118:5766:1:3'  # eRtg Osterwiesenstraße (Vaihingen an der Enz Kleinglattbach)
	KLEINGLATTB__NORDBAHNHOF_2 = 'de:08118:5766:1:4'  # eRtg Sersh. Industriestraße/Horrheim (Vaihingen an der Enz Kleinglattbach)
	NECKARREMS_AM_UNT__SCHLOSSBERG = 'de:08118:5767'
	NECKARREMS_AM_UNT__SCHLOSSBERG_1 = 'de:08118:5767:0:3'  # Rtg Neckargröningen Stadtbahn (Remseck am Neckar Neckarrems)
	NECKARREMS_AM_UNT__SCHLOSSBERG_2 = 'de:08118:5767:0:4'  # Rtg Sportgelände Tennwengert (Remseck am Neckar Neckarrems)
	AM_UNTEREN_SCHLOSSBERG = 'de:08118:5767'
	AM_UNTEREN_SCHLOSSBERG_1 = 'de:08118:5767:0:3'  # Rtg Neckargröningen Stadtbahn (Remseck am Neckar Neckarrems)
	AM_UNTEREN_SCHLOSSBERG_2 = 'de:08118:5767:0:4'  # Rtg Sportgelände Tennwengert (Remseck am Neckar Neckarrems)
	KIRCHHEIM__T__WALD = 'de:08116:5768'
	KIRCHHEIM__T__WALD_1 = 'de:08116:5768:0:3'  # eRtg Bachbrücke (Kirchheim unter Teck Kirchheim (T))
	KIRCHHEIM__T__WALD_2 = 'de:08116:5768:0:4'  # eRtg Ziegelwasen (Kirchheim unter Teck Kirchheim (T))
	WALD = 'de:08116:5768'
	WALD_1 = 'de:08116:5768:0:3'  # eRtg Bachbrücke (Kirchheim unter Teck Kirchheim (T))
	WALD_2 = 'de:08116:5768:0:4'  # eRtg Ziegelwasen (Kirchheim unter Teck Kirchheim (T))
	OBERL__HOCHWANG_RATHAUS = 'de:08116:5769'
	OBERL__HOCHWANG_RATHAUS_1 = 'de:08116:5769:0:3'  # eRtg Untere Straße (Lenningen Oberlenningen)
	OBERL__HOCHWANG_RATHAUS_2 = 'de:08116:5769:0:4'  # eRtg Rathaus (Lenningen Oberlenningen)
	HOCHWANG_RATHAUS = 'de:08116:5769'
	HOCHWANG_RATHAUS_1 = 'de:08116:5769:0:3'  # eRtg Untere Straße (Lenningen Oberlenningen)
	HOCHWANG_RATHAUS_2 = 'de:08116:5769:0:4'  # eRtg Rathaus (Lenningen Oberlenningen)
	OBERBOIHINGEN = 'de:08116:5770'
	OBERBOIHINGEN_1 = 'de:08116:5770:1:1'  # Gleis 1 (Oberboihingen Oberboihingen)
	OBERBOIHINGEN_2 = 'de:08116:5770:2:2'  # Gleis 2 (Oberboihingen Oberboihingen)
	OBERBOIHINGEN_3 = 'de:08116:5770:3:3'  # Rtg ??? (Oberboihingen Oberboihingen)
	BEMPFLINGEN = 'de:08116:5772'
	BEMPFLINGEN_1 = 'de:08116:5772:1:2'  # Gleis 2 (Bempflingen Bempflingen)
	BEMPFLINGEN_2 = 'de:08116:5772:2:1'  # Gleis 1 (Bempflingen Bempflingen)
	BEMPFLINGEN_3 = 'de:08116:5772:3:3'  # Bus (Bempflingen Bempflingen)
	EHNINGEN = 'de:08115:5773'
	EHNINGEN_1 = 'de:08115:5773:1:1'  # Gleis 1 (Ehningen Ehningen)
	EHNINGEN_2 = 'de:08115:5773:1:2'  # Gleis 2 (Ehningen Ehningen)
	EHNINGEN_3 = 'de:08115:5773:3:3'  # eRtg Ankunft/Abfahrt (Ehningen Ehningen)
	GÄRTRINGEN = 'de:08115:5774'
	GÄRTRINGEN_1 = 'de:08115:5774:1:1'  # Gleis 1 (Gärtringen Gärtringen)
	GÄRTRINGEN_2 = 'de:08115:5774:2:2'  # Gleis 2 (Gärtringen Gärtringen)
	GÄRTRINGEN_3 = 'de:08115:5774:3:3'  # eRtg Ankunft/Abfahrt (Gärtringen Gärtringen)
	NUFRINGEN = 'de:08115:5775'
	NUFRINGEN_1 = 'de:08115:5775:1:1'  # Gleis 1 (Nufringen Nufringen)
	NUFRINGEN_2 = 'de:08115:5775:1:2'  # Gleis 2 (Nufringen Nufringen)
	GÄUFELDEN = 'de:08115:5776'
	GÄUFELDEN_1 = 'de:08115:5776:1:1'  # Gleis 1 (Gäufelden Gäufelden)
	GÄUFELDEN_2 = 'de:08115:5776:1:2'  # Gleis 2 (Gäufelden Gäufelden)
	GÄUFELDEN_3 = 'de:08115:5776:2:3'  # Rtg Postamt (Gäufelden Gäufelden)
	GÄUFELDEN_4 = 'de:08115:5776:2:4'  # Rtg Bettäcker (Gäufelden Gäufelden)
	KIRCHHEIM__T__SÜD = 'de:08116:5777:1:2'  # Gleis 2 (Kirchheim unter Teck Kirchheim (T))
	DETTINGEN__T = 'de:08116:5778:1:1'  # Gleis 1 (Dettingen unter Teck Dettingen (T))
	OWEN = 'de:08116:5779'
	OWEN_1 = 'de:08116:5779:1:1'  # Gleis 1 (Owen Owen)
	OWEN_2 = 'de:08116:5779:2:3'  # Rtg Nürtingen/Beuren/Lenningen (Owen Owen)
	BRUCKEN = 'de:08116:5780:1:1'  # Gleis 1 (Lenningen Unterlenningen)
	UNTERLENNINGEN = 'de:08116:5781:1:1'  # Gleis 1 (Lenningen Unterlenningen)
	SERSHEIM = 'de:08118:5782'
	SERSHEIM_1 = 'de:08118:5782:1:2'  # Gleis 2 (Sersheim Sersheim)
	SERSHEIM_2 = 'de:08118:5782:2:1'  # Gleis 1 (Sersheim Sersheim)
	WALHEIM = 'de:08118:5785'
	WALHEIM_1 = 'de:08118:5785:1:1'  # Gleis 1 (Walheim Walheim)
	WALHEIM_2 = 'de:08118:5785:1:2'  # Gleis 2 (Walheim Walheim)
	WALHEIM_3 = 'de:08118:5785:2:3'  # Rtg Oberamteigasse/Besigheim (Walheim Walheim)
	WALHEIM_4 = 'de:08118:5785:2:4'  # Rtg Kirchheim (Walheim Walheim)
	OPPENWEILER = 'de:08119:5786'
	OPPENWEILER_1 = 'de:08119:5786:1:1'  # Gleis 1 (Oppenweiler Oppenweiler)
	OPPENWEILER_2 = 'de:08119:5786:2:2'  # Gleis 2 (Oppenweiler Oppenweiler)
	OPPENWEILER_3 = 'de:08119:5786:5:3'  # Rtg Aichelbach (Oppenweiler Oppenweiler)
	OPPENWEILER_4 = 'de:08119:5786:5:4'  # Rtg Ortsmitte (Oppenweiler Oppenweiler)
	FORNSBACH = 'de:08119:5787'
	FORNSBACH_1 = 'de:08119:5787:1:1'  # Gleis 1 (Murrhardt Fornsbach)
	FORNSBACH_2 = 'de:08119:5787:1:2'  # Gleis 2 (Murrhardt Fornsbach)
	URBACH = 'de:08119:5789'
	URBACH_1 = 'de:08119:5789:1:1'  # Gleis 1 (Urbach Urbach)
	URBACH_2 = 'de:08119:5789:2:2'  # Gleis 2 (Urbach Urbach)
	URBACH_3 = 'de:08119:5789:5:3'  # Bussteig Bf Urbach (Urbach Urbach)
	MIEDELSBACH_STEINENBERG = 'de:08119:5790'
	MIEDELSBACH_STEINENBERG_1 = 'de:08119:5790:1:2'  # Rtg Rudersberg/Welzheim (Schorndorf Miedelsbach)
	MIEDELSBACH_STEINENBERG_2 = 'de:08119:5790:2:1'  # Rtg Schorndorf (Schorndorf Miedelsbach)
	MIEDELSBACH_STEINENBERG_3 = 'de:08119:5790:3:4'  # Rtg Steinenberg (Schorndorfer Str) (Schorndorf Miedelsbach)
	MICHELAU = 'de:08119:5791'
	MICHELAU_1 = 'de:08119:5791:1:1'  # R-Bahn (Rudersberg Schlechtbach)
	MICHELAU_2 = 'de:08119:5791:2:3'  # eRtg Asperglen (Rudersberg Schlechtbach)
	MICHELAU_3 = 'de:08119:5791:2:4'  # eRtg Ortsmitte (Rudersberg Schlechtbach)
	SCHLECHTBACH = 'de:08119:5792:1:1'  # R-Bahn (Rudersberg Schlechtbach)
	FRICKENHAUSEN = 'de:08116:5793'
	FRICKENHAUSEN_1 = 'de:08116:5793:1:1'  # R-Bahn (Frickenhausen Frickenhausen)
	FRICKENHAUSEN_2 = 'de:08116:5793:2:3'  # Rtg Uhlandstr. (Frickenhausen Frickenhausen)
	NEUFFEN = 'de:08116:5794'
	NEUFFEN_1 = 'de:08116:5794:1:1'  # R-Bahn (Neuffen Neuffen)
	NEUFFEN_2 = 'de:08116:5794:2:3'  # Rtg Lindenplatz/Oberer Graben (Neuffen Neuffen)
	SCHWIEBERDINGEN = 'de:08118:5795'
	SCHWIEBERDINGEN_1 = 'de:08118:5795:1:1'  # Gleis 1 (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_2 = 'de:08118:5795:1:2'  # Gleis 2 (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_3 = 'de:08118:5795:2:3'  # eRtg Mitte (Schwieberdingen Schwieberdingen)
	BONL__METZINGER_STRASSE = 'de:08116:5798'
	BONL__METZINGER_STRASSE_1 = 'de:08116:5798:0:3'  # Rtg Kreuzäcker/Reußensteinweg (Filderstadt Bonlanden)
	BONL__METZINGER_STRASSE_2 = 'de:08116:5798:0:4'  # Rtg Marktstr. (Filderstadt Bonlanden)
	SCHWARZWALDSTRASSE_1 = 'de:08118:5799:0:3'  # eRtg Ellental (Bietigheim-Bissingen Bietigheim)
	SCHWARZWALDSTRASSE_2 = 'de:08118:5799:0:4'  # Rtg Hillerplatz (Bietigheim-Bissingen Bietigheim)
	SCHWARZWALDSTRASSE_3 = 'de:08115:7053:0:3'  # Rtg Grabenstraße (Gärtringen Gärtringen)
	SCHWARZWALDSTRASSE_4 = 'de:08115:7053:0:4'  # Rtg Feuerwehrgerätehaus (Gärtringen Gärtringen)
	BIETIGHEIM_SCHWARZWALDSTR = 'de:08118:5799'
	BIETIGHEIM_SCHWARZWALDSTR_1 = 'de:08118:5799:0:3'  # eRtg Ellental (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_SCHWARZWALDSTR_2 = 'de:08118:5799:0:4'  # Rtg Hillerplatz (Bietigheim-Bissingen Bietigheim)
	RUDERSBERG_NORD = 'de:08119:5801'
	RUDERSBERG_NORD_1 = 'de:08119:5801:1:1'  # R-Bahn (Rudersberg Rudersberg)
	RUDERSBERG_NORD_2 = 'de:08119:5801:2:3'  # Richtung Oberndorf Bf/Brunnenplatz (Rudersberg Rudersberg)
	RUDERSBERG_NORD_3 = 'de:08119:5801:2:4'  # Richtung Rudersberg Brühlstr. (Rudersberg Rudersberg)
	RUDERSBERG_NORD_4 = 'de:08119:5801:2:5'  # Rtg Rudersberg Schulzentrum (Rudersberg Rudersberg)
	GERADSTETTEN_REALSCHULE = 'de:08119:5802:0:3'  # Rtg Geradstetten Rathaus & Mitte (Remshalden Geradstetten)
	HAUBERSBRONN_MITTE = 'de:08119:5803:0:1'  # R-Bahn (Schorndorf Haubersbronn)
	SCHORNDORF_HAMMERSCHLAG = 'de:08119:5804:0:1'  # R-Bahn (Schorndorf Schorndorf)
	HAMMERSCHLAG = 'de:08119:5804:0:1'  # R-Bahn (Schorndorf Schorndorf)
	GRUNBACH_KIRCHPLATZ = 'de:08119:5805'
	GRUNBACH_KIRCHPLATZ_1 = 'de:08119:5805:0:3'  # Rtg. Kelterstr./Grunbach (Remshalden Grunbach)
	GRUNBACH_KIRCHPLATZ_2 = 'de:08119:5805:0:4'  # Rtg Buoch (Nordic Walking Park) (Remshalden Grunbach)
	BÖBLINGEN_PANZERKASERNE = 'de:08115:5807'
	BÖBLINGEN_PANZERKASERNE_1 = 'de:08115:5807:0:3'  # Rtg Schönaich Elsenhalde (Böblingen Böblingen)
	BÖBLINGEN_PANZERKASERNE_2 = 'de:08115:5807:0:4'  # Rtg Thermalbad (Böblingen Böblingen)
	PANZERKASERNE = 'de:08115:5807'
	PANZERKASERNE_1 = 'de:08115:5807:0:3'  # Rtg Schönaich Elsenhalde (Böblingen Böblingen)
	PANZERKASERNE_2 = 'de:08115:5807:0:4'  # Rtg Thermalbad (Böblingen Böblingen)
	KORNWESTHEIM_IM_WEIZEN = 'de:08118:5810'
	KORNWESTHEIM_IM_WEIZEN_1 = 'de:08118:5810:0:3'  # eRtg Hallenbad (Kornwestheim Kornwestheim)
	KORNWESTHEIM_IM_WEIZEN_2 = 'de:08118:5810:0:4'  # eRtg Zeppelinstraße (Kornwestheim Kornwestheim)
	IM_WEIZEN = 'de:08118:5810'
	IM_WEIZEN_1 = 'de:08118:5810:0:3'  # eRtg Hallenbad (Kornwestheim Kornwestheim)
	IM_WEIZEN_2 = 'de:08118:5810:0:4'  # eRtg Zeppelinstraße (Kornwestheim Kornwestheim)
	ROSSWAG_SPORTHALLE = 'de:08118:5811'
	ROSSWAG_SPORTHALLE_1 = 'de:08118:5811:0:3'  # Rtg Friedhof (Vaihingen an der Enz Roßwag)
	ROSSWAG_SPORTHALLE_2 = 'de:08118:5811:0:4'  # eRtg Seemühle (Vaihingen an der Enz Roßwag)
	SINDELF__SCHWERTSTRASSE = 'de:08115:5818:0:3'  # eRtg Textilzentrum (Sindelfingen Sindelfingen)
	SCHWERTSTRASSE = 'de:08115:5818:0:3'  # eRtg Textilzentrum (Sindelfingen Sindelfingen)
	NECKARTAILFINGEN_B312 = 'de:08116:5819'
	NECKARTAILFINGEN_B312_1 = 'de:08116:5819:0:3'  # Rtg Tübinger Str/Aich (Neckartailfingen Neckartailfingen)
	NECKARTAILFINGEN_B312_2 = 'de:08116:5819:0:4'  # Rtg Schlaitdorf Neckart. Str (Neckartailfingen Neckartailfingen)
	B312 = 'de:08116:5819'
	B312_1 = 'de:08116:5819:0:3'  # Rtg Tübinger Str/Aich (Neckartailfingen Neckartailfingen)
	B312_2 = 'de:08116:5819:0:4'  # Rtg Schlaitdorf Neckart. Str (Neckartailfingen Neckartailfingen)
	NECKARTAILFINGEN_TÜBINGER_STR = 'de:08116:5820'
	NECKARTAILFINGEN_TÜBINGER_STR_1 = 'de:08116:5820:0:3'  # eRtg Neckartailfingen (Neckartailfingen Neckartailfingen)
	NECKARTAILFINGEN_TÜBINGER_STR_2 = 'de:08116:5820:0:4'  # eRtg Neckarbrücke (Neckartailfingen Neckartailfingen)
	PLOCHINGEN_ESSLINGER_STR = 'de:08116:5822'
	PLOCHINGEN_ESSLINGER_STR_1 = 'de:08116:5822:0:3'  # Rtg Plochingen Bf/ZOB (Plochingen Plochingen)
	PLOCHINGEN_ESSLINGER_STR_2 = 'de:08116:5822:0:4'  # eRtg Pfostenberg (Plochingen Plochingen)
	GEBERSHEIM_CARL_ZEISS_STR = 'de:08115:5824'
	GEBERSHEIM_CARL_ZEISS_STR_1 = 'de:08115:5824:0:3'  # Rtg Rathaus (Leonberg Gebersheim)
	GEBERSHEIM_CARL_ZEISS_STR_2 = 'de:08115:5824:0:4'  # Rtg Leonberg (Leonberg Gebersheim)
	RUTESHEIM_MINIGOLFPLATZ = 'de:08115:5825'
	RUTESHEIM_MINIGOLFPLATZ_1 = 'de:08115:5825:0:4'  # Rtg Perouse (Rutesheim Rutesheim)
	RUTESHEIM_MINIGOLFPLATZ_2 = 'de:08115:5825:0:5'  # Rtg Rutesheim Umgehungstraße (Rutesheim Rutesheim)
	MINIGOLFPLATZ = 'de:08115:5825'
	MINIGOLFPLATZ_1 = 'de:08115:5825:0:4'  # Rtg Perouse (Rutesheim Rutesheim)
	MINIGOLFPLATZ_2 = 'de:08115:5825:0:5'  # Rtg Rutesheim Umgehungstraße (Rutesheim Rutesheim)
	RUTESHEIM_SCHILLERSTRASSE = 'de:08115:5826:0:4'  # Rtg Dieselstraße (Rutesheim Rutesheim)
	RUTESHEIM_FESTHALLE = 'de:08115:5828'
	RUTESHEIM_FESTHALLE_1 = 'de:08115:5828:0:3'  # Rtg Flacht (Rutesheim Rutesheim)
	RUTESHEIM_FESTHALLE_2 = 'de:08115:5828:0:4'  # Rtg Rutesheim (Rutesheim Rutesheim)
	RUTESHEIM_FESTHALLE_3 = 'de:08115:5828:0:5'  # Rtg Gebersheimer Str. in Schulstr. (Rutesheim Rutesheim)
	FESTHALLE_1 = 'de:08115:5828:0:3'  # Rtg Flacht (Rutesheim Rutesheim)
	FESTHALLE_2 = 'de:08115:5828:0:4'  # Rtg Rutesheim (Rutesheim Rutesheim)
	FESTHALLE_3 = 'de:08115:5828:0:5'  # Rtg Gebersheimer Str. in Schulstr. (Rutesheim Rutesheim)
	FESTHALLE_4 = 'de:08117:2033:0:H1'  # Rtg Am Wiesenbach (Uhingen Holzhausen (Uhingen))
	RUTESHEIM_SCHULZENTRUM = 'de:08115:5829:0:3'  # alle Richtungen (Rutesheim Rutesheim)
	RUTESHEIM_GEBERSHEIMER_STRASSE = 'de:08115:5831:0:3'  # Rtg Schillerstraße (Rutesheim Rutesheim)
	GEBERSHEIMER_STRASSE = 'de:08115:5831:0:3'  # Rtg Schillerstraße (Rutesheim Rutesheim)
	GEWERBEGEBIET_STEIGE = 'de:08115:5833'
	GEWERBEGEBIET_STEIGE_1 = 'de:08115:5833:0:1'  # Rtg stadtauswärts (Rutesheim Rutesheim)
	GEWERBEGEBIET_STEIGE_2 = 'de:08115:5833:0:2'  # Rtg Rutesheim (Rutesheim Rutesheim)
	GEWERBEGEBIET_STEIGE_3 = 'de:08115:5833:0:3'  # Rtg stadtauswärts in Drescherstraße (Rutesheim Rutesheim)
	RUTESHEIM_GEWERBEGEBIET_STEIGE = 'de:08115:5833'
	RUTESHEIM_GEWERBEGEBIET_STEIGE_1 = 'de:08115:5833:0:1'  # Rtg stadtauswärts (Rutesheim Rutesheim)
	RUTESHEIM_GEWERBEGEBIET_STEIGE_2 = 'de:08115:5833:0:2'  # Rtg Rutesheim (Rutesheim Rutesheim)
	RUTESHEIM_GEWERBEGEBIET_STEIGE_3 = 'de:08115:5833:0:3'  # Rtg stadtauswärts in Drescherstraße (Rutesheim Rutesheim)
	RUTESHEIM_HEUWEG = 'de:08115:5834'
	RUTESHEIM_HEUWEG_1 = 'de:08115:5834:0:3'  # eRtg Lessingstraße (Rutesheim Rutesheim)
	RUTESHEIM_HEUWEG_2 = 'de:08115:5834:0:4'  # eRtg Rutesheim (Rutesheim Rutesheim)
	HEUWEG = 'de:08115:5834'
	HEUWEG_1 = 'de:08115:5834:0:3'  # eRtg Lessingstraße (Rutesheim Rutesheim)
	HEUWEG_2 = 'de:08115:5834:0:4'  # eRtg Rutesheim (Rutesheim Rutesheim)
	RUTESHEIM_LESSINGSTRASSE = 'de:08115:5835'
	RUTESHEIM_LESSINGSTRASSE_1 = 'de:08115:5835:0:3'  # eRtg Bosch (Rutesheim Rutesheim)
	RUTESHEIM_LESSINGSTRASSE_2 = 'de:08115:5835:0:4'  # eRtg Heuweg (Rutesheim Rutesheim)
	RUTESHEIM_RATHAUS = 'de:08115:5836'
	RUTESHEIM_RATHAUS_1 = 'de:08115:5836:1:3'  # Pos 1 Rtg Dieselstraße Bosch (Rutesheim Rutesheim)
	RUTESHEIM_RATHAUS_2 = 'de:08115:5836:1:4'  # Pos 2 Rtg Drescherstr. Festhalle (Rutesheim Rutesheim)
	PEROUSE_HAUPTSTR = 'de:08115:5837'
	PEROUSE_HAUPTSTR_1 = 'de:08115:5837:0:3'  # Rtg Norden (Rutesheim Perouse)
	PEROUSE_HAUPTSTR_2 = 'de:08115:5837:0:4'  # Kringel (Rutesheim Perouse)
	RIEMENMÜHLE = 'de:08115:5838'
	RIEMENMÜHLE_1 = 'de:08115:5838:0:3'  # Rtg Merkl. / Hausen (Weil der Stadt Merklingen)
	RIEMENMÜHLE_2 = 'de:08115:5838:0:4'  # Rtg Im Tal (Weil der Stadt Merklingen)
	MERKLINGEN_RIEMENMÜHLE = 'de:08115:5838'
	MERKLINGEN_RIEMENMÜHLE_1 = 'de:08115:5838:0:3'  # Rtg Merkl. / Hausen (Weil der Stadt Merklingen)
	MERKLINGEN_RIEMENMÜHLE_2 = 'de:08115:5838:0:4'  # Rtg Im Tal (Weil der Stadt Merklingen)
	MERKLINGEN_INDUSTRIESTR = 'de:08115:5839'
	MERKLINGEN_INDUSTRIESTR_1 = 'de:08115:5839:0:3'  # eRtg Beethovenstraße (Weil der Stadt Merklingen)
	MERKLINGEN_INDUSTRIESTR_2 = 'de:08115:5839:0:4'  # eRtg Hauptstraße (Weil der Stadt Merklingen)
	METZELWIESEN = 'de:08115:5841'
	METZELWIESEN_1 = 'de:08115:5841:0:3'  # eRtg Hauptstraße (Weil der Stadt Merklingen)
	METZELWIESEN_2 = 'de:08115:5841:0:4'  # eRtg Merklinger Straße (Weil der Stadt Merklingen)
	MERKLINGEN_METZELWIESEN = 'de:08115:5841'
	MERKLINGEN_METZELWIESEN_1 = 'de:08115:5841:0:3'  # eRtg Hauptstraße (Weil der Stadt Merklingen)
	MERKLINGEN_METZELWIESEN_2 = 'de:08115:5841:0:4'  # eRtg Merklinger Straße (Weil der Stadt Merklingen)
	DITZ__INDUSTRIEGEBIET_OST = 'de:08118:5844:0:3'  # eRtg Münchinger Str. (Altenw.) (Ditzingen Ditzingen)
	PEROUSER_STRASSE = 'de:08115:5845'
	PEROUSER_STRASSE_1 = 'de:08115:5845:0:3'  # 633p (Renningen Malmsheim)
	PEROUSER_STRASSE_2 = 'de:08115:5845:0:4'  # Alter Steig (Renningen Malmsheim)
	MALMSHEIM_PEROUSER_STR = 'de:08115:5845'
	MALMSHEIM_PEROUSER_STR_1 = 'de:08115:5845:0:3'  # 633p (Renningen Malmsheim)
	MALMSHEIM_PEROUSER_STR_2 = 'de:08115:5845:0:4'  # Alter Steig (Renningen Malmsheim)
	WANKELSTRASSE = 'de:08115:5847'
	WANKELSTRASSE_1 = 'de:08115:5847:0:3'  # eRtg Kirche (Renningen Malmsheim)
	WANKELSTRASSE_2 = 'de:08115:5847:0:4'  # eRtg Rathaus (Renningen Malmsheim)
	MALMSHEIM_WANKELSTRASSE = 'de:08115:5847'
	MALMSHEIM_WANKELSTRASSE_1 = 'de:08115:5847:0:3'  # eRtg Kirche (Renningen Malmsheim)
	MALMSHEIM_WANKELSTRASSE_2 = 'de:08115:5847:0:4'  # eRtg Rathaus (Renningen Malmsheim)
	MALMSH__RATHAUS = 'de:08115:5848'
	MALMSH__RATHAUS_1 = 'de:08115:5848:0:3'  # eRtg Wankelstraße (Renningen Malmsheim)
	MALMSH__RATHAUS_2 = 'de:08115:5848:0:5'  # Rtg Perouser Str. (Renningen Malmsheim)
	MALMSHEIM_LILIENSTRASSE = 'de:08115:5850'
	MALMSHEIM_LILIENSTRASSE_1 = 'de:08115:5850:0:3'  # Rtg Rathaus (Renningen Malmsheim)
	MALMSHEIM_LILIENSTRASSE_2 = 'de:08115:5850:0:4'  # Rtg Renn. Schnallenäcker (Renningen Malmsheim)
	ROBERT_BOSCH_CAMPUS = 'de:08115:5851:0:3'  # eRtg Alle Richtungen (Renningen Malmsheim)
	MALMSHEIM_ROBERT_BOSCH_CAMPUS = 'de:08115:5851:0:3'  # eRtg Alle Richtungen (Renningen Malmsheim)
	TIGERSTRASSE = 'de:08111:5852'
	TIGERSTRASSE_1 = 'de:08111:5852:0:3'  # Rtg Kurmärker Kaserne (Stuttgart Stuttgart)
	TIGERSTRASSE_2 = 'de:08111:5852:0:4'  # Rtg Krehlstr. (Stuttgart Stuttgart)
	AUBERLENWEG = 'de:08111:5854'
	AUBERLENWEG_1 = 'de:08111:5854:0:3'  # Rtg Aspenwaldstr. (Stuttgart Stuttgart)
	AUBERLENWEG_2 = 'de:08111:5854:0:4'  # Rtg Lindpaintnerstr. (Stuttgart Stuttgart)
	MAICH__UNTERRIEDEN_GYMNASIUM = 'de:08115:5856'
	MAICH__UNTERRIEDEN_GYMNASIUM_1 = 'de:08115:5856:0:3'  # eRtg Maichingen Bf (Sindelfingen Maichingen)
	MAICH__UNTERRIEDEN_GYMNASIUM_2 = 'de:08115:5856:0:4'  # Rtg Südost (Sindelfingen Maichingen)
	UNTERRIEDEN_GYMNASIUM = 'de:08115:5856'
	UNTERRIEDEN_GYMNASIUM_1 = 'de:08115:5856:0:3'  # eRtg Maichingen Bf (Sindelfingen Maichingen)
	UNTERRIEDEN_GYMNASIUM_2 = 'de:08115:5856:0:4'  # Rtg Südost (Sindelfingen Maichingen)
	STIFTSGYMNASIUM = 'de:08115:5857:0:3'  # Rtg Gottlieb-Daimler-Schule (Sindelfingen Sindelfingen)
	SINDELF__STIFTSGYMNASIUM = 'de:08115:5857:0:3'  # Rtg Gottlieb-Daimler-Schule (Sindelfingen Sindelfingen)
	NÜRTINGEN_ROSSDORF = 'de:08116:5858:0:1'  # R-Bahn (Nürtingen Nürtingen)
	ROSSDORF = 'de:08116:5858:0:1'  # R-Bahn (Nürtingen Nürtingen)
	FRICKENHAUSEN_KELTERSTR = 'de:08116:5859:1:1'  # R-Bahn (Frickenhausen Frickenhausen)
	MARKGR__BILDUNGSZENTRUM = 'de:08118:5860:0:3'  # eRtg alle Richtungen (Markgröningen Markgröningen)
	FURTBACHSCHULE = 'de:08118:5861:0:3'  # eRtg Ankunft/Asperg Neckarstraße (Möglingen Möglingen)
	MÖGLINGEN_FURTBACHSCHULE = 'de:08118:5861:0:3'  # eRtg Ankunft/Asperg Neckarstraße (Möglingen Möglingen)
	HEYD_SCHULE = 'de:08118:5862'
	HEYD_SCHULE_1 = 'de:08118:5862:0:3'  # Rtg Haus Frank (Markgröningen Markgröningen)
	HEYD_SCHULE_2 = 'de:08118:5862:0:4'  # Rtg Mitte (Markgröningen Markgröningen)
	MARKGR__HEYD_SCHULE = 'de:08118:5862'
	MARKGR__HEYD_SCHULE_1 = 'de:08118:5862:0:3'  # Rtg Haus Frank (Markgröningen Markgröningen)
	MARKGR__HEYD_SCHULE_2 = 'de:08118:5862:0:4'  # Rtg Mitte (Markgröningen Markgröningen)
	NÜRTINGEN_VORSTADT = 'de:08116:5863:1:1'  # R-Bahn (Nürtingen Nürtingen)
	VORSTADT = 'de:08116:5863:1:1'  # R-Bahn (Nürtingen Nürtingen)
	BALZHOLZER_STRASSE = 'de:08116:5864'
	BALZHOLZER_STRASSE_1 = 'de:08116:5864:0:3'  # eRtg Balzholz Rathaus (Beuren Beuren)
	BALZHOLZER_STRASSE_2 = 'de:08116:5864:0:4'  # eRtg Freilichtmuseum (Beuren Beuren)
	BEUREN_BALZHOLZER_STR = 'de:08116:5864'
	BEUREN_BALZHOLZER_STR_1 = 'de:08116:5864:0:3'  # eRtg Balzholz Rathaus (Beuren Beuren)
	BEUREN_BALZHOLZER_STR_2 = 'de:08116:5864:0:4'  # eRtg Freilichtmuseum (Beuren Beuren)
	VAIHINGEN__E__STUTTGARTER_STR = 'de:08118:5865:0:3'  # eRtg Grabenstr. (Stadtm.) (Vaihingen an der Enz Vaihingen (E))
	GEHRENWALD = 'de:08111:5868'
	GEHRENWALD_1 = 'de:08111:5868:0:3'  # Rtg Im Hag (Stuttgart Stuttgart)
	GEHRENWALD_2 = 'de:08111:5868:0:4'  # Rtg Luginsland (Stuttgart Stuttgart)
	KARL_GEORG_PFLEIDERER_STRASSE = 'de:08119:5869:0:3'  # eRtg Baacher Weg (Winnenden Hertmannsweiler)
	HERTM__K__G__PFLEID__STR = 'de:08119:5869:0:3'  # eRtg Baacher Weg (Winnenden Hertmannsweiler)
	HOCHDORF_WEINBERGSTRASSE = 'de:08116:5871:0:3'  # eRtg Roßwälder Straße (Hochdorf Hochdorf (ES))
	WEINBERGSTRASSE_1 = 'de:08116:5871:0:3'  # eRtg Roßwälder Straße (Hochdorf Hochdorf (ES))
	WEINBERGSTRASSE_2 = 'de:08116:6713:0:3'  # eRtg Risshalde / Stgt.-Hauptstraße (Reichenbach an der Fils Reichenbach (F))
	WEINBERGSTRASSE_3 = 'de:08116:6713:0:4'  # eRtg Blumen-Wilhelmstraße (Reichenbach an der Fils Reichenbach (F))
	VAIHINGEN__E__KRANKENHAUS = 'de:08118:5873'
	VAIHINGEN__E__KRANKENHAUS_1 = 'de:08118:5873:0:3'  # eRtg Steinbeisstraße (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__KRANKENHAUS_2 = 'de:08118:5873:0:4'  # eRtg Stadthalle (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__STEINBEISSTR = 'de:08118:5875'
	VAIHINGEN__E__STEINBEISSTR_1 = 'de:08118:5875:0:3'  # eRtg Vaihingen (E) (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__STEINBEISSTR_2 = 'de:08118:5875:0:4'  # eRtg Krankenhaus (Vaihingen an der Enz Vaihingen (E))
	HOCHDORF__E__EBERDINGER_STR = 'de:08118:5876:0:3'  # eRtg Hochdorfer Straße (Eberdingen Hochdorf (Eberdingen))
	EBERDINGER_STRASSE = 'de:08118:5876:0:3'  # eRtg Hochdorfer Straße (Eberdingen Hochdorf (Eberdingen))
	FRICKENHAUSEN_STERNBERGSTR = 'de:08116:5878'
	FRICKENHAUSEN_STERNBERGSTR_1 = 'de:08116:5878:0:3'  # eRtg Kelter (Frickenhausen Frickenhausen)
	FRICKENHAUSEN_STERNBERGSTR_2 = 'de:08116:5878:0:4'  # eRtg Humboldtstraße (Frickenhausen Frickenhausen)
	STERNBERGSTRASSE = 'de:08116:5878'
	STERNBERGSTRASSE_1 = 'de:08116:5878:0:3'  # eRtg Kelter (Frickenhausen Frickenhausen)
	STERNBERGSTRASSE_2 = 'de:08116:5878:0:4'  # eRtg Humboldtstraße (Frickenhausen Frickenhausen)
	VAIHINGEN__E = 'de:08118:5879'
	VAIHINGEN__E_1 = 'de:08118:5879:1:7'  # Gleis 7 (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E_2 = 'de:08118:5879:1:8'  # Gleis 8 (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E_3 = 'de:08118:5879:2:1'  # Gleis 1 (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E_4 = 'de:08118:5879:2:2'  # Gleis 2 (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E_5 = 'de:08118:5879:4:1A'  # Bstg 1A (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E_6 = 'de:08118:5879:4:1B'  # Bstg 1B (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E_7 = 'de:08118:5879:4:2A'  # Bstg 2A (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E_8 = 'de:08118:5879:4:3A'  # Bstg 3A (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E_9 = 'de:08118:5879:4:3B'  # Bstg 3B (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E_10 = 'de:08118:5879:4:4A'  # Bstg 4A (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E_11 = 'de:08118:5879:4:4C'  # Bstg 4C (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E_12 = 'de:08118:5879:4:6'  # Vaihingen (E) (Vaihingen an der Enz Vaihingen (E))
	NEUE_BAHNHOFSTRASSE = 'de:08118:5880'
	NEUE_BAHNHOFSTRASSE_1 = 'de:08118:5880:0:3'  # Rtg Bahnhof (Vaihingen an der Enz Vaihingen (E))
	NEUE_BAHNHOFSTRASSE_2 = 'de:08118:5880:0:4'  # Rtg Grabenstr. (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__NEUE_BAHNHOFSTR = 'de:08118:5880'
	VAIHINGEN__E__NEUE_BAHNHOFSTR_1 = 'de:08118:5880:0:3'  # Rtg Bahnhof (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__NEUE_BAHNHOFSTR_2 = 'de:08118:5880:0:4'  # Rtg Grabenstr. (Vaihingen an der Enz Vaihingen (E))
	DRESCHHALLE = 'de:08118:5881:0:3'  # eRtg Pfarrhaus (Vaihingen an der Enz Gündelbach)
	GÜNDELBACH_DRESCHHALLE = 'de:08118:5881:0:3'  # eRtg Pfarrhaus (Vaihingen an der Enz Gündelbach)
	GÜNDELBACH_HILDENSTR = 'de:08118:5882:0:3'  # eRtg Dreschhalle (Vaihingen an der Enz Gündelbach)
	HILDENSTRASSE = 'de:08118:5882:0:3'  # eRtg Dreschhalle (Vaihingen an der Enz Gündelbach)
	OCHSENBACH_BUCHENRAIN = 'de:08118:5883'
	OCHSENBACH_BUCHENRAIN_1 = 'de:08118:5883:0:3'  # Rtg Kirche (Sachsenheim Ochsenbach)
	OCHSENBACH_BUCHENRAIN_2 = 'de:08118:5883:0:4'  # Rtg Kirchbachhof (Sachsenheim Ochsenbach)
	BUCHENRAIN = 'de:08118:5883'
	BUCHENRAIN_1 = 'de:08118:5883:0:3'  # Rtg Kirche (Sachsenheim Ochsenbach)
	BUCHENRAIN_2 = 'de:08118:5883:0:4'  # Rtg Kirchbachhof (Sachsenheim Ochsenbach)
	OSTERWIESENWEG = 'de:08118:5884'
	OSTERWIESENWEG_1 = 'de:08118:5884:0:3'  # eRtg Grundschule (Vaihingen an der Enz Kleinglattbach)
	OSTERWIESENWEG_2 = 'de:08118:5884:0:4'  # eRtg Nordbahnhof (Vaihingen an der Enz Kleinglattbach)
	KLEINGLATTB__OSTERWIESENWEG = 'de:08118:5884'
	KLEINGLATTB__OSTERWIESENWEG_1 = 'de:08118:5884:0:3'  # eRtg Grundschule (Vaihingen an der Enz Kleinglattbach)
	KLEINGLATTB__OSTERWIESENWEG_2 = 'de:08118:5884:0:4'  # eRtg Nordbahnhof (Vaihingen an der Enz Kleinglattbach)
	NUSSDORFER_STRASSE_1 = 'de:08118:5885:0:3'  # eRtg Ortsmitte (Vaihingen an der Enz Aurich)
	NUSSDORFER_STRASSE_2 = 'de:08118:5885:0:4'  # eRtg Vaihinger Straße (Vaihingen an der Enz Aurich)
	NUSSDORFER_STRASSE_3 = 'de:08118:6468:0:3'  # eRtg Hochdorfer Straße (Eberdingen Eberdingen)
	AURICH_NUSSDORFER_STRASSE = 'de:08118:5885'
	AURICH_NUSSDORFER_STRASSE_1 = 'de:08118:5885:0:3'  # eRtg Ortsmitte (Vaihingen an der Enz Aurich)
	AURICH_NUSSDORFER_STRASSE_2 = 'de:08118:5885:0:4'  # eRtg Vaihinger Straße (Vaihingen an der Enz Aurich)
	GROSSSACHSH__BANNHALDE = 'de:08118:5888'
	GROSSSACHSH__BANNHALDE_1 = 'de:08118:5888:0:3'  # eRtg Langmantel (Sachsenheim Großsachsenheim)
	GROSSSACHSH__BANNHALDE_2 = 'de:08118:5888:0:4'  # eRtg Metterbrücke (Sachsenheim Großsachsenheim)
	BANNHALDE = 'de:08118:5888'
	BANNHALDE_1 = 'de:08118:5888:0:3'  # eRtg Langmantel (Sachsenheim Großsachsenheim)
	BANNHALDE_2 = 'de:08118:5888:0:4'  # eRtg Metterbrücke (Sachsenheim Großsachsenheim)
	WEINGARTSTRASSE = 'de:08116:5889:0:3'  # Bus-Li119/121>DDNH (Denkendorf Denkendorf)
	DENKENDORF_WEINGARTSTRASSE = 'de:08116:5889:0:3'  # Bus-Li119/121>DDNH (Denkendorf Denkendorf)
	ZIMMERER_PFAD = 'de:08118:5890'
	ZIMMERER_PFAD_1 = 'de:08118:5890:0:3'  # eRtg Lichtenstern Gymnasium (Sachsenheim Großsachsenheim)
	ZIMMERER_PFAD_2 = 'de:08118:5890:0:4'  # eRtg Holderbüschle (Sachsenheim Großsachsenheim)
	GROSSSACHSH__ZIMMERER_PFAD = 'de:08118:5890'
	GROSSSACHSH__ZIMMERER_PFAD_1 = 'de:08118:5890:0:3'  # eRtg Lichtenstern Gymnasium (Sachsenheim Großsachsenheim)
	GROSSSACHSH__ZIMMERER_PFAD_2 = 'de:08118:5890:0:4'  # eRtg Holderbüschle (Sachsenheim Großsachsenheim)
	GROSSSACHSH__HOLDERBÜSCHLE = 'de:08118:5891'
	GROSSSACHSH__HOLDERBÜSCHLE_1 = 'de:08118:5891:0:3'  # Rtg Zimmerer Pfad (Sachsenheim Großsachsenheim)
	GROSSSACHSH__HOLDERBÜSCHLE_2 = 'de:08118:5891:0:4'  # Rtg Bietigheim (Sachsenheim Großsachsenheim)
	HOLDERBÜSCHLE = 'de:08118:5891'
	HOLDERBÜSCHLE_1 = 'de:08118:5891:0:3'  # Rtg Zimmerer Pfad (Sachsenheim Großsachsenheim)
	HOLDERBÜSCHLE_2 = 'de:08118:5891:0:4'  # Rtg Bietigheim (Sachsenheim Großsachsenheim)
	ETTERSTRASSE = 'de:08116:5896:0:3'  # Rtg alle Richtungen (Lenningen Schlattstall)
	SCHLATTSTALL_ETTERSTRASSE = 'de:08116:5896:0:3'  # Rtg alle Richtungen (Lenningen Schlattstall)
	OBERL__GUTENBERGER_STR__17 = 'de:08116:5897'
	OBERL__GUTENBERGER_STR__17_1 = 'de:08116:5897:0:3'  # eRtg Nikomödiestraße 34 (Lenningen Oberlenningen)
	OBERL__GUTENBERGER_STR__17_2 = 'de:08116:5897:0:4'  # eRtg Oberlenningen (Lenningen Oberlenningen)
	GUTENBERGER_STRASSE_17 = 'de:08116:5897'
	GUTENBERGER_STRASSE_17_1 = 'de:08116:5897:0:3'  # eRtg Nikomödiestraße 34 (Lenningen Oberlenningen)
	GUTENBERGER_STRASSE_17_2 = 'de:08116:5897:0:4'  # eRtg Oberlenningen (Lenningen Oberlenningen)
	NÜRTINGEN_WEBERSTRASSE = 'de:08116:5898'
	NÜRTINGEN_WEBERSTRASSE_1 = 'de:08116:5898:0:3'  # eRtg Inselbad (Nürtingen Nürtingen)
	NÜRTINGEN_WEBERSTRASSE_2 = 'de:08116:5898:0:4'  # eRtg Nürtingen ZOB (Nürtingen Nürtingen)
	WEBERSTRASSE = 'de:08116:5898'
	WEBERSTRASSE_1 = 'de:08116:5898:0:3'  # eRtg Inselbad (Nürtingen Nürtingen)
	WEBERSTRASSE_2 = 'de:08116:5898:0:4'  # eRtg Nürtingen ZOB (Nürtingen Nürtingen)
	GROSSBETTL__BETTLINGER_FORUM = 'de:08116:5900'
	GROSSBETTL__BETTLINGER_FORUM_1 = 'de:08116:5900:0:3'  # eRtg Nürtinger Straße (Großbettlingen Großbettlingen)
	GROSSBETTL__BETTLINGER_FORUM_2 = 'de:08116:5900:0:4'  # eRtg Rathaus/Herion (Großbettlingen Großbettlingen)
	BETTLINGER_FORUM = 'de:08116:5900'
	BETTLINGER_FORUM_1 = 'de:08116:5900:0:3'  # eRtg Nürtinger Straße (Großbettlingen Großbettlingen)
	BETTLINGER_FORUM_2 = 'de:08116:5900:0:4'  # eRtg Rathaus/Herion (Großbettlingen Großbettlingen)
	GROSSBETTLINGEN_ALBSTR = 'de:08116:5901'
	GROSSBETTLINGEN_ALBSTR_1 = 'de:08116:5901:0:3'  # eRtg Herion (Großbettlingen Großbettlingen)
	GROSSBETTLINGEN_ALBSTR_2 = 'de:08116:5901:0:4'  # eRtg Metzinger Str. (Großbettlingen Großbettlingen)
	FRICKENHAUSEN_MAYBACHSTR = 'de:08116:5902:0:3'  # Rtg Frickenh. Bf (Frickenhausen Frickenhausen)
	KOHLBERG_RATHAUS = 'de:08116:5903'
	KOHLBERG_RATHAUS_1 = 'de:08116:5903:0:3'  # Rtg Kappis/Tischardt (Kohlberg Kohlberg)
	KOHLBERG_RATHAUS_2 = 'de:08116:5903:0:4'  # Rtg Neuffen/Hörnlesweg (Kohlberg Kohlberg)
	WEILH__T__LIMB_SCH__KELTERSTR = 'de:08116:5908:0:3'  # eRtg Bildungszentrum (Weilheim an der Teck Weilheim (T))
	LIMBURGSCHULE_KELTERSTRASSE = 'de:08116:5908:0:3'  # eRtg Bildungszentrum (Weilheim an der Teck Weilheim (T))
	RIEDWIESEN = 'de:08116:5910'
	RIEDWIESEN_1 = 'de:08116:5910:0:3'  # eRtg Aich Siedlung (Aichtal Aich)
	RIEDWIESEN_2 = 'de:08116:5910:0:4'  # eRtg Spitzacker/Bonl. Gutenhalde (Aichtal Aich)
	AICH_RIEDWIESEN = 'de:08116:5910'
	AICH_RIEDWIESEN_1 = 'de:08116:5910:0:3'  # eRtg Aich Siedlung (Aichtal Aich)
	AICH_RIEDWIESEN_2 = 'de:08116:5910:0:4'  # eRtg Spitzacker/Bonl. Gutenhalde (Aichtal Aich)
	WÖHR_TOURS_DEPOT = 'de:08115:5911'
	WÖHR_TOURS_DEPOT_1 = 'de:08115:5911:0:3'  # eRtg Bahnhofstraße (Weissach Weissach (BB))
	WÖHR_TOURS_DEPOT_2 = 'de:08115:5911:0:4'  # eRtg Hindenburgstraße (Weissach Weissach (BB))
	WEISSACH_WÖHR_TOURS_DEPOT = 'de:08115:5911'
	WEISSACH_WÖHR_TOURS_DEPOT_1 = 'de:08115:5911:0:3'  # eRtg Bahnhofstraße (Weissach Weissach (BB))
	WEISSACH_WÖHR_TOURS_DEPOT_2 = 'de:08115:5911:0:4'  # eRtg Hindenburgstraße (Weissach Weissach (BB))
	WEISSACH_BACHSTRASSE = 'de:08115:5912:0:3'  # eRtg Marktplatz (Weissach Weissach (BB))
	WALDENSERSTRASSE = 'de:08115:5915:0:3'  # Rtg Rutesheim Schulzentrum (Rutesheim Perouse)
	PEROUSE_WALDENSERSTR = 'de:08115:5915:0:3'  # Rtg Rutesheim Schulzentrum (Rutesheim Perouse)
	GERLINGEN_RINGSTRASSE = 'de:08118:5916'
	GERLINGEN_RINGSTRASSE_1 = 'de:08118:5916:0:3'  # Rtg Gerlingen (Gerlingen Gerlingen)
	GERLINGEN_RINGSTRASSE_2 = 'de:08118:5916:0:4'  # Rtg Weissach (Gerlingen Gerlingen)
	FORCHENRAINSTRASSE = 'de:08118:5917:0:3'  # eRtg Hermann-Löns-Straße (Gerlingen Gerlingen)
	GERLINGEN_FORCHENRAINSTR = 'de:08118:5917:0:3'  # eRtg Hermann-Löns-Straße (Gerlingen Gerlingen)
	STRUDELBACHHALLE = 'de:08115:5918:0:3'  # eRtg Hohweg (Weissach Weissach (BB))
	WEISSACH_STRUDELBACHHALLE = 'de:08115:5918:0:3'  # eRtg Hohweg (Weissach Weissach (BB))
	FEUERWEHRMAGAZIN = 'de:08118:5920:0:3'  # eRtg Kleinglattbacher Straße (Vaihingen an der Enz Ensingen)
	ENS__FEUERWEHRMAGAZIN = 'de:08118:5920:0:3'  # eRtg Kleinglattbacher Straße (Vaihingen an der Enz Ensingen)
	SINDELFINGEN_GRÜNÄCKERSTR = 'de:08115:5921'
	SINDELFINGEN_GRÜNÄCKERSTR_1 = 'de:08115:5921:0:3'  # eRtg Bartenweg (Sindelfingen Sindelfingen)
	SINDELFINGEN_GRÜNÄCKERSTR_2 = 'de:08115:5921:0:4'  # eRtg Schlüsseläcker (Sindelfingen Sindelfingen)
	GRÜNÄCKERSTRASSE = 'de:08115:5921'
	GRÜNÄCKERSTRASSE_1 = 'de:08115:5921:0:3'  # eRtg Bartenweg (Sindelfingen Sindelfingen)
	GRÜNÄCKERSTRASSE_2 = 'de:08115:5921:0:4'  # eRtg Schlüsseläcker (Sindelfingen Sindelfingen)
	KLEINBOTTWARER_STRASSE = 'de:08118:5922'
	KLEINBOTTWARER_STRASSE_1 = 'de:08118:5922:0:3'  # eRtg Kelter (Steinheim an der Murr Steinheim (M))
	KLEINBOTTWARER_STRASSE_2 = 'de:08118:5922:0:4'  # eRtg Kleinbottwar (Steinheim an der Murr Steinheim (M))
	STEINH__M__KLEINBOTTWARER_STR = 'de:08118:5922'
	STEINH__M__KLEINBOTTWARER_STR_1 = 'de:08118:5922:0:3'  # eRtg Kelter (Steinheim an der Murr Steinheim (M))
	STEINH__M__KLEINBOTTWARER_STR_2 = 'de:08118:5922:0:4'  # eRtg Kleinbottwar (Steinheim an der Murr Steinheim (M))
	WAIBL__GEWERBESTR___AKADEMIE = 'de:08119:5924:0:4'  # eRtg Alte Bundesstraße/Bahnhof (Waiblingen Waiblingen)
	GEWERBESTRASSE__AKADEMIE = 'de:08119:5924:0:4'  # eRtg Alte Bundesstraße/Bahnhof (Waiblingen Waiblingen)
	WAIBLINGEN_OBERE_BAHNHOFSTR = 'de:08119:5925'
	WAIBLINGEN_OBERE_BAHNHOFSTR_1 = 'de:08119:5925:0:3'  # Rtg Blumenstr. (Waiblingen Waiblingen)
	WAIBLINGEN_OBERE_BAHNHOFSTR_2 = 'de:08119:5925:0:4'  # Rtg Bahnhof (Waiblingen Waiblingen)
	OBERE_BAHNHOFSTRASSE = 'de:08119:5925'
	OBERE_BAHNHOFSTRASSE_1 = 'de:08119:5925:0:3'  # Rtg Blumenstr. (Waiblingen Waiblingen)
	OBERE_BAHNHOFSTRASSE_2 = 'de:08119:5925:0:4'  # Rtg Bahnhof (Waiblingen Waiblingen)
	SINDELF__MARKTPL___BÖBL_STR = 'de:08115:5926:0:3'  # eRtg Domo (Sindelfingen Sindelfingen)
	MARKTPLATZ__BÖBLINGER_STRASSE = 'de:08115:5926:0:3'  # eRtg Domo (Sindelfingen Sindelfingen)
	MARKTPLATZ__RATHAUS = 'de:08115:5927'
	MARKTPLATZ__RATHAUS_1 = 'de:08115:5927:0:3'  # Rtg ZOB (Sindelfingen Sindelfingen)
	MARKTPLATZ__RATHAUS_2 = 'de:08115:5927:0:4'  # Rtg Stadthalle (Sindelfingen Sindelfingen)
	SINDELF__MARKTPL___RATHAUS = 'de:08115:5927'
	SINDELF__MARKTPL___RATHAUS_1 = 'de:08115:5927:0:3'  # Rtg ZOB (Sindelfingen Sindelfingen)
	SINDELF__MARKTPL___RATHAUS_2 = 'de:08115:5927:0:4'  # Rtg Stadthalle (Sindelfingen Sindelfingen)
	BARTENWEG = 'de:08115:5929:0:3'  # eRtg Landhaussiedlung (Sindelfingen Sindelfingen)
	SINDELFINGEN_BARTENWEG = 'de:08115:5929:0:3'  # eRtg Landhaussiedlung (Sindelfingen Sindelfingen)
	VAIHINGEN__E__STADTBAHNHOF = 'de:08118:5932'
	VAIHINGEN__E__STADTBAHNHOF_1 = 'de:08118:5932:0:3'  # Bus (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__STADTBAHNHOF_2 = 'de:08118:5932:0:4'  # Bus (Vaihingen an der Enz Vaihingen (E))
	STADTBAHNHOF = 'de:08118:5932'
	STADTBAHNHOF_1 = 'de:08118:5932:0:3'  # Bus (Vaihingen an der Enz Vaihingen (E))
	STADTBAHNHOF_2 = 'de:08118:5932:0:4'  # Bus (Vaihingen an der Enz Vaihingen (E))
	GROSSINGERSHEIM_FRIEDHOF = 'de:08118:5933'
	GROSSINGERSHEIM_FRIEDHOF_1 = 'de:08118:5933:0:3'  # eRtg Schlossstraße (Ingersheim Großingersheim)
	GROSSINGERSHEIM_FRIEDHOF_2 = 'de:08118:5933:0:4'  # eRtg Brühlstraße (Ingersheim Großingersheim)
	GROSSINGERSHEIM_ORTSMITTE = 'de:08118:5934'
	GROSSINGERSHEIM_ORTSMITTE_1 = 'de:08118:5934:0:3'  # Rtg Brühlstr. (Ingersheim Großingersheim)
	GROSSINGERSHEIM_ORTSMITTE_2 = 'de:08118:5934:0:4'  # Rtg Bietigheimer Str. (Ingersheim Großingersheim)
	RUTESHEIM_KEPLERSTRASSE = 'de:08115:5935:0:3'  # eRtg Gewerbegebiet Steige (Rutesheim Rutesheim)
	RUTESH__SPORTHALLE_BÜHL = 'de:08115:5936'
	RUTESH__SPORTHALLE_BÜHL_1 = 'de:08115:5936:0:3'  # eRtg Renningen (Rutesheim Rutesheim)
	RUTESH__SPORTHALLE_BÜHL_2 = 'de:08115:5936:0:4'  # eRtg Bosch (Rutesheim Rutesheim)
	SPORTHALLE_BÜHL = 'de:08115:5936'
	SPORTHALLE_BÜHL_1 = 'de:08115:5936:0:3'  # eRtg Renningen (Rutesheim Rutesheim)
	SPORTHALLE_BÜHL_2 = 'de:08115:5936:0:4'  # eRtg Bosch (Rutesheim Rutesheim)
	RUTESHEIM_WIDDUMHOF = 'de:08115:5937'
	RUTESHEIM_WIDDUMHOF_1 = 'de:08115:5937:0:3'  # eRtg Rathaus (Rutesheim Rutesheim)
	RUTESHEIM_WIDDUMHOF_2 = 'de:08115:5937:0:4'  # eRtg Drescherstr. (Rutesheim Rutesheim)
	WIDDUMHOF = 'de:08115:5937'
	WIDDUMHOF_1 = 'de:08115:5937:0:3'  # eRtg Rathaus (Rutesheim Rutesheim)
	WIDDUMHOF_2 = 'de:08115:5937:0:4'  # eRtg Drescherstr. (Rutesheim Rutesheim)
	IN_DEN_BEETEN = 'de:08118:5940:0:3'  # Nachtbus (Ingersheim Großingersheim)
	GROSSING__IN_DEN_BEETEN = 'de:08118:5940:0:3'  # Nachtbus (Ingersheim Großingersheim)
	DÄTZINGEN_ABZW = 'de:08115:5941'
	DÄTZINGEN_ABZW_1 = 'de:08115:5941:0:3'  # eRtg Lehenweiler Abzweig (Grafenau Dätzingen)
	DÄTZINGEN_ABZW_2 = 'de:08115:5941:0:4'  # eRtg Seeger (Grafenau Dätzingen)
	NEBRINGER_STRASSE = 'de:08115:5946:0:3'  # eRtg Weildorf (Bondorf Bondorf)
	BONDORF_NEBRINGER_STRASSE = 'de:08115:5946:0:3'  # eRtg Weildorf (Bondorf Bondorf)
	WEIL_I__S__HAUPTSTRASSE = 'de:08115:5947'
	WEIL_I__S__HAUPTSTRASSE_1 = 'de:08115:5947:1:3'  # eRtg Rathaus(Backhaus)/See (Weil im Schönbuch Weil im Schönbuch)
	WEIL_I__S__HAUPTSTRASSE_2 = 'de:08115:5947:1:4'  # eRtg Troppel (Weil im Schönbuch Weil im Schönbuch)
	WEIL_I__S__SCHULE_HALLENBAD = 'de:08115:5948:0:3'  # Alle Richtungen (Weil im Schönbuch Weil im Schönbuch)
	SCHULE_HALLENBAD = 'de:08115:5948:0:3'  # Alle Richtungen (Weil im Schönbuch Weil im Schönbuch)
	ROMMELSHAUSEN_KARLSTR = 'de:08119:5949'
	ROMMELSHAUSEN_KARLSTR_1 = 'de:08119:5949:0:3'  # eRtg Lilienstraße/Fellbacher Straße (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_KARLSTR_2 = 'de:08119:5949:0:4'  # eRtg Rumold-Realschule (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_KARLSTR_3 = 'de:08119:5949:0:5'  # Rtg Esslingen (RELEX X20) (Kernen im Remstal Rommelshausen)
	WAIBLINGEN_JESISTRASSE = 'de:08119:5950'
	WAIBLINGEN_JESISTRASSE_1 = 'de:08119:5950:1:3'  # Rtg Devizestr. (Waiblingen Waiblingen)
	WAIBLINGEN_JESISTRASSE_2 = 'de:08119:5950:1:4'  # Rtg Danziger Platz (Waiblingen Waiblingen)
	JESISTRASSE = 'de:08119:5950'
	JESISTRASSE_1 = 'de:08119:5950:1:3'  # Rtg Devizestr. (Waiblingen Waiblingen)
	JESISTRASSE_2 = 'de:08119:5950:1:4'  # Rtg Danziger Platz (Waiblingen Waiblingen)
	BEINSTEIN_MÜHLWEG = 'de:08119:5951'
	BEINSTEIN_MÜHLWEG_1 = 'de:08119:5951:0:3'  # Rtg Beinstein Endersb. Str. (Waiblingen Beinstein)
	BEINSTEIN_MÜHLWEG_2 = 'de:08119:5951:0:4'  # Rtg Waiblingen/Nachtbus Rtg Stetten (Waiblingen Beinstein)
	BAACH = 'de:08119:5952:0:3'  # Ankunft (Weinstadt Schnait)
	SCHNAIT_BAACH = 'de:08119:5952:0:3'  # Ankunft (Weinstadt Schnait)
	QUELLENSTRASSE = 'de:08119:5953:0:3'  # Rtg Endersb. Str. (Waiblingen Beinstein)
	BEINSTEIN_QUELLENSTRASSE = 'de:08119:5953:0:3'  # Rtg Endersb. Str. (Waiblingen Beinstein)
	ALTE_BUNDESSTRASSE = 'de:08119:5955'
	ALTE_BUNDESSTRASSE_1 = 'de:08119:5955:0:3'  # eRtg Eisentalstraße (Waiblingen Waiblingen)
	ALTE_BUNDESSTRASSE_2 = 'de:08119:5955:0:4'  # Rtg Neue Rommelshauser Str. (Waiblingen Waiblingen)
	WAIBLINGEN_ALTE_BUNDESSTR = 'de:08119:5955'
	WAIBLINGEN_ALTE_BUNDESSTR_1 = 'de:08119:5955:0:3'  # eRtg Eisentalstraße (Waiblingen Waiblingen)
	WAIBLINGEN_ALTE_BUNDESSTR_2 = 'de:08119:5955:0:4'  # Rtg Neue Rommelshauser Str. (Waiblingen Waiblingen)
	WAIBLINGEN_EISENTALSTRASSE = 'de:08119:5956:0:3'  # eRtg Schänzle (Waiblingen Waiblingen)
	EISENTALSTRASSE = 'de:08119:5956:0:3'  # eRtg Schänzle (Waiblingen Waiblingen)
	SCHÄNZLE = 'de:08119:5957:0:3'  # eRtg Anton-Schmidt-Straße (Waiblingen Waiblingen)
	WAIBLINGEN_SCHÄNZLE = 'de:08119:5957:0:3'  # eRtg Anton-Schmidt-Straße (Waiblingen Waiblingen)
	WAIBLINGEN_FRIEDHOF = 'de:08119:5958'
	WAIBLINGEN_FRIEDHOF_1 = 'de:08119:5958:0:3'  # Rtg Seniorenzentrum/Querspange (Waiblingen Waiblingen)
	WAIBLINGEN_FRIEDHOF_2 = 'de:08119:5958:0:4'  # eRtg Schwabstraße (Waiblingen Waiblingen)
	GRAUHALDE = 'de:08119:5959'
	GRAUHALDE_1 = 'de:08119:5959:0:3'  # Rtg Norden (Mönchsbrücke) (Schorndorf Schorndorf)
	GRAUHALDE_2 = 'de:08119:5959:0:4'  # Rtg Süden (Lindenweg/Vier Pappeln) (Schorndorf Schorndorf)
	SCHORNDORF_GRAUHALDE = 'de:08119:5959'
	SCHORNDORF_GRAUHALDE_1 = 'de:08119:5959:0:3'  # Rtg Norden (Mönchsbrücke) (Schorndorf Schorndorf)
	SCHORNDORF_GRAUHALDE_2 = 'de:08119:5959:0:4'  # Rtg Süden (Lindenweg/Vier Pappeln) (Schorndorf Schorndorf)
	BURGGYMNASIUM = 'de:08119:5960'
	BURGGYMNASIUM_1 = 'de:08119:5960:0:3'  # Rtg Bahnhof/Grauhalde/SOS (Schorndorf Schorndorf)
	BURGGYMNASIUM_2 = 'de:08119:5960:0:4'  # Rtg Künkelin/Weiler/Bahnhof (Schorndorf Schorndorf)
	SCHORNDORF_BURGGYMNASIUM = 'de:08119:5960'
	SCHORNDORF_BURGGYMNASIUM_1 = 'de:08119:5960:0:3'  # Rtg Bahnhof/Grauhalde/SOS (Schorndorf Schorndorf)
	SCHORNDORF_BURGGYMNASIUM_2 = 'de:08119:5960:0:4'  # Rtg Künkelin/Weiler/Bahnhof (Schorndorf Schorndorf)
	SCHORNDORF_KÜNKELINHALLE = 'de:08119:5961:0:3'  # Rtg Schorndorf Bf (Schorndorf Schorndorf)
	KÜNKELINHALLE = 'de:08119:5961:0:3'  # Rtg Schorndorf Bf (Schorndorf Schorndorf)
	SCHORNDORF_TALSTRASSE = 'de:08119:5962:0:3'  # eRtg Mittlere Brücke (Schorndorf Schorndorf)
	BIRKMANNSWEILER_INDUSTRIEGEB = 'de:08119:5968'
	BIRKMANNSWEILER_INDUSTRIEGEB_1 = 'de:08119:5968:0:3'  # eRtg Lehnenberg Kreuzung (Winnenden Birkmannsweiler)
	BIRKMANNSWEILER_INDUSTRIEGEB_2 = 'de:08119:5968:0:4'  # eRtg Ortsmitte (Winnenden Birkmannsweiler)
	SCHLOSSSTRASSE_1 = 'de:08119:5969:0:3'  # eRtg Haydnstraße (Schorndorf Schorndorf)
	SCHLOSSSTRASSE_2 = 'de:08119:5969:0:4'  # eRtg Künkelinhalle (Schorndorf Schorndorf)
	SCHLOSSSTRASSE_3 = 'de:08118:6718:0:3'  # eRtg Brühlstraße (Ingersheim Kleiningersheim)
	SCHLOSSSTRASSE_4 = 'de:08118:6718:0:4'  # eRtg Blumenstraße (Ingersheim Kleiningersheim)
	SCHORNDORF_SCHLOSSSTRASSE = 'de:08119:5969'
	SCHORNDORF_SCHLOSSSTRASSE_1 = 'de:08119:5969:0:3'  # eRtg Haydnstraße (Schorndorf Schorndorf)
	SCHORNDORF_SCHLOSSSTRASSE_2 = 'de:08119:5969:0:4'  # eRtg Künkelinhalle (Schorndorf Schorndorf)
	GOTTLOB_KAMM_PLATZ = 'de:08119:5970'
	GOTTLOB_KAMM_PLATZ_1 = 'de:08119:5970:0:3'  # eRtg Kreiskrankenhaus (Schorndorf Schorndorf)
	GOTTLOB_KAMM_PLATZ_2 = 'de:08119:5970:0:4'  # eRtg Haydnstraße (Schorndorf Schorndorf)
	SCHORNDORF_GOTTLOB_KAMM_PLATZ = 'de:08119:5970'
	SCHORNDORF_GOTTLOB_KAMM_PLATZ_1 = 'de:08119:5970:0:3'  # eRtg Kreiskrankenhaus (Schorndorf Schorndorf)
	SCHORNDORF_GOTTLOB_KAMM_PLATZ_2 = 'de:08119:5970:0:4'  # eRtg Haydnstraße (Schorndorf Schorndorf)
	RÖHRACHWEG = 'de:08119:5971:0:3'  # eRtg Buchenweg (Schorndorf Schorndorf)
	SCHORNDORF_RÖHRACHWEG = 'de:08119:5971:0:3'  # eRtg Buchenweg (Schorndorf Schorndorf)
	SCHORNDORF_LINDENWEG = 'de:08119:5973:0:3'  # eRtg Grauhalde (Schorndorf Schorndorf)
	LINDENWEG = 'de:08119:5973:0:3'  # eRtg Grauhalde (Schorndorf Schorndorf)
	SCHORNDORF_STUTTGARTER_STR = 'de:08119:5974:0:4'  # eRtg Schorndorf (Schorndorf Schorndorf)
	SCHORNDORF_NEUER_FRIEDHOF = 'de:08119:5975:0:3'  # eRtg Auerhahnstraße (Schorndorf Schorndorf)
	SCHORNDORF_AUERHAHNSTRASSE = 'de:08119:5976:0:3'  # eRtg Epplerinweg (Schorndorf Schorndorf)
	AUERHAHNSTRASSE = 'de:08119:5976:0:3'  # eRtg Epplerinweg (Schorndorf Schorndorf)
	SCHORNDORF_EPPLERINWEG = 'de:08119:5977:0:3'  # eRtg Reinhold-Maier-Platz (Schorndorf Schorndorf)
	EPPLERINWEG = 'de:08119:5977:0:3'  # eRtg Reinhold-Maier-Platz (Schorndorf Schorndorf)
	REINHOLD_MAIER_PLATZ = 'de:08119:5978'
	REINHOLD_MAIER_PLATZ_1 = 'de:08119:5978:0:3'  # Rtg Hohenstaufenstr. (Schorndorf Schorndorf)
	REINHOLD_MAIER_PLATZ_2 = 'de:08119:5978:0:4'  # Rtg Bahnhof (Schorndorf Schorndorf)
	SCHORND__REINHOLD_MAIER_PLATZ = 'de:08119:5978'
	SCHORND__REINHOLD_MAIER_PLATZ_1 = 'de:08119:5978:0:3'  # Rtg Hohenstaufenstr. (Schorndorf Schorndorf)
	SCHORND__REINHOLD_MAIER_PLATZ_2 = 'de:08119:5978:0:4'  # Rtg Bahnhof (Schorndorf Schorndorf)
	SCHORNDORF_HOHENSTAUFENSTR = 'de:08119:5979'
	SCHORNDORF_HOHENSTAUFENSTR_1 = 'de:08119:5979:0:3'  # eRtg Kleingärten (Schorndorf Schorndorf)
	SCHORNDORF_HOHENSTAUFENSTR_2 = 'de:08119:5979:0:4'  # eRtg Reinhold-Maier-Platz (Schorndorf Schorndorf)
	JUGENDHERBERGE_1 = 'de:08119:5981:0:3'  # Rtg Erich-Schumm-Str. (Murrhardt Murrhardt)
	JUGENDHERBERGE_2 = 'de:08117:1305:0:H1'  # Rtg Maitis (Göppingen Hohenstaufen)
	MURRHARDT_JUGENDHERBERGE = 'de:08119:5981:0:3'  # Rtg Erich-Schumm-Str. (Murrhardt Murrhardt)
	STRÜMPFELBACH = 'de:08119:5983'
	STRÜMPFELBACH_1 = 'de:08119:5983:0:3'  # Rtg Backnang (Backnang Strümpfelbach (Backnang))
	STRÜMPFELBACH_2 = 'de:08119:5983:0:4'  # Rtg Oppenweiler (Backnang Strümpfelbach (Backnang))
	HARBACH = 'de:08119:5984'
	HARBACH_1 = 'de:08119:5984:0:3'  # eRtg Bartenbach (Murrhardt Murrhardt)
	HARBACH_2 = 'de:08119:5984:0:4'  # eRtg Hörschbachschule (Murrhardt Murrhardt)
	MURRHARDT_HARBACH = 'de:08119:5984'
	MURRHARDT_HARBACH_1 = 'de:08119:5984:0:3'  # eRtg Bartenbach (Murrhardt Murrhardt)
	MURRHARDT_HARBACH_2 = 'de:08119:5984:0:4'  # eRtg Hörschbachschule (Murrhardt Murrhardt)
	GÖCKELHOF = 'de:08119:5985:0:3'  # eRtg Murrhardt (Murrhardt Kirchenkirnberg)
	KIRCHENK__GÖCKELHOF = 'de:08119:5985:0:3'  # eRtg Murrhardt (Murrhardt Kirchenkirnberg)
	KIRCHENKIRNBERG_SCHLOSSMÜHLE = 'de:08119:5986:0:3'  # eRtg Mettelbach (Murrhardt Kirchenkirnberg)
	SCHLOSSMÜHLE = 'de:08119:5986:0:3'  # eRtg Mettelbach (Murrhardt Kirchenkirnberg)
	KIRCHENKIRNBERG_MARXENHOF = 'de:08119:5987:0:3'  # eRtg Göckelhof (Murrhardt Kirchenkirnberg)
	MARXENHOF = 'de:08119:5987:0:3'  # eRtg Göckelhof (Murrhardt Kirchenkirnberg)
	PLÜDERHAUSEN_GRABENSTRASSE = 'de:08119:5988'
	PLÜDERHAUSEN_GRABENSTRASSE_1 = 'de:08119:5988:0:3'  # Rtg Königsbau (Plüderhausen Plüderhausen)
	PLÜDERHAUSEN_GRABENSTRASSE_2 = 'de:08119:5988:0:4'  # Rtg Postweg (Plüderhausen Plüderhausen)
	OBERBERK__SOS_KINDERDORF = 'de:08119:5989'
	OBERBERK__SOS_KINDERDORF_1 = 'de:08119:5989:0:3'  # Rtg Höhenweg Schurwaldhöhe (Schorndorf Oberberken)
	OBERBERK__SOS_KINDERDORF_2 = 'de:08119:5989:0:4'  # Rtg Schorndorf Adelberg (Schorndorf Oberberken)
	SOS_KINDERDORF = 'de:08119:5989'
	SOS_KINDERDORF_1 = 'de:08119:5989:0:3'  # Rtg Höhenweg Schurwaldhöhe (Schorndorf Oberberken)
	SOS_KINDERDORF_2 = 'de:08119:5989:0:4'  # Rtg Schorndorf Adelberg (Schorndorf Oberberken)
	OBERBERKEN_SCHURWALDHALLE = 'de:08119:5990'
	OBERBERKEN_SCHURWALDHALLE_1 = 'de:08119:5990:0:3'  # eRtg Unterberken Herrenbachstraße (Schorndorf Oberberken)
	OBERBERKEN_SCHURWALDHALLE_2 = 'de:08119:5990:0:4'  # eRtg Unterberken Backhaus (Schorndorf Oberberken)
	SCHURWALDHALLE = 'de:08119:5990'
	SCHURWALDHALLE_1 = 'de:08119:5990:0:3'  # eRtg Unterberken Herrenbachstraße (Schorndorf Oberberken)
	SCHURWALDHALLE_2 = 'de:08119:5990:0:4'  # eRtg Unterberken Backhaus (Schorndorf Oberberken)
	WINNENDEN_WOLFSKLINGENWEG = 'de:08119:5991'
	WINNENDEN_WOLFSKLINGENWEG_1 = 'de:08119:5991:0:3'  # eRtg Burkhardshof/Höfen (Winnenden Winnenden)
	WINNENDEN_WOLFSKLINGENWEG_2 = 'de:08119:5991:0:4'  # eRtg Friedhof (Winnenden Winnenden)
	WOLFSKLINGENWEG = 'de:08119:5991'
	WOLFSKLINGENWEG_1 = 'de:08119:5991:0:3'  # eRtg Burkhardshof/Höfen (Winnenden Winnenden)
	WOLFSKLINGENWEG_2 = 'de:08119:5991:0:4'  # eRtg Friedhof (Winnenden Winnenden)
	BURKHARDSHOF = 'de:08119:5992'
	BURKHARDSHOF_1 = 'de:08119:5992:0:3'  # eRtg Ortsmitte (Winnenden Birkmannsweiler)
	BURKHARDSHOF_2 = 'de:08119:5992:0:4'  # eRtg Winnenden/Höfen (Winnenden Birkmannsweiler)
	BIRKMANNSW__BURKHARDSHOF = 'de:08119:5992'
	BIRKMANNSW__BURKHARDSHOF_1 = 'de:08119:5992:0:3'  # eRtg Ortsmitte (Winnenden Birkmannsweiler)
	BIRKMANNSW__BURKHARDSHOF_2 = 'de:08119:5992:0:4'  # eRtg Winnenden/Höfen (Winnenden Birkmannsweiler)
	LIPPOLDSWEILER_ABZW__DÄFERN = 'de:08119:5993'
	LIPPOLDSWEILER_ABZW__DÄFERN_1 = 'de:08119:5993:0:3'  # Rtg Hohnw. Rathausstr. (Auenwald Lippoldsweiler)
	LIPPOLDSWEILER_ABZW__DÄFERN_2 = 'de:08119:5993:0:4'  # Rtg. BK/Bruch (Auenwald Lippoldsweiler)
	ABZWEIG_DÄFERN = 'de:08119:5993'
	ABZWEIG_DÄFERN_1 = 'de:08119:5993:0:3'  # Rtg Hohnw. Rathausstr. (Auenwald Lippoldsweiler)
	ABZWEIG_DÄFERN_2 = 'de:08119:5993:0:4'  # Rtg. BK/Bruch (Auenwald Lippoldsweiler)
	UNTERW__AICHHOLZHOF = 'de:08119:5994'
	UNTERW__AICHHOLZHOF_1 = 'de:08119:5994:0:3'  # eRtg Weissachstraße (Weissach im Tal Unterweissach)
	UNTERW__AICHHOLZHOF_2 = 'de:08119:5994:0:4'  # eRtg Edeka (Weissach im Tal Unterweissach)
	AICHHOLZHOF = 'de:08119:5994'
	AICHHOLZHOF_1 = 'de:08119:5994:0:3'  # eRtg Weissachstraße (Weissach im Tal Unterweissach)
	AICHHOLZHOF_2 = 'de:08119:5994:0:4'  # eRtg Edeka (Weissach im Tal Unterweissach)
	COTTENW__BILDUNGSZENTRUM = 'de:08119:5995:0:3'  # eRtg Alle Richtungen (Weissach im Tal Cottenweiler)
	HOHNWEILER_FORSTSTRASSE = 'de:08119:5996:0:3'  # Rtg Raiffeisenbank (Auenwald Lippoldsweiler)
	LIPPOLDSW__HOHNW__FORSTSTR = 'de:08119:5996:0:3'  # Rtg Raiffeisenbank (Auenwald Lippoldsweiler)
	FRAUENKREUZ = 'de:08115:5998'
	FRAUENKREUZ_1 = 'de:08115:5998:0:3'  # Rtg Sindelfingen (Leonberg Leonberg)
	FRAUENKREUZ_2 = 'de:08115:5998:0:4'  # Rtg Warmbronn (Leonberg Leonberg)
	LEONBERG_FRAUENKREUZ = 'de:08115:5998'
	LEONBERG_FRAUENKREUZ_1 = 'de:08115:5998:0:3'  # Rtg Sindelfingen (Leonberg Leonberg)
	LEONBERG_FRAUENKREUZ_2 = 'de:08115:5998:0:4'  # Rtg Warmbronn (Leonberg Leonberg)
	MAUSERSTRASSE = 'de:08118:5999'
	MAUSERSTRASSE_1 = 'de:08118:5999:0:3'  # eRtg Ostfriedhof (Ludwigsburg Oßweil)
	MAUSERSTRASSE_2 = 'de:08118:5999:0:4'  # eRtg Westfalenstraße (Ludwigsburg Oßweil)
	OSSWEIL_MAUSERSTRASSE = 'de:08118:5999'
	OSSWEIL_MAUSERSTRASSE_1 = 'de:08118:5999:0:3'  # eRtg Ostfriedhof (Ludwigsburg Oßweil)
	OSSWEIL_MAUSERSTRASSE_2 = 'de:08118:5999:0:4'  # eRtg Westfalenstraße (Ludwigsburg Oßweil)
	OSSWEIL_OSTFRIEDHOF = 'de:08118:6000'
	OSSWEIL_OSTFRIEDHOF_1 = 'de:08118:6000:0:3'  # eRtg Bank (Ludwigsburg Oßweil)
	OSSWEIL_OSTFRIEDHOF_2 = 'de:08118:6000:0:4'  # eRtg Mauserstraße (Ludwigsburg Oßweil)
	OSTFRIEDHOF = 'de:08118:6000'
	OSTFRIEDHOF_1 = 'de:08118:6000:0:3'  # eRtg Bank (Ludwigsburg Oßweil)
	OSTFRIEDHOF_2 = 'de:08118:6000:0:4'  # eRtg Mauserstraße (Ludwigsburg Oßweil)
	VAIHINGEN = 'de:08111:6002'
	VAIHINGEN_1 = 'de:08111:6002:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	VAIHINGEN_2 = 'de:08111:6002:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	VAIHINGEN_3 = 'de:08111:6002:1:3'  # Gleis 3 (Stuttgart Stuttgart)
	VAIHINGEN_4 = 'de:08111:6002:12:4'  # Gleis 4 (Stuttgart Stuttgart)
	VAIHINGEN_5 = 'de:08111:6002:2:11'  # Stadtbahn - U3/U8 Abfahrt (Stuttgart Stuttgart)
	VAIHINGEN_6 = 'de:08111:6002:3:12'  # Stadtbahn - U3/U8 Ankunft (Stuttgart Stuttgart)
	VAIHINGEN_7 = 'de:08111:6002:3:13'  # Stadtbahn - U1 Abfahrt (Stuttgart Stuttgart)
	VAIHINGEN_8 = 'de:08111:6002:4:14'  # Stadtbahn - U1 Ankunft (Stuttgart Stuttgart)
	VAIHINGEN_9 = 'de:08111:6002:5:1'  # Linie 751 Rtg Schwabengalerie (Stuttgart Stuttgart)
	VAIHINGEN_10 = 'de:08111:6002:5:2'  # Bstg 2 (Stuttgart Stuttgart)
	VAIHINGEN_11 = 'de:08111:6002:5:3'  # Linie 86 Rtg Waldburgstr. (Stuttgart Stuttgart)
	VAIHINGEN_12 = 'de:08111:6002:5:4'  # Linie 84 Rtg Schwabengalerie (Stuttgart Stuttgart)
	VAIHINGEN_SCHILLERPLATZ = 'de:08111:6004'
	VAIHINGEN_SCHILLERPLATZ_1 = 'de:08111:6004:1:1'  # Rtg Vaihingen Bf (Stuttgart Stuttgart)
	VAIHINGEN_SCHILLERPLATZ_2 = 'de:08111:6004:1:2'  # Rtg Fauststr. (Stuttgart Stuttgart)
	VAIHINGEN_SCHILLERPLATZ_3 = 'de:08111:6004:2:3'  # Rtg Fanny-Leicht-Str. (Stuttgart Stuttgart)
	VAIHINGEN_BF__OST = 'de:08111:6006:1:7'  # Bstg 7 (Stuttgart Stuttgart)
	BERGHAUSTRASSE = 'de:08111:6007'
	BERGHAUSTRASSE_1 = 'de:08111:6007:0:3'  # Rtg Leinfelden Schmellbachtal (Stuttgart Stuttgart)
	BERGHAUSTRASSE_2 = 'de:08111:6007:0:4'  # Rtg Rohr Mitte (Stuttgart Stuttgart)
	UNIVERSITÄT = 'de:08111:6008'
	UNIVERSITÄT_1 = 'de:08111:6008:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	UNIVERSITÄT_2 = 'de:08111:6008:2:2'  # Gleis 2 (Stuttgart Stuttgart)
	UNIVERSITÄT_3 = 'de:08111:6008:3:3'  # Rtg Schranne (Stuttgart Stuttgart)
	UNIVERSITÄT_4 = 'de:08111:6008:4:4'  # Rtg Universität/Forsthaus I (Stuttgart Stuttgart)
	KALTENTAL = 'de:08111:6009'
	KALTENTAL_1 = 'de:08111:6009:0:1'  # Rtg Engelboldstr. (Stuttgart Stuttgart)
	KALTENTAL_2 = 'de:08111:6009:0:2'  # Rtg Waldeck (Stuttgart Stuttgart)
	KALTENTAL_3 = 'de:08111:6009:0:5'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	WALDECK = 'de:08111:6010'
	WALDECK_1 = 'de:08111:6010:1:1'  # Rtg Kaltental (Stuttgart Stuttgart)
	WALDECK_2 = 'de:08111:6010:1:2'  # Rtg Heslach Vogelrain (Stuttgart Stuttgart)
	WALDECK_3 = 'de:08111:6010:2:3'  # Rtg Im Elsental (Stuttgart Stuttgart)
	WALDECK_4 = 'de:08111:6010:2:4'  # L.82/Ankunft (Stuttgart Stuttgart)
	WALDECK_5 = 'de:08111:6010:3:5'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	KATZENBACHSTRASSE = 'de:08111:6011'
	KATZENBACHSTRASSE_1 = 'de:08111:6011:0:3'  # Rtg Heerstr. (Stuttgart Stuttgart)
	KATZENBACHSTRASSE_2 = 'de:08111:6011:0:4'  # Rtg Am Feldrand (Stuttgart Stuttgart)
	HEGEL_GYMNASIUM = 'de:08111:6013'
	HEGEL_GYMNASIUM_1 = 'de:08111:6013:0:3'  # Rtg Rohr Mitte (Stuttgart Stuttgart)
	HEGEL_GYMNASIUM_2 = 'de:08111:6013:0:4'  # Rtg Waldburgstr. (Stuttgart Stuttgart)
	ROHR_MITTE = 'de:08111:6014'
	ROHR_MITTE_1 = 'de:08111:6014:1:2'  # Rtg Hegelgymnasium/Thingstr. (Stuttgart Stuttgart)
	ROHR_MITTE_2 = 'de:08111:6014:3:1'  # Ankunft und Rtg Berghaustr. (Stuttgart Stuttgart)
	ROHR_MITTE_3 = 'de:08111:6014:4:3'  # Rtg Rohr Bf/Berghaustr. (Stuttgart Stuttgart)
	PESTALOZZI_SCHULE = 'de:08111:6015'
	PESTALOZZI_SCHULE_1 = 'de:08111:6015:0:3'  # Rtg Albblick (Stuttgart Stuttgart)
	PESTALOZZI_SCHULE_2 = 'de:08111:6015:0:4'  # Rtg Waldburgstr. (Stuttgart Stuttgart)
	ERWIN_SCHOETTLE_PLATZ = 'de:08111:6017'
	ERWIN_SCHOETTLE_PLATZ_1 = 'de:08111:6017:1:1'  # Rtg Bihlplatz (Stuttgart Stuttgart)
	ERWIN_SCHOETTLE_PLATZ_2 = 'de:08111:6017:1:2'  # Rtg Marienplatz (Stuttgart Stuttgart)
	ERWIN_SCHOETTLE_PLATZ_3 = 'de:08111:6017:2:3'  # Linie 42 Ankunft (Stuttgart Stuttgart)
	ERWIN_SCHOETTLE_PLATZ_4 = 'de:08111:6017:3:6'  # Rtg Schickhardtschule (Stuttgart Stuttgart)
	NOBELSTRASSE = 'de:08111:6018'
	NOBELSTRASSE_1 = 'de:08111:6018:0:3'  # Rtg Lauchhau (Stuttgart Stuttgart)
	NOBELSTRASSE_2 = 'de:08111:6018:0:4'  # Rtg Schranne/Johannesgraben (Stuttgart Stuttgart)
	MARIENPLATZ = 'de:08111:6019'
	MARIENPLATZ_1 = 'de:08111:6019:1:2'  # Rtg Österreichischer Platz (Stuttgart Stuttgart)
	MARIENPLATZ_2 = 'de:08111:6019:2:1'  # Rtg Erwin-Schöttle-Platz (Stuttgart Stuttgart)
	MARIENPLATZ_3 = 'de:08111:6019:3:7'  # Zahnradbahn 10 Endhst. (Stuttgart Stuttgart)
	MARIENPLATZ_4 = 'de:08111:6019:4:4'  # Rtg Mörikestr. (Stuttgart Stuttgart)
	MARIENPLATZ_5 = 'de:08111:6019:5:3'  # L.41 Rtg Marienhosp./L.43 Rtg Lehenstr. (Stuttgart Stuttgart)
	MARIENPLATZ_6 = 'de:08111:6019:6:5'  # N11 Rtg VB (Stuttgart Stuttgart)
	MARIENPLATZ_7 = 'de:08111:6019:7:6'  # N 1 Rtg SPN (Stuttgart Stuttgart)
	ÖSTERREICHISCHER_PLATZ = 'de:08111:6020'
	ÖSTERREICHISCHER_PLATZ_1 = 'de:08111:6020:1:2'  # Rtg Rathaus/Rotebühlplatz (Stuttgart Stuttgart)
	ÖSTERREICHISCHER_PLATZ_2 = 'de:08111:6020:2:1'  # Rtg Marienplatz (Stuttgart Stuttgart)
	ÖSTERREICHISCHER_PLATZ_3 = 'de:08111:6020:3:3'  # Rtg Marienstr. (Stuttgart Stuttgart)
	ÖSTERREICHISCHER_PLATZ_4 = 'de:08111:6020:4:4'  # Rtg Rathaus (Stuttgart Stuttgart)
	ÖSTERREICHISCHER_PLATZ_5 = 'de:08111:6020:5:2'  # Zusatzhalt Rtg Rathaus (Stuttgart Stuttgart)
	UNIVERSITÄT__SCHLEIFE = 'de:08111:6021'
	UNIVERSITÄT__SCHLEIFE_1 = 'de:08111:6021:0:3'  # Postion 2 Linie X60, X74 (Stuttgart Stuttgart)
	UNIVERSITÄT__SCHLEIFE_2 = 'de:08111:6021:0:4'  # Position 1 Linie 84 (Stuttgart Stuttgart)
	DOROTHEENSTRASSE = 'de:08111:6023:1:3'  # Linien 43/44 Rtg Rathaus (Stuttgart Stuttgart)
	STAATSGALERIE = 'de:08111:6024'
	STAATSGALERIE_1 = 'de:08111:6024:10:1'  # St von Neckartor ab 09/2020 (Stuttgart Stuttgart)
	STAATSGALERIE_2 = 'de:08111:6024:11:2'  # St von Charlottenplatz ab 09/2020 (Stuttgart Stuttgart)
	STAATSGALERIE_3 = 'de:08111:6024:11:3'  # St von Hbf ab ??? (Stuttgart Stuttgart)
	STAATSGALERIE_4 = 'de:08111:6024:5:2'  # Pos. 2 - Bus Li.42 Rtg Hbf (Stuttgart Stuttgart)
	STAATSGALERIE_5 = 'de:08111:6024:5:3'  # Pos. 3 - Bus Li.42 Rtg Friedenstraße (Stuttgart Stuttgart)
	STAATSGALERIE_6 = 'de:08111:6024:6:1'  # Pos. 1 - Bus Li.40 / 44  Rtg Hbf (Stuttgart Stuttgart)
	STAATSGALERIE_7 = 'de:08111:6024:8:5'  # Pos. 5 - Bus Li.40 Rtg Tunnel Ostportal (Stuttgart Stuttgart)
	STAATSGALERIE_8 = 'de:08111:6024:9:4'  # Pos. 4 - Bus Li. N4, N6 Rtg Neckartor (Stuttgart Stuttgart)
	ÖSTERFELD = 'de:08111:6027'
	ÖSTERFELD_1 = 'de:08111:6027:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	ÖSTERFELD_2 = 'de:08111:6027:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	FELLBACH_MOZARTSTRASSE = 'de:08119:6039'
	FELLBACH_MOZARTSTRASSE_1 = 'de:08119:6039:0:3'  # L.60 Rtg Lutherkirche (Fellbach Fellbach)
	FELLBACH_MOZARTSTRASSE_2 = 'de:08119:6039:0:4'  # L.67 Rtg Lutherkirche (Fellbach Fellbach)
	LUTHERKIRCHE = 'de:08119:6040'
	LUTHERKIRCHE_1 = 'de:08119:6040:1:1'  # Stadtbahn - Endhst. (Fellbach Fellbach)
	LUTHERKIRCHE_2 = 'de:08119:6040:2:3'  # Rtg A.Br.Str./Kleinf.friedh./Silcherweg (Fellbach Fellbach)
	LUTHERKIRCHE_3 = 'de:08119:6040:2:5'  # Rtg Schwabenlandhalle/F3 (Fellbach Fellbach)
	LUTHERKIRCHE_4 = 'de:08119:6040:3:4'  # L.60 Rtg Cannst. Pl./L.67 Rtg Silcherweg (Fellbach Fellbach)
	FELLBACH_LUTHERKIRCHE = 'de:08119:6040'
	FELLBACH_LUTHERKIRCHE_1 = 'de:08119:6040:1:1'  # Stadtbahn - Endhst. (Fellbach Fellbach)
	FELLBACH_LUTHERKIRCHE_2 = 'de:08119:6040:2:3'  # Rtg A.Br.Str./Kleinf.friedh./Silcherweg (Fellbach Fellbach)
	FELLBACH_LUTHERKIRCHE_3 = 'de:08119:6040:2:5'  # Rtg Schwabenlandhalle/F3 (Fellbach Fellbach)
	FELLBACH_LUTHERKIRCHE_4 = 'de:08119:6040:3:4'  # L.60 Rtg Cannst. Pl./L.67 Rtg Silcherweg (Fellbach Fellbach)
	CANNSTATTER_PLATZ = 'de:08119:6041:0:4'  # Rtg Stuttgarter Platz (Fellbach Fellbach)
	FELLBACH_CANNSTATTER_PLATZ = 'de:08119:6041:0:4'  # Rtg Stuttgarter Platz (Fellbach Fellbach)
	FELLBACH_STUTTGARTER_PL = 'de:08119:6042:0:4'  # Rtg Pauluskirche (Fellbach Fellbach)
	STUTTGARTER_PLATZ = 'de:08119:6042:0:4'  # Rtg Pauluskirche (Fellbach Fellbach)
	SINDELF__BREUNINGERLAND = 'de:08115:6043'
	SINDELF__BREUNINGERLAND_1 = 'de:08115:6043:1:3'  # Rtg Hofmeister/Schauwerk (Sindelfingen Sindelfingen)
	SINDELF__BREUNINGERLAND_2 = 'de:08115:6043:1:4'  # Rtg Textilzentrum (Sindelfingen Sindelfingen)
	HÖFINGEN_RATHAUS = 'de:08115:6045'
	HÖFINGEN_RATHAUS_1 = 'de:08115:6045:1:3'  # Rtg Hirschlander/Alter Bfweg (Leonberg Höfingen)
	HÖFINGEN_RATHAUS_2 = 'de:08115:6045:1:4'  # Rtg Am Schlossberg (Leonberg Höfingen)
	STEINBACH_LINDENPLATZ = 'de:08119:6046'
	STEINBACH_LINDENPLATZ_1 = 'de:08119:6046:0:3'  # eRtg Dorfhalle (Backnang Steinbach)
	STEINBACH_LINDENPLATZ_2 = 'de:08119:6046:0:4'  # eRtg Sommerrain (Backnang Steinbach)
	LACHENTORSTRASSE = 'de:08115:6049'
	LACHENTORSTRASSE_1 = 'de:08115:6049:0:3'  # Rtg A.Schweitzer-Str. (Leonberg Höfingen)
	LACHENTORSTRASSE_2 = 'de:08115:6049:0:4'  # Rtg Hirschlander Str. (Leonberg Höfingen)
	HÖFINGEN_LACHENTORSTR = 'de:08115:6049'
	HÖFINGEN_LACHENTORSTR_1 = 'de:08115:6049:0:3'  # Rtg A.Schweitzer-Str. (Leonberg Höfingen)
	HÖFINGEN_LACHENTORSTR_2 = 'de:08115:6049:0:4'  # Rtg Hirschlander Str. (Leonberg Höfingen)
	HÖFINGEN_HIRSCHLANDER_STR = 'de:08115:6050'
	HÖFINGEN_HIRSCHLANDER_STR_1 = 'de:08115:6050:0:3'  # Rtg Lachentor/Mercedesstr. (Leonberg Höfingen)
	HÖFINGEN_HIRSCHLANDER_STR_2 = 'de:08115:6050:0:4'  # Rtg Alter Bfweg/Rathaus (Leonberg Höfingen)
	ALTER_BAHNHOFWEG = 'de:08115:6051:1:3'  # vorher Rathaus (Leonberg Höfingen)
	HÖFINGEN_ALTER_BAHNHOFWEG = 'de:08115:6051:1:3'  # vorher Rathaus (Leonberg Höfingen)
	LEONBERG_STROHGÄUSTRASSE = 'de:08115:6054'
	LEONBERG_STROHGÄUSTRASSE_1 = 'de:08115:6054:0:3'  # Rtg Höfingen (Leonberg Leonberg)
	LEONBERG_STROHGÄUSTRASSE_2 = 'de:08115:6054:0:4'  # Rtg Feuerbacher Str. (Leonberg Leonberg)
	STROHGÄUSTRASSE = 'de:08115:6054'
	STROHGÄUSTRASSE_1 = 'de:08115:6054:0:3'  # Rtg Höfingen (Leonberg Leonberg)
	STROHGÄUSTRASSE_2 = 'de:08115:6054:0:4'  # Rtg Feuerbacher Str. (Leonberg Leonberg)
	LEONBERG_FEUERBACHER_STR = 'de:08115:6055'
	LEONBERG_FEUERBACHER_STR_1 = 'de:08115:6055:0:3'  # Rtg Strohgäustr./Gerlingen (Leonberg Leonberg)
	LEONBERG_FEUERBACHER_STR_2 = 'de:08115:6055:0:4'  # Rtg Altstadt (Leonberg Leonberg)
	FEUERBACHER_STRASSE = 'de:08115:6055'
	FEUERBACHER_STRASSE_1 = 'de:08115:6055:0:3'  # Rtg Strohgäustr./Gerlingen (Leonberg Leonberg)
	FEUERBACHER_STRASSE_2 = 'de:08115:6055:0:4'  # Rtg Altstadt (Leonberg Leonberg)
	OSTENDPLATZ = 'de:08111:6060'
	OSTENDPLATZ_1 = 'de:08111:6060:1:1'  # Rtg Bergfriedhof (Stuttgart Stuttgart)
	OSTENDPLATZ_2 = 'de:08111:6060:1:2'  # Rtg Leo-Vetter-Bad (Stuttgart Stuttgart)
	OSTENDPLATZ_3 = 'de:08111:6060:2:3'  # Linien 42/45 Rtg Schw.bergstr./Leo-V-Bad (Stuttgart Stuttgart)
	OSTENDPLATZ_4 = 'de:08111:6060:2:4'  # Linien 42/45 Rtg Wagenburgstr. (Stuttgart Stuttgart)
	HEGEL__SEIDENSTR = 'de:08111:6068'
	HEGEL__SEIDENSTR_1 = 'de:08111:6068:0:3'  # Rtg Russische Kirche (Stuttgart Stuttgart)
	HEGEL__SEIDENSTR_2 = 'de:08111:6068:0:4'  # Rtg Linden-Museum (Stuttgart Stuttgart)
	HEGEL__SEIDENSTRASSE = 'de:08111:6068'
	HEGEL__SEIDENSTRASSE_1 = 'de:08111:6068:0:3'  # Rtg Russische Kirche (Stuttgart Stuttgart)
	HEGEL__SEIDENSTRASSE_2 = 'de:08111:6068:0:4'  # Rtg Linden-Museum (Stuttgart Stuttgart)
	BISMARCKPLATZ = 'de:08111:6069'
	BISMARCKPLATZ_1 = 'de:08111:6069:0:3'  # Rtg Schwabstr. (Stuttgart Stuttgart West)
	BISMARCKPLATZ_2 = 'de:08111:6069:0:4'  # Rtg Schwab-/Bebelstr. (Stuttgart Stuttgart West)
	HÖLDERLINPLATZ = 'de:08111:6070'
	HÖLDERLINPLATZ_1 = 'de:08111:6070:1:1'  # Stadtbahn Endhaltestelle (Stuttgart Stuttgart)
	HÖLDERLINPLATZ_2 = 'de:08111:6070:2:4'  # Rtg Russische Kirche (Stuttgart Stuttgart)
	HÖLDERLINPLATZ_3 = 'de:08111:6070:3:3'  # Rtg Honoldweg (Stuttgart Stuttgart)
	RUSSISCHE_KIRCHE = 'de:08111:6071'
	RUSSISCHE_KIRCHE_1 = 'de:08111:6071:0:1'  # Rtg Hölderlinplatz (Stuttgart Stuttgart)
	RUSSISCHE_KIRCHE_2 = 'de:08111:6071:0:2'  # Rtg Rosenberg-/Seidenstr. (Stuttgart Stuttgart)
	RUSSISCHE_KIRCHE_3 = 'de:08111:6071:0:3'  # Rtg Hölderlinplatz (Stuttgart Stuttgart)
	RUSSISCHE_KIRCHE_4 = 'de:08111:6071:0:4'  # Rtg Hegel-/Seidenstr. (Stuttgart Stuttgart)
	ROSENBERG__SEIDENSTRASSE = 'de:08111:6072'
	ROSENBERG__SEIDENSTRASSE_1 = 'de:08111:6072:1:2'  # Rtg Berliner Platz (Stuttgart Stuttgart)
	ROSENBERG__SEIDENSTRASSE_2 = 'de:08111:6072:2:1'  # Rtg Russische Kirche (Stuttgart Stuttgart)
	ROSENBERG__SEIDENSTRASSE_3 = 'de:08111:6072:3:4'  # Linien 42/43 Rtg Linden-Museum (Stuttgart Stuttgart)
	ROSENBERG__SEIDENSTRASSE_4 = 'de:08111:6072:4:5'  # Linie 41 Rtg Berliner Platz (Stuttgart Stuttgart)
	ROSENBERG__SEIDENSTRASSE_5 = 'de:08111:6072:4:6'  # Nachtbus N2 (Stuttgart Stuttgart)
	ROSENBERG__SEIDENSTRASSE_6 = 'de:08111:6072:5:3'  # Linie 42 Rtg Rosenberg-/Johannesstr. (Stuttgart Stuttgart)
	BERLINER_PLATZ__LIEDERHALLE = 'de:08111:6073'
	BERLINER_PLATZ__LIEDERHALLE_1 = 'de:08111:6073:1:2'  # Rtg Börsenplatz (Stuttgart Stuttgart)
	BERLINER_PLATZ__LIEDERHALLE_2 = 'de:08111:6073:2:1'  # Rtg Rotebühlplatz/Schloss-/Johannesstr. (Stuttgart Stuttgart)
	BERLINER_PLATZ__LIEDERHALLE_3 = 'de:08111:6073:3:7'  # Rtg Rosenberg-/Seidenstr. (Stuttgart Stuttgart)
	BERLINER_PLATZ__LIEDERHALLE_4 = 'de:08111:6073:4:8'  # U4 Rtg Rotebühlplatz (Stuttgart Stuttgart)
	BERLINER_PLATZ__LIEDERHALLE_5 = 'de:08111:6073:5:5'  # Linie 43 Rtg Rosenberg-/Seidenstr. (Stuttgart Stuttgart)
	BERLINER_PLATZ__LIEDERHALLE_6 = 'de:08111:6073:6:3'  # Linie 41 Rtg Schloss-/Johannesstr. (Stuttgart Stuttgart)
	TAL__LANDHAUSSTRASSE = 'de:08111:6078'
	TAL__LANDHAUSSTRASSE_1 = 'de:08111:6078:2:3'  # Linie 45 Rtg Wagenburgstr. (Stuttgart Stuttgart)
	TAL__LANDHAUSSTRASSE_2 = 'de:08111:6078:2:4'  # Linie 45 Rtg Schlachthof / N5/N7 Rtg WP (Stuttgart Stuttgart)
	NECKARPARK = 'de:08111:6080'
	NECKARPARK_1 = 'de:08111:6080:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	NECKARPARK_2 = 'de:08111:6080:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	NECKARPARK_3 = 'de:08111:6080:3:4'  # eRtg Bad Cannstatt (Stuttgart Stuttgart)
	NECKARPARK_4 = 'de:08111:6080:3:5'  # eRtg Untertürkheim (Stuttgart Stuttgart)
	UNTERTÜRKHEIM = 'de:08111:6085'
	UNTERTÜRKHEIM_1 = 'de:08111:6085:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_2 = 'de:08111:6085:1:3'  # Gleis 3 (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_3 = 'de:08111:6085:2:4'  # Gleis 4 (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_4 = 'de:08111:6085:2:5'  # Gleis 5 (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_5 = 'de:08111:6085:3:6'  # Gleis 6 (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_6 = 'de:08111:6085:4:1'  # Stadtbahn - U4 Abfahrt (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_7 = 'de:08111:6085:4:2'  # Stadtbahn - U4 Ankunft (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_8 = 'de:08111:6085:5:3'  # L.60 Rtg Großglocknerstr. (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_9 = 'de:08111:6085:5:5'  # L.61 Rtg Großglocknerstr./Obertürkheim (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_10 = 'de:08111:6085:5:6'  # L.61 Rtg Großglocknerstr./Rotenberg (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_11 = 'de:08111:6085:5:7'  # N7 (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_12 = 'de:08111:6085:6:8'  # Rtg Schlotterbeckstr. (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_13 = 'de:08111:6085:7:7'  # Rtg Wasenstr. (Stuttgart Stuttgart)
	UNTERTÜRKHEIM_14 = 'de:08111:6085:8:4'  # N6 (Stuttgart Stuttgart)
	OBERTÜRKHEIM = 'de:08111:6091'
	OBERTÜRKHEIM_1 = 'de:08111:6091:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	OBERTÜRKHEIM_2 = 'de:08111:6091:1:5'  # Gleis 5 (Stuttgart Stuttgart)
	OBERTÜRKHEIM_3 = 'de:08111:6091:1:6'  # Gleis 6 (Stuttgart Stuttgart)
	OBERTÜRKHEIM_4 = 'de:08111:6091:2:4'  # Gleis 4 (Stuttgart Stuttgart)
	OBERTÜRKHEIM_5 = 'de:08111:6091:3:1'  # L.62/65 Rtg Im Mäder/L.65 Rtg Bf(GP-Str) (Stuttgart Stuttgart)
	OBERTÜRKHEIM_6 = 'de:08111:6091:3:2'  # L.61 Rtg Bf (Göpp.Str) (Stuttgart Stuttgart)
	OBERTÜRKHEIM_7 = 'de:08111:6091:3:7'  # Ersatzhst. i. d. Hafenbahnstraße (Stuttgart Stuttgart)
	OBERTÜRKHEIM_8 = 'de:08111:6091:4:3'  # Bus 101 (Stuttgart Stuttgart)
	EIERNEST = 'de:08111:6098:0:4'  # Rtg Dornhaldenstr. (Stuttgart Stuttgart)
	DORNHALDENSTRASSE = 'de:08111:6099'
	DORNHALDENSTRASSE_1 = 'de:08111:6099:0:3'  # Rtg Lerchenrainschule (Stuttgart Stuttgart)
	DORNHALDENSTRASSE_2 = 'de:08111:6099:0:4'  # Rtg Marienhospital (Stuttgart Stuttgart)
	LERCHENRAINSCHULE = 'de:08111:6100:0:4'  # Ankunft/Rtg Eiernest (Stuttgart Stuttgart)
	MARIENHOSPITAL = 'de:08111:6101'
	MARIENHOSPITAL_1 = 'de:08111:6101:1:3'  # Rtg Dornhaldenstr. (Stuttgart Stuttgart)
	MARIENHOSPITAL_2 = 'de:08111:6101:1:4'  # Rtg Marienplatz (Stuttgart Stuttgart)
	SALZWIESENSTRASSE = 'de:08111:6105'
	SALZWIESENSTRASSE_1 = 'de:08111:6105:0:1'  # Rtg Kirchtalstr. (Stuttgart Stuttgart)
	SALZWIESENSTRASSE_2 = 'de:08111:6105:0:2'  # Rtg Wimpfener Str. (Stuttgart Stuttgart)
	SALZWIESENSTRASSE_3 = 'de:08111:6105:0:3'  # Linie 508 Rtg Korntaler Str. (Stuttgart Stuttgart)
	SALZWIESENSTRASSE_4 = 'de:08111:6105:0:4'  # Linie 508 Rtg Zahn-Nopper-Str. (Stuttgart Stuttgart)
	HOHENSTEINSTRASSE_1 = 'de:08111:6109:1:1'  # Rtg Friedrichswahl (Stuttgart Stuttgart)
	HOHENSTEINSTRASSE_2 = 'de:08111:6109:2:2'  # Rtg Rathaus/Kelterplatz (Stuttgart Stuttgart)
	HOHENSTEINSTRASSE_3 = 'de:08111:6109:3:4'  # Linie 52 Rtg Frankenstr. (Stuttgart Stuttgart)
	HOHENSTEINSTRASSE_4 = 'de:08111:6109:4:3'  # Linie 52 Rtg Zuffenhausen Kelterplatz (Stuttgart Stuttgart)
	HOHENSTEINSTRASSE_5 = 'de:08117:9013:0:H1'  # Rtg John-F.-Kennedy-Str. (Göppingen Göppingen)
	HAUPTBF__ARNULF_KLETT_PLATZ = 'de:08111:6112'
	HAUPTBF__ARNULF_KLETT_PLATZ_1 = 'de:08111:6112:1:4'  # Rtg Schlossplatz (Stuttgart Stuttgart)
	HAUPTBF__ARNULF_KLETT_PLATZ_2 = 'de:08111:6112:2:2'  # Rtg Staatsgalerie (Stuttgart Stuttgart)
	HAUPTBF__ARNULF_KLETT_PLATZ_3 = 'de:08111:6112:2:3'  # Rtg Stadtbibliothek (Stuttgart Stuttgart)
	HAUPTBF__ARNULF_KLETT_PLATZ_4 = 'de:08111:6112:4:1'  # Rtg Börsenplatz (Stuttgart Stuttgart)
	HAUPTBF__ARNULF_KLETT_PLATZ_5 = 'de:08111:6112:5:4'  # Linien 40/42 Rtg Staatsgalerie (Stuttgart Stuttgart)
	HAUPTBF__ARNULF_KLETT_PLATZ_6 = 'de:08111:6112:5:6'  # Linie 44 Rtg Staatsgalerie (Stuttgart Stuttgart)
	HAUPTBF__ARNULF_KLETT_PLATZ_7 = 'de:08111:6112:5:7'  # SEV S-Bahn Rtg Rotebühlplatz/Bad Cannst. (Stuttgart Stuttgart)
	HAUPTBF__ARNULF_KLETT_PLATZ_8 = 'de:08111:6112:6:3'  # Linien 40/42 Rtg Katarinenhospital (Stuttgart Stuttgart)
	HAUPTBF__ARNULF_KLETT_PLATZ_9 = 'de:08111:6112:6:5'  # Linie 44 Rtg Stadtbibliothek (Stuttgart Stuttgart)
	PRAGSATTEL = 'de:08111:6113'
	PRAGSATTEL_1 = 'de:08111:6113:1:1'  # Rtg Löwentorbrücke/Löwentor (Stuttgart Stuttgart)
	PRAGSATTEL_2 = 'de:08111:6113:2:2'  # Rtg Maybachstr./Sieglestr. (Stuttgart Stuttgart)
	PRAGSATTEL_3 = 'de:08111:6113:5:4'  # Rtg A.-Luthuli-Pl./Höhenfreibad (Stuttgart Stuttgart)
	LÖWENTORBRÜCKE = 'de:08111:6114'
	LÖWENTORBRÜCKE_1 = 'de:08111:6114:1:1'  # Rtg Eckartshaldenweg (Stuttgart Stuttgart)
	LÖWENTORBRÜCKE_2 = 'de:08111:6114:1:2'  # Rtg Pragsattel (Stuttgart Stuttgart)
	LÖWENTORBRÜCKE_3 = 'de:08111:6114:2:5'  # NBus auswärts (Stuttgart Stuttgart)
	LÖWENTORBRÜCKE_4 = 'de:08111:6114:2:6'  # NBus einwärts (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN = 'de:08111:6115'
	HAUPTBAHNHOF__OBEN_1 = 'de:08111:6115:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_2 = 'de:08111:6115:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_3 = 'de:08111:6115:2:3'  # Gleis 3 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_4 = 'de:08111:6115:2:4'  # Gleis 4 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_5 = 'de:08111:6115:3:5'  # Gleis 5 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_6 = 'de:08111:6115:3:6'  # Gleis 6 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_7 = 'de:08111:6115:4:7'  # Gleis 7 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_8 = 'de:08111:6115:4:8'  # Gleis 8 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_9 = 'de:08111:6115:5:10'  # Gleis 10 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_10 = 'de:08111:6115:5:9'  # Gleis 9 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_11 = 'de:08111:6115:6:11'  # Gleis 11 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_12 = 'de:08111:6115:6:12'  # Gleis 12 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_13 = 'de:08111:6115:7:13'  # Gleis 13 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_14 = 'de:08111:6115:7:14'  # Gleis 14 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_15 = 'de:08111:6115:8:15'  # Gleis 15 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__OBEN_16 = 'de:08111:6115:8:16'  # Gleis 16 (Stuttgart Stuttgart)
	STADTBIBLIOTHEK_1 = 'de:08111:6116:1:1'  # Rtg Hauptbahnhof (Stuttgart Stuttgart)
	STADTBIBLIOTHEK_2 = 'de:08111:6116:1:2'  # Rtg Pragfriedhof/Eckartshaldenweg (Stuttgart Stuttgart)
	STADTBIBLIOTHEK_3 = 'de:08111:6116:3:3'  # Linie 44 Rtg Hauptbahnhof (Stuttgart Stuttgart)
	STADTBIBLIOTHEK_4 = 'de:08111:6116:3:4'  # Linie 44 Rtg Postdörfle (Stuttgart Stuttgart)
	STADTBIBLIOTHEK_5 = 'de:08117:7637:0:H1'  # Rtg Büchenbronner Str. (Ebersbach an der Fils Ebersbach (F))
	STADTBIBLIOTHEK_6 = 'de:08117:7637:0:R1'  # Rtg Hardtschule (Ebersbach an der Fils Ebersbach (F))
	HAUPTBAHNHOF__TIEF = 'de:08111:6118'
	HAUPTBAHNHOF__TIEF_1 = 'de:08111:6118:1:101'  # Gleis 101 (Stuttgart Stuttgart)
	HAUPTBAHNHOF__TIEF_2 = 'de:08111:6118:1:102'  # Gleis 102 (Stuttgart Stuttgart)
	OLGAECK = 'de:08111:6119'
	OLGAECK_1 = 'de:08111:6119:1:1'  # Rtg Dobelstr./Eugensplatz (Stuttgart Stuttgart)
	OLGAECK_2 = 'de:08111:6119:1:2'  # Rtg Charlottenplatz (Stuttgart Stuttgart)
	OLGAECK_3 = 'de:08111:6119:3:5'  # Linie 42 Rtg Eugensplatz (Stuttgart Stuttgart)
	OLGAECK_4 = 'de:08111:6119:3:6'  # Linie 42 Rtg Charlottenplatz (Stuttgart Stuttgart)
	OLGAECK_5 = 'de:08111:6119:4:3'  # Linie 43 Rtg Wilhelm-/Olgastr. (Stuttgart Stuttgart)
	OLGAECK_6 = 'de:08111:6119:4:4'  # Linie 43 Rtg Dorotheenstr. (Stuttgart Stuttgart)
	EUGENSPLATZ = 'de:08111:6121'
	EUGENSPLATZ_1 = 'de:08111:6121:1:1'  # Rtg Heidehofstr. (Stuttgart Stuttgart)
	EUGENSPLATZ_2 = 'de:08111:6121:1:2'  # Rtg Olgaeck (Stuttgart Stuttgart)
	EUGENSPLATZ_3 = 'de:08111:6121:2:3'  # Linie 42 Rtg Heidehofstr. (Stuttgart Stuttgart)
	EUGENSPLATZ_4 = 'de:08111:6121:2:4'  # Linie 42 Rtg Olgaeck (Stuttgart Stuttgart)
	HEIDEHOFSTRASSE = 'de:08111:6122'
	HEIDEHOFSTRASSE_1 = 'de:08111:6122:1:1'  # Rtg Stafflenbergstr. (Stuttgart Stuttgart)
	HEIDEHOFSTRASSE_2 = 'de:08111:6122:2:2'  # Rtg Olgaeck (Stuttgart Stuttgart)
	HEIDEHOFSTRASSE_3 = 'de:08111:6122:3:3'  # Linie 42 Rtg Straußstaffel (Stuttgart Stuttgart)
	HEIDEHOFSTRASSE_4 = 'de:08111:6122:3:4'  # Linie 42 Rtg Eugensplatz (Stuttgart Stuttgart)
	HEIDEHOFSTRASSE_5 = 'de:08111:6122:3:5'  # NBus N7 Rtg Stafflenb (Stuttgart Stuttgart)
	STELLE = 'de:08111:6127'
	STELLE_1 = 'de:08111:6127:1:1'  # Rtg Ruhbank (Stuttgart Stuttgart)
	STELLE_2 = 'de:08111:6127:1:2'  # Rtg Geroksruhe (Stuttgart Stuttgart)
	STELLE_3 = 'de:08111:6127:2:3'  # Rtg Filderblickweg/Geroksruhe (Stuttgart Stuttgart)
	RUHBANK__FERNSEHTURM = 'de:08111:6128'
	RUHBANK__FERNSEHTURM_1 = 'de:08111:6128:1:7'  # U15 Rtg Silberwald (Stuttgart Stuttgart)
	RUHBANK__FERNSEHTURM_2 = 'de:08111:6128:1:8'  # Rtg Stelle (Stuttgart Stuttgart)
	RUHBANK__FERNSEHTURM_3 = 'de:08111:6128:2:1'  # U7/U8 Rtg Silberwald (Stuttgart Stuttgart)
	RUHBANK__FERNSEHTURM_4 = 'de:08111:6128:2:2'  # Rtg Waldau (Stuttgart Stuttgart)
	RUHBANK__FERNSEHTURM_5 = 'de:08111:6128:3:3'  # Linie 70 Rtg Kolpingsiedlung (Stuttgart Stuttgart)
	RUHBANK__FERNSEHTURM_6 = 'de:08111:6128:3:4'  # Linie 70 Rtg Königsträßle (Stuttgart Stuttgart)
	RUHBANK__FERNSEHTURM_7 = 'de:08111:6128:3:6'  # Ankunft und Abfahrt T70 (Stuttgart Stuttgart)
	RUHBANK__FERNSEHTURM_8 = 'de:08111:6128:4:5'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	SCHEMPPSTRASSE = 'de:08111:6130'
	SCHEMPPSTRASSE_1 = 'de:08111:6130:1:1'  # Rtg Heumaden Bockelstr. (Stuttgart Stuttgart)
	SCHEMPPSTRASSE_2 = 'de:08111:6130:1:2'  # Rtg Sillenbuch (Stuttgart Stuttgart)
	SCHEMPPSTRASSE_3 = 'de:08111:6130:2:4'  # L.65 Rtg Spaich.Str./L.66 Rtg Kirchh.Str (Stuttgart Stuttgart)
	SCHEMPPSTRASSE_4 = 'de:08111:6130:3:3'  # Rtg Kemnater Str. (Stuttgart Stuttgart)
	HEUMADEN_BOCKELSTRASSE = 'de:08111:6133'
	HEUMADEN_BOCKELSTRASSE_1 = 'de:08111:6133:1:1'  # Rtg Heumaden (Stuttgart Stuttgart)
	HEUMADEN_BOCKELSTRASSE_2 = 'de:08111:6133:1:2'  # Rtg Schemppstr. (Stuttgart Stuttgart)
	HEUMADEN_BOCKELSTRASSE_3 = 'de:08111:6133:2:4'  # Rtg Isegrimweg (Stuttgart Stuttgart)
	HEUMADEN_BOCKELSTRASSE_4 = 'de:08111:6133:3:3'  # Rtg Schemppstr./Kemnater Str. (Stuttgart Stuttgart)
	SILLENBUCH = 'de:08111:6134'
	SILLENBUCH_1 = 'de:08111:6134:1:1'  # Rtg Schemppstr. (Stuttgart Stuttgart)
	SILLENBUCH_2 = 'de:08111:6134:1:2'  # Rtg Silberwald (Stuttgart Stuttgart)
	SILLENBUCH_3 = 'de:08111:6134:1:5'  # NBus (Stuttgart Stuttgart)
	SILLENBUCH_4 = 'de:08111:6134:2:3'  # Linie 66 Rtg Kirchheimer Str. (Stuttgart Stuttgart)
	GESCHWISTER_SCHOLL_GYMNASIUM = 'de:08111:6135:0:3'  # Rtg Kemnater Str. (Stuttgart Stuttgart)
	KEMNATER_STRASSE = 'de:08111:6136'
	KEMNATER_STRASSE_1 = 'de:08111:6136:1:4'  # Rtg Schemppstr. (Stuttgart Stuttgart)
	KEMNATER_STRASSE_2 = 'de:08111:6136:2:3'  # Rtg Geschw.-Scholl-Gymn. (Stuttgart Stuttgart)
	KEMNATER_STRASSE_3 = 'de:08111:6136:3:6'  # Rtg Heumaden Bockelstr. (Stuttgart Stuttgart)
	BERNSTEINSTRASSE = 'de:08111:6137'
	BERNSTEINSTRASSE_1 = 'de:08111:6137:0:3'  # Rtg ??? (Stuttgart Stuttgart)
	BERNSTEINSTRASSE_2 = 'de:08111:6137:0:4'  # RTg ??? (Stuttgart Stuttgart)
	ROSSBACHSTRASSE_1 = 'de:08111:6147:0:3'  # Rtg Flachter Str. (Stuttgart Stuttgart)
	ROSSBACHSTRASSE_2 = 'de:08111:6147:0:4'  # Rtg Löwenmarkt (Stuttgart Stuttgart)
	ROSSBACHSTRASSE_3 = 'de:08117:1033:0:H1'  # Rtg Eislingen (Göppingen Göppingen)
	ROSSBACHSTRASSE_4 = 'de:08117:1033:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	WEILIMDORF_LÖWEN_MARKT = 'de:08111:6149'
	WEILIMDORF_LÖWEN_MARKT_1 = 'de:08111:6149:1:1'  # Rtg Landauer Str. (Stuttgart Stuttgart)
	WEILIMDORF_LÖWEN_MARKT_2 = 'de:08111:6149:2:2'  # Rtg Rastatter Str. (Stuttgart Stuttgart)
	WEILIMDORF_LÖWEN_MARKT_3 = 'de:08111:6149:3:3'  # L. 90 Rtg Roßbachstr. (Stuttgart Stuttgart)
	WEILIMDORF_LÖWEN_MARKT_4 = 'de:08111:6149:4:4'  # L. 90 Rtg Fehrbelliner Str. (Stuttgart Stuttgart)
	WEILIMDORF_LÖWEN_MARKT_5 = 'de:08111:6149:5:6'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	HOHEWARTSTRASSE = 'de:08111:6155'
	HOHEWARTSTRASSE_1 = 'de:08111:6155:0:3'  # Rtg Wildensteinstr. (Stuttgart Stuttgart)
	HOHEWARTSTRASSE_2 = 'de:08111:6155:0:4'  # Rtg Föhrich (Stuttgart Stuttgart)
	FEUERBACH = 'de:08111:6157'
	FEUERBACH_1 = 'de:08111:6157:1:1'  # Gleis 1 (Stuttgart Stuttgart)
	FEUERBACH_2 = 'de:08111:6157:1:3'  # Gleis 3 (Stuttgart Stuttgart)
	FEUERBACH_3 = 'de:08111:6157:2:2'  # Gleis 2 (Stuttgart Stuttgart)
	FEUERBACH_4 = 'de:08111:6157:3:1'  # Rtg Maybachstr. (Stuttgart Stuttgart)
	FEUERBACH_5 = 'de:08111:6157:4:2'  # Rtg Wilhelm-Geiger-Pl. (Stuttgart Stuttgart)
	FEUERBACH_6 = 'de:08111:6157:5:3'  # Rtg Kerschensteinerschule/ Feuerb. Bosch (Stuttgart Stuttgart)
	FEUERBACH_7 = 'de:08111:6157:5:4'  # Bus Ankunft (Stuttgart Stuttgart)
	FEUERBACH_8 = 'de:08111:6157:5:5'  # NBus auswärts (Stuttgart Stuttgart)
	FEUERBACH_9 = 'de:08111:6157:6:4'  # Gleis 4 (Stuttgart Stuttgart)
	BOPSER = 'de:08111:6160'
	BOPSER_1 = 'de:08111:6160:1:1'  # Rtg Weinsteige/Waldau (Stuttgart Stuttgart)
	BOPSER_2 = 'de:08111:6160:1:2'  # Rtg Dobelstr. (Stuttgart Stuttgart)
	BOPSER_3 = 'de:08111:6160:2:3'  # N8 (Stuttgart Stuttgart)
	BOPSER_4 = 'de:08111:6160:2:4'  # N8, N9, N10 (Stuttgart Stuttgart)
	DEGERLOCH_ZOB = 'de:08111:6163'
	DEGERLOCH_ZOB_1 = 'de:08111:6163:1:11'  # Bstg 1 (Stuttgart Stuttgart)
	DEGERLOCH_ZOB_2 = 'de:08111:6163:1:12'  # Bstg 2 (Stuttgart Stuttgart)
	DEGERLOCH_ZOB_3 = 'de:08111:6163:1:13'  # Bstg 3 (Stuttgart Stuttgart)
	DEGERLOCH_ZOB_4 = 'de:08111:6163:1:14'  # Bstg 4 (Stuttgart Stuttgart)
	DEGERLOCH_ZOB_5 = 'de:08111:6163:1:15'  # Bstg 5 (Stuttgart Stuttgart)
	LAUSTRASSE = 'de:08111:6164:0:3'  # NBus (Stuttgart Stuttgart)
	DEGERLOCH = 'de:08111:6165'
	DEGERLOCH_1 = 'de:08111:6165:1:1'  # Rtg Albstraße (Stuttgart Stuttgart)
	DEGERLOCH_2 = 'de:08111:6165:1:2'  # Rtg Weinsteige (Stuttgart Stuttgart)
	DEGERLOCH_3 = 'de:08111:6165:1:7'  # Zahnradbahn Endhst. (Stuttgart Stuttgart)
	DEGERLOCH_4 = 'de:08111:6165:2:3'  # Linien 78 / 809 Abfahrt (Stuttgart Stuttgart)
	DEGERLOCH_5 = 'de:08111:6165:3:4'  # Linientaxi 10 / N8 (Stuttgart Stuttgart)
	PEREGRINASTRASSE = 'de:08111:6166'
	PEREGRINASTRASSE_1 = 'de:08111:6166:1:1'  # Rtg Sonnenberg (Stuttgart Stuttgart)
	PEREGRINASTRASSE_2 = 'de:08111:6166:1:2'  # Rtg Albstraße (Stuttgart Stuttgart)
	RIEDSEE = 'de:08111:6168'
	RIEDSEE_1 = 'de:08111:6168:1:1'  # Rtg Möhringen Bf (Stuttgart Stuttgart)
	RIEDSEE_2 = 'de:08111:6168:1:2'  # Rtg Sonnenberg (Stuttgart Stuttgart)
	RIEDSEE_3 = 'de:08111:6168:3:3'  # NBus (Stuttgart Stuttgart)
	MÖHRINGEN_BF = 'de:08111:6169'
	MÖHRINGEN_BF_1 = 'de:08111:6169:1:1'  # Rtg Vaihinger Str. (Stuttgart Möhringen)
	MÖHRINGEN_BF_2 = 'de:08111:6169:2:2'  # Rtg Riedsee (Stuttgart Möhringen)
	MÖHRINGEN_BF_3 = 'de:08111:6169:2:3'  # U3 Rtg Vaihinger Str. (Stuttgart Möhringen)
	MÖHRINGEN_BF_4 = 'de:08111:6169:3:4'  # U3 Rtg Sigmaringer Str. (Stuttgart Möhringen)
	MÖHRINGEN_BF_5 = 'de:08111:6169:4:6'  # Baustellenhaltestelle 72+N9 (Stuttgart Möhringen)
	MÖHRINGEN_BF_6 = 'de:08111:6169:5:8'  # Ers.Hst. i.d. Leinenweberstr Rtg.Rathaus (Stuttgart Möhringen)
	MÖHRINGEN_FREIBAD = 'de:08111:6171'
	MÖHRINGEN_FREIBAD_1 = 'de:08111:6171:0:1'  # Rtg Leinfelden Frank/Fasanenhof (Stuttgart Stuttgart)
	MÖHRINGEN_FREIBAD_2 = 'de:08111:6171:0:2'  # Rtg Rohrer Weg (Stuttgart Stuttgart)
	DORNRÖSCHENWEG = 'de:08111:6172'
	DORNRÖSCHENWEG_1 = 'de:08111:6172:0:3'  # Rtg Lohäckerstr. (Stuttgart Stuttgart)
	DORNRÖSCHENWEG_2 = 'de:08111:6172:0:4'  # Rtg Jelinstr. (Stuttgart Stuttgart)
	LOHÄCKERSTRASSE = 'de:08111:6173'
	LOHÄCKERSTRASSE_1 = 'de:08111:6173:0:3'  # Rtg Kurt-Schumacher-Str. (Stuttgart Stuttgart)
	LOHÄCKERSTRASSE_2 = 'de:08111:6173:0:4'  # Rtg Dornröschenweg (Stuttgart Stuttgart)
	RÜBEZAHLWEG = 'de:08111:6174:0:3'  # Rtg Jelinstr. (Stuttgart Stuttgart)
	STUTTGART_RÜBEZAHLWEG = 'de:08111:6174:0:3'  # Rtg Jelinstr. (Stuttgart Stuttgart)
	KERSCHENSTEINERSCHULE = 'de:08111:6179'
	KERSCHENSTEINERSCHULE_1 = 'de:08111:6179:0:3'  # Rtg Wilhelm-Geiger-Pl. (Stuttgart Stuttgart)
	KERSCHENSTEINERSCHULE_2 = 'de:08111:6179:0:4'  # Rtg Feuerbach Bahnhof (Stuttgart Stuttgart)
	WILHELM_GEIGER_PLATZ = 'de:08111:6180'
	WILHELM_GEIGER_PLATZ_1 = 'de:08111:6180:1:1'  # Rtg Feuerbach Bf (Stuttgart Stuttgart)
	WILHELM_GEIGER_PLATZ_2 = 'de:08111:6180:1:2'  # Rtg Föhrich (Stuttgart Stuttgart)
	WILHELM_GEIGER_PLATZ_3 = 'de:08111:6180:3:3'  # Rtg Föhrich (Stuttgart Stuttgart)
	WILHELM_GEIGER_PLATZ_4 = 'de:08111:6180:3:4'  # Rtg Kerschensteinerschule (Stuttgart Stuttgart)
	DEGERLOCH_EPPLESTRASSE = 'de:08111:6184:1:4'  # Rtg Degerloch ZOB/Zahnradbahnhof (Stuttgart Stuttgart)
	BUCHRAIN_FRIEDHOF = 'de:08111:6200'
	BUCHRAIN_FRIEDHOF_1 = 'de:08111:6200:0:3'  # Rtg Gründgensstr. (Stuttgart Stuttgart)
	BUCHRAIN_FRIEDHOF_2 = 'de:08111:6200:0:4'  # Rtg Kurmärker Kaserne (Stuttgart Stuttgart)
	KREHLSTRASSE = 'de:08111:6201'
	KREHLSTRASSE_1 = 'de:08111:6201:0:3'  # Rtg Schwabengalerie (Stuttgart Stuttgart)
	KREHLSTRASSE_2 = 'de:08111:6201:0:4'  # Rtg Heerstr./Tigerstr. (Stuttgart Stuttgart)
	KURMÄRKER_KASERNE = 'de:08111:6202'
	KURMÄRKER_KASERNE_1 = 'de:08111:6202:0:3'  # Rtg Buchrain Friedhof (Stuttgart Stuttgart)
	KURMÄRKER_KASERNE_2 = 'de:08111:6202:0:4'  # Rtg Tigerstr. (Stuttgart Stuttgart)
	GRÜNDGENSSTRASSE = 'de:08111:6203'
	GRÜNDGENSSTRASSE_1 = 'de:08111:6203:0:3'  # Rtg Sindelfingen Waldheim (Stuttgart Stuttgart)
	GRÜNDGENSSTRASSE_2 = 'de:08111:6203:0:4'  # Rtg Buchrain Friedhof (Stuttgart Stuttgart)
	SCHWAB__BEBELSTRASSE = 'de:08111:6206'
	SCHWAB__BEBELSTRASSE_1 = 'de:08111:6206:1:2'  # Rtg Schloss-/Johannesstr. (Stuttgart Stuttgart)
	SCHWAB__BEBELSTRASSE_2 = 'de:08111:6206:2:1'  # Rtg Arndt-/Spittastr. (Stuttgart Stuttgart)
	SCHWAB__BEBELSTRASSE_3 = 'de:08111:6206:3:3'  # Rtg Bismarckplatz (Stuttgart Stuttgart)
	SCHWAB__BEBELSTRASSE_4 = 'de:08111:6206:4:4'  # Rtg Rosenbergplatz (Stuttgart Stuttgart)
	SCHWAB__BEBELSTRASSE_5 = 'de:08111:6206:4:5'  # NBus N2 Rtg Vogelsang (Stuttgart Stuttgart)
	LINDPAINTNERSTRASSE = 'de:08111:6213'
	LINDPAINTNERSTRASSE_1 = 'de:08111:6213:1:1'  # Rtg Beethovenstr. (Stuttgart Stuttgart)
	LINDPAINTNERSTRASSE_2 = 'de:08111:6213:1:2'  # Rtg Herderplatz (Stuttgart Stuttgart)
	LINDPAINTNERSTRASSE_3 = 'de:08111:6213:2:3'  # Rtg Auberlenweg (Stuttgart Stuttgart)
	LINDPAINTNERSTRASSE_4 = 'de:08111:6213:3:4'  # Rtg Beethovenstr. (Stuttgart Stuttgart)
	BOTNANGER_SATTEL = 'de:08111:6214'
	BOTNANGER_SATTEL_1 = 'de:08111:6214:0:3'  # Rtg Wielandstr. (Stuttgart Stuttgart)
	BOTNANGER_SATTEL_2 = 'de:08111:6214:0:4'  # Rtg Nikolauspflege (Stuttgart Stuttgart)
	SCHLOSS__JOHANNESSTRASSE = 'de:08111:6219'
	SCHLOSS__JOHANNESSTRASSE_1 = 'de:08111:6219:1:2'  # Rtg Berliner Platz (Stuttgart Stuttgart)
	SCHLOSS__JOHANNESSTRASSE_2 = 'de:08111:6219:2:1'  # Rtg Schwab-/Bebelstr. (Stuttgart Stuttgart)
	SCHLOSS__JOHANNESSTRASSE_3 = 'de:08111:6219:3:4'  # Rtg Rosenberg-/Johannesstr. (Stuttgart Stuttgart)
	SCHLOSS__JOHANNESSTRASSE_4 = 'de:08111:6219:4:3'  # Rtg Feuersee (Stuttgart Stuttgart)
	SCHLOSS__JOHANNESSTRASSE_5 = 'de:08111:6219:5:5'  # NBus Rtg auswärts (Stuttgart Stuttgart)
	SCHLACHTHOF = 'de:08111:6223'
	SCHLACHTHOF_1 = 'de:08111:6223:1:1'  # Rtg Raitelsberg (Stuttgart Stuttgart)
	SCHLACHTHOF_2 = 'de:08111:6223:1:2'  # Rtg Wangener-/Landhausstr. (Stuttgart Stuttgart)
	SCHLACHTHOF_3 = 'de:08111:6223:1:5'  # N6 Rtg Wangener-/Landhausstr (Stuttgart Stuttgart)
	SCHLACHTHOF_4 = 'de:08111:6223:2:3'  # Rtg Tal-/Landhausstr. (Stuttgart Stuttgart)
	SCHLACHTHOF_5 = 'de:08111:6223:2:4'  # Rtg NeckarPark (Stadion) (Stuttgart Stuttgart)
	WESTBAHNHOF = 'de:08111:6249'
	WESTBAHNHOF_1 = 'de:08111:6249:2:4'  # Rtg Kleiststr./Herderplatz (Stuttgart Stuttgart)
	WESTBAHNHOF_2 = 'de:08111:6249:2:5'  # Rtg Birkenkopf (Stuttgart Stuttgart)
	WESTBAHNHOF__SCHLEIFE = 'de:08111:6250:1:5'  # Rtg Westbf/Kleiststr./Herderpl. (Stuttgart Stuttgart)
	ROSENSTEINBRÜCKE = 'de:08111:6253'
	ROSENSTEINBRÜCKE_1 = 'de:08111:6253:1:7'  # U13 Rtg Wilhelmsplatz (Stuttgart Stuttgart)
	ROSENSTEINBRÜCKE_2 = 'de:08111:6253:2:8'  # U13 Rtg Glockenstr. (Stuttgart Stuttgart)
	ROSENSTEINBRÜCKE_3 = 'de:08111:6253:3:1'  # U14 Rtg Wilhelma (Stuttgart Stuttgart)
	ROSENSTEINBRÜCKE_4 = 'de:08111:6253:3:5'  # Ersatzhaltestelle in der Wilhelmastr (Stuttgart Stuttgart)
	ROSENSTEINBRÜCKE_5 = 'de:08111:6253:4:2'  # U14 Rtg Mühlsteg (Stuttgart Stuttgart)
	ROSENSTEINBRÜCKE_6 = 'de:08111:6253:4:4'  # Rtg Martinskirche/Mühlsteg (Stuttgart Stuttgart)
	LÖWENTOR = 'de:08111:6261'
	LÖWENTOR_1 = 'de:08111:6261:1:1'  # Rtg Rosensteinpark (Stuttgart Stuttgart)
	LÖWENTOR_2 = 'de:08111:6261:1:6'  # Nbus Rtg Nordb (Stuttgart Stuttgart)
	LÖWENTOR_3 = 'de:08111:6261:2:2'  # Rtg Pragsattel (Stuttgart Stuttgart)
	LÖWENTOR_4 = 'de:08111:6261:5:3'  # Rtg Nordbf (Stuttgart Stuttgart)
	LÖWENTOR_5 = 'de:08111:6261:5:4'  # Rtg Züricher Str. (Stuttgart Stuttgart)
	ZAZENHAUSEN_STEIGLE = 'de:08111:6269'
	ZAZENHAUSEN_STEIGLE_1 = 'de:08111:6269:0:3'  # eRtg Zazenhausen Krone (Stuttgart Stuttgart)
	ZAZENHAUSEN_STEIGLE_2 = 'de:08111:6269:0:4'  # eRtg Bachhalde (Stuttgart Stuttgart)
	MÜHLHAUSEN = 'de:08111:6270'
	MÜHLHAUSEN_1 = 'de:08111:6270:1:1'  # Rtg Auwiesen (Stuttgart Stuttgart)
	MÜHLHAUSEN_2 = 'de:08111:6270:1:2'  # Rtg Hornbach (Stuttgart Stuttgart)
	MÜHLHAUSEN_3 = 'de:08111:6270:2:3'  # Linie 54 Rtg Auwiesen (Stuttgart Stuttgart)
	MÜHLHAUSEN_4 = 'de:08111:6270:3:6'  # Linie 401 Rtg Mühlhausen Schloss (Stuttgart Stuttgart)
	MÜHLHAUSEN_5 = 'de:08111:6270:4:4'  # Linie 54 Rtg Mühlhausen Schloss (Stuttgart Stuttgart)
	MÜHLHAUSEN_6 = 'de:08111:6270:5:3'  # Endhaltestelle U14 (Stuttgart Stuttgart)
	MÜHLHAUSEN_SCHLOSS = 'de:08111:6271'
	MÜHLHAUSEN_SCHLOSS_1 = 'de:08111:6271:0:3'  # Rtg Mühlhausen (Stuttgart Stuttgart)
	MÜHLHAUSEN_SCHLOSS_2 = 'de:08111:6271:0:4'  # Rtg Aalstr. (Stuttgart Stuttgart)
	BACHHALDE = 'de:08111:6272'
	BACHHALDE_1 = 'de:08111:6272:0:3'  # eRtg Zazenhausen Steigle (Stuttgart Stuttgart)
	BACHHALDE_2 = 'de:08111:6272:0:4'  # eRtg Mühlhausen Schloss (Stuttgart Stuttgart)
	MAX_EYTH_SEE = 'de:08111:6273'
	MAX_EYTH_SEE_1 = 'de:08111:6273:1:1'  # Rtg Wagrainäcker (Stuttgart Stuttgart)
	MAX_EYTH_SEE_2 = 'de:08111:6273:2:2'  # Rtg Hofen (Stuttgart Stuttgart)
	MAX_EYTH_SEE_3 = 'de:08111:6273:3:3'  # Linie 54 Rtg Nixenweg (Stuttgart Stuttgart)
	MAX_EYTH_SEE_4 = 'de:08111:6273:4:4'  # Linie 54 Rtg Hofen (Stuttgart Stuttgart)
	ALPSEEWEG = 'de:08111:6274'
	ALPSEEWEG_1 = 'de:08111:6274:0:3'  # Rtg Schneideräckerstr. (Stuttgart Stuttgart)
	ALPSEEWEG_2 = 'de:08111:6274:0:4'  # Rtg Nixenweg (Stuttgart Stuttgart)
	NIXENWEG = 'de:08111:6275'
	NIXENWEG_1 = 'de:08111:6275:0:3'  # Rtg Alpseeweg (Stuttgart Stuttgart)
	NIXENWEG_2 = 'de:08111:6275:0:4'  # Rtg Max-Eyth-See (Stuttgart Stuttgart)
	SCHNEIDERÄCKERSTRASSE = 'de:08111:6276'
	SCHNEIDERÄCKERSTRASSE_1 = 'de:08111:6276:0:3'  # Rtg Neugereut (Stuttgart Stuttgart)
	SCHNEIDERÄCKERSTRASSE_2 = 'de:08111:6276:0:4'  # Rtg Alpseeweg (Stuttgart Stuttgart)
	WILDGANSWEG = 'de:08111:6277'
	WILDGANSWEG_1 = 'de:08111:6277:0:3'  # Rtg Schneideräckerstr. (Stuttgart Stuttgart)
	WILDGANSWEG_2 = 'de:08111:6277:0:4'  # Rtg Alpseeweg (Stuttgart Stuttgart)
	FREIBERG_SCHULZENTRUM = 'de:08111:6286:0:3'  # Endhaltestelle (Stuttgart Stuttgart)
	FREIBERG = 'de:08111:6287'
	FREIBERG_1 = 'de:08111:6287:1:1'  # Rtg Suttnerstr. (Stuttgart Stuttgart)
	FREIBERG_2 = 'de:08111:6287:1:2'  # Rtg Mönchfeld (Stuttgart Stuttgart)
	FREIBERG_3 = 'de:08111:6287:2:3'  # Rtg Forellenweg (Stuttgart Stuttgart)
	FREIBERG_4 = 'de:08111:6287:2:4'  # Linie 54 Ankunft (Stuttgart Stuttgart)
	MÖNCHFELD = 'de:08111:6288:0:1'  # Stadtbahn Endhaltestelle (Stuttgart Stuttgart)
	TAPACHSTRASSE = 'de:08111:6291'
	TAPACHSTRASSE_1 = 'de:08111:6291:1:1'  # Rtg Fürfelder Str. (Stuttgart Stuttgart)
	TAPACHSTRASSE_2 = 'de:08111:6291:2:2'  # Rtg Himmelsleiter (Stuttgart Stuttgart)
	TAPACHSTRASSE_3 = 'de:08111:6291:3:4'  # Linie 52 Rtg Fürfelder Str. (Stuttgart Stuttgart)
	TAPACHSTRASSE_4 = 'de:08111:6291:4:3'  # Linie 52 Rtg Rilke-Realschule (Stuttgart Stuttgart)
	FÜRFELDER_STRASSE = 'de:08111:6292'
	FÜRFELDER_STRASSE_1 = 'de:08111:6292:0:1'  # Rtg Schozacher Str. (Stuttgart Stuttgart)
	FÜRFELDER_STRASSE_2 = 'de:08111:6292:0:2'  # Rtg Tapachstr. (Stuttgart Stuttgart)
	FÜRFELDER_STRASSE_3 = 'de:08111:6292:0:3'  # Linie 52 Rtg Tapachstr. (Stuttgart Stuttgart)
	FÜRFELDER_STRASSE_4 = 'de:08111:6292:0:4'  # Linie 52 Rtg Schozacher Str. (Stuttgart Stuttgart)
	SCHOZACHER_STRASSE = 'de:08111:6293'
	SCHOZACHER_STRASSE_1 = 'de:08111:6293:0:1'  # Rtg Zuffenhausen Kelterplatz (Stuttgart Stuttgart)
	SCHOZACHER_STRASSE_2 = 'de:08111:6293:0:2'  # Rtg Fürfelder Str. (Stuttgart Stuttgart)
	SCHOZACHER_STRASSE_3 = 'de:08111:6293:0:3'  # Linie 52 Rtg Fürfelder Str. (Stuttgart Stuttgart)
	SCHOZACHER_STRASSE_4 = 'de:08111:6293:0:4'  # Linie 52 Rtg Zuffenhausen Kelterplatz (Stuttgart Stuttgart)
	ZUFFENHAUSEN_KELTERPLATZ = 'de:08111:6294'
	ZUFFENHAUSEN_KELTERPLATZ_1 = 'de:08111:6294:1:1'  # Rtg Hohensteinstr. (Stuttgart Stuttgart)
	ZUFFENHAUSEN_KELTERPLATZ_2 = 'de:08111:6294:1:2'  # Rtg Schozacher Str. (Stuttgart Stuttgart)
	ZUFFENHAUSEN_KELTERPLATZ_3 = 'de:08111:6294:2:3'  # Linie 52 Rtg Schozacher Str. (Stuttgart Stuttgart)
	ZUFFENHAUSEN_KELTERPLATZ_4 = 'de:08111:6294:2:4'  # Linie 52 Rtg Hohensteinstr. (Stuttgart Stuttgart)
	ROSENSTEINSTRASSE = 'de:08111:6296:0:3'  # NBus (Stuttgart Stuttgart)
	OBERE_ZIEGELEI = 'de:08111:6320'
	OBERE_ZIEGELEI_1 = 'de:08111:6320:1:1'  # Rtg Gnesener Str. (Stuttgart Stuttgart)
	OBERE_ZIEGELEI_2 = 'de:08111:6320:1:2'  # Rtg Hauptfriedhof (Stuttgart Stuttgart)
	OBERE_ZIEGELEI_3 = 'de:08111:6320:1:3'  # Linien 54/58 Rtg Edelweißweg (Stuttgart Stuttgart)
	OBERE_ZIEGELEI_4 = 'de:08111:6320:2:4'  # Linie 54 Rtg Muckensturm (Stuttgart Stuttgart)
	OBERE_ZIEGELEI_5 = 'de:08111:6320:3:6'  # Linie 58 Rtg Auf der Gans (Stuttgart Stuttgart)
	FRITZ_WALTER_WEG = 'de:08111:6328'
	FRITZ_WALTER_WEG_1 = 'de:08111:6328:0:3'  # Rtg Mercedes-Benz Welt (Stuttgart Stuttgart)
	FRITZ_WALTER_WEG_2 = 'de:08111:6328:0:4'  # Rtg NeckarPark (Stadion) (Stuttgart Stuttgart)
	NECKARPARK__STADION = 'de:08111:6330'
	NECKARPARK__STADION_1 = 'de:08111:6330:1:1'  # Stadtbahn Endhst. (Stuttgart Stuttgart)
	NECKARPARK__STADION_2 = 'de:08111:6330:2:3'  # Rtg Schlachthof (Stuttgart Stuttgart)
	NECKARPARK__STADION_3 = 'de:08111:6330:2:4'  # Rtg Veielbrunnenweg/Elwertstr. (Stuttgart Stuttgart)
	NECKARPARK__STADION_4 = 'de:08111:6330:2:5'  # Rtg Fritz-Walter-Weg (Stuttgart Stuttgart)
	BAD_CANNSTATT_WILHELMSPLATZ = 'de:08111:6332'
	BAD_CANNSTATT_WILHELMSPLATZ_1 = 'de:08111:6332:1:1'  # Rtg Mercedesstr. (Stuttgart Stuttgart)
	BAD_CANNSTATT_WILHELMSPLATZ_2 = 'de:08111:6332:1:6'  # Bus X1 2. Halt+N6 (Stuttgart Stuttgart)
	BAD_CANNSTATT_WILHELMSPLATZ_3 = 'de:08111:6332:2:3'  # Bus 52 Rtg Badstr. (Stuttgart Stuttgart)
	BAD_CANNSTATT_WILHELMSPLATZ_4 = 'de:08111:6332:2:4'  # Bus 56 Rtg Bahnhof/Badstr. (Stuttgart Stuttgart)
	BAD_CANNSTATT_WILHELMSPLATZ_5 = 'de:08111:6332:5:2'  # Rtg Uff-Kirchhof/Daimlerplatz (Stuttgart Stuttgart)
	BAD_CANNSTATT = 'de:08111:6333'
	BAD_CANNSTATT_1 = 'de:08111:6333:1:2'  # Gleis 2 (Stuttgart Stuttgart)
	BAD_CANNSTATT_2 = 'de:08111:6333:1:3'  # Gleis 3 (Stuttgart Stuttgart)
	BAD_CANNSTATT_3 = 'de:08111:6333:2:1'  # Gleis 1 (Stuttgart Stuttgart)
	BAD_CANNSTATT_4 = 'de:08111:6333:5:4'  # Gleis 4 (Stuttgart Stuttgart)
	BAD_CANNSTATT_5 = 'de:08111:6333:5:5'  # Gleis 5 (Stuttgart Stuttgart)
	BAD_CANNSTATT_6 = 'de:08111:6333:6:6'  # Gleis 6 (Stuttgart Stuttgart)
	BAD_CANNSTATT_7 = 'de:08111:6333:6:7'  # Gleis 7 (Stuttgart Stuttgart)
	BAD_CANNSTATT_8 = 'de:08111:6333:7:8'  # Gleis 8 (Stuttgart Stuttgart)
	BAD_CANNSTATT_9 = 'de:08111:6333:8:6'  # Südseite Bahnhof Rtg. NeckarPark (Stuttgart Stuttgart)
	ZAHNRADBAHNHOF = 'de:08111:6340'
	ZAHNRADBAHNHOF_1 = 'de:08111:6340:0:1'  # Rtg Degerloch (Stuttgart Stuttgart)
	ZAHNRADBAHNHOF_2 = 'de:08111:6340:0:2'  # Rtg Nägelestr. (Stuttgart Stuttgart)
	ZAHNRADBAHNHOF_3 = 'de:08111:6340:0:3'  # Linien 70/71 Rtg Königsträßle (Stuttgart Stuttgart)
	ZAHNRADBAHNHOF_4 = 'de:08111:6340:0:4'  # Linie 70 Rtg Degerloch ZOB (Stuttgart Stuttgart)
	STERNHÄULE = 'de:08111:6351'
	STERNHÄULE_1 = 'de:08111:6351:1:2'  # X7 Rtg Landhaus (Stuttgart Stuttgart)
	STERNHÄULE_2 = 'de:08111:6351:1:3'  # Rtg Landhaus/Landhauskrzg. (Stuttgart Stuttgart)
	STERNHÄULE_3 = 'de:08111:6351:2:1'  # X7 Rtg Degerloch (Stuttgart Stuttgart)
	STERNHÄULE_4 = 'de:08111:6351:2:4'  # Rtg Weidachtal (Stuttgart Stuttgart)
	LANDHAUS = 'de:08111:6352'
	LANDHAUS_1 = 'de:08111:6352:1:2'  # Rtg Plieningen (Stuttgart Stuttgart)
	LANDHAUS_2 = 'de:08111:6352:2:1'  # Rtg Salzäcker (Stuttgart Stuttgart)
	LANDHAUS_3 = 'de:08111:6352:3:3'  # Rtg EnBW City (Stuttgart Stuttgart)
	LANDHAUS_4 = 'de:08111:6352:3:4'  # Rtg Sternhäule (Stuttgart Stuttgart)
	LANDHAUSKREUZUNG = 'de:08111:6353'
	LANDHAUSKREUZUNG_1 = 'de:08111:6353:1:3'  # Rtg Kaserne Möhringen (Stuttgart Stuttgart)
	LANDHAUSKREUZUNG_2 = 'de:08111:6353:2:4'  # Rtg Sternhäule (Stuttgart Stuttgart)
	FEUERBACH_BOSCH = 'de:08111:6423'
	FEUERBACH_BOSCH_1 = 'de:08111:6423:0:3'  # Rtg Feuerbach Bf (Stuttgart Stuttgart)
	FEUERBACH_BOSCH_2 = 'de:08111:6423:0:4'  # Rtg Werner & Pfleiderer (Stuttgart Stuttgart)
	ZUFFENHAUSEN_PORSCHE = 'de:08111:6462'
	ZUFFENHAUSEN_PORSCHE_1 = 'de:08111:6462:1:1'  # Rtg Wollinstr. (Stuttgart Stuttgart)
	ZUFFENHAUSEN_PORSCHE_2 = 'de:08111:6462:2:2'  # Rtg Marconistr./Ohmstr. (Stuttgart Stuttgart)
	ZUFFENHAUSEN_PORSCHE_3 = 'de:08111:6462:3:3'  # Rtg Wollinstr. (Stuttgart Stuttgart)
	MORSESTRASSE = 'de:08111:6464'
	MORSESTRASSE_1 = 'de:08111:6464:0:3'  # Rtg Ohmstr. (Zuffenh. Bf) (Stuttgart Stuttgart)
	MORSESTRASSE_2 = 'de:08111:6464:0:4'  # Rtg Marconistraße (Stuttgart Stuttgart)
	ZUFFENHAUSEN = 'de:08111:6465'
	ZUFFENHAUSEN_1 = 'de:08111:6465:1:12'  # Gleis 12 (Stuttgart Stuttgart)
	ZUFFENHAUSEN_2 = 'de:08111:6465:2:11'  # Gleis 11 (Stuttgart Stuttgart)
	ZUFFENHAUSEN_3 = 'de:08111:6465:3:2'  # Gleis 2 (Stuttgart Stuttgart)
	ZUFFENHAUSEN_4 = 'de:08111:6465:4:4'  # Gleis 4 (Stuttgart Stuttgart)
	ZUFFENHAUSEN_5 = 'de:08111:6465:4:5'  # Gleis 5 (Stuttgart Stuttgart)
	ZUFFENHAUSEN_6 = 'de:08111:6465:5:6'  # Gleis 6 (Stuttgart Stuttgart)
	ZUFFENHAUSEN_7 = 'de:08111:6465:7:2'  # Linie 401 Rtg Werner und Pfleiderer (Stuttgart Stuttgart)
	ZUFFENHAUSEN_8 = 'de:08111:6465:7:3'  # Linie 99 Rtg Zahn-Nopper-Str. (Stuttgart Stuttgart)
	ZUFFENHAUSEN_9 = 'de:08111:6465:7:4'  # Linie 508 Rtg Zahn-Nopper-Str. (Stuttgart Stuttgart)
	ZUFFENHAUSEN_10 = 'de:08111:6465:7:7'  # Linie 52 Rtg Hohensteinstr. (Stuttgart Stuttgart)
	ZUFFENHAUSEN_11 = 'de:08111:6465:7:8'  # Linie 401 Rtg Kirchtalstr. (Stuttgart Stuttgart)
	PFLUGFELDER_STRASSE_1 = 'de:08111:6466:0:3'  # eRtg Stammheim (Stuttgart Stuttgart)
	PFLUGFELDER_STRASSE_2 = 'de:08111:6466:0:4'  # eRtg Aussiedlerhöfe (Stuttgart Stuttgart)
	PFLUGFELDER_STRASSE_3 = 'de:08118:6807:0:3'  # Rtg Ludwigsburg (Kornwestheim Kornwestheim)
	PFLUGFELDER_STRASSE_4 = 'de:08118:6807:0:4'  # Rtg Bahnhof (Kornwestheim Kornwestheim)
	OHMSTRASSE__ZUFFENHAUSEN_BF = 'de:08111:6467'
	OHMSTRASSE__ZUFFENHAUSEN_BF_1 = 'de:08111:6467:1:3'  # Rtg Hohensteinstr./Werner und Pfleiderer (Stuttgart Stuttgart)
	OHMSTRASSE__ZUFFENHAUSEN_BF_2 = 'de:08111:6467:2:4'  # Rtg Morsestr./Marconistr. (Stuttgart Stuttgart)
	EBERD__NUSSDORFER_STRASSE = 'de:08118:6468:0:3'  # eRtg Hochdorfer Straße (Eberdingen Eberdingen)
	RATHAUSSTRASSE = 'de:08118:6469:0:3'  # Rtg ??? (Eberdingen Eberdingen)
	EBERDINGEN_RATHAUSSTRASSE = 'de:08118:6469:0:3'  # Rtg ??? (Eberdingen Eberdingen)
	NORMANNENSTRASSE = 'de:08115:6470:0:3'  # eRtg Blammerbergstraße (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_NORMANNENSTR = 'de:08115:6470:0:3'  # eRtg Blammerbergstraße (Weil der Stadt Weil der Stadt)
	BLAMMERBERGSTRASSE = 'de:08115:6471:0:3'  # eRtg Alte Renninger Straße 111 (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_BLAMMERBERGSTR = 'de:08115:6471:0:3'  # eRtg Alte Renninger Straße 111 (Weil der Stadt Weil der Stadt)
	WEIL_D__ST__A__RENN__STR__111 = 'de:08115:6472:0:3'  # eRtg Alte Renninger Straße 49 (Weil der Stadt Weil der Stadt)
	ALTE_RENNINGER_STRASSE_111 = 'de:08115:6472:0:3'  # eRtg Alte Renninger Straße 49 (Weil der Stadt Weil der Stadt)
	ALTE_RENNINGER_STRASSE_49 = 'de:08115:6473:0:3'  # eRtg Alte Renninger Straße 19 (Weil der Stadt Weil der Stadt)
	WEIL_D__STADT_A__RENN__STR__49 = 'de:08115:6473:0:3'  # eRtg Alte Renninger Straße 19 (Weil der Stadt Weil der Stadt)
	WEIL_D__STADT_A__RENN__STR__19 = 'de:08115:6474:0:3'  # eRtg Weil der Stadt (Weil der Stadt Weil der Stadt)
	ALTE_RENNINGER_STRASSE_19 = 'de:08115:6474:0:3'  # eRtg Weil der Stadt (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_GYMNASIUM = 'de:08115:6477:0:3'  # eRtg Alle Richtungen (Weil der Stadt Weil der Stadt)
	JOSEPH_HAYDN_STRASSE = 'de:08115:6478'
	JOSEPH_HAYDN_STRASSE_1 = 'de:08115:6478:0:3'  # eRtg Bad Liebenzell (Weil der Stadt Weil der Stadt)
	JOSEPH_HAYDN_STRASSE_2 = 'de:08115:6478:0:4'  # eRtg Stadtgarten (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_J__HAYDN_STR = 'de:08115:6478'
	WEIL_DER_STADT_J__HAYDN_STR_1 = 'de:08115:6478:0:3'  # eRtg Bad Liebenzell (Weil der Stadt Weil der Stadt)
	WEIL_DER_STADT_J__HAYDN_STR_2 = 'de:08115:6478:0:4'  # eRtg Stadtgarten (Weil der Stadt Weil der Stadt)
	NUFRINGEN_GÄRTRINGER_STR = 'de:08115:6479'
	NUFRINGEN_GÄRTRINGER_STR_1 = 'de:08115:6479:0:3'  # Rtg Carl-Benz-Str. (Nufringen Nufringen)
	NUFRINGEN_GÄRTRINGER_STR_2 = 'de:08115:6479:0:4'  # Rtg Friedhof (Nufringen Nufringen)
	GÄRTRINGER_STRASSE = 'de:08115:6479'
	GÄRTRINGER_STRASSE_1 = 'de:08115:6479:0:3'  # Rtg Carl-Benz-Str. (Nufringen Nufringen)
	GÄRTRINGER_STRASSE_2 = 'de:08115:6479:0:4'  # Rtg Friedhof (Nufringen Nufringen)
	NUFRINGEN_FRIEDHOF = 'de:08115:6480:0:3'  # Rtg Gärtringer Str. (Nufringen Nufringen)
	NUFRINGEN_GEWERBEGEBIET = 'de:08115:6483'
	NUFRINGEN_GEWERBEGEBIET_1 = 'de:08115:6483:0:3'  # Rtg Grabenstr. (Nufringen Nufringen)
	NUFRINGEN_GEWERBEGEBIET_2 = 'de:08115:6483:0:4'  # Rtg Gärtr. Str. (Nufringen Nufringen)
	HIRSCHLANDEN_KREISVERKEHR = 'de:08118:6490'
	HIRSCHLANDEN_KREISVERKEHR_1 = 'de:08118:6490:0:3'  # Rtg Schöckingen (Ditzingen Hirschlanden)
	HIRSCHLANDEN_KREISVERKEHR_2 = 'de:08118:6490:0:4'  # Rtg Turnhalle (Ditzingen Hirschlanden)
	HIRSCHLANDEN_MERCEDESSTR = 'de:08118:6491'
	HIRSCHLANDEN_MERCEDESSTR_1 = 'de:08118:6491:0:3'  # Rtg Hirschl Nord (Ditzingen Hirschlanden)
	HIRSCHLANDEN_MERCEDESSTR_2 = 'de:08118:6491:0:4'  # Rtg Brandström/Hirschlander (Ditzingen Hirschlanden)
	HIRSCHLANDEN_NORD = 'de:08118:6492'
	HIRSCHLANDEN_NORD_1 = 'de:08118:6492:0:3'  # Rtg Rathausplatz (Ditzingen Hirschlanden)
	HIRSCHLANDEN_NORD_2 = 'de:08118:6492:0:4'  # Rtg Mercedesstr. (Ditzingen Hirschlanden)
	DITZENBRUNNER_STRASSE = 'de:08118:6493:0:3'  # eRtg Ditzenbrunner-/Knielstraße (Ditzingen Ditzingen)
	DITZINGEN_DITZENBRUNNER_STR = 'de:08118:6493:0:3'  # eRtg Ditzenbrunner-/Knielstraße (Ditzingen Ditzingen)
	THOMASHARDT_FRIEDHOF = 'de:08116:6494'
	THOMASHARDT_FRIEDHOF_1 = 'de:08116:6494:0:3'  # eRtg Hegenloher Straße (Lichtenwald Thomashardt)
	THOMASHARDT_FRIEDHOF_2 = 'de:08116:6494:0:4'  # eRtg Nachbarschaftsschule (Lichtenwald Thomashardt)
	REICHENB__F__STUTTGARTER_STR = 'de:08116:6495'
	REICHENB__F__STUTTGARTER_STR_1 = 'de:08116:6495:0:3'  # eRtg Stuttgarter-/Hauptstraße (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__STUTTGARTER_STR_2 = 'de:08116:6495:0:4'  # eRtg Ulmer Straße (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__BLUMEN__WILH_STR = 'de:08116:6496'
	REICHENB__F__BLUMEN__WILH_STR_1 = 'de:08116:6496:1:3'  # Rtg Lichtenwald//Risshalde (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__BLUMEN__WILH_STR_2 = 'de:08116:6496:1:4'  # Rtg Reichenbach (Reichenbach an der Fils Reichenbach (F))
	BLUMEN__WILHELMSTRASSE = 'de:08116:6496'
	BLUMEN__WILHELMSTRASSE_1 = 'de:08116:6496:1:3'  # Rtg Lichtenwald//Risshalde (Reichenbach an der Fils Reichenbach (F))
	BLUMEN__WILHELMSTRASSE_2 = 'de:08116:6496:1:4'  # Rtg Reichenbach (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__ULMER_STRASSE = 'de:08116:6498'
	REICHENB__F__ULMER_STRASSE_1 = 'de:08116:6498:1:3'  # Rtg Plochingen (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__ULMER_STRASSE_2 = 'de:08116:6498:1:4'  # Rtg Ebersbach (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__SCHORNDORFER_STR = 'de:08116:6499'
	REICHENB__F__SCHORNDORFER_STR_1 = 'de:08116:6499:0:3'  # eRtg Reichenbachstraße (Reichenbach an der Fils Reichenbach (F))
	REICHENB__F__SCHORNDORFER_STR_2 = 'de:08116:6499:0:4'  # eRtg Blumenstraße (Reichenbach an der Fils Reichenbach (F))
	FELLBACH = 'de:08119:6500'
	FELLBACH_1 = 'de:08119:6500:1:3'  # Gleis 3 (Fellbach Fellbach)
	FELLBACH_2 = 'de:08119:6500:1:4'  # Gleis 4 (Fellbach Fellbach)
	FELLBACH_3 = 'de:08119:6500:2:1'  # Gleis 1 (Fellbach Fellbach)
	FELLBACH_4 = 'de:08119:6500:2:2'  # Gleis 2 (Fellbach Fellbach)
	FELLBACH_5 = 'de:08119:6500:3:3'  # L. 60 Rtg Wernerstr. (Fellbach Fellbach)
	FELLBACH_6 = 'de:08119:6500:3:4'  # L.60 Rtg Stauferstr./L.67 Rtg Steinbeiss (Fellbach Fellbach)
	FELLBACH_7 = 'de:08119:6500:3:5'  # NBus (Fellbach Fellbach)
	FELLBACH_8 = 'de:08119:6500:3:6'  # 210/214 Ri Werner-/Stauferstr (Südseite) (Fellbach Fellbach)
	FELLBACH_9 = 'de:08119:6500:4:7'  # Rtg Stettten/Waiblingen Linie 212 (Fellbach Fellbach)
	SCHLICHTEN_KAISEREICHE = 'de:08119:6501'
	SCHLICHTEN_KAISEREICHE_1 = 'de:08119:6501:0:3'  # eRtg Vier Pappeln (Schorndorf Schlichten)
	SCHLICHTEN_KAISEREICHE_2 = 'de:08119:6501:0:4'  # eRtg Rathaus (Schorndorf Schlichten)
	KAISEREICHE = 'de:08119:6501'
	KAISEREICHE_1 = 'de:08119:6501:0:3'  # eRtg Vier Pappeln (Schorndorf Schlichten)
	KAISEREICHE_2 = 'de:08119:6501:0:4'  # eRtg Rathaus (Schorndorf Schlichten)
	SCHLICHTEN_STRUTSTRASSE = 'de:08119:6502:0:3'  # eRtg Kirche (Schorndorf Schlichten)
	STRUTSTRASSE = 'de:08119:6502:0:3'  # eRtg Kirche (Schorndorf Schlichten)
	BOTNANG = 'de:08111:6503'
	BOTNANG_1 = 'de:08111:6503:1:1'  # Stadtbahn - Endhst. (Stuttgart Stuttgart)
	BOTNANG_2 = 'de:08111:6503:3:3'  # Rtg Eltinger Str. (Stuttgart Stuttgart)
	REICHENBACH__F__NEUFFENSTR = 'de:08116:6504:0:3'  # eRtg Hochhaus (Reichenbach an der Fils Reichenbach (F))
	HERTICHSTRASSE_73 = 'de:08115:6505'
	HERTICHSTRASSE_73_1 = 'de:08115:6505:0:3'  # Rtg Warmbronn (Leonberg Eltingen)
	HERTICHSTRASSE_73_2 = 'de:08115:6505:0:4'  # Rtg Leobad (Leonberg Eltingen)
	ELTINGEN_HERTICHSTR__73 = 'de:08115:6505'
	ELTINGEN_HERTICHSTR__73_1 = 'de:08115:6505:0:3'  # Rtg Warmbronn (Leonberg Eltingen)
	ELTINGEN_HERTICHSTR__73_2 = 'de:08115:6505:0:4'  # Rtg Leobad (Leonberg Eltingen)
	HOLZGERLINGEN_GYMNASIUM = 'de:08115:6506'
	HOLZGERLINGEN_GYMNASIUM_1 = 'de:08115:6506:0:3'  # eRtg Neuweiler (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_GYMNASIUM_2 = 'de:08115:6506:2:4'  # eRtg Holzgerlingen (Holzgerlingen Holzgerlingen)
	TÄFERHALDE = 'de:08119:6509'
	TÄFERHALDE_1 = 'de:08119:6509:0:3'  # Rtg Backnang (Weissach im Tal Unterweissach)
	TÄFERHALDE_2 = 'de:08119:6509:0:4'  # Rtg Unterweissach (Weissach im Tal Unterweissach)
	UNTERWEISSACH_TÄFERHALDE = 'de:08119:6509'
	UNTERWEISSACH_TÄFERHALDE_1 = 'de:08119:6509:0:3'  # Rtg Backnang (Weissach im Tal Unterweissach)
	UNTERWEISSACH_TÄFERHALDE_2 = 'de:08119:6509:0:4'  # Rtg Unterweissach (Weissach im Tal Unterweissach)
	HERWEGHSTRASSE = 'de:08111:6511:0:3'  # Rtg Kleiststr. (Stuttgart Stuttgart)
	TANNBACHBRÜCKE = 'de:08119:6512:1:3'  # Rtg Steinenberg/Rudersb. (Schorndorf Miedelsbach)
	MIEDELSBACH_TANNBACHBRÜCKE = 'de:08119:6512:1:3'  # Rtg Steinenberg/Rudersb. (Schorndorf Miedelsbach)
	OBERRIEXINGEN_TH__HEUSS_STR = 'de:08118:6513:0:3'  # eRtg Roter Baum (Oberriexingen Oberriexingen)
	OBERRIEXINGEN_MÖRIKESTRASSE = 'de:08118:6514:0:3'  # eRtg Rathaus (Oberriexingen Oberriexingen)
	OTTMARSHEIM_CARL_BENZ_STR = 'de:08118:6515:1:3'  # eRtg Carl-Zeiss-Str. (Besigheim Ottmarsheim)
	CARL_BENZ_STRASSE_1 = 'de:08118:6515:1:3'  # eRtg Carl-Zeiss-Str. (Besigheim Ottmarsheim)
	CARL_BENZ_STRASSE_2 = 'de:08425:2585:0:1'  # Rtg Stadtmitte (Laichingen Laichingen)
	CARL_BENZ_STRASSE_3 = 'de:08425:2585:0:2'  # Rtg Merklingen (Laichingen Laichingen)
	OTTMARSHEIM_CARL_ZEISS_STR = 'de:08118:6516:0:3'  # eRtg Ilsfelder Straße (Besigheim Ottmarsheim)
	ROTER_BAUM = 'de:08118:6517:0:3'  # eRtg Mörikestraße (Oberriexingen Oberriexingen)
	OBERRIEXINGEN_ROTER_BAUM = 'de:08118:6517:0:3'  # eRtg Mörikestraße (Oberriexingen Oberriexingen)
	GROSSASPACH_MARBACHER_STR = 'de:08119:6518'
	GROSSASPACH_MARBACHER_STR_1 = 'de:08119:6518:0:3'  # eRtg Fürstenhof (Aspach Großaspach)
	GROSSASPACH_MARBACHER_STR_2 = 'de:08119:6518:0:4'  # eRtg Klöpferbach (Aspach Großaspach)
	SCHMIDEN_HOHENACKERSTR = 'de:08119:6521'
	SCHMIDEN_HOHENACKERSTR_1 = 'de:08119:6521:0:3'  # Rtg Schmiden Rathaus (Fellbach Schmiden)
	SCHMIDEN_HOHENACKERSTR_2 = 'de:08119:6521:0:4'  # Rtg Waiblingen Berufsschulzentrum (Fellbach Schmiden)
	HOHENACKERSTRASSE = 'de:08119:6521'
	HOHENACKERSTRASSE_1 = 'de:08119:6521:0:3'  # Rtg Schmiden Rathaus (Fellbach Schmiden)
	HOHENACKERSTRASSE_2 = 'de:08119:6521:0:4'  # Rtg Waiblingen Berufsschulzentrum (Fellbach Schmiden)
	THOMASHARDT_GEWERBEGEBIET = 'de:08116:6523'
	THOMASHARDT_GEWERBEGEBIET_1 = 'de:08116:6523:0:3'  # Rtg Kreuzung (Lichtenwald Thomashardt)
	THOMASHARDT_GEWERBEGEBIET_2 = 'de:08116:6523:0:4'  # Rtg Hegenloher Str. (Lichtenwald Thomashardt)
	OTTMARSHEIM_BESIGHEIMER_STRASSE = 'de:08118:6524:0:3'  # Rtg Besigheim/Kirchheim (Besigheim Ottmarsheim)
	HOLZGERLINGEN_HÜLBEN = 'de:08115:6525'
	HOLZGERLINGEN_HÜLBEN_1 = 'de:08115:6525:1:1'  # Gleis 1 Rtg Dettenhausen (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_HÜLBEN_2 = 'de:08115:6525:1:2'  # Gleis 2 Rtg Böblingen (Holzgerlingen Holzgerlingen)
	HÜLBEN = 'de:08115:6525'
	HÜLBEN_1 = 'de:08115:6525:1:1'  # Gleis 1 Rtg Dettenhausen (Holzgerlingen Holzgerlingen)
	HÜLBEN_2 = 'de:08115:6525:1:2'  # Gleis 2 Rtg Böblingen (Holzgerlingen Holzgerlingen)
	HÜLBEN_NORD = 'de:08115:6526:0:3'  # Rtg Agnesweg (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_HÜLBEN_NORD = 'de:08115:6526:0:3'  # Rtg Agnesweg (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_ROSSBERGWEG = 'de:08115:6528:0:3'  # Rtg Friedhof (Holzgerlingen Holzgerlingen)
	ROSSBERGWEG = 'de:08115:6528:0:3'  # Rtg Friedhof (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_FRIEDHOF = 'de:08115:6529:0:3'  # Rtg Gymnasium (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_AGNESWEG = 'de:08115:6530:0:3'  # Rtg Anne-Frank-Str. (Holzgerlingen Holzgerlingen)
	AGNESWEG = 'de:08115:6530:0:3'  # Rtg Anne-Frank-Str. (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_ANNE_FRANK_STR = 'de:08115:6531:0:3'  # Rtg Abzw. Holzgerlingen (Holzgerlingen Holzgerlingen)
	ANNE_FRANK_STRASSE = 'de:08115:6531:0:3'  # Rtg Abzw. Holzgerlingen (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_AM_ESCHELBACH = 'de:08115:6532'
	HOLZGERLINGEN_AM_ESCHELBACH_1 = 'de:08115:6532:0:3'  # Rtg Talstr. (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_AM_ESCHELBACH_2 = 'de:08115:6532:0:4'  # Rtg Daimlerstr. (Holzgerlingen Holzgerlingen)
	AM_ESCHELBACH = 'de:08115:6532'
	AM_ESCHELBACH_1 = 'de:08115:6532:0:3'  # Rtg Talstr. (Holzgerlingen Holzgerlingen)
	AM_ESCHELBACH_2 = 'de:08115:6532:0:4'  # Rtg Daimlerstr. (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_DAIMLERSTRASSE = 'de:08115:6533'
	HOLZGERLINGEN_DAIMLERSTRASSE_1 = 'de:08115:6533:0:3'  # Rtg Am Eschelbach (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_DAIMLERSTRASSE_2 = 'de:08115:6533:0:4'  # Rtg R.-Bosch-Str. (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_R__BOSCH_STR = 'de:08115:6534'
	HOLZGERLINGEN_R__BOSCH_STR_1 = 'de:08115:6534:0:3'  # Rtg Daimlerstr. (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_R__BOSCH_STR_2 = 'de:08115:6534:0:4'  # Rtg Siemensstr. (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_SIEMENSSTRASSE = 'de:08115:6535'
	HOLZGERLINGEN_SIEMENSSTRASSE_1 = 'de:08115:6535:0:3'  # Rtg R.-Bosch-Str. (Holzgerlingen Holzgerlingen)
	HOLZGERLINGEN_SIEMENSSTRASSE_2 = 'de:08115:6535:0:4'  # Rtg Max-Brose-Str. (Holzgerlingen Holzgerlingen)
	PLIENINGEN_GARBE = 'de:08111:6554'
	PLIENINGEN_GARBE_1 = 'de:08111:6554:1:1'  # Rtg Windhalmweg (Stuttgart Stuttgart)
	PLIENINGEN_GARBE_2 = 'de:08111:6554:2:2'  # Rtg Im Chausseefeld (Stuttgart Stuttgart)
	PLIENINGEN_GARBE_3 = 'de:08111:6554:2:4'  # Rtg Im Chausseefeld (Stuttgart Stuttgart)
	PLIENINGEN_GARBE_4 = 'de:08111:6554:3:4'  # Rtg Universität Hohenheim (Stuttgart Stuttgart)
	PLIENINGEN_GARBE_5 = 'de:08111:6554:4:3'  # Rtg Windhalmweg/Plieningen (Stuttgart Stuttgart)
	PLIENINGEN = 'de:08111:6555'
	PLIENINGEN_1 = 'de:08111:6555:1:1'  # Stadtbahn - Endhst. (Stuttgart Stuttgart)
	PLIENINGEN_2 = 'de:08111:6555:2:1'  # Rtg Plieningen Garbe (Stuttgart Stuttgart)
	PLIENINGEN_3 = 'de:08111:6555:2:2'  # Plieningen (Stuttgart Stuttgart)
	WALDENBUCHER_PLATZ = 'de:08111:6568:0:3'  # Rtg Leinfeldener Str. (Stuttgart Stuttgart)
	ALBSCHULE = 'de:08111:6570'
	ALBSCHULE_1 = 'de:08111:6570:0:1'  # Rtg Pfullinger Str./Leinfeldener Str. (Stuttgart Stuttgart)
	ALBSCHULE_2 = 'de:08111:6570:0:2'  # Rtg Degerloch ZOB (Stuttgart Stuttgart)
	ALBSCHULE_3 = 'de:08111:6570:0:3'  # L.71 Rtg Leinfeldener Str. (Stuttgart Stuttgart)
	HOHE_EICHE = 'de:08111:6571'
	HOHE_EICHE_1 = 'de:08111:6571:0:3'  # Rtg Weidachtal/Kaserne Möhringen (Stuttgart Stuttgart)
	HOHE_EICHE_2 = 'de:08111:6571:0:4'  # Rtg Tränke (Stuttgart Stuttgart)
	WEIDACHTAL = 'de:08111:6572'
	WEIDACHTAL_1 = 'de:08111:6572:0:3'  # Rtg Sternhäule (Stuttgart Stuttgart)
	WEIDACHTAL_2 = 'de:08111:6572:0:4'  # Rtg Hohe Eiche (Stuttgart Stuttgart)
	FASANENHOF_ABZW = 'de:08111:6580'
	FASANENHOF_ABZW_1 = 'de:08111:6580:1:3'  # L.77 Rtg  Ind-geb./L.806 Rtg Messe West (Stuttgart Stuttgart)
	FASANENHOF_ABZW_2 = 'de:08111:6580:1:4'  # Linien 77/806 Rtg EnBW City (Stuttgart Stuttgart)
	FASANENHOF_ABZWEIG = 'de:08111:6580'
	FASANENHOF_ABZWEIG_1 = 'de:08111:6580:1:3'  # L.77 Rtg  Ind-geb./L.806 Rtg Messe West (Stuttgart Stuttgart)
	FASANENHOF_ABZWEIG_2 = 'de:08111:6580:1:4'  # Linien 77/806 Rtg EnBW City (Stuttgart Stuttgart)
	NEUWIRTSHAUS_WITHAUWEG = 'de:08111:6624'
	NEUWIRTSHAUS_WITHAUWEG_1 = 'de:08111:6624:0:1'  # Rtg Kallenberg (Stuttgart Stuttgart)
	NEUWIRTSHAUS_WITHAUWEG_2 = 'de:08111:6624:0:2'  # Rtg Zuffenhausen (Stuttgart Stuttgart)
	WITHAUWEG = 'de:08111:6624'
	WITHAUWEG_1 = 'de:08111:6624:0:1'  # Rtg Kallenberg (Stuttgart Stuttgart)
	WITHAUWEG_2 = 'de:08111:6624:0:2'  # Rtg Zuffenhausen (Stuttgart Stuttgart)
	URBACH_K__HORNSCHUCH_STR = 'de:08119:6625:0:4'  # Rtg Hauptstr. (Urbach Urbach)
	K__HORNSCHUCH_STRASSE = 'de:08119:6625:0:4'  # Rtg Hauptstr. (Urbach Urbach)
	URBACH_QUELLENWEG = 'de:08119:6626'
	URBACH_QUELLENWEG_1 = 'de:08119:6626:0:3'  # Rtg Hauptstr./Friedenskirche (Urbach Urbach)
	URBACH_QUELLENWEG_2 = 'de:08119:6626:0:4'  # Rtg Birkenallee (Urbach Urbach)
	QUELLENWEG = 'de:08119:6626'
	QUELLENWEG_1 = 'de:08119:6626:0:3'  # Rtg Hauptstr./Friedenskirche (Urbach Urbach)
	QUELLENWEG_2 = 'de:08119:6626:0:4'  # Rtg Birkenallee (Urbach Urbach)
	URBACH_BENZSTRASSE = 'de:08119:6627:0:3'  # Rtg Wasenstr. (Urbach Urbach)
	BENZSTRASSE_1 = 'de:08119:6627:0:3'  # Rtg Wasenstr. (Urbach Urbach)
	BENZSTRASSE_2 = 'de:08115:7054:0:3'  # eRtg Umspannwerk (Herrenberg Herrenberg)
	BENZSTRASSE_3 = 'de:08115:7054:0:4'  # Rtg Herrenberg (Herrenberg Herrenberg)
	URBACH_KAPFFSTR = 'de:08119:6629'
	URBACH_KAPFFSTR_1 = 'de:08119:6629:0:3'  # eRtg Hauptstraße (Urbach Urbach)
	URBACH_KAPFFSTR_2 = 'de:08119:6629:0:4'  # eRtg Hölderlinweg (Urbach Urbach)
	KAPFFSTRASSE = 'de:08119:6629'
	KAPFFSTRASSE_1 = 'de:08119:6629:0:3'  # eRtg Hauptstraße (Urbach Urbach)
	KAPFFSTRASSE_2 = 'de:08119:6629:0:4'  # eRtg Hölderlinweg (Urbach Urbach)
	BACKNANG_MITTELSCHÖNTAL = 'de:08119:6632:0:3'  # eRtg Unterschöntal (Backnang Backnang)
	MITTELSCHÖNTAL = 'de:08119:6632:0:3'  # eRtg Unterschöntal (Backnang Backnang)
	KLEINHEPPACH_KORBER_STRASSE = 'de:08119:6633'
	KLEINHEPPACH_KORBER_STRASSE_1 = 'de:08119:6633:0:3'  # Rtg Heckbachstr. (Korb Kleinheppach)
	KLEINHEPPACH_KORBER_STRASSE_2 = 'de:08119:6633:0:4'  # Rtg Korb Bühl (Korb Kleinheppach)
	KORBER_STRASSE = 'de:08119:6633'
	KORBER_STRASSE_1 = 'de:08119:6633:0:3'  # Rtg Heckbachstr. (Korb Kleinheppach)
	KORBER_STRASSE_2 = 'de:08119:6633:0:4'  # Rtg Korb Bühl (Korb Kleinheppach)
	BEINSTEIN_OBSTHALDE = 'de:08119:6634:0:4'  # Rtg Rossäcker (Waiblingen Beinstein)
	OBSTHALDE = 'de:08119:6634:0:4'  # Rtg Rossäcker (Waiblingen Beinstein)
	BEINSTEIN_KLEINHEPPACHER_STR = 'de:08119:6636'
	BEINSTEIN_KLEINHEPPACHER_STR_1 = 'de:08119:6636:0:3'  # Rtg Rossäcker (Waiblingen Beinstein)
	BEINSTEIN_KLEINHEPPACHER_STR_2 = 'de:08119:6636:0:4'  # Rtg Rathaus/Endersbacher Str. (Waiblingen Beinstein)
	ROSSÄCKER = 'de:08119:6637'
	ROSSÄCKER_1 = 'de:08119:6637:0:3'  # Rtg Hausweinberg (Waiblingen Beinstein)
	ROSSÄCKER_2 = 'de:08119:6637:0:4'  # Rtg Kleinheppacher Str. (Waiblingen Beinstein)
	BEINSTEIN_ROSSÄCKER = 'de:08119:6637'
	BEINSTEIN_ROSSÄCKER_1 = 'de:08119:6637:0:3'  # Rtg Hausweinberg (Waiblingen Beinstein)
	BEINSTEIN_ROSSÄCKER_2 = 'de:08119:6637:0:4'  # Rtg Kleinheppacher Str. (Waiblingen Beinstein)
	BEINSTEIN_HAUSWEINBERG = 'de:08119:6638:1:4'  # Rtg Obsthalde (Waiblingen Beinstein)
	HAUSWEINBERG = 'de:08119:6638:1:4'  # Rtg Obsthalde (Waiblingen Beinstein)
	ALFDORF_SCHULE = 'de:08119:6640'
	ALFDORF_SCHULE_1 = 'de:08119:6640:0:1'  # Rtg Gewerbegebiet (Alfdorf Alfdorf)
	ALFDORF_SCHULE_2 = 'de:08119:6640:0:3'  # eRtg Alle Richtungen (Alfdorf Alfdorf)
	ALFDORF_DÖLLENSTRASSE = 'de:08119:6641'
	ALFDORF_DÖLLENSTRASSE_1 = 'de:08119:6641:0:3'  # Rtg Vordersteinenberg (Alfdorf Alfdorf)
	ALFDORF_DÖLLENSTRASSE_2 = 'de:08119:6641:0:4'  # Rtg Alfdorf (Alfdorf Alfdorf)
	DÖLLENSTRASSE = 'de:08119:6641'
	DÖLLENSTRASSE_1 = 'de:08119:6641:0:3'  # Rtg Vordersteinenberg (Alfdorf Alfdorf)
	DÖLLENSTRASSE_2 = 'de:08119:6641:0:4'  # Rtg Alfdorf (Alfdorf Alfdorf)
	PFAHLBRONN_BREND = 'de:08119:6642:0:3'  # eRtg Höldis (Alfdorf Pfahlbronn)
	BREND = 'de:08119:6642:0:3'  # eRtg Höldis (Alfdorf Pfahlbronn)
	PFAHLBRONN_BURGHOLZ = 'de:08119:6643:0:3'  # eRtg Mannholz (Alfdorf Pfahlbronn)
	BURGHOLZ = 'de:08119:6643:0:3'  # eRtg Mannholz (Alfdorf Pfahlbronn)
	VORDERSTEINENBERG_HÜTTENBÜHL = 'de:08119:6644:0:3'  # eRtg alle Richtungen (Alfdorf Vordersteinenberg)
	HÜTTENBÜHL = 'de:08119:6644:0:3'  # eRtg alle Richtungen (Alfdorf Vordersteinenberg)
	MANNHOLZ = 'de:08119:6645:0:3'  # eRtg Brend (Alfdorf Pfahlbronn)
	PFAHLBRONN_MANNHOLZ = 'de:08119:6645:0:3'  # eRtg Brend (Alfdorf Pfahlbronn)
	PFAHLBR__RIENHARZER_STR_SCHULE = 'de:08119:6646'
	PFAHLBR__RIENHARZER_STR_SCHULE_1 = 'de:08119:6646:0:3'  # Rtg Lorcher Str. (Alfdorf Pfahlbronn)
	PFAHLBR__RIENHARZER_STR_SCHULE_2 = 'de:08119:6646:0:4'  # Rtg Abzw. Eisenbachsee (Alfdorf Pfahlbronn)
	RIENHARZER_STR__SCHULE = 'de:08119:6646'
	RIENHARZER_STR__SCHULE_1 = 'de:08119:6646:0:3'  # Rtg Lorcher Str. (Alfdorf Pfahlbronn)
	RIENHARZER_STR__SCHULE_2 = 'de:08119:6646:0:4'  # Rtg Abzw. Eisenbachsee (Alfdorf Pfahlbronn)
	RIENHARZ = 'de:08119:6647'
	RIENHARZ_1 = 'de:08119:6647:0:3'  # Rtg. Alfdorf Tannhof/Welzheim (Alfdorf Pfahlbronn)
	RIENHARZ_2 = 'de:08119:6647:0:4'  # Rtg. Pfahlbr/Abzw. Eisenb.see (Alfdorf Pfahlbronn)
	PFAHLBRONN_RIENHARZ = 'de:08119:6647'
	PFAHLBRONN_RIENHARZ_1 = 'de:08119:6647:0:3'  # Rtg. Alfdorf Tannhof/Welzheim (Alfdorf Pfahlbronn)
	PFAHLBRONN_RIENHARZ_2 = 'de:08119:6647:0:4'  # Rtg. Pfahlbr/Abzw. Eisenb.see (Alfdorf Pfahlbronn)
	VORDERSTEINENBERG_HEINLESMÜHLE = 'de:08119:6648'
	VORDERSTEINENBERG_HEINLESMÜHLE_1 = 'de:08119:6648:0:3'  # Rtg Hundsberg (Alfdorf Vordersteinenberg)
	VORDERSTEINENBERG_HEINLESMÜHLE_2 = 'de:08119:6648:0:4'  # Rtg Hellershof (Alfdorf Vordersteinenberg)
	HEINLESMÜHLE = 'de:08119:6648'
	HEINLESMÜHLE_1 = 'de:08119:6648:0:3'  # Rtg Hundsberg (Alfdorf Vordersteinenberg)
	HEINLESMÜHLE_2 = 'de:08119:6648:0:4'  # Rtg Hellershof (Alfdorf Vordersteinenberg)
	HARDTWALDHALLE = 'de:08119:6651'
	HARDTWALDHALLE_1 = 'de:08119:6651:0:3'  # eRtg Sportplatz (Aspach Kleinaspach)
	HARDTWALDHALLE_2 = 'de:08119:6651:0:4'  # eRtg Ortsmitte (Aspach Kleinaspach)
	KLEINASP__HARDTWALDHALLE = 'de:08119:6651'
	KLEINASP__HARDTWALDHALLE_1 = 'de:08119:6651:0:3'  # eRtg Sportplatz (Aspach Kleinaspach)
	KLEINASP__HARDTWALDHALLE_2 = 'de:08119:6651:0:4'  # eRtg Ortsmitte (Aspach Kleinaspach)
	KARLSHOF = 'de:08119:6656'
	KARLSHOF_1 = 'de:08119:6656:0:3'  # Rtg Kleinaspach Ortsmitte (Aspach Kleinaspach)
	KARLSHOF_2 = 'de:08119:6656:0:4'  # Rtg Forstboden (Aspach Kleinaspach)
	KLEINASPACH_KARLSHOF = 'de:08119:6656'
	KLEINASPACH_KARLSHOF_1 = 'de:08119:6656:0:3'  # Rtg Kleinaspach Ortsmitte (Aspach Kleinaspach)
	KLEINASPACH_KARLSHOF_2 = 'de:08119:6656:0:4'  # Rtg Forstboden (Aspach Kleinaspach)
	BÜRG_SCHULERHOF = 'de:08119:6659'
	BÜRG_SCHULERHOF_1 = 'de:08119:6659:0:3'  # eRtg Bürg (Winnenden Bürg)
	BÜRG_SCHULERHOF_2 = 'de:08119:6659:0:4'  # eRtg Ortsmitte (Winnenden Bürg)
	SCHULERHOF = 'de:08119:6659'
	SCHULERHOF_1 = 'de:08119:6659:0:3'  # eRtg Bürg (Winnenden Bürg)
	SCHULERHOF_2 = 'de:08119:6659:0:4'  # eRtg Ortsmitte (Winnenden Bürg)
	SCHELMENHOLZ_BBW = 'de:08119:6660'
	SCHELMENHOLZ_BBW_1 = 'de:08119:6660:0:3'  # eRtg Theodor-Heuss-Platz (Winnenden Schelmenholz)
	SCHELMENHOLZ_BBW_2 = 'de:08119:6660:0:4'  # eRtg Lange Weiden (Winnenden Schelmenholz)
	BBW = 'de:08119:6660'
	BBW_1 = 'de:08119:6660:0:3'  # eRtg Theodor-Heuss-Platz (Winnenden Schelmenholz)
	BBW_2 = 'de:08119:6660:0:4'  # eRtg Lange Weiden (Winnenden Schelmenholz)
	ESCHENWEG = 'de:08119:6661'
	ESCHENWEG_1 = 'de:08119:6661:0:3'  # eRtg Hanweiler (Winnenden Schelmenholz)
	ESCHENWEG_2 = 'de:08119:6661:0:4'  # eRtg Forchenwaldstraße (Winnenden Schelmenholz)
	SCHELMENH__ESCHENWEG = 'de:08119:6661'
	SCHELMENH__ESCHENWEG_1 = 'de:08119:6661:0:3'  # eRtg Hanweiler (Winnenden Schelmenholz)
	SCHELMENH__ESCHENWEG_2 = 'de:08119:6661:0:4'  # eRtg Forchenwaldstraße (Winnenden Schelmenholz)
	THEODOR_HEUSS_PLATZ = 'de:08119:6662'
	THEODOR_HEUSS_PLATZ_1 = 'de:08119:6662:0:3'  # eRtg Forchenwaldstraße (Winnenden Schelmenholz)
	THEODOR_HEUSS_PLATZ_2 = 'de:08119:6662:0:4'  # eRtg BBW (Winnenden Schelmenholz)
	SCHELMENH__TH__HEUSS_PLATZ = 'de:08119:6662'
	SCHELMENH__TH__HEUSS_PLATZ_1 = 'de:08119:6662:0:3'  # eRtg Forchenwaldstraße (Winnenden Schelmenholz)
	SCHELMENH__TH__HEUSS_PLATZ_2 = 'de:08119:6662:0:4'  # eRtg BBW (Winnenden Schelmenholz)
	BAACHER_WEG = 'de:08119:6664:0:3'  # eRtg Baacher Straße (Winnenden Hertmannsweiler)
	HERTM__BAACHER_WEG = 'de:08119:6664:0:3'  # eRtg Baacher Straße (Winnenden Hertmannsweiler)
	WIESENTALSTRASSE_1 = 'de:08119:6665:1:3'  # Alle Richtungen (Weinstadt Schnait)
	WIESENTALSTRASSE_2 = 'de:08119:7455:0:3'  # Rtg Hertmannsweiler B14 (Winnenden Hertmannsweiler)
	SCHNAIT_WIESENTALSTRASSE = 'de:08119:6665:1:3'  # Alle Richtungen (Weinstadt Schnait)
	HINTERE_TOS = 'de:08119:6666'
	HINTERE_TOS_1 = 'de:08119:6666:0:3'  # eRtg Sommerrain (Backnang Backnang)
	HINTERE_TOS_2 = 'de:08119:6666:0:4'  # eRtg Stettiner Ring (Backnang Backnang)
	BACKNANG_HINTERE_TOS = 'de:08119:6666'
	BACKNANG_HINTERE_TOS_1 = 'de:08119:6666:0:3'  # eRtg Sommerrain (Backnang Backnang)
	BACKNANG_HINTERE_TOS_2 = 'de:08119:6666:0:4'  # eRtg Stettiner Ring (Backnang Backnang)
	HINTERSTEINENBERG_SÜD = 'de:08119:6668'
	HINTERSTEINENBERG_SÜD_1 = 'de:08119:6668:0:3'  # eRtg Greuthöfle (Alfdorf Vordersteinenberg)
	HINTERSTEINENBERG_SÜD_2 = 'de:08119:6668:0:4'  # eRtg Vordersteinenberg (Alfdorf Vordersteinenberg)
	VORDERST__HINTERST__SÜD = 'de:08119:6668'
	VORDERST__HINTERST__SÜD_1 = 'de:08119:6668:0:3'  # eRtg Greuthöfle (Alfdorf Vordersteinenberg)
	VORDERST__HINTERST__SÜD_2 = 'de:08119:6668:0:4'  # eRtg Vordersteinenberg (Alfdorf Vordersteinenberg)
	VORDERSTEINENBERG_GREUTHÖFLE = 'de:08119:6669'
	VORDERSTEINENBERG_GREUTHÖFLE_1 = 'de:08119:6669:0:3'  # eRtg Wahlenheim (Alfdorf Vordersteinenberg)
	VORDERSTEINENBERG_GREUTHÖFLE_2 = 'de:08119:6669:0:4'  # eRtg Hintersteinenberg Süd (Alfdorf Vordersteinenberg)
	GREUTHÖFLE = 'de:08119:6669'
	GREUTHÖFLE_1 = 'de:08119:6669:0:3'  # eRtg Wahlenheim (Alfdorf Vordersteinenberg)
	GREUTHÖFLE_2 = 'de:08119:6669:0:4'  # eRtg Hintersteinenberg Süd (Alfdorf Vordersteinenberg)
	KAISERSBACH_HEUMADEN = 'de:08119:6671'
	KAISERSBACH_HEUMADEN_1 = 'de:08119:6671:0:3'  # eRtg Eulenhof (Kaisersbach Kaisersbach)
	KAISERSBACH_HEUMADEN_2 = 'de:08119:6671:0:4'  # eRtg Hauptstraße (Kaisersbach Kaisersbach)
	JUX_KINDERGARTEN = 'de:08119:6672'
	JUX_KINDERGARTEN_1 = 'de:08119:6672:0:3'  # eRtg Hüttlen (Spiegelberg Jux)
	JUX_KINDERGARTEN_2 = 'de:08119:6672:0:4'  # eRtg Jux (Spiegelberg Jux)
	KIRCHENK__METTELBACH = 'de:08119:6674:0:3'  # eRtg Marxenhof (Murrhardt Kirchenkirnberg)
	METTELBACH = 'de:08119:6674:0:3'  # eRtg Marxenhof (Murrhardt Kirchenkirnberg)
	LINDENTALER_STRASSE = 'de:08119:6675:0:3'  # eRtg Ortsmitte (Rudersberg Schlechtbach)
	SCHLECHTBACH_LINDENTALER_STR = 'de:08119:6675:0:3'  # eRtg Ortsmitte (Rudersberg Schlechtbach)
	SCHNEIDER = 'de:08119:6676'
	SCHNEIDER_1 = 'de:08119:6676:0:3'  # Rtg Pfarräcker (Waiblingen Neustadt)
	SCHNEIDER_2 = 'de:08119:6676:0:4'  # Rtg Torstr. (Waiblingen Neustadt)
	NEUSTADT_SCHNEIDER = 'de:08119:6676'
	NEUSTADT_SCHNEIDER_1 = 'de:08119:6676:0:3'  # Rtg Pfarräcker (Waiblingen Neustadt)
	NEUSTADT_SCHNEIDER_2 = 'de:08119:6676:0:4'  # Rtg Torstr. (Waiblingen Neustadt)
	SALIERSCHULE = 'de:08119:6677:0:3'  # Rtg Mikrozentrum (Waiblingen Waiblingen)
	WAIBLINGEN_SALIERSCHULE = 'de:08119:6677:0:3'  # Rtg Mikrozentrum (Waiblingen Waiblingen)
	SCHORNDORF_REITHALLE = 'de:08119:6680'
	SCHORNDORF_REITHALLE_1 = 'de:08119:6680:0:3'  # eRtg Dürrbach (Schorndorf Schorndorf)
	SCHORNDORF_REITHALLE_2 = 'de:08119:6680:0:4'  # eRtg Oskar Frech Seebad (Schorndorf Schorndorf)
	REITHALLE = 'de:08119:6680'
	REITHALLE_1 = 'de:08119:6680:0:3'  # eRtg Dürrbach (Schorndorf Schorndorf)
	REITHALLE_2 = 'de:08119:6680:0:4'  # eRtg Oskar Frech Seebad (Schorndorf Schorndorf)
	URBACH_FRIEDENSKIRCHE = 'de:08119:6682'
	URBACH_FRIEDENSKIRCHE_1 = 'de:08119:6682:0:3'  # eRtg Birkenallee (Bahnübergang) (Urbach Urbach)
	URBACH_FRIEDENSKIRCHE_2 = 'de:08119:6682:0:4'  # eRtg Hauptstraße (Urbach Urbach)
	FRIEDENSKIRCHE = 'de:08119:6682'
	FRIEDENSKIRCHE_1 = 'de:08119:6682:0:3'  # eRtg Birkenallee (Bahnübergang) (Urbach Urbach)
	FRIEDENSKIRCHE_2 = 'de:08119:6682:0:4'  # eRtg Hauptstraße (Urbach Urbach)
	BIRKENALLEE__BAHNUNTERFÜHRUNG = 'de:08119:6683'
	BIRKENALLEE__BAHNUNTERFÜHRUNG_1 = 'de:08119:6683:1:3'  # Rtg Ahornstraße (Plüderhausen Plüderhausen)
	BIRKENALLEE__BAHNUNTERFÜHRUNG_2 = 'de:08119:6683:1:4'  # eRtg Steinhalde (Plüderhausen Plüderhausen)
	BIRKENALLEE__BAHNUNTERFÜHRUNG_3 = 'de:08119:6683:1:5'  # Rtg Hohbaum (Plüderhausen Plüderhausen)
	BIRKENALLEE__BAHNUNTERFÜHRUNG_4 = 'de:08119:6683:1:6'  # Rtg Urbach Quellenweg (Plüderhausen Plüderhausen)
	PLÜD__BIRKENALLEE_BAHNUNTERF = 'de:08119:6683'
	PLÜD__BIRKENALLEE_BAHNUNTERF_1 = 'de:08119:6683:1:3'  # Rtg Ahornstraße (Plüderhausen Plüderhausen)
	PLÜD__BIRKENALLEE_BAHNUNTERF_2 = 'de:08119:6683:1:4'  # eRtg Steinhalde (Plüderhausen Plüderhausen)
	PLÜD__BIRKENALLEE_BAHNUNTERF_3 = 'de:08119:6683:1:5'  # Rtg Hohbaum (Plüderhausen Plüderhausen)
	PLÜD__BIRKENALLEE_BAHNUNTERF_4 = 'de:08119:6683:1:6'  # Rtg Urbach Quellenweg (Plüderhausen Plüderhausen)
	WELZHEIM_GSCHWENDER_STR = 'de:08119:6686'
	WELZHEIM_GSCHWENDER_STR_1 = 'de:08119:6686:0:3'  # eRtg Schulzentrum/Steinbeise (Welzheim Welzheim)
	WELZHEIM_GSCHWENDER_STR_2 = 'de:08119:6686:0:4'  # eRtg Kirchplatz/Brunnenstr./Gottlob/O-Wa (Welzheim Welzheim)
	GSCHWENDER_STRASSE = 'de:08119:6686'
	GSCHWENDER_STRASSE_1 = 'de:08119:6686:0:3'  # eRtg Schulzentrum/Steinbeise (Welzheim Welzheim)
	GSCHWENDER_STRASSE_2 = 'de:08119:6686:0:4'  # eRtg Kirchplatz/Brunnenstr./Gottlob/O-Wa (Welzheim Welzheim)
	BACKNANG_STAIGACKER = 'de:08119:6687:0:3'  # eRtg Eichwald (Backnang Backnang)
	STAIGACKER = 'de:08119:6687:0:3'  # eRtg Eichwald (Backnang Backnang)
	BACKNANG_EICHWALD = 'de:08119:6688'
	BACKNANG_EICHWALD_1 = 'de:08119:6688:0:3'  # eRtg Staigacker (Backnang Backnang)
	BACKNANG_EICHWALD_2 = 'de:08119:6688:0:4'  # eRtg Waldfriedhof (Backnang Backnang)
	EICHWALD = 'de:08119:6688'
	EICHWALD_1 = 'de:08119:6688:0:3'  # eRtg Staigacker (Backnang Backnang)
	EICHWALD_2 = 'de:08119:6688:0:4'  # eRtg Waldfriedhof (Backnang Backnang)
	MURRHARDT_SCHWAMMHOF = 'de:08119:6689:0:4'  # eRtg Murrhardt (Murrhardt Murrhardt)
	SCHWAMMHOF = 'de:08119:6689:0:4'  # eRtg Murrhardt (Murrhardt Murrhardt)
	MURRHARDT_WALTERSBERG = 'de:08119:6690'
	MURRHARDT_WALTERSBERG_1 = 'de:08119:6690:0:3'  # eRtg Murrhardt (Murrhardt Murrhardt)
	MURRHARDT_WALTERSBERG_2 = 'de:08119:6690:0:4'  # eRtg Schwammhof (Murrhardt Murrhardt)
	WALTERSBERG = 'de:08119:6690'
	WALTERSBERG_1 = 'de:08119:6690:0:3'  # eRtg Murrhardt (Murrhardt Murrhardt)
	WALTERSBERG_2 = 'de:08119:6690:0:4'  # eRtg Schwammhof (Murrhardt Murrhardt)
	BANRAIN = 'de:08119:6691:0:3'  # eRtg Seehalde (Urbach Urbach)
	URBACH_BANRAIN = 'de:08119:6691:0:3'  # eRtg Seehalde (Urbach Urbach)
	URBACH_SCHLOSS = 'de:08119:6692'
	URBACH_SCHLOSS_1 = 'de:08119:6692:0:3'  # eRtg Banrain (Urbach Urbach)
	URBACH_SCHLOSS_2 = 'de:08119:6692:0:4'  # Rtg Wittumhalle (Urbach Urbach)
	SCHORNDORF_ERLENSTRASSE = 'de:08119:6693:0:3'  # eRtg Mittlere Brücke/Grafenbergweg (neu) (Schorndorf Schorndorf)
	ERLENSTRASSE = 'de:08119:6693:0:3'  # eRtg Mittlere Brücke/Grafenbergweg (neu) (Schorndorf Schorndorf)
	URBACH_SCHORNDORFER_STR = 'de:08119:6694'
	URBACH_SCHORNDORFER_STR_1 = 'de:08119:6694:0:3'  # eRtg Hauptstraße (Urbach Urbach)
	URBACH_SCHORNDORFER_STR_2 = 'de:08119:6694:0:4'  # eRtg Wasenstraße (Urbach Urbach)
	URBACH_HÖLDERLINWEG = 'de:08119:6695'
	URBACH_HÖLDERLINWEG_1 = 'de:08119:6695:0:3'  # Rtg Feuerwehr (Urbach Urbach)
	URBACH_HÖLDERLINWEG_2 = 'de:08119:6695:0:4'  # Rtg Kapffstr. (Urbach Urbach)
	HÖLDERLINWEG = 'de:08119:6695'
	HÖLDERLINWEG_1 = 'de:08119:6695:0:3'  # Rtg Feuerwehr (Urbach Urbach)
	HÖLDERLINWEG_2 = 'de:08119:6695:0:4'  # Rtg Kapffstr. (Urbach Urbach)
	URBACH_FEUERWEHR = 'de:08119:6696'
	URBACH_FEUERWEHR_1 = 'de:08119:6696:0:3'  # Rtg Burgstr./Schloss (Urbach Urbach)
	URBACH_FEUERWEHR_2 = 'de:08119:6696:0:4'  # Rtg Hölderlinweg (Urbach Urbach)
	URBACH_SEEHALDE = 'de:08119:6697'
	URBACH_SEEHALDE_1 = 'de:08119:6697:0:3'  # eRtg Burgstraße (Urbach Urbach)
	URBACH_SEEHALDE_2 = 'de:08119:6697:0:4'  # Rtg Banrain (Urbach Urbach)
	SEEHALDE = 'de:08119:6697'
	SEEHALDE_1 = 'de:08119:6697:0:3'  # eRtg Burgstraße (Urbach Urbach)
	SEEHALDE_2 = 'de:08119:6697:0:4'  # Rtg Banrain (Urbach Urbach)
	URBACH_BURGSTRASSE = 'de:08119:6698'
	URBACH_BURGSTRASSE_1 = 'de:08119:6698:0:3'  # eRtg Feuerwehr (Urbach Urbach)
	URBACH_BURGSTRASSE_2 = 'de:08119:6698:0:4'  # Rtg Seehalde (Urbach Urbach)
	HOFEN_SONNENHALDE = 'de:08118:6700'
	HOFEN_SONNENHALDE_1 = 'de:08118:6700:0:3'  # eRtg Hofen (Bönnigheim Hofen (Bönnigheim))
	HOFEN_SONNENHALDE_2 = 'de:08118:6700:0:4'  # eRtg Erligheim (Bönnigheim Hofen (Bönnigheim))
	DANZIGER_WEG = 'de:08118:6701'
	DANZIGER_WEG_1 = 'de:08118:6701:0:3'  # eRtg Winkels (Sersheim Sersheim)
	DANZIGER_WEG_2 = 'de:08118:6701:0:4'  # eRtg Bahnhofstraße (Sersheim Sersheim)
	SERSHEIM_DANZIGER_WEG = 'de:08118:6701'
	SERSHEIM_DANZIGER_WEG_1 = 'de:08118:6701:0:3'  # eRtg Winkels (Sersheim Sersheim)
	SERSHEIM_DANZIGER_WEG_2 = 'de:08118:6701:0:4'  # eRtg Bahnhofstraße (Sersheim Sersheim)
	KOHLBERG_GOETHESTRASSE = 'de:08116:6703:0:3'  # eRtg Metzinger Straße (Kohlberg Kohlberg)
	KOHLBERG_SCHILLERSTRASSE = 'de:08116:6707:0:3'  # eRtg Linde (Kohlberg Kohlberg)
	ALTENRIET_SPORTPLATZ = 'de:08116:6708:0:3'  # Rtg Schlaitdorf Rathaus (Altenriet Altenriet)
	WASSERWERK = 'de:08116:6709'
	WASSERWERK_1 = 'de:08116:6709:0:3'  # eRtg Nürtinger Straße (Neckartailfingen Neckartailfingen)
	WASSERWERK_2 = 'de:08116:6709:0:4'  # eRtg Halde (Neckartailfingen Neckartailfingen)
	NECKARTAILF__WASSERWERK = 'de:08116:6709'
	NECKARTAILF__WASSERWERK_1 = 'de:08116:6709:0:3'  # eRtg Nürtinger Straße (Neckartailfingen Neckartailfingen)
	NECKARTAILF__WASSERWERK_2 = 'de:08116:6709:0:4'  # eRtg Halde (Neckartailfingen Neckartailfingen)
	WERNAU__N__QUADRIUM = 'de:08116:6710'
	WERNAU__N__QUADRIUM_1 = 'de:08116:6710:0:3'  # eRtg Wernau (N)/Eulenberg (Wernau (Neckar) Wernau (N))
	WERNAU__N__QUADRIUM_2 = 'de:08116:6710:0:4'  # eRtg Kirche Sankt Magnus/Adlerstraße (Wernau (Neckar) Wernau (N))
	QUADRIUM = 'de:08116:6710'
	QUADRIUM_1 = 'de:08116:6710:0:3'  # eRtg Wernau (N)/Eulenberg (Wernau (Neckar) Wernau (N))
	QUADRIUM_2 = 'de:08116:6710:0:4'  # eRtg Kirche Sankt Magnus/Adlerstraße (Wernau (Neckar) Wernau (N))
	REICHENBACH__F__WEINBERGSTR = 'de:08116:6713'
	REICHENBACH__F__WEINBERGSTR_1 = 'de:08116:6713:0:3'  # eRtg Risshalde / Stgt.-Hauptstraße (Reichenbach an der Fils Reichenbach (F))
	REICHENBACH__F__WEINBERGSTR_2 = 'de:08116:6713:0:4'  # eRtg Blumen-Wilhelmstraße (Reichenbach an der Fils Reichenbach (F))
	OBERLENNINGEN_RATHAUS = 'de:08116:6715:1:3'  # eRtg Gutenberger Straße 17 (Lenningen Oberlenningen)
	WOLFENMÜHLE = 'de:08115:6716'
	WOLFENMÜHLE_1 = 'de:08115:6716:0:3'  # eRtg Rathaus (Schönaich Schönaich)
	WOLFENMÜHLE_2 = 'de:08115:6716:0:4'  # eRtg Bühläcker (Schönaich Schönaich)
	SCHÖNAICH_WOLFENMÜHLE = 'de:08115:6716'
	SCHÖNAICH_WOLFENMÜHLE_1 = 'de:08115:6716:0:3'  # eRtg Rathaus (Schönaich Schönaich)
	SCHÖNAICH_WOLFENMÜHLE_2 = 'de:08115:6716:0:4'  # eRtg Bühläcker (Schönaich Schönaich)
	BISSINGEN_GROTZSTR___FRIEDHOF = 'de:08118:6717'
	BISSINGEN_GROTZSTR___FRIEDHOF_1 = 'de:08118:6717:0:3'  # eRtg Grotzstraße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_GROTZSTR___FRIEDHOF_2 = 'de:08118:6717:0:4'  # eRtg Schubartstraße (Bietigheim-Bissingen Bissingen (LB))
	GROTZSTRASSE__FRIEDHOF = 'de:08118:6717'
	GROTZSTRASSE__FRIEDHOF_1 = 'de:08118:6717:0:3'  # eRtg Grotzstraße (Bietigheim-Bissingen Bissingen (LB))
	GROTZSTRASSE__FRIEDHOF_2 = 'de:08118:6717:0:4'  # eRtg Schubartstraße (Bietigheim-Bissingen Bissingen (LB))
	KLEININGERSH__SCHLOSSSTR = 'de:08118:6718'
	KLEININGERSH__SCHLOSSSTR_1 = 'de:08118:6718:0:3'  # eRtg Brühlstraße (Ingersheim Kleiningersheim)
	KLEININGERSH__SCHLOSSSTR_2 = 'de:08118:6718:0:4'  # eRtg Blumenstraße (Ingersheim Kleiningersheim)
	METTERZ__BIETIGHEIMER_STR = 'de:08118:6719:0:3'  # eRtg Helenenburgweg (Bietigheim-Bissingen Metterzimmern)
	BISS__KELTERSTRASSE = 'de:08118:6720'
	BISS__KELTERSTRASSE_1 = 'de:08118:6720:0:3'  # Bus 556/557 (Bietigheim-Bissingen Bissingen (LB))
	BISS__KELTERSTRASSE_2 = 'de:08118:6720:0:4'  # Bus 553/554/540 (Bietigheim-Bissingen Bissingen (LB))
	HOFEN_AUSSIEDLERHÖFE = 'de:08118:6721'
	HOFEN_AUSSIEDLERHÖFE_1 = 'de:08118:6721:0:3'  # eRtg Schlossbergallee (Bönnigheim Hofen (Bönnigheim))
	HOFEN_AUSSIEDLERHÖFE_2 = 'de:08118:6721:0:4'  # eRtg Hofen (Bönnigheim Hofen (Bönnigheim))
	BRUCKENSTRASSE = 'de:08116:6722'
	BRUCKENSTRASSE_1 = 'de:08116:6722:0:3'  # Rtg Kirchheim/Brühl (Wernau (Neckar) Wernau (N))
	BRUCKENSTRASSE_2 = 'de:08116:6722:0:4'  # Rtg Sankt Magnus (Wernau (Neckar) Wernau (N))
	WERNAU__N__BRUCKENSTRASSE = 'de:08116:6722'
	WERNAU__N__BRUCKENSTRASSE_1 = 'de:08116:6722:0:3'  # Rtg Kirchheim/Brühl (Wernau (Neckar) Wernau (N))
	WERNAU__N__BRUCKENSTRASSE_2 = 'de:08116:6722:0:4'  # Rtg Sankt Magnus (Wernau (Neckar) Wernau (N))
	WERNAU__N__ADLERSTRASSE = 'de:08116:6723:0:4'  # Rtg Stadtplatz (Wernau (Neckar) Wernau (N))
	WERNAU__N__KÖNIGSBERGER_STRASSE = 'de:08116:6724:0:3'  # eRtg Adlerstraße (Wernau (Neckar) Wernau (N))
	KIRCHE_SANKT_MAGNUS = 'de:08116:6726'
	KIRCHE_SANKT_MAGNUS_1 = 'de:08116:6726:0:3'  # Rtg Kirchhheim (Wernau (Neckar) Wernau (N))
	KIRCHE_SANKT_MAGNUS_2 = 'de:08116:6726:0:4'  # Rtg Wernau (Wernau (Neckar) Wernau (N))
	WERNAU__N__KIRCHE_SANKT_MAGNUS = 'de:08116:6726'
	WERNAU__N__KIRCHE_SANKT_MAGNUS_1 = 'de:08116:6726:0:3'  # Rtg Kirchhheim (Wernau (Neckar) Wernau (N))
	WERNAU__N__KIRCHE_SANKT_MAGNUS_2 = 'de:08116:6726:0:4'  # Rtg Wernau (Wernau (Neckar) Wernau (N))
	PLATTENHARDT_GRIEBENÄCKER = 'de:08116:6727'
	PLATTENHARDT_GRIEBENÄCKER_1 = 'de:08116:6727:0:3'  # Rtg Schillerstraße (Filderstadt Plattenhardt)
	PLATTENHARDT_GRIEBENÄCKER_2 = 'de:08116:6727:0:4'  # Rtg Bernhausen Bf (Filderstadt Plattenhardt)
	GRIEBENÄCKER = 'de:08116:6727'
	GRIEBENÄCKER_1 = 'de:08116:6727:0:3'  # Rtg Schillerstraße (Filderstadt Plattenhardt)
	GRIEBENÄCKER_2 = 'de:08116:6727:0:4'  # Rtg Bernhausen Bf (Filderstadt Plattenhardt)
	UHLBERG = 'de:08116:6728'
	UHLBERG_1 = 'de:08116:6728:0:3'  # Rtg Filderklinik (Filderstadt Plattenhardt)
	UHLBERG_2 = 'de:08116:6728:0:4'  # Rtg Altenheim (Filderstadt Plattenhardt)
	PLATTENHARDT_UHLBERG = 'de:08116:6728'
	PLATTENHARDT_UHLBERG_1 = 'de:08116:6728:0:3'  # Rtg Filderklinik (Filderstadt Plattenhardt)
	PLATTENHARDT_UHLBERG_2 = 'de:08116:6728:0:4'  # Rtg Altenheim (Filderstadt Plattenhardt)
	SCHWABENGALERIE = 'de:08111:6729'
	SCHWABENGALERIE_1 = 'de:08111:6729:0:3'  # Rtg Schillerpl./Vaihingen ZOB (Stuttgart Stuttgart)
	SCHWABENGALERIE_2 = 'de:08111:6729:0:4'  # Rtg Krehlstr. (Stuttgart Stuttgart)
	PLATTENHARDT_LAILENSÄCKER = 'de:08116:6730'
	PLATTENHARDT_LAILENSÄCKER_1 = 'de:08116:6730:0:3'  # Rtg Weilerhau (Filderstadt Plattenhardt)
	PLATTENHARDT_LAILENSÄCKER_2 = 'de:08116:6730:0:4'  # Rtg Schnecken (Filderstadt Plattenhardt)
	LAILENSÄCKER = 'de:08116:6730'
	LAILENSÄCKER_1 = 'de:08116:6730:0:3'  # Rtg Weilerhau (Filderstadt Plattenhardt)
	LAILENSÄCKER_2 = 'de:08116:6730:0:4'  # Rtg Schnecken (Filderstadt Plattenhardt)
	CARL_SCHMINCKE_STRASSE = 'de:08115:6733'
	CARL_SCHMINCKE_STRASSE_1 = 'de:08115:6733:0:3'  # Rtg Glemseckstr. (Leonberg Eltingen)
	CARL_SCHMINCKE_STRASSE_2 = 'de:08115:6733:0:4'  # Rtg Mörike-Schule (Leonberg Eltingen)
	ELTINGEN_CARL_SCHMINCKE_STR = 'de:08115:6733'
	ELTINGEN_CARL_SCHMINCKE_STR_1 = 'de:08115:6733:0:3'  # Rtg Glemseckstr. (Leonberg Eltingen)
	ELTINGEN_CARL_SCHMINCKE_STR_2 = 'de:08115:6733:0:4'  # Rtg Mörike-Schule (Leonberg Eltingen)
	WARMBRONN_BETRIEBSHOF = 'de:08115:6734:1:3'  # Rtg Sindelfingen (Leonberg Warmbronn)
	BETRIEBSHOF = 'de:08115:6734:1:3'  # Rtg Sindelfingen (Leonberg Warmbronn)
	KRONENBRUNNEN = 'de:08116:6735'
	KRONENBRUNNEN_1 = 'de:08116:6735:0:3'  # Rtg Rathaus (Filderstadt Plattenhardt)
	KRONENBRUNNEN_2 = 'de:08116:6735:0:4'  # Rtg Driefbrunnen (Filderstadt Plattenhardt)
	PLATTENHARDT_KRONENBRUNNEN = 'de:08116:6735'
	PLATTENHARDT_KRONENBRUNNEN_1 = 'de:08116:6735:0:3'  # Rtg Rathaus (Filderstadt Plattenhardt)
	PLATTENHARDT_KRONENBRUNNEN_2 = 'de:08116:6735:0:4'  # Rtg Driefbrunnen (Filderstadt Plattenhardt)
	SPITZÄCKER = 'de:08116:6736'
	SPITZÄCKER_1 = 'de:08116:6736:0:3'  # Rtg Altenheim (Filderstadt Plattenhardt)
	SPITZÄCKER_2 = 'de:08116:6736:0:4'  # Rtg Schönbuchstr. (Filderstadt Plattenhardt)
	PLATTENHARDT_SPITZÄCKER = 'de:08116:6736'
	PLATTENHARDT_SPITZÄCKER_1 = 'de:08116:6736:0:3'  # Rtg Altenheim (Filderstadt Plattenhardt)
	PLATTENHARDT_SPITZÄCKER_2 = 'de:08116:6736:0:4'  # Rtg Schönbuchstr. (Filderstadt Plattenhardt)
	MÖTZINGEN_LINDENHOF = 'de:08115:6739'
	MÖTZINGEN_LINDENHOF_1 = 'de:08115:6739:0:3'  # Rtg Rathaus (Mötzingen Mötzingen)
	MÖTZINGEN_LINDENHOF_2 = 'de:08115:6739:0:4'  # Rtg Unterjettingen Mötzinger Str (Mötzingen Mötzingen)
	LINDENHOF_1 = 'de:08115:6739:0:3'  # Rtg Rathaus (Mötzingen Mötzingen)
	LINDENHOF_2 = 'de:08115:6739:0:4'  # Rtg Unterjettingen Mötzinger Str (Mötzingen Mötzingen)
	LINDENHOF_3 = 'de:08117:119:0:H1'  # Rtg Heiligenäcker (Geislingen an der Steige Geislingen (Steige))
	LINDENHOF_4 = 'de:08117:119:0:R1'  # Rtg Berufsschulzentrum (Geislingen an der Steige Geislingen (Steige))
	LINDENHOF_5 = 'de:08117:205:0:H1'  # Rtg Böhmenkirch (Böhmenkirch Böhmenkirch)
	LINDENHOF_6 = 'de:08117:205:0:R1'  # Rtg Steinenkirch/Geislingen (Böhmenkirch Böhmenkirch)
	SÜDBAHNHOF = 'de:08115:6742:1:1'  # R-Bahn (Böblingen Böblingen)
	BÖBLINGEN_SÜDBAHNHOF = 'de:08115:6742:1:1'  # R-Bahn (Böblingen Böblingen)
	BÖBLINGEN_STETTINER_STR = 'de:08115:6744:0:3'  # eRtg Parkstraße (Landratsamt) (Böblingen Böblingen)
	NEBRINGEN_SINDLINGER_STRASSE = 'de:08115:6745'
	NEBRINGEN_SINDLINGER_STRASSE_1 = 'de:08115:6745:0:1'  # Rtg Öschelbronn (Gäufelden Nebringen)
	NEBRINGEN_SINDLINGER_STRASSE_2 = 'de:08115:6745:0:2'  # Rtg Herrenberg (Gäufelden Nebringen)
	SINDELF__MERCEDES_BENZ_P307 = 'de:08115:6746'
	SINDELF__MERCEDES_BENZ_P307_1 = 'de:08115:6746:0:3'  # eRtg Calwer Brücke (Sindelfingen Sindelfingen)
	SINDELF__MERCEDES_BENZ_P307_2 = 'de:08115:6746:0:4'  # Rtg Mercedes-Benz P 305 (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_P307 = 'de:08115:6746'
	MERCEDES_BENZ_P307_1 = 'de:08115:6746:0:3'  # eRtg Calwer Brücke (Sindelfingen Sindelfingen)
	MERCEDES_BENZ_P307_2 = 'de:08115:6746:0:4'  # Rtg Mercedes-Benz P 305 (Sindelfingen Sindelfingen)
	DÖFFINGEN_DROSSELWEG = 'de:08115:6748:0:3'  # eRtg Elsterweg (Grafenau Döffingen)
	ELSTERWEG = 'de:08115:6749:0:3'  # eRtg Seeger (Grafenau Döffingen)
	DÖFFINGEN_ELSTERWEG = 'de:08115:6749:0:3'  # eRtg Seeger (Grafenau Döffingen)
	BÖBL__DIEZENHALDE__ZENTRUM = 'de:08115:6751:1:3'  # eRtg Freiburger Allee (Böblingen Böblingen)
	DIEZENHALDE__ZENTRUM = 'de:08115:6751:1:3'  # eRtg Freiburger Allee (Böblingen Böblingen)
	MITTENBÜHL = 'de:08115:6752'
	MITTENBÜHL_1 = 'de:08115:6752:0:3'  # eRtg Ulrichweg (Grafenau Döffingen)
	MITTENBÜHL_2 = 'de:08115:6752:0:4'  # eRtg Röte (Grafenau Döffingen)
	DÖFFINGEN_MITTENBÜHL = 'de:08115:6752'
	DÖFFINGEN_MITTENBÜHL_1 = 'de:08115:6752:0:3'  # eRtg Ulrichweg (Grafenau Döffingen)
	DÖFFINGEN_MITTENBÜHL_2 = 'de:08115:6752:0:4'  # eRtg Röte (Grafenau Döffingen)
	ULRICHWEG = 'de:08115:6754'
	ULRICHWEG_1 = 'de:08115:6754:0:3'  # eRtg Seeger (Grafenau Döffingen)
	ULRICHWEG_2 = 'de:08115:6754:0:4'  # eRtg Mittenbühl (Grafenau Döffingen)
	DÖFFINGEN_ULRICHWEG = 'de:08115:6754'
	DÖFFINGEN_ULRICHWEG_1 = 'de:08115:6754:0:3'  # eRtg Seeger (Grafenau Döffingen)
	DÖFFINGEN_ULRICHWEG_2 = 'de:08115:6754:0:4'  # eRtg Mittenbühl (Grafenau Döffingen)
	DÖFFINGEN_RÖTE = 'de:08115:6756'
	DÖFFINGEN_RÖTE_1 = 'de:08115:6756:0:3'  # eRtg Mittenbühl (Grafenau Döffingen)
	DÖFFINGEN_RÖTE_2 = 'de:08115:6756:0:4'  # eRtg Hallenbad (Grafenau Döffingen)
	WOLF_HIRTH_STRASSE = 'de:08116:6758'
	WOLF_HIRTH_STRASSE_1 = 'de:08116:6758:0:3'  # eRtg Dornierstraße (Esslingen am Neckar Sirnau)
	WOLF_HIRTH_STRASSE_2 = 'de:08116:6758:0:4'  # eRtg Betriebshof (Esslingen am Neckar Sirnau)
	SIRNAU_WOLF_HIRTH_STR = 'de:08116:6758'
	SIRNAU_WOLF_HIRTH_STR_1 = 'de:08116:6758:0:3'  # eRtg Dornierstraße (Esslingen am Neckar Sirnau)
	SIRNAU_WOLF_HIRTH_STR_2 = 'de:08116:6758:0:4'  # eRtg Betriebshof (Esslingen am Neckar Sirnau)
	SIRNAU_DORNIERSTR = 'de:08116:6759'
	SIRNAU_DORNIERSTR_1 = 'de:08116:6759:0:3'  # eRtg Körschtal (Esslingen am Neckar Sirnau)
	SIRNAU_DORNIERSTR_2 = 'de:08116:6759:0:4'  # eRtg Wolf-Hirth-Straße (Esslingen am Neckar Sirnau)
	KLINIKUM_ESSLINGEN = 'de:08116:6763:0:3'  # eRtg Hirschlandkopf (Esslingen am Neckar Oberesslingen)
	OBERESSL__KLINIKUM_ESSLINGEN = 'de:08116:6763:0:3'  # eRtg Hirschlandkopf (Esslingen am Neckar Oberesslingen)
	IN_DEN_HOFÄCKERN = 'de:08119:6765:0:3'  # eRtg Im Hummerholz (Leutenbach Weiler zum Stein)
	WEILER_Z__STEIN_I_D__HOFÄCKERN = 'de:08119:6765:0:3'  # eRtg Im Hummerholz (Leutenbach Weiler zum Stein)
	ASPERG_STADTMITTE = 'de:08118:6766'
	ASPERG_STADTMITTE_1 = 'de:08118:6766:1:3'  # eRtg Wilhelmsplatz (Asperg Asperg)
	ASPERG_STADTMITTE_2 = 'de:08118:6766:1:4'  # eRtg Weinstraße (Asperg Asperg)
	WEILER_ZUM_STEIN_RATHAUS = 'de:08119:6767'
	WEILER_ZUM_STEIN_RATHAUS_1 = 'de:08119:6767:1:3'  # eRtg Leutenbach (Leutenbach Weiler zum Stein)
	WEILER_ZUM_STEIN_RATHAUS_2 = 'de:08119:6767:1:4'  # eRtg Im Hummerholz (Leutenbach Weiler zum Stein)
	GOLLENHOFER_STRASSE = 'de:08119:6768:0:3'  # eRtg In den Hofäckern (Leutenbach Weiler zum Stein)
	WEILER_ZUM_STEIN_GOLLENH__STR = 'de:08119:6768:0:3'  # eRtg In den Hofäckern (Leutenbach Weiler zum Stein)
	WEILER_ZUM_STEIN_IM_HUMMERHOLZ = 'de:08119:6769:0:3'  # eRtg Rathaus (Leutenbach Weiler zum Stein)
	IM_HUMMERHOLZ = 'de:08119:6769:0:3'  # eRtg Rathaus (Leutenbach Weiler zum Stein)
	IM_HOLDER = 'de:08116:6770'
	IM_HOLDER_1 = 'de:08116:6770:0:3'  # Rtg Weilstr. (Esslingen am Neckar Pliensauvorstadt)
	IM_HOLDER_2 = 'de:08116:6770:0:4'  # Rtg Zollberg Ob. Eisbergweg (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUV__IM_HOLDER = 'de:08116:6770'
	PLIENSAUV__IM_HOLDER_1 = 'de:08116:6770:0:3'  # Rtg Weilstr. (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUV__IM_HOLDER_2 = 'de:08116:6770:0:4'  # Rtg Zollberg Ob. Eisbergweg (Esslingen am Neckar Pliensauvorstadt)
	HOCHDORF_ASPEN = 'de:08116:6772'
	HOCHDORF_ASPEN_1 = 'de:08116:6772:0:3'  # eRtg Hochdorfer Straße (Hochdorf Hochdorf (ES))
	HOCHDORF_ASPEN_2 = 'de:08116:6772:0:4'  # Rtg Talbachbrücke (Hochdorf Hochdorf (ES))
	HOLZMADEN_SEESTRASSE = 'de:08116:6773:0:3'  # eRtg Bahnhof/Weilheim (Holzmaden Holzmaden)
	NEUFFEN_SCHULE = 'de:08116:6775'
	NEUFFEN_SCHULE_1 = 'de:08116:6775:0:3'  # eRtg Lindenplatz/Kohlberg (Neuffen Neuffen)
	NEUFFEN_SCHULE_2 = 'de:08116:6775:0:4'  # eRtg Uracher Weg (Neuffen Neuffen)
	KLOSTERWEG = 'de:08116:6777:2:4'  # Ersatzhalt Li 151 Rtg Charlottenstr. (Köngen Köngen)
	KÖNGEN_KLOSTERWEG = 'de:08116:6777:2:4'  # Ersatzhalt Li 151 Rtg Charlottenstr. (Köngen Köngen)
	SCHULZENTRUM_AM_BERG = 'de:08116:6779:0:3'  # eRtg Schulzentrum/Stadtmitte (Wendlingen am Neckar Wendlingen (N))
	WENDL__N__SCHULZENTRUM_AM_BERG = 'de:08116:6779:0:3'  # eRtg Schulzentrum/Stadtmitte (Wendlingen am Neckar Wendlingen (N))
	PFAHLBRONN_MITTELWEILER = 'de:08119:6780'
	PFAHLBRONN_MITTELWEILER_1 = 'de:08119:6780:0:3'  # Rtg Buchengehren (Alfdorf Pfahlbronn)
	PFAHLBRONN_MITTELWEILER_2 = 'de:08119:6780:0:4'  # Rtg Wahlenheim (Alfdorf Pfahlbronn)
	MITTELWEILER = 'de:08119:6780'
	MITTELWEILER_1 = 'de:08119:6780:0:3'  # Rtg Buchengehren (Alfdorf Pfahlbronn)
	MITTELWEILER_2 = 'de:08119:6780:0:4'  # Rtg Wahlenheim (Alfdorf Pfahlbronn)
	PFAHLBRONN_BUCHENGEHREN = 'de:08119:6781'
	PFAHLBRONN_BUCHENGEHREN_1 = 'de:08119:6781:0:3'  # Rtg Buchengehren Sägmühle (Alfdorf Pfahlbronn)
	PFAHLBRONN_BUCHENGEHREN_2 = 'de:08119:6781:0:4'  # Rtg Mittelweiler (Alfdorf Pfahlbronn)
	BUCHENGEHREN = 'de:08119:6781'
	BUCHENGEHREN_1 = 'de:08119:6781:0:3'  # Rtg Buchengehren Sägmühle (Alfdorf Pfahlbronn)
	BUCHENGEHREN_2 = 'de:08119:6781:0:4'  # Rtg Mittelweiler (Alfdorf Pfahlbronn)
	BUCHENGEHREN_SÄGMÜHLE = 'de:08119:6782'
	BUCHENGEHREN_SÄGMÜHLE_1 = 'de:08119:6782:0:3'  # eRtg Abzweig Wahlenheim (Alfdorf Pfahlbronn)
	BUCHENGEHREN_SÄGMÜHLE_2 = 'de:08119:6782:0:4'  # Ersatzhaltestelle Rtg. Alfdorf Döllenstr (Alfdorf Pfahlbronn)
	PFAHLBR__BUCHENG__SÄGMÜHLE = 'de:08119:6782'
	PFAHLBR__BUCHENG__SÄGMÜHLE_1 = 'de:08119:6782:0:3'  # eRtg Abzweig Wahlenheim (Alfdorf Pfahlbronn)
	PFAHLBR__BUCHENG__SÄGMÜHLE_2 = 'de:08119:6782:0:4'  # Ersatzhaltestelle Rtg. Alfdorf Döllenstr (Alfdorf Pfahlbronn)
	HÖLDIS = 'de:08119:6784:0:3'  # eRtg Rienharz (Alfdorf Pfahlbronn)
	PFAHLBRONN_HÖLDIS = 'de:08119:6784:0:3'  # eRtg Rienharz (Alfdorf Pfahlbronn)
	RAPPACHSCHULE = 'de:08111:6792:0:4'  # Rtg Gerlinger Str. (Stuttgart Stuttgart)
	ERNST_REUTER_PLATZ = 'de:08111:6794:1:4'  # Rtg Rappachschule (Stuttgart Stuttgart)
	DÜRRBACH = 'de:08119:6796'
	DÜRRBACH_1 = 'de:08119:6796:0:3'  # eRtg Rathaus (Schorndorf Weiler (Schorndorf))
	DÜRRBACH_2 = 'de:08119:6796:0:4'  # eRtg Reithalle (Schorndorf Weiler (Schorndorf))
	WEILER_DÜRRBACH = 'de:08119:6796'
	WEILER_DÜRRBACH_1 = 'de:08119:6796:0:3'  # eRtg Rathaus (Schorndorf Weiler (Schorndorf))
	WEILER_DÜRRBACH_2 = 'de:08119:6796:0:4'  # eRtg Reithalle (Schorndorf Weiler (Schorndorf))
	VORDERST__WAHLENHEIM = 'de:08119:6797'
	VORDERST__WAHLENHEIM_1 = 'de:08119:6797:0:3'  # eRtg Greuthöfle (Alfdorf Vordersteinenberg)
	VORDERST__WAHLENHEIM_2 = 'de:08119:6797:0:4'  # Rtg Hüttenbacher See Abzw. (Alfdorf Vordersteinenberg)
	WAHLENHEIM = 'de:08119:6797'
	WAHLENHEIM_1 = 'de:08119:6797:0:3'  # eRtg Greuthöfle (Alfdorf Vordersteinenberg)
	WAHLENHEIM_2 = 'de:08119:6797:0:4'  # Rtg Hüttenbacher See Abzw. (Alfdorf Vordersteinenberg)
	GRONAU_PREVORST_LÖWEN = 'de:08118:6798'
	GRONAU_PREVORST_LÖWEN_1 = 'de:08118:6798:0:3'  # eRtg Krone (Oberstenfeld Gronau)
	GRONAU_PREVORST_LÖWEN_2 = 'de:08118:6798:0:4'  # eRtg Prevorst Feuerwehr (Oberstenfeld Gronau)
	PREVORST_LÖWEN = 'de:08118:6798'
	PREVORST_LÖWEN_1 = 'de:08118:6798:0:3'  # eRtg Krone (Oberstenfeld Gronau)
	PREVORST_LÖWEN_2 = 'de:08118:6798:0:4'  # eRtg Prevorst Feuerwehr (Oberstenfeld Gronau)
	OSSWEIL_MEHRZWECKHALLE = 'de:08118:6800:0:3'  # alle Richtungen (Ludwigsburg Oßweil)
	MEHRZWECKHALLE = 'de:08118:6800:0:3'  # alle Richtungen (Ludwigsburg Oßweil)
	KLEINHEPPACH_HECKBACHSTR = 'de:08119:6806'
	KLEINHEPPACH_HECKBACHSTR_1 = 'de:08119:6806:0:3'  # Rtg Großheppach Kath. Kirche (Korb Kleinheppach)
	KLEINHEPPACH_HECKBACHSTR_2 = 'de:08119:6806:0:4'  # Rtg Korber Straße (Korb Kleinheppach)
	HECKBACHSTRASSE = 'de:08119:6806'
	HECKBACHSTRASSE_1 = 'de:08119:6806:0:3'  # Rtg Großheppach Kath. Kirche (Korb Kleinheppach)
	HECKBACHSTRASSE_2 = 'de:08119:6806:0:4'  # Rtg Korber Straße (Korb Kleinheppach)
	KORNWESTHEIM_PFLUGFELDER_STR = 'de:08118:6807'
	KORNWESTHEIM_PFLUGFELDER_STR_1 = 'de:08118:6807:0:3'  # Rtg Ludwigsburg (Kornwestheim Kornwestheim)
	KORNWESTHEIM_PFLUGFELDER_STR_2 = 'de:08118:6807:0:4'  # Rtg Bahnhof (Kornwestheim Kornwestheim)
	KORNWESTHEIM_MAX_PLANCK_STR = 'de:08118:6809'
	KORNWESTHEIM_MAX_PLANCK_STR_1 = 'de:08118:6809:0:3'  # Rtg Kornwestheimer Str. (Kornwestheim Kornwestheim)
	KORNWESTHEIM_MAX_PLANCK_STR_2 = 'de:08118:6809:0:4'  # Rtg Bolzstr. (Kornwestheim Kornwestheim)
	KORNWESTHEIM_FREIZEITPARK = 'de:08118:6810'
	KORNWESTHEIM_FREIZEITPARK_1 = 'de:08118:6810:0:3'  # eRtg Talstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_FREIZEITPARK_2 = 'de:08118:6810:0:4'  # eRtg Realschule Remseck (Kornwestheim Kornwestheim)
	FREIZEITPARK = 'de:08118:6810'
	FREIZEITPARK_1 = 'de:08118:6810:0:3'  # eRtg Talstraße (Kornwestheim Kornwestheim)
	FREIZEITPARK_2 = 'de:08118:6810:0:4'  # eRtg Realschule Remseck (Kornwestheim Kornwestheim)
	GEMMRIGHEIM_AMSELWEG = 'de:08118:6811:0:3'  # eRtg Kurze Straße (Gemmrigheim Gemmrigheim)
	KURZE_STRASSE = 'de:08118:6812'
	KURZE_STRASSE_1 = 'de:08118:6812:0:3'  # Rtg Niedere Klinge (Gemmrigheim Gemmrigheim)
	KURZE_STRASSE_2 = 'de:08118:6812:0:4'  # Rtg Papierfabrik/Fährergasse/Walheim (Gemmrigheim Gemmrigheim)
	GEMMRIGHEIM_KURZE_STRASSE = 'de:08118:6812'
	GEMMRIGHEIM_KURZE_STRASSE_1 = 'de:08118:6812:0:3'  # Rtg Niedere Klinge (Gemmrigheim Gemmrigheim)
	GEMMRIGHEIM_KURZE_STRASSE_2 = 'de:08118:6812:0:4'  # Rtg Papierfabrik/Fährergasse/Walheim (Gemmrigheim Gemmrigheim)
	ADLERWEG = 'de:08118:6813:0:3'  # eRtg Amselweg (Gemmrigheim Gemmrigheim)
	GEMMRIGHEIM_ADLERWEG = 'de:08118:6813:0:3'  # eRtg Amselweg (Gemmrigheim Gemmrigheim)
	HAYDNSTRASSE = 'de:08115:6814:0:3'  # Rtg Varnbülerstr. (Leonberg Höfingen)
	HÖFINGEN_HAYDNSTRASSE = 'de:08115:6814:0:3'  # Rtg Varnbülerstr. (Leonberg Höfingen)
	VARNBÜLERSTRASSE = 'de:08115:6815:0:3'  # Rtg Hirschlander Str. (Leonberg Höfingen)
	HÖFINGEN_VARNBÜLERSTRASSE = 'de:08115:6815:0:3'  # Rtg Hirschlander Str. (Leonberg Höfingen)
	HÖFINGEN_A__SCHWEITZER_STR = 'de:08115:6816'
	HÖFINGEN_A__SCHWEITZER_STR_1 = 'de:08115:6816:0:3'  # Rtg Brandströmstr. (Leonberg Höfingen)
	HÖFINGEN_A__SCHWEITZER_STR_2 = 'de:08115:6816:0:4'  # Rtg Lachentorstr. (Leonberg Höfingen)
	HÖFINGEN_ELSA_BRÄNDSTRÖM_STR = 'de:08115:6817:0:3'  # Endstation/Rtg Lachentor/Mercedesstr. (Leonberg Höfingen)
	NECKARWEIH__PETER_HEBEL_STR = 'de:08118:6818'
	NECKARWEIH__PETER_HEBEL_STR_1 = 'de:08118:6818:0:3'  # eRtg Lechtstraße (Ludwigsburg Neckarweihingen)
	NECKARWEIH__PETER_HEBEL_STR_2 = 'de:08118:6818:0:4'  # eRtg Burghaldenstraße (Ludwigsburg Neckarweihingen)
	PETER_HEBEL_STRASSE = 'de:08118:6818'
	PETER_HEBEL_STRASSE_1 = 'de:08118:6818:0:3'  # eRtg Lechtstraße (Ludwigsburg Neckarweihingen)
	PETER_HEBEL_STRASSE_2 = 'de:08118:6818:0:4'  # eRtg Burghaldenstraße (Ludwigsburg Neckarweihingen)
	POPPENWEILER_HOCHBERGER_STR = 'de:08118:6820'
	POPPENWEILER_HOCHBERGER_STR_1 = 'de:08118:6820:0:3'  # Rtg Neckarweihingen (Ludwigsburg Poppenweiler)
	POPPENWEILER_HOCHBERGER_STR_2 = 'de:08118:6820:0:4'  # Rtg Hochberg / Neckargröningen (Ludwigsburg Poppenweiler)
	HOCHDORF_WILHELMSPLATZ = 'de:08118:6821'
	HOCHDORF_WILHELMSPLATZ_1 = 'de:08118:6821:0:3'  # eRtg Poppenweiler Straße (Remseck am Neckar Hochdorf (Remseck))
	HOCHDORF_WILHELMSPLATZ_2 = 'de:08118:6821:0:4'  # eRtg Hochberger Straße (Remseck am Neckar Hochdorf (Remseck))
	HOCHBERG_LINDENSTRASSE = 'de:08118:6822'
	HOCHBERG_LINDENSTRASSE_1 = 'de:08118:6822:0:3'  # eRtg Friedhof (Remseck am Neckar Hochberg)
	HOCHBERG_LINDENSTRASSE_2 = 'de:08118:6822:0:4'  # eRtg Alexandrinenplatz (Remseck am Neckar Hochberg)
	HOCHBERGER_WALD = 'de:08118:6823'
	HOCHBERGER_WALD_1 = 'de:08118:6823:0:3'  # eRtg Waldallee (Remseck am Neckar Hochberg)
	HOCHBERGER_WALD_2 = 'de:08118:6823:0:4'  # Ankunft (Remseck am Neckar Hochberg)
	HOCHBERG_HOCHBERGER_WALD = 'de:08118:6823'
	HOCHBERG_HOCHBERGER_WALD_1 = 'de:08118:6823:0:3'  # eRtg Waldallee (Remseck am Neckar Hochberg)
	HOCHBERG_HOCHBERGER_WALD_2 = 'de:08118:6823:0:4'  # Ankunft (Remseck am Neckar Hochberg)
	WALDALLEE = 'de:08118:6824'
	WALDALLEE_1 = 'de:08118:6824:0:3'  # eRtg Rotweg (Remseck am Neckar Hochberg)
	WALDALLEE_2 = 'de:08118:6824:0:4'  # eRtg Hochberger Wald (Remseck am Neckar Hochberg)
	HOCHBERG_WALDALLEE = 'de:08118:6824'
	HOCHBERG_WALDALLEE_1 = 'de:08118:6824:0:3'  # eRtg Rotweg (Remseck am Neckar Hochberg)
	HOCHBERG_WALDALLEE_2 = 'de:08118:6824:0:4'  # eRtg Hochberger Wald (Remseck am Neckar Hochberg)
	NECKARREMS_FRIEDHOF = 'de:08118:6825'
	NECKARREMS_FRIEDHOF_1 = 'de:08118:6825:0:3'  # eRtg Neckarhalde (Remseck am Neckar Neckarrems)
	NECKARREMS_FRIEDHOF_2 = 'de:08118:6825:0:4'  # eRtg Lindenstraße (Remseck am Neckar Neckarrems)
	NECKARGRÖNINGEN_RAINWIESEN = 'de:08118:6826:0:3'  # alle Rtg (Remseck am Neckar Neckargröningen)
	RAINWIESEN = 'de:08118:6826:0:3'  # alle Rtg (Remseck am Neckar Neckargröningen)
	NECKARGRÖNINGEN_HAUS_D__JUGEND = 'de:08118:6827'
	NECKARGRÖNINGEN_HAUS_D__JUGEND_1 = 'de:08118:6827:0:3'  # eRtg Remseck (Remseck am Neckar Neckargröningen)
	NECKARGRÖNINGEN_HAUS_D__JUGEND_2 = 'de:08118:6827:0:4'  # eRtg Bildungszentrum (Remseck am Neckar Neckargröningen)
	HAUS_DER_JUGEND = 'de:08118:6827'
	HAUS_DER_JUGEND_1 = 'de:08118:6827:0:3'  # eRtg Remseck (Remseck am Neckar Neckargröningen)
	HAUS_DER_JUGEND_2 = 'de:08118:6827:0:4'  # eRtg Bildungszentrum (Remseck am Neckar Neckargröningen)
	KALTENTHALSTRASSE = 'de:08118:6828'
	KALTENTHALSTRASSE_1 = 'de:08118:6828:0:3'  # eRtg Remseck (Remseck am Neckar Aldingen)
	KALTENTHALSTRASSE_2 = 'de:08118:6828:0:4'  # Rtg Oßweiler Weg (Remseck am Neckar Aldingen)
	ALDINGEN_KALTENTHALSTR = 'de:08118:6828'
	ALDINGEN_KALTENTHALSTR_1 = 'de:08118:6828:0:3'  # eRtg Remseck (Remseck am Neckar Aldingen)
	ALDINGEN_KALTENTHALSTR_2 = 'de:08118:6828:0:4'  # Rtg Oßweiler Weg (Remseck am Neckar Aldingen)
	ALDINGEN_GOETHESTRASSE = 'de:08118:6829'
	ALDINGEN_GOETHESTRASSE_1 = 'de:08118:6829:0:3'  # eRtg Mühlweg (Remseck am Neckar Aldingen)
	ALDINGEN_GOETHESTRASSE_2 = 'de:08118:6829:0:4'  # eRtg Berliner Straße (Remseck am Neckar Aldingen)
	HOCHDORF__E__KELTENMUSEUM = 'de:08118:6830'
	HOCHDORF__E__KELTENMUSEUM_1 = 'de:08118:6830:0:3'  # vor Museum (Eberdingen Hochdorf (Eberdingen))
	HOCHDORF__E__KELTENMUSEUM_2 = 'de:08118:6830:0:4'  # Rtg Pulverdingen Ort (Eberdingen Hochdorf (Eberdingen))
	HOCHDORF__E__KELTENMUSEUM_3 = 'de:08118:6830:0:5'  # Rtg Hochdorf Mitte (Eberdingen Hochdorf (Eberdingen))
	KELTENMUSEUM = 'de:08118:6830'
	KELTENMUSEUM_1 = 'de:08118:6830:0:3'  # vor Museum (Eberdingen Hochdorf (Eberdingen))
	KELTENMUSEUM_2 = 'de:08118:6830:0:4'  # Rtg Pulverdingen Ort (Eberdingen Hochdorf (Eberdingen))
	KELTENMUSEUM_3 = 'de:08118:6830:0:5'  # Rtg Hochdorf Mitte (Eberdingen Hochdorf (Eberdingen))
	HOCHDORF__E__RIETER_STR = 'de:08118:6831'
	HOCHDORF__E__RIETER_STR_1 = 'de:08118:6831:0:3'  # Rtg Riet Dorfplatz (Eberdingen Hochdorf (Eberdingen))
	HOCHDORF__E__RIETER_STR_2 = 'de:08118:6831:0:4'  # Rtg Rathaus (Eberdingen Hochdorf (Eberdingen))
	RIETER_STRASSE = 'de:08118:6831'
	RIETER_STRASSE_1 = 'de:08118:6831:0:3'  # Rtg Riet Dorfplatz (Eberdingen Hochdorf (Eberdingen))
	RIETER_STRASSE_2 = 'de:08118:6831:0:4'  # Rtg Rathaus (Eberdingen Hochdorf (Eberdingen))
	ALDINGEN_MÜHLWEG = 'de:08118:6832'
	ALDINGEN_MÜHLWEG_1 = 'de:08118:6832:1:3'  # eRtg Schlosshof (Remseck am Neckar Aldingen)
	ALDINGEN_MÜHLWEG_2 = 'de:08118:6832:1:4'  # eRtg Goethestraße (Remseck am Neckar Aldingen)
	HIRSCHSTRASSE_1 = 'de:08118:6833:0:4'  # eRtg Wettemarkt (Ludwigsburg Oßweil)
	HIRSCHSTRASSE_2 = 'de:08116:7178:1:4'  # Rtg Echterdingen Bf (Leinfelden-Echterdingen Echterdingen)
	HIRSCHSTRASSE_3 = 'de:08116:7178:2:3'  # Rtg Waldhornstr. (Leinfelden-Echterdingen Echterdingen)
	HIRSCHSTRASSE_4 = 'de:08117:7009:0:H1'  # Rtg Sauerbrunnen (Göppingen Faurndau)
	HIRSCHSTRASSE_5 = 'de:08117:7009:0:R1'  # Rtg Marienstr. (Göppingen Faurndau)
	OSSWEIL_HIRSCHSTRASSE = 'de:08118:6833:0:4'  # eRtg Wettemarkt (Ludwigsburg Oßweil)
	KORNWESTHEIM_TALSTRASSE = 'de:08118:6834'
	KORNWESTHEIM_TALSTRASSE_1 = 'de:08118:6834:1:3'  # eRtg Aldinger Straße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_TALSTRASSE_2 = 'de:08118:6834:1:4'  # eRtg Enzstraße (Kornwestheim Kornwestheim)
	KORNWESTHEIM_LENZHALDE = 'de:08118:6835'
	KORNWESTHEIM_LENZHALDE_1 = 'de:08118:6835:0:3'  # Rtg Aldinger Str. (Kornwestheim Kornwestheim)
	KORNWESTHEIM_LENZHALDE_2 = 'de:08118:6835:0:4'  # Rtg Friedhof/Pfarrer (Kornwestheim Kornwestheim)
	STROMBERGGYMNASIUM = 'de:08118:6836'
	STROMBERGGYMNASIUM_1 = 'de:08118:6836:0:3'  # eRtg Stadthalle (Vaihingen an der Enz Vaihingen (E))
	STROMBERGGYMNASIUM_2 = 'de:08118:6836:0:4'  # eRtg Neue Bahnhofstraße (Vaihingen an der Enz Vaihingen (E))
	VAIH__E__STROMBERGGYMNASIUM = 'de:08118:6836'
	VAIH__E__STROMBERGGYMNASIUM_1 = 'de:08118:6836:0:3'  # eRtg Stadthalle (Vaihingen an der Enz Vaihingen (E))
	VAIH__E__STROMBERGGYMNASIUM_2 = 'de:08118:6836:0:4'  # eRtg Neue Bahnhofstraße (Vaihingen an der Enz Vaihingen (E))
	ENDERSBACH_BAHNHOFSTR = 'de:08119:6837'
	ENDERSBACH_BAHNHOFSTR_1 = 'de:08119:6837:0:3'  # Rtg Endersbach Bf (Weinstadt Endersbach)
	ENDERSBACH_BAHNHOFSTR_2 = 'de:08119:6837:0:4'  # Rtg Beim Kreisel/Viadukt (Weinstadt Endersbach)
	BEIM_KREISEL = 'de:08119:6838'
	BEIM_KREISEL_1 = 'de:08119:6838:0:3'  # Rtg Viadukt/Bf Endersbach/Kalko (Weinstadt Endersbach)
	BEIM_KREISEL_2 = 'de:08119:6838:0:4'  # Rtg Poststr. (Weinstadt Endersbach)
	ENDERSBACH_BEIM_KREISEL = 'de:08119:6838'
	ENDERSBACH_BEIM_KREISEL_1 = 'de:08119:6838:0:3'  # Rtg Viadukt/Bf Endersbach/Kalko (Weinstadt Endersbach)
	ENDERSBACH_BEIM_KREISEL_2 = 'de:08119:6838:0:4'  # Rtg Poststr. (Weinstadt Endersbach)
	KALKOFEN = 'de:08119:6839'
	KALKOFEN_1 = 'de:08119:6839:0:3'  # Rtg Beim Kreisel (Weinstadt Endersbach)
	KALKOFEN_2 = 'de:08119:6839:0:4'  # Rtg Beinst. (Endersb- Str.)Ortsmitte (Weinstadt Endersbach)
	ENDERSBACH_KALKOFEN = 'de:08119:6839'
	ENDERSBACH_KALKOFEN_1 = 'de:08119:6839:0:3'  # Rtg Beim Kreisel (Weinstadt Endersbach)
	ENDERSBACH_KALKOFEN_2 = 'de:08119:6839:0:4'  # Rtg Beinst. (Endersb- Str.)Ortsmitte (Weinstadt Endersbach)
	BÖNNIGHEIM_SCHLOSSFELD = 'de:08118:6840:0:3'  # eRtg Aussiedlerhöfe (Bönnigheim Bönnigheim)
	SCHLOSSFELD = 'de:08118:6840:0:3'  # eRtg Aussiedlerhöfe (Bönnigheim Bönnigheim)
	WEIL_WÜRTTEMBERGSTR = 'de:08116:6841:0:3'  # Rtg Mettingen / Hedelfingen (Esslingen am Neckar Weil)
	PALMENWALDSTRASSE = 'de:08116:6842:0:3'  # Alle Richtungen (Esslingen am Neckar Brühl)
	BRÜHL_PALMENWALDSTR = 'de:08116:6842:0:3'  # Alle Richtungen (Esslingen am Neckar Brühl)
	KLEINASPACH_SONNENHOF = 'de:08119:6843'
	KLEINASPACH_SONNENHOF_1 = 'de:08119:6843:0:3'  # eRtg Sinzenburg (Aspach Kleinaspach)
	KLEINASPACH_SONNENHOF_2 = 'de:08119:6843:0:4'  # eRtg Ortsmitte (Aspach Kleinaspach)
	SONNENHOF = 'de:08119:6843'
	SONNENHOF_1 = 'de:08119:6843:0:3'  # eRtg Sinzenburg (Aspach Kleinaspach)
	SONNENHOF_2 = 'de:08119:6843:0:4'  # eRtg Ortsmitte (Aspach Kleinaspach)
	STEINBACH_NEUE_STRASSE = 'de:08119:6844'
	STEINBACH_NEUE_STRASSE_1 = 'de:08119:6844:0:3'  # eRtg Ankunft/Sachsenweiler (Backnang Steinbach)
	STEINBACH_NEUE_STRASSE_2 = 'de:08119:6844:0:4'  # eRtg Lindenplatz (Backnang Steinbach)
	TUXER_WEG = 'de:08119:6845'
	TUXER_WEG_1 = 'de:08119:6845:0:3'  # eRtg Züricher Straße (Backnang Maubach)
	TUXER_WEG_2 = 'de:08119:6845:0:4'  # eRtg Salzburger Straße (Backnang Maubach)
	MAUBACH_TUXER_WEG = 'de:08119:6845'
	MAUBACH_TUXER_WEG_1 = 'de:08119:6845:0:3'  # eRtg Züricher Straße (Backnang Maubach)
	MAUBACH_TUXER_WEG_2 = 'de:08119:6845:0:4'  # eRtg Salzburger Straße (Backnang Maubach)
	GROSSBETTLINGEN_MÖRIKESTRASSE = 'de:08116:6846'
	GROSSBETTLINGEN_MÖRIKESTRASSE_1 = 'de:08116:6846:0:3'  # eRtg Bettlinger Forum (Großbettlingen Großbettlingen)
	GROSSBETTLINGEN_MÖRIKESTRASSE_2 = 'de:08116:6846:0:4'  # Rtg Albstraße (Großbettlingen Großbettlingen)
	NECKARWEIH__OTTO_HAHN_STR = 'de:08118:6847'
	NECKARWEIH__OTTO_HAHN_STR_1 = 'de:08118:6847:0:3'  # eRtg Hermann-Hesse-Straße (Ludwigsburg Neckarweihingen)
	NECKARWEIH__OTTO_HAHN_STR_2 = 'de:08118:6847:0:4'  # eRtg Neckarbrücke (Ludwigsburg Neckarweihingen)
	NECKARWEIH__H__HESSE_STR = 'de:08118:6848:0:4'  # eRtg Otto-Hahn-Straße (Ludwigsburg Neckarweihingen)
	ROSSWAG_SEEMÜHLE = 'de:08118:6849'
	ROSSWAG_SEEMÜHLE_1 = 'de:08118:6849:0:3'  # eRtg Roßwag Sporthalle (Vaihingen an der Enz Roßwag)
	ROSSWAG_SEEMÜHLE_2 = 'de:08118:6849:0:4'  # eRtg Vaihingen (E) (Vaihingen an der Enz Roßwag)
	ROSSWAG_MÜHLHÄUSER_STRASSE = 'de:08118:6850:0:3'  # Rtg Sporthalle (Vaihingen an der Enz Roßwag)
	HAUS_AM_REMSUFER = 'de:08118:6852'
	HAUS_AM_REMSUFER_1 = 'de:08118:6852:0:3'  # eRtg Rathaus (Remseck am Neckar Neckarrems)
	HAUS_AM_REMSUFER_2 = 'de:08118:6852:0:4'  # eRtg Dorfstraße (Remseck am Neckar Neckarrems)
	NECKARREMS_HAUS_AM_REMSUFER = 'de:08118:6852'
	NECKARREMS_HAUS_AM_REMSUFER_1 = 'de:08118:6852:0:3'  # eRtg Rathaus (Remseck am Neckar Neckarrems)
	NECKARREMS_HAUS_AM_REMSUFER_2 = 'de:08118:6852:0:4'  # eRtg Dorfstraße (Remseck am Neckar Neckarrems)
	NECKARREMS_STEIGE = 'de:08118:6853:0:3'  # eRtg Traubenstraße (Remseck am Neckar Neckarrems)
	TRAUBENSTRASSE = 'de:08118:6854:0:3'  # eRtg Neckarhalde (Remseck am Neckar Neckarrems)
	NECKARREMS_TRAUBENSTRASSE = 'de:08118:6854:0:3'  # eRtg Neckarhalde (Remseck am Neckar Neckarrems)
	BÖNNIGHEIM_SCHLOSSBERGALLEE = 'de:08118:6855:0:3'  # eRtg Hofener Straße (Bönnigheim Bönnigheim)
	SCHLOSSBERGALLEE = 'de:08118:6855:0:3'  # eRtg Hofener Straße (Bönnigheim Bönnigheim)
	RÖMERWEG = 'de:08119:6856:0:3'  # Rtg Rathaus (Kirchberg an der Murr Kirchberg (M))
	KIRCHBERG__M__RÖMERWEG = 'de:08119:6856:0:3'  # Rtg Rathaus (Kirchberg an der Murr Kirchberg (M))
	KIRCHBERG__M__PFARRGARTENSTR = 'de:08119:6858:0:3'  # Rtg Kindergarten (Kirchberg an der Murr Kirchberg (M))
	PFARRGARTENSTRASSE = 'de:08119:6858:0:3'  # Rtg Kindergarten (Kirchberg an der Murr Kirchberg (M))
	KIRCHBERG__M__DANZIGER_STR = 'de:08119:6859:0:3'  # Rtg Zwingelh. Str. (Kirchberg an der Murr Kirchberg (M))
	NECKARTAILF__NECKARBRÜCKE = 'de:08116:6860'
	NECKARTAILF__NECKARBRÜCKE_1 = 'de:08116:6860:0:3'  # eRtg Tübinger Straße (Neckartailfingen Neckartailfingen)
	NECKARTAILF__NECKARBRÜCKE_2 = 'de:08116:6860:0:4'  # eRtg Schule (Neckartailfingen Neckartailfingen)
	KIRCHB__M__ZWINGELHÄUSER_STR = 'de:08119:6861'
	KIRCHB__M__ZWINGELHÄUSER_STR_1 = 'de:08119:6861:0:3'  # eRtg Rathaus (Kirchberg an der Murr Kirchberg (M))
	KIRCHB__M__ZWINGELHÄUSER_STR_2 = 'de:08119:6861:0:4'  # eRtg Abzw. Obertorhöfe (Kirchberg an der Murr Kirchberg (M))
	ZWINGELHÄUSER_STRASSE = 'de:08119:6861'
	ZWINGELHÄUSER_STRASSE_1 = 'de:08119:6861:0:3'  # eRtg Rathaus (Kirchberg an der Murr Kirchberg (M))
	ZWINGELHÄUSER_STRASSE_2 = 'de:08119:6861:0:4'  # eRtg Abzw. Obertorhöfe (Kirchberg an der Murr Kirchberg (M))
	WENDLINGEN__N__AUSTRASSE = 'de:08116:6864:0:3'  # Rtg Friedhof (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N__FRIEDHOF = 'de:08116:6865:0:3'  # Rtg Wilhelm-Raabe-Str. (Wendlingen am Neckar Wendlingen (N))
	KIRCHBERG__M__SCHULSTRASSE = 'de:08119:6873:0:3'  # eRtg Hauptschule (Kirchberg an der Murr Kirchberg (M))
	ALLMERSBACH_I_T__WASENFELD = 'de:08119:6874'
	ALLMERSBACH_I_T__WASENFELD_1 = 'de:08119:6874:0:3'  # eRtg Ortsmitte (Allmersbach im Tal Allmersbach im Tal)
	ALLMERSBACH_I_T__WASENFELD_2 = 'de:08119:6874:0:4'  # eRtg Kreuzung (Allmersbach im Tal Allmersbach im Tal)
	WASENFELD = 'de:08119:6874'
	WASENFELD_1 = 'de:08119:6874:0:3'  # eRtg Ortsmitte (Allmersbach im Tal Allmersbach im Tal)
	WASENFELD_2 = 'de:08119:6874:0:4'  # eRtg Kreuzung (Allmersbach im Tal Allmersbach im Tal)
	DEIZISAU_OLGA__ZEPPELINSTR = 'de:08116:6875'
	DEIZISAU_OLGA__ZEPPELINSTR_1 = 'de:08116:6875:0:3'  # Rtg Kirchstr. (Deizisau Deizisau)
	DEIZISAU_OLGA__ZEPPELINSTR_2 = 'de:08116:6875:0:4'  # Rtg Zeppelinstr. (Deizisau Deizisau)
	OLGA__ZEPPELINSTRASSE = 'de:08116:6875'
	OLGA__ZEPPELINSTRASSE_1 = 'de:08116:6875:0:3'  # Rtg Kirchstr. (Deizisau Deizisau)
	OLGA__ZEPPELINSTRASSE_2 = 'de:08116:6875:0:4'  # Rtg Zeppelinstr. (Deizisau Deizisau)
	ZEPPELIN__KIRCHSTRASSE = 'de:08116:6876:0:3'  # eRtg Kirchstraße/Nelkenweg (Deizisau Deizisau)
	DEIZISAU_ZEPPELIN__KIRCHSTR = 'de:08116:6876:0:3'  # eRtg Kirchstraße/Nelkenweg (Deizisau Deizisau)
	DEIZISAU_KIRCHSTR__NELKENWEG = 'de:08116:6877:0:3'  # eRtg Schule (Deizisau Deizisau)
	KIRCHSTRASSE_NELKENWEG = 'de:08116:6877:0:3'  # eRtg Schule (Deizisau Deizisau)
	DEIZISAU_ZEPPELINSTR = 'de:08116:6878:0:3'  # eRtg Plochingen (Deizisau Deizisau)
	ALTE_BERGSTRASSE = 'de:08116:6879'
	ALTE_BERGSTRASSE_1 = 'de:08116:6879:0:3'  # Rtg Rathaus (Deizisau Deizisau)
	ALTE_BERGSTRASSE_2 = 'de:08116:6879:0:4'  # Rtg Wert (Deizisau Deizisau)
	DEIZISAU_ALTE_BERGSTR = 'de:08116:6879'
	DEIZISAU_ALTE_BERGSTR_1 = 'de:08116:6879:0:3'  # Rtg Rathaus (Deizisau Deizisau)
	DEIZISAU_ALTE_BERGSTR_2 = 'de:08116:6879:0:4'  # Rtg Wert (Deizisau Deizisau)
	OLGA__KEPLERSTRASSE = 'de:08116:6880:0:3'  # eRtg Olga-/Zeppelinstraße (Deizisau Deizisau)
	DEIZISAU_OLGA__KEPLERSTR = 'de:08116:6880:0:3'  # eRtg Olga-/Zeppelinstraße (Deizisau Deizisau)
	OSSWEIL_WALTER_FLEX_STR = 'de:08118:6882'
	OSSWEIL_WALTER_FLEX_STR_1 = 'de:08118:6882:0:3'  # Rtg Neckarweihinger Str. (Ludwigsburg Oßweil)
	OSSWEIL_WALTER_FLEX_STR_2 = 'de:08118:6882:0:4'  # Rtg Comburgstr. (Ludwigsburg Oßweil)
	WALTER_FLEX_STRASSE = 'de:08118:6882'
	WALTER_FLEX_STRASSE_1 = 'de:08118:6882:0:3'  # Rtg Neckarweihinger Str. (Ludwigsburg Oßweil)
	WALTER_FLEX_STRASSE_2 = 'de:08118:6882:0:4'  # Rtg Comburgstr. (Ludwigsburg Oßweil)
	NECKARWEIHINGER_STRASSE = 'de:08118:6883'
	NECKARWEIHINGER_STRASSE_1 = 'de:08118:6883:0:3'  # Rtg Westfalenstr. (Ludwigsburg Oßweil)
	NECKARWEIHINGER_STRASSE_2 = 'de:08118:6883:0:4'  # Rtg Walter-Flex-Str. (Ludwigsburg Oßweil)
	OSSWEIL_NECKARWEIHINGER_STR = 'de:08118:6883'
	OSSWEIL_NECKARWEIHINGER_STR_1 = 'de:08118:6883:0:3'  # Rtg Westfalenstr. (Ludwigsburg Oßweil)
	OSSWEIL_NECKARWEIHINGER_STR_2 = 'de:08118:6883:0:4'  # Rtg Walter-Flex-Str. (Ludwigsburg Oßweil)
	ALLEE_BRÄCHTERSTRASSE = 'de:08118:6885'
	ALLEE_BRÄCHTERSTRASSE_1 = 'de:08118:6885:0:3'  # eRtg Allee/Alter Weg (Tamm Tamm)
	ALLEE_BRÄCHTERSTRASSE_2 = 'de:08118:6885:0:4'  # eRtg Hohenstange Stuttgarter Straße (Tamm Tamm)
	TAMM_ALLEE_BRÄCHTERSTR = 'de:08118:6885'
	TAMM_ALLEE_BRÄCHTERSTR_1 = 'de:08118:6885:0:3'  # eRtg Allee/Alter Weg (Tamm Tamm)
	TAMM_ALLEE_BRÄCHTERSTR_2 = 'de:08118:6885:0:4'  # eRtg Hohenstange Stuttgarter Straße (Tamm Tamm)
	TAMM_ALLEE_ALTER_WEG = 'de:08118:6886'
	TAMM_ALLEE_ALTER_WEG_1 = 'de:08118:6886:0:3'  # eRtg Alter Weg (Tamm Tamm)
	TAMM_ALLEE_ALTER_WEG_2 = 'de:08118:6886:0:4'  # eRtg Allee/Brächterstraße (Tamm Tamm)
	ALLEE_ALTER_WEG = 'de:08118:6886'
	ALLEE_ALTER_WEG_1 = 'de:08118:6886:0:3'  # eRtg Alter Weg (Tamm Tamm)
	ALLEE_ALTER_WEG_2 = 'de:08118:6886:0:4'  # eRtg Allee/Brächterstraße (Tamm Tamm)
	TAMM_ALTER_WEG = 'de:08118:6887'
	TAMM_ALTER_WEG_1 = 'de:08118:6887:1:3'  # eRtg Ludwigsburger Straße (Tamm Tamm)
	TAMM_ALTER_WEG_2 = 'de:08118:6887:1:4'  # eRtg Allee/Alter Weg (Tamm Tamm)
	ALTER_WEG = 'de:08118:6887'
	ALTER_WEG_1 = 'de:08118:6887:1:3'  # eRtg Ludwigsburger Straße (Tamm Tamm)
	ALTER_WEG_2 = 'de:08118:6887:1:4'  # eRtg Allee/Alter Weg (Tamm Tamm)
	MARKGR__MITTE_HELENENSTR = 'de:08118:6888:0:3'  # eRtg Münchinger Straße (Markgröningen Markgröningen)
	MITTE_HELENENSTRASSE = 'de:08118:6888:0:3'  # eRtg Münchinger Straße (Markgröningen Markgröningen)
	HASLACH_AUSSIEDLERHÖFE = 'de:08115:6889'
	HASLACH_AUSSIEDLERHÖFE_1 = 'de:08115:6889:0:3'  # Rtg Haslach Randenstraße (Herrenberg Haslach)
	HASLACH_AUSSIEDLERHÖFE_2 = 'de:08115:6889:0:4'  # Rtg Schulmeisterbuche (Herrenberg Haslach)
	GEMMRIGHEIM_SPORTPLATZ = 'de:08118:6890'
	GEMMRIGHEIM_SPORTPLATZ_1 = 'de:08118:6890:0:3'  # Rtg Kirchheim (Gemmrigheim Gemmrigheim)
	GEMMRIGHEIM_SPORTPLATZ_2 = 'de:08118:6890:0:4'  # Rtg Gemmrigheim (Gemmrigheim Gemmrigheim)
	TAILFINGER_STRASSE = 'de:08115:6892:0:3'  # eRtg Bondorf/Weildorf (Gäufelden Öschelbronn (Gäufelden))
	ÖSCHELBRONN_TAILFINGER_STR = 'de:08115:6892:0:3'  # eRtg Bondorf/Weildorf (Gäufelden Öschelbronn (Gäufelden))
	BRETZENACKER_RATHAUS = 'de:08119:6893:0:3'  # alle Richtungen (Berglen Bretzenacker)
	GERLINGEN_HERMANN_LÖNS_STR = 'de:08118:6895:0:3'  # eRtg Golfplatz (Gerlingen Gerlingen)
	MARGARETENWEG = 'de:08118:6897'
	MARGARETENWEG_1 = 'de:08118:6897:0:3'  # Rtg Lukas Kirche (Gerlingen Gerlingen)
	MARGARETENWEG_2 = 'de:08118:6897:0:4'  # Rtg Einkaufszentrum (Gerlingen Gerlingen)
	GERLINGEN_MARGARETENWEG = 'de:08118:6897'
	GERLINGEN_MARGARETENWEG_1 = 'de:08118:6897:0:3'  # Rtg Lukas Kirche (Gerlingen Gerlingen)
	GERLINGEN_MARGARETENWEG_2 = 'de:08118:6897:0:4'  # Rtg Einkaufszentrum (Gerlingen Gerlingen)
	GERLINGEN_ROSENSTR = 'de:08118:6903'
	GERLINGEN_ROSENSTR_1 = 'de:08118:6903:0:3'  # eRtg Vesouler Straße (Gerlingen Gerlingen)
	GERLINGEN_ROSENSTR_2 = 'de:08118:6903:0:4'  # eRtg Leonberger Straße (Gerlingen Gerlingen)
	GERLINGEN_RICHTHOFENSTRASSE = 'de:08118:6904'
	GERLINGEN_RICHTHOFENSTRASSE_1 = 'de:08118:6904:0:3'  # eRtg Falkenweg (Gerlingen Gerlingen)
	GERLINGEN_RICHTHOFENSTRASSE_2 = 'de:08118:6904:0:4'  # eRtg Vesouler Straße (Gerlingen Gerlingen)
	RICHTHOFENSTRASSE = 'de:08118:6904'
	RICHTHOFENSTRASSE_1 = 'de:08118:6904:0:3'  # eRtg Falkenweg (Gerlingen Gerlingen)
	RICHTHOFENSTRASSE_2 = 'de:08118:6904:0:4'  # eRtg Vesouler Straße (Gerlingen Gerlingen)
	VESOULER_STRASSE = 'de:08118:6905'
	VESOULER_STRASSE_1 = 'de:08118:6905:0:3'  # eRtg Richthofenstraße (Gerlingen Gerlingen)
	VESOULER_STRASSE_2 = 'de:08118:6905:0:4'  # eRtg Rosenstraße (Gerlingen Gerlingen)
	GERLINGEN_VESOULER_STR = 'de:08118:6905'
	GERLINGEN_VESOULER_STR_1 = 'de:08118:6905:0:3'  # eRtg Richthofenstraße (Gerlingen Gerlingen)
	GERLINGEN_VESOULER_STR_2 = 'de:08118:6905:0:4'  # eRtg Rosenstraße (Gerlingen Gerlingen)
	ROBERT_BOSCH_PLATZ = 'de:08118:6906'
	ROBERT_BOSCH_PLATZ_1 = 'de:08118:6906:0:3'  # eRtg Waldfriedhof (Gerlingen Gerlingen)
	ROBERT_BOSCH_PLATZ_2 = 'de:08118:6906:0:4'  # eRtg Schillerhöhe Bosch/Post (Gerlingen Gerlingen)
	GERLINGEN_ROBERT_BOSCH_PLATZ = 'de:08118:6906'
	GERLINGEN_ROBERT_BOSCH_PLATZ_1 = 'de:08118:6906:0:3'  # eRtg Waldfriedhof (Gerlingen Gerlingen)
	GERLINGEN_ROBERT_BOSCH_PLATZ_2 = 'de:08118:6906:0:4'  # eRtg Schillerhöhe Bosch/Post (Gerlingen Gerlingen)
	GERLINGEN_WALDFRIEDHOF = 'de:08118:6907:0:3'  # eRtg Robert-Bosch-Platz (Gerlingen Gerlingen)
	BREITWIESENHAUS = 'de:08118:6909:0:1'  # Alle Richtungen (Gerlingen Gerlingen)
	GERLINGEN_BREITWIESENHAUS = 'de:08118:6909:0:1'  # Alle Richtungen (Gerlingen Gerlingen)
	LEUTENBACH_WINNENDER_STR = 'de:08119:6911'
	LEUTENBACH_WINNENDER_STR_1 = 'de:08119:6911:0:3'  # eRtg Winnenden (Leutenbach Leutenbach)
	LEUTENBACH_WINNENDER_STR_2 = 'de:08119:6911:0:4'  # eRtg Rathaus (Leutenbach Leutenbach)
	BACKNANG_K__FERD__BRAUN_STRASSE = 'de:08119:6912:0:3'  # eRtg Lerchenäcker (Backnang Backnang)
	K__FERD__BRAUN_STRASSE = 'de:08119:6912:0:3'  # eRtg Lerchenäcker (Backnang Backnang)
	HÜTTENBÜHLSEE_ABZW = 'de:08119:6914'
	HÜTTENBÜHLSEE_ABZW_1 = 'de:08119:6914:0:3'  # eRtg Birkhof (Alfdorf Vordersteinenberg)
	HÜTTENBÜHLSEE_ABZW_2 = 'de:08119:6914:0:4'  # Rtg Wahlenheim (Alfdorf Vordersteinenberg)
	VORDERST__HÜTTENB_SEE_ABZW = 'de:08119:6914'
	VORDERST__HÜTTENB_SEE_ABZW_1 = 'de:08119:6914:0:3'  # eRtg Birkhof (Alfdorf Vordersteinenberg)
	VORDERST__HÜTTENB_SEE_ABZW_2 = 'de:08119:6914:0:4'  # Rtg Wahlenheim (Alfdorf Vordersteinenberg)
	HELLERSHOF_KREUZUNG = 'de:08119:6915'
	HELLERSHOF_KREUZUNG_1 = 'de:08119:6915:0:3'  # Rtg Cronhütte (Alfdorf Vordersteinenberg)
	HELLERSHOF_KREUZUNG_2 = 'de:08119:6915:0:4'  # Rtg Birkhof (Alfdorf Vordersteinenberg)
	VORDERST__HELLERSHOF_KREUZUNG = 'de:08119:6915'
	VORDERST__HELLERSHOF_KREUZUNG_1 = 'de:08119:6915:0:3'  # Rtg Cronhütte (Alfdorf Vordersteinenberg)
	VORDERST__HELLERSHOF_KREUZUNG_2 = 'de:08119:6915:0:4'  # Rtg Birkhof (Alfdorf Vordersteinenberg)
	KAISERSBACH_TÄLE = 'de:08119:6916'
	KAISERSBACH_TÄLE_1 = 'de:08119:6916:0:3'  # eRtg Ziegelhütte (Kaisersbach Kaisersbach)
	KAISERSBACH_TÄLE_2 = 'de:08119:6916:0:4'  # Rtg. Kaisersb. Cronhütte (Kaisersbach Kaisersbach)
	TÄLE = 'de:08119:6916'
	TÄLE_1 = 'de:08119:6916:0:3'  # eRtg Ziegelhütte (Kaisersbach Kaisersbach)
	TÄLE_2 = 'de:08119:6916:0:4'  # Rtg. Kaisersb. Cronhütte (Kaisersbach Kaisersbach)
	OBERBRÜDENER_STRASSE = 'de:08119:6918'
	OBERBRÜDENER_STRASSE_1 = 'de:08119:6918:0:3'  # Rtg Abzw. Mittelbrüden (Backnang Steinbach)
	OBERBRÜDENER_STRASSE_2 = 'de:08119:6918:0:4'  # Rtg Lindenplatz (Backnang Steinbach)
	STEINBACH_OBERBRÜDENER_STR = 'de:08119:6918'
	STEINBACH_OBERBRÜDENER_STR_1 = 'de:08119:6918:0:3'  # Rtg Abzw. Mittelbrüden (Backnang Steinbach)
	STEINBACH_OBERBRÜDENER_STR_2 = 'de:08119:6918:0:4'  # Rtg Lindenplatz (Backnang Steinbach)
	FORNSBACH_WALDSEE = 'de:08119:6921:0:3'  # eRtg Kindergarten (Murrhardt Fornsbach)
	WALDSEE = 'de:08119:6921:0:3'  # eRtg Kindergarten (Murrhardt Fornsbach)
	KOHLBERG_WALDFRIEDHOF = 'de:08116:6923'
	KOHLBERG_WALDFRIEDHOF_1 = 'de:08116:6923:0:1'  # Rtg Tischardt (Kohlberg Kohlberg)
	KOHLBERG_WALDFRIEDHOF_2 = 'de:08116:6923:0:2'  # Rtg Kohlberg (Kohlberg Kohlberg)
	KAISERSBACH_BIRKHOF = 'de:08119:6924'
	KAISERSBACH_BIRKHOF_1 = 'de:08119:6924:0:3'  # eRtg Hüttenbühl (Kaisersbach Kaisersbach)
	KAISERSBACH_BIRKHOF_2 = 'de:08119:6924:0:4'  # eRtg Heinlesmühle/Cronhütte (Kaisersbach Kaisersbach)
	BIRKHOF_1 = 'de:08119:6924:0:3'  # eRtg Hüttenbühl (Kaisersbach Kaisersbach)
	BIRKHOF_2 = 'de:08119:6924:0:4'  # eRtg Heinlesmühle/Cronhütte (Kaisersbach Kaisersbach)
	BIRKHOF_3 = 'de:08117:10075:0:1'  # Rtg Ave Maria (Deggingen Deggingen)
	BIRKHOF_4 = 'de:08117:10075:0:2'  # Rtg Reichenbach (Deggingen Deggingen)
	VORDERST__HUNDSBERG = 'de:08119:6925'
	VORDERST__HUNDSBERG_1 = 'de:08119:6925:0:3'  # eRtg Cronhütte (Alfdorf Vordersteinenberg)
	VORDERST__HUNDSBERG_2 = 'de:08119:6925:0:4'  # eRtg Hellershof (Alfdorf Vordersteinenberg)
	HUNDSBERG = 'de:08119:6925'
	HUNDSBERG_1 = 'de:08119:6925:0:3'  # eRtg Cronhütte (Alfdorf Vordersteinenberg)
	HUNDSBERG_2 = 'de:08119:6925:0:4'  # eRtg Hellershof (Alfdorf Vordersteinenberg)
	SCHWIEBERD__BOSCH_TOR_IV = 'de:08118:6926'
	SCHWIEBERD__BOSCH_TOR_IV_1 = 'de:08118:6926:0:3'  # eRtg Hermann-Essig-Straße/Möglingen (Schwieberdingen Schwieberdingen)
	SCHWIEBERD__BOSCH_TOR_IV_2 = 'de:08118:6926:0:4'  # eRtg Ludwigsburger Straße (Schwieberdingen Schwieberdingen)
	BOSCH_TOR_IV = 'de:08118:6926'
	BOSCH_TOR_IV_1 = 'de:08118:6926:0:3'  # eRtg Hermann-Essig-Straße/Möglingen (Schwieberdingen Schwieberdingen)
	BOSCH_TOR_IV_2 = 'de:08118:6926:0:4'  # eRtg Ludwigsburger Straße (Schwieberdingen Schwieberdingen)
	GROSSSACHSH__LICHTENSTERN_GYMN = 'de:08118:6929'
	GROSSSACHSH__LICHTENSTERN_GYMN_1 = 'de:08118:6929:0:3'  # eRtg Ludwigsburger Straße (Sachsenheim Großsachsenheim)
	GROSSSACHSH__LICHTENSTERN_GYMN_2 = 'de:08118:6929:0:4'  # eRtg Zimmerer Pfad (Sachsenheim Großsachsenheim)
	LICHTENSTERN_GYMNASIUM = 'de:08118:6929'
	LICHTENSTERN_GYMNASIUM_1 = 'de:08118:6929:0:3'  # eRtg Ludwigsburger Straße (Sachsenheim Großsachsenheim)
	LICHTENSTERN_GYMNASIUM_2 = 'de:08118:6929:0:4'  # eRtg Zimmerer Pfad (Sachsenheim Großsachsenheim)
	HEMMINGEN_MÜNCHINGER_STRASSE = 'de:08118:6930'
	HEMMINGEN_MÜNCHINGER_STRASSE_1 = 'de:08118:6930:0:3'  # Rtg Schwieberdingen / Sägmühle (Hemmingen Hemmingen)
	HEMMINGEN_MÜNCHINGER_STRASSE_2 = 'de:08118:6930:0:4'  # Rtg Hemmingen (Hemmingen Hemmingen)
	UNTERJETT__SINDL__AUSSIEDLERH = 'de:08115:6931'
	UNTERJETT__SINDL__AUSSIEDLERH_1 = 'de:08115:6931:0:3'  # Rtg Sindlingen Ortsmitte (Jettingen Unterjettingen)
	UNTERJETT__SINDL__AUSSIEDLERH_2 = 'de:08115:6931:0:4'  # Rtg Haslach Schelmenäcker (Jettingen Unterjettingen)
	SINDLINGEN_AUSSIEDLERHÖFE = 'de:08115:6931'
	SINDLINGEN_AUSSIEDLERHÖFE_1 = 'de:08115:6931:0:3'  # Rtg Sindlingen Ortsmitte (Jettingen Unterjettingen)
	SINDLINGEN_AUSSIEDLERHÖFE_2 = 'de:08115:6931:0:4'  # Rtg Haslach Schelmenäcker (Jettingen Unterjettingen)
	SCHLOSSHALDENSTRASSE = 'de:08118:6932'
	SCHLOSSHALDENSTRASSE_1 = 'de:08118:6932:0:3'  # eRtg Münchinger Straße (Hemmingen Hemmingen)
	SCHLOSSHALDENSTRASSE_2 = 'de:08118:6932:0:4'  # eRtg Schwieberdinger Straße (Hemmingen Hemmingen)
	HEMMINGEN_SCHLOSSHALDENSTR = 'de:08118:6932'
	HEMMINGEN_SCHLOSSHALDENSTR_1 = 'de:08118:6932:0:3'  # eRtg Münchinger Straße (Hemmingen Hemmingen)
	HEMMINGEN_SCHLOSSHALDENSTR_2 = 'de:08118:6932:0:4'  # eRtg Schwieberdinger Straße (Hemmingen Hemmingen)
	BACKNANG_LERCHENÄCKER = 'de:08119:6933:0:3'  # Rtg Ferd.-Braun-Str. (Backnang Backnang)
	OBERRIEXINGEN_RATHAUS = 'de:08118:6934:0:3'  # eRtg Talhöfe/Unterriexingen (Oberriexingen Oberriexingen)
	SACHSENWEILER_SCHULE = 'de:08119:6935:0:3'  # Rtg Hof (Backnang Sachsenweiler)
	BACKNANG_RICHARD_WAGNER_STR = 'de:08119:6936:0:3'  # Rtg Rosbergstaffel (Backnang Backnang)
	RICHARD_WAGNER_STRASSE = 'de:08119:6936:0:3'  # Rtg Rosbergstaffel (Backnang Backnang)
	FORNSBACHER_WEG = 'de:08119:6937:0:3'  # Rtg Seehofweg (Backnang Backnang)
	BACKNANG_FORNSBACHER_WEG = 'de:08119:6937:0:3'  # Rtg Seehofweg (Backnang Backnang)
	BACKNANG_BEIM_WASSERTURM = 'de:08119:6938'
	BACKNANG_BEIM_WASSERTURM_1 = 'de:08119:6938:0:3'  # Rtg Rossbergstaffel (Backnang Backnang)
	BACKNANG_BEIM_WASSERTURM_2 = 'de:08119:6938:0:4'  # Rtg Dresdener Ring (Backnang Backnang)
	FREIBERG__N__LÖCHLESÄCKER = 'de:08118:6940'
	FREIBERG__N__LÖCHLESÄCKER_1 = 'de:08118:6940:0:3'  # Rtg Freiberg (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__LÖCHLESÄCKER_2 = 'de:08118:6940:0:4'  # Rtg Großingersheim (Freiberg am Neckar Freiberg (N))
	LÖCHLESÄCKER = 'de:08118:6940'
	LÖCHLESÄCKER_1 = 'de:08118:6940:0:3'  # Rtg Freiberg (Freiberg am Neckar Freiberg (N))
	LÖCHLESÄCKER_2 = 'de:08118:6940:0:4'  # Rtg Großingersheim (Freiberg am Neckar Freiberg (N))
	VAIHINGEN__E__KEHLSTR = 'de:08118:6941'
	VAIHINGEN__E__KEHLSTR_1 = 'de:08118:6941:0:3'  # Rtg Stadtbahnhof (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__KEHLSTR_2 = 'de:08118:6941:0:4'  # Rtg Salzäcker (Vaihingen an der Enz Vaihingen (E))
	KEHLSTRASSE = 'de:08118:6941'
	KEHLSTRASSE_1 = 'de:08118:6941:0:3'  # Rtg Stadtbahnhof (Vaihingen an der Enz Vaihingen (E))
	KEHLSTRASSE_2 = 'de:08118:6941:0:4'  # Rtg Salzäcker (Vaihingen an der Enz Vaihingen (E))
	PFAHLBRONN_ABZW__EISENBACHSEE = 'de:08119:6942'
	PFAHLBRONN_ABZW__EISENBACHSEE_1 = 'de:08119:6942:0:3'  # eRtg Lorcher Straße (Alfdorf Pfahlbronn)
	PFAHLBRONN_ABZW__EISENBACHSEE_2 = 'de:08119:6942:0:4'  # Rtg Rienharz (Alfdorf Pfahlbronn)
	ABZWEIG_EISENBACHSEE = 'de:08119:6942'
	ABZWEIG_EISENBACHSEE_1 = 'de:08119:6942:0:3'  # eRtg Lorcher Straße (Alfdorf Pfahlbronn)
	ABZWEIG_EISENBACHSEE_2 = 'de:08119:6942:0:4'  # Rtg Rienharz (Alfdorf Pfahlbronn)
	ZIEGELHÜTTE = 'de:08119:6943:0:3'  # eRtg Hauptstraße (Kaisersbach Kaisersbach)
	KAISERSBACH_ZIEGELHÜTTE = 'de:08119:6943:0:3'  # eRtg Hauptstraße (Kaisersbach Kaisersbach)
	OBERRIEXINGER_STRASSE = 'de:08118:6944'
	OBERRIEXINGER_STRASSE_1 = 'de:08118:6944:0:3'  # eRtg Winkels (Sachsenheim Großsachsenheim)
	OBERRIEXINGER_STRASSE_2 = 'de:08118:6944:0:4'  # eRtg Burgfeldschule (Sachsenheim Großsachsenheim)
	GROSSSACHSH__OBERRIEXING__STR = 'de:08118:6944'
	GROSSSACHSH__OBERRIEXING__STR_1 = 'de:08118:6944:0:3'  # eRtg Winkels (Sachsenheim Großsachsenheim)
	GROSSSACHSH__OBERRIEXING__STR_2 = 'de:08118:6944:0:4'  # eRtg Burgfeldschule (Sachsenheim Großsachsenheim)
	STAUFENSTRASSE_1 = 'de:08116:6945:0:1'  # Rtg Untere Str. (Erkenbrechtsweiler Erkenbrechtsweiler)
	STAUFENSTRASSE_2 = 'de:08116:6945:0:2'  # Rtg Hochwang (Erkenbrechtsweiler Erkenbrechtsweiler)
	STAUFENSTRASSE_3 = 'de:08117:2015:0:H1'  # eRtg Uhinger-/Waldeckstr. (Albershausen Albershausen)
	STAUFENSTRASSE_4 = 'de:08117:2015:0:R1'  # eRtg B297/Kirchheimer Str. (Albershausen Albershausen)
	ERKENBRECHTSWEILER_STAUFENSTR = 'de:08116:6945'
	ERKENBRECHTSWEILER_STAUFENSTR_1 = 'de:08116:6945:0:1'  # Rtg Untere Str. (Erkenbrechtsweiler Erkenbrechtsweiler)
	ERKENBRECHTSWEILER_STAUFENSTR_2 = 'de:08116:6945:0:2'  # Rtg Hochwang (Erkenbrechtsweiler Erkenbrechtsweiler)
	LEHENBRÜCKE = 'de:08118:6946'
	LEHENBRÜCKE_1 = 'de:08118:6946:0:3'  # eRtg Asperg (Asperg Asperg)
	LEHENBRÜCKE_2 = 'de:08118:6946:0:4'  # eRtg Hohenstaufenstraße (Asperg Asperg)
	ASPERG_LEHENBRÜCKE = 'de:08118:6946'
	ASPERG_LEHENBRÜCKE_1 = 'de:08118:6946:0:3'  # eRtg Asperg (Asperg Asperg)
	ASPERG_LEHENBRÜCKE_2 = 'de:08118:6946:0:4'  # eRtg Hohenstaufenstraße (Asperg Asperg)
	ASPERG_BERLINER_STR = 'de:08118:6947:0:3'  # eRtg Schäferstraße (Asperg Asperg)
	ASPERG_STUTTGARTER_STR = 'de:08118:6948:0:3'  # eRtg Berliner Straße (Asperg Asperg)
	HOCHDORF__E__GARTENSTR = 'de:08118:6949:0:3'  # eRtg Rathaus (Eberdingen Hochdorf (Eberdingen))
	TH__HEUSS_STRASSE = 'de:08118:6950:0:3'  # Rtg Pulverdinger Str. (Eberdingen Hochdorf (Eberdingen))
	HOCHDORF__E__TH__HEUSS_STR = 'de:08118:6950:0:3'  # Rtg Pulverdinger Str. (Eberdingen Hochdorf (Eberdingen))
	PULVERDINGER_STRASSE = 'de:08118:6951:0:3'  # eRtg Keltenmuseum (Eberdingen Hochdorf (Eberdingen))
	HOCHDORF__E__PULVERDINGER_STR = 'de:08118:6951:0:3'  # eRtg Keltenmuseum (Eberdingen Hochdorf (Eberdingen))
	HOCHDORF__E__AM_PFAFFENWALD = 'de:08118:6952:0:4'  # eRtg Dorfplatz (Eberdingen Hochdorf (Eberdingen))
	AM_PFAFFENWALD = 'de:08118:6952:0:4'  # eRtg Dorfplatz (Eberdingen Hochdorf (Eberdingen))
	ROMMELSHAUSEN_LILIENSTR = 'de:08119:6954'
	ROMMELSHAUSEN_LILIENSTR_1 = 'de:08119:6954:0:3'  # Rtg Unter den Arkaden/Karlstr. (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_LILIENSTR_2 = 'de:08119:6954:0:4'  # Rtg Rommelshausen Bf (Kernen im Remstal Rommelshausen)
	ENZWEIHINGEN_KLAIBERSTR = 'de:08118:6955'
	ENZWEIHINGEN_KLAIBERSTR_1 = 'de:08118:6955:0:3'  # eRtg Dorfplatz (Vaihingen an der Enz Enzweihingen)
	ENZWEIHINGEN_KLAIBERSTR_2 = 'de:08118:6955:0:4'  # eRtg Beerhaldenstraße (Vaihingen an der Enz Enzweihingen)
	KLAIBERSTRASSE = 'de:08118:6955'
	KLAIBERSTRASSE_1 = 'de:08118:6955:0:3'  # eRtg Dorfplatz (Vaihingen an der Enz Enzweihingen)
	KLAIBERSTRASSE_2 = 'de:08118:6955:0:4'  # eRtg Beerhaldenstraße (Vaihingen an der Enz Enzweihingen)
	GROSSBETTLINGEN_METZINGER_STR = 'de:08116:6956:0:3'  # eRtg Albstraße (Großbettlingen Großbettlingen)
	SCHWIEBERDINGEN_BRESLAUER_STR = 'de:08118:6957:0:3'  # eRtg Stettiner Straße (Schwieberdingen Schwieberdingen)
	SCHWIEBERDINGEN_STETTINER_STR = 'de:08118:6958:0:3'  # eRtg Hermann-Essig-Straße (Schwieberdingen Schwieberdingen)
	SERSHEIM_EINSTEINSTR = 'de:08118:6959:0:3'  # eRtg Am Aischbach (Sersheim Sersheim)
	SERSHEIM_FESSLER_MÜHLE = 'de:08118:6961'
	SERSHEIM_FESSLER_MÜHLE_1 = 'de:08118:6961:0:3'  # eRtg Vaihinger Straße (Sersheim Sersheim)
	SERSHEIM_FESSLER_MÜHLE_2 = 'de:08118:6961:0:4'  # eRtg Metterbrücke (Sersheim Sersheim)
	FESSLER_MÜHLE = 'de:08118:6961'
	FESSLER_MÜHLE_1 = 'de:08118:6961:0:3'  # eRtg Vaihinger Straße (Sersheim Sersheim)
	FESSLER_MÜHLE_2 = 'de:08118:6961:0:4'  # eRtg Metterbrücke (Sersheim Sersheim)
	NUSSDORF_VAIHINGER_STR = 'de:08118:6962'
	NUSSDORF_VAIHINGER_STR_1 = 'de:08118:6962:0:3'  # eRtg Dorfplatz/B10/Aurich (Eberdingen Nussdorf)
	NUSSDORF_VAIHINGER_STR_2 = 'de:08118:6962:0:4'  # eRtg Martinstraße (Eberdingen Nussdorf)
	BESIGHEIM_FREIBAD_FITKOM = 'de:08118:6963'
	BESIGHEIM_FREIBAD_FITKOM_1 = 'de:08118:6963:0:3'  # eRtg Jahnstraße (Besigheim Besigheim)
	BESIGHEIM_FREIBAD_FITKOM_2 = 'de:08118:6963:0:4'  # eRtg Elserring (Besigheim Besigheim)
	FREIBAD_FITKOM = 'de:08118:6963'
	FREIBAD_FITKOM_1 = 'de:08118:6963:0:3'  # eRtg Jahnstraße (Besigheim Besigheim)
	FREIBAD_FITKOM_2 = 'de:08118:6963:0:4'  # eRtg Elserring (Besigheim Besigheim)
	ENSINGEN_ALTES_SCHULHAUS = 'de:08118:6964'
	ENSINGEN_ALTES_SCHULHAUS_1 = 'de:08118:6964:0:3'  # eRtg Feuerwehrhaus (Vaihingen an der Enz Ensingen)
	ENSINGEN_ALTES_SCHULHAUS_2 = 'de:08118:6964:0:4'  # eRtg Kleinglattbacher Straße (Vaihingen an der Enz Ensingen)
	ALTES_SCHULHAUS = 'de:08118:6964'
	ALTES_SCHULHAUS_1 = 'de:08118:6964:0:3'  # eRtg Feuerwehrhaus (Vaihingen an der Enz Ensingen)
	ALTES_SCHULHAUS_2 = 'de:08118:6964:0:4'  # eRtg Kleinglattbacher Straße (Vaihingen an der Enz Ensingen)
	AURICH_REITSTEIGE = 'de:08118:6965'
	AURICH_REITSTEIGE_1 = 'de:08118:6965:0:3'  # eRtg Ortsmitte (Vaihingen an der Enz Aurich)
	AURICH_REITSTEIGE_2 = 'de:08118:6965:0:4'  # eRtg Abzweig (Vaihingen an der Enz Aurich)
	REITSTEIGE = 'de:08118:6965'
	REITSTEIGE_1 = 'de:08118:6965:0:3'  # eRtg Ortsmitte (Vaihingen an der Enz Aurich)
	REITSTEIGE_2 = 'de:08118:6965:0:4'  # eRtg Abzweig (Vaihingen an der Enz Aurich)
	STEINBERG_HALDE = 'de:08119:6966'
	STEINBERG_HALDE_1 = 'de:08119:6966:0:3'  # eRtg Mannenweiler (Murrhardt Murrhardt)
	STEINBERG_HALDE_2 = 'de:08119:6966:0:4'  # eRtg Steinberg (Murrhardt Murrhardt)
	MURRHARDT_STEINBERG_HALDE = 'de:08119:6966'
	MURRHARDT_STEINBERG_HALDE_1 = 'de:08119:6966:0:3'  # eRtg Mannenweiler (Murrhardt Murrhardt)
	MURRHARDT_STEINBERG_HALDE_2 = 'de:08119:6966:0:4'  # eRtg Steinberg (Murrhardt Murrhardt)
	HAUBENWASEN = 'de:08119:6968'
	HAUBENWASEN_1 = 'de:08119:6968:0:3'  # eRtg Petershalde (Alfdorf Pfahlbronn)
	HAUBENWASEN_2 = 'de:08119:6968:0:4'  # eRtg Welzheimer Straße (Alfdorf Pfahlbronn)
	PFAHLBRONN_HAUBENWASEN = 'de:08119:6968'
	PFAHLBRONN_HAUBENWASEN_1 = 'de:08119:6968:0:3'  # eRtg Petershalde (Alfdorf Pfahlbronn)
	PFAHLBRONN_HAUBENWASEN_2 = 'de:08119:6968:0:4'  # eRtg Welzheimer Straße (Alfdorf Pfahlbronn)
	PLIENSAUV__PFEIFFERKLINGE = 'de:08116:6969'
	PLIENSAUV__PFEIFFERKLINGE_1 = 'de:08116:6969:0:3'  # Rtg Pliensaufriedhof (Esslingen am Neckar Pliensauvorstadt)
	PLIENSAUV__PFEIFFERKLINGE_2 = 'de:08116:6969:0:4'  # Rtg Weilstraße (Esslingen am Neckar Pliensauvorstadt)
	PFEIFFERKLINGE = 'de:08116:6969'
	PFEIFFERKLINGE_1 = 'de:08116:6969:0:3'  # Rtg Pliensaufriedhof (Esslingen am Neckar Pliensauvorstadt)
	PFEIFFERKLINGE_2 = 'de:08116:6969:0:4'  # Rtg Weilstraße (Esslingen am Neckar Pliensauvorstadt)
	BÜRGERZENTRUM = 'de:08116:6970'
	BÜRGERZENTRUM_1 = 'de:08116:6970:0:3'  # eRtg Nachbarschaftsschule (Lichtenwald Hegenlohe)
	BÜRGERZENTRUM_2 = 'de:08116:6970:0:4'  # eRtg Thomashardter Straße (Lichtenwald Hegenlohe)
	HEGENLOHE_BÜRGERZENTRUM = 'de:08116:6970'
	HEGENLOHE_BÜRGERZENTRUM_1 = 'de:08116:6970:0:3'  # eRtg Nachbarschaftsschule (Lichtenwald Hegenlohe)
	HEGENLOHE_BÜRGERZENTRUM_2 = 'de:08116:6970:0:4'  # eRtg Thomashardter Straße (Lichtenwald Hegenlohe)
	GROSSBOTTWAR_FLICKENWIESEN = 'de:08118:6971'
	GROSSBOTTWAR_FLICKENWIESEN_1 = 'de:08118:6971:0:3'  # eRtg Kelter (Großbottwar Großbottwar)
	GROSSBOTTWAR_FLICKENWIESEN_2 = 'de:08118:6971:0:4'  # eRtg Sauserhof (Großbottwar Großbottwar)
	FLICKENWIESEN = 'de:08118:6971'
	FLICKENWIESEN_1 = 'de:08118:6971:0:3'  # eRtg Kelter (Großbottwar Großbottwar)
	FLICKENWIESEN_2 = 'de:08118:6971:0:4'  # eRtg Sauserhof (Großbottwar Großbottwar)
	ALDINGEN_HAUS_DER_FEUERWEHR = 'de:08118:6973'
	ALDINGEN_HAUS_DER_FEUERWEHR_1 = 'de:08118:6973:0:3'  # eRtg Stadtbahn/Ostfriedhof (Remseck am Neckar Aldingen)
	ALDINGEN_HAUS_DER_FEUERWEHR_2 = 'de:08118:6973:0:4'  # eRtg Neckarstraße (Remseck am Neckar Aldingen)
	HAUS_DER_FEUERWEHR = 'de:08118:6973'
	HAUS_DER_FEUERWEHR_1 = 'de:08118:6973:0:3'  # eRtg Stadtbahn/Ostfriedhof (Remseck am Neckar Aldingen)
	HAUS_DER_FEUERWEHR_2 = 'de:08118:6973:0:4'  # eRtg Neckarstraße (Remseck am Neckar Aldingen)
	GÜNDELBACH_KATHARINENSTR = 'de:08118:6976:0:3'  # eRtg Hildenstraße (Vaihingen an der Enz Gündelbach)
	MARKGRÖNINGEN_IM_LANGEN_FELD = 'de:08118:6977'
	MARKGRÖNINGEN_IM_LANGEN_FELD_1 = 'de:08118:6977:0:3'  # eRtg Sträßle (Markgröningen Markgröningen)
	MARKGRÖNINGEN_IM_LANGEN_FELD_2 = 'de:08118:6977:0:4'  # eRtg Markgröninger Straße (Markgröningen Markgröningen)
	IM_LANGEN_FELD = 'de:08118:6977'
	IM_LANGEN_FELD_1 = 'de:08118:6977:0:3'  # eRtg Sträßle (Markgröningen Markgröningen)
	IM_LANGEN_FELD_2 = 'de:08118:6977:0:4'  # eRtg Markgröninger Straße (Markgröningen Markgröningen)
	MARKGRÖNINGEN_STEINBEISSTR = 'de:08118:6978'
	MARKGRÖNINGEN_STEINBEISSTR_1 = 'de:08118:6978:0:3'  # eRtg Abzweig Bahnhof (Markgröningen Markgröningen)
	MARKGRÖNINGEN_STEINBEISSTR_2 = 'de:08118:6978:0:4'  # eRtg Klinik/Schule (Markgröningen Markgröningen)
	TANNHOF = 'de:08119:6979'
	TANNHOF_1 = 'de:08119:6979:0:3'  # eRtg Rienharz (Alfdorf Pfahlbronn)
	TANNHOF_2 = 'de:08119:6979:0:4'  # Rtg Welzheim Gschwender Str. (Alfdorf Pfahlbronn)
	ALFDORF_TANNHOF = 'de:08119:6979'
	ALFDORF_TANNHOF_1 = 'de:08119:6979:0:3'  # eRtg Rienharz (Alfdorf Pfahlbronn)
	ALFDORF_TANNHOF_2 = 'de:08119:6979:0:4'  # Rtg Welzheim Gschwender Str. (Alfdorf Pfahlbronn)
	JUGENDDORF = 'de:08118:6980'
	JUGENDDORF_1 = 'de:08118:6980:0:3'  # eRtg Steinwerk (Vaihingen an der Enz Vaihingen (E))
	JUGENDDORF_2 = 'de:08118:6980:0:4'  # eRtg Grabenstr. (Stadtm.) (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__JUGENDDORF = 'de:08118:6980'
	VAIHINGEN__E__JUGENDDORF_1 = 'de:08118:6980:0:3'  # eRtg Steinwerk (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__JUGENDDORF_2 = 'de:08118:6980:0:4'  # eRtg Grabenstr. (Stadtm.) (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__STEINWERK = 'de:08118:6981'
	VAIHINGEN__E__STEINWERK_1 = 'de:08118:6981:0:3'  # eRtg Seemühle (Vaihingen an der Enz Vaihingen (E))
	VAIHINGEN__E__STEINWERK_2 = 'de:08118:6981:0:4'  # eRtg Jugenddorf (Vaihingen an der Enz Vaihingen (E))
	STEINWERK = 'de:08118:6981'
	STEINWERK_1 = 'de:08118:6981:0:3'  # eRtg Seemühle (Vaihingen an der Enz Vaihingen (E))
	STEINWERK_2 = 'de:08118:6981:0:4'  # eRtg Jugenddorf (Vaihingen an der Enz Vaihingen (E))
	PATTONVILLE_MITTE = 'de:08118:6982'
	PATTONVILLE_MITTE_1 = 'de:08118:6982:0:3'  # eRtg Realschule Remseck (Remseck am Neckar Pattonville)
	PATTONVILLE_MITTE_2 = 'de:08118:6982:0:4'  # eRtg Berufsschule (Remseck am Neckar Pattonville)
	WINNENDEN_REMS_MURR_KLINIKUM = 'de:08119:6983:0:3'  # eRtg Alle Richtungen (Winnenden Winnenden)
	REMS_MURR_KLINIKUM = 'de:08119:6983:0:3'  # eRtg Alle Richtungen (Winnenden Winnenden)
	POPPENWEILER_JAHNSTR = 'de:08118:6985'
	POPPENWEILER_JAHNSTR_1 = 'de:08118:6985:0:3'  # eRtg Friedhof (Ludwigsburg Poppenweiler)
	POPPENWEILER_JAHNSTR_2 = 'de:08118:6985:0:4'  # eRtg Poppenweiler Straße (Ludwigsburg Poppenweiler)
	KLEINBOTTWAR_SEESTRASSE = 'de:08118:6986'
	KLEINBOTTWAR_SEESTRASSE_1 = 'de:08118:6986:1:3'  # eRtg Horrenwinkel (Steinheim an der Murr Kleinbottwar)
	KLEINBOTTWAR_SEESTRASSE_2 = 'de:08118:6986:1:4'  # eRtg Waldstraße (Steinheim an der Murr Kleinbottwar)
	HORRENWINKEL = 'de:08118:6987'
	HORRENWINKEL_1 = 'de:08118:6987:0:3'  # eRtg Schulzentrum (Steinheim an der Murr Steinheim (M))
	HORRENWINKEL_2 = 'de:08118:6987:0:4'  # eRtg Seestraße (Steinheim an der Murr Steinheim (M))
	STEINHEIM__M__HORRENWINKEL = 'de:08118:6987'
	STEINHEIM__M__HORRENWINKEL_1 = 'de:08118:6987:0:3'  # eRtg Schulzentrum (Steinheim an der Murr Steinheim (M))
	STEINHEIM__M__HORRENWINKEL_2 = 'de:08118:6987:0:4'  # eRtg Seestraße (Steinheim an der Murr Steinheim (M))
	STEINHEIM__M__SCHULZENTRUM = 'de:08118:6988'
	STEINHEIM__M__SCHULZENTRUM_1 = 'de:08118:6988:0:3'  # eRtg Höpfigheimer Straße (Steinheim an der Murr Steinheim (M))
	STEINHEIM__M__SCHULZENTRUM_2 = 'de:08118:6988:0:4'  # eRtg Horrenwinkel (Steinheim an der Murr Steinheim (M))
	ZAZENHAUSEN_VIADUKT = 'de:08111:6989'
	ZAZENHAUSEN_VIADUKT_1 = 'de:08111:6989:0:3'  # eRtg Zuffenhausen Reibedanz (Stuttgart Stuttgart)
	ZAZENHAUSEN_VIADUKT_2 = 'de:08111:6989:0:4'  # eRtg Zazenhausen Krone (Stuttgart Stuttgart)
	SCHORNDORF_ALTE_STEIGE = 'de:08119:6991:0:3'  # eRtg Alle Richtungen (Schorndorf Schorndorf)
	SCHORNDORF_LEDERSTRASSE = 'de:08119:6992'
	SCHORNDORF_LEDERSTRASSE_1 = 'de:08119:6992:0:3'  # eRtg Baumwasenstraße (Schorndorf Schorndorf)
	SCHORNDORF_LEDERSTRASSE_2 = 'de:08119:6992:0:4'  # eRtg ZOB (Schorndorf Schorndorf)
	LEDERSTRASSE = 'de:08119:6992'
	LEDERSTRASSE_1 = 'de:08119:6992:0:3'  # eRtg Baumwasenstraße (Schorndorf Schorndorf)
	LEDERSTRASSE_2 = 'de:08119:6992:0:4'  # eRtg ZOB (Schorndorf Schorndorf)
	SCHORNDORF_STEINBEISSTR = 'de:08119:6993'
	SCHORNDORF_STEINBEISSTR_1 = 'de:08119:6993:0:3'  # eRtg Grafenbergweg (Schorndorf Schorndorf)
	SCHORNDORF_STEINBEISSTR_2 = 'de:08119:6993:0:4'  # eRtg Waiblinger Straße (Schorndorf Schorndorf)
	SCHORNDORF_GRAFENBERGWEG = 'de:08119:6994'
	SCHORNDORF_GRAFENBERGWEG_1 = 'de:08119:6994:0:3'  # eRtg Talstraße (Schorndorf Schorndorf)
	SCHORNDORF_GRAFENBERGWEG_2 = 'de:08119:6994:0:4'  # eRtg Steinbeisstraße (Schorndorf Schorndorf)
	GRAFENBERGWEG = 'de:08119:6994'
	GRAFENBERGWEG_1 = 'de:08119:6994:0:3'  # eRtg Talstraße (Schorndorf Schorndorf)
	GRAFENBERGWEG_2 = 'de:08119:6994:0:4'  # eRtg Steinbeisstraße (Schorndorf Schorndorf)
	STEINHEIM__M__KREUZWEGÄCKER = 'de:08118:6995'
	STEINHEIM__M__KREUZWEGÄCKER_1 = 'de:08118:6995:0:3'  # Rtg Höpfigheim (Steinheim an der Murr Steinheim (M))
	STEINHEIM__M__KREUZWEGÄCKER_2 = 'de:08118:6995:0:4'  # Rtg Steinheim (Steinheim an der Murr Steinheim (M))
	KREUZWEGÄCKER = 'de:08118:6995'
	KREUZWEGÄCKER_1 = 'de:08118:6995:0:3'  # Rtg Höpfigheim (Steinheim an der Murr Steinheim (M))
	KREUZWEGÄCKER_2 = 'de:08118:6995:0:4'  # Rtg Steinheim (Steinheim an der Murr Steinheim (M))
	GRAB_FRIEDHOF = 'de:08119:6996'
	GRAB_FRIEDHOF_1 = 'de:08119:6996:0:3'  # eRtg Hohenbrach (Großerlach Grab)
	GRAB_FRIEDHOF_2 = 'de:08119:6996:0:4'  # eRtg Trauzenbach (Großerlach Grab)
	REIZENWIESEN = 'de:08119:6997'
	REIZENWIESEN_1 = 'de:08119:6997:0:3'  # Rtg Breitenfürst (Welzheim Welzheim)
	REIZENWIESEN_2 = 'de:08119:6997:0:4'  # Rtg Bausche (Welzheim Welzheim)
	WELZHEIM_REIZENWIESEN = 'de:08119:6997'
	WELZHEIM_REIZENWIESEN_1 = 'de:08119:6997:0:3'  # Rtg Breitenfürst (Welzheim Welzheim)
	WELZHEIM_REIZENWIESEN_2 = 'de:08119:6997:0:4'  # Rtg Bausche (Welzheim Welzheim)
	DEIZISAU_KEPLERSTR = 'de:08116:6998:0:3'  # eRtg Olga-/Zeppelinstraße (Deizisau Deizisau)
	MURR_BIETIGHEIMER_STR = 'de:08118:6999:0:3'  # eRtg Ortsmitte (Murr Murr)
	DITZINGEN = 'de:08118:7000'
	DITZINGEN_1 = 'de:08118:7000:1:2'  # Gleis 2 (Ditzingen Ditzingen)
	DITZINGEN_2 = 'de:08118:7000:1:3'  # Gleis 3 (Ditzingen Ditzingen)
	DITZINGEN_3 = 'de:08118:7000:1:5'  # Bstg 5 (Ditzingen Ditzingen)
	DITZINGEN_4 = 'de:08118:7000:1:7'  # Bstg 7 (Ditzingen Ditzingen)
	DITZINGEN_5 = 'de:08118:7000:2:1'  # Bstg 1 (Ditzingen Ditzingen)
	DITZINGEN_6 = 'de:08118:7000:2:2'  # Bstg 2 (Ditzingen Ditzingen)
	DITZINGEN_7 = 'de:08118:7000:2:3'  # Bstg 3 (Ditzingen Ditzingen)
	DITZINGEN_8 = 'de:08118:7000:2:4'  # Bstg 4 (Ditzingen Ditzingen)
	DITZINGEN_9 = 'de:08118:7000:6:6'  # Bstg 6 (Ditzingen Ditzingen)
	DITZINGEN_GEWERBEGEBIET_SÜD = 'de:08118:7001'
	DITZINGEN_GEWERBEGEBIET_SÜD_1 = 'de:08118:7001:0:3'  # Rtg Gerlingen Hofwiesenstr. (Ditzingen Ditzingen)
	DITZINGEN_GEWERBEGEBIET_SÜD_2 = 'de:08118:7001:0:4'  # Rtg Ditzingen Bf (Ditzingen Ditzingen)
	DITZINGEN_GEWERBEGEBIET_SÜD_3 = 'de:08118:7001:0:5'  # eRtg Thales (Ditzingen Ditzingen)
	HERRENBERG_DAIMLERSTRASSE = 'de:08115:7002:0:3'  # eRtg Affstätter Tal (Herrenberg Herrenberg)
	ECHTERDINGEN = 'de:08116:7003'
	ECHTERDINGEN_1 = 'de:08116:7003:1:1'  # Gleis 1 (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_2 = 'de:08116:7003:1:2'  # Gleis 2 (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_3 = 'de:08116:7003:2:1'  # Bstg 1 (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_4 = 'de:08116:7003:2:3'  # Bstg 3 (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_5 = 'de:08116:7003:3:5'  # Bstg 5 (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_6 = 'de:08116:7003:4:6'  # Bstg 6 (Leinfelden-Echterdingen Echterdingen)
	HERRENBERG_BEETHOVENSTRASSE = 'de:08115:7004'
	HERRENBERG_BEETHOVENSTRASSE_1 = 'de:08115:7004:0:3'  # eRtg Grafenweg (Herrenberg Herrenberg)
	HERRENBERG_BEETHOVENSTRASSE_2 = 'de:08115:7004:0:4'  # eRtg Hallenbad (Herrenberg Herrenberg)
	HERZOGWEG = 'de:08115:7005:0:4'  # eRtg Seniorenzentrum (Herrenberg Herrenberg)
	HERRENBERG_HERZOGWEG = 'de:08115:7005:0:4'  # eRtg Seniorenzentrum (Herrenberg Herrenberg)
	HERRENBERG_GRAFENWEG = 'de:08115:7006'
	HERRENBERG_GRAFENWEG_1 = 'de:08115:7006:1:3'  # eRtg Waldfriedhof (Herrenberg Herrenberg)
	HERRENBERG_GRAFENWEG_2 = 'de:08115:7006:1:4'  # eRtg Herzogweg (Herrenberg Herrenberg)
	GRAFENWEG = 'de:08115:7006'
	GRAFENWEG_1 = 'de:08115:7006:1:3'  # eRtg Waldfriedhof (Herrenberg Herrenberg)
	GRAFENWEG_2 = 'de:08115:7006:1:4'  # eRtg Herzogweg (Herrenberg Herrenberg)
	PORSCHESTRASSE = 'de:08115:7007:0:3'  # eRtg Daimlerstraße (Herrenberg Herrenberg)
	HERRENBERG_PORSCHESTRASSE = 'de:08115:7007:0:3'  # eRtg Daimlerstraße (Herrenberg Herrenberg)
	HERRENBERG_STADTWERKE = 'de:08115:7009'
	HERRENBERG_STADTWERKE_1 = 'de:08115:7009:0:3'  # Rtg Nufringen (Herrenberg Herrenberg)
	HERRENBERG_STADTWERKE_2 = 'de:08115:7009:0:4'  # Rtg Herrenberg (Herrenberg Herrenberg)
	DECKENPFRONN_BRÜHLSTRASSE = 'de:08115:7011'
	DECKENPFRONN_BRÜHLSTRASSE_1 = 'de:08115:7011:0:3'  # Rtg Gärtringen (Deckenpfronn Deckenpfronn)
	DECKENPFRONN_BRÜHLSTRASSE_2 = 'de:08115:7011:0:4'  # Rtg Rathaus (Deckenpfronn Deckenpfronn)
	HILDRIZHAUSER_STRASSE = 'de:08115:7012'
	HILDRIZHAUSER_STRASSE_1 = 'de:08115:7012:0:1'  # Rtg Seniorenzentrum (Herrenberg Herrenberg)
	HILDRIZHAUSER_STRASSE_2 = 'de:08115:7012:0:2'  # Rtg Hindenburgstr. (Herrenberg Herrenberg)
	HILDRIZHAUSER_STRASSE_3 = 'de:08115:7012:0:3'  # eRtg Seniorenzentrum (Herrenberg Herrenberg)
	HILDRIZHAUSER_STRASSE_4 = 'de:08115:7012:0:4'  # Rtg Beethovenstr. (Herrenberg Herrenberg)
	HERRENBERG_HILDRIZHAUSER_STR = 'de:08115:7012'
	HERRENBERG_HILDRIZHAUSER_STR_1 = 'de:08115:7012:0:1'  # Rtg Seniorenzentrum (Herrenberg Herrenberg)
	HERRENBERG_HILDRIZHAUSER_STR_2 = 'de:08115:7012:0:2'  # Rtg Hindenburgstr. (Herrenberg Herrenberg)
	HERRENBERG_HILDRIZHAUSER_STR_3 = 'de:08115:7012:0:3'  # eRtg Seniorenzentrum (Herrenberg Herrenberg)
	HERRENBERG_HILDRIZHAUSER_STR_4 = 'de:08115:7012:0:4'  # Rtg Beethovenstr. (Herrenberg Herrenberg)
	HERRENBERG_REINHOLD_SCHICK_PL = 'de:08115:7013'
	HERRENBERG_REINHOLD_SCHICK_PL_1 = 'de:08115:7013:0:3'  # Rtg Herrenberg ZOB (Herrenberg Herrenberg)
	HERRENBERG_REINHOLD_SCHICK_PL_2 = 'de:08115:7013:0:4'  # Rtg Eichenweg Kalkofenstraße (Herrenberg Herrenberg)
	REINHOLD_SCHICK_PLATZ = 'de:08115:7013'
	REINHOLD_SCHICK_PLATZ_1 = 'de:08115:7013:0:3'  # Rtg Herrenberg ZOB (Herrenberg Herrenberg)
	REINHOLD_SCHICK_PLATZ_2 = 'de:08115:7013:0:4'  # Rtg Eichenweg Kalkofenstraße (Herrenberg Herrenberg)
	UNTERJETT__KATHOLISCHE_KIRCHE = 'de:08115:7015'
	UNTERJETT__KATHOLISCHE_KIRCHE_1 = 'de:08115:7015:0:3'  # Rtg Altes Rathaus Mötzinger Straße (Jettingen Unterjettingen)
	UNTERJETT__KATHOLISCHE_KIRCHE_2 = 'de:08115:7015:0:4'  # Rtg Sindlingen Öschelbronn (Jettingen Unterjettingen)
	OBERJETT__HERRENBERGER_STR = 'de:08115:7016'
	OBERJETT__HERRENBERGER_STR_1 = 'de:08115:7016:0:3'  # Rtg Gemeindezentrum Sindlinger Straße (Jettingen Oberjettingen)
	OBERJETT__HERRENBERGER_STR_2 = 'de:08115:7016:0:4'  # Rtg Öfele Herrenberg (Jettingen Oberjettingen)
	MÖTZINGEN_GOETHE__HÖLDERLINSTR = 'de:08115:7018:0:3'  # Rtg Goethestr. (Mötzingen Mötzingen)
	GOETHE__HÖLDERLINSTR = 'de:08115:7018:0:3'  # Rtg Goethestr. (Mötzingen Mötzingen)
	MÖTZINGEN_GOETHESTRASSE = 'de:08115:7019:0:3'  # Rtg In der Röte (Mötzingen Mötzingen)
	MÖTZINGEN_BRÜHLSTRASSE = 'de:08115:7020'
	MÖTZINGEN_BRÜHLSTRASSE_1 = 'de:08115:7020:0:3'  # eRtg Rathaus (Mötzingen Mötzingen)
	MÖTZINGEN_BRÜHLSTRASSE_2 = 'de:08115:7020:0:4'  # eRtg (Mötzingen Mötzingen)
	SCHIESSTÄLE = 'de:08115:7021:1:3'  # eRtg alle Richtungen (Herrenberg Herrenberg)
	HERRENBERG_SCHIESSTÄLE = 'de:08115:7021:1:3'  # eRtg alle Richtungen (Herrenberg Herrenberg)
	KOHLPLATTE = 'de:08115:7022'
	KOHLPLATTE_1 = 'de:08115:7022:0:3'  # Rtg Altes Rathaus (Jettingen Unterjettingen)
	KOHLPLATTE_2 = 'de:08115:7022:0:4'  # Rtg Imental (Jettingen Unterjettingen)
	UNTERJETTINGEN_KOHLPLATTE = 'de:08115:7022'
	UNTERJETTINGEN_KOHLPLATTE_1 = 'de:08115:7022:0:3'  # Rtg Altes Rathaus (Jettingen Unterjettingen)
	UNTERJETTINGEN_KOHLPLATTE_2 = 'de:08115:7022:0:4'  # Rtg Imental (Jettingen Unterjettingen)
	MÖNCHBERG_GREUTÄCKERSTRASSE = 'de:08115:7023:0:4'  # Rtg auswärts Herrenberg Kayh Gültstein (Herrenberg Mönchberg)
	GREUTÄCKERSTRASSE = 'de:08115:7023:0:4'  # Rtg auswärts Herrenberg Kayh Gültstein (Herrenberg Mönchberg)
	MÖNCHBERG_SCHULE = 'de:08115:7025:0:3'  # Rtg alle Richtungen (Herrenberg Mönchberg)
	BUCHHALDENSCHULE = 'de:08115:7027:0:3'  # Ri West (Aidlingen Aidlingen)
	AIDLINGEN_BUCHHALDENSCHULE = 'de:08115:7027:0:3'  # Ri West (Aidlingen Aidlingen)
	SONNENBERGSCHULE = 'de:08115:7028'
	SONNENBERGSCHULE_1 = 'de:08115:7028:0:3'  # Ri Ost (Aidlingen Aidlingen)
	SONNENBERGSCHULE_2 = 'de:08115:7028:0:4'  # Ri West (Aidlingen Aidlingen)
	AIDLINGEN_SONNENBERGSCHULE = 'de:08115:7028'
	AIDLINGEN_SONNENBERGSCHULE_1 = 'de:08115:7028:0:3'  # Ri Ost (Aidlingen Aidlingen)
	AIDLINGEN_SONNENBERGSCHULE_2 = 'de:08115:7028:0:4'  # Ri West (Aidlingen Aidlingen)
	ALLMENDSTRASSE = 'de:08115:7029'
	ALLMENDSTRASSE_1 = 'de:08115:7029:0:3'  # eRtg Maichingen (Sindelfingen Maichingen)
	ALLMENDSTRASSE_2 = 'de:08115:7029:0:4'  # eRtg Essegger Straße (Sindelfingen Maichingen)
	MAICHINGEN_ALLMENDSTR = 'de:08115:7029'
	MAICHINGEN_ALLMENDSTR_1 = 'de:08115:7029:0:3'  # eRtg Maichingen (Sindelfingen Maichingen)
	MAICHINGEN_ALLMENDSTR_2 = 'de:08115:7029:0:4'  # eRtg Essegger Straße (Sindelfingen Maichingen)
	SCHOPFLOCH = 'de:08115:7030'
	SCHOPFLOCH_1 = 'de:08115:7030:0:3'  # Rtg Warmbronn (Leonberg Eltingen)
	SCHOPFLOCH_2 = 'de:08115:7030:0:4'  # Rtg Niederhofen (Leonberg Eltingen)
	ELTINGEN_SCHOPFLOCH = 'de:08115:7030'
	ELTINGEN_SCHOPFLOCH_1 = 'de:08115:7030:0:3'  # Rtg Warmbronn (Leonberg Eltingen)
	ELTINGEN_SCHOPFLOCH_2 = 'de:08115:7030:0:4'  # Rtg Niederhofen (Leonberg Eltingen)
	BONDORF_HAILFINGER_STR = 'de:08115:7034'
	BONDORF_HAILFINGER_STR_1 = 'de:08115:7034:0:3'  # eRtg Hochwiesenstraße (Bondorf Bondorf)
	BONDORF_HAILFINGER_STR_2 = 'de:08115:7034:0:4'  # eRtg Wartehalle (Bondorf Bondorf)
	HAILFINGER_STRASSE = 'de:08115:7034'
	HAILFINGER_STRASSE_1 = 'de:08115:7034:0:3'  # eRtg Hochwiesenstraße (Bondorf Bondorf)
	HAILFINGER_STRASSE_2 = 'de:08115:7034:0:4'  # eRtg Wartehalle (Bondorf Bondorf)
	WEIL_I__S__KÄLBERSTELLE = 'de:08115:7036'
	WEIL_I__S__KÄLBERSTELLE_1 = 'de:08115:7036:0:3'  # Rtg Bebenhausen Waldhorn (Weil im Schönbuch Weil im Schönbuch)
	WEIL_I__S__KÄLBERSTELLE_2 = 'de:08115:7036:0:4'  # Rtg Dettenhausen Bf/Alte Post (Weil im Schönbuch Weil im Schönbuch)
	KÄLBERSTELLE = 'de:08115:7036'
	KÄLBERSTELLE_1 = 'de:08115:7036:0:3'  # Rtg Bebenhausen Waldhorn (Weil im Schönbuch Weil im Schönbuch)
	KÄLBERSTELLE_2 = 'de:08115:7036:0:4'  # Rtg Dettenhausen Bf/Alte Post (Weil im Schönbuch Weil im Schönbuch)
	WEIL_I__S__UNTERE_HALDE = 'de:08115:7037:1:1'  # R-Bahn (Weil im Schönbuch Weil im Schönbuch)
	HERRENBERG_KÖNIGSBERGER_STRASSE = 'de:08115:7038:0:3'  # eRtg Sankt Martin (Herrenberg Herrenberg)
	IM_STEINGRABEN = 'de:08115:7039:0:3'  # eRtg Akazienweg (Herrenberg Herrenberg)
	HERRENBERG_IM_STEINGRABEN = 'de:08115:7039:0:3'  # eRtg Akazienweg (Herrenberg Herrenberg)
	HERRENBERG_AMSELWEG = 'de:08115:7040:0:3'  # eRtg Finkenweg (Herrenberg Herrenberg)
	HERRENBERG_IM_VOGELSANG = 'de:08115:7042:0:3'  # eRtg Zeisigweg (Herrenberg Herrenberg)
	NIEDERREUTIN = 'de:08115:7046'
	NIEDERREUTIN_1 = 'de:08115:7046:0:3'  # eRtg Goethe-/Hölderlinstraße (Bondorf Bondorf)
	NIEDERREUTIN_2 = 'de:08115:7046:0:4'  # eRtg Bondorf (Bondorf Bondorf)
	BONDORF_NIEDERREUTIN = 'de:08115:7046'
	BONDORF_NIEDERREUTIN_1 = 'de:08115:7046:0:3'  # eRtg Goethe-/Hölderlinstraße (Bondorf Bondorf)
	BONDORF_NIEDERREUTIN_2 = 'de:08115:7046:0:4'  # eRtg Bondorf (Bondorf Bondorf)
	DECKENPFRONN_SIEMENSSTR = 'de:08115:7048:0:3'  # eRtg Oberjesingen (Deckenpfronn Deckenpfronn)
	FRONTALSTRASSE = 'de:08115:7049:0:3'  # eRtg Tübinger Straße (Deckenpfronn Deckenpfronn)
	DECKENPFRONN_FRONTALSTR = 'de:08115:7049:0:3'  # eRtg Tübinger Straße (Deckenpfronn Deckenpfronn)
	NAGOLDER_STEIG = 'de:08115:7050'
	NAGOLDER_STEIG_1 = 'de:08115:7050:0:3'  # Rtg Mötzingen Rathaus (Mötzingen Mötzingen)
	NAGOLDER_STEIG_2 = 'de:08115:7050:0:4'  # Rtg Nagold Mötzinger Str. (Mötzingen Mötzingen)
	MÖTZINGEN_NAGOLDER_STEIG = 'de:08115:7050'
	MÖTZINGEN_NAGOLDER_STEIG_1 = 'de:08115:7050:0:3'  # Rtg Mötzingen Rathaus (Mötzingen Mötzingen)
	MÖTZINGEN_NAGOLDER_STEIG_2 = 'de:08115:7050:0:4'  # Rtg Nagold Mötzinger Str. (Mötzingen Mötzingen)
	ISELSHAUSER_STRASSE = 'de:08115:7051'
	ISELSHAUSER_STRASSE_1 = 'de:08115:7051:0:3'  # eRtg Hoher Baumweg (Mötzingen Mötzingen)
	ISELSHAUSER_STRASSE_2 = 'de:08115:7051:0:4'  # eRtg Nagolder Steig (Mötzingen Mötzingen)
	MÖTZINGEN_ISELSHAUSER_STRASSE = 'de:08115:7051'
	MÖTZINGEN_ISELSHAUSER_STRASSE_1 = 'de:08115:7051:0:3'  # eRtg Hoher Baumweg (Mötzingen Mötzingen)
	MÖTZINGEN_ISELSHAUSER_STRASSE_2 = 'de:08115:7051:0:4'  # eRtg Nagolder Steig (Mötzingen Mötzingen)
	GÄRTR__FEUERWEHRGERÄTEHAUS = 'de:08115:7052'
	GÄRTR__FEUERWEHRGERÄTEHAUS_1 = 'de:08115:7052:0:3'  # Rtg Schwarzwaldstraße (Gärtringen Gärtringen)
	GÄRTR__FEUERWEHRGERÄTEHAUS_2 = 'de:08115:7052:0:4'  # Rtg Rathaus (Gärtringen Gärtringen)
	FEUERWEHRGERÄTEHAUS = 'de:08115:7052'
	FEUERWEHRGERÄTEHAUS_1 = 'de:08115:7052:0:3'  # Rtg Schwarzwaldstraße (Gärtringen Gärtringen)
	FEUERWEHRGERÄTEHAUS_2 = 'de:08115:7052:0:4'  # Rtg Rathaus (Gärtringen Gärtringen)
	GÄRTRINGEN_SCHWARZWALDSTR = 'de:08115:7053'
	GÄRTRINGEN_SCHWARZWALDSTR_1 = 'de:08115:7053:0:3'  # Rtg Grabenstraße (Gärtringen Gärtringen)
	GÄRTRINGEN_SCHWARZWALDSTR_2 = 'de:08115:7053:0:4'  # Rtg Feuerwehrgerätehaus (Gärtringen Gärtringen)
	HERRENBERG_BENZSTRASSE = 'de:08115:7054'
	HERRENBERG_BENZSTRASSE_1 = 'de:08115:7054:0:3'  # eRtg Umspannwerk (Herrenberg Herrenberg)
	HERRENBERG_BENZSTRASSE_2 = 'de:08115:7054:0:4'  # Rtg Herrenberg (Herrenberg Herrenberg)
	HERRENBERG_FRÖBELSCHULE = 'de:08115:7055:0:3'  # eRtg Im Seele (Herrenberg Herrenberg)
	FRÖBELSCHULE = 'de:08115:7055:0:3'  # eRtg Im Seele (Herrenberg Herrenberg)
	IM_SEELE = 'de:08115:7056:0:3'  # eRtg Adlerstraße (Herrenberg Herrenberg)
	HERRENBERG_IM_SEELE = 'de:08115:7056:0:3'  # eRtg Adlerstraße (Herrenberg Herrenberg)
	HERRENBERG_ADLERSTRASSE = 'de:08115:7057:0:3'  # eRtg Im Vogelsang (Herrenberg Herrenberg)
	ZEISIGWEG_1 = 'de:08115:7059:0:3'  # eRtg Amselweg (Herrenberg Herrenberg)
	ZEISIGWEG_2 = 'de:08117:1019:0:H1'  # Ankunft (Eislingen/Fils Eislingen (F))
	HERRENBERG_ZEISIGWEG = 'de:08115:7059:0:3'  # eRtg Amselweg (Herrenberg Herrenberg)
	HERRENBERG_AFFSTÄTTER_TAL = 'de:08115:7065'
	HERRENBERG_AFFSTÄTTER_TAL_1 = 'de:08115:7065:0:3'  # eRtg Stadthalle (Herrenberg Herrenberg)
	HERRENBERG_AFFSTÄTTER_TAL_2 = 'de:08115:7065:0:4'  # Rtg Daimlerstr. (Herrenberg Herrenberg)
	AFFSTÄTTER_TAL = 'de:08115:7065'
	AFFSTÄTTER_TAL_1 = 'de:08115:7065:0:3'  # eRtg Stadthalle (Herrenberg Herrenberg)
	AFFSTÄTTER_TAL_2 = 'de:08115:7065:0:4'  # Rtg Daimlerstr. (Herrenberg Herrenberg)
	HERRENBERG_EICHENWEG = 'de:08115:7066:0:3'  # eRtg Im Steingraben (Herrenberg Herrenberg)
	HERRENBERG_AKAZIENWEG = 'de:08115:7067:0:3'  # eRtg Wittenberger Straße (Herrenberg Herrenberg)
	AKAZIENWEG = 'de:08115:7067:0:3'  # eRtg Wittenberger Straße (Herrenberg Herrenberg)
	HERRENBERG_WITTENBERGER_STRASSE = 'de:08115:7068:0:3'  # eRtg Torgauer Weg (Herrenberg Herrenberg)
	WITTENBERGER_STRASSE = 'de:08115:7068:0:3'  # eRtg Torgauer Weg (Herrenberg Herrenberg)
	HERRENBERG_TORGAUER_WEG = 'de:08115:7069:0:3'  # eRtg Königsberger Straße (Herrenberg Herrenberg)
	TORGAUER_WEG = 'de:08115:7069:0:3'  # eRtg Königsberger Straße (Herrenberg Herrenberg)
	NEUFFEN_OBERER_GRABEN = 'de:08116:7070'
	NEUFFEN_OBERER_GRABEN_1 = 'de:08116:7070:0:3'  # eRtg Neuffen (Neuffen Neuffen)
	NEUFFEN_OBERER_GRABEN_2 = 'de:08116:7070:0:4'  # eRtg Max-Planck-Straße/Albstraße (Neuffen Neuffen)
	OBERER_GRABEN = 'de:08116:7070'
	OBERER_GRABEN_1 = 'de:08116:7070:0:3'  # eRtg Neuffen (Neuffen Neuffen)
	OBERER_GRABEN_2 = 'de:08116:7070:0:4'  # eRtg Max-Planck-Straße/Albstraße (Neuffen Neuffen)
	HASLACH_SCHELMENÄCKER = 'de:08115:7071'
	HASLACH_SCHELMENÄCKER_1 = 'de:08115:7071:0:3'  # Rtg Sindl. Aussiedlerhöfe Unterjettingen (Herrenberg Haslach)
	HASLACH_SCHELMENÄCKER_2 = 'de:08115:7071:0:4'  # Rtg Sindlinger Straße (Herrenberg Haslach)
	SCHELMENÄCKER = 'de:08115:7071'
	SCHELMENÄCKER_1 = 'de:08115:7071:0:3'  # Rtg Sindl. Aussiedlerhöfe Unterjettingen (Herrenberg Haslach)
	SCHELMENÄCKER_2 = 'de:08115:7071:0:4'  # Rtg Sindlinger Straße (Herrenberg Haslach)
	DORFGEMEINSCHAFT_TENNENTAL = 'de:08115:7072'
	DORFGEMEINSCHAFT_TENNENTAL_1 = 'de:08115:7072:0:3'  # Rtg Brühlstraße (Deckenpfronn Deckenpfronn)
	DORFGEMEINSCHAFT_TENNENTAL_2 = 'de:08115:7072:0:4'  # Rtg Gärtingen Grabenstraße (Deckenpfronn Deckenpfronn)
	DECKENPFR__DORFGEM__TENNENTAL = 'de:08115:7072'
	DECKENPFR__DORFGEM__TENNENTAL_1 = 'de:08115:7072:0:3'  # Rtg Brühlstraße (Deckenpfronn Deckenpfronn)
	DECKENPFR__DORFGEM__TENNENTAL_2 = 'de:08115:7072:0:4'  # Rtg Gärtingen Grabenstraße (Deckenpfronn Deckenpfronn)
	GÄRTRINGEN_GRABENSTRASSE = 'de:08115:7073'
	GÄRTRINGEN_GRABENSTRASSE_1 = 'de:08115:7073:0:3'  # Rtg Dorfgemeinschaft Tennental (Gärtringen Gärtringen)
	GÄRTRINGEN_GRABENSTRASSE_2 = 'de:08115:7073:0:4'  # Rtg Schwarzwaldstraße (Gärtringen Gärtringen)
	HERRENBERG_JOHANNESSTR = 'de:08115:7074'
	HERRENBERG_JOHANNESSTR_1 = 'de:08115:7074:0:1'  # Rtg Silcherstr (Herrenberg Herrenberg)
	HERRENBERG_JOHANNESSTR_2 = 'de:08115:7074:0:2'  # Rtg Hallenbad (Herrenberg Herrenberg)
	HERRENBERG_STADTFRIEDHOF = 'de:08115:7075:1:3'  # eRtg Hindenburgstr. (Herrenberg Herrenberg)
	STADTFRIEDHOF = 'de:08115:7075:1:3'  # eRtg Hindenburgstr. (Herrenberg Herrenberg)
	SANKT_MARTIN = 'de:08115:7076:0:3'  # eRtg Vogt-Heß-Schule (Herrenberg Herrenberg)
	HERRENBERG_SANKT_MARTIN = 'de:08115:7076:0:3'  # eRtg Vogt-Heß-Schule (Herrenberg Herrenberg)
	IM_BACKENSCHLAG = 'de:08115:7079'
	IM_BACKENSCHLAG_1 = 'de:08115:7079:0:3'  # eRtg Hailfinger Straße (Bondorf Bondorf)
	IM_BACKENSCHLAG_2 = 'de:08115:7079:0:4'  # eRtg Wartehalle (Bondorf Bondorf)
	BONDORF_IM_BACKENSCHLAG = 'de:08115:7079'
	BONDORF_IM_BACKENSCHLAG_1 = 'de:08115:7079:0:3'  # eRtg Hailfinger Straße (Bondorf Bondorf)
	BONDORF_IM_BACKENSCHLAG_2 = 'de:08115:7079:0:4'  # eRtg Wartehalle (Bondorf Bondorf)
	NEUFFEN_ALBSTRASSE = 'de:08116:7081'
	NEUFFEN_ALBSTRASSE_1 = 'de:08116:7081:0:3'  # eRtg Uracher Weg (Neuffen Neuffen)
	NEUFFEN_ALBSTRASSE_2 = 'de:08116:7081:0:4'  # eRtg Oberer Graben (Neuffen Neuffen)
	URACHER_WEG = 'de:08116:7082'
	URACHER_WEG_1 = 'de:08116:7082:0:3'  # eRtg Schule (Neuffen Neuffen)
	URACHER_WEG_2 = 'de:08116:7082:0:4'  # eRtg Albstraße (Neuffen Neuffen)
	NEUFFEN_URACHER_WEG = 'de:08116:7082'
	NEUFFEN_URACHER_WEG_1 = 'de:08116:7082:0:3'  # eRtg Schule (Neuffen Neuffen)
	NEUFFEN_URACHER_WEG_2 = 'de:08116:7082:0:4'  # eRtg Albstraße (Neuffen Neuffen)
	WINKELS = 'de:08118:7083'
	WINKELS_1 = 'de:08118:7083:0:3'  # eRtg Porsche (Sachsenheim Großsachsenheim)
	WINKELS_2 = 'de:08118:7083:0:4'  # eRtg Oberriexinger Straße (Sachsenheim Großsachsenheim)
	GROSSSACHSENHEIM_WINKELS = 'de:08118:7083'
	GROSSSACHSENHEIM_WINKELS_1 = 'de:08118:7083:0:3'  # eRtg Porsche (Sachsenheim Großsachsenheim)
	GROSSSACHSENHEIM_WINKELS_2 = 'de:08118:7083:0:4'  # eRtg Oberriexinger Straße (Sachsenheim Großsachsenheim)
	GROSSSACHSENHEIM_PORSCHE = 'de:08118:7084:0:3'  # Endstation (Sachsenheim Großsachsenheim)
	BESIGHEIM_KATH__KIRCHE = 'de:08118:7085'
	BESIGHEIM_KATH__KIRCHE_1 = 'de:08118:7085:0:3'  # eRtg Schwalbenhälde (Besigheim Besigheim)
	BESIGHEIM_KATH__KIRCHE_2 = 'de:08118:7085:0:4'  # eRtg Enzplatz (Besigheim Besigheim)
	SCHWALBENHÄLDE = 'de:08118:7086:0:3'  # eRtg Chausseebergstraße (Besigheim Besigheim)
	BESIGHEIM_SCHWALBENHÄLDE = 'de:08118:7086:0:3'  # eRtg Chausseebergstraße (Besigheim Besigheim)
	CHAUSSEEBERGSTRASSE = 'de:08118:7087:0:3'  # eRtg Martinshaus (Besigheim Besigheim)
	BESIGHEIM_CHAUSSEEBERGSTR = 'de:08118:7087:0:3'  # eRtg Martinshaus (Besigheim Besigheim)
	GROSSSACHSENHEIM_AM_FRIEDHOF = 'de:08118:7088'
	GROSSSACHSENHEIM_AM_FRIEDHOF_1 = 'de:08118:7088:0:3'  # Rtg Untermberg (Sachsenheim Großsachsenheim)
	GROSSSACHSENHEIM_AM_FRIEDHOF_2 = 'de:08118:7088:0:4'  # Rtg Großs. Hauptstr. (Sachsenheim Großsachsenheim)
	ALTDORF_OBERE_LIESÄCKER = 'de:08116:7089'
	ALTDORF_OBERE_LIESÄCKER_1 = 'de:08116:7089:0:3'  # eRtg Altdorf Stuttgarter Straße (Altdorf Altdorf (ES))
	ALTDORF_OBERE_LIESÄCKER_2 = 'de:08116:7089:0:4'  # eRtg Schule (Altdorf Altdorf (ES))
	OBERE_LIESÄCKER = 'de:08116:7089'
	OBERE_LIESÄCKER_1 = 'de:08116:7089:0:3'  # eRtg Altdorf Stuttgarter Straße (Altdorf Altdorf (ES))
	OBERE_LIESÄCKER_2 = 'de:08116:7089:0:4'  # eRtg Schule (Altdorf Altdorf (ES))
	WAIBLINGEN_WASSERSTUBENWEG = 'de:08119:7090:0:3'  # eRtg Talstraße (Waiblingen Waiblingen)
	WASSERSTUBENWEG = 'de:08119:7090:0:3'  # eRtg Talstraße (Waiblingen Waiblingen)
	MENDELSSOHNSTRASSE = 'de:08119:7091:0:3'  # eRtg Wasserstubenweg (Waiblingen Waiblingen)
	WAIBLINGEN_MENDELSSOHNSTR = 'de:08119:7091:0:3'  # eRtg Wasserstubenweg (Waiblingen Waiblingen)
	WAIBLINGEN_SCHELLINGSTR = 'de:08119:7092:0:3'  # eRtg Mendelssohnstraße (Waiblingen Waiblingen)
	WAIBLINGEN_ARBEITSAGENTUR_POST = 'de:08119:7093'
	WAIBLINGEN_ARBEITSAGENTUR_POST_1 = 'de:08119:7093:0:3'  # eRtg Emil-Münz-Straße (Waiblingen Waiblingen)
	WAIBLINGEN_ARBEITSAGENTUR_POST_2 = 'de:08119:7093:0:4'  # eRtg Devizesstraße (Waiblingen Waiblingen)
	ARBEITSAGENTUR_POST = 'de:08119:7093'
	ARBEITSAGENTUR_POST_1 = 'de:08119:7093:0:3'  # eRtg Emil-Münz-Straße (Waiblingen Waiblingen)
	ARBEITSAGENTUR_POST_2 = 'de:08119:7093:0:4'  # eRtg Devizesstraße (Waiblingen Waiblingen)
	EMIL_MÜNZ_STRASSE = 'de:08119:7094'
	EMIL_MÜNZ_STRASSE_1 = 'de:08119:7094:0:3'  # Rtg Waiblingen Schwabstraße (Waiblingen Waiblingen)
	EMIL_MÜNZ_STRASSE_2 = 'de:08119:7094:0:4'  # Rtg Arbeitsagentur/Post (Waiblingen Waiblingen)
	WAIBLINGEN_EMIL_MÜNZ_STR = 'de:08119:7094'
	WAIBLINGEN_EMIL_MÜNZ_STR_1 = 'de:08119:7094:0:3'  # Rtg Waiblingen Schwabstraße (Waiblingen Waiblingen)
	WAIBLINGEN_EMIL_MÜNZ_STR_2 = 'de:08119:7094:0:4'  # Rtg Arbeitsagentur/Post (Waiblingen Waiblingen)
	WAIBLINGEN_SCHWABSTR = 'de:08119:7095'
	WAIBLINGEN_SCHWABSTR_1 = 'de:08119:7095:0:3'  # eRtg Friedhof (Waiblingen Waiblingen)
	WAIBLINGEN_SCHWABSTR_2 = 'de:08119:7095:0:4'  # eRtg Emil-Münz-Straße (Waiblingen Waiblingen)
	KIRCHBERG__M__FEUERWEHR = 'de:08119:7096:0:3'  # Rtg Pfarrgartenstr. (Kirchberg an der Murr Kirchberg (M))
	KIRCHBERG__M__RAPPENBERG = 'de:08119:7097:0:3'  # Rtg Danziger Str. (Kirchberg an der Murr Kirchberg (M))
	RAPPENBERG = 'de:08119:7097:0:3'  # Rtg Danziger Str. (Kirchberg an der Murr Kirchberg (M))
	MALMSHEIM_VIEHTRIEB = 'de:08115:7098:0:3'  # Rtg Perouser Str. (Renningen Malmsheim)
	VIEHTRIEB = 'de:08115:7098:0:3'  # Rtg Perouser Str. (Renningen Malmsheim)
	BÖBLINGEN = 'de:08115:7100'
	BÖBLINGEN_1 = 'de:08115:7100:1:4'  # Gleis 4 (Böblingen Böblingen)
	BÖBLINGEN_2 = 'de:08115:7100:1:5'  # Gleis 5 (Böblingen Böblingen)
	BÖBLINGEN_3 = 'de:08115:7100:10:10'  # Bstg 10 (Böblingen Böblingen)
	BÖBLINGEN_4 = 'de:08115:7100:10:13'  # Bstg 13 (Böblingen Böblingen)
	BÖBLINGEN_5 = 'de:08115:7100:10:14'  # Bstg 14 (Böblingen Böblingen)
	BÖBLINGEN_6 = 'de:08115:7100:10:2'  # Bstg 2 (Böblingen Böblingen)
	BÖBLINGEN_7 = 'de:08115:7100:10:3'  # Bstg 3 (Böblingen Böblingen)
	BÖBLINGEN_8 = 'de:08115:7100:10:6'  # Bstg 6 (Böblingen Böblingen)
	BÖBLINGEN_9 = 'de:08115:7100:10:9'  # Bstg 9 (Böblingen Böblingen)
	BÖBLINGEN_10 = 'de:08115:7100:12:15'  # Bstg 15 (Böblingen Böblingen)
	BÖBLINGEN_11 = 'de:08115:7100:2:2'  # Gleis 2 (Böblingen Böblingen)
	BÖBLINGEN_12 = 'de:08115:7100:2:3'  # Gleis 3 (Böblingen Böblingen)
	BÖBLINGEN_13 = 'de:08115:7100:5:1'  # Gleis 1 (Böblingen Böblingen)
	BÖBLINGEN_14 = 'de:08115:7100:9:1'  # Bstg 1 (Böblingen Böblingen)
	BÖBLINGEN_15 = 'de:08115:7100:9:11'  # Bstg 11 (Böblingen Böblingen)
	BÖBLINGEN_16 = 'de:08115:7100:9:12'  # Bstg 12 (Böblingen Böblingen)
	BÖBLINGEN_17 = 'de:08115:7100:9:4'  # Bstg 4 (Böblingen Böblingen)
	BÖBLINGEN_18 = 'de:08115:7100:9:5'  # Bstg 5 (Böblingen Böblingen)
	BÖBLINGEN_19 = 'de:08115:7100:9:7'  # Bstg 7 (Böblingen Böblingen)
	BÖBLINGEN_20 = 'de:08115:7100:9:8'  # Bstg 8 (Böblingen Böblingen)
	ECHTERD__INDUSTRIEGEBIET = 'de:08116:7101'
	ECHTERD__INDUSTRIEGEBIET_1 = 'de:08116:7101:0:3'  # Rtg Bahnhof (Leinfelden-Echterdingen Echterdingen)
	ECHTERD__INDUSTRIEGEBIET_2 = 'de:08116:7101:0:4'  # Rtg Fasanenhof Abzw. (Leinfelden-Echterdingen Echterdingen)
	BAD_1 = 'de:08116:7102:1:3'  # Rtg Leinfelden Bf (Leinfelden-Echterdingen Leinfelden)
	BAD_2 = 'de:08116:7102:1:4'  # Rtg Meisenweg (Leinfelden-Echterdingen Leinfelden)
	BAD_3 = 'de:08117:46:0:H1'  # Rtg Autalhalle (Bad Überkingen Bad Überkingen)
	BAD_4 = 'de:08117:46:0:R1'  # Rtg Im Grund (Bad Überkingen Bad Überkingen)
	LEINFELDEN_BAD = 'de:08116:7102'
	LEINFELDEN_BAD_1 = 'de:08116:7102:1:3'  # Rtg Leinfelden Bf (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_BAD_2 = 'de:08116:7102:1:4'  # Rtg Meisenweg (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_SPIELKARTENMUSEUM = 'de:08116:7103'
	LEINFELDEN_SPIELKARTENMUSEUM_1 = 'de:08116:7103:0:1'  # Rtg Neuer Markt (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_SPIELKARTENMUSEUM_2 = 'de:08116:7103:0:2'  # Rtg Riedweg (Leinfelden-Echterdingen Leinfelden)
	SPIELKARTENMUSEUM = 'de:08116:7103'
	SPIELKARTENMUSEUM_1 = 'de:08116:7103:0:1'  # Rtg Neuer Markt (Leinfelden-Echterdingen Leinfelden)
	SPIELKARTENMUSEUM_2 = 'de:08116:7103:0:2'  # Rtg Riedweg (Leinfelden-Echterdingen Leinfelden)
	MUSBERG_SEEBRUCKENMÜHLE = 'de:08116:7104'
	MUSBERG_SEEBRUCKENMÜHLE_1 = 'de:08116:7104:1:3'  # Rtg Steinenbronn Goldäcker (Leinfelden-Echterdingen Musberg)
	MUSBERG_SEEBRUCKENMÜHLE_2 = 'de:08116:7104:1:4'  # Rtg Leinfelden Sportzentrum (Leinfelden-Echterdingen Musberg)
	SEEBRUCKENMÜHLE = 'de:08116:7104'
	SEEBRUCKENMÜHLE_1 = 'de:08116:7104:1:3'  # Rtg Steinenbronn Goldäcker (Leinfelden-Echterdingen Musberg)
	SEEBRUCKENMÜHLE_2 = 'de:08116:7104:1:4'  # Rtg Leinfelden Sportzentrum (Leinfelden-Echterdingen Musberg)
	WANDERWEG = 'de:08116:7105'
	WANDERWEG_1 = 'de:08116:7105:0:1'  # Rtg Leinfelden Riedweg (Leinfelden-Echterdingen Musberg)
	WANDERWEG_2 = 'de:08116:7105:0:2'  # Rtg Musberg Friedhof/Kirche (Leinfelden-Echterdingen Musberg)
	WANDERWEG_3 = 'de:08116:7105:0:3'  # Linie 38 Rtg Musberg Kirche (Leinfelden-Echterdingen Musberg)
	MUSBERG_WANDERWEG = 'de:08116:7105'
	MUSBERG_WANDERWEG_1 = 'de:08116:7105:0:1'  # Rtg Leinfelden Riedweg (Leinfelden-Echterdingen Musberg)
	MUSBERG_WANDERWEG_2 = 'de:08116:7105:0:2'  # Rtg Musberg Friedhof/Kirche (Leinfelden-Echterdingen Musberg)
	MUSBERG_WANDERWEG_3 = 'de:08116:7105:0:3'  # Linie 38 Rtg Musberg Kirche (Leinfelden-Echterdingen Musberg)
	STEINENBRONN_KIRCHE = 'de:08115:7106'
	STEINENBRONN_KIRCHE_1 = 'de:08115:7106:1:3'  # Rtg Waldenbuch Hasenhof (Steinenbronn Steinenbronn)
	STEINENBRONN_KIRCHE_2 = 'de:08115:7106:1:4'  # Rtg Im Alten See/Schönaich Sulzbachtal (Steinenbronn Steinenbronn)
	STEINENBRONN_IM_ALTEN_SEE = 'de:08115:7108'
	STEINENBRONN_IM_ALTEN_SEE_1 = 'de:08115:7108:0:3'  # Rtg Kirche (Steinenbronn Steinenbronn)
	STEINENBRONN_IM_ALTEN_SEE_2 = 'de:08115:7108:0:4'  # Rtg Goldäcker (Steinenbronn Steinenbronn)
	IM_ALTEN_SEE = 'de:08115:7108'
	IM_ALTEN_SEE_1 = 'de:08115:7108:0:3'  # Rtg Kirche (Steinenbronn Steinenbronn)
	IM_ALTEN_SEE_2 = 'de:08115:7108:0:4'  # Rtg Goldäcker (Steinenbronn Steinenbronn)
	STEINENBRONN_GOLDÄCKER = 'de:08115:7111'
	STEINENBRONN_GOLDÄCKER_1 = 'de:08115:7111:1:3'  # Rtg Im Alten See/Waldenbuch Postamt (Steinenbronn Steinenbronn)
	STEINENBRONN_GOLDÄCKER_2 = 'de:08115:7111:1:4'  # Rtg Musb. Seebruckenmühle (Steinenbronn Steinenbronn)
	GOLDÄCKER = 'de:08115:7111'
	GOLDÄCKER_1 = 'de:08115:7111:1:3'  # Rtg Im Alten See/Waldenbuch Postamt (Steinenbronn Steinenbronn)
	GOLDÄCKER_2 = 'de:08115:7111:1:4'  # Rtg Musb. Seebruckenmühle (Steinenbronn Steinenbronn)
	MUSB__SEEBRUCKENMÜHLE__KLÄRW = 'de:08116:7112'
	MUSB__SEEBRUCKENMÜHLE__KLÄRW_1 = 'de:08116:7112:1:3'  # Rtg Steinenbronn Goldäcker (Leinfelden-Echterdingen Musberg)
	MUSB__SEEBRUCKENMÜHLE__KLÄRW_2 = 'de:08116:7112:1:4'  # Rtg Eselsmühle (Leinfelden-Echterdingen Musberg)
	SEEBRUCKENMÜHLE__KLÄRWERK = 'de:08116:7112'
	SEEBRUCKENMÜHLE__KLÄRWERK_1 = 'de:08116:7112:1:3'  # Rtg Steinenbronn Goldäcker (Leinfelden-Echterdingen Musberg)
	SEEBRUCKENMÜHLE__KLÄRWERK_2 = 'de:08116:7112:1:4'  # Rtg Eselsmühle (Leinfelden-Echterdingen Musberg)
	HULB = 'de:08115:7115'
	HULB_1 = 'de:08115:7115:1:1'  # Gleis 1 (Böblingen Böblingen)
	HULB_2 = 'de:08115:7115:1:2'  # Gleis 2 (Böblingen Böblingen)
	HULB_3 = 'de:08115:7115:3:3'  # eRtg Dornierstraße (Böblingen Böblingen)
	HULB_4 = 'de:08115:7115:6:4'  # Bus Südseite (Böblingen Böblingen)
	STEINENBRONN_ROHRER_STRASSE = 'de:08115:7116:0:1'  # Rtg Sindelfinger Str (Steinenbronn Steinenbronn)
	ROHRER_STRASSE = 'de:08115:7116:0:1'  # Rtg Sindelfinger Str (Steinenbronn Steinenbronn)
	STEINENBRONN_KRING = 'de:08115:7118'
	STEINENBRONN_KRING_1 = 'de:08115:7118:0:1'  # Rtg Im Alten See (Steinenbronn Steinenbronn)
	STEINENBRONN_KRING_2 = 'de:08115:7118:0:2'  # Rtg Maurer (Steinenbronn Steinenbronn)
	KRING = 'de:08115:7118'
	KRING_1 = 'de:08115:7118:0:1'  # Rtg Im Alten See (Steinenbronn Steinenbronn)
	KRING_2 = 'de:08115:7118:0:2'  # Rtg Maurer (Steinenbronn Steinenbronn)
	MAURER = 'de:08115:7119'
	MAURER_1 = 'de:08115:7119:0:1'  # Rtg Kring (Steinenbronn Steinenbronn)
	MAURER_2 = 'de:08115:7119:0:2'  # Rtg Kirche (Steinenbronn Steinenbronn)
	STEINENBRONN_MAURER = 'de:08115:7119'
	STEINENBRONN_MAURER_1 = 'de:08115:7119:0:1'  # Rtg Kring (Steinenbronn Steinenbronn)
	STEINENBRONN_MAURER_2 = 'de:08115:7119:0:2'  # Rtg Kirche (Steinenbronn Steinenbronn)
	STEINENBRONN_LINDENSTRASSE = 'de:08115:7120'
	STEINENBRONN_LINDENSTRASSE_1 = 'de:08115:7120:0:1'  # Rtg Goldäcker (Steinenbronn Steinenbronn)
	STEINENBRONN_LINDENSTRASSE_2 = 'de:08115:7120:0:2'  # Rtg Waldenbuch (Steinenbronn Steinenbronn)
	GERLINGEN = 'de:08118:7140'
	GERLINGEN_1 = 'de:08118:7140:1:1'  # Stadtbahn - Endhst. (Gerlingen Gerlingen)
	GERLINGEN_2 = 'de:08118:7140:2:4'  # Bus-Li98 Rtg Ditzingen Bf (Gerlingen Gerlingen)
	GERLINGEN_RATHAUS = 'de:08118:7141:1:3'  # Rtg Alte Steige (Gerlingen Gerlingen)
	RIEDBRUNNEN = 'de:08115:7151:0:3'  # alle Richtungen (Gärtringen Gärtringen)
	GÄRTRINGEN_RIEDBRUNNEN = 'de:08115:7151:0:3'  # alle Richtungen (Gärtringen Gärtringen)
	SCHELMENWIESEN = 'de:08115:7152'
	SCHELMENWIESEN_1 = 'de:08115:7152:0:3'  # Rtg Bahnhof (Gärtringen Gärtringen)
	SCHELMENWIESEN_2 = 'de:08115:7152:0:4'  # Rtg Riedbrunnen (Gärtringen Gärtringen)
	GÄRTRINGEN_SCHELMENWIESEN = 'de:08115:7152'
	GÄRTRINGEN_SCHELMENWIESEN_1 = 'de:08115:7152:0:3'  # Rtg Bahnhof (Gärtringen Gärtringen)
	GÄRTRINGEN_SCHELMENWIESEN_2 = 'de:08115:7152:0:4'  # Rtg Riedbrunnen (Gärtringen Gärtringen)
	LAMMTAL = 'de:08115:7153'
	LAMMTAL_1 = 'de:08115:7153:0:3'  # Rtg Rathaus (Gärtringen Gärtringen)
	LAMMTAL_2 = 'de:08115:7153:0:4'  # Rtg Aidlingen/Deufringen (Gärtringen Gärtringen)
	GÄRTRINGEN_LAMMTAL = 'de:08115:7153'
	GÄRTRINGEN_LAMMTAL_1 = 'de:08115:7153:0:3'  # Rtg Rathaus (Gärtringen Gärtringen)
	GÄRTRINGEN_LAMMTAL_2 = 'de:08115:7153:0:4'  # Rtg Aidlingen/Deufringen (Gärtringen Gärtringen)
	LEINFELDEN_FRANK = 'de:08116:7173'
	LEINFELDEN_FRANK_1 = 'de:08116:7173:0:1'  # Rtg Unteraichen (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_FRANK_2 = 'de:08116:7173:0:2'  # Rtg Möhringen Freibad (Leinfelden-Echterdingen Leinfelden)
	FRANK = 'de:08116:7173'
	FRANK_1 = 'de:08116:7173:0:1'  # Rtg Unteraichen (Leinfelden-Echterdingen Leinfelden)
	FRANK_2 = 'de:08116:7173:0:2'  # Rtg Möhringen Freibad (Leinfelden-Echterdingen Leinfelden)
	LEINF__LEINFELDER_HAUS = 'de:08116:7176'
	LEINF__LEINFELDER_HAUS_1 = 'de:08116:7176:0:3'  # Rtg Echterd. Gymnasium (Leinfelden-Echterdingen Leinfelden)
	LEINF__LEINFELDER_HAUS_2 = 'de:08116:7176:0:4'  # Rtg Neuer Markt (Leinfelden-Echterdingen Leinfelden)
	LEINFELDER_HAUS = 'de:08116:7176'
	LEINFELDER_HAUS_1 = 'de:08116:7176:0:3'  # Rtg Echterd. Gymnasium (Leinfelden-Echterdingen Leinfelden)
	LEINFELDER_HAUS_2 = 'de:08116:7176:0:4'  # Rtg Neuer Markt (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_SPORTZENTRUM = 'de:08116:7177'
	LEINFELDEN_SPORTZENTRUM_1 = 'de:08116:7177:0:3'  # Rtg Musberg Seebruckenmühle (Leinfelden-Echterdingen Leinfelden)
	LEINFELDEN_SPORTZENTRUM_2 = 'de:08116:7177:0:4'  # Rtg Echterdingen Kelterrain (Leinfelden-Echterdingen Leinfelden)
	ECHTERDINGEN_HIRSCHSTR = 'de:08116:7178'
	ECHTERDINGEN_HIRSCHSTR_1 = 'de:08116:7178:1:4'  # Rtg Echterdingen Bf (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_HIRSCHSTR_2 = 'de:08116:7178:2:3'  # Rtg Waldhornstr. (Leinfelden-Echterdingen Echterdingen)
	WALDHORNSTRASSE = 'de:08116:7179'
	WALDHORNSTRASSE_1 = 'de:08116:7179:0:3'  # Rtg Bonländer Str./Tübinger Str. (Leinfelden-Echterdingen Echterdingen)
	WALDHORNSTRASSE_2 = 'de:08116:7179:0:4'  # Rtg Hirschstr. (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_WALDHORNSTRASSE = 'de:08116:7179'
	ECHTERDINGEN_WALDHORNSTRASSE_1 = 'de:08116:7179:0:3'  # Rtg Bonländer Str./Tübinger Str. (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_WALDHORNSTRASSE_2 = 'de:08116:7179:0:4'  # Rtg Hirschstr. (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_STADIONSTR = 'de:08116:7180'
	ECHTERDINGEN_STADIONSTR_1 = 'de:08116:7180:1:1'  # Rtg Flughafen (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_STADIONSTR_2 = 'de:08116:7180:1:2'  # Rtg Fasanenhof (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_STADIONSTR_3 = 'de:08116:7180:1:4'  # Rtg Süd (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_STADIONSTR_4 = 'de:08116:7180:2:3'  # Rtg Heilbronnerstr. (Leinfelden-Echterdingen Echterdingen)
	STADIONSTRASSE = 'de:08116:7180'
	STADIONSTRASSE_1 = 'de:08116:7180:1:1'  # Rtg Flughafen (Leinfelden-Echterdingen Echterdingen)
	STADIONSTRASSE_2 = 'de:08116:7180:1:2'  # Rtg Fasanenhof (Leinfelden-Echterdingen Echterdingen)
	STADIONSTRASSE_3 = 'de:08116:7180:1:4'  # Rtg Süd (Leinfelden-Echterdingen Echterdingen)
	STADIONSTRASSE_4 = 'de:08116:7180:2:3'  # Rtg Heilbronnerstr. (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_MESSE_WEST = 'de:08116:7182'
	ECHTERDINGEN_MESSE_WEST_1 = 'de:08116:7182:0:3'  # Rtg Im Köpfert/Fas. Abzw./Echterd. Bf (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_MESSE_WEST_2 = 'de:08116:7182:0:4'  # Rtg Flughafen SAB (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_MESSE_WEST_3 = 'de:08116:7182:1:1'  # Rtg Flughafen (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_MESSE_WEST_4 = 'de:08116:7182:2:2'  # Rtg Stadionstr (Leinfelden-Echterdingen Echterdingen)
	MESSE_WEST = 'de:08116:7182'
	MESSE_WEST_1 = 'de:08116:7182:0:3'  # Rtg Im Köpfert/Fas. Abzw./Echterd. Bf (Leinfelden-Echterdingen Echterdingen)
	MESSE_WEST_2 = 'de:08116:7182:0:4'  # Rtg Flughafen SAB (Leinfelden-Echterdingen Echterdingen)
	MESSE_WEST_3 = 'de:08116:7182:1:1'  # Rtg Flughafen (Leinfelden-Echterdingen Echterdingen)
	MESSE_WEST_4 = 'de:08116:7182:2:2'  # Rtg Stadionstr (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_GYMNASIUM = 'de:08116:7185'
	ECHTERDINGEN_GYMNASIUM_1 = 'de:08116:7185:0:3'  # Rtg Stangen (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_GYMNASIUM_2 = 'de:08116:7185:0:4'  # Rtg Leinfelder Haus (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_STANGEN = 'de:08116:7186'
	ECHTERDINGEN_STANGEN_1 = 'de:08116:7186:0:3'  # Rtg M.-Luther-Straße (Leinfelden-Echterdingen Echterdingen)
	ECHTERDINGEN_STANGEN_2 = 'de:08116:7186:0:4'  # Rtg Gymnasium (Leinfelden-Echterdingen Echterdingen)
	STANGEN = 'de:08116:7186'
	STANGEN_1 = 'de:08116:7186:0:3'  # Rtg M.-Luther-Straße (Leinfelden-Echterdingen Echterdingen)
	STANGEN_2 = 'de:08116:7186:0:4'  # Rtg Gymnasium (Leinfelden-Echterdingen Echterdingen)
	SINDELF__MÖNCHSBRUNNEN = 'de:08115:7201'
	SINDELF__MÖNCHSBRUNNEN_1 = 'de:08115:7201:0:3'  # Rtg Messehalle (Sindelfingen Sindelfingen)
	SINDELF__MÖNCHSBRUNNEN_2 = 'de:08115:7201:0:4'  # Rtg Waldheim (Sindelfingen Sindelfingen)
	MÖNCHSBRUNNEN = 'de:08115:7201'
	MÖNCHSBRUNNEN_1 = 'de:08115:7201:0:3'  # Rtg Messehalle (Sindelfingen Sindelfingen)
	MÖNCHSBRUNNEN_2 = 'de:08115:7201:0:4'  # Rtg Waldheim (Sindelfingen Sindelfingen)
	ZWERCHWEG = 'de:08115:7202:1:1'  # Gleis 1 (Herrenberg Herrenberg)
	HERRENBERG_ZWERCHWEG = 'de:08115:7202:1:1'  # Gleis 1 (Herrenberg Herrenberg)
	LEONBERG = 'de:08115:7301'
	LEONBERG_1 = 'de:08115:7301:1:1'  # Gleis 1 (Leonberg Leonberg)
	LEONBERG_2 = 'de:08115:7301:1:8'  # Bstg 8 (Leonberg Leonberg)
	LEONBERG_3 = 'de:08115:7301:2:2'  # Gleis 2 (Leonberg Leonberg)
	LEONBERG_4 = 'de:08115:7301:3:1'  # Bstg 1 (Leonberg Leonberg)
	LEONBERG_5 = 'de:08115:7301:3:2'  # Bstg 2 (Leonberg Leonberg)
	LEONBERG_6 = 'de:08115:7301:3:3'  # Bstg 3 (Leonberg Leonberg)
	LEONBERG_7 = 'de:08115:7301:3:4'  # Bstg 4 (Leonberg Leonberg)
	LEONBERG_8 = 'de:08115:7301:3:5'  # Bstg 5 (Leonberg Leonberg)
	LEONBERG_9 = 'de:08115:7301:3:6'  # Bstg 6 (Leonberg Leonberg)
	LEONBERG_10 = 'de:08115:7301:3:7'  # Ruftaxi (Leonberg Leonberg)
	RENNINGEN = 'de:08115:7302'
	RENNINGEN_1 = 'de:08115:7302:1:2'  # Gleis 2 (Renningen Renningen)
	RENNINGEN_2 = 'de:08115:7302:1:3'  # Gleis 3 (Renningen Renningen)
	RENNINGEN_3 = 'de:08115:7302:2:3'  # Rtg alle Richtungen (Renningen Renningen)
	LEOBAD = 'de:08115:7304'
	LEOBAD_1 = 'de:08115:7304:0:3'  # Rtg Hertichstr. (Leonberg Eltingen)
	LEOBAD_2 = 'de:08115:7304:0:4'  # Rtg Schminckestr. (Leonberg Eltingen)
	ELTINGEN_LEOBAD = 'de:08115:7304'
	ELTINGEN_LEOBAD_1 = 'de:08115:7304:0:3'  # Rtg Hertichstr. (Leonberg Eltingen)
	ELTINGEN_LEOBAD_2 = 'de:08115:7304:0:4'  # Rtg Schminckestr. (Leonberg Eltingen)
	LÄNGENBÜHL = 'de:08115:7305:0:4'  # Rtg Hertichstr. 73 (Leonberg Eltingen)
	ELTINGEN_LÄNGENBÜHL = 'de:08115:7305:0:4'  # Rtg Hertichstr. 73 (Leonberg Eltingen)
	ASPERG = 'de:08118:7400'
	ASPERG_1 = 'de:08118:7400:1:3'  # Gleis 3 (Asperg Asperg)
	ASPERG_2 = 'de:08118:7400:1:4'  # Gleis 4 (Asperg Asperg)
	ASPERG_3 = 'de:08118:7400:2:1'  # Rtg Nord (Asperg Asperg)
	ASPERG_4 = 'de:08118:7400:2:6'  # Rtg Süd (Asperg Asperg)
	ASPERG_5 = 'de:08118:7400:3:3'  # Rtg Nord (Asperg Asperg)
	LUDWIGSBURG_BAHNHOF__ARENA = 'de:08118:7401'
	LUDWIGSBURG_BAHNHOF__ARENA_1 = 'de:08118:7401:1:3'  # Rtg Bahnhof/Ruhrstr (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_BAHNHOF__ARENA_2 = 'de:08118:7401:1:4'  # Rtg Hoferstr (Ludwigsburg Ludwigsburg)
	BAHNHOF__ARENA = 'de:08118:7401'
	BAHNHOF__ARENA_1 = 'de:08118:7401:1:3'  # Rtg Bahnhof/Ruhrstr (Ludwigsburg Ludwigsburg)
	BAHNHOF__ARENA_2 = 'de:08118:7401:1:4'  # Rtg Hoferstr (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG = 'de:08118:7402'
	LUDWIGSBURG_1 = 'de:08118:7402:1:2'  # Gleis 2 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_2 = 'de:08118:7402:1:3'  # Gleis 3 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_3 = 'de:08118:7402:2:4'  # Gleis 4 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_4 = 'de:08118:7402:3:1'  # Gleis 1 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_5 = 'de:08118:7402:4:1'  # Bstg 1 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_6 = 'de:08118:7402:4:10'  # Bstg 10 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_7 = 'de:08118:7402:4:13'  # Bstg 13 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_8 = 'de:08118:7402:4:2'  # Bstg 2 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_9 = 'de:08118:7402:4:6'  # Bstg 6 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_10 = 'de:08118:7402:5:11'  # Bstg 11 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_11 = 'de:08118:7402:5:12'  # Bstg 12 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_12 = 'de:08118:7402:5:15'  # Bstg 15 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_13 = 'de:08118:7402:5:16'  # Bstg 16 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_14 = 'de:08118:7402:5:17'  # Bstg 17 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_15 = 'de:08118:7402:5:20'  # Ludwigsburg (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_16 = 'de:08118:7402:5:3'  # Bstg 3 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_17 = 'de:08118:7402:5:4'  # Bstg 4 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_18 = 'de:08118:7402:5:5'  # Bstg 5 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_19 = 'de:08118:7402:5:7'  # Bstg 7 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_20 = 'de:08118:7402:5:8'  # Bstg 8 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_21 = 'de:08118:7402:5:9'  # Bstg 9 (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_22 = 'de:08118:7402:6:18'  # Bstg 18 (Ludwigsburg Ludwigsburg)
	FAVORITEPARK = 'de:08118:7403'
	FAVORITEPARK_1 = 'de:08118:7403:1:2'  # Gleis 2 (Ludwigsburg Eglosheim)
	FAVORITEPARK_2 = 'de:08118:7403:2:1'  # Gleis 1 (Ludwigsburg Eglosheim)
	TAMM = 'de:08118:7404'
	TAMM_1 = 'de:08118:7404:1:2'  # Gleis 2 (Tamm Tamm)
	TAMM_2 = 'de:08118:7404:1:3'  # Gleis 3 (Tamm Tamm)
	TAMM_3 = 'de:08118:7404:2:3'  # Rtg Ludwigsburger Str. (Tamm Tamm)
	TAMM_4 = 'de:08118:7404:2:4'  # Rtg Alter Weg (Tamm Tamm)
	BIETIGHEIM_SCHILLERPLATZ = 'de:08118:7405'
	BIETIGHEIM_SCHILLERPLATZ_1 = 'de:08118:7405:0:3'  # eRtg Kronenzentrum (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_SCHILLERPLATZ_2 = 'de:08118:7405:0:4'  # eRtg Bolzstraße (Bietigheim-Bissingen Bietigheim)
	BISSINGEN_GEROKSTRASSE = 'de:08118:7406'
	BISSINGEN_GEROKSTRASSE_1 = 'de:08118:7406:0:3'  # eRtg Kelterstraße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_GEROKSTRASSE_2 = 'de:08118:7406:0:4'  # eRtg Rathaus (Bietigheim-Bissingen Bissingen (LB))
	GEROKSTRASSE = 'de:08118:7406'
	GEROKSTRASSE_1 = 'de:08118:7406:0:3'  # eRtg Kelterstraße (Bietigheim-Bissingen Bissingen (LB))
	GEROKSTRASSE_2 = 'de:08118:7406:0:4'  # eRtg Rathaus (Bietigheim-Bissingen Bissingen (LB))
	BIETIGHEIM_STEINHEIMER_STR = 'de:08118:7407'
	BIETIGHEIM_STEINHEIMER_STR_1 = 'de:08118:7407:0:3'  # eRtg Gustav-Rau-Straße (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_STEINHEIMER_STR_2 = 'de:08118:7407:0:4'  # Rtg Höpfigheimer Str. (Bietigheim-Bissingen Bietigheim)
	PLEIDELSHEIMER_STRASSE = 'de:08118:7408:0:4'  # Rtg Steinheimer Str. (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_PLEIDELSHEIMER_STR = 'de:08118:7408:0:4'  # Rtg Steinheimer Str. (Bietigheim-Bissingen Bietigheim)
	SCHLOSSBERGHALLE = 'de:08116:7409'
	SCHLOSSBERGHALLE_1 = 'de:08116:7409:0:3'  # eRtg Katholische Kirche (Dettingen unter Teck Dettingen (T))
	SCHLOSSBERGHALLE_2 = 'de:08116:7409:0:4'  # eRtg Obere Straße (Dettingen unter Teck Dettingen (T))
	DETT__T__SCHLOSSBERGHALLE = 'de:08116:7409'
	DETT__T__SCHLOSSBERGHALLE_1 = 'de:08116:7409:0:3'  # eRtg Katholische Kirche (Dettingen unter Teck Dettingen (T))
	DETT__T__SCHLOSSBERGHALLE_2 = 'de:08116:7409:0:4'  # eRtg Obere Straße (Dettingen unter Teck Dettingen (T))
	GROSSBETTLINGEN_RATHAUS = 'de:08116:7413'
	GROSSBETTLINGEN_RATHAUS_1 = 'de:08116:7413:0:3'  # eRtg Bettlinger Forum/Herion (Großbettlingen Großbettlingen)
	GROSSBETTLINGEN_RATHAUS_2 = 'de:08116:7413:0:4'  # eRtg Heerweg (Großbettlingen Großbettlingen)
	BIETIGHEIM_HÖPFIGHEIMER_STR = 'de:08118:7414:0:3'  # eRtg Pleidelsheimer Straße (Bietigheim-Bissingen Bietigheim)
	SERSHEIM_INDUSTRIESTR = 'de:08118:7415'
	SERSHEIM_INDUSTRIESTR_1 = 'de:08118:7415:0:3'  # Rtg Kleinglattbach (Sersheim Sersheim)
	SERSHEIM_INDUSTRIESTR_2 = 'de:08118:7415:0:4'  # Rtg Sersheim (Sersheim Sersheim)
	ENZWEIHINGEN_BEERHALDENSTR = 'de:08118:7416'
	ENZWEIHINGEN_BEERHALDENSTR_1 = 'de:08118:7416:0:3'  # eRtg Enzweihingen B10 (Vaihingen an der Enz Enzweihingen)
	ENZWEIHINGEN_BEERHALDENSTR_2 = 'de:08118:7416:0:4'  # eRtg Rieter Tal (Vaihingen an der Enz Enzweihingen)
	BEERHALDENSTRASSE = 'de:08118:7416'
	BEERHALDENSTRASSE_1 = 'de:08118:7416:0:3'  # eRtg Enzweihingen B10 (Vaihingen an der Enz Enzweihingen)
	BEERHALDENSTRASSE_2 = 'de:08118:7416:0:4'  # eRtg Rieter Tal (Vaihingen an der Enz Enzweihingen)
	SERSHEIM_BAHNHOFSTR = 'de:08118:7417'
	SERSHEIM_BAHNHOFSTR_1 = 'de:08118:7417:1:3'  # Rtg Danziger Weg (Sersheim Sersheim)
	SERSHEIM_BAHNHOFSTR_2 = 'de:08118:7417:1:4'  # Rtg Vaihinger Straße (Sersheim Sersheim)
	BILDUNGSZENTRUM_WEST = 'de:08118:7419:0:3'  # alle Richtungen (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_BILDUNGSZ__WEST = 'de:08118:7419:0:3'  # alle Richtungen (Ludwigsburg Ludwigsburg)
	SERSHEIM_SEDANSTRASSE = 'de:08118:7421'
	SERSHEIM_SEDANSTRASSE_1 = 'de:08118:7421:0:3'  # eRtg Bahnhofstraße (Sersheim Sersheim)
	SERSHEIM_SEDANSTRASSE_2 = 'de:08118:7421:0:4'  # eRtg Industriestraße (Sersheim Sersheim)
	SEDANSTRASSE = 'de:08118:7421'
	SEDANSTRASSE_1 = 'de:08118:7421:0:3'  # eRtg Bahnhofstraße (Sersheim Sersheim)
	SEDANSTRASSE_2 = 'de:08118:7421:0:4'  # eRtg Industriestraße (Sersheim Sersheim)
	BISSINGEN_NEUER_FRIEDHOF = 'de:08118:7422'
	BISSINGEN_NEUER_FRIEDHOF_1 = 'de:08118:7422:0:3'  # Ankunft (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_NEUER_FRIEDHOF_2 = 'de:08118:7422:0:4'  # eRtg Zahnklinik (Bietigheim-Bissingen Bissingen (LB))
	ERLIGHEIMER_STRASSE = 'de:08118:7423'
	ERLIGHEIMER_STRASSE_1 = 'de:08118:7423:0:3'  # eRtg Erligheim (Löchgau Löchgau)
	ERLIGHEIMER_STRASSE_2 = 'de:08118:7423:0:4'  # eRtg Wette (Löchgau Löchgau)
	LÖCHGAU_ERLIGHEIMER_STRASSE = 'de:08118:7423'
	LÖCHGAU_ERLIGHEIMER_STRASSE_1 = 'de:08118:7423:0:3'  # eRtg Erligheim (Löchgau Löchgau)
	LÖCHGAU_ERLIGHEIMER_STRASSE_2 = 'de:08118:7423:0:4'  # eRtg Wette (Löchgau Löchgau)
	BISSINGEN_SÜDSTRASSE = 'de:08118:7424'
	BISSINGEN_SÜDSTRASSE_1 = 'de:08118:7424:0:3'  # bis j05 (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_SÜDSTRASSE_2 = 'de:08118:7424:0:4'  # ab j06 (Bietigheim-Bissingen Bissingen (LB))
	SÜDSTRASSE = 'de:08118:7424'
	SÜDSTRASSE_1 = 'de:08118:7424:0:3'  # bis j05 (Bietigheim-Bissingen Bissingen (LB))
	SÜDSTRASSE_2 = 'de:08118:7424:0:4'  # ab j06 (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_LINDENSTRASSE = 'de:08118:7425'
	BISSINGEN_LINDENSTRASSE_1 = 'de:08118:7425:0:3'  # eRtg Südstraße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_LINDENSTRASSE_2 = 'de:08118:7425:0:4'  # eRtg Eichenweg (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_EICHENWEG = 'de:08118:7426'
	BISSINGEN_EICHENWEG_1 = 'de:08118:7426:0:3'  # eRtg Lindenstraße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_EICHENWEG_2 = 'de:08118:7426:0:4'  # eRtg Entenäcker (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_ENTENÄCKER = 'de:08118:7427'
	BISSINGEN_ENTENÄCKER_1 = 'de:08118:7427:0:3'  # eRtg Eichenweg (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_ENTENÄCKER_2 = 'de:08118:7427:0:4'  # eRtg Westendstraße (Bietigheim-Bissingen Bissingen (LB))
	ENTENÄCKER = 'de:08118:7427'
	ENTENÄCKER_1 = 'de:08118:7427:0:3'  # eRtg Eichenweg (Bietigheim-Bissingen Bissingen (LB))
	ENTENÄCKER_2 = 'de:08118:7427:0:4'  # eRtg Westendstraße (Bietigheim-Bissingen Bissingen (LB))
	WESTENDSTRASSE = 'de:08118:7428'
	WESTENDSTRASSE_1 = 'de:08118:7428:0:3'  # eRtg Entenäcker (Bietigheim-Bissingen Bissingen (LB))
	WESTENDSTRASSE_2 = 'de:08118:7428:0:4'  # eRtg Birkenweg (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_WESTENDSTRASSE = 'de:08118:7428'
	BISSINGEN_WESTENDSTRASSE_1 = 'de:08118:7428:0:3'  # eRtg Entenäcker (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_WESTENDSTRASSE_2 = 'de:08118:7428:0:4'  # eRtg Birkenweg (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_BIRKENWEG = 'de:08118:7429'
	BISSINGEN_BIRKENWEG_1 = 'de:08118:7429:0:3'  # eRtg Westendstraße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_BIRKENWEG_2 = 'de:08118:7429:0:4'  # eRtg Kelterstraße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_HALLENBAD = 'de:08118:7430'
	BISSINGEN_HALLENBAD_1 = 'de:08118:7430:0:3'  # eRtg Bissinger Straße (Bietigheim-Bissingen Bissingen (LB))
	BISSINGEN_HALLENBAD_2 = 'de:08118:7430:0:4'  # eRtg Bietigheim (Bietigheim-Bissingen Bissingen (LB))
	SCHILLERSCHULE_1 = 'de:08118:7431:0:3'  # alle Richtungen (Bietigheim-Bissingen Bissingen (LB))
	SCHILLERSCHULE_2 = 'de:08119:7549:0:3'  # Rtg Beethovenstr (Waiblingen Bittenfeld)
	BISSINGEN__LB__SCHILLERSCHULE = 'de:08118:7431:0:3'  # alle Richtungen (Bietigheim-Bissingen Bissingen (LB))
	MAINZER_ALLEE = 'de:08118:7432'
	MAINZER_ALLEE_1 = 'de:08118:7432:0:3'  # eRtg Kornbeckstraße (Ludwigsburg Ludwigsburg)
	MAINZER_ALLEE_2 = 'de:08118:7432:0:4'  # eRtg Harteneckstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_MAINZER_ALLEE = 'de:08118:7432'
	LUDWIGSBURG_MAINZER_ALLEE_1 = 'de:08118:7432:0:3'  # eRtg Kornbeckstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_MAINZER_ALLEE_2 = 'de:08118:7432:0:4'  # eRtg Harteneckstraße (Ludwigsburg Ludwigsburg)
	BERLINER_PLATZ = 'de:08118:7433'
	BERLINER_PLATZ_1 = 'de:08118:7433:0:3'  # eRtg Danziger Straße (Ludwigsburg Ludwigsburg)
	BERLINER_PLATZ_2 = 'de:08118:7433:0:4'  # eRtg Neckarstraße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_BERLINER_PLATZ = 'de:08118:7433'
	LUDWIGSBURG_BERLINER_PLATZ_1 = 'de:08118:7433:0:3'  # eRtg Danziger Straße (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_BERLINER_PLATZ_2 = 'de:08118:7433:0:4'  # eRtg Neckarstraße (Ludwigsburg Ludwigsburg)
	ALTER_OSSWEILER_WEG = 'de:08118:7435:0:3'  # Rtg Berliner Platz (Ludwigsburg Ludwigsburg)
	LUDW__ALTER_OSSWEILER_WEG = 'de:08118:7435:0:3'  # Rtg Berliner Platz (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_FRIEDRICHSTRASSE = 'de:08118:7436'
	LUDWIGSBURG_FRIEDRICHSTRASSE_1 = 'de:08118:7436:0:3'  # eRtg Remseck (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_FRIEDRICHSTRASSE_2 = 'de:08118:7436:0:4'  # eRtg Ludwigsburg (Ludwigsburg Ludwigsburg)
	LUDW__THEO_LORCH_WERKSTÄTTEN = 'de:08118:7438'
	LUDW__THEO_LORCH_WERKSTÄTTEN_1 = 'de:08118:7438:0:3'  # eRtg Ludwigsburg (Ludwigsburg Ludwigsburg)
	LUDW__THEO_LORCH_WERKSTÄTTEN_2 = 'de:08118:7438:0:4'  # eRtg Grünbühl/Pattonville (Ludwigsburg Ludwigsburg)
	BIETIGHEIM_FR__EBERT_STRASSE = 'de:08118:7440'
	BIETIGHEIM_FR__EBERT_STRASSE_1 = 'de:08118:7440:0:3'  # eRtg Grünwiesenstraße (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_FR__EBERT_STRASSE_2 = 'de:08118:7440:0:4'  # eRtg Bolzstraße (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_GRÜNWIESENSTR = 'de:08118:7441'
	BIETIGHEIM_GRÜNWIESENSTR_1 = 'de:08118:7441:0:3'  # eRtg Schillerplatz (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_GRÜNWIESENSTR_2 = 'de:08118:7441:0:4'  # eRtg Friedrich-Ebert-Straße (Bietigheim-Bissingen Bietigheim)
	GRÜNWIESENSTRASSE = 'de:08118:7441'
	GRÜNWIESENSTRASSE_1 = 'de:08118:7441:0:3'  # eRtg Schillerplatz (Bietigheim-Bissingen Bietigheim)
	GRÜNWIESENSTRASSE_2 = 'de:08118:7441:0:4'  # eRtg Friedrich-Ebert-Straße (Bietigheim-Bissingen Bietigheim)
	MAICHINGEN_ULMENSTRASSE = 'de:08115:7444'
	MAICHINGEN_ULMENSTRASSE_1 = 'de:08115:7444:0:3'  # Rtg Nordost (Sindelfingen Maichingen)
	MAICHINGEN_ULMENSTRASSE_2 = 'de:08115:7444:0:4'  # Rtg Südwest (Sindelfingen Maichingen)
	NELLMERSBACH_GEWERBEGEBIET = 'de:08119:7445'
	NELLMERSBACH_GEWERBEGEBIET_1 = 'de:08119:7445:1:3'  # eRtg Waldrems B14 (Leutenbach Nellmersbach)
	NELLMERSBACH_GEWERBEGEBIET_2 = 'de:08119:7445:1:4'  # eRtg Bahnhofstraße (Leutenbach Nellmersbach)
	KARL_MAI_ALLEE = 'de:08118:7446:0:4'  # Rtg Auwiesenbrücke (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_KARL_MAI_ALLEE = 'de:08118:7446:0:4'  # Rtg Auwiesenbrücke (Bietigheim-Bissingen Bietigheim)
	ENZWEIHINGEN_BUOLWEG = 'de:08118:7448'
	ENZWEIHINGEN_BUOLWEG_1 = 'de:08118:7448:0:3'  # Rtg B10 Vaihingen (Vaihingen an der Enz Enzweihingen)
	ENZWEIHINGEN_BUOLWEG_2 = 'de:08118:7448:0:4'  # Rtg Beerhaldenstr. (Vaihingen an der Enz Enzweihingen)
	BUOLWEG = 'de:08118:7448'
	BUOLWEG_1 = 'de:08118:7448:0:3'  # Rtg B10 Vaihingen (Vaihingen an der Enz Enzweihingen)
	BUOLWEG_2 = 'de:08118:7448:0:4'  # Rtg Beerhaldenstr. (Vaihingen an der Enz Enzweihingen)
	SPIEGELB__PARKPLATZ_ZOLLSTOCK = 'de:08119:7449'
	SPIEGELB__PARKPLATZ_ZOLLSTOCK_1 = 'de:08119:7449:0:3'  # eRtg Hüttlen (Spiegelberg Spiegelberg)
	SPIEGELB__PARKPLATZ_ZOLLSTOCK_2 = 'de:08119:7449:0:4'  # eRtg Nassach (Spiegelberg Spiegelberg)
	PARKPLATZ_ZOLLSTOCK = 'de:08119:7449'
	PARKPLATZ_ZOLLSTOCK_1 = 'de:08119:7449:0:3'  # eRtg Hüttlen (Spiegelberg Spiegelberg)
	PARKPLATZ_ZOLLSTOCK_2 = 'de:08119:7449:0:4'  # eRtg Nassach (Spiegelberg Spiegelberg)
	BIETIGH__KAMMGARNSPINN__B27 = 'de:08118:7450:0:4'  # eRtg Auwiesenbrücke (Bietigheim-Bissingen Bietigheim)
	KAMMGARNSPINNEREI_B27 = 'de:08118:7450:0:4'  # eRtg Auwiesenbrücke (Bietigheim-Bissingen Bietigheim)
	NECKARW__IMMAN__DORNFELD_STR = 'de:08118:7451'
	NECKARW__IMMAN__DORNFELD_STR_1 = 'de:08118:7451:0:3'  # eRtg Dampfkraftwerk (Ludwigsburg Neckarweihingen)
	NECKARW__IMMAN__DORNFELD_STR_2 = 'de:08118:7451:0:4'  # eRtg Wendeplatte (Ludwigsburg Neckarweihingen)
	IMMANUEL_DORNFELD_STRASSE = 'de:08118:7451'
	IMMANUEL_DORNFELD_STRASSE_1 = 'de:08118:7451:0:3'  # eRtg Dampfkraftwerk (Ludwigsburg Neckarweihingen)
	IMMANUEL_DORNFELD_STRASSE_2 = 'de:08118:7451:0:4'  # eRtg Wendeplatte (Ludwigsburg Neckarweihingen)
	WILHELM_FÖLL_STRASSE = 'de:08119:7452:0:3'  # eRtg Strümpfelbach (Backnang Strümpfelbach (Backnang))
	STRÜMPFELB___BK___W__FÖLL_STR = 'de:08119:7452:0:3'  # eRtg Strümpfelbach (Backnang Strümpfelbach (Backnang))
	GEGEN_EICH = 'de:08118:7454:0:3'  # Rtg Neckarweihinger Str. (Ludwigsburg Oßweil)
	OSSWEIL_GEGEN_EICH = 'de:08118:7454:0:3'  # Rtg Neckarweihinger Str. (Ludwigsburg Oßweil)
	HERTMANNSW__WIESENTALSTR = 'de:08119:7455:0:3'  # Rtg Hertmannsweiler B14 (Winnenden Hertmannsweiler)
	HERTMANNSWEILER_HECKENWEG = 'de:08119:7456:0:3'  # Rtg K.-G.-Pfleiderer-Str. (Winnenden Hertmannsweiler)
	OBERBRÜDEN_KELTERPLATZ = 'de:08119:7458:0:03'  # alle Richtungen (Auenwald Oberbrüden)
	WANDERPARKPLATZ_ROTER_STICH = 'de:08119:7459'
	WANDERPARKPLATZ_ROTER_STICH_1 = 'de:08119:7459:0:3'  # Rtg Hößlinswart (Berglen Hößlinswart)
	WANDERPARKPLATZ_ROTER_STICH_2 = 'de:08119:7459:0:4'  # Rtg Rohrbronn (Berglen Hößlinswart)
	HÖSSLINSW__W_PARKPL__ROT__STICH = 'de:08119:7459'
	HÖSSLINSW__W_PARKPL__ROT__STICH_1 = 'de:08119:7459:0:3'  # Rtg Hößlinswart (Berglen Hößlinswart)
	HÖSSLINSW__W_PARKPL__ROT__STICH_2 = 'de:08119:7459:0:4'  # Rtg Rohrbronn (Berglen Hößlinswart)
	HÄLDE = 'de:08118:7460'
	HÄLDE_1 = 'de:08118:7460:0:3'  # Rtg Hemm. Schwieberd. Str. (Hemmingen Hemmingen)
	HÄLDE_2 = 'de:08118:7460:0:4'  # Rtg Schwieberdingen (Hemmingen Hemmingen)
	HEMMINGEN_HÄLDE = 'de:08118:7460'
	HEMMINGEN_HÄLDE_1 = 'de:08118:7460:0:3'  # Rtg Hemm. Schwieberd. Str. (Hemmingen Hemmingen)
	HEMMINGEN_HÄLDE_2 = 'de:08118:7460:0:4'  # Rtg Schwieberdingen (Hemmingen Hemmingen)
	BIETIGHEIM_MÜHLWIESEN = 'de:08118:7471'
	BIETIGHEIM_MÜHLWIESEN_1 = 'de:08118:7471:0:3'  # Rtg Kronenzentrum (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_MÜHLWIESEN_2 = 'de:08118:7471:0:4'  # Rtg Schillerplatz/Auwiesenbrücke (Bietigheim-Bissingen Bietigheim)
	MÜHLWIESEN = 'de:08118:7471'
	MÜHLWIESEN_1 = 'de:08118:7471:0:3'  # Rtg Kronenzentrum (Bietigheim-Bissingen Bietigheim)
	MÜHLWIESEN_2 = 'de:08118:7471:0:4'  # Rtg Schillerplatz/Auwiesenbrücke (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_FISCHERPFAD = 'de:08118:7472'
	BIETIGHEIM_FISCHERPFAD_1 = 'de:08118:7472:0:3'  # Rtg Ellental (Bietigheim-Bissingen Bietigheim)
	BIETIGHEIM_FISCHERPFAD_2 = 'de:08118:7472:0:4'  # Rtg Badepark (Bietigheim-Bissingen Bietigheim)
	FISCHERPFAD = 'de:08118:7472'
	FISCHERPFAD_1 = 'de:08118:7472:0:3'  # Rtg Ellental (Bietigheim-Bissingen Bietigheim)
	FISCHERPFAD_2 = 'de:08118:7472:0:4'  # Rtg Badepark (Bietigheim-Bissingen Bietigheim)
	BISSINGEN_BORSIGSTRASSE = 'de:08118:7474:0:3'  # Rtg ZOB (Bietigheim-Bissingen Bissingen (LB))
	UNTERMBERG_ENZSTRASSE = 'de:08118:7477'
	UNTERMBERG_ENZSTRASSE_1 = 'de:08118:7477:0:3'  # Rtg Am Türmle (Bietigheim-Bissingen Untermberg)
	UNTERMBERG_ENZSTRASSE_2 = 'de:08118:7477:0:4'  # Rtg Bissinger Str. (Bietigheim-Bissingen Untermberg)
	IN_DEN_ÄULESWIESEN = 'de:08119:7480'
	IN_DEN_ÄULESWIESEN_1 = 'de:08119:7480:0:3'  # Rtg Königsbronnhof (Allmersbach im Tal Allmersbach im Tal)
	IN_DEN_ÄULESWIESEN_2 = 'de:08119:7480:0:4'  # Rtg Ortsmitte (Allmersbach im Tal Allmersbach im Tal)
	ALLMERSB__I_T__IN_DEN_ÄULESW = 'de:08119:7480'
	ALLMERSB__I_T__IN_DEN_ÄULESW_1 = 'de:08119:7480:0:3'  # Rtg Königsbronnhof (Allmersbach im Tal Allmersbach im Tal)
	ALLMERSB__I_T__IN_DEN_ÄULESW_2 = 'de:08119:7480:0:4'  # Rtg Ortsmitte (Allmersbach im Tal Allmersbach im Tal)
	OPPENWEILER_AICHELBACH = 'de:08119:7485'
	OPPENWEILER_AICHELBACH_1 = 'de:08119:7485:0:3'  # Rtg Zell (Oppenweiler Oppenweiler)
	OPPENWEILER_AICHELBACH_2 = 'de:08119:7485:0:4'  # Rtg Oppenweiler Bahnhof (Oppenweiler Oppenweiler)
	AICHELBACH = 'de:08119:7485'
	AICHELBACH_1 = 'de:08119:7485:0:3'  # Rtg Zell (Oppenweiler Oppenweiler)
	AICHELBACH_2 = 'de:08119:7485:0:4'  # Rtg Oppenweiler Bahnhof (Oppenweiler Oppenweiler)
	OPPENWEILER_ZELL = 'de:08119:7486'
	OPPENWEILER_ZELL_1 = 'de:08119:7486:0:3'  # Rtg Dieselstr. (Oppenweiler Oppenweiler)
	OPPENWEILER_ZELL_2 = 'de:08119:7486:0:4'  # Rtg Aichelbach (Oppenweiler Oppenweiler)
	OPPENWEILER_DIESELSTR = 'de:08119:7487:0:3'  # Rtg Post (Oppenweiler Oppenweiler)
	SULZB__M__HASELBACHMÜHLE_ABZW = 'de:08119:7488'
	SULZB__M__HASELBACHMÜHLE_ABZW_1 = 'de:08119:7488:0:1'  # Rtg Zwerenberg (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__HASELBACHMÜHLE_ABZW_2 = 'de:08119:7488:0:2'  # Rtg Bartenbach (Sulzbach an der Murr Sulzbach (M))
	HASELBACHMÜHLE_ABZWEIG = 'de:08119:7488'
	HASELBACHMÜHLE_ABZWEIG_1 = 'de:08119:7488:0:1'  # Rtg Zwerenberg (Sulzbach an der Murr Sulzbach (M))
	HASELBACHMÜHLE_ABZWEIG_2 = 'de:08119:7488:0:2'  # Rtg Bartenbach (Sulzbach an der Murr Sulzbach (M))
	BURGSTALL__M = 'de:08119:7500'
	BURGSTALL__M_1 = 'de:08119:7500:2:1'  # Gleis 1 (Burgstetten Burgstall (Burgstetten))
	BURGSTALL__M_2 = 'de:08119:7500:3:3'  # Rtg Murrbrücke (Burgstetten Burgstall (Burgstetten))
	BURGSTALL__M_3 = 'de:08119:7500:3:4'  # Rtg Kelterweg (Burgstetten Burgstall (Burgstetten))
	KIRCHBERG__M = 'de:08119:7502'
	KIRCHBERG__M_1 = 'de:08119:7502:1:2'  # Gleis 2 (Kirchberg an der Murr Kirchberg (M))
	KIRCHBERG__M_2 = 'de:08119:7502:2:1'  # Gleis 1 (Kirchberg an der Murr Kirchberg (M))
	KIRCHBERG__M_3 = 'de:08119:7502:3:3'  # eRtg Ankunft/Abfahrt Bus (Kirchberg an der Murr Kirchberg (M))
	MARBACH__N = 'de:08118:7503'
	MARBACH__N_1 = 'de:08118:7503:1:1'  # Gleis 1 (Marbach am Neckar Marbach (N))
	MARBACH__N_2 = 'de:08118:7503:2:2'  # Gleis 2 (Marbach am Neckar Marbach (N))
	MARBACH__N_3 = 'de:08118:7503:2:3'  # Gleis 3 (Marbach am Neckar Marbach (N))
	MARBACH__N_4 = 'de:08118:7503:3:0'  # Bstg 0 (Marbach am Neckar Marbach (N))
	MARBACH__N_5 = 'de:08118:7503:3:1'  # Bstg 1 (Marbach am Neckar Marbach (N))
	MARBACH__N_6 = 'de:08118:7503:3:3'  # Bstg 3 (Marbach am Neckar Marbach (N))
	MARBACH__N_7 = 'de:08118:7503:3:4'  # Bstg 4 (Marbach am Neckar Marbach (N))
	MARBACH__N_8 = 'de:08118:7503:4:5'  # Bstg 5 (Marbach am Neckar Marbach (N))
	MARBACH__N_9 = 'de:08118:7503:4:6'  # Bstg 6 (Marbach am Neckar Marbach (N))
	MARBACH__N_10 = 'de:08118:7503:4:7'  # Bstg 7 (Marbach am Neckar Marbach (N))
	GROSSERLACH_SCHULE = 'de:08119:7504'
	GROSSERLACH_SCHULE_1 = 'de:08119:7504:0:3'  # existiert nicht (Großerlach Großerlach)
	GROSSERLACH_SCHULE_2 = 'de:08119:7504:0:4'  # Rtg Liemersbach_Abzw. Hohenegarten (Großerlach Großerlach)
	LIEMERSBACH = 'de:08119:7505'
	LIEMERSBACH_1 = 'de:08119:7505:0:3'  # Rtg Großerlach/Mainhardt (Großerlach Großerlach)
	LIEMERSBACH_2 = 'de:08119:7505:0:4'  # Rtg Grab/Murrhardt (Großerlach Großerlach)
	GROSSERLACH_LIEMERSBACH = 'de:08119:7505'
	GROSSERLACH_LIEMERSBACH_1 = 'de:08119:7505:0:3'  # Rtg Großerlach/Mainhardt (Großerlach Großerlach)
	GROSSERLACH_LIEMERSBACH_2 = 'de:08119:7505:0:4'  # Rtg Grab/Murrhardt (Großerlach Großerlach)
	GRAB_SCHÖNBRONN = 'de:08119:7506:0:3'  # eRtg Schwalbenflug (Großerlach Grab)
	SCHÖNBRONN = 'de:08119:7506:0:3'  # eRtg Schwalbenflug (Großerlach Grab)
	LIEMANNSKLINGE_ABZWEIG = 'de:08119:7507'
	LIEMANNSKLINGE_ABZWEIG_1 = 'de:08119:7507:0:3'  # eRtg Berwinkel (Sulzbach an der Murr Sulzbach (M))
	LIEMANNSKLINGE_ABZWEIG_2 = 'de:08119:7507:0:4'  # eRtg Eschenstruet (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__LIEMANNSKLINGE_ABZW = 'de:08119:7507'
	SULZB__M__LIEMANNSKLINGE_ABZW_1 = 'de:08119:7507:0:3'  # eRtg Berwinkel (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__LIEMANNSKLINGE_ABZW_2 = 'de:08119:7507:0:4'  # eRtg Eschenstruet (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__ESCHENSTRUET = 'de:08119:7508'
	SULZB__M__ESCHENSTRUET_1 = 'de:08119:7508:0:3'  # eRtg Liemannsklinge Abzweig (Sulzbach an der Murr Sulzbach (M))
	SULZB__M__ESCHENSTRUET_2 = 'de:08119:7508:0:4'  # eRtg Bushof Abzw (Sulzbach an der Murr Sulzbach (M))
	ESCHENSTRUET = 'de:08119:7508'
	ESCHENSTRUET_1 = 'de:08119:7508:0:3'  # eRtg Liemannsklinge Abzweig (Sulzbach an der Murr Sulzbach (M))
	ESCHENSTRUET_2 = 'de:08119:7508:0:4'  # eRtg Bushof Abzw (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__BUSHOF_ABZW = 'de:08119:7509'
	SULZBACH__M__BUSHOF_ABZW_1 = 'de:08119:7509:0:3'  # eRtg Eschenstruet (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__BUSHOF_ABZW_2 = 'de:08119:7509:0:4'  # eRtg Hager (Sulzbach an der Murr Sulzbach (M))
	BUSHOF_ABZWEIG = 'de:08119:7509'
	BUSHOF_ABZWEIG_1 = 'de:08119:7509:0:3'  # eRtg Eschenstruet (Sulzbach an der Murr Sulzbach (M))
	BUSHOF_ABZWEIG_2 = 'de:08119:7509:0:4'  # eRtg Hager (Sulzbach an der Murr Sulzbach (M))
	HAGER = 'de:08119:7510'
	HAGER_1 = 'de:08119:7510:0:3'  # eRtg Bushof Abzweig (Sulzbach an der Murr Sulzbach (M))
	HAGER_2 = 'de:08119:7510:0:4'  # eRtg Zwerenberg (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__HAGER = 'de:08119:7510'
	SULZBACH__M__HAGER_1 = 'de:08119:7510:0:3'  # eRtg Bushof Abzweig (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__HAGER_2 = 'de:08119:7510:0:4'  # eRtg Zwerenberg (Sulzbach an der Murr Sulzbach (M))
	ZWERENBERG = 'de:08119:7511'
	ZWERENBERG_1 = 'de:08119:7511:0:3'  # eRtg Hager (Sulzbach an der Murr Sulzbach (M))
	ZWERENBERG_2 = 'de:08119:7511:0:4'  # eRtg Bartenbach (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__ZWERENBERG = 'de:08119:7511'
	SULZBACH__M__ZWERENBERG_1 = 'de:08119:7511:0:3'  # eRtg Hager (Sulzbach an der Murr Sulzbach (M))
	SULZBACH__M__ZWERENBERG_2 = 'de:08119:7511:0:4'  # eRtg Bartenbach (Sulzbach an der Murr Sulzbach (M))
	BACKNANG_CHRISTOPHSTR = 'de:08119:7512'
	BACKNANG_CHRISTOPHSTR_1 = 'de:08119:7512:0:3'  # eRtg Im Wiesengrund (Backnang Backnang)
	BACKNANG_CHRISTOPHSTR_2 = 'de:08119:7512:0:4'  # eRtg AOK/Waldhorn (Backnang Backnang)
	CHRISTOPHSTRASSE_1 = 'de:08119:7512:0:3'  # eRtg Im Wiesengrund (Backnang Backnang)
	CHRISTOPHSTRASSE_2 = 'de:08119:7512:0:4'  # eRtg AOK/Waldhorn (Backnang Backnang)
	CHRISTOPHSTRASSE_3 = 'de:08117:4003:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	CHRISTOPHSTRASSE_4 = 'de:08117:4003:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	BACKNANG_MAUBACHER_STR = 'de:08119:7513'
	BACKNANG_MAUBACHER_STR_1 = 'de:08119:7513:0:5'  # Ersatz Rtg. M.-Born-Gym (Backnang Backnang)
	BACKNANG_MAUBACHER_STR_2 = 'de:08119:7513:0:6'  # Ersatz Rtg. Blumenstr. (Backnang Backnang)
	MAUBACHER_STRASSE = 'de:08119:7513'
	MAUBACHER_STRASSE_1 = 'de:08119:7513:0:5'  # Ersatz Rtg. M.-Born-Gym (Backnang Backnang)
	MAUBACHER_STRASSE_2 = 'de:08119:7513:0:6'  # Ersatz Rtg. Blumenstr. (Backnang Backnang)
	GROSSERLACH_ERLACH = 'de:08119:7514'
	GROSSERLACH_ERLACH_1 = 'de:08119:7514:0:3'  # eRtg Hohenbrach (Großerlach Großerlach)
	GROSSERLACH_ERLACH_2 = 'de:08119:7514:0:4'  # eRtg Murrhardt/Mainhardt (Großerlach Großerlach)
	GRAB_MORBACH = 'de:08119:7515'
	GRAB_MORBACH_1 = 'de:08119:7515:0:3'  # eRtg Butzberg (Großerlach Grab)
	GRAB_MORBACH_2 = 'de:08119:7515:0:4'  # eRtg Grab (Großerlach Grab)
	MORBACH = 'de:08119:7515'
	MORBACH_1 = 'de:08119:7515:0:3'  # eRtg Butzberg (Großerlach Grab)
	MORBACH_2 = 'de:08119:7515:0:4'  # eRtg Grab (Großerlach Grab)
	FORUM_AM_SCHLOSSPARK = 'de:08118:7516:1:3'  # eRtg Rathaus (Ludwigsburg Ludwigsburg)
	LUDW__FORUM_AM_SCHLOSSPARK = 'de:08118:7516:1:3'  # eRtg Rathaus (Ludwigsburg Ludwigsburg)
	SCHWALBENFLUG = 'de:08119:7518'
	SCHWALBENFLUG_1 = 'de:08119:7518:0:3'  # Rtg Schönbronn (Großerlach Grab)
	SCHWALBENFLUG_2 = 'de:08119:7518:0:4'  # Rtg Grab (Großerlach Grab)
	GRAB_SCHWALBENFLUG = 'de:08119:7518'
	GRAB_SCHWALBENFLUG_1 = 'de:08119:7518:0:3'  # Rtg Schönbronn (Großerlach Grab)
	GRAB_SCHWALBENFLUG_2 = 'de:08119:7518:0:4'  # Rtg Grab (Großerlach Grab)
	GRAB_MANNENWEILER = 'de:08119:7519:0:3'  # eRtg Grab/Steinberg (Großerlach Grab)
	MANNENWEILER = 'de:08119:7519:0:3'  # eRtg Grab/Steinberg (Großerlach Grab)
	TAUBENHOF = 'de:08119:7524'
	TAUBENHOF_1 = 'de:08119:7524:0:3'  # eRtg Langenberg (Welzheim Welzheim)
	TAUBENHOF_2 = 'de:08119:7524:0:4'  # Rtg Bausche (Welzheim Welzheim)
	WELZHEIM_TAUBENHOF = 'de:08119:7524'
	WELZHEIM_TAUBENHOF_1 = 'de:08119:7524:0:3'  # eRtg Langenberg (Welzheim Welzheim)
	WELZHEIM_TAUBENHOF_2 = 'de:08119:7524:0:4'  # Rtg Bausche (Welzheim Welzheim)
	LANGENBERG = 'de:08119:7525'
	LANGENBERG_1 = 'de:08119:7525:0:3'  # eRtg Edelmannshof (Welzheim Welzheim)
	LANGENBERG_2 = 'de:08119:7525:0:4'  # eRtg Vorderhundsberg (Welzheim Welzheim)
	WELZHEIM_LANGENBERG = 'de:08119:7525'
	WELZHEIM_LANGENBERG_1 = 'de:08119:7525:0:3'  # eRtg Edelmannshof (Welzheim Welzheim)
	WELZHEIM_LANGENBERG_2 = 'de:08119:7525:0:4'  # eRtg Vorderhundsberg (Welzheim Welzheim)
	WELZHEIM_OBERSTEINENBERG = 'de:08119:7526:0:3'  # eRtg Edelmannshof (Welzheim Welzheim)
	OBERSTEINENBERG = 'de:08119:7526:0:3'  # eRtg Edelmannshof (Welzheim Welzheim)
	LETTENSTICH = 'de:08119:7527:0:3'  # eRtg Vorderhundsberg (Welzheim Welzheim)
	WELZHEIM_LETTENSTICH = 'de:08119:7527:0:3'  # eRtg Vorderhundsberg (Welzheim Welzheim)
	VORDERHUNDSBERG = 'de:08119:7528:0:3'  # eRtg alle Richtungen (Welzheim Welzheim)
	WELZHEIM_VORDERHUNDSBERG = 'de:08119:7528:0:3'  # eRtg alle Richtungen (Welzheim Welzheim)
	RUDERSBG__BRONNWIESENWEG = 'de:08119:7529:0:3'  # eRtg Alle Richtungen (Rudersberg Rudersberg)
	BRONNWIESENWEG = 'de:08119:7529:0:3'  # eRtg Alle Richtungen (Rudersberg Rudersberg)
	EULENHOF = 'de:08119:7530'
	EULENHOF_1 = 'de:08119:7530:0:3'  # eRtg Ebnisee (Kaisersbach Kaisersbach)
	EULENHOF_2 = 'de:08119:7530:0:4'  # eRtg Heumaden (Kaisersbach Kaisersbach)
	KAISERSBACH_EULENHOF = 'de:08119:7530'
	KAISERSBACH_EULENHOF_1 = 'de:08119:7530:0:3'  # eRtg Ebnisee (Kaisersbach Kaisersbach)
	KAISERSBACH_EULENHOF_2 = 'de:08119:7530:0:4'  # eRtg Heumaden (Kaisersbach Kaisersbach)
	BUTZBERG = 'de:08119:7538:0:3'  # eRtg Mannenweiler (Großerlach Grab)
	GRAB_BUTZBERG = 'de:08119:7538:0:3'  # eRtg Mannenweiler (Großerlach Grab)
	KAISERSBACH_FORSTSTRASSE = 'de:08119:7542'
	KAISERSBACH_FORSTSTRASSE_1 = 'de:08119:7542:0:3'  # eRtg Hauptstraße (Kaisersbach Kaisersbach)
	KAISERSBACH_FORSTSTRASSE_2 = 'de:08119:7542:0:4'  # eRtg Grairich (Kaisersbach Kaisersbach)
	NEUSTADT_ALTE_SCHULE = 'de:08119:7545'
	NEUSTADT_ALTE_SCHULE_1 = 'de:08119:7545:0:3'  # Rtg Rathaus (Waiblingen Neustadt)
	NEUSTADT_ALTE_SCHULE_2 = 'de:08119:7545:0:4'  # Rtg Alte Waiblinger Str. (Waiblingen Neustadt)
	ALTE_SCHULE = 'de:08119:7545'
	ALTE_SCHULE_1 = 'de:08119:7545:0:3'  # Rtg Rathaus (Waiblingen Neustadt)
	ALTE_SCHULE_2 = 'de:08119:7545:0:4'  # Rtg Alte Waiblinger Str. (Waiblingen Neustadt)
	PFARRÄCKER = 'de:08119:7547'
	PFARRÄCKER_1 = 'de:08119:7547:0:3'  # Rtg Neustadt-Hohenacker (Waiblingen Neustadt)
	PFARRÄCKER_2 = 'de:08119:7547:0:4'  # Rtg Schneider (Waiblingen Neustadt)
	NEUSTADT_PFARRÄCKER = 'de:08119:7547'
	NEUSTADT_PFARRÄCKER_1 = 'de:08119:7547:0:3'  # Rtg Neustadt-Hohenacker (Waiblingen Neustadt)
	NEUSTADT_PFARRÄCKER_2 = 'de:08119:7547:0:4'  # Rtg Schneider (Waiblingen Neustadt)
	HOHENACKER_KRIEGSBERG = 'de:08119:7548'
	HOHENACKER_KRIEGSBERG_1 = 'de:08119:7548:0:3'  # Rtg Bildäckerstr. (Waiblingen Hohenacker)
	HOHENACKER_KRIEGSBERG_2 = 'de:08119:7548:0:4'  # Rtg Neustadt-Hohenacker (Waiblingen Hohenacker)
	KRIEGSBERG = 'de:08119:7548'
	KRIEGSBERG_1 = 'de:08119:7548:0:3'  # Rtg Bildäckerstr. (Waiblingen Hohenacker)
	KRIEGSBERG_2 = 'de:08119:7548:0:4'  # Rtg Neustadt-Hohenacker (Waiblingen Hohenacker)
	BITTENFELD_SCHILLERSCHULE = 'de:08119:7549:0:3'  # Rtg Beethovenstr (Waiblingen Bittenfeld)
	BITTENFELD_BEETHOVENSTR = 'de:08119:7550:0:3'  # Rtg Friedhof (Waiblingen Bittenfeld)
	BITTENFELD_FRIEDHOF = 'de:08119:7551:0:3'  # Rtg Altes Rathaus (Waiblingen Bittenfeld)
	NASTSTRASSE = 'de:08118:7555'
	NASTSTRASSE_1 = 'de:08118:7555:0:3'  # eRtg Neckarbrücke (Ludwigsburg Ludwigsburg)
	NASTSTRASSE_2 = 'de:08118:7555:0:4'  # Rtg Harteneckstr (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_NASTSTRASSE = 'de:08118:7555'
	LUDWIGSBURG_NASTSTRASSE_1 = 'de:08118:7555:0:3'  # eRtg Neckarbrücke (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_NASTSTRASSE_2 = 'de:08118:7555:0:4'  # Rtg Harteneckstr (Ludwigsburg Ludwigsburg)
	OSSWEIL_MÜHLHÄUSER_STR = 'de:08118:7556:1:3'  # Rtg Friesenstraße (Ludwigsburg Oßweil)
	PALAZZO = 'de:08118:7560:0:3'  # eRtg Bietigheim (Freiberg am Neckar Freiberg (N))
	FREIBERG__N__PALAZZO = 'de:08118:7560:0:3'  # eRtg Bietigheim (Freiberg am Neckar Freiberg (N))
	LUDWIGSBURG_GARTENSTRASSE = 'de:08118:7563:0:3'  # eRtg Arsenalplatz (Ludwigsburg Ludwigsburg)
	LÖCHGAU_STEINBACHBRÜCKE = 'de:08118:7564'
	LÖCHGAU_STEINBACHBRÜCKE_1 = 'de:08118:7564:0:3'  # eRtg Heinkelstraße (Löchgau Löchgau)
	LÖCHGAU_STEINBACHBRÜCKE_2 = 'de:08118:7564:0:4'  # eRtg Besigheimer Straße (Löchgau Löchgau)
	STEINBACHBRÜCKE = 'de:08118:7564'
	STEINBACHBRÜCKE_1 = 'de:08118:7564:0:3'  # eRtg Heinkelstraße (Löchgau Löchgau)
	STEINBACHBRÜCKE_2 = 'de:08118:7564:0:4'  # eRtg Besigheimer Straße (Löchgau Löchgau)
	LÖCHGAU_HEINKELSTRASSE = 'de:08118:7565'
	LÖCHGAU_HEINKELSTRASSE_1 = 'de:08118:7565:0:3'  # eRtg Kirschhaldenweg (Löchgau Löchgau)
	LÖCHGAU_HEINKELSTRASSE_2 = 'de:08118:7565:0:4'  # eRtg Steinbachbrücke (Löchgau Löchgau)
	OBERNDORF = 'de:08119:7567:1:1'  # R-Bahn (Rudersberg Rudersberg)
	EDELMANNSHOF = 'de:08119:7568:0:3'  # eRtg Obersteinenberg/Langenberg (Rudersberg Rudersberg)
	RUDERSBERG_EDELMANNSHOF = 'de:08119:7568:0:3'  # eRtg Obersteinenberg/Langenberg (Rudersberg Rudersberg)
	AM_MÜHLBACH = 'de:08119:7569'
	AM_MÜHLBACH_1 = 'de:08119:7569:0:3'  # eRtg Mittlere Brücke (Schorndorf Schorndorf)
	AM_MÜHLBACH_2 = 'de:08119:7569:0:4'  # eRtg Grabenstraße (Schorndorf Schorndorf)
	SCHORNDORF_AM_MÜHLBACH = 'de:08119:7569'
	SCHORNDORF_AM_MÜHLBACH_1 = 'de:08119:7569:0:3'  # eRtg Mittlere Brücke (Schorndorf Schorndorf)
	SCHORNDORF_AM_MÜHLBACH_2 = 'de:08119:7569:0:4'  # eRtg Grabenstraße (Schorndorf Schorndorf)
	BAUMWASENSTRASSE = 'de:08119:7570'
	BAUMWASENSTRASSE_1 = 'de:08119:7570:0:3'  # eRtg Waiblinger Straße (Schorndorf Schorndorf)
	BAUMWASENSTRASSE_2 = 'de:08119:7570:0:4'  # eRtg Lederstraße (Schorndorf Schorndorf)
	SCHORNDORF_BAUMWASENSTRASSE = 'de:08119:7570'
	SCHORNDORF_BAUMWASENSTRASSE_1 = 'de:08119:7570:0:3'  # eRtg Waiblinger Straße (Schorndorf Schorndorf)
	SCHORNDORF_BAUMWASENSTRASSE_2 = 'de:08119:7570:0:4'  # eRtg Lederstraße (Schorndorf Schorndorf)
	KONRAD_HAUSSMANN_WEG = 'de:08119:7575'
	KONRAD_HAUSSMANN_WEG_1 = 'de:08119:7575:0:3'  # eRtg Schlipfweg (Schorndorf Schorndorf)
	KONRAD_HAUSSMANN_WEG_2 = 'de:08119:7575:0:4'  # eRtg Kreisverkehr Welzh. Straße (Schorndorf Schorndorf)
	SCHORNDORF_K__HAUSSMANN_WEG = 'de:08119:7575'
	SCHORNDORF_K__HAUSSMANN_WEG_1 = 'de:08119:7575:0:3'  # eRtg Schlipfweg (Schorndorf Schorndorf)
	SCHORNDORF_K__HAUSSMANN_WEG_2 = 'de:08119:7575:0:4'  # eRtg Kreisverkehr Welzh. Straße (Schorndorf Schorndorf)
	WIESENSTR = 'de:08119:7576:0:3'  # eRtg Wiesenstraße (Schorndorf Schorndorf)
	SCHORNDORF_WIESENSTR = 'de:08119:7576:0:3'  # eRtg Wiesenstraße (Schorndorf Schorndorf)
	WENDL__N__ANNE_FRANK_SCHULE = 'de:08116:7577'
	WENDL__N__ANNE_FRANK_SCHULE_1 = 'de:08116:7577:0:3'  # Rtg Kapellenstr./Schulzentrum (Wendlingen am Neckar Wendlingen (N))
	WENDL__N__ANNE_FRANK_SCHULE_2 = 'de:08116:7577:0:4'  # Rtg Sporthalle (Wendlingen am Neckar Wendlingen (N))
	ANNE_FRANK_SCHULE = 'de:08116:7577'
	ANNE_FRANK_SCHULE_1 = 'de:08116:7577:0:3'  # Rtg Kapellenstr./Schulzentrum (Wendlingen am Neckar Wendlingen (N))
	ANNE_FRANK_SCHULE_2 = 'de:08116:7577:0:4'  # Rtg Sporthalle (Wendlingen am Neckar Wendlingen (N))
	WENDLINGEN__N__SPORTHALLE = 'de:08116:7578:0:3'  # eRtg Schulzentrum (Wendlingen am Neckar Wendlingen (N))
	SPORTHALLE = 'de:08116:7578:0:3'  # eRtg Schulzentrum (Wendlingen am Neckar Wendlingen (N))
	ENSINGEN_BERTHA_BENZ_STR = 'de:08118:7579'
	ENSINGEN_BERTHA_BENZ_STR_1 = 'de:08118:7579:0:3'  # eRtg Vaihingen (E) (Vaihingen an der Enz Ensingen)
	ENSINGEN_BERTHA_BENZ_STR_2 = 'de:08118:7579:0:4'  # eRtg Grundschule (Vaihingen an der Enz Ensingen)
	BERTHA_BENZ_STRASSE = 'de:08118:7579'
	BERTHA_BENZ_STRASSE_1 = 'de:08118:7579:0:3'  # eRtg Vaihingen (E) (Vaihingen an der Enz Ensingen)
	BERTHA_BENZ_STRASSE_2 = 'de:08118:7579:0:4'  # eRtg Grundschule (Vaihingen an der Enz Ensingen)
	SPIEGELBERG_DENTELTAL = 'de:08119:7580'
	SPIEGELBERG_DENTELTAL_1 = 'de:08119:7580:0:3'  # eRtg Vorderbüchelberg (Spiegelberg Spiegelberg)
	SPIEGELBERG_DENTELTAL_2 = 'de:08119:7580:0:4'  # Rtg Spiegelberg Löwensteiner Str. (Spiegelberg Spiegelberg)
	DENTELTAL = 'de:08119:7580'
	DENTELTAL_1 = 'de:08119:7580:0:3'  # eRtg Vorderbüchelberg (Spiegelberg Spiegelberg)
	DENTELTAL_2 = 'de:08119:7580:0:4'  # Rtg Spiegelberg Löwensteiner Str. (Spiegelberg Spiegelberg)
	WETZSTEINSTOLLEN = 'de:08119:7581:0:3'  # eRtg Nassach (Spiegelberg Jux)
	JUX_WETZSTEINSTOLLEN = 'de:08119:7581:0:3'  # eRtg Nassach (Spiegelberg Jux)
	BRUNNENSTRASSE_1 = 'de:08119:7582:0:3'  # eRtg Busbahnhof (Welzheim Welzheim)
	BRUNNENSTRASSE_2 = 'de:08117:122:0:H1'  # Rtg Brückenstr. (Gingen an der Fils Gingen (F))
	BRUNNENSTRASSE_3 = 'de:08117:122:0:R1'  # Rtg Ev. Kindergarten (Gingen an der Fils Gingen (F))
	BRUNNENSTRASSE_4 = 'de:08117:4918:0:H1'  # Rtg Hofhalde (Göppingen Bartenbach (Göppingen))
	BRUNNENSTRASSE_5 = 'de:08117:4918:0:R1'  # Rtg Raiffeisenstr. (Göppingen Bartenbach (Göppingen))
	WELZHEIM_BRUNNENSTRASSE = 'de:08119:7582:0:3'  # eRtg Busbahnhof (Welzheim Welzheim)
	GUTENBERGER_STRASSE_66 = 'de:08116:7583:0:3'  # eRtg Etterstraße (Lenningen Oberlenningen)
	OBERL__GUTENBERGER_STR__66 = 'de:08116:7583:0:3'  # eRtg Etterstraße (Lenningen Oberlenningen)
	OBERLENNINGEN_NIKOMÖDIESTR__34 = 'de:08116:7584'
	OBERLENNINGEN_NIKOMÖDIESTR__34_1 = 'de:08116:7584:0:3'  # Ankunft (Lenningen Oberlenningen)
	OBERLENNINGEN_NIKOMÖDIESTR__34_2 = 'de:08116:7584:0:4'  # eRtg Gutenberger Straße 47 (Lenningen Oberlenningen)
	NIKOMÖDIESTRASSE_34 = 'de:08116:7584'
	NIKOMÖDIESTRASSE_34_1 = 'de:08116:7584:0:3'  # Ankunft (Lenningen Oberlenningen)
	NIKOMÖDIESTRASSE_34_2 = 'de:08116:7584:0:4'  # eRtg Gutenberger Straße 47 (Lenningen Oberlenningen)
	REALSCHULE_IM_AURAIN = 'de:08118:7585:0:3'  # eRtg Ellentalgymnasien (Bietigheim-Bissingen Bietigheim)
	BIETIGH__REALSCHULE_IM_AURAIN = 'de:08118:7585:0:3'  # eRtg Ellentalgymnasien (Bietigheim-Bissingen Bietigheim)
	GUTENBERGER_STRASSE_47 = 'de:08116:7589:0:3'  # eRtg Gutenberger Straße 17 (Lenningen Oberlenningen)
	OBERL__GUTENBERGER_STR__47 = 'de:08116:7589:0:3'  # eRtg Gutenberger Straße 17 (Lenningen Oberlenningen)
	HEGNACHER_STRASSE = 'de:08119:7596'
	HEGNACHER_STRASSE_1 = 'de:08119:7596:0:3'  # Rtg Bittenfeld(er Str.) (Waiblingen Hohenacker)
	HEGNACHER_STRASSE_2 = 'de:08119:7596:0:4'  # Rtg Ortsmitte (Waiblingen Hohenacker)
	HOHENACKER_HEGNACHER_STR = 'de:08119:7596'
	HOHENACKER_HEGNACHER_STR_1 = 'de:08119:7596:0:3'  # Rtg Bittenfeld(er Str.) (Waiblingen Hohenacker)
	HOHENACKER_HEGNACHER_STR_2 = 'de:08119:7596:0:4'  # Rtg Ortsmitte (Waiblingen Hohenacker)
	TORSTRASSE = 'de:08119:7597'
	TORSTRASSE_1 = 'de:08119:7597:0:3'  # Rtg Schneider (Waiblingen Neustadt)
	TORSTRASSE_2 = 'de:08119:7597:0:4'  # Rtg Rathaus (Waiblingen Neustadt)
	NEUSTADT_TORSTRASSE = 'de:08119:7597'
	NEUSTADT_TORSTRASSE_1 = 'de:08119:7597:0:3'  # Rtg Schneider (Waiblingen Neustadt)
	NEUSTADT_TORSTRASSE_2 = 'de:08119:7597:0:4'  # Rtg Rathaus (Waiblingen Neustadt)
	LÖCHGAU__SPORTPLATZ = 'de:08118:7598:0:3'  # eRtg Post (Löchgau Löchgau)
	MURRBÄDER = 'de:08119:7599:0:3'  # eRtg Alle Richtungen (Backnang Backnang)
	BACKNANG_MURRBÄDER = 'de:08119:7599:0:3'  # eRtg Alle Richtungen (Backnang Backnang)
	BACKNANG = 'de:08119:7600'
	BACKNANG_1 = 'de:08119:7600:1:1'  # Gleis 1 (Backnang Backnang)
	BACKNANG_2 = 'de:08119:7600:2:3'  # Gleis 3 (Backnang Backnang)
	BACKNANG_3 = 'de:08119:7600:3:1A'  # Bstg 1A (Backnang Backnang)
	BACKNANG_4 = 'de:08119:7600:3:2A'  # Bstg 2A (Backnang Backnang)
	BACKNANG_5 = 'de:08119:7600:3:3A'  # Bstg 3A (Backnang Backnang)
	BACKNANG_6 = 'de:08119:7600:3:4A'  # Bstg 4A (Backnang Backnang)
	BACKNANG_7 = 'de:08119:7600:3:5A'  # Bstg 5A (Backnang Backnang)
	BACKNANG_8 = 'de:08119:7600:4:17'  # Pseudo Ankunft (Backnang Backnang)
	BACKNANG_9 = 'de:08119:7600:4:1B'  # Bstg 1B (Backnang Backnang)
	BACKNANG_10 = 'de:08119:7600:4:1C'  # Bstg 1C (Backnang Backnang)
	BACKNANG_11 = 'de:08119:7600:4:2B'  # Bstg 2B (Backnang Backnang)
	BACKNANG_12 = 'de:08119:7600:4:2C'  # Bstg 2C (Backnang Backnang)
	BACKNANG_13 = 'de:08119:7600:4:3B'  # Bstg 3B (Backnang Backnang)
	BACKNANG_14 = 'de:08119:7600:4:4B'  # Bstg 4B (Backnang Backnang)
	BACKNANG_15 = 'de:08119:7600:4:4C'  # Bstg 4C (Backnang Backnang)
	BACKNANG_16 = 'de:08119:7600:5:4'  # Gleis 4 (Backnang Backnang)
	BACKNANG_17 = 'de:08119:7600:5:5'  # Gleis 5 (Backnang Backnang)
	MAUBACH = 'de:08119:7601'
	MAUBACH_1 = 'de:08119:7601:1:2'  # Gleis 2 (Backnang Maubach)
	MAUBACH_2 = 'de:08119:7601:2:1'  # Gleis 1 (Backnang Maubach)
	MAUBACH_3 = 'de:08119:7601:3:3'  # Alle Richtungen (Backnang Maubach)
	BAHNHOF__DAMMSTRASSE = 'de:08119:7602:0:3'  # SEV BC-Regio (Waiblingen Waiblingen)
	WAIBLINGEN_BAHNHOF__DAMMSTR = 'de:08119:7602:0:3'  # SEV BC-Regio (Waiblingen Waiblingen)
	KORNTAL = 'de:08118:7603'
	KORNTAL_1 = 'de:08118:7603:1:3'  # Gleis 3 (Korntal-Münchingen Korntal)
	KORNTAL_2 = 'de:08118:7603:2:1'  # Gleis 1 (Korntal-Münchingen Korntal)
	KORNTAL_3 = 'de:08118:7603:3:7'  # Gleis 7 (Korntal-Münchingen Korntal)
	KORNTAL_4 = 'de:08118:7603:4:3'  # Bus Ankunft/Abfahrt (Korntal-Münchingen Korntal)
	WAIBLINGEN = 'de:08119:7604'
	WAIBLINGEN_1 = 'de:08119:7604:1:6'  # Gleis 6 (Waiblingen Waiblingen)
	WAIBLINGEN_2 = 'de:08119:7604:1:7'  # Gleis 7 (Waiblingen Waiblingen)
	WAIBLINGEN_3 = 'de:08119:7604:2:5'  # Gleis 5 (Waiblingen Waiblingen)
	WAIBLINGEN_4 = 'de:08119:7604:3:3'  # Gleis 3 (Waiblingen Waiblingen)
	WAIBLINGEN_5 = 'de:08119:7604:4:1'  # Gleis 1 (Waiblingen Waiblingen)
	WAIBLINGEN_6 = 'de:08119:7604:5:2'  # Bstg 2 (Waiblingen Waiblingen)
	WAIBLINGEN_7 = 'de:08119:7604:5:3'  # Bstg 3 (Waiblingen Waiblingen)
	WAIBLINGEN_8 = 'de:08119:7604:5:4'  # Bstg 4 (Waiblingen Waiblingen)
	WAIBLINGEN_9 = 'de:08119:7604:5:5'  # Bstg 5 (Waiblingen Waiblingen)
	WAIBLINGEN_10 = 'de:08119:7604:5:6'  # Bstg 6 (Waiblingen Waiblingen)
	WAIBLINGEN_11 = 'de:08119:7604:5:7'  # Bstg 7 (Waiblingen Waiblingen)
	WAIBLINGEN_12 = 'de:08119:7604:5:8'  # Bstg 8 (Waiblingen Waiblingen)
	WAIBLINGEN_13 = 'de:08119:7604:6:10'  # Bstg 10 (Waiblingen Waiblingen)
	WAIBLINGEN_14 = 'de:08119:7604:6:20'  # Pseudo Ankunft OVR (Waiblingen Waiblingen)
	WAIBLINGEN_15 = 'de:08119:7604:7:1'  # Bstg 1 (Waiblingen Waiblingen)
	WAIBLINGEN_16 = 'de:08119:7604:7:9'  # Bstg 9 (Waiblingen Waiblingen)
	WINNENDEN = 'de:08119:7605'
	WINNENDEN_1 = 'de:08119:7605:1:2'  # Gleis 2 (Winnenden Winnenden)
	WINNENDEN_2 = 'de:08119:7605:1:3'  # Gleis 3 (Winnenden Winnenden)
	WINNENDEN_3 = 'de:08119:7605:1:5'  # Ersatz-Bstg 5 (Winnenden Winnenden)
	WINNENDEN_4 = 'de:08119:7605:2:1'  # Bstg 1 (Winnenden Winnenden)
	WINNENDEN_5 = 'de:08119:7605:2:5'  # Bstg 5 (Winnenden Winnenden)
	WINNENDEN_6 = 'de:08119:7605:3:2'  # Bstg 2 (Winnenden Winnenden)
	WINNENDEN_7 = 'de:08119:7605:3:3'  # Bstg 3 (Winnenden Winnenden)
	WINNENDEN_8 = 'de:08119:7605:3:4'  # Bstg 4 (Winnenden Winnenden)
	WINNENDEN_9 = 'de:08119:7605:3:6'  # Bstg 6 (Winnenden Winnenden)
	WINNENDEN_10 = 'de:08119:7605:3:7'  # Bstg 7 (Winnenden Winnenden)
	WINNENDEN_11 = 'de:08119:7605:3:8'  # Bstg 8 (Winnenden Winnenden)
	WINNENDEN_12 = 'de:08119:7605:3:A'  # Ankunftssteig (Winnenden Winnenden)
	KORNTAL_GYMNASIUM = 'de:08118:7606:0:1'  # R-Bahn (Korntal-Münchingen Korntal)
	MÜNCHINGEN_RÜHRBERG = 'de:08118:7607:0:1'  # R-Bahn (Korntal-Münchingen Münchingen)
	RÜHRBERG = 'de:08118:7607:0:1'  # R-Bahn (Korntal-Münchingen Münchingen)
	SCHORNDORF_ROBERT_BOSCH_STR = 'de:08119:7631'
	SCHORNDORF_ROBERT_BOSCH_STR_1 = 'de:08119:7631:0:3'  # Rtg Welzheimer Str. (Schorndorf Schorndorf)
	SCHORNDORF_ROBERT_BOSCH_STR_2 = 'de:08119:7631:0:4'  # Rtg Schorndorf Bf (Schorndorf Schorndorf)
	SCHORNDORF_GÄNSBÜHL = 'de:08119:7632:0:3'  # Rtg R.-Maier-Platz (Schorndorf Schorndorf)
	GÄNSBÜHL = 'de:08119:7632:0:3'  # Rtg R.-Maier-Platz (Schorndorf Schorndorf)
	LUDWIGSBURG_WERNERSTR = 'de:08118:7641'
	LUDWIGSBURG_WERNERSTR_1 = 'de:08118:7641:0:3'  # Rtg Hoferstraße ZOB (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_WERNERSTR_2 = 'de:08118:7641:0:4'  # Rtg Osterholzallee Eglosheim (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_ROBERT_KOCH_STR = 'de:08118:7642:0:3'  # Rtg Auf dem Wasen (Ludwigsburg Ludwigsburg)
	SCHLIEFFENSTRASSE = 'de:08118:7643:0:3'  # Rtg Mann + Hummel (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_SCHLIEFFENSTR = 'de:08118:7643:0:3'  # Rtg Mann + Hummel (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_WILHELM_FEIN_STR = 'de:08118:7644:0:3'  # Rtg Schlieffenstr. (Ludwigsburg Ludwigsburg)
	WILHELM_FEIN_STRASSE = 'de:08118:7644:0:3'  # Rtg Schlieffenstr. (Ludwigsburg Ludwigsburg)
	LUDWIGSBURG_URBAN_HARBOR = 'de:08118:7645:0:3'  # Rtg Wilhelm-Flein-Str. (Ludwigsburg Ludwigsburg)
	URBAN_HARBOR = 'de:08118:7645:0:3'  # Rtg Wilhelm-Flein-Str. (Ludwigsburg Ludwigsburg)
	ALLEENFELD = 'de:08118:7658'
	ALLEENFELD_1 = 'de:08118:7658:0:1'  # Rtg Löchgau (Freudental Freudental)
	ALLEENFELD_2 = 'de:08118:7658:0:2'  # Rtg Post (Freudental Freudental)
	FREUDENTAL_ALLEENFELD = 'de:08118:7658'
	FREUDENTAL_ALLEENFELD_1 = 'de:08118:7658:0:1'  # Rtg Löchgau (Freudental Freudental)
	FREUDENTAL_ALLEENFELD_2 = 'de:08118:7658:0:2'  # Rtg Post (Freudental Freudental)
	OCHSENBACH_KINDERGARTEN = 'de:08118:7661'
	OCHSENBACH_KINDERGARTEN_1 = 'de:08118:7661:0:3'  # Rtg Rtg Häfnerhaslach (Sachsenheim Ochsenbach)
	OCHSENBACH_KINDERGARTEN_2 = 'de:08118:7661:0:4'  # Rtg Spielberg (Sachsenheim Ochsenbach)
	RUDERSBERGER_STRASSE = 'de:08119:7662:0:3'  # Rtg Tannbachbrücke/Bf (Schorndorf Miedelsbach)
	MIEDELSBACH_RUDERSBERGER_STR = 'de:08119:7662:0:3'  # Rtg Tannbachbrücke/Bf (Schorndorf Miedelsbach)
	OBERNDORF_MANNENBERGER_STRASSE = 'de:08119:7663'
	OBERNDORF_MANNENBERGER_STRASSE_1 = 'de:08119:7663:0:3'  # Rtg Mannenberg (Rudersberg Rudersberg)
	OBERNDORF_MANNENBERGER_STRASSE_2 = 'de:08119:7663:0:4'  # Rtg Brunnenplatz (Rudersberg Rudersberg)
	RUDERSB__OBERND__MANNENB__STR = 'de:08119:7663'
	RUDERSB__OBERND__MANNENB__STR_1 = 'de:08119:7663:0:3'  # Rtg Mannenberg (Rudersberg Rudersberg)
	RUDERSB__OBERND__MANNENB__STR_2 = 'de:08119:7663:0:4'  # Rtg Brunnenplatz (Rudersberg Rudersberg)
	ROMMELSHAUSEN = 'de:08119:7701'
	ROMMELSHAUSEN_1 = 'de:08119:7701:1:1'  # Gleis 1 (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_2 = 'de:08119:7701:2:2'  # Gleis 2 (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_3 = 'de:08119:7701:3:4'  # eRtg Waiblingen (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_4 = 'de:08119:7701:4:3'  # eRtg Kernen (Kernen im Remstal Rommelshausen)
	ROMMELSHAUSEN_5 = 'de:08119:7701:4:5'  # Rtg Kernen Ausrücker (Kernen im Remstal Rommelshausen)
	SCHORNDORF = 'de:08119:7703'
	SCHORNDORF_1 = 'de:08119:7703:1:1'  # Gleis 1 (Schorndorf Schorndorf)
	SCHORNDORF_2 = 'de:08119:7703:1:14'  # Gleis 14 (Schorndorf Schorndorf)
	SCHORNDORF_3 = 'de:08119:7703:2:2'  # Gleis 2 (Schorndorf Schorndorf)
	SCHORNDORF_4 = 'de:08119:7703:2:3'  # Gleis 3 (Schorndorf Schorndorf)
	SCHORNDORF_5 = 'de:08119:7703:3:5'  # Gleis 5 (Schorndorf Schorndorf)
	SCHORNDORF_6 = 'de:08119:7703:4:5'  # Bstg 5 (Schorndorf Schorndorf)
	SCHORNDORF_7 = 'de:08119:7703:5:1'  # Bstg 1 (Schorndorf Schorndorf)
	SCHORNDORF_8 = 'de:08119:7703:5:9'  # Ankunftssteig (Schorndorf Schorndorf)
	SCHORNDORF_9 = 'de:08119:7703:6:2'  # Bstg 2 (Schorndorf Schorndorf)
	SCHORNDORF_10 = 'de:08119:7703:6:3'  # Bstg 3 (Schorndorf Schorndorf)
	SCHORNDORF_11 = 'de:08119:7703:6:4'  # Bstg 4 (Schorndorf Schorndorf)
	SCHORNDORF_12 = 'de:08119:7703:6:6'  # Ersatzverkehr (Schorndorf Schorndorf)
	ENDERSBACH = 'de:08119:7704'
	ENDERSBACH_1 = 'de:08119:7704:1:2'  # Gleis 2 (Weinstadt Endersbach)
	ENDERSBACH_2 = 'de:08119:7704:1:3'  # Gleis 3 (Weinstadt Endersbach)
	ENDERSBACH_3 = 'de:08119:7704:2:4'  # Bussteig 1 (Weinstadt Endersbach)
	ENDERSBACH_4 = 'de:08119:7704:2:5'  # Bussteig 2 (Weinstadt Endersbach)
	ENDERSBACH_5 = 'de:08119:7704:2:6'  # Bussteig 3 (Weinstadt Endersbach)
	ENDERSBACH_6 = 'de:08119:7704:3:7'  # Bussteig 5 (Weinstadt Endersbach)
	ENDERSBACH_7 = 'de:08119:7704:3:8'  # Bussteig 4 (Weinstadt Endersbach)
	MANFRED_VON_ARDENNE_ALLEE = 'de:08119:7738:0:3'  # Rtg K.-Ferd.-Braun-Str. (Backnang Backnang)
	BACKNANG_M__VON_ARDENNE_ALLEE = 'de:08119:7738:0:3'  # Rtg K.-Ferd.-Braun-Str. (Backnang Backnang)
	ESSLINGEN__N = 'de:08116:7800'
	ESSLINGEN__N_1 = 'de:08116:7800:1:7'  # Gleis 7 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N_2 = 'de:08116:7800:1:8'  # Gleis 8 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N_3 = 'de:08116:7800:2:3'  # Gleis 3 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N_4 = 'de:08116:7800:2:5'  # Gleis 5 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N_5 = 'de:08116:7800:2:6'  # Gleis 6 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__ZOB = 'de:08116:7801'
	ESSLINGEN__N__ZOB_1 = 'de:08116:7801:1:4'  # Bstg 4 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__ZOB_2 = 'de:08116:7801:3:1'  # Bstg 1 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__ZOB_3 = 'de:08116:7801:3:2'  # Bstg 2 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__ZOB_4 = 'de:08116:7801:3:3'  # Bstg 3 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__ZOB_5 = 'de:08116:7801:3:5'  # Bstg 5 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__ZOB_6 = 'de:08116:7801:3:6'  # Bstg 6 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__ZOB_7 = 'de:08116:7801:3:7'  # Bstg 7 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__ZOB_8 = 'de:08116:7801:3:8'  # Bstg 8 (Esslingen am Neckar Esslingen (N))
	ESSLINGEN__N__ZOB_9 = 'de:08116:7801:3:9'  # Bstg 9 (Esslingen am Neckar Esslingen (N))
	PLOCHINGEN = 'de:08116:7802'
	PLOCHINGEN_1 = 'de:08116:7802:1:10'  # Gleis 10 (Plochingen Plochingen)
	PLOCHINGEN_2 = 'de:08116:7802:1:7'  # Gleis 7 (Plochingen Plochingen)
	PLOCHINGEN_3 = 'de:08116:7802:1:9'  # Gleis 9 (Plochingen Plochingen)
	PLOCHINGEN_4 = 'de:08116:7802:2:1'  # Gleis 1 (Plochingen Plochingen)
	PLOCHINGEN_5 = 'de:08116:7802:2:3'  # Gleis 3 (Plochingen Plochingen)
	PLOCHINGEN_6 = 'de:08116:7802:2:4'  # Gleis 4 (Plochingen Plochingen)
	PLOCHINGEN_7 = 'de:08116:7802:2:6'  # Gleis 6 (Plochingen Plochingen)
	PLOCHINGEN_8 = 'de:08116:7802:3:1'  # Bstg 1 (Plochingen Plochingen)
	PLOCHINGEN_9 = 'de:08116:7802:3:2'  # Bstg 2 (Plochingen Plochingen)
	PLOCHINGEN_10 = 'de:08116:7802:3:3'  # Bstg 3 (Plochingen Plochingen)
	PLOCHINGEN_11 = 'de:08116:7802:3:4'  # Bstg 4 (Plochingen Plochingen)
	PLOCHINGEN_12 = 'de:08116:7802:3:5'  # Bstg 5 (Plochingen Plochingen)
	PLOCHINGEN_13 = 'de:08116:7802:3:nix'  # Plochingen (Plochingen Plochingen)
	TISCHARDT_FRICKENHÄUSER_STR = 'de:08116:7805'
	TISCHARDT_FRICKENHÄUSER_STR_1 = 'de:08116:7805:0:3'  # eRtg Reutte (Frickenhausen Tischardt)
	TISCHARDT_FRICKENHÄUSER_STR_2 = 'de:08116:7805:0:4'  # eRtg Rathaus (Frickenhausen Tischardt)
	FRICKENHÄUSER_STRASSE = 'de:08116:7805'
	FRICKENHÄUSER_STRASSE_1 = 'de:08116:7805:0:3'  # eRtg Reutte (Frickenhausen Tischardt)
	FRICKENHÄUSER_STRASSE_2 = 'de:08116:7805:0:4'  # eRtg Rathaus (Frickenhausen Tischardt)
	PFORZHEIMER_STRASSE = 'de:08118:7811:0:3'  # eRtg Herrenwiesen (Vaihingen an der Enz Horrheim)
	HORRHEIM_PFORZHEIMER_STRASSE = 'de:08118:7811:0:3'  # eRtg Herrenwiesen (Vaihingen an der Enz Horrheim)
	WELZHEIM_NATURFREUNDEHAUS = 'de:08119:7818:0:4'  # eRtg Tannhof (Welzheim Welzheim)
	KÖNGEN_KIRCHHEIMER_STR = 'de:08116:7819'
	KÖNGEN_KIRCHHEIMER_STR_1 = 'de:08116:7819:0:3'  # Rtg Neuhausen (Köngen Köngen)
	KÖNGEN_KIRCHHEIMER_STR_2 = 'de:08116:7819:0:4'  # Rtg Wendlingen (Köngen Köngen)
	OCHSENWANG_BISSINGER_STR = 'de:08116:7821'
	OCHSENWANG_BISSINGER_STR_1 = 'de:08116:7821:0:3'  # Rtg Schopfloch (Bissingen an der Teck Ochsenwang)
	OCHSENWANG_BISSINGER_STR_2 = 'de:08116:7821:0:4'  # Rtg Bissingen Rathaus (Bissingen an der Teck Ochsenwang)
	BALTMANNSWEILER_FESTPLATZ = 'de:08116:7822'
	BALTMANNSWEILER_FESTPLATZ_1 = 'de:08116:7822:0:3'  # Rtg Tiefgarage/Waldhorn/Ortsmitte (Baltmannsweiler Baltmannsweiler)
	BALTMANNSWEILER_FESTPLATZ_2 = 'de:08116:7822:0:4'  # Rtg Schurwaldhöhe (Baltmannsweiler Baltmannsweiler)
	KORNBERGWEG = 'de:08116:7823'
	KORNBERGWEG_1 = 'de:08116:7823:0:3'  # Rtg Stuifenstr. (Plochingen Plochingen)
	KORNBERGWEG_2 = 'de:08116:7823:0:4'  # Rtg Lettenäcker (Plochingen Plochingen)
	PLOCHINGEN_KORNBERGWEG = 'de:08116:7823'
	PLOCHINGEN_KORNBERGWEG_1 = 'de:08116:7823:0:3'  # Rtg Stuifenstr. (Plochingen Plochingen)
	PLOCHINGEN_KORNBERGWEG_2 = 'de:08116:7823:0:4'  # Rtg Lettenäcker (Plochingen Plochingen)
	PEROUSE_SPORTGELÄNDE_AISCHBACH = 'de:08115:7831'
	PEROUSE_SPORTGELÄNDE_AISCHBACH_1 = 'de:08115:7831:0:3'  # Rtg W.-Kopp-Str. (Rutesheim Perouse)
	PEROUSE_SPORTGELÄNDE_AISCHBACH_2 = 'de:08115:7831:0:4'  # Rtg Heimsh. Leonb. Str. (Rutesheim Perouse)
	SPORTGELÄNDE_AISCHBACH = 'de:08115:7831'
	SPORTGELÄNDE_AISCHBACH_1 = 'de:08115:7831:0:3'  # Rtg W.-Kopp-Str. (Rutesheim Perouse)
	SPORTGELÄNDE_AISCHBACH_2 = 'de:08115:7831:0:4'  # Rtg Heimsh. Leonb. Str. (Rutesheim Perouse)
	WILHELM_KOPP_STRASSE = 'de:08115:7832'
	WILHELM_KOPP_STRASSE_1 = 'de:08115:7832:0:3'  # Rtg Hauptstr.(Kreisel) (Rutesheim Perouse)
	WILHELM_KOPP_STRASSE_2 = 'de:08115:7832:0:4'  # Rtg Sportst. Aischbach (Rutesheim Perouse)
	PEROUSE_WILHELM_KOPP_STRASSE = 'de:08115:7832'
	PEROUSE_WILHELM_KOPP_STRASSE_1 = 'de:08115:7832:0:3'  # Rtg Hauptstr.(Kreisel) (Rutesheim Perouse)
	PEROUSE_WILHELM_KOPP_STRASSE_2 = 'de:08115:7832:0:4'  # Rtg Sportst. Aischbach (Rutesheim Perouse)
	DARMSHEIM_HÄSLACH = 'de:08115:7836:0:2'  # Rtg Döffingen/Dätzingen (Sindelfingen Darmsheim)
	HÄSLACH = 'de:08115:7836:0:2'  # Rtg Döffingen/Dätzingen (Sindelfingen Darmsheim)
	ROSSWAG_FRIEDHOF = 'de:08118:7841:0:3'  # Rtg Silvanerstr. (Vaihingen an der Enz Roßwag)
	SILVANERSTRASSE = 'de:08118:7842:0:3'  # Rtg Mühlhäuserstr. (Vaihingen an der Enz Roßwag)
	ROSSWAG_SILVANERSTRASSE = 'de:08118:7842:0:3'  # Rtg Mühlhäuserstr. (Vaihingen an der Enz Roßwag)
	NEBRINGEN_BETTÄCKER = 'de:08115:7920'
	NEBRINGEN_BETTÄCKER_1 = 'de:08115:7920:0:3'  # Rtg Bahnhofstraße (Gäufelden Nebringen)
	NEBRINGEN_BETTÄCKER_2 = 'de:08115:7920:0:4'  # Rtg Öschelbronn (Gäufelden Nebringen)
	NEBRINGEN_BETTÄCKER_3 = 'de:08115:7920:0:5'  # Ersatz Rtg Postamt / Bahnhof (Gäufelden Nebringen)
	BETTÄCKER = 'de:08115:7920'
	BETTÄCKER_1 = 'de:08115:7920:0:3'  # Rtg Bahnhofstraße (Gäufelden Nebringen)
	BETTÄCKER_2 = 'de:08115:7920:0:4'  # Rtg Öschelbronn (Gäufelden Nebringen)
	BETTÄCKER_3 = 'de:08115:7920:0:5'  # Ersatz Rtg Postamt / Bahnhof (Gäufelden Nebringen)
	BÖBLINGEN_BAHNHOF__FLUGFELD = 'de:08115:7921'
	BÖBLINGEN_BAHNHOF__FLUGFELD_1 = 'de:08115:7921:0:3'  # Ri Ost (Böblingen Böblingen)
	BÖBLINGEN_BAHNHOF__FLUGFELD_2 = 'de:08115:7921:0:4'  # Ri West (Böblingen Böblingen)
	BAHNHOF__FLUGFELD = 'de:08115:7921'
	BAHNHOF__FLUGFELD_1 = 'de:08115:7921:0:3'  # Ri Ost (Böblingen Böblingen)
	BAHNHOF__FLUGFELD_2 = 'de:08115:7921:0:4'  # Ri West (Böblingen Böblingen)
	ÖSCHELBRONN_MÖTZINGER_STR = 'de:08115:7928'
	ÖSCHELBRONN_MÖTZINGER_STR_1 = 'de:08115:7928:0:3'  # Rtg Stuttgarter Str. (Gäufelden Öschelbronn (Gäufelden))
	ÖSCHELBRONN_MÖTZINGER_STR_2 = 'de:08115:7928:0:4'  # Rtg Radsporthalle (Gäufelden Öschelbronn (Gäufelden))
	TAILFINGEN_MAUERÄCKER = 'de:08115:7930'
	TAILFINGEN_MAUERÄCKER_1 = 'de:08115:7930:0:3'  # Rtg Rathaus (Gäufelden Tailfingen)
	TAILFINGEN_MAUERÄCKER_2 = 'de:08115:7930:0:4'  # Rtg Nebringen Bahnhofstr. (Gäufelden Tailfingen)
	MAUERÄCKER = 'de:08115:7930'
	MAUERÄCKER_1 = 'de:08115:7930:0:3'  # Rtg Rathaus (Gäufelden Tailfingen)
	MAUERÄCKER_2 = 'de:08115:7930:0:4'  # Rtg Nebringen Bahnhofstr. (Gäufelden Tailfingen)
	BOLZPLATZ = 'de:08116:7934:0:3'  # eRtg alle Richtungen (Baltmannsweiler Hohengehren)
	HOHENGEHREN_BOLZPLATZ = 'de:08116:7934:0:3'  # eRtg alle Richtungen (Baltmannsweiler Hohengehren)
	HOHENGEHREN_ORTSMITTE = 'de:08116:7935'
	HOHENGEHREN_ORTSMITTE_1 = 'de:08116:7935:0:3'  # eRtg Bolzplatz (Baltmannsweiler Hohengehren)
	HOHENGEHREN_ORTSMITTE_2 = 'de:08116:7935:0:4'  # eRtg Waldhorn (Baltmannsweiler Hohengehren)
	ZIZISHAUSEN_OBERBOIHINGER_STR = 'de:08116:7936'
	ZIZISHAUSEN_OBERBOIHINGER_STR_1 = 'de:08116:7936:0:3'  # Rtg Oberb. Stattmannstraße (Nürtingen Zizishausen)
	ZIZISHAUSEN_OBERBOIHINGER_STR_2 = 'de:08116:7936:0:4'  # Rtg Nürtingen Weberstraße (Nürtingen Zizishausen)
	OBERBOIHINGER_STRASSE = 'de:08116:7936'
	OBERBOIHINGER_STRASSE_1 = 'de:08116:7936:0:3'  # Rtg Oberb. Stattmannstraße (Nürtingen Zizishausen)
	OBERBOIHINGER_STRASSE_2 = 'de:08116:7936:0:4'  # Rtg Nürtingen Weberstraße (Nürtingen Zizishausen)
	MÜNCHINGEN_LINGWIESEN = 'de:08118:7940'
	MÜNCHINGEN_LINGWIESEN_1 = 'de:08118:7940:0:3'  # eRtg Ziegeleistraße (Korntal-Münchingen Münchingen)
	MÜNCHINGEN_LINGWIESEN_2 = 'de:08118:7940:0:4'  # eRtg Müllerheim (Korntal-Münchingen Münchingen)
	LINGWIESEN = 'de:08118:7940'
	LINGWIESEN_1 = 'de:08118:7940:0:3'  # eRtg Ziegeleistraße (Korntal-Münchingen Münchingen)
	LINGWIESEN_2 = 'de:08118:7940:0:4'  # eRtg Müllerheim (Korntal-Münchingen Münchingen)
	WARNENBERG = 'de:08116:7957:0:4'  # Rtg Haugennest (Oberboihingen Oberboihingen)
	OBERBOIHINGEN_WARNENBERG = 'de:08116:7957:0:4'  # Rtg Haugennest (Oberboihingen Oberboihingen)
	OBERBOIHINGEN_HAUGENNEST = 'de:08116:7958'
	OBERBOIHINGEN_HAUGENNEST_1 = 'de:08116:7958:0:3'  # Rtg Warnenberg (Oberboihingen Oberboihingen)
	OBERBOIHINGEN_HAUGENNEST_2 = 'de:08116:7958:0:4'  # Rtg Ortsmitte (Oberboihingen Oberboihingen)
	HAUGENNEST = 'de:08116:7958'
	HAUGENNEST_1 = 'de:08116:7958:0:3'  # Rtg Warnenberg (Oberboihingen Oberboihingen)
	HAUGENNEST_2 = 'de:08116:7958:0:4'  # Rtg Ortsmitte (Oberboihingen Oberboihingen)
	FRAUENLÄNDERSTRASSE = 'de:08119:7974'
	FRAUENLÄNDERSTRASSE_1 = 'de:08119:7974:0:3'  # Rtg Diakonie (Kernen im Remstal Stetten (i. R.))
	FRAUENLÄNDERSTRASSE_2 = 'de:08119:7974:0:4'  # Rtg Endersbacher Straße (Kernen im Remstal Stetten (i. R.))
	STETTEN_I_R__FRAUENLÄNDERSTR = 'de:08119:7974'
	STETTEN_I_R__FRAUENLÄNDERSTR_1 = 'de:08119:7974:0:3'  # Rtg Diakonie (Kernen im Remstal Stetten (i. R.))
	STETTEN_I_R__FRAUENLÄNDERSTR_2 = 'de:08119:7974:0:4'  # Rtg Endersbacher Straße (Kernen im Remstal Stetten (i. R.))
	STADTKIRCHE = 'de:08117:1:0:H1'  # Rtg Schildwachtweg (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__STADTKIRCHE = 'de:08117:1:0:H1'  # Rtg Schildwachtweg (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__ST__RATHAUS = 'de:08117:2'
	GEISLINGEN__ST__RATHAUS_1 = 'de:08117:2:0:2'  # Rtg Schildwachtweg (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__ST__RATHAUS_2 = 'de:08117:2:0:H1'  # Rtg Geisl.(St) ZOB (Geislingen an der Steige Geislingen (Steige))
	KRANKENHAUS__SEEBACH = 'de:08117:4'
	KRANKENHAUS__SEEBACH_1 = 'de:08117:4:1:H1'  # Rtg Seebach Paulusk. (Geislingen an der Steige Geislingen (Steige))
	KRANKENHAUS__SEEBACH_2 = 'de:08117:4:2:R1'  # Rtg Geisl.(St) ZOB (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__KRANKENHAUS__SEEBACH = 'de:08117:4'
	GEISL__ST__KRANKENHAUS__SEEBACH_1 = 'de:08117:4:1:H1'  # Rtg Seebach Paulusk. (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__KRANKENHAUS__SEEBACH_2 = 'de:08117:4:2:R1'  # Rtg Geisl.(St) ZOB (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__ST__PAULUSKIRCHE = 'de:08117:5'
	GEISLINGEN__ST__PAULUSKIRCHE_1 = 'de:08117:5:0:H1'  # Rtg Nel Mezzo (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__ST__PAULUSKIRCHE_2 = 'de:08117:5:0:R1'  # Rtg Seebach Krankenh. (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__NEL_MEZZO = 'de:08117:6'
	GEISL__ST__NEL_MEZZO_1 = 'de:08117:6:0:H1'  # Rtg Wilhelmstraße (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__NEL_MEZZO_2 = 'de:08117:6:0:R1'  # Rtg Geisl.(St) ZOB (Geislingen an der Steige Geislingen (Steige))
	NEL_MEZZO = 'de:08117:6'
	NEL_MEZZO_1 = 'de:08117:6:0:H1'  # Rtg Wilhelmstraße (Geislingen an der Steige Geislingen (Steige))
	NEL_MEZZO_2 = 'de:08117:6:0:R1'  # Rtg Geisl.(St) ZOB (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__WILHELMSTRASSE = 'de:08117:7'
	GEISL__ST__WILHELMSTRASSE_1 = 'de:08117:7:1:H1'  # Rtg Sternplatz ZOH (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__WILHELMSTRASSE_2 = 'de:08117:7:2:R1'  # Rtg Nel Mezzo (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__STERNPLATZ_ZOH = 'de:08117:8'
	GEISL__ST__STERNPLATZ_ZOH_1 = 'de:08117:8:0:A'  # Rtg Frauenstr. (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__STERNPLATZ_ZOH_2 = 'de:08117:8:0:B'  # Rtg Werastr. (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__STERNPLATZ_ZOH_3 = 'de:08117:8:0:C'  # Rtg Krone/Altenstadt (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__STERNPLATZ_ZOH_4 = 'de:08117:8:0:D'  # Rtg ZOB (Geislingen an der Steige Geislingen (Steige))
	STERNPLATZ_ZOH = 'de:08117:8'
	STERNPLATZ_ZOH_1 = 'de:08117:8:0:A'  # Rtg Frauenstr. (Geislingen an der Steige Geislingen (Steige))
	STERNPLATZ_ZOH_2 = 'de:08117:8:0:B'  # Rtg Werastr. (Geislingen an der Steige Geislingen (Steige))
	STERNPLATZ_ZOH_3 = 'de:08117:8:0:C'  # Rtg Krone/Altenstadt (Geislingen an der Steige Geislingen (Steige))
	STERNPLATZ_ZOH_4 = 'de:08117:8:0:D'  # Rtg ZOB (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__KRONE_ALTENSTADT = 'de:08117:9'
	GEISL__ST__KRONE_ALTENSTADT_1 = 'de:08117:9:0:H1'  # Rtg Adler Altenstadt (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__KRONE_ALTENSTADT_2 = 'de:08117:9:0:R1'  # Rtg Michelberggymnasium (Geislingen an der Steige Geislingen (Steige))
	KRONE_ALTENSTADT = 'de:08117:9'
	KRONE_ALTENSTADT_1 = 'de:08117:9:0:H1'  # Rtg Adler Altenstadt (Geislingen an der Steige Geislingen (Steige))
	KRONE_ALTENSTADT_2 = 'de:08117:9:0:R1'  # Rtg Michelberggymnasium (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__ADLER_ALTENSTADT = 'de:08117:10'
	GEISL__ST__ADLER_ALTENSTADT_1 = 'de:08117:10:0:H1'  # Rtg Michelberggymnasium (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__ADLER_ALTENSTADT_2 = 'de:08117:10:0:R1'  # Rtg Krone/Altenstadt (Geislingen an der Steige Geislingen (Steige))
	ADLER_ALTENSTADT = 'de:08117:10'
	ADLER_ALTENSTADT_1 = 'de:08117:10:0:H1'  # Rtg Michelberggymnasium (Geislingen an der Steige Geislingen (Steige))
	ADLER_ALTENSTADT_2 = 'de:08117:10:0:R1'  # Rtg Krone/Altenstadt (Geislingen an der Steige Geislingen (Steige))
	BRUCKWIESEN = 'de:08117:11'
	BRUCKWIESEN_1 = 'de:08117:11:0:H1'  # Rtg Kuchen (Geislingen an der Steige Geislingen (Steige))
	BRUCKWIESEN_2 = 'de:08117:11:0:R1'  # Rtg Adler Altenstadt (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__BRUCKWIESEN = 'de:08117:11'
	GEISL__ST__BRUCKWIESEN_1 = 'de:08117:11:0:H1'  # Rtg Kuchen (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__BRUCKWIESEN_2 = 'de:08117:11:0:R1'  # Rtg Adler Altenstadt (Geislingen an der Steige Geislingen (Steige))
	KUCHEN_MOLTKESTR = 'de:08117:12'
	KUCHEN_MOLTKESTR_1 = 'de:08117:12:0:H1'  # Rtg Löwen (Kuchen Kuchen (F))
	KUCHEN_MOLTKESTR_2 = 'de:08117:12:0:R1'  # Rtg Geisl.(St.) (Kuchen Kuchen (F))
	MOLTKESTRASSE = 'de:08117:12'
	MOLTKESTRASSE_1 = 'de:08117:12:0:H1'  # Rtg Löwen (Kuchen Kuchen (F))
	MOLTKESTRASSE_2 = 'de:08117:12:0:R1'  # Rtg Geisl.(St.) (Kuchen Kuchen (F))
	KUCHEN_LÖWEN = 'de:08117:13:0:H1'  # Rtg Schillerstr./Gingen (Kuchen Kuchen (F))
	KUCHEN_SCHILLERSTR = 'de:08117:14:0:H1'  # Rtg Gemeindezentrum (Kuchen Kuchen (F))
	KUCHEN_GEMEINDEZENTRUM = 'de:08117:15:0:H1'  # Rtg Färberstr. (Kuchen Kuchen (F))
	GINGEN__F__BAHNHOFSTR = 'de:08117:17:0:H1'  # Rtg Buchsteiner (Gingen an der Fils Gingen (F))
	GINGEN__F__BUCHSTEINER = 'de:08117:18:0:H1'  # Rtg Kath. Kirche (Gingen an der Fils Gingen (F))
	BUCHSTEINER = 'de:08117:18:0:H1'  # Rtg Kath. Kirche (Gingen an der Fils Gingen (F))
	KATH__KIRCHE = 'de:08117:19:0:H1'  # Rtg Ev. Kindergarten (Gingen an der Fils Gingen (F))
	GINGEN__F__KATH__KIRCHE = 'de:08117:19:0:H1'  # Rtg Ev. Kindergarten (Gingen an der Fils Gingen (F))
	GINGEN__F__RATHAUS = 'de:08117:20:0:H1'  # Ankunft (Gingen an der Fils Gingen (F))
	GROSSE_GASSE = 'de:08117:22:0:R1'  # Rtg Kuchen (Gingen an der Fils Gingen (F))
	GINGEN__F__GROSSE_GASSE = 'de:08117:22:0:R1'  # Rtg Kuchen (Gingen an der Fils Gingen (F))
	KUCHEN_FÄRBERSTR = 'de:08117:23'
	KUCHEN_FÄRBERSTR_1 = 'de:08117:23:0:H1'  # Rtg Gingen Bahnhofstraße (Kuchen Kuchen (F))
	KUCHEN_FÄRBERSTR_2 = 'de:08117:23:0:R1'  # Rtg Feuerwache (Kuchen Kuchen (F))
	FÄRBERSTRASSE = 'de:08117:23'
	FÄRBERSTRASSE_1 = 'de:08117:23:0:H1'  # Rtg Gingen Bahnhofstraße (Kuchen Kuchen (F))
	FÄRBERSTRASSE_2 = 'de:08117:23:0:R1'  # Rtg Feuerwache (Kuchen Kuchen (F))
	KUCHEN_FEUERWACHE = 'de:08117:24:0:R1'  # Rtg Gerokweg (Kuchen Kuchen (F))
	FEUERWACHE_1 = 'de:08117:24:0:R1'  # Rtg Gerokweg (Kuchen Kuchen (F))
	FEUERWACHE_2 = 'de:08117:4001:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	FEUERWACHE_3 = 'de:08117:4001:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GEROKWEG = 'de:08117:25:0:R1'  # Rtg Rathaus (Kuchen Kuchen (F))
	KUCHEN_GEROKWEG = 'de:08117:25:0:R1'  # Rtg Rathaus (Kuchen Kuchen (F))
	KUCHEN_RATHAUS = 'de:08117:26:0:R1'  # Rtg Moltkestr. (Kuchen Kuchen (F))
	SONNE_CENTER = 'de:08117:36'
	SONNE_CENTER_1 = 'de:08117:36:0:H1'  # Sonne Center (Geislingen an der Steige Geislingen (Steige))
	SONNE_CENTER_2 = 'de:08117:36:0:R1'  # Rtg Stadtkirche (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__SONNE_CENTER = 'de:08117:36'
	GEISL__ST__SONNE_CENTER_1 = 'de:08117:36:0:H1'  # Sonne Center (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__SONNE_CENTER_2 = 'de:08117:36:0:R1'  # Rtg Stadtkirche (Geislingen an der Steige Geislingen (Steige))
	WERASTRASSE = 'de:08117:41'
	WERASTRASSE_1 = 'de:08117:41:1:H1'  # Rtg Michelberggymnasium (Geislingen an der Steige Geislingen (Steige))
	WERASTRASSE_2 = 'de:08117:41:2:R1'  # Rtg Sternplz ZOH (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__WERASTRASSE = 'de:08117:41'
	GEISL__ST__WERASTRASSE_1 = 'de:08117:41:1:H1'  # Rtg Michelberggymnasium (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__WERASTRASSE_2 = 'de:08117:41:2:R1'  # Rtg Sternplz ZOH (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__FRAUENSTRASSE = 'de:08117:42'
	GEISL__ST__FRAUENSTRASSE_1 = 'de:08117:42:1:H1'  # Rtg Bebelstr. (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__FRAUENSTRASSE_2 = 'de:08117:42:2:R1'  # Rtg Werastr. (Geislingen an der Steige Geislingen (Steige))
	FRAUENSTRASSE = 'de:08117:42'
	FRAUENSTRASSE_1 = 'de:08117:42:1:H1'  # Rtg Bebelstr. (Geislingen an der Steige Geislingen (Steige))
	FRAUENSTRASSE_2 = 'de:08117:42:2:R1'  # Rtg Werastr. (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__BEBELSTRASSE = 'de:08117:43'
	GEISL__ST__BEBELSTRASSE_1 = 'de:08117:43:1:H1'  # Rtg Neuwiesen (B466) (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__BEBELSTRASSE_2 = 'de:08117:43:2:R1'  # Rtg Frauenstr. (Geislingen an der Steige Geislingen (Steige))
	BEBELSTRASSE = 'de:08117:43'
	BEBELSTRASSE_1 = 'de:08117:43:1:H1'  # Rtg Neuwiesen (B466) (Geislingen an der Steige Geislingen (Steige))
	BEBELSTRASSE_2 = 'de:08117:43:2:R1'  # Rtg Frauenstr. (Geislingen an der Steige Geislingen (Steige))
	NEUWIESEN__B466 = 'de:08117:44'
	NEUWIESEN__B466_1 = 'de:08117:44:0:H1'  # Rtg Neuw. Gewerbegebiet (Geislingen an der Steige Geislingen (Steige))
	NEUWIESEN__B466_2 = 'de:08117:44:0:R1'  # Rtg Bebelstraße (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__NEUWIESEN__B466 = 'de:08117:44'
	GEISL__ST__NEUWIESEN__B466_1 = 'de:08117:44:0:H1'  # Rtg Neuw. Gewerbegebiet (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__NEUWIESEN__B466_2 = 'de:08117:44:0:R1'  # Rtg Bebelstraße (Geislingen an der Steige Geislingen (Steige))
	IM_GRUND = 'de:08117:45'
	IM_GRUND_1 = 'de:08117:45:0:H1'  # Rtg Bad (Bad Überkingen Bad Überkingen)
	IM_GRUND_2 = 'de:08117:45:0:R1'  # Rtg Geislingen (St.) (Bad Überkingen Bad Überkingen)
	BAD_ÜBERKINGEN_IM_GRUND = 'de:08117:45'
	BAD_ÜBERKINGEN_IM_GRUND_1 = 'de:08117:45:0:H1'  # Rtg Bad (Bad Überkingen Bad Überkingen)
	BAD_ÜBERKINGEN_IM_GRUND_2 = 'de:08117:45:0:R1'  # Rtg Geislingen (St.) (Bad Überkingen Bad Überkingen)
	BAD_ÜBERKINGEN_BAD = 'de:08117:46'
	BAD_ÜBERKINGEN_BAD_1 = 'de:08117:46:0:H1'  # Rtg Autalhalle (Bad Überkingen Bad Überkingen)
	BAD_ÜBERKINGEN_BAD_2 = 'de:08117:46:0:R1'  # Rtg Im Grund (Bad Überkingen Bad Überkingen)
	BAD_ÜBERKINGEN_AUTALHALLE = 'de:08117:47'
	BAD_ÜBERKINGEN_AUTALHALLE_1 = 'de:08117:47:0:H1'  # Rtg Hausen (F) (Bad Überkingen Bad Überkingen)
	BAD_ÜBERKINGEN_AUTALHALLE_2 = 'de:08117:47:0:R1'  # Rtg Bad (Bad Überkingen Bad Überkingen)
	AUTALHALLE = 'de:08117:47'
	AUTALHALLE_1 = 'de:08117:47:0:H1'  # Rtg Hausen (F) (Bad Überkingen Bad Überkingen)
	AUTALHALLE_2 = 'de:08117:47:0:R1'  # Rtg Bad (Bad Überkingen Bad Überkingen)
	HAUSEN__F__B466 = 'de:08117:48'
	HAUSEN__F__B466_1 = 'de:08117:48:0:H1'  # Rtg Reichenbach i. T. (Bad Überkingen Hausen (Bad Überkingen))
	HAUSEN__F__B466_2 = 'de:08117:48:0:R1'  # Rtg Siedlung (Bad Überkingen Hausen (Bad Überkingen))
	B466 = 'de:08117:48'
	B466_1 = 'de:08117:48:0:H1'  # Rtg Reichenbach i. T. (Bad Überkingen Hausen (Bad Überkingen))
	B466_2 = 'de:08117:48:0:R1'  # Rtg Siedlung (Bad Überkingen Hausen (Bad Überkingen))
	REICHENBACH_I_T__ABZW__B466 = 'de:08117:49'
	REICHENBACH_I_T__ABZW__B466_1 = 'de:08117:49:0:H1'  # Rtg Rathaus (Deggingen Reichenbach im Täle (Deggingen))
	REICHENBACH_I_T__ABZW__B466_2 = 'de:08117:49:0:R1'  # Rtg Hausen (F) (Deggingen Reichenbach im Täle (Deggingen))
	ABZWEIG_B466 = 'de:08117:49'
	ABZWEIG_B466_1 = 'de:08117:49:0:H1'  # Rtg Rathaus (Deggingen Reichenbach im Täle (Deggingen))
	ABZWEIG_B466_2 = 'de:08117:49:0:R1'  # Rtg Hausen (F) (Deggingen Reichenbach im Täle (Deggingen))
	REICHENBACH_I_T__TURNHALLE = 'de:08117:50:0:R1'  # Rtg Deggingen (Deggingen Reichenbach im Täle (Deggingen))
	REICHENBACH_I_T__RATHAUS = 'de:08117:51:0:H1'  # Rtg Turnhalle (Deggingen Reichenbach im Täle (Deggingen))
	DEGGINGEN_AVE_MARIA = 'de:08117:52'
	DEGGINGEN_AVE_MARIA_1 = 'de:08117:52:0:H1'  # Rtg Friedhof (Deggingen Deggingen)
	DEGGINGEN_AVE_MARIA_2 = 'de:08117:52:0:R1'  # Rtg Reichenbach i. T. (Deggingen Deggingen)
	AVE_MARIA = 'de:08117:52'
	AVE_MARIA_1 = 'de:08117:52:0:H1'  # Rtg Friedhof (Deggingen Deggingen)
	AVE_MARIA_2 = 'de:08117:52:0:R1'  # Rtg Reichenbach i. T. (Deggingen Deggingen)
	DEGGINGEN_FRIEDHOF = 'de:08117:53'
	DEGGINGEN_FRIEDHOF_1 = 'de:08117:53:0:H1'  # Rtg Bad Ditzenbach (Deggingen Deggingen)
	DEGGINGEN_FRIEDHOF_2 = 'de:08117:53:0:R1'  # Rtg Ave Maria (Deggingen Deggingen)
	BAD_DITZENBACH_KURKLINIK = 'de:08117:54'
	BAD_DITZENBACH_KURKLINIK_1 = 'de:08117:54:0:H1'  # Rtg Vinzenz Therme (Bad Ditzenbach Bad Ditzenbach)
	BAD_DITZENBACH_KURKLINIK_2 = 'de:08117:54:0:R1'  # Rtg Deggingen (Bad Ditzenbach Bad Ditzenbach)
	KURKLINIK = 'de:08117:54'
	KURKLINIK_1 = 'de:08117:54:0:H1'  # Rtg Vinzenz Therme (Bad Ditzenbach Bad Ditzenbach)
	KURKLINIK_2 = 'de:08117:54:0:R1'  # Rtg Deggingen (Bad Ditzenbach Bad Ditzenbach)
	BAD_DITZENBACH_VINZENZ_THERME = 'de:08117:55'
	BAD_DITZENBACH_VINZENZ_THERME_1 = 'de:08117:55:1:H1'  # Rtg Gosbach (Bad Ditzenbach Bad Ditzenbach)
	BAD_DITZENBACH_VINZENZ_THERME_2 = 'de:08117:55:2:R1'  # Rtg Kurklinik (Bad Ditzenbach Bad Ditzenbach)
	BAD_DITZENBACH_VINZENZ_THERME_3 = 'de:08117:55:3:R2'  # Rtg Auendorfer Straße (Bad Ditzenbach Bad Ditzenbach)
	VINZENZ_THERME = 'de:08117:55'
	VINZENZ_THERME_1 = 'de:08117:55:1:H1'  # Rtg Gosbach (Bad Ditzenbach Bad Ditzenbach)
	VINZENZ_THERME_2 = 'de:08117:55:2:R1'  # Rtg Kurklinik (Bad Ditzenbach Bad Ditzenbach)
	VINZENZ_THERME_3 = 'de:08117:55:3:R2'  # Rtg Auendorfer Straße (Bad Ditzenbach Bad Ditzenbach)
	ABZWEIG_DRACKENSTEIN = 'de:08117:57'
	ABZWEIG_DRACKENSTEIN_1 = 'de:08117:57:0:H1'  # Rtg Wiesensteiger Str (Bad Ditzenbach Gosbach)
	ABZWEIG_DRACKENSTEIN_2 = 'de:08117:57:0:R1'  # Rtg Hirsch/Lamm (Bad Ditzenbach Gosbach)
	GOSBACH_ABZW__DRACKENSTEIN = 'de:08117:57'
	GOSBACH_ABZW__DRACKENSTEIN_1 = 'de:08117:57:0:H1'  # Rtg Wiesensteiger Str (Bad Ditzenbach Gosbach)
	GOSBACH_ABZW__DRACKENSTEIN_2 = 'de:08117:57:0:R1'  # Rtg Hirsch/Lamm (Bad Ditzenbach Gosbach)
	MÜHLHAUSEN_I_T__RATHAUS = 'de:08117:58:0:R1'  # Rtg Wiesensteig (Mühlhausen im Täle Mühlhausen im Täle)
	MÜHLHAUSEN_I_T__BAHNHOFSTR = 'de:08117:59'
	MÜHLHAUSEN_I_T__BAHNHOFSTR_1 = 'de:08117:59:0:H1'  # Rtg Rathaus (Mühlhausen im Täle Mühlhausen im Täle)
	MÜHLHAUSEN_I_T__BAHNHOFSTR_2 = 'de:08117:59:0:R1'  # Rtg Wiesensteig (Mühlhausen im Täle Mühlhausen im Täle)
	WIESENSTEIG_SCHÖNTALWEG = 'de:08117:60'
	WIESENSTEIG_SCHÖNTALWEG_1 = 'de:08117:60:0:H1'  # Schöntalweg (Wiesensteig Wiesensteig)
	WIESENSTEIG_SCHÖNTALWEG_2 = 'de:08117:60:0:R1'  # Schöntalweg (Wiesensteig Wiesensteig)
	SCHÖNTALWEG = 'de:08117:60'
	SCHÖNTALWEG_1 = 'de:08117:60:0:H1'  # Schöntalweg (Wiesensteig Wiesensteig)
	SCHÖNTALWEG_2 = 'de:08117:60:0:R1'  # Schöntalweg (Wiesensteig Wiesensteig)
	WIESENSTEIG_POSTAMT = 'de:08117:61:0:H1'  # eRtg Reußenstein (Wiesensteig Wiesensteig)
	WIESENSTEIG_RATHAUS = 'de:08117:64'
	WIESENSTEIG_RATHAUS_1 = 'de:08117:64:0:H1'  # Rtg Brunnengarten/Ankunft (Wiesensteig Wiesensteig)
	WIESENSTEIG_RATHAUS_2 = 'de:08117:64:0:R1'  # Rtg Gasthaus See (Wiesensteig Wiesensteig)
	BRUNNENGARTEN = 'de:08117:65:0:R1'  # Rtg Schönthalweg (Wiesensteig Wiesensteig)
	WIESENSTEIG_BRUNNENGARTEN = 'de:08117:65:0:R1'  # Rtg Schönthalweg (Wiesensteig Wiesensteig)
	MÜHLHAUSEN_I_T__KIRCHE = 'de:08117:66:0:H1'  # Rtg Bahnhofstraße (Mühlhausen im Täle Mühlhausen im Täle)
	GOSBACH_WIESENSTEIGER_STR = 'de:08117:71'
	GOSBACH_WIESENSTEIGER_STR_1 = 'de:08117:71:0:H1'  # Rtg Mühlhausen i. T. (Bad Ditzenbach Gosbach)
	GOSBACH_WIESENSTEIGER_STR_2 = 'de:08117:71:0:R1'  # Rtg Abzw. Drackenstein (Bad Ditzenbach Gosbach)
	WIESENSTEIGER_STRASSE = 'de:08117:71'
	WIESENSTEIGER_STRASSE_1 = 'de:08117:71:0:H1'  # Rtg Mühlhausen i. T. (Bad Ditzenbach Gosbach)
	WIESENSTEIGER_STRASSE_2 = 'de:08117:71:0:R1'  # Rtg Abzw. Drackenstein (Bad Ditzenbach Gosbach)
	GOSBACH_HIRSCH = 'de:08117:70'
	GOSBACH_HIRSCH_1 = 'de:08117:70:0:2'  # Rtg Lamm (Bad Ditzenbach Gosbach)
	GOSBACH_HIRSCH_2 = 'de:08117:70:0:R1'  # Rtg Bad Ditzenbach (Bad Ditzenbach Gosbach)
	HIRSCH_1 = 'de:08117:70:0:2'  # Rtg Lamm (Bad Ditzenbach Gosbach)
	HIRSCH_2 = 'de:08117:70:0:R1'  # Rtg Bad Ditzenbach (Bad Ditzenbach Gosbach)
	HIRSCH_3 = 'de:08117:90:0:H1'  # Rtg Siedlung (Bad Überkingen Hausen (Bad Überkingen))
	HIRSCH_4 = 'de:08117:90:0:R1'  # Rtg Bad Überkingen (Bad Überkingen Hausen (Bad Überkingen))
	HIRSCH_5 = 'de:08117:7680:0:H1'  # Rtg Treffelhausen (Böhmenkirch Schnittlingen)
	HIRSCH_6 = 'de:08117:7680:0:R1'  # Rtg Siedlung (Böhmenkirch Schnittlingen)
	GÖPPINGEN_HANS_SEYFF_STR = 'de:08117:74:0:H1'  # Rtg Gartenfreude (Göppingen Göppingen)
	HANS_SEYFF_STRASSE = 'de:08117:74:0:H1'  # Rtg Gartenfreude (Göppingen Göppingen)
	DEGGINGEN_BIRKHOF = 'de:08117:10075'
	DEGGINGEN_BIRKHOF_1 = 'de:08117:10075:0:1'  # Rtg Ave Maria (Deggingen Deggingen)
	DEGGINGEN_BIRKHOF_2 = 'de:08117:10075:0:2'  # Rtg Reichenbach (Deggingen Deggingen)
	GÖPPINGEN_PLANCKWEG = 'de:08117:79'
	GÖPPINGEN_PLANCKWEG_1 = 'de:08117:79:0:H1'  # Planckweg (Göppingen Göppingen)
	GÖPPINGEN_PLANCKWEG_2 = 'de:08117:79:0:R1'  # Planckweg (Göppingen Göppingen)
	PLANCKWEG = 'de:08117:79'
	PLANCKWEG_1 = 'de:08117:79:0:H1'  # Planckweg (Göppingen Göppingen)
	PLANCKWEG_2 = 'de:08117:79:0:R1'  # Planckweg (Göppingen Göppingen)
	BIRENBACH_BRÜCKENSTR = 'de:08117:82:0:H1'  # Rtg Schurwaldweg (Birenbach Birenbach)
	GEISL__ST__MICHELBERGGYMNASIUM = 'de:08117:84'
	GEISL__ST__MICHELBERGGYMNASIUM_1 = 'de:08117:84:0:H1'  # Rtg Sternplatz ZOH (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__MICHELBERGGYMNASIUM_2 = 'de:08117:84:0:R1'  # Rtg Bebelstr (Geislingen an der Steige Geislingen (Steige))
	MICHELBERGGYMNASIUM = 'de:08117:84'
	MICHELBERGGYMNASIUM_1 = 'de:08117:84:0:H1'  # Rtg Sternplatz ZOH (Geislingen an der Steige Geislingen (Steige))
	MICHELBERGGYMNASIUM_2 = 'de:08117:84:0:R1'  # Rtg Bebelstr (Geislingen an der Steige Geislingen (Steige))
	GINGEN__F__BRÜCKENSTR = 'de:08117:86'
	GINGEN__F__BRÜCKENSTR_1 = 'de:08117:86:0:H1'  # Rtg Süßen (Gingen an der Fils Gingen (F))
	GINGEN__F__BRÜCKENSTR_2 = 'de:08117:86:0:R1'  # Rtg Brückenstr. (Gingen an der Fils Gingen (F))
	GINGEN__F__EV__KINDERGARTEN = 'de:08117:87'
	GINGEN__F__EV__KINDERGARTEN_1 = 'de:08117:87:0:H1'  # Rtg Brunnenstr. (Gingen an der Fils Gingen (F))
	GINGEN__F__EV__KINDERGARTEN_2 = 'de:08117:87:0:R1'  # Rtg Rathaus (Gingen an der Fils Gingen (F))
	EV__KINDERGARTEN_1 = 'de:08117:87:0:H1'  # Rtg Brunnenstr. (Gingen an der Fils Gingen (F))
	EV__KINDERGARTEN_2 = 'de:08117:87:0:R1'  # Rtg Rathaus (Gingen an der Fils Gingen (F))
	EV__KINDERGARTEN_3 = 'de:08136:7012:0:H1'  # Ev. Kindergarten (Lorch Lorch)
	HAUSEN__F__HIRSCH = 'de:08117:90'
	HAUSEN__F__HIRSCH_1 = 'de:08117:90:0:H1'  # Rtg Siedlung (Bad Überkingen Hausen (Bad Überkingen))
	HAUSEN__F__HIRSCH_2 = 'de:08117:90:0:R1'  # Rtg Bad Überkingen (Bad Überkingen Hausen (Bad Überkingen))
	HAUSEN__F__SIEDLUNG = 'de:08117:91'
	HAUSEN__F__SIEDLUNG_1 = 'de:08117:91:0:H1'  # Rtg Unterböhringen (Bad Überkingen Hausen (Bad Überkingen))
	HAUSEN__F__SIEDLUNG_2 = 'de:08117:91:0:R1'  # Rtg Hirsch (Bad Überkingen Hausen (Bad Überkingen))
	UNTERBÖHRINGEN = 'de:08117:92'
	UNTERBÖHRINGEN_1 = 'de:08117:92:0:H1'  # Ankunft (Bad Überkingen Unterböhringen)
	UNTERBÖHRINGEN_2 = 'de:08117:92:0:R1'  # Rtg Hausen (F) (Bad Überkingen Unterböhringen)
	GEISL__ST__ÄUSSERE_HAUPTSTR = 'de:08117:95'
	GEISL__ST__ÄUSSERE_HAUPTSTR_1 = 'de:08117:95:0:H1'  # Äußere Hauptstraße (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__ÄUSSERE_HAUPTSTR_2 = 'de:08117:95:0:R1'  # Rtg Friedhof (Geislingen an der Steige Geislingen (Steige))
	ÄUSSERE_HAUPTSTRASSE = 'de:08117:95'
	ÄUSSERE_HAUPTSTRASSE_1 = 'de:08117:95:0:H1'  # Äußere Hauptstraße (Geislingen an der Steige Geislingen (Steige))
	ÄUSSERE_HAUPTSTRASSE_2 = 'de:08117:95:0:R1'  # Rtg Friedhof (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__FRIEDHOF = 'de:08117:96:0:R1'  # Rtg Stadtkirche (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__SCHILDWACHTWEG = 'de:08117:100:0:H1'  # Rtg Wilhelmshöhe (Geislingen an der Steige Geislingen (Steige))
	SCHILDWACHTWEG = 'de:08117:100:0:H1'  # Rtg Wilhelmshöhe (Geislingen an der Steige Geislingen (Steige))
	WILHELMSHÖHE = 'de:08117:101:0:H1'  # Rtg Schulth.-Sch.-Str. (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__WILHELMSHÖHE = 'de:08117:101:0:H1'  # Rtg Schulth.-Sch.-Str. (Geislingen an der Steige Geislingen (Steige))
	SCHULTHEISS_SCHNEIDER_STRASSE = 'de:08117:102:0:H1'  # Rtg Y-Häußer (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__SCHULTH__SCH__STR = 'de:08117:102:0:H1'  # Rtg Y-Häußer (Geislingen an der Steige Geislingen (Steige))
	Y_HÄUSER = 'de:08117:103:0:H1'  # Rtg Birkenweg (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__Y_HÄUSER = 'de:08117:103:0:H1'  # Rtg Birkenweg (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__BIRKENWEG = 'de:08117:104:0:H1'  # Rtg Neuwiesen (B466) (Geislingen an der Steige Geislingen (Steige))
	KATZENLOCH = 'de:08117:105'
	KATZENLOCH_1 = 'de:08117:105:0:H1'  # Rtg Schlosshalde (Geislingen an der Steige Geislingen (Steige))
	KATZENLOCH_2 = 'de:08117:105:0:R1'  # Rtg Geisl.(St) ZOB (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__ST__KATZENLOCH = 'de:08117:105'
	GEISLINGEN__ST__KATZENLOCH_1 = 'de:08117:105:0:H1'  # Rtg Schlosshalde (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__ST__KATZENLOCH_2 = 'de:08117:105:0:R1'  # Rtg Geisl.(St) ZOB (Geislingen an der Steige Geislingen (Steige))
	SCHLOSSHALDE = 'de:08117:106'
	SCHLOSSHALDE_1 = 'de:08117:106:0:H1'  # Rtg Weiler ob Helfenstein (Geislingen an der Steige Geislingen (Steige))
	SCHLOSSHALDE_2 = 'de:08117:106:0:R1'  # Rtg Katzenloch (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__SCHLOSSHALDE = 'de:08117:106'
	GEISL__ST__SCHLOSSHALDE_1 = 'de:08117:106:0:H1'  # Rtg Weiler ob Helfenstein (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__SCHLOSSHALDE_2 = 'de:08117:106:0:R1'  # Rtg Katzenloch (Geislingen an der Steige Geislingen (Steige))
	WEILER_HELFENSTEIN = 'de:08117:107'
	WEILER_HELFENSTEIN_1 = 'de:08117:107:0:H1'  # Rtg Unterdorf (Geislingen an der Steige Weiler ob Helfenstn.)
	WEILER_HELFENSTEIN_2 = 'de:08117:107:0:R1'  # Rtg Geisl.(St) (Geislingen an der Steige Weiler ob Helfenstn.)
	WEILER_OB_HELFENSTEIN = 'de:08117:107'
	WEILER_OB_HELFENSTEIN_1 = 'de:08117:107:0:H1'  # Rtg Unterdorf (Geislingen an der Steige Weiler ob Helfenstn.)
	WEILER_OB_HELFENSTEIN_2 = 'de:08117:107:0:R1'  # Rtg Geisl.(St) (Geislingen an der Steige Weiler ob Helfenstn.)
	WEILER_OB_HELFENSTN__UNTERDORF = 'de:08117:108:0:H1'  # Rtg Oberdorf (Geislingen an der Steige Weiler ob Helfenstn.)
	UNTERDORF = 'de:08117:108:0:H1'  # Rtg Oberdorf (Geislingen an der Steige Weiler ob Helfenstn.)
	WEILER_OB_HELFENSTN__OBERDORF = 'de:08117:109:0:H1'  # Rtg Geislingen (Steige) (Geislingen an der Steige Weiler ob Helfenstn.)
	OBERDORF = 'de:08117:109:0:H1'  # Rtg Geislingen (Steige) (Geislingen an der Steige Weiler ob Helfenstn.)
	GEISLINGEN__STEIGE__WEILER_LINDENHOF = 'de:08117:111'
	GEISLINGEN__STEIGE__WEILER_LINDENHOF_1 = 'de:08117:111:0:H1'  # Weiler Lindenhof (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__STEIGE__WEILER_LINDENHOF_2 = 'de:08117:111:0:R1'  # Weiler Lindenhof (Geislingen an der Steige Geislingen (Steige))
	WEILER_LINDENHOF = 'de:08117:111'
	WEILER_LINDENHOF_1 = 'de:08117:111:0:H1'  # Weiler Lindenhof (Geislingen an der Steige Geislingen (Steige))
	WEILER_LINDENHOF_2 = 'de:08117:111:0:R1'  # Weiler Lindenhof (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__NEUW__GEWERBEGEBIET = 'de:08117:114:0:H1'  # Rtg Neuwiesen (B466) (Geislingen an der Steige Geislingen (Steige))
	NEUWIESEN_GEWERBEGEBIET = 'de:08117:114:0:H1'  # Rtg Neuwiesen (B466) (Geislingen an der Steige Geislingen (Steige))
	MILCHZENTRALE = 'de:08117:115:0:H1'  # Rtg Kantstr. (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__MILCHZENTRALE = 'de:08117:115:0:H1'  # Rtg Kantstr. (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__ST__KANTSTR = 'de:08117:116'
	GEISLINGEN__ST__KANTSTR_1 = 'de:08117:116:0:H1'  # Rtg Wölkbad (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__ST__KANTSTR_2 = 'de:08117:116:0:R1'  # Rtg Weing./Tegelberg (Geislingen an der Steige Geislingen (Steige))
	WÖLKBAD = 'de:08117:117'
	WÖLKBAD_1 = 'de:08117:117:0:H1'  # Rtg Berufsschulzentrum (Geislingen an der Steige Geislingen (Steige))
	WÖLKBAD_2 = 'de:08117:117:0:R1'  # Rtg Kantstr. (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__WÖLKBAD = 'de:08117:117'
	GEISL__ST__WÖLKBAD_1 = 'de:08117:117:0:H1'  # Rtg Berufsschulzentrum (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__WÖLKBAD_2 = 'de:08117:117:0:R1'  # Rtg Kantstr. (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__BERUFSSCHULZENTRUM = 'de:08117:118'
	GEISL__ST__BERUFSSCHULZENTRUM_1 = 'de:08117:118:0:H1'  # Rtg Lindenschule/Kuchen (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__BERUFSSCHULZENTRUM_2 = 'de:08117:118:0:R1'  # Rtg Wölkbad/Geislingen Bf (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__LINDENHOF = 'de:08117:119'
	GEISL__ST__LINDENHOF_1 = 'de:08117:119:0:H1'  # Rtg Heiligenäcker (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__LINDENHOF_2 = 'de:08117:119:0:R1'  # Rtg Berufsschulzentrum (Geislingen an der Steige Geislingen (Steige))
	HEILIGENÄCKER = 'de:08117:120:0:H1'  # Rtg Lindenhof (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__HEILIGENÄCKER = 'de:08117:120:0:H1'  # Rtg Lindenhof (Geislingen an der Steige Geislingen (Steige))
	SPARWIESEN_GRUNDSCHULE = 'de:08117:10121:0:3'  # eRtg alle Richtungen (Uhingen Sparwiesen)
	GINGEN__F__BRUNNENSTR = 'de:08117:122'
	GINGEN__F__BRUNNENSTR_1 = 'de:08117:122:0:H1'  # Rtg Brückenstr. (Gingen an der Fils Gingen (F))
	GINGEN__F__BRUNNENSTR_2 = 'de:08117:122:0:R1'  # Rtg Ev. Kindergarten (Gingen an der Fils Gingen (F))
	GEISL__ST__WEING__TEGELBERG = 'de:08117:125:0:R1'  # Rtg Sternplatz ZOH (Geislingen an der Steige Geislingen (Steige))
	WEINGÄRTEN_TEGELBERG = 'de:08117:125:0:R1'  # Rtg Sternplatz ZOH (Geislingen an der Steige Geislingen (Steige))
	RIESENGEBIRGSTRASSE = 'de:08117:133:0:H1'  # Rtg Wangen (Göppingen Faurndau)
	FAURNDAU_RIESENGEBIRGSTR = 'de:08117:133:0:H1'  # Rtg Wangen (Göppingen Faurndau)
	GEISL__ST__LINDENSCHULE = 'de:08117:143:0:H1'  # Rtg Heiligenäcker (Geislingen an der Steige Geislingen (Steige))
	LINDENSCHULE = 'de:08117:143:0:H1'  # Rtg Heiligenäcker (Geislingen an der Steige Geislingen (Steige))
	EBERSBACH__F = 'de:08117:151'
	EBERSBACH__F_1 = 'de:08117:151:1:R1'  # Bus (Ebersbach an der Fils Ebersbach (F))
	EBERSBACH__F_2 = 'de:08117:151:20:1'  # Gleis 1 (Ebersbach an der Fils Ebersbach (F))
	EBERSBACH__F_3 = 'de:08117:151:21:2'  # Gleis 2 (Ebersbach an der Fils Ebersbach (F))
	UHINGEN_1 = 'de:08117:10152:1:3'  # Rtg Faurndau (Uhingen Uhingen)
	UHINGEN_2 = 'de:08117:152:1:R1'  # Rtg Ebersbach (Uhingen Uhingen)
	UHINGEN_3 = 'de:08117:152:20:1'  # Gleis 1 (Uhingen Uhingen)
	UHINGEN_4 = 'de:08117:152:21:2'  # Gleis 2 (Uhingen Uhingen)
	FAURNDAU = 'de:08117:153'
	FAURNDAU_1 = 'de:08117:153:1:H1'  # Rtg Mercedesstr. (Göppingen Faurndau)
	FAURNDAU_2 = 'de:08117:153:1:R1'  # Rtg Goethestr. (Göppingen Faurndau)
	FAURNDAU_3 = 'de:08117:153:20:1'  # Gleis 1 (Göppingen Faurndau)
	FAURNDAU_4 = 'de:08117:153:21:2'  # Gleis 2 (Göppingen Faurndau)
	GÖPPINGEN_1 = 'de:08117:154:20:1'  # Gleis 1 (Göppingen Göppingen)
	GÖPPINGEN_2 = 'de:08117:154:22:4'  # Gleis 4 (Göppingen Göppingen)
	GÖPPINGEN_3 = 'de:08117:154:23:6'  # Gleis 6 (Göppingen Göppingen)
	GÖPPINGEN_4 = 'de:08117:154:23:7'  # Gleis 7 (Göppingen Göppingen)
	GÖPPINGEN_5 = 'de:08117:9001:0:A'  # Bussteig A (Göppingen Göppingen)
	GÖPPINGEN_6 = 'de:08117:9001:0:B'  # Bussteig B (Göppingen Göppingen)
	GÖPPINGEN_7 = 'de:08117:9001:0:C'  # Bussteig C (Göppingen Göppingen)
	GÖPPINGEN_8 = 'de:08117:9001:0:D'  # Bussteig D (Göppingen Göppingen)
	GÖPPINGEN_9 = 'de:08117:9001:0:E'  # Bussteig E (Göppingen Göppingen)
	GÖPPINGEN_10 = 'de:08117:9001:0:F'  # Bussteig F (Göppingen Göppingen)
	GÖPPINGEN_11 = 'de:08117:9001:0:G'  # Bussteig G (Göppingen Göppingen)
	GÖPPINGEN_12 = 'de:08117:9001:0:H'  # Bussteig H (Göppingen Göppingen)
	GÖPPINGEN_13 = 'de:08117:9001:0:I'  # Bussteig I (Göppingen Göppingen)
	GÖPPINGEN_14 = 'de:08117:9001:0:K'  # Bussteig K (Göppingen Göppingen)
	GÖPPINGEN_15 = 'de:08117:9001:0:L'  # Bussteig L (Göppingen Göppingen)
	GÖPPINGEN_16 = 'de:08117:9001:0:M'  # Bussteig M (Göppingen Göppingen)
	GÖPPINGEN_17 = 'de:08117:9001:0:N'  # Bussteig N (Göppingen Göppingen)
	GÖPPINGEN_18 = 'de:08117:9001:0:O'  # Bussteig O (Göppingen Göppingen)
	GÖPPINGEN_19 = 'de:08117:9001:0:P'  # Bussteig P (Göppingen Göppingen)
	GÖPPINGEN_20 = 'de:08117:9001:0:R'  # Bussteig R (Göppingen Göppingen)
	GÖPPINGEN_21 = 'de:08117:9001:0:Z'  # Bussteig Z (Göppingen Göppingen)
	EISLINGEN__F = 'de:08117:155'
	EISLINGEN__F_1 = 'de:08117:155:20:1'  # Gleis 2 (Eislingen/Fils Eislingen (F))
	EISLINGEN__F_2 = 'de:08117:155:21:21'  # Gleis 1 (Eislingen/Fils Eislingen (F))
	EISLINGEN__F_3 = 'de:08117:155:21:3'  # Gleis 3 (Eislingen/Fils Eislingen (F))
	SALACH = 'de:08117:156'
	SALACH_1 = 'de:08117:156:20:1'  # Gleis 1 (Salach Salach)
	SALACH_2 = 'de:08117:156:21:2'  # Gleis 2 (Salach Salach)
	SÜSSEN_1 = 'de:08117:157:20:1'  # Gleis 1 (Süßen Süßen)
	SÜSSEN_2 = 'de:08117:157:21:2'  # Gleis 2 (Süßen Süßen)
	SÜSSEN_3 = 'de:08117:157:21:3'  # Gleis 3 (Süßen Süßen)
	SÜSSEN_4 = 'de:08117:301:0:A'  # eRtg Gingen (Süßen Süßen)
	SÜSSEN_5 = 'de:08117:301:0:C'  # Linie 7689 (Reichenbach) (Süßen Süßen)
	SÜSSEN_6 = 'de:08117:301:0:D'  # Linie 7689 (Wißgoldingen) (Süßen Süßen)
	SÜSSEN_7 = 'de:08117:301:0:F'  # Linie 7680 (Süßen Süßen)
	SÜSSEN_8 = 'de:08117:301:0:G'  # Linie 7688 (Göppingen) (Süßen Süßen)
	GINGEN__F = 'de:08117:158'
	GINGEN__F_1 = 'de:08117:158:20:1'  # Gleis 1 (Gingen an der Fils Gingen (F))
	GINGEN__F_2 = 'de:08117:158:21:2'  # Gleis 2 (Gingen an der Fils Gingen (F))
	KUCHEN = 'de:08117:159'
	KUCHEN_1 = 'de:08117:159:20:1'  # Gleis 1 (Kuchen Kuchen (F))
	KUCHEN_2 = 'de:08117:159:21:2'  # Gleis 2 (Kuchen Kuchen (F))
	GEISLINGEN_WEST = 'de:08117:160'
	GEISLINGEN_WEST_1 = 'de:08117:160:20:1'  # Gleis 101 (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN_WEST_2 = 'de:08117:160:21:2'  # Gleis 103 (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__STEIGE_1 = 'de:08117:161:20:1'  # Gleis 1 (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__STEIGE_2 = 'de:08117:161:21:2'  # Gleis 2 (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__STEIGE_3 = 'de:08117:161:21:3'  # Gleis 3 (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__STEIGE_4 = 'de:08117:6001:0:A'  # Bstg A (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__STEIGE_5 = 'de:08117:6001:0:B'  # Bstg B (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__STEIGE_6 = 'de:08117:6001:0:C'  # Bstg C (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__STEIGE_7 = 'de:08117:6001:0:D'  # Bstg D (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__STEIGE_8 = 'de:08117:6001:0:E'  # Bstg E (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__STEIGE_9 = 'de:08117:6001:0:F'  # Bstg F (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__STEIGE_10 = 'de:08117:6001:0:G'  # Bstg G (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__STEIGE_11 = 'de:08117:6001:0:H'  # Bstg H (Geislingen an der Steige Geislingen (Steige))
	EYBACH_OSSMANNSWEILER = 'de:08117:187'
	EYBACH_OSSMANNSWEILER_1 = 'de:08117:187:0:H1'  # Rtg Pfingsthalde (Geislingen an der Steige Eybach)
	EYBACH_OSSMANNSWEILER_2 = 'de:08117:187:0:R1'  # Rtg Geisl. (St.) (Geislingen an der Steige Eybach)
	OSSMANNSWEILER = 'de:08117:187'
	OSSMANNSWEILER_1 = 'de:08117:187:0:H1'  # Rtg Pfingsthalde (Geislingen an der Steige Eybach)
	OSSMANNSWEILER_2 = 'de:08117:187:0:R1'  # Rtg Geisl. (St.) (Geislingen an der Steige Eybach)
	GEISL__ST__EISENBAHNBRÜCKE = 'de:08117:199'
	GEISL__ST__EISENBAHNBRÜCKE_1 = 'de:08117:199:0:H1'  # Rtg Stötten (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__EISENBAHNBRÜCKE_2 = 'de:08117:199:0:R1'  # Rtg Weing./Tegelberg (Geislingen an der Steige Geislingen (Steige))
	EISENBAHNBRÜCKE = 'de:08117:199'
	EISENBAHNBRÜCKE_1 = 'de:08117:199:0:H1'  # Rtg Stötten (Geislingen an der Steige Geislingen (Steige))
	EISENBAHNBRÜCKE_2 = 'de:08117:199:0:R1'  # Rtg Weing./Tegelberg (Geislingen an der Steige Geislingen (Steige))
	BÖHMENKIRCH_FRIEDHOF = 'de:08117:201'
	BÖHMENKIRCH_FRIEDHOF_1 = 'de:08117:201:0:2'  # Rtg Rathaus (Böhmenkirch Böhmenkirch)
	BÖHMENKIRCH_FRIEDHOF_2 = 'de:08117:201:0:H1'  # Rtg Post (Böhmenkirch Böhmenkirch)
	BÖHMENKIRCH_POST = 'de:08117:202'
	BÖHMENKIRCH_POST_1 = 'de:08117:202:0:H1'  # Rtg Söhnstetten (Böhmenkirch Böhmenkirch)
	BÖHMENKIRCH_POST_2 = 'de:08117:202:0:R1'  # Rtg Lauterstein/Geislingen (Böhmenkirch Böhmenkirch)
	BÖHMENKIRCH_RATHAUS = 'de:08117:203'
	BÖHMENKIRCH_RATHAUS_1 = 'de:08117:203:0:H1'  # Rtg Söhnstetten (Böhmenkirch Böhmenkirch)
	BÖHMENKIRCH_RATHAUS_2 = 'de:08117:203:0:R1'  # Rtg Lauterstein/Geislingen (Böhmenkirch Böhmenkirch)
	BÖHMENKIRCH_KIRCHSTRASSE = 'de:08117:204'
	BÖHMENKIRCH_KIRCHSTRASSE_1 = 'de:08117:204:0:H1'  # Rtg Böhmenkirch (Böhmenkirch Böhmenkirch)
	BÖHMENKIRCH_KIRCHSTRASSE_2 = 'de:08117:204:0:R1'  # Rtg Steinenkirch/Geislingen (Böhmenkirch Böhmenkirch)
	KIRCHSTRASSE_1 = 'de:08117:204:0:H1'  # Rtg Böhmenkirch (Böhmenkirch Böhmenkirch)
	KIRCHSTRASSE_2 = 'de:08117:204:0:R1'  # Rtg Steinenkirch/Geislingen (Böhmenkirch Böhmenkirch)
	KIRCHSTRASSE_3 = 'de:08117:3308:0:H1'  # Rtg Eschenbach (Heiningen Heiningen (GP))
	BÖHMENKIRCH_LINDENHOF = 'de:08117:205'
	BÖHMENKIRCH_LINDENHOF_1 = 'de:08117:205:0:H1'  # Rtg Böhmenkirch (Böhmenkirch Böhmenkirch)
	BÖHMENKIRCH_LINDENHOF_2 = 'de:08117:205:0:R1'  # Rtg Steinenkirch/Geislingen (Böhmenkirch Böhmenkirch)
	STEINENKIRCH = 'de:08117:206'
	STEINENKIRCH_1 = 'de:08117:206:0:H1'  # Rtg Böhmenkirch (Böhmenkirch Steinenkirch)
	STEINENKIRCH_2 = 'de:08117:206:0:R1'  # Rtg Geislingen (Böhmenkirch Steinenkirch)
	EYBACH_ROGGENMÜHLE = 'de:08117:207'
	EYBACH_ROGGENMÜHLE_1 = 'de:08117:207:0:H1'  # Rtg Steinenkirch (Geislingen an der Steige Eybach)
	EYBACH_ROGGENMÜHLE_2 = 'de:08117:207:0:R1'  # Rtg Ortsmitte (Geislingen an der Steige Eybach)
	ROGGENMÜHLE = 'de:08117:207'
	ROGGENMÜHLE_1 = 'de:08117:207:0:H1'  # Rtg Steinenkirch (Geislingen an der Steige Eybach)
	ROGGENMÜHLE_2 = 'de:08117:207:0:R1'  # Rtg Ortsmitte (Geislingen an der Steige Eybach)
	EYBACH_ORTSMITTE = 'de:08117:208'
	EYBACH_ORTSMITTE_1 = 'de:08117:208:0:H1'  # Roggenmühle (Geislingen an der Steige Eybach)
	EYBACH_ORTSMITTE_2 = 'de:08117:208:0:R1'  # Rtg Eybtalhalle (Geislingen an der Steige Eybach)
	PFINGSTHALDE = 'de:08117:209'
	PFINGSTHALDE_1 = 'de:08117:209:0:H1'  # Rtg Eybtalhalle (Geislingen an der Steige Eybach)
	PFINGSTHALDE_2 = 'de:08117:209:0:R1'  # Rtg Oßmannsweiler (Geislingen an der Steige Eybach)
	EYBACH_PFINGSTHALDE = 'de:08117:209'
	EYBACH_PFINGSTHALDE_1 = 'de:08117:209:0:H1'  # Rtg Eybtalhalle (Geislingen an der Steige Eybach)
	EYBACH_PFINGSTHALDE_2 = 'de:08117:209:0:R1'  # Rtg Oßmannsweiler (Geislingen an der Steige Eybach)
	SC_STADION = 'de:08117:210'
	SC_STADION_1 = 'de:08117:210:0:H1'  # Rtg Eybach (Geislingen an der Steige Geislingen (Steige))
	SC_STADION_2 = 'de:08117:210:0:R1'  # Rtg Eisenbahnbrücke (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__SC_STADION = 'de:08117:210'
	GEISL__ST__SC_STADION_1 = 'de:08117:210:0:H1'  # Rtg Eybach (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__SC_STADION_2 = 'de:08117:210:0:R1'  # Rtg Eisenbahnbrücke (Geislingen an der Steige Geislingen (Steige))
	VOSSSTRASSE = 'de:08117:211'
	VOSSSTRASSE_1 = 'de:08117:211:0:H1'  # Rtg Weing./Tegelberg (Geislingen an der Steige Geislingen (Steige))
	VOSSSTRASSE_2 = 'de:08117:211:0:R1'  # Rtg Krankenhaus (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__ST__VOSSSTR = 'de:08117:211'
	GEISLINGEN__ST__VOSSSTR_1 = 'de:08117:211:0:H1'  # Rtg Weing./Tegelberg (Geislingen an der Steige Geislingen (Steige))
	GEISLINGEN__ST__VOSSSTR_2 = 'de:08117:211:0:R1'  # Rtg Krankenhaus (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__KRANKENHAUS = 'de:08117:212'
	GEISL__ST__KRANKENHAUS_1 = 'de:08117:212:0:H1'  # Rtg Vossstr. (Geislingen an der Steige Geislingen (Steige))
	GEISL__ST__KRANKENHAUS_2 = 'de:08117:212:0:R1'  # Rtg Geisl.(St) ZOB (Geislingen an der Steige Geislingen (Steige))
	WALDHAUSEN_ORTSMITTE = 'de:08117:213'
	WALDHAUSEN_ORTSMITTE_1 = 'de:08117:213:0:H1'  # Ankunft (Geislingen an der Steige Waldhausen (Geislingen))
	WALDHAUSEN_ORTSMITTE_2 = 'de:08117:213:0:R1'  # Rtg Eybach (Geislingen an der Steige Waldhausen (Geislingen))
	WALDHAUSEN_CHRISTOFSHOF = 'de:08117:10214'
	WALDHAUSEN_CHRISTOFSHOF_1 = 'de:08117:10214:0:1'  # Rtg Waldhausen (Geislingen an der Steige Waldhausen (Geislingen))
	WALDHAUSEN_CHRISTOFSHOF_2 = 'de:08117:10214:0:2'  # Rtg Eybach (Geislingen an der Steige Waldhausen (Geislingen))
	CHRISTOFSHOF = 'de:08117:10214'
	CHRISTOFSHOF_1 = 'de:08117:10214:0:1'  # Rtg Waldhausen (Geislingen an der Steige Waldhausen (Geislingen))
	CHRISTOFSHOF_2 = 'de:08117:10214:0:2'  # Rtg Eybach (Geislingen an der Steige Waldhausen (Geislingen))
	HEIDHÖFE_SCHAFHAUS = 'de:08117:215:0:H1'  # Rtg Friedhof (Böhmenkirch Böhmenkirch)
	BÖHMENKIRCH_HEIDHÖFE_SCHAFHAUS = 'de:08117:215:0:H1'  # Rtg Friedhof (Böhmenkirch Böhmenkirch)
	EYBACH_EYBTALHALLE = 'de:08117:216:0:H1'  # Rtg Ortsmitte (Geislingen an der Steige Eybach)
	EYBTALHALLE = 'de:08117:216:0:H1'  # Rtg Ortsmitte (Geislingen an der Steige Eybach)
	BREITER_WEG_1 = 'de:08117:217:0:H1'  # Breiter Weg (Böhmenkirch Böhmenkirch)
	BREITER_WEG_2 = 'de:08235:10208:0:3'  # Rtg Ebhausen Rathaus (Ebhausen Ebhausen)
	BREITER_WEG_3 = 'de:08235:10208:0:4'  # Rtg Altensteig Berneck (Ebhausen Ebhausen)
	BÖHMENKIRCH_BREITER_WEG = 'de:08117:217:0:H1'  # Breiter Weg (Böhmenkirch Böhmenkirch)
	HEIDHÖFE_ALLEENSTRASSE = 'de:08117:10218:0:1'  # Rtg Böhmenkirch (Böhmenkirch Böhmenkirch)
	BÖHMENKIR__HEIDHÖFE_ALLEENSTR = 'de:08117:10218:0:1'  # Rtg Böhmenkirch (Böhmenkirch Böhmenkirch)
	AUFHAUSEN = 'de:08117:257:0:H1'  # Rtg Wannenhöfe (Geislingen an der Steige Aufhausen (Geislingen))
	TÜRKHEIM = 'de:08117:258'
	TÜRKHEIM_1 = 'de:08117:258:0:H1'  # Rtg Aufhausen/Nellingen (Geislingen an der Steige Türkheim)
	TÜRKHEIM_2 = 'de:08117:258:0:R1'  # Rtg Geislingen (Geislingen an der Steige Türkheim)
	GEWERBE = 'de:8117:275:0:H1'  # Rtg Boschhof (Geislingen an der Steige Türkheim)
	TÜRKHEIM_GEWERBE = 'de:8117:275:0:H1'  # Rtg Boschhof (Geislingen an der Steige Türkheim)
	SÜSSEN_MÜHLSTRASSE = 'de:08117:303:0:R1'  # Rtg Stiegelwiese (Süßen Süßen)
	DONZDORF_STEINGARTENSCHULE = 'de:08117:310:0:H1'  # Rtg Lautergarten (Donzdorf Donzdorf)
	STEINGARTENSCHULE = 'de:08117:310:0:H1'  # Rtg Lautergarten (Donzdorf Donzdorf)
	STEIGHOF_ABZWEIG_TREFFELHAUSEN = 'de:08117:315'
	STEIGHOF_ABZWEIG_TREFFELHAUSEN_1 = 'de:08117:315:0:H1'  # Rtg Böhmenkirch (Lauterstein Lauterstein)
	STEIGHOF_ABZWEIG_TREFFELHAUSEN_2 = 'de:08117:315:0:R1'  # Rtg Weißenstein (Lauterstein Lauterstein)
	LSTEIN__STEIGHOF_ABZW__TREFHSN = 'de:08117:315'
	LSTEIN__STEIGHOF_ABZW__TREFHSN_1 = 'de:08117:315:0:H1'  # Rtg Böhmenkirch (Lauterstein Lauterstein)
	LSTEIN__STEIGHOF_ABZW__TREFHSN_2 = 'de:08117:315:0:R1'  # Rtg Weißenstein (Lauterstein Lauterstein)
	BÖHMENKIRCH_ALBSPORTHALLE = 'de:08117:316'
	BÖHMENKIRCH_ALBSPORTHALLE_1 = 'de:08117:316:0:H1'  # Rtg Böhmenkirch (Böhmenkirch Böhmenkirch)
	BÖHMENKIRCH_ALBSPORTHALLE_2 = 'de:08117:316:0:R1'  # Rtg Lauterstein (Böhmenkirch Böhmenkirch)
	ALBSPORTHALLE = 'de:08117:316'
	ALBSPORTHALLE_1 = 'de:08117:316:0:H1'  # Rtg Böhmenkirch (Böhmenkirch Böhmenkirch)
	ALBSPORTHALLE_2 = 'de:08117:316:0:R1'  # Rtg Lauterstein (Böhmenkirch Böhmenkirch)
	TREFFELHAUSEN_SONNE = 'de:08117:317'
	TREFFELHAUSEN_SONNE_1 = 'de:08117:317:0:H1'  # Rtg Lamm (Böhmenkirch Treffelhausen)
	TREFFELHAUSEN_SONNE_2 = 'de:08117:317:0:R1'  # Rtg Schnittlingen (Böhmenkirch Treffelhausen)
	SCHNITTLINGEN_SIEDLUNG = 'de:08117:318'
	SCHNITTLINGEN_SIEDLUNG_1 = 'de:08117:318:0:H1'  # Rtg Hirsch (Böhmenkirch Schnittlingen)
	SCHNITTLINGEN_SIEDLUNG_2 = 'de:08117:318:0:R1'  # Rtg Donzdorf (Böhmenkirch Schnittlingen)
	WETTERWARTE = 'de:08117:319'
	WETTERWARTE_1 = 'de:08117:319:0:H1'  # Rtg Donzdorf (Böhmenkirch Schnittlingen)
	WETTERWARTE_2 = 'de:08117:319:0:R1'  # Rtg Stötten (Böhmenkirch Schnittlingen)
	SCHNITTLINGEN_WETTERWARTE = 'de:08117:319'
	SCHNITTLINGEN_WETTERWARTE_1 = 'de:08117:319:0:H1'  # Rtg Donzdorf (Böhmenkirch Schnittlingen)
	SCHNITTLINGEN_WETTERWARTE_2 = 'de:08117:319:0:R1'  # Rtg Stötten (Böhmenkirch Schnittlingen)
	STÖTTEN_AUSSIEDLERHÖFE = 'de:08117:320'
	STÖTTEN_AUSSIEDLERHÖFE_1 = 'de:08117:320:0:H1'  # Rtg Rathaus (Geislingen an der Steige Stötten)
	STÖTTEN_AUSSIEDLERHÖFE_2 = 'de:08117:320:0:R1'  # Rtg Geisl.(St.) (Geislingen an der Steige Stötten)
	NENNINGEN_RATHAUS = 'de:08117:329'
	NENNINGEN_RATHAUS_1 = 'de:08117:329:0:H1'  # Rtg Lauterstein (Lauterstein Nenningen)
	NENNINGEN_RATHAUS_2 = 'de:08117:329:0:R1'  # Rtg Kirche (Lauterstein Nenningen)
	LAUTERSTEIN_ABZW__DEGENFELD = 'de:08117:330'
	LAUTERSTEIN_ABZW__DEGENFELD_1 = 'de:08117:330:0:H1'  # Rtg Weißenstein (Lauterstein Lauterstein)
	LAUTERSTEIN_ABZW__DEGENFELD_2 = 'de:08117:330:0:R1'  # Rtg Nenningen (Lauterstein Lauterstein)
	ABZWEIG_DEGENFELD = 'de:08117:330'
	ABZWEIG_DEGENFELD_1 = 'de:08117:330:0:H1'  # Rtg Weißenstein (Lauterstein Lauterstein)
	ABZWEIG_DEGENFELD_2 = 'de:08117:330:0:R1'  # Rtg Nenningen (Lauterstein Lauterstein)
	WANNENHÖFE = 'de:08117:497'
	WANNENHÖFE_1 = 'de:08117:497:0:H1'  # Rtg Nellingen (Geislingen an der Steige Aufhausen (Geislingen))
	WANNENHÖFE_2 = 'de:08117:497:0:R1'  # Rtg Aufhausen (Geislingen an der Steige Aufhausen (Geislingen))
	AUFHAUSEN_WANNENHÖFE = 'de:08117:497'
	AUFHAUSEN_WANNENHÖFE_1 = 'de:08117:497:0:H1'  # Rtg Nellingen (Geislingen an der Steige Aufhausen (Geislingen))
	AUFHAUSEN_WANNENHÖFE_2 = 'de:08117:497:0:R1'  # Rtg Aufhausen (Geislingen an der Steige Aufhausen (Geislingen))
	AUFHAUSEN_SCHULE = 'de:08117:10498:0:1'  # Rtg Aufhausen (Geislingen an der Steige Aufhausen (Geislingen))
	AUFHAUSEN_KRÖNERHOF = 'de:08117:511'
	AUFHAUSEN_KRÖNERHOF_1 = 'de:08117:511:0:H1'  # Rtg Aufhausen (Geislingen an der Steige Aufhausen (Geislingen))
	AUFHAUSEN_KRÖNERHOF_2 = 'de:08117:511:0:R1'  # Rtg Boschhof (Geislingen an der Steige Aufhausen (Geislingen))
	KRÖNERHOF = 'de:08117:511'
	KRÖNERHOF_1 = 'de:08117:511:0:H1'  # Rtg Aufhausen (Geislingen an der Steige Aufhausen (Geislingen))
	KRÖNERHOF_2 = 'de:08117:511:0:R1'  # Rtg Boschhof (Geislingen an der Steige Aufhausen (Geislingen))
	TÜRKHEIM_BOSCHHOF = 'de:08117:512'
	TÜRKHEIM_BOSCHHOF_1 = 'de:08117:512:0:H1'  # Rtg Aufhausen (Geislingen an der Steige Türkheim)
	TÜRKHEIM_BOSCHHOF_2 = 'de:08117:512:0:R1'  # Rtg Türkheim (Geislingen an der Steige Türkheim)
	BOSCHHOF = 'de:08117:512'
	BOSCHHOF_1 = 'de:08117:512:0:H1'  # Rtg Aufhausen (Geislingen an der Steige Türkheim)
	BOSCHHOF_2 = 'de:08117:512:0:R1'  # Rtg Türkheim (Geislingen an der Steige Türkheim)
	GÖPPINGEN_KANTSTR = 'de:08117:1000'
	GÖPPINGEN_KANTSTR_1 = 'de:08117:1000:0:H1'  # Rtg Stauferpark (Göppingen Göppingen)
	GÖPPINGEN_KANTSTR_2 = 'de:08117:1000:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_SCHILLERPLATZ = 'de:08117:1001:0:H1'  # Rtg Rathaus (Göppingen Göppingen)
	KAISERBAU__AGENTUR_FÜR_ARBEIT = 'de:08117:1004'
	KAISERBAU__AGENTUR_FÜR_ARBEIT_1 = 'de:08117:1004:0:H1'  # RBS Rtg Eislingen (Göppingen Göppingen)
	KAISERBAU__AGENTUR_FÜR_ARBEIT_2 = 'de:08117:1004:0:H2'  # OHB Rtg Heiningen (Göppingen Göppingen)
	KAISERBAU__AGENTUR_FÜR_ARBEIT_3 = 'de:08117:1004:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPP__KAISERBAU__AG__FÜR_ARB = 'de:08117:1004'
	GÖPP__KAISERBAU__AG__FÜR_ARB_1 = 'de:08117:1004:0:H1'  # RBS Rtg Eislingen (Göppingen Göppingen)
	GÖPP__KAISERBAU__AG__FÜR_ARB_2 = 'de:08117:1004:0:H2'  # OHB Rtg Heiningen (Göppingen Göppingen)
	GÖPP__KAISERBAU__AG__FÜR_ARB_3 = 'de:08117:1004:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPP__ULMER__HEININGER_STR = 'de:08117:1005:0:H1'  # Rtg Fuchseckstr. (Göppingen Göppingen)
	ULMER__HEININGER_STR = 'de:08117:1005:0:H1'  # Rtg Fuchseckstr. (Göppingen Göppingen)
	GÖPPINGEN_FREIBAD = 'de:08117:1006'
	GÖPPINGEN_FREIBAD_1 = 'de:08117:1006:0:H1'  # Rtg Eislingen (Göppingen Göppingen)
	GÖPPINGEN_FREIBAD_2 = 'de:08117:1006:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_REUTLINGER_STR = 'de:08117:1007'
	GÖPPINGEN_REUTLINGER_STR_1 = 'de:08117:1007:0:H1'  # Rtg Eislingen (Göppingen Göppingen)
	GÖPPINGEN_REUTLINGER_STR_2 = 'de:08117:1007:0:R1'  # Rtg Freibad (Göppingen Göppingen)
	IMMELMANNZENTRUM = 'de:08117:1008'
	IMMELMANNZENTRUM_1 = 'de:08117:1008:0:H1'  # Rtg Eislingen (Göppingen Göppingen)
	IMMELMANNZENTRUM_2 = 'de:08117:1008:0:R1'  # Rtg Göppingen (Göppingen Göppingen)
	GÖPPINGEN_IMMELMANNZENTRUM = 'de:08117:1008'
	GÖPPINGEN_IMMELMANNZENTRUM_1 = 'de:08117:1008:0:H1'  # Rtg Eislingen (Göppingen Göppingen)
	GÖPPINGEN_IMMELMANNZENTRUM_2 = 'de:08117:1008:0:R1'  # Rtg Göppingen (Göppingen Göppingen)
	EISLINGEN__F__STEINBEISSTR = 'de:08117:1009:0:H1'  # Rtg Liebfrauenkirche (Eislingen/Fils Eislingen (F))
	LIEBFRAUENKIRCHE = 'de:08117:1010'
	LIEBFRAUENKIRCHE_1 = 'de:08117:1010:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	LIEBFRAUENKIRCHE_2 = 'de:08117:1010:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__LIEBFRAUENKIRCHE = 'de:08117:1010'
	EISLINGEN__F__LIEBFRAUENKIRCHE_1 = 'de:08117:1010:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__LIEBFRAUENKIRCHE_2 = 'de:08117:1010:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	WELTESTRASSE = 'de:08117:1011'
	WELTESTRASSE_1 = 'de:08117:1011:0:H1'  # Rtg Eislingen/Salach (Eislingen/Fils Eislingen (F))
	WELTESTRASSE_2 = 'de:08117:1011:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__WELTESTR = 'de:08117:1011'
	EISLINGEN__F__WELTESTR_1 = 'de:08117:1011:0:H1'  # Rtg Eislingen/Salach (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__WELTESTR_2 = 'de:08117:1011:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__SILCHERSCHULE = 'de:08117:1012'
	EISLINGEN__F__SILCHERSCHULE_1 = 'de:08117:1012:0:H1'  # Rtg Eislingen/Salach (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__SILCHERSCHULE_2 = 'de:08117:1012:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	SILCHERSCHULE = 'de:08117:1012'
	SILCHERSCHULE_1 = 'de:08117:1012:0:H1'  # Rtg Eislingen/Salach (Eislingen/Fils Eislingen (F))
	SILCHERSCHULE_2 = 'de:08117:1012:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__HALLENBAD = 'de:08117:1013'
	EISLINGEN__F__HALLENBAD_1 = 'de:08117:1013:0:H1'  # Rtg Eislingen Bhf (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__HALLENBAD_2 = 'de:08117:1013:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__WASENHOF = 'de:08117:1015'
	EISLINGEN__F__WASENHOF_1 = 'de:08117:1015:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__WASENHOF_2 = 'de:08117:1015:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	WASENHOF = 'de:08117:1015'
	WASENHOF_1 = 'de:08117:1015:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	WASENHOF_2 = 'de:08117:1015:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	ZIEGELBACHSTRASSE = 'de:08117:1016'
	ZIEGELBACHSTRASSE_1 = 'de:08117:1016:0:H1'  # Rtg Eislingen Vogelgarten (Eislingen/Fils Eislingen (F))
	ZIEGELBACHSTRASSE_2 = 'de:08117:1016:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__ZIEGELBACHSTR = 'de:08117:1016'
	EISLINGEN__F__ZIEGELBACHSTR_1 = 'de:08117:1016:0:H1'  # Rtg Eislingen Vogelgarten (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__ZIEGELBACHSTR_2 = 'de:08117:1016:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISL__F__ZIEGELBACHKLINGE = 'de:08117:1018'
	EISL__F__ZIEGELBACHKLINGE_1 = 'de:08117:1018:0:H1'  # Rtg Eislingen Vogelgarten (Eislingen/Fils Eislingen (F))
	EISL__F__ZIEGELBACHKLINGE_2 = 'de:08117:1018:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	ZIEGELBACHKLINGE = 'de:08117:1018'
	ZIEGELBACHKLINGE_1 = 'de:08117:1018:0:H1'  # Rtg Eislingen Vogelgarten (Eislingen/Fils Eislingen (F))
	ZIEGELBACHKLINGE_2 = 'de:08117:1018:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__ZEISIGWEG = 'de:08117:1019:0:H1'  # Ankunft (Eislingen/Fils Eislingen (F))
	EISL__F__TALSTRASSE = 'de:08117:1020'
	EISL__F__TALSTRASSE_1 = 'de:08117:1020:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	EISL__F__TALSTRASSE_2 = 'de:08117:1020:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__DANZIGER_STR = 'de:08117:1021'
	EISLINGEN__F__DANZIGER_STR_1 = 'de:08117:1021:0:H1'  # Rtg Vogelgarten (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__DANZIGER_STR_2 = 'de:08117:1021:0:R1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	SÄNGERSTRASSE = 'de:08117:1024'
	SÄNGERSTRASSE_1 = 'de:08117:1024:0:H1'  # Rtg Vogelgarten (Eislingen/Fils Eislingen (F))
	SÄNGERSTRASSE_2 = 'de:08117:1024:0:R1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__SÄNGERSTR = 'de:08117:1024'
	EISLINGEN__F__SÄNGERSTR_1 = 'de:08117:1024:0:H1'  # Rtg Vogelgarten (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__SÄNGERSTR_2 = 'de:08117:1024:0:R1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	OESCH_SCHULEN = 'de:08117:1025'
	OESCH_SCHULEN_1 = 'de:08117:1025:0:H1'  # Rtg Ottenbach (Eislingen/Fils Eislingen (F))
	OESCH_SCHULEN_2 = 'de:08117:1025:0:R1'  # Rtg Eislingen Vogelgarten (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__OESCH_SCHULEN = 'de:08117:1025'
	EISLINGEN__F__OESCH_SCHULEN_1 = 'de:08117:1025:0:H1'  # Rtg Ottenbach (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__OESCH_SCHULEN_2 = 'de:08117:1025:0:R1'  # Rtg Eislingen Vogelgarten (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__FRIEDHOF_NORD = 'de:08117:1028'
	EISLINGEN__F__FRIEDHOF_NORD_1 = 'de:08117:1028:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__FRIEDHOF_NORD_2 = 'de:08117:1028:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	FRIEDHOF_NORD_1 = 'de:08117:1028:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	FRIEDHOF_NORD_2 = 'de:08117:1028:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	FRIEDHOF_NORD_3 = 'de:08117:9012:0:H1'  # Rtg Hohenstaufen (Göppingen Göppingen)
	FRIEDHOF_NORD_4 = 'de:08117:9012:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	EISLINGEN__F__STADTMITTE = 'de:08117:1029'
	EISLINGEN__F__STADTMITTE_1 = 'de:08117:1029:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__STADTMITTE_2 = 'de:08117:1029:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__STADTMITTE_3 = 'de:08117:1029:2:3'  # Rtg Talstraße (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__STADTMITTE_4 = 'de:08117:1029:2:4'  # Rtg Hbf Waldhorn (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__WALDHORN = 'de:08117:1031'
	EISLINGEN__F__WALDHORN_1 = 'de:08117:1031:0:H1'  # Rtg Eislingen/Salach (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__WALDHORN_2 = 'de:08117:1031:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISL__F__IN_DEN_WEINGÄRTEN = 'de:08117:1032'
	EISL__F__IN_DEN_WEINGÄRTEN_1 = 'de:08117:1032:0:H1'  # Rtg Eislingen (Eislingen/Fils Eislingen (F))
	EISL__F__IN_DEN_WEINGÄRTEN_2 = 'de:08117:1032:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	IN_DEN_WEINGÄRTEN = 'de:08117:1032'
	IN_DEN_WEINGÄRTEN_1 = 'de:08117:1032:0:H1'  # Rtg Eislingen (Eislingen/Fils Eislingen (F))
	IN_DEN_WEINGÄRTEN_2 = 'de:08117:1032:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	GÖPPINGEN_ROSSBACHSTR = 'de:08117:1033'
	GÖPPINGEN_ROSSBACHSTR_1 = 'de:08117:1033:0:H1'  # Rtg Eislingen (Göppingen Göppingen)
	GÖPPINGEN_ROSSBACHSTR_2 = 'de:08117:1033:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_STADTWERKE = 'de:08117:1034'
	GÖPPINGEN_STADTWERKE_1 = 'de:08117:1034:0:H1'  # Rtg Eislingen (Göppingen Göppingen)
	GÖPPINGEN_STADTWERKE_2 = 'de:08117:1034:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_BAUHOF = 'de:08117:1035:0:H1'  # Rtg Stadtwerke (Göppingen Göppingen)
	BETZSTRASSE = 'de:08117:1036:0:R1'  # Rtg Marktstr. (Göppingen Göppingen)
	GÖPPINGEN_BETZSTR = 'de:08117:1036:0:R1'  # Rtg Marktstr. (Göppingen Göppingen)
	GÖPPINGEN_MARKTSTR = 'de:08117:1037:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	HAUPTSTR__BAHNHOF = 'de:08117:1014'
	HAUPTSTR__BAHNHOF_1 = 'de:08117:1014:0:H1'  # Rtg Eislingen Nord/Salach (Eislingen/Fils Eislingen (F))
	HAUPTSTR__BAHNHOF_2 = 'de:08117:1014:0:R1'  # Rtg Eislingen Süd/Göppingen (Eislingen/Fils Eislingen (F))
	EISL__F__HAUPTSTR__BAHNHOF = 'de:08117:1014'
	EISL__F__HAUPTSTR__BAHNHOF_1 = 'de:08117:1014:0:H1'  # Rtg Eislingen Nord/Salach (Eislingen/Fils Eislingen (F))
	EISL__F__HAUPTSTR__BAHNHOF_2 = 'de:08117:1014:0:R1'  # Rtg Eislingen Süd/Göppingen (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__MOZARTSTR = 'de:08117:1040'
	EISLINGEN__F__MOZARTSTR_1 = 'de:08117:1040:0:H1'  # Rtg Fahrzeugwerke (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__MOZARTSTR_2 = 'de:08117:1040:0:R1'  # Rtg Beundstr. (Eislingen/Fils Eislingen (F))
	EISL__F__SOLITUDESTR = 'de:08117:1043:0:R1'  # Rtg Immelmannstr. (Eislingen/Fils Eislingen (F))
	SOLITUDESTRASSE = 'de:08117:1043:0:R1'  # Rtg Immelmannstr. (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__FRIEDHOF_SÜD = 'de:08117:1044'
	EISLINGEN__F__FRIEDHOF_SÜD_1 = 'de:08117:1044:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__FRIEDHOF_SÜD_2 = 'de:08117:1044:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	FRIEDHOF_SÜD_1 = 'de:08117:1044:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	FRIEDHOF_SÜD_2 = 'de:08117:1044:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	FRIEDHOF_SÜD_3 = 'de:08117:9301:0:H1'  # Rtg Galgenberg (Göppingen Göppingen)
	FRIEDHOF_SÜD_4 = 'de:08117:9301:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	EISLINGEN__F__BEUNDSTR = 'de:08117:1045'
	EISLINGEN__F__BEUNDSTR_1 = 'de:08117:1045:0:H1'  # Rtg Eislingen Nord/Salach (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__BEUNDSTR_2 = 'de:08117:1045:0:R1'  # Rtg Eislingen Süd/Göppingen (Eislingen/Fils Eislingen (F))
	BEUNDSTRASSE = 'de:08117:1045'
	BEUNDSTRASSE_1 = 'de:08117:1045:0:H1'  # Rtg Eislingen Nord/Salach (Eislingen/Fils Eislingen (F))
	BEUNDSTRASSE_2 = 'de:08117:1045:0:R1'  # Rtg Eislingen Süd/Göppingen (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__HOLZHEIMER_STR = 'de:08117:1047'
	EISLINGEN__F__HOLZHEIMER_STR_1 = 'de:08117:1047:0:H1'  # Rtg Weltestr. (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__HOLZHEIMER_STR_2 = 'de:08117:1047:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	HOLZHEIMER_STRASSE = 'de:08117:1047'
	HOLZHEIMER_STRASSE_1 = 'de:08117:1047:0:H1'  # Rtg Weltestr. (Eislingen/Fils Eislingen (F))
	HOLZHEIMER_STRASSE_2 = 'de:08117:1047:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	SCHLOSSPLATZ_BAHNHOF = 'de:08117:1049'
	SCHLOSSPLATZ_BAHNHOF_1 = 'de:08117:1049:0:H1'  # Rtg Eislingen (Eislingen/Fils Eislingen (F))
	SCHLOSSPLATZ_BAHNHOF_2 = 'de:08117:1049:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISL__F__SCHLOSSPLATZ_BAHNHOF = 'de:08117:1049'
	EISL__F__SCHLOSSPLATZ_BAHNHOF_1 = 'de:08117:1049:0:H1'  # Rtg Eislingen (Eislingen/Fils Eislingen (F))
	EISL__F__SCHLOSSPLATZ_BAHNHOF_2 = 'de:08117:1049:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	BARBAROSSASEE = 'de:08117:1050'
	BARBAROSSASEE_1 = 'de:08117:1050:0:H1'  # Rtg Stauferpark (Göppingen Göppingen)
	BARBAROSSASEE_2 = 'de:08117:1050:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_BARBAROSSASEE = 'de:08117:1050'
	GÖPPINGEN_BARBAROSSASEE_1 = 'de:08117:1050:0:H1'  # Rtg Stauferpark (Göppingen Göppingen)
	GÖPPINGEN_BARBAROSSASEE_2 = 'de:08117:1050:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_HAILINGSTRASSE = 'de:08117:1051'
	GÖPPINGEN_HAILINGSTRASSE_1 = 'de:08117:1051:0:H1'  # Rtg Stauferpark (Göppingen Göppingen)
	GÖPPINGEN_HAILINGSTRASSE_2 = 'de:08117:1051:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	HAILINGSTRASSE = 'de:08117:1051'
	HAILINGSTRASSE_1 = 'de:08117:1051:0:H1'  # Rtg Stauferpark (Göppingen Göppingen)
	HAILINGSTRASSE_2 = 'de:08117:1051:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_MÖRIKESTR = 'de:08117:1053:0:R1'  # Rtg Barbarossasee (Göppingen Göppingen)
	WILHELMSHILFE_1 = 'de:08117:1056:0:H1'  # Rtg Dürestr. (Göppingen Göppingen)
	WILHELMSHILFE_2 = 'de:08117:4014:0:H1'  # Rtg Rechberghausen (Göppingen Bartenbach (Göppingen))
	WILHELMSHILFE_3 = 'de:08117:4014:0:R1'  # Rtg Hst Lorch./Lerchen. (Göppingen Bartenbach (Göppingen))
	WILHELMSHILFE_4 = 'de:08117:4014:0:R2'  # Rtg Hst. Engenlauch/Schule (Göppingen Bartenbach (Göppingen))
	GÖPPINGEN_WILHELMSHILFE = 'de:08117:1056:0:H1'  # Rtg Dürestr. (Göppingen Göppingen)
	GÖPPINGEN_SCHÜTZENSTR = 'de:08117:1057'
	GÖPPINGEN_SCHÜTZENSTR_1 = 'de:08117:1057:0:H1'  # Linie 13/91/7688 (Göppingen Göppingen)
	GÖPPINGEN_SCHÜTZENSTR_2 = 'de:08117:1057:0:H2'  # Linie 1/3/6/16 (Göppingen Göppingen)
	GÖPPINGEN_SCHÜTZENSTR_3 = 'de:08117:1057:0:H3'  # Linie 4/11/94/31/32/33 (Göppingen Göppingen)
	GÖPPINGEN_KORNHAUSPLATZ = 'de:08117:1058:0:H1'  # Rtg Marktplatz (Göppingen Göppingen)
	KORNHAUSPLATZ = 'de:08117:1058:0:H1'  # Rtg Marktplatz (Göppingen Göppingen)
	STORCHEN = 'de:08117:1059:0:H1'  # Rtg Schillerplatz (Göppingen Göppingen)
	GÖPPINGEN_STORCHEN = 'de:08117:1059:0:H1'  # Rtg Schillerplatz (Göppingen Göppingen)
	GÖPPINGEN_VORDERE_KARLSTR = 'de:08117:1060:0:H1'  # Rtg Ulmer-/Heininger Str. (Göppingen Göppingen)
	VORDERE_KARLSTRASSE = 'de:08117:1060:0:H1'  # Rtg Ulmer-/Heininger Str. (Göppingen Göppingen)
	GÖPPINGEN_WERTSTOFFHOF = 'de:08117:1061:0:H1'  # Rtg Roßbachstr. (Göppingen Göppingen)
	WERTSTOFFHOF = 'de:08117:1061:0:H1'  # Rtg Roßbachstr. (Göppingen Göppingen)
	GÖPPINGEN_LENBACHSTR = 'de:08117:1062'
	GÖPPINGEN_LENBACHSTR_1 = 'de:08117:1062:0:H1'  # Rtg Galgenberg (Göppingen Göppingen)
	GÖPPINGEN_LENBACHSTR_2 = 'de:08117:1062:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	LENBACHSTRASSE = 'de:08117:1062'
	LENBACHSTRASSE_1 = 'de:08117:1062:0:H1'  # Rtg Galgenberg (Göppingen Göppingen)
	LENBACHSTRASSE_2 = 'de:08117:1062:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_GARTENFREUNDE = 'de:08117:1064:0:H1'  # Ankunft (Göppingen Göppingen)
	GARTENFREUNDE = 'de:08117:1064:0:H1'  # Ankunft (Göppingen Göppingen)
	HEUBACHSTRASSE = 'de:08117:1065:0:R1'  # Richtung ZOB (Göppingen Göppingen)
	GÖPPINGEN_HEUBACHSTR = 'de:08117:1065:0:R1'  # Richtung ZOB (Göppingen Göppingen)
	SCHURWALDWEG = 'de:08117:1100:0:H1'  # Rtg Geißberg (Birenbach Birenbach)
	BIRENBACH_SCHURWALDWEG = 'de:08117:1100:0:H1'  # Rtg Geißberg (Birenbach Birenbach)
	BIRENBACH_GEISSBERG = 'de:08117:1101:0:H1'  # Rtg Rathaus (Birenbach Birenbach)
	GEISSBERG = 'de:08117:1101:0:H1'  # Rtg Rathaus (Birenbach Birenbach)
	BIRENBACH_RATHAUS_1 = 'de:08117:1102:0:H1'  # Rtg Bahnhofstr/Krettenhof (Birenbach Birenbach)
	BIRENBACH_RATHAUS_2 = 'de:08117:11102:0:2'  # Rtg Brückenstr (Birenbach Birenbach)
	BIRENBACH_BAHNHOFSTR = 'de:08117:1103'
	BIRENBACH_BAHNHOFSTR_1 = 'de:08117:1103:0:H1'  # Rtg Wäschenbeuren (Birenbach Birenbach)
	BIRENBACH_BAHNHOFSTR_2 = 'de:08117:1103:0:R1'  # Rtg Rechberghausen (Birenbach Birenbach)
	KRETTENHOF = 'de:08117:1104'
	KRETTENHOF_1 = 'de:08117:1104:0:H1'  # Rtg Wäschenbeuren (Göppingen Göppingen)
	KRETTENHOF_2 = 'de:08117:1104:0:R1'  # Rtg Rechberghausen (Göppingen Göppingen)
	GÖPPINGEN_KRETTENHOF = 'de:08117:1104'
	GÖPPINGEN_KRETTENHOF_1 = 'de:08117:1104:0:H1'  # Rtg Wäschenbeuren (Göppingen Göppingen)
	GÖPPINGEN_KRETTENHOF_2 = 'de:08117:1104:0:R1'  # Rtg Rechberghausen (Göppingen Göppingen)
	WÄSCHENBEUREN_SCHÜTZENHOF = 'de:08117:1105'
	WÄSCHENBEUREN_SCHÜTZENHOF_1 = 'de:08117:1105:0:H1'  # Rtg Schw. Gmünd (Wäschenbeuren Wäschenbeuren)
	WÄSCHENBEUREN_SCHÜTZENHOF_2 = 'de:08117:1105:0:R1'  # Rtg Göppingen (Wäschenbeuren Wäschenbeuren)
	SCHÜTZENHOF = 'de:08117:1105'
	SCHÜTZENHOF_1 = 'de:08117:1105:0:H1'  # Rtg Schw. Gmünd (Wäschenbeuren Wäschenbeuren)
	SCHÜTZENHOF_2 = 'de:08117:1105:0:R1'  # Rtg Göppingen (Wäschenbeuren Wäschenbeuren)
	WÄSCHENBEUREN_MARKTPLATZ = 'de:08117:1106'
	WÄSCHENBEUREN_MARKTPLATZ_1 = 'de:08117:1106:0:H1'  # Rtg Schw. Gmünd (Wäschenbeuren Wäschenbeuren)
	WÄSCHENBEUREN_MARKTPLATZ_2 = 'de:08117:1106:0:R1'  # Rtg Göppingen (Wäschenbeuren Wäschenbeuren)
	WÄSCHENBEUREN_BAHNHOFSTR = 'de:08117:1107:0:H1'  # Rtg Schillerstr. (Wäschenbeuren Wäschenbeuren)
	WÄSCHENBEUREN_SCHILLERSTR = 'de:08117:1108:0:H1'  # Rtg Friedhof (Wäschenbeuren Wäschenbeuren)
	WÄSCHENBEUREN_FRIEDHOF = 'de:08117:1109'
	WÄSCHENBEUREN_FRIEDHOF_1 = 'de:08117:1109:0:H1'  # Rtg Schw. Gmünd (Wäschenbeuren Wäschenbeuren)
	WÄSCHENBEUREN_FRIEDHOF_2 = 'de:08117:1109:0:R1'  # Rtg Göppingen (Wäschenbeuren Wäschenbeuren)
	WANGEN_SIEMENSSTR = 'de:08117:1136'
	WANGEN_SIEMENSSTR_1 = 'de:08117:1136:0:H1'  # Rtg Hanglich (Wangen Wangen (GP))
	WANGEN_SIEMENSSTR_2 = 'de:08117:1136:0:R1'  # Rtg Faurndau (Wangen Wangen (GP))
	RECHBERGHAUSEN_HAUFFSTR = 'de:08117:1201:0:R1'  # Rtg Reichenhardtstraße (Rechberghausen Rechberghausen)
	FAURNDAU_KESSELTOBELSTR = 'de:08117:1203'
	FAURNDAU_KESSELTOBELSTR_1 = 'de:08117:1203:0:H1'  # Rtg Rechberghausen (Göppingen Faurndau)
	FAURNDAU_KESSELTOBELSTR_2 = 'de:08117:1203:0:R1'  # Rtg Salamanderstr. (Göppingen Faurndau)
	KESSELTOBELSTRASSE = 'de:08117:1203'
	KESSELTOBELSTRASSE_1 = 'de:08117:1203:0:H1'  # Rtg Rechberghausen (Göppingen Faurndau)
	KESSELTOBELSTRASSE_2 = 'de:08117:1203:0:R1'  # Rtg Salamanderstr. (Göppingen Faurndau)
	RECHBERGHAUSEN_MAYBACHSTR = 'de:08117:1204:0:H1'  # Rtg Grundschule (Rechberghausen Rechberghausen)
	WÄSCHENBEUREN_WÄSCHERHOFSTR = 'de:08117:1205'
	WÄSCHENBEUREN_WÄSCHERHOFSTR_1 = 'de:08117:1205:0:H1'  # Rtg Maitis (Wäschenbeuren Wäschenbeuren)
	WÄSCHENBEUREN_WÄSCHERHOFSTR_2 = 'de:08117:1205:0:R1'  # Rtg Göppingen (Wäschenbeuren Wäschenbeuren)
	WÄSCHERHOFSTRASSE = 'de:08117:1205'
	WÄSCHERHOFSTRASSE_1 = 'de:08117:1205:0:H1'  # Rtg Maitis (Wäschenbeuren Wäschenbeuren)
	WÄSCHERHOFSTRASSE_2 = 'de:08117:1205:0:R1'  # Rtg Göppingen (Wäschenbeuren Wäschenbeuren)
	MAITIS_ORTSMITTE = 'de:08117:1206:0:H1'  # Rtg Limesweg (Göppingen Maitis)
	LENGLINGEN_ZIEGERHOF = 'de:08117:1207'
	LENGLINGEN_ZIEGERHOF_1 = 'de:08117:1207:0:H1'  # Rtg Reitprechts (Göppingen Lenglingen)
	LENGLINGEN_ZIEGERHOF_2 = 'de:08117:1207:0:R1'  # Rtg Maitis (Göppingen Lenglingen)
	ZIEGERHOF = 'de:08117:1207'
	ZIEGERHOF_1 = 'de:08117:1207:0:H1'  # Rtg Reitprechts (Göppingen Lenglingen)
	ZIEGERHOF_2 = 'de:08117:1207:0:R1'  # Rtg Maitis (Göppingen Lenglingen)
	LENGLINGEN_ORTSMITTE = 'de:08117:1208'
	LENGLINGEN_ORTSMITTE_1 = 'de:08117:1208:0:H1'  # Rtg Reitprechts (Göppingen Lenglingen)
	LENGLINGEN_ORTSMITTE_2 = 'de:08117:1208:0:R1'  # Rtg Maitis (Göppingen Lenglingen)
	HIRSCHHOF = 'de:08117:11209:0:1'  # Rtg Lenglingen (Göppingen Lenglingen)
	LENGLINGEN_HIRSCHHOF = 'de:08117:11209:0:1'  # Rtg Lenglingen (Göppingen Lenglingen)
	MAITIS_STUIFENSTR = 'de:08117:1225:0:H1'  # Rtg Lenglingen (Göppingen Maitis)
	MAITIS_LIMESWEG = 'de:08117:1216:0:H1'  # Rtg Wäschenbeuren (Göppingen Maitis)
	SALAMANDERSTRASSE = 'de:08117:1220'
	SALAMANDERSTRASSE_1 = 'de:08117:1220:0:H1'  # Rtg Iltisweg (Göppingen Faurndau)
	SALAMANDERSTRASSE_2 = 'de:08117:1220:0:R1'  # Rtg Lehlstr. (Göppingen Faurndau)
	FAURNDAU_SALAMANDERSTR = 'de:08117:1220'
	FAURNDAU_SALAMANDERSTR_1 = 'de:08117:1220:0:H1'  # Rtg Iltisweg (Göppingen Faurndau)
	FAURNDAU_SALAMANDERSTR_2 = 'de:08117:1220:0:R1'  # Rtg Lehlstr. (Göppingen Faurndau)
	MAITIS_STAUFERWEG = 'de:08117:1226'
	MAITIS_STAUFERWEG_1 = 'de:08117:1226:0:H1'  # Rtg Lenglingen (Göppingen Maitis)
	MAITIS_STAUFERWEG_2 = 'de:08117:1226:0:R1'  # Rtg. Wäschenbeuren (Göppingen Maitis)
	STAUFERWEG = 'de:08117:1226'
	STAUFERWEG_1 = 'de:08117:1226:0:H1'  # Rtg Lenglingen (Göppingen Maitis)
	STAUFERWEG_2 = 'de:08117:1226:0:R1'  # Rtg. Wäschenbeuren (Göppingen Maitis)
	GÖPP__WALDCAFE_WANNENHOF = 'de:08117:1302'
	GÖPP__WALDCAFE_WANNENHOF_1 = 'de:08117:1302:0:H1'  # Rtg Hohenstaufen (Göppingen Göppingen)
	GÖPP__WALDCAFE_WANNENHOF_2 = 'de:08117:1302:0:R1'  # Rtg Göppingen (Göppingen Göppingen)
	WALDCAFE_WANNENHOF = 'de:08117:1302'
	WALDCAFE_WANNENHOF_1 = 'de:08117:1302:0:H1'  # Rtg Hohenstaufen (Göppingen Göppingen)
	WALDCAFE_WANNENHOF_2 = 'de:08117:1302:0:R1'  # Rtg Göppingen (Göppingen Göppingen)
	GÖPPINGEN_ABZW__HOHREIN = 'de:08117:1303'
	GÖPPINGEN_ABZW__HOHREIN_1 = 'de:08117:1303:0:H1'  # Rtg Hohenstaufen (Göppingen Göppingen)
	GÖPPINGEN_ABZW__HOHREIN_2 = 'de:08117:1303:0:R1'  # Rtg Waldcafe Wannenhof (Göppingen Göppingen)
	ABZWEIG_HOHREIN = 'de:08117:1303'
	ABZWEIG_HOHREIN_1 = 'de:08117:1303:0:H1'  # Rtg Hohenstaufen (Göppingen Göppingen)
	ABZWEIG_HOHREIN_2 = 'de:08117:1303:0:R1'  # Rtg Waldcafe Wannenhof (Göppingen Göppingen)
	RATHAUS_DORFPLATZ = 'de:08117:1304'
	RATHAUS_DORFPLATZ_1 = 'de:08117:1304:0:H1'  # Rtg Reichsdorfstr. (Göppingen Hohenstaufen)
	RATHAUS_DORFPLATZ_2 = 'de:08117:1304:0:R1'  # Rtg Göppingen (Göppingen Hohenstaufen)
	HOHENST__RATHAUS_DORFPLATZ = 'de:08117:1304'
	HOHENST__RATHAUS_DORFPLATZ_1 = 'de:08117:1304:0:H1'  # Rtg Reichsdorfstr. (Göppingen Hohenstaufen)
	HOHENST__RATHAUS_DORFPLATZ_2 = 'de:08117:1304:0:R1'  # Rtg Göppingen (Göppingen Hohenstaufen)
	HOHENST__JUGENDHERBERGE = 'de:08117:1305:0:H1'  # Rtg Maitis (Göppingen Hohenstaufen)
	REICHSDORFSTRASSE = 'de:08117:1306'
	REICHSDORFSTRASSE_1 = 'de:08117:1306:0:H1'  # Rtg Jugendherberge (Göppingen Hohenstaufen)
	REICHSDORFSTRASSE_2 = 'de:08117:1306:0:R1'  # Rtg Rathaus/Dorfplatz (Göppingen Hohenstaufen)
	HOHENSTAUFEN_REICHSDORFSTR = 'de:08117:1306'
	HOHENSTAUFEN_REICHSDORFSTR_1 = 'de:08117:1306:0:H1'  # Rtg Jugendherberge (Göppingen Hohenstaufen)
	HOHENSTAUFEN_REICHSDORFSTR_2 = 'de:08117:1306:0:R1'  # Rtg Rathaus/Dorfplatz (Göppingen Hohenstaufen)
	STERNKREUZUNG = 'de:08117:2001'
	STERNKREUZUNG_1 = 'de:08117:2001:0:H1'  # eRtg Märklin (Göppingen Göppingen)
	STERNKREUZUNG_2 = 'de:08117:2001:0:R1'  # eRtg ZOB (Göppingen Göppingen)
	GÖPPINGEN_STERNKREUZUNG = 'de:08117:2001'
	GÖPPINGEN_STERNKREUZUNG_1 = 'de:08117:2001:0:H1'  # eRtg Märklin (Göppingen Göppingen)
	GÖPPINGEN_STERNKREUZUNG_2 = 'de:08117:2001:0:R1'  # eRtg ZOB (Göppingen Göppingen)
	MÄRKLINEUM = 'de:08117:2002'
	MÄRKLINEUM_1 = 'de:08117:2002:0:H1'  # Rtg Boeringer / Lehlestr. (Göppingen Göppingen)
	MÄRKLINEUM_2 = 'de:08117:2002:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_MÄRKLINEUM = 'de:08117:2002'
	GÖPPINGEN_MÄRKLINEUM_1 = 'de:08117:2002:0:H1'  # Rtg Boeringer / Lehlestr. (Göppingen Göppingen)
	GÖPPINGEN_MÄRKLINEUM_2 = 'de:08117:2002:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_BOEHRINGER_AREAL = 'de:08117:2003'
	GÖPPINGEN_BOEHRINGER_AREAL_1 = 'de:08117:2003:0:H1'  # Rtg Faurndau (Göppingen Göppingen)
	GÖPPINGEN_BOEHRINGER_AREAL_2 = 'de:08117:2003:0:H2'  # Rtg BZ Öde (Göppingen Göppingen)
	BOEHRINGER_AREAL = 'de:08117:2003'
	BOEHRINGER_AREAL_1 = 'de:08117:2003:0:H1'  # Rtg Faurndau (Göppingen Göppingen)
	BOEHRINGER_AREAL_2 = 'de:08117:2003:0:H2'  # Rtg BZ Öde (Göppingen Göppingen)
	LEHLESTRASSE = 'de:08117:2004'
	LEHLESTRASSE_1 = 'de:08117:2004:0:H1'  # Rtg Elektra (Göppingen Faurndau)
	LEHLESTRASSE_2 = 'de:08117:2004:0:R1'  # Rtg Märklin (Göppingen Faurndau)
	FAURNDAU_LEHLESTR = 'de:08117:2004'
	FAURNDAU_LEHLESTR_1 = 'de:08117:2004:0:H1'  # Rtg Elektra (Göppingen Faurndau)
	FAURNDAU_LEHLESTR_2 = 'de:08117:2004:0:R1'  # Rtg Märklin (Göppingen Faurndau)
	GOETHESTR = 'de:08117:2005'
	GOETHESTR_1 = 'de:08117:2005:0:H1'  # Rtg Bahnhof (Göppingen Faurndau)
	GOETHESTR_2 = 'de:08117:2005:0:R1'  # Rtg Lehlestr. (Göppingen Faurndau)
	FAURNDAU_GOETHESTR = 'de:08117:2005'
	FAURNDAU_GOETHESTR_1 = 'de:08117:2005:0:H1'  # Rtg Bahnhof (Göppingen Faurndau)
	FAURNDAU_GOETHESTR_2 = 'de:08117:2005:0:R1'  # Rtg Lehlestr. (Göppingen Faurndau)
	MERCEDESSTR = 'de:08117:2007'
	MERCEDESSTR_1 = 'de:08117:2007:0:H1'  # Rtg Uhingen (Uhingen Uhingen)
	MERCEDESSTR_2 = 'de:08117:2007:0:R1'  # Rtg Faurndau (Uhingen Uhingen)
	UHINGEN_MERCEDESSTR = 'de:08117:2007'
	UHINGEN_MERCEDESSTR_1 = 'de:08117:2007:0:H1'  # Rtg Uhingen (Uhingen Uhingen)
	UHINGEN_MERCEDESSTR_2 = 'de:08117:2007:0:R1'  # Rtg Faurndau (Uhingen Uhingen)
	UHINGEN_FA__ALLGAIER = 'de:08117:2008'
	UHINGEN_FA__ALLGAIER_1 = 'de:08117:2008:0:H1'  # Rtg Uhingen (Uhingen Uhingen)
	UHINGEN_FA__ALLGAIER_2 = 'de:08117:2008:0:R1'  # Rtg Faurndau (Uhingen Uhingen)
	FA__ALLGAIER = 'de:08117:2008'
	FA__ALLGAIER_1 = 'de:08117:2008:0:H1'  # Rtg Uhingen (Uhingen Uhingen)
	FA__ALLGAIER_2 = 'de:08117:2008:0:R1'  # Rtg Faurndau (Uhingen Uhingen)
	HIEBERSCHULE = 'de:08117:2009:0:H1'  # Hieberschule (Uhingen Uhingen)
	UHINGEN_HIEBERSCHULE = 'de:08117:2009:0:H1'  # Hieberschule (Uhingen Uhingen)
	UHINGEN_SCHULSTR = 'de:08117:2039'
	UHINGEN_SCHULSTR_1 = 'de:08117:2039:0:H1'  # Rtg Volksbank (Uhingen Uhingen)
	UHINGEN_SCHULSTR_2 = 'de:08117:2039:0:R1'  # Rtg Allgaier (Uhingen Uhingen)
	UHINGEN_UDITORIUM = 'de:08117:2011'
	UHINGEN_UDITORIUM_1 = 'de:08117:2011:0:H1'  # eRtg Bleicherei / Albershausen (Uhingen Uhingen)
	UHINGEN_UDITORIUM_2 = 'de:08117:2011:0:R1'  # Rtg Faurndau (Uhingen Uhingen)
	UHINGEN_UDITORIUM_3 = 'de:08117:2011:2:3'  # Rtg Göppingen (Uhingen Uhingen)
	UDITORIUM = 'de:08117:2011'
	UDITORIUM_1 = 'de:08117:2011:0:H1'  # eRtg Bleicherei / Albershausen (Uhingen Uhingen)
	UDITORIUM_2 = 'de:08117:2011:0:R1'  # Rtg Faurndau (Uhingen Uhingen)
	UDITORIUM_3 = 'de:08117:2011:2:3'  # Rtg Göppingen (Uhingen Uhingen)
	ZIEGELBERGWEG = 'de:08117:2012'
	ZIEGELBERGWEG_1 = 'de:08117:2012:0:H1'  # eRtg Uhinger Straße (Uhingen Uhingen)
	ZIEGELBERGWEG_2 = 'de:08117:2012:0:R1'  # eRtg Volksbank (Uhingen Uhingen)
	UHINGEN_ZIEGELBERGWEG = 'de:08117:2012'
	UHINGEN_ZIEGELBERGWEG_1 = 'de:08117:2012:0:H1'  # eRtg Uhinger Straße (Uhingen Uhingen)
	UHINGEN_ZIEGELBERGWEG_2 = 'de:08117:2012:0:R1'  # eRtg Volksbank (Uhingen Uhingen)
	UHINGEN_MARIENSTR = 'de:08117:2042:0:R1'  # Rtg Uditorium (Uhingen Uhingen)
	ALBERSHAUSEN_STAUFENSTR = 'de:08117:2015'
	ALBERSHAUSEN_STAUFENSTR_1 = 'de:08117:2015:0:H1'  # eRtg Uhinger-/Waldeckstr. (Albershausen Albershausen)
	ALBERSHAUSEN_STAUFENSTR_2 = 'de:08117:2015:0:R1'  # eRtg B297/Kirchheimer Str. (Albershausen Albershausen)
	WALDECKSTRASSE_1 = 'de:08117:2016:0:H1'  # eRtg Ortsmitte / Tannenhöfe (Albershausen Albershausen)
	WALDECKSTRASSE_2 = 'de:08117:2016:0:R1'  # eRtg Uhinger Str. (Albershausen Albershausen)
	WALDECKSTRASSE_3 = 'de:08117:9203:0:H1'  # Waldeckstraße (Göppingen Göppingen)
	ALBERSHAUSEN_WALDECKSTR = 'de:08117:2016'
	ALBERSHAUSEN_WALDECKSTR_1 = 'de:08117:2016:0:H1'  # eRtg Ortsmitte / Tannenhöfe (Albershausen Albershausen)
	ALBERSHAUSEN_WALDECKSTR_2 = 'de:08117:2016:0:R1'  # eRtg Uhinger Str. (Albershausen Albershausen)
	ALBERSHAUSEN_AM_BRÜNNELE = 'de:08117:2018:0:H1'  # Rtg Finkenweg (Albershausen Albershausen)
	AM_BRÜNNELE = 'de:08117:2018:0:H1'  # Rtg Finkenweg (Albershausen Albershausen)
	ALBERSHAUSEN_FINKENWEG = 'de:08117:2019'
	ALBERSHAUSEN_FINKENWEG_1 = 'de:08117:2019:0:H1'  # Rtg Im Morgen (Albershausen Albershausen)
	ALBERSHAUSEN_FINKENWEG_2 = 'de:08117:2019:0:R1'  # Rtg Frühlingstr. (Albershausen Albershausen)
	IM_MORGEN = 'de:08117:2020'
	IM_MORGEN_1 = 'de:08117:2020:0:H1'  # Rtg Gemeindeländerweg (Albershausen Albershausen)
	IM_MORGEN_2 = 'de:08117:2020:0:R1'  # Rtg Finkenweg (Albershausen Albershausen)
	ALBERSHAUSEN_IM_MORGEN = 'de:08117:2020'
	ALBERSHAUSEN_IM_MORGEN_1 = 'de:08117:2020:0:H1'  # Rtg Gemeindeländerweg (Albershausen Albershausen)
	ALBERSHAUSEN_IM_MORGEN_2 = 'de:08117:2020:0:R1'  # Rtg Finkenweg (Albershausen Albershausen)
	ALBERSH__GEMEINDELÄNDERWEG = 'de:08117:2021'
	ALBERSH__GEMEINDELÄNDERWEG_1 = 'de:08117:2021:0:H1'  # Rtg Bünzwanger Str./Schule (Albershausen Albershausen)
	ALBERSH__GEMEINDELÄNDERWEG_2 = 'de:08117:2021:0:R1'  # Rtg Im Morgen (Albershausen Albershausen)
	GEMEINDELÄNDERWEG = 'de:08117:2021'
	GEMEINDELÄNDERWEG_1 = 'de:08117:2021:0:H1'  # Rtg Bünzwanger Str./Schule (Albershausen Albershausen)
	GEMEINDELÄNDERWEG_2 = 'de:08117:2021:0:R1'  # Rtg Im Morgen (Albershausen Albershausen)
	ALBERSH__INDUSTRIEGEBIET = 'de:08117:2023'
	ALBERSH__INDUSTRIEGEBIET_1 = 'de:08117:2023:0:H2'  # Linie 178/N91 (Albershausen Albershausen)
	ALBERSH__INDUSTRIEGEBIET_2 = 'de:08117:2023:0:R1'  # Linie 911,914 Rtg Albershausen (Albershausen Albershausen)
	ALBERSHAUSEN_FRÜHLINGSTR = 'de:08117:2024:0:H1'  # Rtg Lenaustr. (Albershausen Albershausen)
	FRÜHLINGSTRASSE_1 = 'de:08117:2024:0:H1'  # Rtg Lenaustr. (Albershausen Albershausen)
	FRÜHLINGSTRASSE_2 = 'de:08117:3314:0:H1'  # Rtg Gemeindehaus (Eschenbach Eschenbach)
	LENAUSTRASSE = 'de:08117:2026:0:H1'  # Rtg Ortsmitte (Albershausen Albershausen)
	ALBERSHAUSEN_LENAUSTR = 'de:08117:2026:0:H1'  # Rtg Ortsmitte (Albershausen Albershausen)
	UHINGEN_TURNHALLE = 'de:08117:2029:0:R1'  # Rtg Uditorium (Uhingen Uhingen)
	UHINGEN_FREIBAD = 'de:08117:2030'
	UHINGEN_FREIBAD_1 = 'de:08117:2030:0:1'  # Rtg Am Wiesenbach (Uhingen Uhingen)
	UHINGEN_FREIBAD_2 = 'de:08117:2030:0:H1'  # Rtg Turnhalle (Uhingen Uhingen)
	HOLZHAUSEN_VEILCHENSTR = 'de:08117:2032:0:H1'  # Rtg Festhalle (Uhingen Holzhausen (Uhingen))
	VEILCHENSTRASSE = 'de:08117:2032:0:H1'  # Rtg Festhalle (Uhingen Holzhausen (Uhingen))
	HOLZHAUSEN_FESTHALLE = 'de:08117:2033:0:H1'  # Rtg Am Wiesenbach (Uhingen Holzhausen (Uhingen))
	HOLZHAUSEN_AM_WIESENBACH = 'de:08117:2034'
	HOLZHAUSEN_AM_WIESENBACH_1 = 'de:08117:2034:0:H1'  # Rtg Wangen (Uhingen Holzhausen (Uhingen))
	HOLZHAUSEN_AM_WIESENBACH_2 = 'de:08117:2034:0:R1'  # Rtg Uhingen (Uhingen Holzhausen (Uhingen))
	AM_WIESENBACH = 'de:08117:2034'
	AM_WIESENBACH_1 = 'de:08117:2034:0:H1'  # Rtg Wangen (Uhingen Holzhausen (Uhingen))
	AM_WIESENBACH_2 = 'de:08117:2034:0:R1'  # Rtg Uhingen (Uhingen Holzhausen (Uhingen))
	UHINGEN_SEESTR = 'de:08117:2035'
	UHINGEN_SEESTR_1 = 'de:08117:2035:0:H1'  # Rtg Fabrikstr. (Uhingen Uhingen)
	UHINGEN_SEESTR_2 = 'de:08117:2035:0:R1'  # Rtg. Baiereck (Uhingen Uhingen)
	UHINGEN_FABRIKSTR = 'de:08117:2036:0:H1'  # Rtg Bleicherei (Uhingen Uhingen)
	BLEICHEREI_1 = 'de:08117:12037:0:4'  # Rtg. Uhingen Seestr. (Uhingen Uhingen)
	BLEICHEREI_2 = 'de:08117:2037:0:H1'  # Rtg Marienstr. (Uhingen Uhingen)
	UHINGEN_BLEICHEREI_1 = 'de:08117:12037:0:4'  # Rtg. Uhingen Seestr. (Uhingen Uhingen)
	UHINGEN_BLEICHEREI_2 = 'de:08117:2037:0:H1'  # Rtg Marienstr. (Uhingen Uhingen)
	GÖPPINGEN_FISCHSTRASSE = 'de:08117:2038'
	GÖPPINGEN_FISCHSTRASSE_1 = 'de:08117:2038:0:H1'  # Rtg Saust./Sternkreuzung/Christophsb. (Göppingen Göppingen)
	GÖPPINGEN_FISCHSTRASSE_2 = 'de:08117:2038:0:H2'  # Rtg Sternkreuzung (Göppingen Göppingen)
	FISCHSTRASSE = 'de:08117:2038'
	FISCHSTRASSE_1 = 'de:08117:2038:0:H1'  # Rtg Saust./Sternkreuzung/Christophsb. (Göppingen Göppingen)
	FISCHSTRASSE_2 = 'de:08117:2038:0:H2'  # Rtg Sternkreuzung (Göppingen Göppingen)
	EINFELDSTRASSE = 'de:08117:2040:0:H1'  # Rtg Veilchenstr. (Uhingen Holzhausen (Uhingen))
	HOLZHAUSEN_EINFELDSTR = 'de:08117:2040:0:H1'  # Rtg Veilchenstr. (Uhingen Holzhausen (Uhingen))
	ALBERSHAUSEN_ORTSMITTE = 'de:08117:2044'
	ALBERSHAUSEN_ORTSMITTE_1 = 'de:08117:2044:0:H1'  # eRtg Industriegebiet (Albershausen Albershausen)
	ALBERSHAUSEN_ORTSMITTE_2 = 'de:08117:2044:0:R1'  # eRtg Uhinger-/Waldeckstraße (Albershausen Albershausen)
	ALBERSHAUSEN_BÜNZWANGER_STR = 'de:08117:2045:0:H1'  # Rtg Frühlingstr. (Albershausen Albershausen)
	BÜNZWANGER_STRASSE_1 = 'de:08117:2045:0:H1'  # Rtg Frühlingstr. (Albershausen Albershausen)
	BÜNZWANGER_STRASSE_2 = 'de:08117:9825:0:H1'  # Rtg Ortsstr. (Uhingen Uhingen)
	BÜNZWANGER_STRASSE_3 = 'de:08117:9825:0:R1'  # Rtg Volksbank (Uhingen Uhingen)
	HATTENHOFEN_ZELLER_STR = 'de:08117:2050'
	HATTENHOFEN_ZELLER_STR_1 = 'de:08117:2050:0:H1'  # Rtg Graubachtal (Hattenhofen Hattenhofen)
	HATTENHOFEN_ZELLER_STR_2 = 'de:08117:2050:0:R1'  # Rtg Zell (A) (Hattenhofen Hattenhofen)
	HATTENH__BRUCKWIESENSTR = 'de:08117:2051:0:R1'  # Rtg Zell/Göppingen (Hattenhofen Hattenhofen)
	BRUCKWIESENSTRASSE = 'de:08117:2051:0:R1'  # Rtg Zell/Göppingen (Hattenhofen Hattenhofen)
	HATTENHOFEN_RATHAUS = 'de:08117:2052'
	HATTENHOFEN_RATHAUS_1 = 'de:08117:2052:0:H1'  # Rtg Hattenhofen (Hattenhofen Hattenhofen)
	HATTENHOFEN_RATHAUS_2 = 'de:08117:2052:0:R1'  # Rtg Göppingen (Hattenhofen Hattenhofen)
	HATTENHOFEN_HAUPTSTRASSE = 'de:08117:2053:0:H1'  # Rtg Friedhof (Hattenhofen Hattenhofen)
	HATTENHOFEN_FRIEDHOF_1 = 'de:08117:12054:0:2'  # Rtg Rathaus (Hattenhofen Hattenhofen)
	HATTENHOFEN_FRIEDHOF_2 = 'de:08117:2054:0:H1'  # Rtg Schlierbach (Hattenhofen Hattenhofen)
	GRAUBACHTAL = 'de:08117:2059'
	GRAUBACHTAL_1 = 'de:08117:2059:0:H1'  # Rtg Bruckw. Spielplatz (Hattenhofen Hattenhofen)
	GRAUBACHTAL_2 = 'de:08117:2059:0:R1'  # Rtg Zeller Str. (Hattenhofen Hattenhofen)
	HATTENHOFEN_GRAUBACHTAL = 'de:08117:2059'
	HATTENHOFEN_GRAUBACHTAL_1 = 'de:08117:2059:0:H1'  # Rtg Bruckw. Spielplatz (Hattenhofen Hattenhofen)
	HATTENHOFEN_GRAUBACHTAL_2 = 'de:08117:2059:0:R1'  # Rtg Zeller Str. (Hattenhofen Hattenhofen)
	AICHELBERG_BÜRGERHAUS = 'de:08117:2061:0:H1'  # Rtg Kreuzung (Aichelberg Aichelberg (GP))
	AICHELBERG_KREUZUNG = 'de:08117:2060'
	AICHELBERG_KREUZUNG_1 = 'de:08117:2060:0:H1'  # Rtg Bürgerhaus (Aichelberg Aichelberg (GP))
	AICHELBERG_KREUZUNG_2 = 'de:08117:2060:0:R1'  # Rtg Zell (Aichelberg Aichelberg (GP))
	HATTENHOFEN_BRUCKW__SPIELPLATZ = 'de:08117:2062:0:H1'  # Rtg Hattenhofen (Hattenhofen Hattenhofen)
	BRUCKWIESEN_SPIELPLATZ = 'de:08117:2062:0:H1'  # Rtg Hattenhofen (Hattenhofen Hattenhofen)
	JEBENHAUSEN_FALKENSTRASSE = 'de:08117:2098:0:H1'  # Rtg Kleingartenanlage (Göppingen Jebenhausen)
	FALKENSTRASSE = 'de:08117:2098:0:H1'  # Rtg Kleingartenanlage (Göppingen Jebenhausen)
	JEBENHAUSEN_KLEINGARTENANLAGE = 'de:08117:2099:0:H1'  # Rtg Fasanenstraße (Göppingen Jebenhausen)
	KLEINGARTENANLAGE = 'de:08117:2099:0:H1'  # Rtg Fasanenstraße (Göppingen Jebenhausen)
	JEBENHAUSEN_HERDWEG = 'de:08117:2100'
	JEBENHAUSEN_HERDWEG_1 = 'de:08117:2100:0:H1'  # Rtg Linde (Göppingen Jebenhausen)
	JEBENHAUSEN_HERDWEG_2 = 'de:08117:2100:0:R1'  # Rtg Göppingen (Göppingen Jebenhausen)
	JEBENHAUSEN_LINDE = 'de:08117:2101'
	JEBENHAUSEN_LINDE_1 = 'de:08117:2101:0:H1'  # Rtg Heimbach (Göppingen Jebenhausen)
	JEBENHAUSEN_LINDE_2 = 'de:08117:2101:0:R1'  # Rtg Herdweg (Göppingen Jebenhausen)
	JEBENHAUSEN_ALTE_KIRCHE = 'de:08117:2102'
	JEBENHAUSEN_ALTE_KIRCHE_1 = 'de:08117:2102:0:H1'  # Rtg Industriegebiet Süd (Göppingen Jebenhausen)
	JEBENHAUSEN_ALTE_KIRCHE_2 = 'de:08117:2102:0:R1'  # Rtg Linde (Göppingen Jebenhausen)
	ALTE_KIRCHE = 'de:08117:2102'
	ALTE_KIRCHE_1 = 'de:08117:2102:0:H1'  # Rtg Industriegebiet Süd (Göppingen Jebenhausen)
	ALTE_KIRCHE_2 = 'de:08117:2102:0:R1'  # Rtg Linde (Göppingen Jebenhausen)
	BEZGENRIET_BEZIRKSAMT = 'de:08117:2103'
	BEZGENRIET_BEZIRKSAMT_1 = 'de:08117:2103:0:H1'  # Rtg Schlopfenberg (Göppingen Bezgenriet)
	BEZGENRIET_BEZIRKSAMT_2 = 'de:08117:2103:0:R1'  # Rtg Jebenhausen (Göppingen Bezgenriet)
	BEZIRKSAMT_1 = 'de:08117:2103:0:H1'  # Rtg Schlopfenberg (Göppingen Bezgenriet)
	BEZIRKSAMT_2 = 'de:08117:2103:0:R1'  # Rtg Jebenhausen (Göppingen Bezgenriet)
	BEZIRKSAMT_3 = 'de:08136:7821:0:1'  # Bezirksamt (Waldstetten Wißgoldingen)
	BEZIRKSAMT_4 = 'de:08136:7821:0:R1'  # Bezirksamt (Waldstetten Wißgoldingen)
	SCHOPFLENBERG = 'de:08117:2104'
	SCHOPFLENBERG_1 = 'de:08117:2104:0:H1'  # Rtg Hattenhofen (Göppingen Bezgenriet)
	SCHOPFLENBERG_2 = 'de:08117:2104:0:R1'  # Rtg Hattenhofen (Göppingen Bezgenriet)
	BEZGENRIET_SCHOPFLENBERG = 'de:08117:2104'
	BEZGENRIET_SCHOPFLENBERG_1 = 'de:08117:2104:0:H1'  # Rtg Hattenhofen (Göppingen Bezgenriet)
	BEZGENRIET_SCHOPFLENBERG_2 = 'de:08117:2104:0:R1'  # Rtg Hattenhofen (Göppingen Bezgenriet)
	BAHNHOFSALLEE = 'de:08117:2106'
	BAHNHOFSALLEE_1 = 'de:08117:2106:0:H1'  # Rtg Ortsmitte (Bad Boll Bad Boll)
	BAHNHOFSALLEE_2 = 'de:08117:2106:0:R1'  # Rtg Sehningen (Bad Boll Bad Boll)
	BAD_BOLL_BAHNHOFSALLEE = 'de:08117:2106'
	BAD_BOLL_BAHNHOFSALLEE_1 = 'de:08117:2106:0:H1'  # Rtg Ortsmitte (Bad Boll Bad Boll)
	BAD_BOLL_BAHNHOFSALLEE_2 = 'de:08117:2106:0:R1'  # Rtg Sehningen (Bad Boll Bad Boll)
	BAD_BOLL_ORTSMITTE = 'de:08117:2107'
	BAD_BOLL_ORTSMITTE_1 = 'de:08117:2107:0:H1'  # Rtg Gruibinger Straße (Bad Boll Bad Boll)
	BAD_BOLL_ORTSMITTE_2 = 'de:08117:2107:0:R1'  # Rtg Bahnhofsallee (Bad Boll Bad Boll)
	GRUIBINGER_STRASSE = 'de:08117:2108'
	GRUIBINGER_STRASSE_1 = 'de:08117:2108:0:H1'  # Rtg Eichhalde (Bad Boll Bad Boll)
	GRUIBINGER_STRASSE_2 = 'de:08117:2108:0:R1'  # Rtg Ortsmitte (Bad Boll Bad Boll)
	BAD_BOLL_GRUIBINGER_STR = 'de:08117:2108'
	BAD_BOLL_GRUIBINGER_STR_1 = 'de:08117:2108:0:H1'  # Rtg Eichhalde (Bad Boll Bad Boll)
	BAD_BOLL_GRUIBINGER_STR_2 = 'de:08117:2108:0:R1'  # Rtg Ortsmitte (Bad Boll Bad Boll)
	EICHHALDE = 'de:08117:2109'
	EICHHALDE_1 = 'de:08117:2109:0:H1'  # Rtg Ev. Akademie (Bad Boll Bad Boll)
	EICHHALDE_2 = 'de:08117:2109:0:R1'  # Rtg Gruibinger Str (Bad Boll Bad Boll)
	BAD_BOLL_EICHHALDE = 'de:08117:2109'
	BAD_BOLL_EICHHALDE_1 = 'de:08117:2109:0:H1'  # Rtg Ev. Akademie (Bad Boll Bad Boll)
	BAD_BOLL_EICHHALDE_2 = 'de:08117:2109:0:R1'  # Rtg Gruibinger Str (Bad Boll Bad Boll)
	BAD_BOLL_ECKW__ORTSEINGANG = 'de:08117:2111:0:H1'  # Rtg WALA (Bad Boll Bad Boll)
	ECKWÄLDEN_ORTSEINGANG = 'de:08117:2111:0:H1'  # Rtg WALA (Bad Boll Bad Boll)
	ZELL__A__GÖPPINGER_STRASSE = 'de:08117:2117'
	ZELL__A__GÖPPINGER_STRASSE_1 = 'de:08117:2117:0:H1'  # Rtg Schillerstr. (Zell unter Aichelberg Zell (A))
	ZELL__A__GÖPPINGER_STRASSE_2 = 'de:08117:2117:0:R1'  # Rtg Bad Boll (Zell unter Aichelberg Zell (A))
	ZELL__A__SCHILLERSTR = 'de:08117:2113'
	ZELL__A__SCHILLERSTR_1 = 'de:08117:2113:0:H1'  # Schillerstraße (Zell unter Aichelberg Zell (A))
	ZELL__A__SCHILLERSTR_2 = 'de:08117:2113:0:R1'  # Rtg Göppinger Straße (Zell unter Aichelberg Zell (A))
	ZELL__A__KATH__KIRCHE_1 = 'de:08117:12114:0:1'  # Rtg Schillerstraße (Zell unter Aichelberg Zell (A))
	ZELL__A__KATH__KIRCHE_2 = 'de:08117:12114:0:2'  # Rtg Hattenh./Ohmden (Zell unter Aichelberg Zell (A))
	ZELL__A__KATH__KIRCHE_3 = 'de:08117:2114:0:H1'  # Rtg Kirchheimer Straße (Zell unter Aichelberg Zell (A))
	ZELL__A__KATH__KIRCHE_4 = 'de:08117:2114:0:R1'  # Rtg Schillerstr. (Zell unter Aichelberg Zell (A))
	ZELL__A__KIRCHHEIMER_STRASSE = 'de:08117:2115'
	ZELL__A__KIRCHHEIMER_STRASSE_1 = 'de:08117:2115:0:H1'  # Rtg Weilheimer Straße (Zell unter Aichelberg Zell (A))
	ZELL__A__KIRCHHEIMER_STRASSE_2 = 'de:08117:2115:0:R1'  # Rtg Kath. Kirche (Zell unter Aichelberg Zell (A))
	ZELL__A__WEILHEIMER_STRASSE = 'de:08117:2116'
	ZELL__A__WEILHEIMER_STRASSE_1 = 'de:08117:2116:0:H1'  # Rtg Aichelberg (Zell unter Aichelberg Zell (A))
	ZELL__A__WEILHEIMER_STRASSE_2 = 'de:08117:2116:0:R1'  # Rtg Kirchheimer Straße (Zell unter Aichelberg Zell (A))
	WEILHEIMER_STRASSE = 'de:08117:2116'
	WEILHEIMER_STRASSE_1 = 'de:08117:2116:0:H1'  # Rtg Aichelberg (Zell unter Aichelberg Zell (A))
	WEILHEIMER_STRASSE_2 = 'de:08117:2116:0:R1'  # Rtg Kirchheimer Straße (Zell unter Aichelberg Zell (A))
	INDUSTRIEGEBIET_SÜD = 'de:08117:2118:0:H1'  # Rtg Bezgenriet (Göppingen Jebenhausen)
	JEBENH__INDUSTRIEGEBIET_SÜD = 'de:08117:2118:0:H1'  # Rtg Bezgenriet (Göppingen Jebenhausen)
	WALA = 'de:08117:2119:0:H1'  # Rtg Eckwälden (Bad Boll Bad Boll)
	BAD_BOLL_WALA = 'de:08117:2119:0:H1'  # Rtg Eckwälden (Bad Boll Bad Boll)
	SUDETENSTRASSE_1 = 'de:08117:2151:0:H1'  # Rtg Waldeckschule (Göppingen Jebenhausen)
	SUDETENSTRASSE_2 = 'de:08117:7614:0:H1'  # Rtg Marrenstraße (Donzdorf Donzdorf)
	SUDETENSTRASSE_3 = 'de:08117:7614:0:R1'  # Rtg Adolf-Kolping-Str (Donzdorf Donzdorf)
	JEBENHAUSEN_SUDETENSTRASSE = 'de:08117:2151:0:H1'  # Rtg Waldeckschule (Göppingen Jebenhausen)
	VORDERER_BERG = 'de:08117:2152:0:H1'  # Rtg Linde (Göppingen Jebenhausen)
	JEBENHAUSEN_VORDERER_BERG = 'de:08117:2152:0:H1'  # Rtg Linde (Göppingen Jebenhausen)
	FASANENSTRASSE = 'de:08117:2153:0:H1'  # Rtg Wasenstraße (Göppingen Jebenhausen)
	JEBENHAUSEN_FASANENSTRASSE = 'de:08117:2153:0:H1'  # Rtg Wasenstraße (Göppingen Jebenhausen)
	JEBENHAUSEN_WASENSTRASSE = 'de:08117:2154'
	JEBENHAUSEN_WASENSTRASSE_1 = 'de:08117:2154:0:H1'  # Rtg Falkenstraße (Göppingen Jebenhausen)
	JEBENHAUSEN_WASENSTRASSE_2 = 'de:08117:2154:0:R1'  # Rtg Vorderer Berg (Göppingen Jebenhausen)
	WALDECKSCHULE = 'de:08117:2155:0:H1'  # Rtg Wasenstraße (Göppingen Jebenhausen)
	JEBENHAUSEN_WALDECKSCHULE = 'de:08117:2155:0:H1'  # Rtg Wasenstraße (Göppingen Jebenhausen)
	REUSTADT = 'de:08117:2156'
	REUSTADT_1 = 'de:08117:2156:0:H1'  # Rtg Hattenhofen (Hattenhofen Hattenhofen)
	REUSTADT_2 = 'de:08117:2156:0:R1'  # Rtg Göppingen (Hattenhofen Hattenhofen)
	HATTENHOFEN_REUSTADT = 'de:08117:2156'
	HATTENHOFEN_REUSTADT_1 = 'de:08117:2156:0:H1'  # Rtg Hattenhofen (Hattenhofen Hattenhofen)
	HATTENHOFEN_REUSTADT_2 = 'de:08117:2156:0:R1'  # Rtg Göppingen (Hattenhofen Hattenhofen)
	UHINGEN_BURGSTALL = 'de:08117:2157'
	UHINGEN_BURGSTALL_1 = 'de:08117:2157:0:H1'  # Rtg Sparwiesen (Uhingen Uhingen)
	UHINGEN_BURGSTALL_2 = 'de:08117:2157:0:R1'  # Rtg Uhingen (Uhingen Uhingen)
	BURGSTALL = 'de:08117:2157'
	BURGSTALL_1 = 'de:08117:2157:0:H1'  # Rtg Sparwiesen (Uhingen Uhingen)
	BURGSTALL_2 = 'de:08117:2157:0:R1'  # Rtg Uhingen (Uhingen Uhingen)
	HEIMBACH = 'de:08117:2189:0:H1'  # Ankunft (Göppingen Jebenhausen)
	JEBENHAUSEN_HEIMBACH = 'de:08117:2189:0:H1'  # Ankunft (Göppingen Jebenhausen)
	UHINGEN_RATHAUS = 'de:08117:2201:0:H1'  # Rtg Sparwiesen (Uhingen Uhingen)
	UHINGEN_WASSERBERG = 'de:08117:2202'
	UHINGEN_WASSERBERG_1 = 'de:08117:2202:0:H1'  # Rtg Sparwiesen (Uhingen Uhingen)
	UHINGEN_WASSERBERG_2 = 'de:08117:2202:0:R1'  # Rtg Uhingen (Uhingen Uhingen)
	WASSERBERG = 'de:08117:2202'
	WASSERBERG_1 = 'de:08117:2202:0:H1'  # Rtg Sparwiesen (Uhingen Uhingen)
	WASSERBERG_2 = 'de:08117:2202:0:R1'  # Rtg Uhingen (Uhingen Uhingen)
	UHINGEN_WEILENBERGER_HOF = 'de:08117:2203'
	UHINGEN_WEILENBERGER_HOF_1 = 'de:08117:2203:0:H1'  # Rtg Sparwiesen (Uhingen Uhingen)
	UHINGEN_WEILENBERGER_HOF_2 = 'de:08117:2203:0:R1'  # Rtg Uhingen (Uhingen Uhingen)
	WEILENBERGER_HOF = 'de:08117:2203'
	WEILENBERGER_HOF_1 = 'de:08117:2203:0:H1'  # Rtg Sparwiesen (Uhingen Uhingen)
	WEILENBERGER_HOF_2 = 'de:08117:2203:0:R1'  # Rtg Uhingen (Uhingen Uhingen)
	SPARWIESEN_STEINIGE_HALDE = 'de:08117:2204'
	SPARWIESEN_STEINIGE_HALDE_1 = 'de:08117:2204:0:H1'  # Rtg Schopflenberg (Uhingen Sparwiesen)
	SPARWIESEN_STEINIGE_HALDE_2 = 'de:08117:2204:0:R1'  # Rtg Uhingen (Uhingen Sparwiesen)
	STEINIGE_HALDE = 'de:08117:2204'
	STEINIGE_HALDE_1 = 'de:08117:2204:0:H1'  # Rtg Schopflenberg (Uhingen Sparwiesen)
	STEINIGE_HALDE_2 = 'de:08117:2204:0:R1'  # Rtg Uhingen (Uhingen Sparwiesen)
	GÖPPINGEN_MARKTPLATZ = 'de:08117:3002:0:H1'  # Rtg Bleichstr. (Göppingen Göppingen)
	GÖPPINGEN_BRÜCKENSTR = 'de:08117:3003:0:R1'  # Rtg Kaiserbau (Ag. für Arb.) (Göppingen Göppingen)
	FUCHSECKSTRASSE_1 = 'de:08117:3004:0:H1'  # Rtg Holzheim (Göppingen Göppingen)
	FUCHSECKSTRASSE_2 = 'de:08117:3004:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	FUCHSECKSTRASSE_3 = 'de:08117:7613:0:H1'  # Rtg Schlater Str. (Süßen Süßen)
	FUCHSECKSTRASSE_4 = 'de:08117:7613:0:R1'  # Ankunft (Süßen Süßen)
	GÖPPINGEN_FUCHSECKSTR = 'de:08117:3004'
	GÖPPINGEN_FUCHSECKSTR_1 = 'de:08117:3004:0:H1'  # Rtg Holzheim (Göppingen Göppingen)
	GÖPPINGEN_FUCHSECKSTR_2 = 'de:08117:3004:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_WASSERBERGSTR = 'de:08117:3005'
	GÖPPINGEN_WASSERBERGSTR_1 = 'de:08117:3005:0:H1'  # Rtg Holzheim (Göppingen Göppingen)
	GÖPPINGEN_WASSERBERGSTR_2 = 'de:08117:3005:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	WASSERBERGSTRASSE = 'de:08117:3005'
	WASSERBERGSTRASSE_1 = 'de:08117:3005:0:H1'  # Rtg Holzheim (Göppingen Göppingen)
	WASSERBERGSTRASSE_2 = 'de:08117:3005:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_SANKT_PAULS_KIRCHE = 'de:08117:3006'
	GÖPPINGEN_SANKT_PAULS_KIRCHE_1 = 'de:08117:3006:0:H1'  # Rtg Holzheim (Göppingen Göppingen)
	GÖPPINGEN_SANKT_PAULS_KIRCHE_2 = 'de:08117:3006:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	SANKT_PAULS_KIRCHE = 'de:08117:3006'
	SANKT_PAULS_KIRCHE_1 = 'de:08117:3006:0:H1'  # Rtg Holzheim (Göppingen Göppingen)
	SANKT_PAULS_KIRCHE_2 = 'de:08117:3006:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPP__INDUSTRIEGEB__OST = 'de:08117:3007'
	GÖPP__INDUSTRIEGEB__OST_1 = 'de:08117:3007:0:H1'  # Rtg Holzheim (Göppingen Göppingen)
	GÖPP__INDUSTRIEGEB__OST_2 = 'de:08117:3007:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	WIELANDSTRASSE_BF = 'de:08117:3008'
	WIELANDSTRASSE_BF_1 = 'de:08117:3008:0:H1'  # Rtg Rigipark (Göppingen Holzheim (Göppingen))
	WIELANDSTRASSE_BF_2 = 'de:08117:3008:0:R1'  # Rtg Göppingen (Göppingen Holzheim (Göppingen))
	HOLZHEIM_WIELANDSTRASSE_BF = 'de:08117:3008'
	HOLZHEIM_WIELANDSTRASSE_BF_1 = 'de:08117:3008:0:H1'  # Rtg Rigipark (Göppingen Holzheim (Göppingen))
	HOLZHEIM_WIELANDSTRASSE_BF_2 = 'de:08117:3008:0:R1'  # Rtg Göppingen (Göppingen Holzheim (Göppingen))
	BUCHRAIN = 'de:08117:3009:0:R1'  # Rtg Rigipark (Göppingen Holzheim (Göppingen))
	HOLZHEIM_BUCHRAIN = 'de:08117:3009:0:R1'  # Rtg Rigipark (Göppingen Holzheim (Göppingen))
	HOLZHEIM_RIGIPARK = 'de:08117:3010'
	HOLZHEIM_RIGIPARK_1 = 'de:08117:3010:0:H1'  # Rtg Steinstraße (Göppingen Holzheim (Göppingen))
	HOLZHEIM_RIGIPARK_2 = 'de:08117:3010:0:R1'  # Rtg Wielandstraße/Bf (Göppingen Holzheim (Göppingen))
	RIGIPARK = 'de:08117:3010'
	RIGIPARK_1 = 'de:08117:3010:0:H1'  # Rtg Steinstraße (Göppingen Holzheim (Göppingen))
	RIGIPARK_2 = 'de:08117:3010:0:R1'  # Rtg Wielandstraße/Bf (Göppingen Holzheim (Göppingen))
	GÖPP__ST__GOTTH__WEILERBACHWEG = 'de:08117:3011'
	GÖPP__ST__GOTTH__WEILERBACHWEG_1 = 'de:08117:3011:0:H1'  # Rtg Manzen Ob. Siedlung (Göppingen Göppingen)
	GÖPP__ST__GOTTH__WEILERBACHWEG_2 = 'de:08117:3011:0:R1'  # Rtg Holzheim (Göppingen Göppingen)
	ST__GOTTH__WEILERBACHWEG = 'de:08117:3011'
	ST__GOTTH__WEILERBACHWEG_1 = 'de:08117:3011:0:H1'  # Rtg Manzen Ob. Siedlung (Göppingen Göppingen)
	ST__GOTTH__WEILERBACHWEG_2 = 'de:08117:3011:0:R1'  # Rtg Holzheim (Göppingen Göppingen)
	MANZEN_OBERE_SIEDLUNG = 'de:08117:3012'
	MANZEN_OBERE_SIEDLUNG_1 = 'de:08117:3012:0:H1'  # Rtg Manzen Spitzenbergstr. (Göppingen Göppingen)
	MANZEN_OBERE_SIEDLUNG_2 = 'de:08117:3012:0:R1'  # Rtg Manzen Spitzenbergstr. (Göppingen Göppingen)
	GÖPP__MANZEN_OB__SIEDLUNG = 'de:08117:3012'
	GÖPP__MANZEN_OB__SIEDLUNG_1 = 'de:08117:3012:0:H1'  # Rtg Manzen Spitzenbergstr. (Göppingen Göppingen)
	GÖPP__MANZEN_OB__SIEDLUNG_2 = 'de:08117:3012:0:R1'  # Rtg Manzen Spitzenbergstr. (Göppingen Göppingen)
	GÖPP__URSENW__LINDENPLATZ = 'de:08117:3013:0:H1'  # Rtg Ursenw. Kiefernsteige (Göppingen Göppingen)
	URSENWANG_LINDENPLATZ = 'de:08117:3013:0:H1'  # Rtg Ursenw. Kiefernsteige (Göppingen Göppingen)
	URSENWANG_KIEFERNSTEIGE = 'de:08117:3014:0:H1'  # Rtg Ursenwang Eschenstr. (Göppingen Göppingen)
	GÖPP__URSENW__KIEFERNSTEIGE = 'de:08117:3014:0:H1'  # Rtg Ursenwang Eschenstr. (Göppingen Göppingen)
	URSENWANG_ESCHENSTR = 'de:08117:3015:0:H1'  # Rtg Schlat (Göppingen Göppingen)
	GÖPPINGEN_URSENWANG_ESCHENSTR = 'de:08117:3015:0:H1'  # Rtg Schlat (Göppingen Göppingen)
	MANZEN_MANZENSTR = 'de:08117:3017:0:R1'  # Rtg Ursenw. Grundschule (Göppingen Göppingen)
	GÖPP__MANZEN_MANZENSTR = 'de:08117:3017:0:R1'  # Rtg Ursenw. Grundschule (Göppingen Göppingen)
	SCHLAT_KIRCHE = 'de:08117:3018'
	SCHLAT_KIRCHE_1 = 'de:08117:3018:0:H1'  # Rtg Bachstr. (Schlat Schlat)
	SCHLAT_KIRCHE_2 = 'de:08117:3018:0:R1'  # Rtg Enzbachweg (Schlat Schlat)
	SCHLAT_BACHSTR = 'de:08117:3020:0:H1'  # Rtg Kirche (Schlat Schlat)
	ROMMENTALER_STRASSE = 'de:08117:3022:0:H1'  # Rtg Schlater Str. (Schlat Schlat)
	SCHLAT_ROMMENTALER_STR = 'de:08117:3022:0:H1'  # Rtg Schlater Str. (Schlat Schlat)
	MANZEN_SPITZENBERGSTR = 'de:08117:3023:0:R1'  # Rtg Manzen Sportplatz (Göppingen Göppingen)
	GÖPP__MANZEN_SPITZENBERGSTR = 'de:08117:3023:0:R1'  # Rtg Manzen Sportplatz (Göppingen Göppingen)
	GÖPP__MANZEN_SPORTPLATZ = 'de:08117:3024:0:R1'  # Rtg Manzen Manzenstr. (Göppingen Göppingen)
	MANZEN_SPORTPLATZ = 'de:08117:3024:0:R1'  # Rtg Manzen Manzenstr. (Göppingen Göppingen)
	HOLZHEIM_STEINSTRASSE = 'de:08117:3025'
	HOLZHEIM_STEINSTRASSE_1 = 'de:08117:3025:0:H1'  # Rtg Göppingen (Göppingen Holzheim (Göppingen))
	HOLZHEIM_STEINSTRASSE_2 = 'de:08117:3025:0:R1'  # Rtg Buchrain (Göppingen Holzheim (Göppingen))
	STEINSTRASSE = 'de:08117:3025'
	STEINSTRASSE_1 = 'de:08117:3025:0:H1'  # Rtg Göppingen (Göppingen Holzheim (Göppingen))
	STEINSTRASSE_2 = 'de:08117:3025:0:R1'  # Rtg Buchrain (Göppingen Holzheim (Göppingen))
	ENZBACHWEG = 'de:08117:3026:0:H1'  # Rtg Rommentaler Str. (Schlat Schlat)
	SCHLAT_ENZBACHWEG = 'de:08117:3026:0:H1'  # Rtg Rommentaler Str. (Schlat Schlat)
	KREUZUNG_GRUIBINGEN_AUENDORF = 'de:08117:3224'
	KREUZUNG_GRUIBINGEN_AUENDORF_1 = 'de:08117:3224:1:H1'  # Rtg Kornbergsattel (Gammelshausen Gammelshausen)
	KREUZUNG_GRUIBINGEN_AUENDORF_2 = 'de:08117:3224:1:R1'  # Rtg Kirche (Gammelshausen Gammelshausen)
	KREUZUNG_GRUIBINGEN_AUENDORF_3 = 'de:08117:3224:2:H2'  # Rtg Auerndorf (Gammelshausen Gammelshausen)
	KREUZUNG_GRUIBINGEN_AUENDORF_4 = 'de:08117:3224:2:R2'  # Rtg Kirche (Gammelshausen Gammelshausen)
	GAMMELSH__KRZG__GRUIB__AUEND = 'de:08117:3224'
	GAMMELSH__KRZG__GRUIB__AUEND_1 = 'de:08117:3224:1:H1'  # Rtg Kornbergsattel (Gammelshausen Gammelshausen)
	GAMMELSH__KRZG__GRUIB__AUEND_2 = 'de:08117:3224:1:R1'  # Rtg Kirche (Gammelshausen Gammelshausen)
	GAMMELSH__KRZG__GRUIB__AUEND_3 = 'de:08117:3224:2:H2'  # Rtg Auerndorf (Gammelshausen Gammelshausen)
	GAMMELSH__KRZG__GRUIB__AUEND_4 = 'de:08117:3224:2:R2'  # Rtg Kirche (Gammelshausen Gammelshausen)
	FUCHSECKSATTEL = 'de:08117:3225'
	FUCHSECKSATTEL_1 = 'de:08117:3225:0:H1'  # Rtg Volksbank (Bad Ditzenbach Auendorf)
	FUCHSECKSATTEL_2 = 'de:08117:3225:0:R1'  # Rtg Gammelshausen (Bad Ditzenbach Auendorf)
	AUENDORF_FUCHSECKSATTEL = 'de:08117:3225'
	AUENDORF_FUCHSECKSATTEL_1 = 'de:08117:3225:0:H1'  # Rtg Volksbank (Bad Ditzenbach Auendorf)
	AUENDORF_FUCHSECKSATTEL_2 = 'de:08117:3225:0:R1'  # Rtg Gammelshausen (Bad Ditzenbach Auendorf)
	AUENDORF_VOLKSBANK = 'de:08117:3226'
	AUENDORF_VOLKSBANK_1 = 'de:08117:3226:0:H1'  # Rtg Gosbach (Bad Ditzenbach Auendorf)
	AUENDORF_VOLKSBANK_2 = 'de:08117:3226:0:R1'  # Rtg Fuchsecksattel (Bad Ditzenbach Auendorf)
	AUENDORF_DITZENBACHER_STR = 'de:08117:3227'
	AUENDORF_DITZENBACHER_STR_1 = 'de:08117:3227:0:H1'  # Rtg Grundschule (Bad Ditzenbach Auendorf)
	AUENDORF_DITZENBACHER_STR_2 = 'de:08117:3227:0:R1'  # Rtg Volksbank (Bad Ditzenbach Auendorf)
	DITZENBACHER_STR = 'de:08117:3227'
	DITZENBACHER_STR_1 = 'de:08117:3227:0:H1'  # Rtg Grundschule (Bad Ditzenbach Auendorf)
	DITZENBACHER_STR_2 = 'de:08117:3227:0:R1'  # Rtg Volksbank (Bad Ditzenbach Auendorf)
	MÜHLHAUSEN_I_T__AM_WARMEN = 'de:08117:3230'
	MÜHLHAUSEN_I_T__AM_WARMEN_1 = 'de:08117:3230:0:H1'  # Rtg Bahnhofstraße (Mühlhausen im Täle Mühlhausen im Täle)
	MÜHLHAUSEN_I_T__AM_WARMEN_2 = 'de:08117:3230:0:R1'  # Rtg Gruibingen (Mühlhausen im Täle Mühlhausen im Täle)
	AM_WARMEN = 'de:08117:3230'
	AM_WARMEN_1 = 'de:08117:3230:0:H1'  # Rtg Bahnhofstraße (Mühlhausen im Täle Mühlhausen im Täle)
	AM_WARMEN_2 = 'de:08117:3230:0:R1'  # Rtg Gruibingen (Mühlhausen im Täle Mühlhausen im Täle)
	GRUIBINGEN_MÜHLHAUSER_STR = 'de:08117:3231:0:R1'  # Rtg Rathaus (Gruibingen Gruibingen)
	MÜHLHAUSER_STR = 'de:08117:3231:0:R1'  # Rtg Rathaus (Gruibingen Gruibingen)
	GRUIBINGEN_RATHAUS = 'de:08117:3232'
	GRUIBINGEN_RATHAUS_1 = 'de:08117:3232:0:3'  # Rtg Mühlhausen (Gruibingen Gruibingen)
	GRUIBINGEN_RATHAUS_2 = 'de:08117:3232:0:R1'  # Rtg Grundschule (Gruibingen Gruibingen)
	GRUIBINGEN_GRUNDSCHULE = 'de:08117:3233:0:H1'  # Rtg Brückenweg (Gruibingen Gruibingen)
	GRUIBINGEN_BRÜCKENWEG = 'de:08117:3234'
	GRUIBINGEN_BRÜCKENWEG_1 = 'de:08117:3234:0:H1'  # Rtg Gasthaus Lamm (Gruibingen Gruibingen)
	GRUIBINGEN_BRÜCKENWEG_2 = 'de:08117:3234:0:R1'  # Rtg Kornbergsattel (Gruibingen Gruibingen)
	BRÜCKENWEG = 'de:08117:3234'
	BRÜCKENWEG_1 = 'de:08117:3234:0:H1'  # Rtg Gasthaus Lamm (Gruibingen Gruibingen)
	BRÜCKENWEG_2 = 'de:08117:3234:0:R1'  # Rtg Kornbergsattel (Gruibingen Gruibingen)
	KORNBERGSATTEL = 'de:08117:3235'
	KORNBERGSATTEL_1 = 'de:08117:3235:0:H1'  # Rtg Brückenweg (Gruibingen Gruibingen)
	KORNBERGSATTEL_2 = 'de:08117:3235:0:R1'  # Rtg Gammelshausen (Gruibingen Gruibingen)
	GRUIBINGEN_KORNBERGSATTEL = 'de:08117:3235'
	GRUIBINGEN_KORNBERGSATTEL_1 = 'de:08117:3235:0:H1'  # Rtg Brückenweg (Gruibingen Gruibingen)
	GRUIBINGEN_KORNBERGSATTEL_2 = 'de:08117:3235:0:R1'  # Rtg Gammelshausen (Gruibingen Gruibingen)
	GRUIBINGEN_KATH__KIRCHE = 'de:08117:3239:0:H1'  # Rtg Mühlhausen i. T. (Gruibingen Gruibingen)
	GÖPPINGEN_HOCHSCHULE = 'de:08117:3301'
	GÖPPINGEN_HOCHSCHULE_1 = 'de:08117:3301:0:H1'  # Rtg Ulmer-/Heininger Str. (Göppingen Göppingen)
	GÖPPINGEN_HOCHSCHULE_2 = 'de:08117:3301:0:R1'  # Rtg Bahnhofssteg (Göppingen Göppingen)
	GÖPPINGEN_DIESELSTR = 'de:08117:3302'
	GÖPPINGEN_DIESELSTR_1 = 'de:08117:3302:0:H1'  # Rtg Heiningen (Göppingen Göppingen)
	GÖPPINGEN_DIESELSTR_2 = 'de:08117:3302:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPP__HEININGER__JAHNSTR = 'de:08117:3303'
	GÖPP__HEININGER__JAHNSTR_1 = 'de:08117:3303:0:H1'  # Rtg Heiningen (Göppingen Göppingen)
	GÖPP__HEININGER__JAHNSTR_2 = 'de:08117:3303:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	HEININGER__JAHNSTRASSE = 'de:08117:3303'
	HEININGER__JAHNSTRASSE_1 = 'de:08117:3303:0:H1'  # Rtg Heiningen (Göppingen Göppingen)
	HEININGER__JAHNSTRASSE_2 = 'de:08117:3303:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPP__POLIZEIPRÄSIDIUM_EINSATZ = 'de:08117:3305'
	GÖPP__POLIZEIPRÄSIDIUM_EINSATZ_1 = 'de:08117:3305:0:H1'  # Rtg Heiningen (Göppingen Göppingen)
	GÖPP__POLIZEIPRÄSIDIUM_EINSATZ_2 = 'de:08117:3305:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	POLIZEIPRÄSIDIUM_EINSATZ = 'de:08117:3305'
	POLIZEIPRÄSIDIUM_EINSATZ_1 = 'de:08117:3305:0:H1'  # Rtg Heiningen (Göppingen Göppingen)
	POLIZEIPRÄSIDIUM_EINSATZ_2 = 'de:08117:3305:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	HEININGEN_AUSSIEDLERHÖFE = 'de:08117:3306'
	HEININGEN_AUSSIEDLERHÖFE_1 = 'de:08117:3306:0:H1'  # Rtg Adler (Heiningen Heiningen (GP))
	HEININGEN_AUSSIEDLERHÖFE_2 = 'de:08117:3306:0:R1'  # Rtg Göppingen (Heiningen Heiningen (GP))
	ADLER = 'de:08117:3307'
	ADLER_1 = 'de:08117:3307:0:H1'  # Rtg Kirchstr. (Heiningen Heiningen (GP))
	ADLER_2 = 'de:08117:3307:0:R1'  # Rtg Aussiedlerhöfe (Heiningen Heiningen (GP))
	HEININGEN_ADLER = 'de:08117:3307'
	HEININGEN_ADLER_1 = 'de:08117:3307:0:H1'  # Rtg Kirchstr. (Heiningen Heiningen (GP))
	HEININGEN_ADLER_2 = 'de:08117:3307:0:R1'  # Rtg Aussiedlerhöfe (Heiningen Heiningen (GP))
	HEININGEN_KIRCHSTR = 'de:08117:3308:0:H1'  # Rtg Eschenbach (Heiningen Heiningen (GP))
	HEININGEN_UHLANDSTRASSE = 'de:08117:3309'
	HEININGEN_UHLANDSTRASSE_1 = 'de:08117:3309:0:H1'  # Rtg Mörikestr. (Heiningen Heiningen (GP))
	HEININGEN_UHLANDSTRASSE_2 = 'de:08117:3309:0:R1'  # Rtg Adler (Heiningen Heiningen (GP))
	HEININGEN_MÖRIKESTRASSE = 'de:08117:3310'
	HEININGEN_MÖRIKESTRASSE_1 = 'de:08117:3310:0:H1'  # Rtg Breitestraße (Heiningen Heiningen (GP))
	HEININGEN_MÖRIKESTRASSE_2 = 'de:08117:3310:0:R1'  # Rtg Uhlandstraße (Heiningen Heiningen (GP))
	BREITESTRASSE = 'de:08117:3311'
	BREITESTRASSE_1 = 'de:08117:3311:0:H1'  # Rtg Lindenstr. (Heiningen Heiningen (GP))
	BREITESTRASSE_2 = 'de:08117:3311:0:R1'  # Rtg Mörikestraße (Heiningen Heiningen (GP))
	HEININGEN_BREITESTR = 'de:08117:3311'
	HEININGEN_BREITESTR_1 = 'de:08117:3311:0:H1'  # Rtg Lindenstr. (Heiningen Heiningen (GP))
	HEININGEN_BREITESTR_2 = 'de:08117:3311:0:R1'  # Rtg Mörikestraße (Heiningen Heiningen (GP))
	HEININGEN_FRIEDHOF = 'de:08117:3312'
	HEININGEN_FRIEDHOF_1 = 'de:08117:3312:0:H1'  # Rtg Eschenbach (Heiningen Heiningen (GP))
	HEININGEN_FRIEDHOF_2 = 'de:08117:3312:0:R1'  # Rtg Lindenstr. (Heiningen Heiningen (GP))
	ALTER_BAHNHOF = 'de:08117:3320'
	ALTER_BAHNHOF_1 = 'de:08117:3320:0:H1'  # Rtg Gammelshausen (Heiningen Heiningen (GP))
	ALTER_BAHNHOF_2 = 'de:08117:3320:0:R1'  # Rtg Lindenstr. (Heiningen Heiningen (GP))
	HEININGEN_ALTER_BAHNHOF = 'de:08117:3320'
	HEININGEN_ALTER_BAHNHOF_1 = 'de:08117:3320:0:H1'  # Rtg Gammelshausen (Heiningen Heiningen (GP))
	HEININGEN_ALTER_BAHNHOF_2 = 'de:08117:3320:0:R1'  # Rtg Lindenstr. (Heiningen Heiningen (GP))
	ESCHENBACH_FRÜHLINGSTR = 'de:08117:3314:0:H1'  # Rtg Gemeindehaus (Eschenbach Eschenbach)
	ESCHENBACH_GEMEINDEHAUS = 'de:08117:3315:0:H1'  # Rtg Grundschule (Eschenbach Eschenbach)
	ESCHENBACH_BACKHAUS = 'de:08117:3316:0:H1'  # Backhaus (Eschenbach Eschenbach)
	ESCHENBACH_GRUNDSCHULE = 'de:08117:3317:0:H1'  # Rtg Heiningen (Eschenbach Eschenbach)
	ESCHENBACH_FEUERWEHRHAUS = 'de:08117:3318:0:H1'  # Rtg Gewerbepark/Heiningen (Eschenbach Eschenbach)
	GEWERBEPARK = 'de:08117:3319:0:H1'  # Rtg Göppingen (Eschenbach Eschenbach)
	ESCHENBACH_GEWERBEPARK = 'de:08117:3319:0:H1'  # Rtg Göppingen (Eschenbach Eschenbach)
	GAMMELSHAUSEN_KREISVERKEHR = 'de:08117:3322'
	GAMMELSHAUSEN_KREISVERKEHR_1 = 'de:08117:3322:0:H1'  # Rtg Kirche (Gammelshausen Gammelshausen)
	GAMMELSHAUSEN_KREISVERKEHR_2 = 'de:08117:3322:0:R1'  # Rtg Dürnau (Gammelshausen Gammelshausen)
	GAMMELSHAUSEN_KREISVERKEHR_3 = 'de:08117:3322:2:3'  # Rtg Heiningen (Gammelshausen Gammelshausen)
	GAMMELSHAUSEN_KREISVERKEHR_4 = 'de:08117:3322:2:4'  # Rtg Dürnau Abzw. Kornberghalle (Gammelshausen Gammelshausen)
	GAMMELSHAUSEN_KIRCHE = 'de:08117:3323'
	GAMMELSHAUSEN_KIRCHE_1 = 'de:08117:3323:0:H1'  # Rtg Krzg. Gruib. Auend. (Gammelshausen Gammelshausen)
	GAMMELSHAUSEN_KIRCHE_2 = 'de:08117:3323:0:R1'  # Rtg Gam. Kreisverkehr (Gammelshausen Gammelshausen)
	GAMMELSHAUSEN_WEINGARTENSTR = 'de:08117:3324:0:H1'  # Rtg Eschenbacher Str. (Gammelshausen Gammelshausen)
	GAMMELSH__ESCHENBACHER_STR = 'de:08117:3325:0:H1'  # Rtg Kreisverkehr (Gammelshausen Gammelshausen)
	ESCHENBACHER_STR = 'de:08117:3325:0:H1'  # Rtg Kreisverkehr (Gammelshausen Gammelshausen)
	DÜRNAU_GAMMELSH__STR = 'de:08117:3327:0:H1'  # Rtg Abzw. Kornberghalle (Dürnau Dürnau)
	GAMMELSHAUSER_STR = 'de:08117:3327:0:H1'  # Rtg Abzw. Kornberghalle (Dürnau Dürnau)
	DÜRNAU_ABZW__KORNBERGHALLE = 'de:08117:3328'
	DÜRNAU_ABZW__KORNBERGHALLE_1 = 'de:08117:3328:0:H1'  # Rtg Rathaus (Dürnau Dürnau)
	DÜRNAU_ABZW__KORNBERGHALLE_2 = 'de:08117:3328:0:R1'  # Rtg Gammelshausen (Dürnau Dürnau)
	ABZWEIG_KORNBERGHALLE = 'de:08117:3328'
	ABZWEIG_KORNBERGHALLE_1 = 'de:08117:3328:0:H1'  # Rtg Rathaus (Dürnau Dürnau)
	ABZWEIG_KORNBERGHALLE_2 = 'de:08117:3328:0:R1'  # Rtg Gammelshausen (Dürnau Dürnau)
	BAD_BOLL_DÜRNAUER_STR = 'de:08117:3330:0:H1'  # Rtg Gruibinger Straße (Bad Boll Bad Boll)
	DÜRNAUER_STRASSE = 'de:08117:3330:0:H1'  # Rtg Gruibinger Straße (Bad Boll Bad Boll)
	BOLLER_STRASSE = 'de:08117:3331'
	BOLLER_STRASSE_1 = 'de:08117:3331:0:H1'  # Rtg Bad Boll (Dürnau Dürnau)
	BOLLER_STRASSE_2 = 'de:08117:3331:0:R1'  # Rtg Gammelshausen (Dürnau Dürnau)
	DÜRNAU_BOLLER_STR = 'de:08117:3331'
	DÜRNAU_BOLLER_STR_1 = 'de:08117:3331:0:H1'  # Rtg Bad Boll (Dürnau Dürnau)
	DÜRNAU_BOLLER_STR_2 = 'de:08117:3331:0:R1'  # Rtg Gammelshausen (Dürnau Dürnau)
	BAD_BOLL_GEMEINSCHAFTSSCHULE = 'de:08117:3332:0:H1'  # Rtg Im Steigle (Bad Boll Bad Boll)
	GEMEINSCHAFTSSCHULE = 'de:08117:3332:0:H1'  # Rtg Im Steigle (Bad Boll Bad Boll)
	BAD_BOLL_KIRCHPLATZ = 'de:08117:3334:0:R1'  # Rtg Dürnau (Bad Boll Bad Boll)
	SEHNINGEN = 'de:08117:3335'
	SEHNINGEN_1 = 'de:08117:3335:0:H1'  # Rtg Bahnhofsallee (Bad Boll Bad Boll)
	SEHNINGEN_2 = 'de:08117:3335:0:H2'  # Im Steigle Rtg Bahnhofsallee (Bad Boll Bad Boll)
	SEHNINGEN_3 = 'de:08117:3335:0:R1'  # Hauptstraße Rtg Bezgenriet (Bad Boll Bad Boll)
	SEHNINGEN_4 = 'de:08117:3335:0:R2'  # Göppinger Str. Rtg Betzgenried (Bad Boll Bad Boll)
	BAD_BOLL_SEHNINGEN = 'de:08117:3335'
	BAD_BOLL_SEHNINGEN_1 = 'de:08117:3335:0:H1'  # Rtg Bahnhofsallee (Bad Boll Bad Boll)
	BAD_BOLL_SEHNINGEN_2 = 'de:08117:3335:0:H2'  # Im Steigle Rtg Bahnhofsallee (Bad Boll Bad Boll)
	BAD_BOLL_SEHNINGEN_3 = 'de:08117:3335:0:R1'  # Hauptstraße Rtg Bezgenriet (Bad Boll Bad Boll)
	BAD_BOLL_SEHNINGEN_4 = 'de:08117:3335:0:R2'  # Göppinger Str. Rtg Betzgenried (Bad Boll Bad Boll)
	BAD_BOLL_EV__AKAD__REHA_KLINIK = 'de:08117:3336:0:H1'  # Rtg Eckw. Ortseingang (Bad Boll Bad Boll)
	EV__AKADEMIE_REHA_KLINIK = 'de:08117:3336:0:H1'  # Rtg Eckw. Ortseingang (Bad Boll Bad Boll)
	DÜRNAU_ORTSMITTE = 'de:08117:3339'
	DÜRNAU_ORTSMITTE_1 = 'de:08117:3339:0:H1'  # Rtg Boller Straße (Dürnau Dürnau)
	DÜRNAU_ORTSMITTE_2 = 'de:08117:3339:0:R1'  # Rtg Abzw. Kornberghalle (Dürnau Dürnau)
	HEININGEN_LINDENSTR = 'de:08117:3378'
	HEININGEN_LINDENSTR_1 = 'de:08117:3378:0:H1'  # Rtg Friedhof (Heiningen Heiningen (GP))
	HEININGEN_LINDENSTR_2 = 'de:08117:3378:0:R1'  # Rtg Breitestr. (Heiningen Heiningen (GP))
	HEININGEN_LINDENSTR_3 = 'de:08117:3378:0:R2'  # Rtg Adler (Heiningen Heiningen (GP))
	GÖPPINGEN_H__HESSE_REALSCHULE = 'de:08117:3383'
	GÖPPINGEN_H__HESSE_REALSCHULE_1 = 'de:08117:3383:0:H1'  # Rtg Heiningen (Göppingen Göppingen)
	GÖPPINGEN_H__HESSE_REALSCHULE_2 = 'de:08117:3383:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	HERMANN_HESSE_REALSCHULE = 'de:08117:3383'
	HERMANN_HESSE_REALSCHULE_1 = 'de:08117:3383:0:H1'  # Rtg Heiningen (Göppingen Göppingen)
	HERMANN_HESSE_REALSCHULE_2 = 'de:08117:3383:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	BAHNHOFSSTEG = 'de:08117:3385'
	BAHNHOFSSTEG_1 = 'de:08117:3385:0:H1'  # Rtg Hochschule (Göppingen Göppingen)
	BAHNHOFSSTEG_2 = 'de:08117:3385:0:R1'  # Rtg Brückenstr. (Göppingen Göppingen)
	GÖPPINGEN_BAHNHOFSSTEG = 'de:08117:3385'
	GÖPPINGEN_BAHNHOFSSTEG_1 = 'de:08117:3385:0:H1'  # Rtg Hochschule (Göppingen Göppingen)
	GÖPPINGEN_BAHNHOFSSTEG_2 = 'de:08117:3385:0:R1'  # Rtg Brückenstr. (Göppingen Göppingen)
	GÖPPINGEN_SCHILLER_REALSCHULE = 'de:08117:3386:0:R1'  # Rtg Friedrichstraße (Göppingen Göppingen)
	SCHILLER_REALSCHULE = 'de:08117:3386:0:R1'  # Rtg Friedrichstraße (Göppingen Göppingen)
	HOHENSTADT_KIRCHE = 'de:08117:3501:0:H1'  # Ankunft (Hohenstadt Hohenstadt (GP))
	OBERDRACKENSTEIN_RATHAUS = 'de:08117:3502'
	OBERDRACKENSTEIN_RATHAUS_1 = 'de:08117:3502:0:H1'  # Rtg Göserhof (Drackenstein Oberdrackenstein)
	OBERDRACKENSTEIN_RATHAUS_2 = 'de:08117:3502:0:R1'  # Rtg Drackenstein (Drackenstein Oberdrackenstein)
	DRACKENST__UNTDR_STEIN_KIRCHE = 'de:08117:3503'
	DRACKENST__UNTDR_STEIN_KIRCHE_1 = 'de:08117:3503:0:H1'  # Rtg Oberdrackenstein (Drackenstein Drackenstein)
	DRACKENST__UNTDR_STEIN_KIRCHE_2 = 'de:08117:3503:0:R1'  # Rtg Gosbach (Drackenstein Drackenstein)
	UNTERDRACKENSTEIN_KIRCHE = 'de:08117:3503'
	UNTERDRACKENSTEIN_KIRCHE_1 = 'de:08117:3503:0:H1'  # Rtg Oberdrackenstein (Drackenstein Drackenstein)
	UNTERDRACKENSTEIN_KIRCHE_2 = 'de:08117:3503:0:R1'  # Rtg Gosbach (Drackenstein Drackenstein)
	GOSBACH_LAMM = 'de:08117:3504'
	GOSBACH_LAMM_1 = 'de:08117:3504:0:H1'  # Rtg Einkaufszentrum (Bad Ditzenbach Gosbach)
	GOSBACH_LAMM_2 = 'de:08117:3504:0:R1'  # Rtg Hirsch (Bad Ditzenbach Gosbach)
	BAD_DITZENB__GRUNDSCHULE = 'de:08117:3505'
	BAD_DITZENB__GRUNDSCHULE_1 = 'de:08117:3505:0:H1'  # Rtg Gosbach Auendorf (Bad Ditzenbach Bad Ditzenbach)
	BAD_DITZENB__GRUNDSCHULE_2 = 'de:08117:3505:0:R1'  # Rtg Kurklinik (Bad Ditzenbach Bad Ditzenbach)
	HOHENSTADT_ABZW__WALTERTAL = 'de:08117:3506'
	HOHENSTADT_ABZW__WALTERTAL_1 = 'de:08117:3506:0:H1'  # Rtg Kirche (Hohenstadt Hohenstadt (GP))
	HOHENSTADT_ABZW__WALTERTAL_2 = 'de:08117:3506:0:R1'  # Rtg Oberdrackenstein (Hohenstadt Hohenstadt (GP))
	ABZWEIG_WALTERTAL = 'de:08117:3506'
	ABZWEIG_WALTERTAL_1 = 'de:08117:3506:0:H1'  # Rtg Kirche (Hohenstadt Hohenstadt (GP))
	ABZWEIG_WALTERTAL_2 = 'de:08117:3506:0:R1'  # Rtg Oberdrackenstein (Hohenstadt Hohenstadt (GP))
	GOSBACH_EINKAUFSZENTRUM = 'de:08117:3510'
	GOSBACH_EINKAUFSZENTRUM_1 = 'de:08117:3510:0:H1'  # Rtg Drackenstein (Bad Ditzenbach Gosbach)
	GOSBACH_EINKAUFSZENTRUM_2 = 'de:08117:3510:0:R1'  # Rtg Lamm (Bad Ditzenbach Gosbach)
	GÖPPINGEN_FEUERWACHE = 'de:08117:4001'
	GÖPPINGEN_FEUERWACHE_1 = 'de:08117:4001:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	GÖPPINGEN_FEUERWACHE_2 = 'de:08117:4001:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_FRIEDRICHSTRASSE = 'de:08117:4002'
	GÖPPINGEN_FRIEDRICHSTRASSE_1 = 'de:08117:4002:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	GÖPPINGEN_FRIEDRICHSTRASSE_2 = 'de:08117:4002:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_CHRISTOPHSTR = 'de:08117:4003'
	GÖPPINGEN_CHRISTOPHSTR_1 = 'de:08117:4003:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	GÖPPINGEN_CHRISTOPHSTR_2 = 'de:08117:4003:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_KUNSTHALLE = 'de:08117:4004'
	GÖPPINGEN_KUNSTHALLE_1 = 'de:08117:4004:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	GÖPPINGEN_KUNSTHALLE_2 = 'de:08117:4004:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	KUNSTHALLE = 'de:08117:4004'
	KUNSTHALLE_1 = 'de:08117:4004:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	KUNSTHALLE_2 = 'de:08117:4004:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPP__EWS_ARENA_NÖRDL_RINGSTR = 'de:08117:4006:0:H1'  # Rtg Mozartstr. (Göppingen Göppingen)
	EWS_ARENA_NÖRDL__RINGSTR = 'de:08117:4006:0:H1'  # Rtg Mozartstr. (Göppingen Göppingen)
	GÖPPINGEN_MOZARTSTR = 'de:08117:4007:0:H1'  # Rtg Bartenb. Rotes Reusch (Göppingen Göppingen)
	ROTES_REUSCH = 'de:08117:4008'
	ROTES_REUSCH_1 = 'de:08117:4008:0:H1'  # Rtg Lorcher Str. (Göppingen Bartenbach (Göppingen))
	ROTES_REUSCH_2 = 'de:08117:4008:0:R1'  # Rtg EWS Arena (Göppingen Bartenbach (Göppingen))
	BARTENBACH_ROTES_REUSCH = 'de:08117:4008'
	BARTENBACH_ROTES_REUSCH_1 = 'de:08117:4008:0:H1'  # Rtg Lorcher Str. (Göppingen Bartenbach (Göppingen))
	BARTENBACH_ROTES_REUSCH_2 = 'de:08117:4008:0:R1'  # Rtg EWS Arena (Göppingen Bartenbach (Göppingen))
	BARTENBACH_RAIFFEISENSTR = 'de:08117:4011:0:R1'  # Rtg Am Meerberg (Göppingen Bartenbach (Göppingen))
	BARTENB__SCHELMENSTR__FRIEDHOF = 'de:08117:4012:0:H1'  # Rtg Im Engenlauch (Göppingen Bartenbach (Göppingen))
	SCHELMENSTRASSE_FRIEDHOF = 'de:08117:4012:0:H1'  # Rtg Im Engenlauch (Göppingen Bartenbach (Göppingen))
	BARTENBACH_IM_ENGENLAUCH = 'de:08117:4013'
	BARTENBACH_IM_ENGENLAUCH_1 = 'de:08117:4013:0:H1'  # Rtg Theater (Göppingen Bartenbach (Göppingen))
	BARTENBACH_IM_ENGENLAUCH_2 = 'de:08117:4013:0:R1'  # Rtg Schurwaldschule (Göppingen Bartenbach (Göppingen))
	IM_ENGENLAUCH = 'de:08117:4013'
	IM_ENGENLAUCH_1 = 'de:08117:4013:0:H1'  # Rtg Theater (Göppingen Bartenbach (Göppingen))
	IM_ENGENLAUCH_2 = 'de:08117:4013:0:R1'  # Rtg Schurwaldschule (Göppingen Bartenbach (Göppingen))
	BARTENBACH_WILHELMSHILFE = 'de:08117:4014'
	BARTENBACH_WILHELMSHILFE_1 = 'de:08117:4014:0:H1'  # Rtg Rechberghausen (Göppingen Bartenbach (Göppingen))
	BARTENBACH_WILHELMSHILFE_2 = 'de:08117:4014:0:R1'  # Rtg Hst Lorch./Lerchen. (Göppingen Bartenbach (Göppingen))
	BARTENBACH_WILHELMSHILFE_3 = 'de:08117:4014:0:R2'  # Rtg Hst. Engenlauch/Schule (Göppingen Bartenbach (Göppingen))
	THEATER_IM_BAHNHOF = 'de:08117:4015'
	THEATER_IM_BAHNHOF_1 = 'de:08117:4015:0:H1'  # Rtg Feuerwehr (Rechberghausen Rechberghausen)
	THEATER_IM_BAHNHOF_2 = 'de:08117:4015:0:R1'  # Rtg Wilhelmshilfe (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_THEATER_IM_BHF = 'de:08117:4015'
	RECHBERGHAUSEN_THEATER_IM_BHF_1 = 'de:08117:4015:0:H1'  # Rtg Feuerwehr (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_THEATER_IM_BHF_2 = 'de:08117:4015:0:R1'  # Rtg Wilhelmshilfe (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_FEUERWEHR = 'de:08117:4016'
	RECHBERGHAUSEN_FEUERWEHR_1 = 'de:08117:4016:0:H1'  # Rtg Schlossmarkt (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_FEUERWEHR_2 = 'de:08117:4016:0:R1'  # Rtg Schurwaldsch. (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_GRUNDSCHULE = 'de:08117:4018'
	RECHBERGHAUSEN_GRUNDSCHULE_1 = 'de:08117:4018:0:H1'  # Rtg Feuerwehr (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_GRUNDSCHULE_2 = 'de:08117:4018:0:R1'  # Rtg Hauffstraße (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_EBENE_SCHULE = 'de:08117:4019:0:H1'  # alle Rtg (Rechberghausen Rechberghausen)
	EBENE_SCHULE = 'de:08117:4019:0:H1'  # alle Rtg (Rechberghausen Rechberghausen)
	RECHBGH__KIRCHPLATZ_RATHAUS = 'de:08117:4020:0:H1'  # Rtg Feuerwehr (Rechberghausen Rechberghausen)
	KIRCHPLATZ_RATHAUS = 'de:08117:4020:0:H1'  # Rtg Feuerwehr (Rechberghausen Rechberghausen)
	ZIEGELSTRASSE_1 = 'de:08117:4021:0:H1'  # Rtg Sonnenbrücke (Rechberghausen Rechberghausen)
	ZIEGELSTRASSE_2 = 'de:08117:6018:0:R1'  # Rtg Baumgarten (Salach Salach)
	RECHBERGHAUSEN_ZIEGELSTRASSE = 'de:08117:4021:0:H1'  # Rtg Sonnenbrücke (Rechberghausen Rechberghausen)
	RECHBGH__GRAF_DEGENFELD_STR = 'de:08117:4024'
	RECHBGH__GRAF_DEGENFELD_STR_1 = 'de:08117:4024:0:H1'  # Rtg Panoramastraße (Rechberghausen Rechberghausen)
	RECHBGH__GRAF_DEGENFELD_STR_2 = 'de:08117:4024:0:R1'  # Rtg Sonnenbrücke (Rechberghausen Rechberghausen)
	GRAF_DEGENFELD_STRASSE = 'de:08117:4024'
	GRAF_DEGENFELD_STRASSE_1 = 'de:08117:4024:0:H1'  # Rtg Panoramastraße (Rechberghausen Rechberghausen)
	GRAF_DEGENFELD_STRASSE_2 = 'de:08117:4024:0:R1'  # Rtg Sonnenbrücke (Rechberghausen Rechberghausen)
	PANORAMASTR = 'de:08117:4025'
	PANORAMASTR_1 = 'de:08117:4025:0:H1'  # Rtg Schlosshofstraße (Rechberghausen Rechberghausen)
	PANORAMASTR_2 = 'de:08117:4025:0:R1'  # Rtg Graf-Degenfeld-Str. (Rechberghausen Rechberghausen)
	RECHBGH__PANORAMASTRASSE = 'de:08117:4025'
	RECHBGH__PANORAMASTRASSE_1 = 'de:08117:4025:0:H1'  # Rtg Schlosshofstraße (Rechberghausen Rechberghausen)
	RECHBGH__PANORAMASTRASSE_2 = 'de:08117:4025:0:R1'  # Rtg Graf-Degenfeld-Str. (Rechberghausen Rechberghausen)
	RECHBGH__SCHLOSSHOFSTRASSE = 'de:08117:4026:0:H1'  # Rtg Panoramastraße (Rechberghausen Rechberghausen)
	SCHLOSSHOFSTRASSE = 'de:08117:4026:0:H1'  # Rtg Panoramastraße (Rechberghausen Rechberghausen)
	SCHLOSSMARKT = 'de:08117:4027'
	SCHLOSSMARKT_1 = 'de:08117:4027:0:H1'  # Rtg Kreuzung Adelberg (Rechberghausen Rechberghausen)
	SCHLOSSMARKT_2 = 'de:08117:4027:0:R1'  # Rtg Feuerwehr (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_SCHLOSSMARKT = 'de:08117:4027'
	RECHBERGHAUSEN_SCHLOSSMARKT_1 = 'de:08117:4027:0:H1'  # Rtg Kreuzung Adelberg (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_SCHLOSSMARKT_2 = 'de:08117:4027:0:R1'  # Rtg Feuerwehr (Rechberghausen Rechberghausen)
	KREUZUNG_ADELBERG = 'de:08117:4028'
	KREUZUNG_ADELBERG_1 = 'de:08117:4028:0:H1'  # Rtg Abzweig Oberwälden (Rechberghausen Rechberghausen)
	KREUZUNG_ADELBERG_2 = 'de:08117:4028:0:R1'  # Rtg Schlossmarkt (Rechberghausen Rechberghausen)
	RECHBGH__KRZG__ADELBERG = 'de:08117:4028'
	RECHBGH__KRZG__ADELBERG_1 = 'de:08117:4028:0:H1'  # Rtg Abzweig Oberwälden (Rechberghausen Rechberghausen)
	RECHBGH__KRZG__ADELBERG_2 = 'de:08117:4028:0:R1'  # Rtg Schlossmarkt (Rechberghausen Rechberghausen)
	ZELL_RECHBERGHÄUSER_STR = 'de:08117:4030'
	ZELL_RECHBERGHÄUSER_STR_1 = 'de:08117:4030:0:H1'  # Rtg Untere Bruck (Börtlingen Börtlingen)
	ZELL_RECHBERGHÄUSER_STR_2 = 'de:08117:4030:0:R1'  # Rtg Rechberghausen (Börtlingen Börtlingen)
	BÖRTL__ZELL_RECHBERGH__STR = 'de:08117:4030'
	BÖRTL__ZELL_RECHBERGH__STR_1 = 'de:08117:4030:0:H1'  # Rtg Untere Bruck (Börtlingen Börtlingen)
	BÖRTL__ZELL_RECHBERGH__STR_2 = 'de:08117:4030:0:R1'  # Rtg Rechberghausen (Börtlingen Börtlingen)
	UNTERE_BRUCK = 'de:08117:4031'
	UNTERE_BRUCK_1 = 'de:08117:4031:0:H1'  # Rtg Kreissparkasse (Börtlingen Börtlingen)
	UNTERE_BRUCK_2 = 'de:08117:4031:0:R1'  # Rtg Zell Rechbergh. Str. (Börtlingen Börtlingen)
	BÖRTLINGEN_UNTERE_BRUCK = 'de:08117:4031'
	BÖRTLINGEN_UNTERE_BRUCK_1 = 'de:08117:4031:0:H1'  # Rtg Kreissparkasse (Börtlingen Börtlingen)
	BÖRTLINGEN_UNTERE_BRUCK_2 = 'de:08117:4031:0:R1'  # Rtg Zell Rechbergh. Str. (Börtlingen Börtlingen)
	BÖRTLINGEN_LÖWEN = 'de:08117:4032:0:R1'  # Rtg Untere Bruck (Börtlingen Börtlingen)
	BÖRTLINGEN_KREISSPARKASSE = 'de:08117:4033'
	BÖRTLINGEN_KREISSPARKASSE_1 = 'de:08117:4033:0:H1'  # Rtg Friedhof (Börtlingen Börtlingen)
	BÖRTLINGEN_KREISSPARKASSE_2 = 'de:08117:4033:0:R1'  # Rtg Löwen (Börtlingen Börtlingen)
	BÖRTLINGEN_FRIEDHOF = 'de:08117:4034'
	BÖRTLINGEN_FRIEDHOF_1 = 'de:08117:4034:0:H1'  # Rtg Industriegebiet (Börtlingen Börtlingen)
	BÖRTLINGEN_FRIEDHOF_2 = 'de:08117:4034:0:R1'  # Rtg Kreissparkasse (Börtlingen Börtlingen)
	BREECH_KRZG__RATTENHARZ = 'de:08117:4035'
	BREECH_KRZG__RATTENHARZ_1 = 'de:08117:4035:0:H1'  # Rtg Breech Wendeplatte (Börtlingen Börtlingen)
	BREECH_KRZG__RATTENHARZ_2 = 'de:08117:4035:0:R1'  # Rtg Industriegebiet (Börtlingen Börtlingen)
	BÖRTL__BREECH_KRZG__RATTENH = 'de:08117:4035'
	BÖRTL__BREECH_KRZG__RATTENH_1 = 'de:08117:4035:0:H1'  # Rtg Breech Wendeplatte (Börtlingen Börtlingen)
	BÖRTL__BREECH_KRZG__RATTENH_2 = 'de:08117:4035:0:R1'  # Rtg Industriegebiet (Börtlingen Börtlingen)
	BÖRTLINGEN_INDUSTRIEGEBIET = 'de:08117:4036'
	BÖRTLINGEN_INDUSTRIEGEBIET_1 = 'de:08117:4036:0:H1'  # Rtg Breech Krzg. Rattenh. (Börtlingen Börtlingen)
	BÖRTLINGEN_INDUSTRIEGEBIET_2 = 'de:08117:4036:0:R1'  # Rtg Friedhof (Börtlingen Börtlingen)
	WANGENER_STRASSE_1 = 'de:08117:4038:0:H1'  # Rtg Ziegelstraße (Rechberghausen Rechberghausen)
	WANGENER_STRASSE_2 = 'de:08117:4908:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	WANGENER_STRASSE_3 = 'de:08117:4908:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	RECHBGH__WANGENER_STRASSE = 'de:08117:4038:0:H1'  # Rtg Ziegelstraße (Rechberghausen Rechberghausen)
	RECHBGH__REICHENHARDTSTRASSE = 'de:08117:4039'
	RECHBGH__REICHENHARDTSTRASSE_1 = 'de:08117:4039:0:H1'  # Rtg Bergstraße (Rechberghausen Rechberghausen)
	RECHBGH__REICHENHARDTSTRASSE_2 = 'de:08117:4039:0:R1'  # Rtg Lindach (Rechberghausen Rechberghausen)
	REICHENHARDTSTRASSE = 'de:08117:4039'
	REICHENHARDTSTRASSE_1 = 'de:08117:4039:0:H1'  # Rtg Bergstraße (Rechberghausen Rechberghausen)
	REICHENHARDTSTRASSE_2 = 'de:08117:4039:0:R1'  # Rtg Lindach (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_BERGSTRASSE = 'de:08117:4040'
	RECHBERGHAUSEN_BERGSTRASSE_1 = 'de:08117:4040:0:H1'  # Rtg Wangener Straße (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_BERGSTRASSE_2 = 'de:08117:4040:0:R1'  # Rtg Reichenhardtstraße (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_SONNENBRÜCKE = 'de:08117:4042'
	RECHBERGHAUSEN_SONNENBRÜCKE_1 = 'de:08117:4042:0:H1'  # Rtg Graf-Degenfeld-Str. (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_SONNENBRÜCKE_2 = 'de:08117:4042:0:R1'  # Rtg Ortsmitte (Rechberghausen Rechberghausen)
	SONNENBRÜCKE = 'de:08117:4042'
	SONNENBRÜCKE_1 = 'de:08117:4042:0:H1'  # Rtg Graf-Degenfeld-Str. (Rechberghausen Rechberghausen)
	SONNENBRÜCKE_2 = 'de:08117:4042:0:R1'  # Rtg Ortsmitte (Rechberghausen Rechberghausen)
	RECHBERGHAUSEN_LINDACH = 'de:08117:4043:0:H1'  # Rtg Maybachstr. (Rechberghausen Rechberghausen)
	LINDACH = 'de:08117:4043:0:H1'  # Rtg Maybachstr. (Rechberghausen Rechberghausen)
	BÖRTL__BREECH_WENDEPLATTE = 'de:08117:4050:0:H1'  # Ankunft (Börtlingen Börtlingen)
	BREECH_WENDEPLATTE = 'de:08117:4050:0:H1'  # Ankunft (Börtlingen Börtlingen)
	GÖPP__EWS_ARENA_LORCHER_STR = 'de:08117:4901'
	GÖPP__EWS_ARENA_LORCHER_STR_1 = 'de:08117:4901:0:H1'  # Rtg Bartenbach (Göppingen Göppingen)
	GÖPP__EWS_ARENA_LORCHER_STR_2 = 'de:08117:4901:0:R1'  # Rtg LRA (Göppingen Göppingen)
	EWS_ARENA_LORCHER_STR = 'de:08117:4901'
	EWS_ARENA_LORCHER_STR_1 = 'de:08117:4901:0:H1'  # Rtg Bartenbach (Göppingen Göppingen)
	EWS_ARENA_LORCHER_STR_2 = 'de:08117:4901:0:R1'  # Rtg LRA (Göppingen Göppingen)
	GÖPPINGEN_LANDRATSAMT = 'de:08117:4902:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_A__SCHWEITZER_SCHULE = 'de:08117:4904:0:R1'  # Rtg Barbarossa-Themen (Göppingen Göppingen)
	ALBERT_SCHWEITZER_SCHULE = 'de:08117:4904:0:R1'  # Rtg Barbarossa-Themen (Göppingen Göppingen)
	REUSCHKIRCHE = 'de:08117:4905'
	REUSCHKIRCHE_1 = 'de:08117:4905:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	REUSCHKIRCHE_2 = 'de:08117:4905:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_REUSCHKIRCHE = 'de:08117:4905'
	GÖPPINGEN_REUSCHKIRCHE_1 = 'de:08117:4905:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	GÖPPINGEN_REUSCHKIRCHE_2 = 'de:08117:4905:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	TURNERSCHAFT_STADION = 'de:08117:4906'
	TURNERSCHAFT_STADION_1 = 'de:08117:4906:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	TURNERSCHAFT_STADION_2 = 'de:08117:4906:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_TURNERSCHAFT_STADION = 'de:08117:4906'
	GÖPPINGEN_TURNERSCHAFT_STADION_1 = 'de:08117:4906:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	GÖPPINGEN_TURNERSCHAFT_STADION_2 = 'de:08117:4906:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_EUGENSTR = 'de:08117:4907'
	GÖPPINGEN_EUGENSTR_1 = 'de:08117:4907:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	GÖPPINGEN_EUGENSTR_2 = 'de:08117:4907:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_WANGENER_STR = 'de:08117:4908'
	GÖPPINGEN_WANGENER_STR_1 = 'de:08117:4908:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	GÖPPINGEN_WANGENER_STR_2 = 'de:08117:4908:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_SCHILLERSTR = 'de:08117:4037'
	GÖPPINGEN_SCHILLERSTR_1 = 'de:08117:4037:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	GÖPPINGEN_SCHILLERSTR_2 = 'de:08117:4037:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_CHRISTKÖNIGSKIRCHE = 'de:08117:4910'
	GÖPPINGEN_CHRISTKÖNIGSKIRCHE_1 = 'de:08117:4910:0:H1'  # Rtg Bartenbach (Göppingen Göppingen)
	GÖPPINGEN_CHRISTKÖNIGSKIRCHE_2 = 'de:08117:4910:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	CHRISTKÖNIGSKIRCHE = 'de:08117:4910'
	CHRISTKÖNIGSKIRCHE_1 = 'de:08117:4910:0:H1'  # Rtg Bartenbach (Göppingen Göppingen)
	CHRISTKÖNIGSKIRCHE_2 = 'de:08117:4910:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	HOFHALDE = 'de:08117:4912'
	HOFHALDE_1 = 'de:08117:4912:0:H1'  # Rtg Sportplatz (Göppingen Bartenbach (Göppingen))
	HOFHALDE_2 = 'de:08117:4912:0:R1'  # Rtg Brunnenstraße (Göppingen Bartenbach (Göppingen))
	BARTENBACH_HOFHALDE = 'de:08117:4912'
	BARTENBACH_HOFHALDE_1 = 'de:08117:4912:0:H1'  # Rtg Sportplatz (Göppingen Bartenbach (Göppingen))
	BARTENBACH_HOFHALDE_2 = 'de:08117:4912:0:R1'  # Rtg Brunnenstraße (Göppingen Bartenbach (Göppingen))
	BARTENBACH_SPORTPLATZ = 'de:08117:4914'
	BARTENBACH_SPORTPLATZ_1 = 'de:08117:4914:0:H1'  # Rtg Göpp. Lerchenbg. (Göppingen Bartenbach (Göppingen))
	BARTENBACH_SPORTPLATZ_2 = 'de:08117:4914:0:R1'  # Rtg Hofhalde (Göppingen Bartenbach (Göppingen))
	GÖPP__LERCHENBG__ORTSMITTE = 'de:08117:4915'
	GÖPP__LERCHENBG__ORTSMITTE_1 = 'de:08117:4915:0:H1'  # Rtg Hohrein Ortsmitte (Göppingen Göppingen)
	GÖPP__LERCHENBG__ORTSMITTE_2 = 'de:08117:4915:0:R1'  # Rtg Bartenbach (Göppingen Göppingen)
	LERCHENBERG_ORTSMITTE = 'de:08117:4915'
	LERCHENBERG_ORTSMITTE_1 = 'de:08117:4915:0:H1'  # Rtg Hohrein Ortsmitte (Göppingen Göppingen)
	LERCHENBERG_ORTSMITTE_2 = 'de:08117:4915:0:R1'  # Rtg Bartenbach (Göppingen Göppingen)
	HOHREIN_ORTSMITTE = 'de:08117:4916:0:R1'  # Rtg Lerchenbg. Ortsmitte (Göppingen Göppingen)
	GÖPP__HOHREIN_ORTSMITTE = 'de:08117:4916:0:R1'  # Rtg Lerchenbg. Ortsmitte (Göppingen Göppingen)
	BARTENBACH_AM_MEERBACH = 'de:08117:4917'
	BARTENBACH_AM_MEERBACH_1 = 'de:08117:4917:0:H1'  # Rtg Hst. Schelmenstr./Friedhof (Göppingen Bartenbach (Göppingen))
	BARTENBACH_AM_MEERBACH_2 = 'de:08117:4917:0:H2'  # Rtg Hst Wilhelmshilfe (Göppingen Bartenbach (Göppingen))
	BARTENBACH_AM_MEERBACH_3 = 'de:08117:4917:0:R1'  # Rtg Göppingen (Göppingen Bartenbach (Göppingen))
	AM_MEERBACH = 'de:08117:4917'
	AM_MEERBACH_1 = 'de:08117:4917:0:H1'  # Rtg Hst. Schelmenstr./Friedhof (Göppingen Bartenbach (Göppingen))
	AM_MEERBACH_2 = 'de:08117:4917:0:H2'  # Rtg Hst Wilhelmshilfe (Göppingen Bartenbach (Göppingen))
	AM_MEERBACH_3 = 'de:08117:4917:0:R1'  # Rtg Göppingen (Göppingen Bartenbach (Göppingen))
	BARTENBACH_BRUNNENSTRASSE = 'de:08117:4918'
	BARTENBACH_BRUNNENSTRASSE_1 = 'de:08117:4918:0:H1'  # Rtg Hofhalde (Göppingen Bartenbach (Göppingen))
	BARTENBACH_BRUNNENSTRASSE_2 = 'de:08117:4918:0:R1'  # Rtg Raiffeisenstr. (Göppingen Bartenbach (Göppingen))
	GÖPPINGEN_HOHENSTÄDTER_WEG = 'de:08117:4920'
	GÖPPINGEN_HOHENSTÄDTER_WEG_1 = 'de:08117:4920:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	GÖPPINGEN_HOHENSTÄDTER_WEG_2 = 'de:08117:4920:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	HOHENSTÄDTER_WEG = 'de:08117:4920'
	HOHENSTÄDTER_WEG_1 = 'de:08117:4920:0:H1'  # Rtg Reusch/Bartenbach (Göppingen Göppingen)
	HOHENSTÄDTER_WEG_2 = 'de:08117:4920:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	FAURNDAU_UHLANDSTR = 'de:08117:5002:0:H1'  # Rtg Panoramastr./Bahnhof (Göppingen Faurndau)
	FAURNDAU_PANORAMASTR__BAHNHOF = 'de:08117:5003'
	FAURNDAU_PANORAMASTR__BAHNHOF_1 = 'de:08117:5003:0:H1'  # Rtg Buchenstr. (Göppingen Faurndau)
	FAURNDAU_PANORAMASTR__BAHNHOF_2 = 'de:08117:5003:0:R1'  # Rtg Hirschplatz (Göppingen Faurndau)
	PANORAMASTRASSE_BAHNHOF = 'de:08117:5003'
	PANORAMASTRASSE_BAHNHOF_1 = 'de:08117:5003:0:H1'  # Rtg Buchenstr. (Göppingen Faurndau)
	PANORAMASTRASSE_BAHNHOF_2 = 'de:08117:5003:0:R1'  # Rtg Hirschplatz (Göppingen Faurndau)
	FAURNDAU_BUCHENSTR = 'de:08117:5004:0:H1'  # Rtg Haier Waldorfsch. (Göppingen Faurndau)
	FAURND__HAIER_WALDORFSCH = 'de:08117:5005:0:H1'  # eRtg Stuttgarter Straße/Elektra (Göppingen Faurndau)
	HAIER_WALDORFSCHULE = 'de:08117:5005:0:H1'  # eRtg Stuttgarter Straße/Elektra (Göppingen Faurndau)
	HAIER_HAIERSCHULE = 'de:08117:5006:0:H1'  # Rtg Reisengebirgstr. (Göppingen Faurndau)
	FAURNDAU_HAIER_HAIERSCHULE = 'de:08117:5006:0:H1'  # Rtg Reisengebirgstr. (Göppingen Faurndau)
	WANGEN_HANGLICH = 'de:08117:5007'
	WANGEN_HANGLICH_1 = 'de:08117:5007:0:H1'  # Rtg Holzhäuser Str. (Wangen Wangen (GP))
	WANGEN_HANGLICH_2 = 'de:08117:5007:0:R1'  # Rtg Siemensstr. (Wangen Wangen (GP))
	HANGLICH = 'de:08117:5007'
	HANGLICH_1 = 'de:08117:5007:0:H1'  # Rtg Holzhäuser Str. (Wangen Wangen (GP))
	HANGLICH_2 = 'de:08117:5007:0:R1'  # Rtg Siemensstr. (Wangen Wangen (GP))
	HOLZHÄUSER_STRASSE = 'de:08117:5008'
	HOLZHÄUSER_STRASSE_1 = 'de:08117:5008:0:H1'  # Rtg Schulstr. (Wangen Wangen (GP))
	HOLZHÄUSER_STRASSE_2 = 'de:08117:5008:0:R1'  # Rtg Hanglich (Wangen Wangen (GP))
	WANGEN_HOLZHÄUSER_STR = 'de:08117:5008'
	WANGEN_HOLZHÄUSER_STR_1 = 'de:08117:5008:0:H1'  # Rtg Schulstr. (Wangen Wangen (GP))
	WANGEN_HOLZHÄUSER_STR_2 = 'de:08117:5008:0:R1'  # Rtg Hanglich (Wangen Wangen (GP))
	WANGEN_SCHULSTR = 'de:08117:5010'
	WANGEN_SCHULSTR_1 = 'de:08117:5010:0:H1'  # Rtg Eichbühl (Wangen Wangen (GP))
	WANGEN_SCHULSTR_2 = 'de:08117:5010:0:R1'  # Rtg Holzhäuser Str. (Wangen Wangen (GP))
	EICHBÜHL = 'de:08117:5011'
	EICHBÜHL_1 = 'de:08117:5011:0:H1'  # Rtg Rechberghausen (Wangen Wangen (GP))
	EICHBÜHL_2 = 'de:08117:5011:0:R1'  # Rtg Wangen/Göppingen (Wangen Wangen (GP))
	WANGEN_EICHBÜHL = 'de:08117:5011'
	WANGEN_EICHBÜHL_1 = 'de:08117:5011:0:H1'  # Rtg Rechberghausen (Wangen Wangen (GP))
	WANGEN_EICHBÜHL_2 = 'de:08117:5011:0:R1'  # Rtg Wangen/Göppingen (Wangen Wangen (GP))
	OBERWÄLDEN_UHLANDSTRASSE = 'de:08117:5013'
	OBERWÄLDEN_UHLANDSTRASSE_1 = 'de:08117:5013:0:H1'  # Rtg Backhausen (Wangen Oberwälden)
	OBERWÄLDEN_UHLANDSTRASSE_2 = 'de:08117:5013:0:R1'  # Rtg Rechberghausen (Wangen Oberwälden)
	OBERWÄLDEN_BACKHAUS = 'de:08117:5015:0:H1'  # Rtg Uhlandstraße (Wangen Oberwälden)
	WANGEN_ABZW__OBERWÄLDEN = 'de:08117:5016'
	WANGEN_ABZW__OBERWÄLDEN_1 = 'de:08117:5016:0:H1'  # Rtg Zachersmühle (Wangen Wangen (GP))
	WANGEN_ABZW__OBERWÄLDEN_2 = 'de:08117:5016:0:R1'  # Rtg Kreuzung Adelberg (Wangen Wangen (GP))
	ABZWEIG_OBERWÄLDEN = 'de:08117:5016'
	ABZWEIG_OBERWÄLDEN_1 = 'de:08117:5016:0:H1'  # Rtg Zachersmühle (Wangen Wangen (GP))
	ABZWEIG_OBERWÄLDEN_2 = 'de:08117:5016:0:R1'  # Rtg Kreuzung Adelberg (Wangen Wangen (GP))
	SPARWIESEN_GEWERBEGEBIET = 'de:08117:5020'
	SPARWIESEN_GEWERBEGEBIET_1 = 'de:08117:5020:0:H1'  # Rtg Schopflenberg (Uhingen Sparwiesen)
	SPARWIESEN_GEWERBEGEBIET_2 = 'de:08117:5020:0:R1'  # Rtg Uhingen (Uhingen Sparwiesen)
	SCHLOSS_FILSECK_1 = 'de:08117:15021:0:2'  # Rtg Faurndau (Göppingen Faurndau)
	SCHLOSS_FILSECK_2 = 'de:08117:5021:0:H1'  # Rtg Sparwiesen (Göppingen Faurndau)
	FAURNDAU_SCHLOSS_FILSECK_1 = 'de:08117:15021:0:2'  # Rtg Faurndau (Göppingen Faurndau)
	FAURNDAU_SCHLOSS_FILSECK_2 = 'de:08117:5021:0:H1'  # Rtg Sparwiesen (Göppingen Faurndau)
	EICHENHOF = 'de:08117:6003'
	EICHENHOF_1 = 'de:08117:6003:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	EICHENHOF_2 = 'de:08117:6003:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__EICHENHOF = 'de:08117:6003'
	EISLINGEN__F__EICHENHOF_1 = 'de:08117:6003:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	EISLINGEN__F__EICHENHOF_2 = 'de:08117:6003:0:R1'  # Rtg Göppingen (Eislingen/Fils Eislingen (F))
	SALACH_MARKTPLATZ = 'de:08117:6004'
	SALACH_MARKTPLATZ_1 = 'de:08117:6004:0:H1'  # Rtg Süßen/Ottenbach (Salach Salach)
	SALACH_MARKTPLATZ_2 = 'de:08117:6004:0:R1'  # Rtg Eislingen/Göppingen (Salach Salach)
	SALACH_EISLINGER_STR = 'de:08117:6007'
	SALACH_EISLINGER_STR_1 = 'de:08117:6007:0:H1'  # Rtg Salach (Salach Salach)
	SALACH_EISLINGER_STR_2 = 'de:08117:6007:0:R1'  # Rtg Eislingen (Salach Salach)
	EISLINGER_STRASSE = 'de:08117:6007'
	EISLINGER_STRASSE_1 = 'de:08117:6007:0:H1'  # Rtg Salach (Salach Salach)
	EISLINGER_STRASSE_2 = 'de:08117:6007:0:R1'  # Rtg Eislingen (Salach Salach)
	EISL__F__KRUMMW__BRÜCKENSTR = 'de:08117:6010'
	EISL__F__KRUMMW__BRÜCKENSTR_1 = 'de:08117:6010:0:H1'  # Rtg Krummw. Schwarzeng. (Eislingen/Fils Eislingen (F))
	EISL__F__KRUMMW__BRÜCKENSTR_2 = 'de:08117:6010:0:R1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	KRUMMWÄLDEN_BRÜCKENSTR = 'de:08117:6010'
	KRUMMWÄLDEN_BRÜCKENSTR_1 = 'de:08117:6010:0:H1'  # Rtg Krummw. Schwarzeng. (Eislingen/Fils Eislingen (F))
	KRUMMWÄLDEN_BRÜCKENSTR_2 = 'de:08117:6010:0:R1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	EISL__F__KRUMMW__SCHWARZENG = 'de:08117:6011'
	EISL__F__KRUMMW__SCHWARZENG_1 = 'de:08117:6011:0:H1'  # Rtg Krzg. Eschenbäche (Eislingen/Fils Eislingen (F))
	EISL__F__KRUMMW__SCHWARZENG_2 = 'de:08117:6011:0:R1'  # Rtg Krummw. Brückenstr. (Eislingen/Fils Eislingen (F))
	KRUMMW__SCHWARZENGASSE = 'de:08117:6011'
	KRUMMW__SCHWARZENGASSE_1 = 'de:08117:6011:0:H1'  # Rtg Krzg. Eschenbäche (Eislingen/Fils Eislingen (F))
	KRUMMW__SCHWARZENGASSE_2 = 'de:08117:6011:0:R1'  # Rtg Krummw. Brückenstr. (Eislingen/Fils Eislingen (F))
	KREUZUNG_ESCHENBÄCHE = 'de:08117:6012'
	KREUZUNG_ESCHENBÄCHE_1 = 'de:08117:6012:0:H1'  # Rtg Ottenbach (Eislingen/Fils Eislingen (F))
	KREUZUNG_ESCHENBÄCHE_2 = 'de:08117:6012:0:R1'  # Rtg Krummw. Schwarzeng. (Eislingen/Fils Eislingen (F))
	EISL__F__KRZG__ESCHENBÄCHE = 'de:08117:6012'
	EISL__F__KRZG__ESCHENBÄCHE_1 = 'de:08117:6012:0:H1'  # Rtg Ottenbach (Eislingen/Fils Eislingen (F))
	EISL__F__KRZG__ESCHENBÄCHE_2 = 'de:08117:6012:0:R1'  # Rtg Krummw. Schwarzeng. (Eislingen/Fils Eislingen (F))
	BREITFELDERHOF = 'de:08117:6013'
	BREITFELDERHOF_1 = 'de:08117:6013:0:H1'  # Rtg Siedlung (Ottenbach Ottenbach)
	BREITFELDERHOF_2 = 'de:08117:6013:0:R1'  # Rtg Eisl.(F) (Ottenbach Ottenbach)
	OTTENBACH_BREITFELDERHOF = 'de:08117:6013'
	OTTENBACH_BREITFELDERHOF_1 = 'de:08117:6013:0:H1'  # Rtg Siedlung (Ottenbach Ottenbach)
	OTTENBACH_BREITFELDERHOF_2 = 'de:08117:6013:0:R1'  # Rtg Eisl.(F) (Ottenbach Ottenbach)
	OTTENBACH_SIEDLUNG = 'de:08117:6014'
	OTTENBACH_SIEDLUNG_1 = 'de:08117:6014:0:H1'  # Rtg Gasthaus Krone (Ottenbach Ottenbach)
	OTTENBACH_SIEDLUNG_2 = 'de:08117:6014:0:R1'  # Rtg Breitfelderhof (Ottenbach Ottenbach)
	OTTENBACH_KITZENER_STR = 'de:08117:6015:0:R1'  # Ankunft (Ottenbach Ottenbach)
	KITZENER_STRASSE = 'de:08117:6015:0:R1'  # Ankunft (Ottenbach Ottenbach)
	OTTENBACH_ADLERSTR = 'de:08117:6016:0:R1'  # Rtg Kitzener Str. (Ottenbach Ottenbach)
	OTTENBACH_GASTHAUS_KRONE = 'de:08117:6017:0:H1'  # Rtg Adlerstr. (Ottenbach Ottenbach)
	GASTHAUS_KRONE = 'de:08117:6017:0:H1'  # Rtg Adlerstr. (Ottenbach Ottenbach)
	SALACH_ZIEGELSTR = 'de:08117:6018:0:R1'  # Rtg Baumgarten (Salach Salach)
	SALACH_BAUMGARTEN = 'de:08117:6019'
	SALACH_BAUMGARTEN_1 = 'de:08117:6019:0:H1'  # Rtg Ottenbach (Salach Salach)
	SALACH_BAUMGARTEN_2 = 'de:08117:6019:0:R1'  # Rtg Eislingen (Salach Salach)
	BAUMGARTEN = 'de:08117:6019'
	BAUMGARTEN_1 = 'de:08117:6019:0:H1'  # Rtg Ottenbach (Salach Salach)
	BAUMGARTEN_2 = 'de:08117:6019:0:R1'  # Rtg Eislingen (Salach Salach)
	SALACH_RINGSTR = 'de:08117:6021'
	SALACH_RINGSTR_1 = 'de:08117:6021:0:H1'  # Rtg Ottenbach (Salach Salach)
	SALACH_RINGSTR_2 = 'de:08117:6021:0:R1'  # Rtg Eislingen (Salach Salach)
	BRÜHLWEG = 'de:08117:6022'
	BRÜHLWEG_1 = 'de:08117:6022:0:H1'  # Rtg Ottenbach (Salach Salach)
	BRÜHLWEG_2 = 'de:08117:6022:0:R1'  # Rtg Eislingen (Salach Salach)
	SALACH_BRÜHLWEG = 'de:08117:6022'
	SALACH_BRÜHLWEG_1 = 'de:08117:6022:0:H1'  # Rtg Ottenbach (Salach Salach)
	SALACH_BRÜHLWEG_2 = 'de:08117:6022:0:R1'  # Rtg Eislingen (Salach Salach)
	CHRISTOPHSBAD = 'de:08117:7002'
	CHRISTOPHSBAD_1 = 'de:08117:7002:0:H1'  # Rtg Faurndau (Göppingen Göppingen)
	CHRISTOPHSBAD_2 = 'de:08117:7002:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPP__CHRISTOPHSBAD = 'de:08117:7002'
	GÖPP__CHRISTOPHSBAD_1 = 'de:08117:7002:0:H1'  # Rtg Faurndau (Göppingen Göppingen)
	GÖPP__CHRISTOPHSBAD_2 = 'de:08117:7002:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_PFINGSTWASEN = 'de:08117:7003'
	GÖPPINGEN_PFINGSTWASEN_1 = 'de:08117:7003:0:H1'  # Rtg Faurndau (Göppingen Göppingen)
	GÖPPINGEN_PFINGSTWASEN_2 = 'de:08117:7003:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	PFINGSTWASEN = 'de:08117:7003'
	PFINGSTWASEN_1 = 'de:08117:7003:0:H1'  # Rtg Faurndau (Göppingen Göppingen)
	PFINGSTWASEN_2 = 'de:08117:7003:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	FAURNDAU_MARIENSTR = 'de:08117:7006'
	FAURNDAU_MARIENSTR_1 = 'de:08117:7006:0:H1'  # Rtg Hirschstr. (Göppingen Faurndau)
	FAURNDAU_MARIENSTR_2 = 'de:08117:7006:0:R1'  # Rtg Ludwigstr. (Göppingen Faurndau)
	SAUERBRUNNEN = 'de:08117:7007:0:H1'  # Rtg Filseckstr. (Göppingen Faurndau)
	FAURNDAU_SAUERBRUNNEN = 'de:08117:7007:0:H1'  # Rtg Filseckstr. (Göppingen Faurndau)
	HIRSCHPLATZ = 'de:08117:7008:0:H1'  # Rtg Marienplatz (Göppingen Faurndau)
	FAURNDAU_HIRSCHPLATZ = 'de:08117:7008:0:H1'  # Rtg Marienplatz (Göppingen Faurndau)
	FAURNDAU_HIRSCHSTR = 'de:08117:7009'
	FAURNDAU_HIRSCHSTR_1 = 'de:08117:7009:0:H1'  # Rtg Sauerbrunnen (Göppingen Faurndau)
	FAURNDAU_HIRSCHSTR_2 = 'de:08117:7009:0:R1'  # Rtg Marienstr. (Göppingen Faurndau)
	FILSECKSTRASSE = 'de:08117:7010'
	FILSECKSTRASSE_1 = 'de:08117:7010:0:H1'  # Rtg Schloss Fleck (Göppingen Faurndau)
	FILSECKSTRASSE_2 = 'de:08117:7010:0:R1'  # Rtg Hirschstr. (Göppingen Faurndau)
	FAURNDAU_FILSECKSTR = 'de:08117:7010'
	FAURNDAU_FILSECKSTR_1 = 'de:08117:7010:0:H1'  # Rtg Schloss Fleck (Göppingen Faurndau)
	FAURNDAU_FILSECKSTR_2 = 'de:08117:7010:0:R1'  # Rtg Hirschstr. (Göppingen Faurndau)
	FAURNDAU_ILTISWEG = 'de:08117:7001:0:H1'  # Rtg Kesseltobelstr. (Göppingen Faurndau)
	ILTISWEG = 'de:08117:7001:0:H1'  # Rtg Kesseltobelstr. (Göppingen Faurndau)
	SPARWIESEN_ALBSTRASSE = 'de:08117:7013'
	SPARWIESEN_ALBSTRASSE_1 = 'de:08117:7013:0:H1'  # Rtg Schopflenberg (Uhingen Sparwiesen)
	SPARWIESEN_ALBSTRASSE_2 = 'de:08117:7013:0:R1'  # Rtg Uhingen (Uhingen Sparwiesen)
	SPARWIESEN_MARKTPLATZ = 'de:08117:7014'
	SPARWIESEN_MARKTPLATZ_1 = 'de:08117:7014:0:H1'  # Rtg Schopflenberg (Uhingen Sparwiesen)
	SPARWIESEN_MARKTPLATZ_2 = 'de:08117:7014:0:R1'  # Rtg Uhingen (Uhingen Sparwiesen)
	FAURNDAU_SCHUBARTSTR = 'de:08117:7015'
	FAURNDAU_SCHUBARTSTR_1 = 'de:08117:7015:0:H1'  # Rtg Ludwigstr. (Göppingen Faurndau)
	FAURNDAU_SCHUBARTSTR_2 = 'de:08117:7015:0:R1'  # Rtg Göppingen (Göppingen Faurndau)
	FAURNDAU_LUDWIGSTR = 'de:08117:7016'
	FAURNDAU_LUDWIGSTR_1 = 'de:08117:7016:0:H1'  # Rtg Marienstr. (Göppingen Faurndau)
	FAURNDAU_LUDWIGSTR_2 = 'de:08117:7016:0:R1'  # Rtg Schubartstr. (Göppingen Faurndau)
	WÄSCHENBEUREN_WIESENTAL = 'de:08117:7017'
	WÄSCHENBEUREN_WIESENTAL_1 = 'de:08117:7017:0:H1'  # Rtg Schw. Gmünd (Wäschenbeuren Wäschenbeuren)
	WÄSCHENBEUREN_WIESENTAL_2 = 'de:08117:7017:0:R1'  # Rtg Göppingen (Wäschenbeuren Wäschenbeuren)
	WIESENTAL = 'de:08117:7017'
	WIESENTAL_1 = 'de:08117:7017:0:H1'  # Rtg Schw. Gmünd (Wäschenbeuren Wäschenbeuren)
	WIESENTAL_2 = 'de:08117:7017:0:R1'  # Rtg Göppingen (Wäschenbeuren Wäschenbeuren)
	ROSSWÄLDEN_DORFSTRASSE = 'de:08117:7403'
	ROSSWÄLDEN_DORFSTRASSE_1 = 'de:08117:7403:0:H1'  # Rtg Schlierbach (Ebersbach an der Fils Roßwälden)
	ROSSWÄLDEN_DORFSTRASSE_2 = 'de:08117:7403:0:R1'  # Rtg Ebersbach (Ebersbach an der Fils Roßwälden)
	SCHLIERBACH_EBERSBACHER_STR = 'de:08117:7404'
	SCHLIERBACH_EBERSBACHER_STR_1 = 'de:08117:7404:0:H1'  # Rtg Birkhöfe (Schlierbach Schlierbach)
	SCHLIERBACH_EBERSBACHER_STR_2 = 'de:08117:7404:0:R1'  # Rtg Kirche (Schlierbach Schlierbach)
	EBERSBACHER_STRASSE = 'de:08117:7404'
	EBERSBACHER_STRASSE_1 = 'de:08117:7404:0:H1'  # Rtg Birkhöfe (Schlierbach Schlierbach)
	EBERSBACHER_STRASSE_2 = 'de:08117:7404:0:R1'  # Rtg Kirche (Schlierbach Schlierbach)
	ROSSWÄLDEN_KINDERGARTEN = 'de:08117:7405:0:R1'  # Rtg Kirche (Ebersbach an der Fils Roßwälden)
	ROSSWÄLDEN_KIRCHE = 'de:08117:7406'
	ROSSWÄLDEN_KIRCHE_1 = 'de:08117:7406:0:H1'  # Rtg Schlierbach (Ebersbach an der Fils Roßwälden)
	ROSSWÄLDEN_KIRCHE_2 = 'de:08117:7406:0:R1'  # Rtg Ebersbach (Ebersbach an der Fils Roßwälden)
	WEILER__F__DREIECK = 'de:08117:7407'
	WEILER__F__DREIECK_1 = 'de:08117:7407:0:H1'  # Rtg Roßwälden (Ebersbach an der Fils Weiler (Ebersbach))
	WEILER__F__DREIECK_2 = 'de:08117:7407:0:R1'  # Rtg Ebersbach (Ebersbach an der Fils Weiler (Ebersbach))
	DREIECK = 'de:08117:7407'
	DREIECK_1 = 'de:08117:7407:0:H1'  # Rtg Roßwälden (Ebersbach an der Fils Weiler (Ebersbach))
	DREIECK_2 = 'de:08117:7407:0:R1'  # Rtg Ebersbach (Ebersbach an der Fils Weiler (Ebersbach))
	ROSSWÄLDEN_IN_DEN_MESSENWIESEN = 'de:08117:7408:0:H1'  # Rtg Birkhöfe (Ebersbach an der Fils Roßwälden)
	IN_DEN_MESSENWIESEN = 'de:08117:7408:0:H1'  # Rtg Birkhöfe (Ebersbach an der Fils Roßwälden)
	FILSBRÜCKE = 'de:08117:7409'
	FILSBRÜCKE_1 = 'de:08117:7409:0:5'  # Rtg Ebersbach (Kirchheimer Straße) (Ebersbach an der Fils Ebersbach (F))
	FILSBRÜCKE_2 = 'de:08117:7409:0:H1'  # Rtg Schulz. Raichberg (Ebersbach an der Fils Ebersbach (F))
	FILSBRÜCKE_3 = 'de:08117:7409:0:R1'  # Rtg Stadtmitte (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__FILSBRÜCKE = 'de:08117:7409'
	EBERSB__F__FILSBRÜCKE_1 = 'de:08117:7409:0:5'  # Rtg Ebersbach (Kirchheimer Straße) (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__FILSBRÜCKE_2 = 'de:08117:7409:0:H1'  # Rtg Schulz. Raichberg (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__FILSBRÜCKE_3 = 'de:08117:7409:0:R1'  # Rtg Stadtmitte (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__HARDTSCHULE = 'de:08117:7411:0:H1'  # Rtg Cafe Krieg (Ebersbach an der Fils Ebersbach (F))
	HARDTSCHULE = 'de:08117:7411:0:H1'  # Rtg Cafe Krieg (Ebersbach an der Fils Ebersbach (F))
	EBERSBACH__F__RATHAUS = 'de:08117:7412:0:R1'  # Rtg Filsbrücke (Ebersbach an der Fils Ebersbach (F))
	BIRKHÖFE = 'de:08117:7414'
	BIRKHÖFE_1 = 'de:08117:7414:0:H1'  # Rtg Ebersbach (Ebersbach an der Fils Roßwälden)
	BIRKHÖFE_2 = 'de:08117:7414:0:R1'  # Rtg Schlierbach (Ebersbach an der Fils Roßwälden)
	ROSSWÄLDEN_BIRKHÖFE = 'de:08117:7414'
	ROSSWÄLDEN_BIRKHÖFE_1 = 'de:08117:7414:0:H1'  # Rtg Ebersbach (Ebersbach an der Fils Roßwälden)
	ROSSWÄLDEN_BIRKHÖFE_2 = 'de:08117:7414:0:R1'  # Rtg Schlierbach (Ebersbach an der Fils Roßwälden)
	SÜSSEN_GEORG_ELSER_WEG = 'de:08117:7600:0:3'  # Rtg Rabenwiesen (Süßen Süßen)
	GEORG_ELSER_WEG = 'de:08117:7600:0:3'  # Rtg Rabenwiesen (Süßen Süßen)
	FAHRZEUGWERKE = 'de:08117:7602'
	FAHRZEUGWERKE_1 = 'de:08117:7602:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	FAHRZEUGWERKE_2 = 'de:08117:7602:0:R1'  # Rtg Eislingen/Göppingen (Eislingen/Fils Eislingen (F))
	EISL__F__FAHRZEUGWERKE = 'de:08117:7602'
	EISL__F__FAHRZEUGWERKE_1 = 'de:08117:7602:0:H1'  # Rtg Salach (Eislingen/Fils Eislingen (F))
	EISL__F__FAHRZEUGWERKE_2 = 'de:08117:7602:0:R1'  # Rtg Eislingen/Göppingen (Eislingen/Fils Eislingen (F))
	QUERSTRASSE = 'de:08117:7605'
	QUERSTRASSE_1 = 'de:08117:7605:0:H1'  # Rtg Fuchseckstr. (Salach Salach)
	QUERSTRASSE_2 = 'de:08117:7605:0:R1'  # Rtg Eislinger Str (Salach Salach)
	SALACH_QUERSTR = 'de:08117:7605'
	SALACH_QUERSTR_1 = 'de:08117:7605:0:H1'  # Rtg Fuchseckstr. (Salach Salach)
	SALACH_QUERSTR_2 = 'de:08117:7605:0:R1'  # Rtg Eislinger Str (Salach Salach)
	DUGENDORF = 'de:08117:7607'
	DUGENDORF_1 = 'de:08117:7607:0:H1'  # Rtg Süßen (Salach Salach)
	DUGENDORF_2 = 'de:08117:7607:0:R1'  # Rtg Eislingen (Salach Salach)
	SALACH_DUGENDORF = 'de:08117:7607'
	SALACH_DUGENDORF_1 = 'de:08117:7607:0:H1'  # Rtg Süßen (Salach Salach)
	SALACH_DUGENDORF_2 = 'de:08117:7607:0:R1'  # Rtg Eislingen (Salach Salach)
	BRIEFZENTRUM = 'de:08117:7608'
	BRIEFZENTRUM_1 = 'de:08117:7608:0:H1'  # Rtg Süßen (Salach Salach)
	BRIEFZENTRUM_2 = 'de:08117:7608:0:R1'  # Rtg Eislingen (Salach Salach)
	SALACH_BRIEFZENTRUM = 'de:08117:7608'
	SALACH_BRIEFZENTRUM_1 = 'de:08117:7608:0:H1'  # Rtg Süßen (Salach Salach)
	SALACH_BRIEFZENTRUM_2 = 'de:08117:7608:0:R1'  # Rtg Eislingen (Salach Salach)
	RABENWIESEN = 'de:08117:7609:0:H1'  # Rtg Schlater Str. (Süßen Süßen)
	SÜSSEN_RABENWIESEN = 'de:08117:7609:0:H1'  # Rtg Schlater Str. (Süßen Süßen)
	SÜSSEN_KAPELLENSTR = 'de:08117:7610:0:H1'  # Rtg Stiegelwiese (Süßen Süßen)
	TOBELKREUZUNG = 'de:08117:7611:0:R1'  # Rtg Dugendorf (Süßen Süßen)
	SÜSSEN_TOBELKREUZUNG = 'de:08117:7611:0:R1'  # Rtg Dugendorf (Süßen Süßen)
	SÜSSEN_STIEGELWIESE = 'de:08117:7612:0:R1'  # Rtg Fuchseckstr. (Süßen Süßen)
	STIEGELWIESE = 'de:08117:7612:0:R1'  # Rtg Fuchseckstr. (Süßen Süßen)
	SÜSSEN_FUCHSECKSTR = 'de:08117:7613'
	SÜSSEN_FUCHSECKSTR_1 = 'de:08117:7613:0:H1'  # Rtg Schlater Str. (Süßen Süßen)
	SÜSSEN_FUCHSECKSTR_2 = 'de:08117:7613:0:R1'  # Ankunft (Süßen Süßen)
	DONZDORF_SUDETENSTR = 'de:08117:7614'
	DONZDORF_SUDETENSTR_1 = 'de:08117:7614:0:H1'  # Rtg Marrenstraße (Donzdorf Donzdorf)
	DONZDORF_SUDETENSTR_2 = 'de:08117:7614:0:R1'  # Rtg Adolf-Kolping-Str (Donzdorf Donzdorf)
	SÜSSEN_SCHLATER_STR = 'de:08117:7615'
	SÜSSEN_SCHLATER_STR_1 = 'de:08117:7615:0:H1'  # Rtg Göppingen (Süßen Süßen)
	SÜSSEN_SCHLATER_STR_2 = 'de:08117:7615:0:R1'  # Rtg Fuchseckstr. (Süßen Süßen)
	SCHLATER_STRASSE = 'de:08117:7615'
	SCHLATER_STRASSE_1 = 'de:08117:7615:0:H1'  # Rtg Göppingen (Süßen Süßen)
	SCHLATER_STRASSE_2 = 'de:08117:7615:0:R1'  # Rtg Fuchseckstr. (Süßen Süßen)
	SÜSSEN_RATHAUS = 'de:08117:7616'
	SÜSSEN_RATHAUS_1 = 'de:08117:7616:0:H1'  # Rtg Süßen Bf (Süßen Süßen)
	SÜSSEN_RATHAUS_2 = 'de:08117:7616:0:R1'  # Rtg stadtauswärts (Süßen Süßen)
	SÜSSEN_UMSPANNWERK = 'de:08117:7618'
	SÜSSEN_UMSPANNWERK_1 = 'de:08117:7618:0:H1'  # Rtg Donzdorf (Süßen Süßen)
	SÜSSEN_UMSPANNWERK_2 = 'de:08117:7618:0:R1'  # Rtg Süßen Bf (Süßen Süßen)
	UMSPANNWERK = 'de:08117:7618'
	UMSPANNWERK_1 = 'de:08117:7618:0:H1'  # Rtg Donzdorf (Süßen Süßen)
	UMSPANNWERK_2 = 'de:08117:7618:0:R1'  # Rtg Süßen Bf (Süßen Süßen)
	DONZDORF_BAHNHOF = 'de:08117:7619'
	DONZDORF_BAHNHOF_1 = 'de:08117:7619:1:H1'  # Rtg Lauterstein (Donzdorf Donzdorf)
	DONZDORF_BAHNHOF_2 = 'de:08117:7619:1:R1'  # Rtg Süßen (Donzdorf Donzdorf)
	ABZWEIG_REICHENBACH = 'de:08117:7620'
	ABZWEIG_REICHENBACH_1 = 'de:08117:7620:0:H1'  # Rtg Bahnhof (Donzdorf Donzdorf)
	ABZWEIG_REICHENBACH_2 = 'de:08117:7620:0:R1'  # Rtg Süßen (Donzdorf Donzdorf)
	DONZDORF_ABZW__REICHENBACH = 'de:08117:7620'
	DONZDORF_ABZW__REICHENBACH_1 = 'de:08117:7620:0:H1'  # Rtg Bahnhof (Donzdorf Donzdorf)
	DONZDORF_ABZW__REICHENBACH_2 = 'de:08117:7620:0:R1'  # Rtg Süßen (Donzdorf Donzdorf)
	MARRENSTRASSE = 'de:08117:7621'
	MARRENSTRASSE_1 = 'de:08117:7621:0:H1'  # Rtg Abz. Reichenbach (Donzdorf Donzdorf)
	MARRENSTRASSE_2 = 'de:08117:7621:0:R1'  # Rtg Sudentenstraße (Donzdorf Donzdorf)
	DONZDORF_MARRENSTR = 'de:08117:7621'
	DONZDORF_MARRENSTR_1 = 'de:08117:7621:0:H1'  # Rtg Abz. Reichenbach (Donzdorf Donzdorf)
	DONZDORF_MARRENSTR_2 = 'de:08117:7621:0:R1'  # Rtg Sudentenstraße (Donzdorf Donzdorf)
	DONZDORF_STADTHALLE = 'de:08117:7622'
	DONZDORF_STADTHALLE_1 = 'de:08117:7622:0:2'  # Rtg R1 Bahnhof (Donzdorf Donzdorf)
	DONZDORF_STADTHALLE_2 = 'de:08117:7622:0:H1'  # Rtg Schulzentrum / Bahnhof (Donzdorf Donzdorf)
	DONZDORF_SCHULZENTRUM = 'de:08117:7623:0:H1'  # Rtg Steingartenschule (Donzdorf Donzdorf)
	LAUTERGARTEN = 'de:08117:7624'
	LAUTERGARTEN_1 = 'de:08117:7624:0:H1'  # Rtg Abzw. Winzingen (Donzdorf Donzdorf)
	LAUTERGARTEN_2 = 'de:08117:7624:0:R1'  # Rtg Stadthalle/Schulzentrum (Donzdorf Donzdorf)
	DONZDORF_LAUTERGARTEN = 'de:08117:7624'
	DONZDORF_LAUTERGARTEN_1 = 'de:08117:7624:0:H1'  # Rtg Abzw. Winzingen (Donzdorf Donzdorf)
	DONZDORF_LAUTERGARTEN_2 = 'de:08117:7624:0:R1'  # Rtg Stadthalle/Schulzentrum (Donzdorf Donzdorf)
	DONZDORF_ABZW__WINZINGEN = 'de:08117:7625'
	DONZDORF_ABZW__WINZINGEN_1 = 'de:08117:7625:0:H1'  # Rtg Hagenbuch (Donzdorf Donzdorf)
	DONZDORF_ABZW__WINZINGEN_2 = 'de:08117:7625:0:R1'  # Rtg Lautergarten (Donzdorf Donzdorf)
	ABZWEIG_WINZINGEN = 'de:08117:7625'
	ABZWEIG_WINZINGEN_1 = 'de:08117:7625:0:H1'  # Rtg Hagenbuch (Donzdorf Donzdorf)
	ABZWEIG_WINZINGEN_2 = 'de:08117:7625:0:R1'  # Rtg Lautergarten (Donzdorf Donzdorf)
	HAGENBUCH = 'de:08117:7626'
	HAGENBUCH_1 = 'de:08117:7626:0:H1'  # Rtg Winzingen (Donzdorf Donzdorf)
	HAGENBUCH_2 = 'de:08117:7626:0:R1'  # Rtg Abz Winzingen (Donzdorf Donzdorf)
	DONZDORF_HAGENBUCH = 'de:08117:7626'
	DONZDORF_HAGENBUCH_1 = 'de:08117:7626:0:H1'  # Rtg Winzingen (Donzdorf Donzdorf)
	DONZDORF_HAGENBUCH_2 = 'de:08117:7626:0:R1'  # Rtg Abz Winzingen (Donzdorf Donzdorf)
	REICHENBACH_U_R__RATHAUS = 'de:08117:7631:0:H1'  # Rtg Turnhalle (Donzdorf Reichenbach unter Rechberg (Donzdorf))
	DONZDORF_ADOLF_KOLPING_STR = 'de:08117:7632'
	DONZDORF_ADOLF_KOLPING_STR_1 = 'de:08117:7632:0:H1'  # Rtg Sudentenstr (Donzdorf Donzdorf)
	DONZDORF_ADOLF_KOLPING_STR_2 = 'de:08117:7632:0:R1'  # Rtg Abz Reichenbach (Donzdorf Donzdorf)
	ADOLF_KOLPING_STRASSE = 'de:08117:7632'
	ADOLF_KOLPING_STRASSE_1 = 'de:08117:7632:0:H1'  # Rtg Sudentenstr (Donzdorf Donzdorf)
	ADOLF_KOLPING_STRASSE_2 = 'de:08117:7632:0:R1'  # Rtg Abz Reichenbach (Donzdorf Donzdorf)
	EBERSBACH__F__STADTBIBLIOTHEK = 'de:08117:7637'
	EBERSBACH__F__STADTBIBLIOTHEK_1 = 'de:08117:7637:0:H1'  # Rtg Büchenbronner Str. (Ebersbach an der Fils Ebersbach (F))
	EBERSBACH__F__STADTBIBLIOTHEK_2 = 'de:08117:7637:0:R1'  # Rtg Hardtschule (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__BÜCHENBR__GEMEINDEH = 'de:08117:7638'
	EBERSB__F__BÜCHENBR__GEMEINDEH_1 = 'de:08117:7638:0:H1'  # Rtg Forsthaus (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__BÜCHENBR__GEMEINDEH_2 = 'de:08117:7638:0:R1'  # Rtg Siedlung (Ebersbach an der Fils Ebersbach (F))
	BÜCHENBRONN_GEMEINDEHAUS = 'de:08117:7638'
	BÜCHENBRONN_GEMEINDEHAUS_1 = 'de:08117:7638:0:H1'  # Rtg Forsthaus (Ebersbach an der Fils Ebersbach (F))
	BÜCHENBRONN_GEMEINDEHAUS_2 = 'de:08117:7638:0:R1'  # Rtg Siedlung (Ebersbach an der Fils Ebersbach (F))
	KRAPFENREUT = 'de:08117:7639'
	KRAPFENREUT_1 = 'de:08117:7639:0:H1'  # Rtg Forsthaus/Norden (Ebersbach an der Fils Ebersbach (F))
	KRAPFENREUT_2 = 'de:08117:7639:0:R1'  # Rtg Süden (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__KRAPFENREUT = 'de:08117:7639'
	EBERSB__F__KRAPFENREUT_1 = 'de:08117:7639:0:H1'  # Rtg Forsthaus/Norden (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__KRAPFENREUT_2 = 'de:08117:7639:0:R1'  # Rtg Süden (Ebersbach an der Fils Ebersbach (F))
	RAUHWIESENSTRASSE = 'de:08117:7640'
	RAUHWIESENSTRASSE_1 = 'de:08117:7640:0:H1'  # Rtg Rathaus (Donzdorf Winzingen)
	RAUHWIESENSTRASSE_2 = 'de:08117:7640:0:R1'  # Rtg Donzdorf (Donzdorf Winzingen)
	WINZINGEN_RAUHWIESENSTR = 'de:08117:7640'
	WINZINGEN_RAUHWIESENSTR_1 = 'de:08117:7640:0:H1'  # Rtg Rathaus (Donzdorf Winzingen)
	WINZINGEN_RAUHWIESENSTR_2 = 'de:08117:7640:0:R1'  # Rtg Donzdorf (Donzdorf Winzingen)
	WINZINGEN_RATHAUS = 'de:08117:7641'
	WINZINGEN_RATHAUS_1 = 'de:08117:7641:0:H1'  # Rtg Reichenbach u.R. (Donzdorf Winzingen)
	WINZINGEN_RATHAUS_2 = 'de:08117:7641:0:R1'  # Rtg Rauhwiesenstr. (Donzdorf Winzingen)
	WINZINGEN_SPORTPLATZ = 'de:08117:7642'
	WINZINGEN_SPORTPLATZ_1 = 'de:08117:7642:0:H1'  # Rtg Wißgoldingen (Donzdorf Winzingen)
	WINZINGEN_SPORTPLATZ_2 = 'de:08117:7642:0:R1'  # Rtg Reichenbach u.R. (Donzdorf Winzingen)
	GRÜNBACH = 'de:08117:7644'
	GRÜNBACH_1 = 'de:08117:7644:0:H1'  # Rtg Nenningen Kirche (Donzdorf Donzdorf)
	GRÜNBACH_2 = 'de:08117:7644:0:R1'  # Rtg Abz Winzingen (Donzdorf Donzdorf)
	DONZDORF_GRÜNBACH = 'de:08117:7644'
	DONZDORF_GRÜNBACH_1 = 'de:08117:7644:0:H1'  # Rtg Nenningen Kirche (Donzdorf Donzdorf)
	DONZDORF_GRÜNBACH_2 = 'de:08117:7644:0:R1'  # Rtg Abz Winzingen (Donzdorf Donzdorf)
	ZACHERSMÜHLE = 'de:08117:7653'
	ZACHERSMÜHLE_1 = 'de:08117:7653:0:H1'  # Rtg Abzweig beim Kloster (Adelberg Adelberg)
	ZACHERSMÜHLE_2 = 'de:08117:7653:0:R1'  # Rtg Abzweig Oberwälden (Adelberg Adelberg)
	ADELBERG_ZACHERSMÜHLE = 'de:08117:7653'
	ADELBERG_ZACHERSMÜHLE_1 = 'de:08117:7653:0:H1'  # Rtg Abzweig beim Kloster (Adelberg Adelberg)
	ADELBERG_ZACHERSMÜHLE_2 = 'de:08117:7653:0:R1'  # Rtg Abzweig Oberwälden (Adelberg Adelberg)
	ABZWEIG_BEIM_KLOSTER = 'de:08117:7654'
	ABZWEIG_BEIM_KLOSTER_1 = 'de:08117:7654:0:H1'  # Rtg Volksbank (Adelberg Adelberg)
	ABZWEIG_BEIM_KLOSTER_2 = 'de:08117:7654:0:R1'  # Rtg Zachersmühle (Adelberg Adelberg)
	ADELBERG_ABZW__BEIM_KLOSTER = 'de:08117:7654'
	ADELBERG_ABZW__BEIM_KLOSTER_1 = 'de:08117:7654:0:H1'  # Rtg Volksbank (Adelberg Adelberg)
	ADELBERG_ABZW__BEIM_KLOSTER_2 = 'de:08117:7654:0:R1'  # Rtg Zachersmühle (Adelberg Adelberg)
	ADELBERG_VOLKSBANK = 'de:08117:7656'
	ADELBERG_VOLKSBANK_1 = 'de:08117:7656:0:H1'  # Rtg. SOS Kinderdorf (Adelberg Adelberg)
	ADELBERG_VOLKSBANK_2 = 'de:08117:7656:0:R1'  # Rtg Abzw. Kloster (Adelberg Adelberg)
	BAIERECK_KRONE = 'de:08117:7663'
	BAIERECK_KRONE_1 = 'de:08117:7663:0:H1'  # Rtg Am Kugelrain (Uhingen Baiereck)
	BAIERECK_KRONE_2 = 'de:08117:7663:0:R1'  # Rtg Herrenbachstr. (Uhingen Baiereck)
	UHINGEN_NASSACH_UNTERHÜTT = 'de:08117:7664'
	UHINGEN_NASSACH_UNTERHÜTT_1 = 'de:08117:7664:0:H1'  # Rtg Baiereck (Uhingen Uhingen)
	UHINGEN_NASSACH_UNTERHÜTT_2 = 'de:08117:7664:0:R1'  # Rtg Nassach Bolzplatz (Uhingen Uhingen)
	NASSACH_UNTERHÜTT = 'de:08117:7664'
	NASSACH_UNTERHÜTT_1 = 'de:08117:7664:0:H1'  # Rtg Baiereck (Uhingen Uhingen)
	NASSACH_UNTERHÜTT_2 = 'de:08117:7664:0:R1'  # Rtg Nassach Bolzplatz (Uhingen Uhingen)
	WEISSENSTEIN_ALTES_RATHAUS = 'de:08117:7666'
	WEISSENSTEIN_ALTES_RATHAUS_1 = 'de:08117:7666:0:H1'  # Rtg Böhmenkirch (Lauterstein Weißenstein)
	WEISSENSTEIN_ALTES_RATHAUS_2 = 'de:08117:7666:0:R1'  # Rtg Nenningen (Lauterstein Weißenstein)
	AM_KUGELRAIN = 'de:08117:7669:0:R1'  # Rtg Krone (Uhingen Baiereck)
	BAIERECK_AM_KUGELRAIN = 'de:08117:7669:0:R1'  # Rtg Krone (Uhingen Baiereck)
	BÜCHENBRONN_SIEDLUNG = 'de:08117:7672'
	BÜCHENBRONN_SIEDLUNG_1 = 'de:08117:7672:0:H1'  # Rtg Gemeindeh. (Ebersbach an der Fils Ebersbach (F))
	BÜCHENBRONN_SIEDLUNG_2 = 'de:08117:7672:0:R1'  # Rtg Im Hegnach (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__BÜCHENBR__SIEDLUNG = 'de:08117:7672'
	EBERSB__F__BÜCHENBR__SIEDLUNG_1 = 'de:08117:7672:0:H1'  # Rtg Gemeindeh. (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__BÜCHENBR__SIEDLUNG_2 = 'de:08117:7672:0:R1'  # Rtg Im Hegnach (Ebersbach an der Fils Ebersbach (F))
	BÜCHENBRONN_FORSTHAUS = 'de:08117:7673'
	BÜCHENBRONN_FORSTHAUS_1 = 'de:08117:7673:0:H1'  # Rtg Königseiche (Ebersbach an der Fils Ebersbach (F))
	BÜCHENBRONN_FORSTHAUS_2 = 'de:08117:7673:0:R1'  # Rtg Gemeindehaus (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__BÜCHENBR__FORSTHAUS = 'de:08117:7673'
	EBERSB__F__BÜCHENBR__FORSTHAUS_1 = 'de:08117:7673:0:H1'  # Rtg Königseiche (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__BÜCHENBR__FORSTHAUS_2 = 'de:08117:7673:0:R1'  # Rtg Gemeindehaus (Ebersbach an der Fils Ebersbach (F))
	BÜCHENBRONN_KÖNIGSEICHE = 'de:08117:7674:0:H1'  # Rtg Krapfenreut (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__BÜCHENBR__KGS_EICHE = 'de:08117:7674:0:H1'  # Rtg Krapfenreut (Ebersbach an der Fils Ebersbach (F))
	TREFFELHAUSEN_LAMM = 'de:08117:7675:0:R1'  # Rtg Sonne (Böhmenkirch Treffelhausen)
	BAIERECK_KINDERGARTEN = 'de:08117:7676'
	BAIERECK_KINDERGARTEN_1 = 'de:08117:7676:0:H1'  # Rtg Baiereck (Uhingen Baiereck)
	BAIERECK_KINDERGARTEN_2 = 'de:08117:7676:0:R1'  # Rtg Nassach (Uhingen Baiereck)
	HERRENBACHSTRASSE = 'de:08117:7677'
	HERRENBACHSTRASSE_1 = 'de:08117:7677:0:H1'  # Rtg Baiereck (Uhingen Baiereck)
	HERRENBACHSTRASSE_2 = 'de:08117:7677:0:R1'  # Rtg Nassach (Uhingen Baiereck)
	BAIERECK_HERRENBACHSTR = 'de:08117:7677'
	BAIERECK_HERRENBACHSTR_1 = 'de:08117:7677:0:H1'  # Rtg Baiereck (Uhingen Baiereck)
	BAIERECK_HERRENBACHSTR_2 = 'de:08117:7677:0:R1'  # Rtg Nassach (Uhingen Baiereck)
	NASSACH_FRIEDHOF = 'de:08117:7678'
	NASSACH_FRIEDHOF_1 = 'de:08117:7678:0:H1'  # Rtg Nassach Bolzplatz (Uhingen Uhingen)
	NASSACH_FRIEDHOF_2 = 'de:08117:7678:0:R1'  # Rtg Nassachmühle (Uhingen Uhingen)
	UHINGEN_NASSACH_FRIEDHOF = 'de:08117:7678'
	UHINGEN_NASSACH_FRIEDHOF_1 = 'de:08117:7678:0:H1'  # Rtg Nassach Bolzplatz (Uhingen Uhingen)
	UHINGEN_NASSACH_FRIEDHOF_2 = 'de:08117:7678:0:R1'  # Rtg Nassachmühle (Uhingen Uhingen)
	UHINGEN_NASSACH_BOLZPLATZ = 'de:08117:7679'
	UHINGEN_NASSACH_BOLZPLATZ_1 = 'de:08117:7679:0:H1'  # Rtg Nassach Unterhütt (Uhingen Uhingen)
	UHINGEN_NASSACH_BOLZPLATZ_2 = 'de:08117:7679:0:R1'  # Rtg Nassach Friedhof (Uhingen Uhingen)
	NASSACH_BOLZPLATZ = 'de:08117:7679'
	NASSACH_BOLZPLATZ_1 = 'de:08117:7679:0:H1'  # Rtg Nassach Unterhütt (Uhingen Uhingen)
	NASSACH_BOLZPLATZ_2 = 'de:08117:7679:0:R1'  # Rtg Nassach Friedhof (Uhingen Uhingen)
	SCHNITTLINGEN_HIRSCH = 'de:08117:7680'
	SCHNITTLINGEN_HIRSCH_1 = 'de:08117:7680:0:H1'  # Rtg Treffelhausen (Böhmenkirch Schnittlingen)
	SCHNITTLINGEN_HIRSCH_2 = 'de:08117:7680:0:R1'  # Rtg Siedlung (Böhmenkirch Schnittlingen)
	NENNINGEN_KIRCHE = 'de:08117:7682'
	NENNINGEN_KIRCHE_1 = 'de:08117:7682:0:2'  # Rtg Rathaus (Lauterstein Nenningen)
	NENNINGEN_KIRCHE_2 = 'de:08117:7682:0:R1'  # Rtg Donzdorf (Lauterstein Nenningen)
	TREFFELHAUSEN_SCHULE = 'de:08117:17683:0:1'  # Rtg Geislingen (Böhmenkirch Treffelhausen)
	STÖTTEN_RATHAUS = 'de:08117:7686'
	STÖTTEN_RATHAUS_1 = 'de:08117:7686:0:H1'  # Rtg Schnittlingen (Geislingen an der Steige Stötten)
	STÖTTEN_RATHAUS_2 = 'de:08117:7686:0:R1'  # Rtg Aussiedlerhöfe (Geislingen an der Steige Stötten)
	SKILIFT_1 = 'de:08117:7688:0:H1'  # Rtg Böhmenkich (Böhmenkirch Treffelhausen)
	SKILIFT_2 = 'de:08117:7688:0:R1'  # Rtg Geislingen (Böhmenkirch Treffelhausen)
	SKILIFT_3 = 'de:08136:7041:0:1'  # eRtg Bruck (Lorch Lorch)
	SKILIFT_4 = 'de:08136:7041:0:2'  # eRtg Kaiserlinde (Lorch Lorch)
	TREFFELHAUSEN_SKILIFT = 'de:08117:7688'
	TREFFELHAUSEN_SKILIFT_1 = 'de:08117:7688:0:H1'  # Rtg Böhmenkich (Böhmenkirch Treffelhausen)
	TREFFELHAUSEN_SKILIFT_2 = 'de:08117:7688:0:R1'  # Rtg Geislingen (Böhmenkirch Treffelhausen)
	SCHLIERBACH_AUCHTERTSTR = 'de:08117:7801'
	SCHLIERBACH_AUCHTERTSTR_1 = 'de:08117:7801:0:H1'  # eRtg Pflegeheim (Schlierbach Schlierbach)
	SCHLIERBACH_AUCHTERTSTR_2 = 'de:08117:7801:0:R1'  # eRtg Siemensstraße (Schlierbach Schlierbach)
	AUCHTERTSTRASSE = 'de:08117:7801'
	AUCHTERTSTRASSE_1 = 'de:08117:7801:0:H1'  # eRtg Pflegeheim (Schlierbach Schlierbach)
	AUCHTERTSTRASSE_2 = 'de:08117:7801:0:R1'  # eRtg Siemensstraße (Schlierbach Schlierbach)
	HATTENHOFER_STRASSE = 'de:08117:7802'
	HATTENHOFER_STRASSE_1 = 'de:08117:7802:0:H1'  # eRtg Kirche (Schlierbach Schlierbach)
	HATTENHOFER_STRASSE_2 = 'de:08117:7802:0:R1'  # eRtg Pflegeheim (Schlierbach Schlierbach)
	SCHLIERB__HATTENH__STR = 'de:08117:7802'
	SCHLIERB__HATTENH__STR_1 = 'de:08117:7802:0:H1'  # eRtg Kirche (Schlierbach Schlierbach)
	SCHLIERB__HATTENH__STR_2 = 'de:08117:7802:0:R1'  # eRtg Pflegeheim (Schlierbach Schlierbach)
	SCHLIERBACH_KIRCHE = 'de:08117:7803'
	SCHLIERBACH_KIRCHE_1 = 'de:08117:7803:0:H1'  # eRtg Bachbrücke (Schlierbach Schlierbach)
	SCHLIERBACH_KIRCHE_2 = 'de:08117:7803:0:R1'  # eRtg Hattenhofer Straße (Schlierbach Schlierbach)
	SCHLIERBACH_BACHBRÜCKE = 'de:08117:7804'
	SCHLIERBACH_BACHBRÜCKE_1 = 'de:08117:7804:0:H1'  # eRtg Wald (Schlierbach Schlierbach)
	SCHLIERBACH_BACHBRÜCKE_2 = 'de:08117:7804:0:R1'  # eRtg Gaiserstraße (Schlierbach Schlierbach)
	BACHBRÜCKE = 'de:08117:7804'
	BACHBRÜCKE_1 = 'de:08117:7804:0:H1'  # eRtg Wald (Schlierbach Schlierbach)
	BACHBRÜCKE_2 = 'de:08117:7804:0:R1'  # eRtg Gaiserstraße (Schlierbach Schlierbach)
	SCHLIERBACH_TANNENHÖFE = 'de:08117:7800'
	SCHLIERBACH_TANNENHÖFE_1 = 'de:08117:7800:0:H1'  # eRtg Siemensstraße (Schlierbach Schlierbach)
	SCHLIERBACH_TANNENHÖFE_2 = 'de:08117:7800:0:R1'  # eRtg Industriegebiet (Schlierbach Schlierbach)
	TANNENHÖFE = 'de:08117:7800'
	TANNENHÖFE_1 = 'de:08117:7800:0:H1'  # eRtg Siemensstraße (Schlierbach Schlierbach)
	TANNENHÖFE_2 = 'de:08117:7800:0:R1'  # eRtg Industriegebiet (Schlierbach Schlierbach)
	SCHLIERBACH_SIEMENSSTR = 'de:08117:7799'
	SCHLIERBACH_SIEMENSSTR_1 = 'de:08117:7799:0:H1'  # eRtg Auchtertstraße (Schlierbach Schlierbach)
	SCHLIERBACH_SIEMENSSTR_2 = 'de:08117:7799:0:R1'  # eRtg Tannenhöfe (Schlierbach Schlierbach)
	SCHLIERBACH_PFLEGEHEIM = 'de:08117:7798'
	SCHLIERBACH_PFLEGEHEIM_1 = 'de:08117:7798:0:H1'  # eRtg Hattenhofer Straße (Schlierbach Schlierbach)
	SCHLIERBACH_PFLEGEHEIM_2 = 'de:08117:7798:0:R1'  # eRtg Auchtertstraße (Schlierbach Schlierbach)
	FELDWIESEN = 'de:08117:8002'
	FELDWIESEN_1 = 'de:08117:8002:0:H1'  # Rtg Pläckertstr. (Ebersbach an der Fils Bünzwangen)
	FELDWIESEN_2 = 'de:08117:8002:0:R1'  # Rtg Bünzwanger Str. (Ebersbach an der Fils Bünzwangen)
	BÜNZWANGEN_FELDWIESEN = 'de:08117:8002'
	BÜNZWANGEN_FELDWIESEN_1 = 'de:08117:8002:0:H1'  # Rtg Pläckertstr. (Ebersbach an der Fils Bünzwangen)
	BÜNZWANGEN_FELDWIESEN_2 = 'de:08117:8002:0:R1'  # Rtg Bünzwanger Str. (Ebersbach an der Fils Bünzwangen)
	PLÄCKERTSTRASSE = 'de:08117:8003'
	PLÄCKERTSTRASSE_1 = 'de:08117:8003:0:H1'  # Rtg Silcherstr. (Ebersbach an der Fils Bünzwangen)
	PLÄCKERTSTRASSE_2 = 'de:08117:8003:0:R1'  # Rtg Ortsstr. (Ebersbach an der Fils Bünzwangen)
	BÜNZWANGEN_PLÄCKERTSTR = 'de:08117:8003'
	BÜNZWANGEN_PLÄCKERTSTR_1 = 'de:08117:8003:0:H1'  # Rtg Silcherstr. (Ebersbach an der Fils Bünzwangen)
	BÜNZWANGEN_PLÄCKERTSTR_2 = 'de:08117:8003:0:R1'  # Rtg Ortsstr. (Ebersbach an der Fils Bünzwangen)
	BÜNZWANGEN_SILCHERSTR = 'de:08117:8004'
	BÜNZWANGEN_SILCHERSTR_1 = 'de:08117:8004:0:H1'  # Rtg Siedlung (Ebersbach an der Fils Bünzwangen)
	BÜNZWANGEN_SILCHERSTR_2 = 'de:08117:8004:0:R1'  # Rtg Pläckertstr. (Ebersbach an der Fils Bünzwangen)
	BÜNZWANGEN_SIEDLUNG = 'de:08117:8005'
	BÜNZWANGEN_SIEDLUNG_1 = 'de:08117:8005:0:H1'  # Rtg Seeweg (Ebersbach an der Fils Bünzwangen)
	BÜNZWANGEN_SIEDLUNG_2 = 'de:08117:8005:0:R1'  # Rtg Silcherstr. (Ebersbach an der Fils Bünzwangen)
	EBERSB__F__PUSTEBLUMENKREISEL = 'de:08117:8008'
	EBERSB__F__PUSTEBLUMENKREISEL_1 = 'de:08117:8008:0:H1'  # Rtg Leintelstr/Hardtschule (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__PUSTEBLUMENKREISEL_2 = 'de:08117:8008:0:R1'  # Rtg Ebersbach Bahnhof (Ebersbach an der Fils Ebersbach (F))
	PUSTEBLUMENKREISEL = 'de:08117:8008'
	PUSTEBLUMENKREISEL_1 = 'de:08117:8008:0:H1'  # Rtg Leintelstr/Hardtschule (Ebersbach an der Fils Ebersbach (F))
	PUSTEBLUMENKREISEL_2 = 'de:08117:8008:0:R1'  # Rtg Ebersbach Bahnhof (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__BÜCHENBRONNER_STR = 'de:08117:8012'
	EBERSB__F__BÜCHENBRONNER_STR_1 = 'de:08117:8012:0:H1'  # Rtg Hegnach (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__BÜCHENBRONNER_STR_2 = 'de:08117:8012:0:R1'  # Rtg Cafe Krieg (Ebersbach an der Fils Ebersbach (F))
	BÜCHENBRONNER_STRASSE = 'de:08117:8012'
	BÜCHENBRONNER_STRASSE_1 = 'de:08117:8012:0:H1'  # Rtg Hegnach (Ebersbach an der Fils Ebersbach (F))
	BÜCHENBRONNER_STRASSE_2 = 'de:08117:8012:0:R1'  # Rtg Cafe Krieg (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__IM_HEGNACH = 'de:08117:8013'
	EBERSB__F__IM_HEGNACH_1 = 'de:08117:8013:0:H1'  # Rtg Büchenbronn Siedlung (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__IM_HEGNACH_2 = 'de:08117:8013:0:R1'  # Rtg Büchenbronner Str. (Ebersbach an der Fils Ebersbach (F))
	IM_HEGNACH = 'de:08117:8013'
	IM_HEGNACH_1 = 'de:08117:8013:0:H1'  # Rtg Büchenbronn Siedlung (Ebersbach an der Fils Ebersbach (F))
	IM_HEGNACH_2 = 'de:08117:8013:0:R1'  # Rtg Büchenbronner Str. (Ebersbach an der Fils Ebersbach (F))
	EBERSBACH__F__STADTMITTE = 'de:08117:8014:0:H1'  # Rtg Ebersbach Bahnhof (Ebersbach an der Fils Ebersbach (F))
	LEINTELSTRASSE = 'de:08117:8015'
	LEINTELSTRASSE_1 = 'de:08117:8015:0:H1'  # Rtg Stadtbibliothek (Ebersbach an der Fils Ebersbach (F))
	LEINTELSTRASSE_2 = 'de:08117:8015:0:R1'  # Rtg Pusteblumenkreisel (Ebersbach an der Fils Ebersbach (F))
	EBERSBACH__F__LEINTELSTRASSE = 'de:08117:8015'
	EBERSBACH__F__LEINTELSTRASSE_1 = 'de:08117:8015:0:H1'  # Rtg Stadtbibliothek (Ebersbach an der Fils Ebersbach (F))
	EBERSBACH__F__LEINTELSTRASSE_2 = 'de:08117:8015:0:R1'  # Rtg Pusteblumenkreisel (Ebersbach an der Fils Ebersbach (F))
	SPORTPLÄTZE = 'de:08117:9000:0:R1'  # Rtg Friedhof Nord (Göppingen Göppingen)
	GÖPPINGEN_SPORTPLÄTZE = 'de:08117:9000:0:R1'  # Rtg Friedhof Nord (Göppingen Göppingen)
	GÖPPINGEN_LESSINGSTR = 'de:08117:9002:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_KEPLERSTR = 'de:08117:9003'
	GÖPPINGEN_KEPLERSTR_1 = 'de:08117:9003:0:H1'  # Rtg Jebenhausen (Göppingen Göppingen)
	GÖPPINGEN_KEPLERSTR_2 = 'de:08117:9003:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GAUSSWEG = 'de:08117:9004'
	GAUSSWEG_1 = 'de:08117:9004:0:H1'  # Rtg Klinik a. Eichert (Göppingen Göppingen)
	GAUSSWEG_2 = 'de:08117:9004:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_GAUSSWEG = 'de:08117:9004'
	GÖPPINGEN_GAUSSWEG_1 = 'de:08117:9004:0:H1'  # Rtg Klinik a. Eichert (Göppingen Göppingen)
	GÖPPINGEN_GAUSSWEG_2 = 'de:08117:9004:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	KLINIK_AM_EICHERT = 'de:08117:9006:0:H1'  # Klinik am Eichert (Göppingen Göppingen)
	GÖPP__KLINIK_AM_EICHERT = 'de:08117:9006:0:H1'  # Klinik am Eichert (Göppingen Göppingen)
	GÖPPINGEN_STADTHALLE = 'de:08117:9007:0:R1'  # Ankunft (Göppingen Göppingen)
	SCHULBAD = 'de:08117:9008'
	SCHULBAD_1 = 'de:08117:9008:0:A'  # Linie 2/15 (Göppingen Göppingen)
	SCHULBAD_2 = 'de:08117:9008:0:D'  # Linie 20/21/93 (Göppingen Göppingen)
	GÖPPINGEN_SCHULBAD = 'de:08117:9008'
	GÖPPINGEN_SCHULBAD_1 = 'de:08117:9008:0:A'  # Linie 2/15 (Göppingen Göppingen)
	GÖPPINGEN_SCHULBAD_2 = 'de:08117:9008:0:D'  # Linie 20/21/93 (Göppingen Göppingen)
	GÖPPINGEN_DÜRERSTR = 'de:08117:9010'
	GÖPPINGEN_DÜRERSTR_1 = 'de:08117:9010:0:H1'  # Rtg Hohenstaufen (Göppingen Göppingen)
	GÖPPINGEN_DÜRERSTR_2 = 'de:08117:9010:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	DÜRERSTRASSE = 'de:08117:9010'
	DÜRERSTRASSE_1 = 'de:08117:9010:0:H1'  # Rtg Hohenstaufen (Göppingen Göppingen)
	DÜRERSTRASSE_2 = 'de:08117:9010:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_HOGY = 'de:08117:9011'
	GÖPPINGEN_HOGY_1 = 'de:08117:9011:0:H1'  # Rtg Hohenstaufen (Göppingen Göppingen)
	GÖPPINGEN_HOGY_2 = 'de:08117:9011:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	HOGY = 'de:08117:9011'
	HOGY_1 = 'de:08117:9011:0:H1'  # Rtg Hohenstaufen (Göppingen Göppingen)
	HOGY_2 = 'de:08117:9011:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_FRIEDHOF_NORD = 'de:08117:9012'
	GÖPPINGEN_FRIEDHOF_NORD_1 = 'de:08117:9012:0:H1'  # Rtg Hohenstaufen (Göppingen Göppingen)
	GÖPPINGEN_FRIEDHOF_NORD_2 = 'de:08117:9012:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_HOHENSTEINSTR = 'de:08117:9013:0:H1'  # Rtg John-F.-Kennedy-Str. (Göppingen Göppingen)
	GÖPPINGEN_SAUSTEIGE = 'de:08117:9014:0:H1'  # Rtg Keplerstr. (Göppingen Göppingen)
	SAUSTEIGE = 'de:08117:9014:0:H1'  # Rtg Keplerstr. (Göppingen Göppingen)
	GÖPP__JOHN_F__KENNEDY_STR = 'de:08117:9015'
	GÖPP__JOHN_F__KENNEDY_STR_1 = 'de:08117:9015:0:H1'  # Rtg Hohenstaufen (Göppingen Göppingen)
	GÖPP__JOHN_F__KENNEDY_STR_2 = 'de:08117:9015:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	JOHN_F__KENNEDY_STR = 'de:08117:9015'
	JOHN_F__KENNEDY_STR_1 = 'de:08117:9015:0:H1'  # Rtg Hohenstaufen (Göppingen Göppingen)
	JOHN_F__KENNEDY_STR_2 = 'de:08117:9015:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	STAUFERPARK = 'de:08117:9016'
	STAUFERPARK_1 = 'de:08117:9016:0:H1'  # Rtg Stauferpark (Göppingen Göppingen)
	STAUFERPARK_2 = 'de:08117:9016:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_STAUFERPARK = 'de:08117:9016'
	GÖPPINGEN_STAUFERPARK_1 = 'de:08117:9016:0:H1'  # Rtg Stauferpark (Göppingen Göppingen)
	GÖPPINGEN_STAUFERPARK_2 = 'de:08117:9016:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	PAUL_KOEPFF_WEG = 'de:08117:9018'
	PAUL_KOEPFF_WEG_1 = 'de:08117:9018:0:H1'  # Rtg Galgenberg (Göppingen Göppingen)
	PAUL_KOEPFF_WEG_2 = 'de:08117:9018:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_PAUL_KOEPFF_WEG = 'de:08117:9018'
	GÖPPINGEN_PAUL_KOEPFF_WEG_1 = 'de:08117:9018:0:H1'  # Rtg Galgenberg (Göppingen Göppingen)
	GÖPPINGEN_PAUL_KOEPFF_WEG_2 = 'de:08117:9018:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	ROSEGGERWEG = 'de:08117:9020:0:H1'  # Rtg Lise-Meitner-Straße (Göppingen Göppingen)
	GÖPPINGEN_ROSEGGERWEG = 'de:08117:9020:0:H1'  # Rtg Lise-Meitner-Straße (Göppingen Göppingen)
	GÖPP__LISE_MEITNER_STRASSE = 'de:08117:9021:0:H1'  # Rtg Stadthalle (Göppingen Göppingen)
	LISE_MEITNER_STRASSE_1 = 'de:08117:9021:0:H1'  # Rtg Stadthalle (Göppingen Göppingen)
	LISE_MEITNER_STRASSE_2 = 'de:08235:4403:0:3'  # Rtg Netz/IHK (Nagold Nagold)
	GÖPPINGEN_GOLFPARK = 'de:08117:9028:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GOLFPARK = 'de:08117:9028:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_BLEICHSTR = 'de:08117:9025:0:H3'  # Rtg Feuerwache (Göppingen Göppingen)
	BLEICHSTRASSE = 'de:08117:9025:0:H3'  # Rtg Feuerwache (Göppingen Göppingen)
	GÖPP__MANFRED_WÖRNER_STR = 'de:08117:9027:0:H1'  # Rtg Golfpark (Göppingen Göppingen)
	MANFRED_WÖRNER_STRASSE = 'de:08117:9027:0:H1'  # Rtg Golfpark (Göppingen Göppingen)
	DR__HERBERT_KÖNIG_PLATZ = 'de:08117:1054'
	DR__HERBERT_KÖNIG_PLATZ_1 = 'de:08117:1054:0:H1'  # Rtg Stauferpark (Göppingen Göppingen)
	DR__HERBERT_KÖNIG_PLATZ_2 = 'de:08117:1054:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPP__DR__HERBERT_KÖNIG_PLATZ = 'de:08117:1054'
	GÖPP__DR__HERBERT_KÖNIG_PLATZ_1 = 'de:08117:1054:0:H1'  # Rtg Stauferpark (Göppingen Göppingen)
	GÖPP__DR__HERBERT_KÖNIG_PLATZ_2 = 'de:08117:1054:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	DR__PFEIFFER_STRASSE = 'de:08117:9201'
	DR__PFEIFFER_STRASSE_1 = 'de:08117:9201:0:H1'  # Rtg Klinik a. Eichert (Göppingen Göppingen)
	DR__PFEIFFER_STRASSE_2 = 'de:08117:9201:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_DR__PFEIFFER_STR = 'de:08117:9201'
	GÖPPINGEN_DR__PFEIFFER_STR_1 = 'de:08117:9201:0:H1'  # Rtg Klinik a. Eichert (Göppingen Göppingen)
	GÖPPINGEN_DR__PFEIFFER_STR_2 = 'de:08117:9201:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_RÖNTGENWEG = 'de:08117:9202'
	GÖPPINGEN_RÖNTGENWEG_1 = 'de:08117:9202:0:H1'  # Rtg Klinik a. Eichert (Göppingen Göppingen)
	GÖPPINGEN_RÖNTGENWEG_2 = 'de:08117:9202:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	RÖNTGENWEG = 'de:08117:9202'
	RÖNTGENWEG_1 = 'de:08117:9202:0:H1'  # Rtg Klinik a. Eichert (Göppingen Göppingen)
	RÖNTGENWEG_2 = 'de:08117:9202:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_WALDECKSTR = 'de:08117:9203:0:H1'  # Waldeckstraße (Göppingen Göppingen)
	GÖPPINGEN_HUGSTR = 'de:08117:9204:0:R1'  # Rtg Lessingstraße Göppingen (Göppingen Göppingen)
	HUGSTRASSE = 'de:08117:9204:0:R1'  # Rtg Lessingstraße Göppingen (Göppingen Göppingen)
	GÖPPINGEN_FRIEDHOF_SÜD = 'de:08117:9301'
	GÖPPINGEN_FRIEDHOF_SÜD_1 = 'de:08117:9301:0:H1'  # Rtg Galgenberg (Göppingen Göppingen)
	GÖPPINGEN_FRIEDHOF_SÜD_2 = 'de:08117:9301:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_AM_GALGENBERG = 'de:08117:9302'
	GÖPPINGEN_AM_GALGENBERG_1 = 'de:08117:9302:0:H1'  # Rtg Galgenberg (Göppingen Göppingen)
	GÖPPINGEN_AM_GALGENBERG_2 = 'de:08117:9302:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	AM_GALGENBERG = 'de:08117:9302'
	AM_GALGENBERG_1 = 'de:08117:9302:0:H1'  # Rtg Galgenberg (Göppingen Göppingen)
	AM_GALGENBERG_2 = 'de:08117:9302:0:R1'  # Rtg Göppingen ZOB (Göppingen Göppingen)
	GÖPPINGEN_RATHAUS = 'de:08117:9303:0:H1'  # Rtg Kornhausplatz (Göppingen Göppingen)
	GÖPPINGEN_BARBAROSSA_THERMEN = 'de:08117:9401:0:R1'  # Rtg Schillerstr. (Göppingen Göppingen)
	BARBAROSSA_THERMEN = 'de:08117:9401:0:R1'  # Rtg Schillerstr. (Göppingen Göppingen)
	SCHEERSTR__DR_ENGEL_REALSCHULE = 'de:08117:9812:0:H1'  # Rtg Friedhof Süd (Eislingen/Fils Eislingen (F))
	EISL__F__SCHEERSTR__REALSCHULE = 'de:08117:9812:0:H1'  # Rtg Friedhof Süd (Eislingen/Fils Eislingen (F))
	NASSACHMÜHLE = 'de:08117:9822'
	NASSACHMÜHLE_1 = 'de:08117:9822:0:H1'  # Rtg Nassach Friedhof (Uhingen Uhingen)
	NASSACHMÜHLE_2 = 'de:08117:9822:0:R1'  # Rtg Diegelsberg (Uhingen Uhingen)
	UHINGEN_NASSACHMÜHLE = 'de:08117:9822'
	UHINGEN_NASSACHMÜHLE_1 = 'de:08117:9822:0:H1'  # Rtg Nassach Friedhof (Uhingen Uhingen)
	UHINGEN_NASSACHMÜHLE_2 = 'de:08117:9822:0:R1'  # Rtg Diegelsberg (Uhingen Uhingen)
	DIEGELSBERG = 'de:08117:9823:0:H1'  # Rtg Nassachmühle Schule (Uhingen Uhingen)
	UHINGEN_DIEGELSBERG = 'de:08117:9823:0:H1'  # Rtg Nassachmühle Schule (Uhingen Uhingen)
	UHINGEN_BÜNZWANGER_STRASSE = 'de:08117:9825'
	UHINGEN_BÜNZWANGER_STRASSE_1 = 'de:08117:9825:0:H1'  # Rtg Ortsstr. (Uhingen Uhingen)
	UHINGEN_BÜNZWANGER_STRASSE_2 = 'de:08117:9825:0:R1'  # Rtg Volksbank (Uhingen Uhingen)
	NASSACHMÜHLE_SCHULE = 'de:08117:9826:0:H1'  # Rtg Diegelsberg (Uhingen Uhingen)
	UHINGEN_NASSACHMÜHLE_SCHULE = 'de:08117:9826:0:H1'  # Rtg Diegelsberg (Uhingen Uhingen)
	URSENWANG_GRUNDSCHULE = 'de:08117:9833:0:H1'  # Rtg Ursenw. Lindenplatz (Göppingen Göppingen)
	GÖPP__URSENW__GRUNDSCHULE = 'de:08117:9833:0:H1'  # Rtg Ursenw. Lindenplatz (Göppingen Göppingen)
	STAUFENECKSCHULE = 'de:08117:9862'
	STAUFENECKSCHULE_1 = 'de:08117:9862:0:H1'  # Rtg Baumgarten (Salach Salach)
	STAUFENECKSCHULE_2 = 'de:08117:9862:0:R1'  # Rtg Brühlweg (Salach Salach)
	SALACH_STAUFENECKSCHULE = 'de:08117:9862'
	SALACH_STAUFENECKSCHULE_1 = 'de:08117:9862:0:H1'  # Rtg Baumgarten (Salach Salach)
	SALACH_STAUFENECKSCHULE_2 = 'de:08117:9862:0:R1'  # Rtg Brühlweg (Salach Salach)
	RAICHBERG_SCHULZENTRUM = 'de:08117:9881:0:H1'  # alle Richtungen (Ebersbach an der Fils Ebersbach (F))
	EBERSB__F__RAICHBERG_SCHULZ = 'de:08117:9881:0:H1'  # alle Richtungen (Ebersbach an der Fils Ebersbach (F))
	WHG = 'de:08117:9895:0:H1'  # Rtg Bahnhofssteg (Göppingen Göppingen)
	GÖPPINGEN_WHG = 'de:08117:9895:0:H1'  # Rtg Bahnhofssteg (Göppingen Göppingen)
	BERUFSSCHULZENTRUM_ÖDE = 'de:08117:9896'
	BERUFSSCHULZENTRUM_ÖDE_1 = 'de:08117:9896:0:H1'  # Rtg Alemannenstr. (Göppingen Göppingen)
	BERUFSSCHULZENTRUM_ÖDE_2 = 'de:08117:9896:0:R1'  # Rtg Mercedesstraße (Göppingen Göppingen)
	GÖPPINGEN_BSZ_ÖDE = 'de:08117:9896'
	GÖPPINGEN_BSZ_ÖDE_1 = 'de:08117:9896:0:H1'  # Rtg Alemannenstr. (Göppingen Göppingen)
	GÖPPINGEN_BSZ_ÖDE_2 = 'de:08117:9896:0:R1'  # Rtg Mercedesstraße (Göppingen Göppingen)
	WÄSCHENBEUREN_STAUFERSCHULE = 'de:08117:9911:0:H1'  # Rtg Wäscherhofstr. (Wäschenbeuren Wäschenbeuren)
	ERGENZINGEN = 'de:08416:2052'
	ERGENZINGEN_1 = 'de:08416:2052:2:1'  # Gleis 2 (Rottenburg am Neckar Ergenzingen)
	ERGENZINGEN_2 = 'de:08416:2052:3:2'  # Gleis 1 (Rottenburg am Neckar Ergenzingen)
	HÄSLACH_RATHAUS = 'de:08415:24319'
	HÄSLACH_RATHAUS_1 = 'de:08415:24319:0:0'  # Rtg Häslach Süd (Walddorfhäslach Häslach)
	HÄSLACH_RATHAUS_2 = 'de:08415:24319:0:4'  # Rtg Altenriet Schule (Walddorfhäslach Häslach)
	DETTENHAUSEN_TÜBINGER_STR = 'de:08416:6352'
	DETTENHAUSEN_TÜBINGER_STR_1 = 'de:08416:6352:0:1'  # Rtg Alte Post (Dettenhausen Dettenhausen)
	DETTENHAUSEN_TÜBINGER_STR_2 = 'de:08416:6352:0:4'  # Rtg Bf/Weil i.S. Kälberstelle/Pfrond. Sp (Dettenhausen Dettenhausen)
	DETTENHAUSEN_ALTE_POST = 'de:08416:6353'
	DETTENHAUSEN_ALTE_POST_1 = 'de:08416:6353:0:1'  # Rtg Waldenbuch Bonholz (Dettenhausen Dettenhausen)
	DETTENHAUSEN_ALTE_POST_2 = 'de:08416:6353:0:4'  # Rtg Tübinger Str. (Dettenhausen Dettenhausen)
	ALTE_POST = 'de:08416:6353'
	ALTE_POST_1 = 'de:08416:6353:0:1'  # Rtg Waldenbuch Bonholz (Dettenhausen Dettenhausen)
	ALTE_POST_2 = 'de:08416:6353:0:4'  # Rtg Tübinger Str. (Dettenhausen Dettenhausen)
	DETTENHAUSEN_SCHULE = 'de:08416:6354'
	DETTENHAUSEN_SCHULE_1 = 'de:08416:6354:0:1'  # eRtg Altes Rathaus (Dettenhausen Dettenhausen)
	DETTENHAUSEN_SCHULE_2 = 'de:08416:6354:0:4'  # eRtg Bahnhofstraße (Dettenhausen Dettenhausen)
	DETTENHAUSEN = 'de:08416:6349'
	DETTENHAUSEN_1 = 'de:08416:6349:1:1'  # Gleis 1 (Dettenhausen Dettenhausen)
	DETTENHAUSEN_2 = 'de:08416:6349:1:2'  # Gleis 2 (Dettenhausen Dettenhausen)
	DETTENHAUSEN_3 = 'de:08416:6349:2:1'  # eRtg Alte Post (Dettenhausen Dettenhausen)
	DETTENHAUSEN_4 = 'de:08416:6349:2:4'  # eRtg Kälberstelle (Dettenhausen Dettenhausen)
	DETTENHAUSEN_TORSTR___BF = 'de:08416:26357:0:1'  # Rtg Rosenstr. (Dettenhausen Dettenhausen)
	TORSTRASSE__BAHNHOF = 'de:08416:26357:0:1'  # Rtg Rosenstr. (Dettenhausen Dettenhausen)
	DETTENHAUSEN_ALTES_RATHAUS = 'de:08416:6362'
	DETTENHAUSEN_ALTES_RATHAUS_1 = 'de:08416:6362:0:1'  # eRtg Schule (Dettenhausen Dettenhausen)
	DETTENHAUSEN_ALTES_RATHAUS_2 = 'de:08416:6362:0:4'  # eRtg Turnhalle (Dettenhausen Dettenhausen)
	ZIEGELWASENSTRASSE = 'de:08415:28280:0:1'  # Rtg Großbettlingen (Grafenberg Grafenberg)
	GRAFENBERG_ZIEGELWASENSTR = 'de:08415:28280:0:1'  # Rtg Großbettlingen (Grafenberg Grafenberg)
	DONNSTETTEN_RATHAUS = 'de:08415:28624:0:4'  # Rtg Bad Urach (Römerstein Donnstetten)
	GUTENBERGER_STRASSE = 'de:08415:28646'
	GUTENBERGER_STRASSE_1 = 'de:08415:28646:0:1'  # eRtg Rathaus (Römerstein Donnstetten)
	GUTENBERGER_STRASSE_2 = 'de:08415:28646:0:4'  # eRtg Fuchslöcher (Römerstein Donnstetten)
	DONNSTETTEN_GUTENBERGER_STR = 'de:08415:28646'
	DONNSTETTEN_GUTENBERGER_STR_1 = 'de:08415:28646:0:1'  # eRtg Rathaus (Römerstein Donnstetten)
	DONNSTETTEN_GUTENBERGER_STR_2 = 'de:08415:28646:0:4'  # eRtg Fuchslöcher (Römerstein Donnstetten)
	DONNSTETTEN_FUCHSLÖCHER = 'de:08415:28678'
	DONNSTETTEN_FUCHSLÖCHER_1 = 'de:08415:28678:0:1'  # Rtg Schopfloch Schlatterhöhe B465 (Römerstein Donnstetten)
	DONNSTETTEN_FUCHSLÖCHER_2 = 'de:08415:28678:0:4'  # Rtg Gutenberger Str./Rathaus (Römerstein Donnstetten)
	FUCHSLÖCHER = 'de:08415:28678'
	FUCHSLÖCHER_1 = 'de:08415:28678:0:1'  # Rtg Schopfloch Schlatterhöhe B465 (Römerstein Donnstetten)
	FUCHSLÖCHER_2 = 'de:08415:28678:0:4'  # Rtg Gutenberger Str./Rathaus (Römerstein Donnstetten)
	WALDDORF_JAHNSTRASSE = 'de:08415:29042'
	WALDDORF_JAHNSTRASSE_1 = 'de:08415:29042:0:1'  # Rtg Dettenhausen (Walddorfhäslach Walddorf)
	WALDDORF_JAHNSTRASSE_2 = 'de:08415:29042:0:2'  # Rtg Rathaus (Walddorfhäslach Walddorf)
	GUSTAV_WERNER_SCHULE = 'de:08415:29044:0:1'  # Rtg Häslach Süd (Walddorfhäslach Walddorf)
	WALDDORF_GUSTAV_WERNER_SCHULE = 'de:08415:29044:0:1'  # Rtg Häslach Süd (Walddorfhäslach Walddorf)
	EISBERG_L362 = 'de:08235:9350'
	EISBERG_L362_1 = 'de:08235:9350:0:3'  # Rtg Herrenberg Oberjettingen (Nagold Nagold)
	EISBERG_L362_2 = 'de:08235:9350:0:4'  # Rtg ZOB (Nagold Nagold)
	NAGOLD_EISBERG_L362 = 'de:08235:9350'
	NAGOLD_EISBERG_L362_1 = 'de:08235:9350:0:3'  # Rtg Herrenberg Oberjettingen (Nagold Nagold)
	NAGOLD_EISBERG_L362_2 = 'de:08235:9350:0:4'  # Rtg ZOB (Nagold Nagold)
	WALDLUST = 'de:08235:10069'
	WALDLUST_1 = 'de:08235:10069:0:3'  # Rtg Eisberg B28 Mötzingen (Nagold Nagold)
	WALDLUST_2 = 'de:08235:10069:0:4'  # Rtg ZOB (Nagold Nagold)
	NAGOLD_WALDLUST = 'de:08235:10069'
	NAGOLD_WALDLUST_1 = 'de:08235:10069:0:3'  # Rtg Eisberg B28 Mötzingen (Nagold Nagold)
	NAGOLD_WALDLUST_2 = 'de:08235:10069:0:4'  # Rtg ZOB (Nagold Nagold)
	NAGOLD_VOGELSANG = 'de:08235:10070'
	NAGOLD_VOGELSANG_1 = 'de:08235:10070:0:3'  # Rtg Waldlust Mötzingen (Nagold Nagold)
	NAGOLD_VOGELSANG_2 = 'de:08235:10070:0:4'  # Rtg ZOB (Nagold Nagold)
	NAGOLD_MÖTZINGER_STRASSE = 'de:08235:10072'
	NAGOLD_MÖTZINGER_STRASSE_1 = 'de:08235:10072:0:3'  # Rtg Mötzingen (Nagold Nagold)
	NAGOLD_MÖTZINGER_STRASSE_2 = 'de:08235:10072:0:4'  # Rtg Waldlust (Nagold Nagold)
	BERGWALD = 'de:08235:1500'
	BERGWALD_1 = 'de:08235:1500:0:3'  # eRtg Rathaus (Gechingen Gechingen)
	BERGWALD_2 = 'de:08235:1500:0:4'  # eRtg Bergsiedlung (Keplerstraße) (Gechingen Gechingen)
	GECHINGEN_BERGWALD = 'de:08235:1500'
	GECHINGEN_BERGWALD_1 = 'de:08235:1500:0:3'  # eRtg Rathaus (Gechingen Gechingen)
	GECHINGEN_BERGWALD_2 = 'de:08235:1500:0:4'  # eRtg Bergsiedlung (Keplerstraße) (Gechingen Gechingen)
	GECHINGEN_GARTENSTRASSE = 'de:08235:2540'
	GECHINGEN_GARTENSTRASSE_1 = 'de:08235:2540:0:1'  # Rtg Rathaus (Gechingen Gechingen)
	GECHINGEN_GARTENSTRASSE_2 = 'de:08235:2540:0:2'  # Rtg Bergwald (Gechingen Gechingen)
	GECHINGEN_RATHAUS = 'de:08235:1510'
	GECHINGEN_RATHAUS_1 = 'de:08235:1510:1:1'  # eRtg Im Gailer (Gechingen Gechingen)
	GECHINGEN_RATHAUS_2 = 'de:08235:1510:1:4'  # eRtg Bergwald (Gechingen Gechingen)
	GECHINGEN_IM_GAILER = 'de:08235:1530'
	GECHINGEN_IM_GAILER_1 = 'de:08235:1530:0:1'  # eRtg Kirchberg (Gechingen Gechingen)
	GECHINGEN_IM_GAILER_2 = 'de:08235:1530:0:2'  # eRtg Rathaus (Gechingen Gechingen)
	IM_GAILER = 'de:08235:1530'
	IM_GAILER_1 = 'de:08235:1530:0:1'  # eRtg Kirchberg (Gechingen Gechingen)
	IM_GAILER_2 = 'de:08235:1530:0:2'  # eRtg Rathaus (Gechingen Gechingen)
	NAGOLD_FERDINAND_PORSCHE_STR = 'de:08235:10092'
	NAGOLD_FERDINAND_PORSCHE_STR_1 = 'de:08235:10092:0:3'  # eRtg Martini (Nagold Nagold)
	NAGOLD_FERDINAND_PORSCHE_STR_2 = 'de:08235:10092:0:4'  # eRtg Gottlieb-Daimler-Str. (Nagold Nagold)
	FERDINAND_PORSCHE_STR = 'de:08235:10092'
	FERDINAND_PORSCHE_STR_1 = 'de:08235:10092:0:3'  # eRtg Martini (Nagold Nagold)
	FERDINAND_PORSCHE_STR_2 = 'de:08235:10092:0:4'  # eRtg Gottlieb-Daimler-Str. (Nagold Nagold)
	GOTTLIEB_DAIMLER_STRASSE = 'de:08235:10093'
	GOTTLIEB_DAIMLER_STRASSE_1 = 'de:08235:10093:0:3'  # eRtg Ferdinand-Porsche-Straße (Nagold Nagold)
	GOTTLIEB_DAIMLER_STRASSE_2 = 'de:08235:10093:0:4'  # eRtg Rübenacker (Nagold Nagold)
	NAGOLD_GOTTLIEB_DAIMLER_STRASSE = 'de:08235:10093'
	NAGOLD_GOTTLIEB_DAIMLER_STRASSE_1 = 'de:08235:10093:0:3'  # eRtg Ferdinand-Porsche-Straße (Nagold Nagold)
	NAGOLD_GOTTLIEB_DAIMLER_STRASSE_2 = 'de:08235:10093:0:4'  # eRtg Rübenacker (Nagold Nagold)
	NAGOLD_HOHER_BAUM_WEG = 'de:08235:4401:0:3'  # eRtg Gottlieb-Daimler-Straße (Nagold Nagold)
	HOHER_BAUM_WEG = 'de:08235:4401:0:3'  # eRtg Gottlieb-Daimler-Straße (Nagold Nagold)
	NAGOLD_LISE_MEITNER_STRASSE = 'de:08235:4403:0:3'  # Rtg Netz/IHK (Nagold Nagold)
	NAGOLD_INGPARK_SÜD = 'de:08235:10399'
	NAGOLD_INGPARK_SÜD_1 = 'de:08235:10399:0:3'  # Rtg INGpark Nord (Nagold Nagold)
	NAGOLD_INGPARK_SÜD_2 = 'de:08235:10399:0:4'  # Rtg Eisberg L362 (Nagold Nagold)
	INGPARK_SÜD = 'de:08235:10399'
	INGPARK_SÜD_1 = 'de:08235:10399:0:3'  # Rtg INGpark Nord (Nagold Nagold)
	INGPARK_SÜD_2 = 'de:08235:10399:0:4'  # Rtg Eisberg L362 (Nagold Nagold)
	BERNECK_B28 = 'de:08235:10156'
	BERNECK_B28_1 = 'de:08235:10156:0:3'  # Rtg Ebhausen (Altensteig Berneck)
	BERNECK_B28_2 = 'de:08235:10156:0:4'  # Rtg Altensteig (Altensteig Berneck)
	B28 = 'de:08235:10156'
	B28_1 = 'de:08235:10156:0:3'  # Rtg Ebhausen (Altensteig Berneck)
	B28_2 = 'de:08235:10156:0:4'  # Rtg Altensteig (Altensteig Berneck)
	EBHAUSEN_BREITER_WEG = 'de:08235:10208'
	EBHAUSEN_BREITER_WEG_1 = 'de:08235:10208:0:3'  # Rtg Ebhausen Rathaus (Ebhausen Ebhausen)
	EBHAUSEN_BREITER_WEG_2 = 'de:08235:10208:0:4'  # Rtg Altensteig Berneck (Ebhausen Ebhausen)
	EBHAUSEN_RATHAUS = 'de:08235:10207'
	EBHAUSEN_RATHAUS_1 = 'de:08235:10207:0:3'  # Rtg In der Misse Altensteig (Ebhausen Ebhausen)
	EBHAUSEN_RATHAUS_2 = 'de:08235:10207:0:4'  # Rtg Breiter Weg Nagold (Ebhausen Ebhausen)
	IN_DER_MISSE = 'de:08235:10206'
	IN_DER_MISSE_1 = 'de:08235:10206:0:3'  # Rtg Rohrdorf Bf Nagold (Ebhausen Ebhausen)
	IN_DER_MISSE_2 = 'de:08235:10206:0:4'  # Rtg Rathaus Altensteig (Ebhausen Ebhausen)
	EBHAUSEN_IN_DER_MISSE = 'de:08235:10206'
	EBHAUSEN_IN_DER_MISSE_1 = 'de:08235:10206:0:3'  # Rtg Rohrdorf Bf Nagold (Ebhausen Ebhausen)
	EBHAUSEN_IN_DER_MISSE_2 = 'de:08235:10206:0:4'  # Rtg Rathaus Altensteig (Ebhausen Ebhausen)
	ROHRDORF_BF = 'de:08235:10165'
	ROHRDORF_BF_1 = 'de:08235:10165:0:3'  # Rtg Gamp Nagold (Rohrdorf Rohrdorf (CW))
	ROHRDORF_BF_2 = 'de:08235:10165:0:4'  # Rtg In der Misse Altensteig (Rohrdorf Rohrdorf (CW))
	ROHRDORF_GAMP = 'de:08235:10166'
	ROHRDORF_GAMP_1 = 'de:08235:10166:0:3'  # Rtg Nagold (Rohrdorf Rohrdorf (CW))
	ROHRDORF_GAMP_2 = 'de:08235:10166:0:4'  # Rtg Bahnhof Altensteig (Rohrdorf Rohrdorf (CW))
	GAMP = 'de:08235:10166'
	GAMP_1 = 'de:08235:10166:0:3'  # Rtg Nagold (Rohrdorf Rohrdorf (CW))
	GAMP_2 = 'de:08235:10166:0:4'  # Rtg Bahnhof Altensteig (Rohrdorf Rohrdorf (CW))
	WAGON = 'de:08235:9332:0:1'  # Rtg Hoher-Baum-Weg (Nagold Nagold)
	NAGOLD_WAGON = 'de:08235:9332:0:1'  # Rtg Hoher-Baum-Weg (Nagold Nagold)
	NAGOLD_EINKAUFSZENTRUM = 'de:08235:10122'
	NAGOLD_EINKAUFSZENTRUM_1 = 'de:08235:10122:0:3'  # Rtg ZOB (Nagold Nagold)
	NAGOLD_EINKAUFSZENTRUM_2 = 'de:08235:10122:0:4'  # Rtg IMMO (Nagold Nagold)
	ALTENSTEIGER_STRASSE = 'de:08235:10112'
	ALTENSTEIGER_STRASSE_1 = 'de:08235:10112:0:3'  # Rtg ZOB (Nagold Nagold)
	ALTENSTEIGER_STRASSE_2 = 'de:08235:10112:0:4'  # Rtg Rohrdorf (Nagold Nagold)
	NAGOLD_ALTENSTEIGER_STR = 'de:08235:10112'
	NAGOLD_ALTENSTEIGER_STR_1 = 'de:08235:10112:0:3'  # Rtg ZOB (Nagold Nagold)
	NAGOLD_ALTENSTEIGER_STR_2 = 'de:08235:10112:0:4'  # Rtg Rohrdorf (Nagold Nagold)
	NAGOLD_ZOB = 'de:08235:9340'
	NAGOLD_ZOB_1 = 'de:08235:9340:1:3'  # Steig 3 (Nagold Nagold)
	NAGOLD_ZOB_2 = 'de:08235:9340:1:4'  # Steig 4 (Nagold Nagold)
	ALTENSTEIG_RATHAUS = 'de:08235:7881:1:3'  # Alle Richtungen (Altensteig Altensteig)
	GECHINGEN_KIRCHBERG = 'de:08235:1535'
	GECHINGEN_KIRCHBERG_1 = 'de:08235:1535:0:3'  # eRtg Rathaus (Gechingen Gechingen)
	GECHINGEN_KIRCHBERG_2 = 'de:08235:1535:0:4'  # eRtg Im Gailer (Gechingen Gechingen)
	KIRCHBERG = 'de:08235:1535'
	KIRCHBERG_1 = 'de:08235:1535:0:3'  # eRtg Rathaus (Gechingen Gechingen)
	KIRCHBERG_2 = 'de:08235:1535:0:4'  # eRtg Im Gailer (Gechingen Gechingen)
	NETZ___IHK = 'de:08235:9333'
	NETZ___IHK_1 = 'de:08235:9333:0:3'  # Rtg Mötz. Iselshauser Str. (Nagold Nagold)
	NETZ___IHK_2 = 'de:08235:9333:0:4'  # Rtg Wagon (Nagold Nagold)
	NAGOLD_NETZ___IHK = 'de:08235:9333'
	NAGOLD_NETZ___IHK_1 = 'de:08235:9333:0:3'  # Rtg Mötz. Iselshauser Str. (Nagold Nagold)
	NAGOLD_NETZ___IHK_2 = 'de:08235:9333:0:4'  # Rtg Wagon (Nagold Nagold)
	VOLLM_WEG_BIRKENWEG = 'de:08235:10343'
	VOLLM_WEG_BIRKENWEG_1 = 'de:08235:10343:0:3'  # Rtg F.-Porsche-Str. (Nagold Nagold)
	VOLLM_WEG_BIRKENWEG_2 = 'de:08235:10343:0:4'  # Rtg Kindergarten (Nagold Nagold)
	NAGOLD_VOLLM_WEG_BIRKENWEG = 'de:08235:10343'
	NAGOLD_VOLLM_WEG_BIRKENWEG_1 = 'de:08235:10343:0:3'  # Rtg F.-Porsche-Str. (Nagold Nagold)
	NAGOLD_VOLLM_WEG_BIRKENWEG_2 = 'de:08235:10343:0:4'  # Rtg Kindergarten (Nagold Nagold)
	NAGOLD_KINDERGARTEN = 'de:08235:10124'
	NAGOLD_KINDERGARTEN_1 = 'de:08235:10124:0:3'  # Rtg Vollmaringer-/Birkenweg (Nagold Nagold)
	NAGOLD_KINDERGARTEN_2 = 'de:08235:10124:0:4'  # Rtg Heckengäustr. (Nagold Nagold)
	HECKENGÄUSTRASSE = 'de:08235:10118'
	HECKENGÄUSTRASSE_1 = 'de:08235:10118:0:3'  # Rtg Heckenrosenweg (Nagold Nagold)
	HECKENGÄUSTRASSE_2 = 'de:08235:10118:0:4'  # Rtg Kindergarten (Nagold Nagold)
	NAGOLD_HECKENGÄUSTRASSE = 'de:08235:10118'
	NAGOLD_HECKENGÄUSTRASSE_1 = 'de:08235:10118:0:3'  # Rtg Heckenrosenweg (Nagold Nagold)
	NAGOLD_HECKENGÄUSTRASSE_2 = 'de:08235:10118:0:4'  # Rtg Kindergarten (Nagold Nagold)
	HECKENROSENWEG = 'de:08235:10119'
	HECKENROSENWEG_1 = 'de:08235:10119:0:3'  # Rtg Heckengäustraße (Nagold Nagold)
	HECKENROSENWEG_2 = 'de:08235:10119:0:4'  # Rtg ZOB (Nagold Nagold)
	NAGOLD_HECKENROSENWEG = 'de:08235:10119'
	NAGOLD_HECKENROSENWEG_1 = 'de:08235:10119:0:3'  # Rtg Heckengäustraße (Nagold Nagold)
	NAGOLD_HECKENROSENWEG_2 = 'de:08235:10119:0:4'  # Rtg ZOB (Nagold Nagold)
	NAGOLD_IMMO = 'de:08235:10145'
	NAGOLD_IMMO_1 = 'de:08235:10145:0:3'  # Rtg Heckenrosenweg (Nagold Nagold)
	NAGOLD_IMMO_2 = 'de:08235:10145:0:4'  # Rtg Einkaufszentrum (Nagold Nagold)
	IMMO = 'de:08235:10145'
	IMMO_1 = 'de:08235:10145:0:3'  # Rtg Heckenrosenweg (Nagold Nagold)
	IMMO_2 = 'de:08235:10145:0:4'  # Rtg Einkaufszentrum (Nagold Nagold)
	LEMBERGSCHULE = 'de:08235:10135'
	LEMBERGSCHULE_1 = 'de:08235:10135:0:1'  # Bstg 1 (Nagold Nagold)
	LEMBERGSCHULE_2 = 'de:08235:10135:0:3'  # Rtg Nagold Rentschler ZOB (Nagold Nagold)
	NAGOLD_LEMBERGSCHULE = 'de:08235:10135'
	NAGOLD_LEMBERGSCHULE_1 = 'de:08235:10135:0:1'  # Bstg 1 (Nagold Nagold)
	NAGOLD_LEMBERGSCHULE_2 = 'de:08235:10135:0:3'  # Rtg Nagold Rentschler ZOB (Nagold Nagold)
	HEIMSHEIM_LEONBERGER_STR = 'de:08236:3514'
	HEIMSHEIM_LEONBERGER_STR_1 = 'de:08236:3514:0:1'  # Rtg Perouse (Heimsheim Heimsheim)
	HEIMSHEIM_LEONBERGER_STR_2 = 'de:08236:3514:0:2'  # Rtg Heimsheim Marktplatz (Heimsheim Heimsheim)
	HEIMSHEIM_SEE = 'de:08236:3510'
	HEIMSHEIM_SEE_1 = 'de:08236:3510:0:1'  # Rtg Marktplatz (Heimsheim Heimsheim)
	HEIMSHEIM_SEE_2 = 'de:08236:3510:0:2'  # Rtg Leonberger Str. (Heimsheim Heimsheim)
	HEIMSHEIM_SCHAFWÄSCHE = 'de:08236:3509'
	HEIMSHEIM_SCHAFWÄSCHE_1 = 'de:08236:3509:0:1'  # Rtg Perouse (Heimsheim Heimsheim)
	HEIMSHEIM_SCHAFWÄSCHE_2 = 'de:08236:3509:0:2'  # Rtg Friolzheim (Heimsheim Heimsheim)
	SCHAFWÄSCHE = 'de:08236:3509'
	SCHAFWÄSCHE_1 = 'de:08236:3509:0:1'  # Rtg Perouse (Heimsheim Heimsheim)
	SCHAFWÄSCHE_2 = 'de:08236:3509:0:2'  # Rtg Friolzheim (Heimsheim Heimsheim)
	HEIMSHEIM_MARKTPLATZ = 'de:08236:3513'
	HEIMSHEIM_MARKTPLATZ_1 = 'de:08236:3513:0:1'  # Rtg Rutesheim/Hausen (Heimsheim Heimsheim)
	HEIMSHEIM_MARKTPLATZ_2 = 'de:08236:3513:0:2'  # Rtg See (Heimsheim Heimsheim)
	HEIMSHEIM_EGELSEE = 'de:08236:3507'
	HEIMSHEIM_EGELSEE_1 = 'de:08236:3507:0:1'  # Rtg Süden (Heimsheim Heimsheim)
	HEIMSHEIM_EGELSEE_2 = 'de:08236:3507:0:2'  # Rtg Dieb (Heimsheim Heimsheim)
	LEHNINGEN_MÜHLHAUSENER_STRASSE = 'de:08236:3607'
	LEHNINGEN_MÜHLHAUSENER_STRASSE_1 = 'de:08236:3607:0:3'  # eRtg Würmtalstraße (Tiefenbronn Lehningen)
	LEHNINGEN_MÜHLHAUSENER_STRASSE_2 = 'de:08236:3607:0:4'  # eRtg Rathaus (Tiefenbronn Lehningen)
	MÜHLHAUSENER_STRASSE = 'de:08236:3607'
	MÜHLHAUSENER_STRASSE_1 = 'de:08236:3607:0:3'  # eRtg Würmtalstraße (Tiefenbronn Lehningen)
	MÜHLHAUSENER_STRASSE_2 = 'de:08236:3607:0:4'  # eRtg Rathaus (Tiefenbronn Lehningen)
	LUDWIG_UHLAND_SCHULE = 'de:08236:3515:0:1'  # Rtg Brunnenstr. (Heimsheim Heimsheim)
	HEIMSH__LUDWIG_UHLAND_SCHULE = 'de:08236:3515:0:1'  # Rtg Brunnenstr. (Heimsheim Heimsheim)
	LEHNINGEN_RATHAUS = 'de:08236:3609'
	LEHNINGEN_RATHAUS_1 = 'de:08236:3609:0:3'  # Rtg Mühlhauser Str. (Tiefenbronn Lehningen)
	LEHNINGEN_RATHAUS_2 = 'de:08236:3609:0:4'  # Rtg Hausen (Tiefenbronn Lehningen)
	VOLLZUGSANSTALT = 'de:08236:3508:0:1'  # Rtg See (Heimsheim Heimsheim)
	HEIMSHEIM_VOLLZUGSANSTALT = 'de:08236:3508:0:1'  # Rtg See (Heimsheim Heimsheim)
	HEIMSHEIM_DIEB = 'de:08236:3418'
	HEIMSHEIM_DIEB_1 = 'de:08236:3418:0:1'  # Rtg Egelsee (Heimsheim Heimsheim)
	HEIMSHEIM_DIEB_2 = 'de:08236:3418:0:2'  # Rtg Geissberg (Heimsheim Heimsheim)
	DIEB = 'de:08236:3418'
	DIEB_1 = 'de:08236:3418:0:1'  # Rtg Egelsee (Heimsheim Heimsheim)
	DIEB_2 = 'de:08236:3418:0:2'  # Rtg Geissberg (Heimsheim Heimsheim)
	WÜSTENROT_FIRMA_ZÜGEL = 'de:08125:983'
	WÜSTENROT_FIRMA_ZÜGEL_1 = 'de:08125:983:0:1'  # eRtg Schule (Wüstenrot Wüstenrot)
	WÜSTENROT_FIRMA_ZÜGEL_2 = 'de:08125:983:0:2'  # eRtg Schönblickstraße (Wüstenrot Wüstenrot)
	FIRMA_ZÜGEL = 'de:08125:983'
	FIRMA_ZÜGEL_1 = 'de:08125:983:0:1'  # eRtg Schule (Wüstenrot Wüstenrot)
	FIRMA_ZÜGEL_2 = 'de:08125:983:0:2'  # eRtg Schönblickstraße (Wüstenrot Wüstenrot)
	LANGHANS = 'de:08125:4672'
	LANGHANS_1 = 'de:08125:4672:0:1'  # eRtg Apotheke (Beilstein Beilstein)
	LANGHANS_2 = 'de:08125:4672:0:2'  # eRtg Schulzentrum (Beilstein Beilstein)
	BEILSTEIN_LANGHANS = 'de:08125:4672'
	BEILSTEIN_LANGHANS_1 = 'de:08125:4672:0:1'  # eRtg Apotheke (Beilstein Beilstein)
	BEILSTEIN_LANGHANS_2 = 'de:08125:4672:0:2'  # eRtg Schulzentrum (Beilstein Beilstein)
	BEILSTEIN_SCHULZENTRUM = 'de:08125:32310:0:1'  # eRtg Langhans (Beilstein Beilstein)
	WÜSTENROT_NEULAUTERN = 'de:08125:1022'
	WÜSTENROT_NEULAUTERN_1 = 'de:08125:1022:0:1'  # eRtg Lohmühle (Wüstenrot Wüstenrot)
	WÜSTENROT_NEULAUTERN_2 = 'de:08125:1022:0:2'  # eRtg Lautertal (Wüstenrot Wüstenrot)
	NEULAUTERN = 'de:08125:1022'
	NEULAUTERN_1 = 'de:08125:1022:0:1'  # eRtg Lohmühle (Wüstenrot Wüstenrot)
	NEULAUTERN_2 = 'de:08125:1022:0:2'  # eRtg Lautertal (Wüstenrot Wüstenrot)
	WÜSTENROT_LAUTERTAL = 'de:08125:1991'
	WÜSTENROT_LAUTERTAL_1 = 'de:08125:1991:0:1'  # eRtg Neulautern (Wüstenrot Wüstenrot)
	WÜSTENROT_LAUTERTAL_2 = 'de:08125:1991:0:2'  # eRtg Gieshof (Wüstenrot Wüstenrot)
	SCHÖNBLICKSTRASSE = 'de:08125:44219'
	SCHÖNBLICKSTRASSE_1 = 'de:08125:44219:0:1'  # eRtg Firma Zügel (Wüstenrot Wüstenrot)
	SCHÖNBLICKSTRASSE_2 = 'de:08125:44219:0:2'  # eRtg Schmellenhof (Wüstenrot Wüstenrot)
	WÜSTENROT_SCHÖNBLICKSTR = 'de:08125:44219'
	WÜSTENROT_SCHÖNBLICKSTR_1 = 'de:08125:44219:0:1'  # eRtg Firma Zügel (Wüstenrot Wüstenrot)
	WÜSTENROT_SCHÖNBLICKSTR_2 = 'de:08125:44219:0:2'  # eRtg Schmellenhof (Wüstenrot Wüstenrot)
	WÜSTENROT_SCHULE = 'de:08125:984'
	WÜSTENROT_SCHULE_1 = 'de:08125:984:0:1'  # eRtg Freibad (Wüstenrot Wüstenrot)
	WÜSTENROT_SCHULE_2 = 'de:08125:984:0:2'  # eRtg Firma Zügel (Wüstenrot Wüstenrot)
	VORDERBÜCHELBERG_ABZWEIG = 'de:08125:2024'
	VORDERBÜCHELBERG_ABZWEIG_1 = 'de:08125:2024:0:1'  # eRtg Firma Zügel (Wüstenrot Wüstenrot)
	VORDERBÜCHELBERG_ABZWEIG_2 = 'de:08125:2024:0:2'  # eRtg Neulautern (Wüstenrot Wüstenrot)
	WÜSTENR__VORDERBÜCHELBG__ABZW = 'de:08125:2024'
	WÜSTENR__VORDERBÜCHELBG__ABZW_1 = 'de:08125:2024:0:1'  # eRtg Firma Zügel (Wüstenrot Wüstenrot)
	WÜSTENR__VORDERBÜCHELBG__ABZW_2 = 'de:08125:2024:0:2'  # eRtg Neulautern (Wüstenrot Wüstenrot)
	BEILSTEIN_APOTHEKE = 'de:08125:4673'
	BEILSTEIN_APOTHEKE_1 = 'de:08125:4673:0:1'  # eRtg Breslauer Straße/Kath. Kirche (Beilstein Beilstein)
	BEILSTEIN_APOTHEKE_2 = 'de:08125:4673:0:2'  # eRtg Langhans (Beilstein Beilstein)
	BEILSTEIN_APOTHEKE_3 = 'de:08125:4673:0:5'  # Nachtbus (Beilstein Beilstein)
	BEILSTEIN_BRESLAUER_STRASSE = 'de:08125:4676'
	BEILSTEIN_BRESLAUER_STRASSE_1 = 'de:08125:4676:0:1'  # eRtg Schmidhausen Abzweig (Beilstein Beilstein)
	BEILSTEIN_BRESLAUER_STRASSE_2 = 'de:08125:4676:0:2'  # eRtg Apotheke (Beilstein Beilstein)
	BEILSTEIN_SCHMIDHAUSEN_ABZW = 'de:08125:4677'
	BEILSTEIN_SCHMIDHAUSEN_ABZW_1 = 'de:08125:4677:0:1'  # eRtg Schule (Beilstein Beilstein)
	BEILSTEIN_SCHMIDHAUSEN_ABZW_2 = 'de:08125:4677:0:2'  # eRtg Breslauer Straße (Beilstein Beilstein)
	SCHMIDHAUSEN_ABZWEIG = 'de:08125:4677'
	SCHMIDHAUSEN_ABZWEIG_1 = 'de:08125:4677:0:1'  # eRtg Schule (Beilstein Beilstein)
	SCHMIDHAUSEN_ABZWEIG_2 = 'de:08125:4677:0:2'  # eRtg Breslauer Straße (Beilstein Beilstein)
	TRIPSDRILL_ERLEBNISPARK = 'de:08125:5444:0:1'  # eRtg Alle Richtungen (Cleebronn Cleebronn)
	CLEEB__TRIPSDRILL_ERLEBNISPARK = 'de:08125:5444:0:1'  # eRtg Alle Richtungen (Cleebronn Cleebronn)
	WILDPARADIES_TRIPSDRILL = 'de:08125:4714'
	WILDPARADIES_TRIPSDRILL_1 = 'de:08125:4714:0:1'  # eRtg Tripsdrill Erlebnispark (Cleebronn Cleebronn)
	WILDPARADIES_TRIPSDRILL_2 = 'de:08125:4714:0:2'  # eRtg  Freudental/Ditzingen (Cleebronn Cleebronn)
	CLEEBR_WILDPARADIES_TRIPSDRILL = 'de:08125:4714'
	CLEEBR_WILDPARADIES_TRIPSDRILL_1 = 'de:08125:4714:0:1'  # eRtg Tripsdrill Erlebnispark (Cleebronn Cleebronn)
	CLEEBR_WILDPARADIES_TRIPSDRILL_2 = 'de:08125:4714:0:2'  # eRtg  Freudental/Ditzingen (Cleebronn Cleebronn)
	WÜSTENROT_LOHMÜHLE = 'de:08125:978'
	WÜSTENROT_LOHMÜHLE_1 = 'de:08125:978:0:1'  # eRtg Stangenbach (Wüstenrot Wüstenrot)
	WÜSTENROT_LOHMÜHLE_2 = 'de:08125:978:0:2'  # eRtg Neulautern (Wüstenrot Wüstenrot)
	LOHMÜHLE = 'de:08125:978'
	LOHMÜHLE_1 = 'de:08125:978:0:1'  # eRtg Stangenbach (Wüstenrot Wüstenrot)
	LOHMÜHLE_2 = 'de:08125:978:0:2'  # eRtg Neulautern (Wüstenrot Wüstenrot)
	STANGENBACH = 'de:08125:981'
	STANGENBACH_1 = 'de:08125:981:0:1'  # eRtg Schmellenhof (Wüstenrot Wüstenrot)
	STANGENBACH_2 = 'de:08125:981:0:2'  # eRtg Lohmühle (Wüstenrot Wüstenrot)
	WÜSTENROT_STANGENBACH = 'de:08125:981'
	WÜSTENROT_STANGENBACH_1 = 'de:08125:981:0:1'  # eRtg Schmellenhof (Wüstenrot Wüstenrot)
	WÜSTENROT_STANGENBACH_2 = 'de:08125:981:0:2'  # eRtg Lohmühle (Wüstenrot Wüstenrot)
	WÜSTENROT_SCHMELLENHOF = 'de:08125:982'
	WÜSTENROT_SCHMELLENHOF_1 = 'de:08125:982:0:1'  # eRtg Schönblickstraße (Wüstenrot Wüstenrot)
	WÜSTENROT_SCHMELLENHOF_2 = 'de:08125:982:0:2'  # eRtg Stangenbach (Wüstenrot Wüstenrot)
	SCHMELLENHOF = 'de:08125:982'
	SCHMELLENHOF_1 = 'de:08125:982:0:1'  # eRtg Schönblickstraße (Wüstenrot Wüstenrot)
	SCHMELLENHOF_2 = 'de:08125:982:0:2'  # eRtg Stangenbach (Wüstenrot Wüstenrot)
	WÜSTENROT_SILBERSTOLLEN = 'de:08125:32451:0:3'  # eRtg Schmellenhof (Wüstenrot Wüstenrot)
	SILBERSTOLLEN = 'de:08125:32451:0:3'  # eRtg Schmellenhof (Wüstenrot Wüstenrot)
	STANGENBACH_HAUS_WALDESRUH = 'de:08125:32453'
	STANGENBACH_HAUS_WALDESRUH_1 = 'de:08125:32453:0:1'  # Rtg Schmellenhof (Wüstenrot Wüstenrot)
	STANGENBACH_HAUS_WALDESRUH_2 = 'de:08125:32453:0:2'  # Rtg Stangenbach (Wüstenrot Wüstenrot)
	WÜSTENROT_ST_B__HAUS_WALDESRUH = 'de:08125:32453'
	WÜSTENROT_ST_B__HAUS_WALDESRUH_1 = 'de:08125:32453:0:1'  # Rtg Schmellenhof (Wüstenrot Wüstenrot)
	WÜSTENROT_ST_B__HAUS_WALDESRUH_2 = 'de:08125:32453:0:2'  # Rtg Stangenbach (Wüstenrot Wüstenrot)
	WALDHAUSEN = 'de:08136:7031'
	WALDHAUSEN_1 = 'de:08136:7031:1:H1'  # eRtg Plüderhausen (Lorch Waldhausen b Schornd (Lorch))
	WALDHAUSEN_2 = 'de:08136:7031:1:R1'  # eRtg Weitmars (Lorch Waldhausen b Schornd (Lorch))
	WALDHAUSEN_3 = 'de:08136:7031:2:1'  # Gleis 1 (Lorch Waldhausen b Schornd (Lorch))
	WALDHAUSEN_4 = 'de:08136:7031:3:2'  # Gleis 2 (Lorch Waldhausen b Schornd (Lorch))
	LORCH = 'de:08136:7005'
	LORCH_1 = 'de:08136:7005:1:1'  # Bussteig 1 (Lorch Lorch)
	LORCH_2 = 'de:08136:7005:1:3'  # Bussteig 3 (Lorch Lorch)
	LORCH_3 = 'de:08136:7005:2:1'  # Gleis 1 (Lorch Lorch)
	LORCH_4 = 'de:08136:7005:3:2'  # Gleis 2 (Lorch Lorch)
	SCHWÄBISCH_GMÜND = 'de:08136:3077'
	SCHWÄBISCH_GMÜND_1 = 'de:08136:3077:0:1'  # Gleis 1 (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHWÄBISCH_GMÜND_2 = 'de:08136:3077:0:2'  # Gleis 2 (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHWÄBISCH_GMÜND_3 = 'de:08136:3077:1:0'  # Ankunftssteig (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHWÄBISCH_GMÜND_4 = 'de:08136:3077:1:1B'  # Busstg. 1B (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHWÄBISCH_GMÜND_5 = 'de:08136:3077:1:H1'  # Schwäbisch Gmünd (Schwäbisch Gmünd Schwäbisch Gmünd)
	MAINHARDT_WENDEPLATTE = 'de:08127:28270:0:1'  # eRtg Alle Richtungen (Mainhardt Mainhardt)
	B14 = 'de:08127:28102:0:2'  # Rtg Großerlach (Mainhardt Mainhardt)
	MAINHARDT_B14 = 'de:08127:28102:0:2'  # Rtg Großerlach (Mainhardt Mainhardt)
	MAINHARDT_ABZW__HOHENEGARTEN = 'de:08127:27509'
	MAINHARDT_ABZW__HOHENEGARTEN_1 = 'de:08127:27509:0:1'  # Rtg. Hohenegarten (Mainhardt Mainhardt)
	MAINHARDT_ABZW__HOHENEGARTEN_2 = 'de:08127:27509:0:2'  # Rtg. Mainhardt/B14 (Mainhardt Mainhardt)
	ABZW__HOHENEGARTEN = 'de:08127:27509'
	ABZW__HOHENEGARTEN_1 = 'de:08127:27509:0:1'  # Rtg. Hohenegarten (Mainhardt Mainhardt)
	ABZW__HOHENEGARTEN_2 = 'de:08127:27509:0:2'  # Rtg. Mainhardt/B14 (Mainhardt Mainhardt)
	MAINH__ABZW__HOHENEG__B14 = 'de:08127:28273'
	MAINH__ABZW__HOHENEG__B14_1 = 'de:08127:28273:0:1'  # eRtg Mainhardt (Mainhardt Mainhardt)
	MAINH__ABZW__HOHENEG__B14_2 = 'de:08127:28273:0:2'  # eRtg Großerlach (Mainhardt Mainhardt)
	ABZW__HOHENEGARTEN_B14 = 'de:08127:28273'
	ABZW__HOHENEGARTEN_B14_1 = 'de:08127:28273:0:1'  # eRtg Mainhardt (Mainhardt Mainhardt)
	ABZW__HOHENEGARTEN_B14_2 = 'de:08127:28273:0:2'  # eRtg Großerlach (Mainhardt Mainhardt)
	MASSSTABFABRIK = 'de:08127:28268'
	MASSSTABFABRIK_1 = 'de:08127:28268:0:1'  # eRtg Turmuhrenm./Industrie (Mainhardt Mainhardt)
	MASSSTABFABRIK_2 = 'de:08127:28268:0:2'  # eRtg Abzw Hohenegarten/Hohenstraßen (Mainhardt Mainhardt)
	MAINHARDT_MASSSTABFABRIK = 'de:08127:28268'
	MAINHARDT_MASSSTABFABRIK_1 = 'de:08127:28268:0:1'  # eRtg Turmuhrenm./Industrie (Mainhardt Mainhardt)
	MAINHARDT_MASSSTABFABRIK_2 = 'de:08127:28268:0:2'  # eRtg Abzw Hohenegarten/Hohenstraßen (Mainhardt Mainhardt)
	TURMUHRENMUSEUM = 'de:08127:28266:0:1'  # eRtg Wendeplatte (Mainhardt Mainhardt)
	MAINHARDT_TURMUHRENMUSEUM = 'de:08127:28266:0:1'  # eRtg Wendeplatte (Mainhardt Mainhardt)
	GSCHWEND_MARKTPLATZ = 'de:08136:6512'
	GSCHWEND_MARKTPLATZ_1 = 'de:08136:6512:0:3'  # eRtg Hintersteinenberg (Gschwend Gschwend)
	GSCHWEND_MARKTPLATZ_2 = 'de:08136:6512:0:4'  # eRtg Fa Kunz (Gschwend Gschwend)
	LORCH_BRUCK = 'de:08136:7000'
	LORCH_BRUCK_1 = 'de:08136:7000:0:1'  # eRtg Kloster (Lorch Lorch)
	LORCH_BRUCK_2 = 'de:08136:7000:0:2'  # eRtg Skilift (Lorch Lorch)
	BRUCK = 'de:08136:7000'
	BRUCK_1 = 'de:08136:7000:0:1'  # eRtg Kloster (Lorch Lorch)
	BRUCK_2 = 'de:08136:7000:0:2'  # eRtg Skilift (Lorch Lorch)
	KLOSTER = 'de:08136:7017'
	KLOSTER_1 = 'de:08136:7017:0:1'  # eRtg Lorch (Lorch Lorch)
	KLOSTER_2 = 'de:08136:7017:0:2'  # eRtg Bruck (Lorch Lorch)
	LORCH_KLOSTER = 'de:08136:7017'
	LORCH_KLOSTER_1 = 'de:08136:7017:0:1'  # eRtg Lorch (Lorch Lorch)
	LORCH_KLOSTER_2 = 'de:08136:7017:0:2'  # eRtg Bruck (Lorch Lorch)
	B_297_HOHE_LINDE = 'de:08136:7002'
	B_297_HOHE_LINDE_1 = 'de:08136:7002:0:H1'  # Rtg Schw. Gmünd (Lorch Kirneck)
	B_297_HOHE_LINDE_2 = 'de:08136:7002:0:R1'  # Rtg Göppingen (Lorch Kirneck)
	KIRNECK_B_297_HOHE_LINDE = 'de:08136:7002'
	KIRNECK_B_297_HOHE_LINDE_1 = 'de:08136:7002:0:H1'  # Rtg Schw. Gmünd (Lorch Kirneck)
	KIRNECK_B_297_HOHE_LINDE_2 = 'de:08136:7002:0:R1'  # Rtg Göppingen (Lorch Kirneck)
	OBERKIRNECK_ORTSMITTE_KORNSTR = 'de:08136:7024:0:H1'  # Ortsmitte/Kornstr. (Lorch Oberkirneck)
	ORTSMITTE_KORNSTR = 'de:08136:7024:0:H1'  # Ortsmitte/Kornstr. (Lorch Oberkirneck)
	OBERKIRNECK_B_297_KORNSTR = 'de:08136:7023:0:H1'  # Rtg Schw. Gmünd (Lorch Oberkirneck)
	B_297_KORNSTR = 'de:08136:7023:0:H1'  # Rtg Schw. Gmünd (Lorch Oberkirneck)
	LORCH_MAIERHOFSTRASSE = 'de:08136:7045'
	LORCH_MAIERHOFSTRASSE_1 = 'de:08136:7045:0:H1'  # Rtg Schw. Gmünd (Lorch Lorch)
	LORCH_MAIERHOFSTRASSE_2 = 'de:08136:7045:0:R1'  # Rtg Göppingen (Lorch Lorch)
	MAIERHOFSTRASSE = 'de:08136:7045'
	MAIERHOFSTRASSE_1 = 'de:08136:7045:0:H1'  # Rtg Schw. Gmünd (Lorch Lorch)
	MAIERHOFSTRASSE_2 = 'de:08136:7045:0:R1'  # Rtg Göppingen (Lorch Lorch)
	HAGWEG = 'de:08136:7028:0:H1'  # Hagweg (Lorch Unterkirneck)
	UNTERKIRNECK_HAGWEG = 'de:08136:7028:0:H1'  # Hagweg (Lorch Unterkirneck)
	LORCH_RATTENHARZ_PULZHOFWEG = 'de:08136:7026'
	LORCH_RATTENHARZ_PULZHOFWEG_1 = 'de:08136:7026:0:H1'  # Rtg Schw. Gmünd (Lorch Lorch)
	LORCH_RATTENHARZ_PULZHOFWEG_2 = 'de:08136:7026:0:R1'  # Rtg Göppingen (Lorch Lorch)
	RATTENHARZ_PULZHOFWEG = 'de:08136:7026'
	RATTENHARZ_PULZHOFWEG_1 = 'de:08136:7026:0:H1'  # Rtg Schw. Gmünd (Lorch Lorch)
	RATTENHARZ_PULZHOFWEG_2 = 'de:08136:7026:0:R1'  # Rtg Göppingen (Lorch Lorch)
	LORCH_RATTENH_KAISERST_KIRCHE = 'de:08136:7025'
	LORCH_RATTENH_KAISERST_KIRCHE_1 = 'de:08136:7025:0:H1'  # Rtg Schw. Gmünd (Lorch Lorch)
	LORCH_RATTENH_KAISERST_KIRCHE_2 = 'de:08136:7025:0:R1'  # Rtg Göppingen (Lorch Lorch)
	RATTENHARZ_KAISERSTR__KIRCHE = 'de:08136:7025'
	RATTENHARZ_KAISERSTR__KIRCHE_1 = 'de:08136:7025:0:H1'  # Rtg Schw. Gmünd (Lorch Lorch)
	RATTENHARZ_KAISERSTR__KIRCHE_2 = 'de:08136:7025:0:R1'  # Rtg Göppingen (Lorch Lorch)
	WALDHAUSEN_ORTSMITTE_KIRCHE = 'de:08136:7033'
	WALDHAUSEN_ORTSMITTE_KIRCHE_1 = 'de:08136:7033:0:H1'  # Rtg Schw. Gmünd (Lorch Waldhausen b Schornd (Lorch))
	WALDHAUSEN_ORTSMITTE_KIRCHE_2 = 'de:08136:7033:0:R1'  # Rtg Göppingen (Lorch Waldhausen b Schornd (Lorch))
	ORTSMITTE_KIRCHE = 'de:08136:7033'
	ORTSMITTE_KIRCHE_1 = 'de:08136:7033:0:H1'  # Rtg Schw. Gmünd (Lorch Waldhausen b Schornd (Lorch))
	ORTSMITTE_KIRCHE_2 = 'de:08136:7033:0:R1'  # Rtg Göppingen (Lorch Waldhausen b Schornd (Lorch))
	WALDHAUSEN_REMSTALHALLE = 'de:08136:7034'
	WALDHAUSEN_REMSTALHALLE_1 = 'de:08136:7034:0:H1'  # Rtg Schw. Gmünd (Lorch Waldhausen b Schornd (Lorch))
	WALDHAUSEN_REMSTALHALLE_2 = 'de:08136:7034:0:R1'  # Rtg Göppingen (Lorch Waldhausen b Schornd (Lorch))
	REMSTALHALLE = 'de:08136:7034'
	REMSTALHALLE_1 = 'de:08136:7034:0:H1'  # Rtg Schw. Gmünd (Lorch Waldhausen b Schornd (Lorch))
	REMSTALHALLE_2 = 'de:08136:7034:0:R1'  # Rtg Göppingen (Lorch Waldhausen b Schornd (Lorch))
	INDUSTRIEGEBIET_FA__KNECHT = 'de:08136:7013'
	INDUSTRIEGEBIET_FA__KNECHT_1 = 'de:08136:7013:0:H1'  # Rtg Schw. Gmünd (Lorch Lorch)
	INDUSTRIEGEBIET_FA__KNECHT_2 = 'de:08136:7013:0:R1'  # Rtg Göppingen (Lorch Lorch)
	LORCH_IND_GEBIET_FA__KNECHT = 'de:08136:7013'
	LORCH_IND_GEBIET_FA__KNECHT_1 = 'de:08136:7013:0:H1'  # Rtg Schw. Gmünd (Lorch Lorch)
	LORCH_IND_GEBIET_FA__KNECHT_2 = 'de:08136:7013:0:R1'  # Rtg Göppingen (Lorch Lorch)
	LORCH_STUTTGARTER__MARIENSTR = 'de:08136:7021:0:1'  # Rtg Ind-geb./Fa. Knecht (Lorch Lorch)
	STUTTGARTER__MARIENSTR = 'de:08136:7021:0:1'  # Rtg Ind-geb./Fa. Knecht (Lorch Lorch)
	LORCH_SONNE = 'de:08136:7014'
	LORCH_SONNE_1 = 'de:08136:7014:0:H1'  # vom Bhf. Lorch (Lorch Lorch)
	LORCH_SONNE_2 = 'de:08136:7014:0:R1'  # Rtg Lorch Bahnhof (Lorch Lorch)
	KIESÄCKER_FA__WOLLIN = 'de:08136:7032'
	KIESÄCKER_FA__WOLLIN_1 = 'de:08136:7032:0:H1'  # Rtg Schw. Gmünd (Lorch Waldhausen b Schornd (Lorch))
	KIESÄCKER_FA__WOLLIN_2 = 'de:08136:7032:0:R1'  # Rtg Göppingen (Lorch Waldhausen b Schornd (Lorch))
	WALDH__KIESÄCKER_FA__WOLLIN = 'de:08136:7032'
	WALDH__KIESÄCKER_FA__WOLLIN_1 = 'de:08136:7032:0:H1'  # Rtg Schw. Gmünd (Lorch Waldhausen b Schornd (Lorch))
	WALDH__KIESÄCKER_FA__WOLLIN_2 = 'de:08136:7032:0:R1'  # Rtg Göppingen (Lorch Waldhausen b Schornd (Lorch))
	KREISSTRASSE = 'de:08136:7029'
	KREISSTRASSE_1 = 'de:08136:7029:0:H1'  # Rtg Schw. Gmünd (Lorch Unterkirneck)
	KREISSTRASSE_2 = 'de:08136:7029:0:R1'  # Rtg Göppingen (Lorch Unterkirneck)
	UNTERKIRNECK_KREISSTRASSE = 'de:08136:7029'
	UNTERKIRNECK_KREISSTRASSE_1 = 'de:08136:7029:0:H1'  # Rtg Schw. Gmünd (Lorch Unterkirneck)
	UNTERKIRNECK_KREISSTRASSE_2 = 'de:08136:7029:0:R1'  # Rtg Göppingen (Lorch Unterkirneck)
	WALDHAUSEN__LORCH__MÜHLSTR = 'de:08136:7048'
	WALDHAUSEN__LORCH__MÜHLSTR_1 = 'de:08136:7048:0:1'  # Rtg Waldhausen (Lorch Waldhausen b Schornd (Lorch))
	WALDHAUSEN__LORCH__MÜHLSTR_2 = 'de:08136:7048:0:2'  # Rtg Weitmars (Lorch Waldhausen b Schornd (Lorch))
	WEITMARS_TALSTR = 'de:08136:7039'
	WEITMARS_TALSTR_1 = 'de:08136:7039:0:1'  # eRtg Lorch (Lorch Lorch)
	WEITMARS_TALSTR_2 = 'de:08136:7039:0:2'  # eRtg Edenhof (Lorch Lorch)
	WALKERSBACHER_TAL = 'de:08136:7035'
	WALKERSBACHER_TAL_1 = 'de:08136:7035:0:1'  # eRtg Edenhof (Lorch Lorch)
	WALKERSBACHER_TAL_2 = 'de:08136:7035:0:2'  # eRtg Metzelhof (Lorch Lorch)
	WEITMARS_WALKERSBACHER_TAL = 'de:08136:7035'
	WEITMARS_WALKERSBACHER_TAL_1 = 'de:08136:7035:0:1'  # eRtg Edenhof (Lorch Lorch)
	WEITMARS_WALKERSBACHER_TAL_2 = 'de:08136:7035:0:2'  # eRtg Metzelhof (Lorch Lorch)
	EDENHOF = 'de:08136:7001'
	EDENHOF_1 = 'de:08136:7001:0:1'  # eRtg Weitmars (Lorch Lorch)
	EDENHOF_2 = 'de:08136:7001:0:2'  # eRtg Walkersbachtal (Lorch Lorch)
	LORCH_EDENHOF = 'de:08136:7001'
	LORCH_EDENHOF_1 = 'de:08136:7001:0:1'  # eRtg Weitmars (Lorch Lorch)
	LORCH_EDENHOF_2 = 'de:08136:7001:0:2'  # eRtg Walkersbachtal (Lorch Lorch)
	METZELHOF = 'de:08136:7022'
	METZELHOF_1 = 'de:08136:7022:0:1'  # eRtg Walkersbachtal (Lorch Lorch)
	METZELHOF_2 = 'de:08136:7022:0:2'  # eRtg Walkersbach (Lorch Lorch)
	LORCH_METZELHOF = 'de:08136:7022'
	LORCH_METZELHOF_1 = 'de:08136:7022:0:1'  # eRtg Walkersbachtal (Lorch Lorch)
	LORCH_METZELHOF_2 = 'de:08136:7022:0:2'  # eRtg Walkersbach (Lorch Lorch)
	SCHW__GMÜND_REITP__NEUBR_GASSE = 'de:08136:3518:0:R1'  # Reitprechts Neubrunnengasse (Schwäbisch Gmünd Schwäbisch Gmünd)
	REITPRECHTS_NEUBRUNNENGASSE = 'de:08136:3518:0:R1'  # Reitprechts Neubrunnengasse (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHW__GMÜND_REITP__AM_URSPRING = 'de:08136:3505:0:H1'  # Reitprechts Am Urspring (Schwäbisch Gmünd Schwäbisch Gmünd)
	REITPRECHTS_AM_URSPRING = 'de:08136:3505:0:H1'  # Reitprechts Am Urspring (Schwäbisch Gmünd Schwäbisch Gmünd)
	ADLER_APOTHEKE = 'de:08136:3508'
	ADLER_APOTHEKE_1 = 'de:08136:3508:0:H1'  # Rtg Schw. Gmünd (Schwäbisch Gmünd Straßdorf)
	ADLER_APOTHEKE_2 = 'de:08136:3508:0:R1'  # Rtg Göppingen (Schwäbisch Gmünd Straßdorf)
	STRASSDORF_ADLER_APOTHEKE = 'de:08136:3508'
	STRASSDORF_ADLER_APOTHEKE_1 = 'de:08136:3508:0:H1'  # Rtg Schw. Gmünd (Schwäbisch Gmünd Straßdorf)
	STRASSDORF_ADLER_APOTHEKE_2 = 'de:08136:3508:0:R1'  # Rtg Göppingen (Schwäbisch Gmünd Straßdorf)
	STRASSDORF_TH__STORM_STR__SDLG = 'de:08136:3513'
	STRASSDORF_TH__STORM_STR__SDLG_1 = 'de:08136:3513:0:H1'  # Rtg Schw. Gmünd (Schwäbisch Gmünd Straßdorf)
	STRASSDORF_TH__STORM_STR__SDLG_2 = 'de:08136:3513:0:R1'  # Rtg Göppingen (Schwäbisch Gmünd Straßdorf)
	TH__STORM_STR__SIEDLUNG = 'de:08136:3513'
	TH__STORM_STR__SIEDLUNG_1 = 'de:08136:3513:0:H1'  # Rtg Schw. Gmünd (Schwäbisch Gmünd Straßdorf)
	TH__STORM_STR__SIEDLUNG_2 = 'de:08136:3513:0:R1'  # Rtg Göppingen (Schwäbisch Gmünd Straßdorf)
	METLANGEN_ORT = 'de:08136:3502:0:R1'  # Metlangen Ort (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHWÄBISCH_GMÜND_METLANGEN_ORT = 'de:08136:3502:0:R1'  # Metlangen Ort (Schwäbisch Gmünd Schwäbisch Gmünd)
	METLANGEN_ABZWEIG = 'de:08136:3503:0:H1'  # Metlangen Abzweig (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHW__GMÜND_METLANGEN_ABZW = 'de:08136:3503:0:H1'  # Metlangen Abzweig (Schwäbisch Gmünd Schwäbisch Gmünd)
	WISSGOLDINGEN_BEZIRKSAMT = 'de:08136:7821'
	WISSGOLDINGEN_BEZIRKSAMT_1 = 'de:08136:7821:0:1'  # Bezirksamt (Waldstetten Wißgoldingen)
	WISSGOLDINGEN_BEZIRKSAMT_2 = 'de:08136:7821:0:R1'  # Bezirksamt (Waldstetten Wißgoldingen)
	WISSGOLDINGEN_LINDENSTRASSE = 'de:08136:7822:0:1'  # Lindenstraße (Waldstetten Wißgoldingen)
	LORCH_PFÄFFLESTR__SCHILLERSCH = 'de:08136:7046'
	LORCH_PFÄFFLESTR__SCHILLERSCH_1 = 'de:08136:7046:0:1'  # Rtg Stuttgarter-/Marienstr. (Lorch Lorch)
	LORCH_PFÄFFLESTR__SCHILLERSCH_2 = 'de:08136:7046:0:2'  # Rtg Lorch Bf (Lorch Lorch)
	PFÄFFLESTRASSE_SCHILLERSCHULE = 'de:08136:7046'
	PFÄFFLESTRASSE_SCHILLERSCHULE_1 = 'de:08136:7046:0:1'  # Rtg Stuttgarter-/Marienstr. (Lorch Lorch)
	PFÄFFLESTRASSE_SCHILLERSCHULE_2 = 'de:08136:7046:0:2'  # Rtg Lorch Bf (Lorch Lorch)
	LORCH_SCHÄFERSFELD_SCHULEN = 'de:08136:7027'
	LORCH_SCHÄFERSFELD_SCHULEN_1 = 'de:08136:7027:0:1'  # eRtg Alle Richtungen (Lorch Lorch)
	LORCH_SCHÄFERSFELD_SCHULEN_2 = 'de:08136:7027:0:H1'  # Schäfersfeld Schulen (Lorch Lorch)
	SCHÄFERSFELD_SCHULEN = 'de:08136:7027'
	SCHÄFERSFELD_SCHULEN_1 = 'de:08136:7027:0:1'  # eRtg Alle Richtungen (Lorch Lorch)
	SCHÄFERSFELD_SCHULEN_2 = 'de:08136:7027:0:H1'  # Schäfersfeld Schulen (Lorch Lorch)
	GSCHWEND_WELZHEIMER_STRASSE = 'de:08136:6514'
	GSCHWEND_WELZHEIMER_STRASSE_1 = 'de:08136:6514:0:3'  # Rtg Marktplatz (Gschwend Gschwend)
	GSCHWEND_WELZHEIMER_STRASSE_2 = 'de:08136:6514:0:4'  # Rtg Dinglesmad (Gschwend Gschwend)
	GSCHWEND_DINGLESMAD = 'de:08136:6537'
	GSCHWEND_DINGLESMAD_1 = 'de:08136:6537:0:3'  # Rtg Welzheimer Str. (Gschwend Gschwend)
	GSCHWEND_DINGLESMAD_2 = 'de:08136:6537:0:4'  # Rtg. Welzheim (Gschwend Gschwend)
	DINGLESMAD = 'de:08136:6537'
	DINGLESMAD_1 = 'de:08136:6537:0:3'  # Rtg Welzheimer Str. (Gschwend Gschwend)
	DINGLESMAD_2 = 'de:08136:6537:0:4'  # Rtg. Welzheim (Gschwend Gschwend)
	BRANDHOF_L_1080 = 'de:08136:6538'
	BRANDHOF_L_1080_1 = 'de:08136:6538:0:3'  # Rtg Dinglesmad (Gschwend Gschwend)
	BRANDHOF_L_1080_2 = 'de:08136:6538:0:4'  # Rtg Kaisersbach (Gschwend Gschwend)
	GSCHWEND_BRANDHOF_L_1080 = 'de:08136:6538'
	GSCHWEND_BRANDHOF_L_1080_1 = 'de:08136:6538:0:3'  # Rtg Dinglesmad (Gschwend Gschwend)
	GSCHWEND_BRANDHOF_L_1080_2 = 'de:08136:6538:0:4'  # Rtg Kaisersbach (Gschwend Gschwend)
	KATH__KINDERGARTEN = 'de:08136:7016:0:H1'  # Kath. Kindergarten (Lorch Lorch)
	LORCH_KATH__KINDERGARTEN = 'de:08136:7016:0:H1'  # Kath. Kindergarten (Lorch Lorch)
	LORCH_EV__KINDERGARTEN = 'de:08136:7012:0:H1'  # Ev. Kindergarten (Lorch Lorch)
	DÜRERWEG = 'de:08136:7011:0:H1'  # Dürerweg (Lorch Lorch)
	LORCH_DÜRERWEG = 'de:08136:7011:0:H1'  # Dürerweg (Lorch Lorch)
	NOLDEWEG = 'de:08136:7003:0:H1'  # Noldeweg (Lorch Lorch)
	LORCH_NOLDEWEG = 'de:08136:7003:0:H1'  # Noldeweg (Lorch Lorch)
	LORCH_BEETHOVENSTR = 'de:08136:7006:0:H1'  # Beethovenstraße (Lorch Lorch)
	BEETHOVENSTRASSE_35 = 'de:08136:7007:0:H1'  # Beethovenstraße 35 (Lorch Lorch)
	LORCH_BEETHOVENSTR__35 = 'de:08136:7007:0:H1'  # Beethovenstraße 35 (Lorch Lorch)
	BEETHOVENSTRASSE_ABZW = 'de:08136:7008:0:H1'  # Beethovenstraße Abzw. (Lorch Lorch)
	LORCH_BEETHOVENSTR__ABZW = 'de:08136:7008:0:H1'  # Beethovenstraße Abzw. (Lorch Lorch)
	LORCH_SONNENWEG = 'de:08136:7020:0:H1'  # Sonnenweg (Lorch Lorch)
	LORCH_BERGSTRASSE = 'de:08136:7009:0:H1'  # Bergstraße (Lorch Lorch)
	REBHUHNWEG = 'de:08136:7019:0:H1'  # Rebhuhnweg (Lorch Lorch)
	LORCH_REBHUHNWEG = 'de:08136:7019:0:H1'  # Rebhuhnweg (Lorch Lorch)
	AUF_DEM_BÜHL = 'de:08136:7004:0:H1'  # Auf dem Bühl (Lorch Lorch)
	LORCH_AUF_DEM_BÜHL = 'de:08136:7004:0:H1'  # Auf dem Bühl (Lorch Lorch)
	LORCH_HOCHHAUS = 'de:08136:7015:0:H1'  # Hochhaus (Lorch Lorch)
	LORCH_PANORAMASTR = 'de:08136:7018:0:H1'  # Panoramastraße (Lorch Lorch)
	AM_KREUZWIESLE = 'de:08136:7043:0:H1'  # Am Kreuzwiesle (Lorch Lorch)
	LORCH_AM_KREUZWIESLE = 'de:08136:7043:0:H1'  # Am Kreuzwiesle (Lorch Lorch)
	REMSSTRASSE = 'de:08136:7047:0:H1'  # Remsstraße (Lorch Lorch)
	LORCH_REMSSTRASSE = 'de:08136:7047:0:H1'  # Remsstraße (Lorch Lorch)
	HOHBERGSTRASSE = 'de:08136:7036:0:1'  # Rtg. Weitmars Kirche (Lorch Weitmars)
	WEITMARS_HOHBERGSTRASSE = 'de:08136:7036:0:1'  # Rtg. Weitmars Kirche (Lorch Weitmars)
	WEITMARS_KIRCHE = 'de:08136:7037:0:1'  # Rtg. Weitmars Rechbergstr. (Lorch Weitmars)
	WEITMARS_RECHBERGSTRASSE = 'de:08136:7038:0:3'  # Rtg. Weitmars Talstr. (Lorch Weitmars)
	LORCH_WILHELMSTRASSE = 'de:08136:7042'
	LORCH_WILHELMSTRASSE_1 = 'de:08136:7042:0:1'  # Rtg (Lorch Lorch)
	LORCH_WILHELMSTRASSE_2 = 'de:08136:7042:0:2'  # Rtg (Lorch Lorch)
	LORCH_SKILIFT = 'de:08136:7041'
	LORCH_SKILIFT_1 = 'de:08136:7041:0:1'  # eRtg Bruck (Lorch Lorch)
	LORCH_SKILIFT_2 = 'de:08136:7041:0:2'  # eRtg Kaiserlinde (Lorch Lorch)
	LORCH_STAUFERSCHULE = 'de:08136:7049:0:1'  # Rtg Sonne (Lorch Lorch)
	SCHWÄBISCH_GMÜND_SCHILLERSTR = 'de:08136:3029'
	SCHWÄBISCH_GMÜND_SCHILLERSTR_1 = 'de:08136:3029:0:H1'  # Rtg Schw. Gmünd (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHWÄBISCH_GMÜND_SCHILLERSTR_2 = 'de:08136:3029:0:R1'  # Rtg Göppingen (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHW__GD__HINTERE_SCHMIEDGASSE = 'de:08136:3028:0:1'  # eRtg Kappelgasse (Schwäbisch Gmünd Schwäbisch Gmünd)
	HINTERE_SCHMIEDGASSE = 'de:08136:3028:0:1'  # eRtg Kappelgasse (Schwäbisch Gmünd Schwäbisch Gmünd)
	LEDERGASSE = 'de:08136:3037'
	LEDERGASSE_1 = 'de:08136:3037:0:1'  # Rtg Schwäbisch Gmünd Bf (Schwäbisch Gmünd Schwäbisch Gmünd)
	LEDERGASSE_2 = 'de:08136:3037:0:2'  # eRtg Vordere Schmiedgasse (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHWÄBISCH_GMÜND_LEDERGASSE = 'de:08136:3037'
	SCHWÄBISCH_GMÜND_LEDERGASSE_1 = 'de:08136:3037:0:1'  # Rtg Schwäbisch Gmünd Bf (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHWÄBISCH_GMÜND_LEDERGASSE_2 = 'de:08136:3037:0:2'  # eRtg Vordere Schmiedgasse (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHWÄBISCH_GMÜND_PFITZERSTRASSE = 'de:08136:3054'
	SCHWÄBISCH_GMÜND_PFITZERSTRASSE_1 = 'de:08136:3054:0:1'  # eRtg Abzw. Rehnenhof (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHWÄBISCH_GMÜND_PFITZERSTRASSE_2 = 'de:08136:3054:0:2'  # eRtg Hintere Schmiedgasse (Schwäbisch Gmünd Schwäbisch Gmünd)
	PFITZERSTRASSE = 'de:08136:3054'
	PFITZERSTRASSE_1 = 'de:08136:3054:0:1'  # eRtg Abzw. Rehnenhof (Schwäbisch Gmünd Schwäbisch Gmünd)
	PFITZERSTRASSE_2 = 'de:08136:3054:0:2'  # eRtg Hintere Schmiedgasse (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHW__GMÜND_ABZW__REHNENH__WETZGAU = 'de:08136:3058'
	SCHW__GMÜND_ABZW__REHNENH__WETZGAU_1 = 'de:08136:3058:0:1'  # eRtg Möbel Wörner (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHW__GMÜND_ABZW__REHNENH__WETZGAU_2 = 'de:08136:3058:0:2'  # eRtg Pfitzerstraße (Schwäbisch Gmünd Schwäbisch Gmünd)
	ABZW__REHNENHAU_WETZGAU = 'de:08136:3058'
	ABZW__REHNENHAU_WETZGAU_1 = 'de:08136:3058:0:1'  # eRtg Möbel Wörner (Schwäbisch Gmünd Schwäbisch Gmünd)
	ABZW__REHNENHAU_WETZGAU_2 = 'de:08136:3058:0:2'  # eRtg Pfitzerstraße (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHW__GMÜND_VORD__SCHMIEDGASSE = 'de:08136:3065:0:1'  # eRtg Pfitzerstraße (Schwäbisch Gmünd Schwäbisch Gmünd)
	VORDERE_SCHMIEDGASSE = 'de:08136:3065:0:1'  # eRtg Pfitzerstraße (Schwäbisch Gmünd Schwäbisch Gmünd)
	SCHWÄBISCH_GMÜND_KAPPELGASSE = 'de:08136:3078:0:1'  # eRtg Schw. Gmünd (Schwäbisch Gmünd Schwäbisch Gmünd)
	KAPPELGASSE = 'de:08136:3078:0:1'  # eRtg Schw. Gmünd (Schwäbisch Gmünd Schwäbisch Gmünd)
	GERSTETTEN_WARTEHALLE = 'de:08135:1138:0:1'  # Alle Richtungen (Gerstetten Gerstetten)
	GUSSENSTADT_SIEDLUNG = 'de:08135:1156'
	GUSSENSTADT_SIEDLUNG_1 = 'de:08135:1156:0:1'  # Rtg Dorfplatz (Gerstetten Gussenstadt)
	GUSSENSTADT_SIEDLUNG_2 = 'de:08135:1156:0:2'  # Rtg Gerstetten (Gerstetten Gussenstadt)
	GUSSENSTADT_DORFPLATZ = 'de:08135:1147'
	GUSSENSTADT_DORFPLATZ_1 = 'de:08135:1147:0:1'  # Rtg Geislingen (Gerstetten Gussenstadt)
	GUSSENSTADT_DORFPLATZ_2 = 'de:08135:1147:0:2'  # Rtg Gerstetten (Gerstetten Gussenstadt)
	LORCH_FIRMA_BINZ = 'de:08136:7051'
	LORCH_FIRMA_BINZ_1 = 'de:08136:7051:0:H1'  # Rtg Schw. Gmünd (Lorch Lorch)
	LORCH_FIRMA_BINZ_2 = 'de:08136:7051:0:R1'  # Rtg Göppingen (Lorch Lorch)
	FIRMA_BINZ = 'de:08136:7051'
	FIRMA_BINZ_1 = 'de:08136:7051:0:H1'  # Rtg Schw. Gmünd (Lorch Lorch)
	FIRMA_BINZ_2 = 'de:08136:7051:0:R1'  # Rtg Göppingen (Lorch Lorch)
	MERKLINGEN = 'de:08425:2626'
	MERKLINGEN_1 = 'de:08425:2626:1:A'  # Bussteig A (Merklingen Merklingen)
	MERKLINGEN_2 = 'de:08425:2626:1:C'  # Bussteig C (Merklingen Merklingen)
	MERKLINGEN_3 = 'de:08425:2626:2:1'  # Gleis 1 (Merklingen Merklingen)
	MERKLINGEN_4 = 'de:08425:2626:3:4'  # Gleis 4 (Merklingen Merklingen)
	MACHTOLSHEIM_MERKLINGER_STR = 'de:08425:2634'
	MACHTOLSHEIM_MERKLINGER_STR_1 = 'de:08425:2634:0:1'  # Rtg Merklingen (Laichingen Machtolsheim)
	MACHTOLSHEIM_MERKLINGER_STR_2 = 'de:08425:2634:0:2'  # Rtg Bühlenhausen (Laichingen Machtolsheim)
	MACHTOLSHEIM_HAUPTSTR = 'de:08425:2640'
	MACHTOLSHEIM_HAUPTSTR_1 = 'de:08425:2640:0:1'  # Rtg Laichingen (Laichingen Machtolsheim)
	MACHTOLSHEIM_HAUPTSTR_2 = 'de:08425:2640:0:2'  # Rtg Bühlenhausen (Laichingen Machtolsheim)
	SCHALKSTETTEN_SCHULE = 'de:08425:2577:0:1'  # Alle Richtungen (Amstetten Schalkstetten)
	SCHALKSTETTEN_BINSENSTR = 'de:08425:2578:0:1'  # Alle Richtungen (Amstetten Schalkstetten)
	BINSENSTRASSE = 'de:08425:2578:0:1'  # Alle Richtungen (Amstetten Schalkstetten)
	MACHTOLSHEIM_KIRCHE = 'de:08425:2641:0:1'  # auswärts (Laichingen Machtolsheim)
	WIDDERSTALL = 'de:08425:2625'
	WIDDERSTALL_1 = 'de:08425:2625:0:1'  # Rtg Hohenstadt (Merklingen Merklingen)
	WIDDERSTALL_2 = 'de:08425:2625:0:2'  # Rtg Merklingen (Merklingen Merklingen)
	MERKLINGEN_WIDDERSTALL = 'de:08425:2625'
	MERKLINGEN_WIDDERSTALL_1 = 'de:08425:2625:0:1'  # Rtg Hohenstadt (Merklingen Merklingen)
	MERKLINGEN_WIDDERSTALL_2 = 'de:08425:2625:0:2'  # Rtg Merklingen (Merklingen Merklingen)
	MERKLINGEN_GEWERBEGEBIET = 'de:08425:2635'
	MERKLINGEN_GEWERBEGEBIET_1 = 'de:08425:2635:0:1'  # auswärts (Merklingen Merklingen)
	MERKLINGEN_GEWERBEGEBIET_2 = 'de:08425:2635:0:2'  # einwärts (Merklingen Merklingen)
	NELLINGEN_RATHAUS = 'de:08425:2613'
	NELLINGEN_RATHAUS_1 = 'de:08425:2613:0:1'  # auswärts (Nellingen Nellingen (UL))
	NELLINGEN_RATHAUS_2 = 'de:08425:2613:0:2'  # einwärts (Nellingen Nellingen (UL))
	LAICHINGEN_ZOB = 'de:08425:2638'
	LAICHINGEN_ZOB_1 = 'de:08425:2638:0:B'  # L30_350 (Laichingen Laichingen)
	LAICHINGEN_ZOB_2 = 'de:08425:2638:0:D'  # L334_339_369 (Laichingen Laichingen)
	LAICHINGEN_ZOB_3 = 'de:08425:2638:0:E'  # L335 (Laichingen Laichingen)
	GEISLINGER_STRASSE = 'de:08425:2660'
	GEISLINGER_STRASSE_1 = 'de:08425:2660:0:1'  # Ortsmitte (Laichingen Laichingen)
	GEISLINGER_STRASSE_2 = 'de:08425:2660:0:2'  # Merklingen (Laichingen Laichingen)
	LAICHINGEN_GEISLINGER_STR = 'de:08425:2660'
	LAICHINGEN_GEISLINGER_STR_1 = 'de:08425:2660:0:1'  # Ortsmitte (Laichingen Laichingen)
	LAICHINGEN_GEISLINGER_STR_2 = 'de:08425:2660:0:2'  # Merklingen (Laichingen Laichingen)
	LAICHINGEN_SCHULZENTRUM = 'de:08425:2653:0:3'  # Linie 360 (Laichingen Laichingen)
	MERKLINGEN_LINDENSTR = 'de:08425:2636'
	MERKLINGEN_LINDENSTR_1 = 'de:08425:2636:0:1'  # Ortsmitte (Merklingen Merklingen)
	MERKLINGEN_LINDENSTR_2 = 'de:08425:2636:0:2'  # Machtolsheim (Merklingen Merklingen)
	OBERE_GASSE = 'de:08425:2621'
	OBERE_GASSE_1 = 'de:08425:2621:0:1'  # Rtg Merklingen (Nellingen Nellingen (UL))
	OBERE_GASSE_2 = 'de:08425:2621:0:2'  # Rtg Rathaus (Nellingen Nellingen (UL))
	NELLINGEN_OBERE_GASSE = 'de:08425:2621'
	NELLINGEN_OBERE_GASSE_1 = 'de:08425:2621:0:1'  # Rtg Merklingen (Nellingen Nellingen (UL))
	NELLINGEN_OBERE_GASSE_2 = 'de:08425:2621:0:2'  # Rtg Rathaus (Nellingen Nellingen (UL))
	AUFHAUSER_STR = 'de:08425:2620'
	AUFHAUSER_STR_1 = 'de:08425:2620:0:1'  # Rtg Aufhausen (Nellingen Nellingen (UL))
	AUFHAUSER_STR_2 = 'de:08425:2620:0:2'  # Rtg Ortsmitte (Nellingen Nellingen (UL))
	NELLINGEN_AUFHAUSER_STR = 'de:08425:2620'
	NELLINGEN_AUFHAUSER_STR_1 = 'de:08425:2620:0:1'  # Rtg Aufhausen (Nellingen Nellingen (UL))
	NELLINGEN_AUFHAUSER_STR_2 = 'de:08425:2620:0:2'  # Rtg Ortsmitte (Nellingen Nellingen (UL))
	WIRTSCHAFTSSCHULE = 'de:08425:2655'
	WIRTSCHAFTSSCHULE_1 = 'de:08425:2655:0:1'  # Rtg Stadtmitte (Laichingen Laichingen)
	WIRTSCHAFTSSCHULE_2 = 'de:08425:2655:0:2'  # Rtg Merklingen (Laichingen Laichingen)
	LAICHINGEN_WIRTSCHAFTSSCHULE = 'de:08425:2655'
	LAICHINGEN_WIRTSCHAFTSSCHULE_1 = 'de:08425:2655:0:1'  # Rtg Stadtmitte (Laichingen Laichingen)
	LAICHINGEN_WIRTSCHAFTSSCHULE_2 = 'de:08425:2655:0:2'  # Rtg Merklingen (Laichingen Laichingen)
	LAICHINGEN_CARL_BENZ_STR = 'de:08425:2585'
	LAICHINGEN_CARL_BENZ_STR_1 = 'de:08425:2585:0:1'  # Rtg Stadtmitte (Laichingen Laichingen)
	LAICHINGEN_CARL_BENZ_STR_2 = 'de:08425:2585:0:2'  # Rtg Merklingen (Laichingen Laichingen)
	LAICHINGEN_RUDOLF_DIESEL_STR = 'de:08425:2591:0:2'  # Rtg Merklingen (Laichingen Laichingen)
	RUDOLF_DIESEL_STR = 'de:08425:2591:0:2'  # Rtg Merklingen (Laichingen Laichingen)
